      module db_sql

use sqlite

      implicit none

      !Add in types from decimal.f90
      integer, parameter  :: long_sql = selected_int_kind(9)  ! >= 9-digit integers 
                                                              !  (close to integer*4)
      integer, parameter :: sip_sql = selected_real_kind(6,37) 
!      integer, parameter :: dop_sql = selected_real_kind(15,307) ! in sqlite

      !-----------------------------------------------------------------------
      ! PARAMETERS FOR CHARACTER STRING LENGTHS
      !
      ! c1, c2, c3, c4, c5: various lengths
      ! c5: length of line in the input
      !-----------------------------------------------------------------------
      integer(long_sql), parameter :: c1 = 16, c2 = 32, & 
                                  c3 = 64, c4 = 80, c5 = 240, c6 =720
      !Below are global originally - will need to be set via functions
      !character(len=c5)          ::ddname
      !integer(long_sql)          ::ddlaenge, itmpin = 24
      logical(kind=4)             ::test0 = .false.
      
      integer(long_sql), parameter  :: DBVERSION = 2000

      integer, save :: dbsqllog=6
      

      private
      public :: dbsqllog
      public :: dbgetversion,dbinit,dbopen,dbrdrefdb,dbwrrefdb, &
                dbgetnrec,dbgetentries,dbrdgeo,dbhasrefdb,dbrdpes, &
                dbrdcpes,dbrdgra,dbrdhes,dbrddip,dbrdmoco,dbrdtrans, &
                dbrdctrans,dbrdnact,dbrdapes,dbrdagra,dbrdahes, &
                dbrdrefdbgeom,dbrdrefdbmaps,dbclose, &
                dbrdcasoc,dbrdentry,dbrdnm,dbrdadip,&
                dbrdsymrep,dbwrgeo,dbwrpes,dbwrcpes,dbwrgra,&
                dbwrhes,dbwrdip,dbwrmoco,dbwrtrans,dbwrctrans, &
                dbwrnact,dbwrapes,dbwragra,dbwrahes, dbwrcasoc, &
                dbwrsymrep,dberrmsg,dbwrentry,dbstarttrans,dbwrnm,&
                dbwradip,dbdelallentries,dbcommittrans,dbwrcidat,&
                dbrdcivec,dbwrcivec,dbgetallversion,dbrdinfo,dbwrinfo,&
                dbwrtemplate,dbrdcidat,dbrdtemplate,dbrdqcfreq

! Not used routines
      public :: dbaborttrans

      contains

!----------------
! iunit: details written to iunit
!----------------

!#######################################################################
!  Subroutine to check if table is present and has entries
!  Doesn't do count(*), instead just sees if there is one entry
!  Enough to know if has stuff in it
!
!  db - open SQL database
!  tablename - name of table to query
!  Dispname - display name of table
!  tableflag - whether table has anything in it
!#######################################################################
subroutine dbtableentry(db,iunit, tablename, dispname, tableflag)

   implicit none

   type(SQLITE_DATABASE), intent(inout)     :: db
   integer(long_sql), intent(in)                :: iunit
   character(len=*), intent(in)             :: tablename, dispname
   logical(kind=4), intent(inout)            :: tableflag

   type(SQLITE_STATEMENT)                   :: stmt
   logical(kind=4)                          :: finished
   type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
   character(len=c6)                              ::sql_string

   allocate(cols_p(2)) !How many possible columns
   !Use new exciting method to replace old count(*)
   !Build SQL string
   sql_string = "SELECT EXISTS (SELECT 1 FROM " // trim(tablename) // ")"

   call sqlite3_column_props( cols_p(1), 'present', SQLITE_INT )
   call sqlite3_prepare( db,&
   sql_string,&
      stmt, cols_p)
   call sqlite3_next_row( stmt, cols_p, finished )
!   tableflag = cols_p(1)%int_value
   if (cols_p(1)%int_value .eq. 1) then
      tableflag = .true.
   else
      tableflag = .false.
   endif
   if (db%error.ne.SQLITE_OK) then
      write (iunit,'(a20,a15)')  trim(dispname),'    Not Present'
   else if (tableflag) then
      write (iunit,'(a20,a15)') trim(dispname),'     Included'
   else if (.not.tableflag) then
      write (iunit,'(a20,a20)') trim(dispname),'    Not Included'
   endif

   !if (allocated(cols_p))deallocate(cols_p)

end subroutine dbtableentry
!#######################################################################
!
!   Subroutine to see which tables are present and have entries in db
!
!   sql_vers = 1 -Have to check if table is present and has at least 1 entry
!           = 2   -Check table list to see what is present, should have entry if there?
!  db - SQL database that is open
! sql_vers version of the database
! iunit   unit attached to DB
!lpesdb   is pes table present
!lgradb   is grad table present
!lhesdb   is hes table present
!lapesdb  is adiabatic pes table present
!lagradb  is adibatic grad  table present
!lahesdb  is hessian table present
!ldipdb   is dipole table present
!ladipdb  is adiabatic dipole table present
!lmocodb  is MO coefficients table present
!lnactdb  is non-adiabatic couplings tabe present
!ltransdb is ADT matrix table present
!lnmdb    is normal mode tabe present
!lcasocdb is (complex) adiabatic SOC table present
!lsocbas  Calculations uses the SOC basis in place of usual Spin basis
!lsymdb
!lpesdb   is complex pes table present
!ltransdb is complex ADT matrix table present
!
!#######################################################################

      subroutine dbgetentries(db, iunit,lpesdb,lgradb,lhesdb,lapesdb,&
                              lagradb,lahesdb,ldipdb,ladipdb,lmocodb,&
                              lnactdb,ltransdb,lnmdb,lcasocdb,lsymdb,&
                              lcpesdb,lctransdb,lcivecdb,ltemplate,&
                              lqcfreq)

      implicit none

      type(SQLITE_DATABASE), intent(inout)     :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: iunit
      logical(kind=4), intent(out)             :: lpesdb,lgradb,lhesdb,&
                                                  lapesdb,lagradb,lahesdb,&
                                                  ldipdb,ladipdb,lmocodb,&
                                                  lnactdb,ltransdb,lnmdb,&
                                                  lcasocdb,lsymdb,lcpesdb,&
                                                  lctransdb,lcivecdb,&
                                                  ltemplate,lqcfreq
      type(SQLITE_STATEMENT)                   :: stmt
      logical(kind=4)                          :: finished
      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      character(len=c3)                        :: routine
      integer(long_sql)                            :: i,table_count
      character(len=80), dimension(:), allocatable :: table_list !List of tables
      character(len=80)                          ::test_string

      write(iunit,'(/,a)') '*** Checking DB Entries ***'

      routine = 'dbgetentries'
      sql_vers = db%db_vers
      allocate(cols_p(2)) !How many possible columns
      if (sql_vers .gt. 2) then
         !Find out how many entries are present
         call sqlite3_column_props( cols_p(1), 'name', SQLITE_INT )
         call sqlite3_prepare( db,&
         "SELECT count(distinct(name)) FROM sqlite_master WHERE type='table'",&
               stmt, cols_p)
         call sqlite3_next_row( stmt, cols_p, finished )
         table_count = cols_p(1)%int_value
         !Know that the column that will be returned is a char
         allocate(table_list(table_count))
         call sqlite3_column_props( cols_p(1), 'name', SQLITE_CHAR )
         call sqlite3_prepare( db,&
            "SELECT name FROM sqlite_master WHERE type='table'",&
            stmt, cols_p)
         i = 1
         do
            call sqlite3_next_row( stmt, cols_p, finished )
            if (finished) exit
            table_list(i) =cols_p(1)%char_value
            i = i + 1
         enddo
         !write(*,*) table_list !List of tables in db
         !Check which tables are present and set flags
         !PES
         test_string = 'pes'
         call checktable(test_string ,table_list,lpesdb, "PES",iunit)

         test_string = 'cpes'
         call checktable(test_string ,table_list,lcpesdb, &
              "Complex PES",iunit)

         !aPES
         test_string = 'apes'
         call checktable(test_string ,table_list,lapesdb, "Adiab PES",iunit)

         !Grad
         test_string = 'gra'
         call checktable(test_string ,table_list,lgradb,'Gradient', iunit)

         !aGrad
         test_string = 'agra'
         call checktable(test_string ,table_list,lagradb,&
              'Adiab Gradient', iunit)

         !Check is Hessian present
         test_string = 'hes'   !as if present should be a 1
         call checktable(test_string ,table_list,lhesdb,'Hessian', iunit)
         !Check is adiabatic Hessian present
         test_string = 'ahes'
         call checktable(test_string ,table_list,lahesdb,'Adiab Hessian', iunit)

         !Check is dipole present
         test_string = 'dip'
         call checktable(test_string ,table_list,ldipdb,'Dipole', iunit)
         !Check is adiabatic dipole present
         test_string = 'adip'
         call checktable(test_string ,table_list,ladipdb,'Adiab Dipole', iunit)

         !Check are molecular orbitals present
         test_string = 'moco'
         call checktable(test_string ,table_list,lmocodb,'MO Coeffs', iunit)

         !Check are nonadiabatic couplings present
         test_string = 'nact'
         call checktable(test_string ,table_list,lnactdb,'NAC', iunit)

         !Check if transformation matrix present
         test_string = 'trans'
         call checktable(test_string ,table_list,ltransdb,'ADT Matrix', iunit)

         test_string = 'ctrans'
         call checktable(test_string ,table_list,lctransdb,'Complex ADT',iunit)

         !Check are Normal Modes present
         test_string = 'nmode'
         call checktable(test_string ,table_list,lnmdb,'Normal Modes', iunit)

         !Check are Adiabatic Spin Orbit Coupling present
         test_string = 'casoc'
         call checktable(test_string ,table_list,lcasocdb,'SOC', iunit)

         !Check are Symmetry Replicas present
         test_string = 'symrep'
         call checktable(test_string ,table_list,lsymdb,'Symmetry replicas', &
              iunit)

         !Check are CI vectors present
         test_string = 'civec'
         call checktable(test_string ,table_list,lcivecdb,'CI vectors', iunit)

         !Check is template.dat stored
         test_string = 'template'
         call checktable(test_string ,table_list,ltemplate,'Template.dat', iunit)

         !Check is QC frequency file stored
         test_string = 'qcfreq'
         call checktable(test_string ,table_list,lqcfreq,'QC frequency file',&
              iunit)

         call sqlite3_finalize( stmt )

         if(allocated(table_list))deallocate(table_list)

      else if (sql_vers .eq. 2) then
         !Find out how many entries are present
         call sqlite3_column_props( cols_p(1), 'name', SQLITE_INT )
         call sqlite3_prepare( db,&
         "SELECT count(distinct(name)) FROM sqlite_master WHERE type='table'",&
         stmt, cols_p)
         call sqlite3_next_row( stmt, cols_p, finished )
         table_count = cols_p(1)%int_value
         !Know that the column that will be returned is a char
         allocate(table_list(table_count))
         call sqlite3_column_props( cols_p(1), 'name', SQLITE_CHAR )
         call sqlite3_prepare( db,&
            "SELECT name FROM sqlite_master WHERE type='table'",&
            stmt, cols_p)
         i = 1
         do
            call sqlite3_next_row( stmt, cols_p, finished )
            if (finished) exit
            table_list(i) =cols_p(1)%char_value
            i = i + 1
         enddo
         !write(*,*) table_list !List of tables in db
         !Check which tables are present and set flags
         !PES
         test_string = 'pes'
         call checktable(test_string ,table_list,lpesdb, "PES",iunit)

         test_string = 'cpes'
         call checktable(test_string ,table_list,lcpesdb, &
                   "Complex PES",iunit)

         !aPES
         test_string = 'apes'
         call checktable(test_string ,table_list,lapesdb, "Adiab PES",iunit)

         !Grad
         test_string = 'gra'
         call checktable(test_string ,table_list,lgradb,'Gradient', iunit)

         !aGrad
         test_string = 'agra'
         call checktable(test_string ,table_list,lagradb,'Adiab Gradient', iunit)

         !Check is Hessian present
         test_string = 'hes_1_1'   !as if present should be a 1
         call checktable(test_string ,table_list,lhesdb,'Hessian', iunit)
         !Check is adiabatic Hessian present
         test_string = 'ahes_1'
         call checktable(test_string ,table_list,lahesdb,'Adiab Hessian', iunit)

         !Check is dipole present
         test_string = 'dip'
         call checktable(test_string ,table_list,ldipdb,'Dipole', iunit)
         !Check is adiabatic dipole present
         test_string = 'adip'
         call checktable(test_string ,table_list,ladipdb,'Adiab Dipole', iunit)

         !Check are molecular orbitals present
         test_string = 'moco_1'
         call checktable(test_string ,table_list,lmocodb,'MO Coeffs', iunit)

         !Check are nonadiabatic couplings present
         test_string = 'nact'
         call checktable(test_string ,table_list,lnactdb,'NAC', iunit)

         !Check if transformation matrix present
         test_string = 'trans'
         call checktable(test_string ,table_list,ltransdb,'ADT Matrix', iunit)

         test_string = 'ctrans'
         call checktable(test_string ,table_list,lctransdb,'ADT Matrix', iunit)

         !Check are Normal Modes present
         test_string = 'nmode'
         call checktable(test_string ,table_list,lnmdb,'Normal Modes', iunit)

         !Check are Adiabatic Spin Orbit Coupling present
         test_string = 'casoc'
         call checktable(test_string ,table_list,lcasocdb,'SOC', iunit)

         !Check are Symmetry Replicas present
         test_string = 'symrep'
         call checktable(test_string ,table_list,lsymdb,'Symmetry replicas', iunit)

         call sqlite3_finalize( stmt )

         if(allocated(table_list))deallocate(table_list)

      elseif (sql_vers .eq. 1) then
         !Use new exciting method to replace old count(*)
         !Just checks if there is one row, so doesn't count through everything
         !Check if PES present
         !Due to how complex numbers implemented have to check PES contents
         call dbtableentry(db,iunit,'pes','PES',lpesdb)
         !Know PES is there, read in first entry to see complex
         call sqlite3_column_props( cols_p(1), 'compdim', SQLITE_INT )
         call sqlite3_prepare( db,&
            'SELECT compdim FROM pes where rowid=1',&
            stmt, cols_p)
         call sqlite3_next_row( stmt, cols_p, finished )

         !Check is complex diabatic PES present
         call dbtableentry(db,iunit,'cpes','Complex PES',lcpesdb)

         !Check is adiabatic PES present
         call dbtableentry(db,iunit,'apes','Adiab PES',lapesdb)

         !Check is Gradient present
         call dbtableentry(db,iunit,'gra','Gradient',lgradb)
         !Check is adiabatic gradient present
         call dbtableentry(db,iunit,'agra','Adiab Gradient',lagradb)

         !Check is Hessian present
         call dbtableentry(db,iunit,'hes','Hessian',lhesdb)
         !Check is adiabatic Hessian present
         call dbtableentry(db,iunit,'ahes','Adiab Hessian',lahesdb)

         !Check is dipole present
         call dbtableentry(db,iunit,'dip','Dipole',ldipdb)
         !Check is adiabatic dipole present
         call dbtableentry(db,iunit,'adip','Adiab Dipole',ladipdb)

         !Check are molecular orbitals present
         call dbtableentry(db,iunit,'moco','MO Coeffs',lmocodb) 

         !Check are nonadiabatic couplings present
         call dbtableentry(db,iunit,'nact','NAC',lnactdb) 

         !Check is transition matrix present
         call dbtableentry(db,iunit,'trans','ADT Matrix',ltransdb) 

         !Check is complex transition matrix present
         call dbtableentry(db,iunit,'ctrans','Complex ADT',lctransdb) 

         !Check are Normal Modes present
         call dbtableentry(db,iunit,'nmode','Normal Modes',lnmdb) 

         !Check are Adiabatic Spin Orbit Coupling present
         call dbtableentry(db,iunit,'casoc','SOC',lcasocdb)

         !Check are Symmetry Replicas present
         call dbtableentry(db,iunit,'symrep','Symmetry replicas',lsymdb)

         !Use old method to check what is there - left as example
         ! call sqlite3_column_props( cols_p(1), 'id', SQLITE_INT )
         ! call sqlite3_column_props( cols_p(2), 'co', SQLITE_INT )
         ! call sqlite3_prepare( db, 'SELECT count(distinct(id)) FROM symrep', stmt,&
         !                      cols_p)
         ! call sqlite3_next_row( stmt, cols_p, finished )
         ! lsymdb = cols_p(1)%int_value > 0
         ! if (db%error.ne.SQLITE_OK) then
         !    write (iunit,'(a20,a15)') ' Symmetry replicas  ','Not Present    '
         ! else if (lsymdb) then
         !    write (iunit,'(a20,a15)') ' Symmetry replicas  ','Included       '
         ! else if (.not.lsymdb) then
         !    write (iunit,'(a20,a15)') ' Symmetry replicas  ','Not Included   '
         ! endif

         call sqlite3_finalize( stmt )
      endif 
      deallocate(cols_p)

      end subroutine dbgetentries

!----------------
      subroutine dbgetversion(db, dbvers)
      implicit none

      type(SQLITE_DATABASE), intent(inout)        :: db
      integer(long_sql), intent(out)              :: dbvers

      type(SQLITE_STATEMENT)                      :: stmt
      type(SQLITE_COLUMN),dimension(:),pointer    :: cols_p
      logical(kind=4)                             :: finished

      call sqlite3_query_table(db, 'versions', cols_p)
      call sqlite3_prepare(db, 'SELECT quanticsversion FROM versions',&
                           stmt, cols_p)
      call sqlite3_next_row(stmt, cols_p, finished)
      if (finished) then
         dbvers = -1
         db%db_vers=-1
      else
         dbvers = cols_p(2)%int_value
         db%db_vers=dbvers
      endif
      deallocate(cols_p)
      call sqlite3_finalize( stmt )

      end subroutine dbgetversion

!----------------
      subroutine dbgetallversion(db, dbvers,quanticsvers)
         implicit none
   
         type(SQLITE_DATABASE), intent(inout)        :: db
         integer(long_sql), intent(out)              :: dbvers
         real(dop_sql), intent(out)                  :: quanticsvers
   
         type(SQLITE_STATEMENT)                      :: stmt
         type(SQLITE_COLUMN),dimension(:),pointer    :: cols_p
         logical(kind=4)                             :: finished
   
         call sqlite3_query_table(db, 'versions', cols_p)
         call sqlite3_prepare(db, 'SELECT * FROM versions',&
                              stmt, cols_p)
         call sqlite3_next_row(stmt, cols_p, finished)
         if (finished) then
            dbvers = -1
         else
            quanticsvers = cols_p(1)%double_value
            dbvers = cols_p(2)%int_value
            db%db_vers = cols_p(2)%int_value
         endif
         deallocate(cols_p)
         call sqlite3_finalize( stmt )
   
         end subroutine dbgetallversion
!----------------
!     This contains the reference for the SQL schema

!     db is the new db info
!     sql_vers the sql version to use to create db 
!         infoarray(1) = natom
!         infoarray(2) = nstates
!         infoarray(3) = nmodes
!         infoarray(4) = nbasis
!         infoarray(5) = nsymop       
!         infoarray(6) = maxspblk     
!         infoarray(7) = nqcstates
!
!        flagarray(1) = lpesdb
!        flagarray(2) = lapesdb
!        flagarray(3) = lnmdb
!        flagarray(4) = lnactdb
!        flagarray(5) = ltransdb
!        flagarray(6) = lgradb
!        flagarray(7) = lagradb
!        flagarray(8) = lhesdb
!        flagarray(9) = lahesdb
!        flagarray(10) = lmocodb
!        flagarray(11) = lsymdb
!        flagarray(12) = ldipdb
!        flagarray(13) = ladipdb
!        flagarray(14) = lcasocdb
!        flagarray(15) = lcivecdb
!        flagarray(16) = lcpesdb  
!        flagarray(17) = lctransdb  
!----------------
      subroutine dbinit(db, sql_vers, quantics_vers, infoarray,flagarray)
      implicit none

      type(SQLITE_DATABASE), intent(inout)        :: db
      integer(long_sql), intent(inout)            :: sql_vers!SQL database version
      real(dop_sql), intent(in)                   :: quantics_vers  
      integer(long_sql), dimension(:), intent(in) :: infoarray   !Info for setting 
                                                                 ! up DB
      logical(kind=4), dimension(:), intent(in)   :: flagarray   !Flags for setting
                                                                 ! up DB
      character(len=200000)                       :: cmd
      logical(kind=4)                             :: busy
      character(len=c5)                           :: message
      character(len=c3)                           :: routine
  
      busy = .true.
      routine = "DBinit (db_sql.F90)"
      ! might need a sleep in here or something
      do while (busy)
         call sqlite3_begin(db, SQLITE_TRANS_IMMEDIATE)
         busy = db%error.eq.SQLITE_BUSY
      enddo
      if (db%error.ne.SQLITE_OK) then
         write(message,'(2a)') 'sql error while beginning transaction: ', &
                               db%errmsg
         call dberrmsg( routine, message )
      endif

      !SQL version will be tested in call to a create table subroutine
      !Set  SQL version of the file
      db%db_vers = sql_vers
         !Foreign keys must be on for ID to link all entries in tables
         cmd = 'PRAGMA foreign_keys=ON'
         call sqlite3_do(db, cmd)

         !Create the tables one at a time
         !First the reference tables
         !Version table
         call creversion(db,cmd)
         !Create refdb
         call crerefdb(db,cmd)
         !Create refdbgeom
         call crerefdbgeom(db,cmd)
         !Create symmref
         call crerefsym(db,infoarray(1),cmd)

         !Create info table
         call creinfo(db,cmd)

         !Create template file table
         call cretemplate(db,cmd)

         !Create dbentry
         call credbent(db,cmd)
         !write(*,*) trim(cmd)

         !Now the tables with data in them
         !Create geom table
         !inforarray(1) is natom
         call cregeo(db,infoarray(1),cmd)
         !write(*,*) trim(cmd)

         !Create PES table
         if (flagarray(1) .eqv. .true.) then
            !infoarray(2) is nstates
            call crepes(db,infoarray(2),cmd)
         endif

         !Create Adibatic PES table
         if (flagarray(2) .eqv. .true.) then
            !infoarray(2) is nstates
            call creapes(db,infoarray(2),cmd)
            !write(*,*) trim(cmd)
         endif
         
         !Create normal mode table
         if (flagarray(3) .eqv. .true.) then
            !infoarray(3) is nmodes
            call crenmodes(db,infoarray(3),cmd)
            !write(*,*) trim(cmd)
         endif

         !Create nact table
         if (flagarray(4) .eqv. .true.) then
            !inforarray(1) is natom
            call crenact(db,infoarray(1),cmd)
            !write(*,*) trim(cmd)
         endif

         !Create trans table
         if (flagarray(5) .eqv. .true.) then
            !infoarray(2) is nstates
            call cretrans(db,infoarray(2),cmd)
         endif

         !Create grad table
         if (flagarray(6) .eqv. .true.) then
            !infoarray(2) is nstates
            !infoarray(1) is natom
            call cregrad(db,infoarray(2),infoarray(1),cmd)
            !write(*,*) trim(cmd)
         endif

         !Create Adiabatic grad table
         if (flagarray(7) .eqv. .true.) then
            !infoarray(2) is nstates
            !infoarray(1) is natom
            call creagrad(db,infoarray(2),infoarray(1),cmd)
            !write(*,*) trim(cmd)
         endif

         !Create Hessian table
         if (flagarray(8) .eqv. .true.) then
            !infoarray(2) is nstates
            !inforarray(1) is natom
            call crehess(db,sql_vers,infoarray(2),infoarray(1),cmd)
            !write(*,*) trim(cmd)
         endif

         !Create Adiabatic Hessian table
         if (flagarray(9) .eqv. .true.) then
            !infoarray(2) is nstates
            !inforarray(1) is natom
            call creahess(db,sql_vers,infoarray(2),infoarray(1),cmd)
            !write(*,*) trim(cmd)
         endif

         !Create MOCO table
         if (flagarray(10) .eqv. .true.) then
            !infoarray(4) is nbasis
            call cremoco(db,sql_vers,infoarray(4),cmd)
            !write(*,*) trim(cmd)
         endif

         !Create symrep table
         if (flagarray(11) .eqv. .true.) then
            !inforarray(5) is nsymop
            call cresymrep(db,infoarray(5),cmd)
         endif

         !Create dip table
         if (flagarray(12) .eqv. .true.) then
            call credip(db,cmd)
         endif

         !Create adip table
         if (flagarray(13) .eqv. .true.) then
            call creadip(db,cmd)
         endif

         !Create adiabatic spin orbit couping
         if (flagarray(14) .eqv. .true.) then
            !infoarray(7) is nqcstates
            call crecasoc(db,infoarray(7),cmd)
         endif

         !CI vector stuff 
         if (flagarray(15) .eqv. .true.) then
            call crecivec(db,infoarray(6),cmd)
         endif

         if (flagarray(16) .eqv. .true.) then
            !infoarray(2) is nstates
            call crecpes(db,infoarray(2),cmd)
         endif

         if (flagarray(17) .eqv. .true.) then
            !infoarray(2) is nstates
            call crectrans(db,infoarray(2),cmd)
         endif

         !Create the quantum chemistry file table
         if (flagarray(18)) call creqcfreq(db,cmd)

      call sqlite3_commit( db )
      !Check that tables are committed
      if (db%error.ne.SQLITE_OK) then
         write(message,'(2a)') 'sql error while creating tables: ', &
                               db%errmsg
         call dberrmsg( routine, message )
      endif

      call dbwrversion( db,quantics_vers,sql_vers )

      if (db%error.ne.SQLITE_OK) then
         write(message,'(2a)') 'sql error while committing transaction: ', &
                               db%errmsg
         call dberrmsg( routine, message )
      endif
      end subroutine dbinit

!----------------
      subroutine dbgetnrec( db, nrec )
      implicit none
      type(SQLITE_DATABASE)                    :: db
      integer(long_sql), intent(out)               :: nrec

      type(SQLITE_STATEMENT)                   :: stmt
      logical(kind=4)                          :: finished
      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p

      allocate(cols_p(1))
      call sqlite3_column_props( cols_p(1), 'id', SQLITE_INT )
      call sqlite3_prepare( db, 'SELECT count(id) FROM dbentry', stmt,&
                            cols_p)
      call sqlite3_next_row( stmt, cols_p, finished )
      nrec = cols_p(1)%int_value

      deallocate(cols_p)
      call sqlite3_finalize( stmt )

      end subroutine dbgetnrec

!#######################################################################
!
! The following routines READ the DB tables and load into arrays.
! The range n = id0 - id is read into array(:,:,n). 
! If id < 0 than only the single record id0 is read into array(:,:,1)
!
! Set up WHERE clause to Select statement, only want given row id
! Combine WHERE with a select. This pulls in ALL columns in an entry
!#######################################################################

! Subroutine to read in the dbentry - primary index
      subroutine dbrdentry( db, id0, id, stat, timestamp )

      implicit none

      type(SQLITE_DATABASE), intent(in)        :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: id0,id
      integer(long_sql), dimension(:), intent(out) :: stat
      integer(long_sql), intent(out)               :: timestamp
      integer(long_sql)                            :: n

      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      type(SQLITE_STATEMENT)                   :: stmt
      character(len=c1)                        :: whereclause
      logical(kind=4)                          :: finished
      character(len=c5)                        :: message
      character(len=c3)                        :: routine
         
      routine = "DBRDentry (db_sql.F90)"
      sql_vers = db%db_vers
      !Read info from dbentry
      !This should get the column info and set cols_p to correct types
      call sqlite3_query_table( db, 'dbentry', cols_p )

      if (sql_vers .ge. 2) then
         !Get stat value and time stamp
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'dbentry', cols_p, stmt, &
                                 trim(whereclause))
            call sqlite3_next_row(stmt, cols_p, finished)
            if (finished) then
               write(message,'(1a,i0)') 'no such DB entry: ', id0
               call dberrmsg( routine, message )
            endif
            call sqlite3_get_column(cols_p(2), stat(1))
            call sqlite3_get_column(cols_p(3), timestamp)
            call sqlite3_finalize( stmt )  

         else
            call sqlite3_prepare_select(db, 'dbentry', cols_p, stmt)
            do n=id0,id
               call sqlite3_next_row(stmt, cols_p, finished)
               call sqlite3_get_column(cols_p(2), stat(n))
               call sqlite3_get_column(cols_p(3), timestamp) 
            enddo
            call sqlite3_finalize(stmt) 
         endif

      else
         !Get stat value
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'dbentry', cols_p, stmt, &
                                 trim(whereclause))
            call sqlite3_next_row(stmt, cols_p, finished)
            if (finished) then
               write(message,'(1a,i0)') 'no such DB entry: ', id0
               call dberrmsg( routine, message )
            endif
            call sqlite3_get_column(cols_p(2), stat(1))
            timestamp = -1
            call sqlite3_finalize( stmt )  

         else
            do n=id0,id
               write(whereclause, '(a,i0)') 'WHERE id=', n
               call sqlite3_prepare_select(db, 'dbentry', cols_p, stmt, &
                                 trim(whereclause))
               call sqlite3_next_row(stmt, cols_p, finished)
               if (finished) then
                  write(message,'(1a,i0)') 'no such DB entry: ', n
                  call dberrmsg( routine, message )
               endif
               call sqlite3_get_column(cols_p(2), stat(n))
               timestamp = -1
               call sqlite3_finalize( stmt )  
            enddo

         endif
      endif

      deallocate (cols_p)
      end subroutine dbrdentry

!-----------------------------------------------------------------------
! Subroutine to read in geometry from table
!  SQL version 1 row per coordinate
!  SQL version 2 each row is a single geometry
! dof is degrees of freedom which is natm * 3 (x,y,z) per atom
! geom(1:ndof,n)  is returned geometries  (ndof = 3*natm)
! ID is first column

      subroutine dbrdgeo( db, natoms, id0, id, geom )
      implicit none
      type(SQLITE_DATABASE), intent(in)        :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: id0,id
      integer(long_sql), intent(in)                :: natoms !Number of atoms
      real(dop_sql), dimension(:,:)                :: geom

      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      type(SQLITE_STATEMENT)                   :: stmt
      character(len=c1)                        :: whereclause
      logical(kind=4)                          :: finished
      integer(long_sql)                            :: dof,n

      !error reporting
      character(len=c3)                        :: routine
      character(len=c5)                        :: message

      sql_vers = db%db_vers
      routine = "dbrdgeo"

      call sqlite3_query_table( db, 'geo', cols_p )

      if (sql_vers .eq. 3) then
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'geo', cols_p, stmt, &
                                        trim(whereclause))
            do
               call sqlite3_next_row(stmt, cols_p, finished)
               if (finished) exit
               call sqlite3_get_column(cols_p(2), dof)
               call sqlite3_get_column(cols_p(4), geom(dof,1))
            enddo
            call sqlite3_finalize( stmt )
         else
            do n=id0,id
               write(whereclause, '(a,i0)') 'WHERE id=', n
               call sqlite3_prepare_select(db, 'geo', cols_p, stmt, &
                                           trim(whereclause))
               do
                  call sqlite3_next_row(stmt, cols_p, finished)
                  if (finished) exit
                  call sqlite3_get_column(cols_p(2), dof)
                  call sqlite3_get_column(cols_p(4), geom(dof,n))
               enddo
               call sqlite3_finalize( stmt )
            enddo
         endif

      else if ((sql_vers .eq. 2) .or. (sql_vers .ge. 4)) then
         dof=3*natoms
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'geo', cols_p, stmt, &
                                        trim(whereclause))
            call sqlite3_next_row(stmt, cols_p, finished)
            geom(1:dof,1) = cols_p(2:)%double_value
            call sqlite3_finalize( stmt )
         else
            call sqlite3_prepare_select(db, 'geo', cols_p, stmt)
            do n=id0,id
               call sqlite3_next_row(stmt, cols_p, finished)
               geom(1:dof,n) = cols_p(2:)%double_value
            enddo
            call sqlite3_finalize(stmt)
         endif

      else if (sql_vers .eq. 1) then
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'geo', cols_p, stmt, &
                                        trim(whereclause))
            do
               call sqlite3_next_row(stmt, cols_p, finished)
               if (finished) exit
               call sqlite3_get_column(cols_p(2), dof)
               call sqlite3_get_column(cols_p(3), geom(dof,1))
            enddo
            call sqlite3_finalize( stmt )
         else
            do n=id0,id
               write(whereclause, '(a,i0)') 'WHERE id=', n
               call sqlite3_prepare_select(db, 'geo', cols_p, stmt, &
                                           trim(whereclause))
               do
                  call sqlite3_next_row(stmt, cols_p, finished)
                  if (finished) exit
                  call sqlite3_get_column(cols_p(2), dof)
                  call sqlite3_get_column(cols_p(3), geom(dof,n))
               enddo
               call sqlite3_finalize( stmt )
            enddo
         endif

      endif

      !Check for DB errors
      if (db%error.ne.SQLITE_OK) then
         write(message,'(2a)') 'sql error while reading geometry: ', &
                               db%errmsg
         call dberrmsg( routine, message )
      endif

      deallocate (cols_p)

      end subroutine dbrdgeo

!-----------------------------------------------------------------------

      subroutine dbrdpes( db, nstates, id0, id, pes )
      implicit none
      type(SQLITE_DATABASE)                    :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: id0,id
      integer(long_sql), intent(in)                :: nstates   !number of states
      real(dop_sql), dimension(:,:,:)              :: pes

      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      type(SQLITE_STATEMENT)                   :: stmt
      character(len=c1)                        :: whereclause
      logical(kind=4)                          :: finished
      integer(long_sql)                            :: st1, st2, index, n

      sql_vers = db%db_vers

      call sqlite3_query_table( db, 'pes', cols_p )

      if (sql_vers .eq. 3) then
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'pes', cols_p, stmt, &
                                        trim(whereclause))
            do
               call sqlite3_next_row(stmt, cols_p, finished)
               if (finished) exit
               call sqlite3_get_column( cols_p(2), st1 )
               call sqlite3_get_column( cols_p(3), st2 )
               call sqlite3_get_column( cols_p(4), pes(st2,st1,1) )
            enddo
            call sqlite3_finalize( stmt )
         else
            do n=id0,id
               write(whereclause, '(a,i0)') 'WHERE id=', n
               call sqlite3_prepare_select(db, 'pes', cols_p, stmt, &
                                           trim(whereclause))
               do
                  call sqlite3_next_row(stmt, cols_p, finished)
                  if (finished) exit
                  call sqlite3_get_column( cols_p(2), st1 )
                  call sqlite3_get_column( cols_p(3), st2 )
                  call sqlite3_get_column( cols_p(4), pes(st2, st1,n) )
               enddo
               call sqlite3_finalize( stmt )
            enddo
         endif

      else if ((sql_vers .eq. 2) .or. (sql_vers .ge. 4)) then
         !Everything is in one row
         !Need to loop over states
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'pes', cols_p, stmt, &
                                        trim(whereclause))
            call sqlite3_next_row(stmt, cols_p, finished)
            index = 0
            do st1 = 1, nstates
               do st2 = st1, nstates
                  call sqlite3_get_column( cols_p(2+index), pes(st2,st1,1) )
                  index = index+1
               enddo
            enddo
            call sqlite3_finalize( stmt )
         else
            call sqlite3_prepare_select(db, 'pes', cols_p, stmt)
            do n=id0,id
               call sqlite3_next_row(stmt, cols_p, finished)
               index = 0
               do st1 = 1, nstates
                  do st2 = st1, nstates
                     call sqlite3_get_column( cols_p(2+index), pes(st2,st1,n) )
                     index = index+1
                  enddo
               enddo
            enddo
            call sqlite3_finalize(stmt)
         endif

      else if (sql_vers .eq. 1) then
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'pes', cols_p, stmt, &
                                        trim(whereclause))
            do
               call sqlite3_next_row(stmt, cols_p, finished)
               if (finished) exit
               call sqlite3_get_column( cols_p(2), st1 )
               call sqlite3_get_column( cols_p(3), st2 )
               !call sqlite3_get_column( cols_p(4), c )
               call sqlite3_get_column( cols_p(5), pes(st2,st1,1) )
            enddo
            call sqlite3_finalize( stmt )
         else
            do n=id0,id
               write(whereclause, '(a,i0)') 'WHERE id=', n
               call sqlite3_prepare_select(db, 'pes', cols_p, stmt, &
                                           trim(whereclause))
               do
                  call sqlite3_next_row(stmt, cols_p, finished)
                  if (finished) exit
                  call sqlite3_get_column( cols_p(2), st1 )
                  call sqlite3_get_column( cols_p(3), st2 )
                  !call sqlite3_get_column( cols_p(4), c )
                  call sqlite3_get_column( cols_p(5), pes(st2, st1,n) )
               enddo
               call sqlite3_finalize( stmt )
            enddo
         endif

      endif

! complete matrix (lower triangle was stored)
      if (id .lt. 0) then
         do st1=1,nstates-1
            do st2=st1+1,nstates
               pes(st1,st2,1)=pes(st2,st1,1)
            enddo
         enddo
      else
         do n=id0,id
            do st1=1,nstates-1
               do st2=st1+1,nstates
                  pes(st1,st2,n)=pes(st2,st1,n)
               enddo
            enddo
         enddo
      endif

      deallocate (cols_p)

      end subroutine dbrdpes

!-----------------------------------------------------------------------

      subroutine dbrdcpes( db, nstates, id0, id, cpes )
      implicit none
      type(SQLITE_DATABASE)                    :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: id0,id
      integer(long_sql), intent(in)                :: nstates   !number of states
      real(dop_sql), dimension(:,:,:)              :: cpes

      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      type(SQLITE_STATEMENT)                   :: stmt
      character(len=c1)                        :: whereclause
      logical(kind=4)                          :: finished
      integer(long_sql)                            :: st1, st2, index, n

      sql_vers = db%db_vers

      call sqlite3_query_table( db, 'cpes', cols_p )

      if (sql_vers .eq. 3) then
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'cpes', cols_p, stmt, &
                                        trim(whereclause))
            do
               call sqlite3_next_row(stmt, cols_p, finished)
               if (finished) exit
               call sqlite3_get_column( cols_p(2), st1 )
               call sqlite3_get_column( cols_p(3), st2 )
               call sqlite3_get_column( cols_p(4), cpes(st2,st1,1) )
            enddo
            call sqlite3_finalize( stmt )
         else
            do n=id0,id
               write(whereclause, '(a,i0)') 'WHERE id=', n
               call sqlite3_prepare_select(db, 'cpes', cols_p, stmt, &
                                           trim(whereclause))
               do
                  call sqlite3_next_row(stmt, cols_p, finished)
                  if (finished) exit
                  call sqlite3_get_column( cols_p(2), st1 )
                  call sqlite3_get_column( cols_p(3), st2 )
                  call sqlite3_get_column( cols_p(4), cpes(st2, st1,n) )
               enddo
               call sqlite3_finalize( stmt )
            enddo
         endif

      else if ((sql_vers .eq. 2) .or. (sql_vers .ge. 4)) then
         !Everything is in one row
         !Need to loop over states
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'cpes', cols_p, stmt, &
                                        trim(whereclause))
            call sqlite3_next_row(stmt, cols_p, finished)
            index = 0
            do st1 = 1, nstates
               do st2 = st1, nstates
                  call sqlite3_get_column( cols_p(2+index), cpes(st2,st1,1) )
                  index = index+1
               enddo
            enddo
            call sqlite3_finalize( stmt )
         else
            call sqlite3_prepare_select(db, 'cpes', cols_p, stmt)
            do n=id0,id
               call sqlite3_next_row(stmt, cols_p, finished)
               index = 0
               do st1 = 1, nstates
                  do st2 = st1, nstates
                     call sqlite3_get_column( cols_p(2+index), cpes(st2,st1,n) )
                     index = index+1
                  enddo
               enddo
            enddo
            call sqlite3_finalize(stmt)
         endif

      else if (sql_vers .eq. 1) then
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'cpes', cols_p, stmt, &
                                        trim(whereclause))
            do
               call sqlite3_next_row(stmt, cols_p, finished)
               if (finished) exit
               call sqlite3_get_column( cols_p(2), st1 )
               call sqlite3_get_column( cols_p(3), st2 )
               !call sqlite3_get_column( cols_p(4), c )
               call sqlite3_get_column( cols_p(5), cpes(st2,st1,1) )
            enddo
            call sqlite3_finalize( stmt )
         else
            do n=id0,id
               write(whereclause, '(a,i0)') 'WHERE id=', n
               call sqlite3_prepare_select(db, 'cpes', cols_p, stmt, &
                                           trim(whereclause))
               do
                  call sqlite3_next_row(stmt, cols_p, finished)
                  if (finished) exit
                  call sqlite3_get_column( cols_p(2), st1 )
                  call sqlite3_get_column( cols_p(3), st2 )
                  !call sqlite3_get_column( cols_p(4), c )
                  call sqlite3_get_column( cols_p(5), cpes(st2, st1,n) )
               enddo
               call sqlite3_finalize( stmt )
            enddo
         endif

      endif

! complete matrix (lower triangle was stored). Negative as complex conjugate.
      if (id .lt. 0) then
         do st1=1,nstates-1
            do st2=st1+1,nstates
               cpes(st1,st2,1)=-cpes(st2,st1,1)
            enddo
         enddo
      else
         do n=id0,id
            do st1=1,nstates-1
               do st2=st1+1,nstates
                  cpes(st1,st2,n)=-cpes(st2,st1,n)
               enddo
            enddo
         enddo
      endif

      deallocate (cols_p)

      end subroutine dbrdcpes

!-----------------------------------------------------------------------

      subroutine dbrdgra( db,nstates, natoms, id0, id, grad )
      implicit none
      type(SQLITE_DATABASE)                    :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: nstates   !number of states
      integer(long_sql), intent(in)                :: natoms !Number of atoms
      integer(long_sql), intent(in)                :: id0, id
      real(dop_sql), dimension(:,:,:,:)            :: grad

      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      type(SQLITE_STATEMENT)                   :: stmt
      character(len=c1)                        :: whereclause
      logical(kind=4)                          :: finished
      integer(long_sql)                            :: st1, st2, dof, index, n, ndof

      sql_vers = db%db_vers
      ndof = natoms*3

      call sqlite3_query_table( db, 'gra', cols_p )

      if (sql_vers .ge. 4) then
         !All in one row
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'gra', cols_p, stmt, &
                                        trim(whereclause))
            do st1 = 1, nstates
               do st2 = st1, nstates
                  index = 0
                  call sqlite3_next_row(stmt, cols_p, finished)
                  do dof = 1, ndof
                     grad(dof,st2,st1,1) = cols_p(4 + index)%double_value
                     index = index + 1
                  enddo
               enddo
            enddo
            call sqlite3_finalize( stmt )
         else
            call sqlite3_prepare_select(db, 'gra', cols_p, stmt)
            do n=id0,id
               do st1 = 1, nstates
                  do st2 = st1, nstates
                     index = 0
                     call sqlite3_next_row(stmt, cols_p, finished)
                     do dof = 1, ndof
                        grad(dof,st2,st1,n) = cols_p(4 + index)%double_value
                        index = index + 1
                     enddo
                  enddo
               enddo
            enddo
            call sqlite3_finalize(stmt)
         endif

      else if (sql_vers .eq. 3) then
         !Have to loop based on id
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'gra', cols_p, stmt, &
                                        trim(whereclause))
            do 
               call sqlite3_next_row(stmt, cols_p, finished)
               if (finished) exit
               call sqlite3_get_column( cols_p(2), st1 )
               call sqlite3_get_column( cols_p(3), st2 )
               call sqlite3_get_column( cols_p(4), dof )
               call sqlite3_get_column( cols_p(6), grad(dof,st2,st1,1) )
            enddo
            call sqlite3_finalize( stmt )
         else
            do n=id0,id
               write(whereclause, '(a,i0)') 'WHERE id=', n
               call sqlite3_prepare_select(db, 'gra', cols_p, stmt, &
                                           trim(whereclause))
               do 
                  call sqlite3_next_row(stmt, cols_p, finished)
                  if (finished) exit
                  call sqlite3_get_column( cols_p(2), st1 )
                  call sqlite3_get_column( cols_p(3), st2 )
                  call sqlite3_get_column( cols_p(4), dof )
                  call sqlite3_get_column( cols_p(6), grad(dof,st2,st1,n) )
               enddo
               call sqlite3_finalize( stmt )
            enddo
         endif

      else if (sql_vers .eq. 2) then
         !All in one row
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'gra', cols_p, stmt, &
                                        trim(whereclause))
            call sqlite3_next_row(stmt, cols_p, finished)
            !ndof = size(cols_p)
            index = 0
            do st1 = 1, nstates
               do st2 = st1, nstates
                  do dof = 1, natoms*3
                     grad(dof,st2,st1,1) = cols_p(2 + index)%double_value
                     index = index + 1
                  enddo
               enddo
            enddo
            call sqlite3_finalize( stmt )
         else
            call sqlite3_prepare_select(db, 'gra', cols_p, stmt)
            do n=id0,id
               call sqlite3_next_row(stmt, cols_p, finished)
               !ndof = size(cols_p)
               index = 0
               do st1 = 1, nstates
                  do st2 = st1, nstates
                     do dof = 1, natoms*3
                        grad(dof,st2,st1,n) = cols_p(2 + index)%double_value
                        index = index + 1
                     enddo
                  enddo
               enddo
            enddo
            call sqlite3_finalize(stmt)
         endif

      else if (sql_vers .eq. 1) then
         !Have to loop based on id
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'gra', cols_p, stmt, &
                                        trim(whereclause))
            do 
               call sqlite3_next_row(stmt, cols_p, finished)
               if (finished) exit
               call sqlite3_get_column( cols_p(2), st1 )
               call sqlite3_get_column( cols_p(3), st2 )
               call sqlite3_get_column( cols_p(4), dof )
               call sqlite3_get_column( cols_p(5), grad(dof,st2,st1,1) )
            enddo
            call sqlite3_finalize( stmt )
         else
            do n=id0,id
               write(whereclause, '(a,i0)') 'WHERE id=', n
               call sqlite3_prepare_select(db, 'gra', cols_p, stmt, &
                                           trim(whereclause))
               do 
                  call sqlite3_next_row(stmt, cols_p, finished)
                  if (finished) exit
                  call sqlite3_get_column( cols_p(2), st1 )
                  call sqlite3_get_column( cols_p(3), st2 )
                  call sqlite3_get_column( cols_p(4), dof )
                  call sqlite3_get_column( cols_p(5), grad(dof,st2,st1,n) )
               enddo
               call sqlite3_finalize( stmt )
            enddo
         endif

      endif

! Complete matrix (lower triangle stored)
      if (id0 .lt. 0) then
         do st1=1,nstates-1
            do st2=st1+1,nstates
               grad(1:ndof,st1,st2,1) = grad(1:ndof,st2,st1,1)
            enddo
         enddo
      else
         do n=id0,id
            do st1=1,nstates-1
               do st2=st1+1,nstates
                  grad(1:ndof,st1,st2,n) = grad(1:ndof,st2,st1,n)
               enddo
            enddo
         enddo
      endif

      deallocate (cols_p)

      end subroutine dbrdgra

!-----------------------------------------------------------------------

      subroutine dbrdhes( db,nstates, natoms, id0, id, hess )
      implicit none
      type(SQLITE_DATABASE)                    :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: nstates   !number of states
      integer(long_sql), intent(in)                :: natoms !Number of atoms
      integer(long_sql), intent(in)                :: id0,id
      real(dop_sql), dimension(:,:,:,:,:)          :: hess

      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      type(SQLITE_STATEMENT)                   :: stmt
      character(len=c1)                        :: whereclause, tabname
      logical(kind=4)                          :: finished
      integer(long_sql)                            :: st1, st2, dof1, dof2, &
                                                  index, n, rc, ndof
      sql_vers = db%db_vers
      ndof = natoms*3

      if (sql_vers .ge. 4) then
         !One table for all Hess
         write(tabname,'(A4)') "hes "
         call sqlite3_query_table( db, trim(tabname), cols_p )
         
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, trim(tabname), cols_p, &
                    stmt, trim(whereclause))

            !Loop over states
            do st1 = 1,nstates
               do st2 = st1,nstates
               !Loop over 1st coord
                  do dof1 =1, ndof
                     call sqlite3_next_row(stmt, cols_p, finished)
                     !Loop over 2nd coord
                     index = 5
                     do dof2 =1, ndof
                        hess(dof2,dof1,st2,st1,1) = &
                           cols_p(index)%double_value
                        hess(dof2,dof1,st1,st2,1) = &
                           cols_p(index)%double_value
                        index = index + 1
                     enddo
                  enddo
               enddo
            end do
            call sqlite3_finalize(stmt)

         else
            call sqlite3_prepare_select(db, trim(tabname), cols_p, stmt)
            do n=id0,id
               !Loop over states
               do st1 = 1,nstates
                  do st2 = st1,nstates
                  !Loop over 1st coord
                     do dof1 =1, natoms*3
                        call sqlite3_next_row(stmt, cols_p, finished)
                        !Loop over 2nd coord
                        index = 5
                        do dof2 =1, natoms*3
                           hess(dof2,dof1,st2,st1,n) = &
                              cols_p(index)%double_value
                           hess(dof2,dof1,st1,st2,n) = &
                              cols_p(index)%double_value
                           index = index + 1
                        enddo
                     enddo
                  enddo
               end do
            end do
            call sqlite3_finalize(stmt)  
            
         end if

      else if (sql_vers .eq. 3) then
         write(tabname,'(A4)') "hes "
         call sqlite3_query_table( db, trim(tabname), cols_p )

         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, trim(tabname), cols_p, &
                    stmt, trim(whereclause))
            do 
               call sqlite3_next_row(stmt, cols_p, finished)
               if (finished) exit
               call sqlite3_get_column( cols_p(2), st1 )
               call sqlite3_get_column( cols_p(3), st2 )
               call sqlite3_get_column( cols_p(4), dof1 )
               call sqlite3_get_column( cols_p(5), dof2 )
               call sqlite3_get_column( cols_p(8), hess(dof1,dof2,st2,st1,1))
            enddo
            call sqlite3_finalize( stmt )
         else
            do n=id0,id
               write(whereclause, '(a,i0)') 'WHERE id=', n
               call sqlite3_prepare_select(db, trim(tabname), cols_p, &
                       stmt, trim(whereclause))
               do 
                  call sqlite3_next_row(stmt, cols_p, finished)
                  if (finished) exit
                  call sqlite3_get_column( cols_p(2), st1 )
                  call sqlite3_get_column( cols_p(3), st2 )
                  call sqlite3_get_column( cols_p(4), dof1 )
                  call sqlite3_get_column( cols_p(5), dof2 )
                  call sqlite3_get_column( cols_p(8), &
                       hess(dof1,dof2,st2,st1,n) )
               enddo
               call sqlite3_finalize( stmt )
            enddo
         endif

      else if (sql_vers .eq. 2) then
         !In version 2 or greater each states has a unique Hess table
         !Need to loop over each state
         do st1=1, nstates
            !Now loop over second elec state
            do st2 = st1, nstates
               !Get table name, is hes_st2_st1 and query table
               write(tabname,'(A4,I0,a1,i0)') "hes_", st2,"_",st1
               !Get table info to set up columns
               call sqlite3_query_table( db, trim(tabname), cols_p )

               !All data is in one row
               if (id .lt. 0) then
                  write(whereclause, '(a,i0)') 'WHERE id=', id0
                  call sqlite3_prepare_select(db, trim(tabname), cols_p, &
                          stmt, trim(whereclause))
                  call sqlite3_next_row(stmt, cols_p, finished)
                  !keep track of position in cols_p
                  index = 0
                  !Loop over 1st coord
                  do dof1 =1, natoms*3
                     !Loop over 2nd coord
                     do dof2 =1, natoms*3
                        hess(dof2,dof1,st2,st1,1) = &
                           cols_p(2 + index)%double_value
                        index = index + 1
                     enddo
                  enddo
                  call sqlite3_finalize( stmt )
               else
                  call sqlite3_prepare_select(db, trim(tabname), cols_p, &
                  stmt)
                  call sqlite3_bind_all(db,stmt,cols_p(2:),rc)
                  do n=id0,id
                     call sqlite3_next_row(stmt, cols_p, finished)
                     !keep track of position in cols_p
                     index = 0
                     !Loop over 1st coord
                     do dof1 =1, natoms*3
                        !Loop over 2nd coord
                        do dof2 =1, natoms*3
                           hess(dof2,dof1,st2,st1,n) = &
                              cols_p(2 + index)%double_value
                           index = index + 1
                        enddo
                     enddo
                  enddo
                  call sqlite3_finalize(stmt)
               endif
            enddo
         enddo

      else if (sql_vers .eq. 1) then
         write(tabname,'(A4)') "hes "
         call sqlite3_query_table( db, trim(tabname), cols_p )

         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, trim(tabname), cols_p, &
                    stmt, trim(whereclause))
            do 
               call sqlite3_next_row(stmt, cols_p, finished)
               if (finished) exit
               call sqlite3_get_column( cols_p(2), st1 )
               call sqlite3_get_column( cols_p(3), st2 )
               call sqlite3_get_column( cols_p(4), dof1 )
               call sqlite3_get_column( cols_p(5), dof2 )
               call sqlite3_get_column( cols_p(6), hess(dof1,dof2,st2,st1,1))
            enddo
            call sqlite3_finalize( stmt )
         else
            do n=id0,id
               write(whereclause, '(a,i0)') 'WHERE id=', n
               call sqlite3_prepare_select(db, trim(tabname), cols_p, &
                       stmt, trim(whereclause))
               do 
                  call sqlite3_next_row(stmt, cols_p, finished)
                  if (finished) exit
                  call sqlite3_get_column( cols_p(2), st1 )
                  call sqlite3_get_column( cols_p(3), st2 )
                  call sqlite3_get_column( cols_p(4), dof1 )
                  call sqlite3_get_column( cols_p(5), dof2 )
                  call sqlite3_get_column( cols_p(6), &
                       hess(dof1,dof2,st2,st1,n) )
               enddo
               call sqlite3_finalize( stmt )
            enddo
         endif

      endif

! complete matrices (lower triangle stored)
      if (id .lt. 0) then
         do st1=1,nstates-1
            do st2=st1+1,nstates
               hess(1:ndof,1:ndof,st1,st2,1) = hess(1:ndof,1:ndof,st2,st1,1)
            enddo
         enddo
      else
         do n=id0,id
            do st1=1,nstates-1
               do st2=st1+1,nstates
                  hess(1:ndof,1:ndof,st1,st2,n) = hess(1:ndof,1:ndof,st2,st1,n)
               enddo
            enddo
         enddo
      endif

      deallocate (cols_p)

      end subroutine dbrdhes

!-----------------------------------------------------------------------

      !Read in adiabatic pes
      subroutine dbrdapes( db, nstates, id0, id, apes )
      implicit none
      type(SQLITE_DATABASE)                    :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: nstates   !number of states
      integer(long_sql), intent(in)                :: id0,id
      real(dop_sql), dimension(:,:,:)              :: apes
      integer(long_sql)                            :: n

      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      type(SQLITE_STATEMENT)                   :: stmt
      character(len=c1)                        :: whereclause
      logical(kind=4)                          :: finished
      integer(long_sql)                            :: st1, st2
      
      sql_vers = db%db_vers

      call sqlite3_query_table( db, 'apes', cols_p )

      if ((sql_vers .eq. 2) .or. (sql_vers .ge. 4)) then
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'apes', cols_p, stmt, &
                                        trim(whereclause))
            !All on one line
            call sqlite3_next_row(stmt, cols_p, finished)
            do st1 = 1, nstates
               call sqlite3_get_column( cols_p(1+st1), apes(st1, st1,1) )
            enddo
            call sqlite3_finalize( stmt )
         else
            call sqlite3_prepare_select(db, 'apes', cols_p, stmt)
            do n=id0,id
               !All on one line
               call sqlite3_next_row(stmt, cols_p, finished)
               do st1 = 1, nstates
                  call sqlite3_get_column( cols_p(1+st1), apes(st1, st1,n) )
               enddo
            enddo
            call sqlite3_finalize(stmt)
         endif

      else
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'apes', cols_p, stmt, &
                                        trim(whereclause))
            do
               call sqlite3_next_row(stmt, cols_p, finished)
               if (finished) exit
               call sqlite3_get_column( cols_p(2), st1 )
               call sqlite3_get_column( cols_p(3), st2 )
               call sqlite3_get_column( cols_p(4), apes(st2, st1,1) )
            enddo
            call sqlite3_finalize( stmt )
         else
            do n=id0,id
               write(whereclause, '(a,i0)') 'WHERE id=', n
               call sqlite3_prepare_select(db, 'apes', cols_p, stmt, &
                                           trim(whereclause))
               do
                  call sqlite3_next_row(stmt, cols_p, finished)
                  if (finished) exit
                  call sqlite3_get_column( cols_p(2), st1 )
                  call sqlite3_get_column( cols_p(3), st2 )
                  call sqlite3_get_column( cols_p(4), apes(st2, st1,n) )
               enddo
               call sqlite3_finalize( stmt )
            enddo
         endif
      endif

      deallocate (cols_p)

      end subroutine dbrdapes

!-----------------------------------------------------------------------

      subroutine dbrdagra( db, nstates, natoms, id0, id, agrad )
      implicit none
      type(SQLITE_DATABASE)                    :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: nstates   !number of states
      integer(long_sql), intent(in)                :: natoms !Number of atoms
      integer(long_sql), intent(in)                :: id0,id
      real(dop_sql), dimension(:,:,:,:)            :: agrad

      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      type(SQLITE_STATEMENT)                   :: stmt
      character(len=c1)                        :: whereclause
      logical(kind=4)                          :: finished
      integer(long_sql)                            :: st1, dof, index, n
      
      sql_vers = db%db_vers

      call sqlite3_query_table( db, 'agra', cols_p )

      if (sql_vers .ge. 4) then
         !All in one row
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'agra', cols_p, stmt, &
            trim(whereclause))
            do st1 = 1, nstates
               call sqlite3_next_row(stmt, cols_p, finished)
               !ndof = size(cols_p)
               index = 0
               do dof = 1, natoms*3
                  agrad(dof,st1,st1,1) = cols_p(3 + index)%double_value
                  index = index + 1
               enddo
            enddo
            call sqlite3_finalize( stmt )
         else 
            call sqlite3_prepare_select(db, 'agra', cols_p, stmt)
            do n=id0,id
               do st1 = 1, nstates
                  call sqlite3_next_row(stmt, cols_p, finished)
                  !ndof = size(cols_p)
                  index = 0
                  do dof = 1, natoms*3
                     agrad(dof,st1,st1,n) = cols_p(3 + index)%double_value
                     index = index + 1
                  enddo
               enddo
            enddo
            call sqlite3_finalize(stmt)
         endif

      else if (sql_vers .eq. 3) then
         !Have to loop based on id
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'agra', cols_p, stmt, &
            trim(whereclause))
            do
               call sqlite3_next_row(stmt, cols_p, finished)
               if (finished) exit
               call sqlite3_get_column( cols_p(2), st1 )
               call sqlite3_get_column( cols_p(3), dof )
               call sqlite3_get_column( cols_p(5), agrad(dof,st1,st1,1) )
            enddo
            call sqlite3_finalize( stmt )
         else
            do n=id0,id
               write(whereclause, '(a,i0)') 'WHERE id=', n
               call sqlite3_prepare_select(db, 'agra', cols_p, stmt, &
               trim(whereclause))
               do
                  call sqlite3_next_row(stmt, cols_p, finished)
                  if (finished) exit
                  call sqlite3_get_column( cols_p(2), st1 )
                  call sqlite3_get_column( cols_p(3), dof )
                  call sqlite3_get_column( cols_p(5), agrad(dof,st1,st1,n) )
               enddo
               call sqlite3_finalize( stmt )
            enddo
         endif

      else if (sql_vers .eq. 2) then
         !All in one row
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'agra', cols_p, stmt, &
            trim(whereclause))
            call sqlite3_next_row(stmt, cols_p, finished)
            !ndof = size(cols_p)
            index = 0
            do st1 = 1, nstates
               do dof = 1, natoms*3
                  agrad(dof,st1,st1,1) = cols_p(2 + index)%double_value
                  index = index + 1
               enddo
            enddo
            call sqlite3_finalize( stmt )
         else 
            call sqlite3_prepare_select(db, 'agra', cols_p, stmt)
            do n=id0,id
               call sqlite3_next_row(stmt, cols_p, finished)
               !ndof = size(cols_p)
               index = 0
               do st1 = 1, nstates
                  do dof = 1, natoms*3
                     agrad(dof,st1,st1,n) = cols_p(2 + index)%double_value
                     index = index + 1
                  enddo
               enddo
            enddo
            call sqlite3_finalize(stmt)
         endif

      else if (sql_vers .eq. 1) then
         !Have to loop based on id
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'agra', cols_p, stmt, &
            trim(whereclause))
            do
               call sqlite3_next_row(stmt, cols_p, finished)
               if (finished) exit
               call sqlite3_get_column( cols_p(2), st1 )
               call sqlite3_get_column( cols_p(3), dof )
               call sqlite3_get_column( cols_p(4), agrad(dof,st1,st1,1) )
            enddo
            call sqlite3_finalize( stmt )
         else
            do n=id0,id
               write(whereclause, '(a,i0)') 'WHERE id=', n
               call sqlite3_prepare_select(db, 'agra', cols_p, stmt, &
               trim(whereclause))
               do
                  call sqlite3_next_row(stmt, cols_p, finished)
                  if (finished) exit
                  call sqlite3_get_column( cols_p(2), st1 )
                  call sqlite3_get_column( cols_p(3), dof )
                  call sqlite3_get_column( cols_p(4), agrad(dof,st1,st1,n) )
               enddo
               call sqlite3_finalize( stmt )
            enddo
         endif

      endif

      deallocate (cols_p)

      end subroutine dbrdagra

!-----------------------------------------------------------------------

      subroutine dbrdahes( db,nstates, natoms, id0, id, ahess )
      implicit none
      type(SQLITE_DATABASE)                    :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: nstates   !number of states
      integer(long_sql), intent(in)                :: natoms !Number of atoms
      integer(long_sql), intent(in)                :: id0,id
      real(dop_sql), dimension(:,:,:,:,:)          :: ahess

      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      type(SQLITE_STATEMENT)                   :: stmt
      character(len=c1)                        :: whereclause, tabname
      logical(kind=4)                          :: finished
      integer(long_sql)                            :: st1, dof1, dof2, index,n,rc

      sql_vers = db%db_vers
      
      if (sql_vers .ge. 4) then
         call sqlite3_query_table(db, 'ahes', cols_p)
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'ahes', cols_p, stmt, &
                                    trim(whereclause))
            !Loop over states
            do st1 = 1,nstates
               !Loop over 1st coord
               do dof1 =1, natoms*3
                  call sqlite3_next_row(stmt, cols_p, finished)
                  !Loop over 2nd coord
                  index = 4
                  do dof2 =1, natoms*3
                     ahess(dof2,dof1,st1,st1,1) = &
                        cols_p(index)%double_value
                     index = index + 1
                  enddo
               enddo
            end do
            call sqlite3_finalize(stmt)
         else
            call sqlite3_prepare_select(db, 'ahes', cols_p, stmt)
            do n=id0,id
               do st1 = 1,nstates
                  !Loop over 1st coord
                  do dof1 =1, natoms*3
                     call sqlite3_next_row(stmt, cols_p, finished)
                     !Loop over 2nd coord
                     index = 4
                     do dof2 =1, natoms*3
                        ahess(dof2,dof1,st1,st1,n) = &
                           cols_p(index)%double_value
                        index = index + 1
                     enddo
                  enddo
               end do
            end do
            call sqlite3_finalize(stmt)
         end if

      else if (sql_vers .eq. 3) then
         if (id .lt. 0) then
            call sqlite3_query_table( db, 'ahes', cols_p )
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'ahes', cols_p, stmt, &
                                    trim(whereclause))
            do
               call sqlite3_next_row(stmt, cols_p, finished)
               if (finished) exit
               call sqlite3_get_column( cols_p(2), st1 )
               call sqlite3_get_column( cols_p(3), dof1 )
               call sqlite3_get_column( cols_p(4), dof2 )
               call sqlite3_get_column( cols_p(7), &
                        ahess(dof2,dof1,st1,st1,1) )
            enddo
            call sqlite3_finalize( stmt )
         else
            do n=id0,id
               call sqlite3_query_table( db, 'ahes', cols_p )
               write(whereclause, '(a,i0)') 'WHERE id=', n
               call sqlite3_prepare_select(db, 'ahes', cols_p, stmt, &
                                    trim(whereclause))
               do
                  call sqlite3_next_row(stmt, cols_p, finished)
                  if (finished) exit
                  call sqlite3_get_column( cols_p(2), st1 )
                  call sqlite3_get_column( cols_p(3), dof1 )
                  call sqlite3_get_column( cols_p(4), dof2 )
                  call sqlite3_get_column( cols_p(7), &
                           ahess(dof2,dof1,st1,st1,n) )
               enddo
               call sqlite3_finalize( stmt )
            enddo
         endif

      else if (sql_vers .eq. 2) then
         !In version 2 or greater each states has a unique Hess table
         !Need to loop over each state
         do st1=1, nstates
            !Get table name, is hes_st1 and query table
            write(tabname,'(A5,I0)') "ahes_", st1
            !Get table info to set up columns
            call sqlite3_query_table( db, trim(tabname), cols_p )

            if (id .lt. 0) then
               !All data is in one row
               write(whereclause, '(a,i0)') 'WHERE id=', id0
               call sqlite3_prepare_select(db, trim(tabname), cols_p, stmt, &
                                           trim(whereclause))
               call sqlite3_next_row(stmt, cols_p, finished)
               !keep track of position in cols_p
               index = 0
               !Loop over 1st coord
               do dof1 =1, natoms*3
                  !Loop over 2nd coord
                  do dof2 =1, natoms*3
                     ahess(dof1,dof2,st1,st1,1) = &
                                cols_p(2 + index)%double_value
                     index = index + 1
                  enddo
               enddo
               call sqlite3_finalize( stmt )
            else
               call sqlite3_prepare_select(db, trim(tabname), cols_p, stmt)
               call sqlite3_bind_all(db,stmt,cols_p(2:),rc)
               do n=id0,id
                  !All data is in one row
                  call sqlite3_next_row(stmt, cols_p, finished)
                  !keep track of position in cols_p
                  index = 0
                  !Loop over 1st coord
                  do dof1 =1, natoms*3
                     !Loop over 2nd coord
                     do dof2 =1, natoms*3
                        ahess(dof1,dof2,st1,st1,n) = &
                                   cols_p(2 + index)%double_value
                        index = index + 1
                     enddo
                  enddo
               enddo
               call sqlite3_finalize(stmt)
            endif
         enddo

      else if (sql_vers .eq. 1) then
         if (id .lt. 0) then
            call sqlite3_query_table( db, 'ahes', cols_p )
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'ahes', cols_p, stmt, &
                                    trim(whereclause))
            do
               call sqlite3_next_row(stmt, cols_p, finished)
               if (finished) exit
               call sqlite3_get_column( cols_p(2), st1 )
               call sqlite3_get_column( cols_p(3), dof1 )
               call sqlite3_get_column( cols_p(4), dof2 )
               call sqlite3_get_column( cols_p(5), &
                        ahess(dof2,dof1,st1,st1,1) )
            enddo
            call sqlite3_finalize( stmt )
         else
            do n=id0,id
               call sqlite3_query_table( db, 'ahes', cols_p )
               write(whereclause, '(a,i0)') 'WHERE id=', n
               call sqlite3_prepare_select(db, 'ahes', cols_p, stmt, &
                                    trim(whereclause))
               do
                  call sqlite3_next_row(stmt, cols_p, finished)
                  if (finished) exit
                  call sqlite3_get_column( cols_p(2), st1 )
                  call sqlite3_get_column( cols_p(3), dof1 )
                  call sqlite3_get_column( cols_p(4), dof2 )
                  call sqlite3_get_column( cols_p(5), &
                           ahess(dof2,dof1,st1,st1,n) )
               enddo
               call sqlite3_finalize( stmt )
            enddo
         endif

      endif

      deallocate (cols_p)

      end subroutine dbrdahes

!-----------------------------------------------------------------------

      subroutine dbrddip( db, id0, id, dip )
      implicit none
      type(SQLITE_DATABASE)                    :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: id0,id
      real(dop_sql), dimension(:,:,:,:)            :: dip

      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      type(SQLITE_STATEMENT)                   :: stmt
      character(len=c1)                        :: whereclause
      logical(kind=4)                          :: finished
      integer(long_sql)                            :: st1, st2,n
      character(len=c5)                        :: message
      character(len=c3)                        :: routine

      sql_vers = db%db_vers
      routine = "DBRDdip (db_sql.F90)"

      if (sql_vers .gt. 1) then
         write(message,'(2a)') 'dipole not implemented with SQL DB ver > 1'
         call dberrmsg( routine, message )
      endif

      call sqlite3_query_table( db, 'dip', cols_p )
      if (id .lt. 0) then
         write(whereclause, '(a,i0)') 'WHERE id=', id0
         call sqlite3_prepare_select(db, 'dip', cols_p, stmt, &
                                     trim(whereclause))
         do
            call sqlite3_next_row(stmt, cols_p, finished)
            if (finished) exit
            call sqlite3_get_column( cols_p(2), st1 )
            call sqlite3_get_column( cols_p(3), st2 )
            call sqlite3_get_column( cols_p(4), dip(1,st2,st1,1) )
            call sqlite3_get_column( cols_p(5), dip(2,st2,st1,1) )
            call sqlite3_get_column( cols_p(6), dip(3,st2,st1,1) )
         enddo
         call sqlite3_finalize( stmt )
      else
         do n=id0,id
            write(whereclause, '(a,i0)') 'WHERE id=', n
            call sqlite3_prepare_select(db, 'dip', cols_p, stmt, &
                                        trim(whereclause))
            do
               call sqlite3_next_row(stmt, cols_p, finished)
               if (finished) exit
               call sqlite3_get_column( cols_p(2), st1 )
               call sqlite3_get_column( cols_p(3), st2 )
               call sqlite3_get_column( cols_p(4), dip(1,st2,st1,n) )
               call sqlite3_get_column( cols_p(5), dip(2,st2,st1,n) )
               call sqlite3_get_column( cols_p(6), dip(3,st2,st1,n) )
            enddo
            call sqlite3_finalize( stmt )
         enddo
      endif

      deallocate (cols_p)

      end subroutine dbrddip

!-----------------------------------------------------------------------

      subroutine dbrdadip( db, id0, id, adip )
      implicit none
      type(SQLITE_DATABASE)                    :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: id0,id
      real(dop_sql), dimension(:,:,:,:)            :: adip

      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      type(SQLITE_STATEMENT)                   :: stmt
      character(len=c1)                        :: whereclause
      logical(kind=4)                          :: finished
      integer(long_sql)                            :: st1, st2, n
      character(len=c5)                        :: message
      character(len=c3)                        :: routine

      sql_vers = db%db_vers

      if (sql_vers .gt. 1) then
         write(message,'(2a)') 'dipole not implemented with SQL DB ver > 1'
         call dberrmsg( routine, message )
      endif

      call sqlite3_query_table( db, 'adip', cols_p )

      if (id .lt. 0) then
         write(whereclause, '(a,i0)') 'WHERE id=', id0
         call sqlite3_prepare_select(db, 'adip', cols_p, stmt, &
                                     trim(whereclause))
         do
            call sqlite3_next_row(stmt, cols_p, finished)
            if (finished) exit
            call sqlite3_get_column( cols_p(2), st1 )
            call sqlite3_get_column( cols_p(3), st2 )
            call sqlite3_get_column( cols_p(4), adip(1,st2,st1,1) )
            call sqlite3_get_column( cols_p(5), adip(2,st2,st1,1) )
            call sqlite3_get_column( cols_p(6), adip(3,st2,st1,1) )
         enddo
         call sqlite3_finalize( stmt )
      else
         do n=id0,id
            write(whereclause, '(a,i0)') 'WHERE id=', n
            call sqlite3_prepare_select(db, 'adip', cols_p, stmt, &
                                        trim(whereclause))
            do
               call sqlite3_next_row(stmt, cols_p, finished)
               if (finished) exit
               call sqlite3_get_column( cols_p(2), st1 )
               call sqlite3_get_column( cols_p(3), st2 )
               call sqlite3_get_column( cols_p(4), adip(1,st2,st1,n) )
               call sqlite3_get_column( cols_p(5), adip(2,st2,st1,n) )
               call sqlite3_get_column( cols_p(6), adip(3,st2,st1,n) )
            enddo
            call sqlite3_finalize( stmt )
         enddo
      endif

      deallocate (cols_p)

      end subroutine dbrdadip

!-----------------------------------------------------------------------

      subroutine dbrdmoco( db, nbasis, id0, id, moco )
      implicit none
      type(SQLITE_DATABASE)                    :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: nbasis   !number of basis functions
      integer(long_sql), intent(in)                :: id0,id
      real(sip_sql), dimension(:,:,:)              :: moco

      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      type(SQLITE_STATEMENT)                   :: stmt
      character(len=c1)                        :: whereclause, tabname
      logical(kind=4)                          :: finished
      integer(long_sql)                            :: mo1, mo2, n, rc, ao1

      sql_vers = db%db_vers

      if (sql_vers .ge. 4) then
         !One row per MO, one column per AO
         call sqlite3_query_table( db, 'moco', cols_p )
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'moco', cols_p, stmt, &
                                        trim(whereclause))
            do ao1=1,nbasis
               call sqlite3_next_row(stmt, cols_p, finished)
               moco(1:nbasis,ao1,1) = real(cols_p(3:nbasis+2)%double_value)
            enddo
            call sqlite3_finalize( stmt )
         else
            call sqlite3_prepare_select(db, 'moco', cols_p, stmt)
            do n=id0,id
               do mo1=1,nbasis
                  call sqlite3_next_row(stmt, cols_p, finished)
                  moco(1:nbasis,mo1,n) = real(cols_p(3:nbasis+2)%double_value)
               enddo
            end do
            call sqlite3_finalize(stmt)
         end if

      else if (sql_vers .eq. 2) then
         !Have a table per basis function
         do mo1 = 1, nbasis
            !Get table name
            write(tabname,'(A5,I0)') "moco_", mo1
            call sqlite3_query_table( db, trim(tabname), cols_p )

            !All data is in one row
            if (id .lt. 0) then
               write(whereclause, '(a,i0)') 'WHERE id=', id0
               call sqlite3_prepare_select(db, trim(tabname), cols_p, stmt, &
                                           trim(whereclause))
               call sqlite3_next_row(stmt, cols_p, finished)
               !Loop over other basis
               do mo2 =1, nbasis
                  call sqlite3_get_column( cols_p(1+mo2), moco(mo2,mo1,1) )
               enddo
               call sqlite3_finalize( stmt )
            else
               call sqlite3_prepare_select(db, trim(tabname), cols_p, stmt)
               call sqlite3_bind_all(db,stmt,cols_p(2:),rc)
               do n=id0,id
                  call sqlite3_next_row(stmt, cols_p, finished)
                  !Loop over other basis
                  ! do mo2 =1, nbasis
                  !    call sqlite3_get_column( cols_p(1+mo2), moco(mo2,mo1,n) )
                  ! enddo
                  !Try slicing - should improve
                  moco(1:nbasis,mo1,n) = real(cols_p(2:)%double_value)
               enddo
               call sqlite3_finalize(stmt)
            endif
         enddo

      else
         call sqlite3_query_table( db, 'moco', cols_p )
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'moco', cols_p, stmt, &
                                       trim(whereclause))
            do
               call sqlite3_next_row(stmt, cols_p, finished)
               if (finished) exit
               call sqlite3_get_column( cols_p(2), mo1 )
               call sqlite3_get_column( cols_p(3), mo2 )
               call sqlite3_get_column( cols_p(4), moco(mo2,mo1,1) )
            enddo
            call sqlite3_finalize( stmt )
         else
            do n=id0,id
               write(whereclause, '(a,i0)') 'WHERE id=', n
               call sqlite3_prepare_select(db, 'moco', cols_p, stmt, &
                                          trim(whereclause))
               do
                  call sqlite3_next_row(stmt, cols_p, finished)
                  if (finished) exit
                  call sqlite3_get_column( cols_p(2), mo1 )
                  call sqlite3_get_column( cols_p(3), mo2 )
                  call sqlite3_get_column( cols_p(4), moco(mo2,mo1,n) )
               enddo
               call sqlite3_finalize( stmt )
            enddo
         endif
      endif

      deallocate (cols_p)

      end subroutine dbrdmoco

!-----------------------------------------------------------------------

      subroutine dbrdnact( db, natoms, dercpdim, id0, id, nact )
      implicit none
      type(SQLITE_DATABASE)                    :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: id0,id
      integer(long_sql), intent(in)                :: natoms !Number of atoms
      integer(long_sql), intent(in)                :: dercpdim !Number of NACTs
      real(dop_sql), dimension(:,:,:)              :: nact

      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      type(SQLITE_STATEMENT)                   :: stmt
      character(len=c1)                        :: whereclause
      logical(kind=4)                          :: finished
      integer(long_sql)                            :: dof, c, n
      
      sql_vers = db%db_vers

      call sqlite3_query_table( db, 'nact', cols_p )

      if (sql_vers .ge. 4) then
         !All in one row
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'nact', cols_p, stmt, &
                                        trim(whereclause))
            do c=1,dercpdim
               call sqlite3_next_row(stmt, cols_p, finished)
               do dof = 1, natoms*3
                  nact(dof,c,1) = cols_p(4 + dof)%double_value
               enddo
            enddo
            call sqlite3_finalize( stmt )

         else
            call sqlite3_prepare_select(db, 'nact', cols_p, stmt)
            do n=id0,id
               do c=1,dercpdim
                  call sqlite3_next_row(stmt, cols_p, finished)
                  do dof = 1, natoms*3
                     nact(dof,c,n) = cols_p(4 + dof)%double_value
                  enddo
               enddo
            enddo
            call sqlite3_finalize(stmt)
         endif

      else if (sql_vers .eq. 3) then
         !In a row per dof
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'nact', cols_p, stmt, &
                                        trim(whereclause))
            do
               call sqlite3_next_row(stmt, cols_p, finished)
               if (finished) exit
               call sqlite3_get_column( cols_p(2), c )
               call sqlite3_get_column( cols_p(3), dof )
               call sqlite3_get_column( cols_p(7), nact(dof, c,1) )
            enddo
            call sqlite3_finalize( stmt )
         else
            do n=id0,id
               write(whereclause, '(a,i0)') 'WHERE id=', n
               call sqlite3_prepare_select(db, 'nact', cols_p, stmt, &
                                           trim(whereclause))
               do
                  call sqlite3_next_row(stmt, cols_p, finished)
                  if (finished) exit
                  call sqlite3_get_column( cols_p(2), c )
                  call sqlite3_get_column( cols_p(3), dof )
                  call sqlite3_get_column( cols_p(7), nact(dof, c,n) )
               enddo
               call sqlite3_finalize( stmt )
            enddo
         endif

      else if (sql_vers .eq. 2) then
         !All in one row
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'nact', cols_p, stmt, &
                                        trim(whereclause))
            do 
               call sqlite3_next_row(stmt, cols_p, finished)
               if (finished) exit
               !Get dercp value
               call sqlite3_get_column( cols_p(2), c )
               do dof=1,natoms*3
                  nact(dof,c,1) = cols_p(2+dof)%double_value
               enddo
            enddo
            call sqlite3_finalize( stmt )
         else
            do n=id0,id
               write(whereclause, '(a,i0)') 'WHERE id=', n
               call sqlite3_prepare_select(db, 'nact', cols_p, stmt)
               do 
                  call sqlite3_next_row(stmt, cols_p, finished)
                  if (finished) exit
                  !Get dercp value
                  call sqlite3_get_column( cols_p(2), c )
                  do dof=1,natoms*3
                     nact(dof,c,n) = cols_p(2+dof)%double_value
                  enddo
               enddo
               call sqlite3_reset(stmt)
            enddo
            call sqlite3_finalize(stmt)
         endif

      else if (sql_vers .eq. 1) then
         !In a row per dof
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'nact', cols_p, stmt, &
                                        trim(whereclause))
            do
               call sqlite3_next_row(stmt, cols_p, finished)
               if (finished) exit
               call sqlite3_get_column( cols_p(2), c )
               call sqlite3_get_column( cols_p(3), dof )
               call sqlite3_get_column( cols_p(4), nact(dof, c,1) )
            enddo
            call sqlite3_finalize( stmt )
         else
            do n=id0,id
               write(whereclause, '(a,i0)') 'WHERE id=', n
               call sqlite3_prepare_select(db, 'nact', cols_p, stmt, &
                                           trim(whereclause))
               do
                  call sqlite3_next_row(stmt, cols_p, finished)
                  if (finished) exit
                  call sqlite3_get_column( cols_p(2), c )
                  call sqlite3_get_column( cols_p(3), dof )
                  call sqlite3_get_column( cols_p(4), nact(dof, c,n) )
               enddo
               call sqlite3_finalize( stmt )
            enddo
         endif

      endif

      deallocate (cols_p)

      end subroutine dbrdnact

!-----------------------------------------------------------------------

      subroutine dbrdtrans( db, nstates, id0, id, diabtrans )
      implicit none
      type(SQLITE_DATABASE)                    ::db 
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: nstates   !number of states
      integer(long_sql), intent(in)                :: id0,id
      real(dop_sql), dimension(:,:,:)              :: diabtrans

      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      type(SQLITE_STATEMENT)                   :: stmt
      character(len=c1)                        :: whereclause
      logical(kind=4)                          :: finished
      integer(long_sql)                            :: st1, st2, index, n

      sql_vers = db%db_vers

      call sqlite3_query_table( db, 'trans', cols_p )

      if (sql_vers .eq. 3) then
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'trans', cols_p, stmt, &
                                        trim(whereclause))
            do
               call sqlite3_next_row(stmt, cols_p, finished)
               if (finished) exit
               call sqlite3_get_column( cols_p(2), st1 )
               call sqlite3_get_column( cols_p(3), st2 )
               call sqlite3_get_column( cols_p(4), diabtrans(st2,st1,1) )
            enddo
            call sqlite3_finalize( stmt )
         else
            do n=id0,id
               write(whereclause, '(a,i0)') 'WHERE id=', n
               call sqlite3_prepare_select(db, 'trans', cols_p, stmt, &
                                           trim(whereclause))
               do
                  call sqlite3_next_row(stmt, cols_p, finished)
                  if (finished) exit
                  call sqlite3_get_column( cols_p(2), st1 )
                  call sqlite3_get_column( cols_p(3), st2 )
                  call sqlite3_get_column( cols_p(4), diabtrans(st2,st1,n) )
               enddo
               call sqlite3_finalize( stmt )
            enddo
         endif

      else if ((sql_vers .eq. 2) .or. (sql_vers .ge. 4)) then
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'trans', cols_p, stmt, &
                                        trim(whereclause))
            call sqlite3_next_row(stmt, cols_p, finished)
            index = 0
            do st1 = 1, nstates
               do st2 =1, nstates   
                  call sqlite3_get_column( cols_p(2 + index),&
                       diabtrans(st2,st1,1) )
                  index = index + 1
               enddo
            enddo
            call sqlite3_finalize( stmt )
         else
            call sqlite3_prepare_select(db, 'trans', cols_p, stmt)
            do n=id0,id
               call sqlite3_next_row(stmt, cols_p, finished)
               index = 0
               do st1 = 1, nstates
                  do st2 =1, nstates   
                     call sqlite3_get_column( cols_p(2 + index),&
                          diabtrans(st2,st1,n) )
                     index = index + 1
                  enddo
               enddo
            enddo
            call sqlite3_finalize(stmt)
         endif

      else
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'trans', cols_p, stmt, &
                                        trim(whereclause))
            do
               call sqlite3_next_row(stmt, cols_p, finished)
               if (finished) exit
               call sqlite3_get_column( cols_p(2), st1 )
               call sqlite3_get_column( cols_p(3), st2 )
               !call sqlite3_get_column( cols_p(4), c )
               call sqlite3_get_column( cols_p(5), diabtrans(st2,st1,1) )
            enddo
            call sqlite3_finalize( stmt )
         else
            do n=id0,id
               write(whereclause, '(a,i0)') 'WHERE id=', n
               call sqlite3_prepare_select(db, 'trans', cols_p, stmt, &
                                           trim(whereclause))
               do
                  call sqlite3_next_row(stmt, cols_p, finished)
                  if (finished) exit
                  call sqlite3_get_column( cols_p(2), st1 )
                  call sqlite3_get_column( cols_p(3), st2 )
                  !call sqlite3_get_column( cols_p(4), c )
                  call sqlite3_get_column( cols_p(5), diabtrans(st2,st1,n) )
               enddo
               call sqlite3_finalize( stmt )
            enddo
         endif
      endif

      deallocate (cols_p)

      end subroutine dbrdtrans

!-----------------------------------------------------------------------

      subroutine dbrdctrans( db, nstates, id0, id, diabctrans )
      implicit none
      type(SQLITE_DATABASE)                    ::db 
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: nstates   !number of states
      integer(long_sql), intent(in)                :: id0,id
      real(dop_sql), dimension(:,:,:)              :: diabctrans

      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      type(SQLITE_STATEMENT)                   :: stmt
      character(len=c1)                        :: whereclause
      logical(kind=4)                          :: finished
      integer(long_sql)                            :: st1, st2, index, n

      sql_vers = db%db_vers

      call sqlite3_query_table( db, 'ctrans', cols_p )

      if (sql_vers .eq. 3) then
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'ctrans', cols_p, stmt, &
                                        trim(whereclause))
            do
               call sqlite3_next_row(stmt, cols_p, finished)
               if (finished) exit
               call sqlite3_get_column( cols_p(2), st1 )
               call sqlite3_get_column( cols_p(3), st2 )
               call sqlite3_get_column( cols_p(4), diabctrans(st2,st1,1) )
            enddo
            call sqlite3_finalize( stmt )
         else
            do n=id0,id
               write(whereclause, '(a,i0)') 'WHERE id=', n
               call sqlite3_prepare_select(db, 'ctrans', cols_p, stmt, &
                                           trim(whereclause))
               do
                  call sqlite3_next_row(stmt, cols_p, finished)
                  if (finished) exit
                  call sqlite3_get_column( cols_p(2), st1 )
                  call sqlite3_get_column( cols_p(3), st2 )
                  call sqlite3_get_column( cols_p(4), diabctrans(st2,st1,n) )
               enddo
               call sqlite3_finalize( stmt )
            enddo
         endif

      else if ((sql_vers .eq. 2) .or. (sql_vers .ge. 4)) then
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'ctrans', cols_p, stmt, &
                                        trim(whereclause))
            call sqlite3_next_row(stmt, cols_p, finished)
            index = 0
            do st1 = 1, nstates
               do st2 =1, nstates   
                  call sqlite3_get_column( cols_p(2 + index),&
                       diabctrans(st2,st1,1) )
                  index = index + 1
               enddo
            enddo
            call sqlite3_finalize( stmt )
         else
            call sqlite3_prepare_select(db, 'ctrans', cols_p, stmt)
            do n=id0,id
               call sqlite3_next_row(stmt, cols_p, finished)
               index = 0
               do st1 = 1, nstates
                  do st2 =1, nstates   
                     call sqlite3_get_column( cols_p(2 + index),&
                          diabctrans(st2,st1,n) )
                     index = index + 1
                  enddo
               enddo
            enddo
            call sqlite3_finalize(stmt)
         endif

      else
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'ctrans', cols_p, stmt, &
                                        trim(whereclause))
            do
               call sqlite3_next_row(stmt, cols_p, finished)
               if (finished) exit
               call sqlite3_get_column( cols_p(2), st1 )
               call sqlite3_get_column( cols_p(3), st2 )
               !call sqlite3_get_column( cols_p(4), c )
               call sqlite3_get_column( cols_p(5), diabctrans(st2,st1,1) )
            enddo
            call sqlite3_finalize( stmt )
         else
            do n=id0,id
               write(whereclause, '(a,i0)') 'WHERE id=', n
               call sqlite3_prepare_select(db, 'ctrans', cols_p, stmt, &
                                           trim(whereclause))
               do
                  call sqlite3_next_row(stmt, cols_p, finished)
                  if (finished) exit
                  call sqlite3_get_column( cols_p(2), st1 )
                  call sqlite3_get_column( cols_p(3), st2 )
                  !call sqlite3_get_column( cols_p(4), c )
                  call sqlite3_get_column( cols_p(5), diabctrans(st2,st1,n) )
               enddo
               call sqlite3_finalize( stmt )
            enddo
         endif
      endif

      deallocate (cols_p)

      end subroutine dbrdctrans

!-----------------------------------------------------------------------

      subroutine dbrdnm( db, nmode, id0, id, nm )
      implicit none
      type(SQLITE_DATABASE)                    :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: id0,id, nmode
      real(dop_sql), dimension(:,:)                :: nm

      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      type(SQLITE_STATEMENT)                   :: stmt
      character(len=c1)                        :: whereclause
      logical(kind=4)                          :: finished
      integer(long_sql)                            :: n,f,index

      sql_vers = db%db_vers

      call sqlite3_query_table( db, 'nmode', cols_p )

      if ((sql_vers .eq. 2) .or. (sql_vers .ge. 4)) then
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'nmode', cols_p, stmt, &
                                        trim(whereclause))
            call sqlite3_next_row(stmt, cols_p, finished)
            index = 0
            do f = 1,nmode
               call sqlite3_get_column( cols_p(2+index), nm(f,1) )
               index = index + 1
            enddo
            call sqlite3_finalize( stmt )
         else
            call sqlite3_prepare_select(db, 'nmode', cols_p, stmt)
            do n=id0,id
               call sqlite3_next_row(stmt, cols_p, finished)
               index = 0
               do f = 1,nmode
                  call sqlite3_get_column( cols_p(2+index), nm(f,n) )
                  index = index + 1
               enddo
            enddo 
            call sqlite3_finalize(stmt)
         endif

      else
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'nmode', cols_p, stmt, &
                                        trim(whereclause))
            do
               call sqlite3_next_row(stmt, cols_p, finished)
               if (finished) exit
               call sqlite3_get_column( cols_p(2), f )
               call sqlite3_get_column( cols_p(3), nm(f,1) )
            enddo
            call sqlite3_finalize( stmt )
         else
            do n=id0,id
               write(whereclause, '(a,i0)') 'WHERE id=', n
               call sqlite3_prepare_select(db, 'nmode', cols_p, stmt, &
                                           trim(whereclause))
               do
                  call sqlite3_next_row(stmt, cols_p, finished)
                  if (finished) exit
                  call sqlite3_get_column( cols_p(2), f )
                  call sqlite3_get_column( cols_p(3), nm(f,n) )
               enddo
               call sqlite3_finalize( stmt )
            enddo
         endif
      endif 

      deallocate (cols_p)

      end subroutine dbrdnm

!-----------------------------------------------------------------------

      subroutine dbrdcasoc( db, nstates, id0, id, asoc )
      implicit none
      type(SQLITE_DATABASE)                    :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: nstates   !number of states
      integer(long_sql), intent(in)                :: id0,id
      complex(dop_sql), dimension(:,:,:)           :: asoc
      real(dop_sql), dimension(nstates*nstates)    :: dsoc
      real(dop_sql), dimension(nstates*nstates)    :: zsoc

      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      type(SQLITE_STATEMENT)                   :: stmt
      character(len=c1)                        :: whereclause
      logical(kind=4)                          :: finished
      integer(long_sql)                            :: st1,st2,n,socdim,index
      real(dop_sql)                                :: realval,complexval

      sql_vers = db%db_vers
      socdim = nstates*(nstates-1)/2
      asoc(:,:,:) = 0.0

      if (sql_vers .ge. 4) then
         call sqlite3_query_table( db, 'casoc', cols_p )

         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'casoc', cols_p, stmt, &
                                        trim(whereclause))
            call sqlite3_next_row(stmt, cols_p, finished)  ! get real part of SOC
            do index = 1,socdim
               call sqlite3_get_column( cols_p(index+2), dsoc(index))
            enddo
            call sqlite3_next_row(stmt, cols_p, finished) ! get imag part of SOC
            do index = 1,socdim
               call sqlite3_get_column( cols_p(index+2), zsoc(index))
            enddo

! fill lower triangle
            index = 1
            do st1 = 1, nstates-1
               do st2 = st1+1, nstates
                  asoc(st2,st1,1) = cmplx(dsoc(index),zsoc(index),dop_sql)
                  asoc(st1,st2,1) = dconjg(asoc(st2,st1,1))
                  index = index+1
               enddo
            enddo

            call sqlite3_finalize( stmt )

         else
            call sqlite3_prepare_select(db, 'casoc', cols_p, stmt)
            do n=id0,id
               call sqlite3_next_row(stmt, cols_p, finished)  ! get real part of SOC
               do index = 1,socdim
                  call sqlite3_get_column( cols_p(index+2), dsoc(index))
               enddo
               call sqlite3_next_row(stmt, cols_p, finished) ! get imag part of SOC
               do index = 1,socdim
                  call sqlite3_get_column( cols_p(index+2), zsoc(index))
               enddo

               index = 1
               do st1 = 1, nstates-1
                  do st2 = st1+1, nstates
                     asoc(st2,st1,n) = cmplx(dsoc(index),zsoc(index),dop_sql)
                     asoc(st1,st2,n) = dconjg(asoc(st2,st1,n))
                     index = index+1
                  enddo
               enddo

            enddo
            call sqlite3_finalize( stmt )
         endif

      else
         call sqlite3_query_table( db, 'casoc', cols_p )

         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'casoc', cols_p, stmt, &
                                        trim(whereclause))
            do st2 = 2,nstates   ! upper triangle sequentially
               do st1 = 1,st2-1
!                   c = (st2-1)*(st2-2)/2+st1   ! old ASOC index. Not used.
                  call sqlite3_next_row(stmt, cols_p, finished)
                  call sqlite3_get_column( cols_p(3), realval )
                  call sqlite3_get_column( cols_p(4), complexval )
                  asoc(st1,st2,1) = cmplx(realval, complexval, dop_sql)
                  asoc(st2,st1,1) = dconjg(asoc(st1,st2,1))
               enddo
            enddo
            call sqlite3_finalize( stmt )

         else
            do n=id0,id
               write(whereclause, '(a,i0)') 'WHERE id=', n
               call sqlite3_prepare_select(db, 'casoc', cols_p, stmt, &
                                           trim(whereclause))
               do st2 = 2,nstates   ! upper triangle sequentially
                  do st1 = 1,st2-1
!                      c = (st2-1)*(st2-2)/2+st1   ! old ASOC index. Not used.
                     call sqlite3_next_row(stmt, cols_p, finished)
                     call sqlite3_get_column( cols_p(3), realval )
                     call sqlite3_get_column( cols_p(4), complexval )
                     asoc(st1,st2,1) = cmplx(realval, complexval, dop_sql)
                     asoc(st2,st1,1) = dconjg(asoc(st1,st2,1))
                  enddo
               enddo
               call sqlite3_finalize( stmt )
            enddo
         endif

      endif

      deallocate (cols_p)

      end subroutine dbrdcasoc

!-----------------------------------------------------------------------

      subroutine dbrdcivec( db, nsmult, nsta, id0, id, ciconfig, cicoeff )

      implicit none

      type(SQLITE_DATABASE)                        :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: nsmult
      integer(long_sql), dimension(nsmult), intent(in) :: nsta
      integer(long_sql), intent(in)                :: id0,id
      integer(long_sql)                            :: n,index,i,j,k
      integer(long_sql), dimension(nsmult)         :: nconf
      character(len=*), dimension(:,:,:)           :: ciconfig
      real(dop_sql), dimension(:,:,:,:)            :: cicoeff

      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      type(SQLITE_STATEMENT)                   :: stmt
      character(len=c1)                        :: whereclause
      logical(kind=4)                          :: finished
      character(len=c5)                        :: message
      character(len=c3)                        :: routine
 
       sql_vers = db%db_vers
 
       if (sql_vers .ge. 4) then

! First get no. of configs for each spin block
         if (id .ne. 0) then
            call sqlite3_query_table( db, 'civec_dat', cols_p )
            call sqlite3_prepare_select(db, 'civec_dat', cols_p, stmt)
            do i = 1, nsmult
               call sqlite3_next_row(stmt, cols_p, finished)
               call sqlite3_get_column( cols_p(3), j )
               call sqlite3_get_column( cols_p(4), nconf(i) )
            enddo
            call sqlite3_finalize(stmt)
         endif

! Now read CIVecs
         call sqlite3_query_table( db, 'civec', cols_p )
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'civec', cols_p, stmt, &
                                        trim(whereclause))
            do i = 1,nsmult
               do j = 1,nconf(i)
                  call sqlite3_next_row(stmt, cols_p, finished)

                  call sqlite3_get_column( cols_p(4), ciconfig(j,i,1) )
                  index=4
                  do k = 1,nsta(i)
                     index = index+1
                     call sqlite3_get_column( cols_p(index), cicoeff(j,k,i,1) )
                  enddo
               enddo
            enddo
            call sqlite3_finalize( stmt )
         else
            call sqlite3_prepare_select(db, 'civec', cols_p, stmt)
            do n=id0,id
               do i = 1,nsmult
                  do j = 1,nconf(i)
                     call sqlite3_next_row(stmt, cols_p, finished)

                     call sqlite3_get_column( cols_p(4), ciconfig(j,i,n) )
                     index=4
                     do k = 1,nsta(i)
                        index = index+1
                        call sqlite3_get_column( cols_p(index), cicoeff(j,k,i,n) )
                     enddo
                  enddo
               enddo
            enddo
            call sqlite3_finalize(stmt)
         endif

      else
         routine = 'DBrdcivec (db_sql.f90)'
         write(message,'(a)') &
              'CIVec table cannot be read for DB version <4'
         call dberrmsg( routine, message )
      endif

      deallocate (cols_p)

      end subroutine dbrdcivec

!-----------------------------------------------------------------------

      subroutine dbrdsymrep( db, nop, id0, id, rep )
      implicit none
      type(SQLITE_DATABASE)                    :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: id0,id
      integer(long_sql), intent(in)                :: nop   ! No. symmetry operations
      integer(long_sql), dimension(:,:), intent(inout) :: rep

      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      type(SQLITE_STATEMENT)                   :: stmt
      character(len=c1)                        :: whereclause
      logical(kind=4)                          :: finished
      integer(long_sql)                            :: nr,n
      
      sql_vers = db%db_vers
      
      call sqlite3_query_table( db, 'symrep', cols_p )

      if ((sql_vers .eq. 2) .or. (sql_vers .ge. 4)) then
         !All in a row
         !Slice out the ID which is first column
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'symrep', cols_p, stmt, &
                          trim(whereclause))
            call sqlite3_next_row(stmt, cols_p, finished)
            rep(1:nop,1) = cols_p(2:nop+1)%int_value
            call sqlite3_finalize( stmt )
         else
            call sqlite3_prepare_select(db, 'symrep', cols_p, stmt)
            do n=id0,id
               call sqlite3_next_row(stmt, cols_p, finished)
               rep(1:nop,n) = cols_p(2:nop+1)%int_value
            enddo
            call sqlite3_finalize( stmt )
         endif

      else
         !Entry per sym op per dbentry
         if (id .lt. 0) then
            write(whereclause, '(a,i0)') 'WHERE id=', id0
            call sqlite3_prepare_select(db, 'symrep', cols_p, stmt, &
                       trim(whereclause))
            do
               call sqlite3_next_row(stmt, cols_p, finished)
               if (finished) exit
               call sqlite3_get_column( cols_p(2), nr )
               call sqlite3_get_column( cols_p(3), rep(nr,1) )
            enddo
            call sqlite3_finalize( stmt )
         else
            do n=id0,id
               write(whereclause, '(a,i0)') 'WHERE id=', n
               call sqlite3_prepare_select(db, 'symrep', cols_p, stmt, &
                          trim(whereclause))
               do
                  call sqlite3_next_row(stmt, cols_p, finished)
                  if (finished) exit
                  call sqlite3_get_column( cols_p(2), nr )
                  call sqlite3_get_column( cols_p(3), rep(nr,n) )
               enddo
               call sqlite3_finalize( stmt )
            enddo
         endif
      endif

      deallocate (cols_p)

      end subroutine dbrdsymrep

!#######################################################################
!
! Following routines are for reading the REFDB parts of the DB
!
!#######################################################################

      logical(kind=4) function dbhasrefdb(db)
      implicit none
      type(SQLITE_DATABASE)                    :: db

      type(SQLITE_STATEMENT)                   :: stmt
      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      logical(kind=4)                          :: finished

      call sqlite3_query_table(db, 'refdb', cols_p)
      call sqlite3_prepare_select(db, 'refdb', cols_p, stmt)

      finished = .false.
      call sqlite3_next_row(stmt, cols_p, finished)
      deallocate(cols_p)
      dbhasrefdb = .not.finished
      call sqlite3_finalize( stmt )

      end function dbhasrefdb

!-----------------------------------------------------------------------
!Subroutine to read in the reference infomation from refdb
!
! Earlier DBs had a logicals at positions 11,12 that were not related to
! DB info so were removed (lddsoc,lsocbas). Hence reading of nsocstate 
! and multiplicity data (nddmult) only if >13 entries (position 13 was 
! nsocstates that is now at 11).
!-----------------------------------------------------------------------

      subroutine dbrdrefdb(db,natm, nroot, nddstate, nbasis, dde0, dimref, &
                 nfam, dercpdim,dbpntgp,nsymop,nsocstate,nddmult)
      implicit none

      type(SQLITE_DATABASE)                    :: db
      integer(long_sql)                            :: sql_vers
      integer(long_sql)                            :: i
      integer(long_sql), intent(out)               :: natm,nroot,nddstate,nbasis
      integer(long_sql), dimension(7), intent(out) :: nddmult
      integer(long_sql), intent(out)               :: nsocstate
      integer(long_sql), intent(out)               :: dbpntgp,nsymop
      real(kind=dop_sql), intent(out)              :: dde0
      integer(long_sql), intent(out)               :: dimref,nfam
      integer(long_sql), intent(out)               :: dercpdim

      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      type(SQLITE_STATEMENT)                   :: stmt
      logical(kind=4)                          :: finished
      character(len=c5)                        :: message
      character(len=c3)                        :: routine

      finished = .false.
      routine = "DBRDrefdb (db_sql.F90)"
      sql_vers = db%db_vers
      if (.not.db%db_open) then
         write(message,'(2a)') 'using a non-open SQL DB', &
                               db%errmsg
         call dberrmsg( routine, message )
      endif
      call sqlite3_query_table(db, 'refdb', cols_p)
      call sqlite3_prepare_select(db, 'refdb', cols_p, stmt)
      call sqlite3_next_row(stmt, cols_p, finished)

      if (db%error.ne.SQLITE_OK) then
         write(message,'(2a)') 'sql error while reading refdb: ', &
                               db%errmsg
         call dberrmsg( routine, message )
      endif
      if (finished) then
         write(message,'(2a)') 'dbrdrefdb: No refdb data in SQL DB'
         call dberrmsg( routine, message )
      endif

      ! Values that are not present will be set to 0 or 0.0
      call sqlite3_get_column(cols_p(1), natm)
      call sqlite3_get_column(cols_p(2), nroot)
      call sqlite3_get_column(cols_p(3), nddstate)
      call sqlite3_get_column(cols_p(4), nbasis)
      call sqlite3_get_column(cols_p(5), dde0)
      call sqlite3_get_column(cols_p(6), dimref)
      call sqlite3_get_column(cols_p(7), nfam)
      call sqlite3_get_column(cols_p(8), dercpdim)
      call sqlite3_get_column(cols_p(9), dbpntgp)
      call sqlite3_get_column(cols_p(10), nsymop)
      if (size(cols_p) .gt. 13) then   
         call sqlite3_get_column(cols_p(11), nsocstate)
         do i=1,7
            call sqlite3_get_column(cols_p(11+i), nddmult(i))
         enddo
      else
         nsocstate = nddstate
         nddmult(1) = nddstate
         nddmult(2:7) = 0
      endif

      call sqlite3_finalize( stmt )
      deallocate(cols_p)

      end subroutine dbrdrefdb

!-----------------------------------------------------------------------
!Subroutine to read out the reference geometry
!Different with different sql versions

      subroutine dbrdrefdbgeom(db, natm, mref, atnam, xref)
      implicit none

      type(SQLITE_DATABASE), intent(out)       :: db  !Database object
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(out)               :: natm   !Number of atoms (why out?)
      real(kind=dop_sql), dimension(:), intent(out), &
         allocatable, optional                 :: mref   !Mass of atoms
      character(len=c1), dimension(:), intent(out), & 
         allocatable, optional                 ::atnam   !Number of atom
      real(kind=dop_sql), dimension(:,:), intent(out), &  
         allocatable, optional                 :: xref   !Coordinate reference

      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p !To read back from database
      type(SQLITE_STATEMENT)                   :: stmt   !Statement to send to database
      logical(kind=4)                          :: finished  !Check database finished
      character(len=c5)                        :: message
      character(len=c3)                        :: routine
      integer(long_sql)                            :: atno,iatm
      character(len=c1)                        :: tmpnam

      finished = .false.
      routine = "DBRDrefdbgeom (db_sql.F90)"
      !I already know this information from refdb - why not pass over?
      sql_vers = db%db_vers
      call sqlite3_query_table(db, 'refdb', cols_p)
      call sqlite3_prepare_select(db, 'refdb', cols_p, stmt)
      call sqlite3_next_row(stmt, cols_p, finished)
      if (db%error.ne.SQLITE_OK) then
         write(message,'(2a)') 'sql error while reading refdb: ', &
                               db%errmsg
         call dberrmsg( routine, message )
      endif
      if (finished) then
         write(message,'(2a)') 'No refdb data in SQL DB'
         call dberrmsg( routine, message )
      endif
      call sqlite3_get_column(cols_p(1), natm)
      call sqlite3_finalize( stmt )
      deallocate(cols_p)

      if (.not. present(mref) .and..not. present(atnam) &
         .and..not. present(xref)) return !Not ready to receive data
      !Now get the reference geometry in cart
      call sqlite3_query_table(db, 'refdbrefgeom', cols_p)
      call sqlite3_prepare_select(db, 'refdbrefgeom', cols_p, stmt)
      if (present(mref)) then
         if ( allocated(mref) ) then
            deallocate(mref)
         endif
         allocate( mref(natm) )
         do atno = 1, natm
            call sqlite3_next_row(stmt, cols_p, finished)
            if (db%error.ne.SQLITE_OK) then
               write(message,'(2a)') 'sql error while reading refdb masses: ', &
                                     db%errmsg
               call dberrmsg( routine, message )
            endif
            if (finished.and.atno.lt.natm) then
               write(message,'(2a)') 'refdb data in SQL DB has too&
     & few atom masses'
               call dberrmsg( routine, message )
            endif
            call sqlite3_get_column(cols_p(1), iatm)
            call sqlite3_get_column(cols_p(2), mref(iatm))
         enddo
         call sqlite3_reset( stmt )
      endif
      if (present(atnam)) then
         if ( allocated(atnam) ) then
            deallocate(atnam)
         endif
         allocate( atnam(natm) )
         do atno = 1, natm
            call sqlite3_next_row(stmt, cols_p, finished)
            if (db%error.ne.SQLITE_OK) then
               write(message,'(2a)') 'sql error while reading refdb names: ', &
                                     db%errmsg
               call dberrmsg( routine, message )
            endif
            if (finished.and.atno.lt.natm) then
               write(message,'(2a)') 'refdb data in SQL DB has too&
     & few atom names'
               call dberrmsg( routine, message )
            endif
            call sqlite3_get_column(cols_p(1), iatm)
            call sqlite3_get_column(cols_p(3), tmpnam)
            atnam(iatm) = tmpnam
         enddo
         call sqlite3_reset( stmt )
      endif
      if (present(xref)) then
         if ( allocated(xref) ) then
            deallocate(xref)
         endif
         allocate( xref(3,natm) )
         do atno = 1, natm
            call sqlite3_next_row(stmt, cols_p, finished)
            if (db%error.ne.SQLITE_OK) then
               write(message,'(2a)') 'sql error while reading refdb geom: ', &
                                     db%errmsg
               call dberrmsg( routine, message )
            endif
            if (finished.and.atno.lt.natm) then
               write(message,'(2a)') 'refdb data in SQL DB has too&
     & few atom coords'
               call dberrmsg( routine, message )
            endif
            call sqlite3_get_column(cols_p(1), iatm)
            call sqlite3_get_column(cols_p(4), xref(1,iatm))
            call sqlite3_get_column(cols_p(5), xref(2,iatm))
            call sqlite3_get_column(cols_p(6), xref(3,iatm))
         enddo
      endif

      deallocate(cols_p)
      
      call sqlite3_finalize( stmt )

      end subroutine dbrdrefdbgeom

!-----------------------------------------------------------------------

      subroutine dbrdrefdbmaps(db,natm,nsymop,dbsymmap)
      implicit none

      type(SQLITE_DATABASE), intent(out)       :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(inout)               :: natm
      integer(long_sql), intent(inout)               :: nsymop
      integer(long_sql), dimension(:,:), intent(inout), &
         allocatable                           :: dbsymmap

      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      type(SQLITE_STATEMENT)                   :: stmt
      logical(kind=4)                          :: finished
      character(len=c5)                        :: message
      character(len=c3)                        :: routine
      integer(long_sql)                            :: nsym,iatm,n
  
      sql_vers = db%db_vers
      routine = "DBRDrefdbmaps (db_sql.F90)"

      if (nsymop .le. 1) return    ! Nothing to read

      if ( allocated(dbsymmap) ) then
         deallocate(dbsymmap)
      endif
      allocate( dbsymmap(natm,nsymop) )

      call sqlite3_query_table(db, 'refdbsymmaps', cols_p)

      if (sql_vers .eq. 2) then
         !New method. There is a row per symop
         call sqlite3_prepare_select(db, 'refdbsymmaps', cols_p, stmt)
         do n = 1,nsymop
            call sqlite3_next_row(stmt, cols_p, finished)
            dbsymmap(:,n) = cols_p(2:)%int_value
         enddo

      else

         call sqlite3_prepare_select(db, 'refdbsymmaps', cols_p, stmt)
         do n = 1,nsymop*natm
            call sqlite3_next_row(stmt, cols_p, finished)
            if (db%error.ne.SQLITE_OK) then
               write(message,'(2a)') 'sql error while reading refdb names: ', &
                                    db%errmsg
               call dberrmsg( routine, message )
            endif
            if (finished.and.n.lt.natm*nsymop) then
               write(message,'(2a)') 'refdb data in SQL DB has too&
                         & few elements in symmetry maps'
               call dberrmsg( routine, message )
            endif
            call sqlite3_get_column(cols_p(1), nsym)
            call sqlite3_get_column(cols_p(2), iatm)
            call sqlite3_get_column(cols_p(3), dbsymmap(iatm,nsym))
         enddo
         call sqlite3_reset( stmt )
      endif
      deallocate(cols_p)
      
      call sqlite3_finalize( stmt )

      end subroutine dbrdrefdbmaps

!-----------------------------------------------------------------------

      !Read in info table
      !Tells you what the calculations were done
      subroutine dbrdinfo(db, method, basis, prog )
         type(SQLITE_DATABASE), intent(inout)        :: db
         integer(long_sql)                            :: sql_vers  !SQL database version
         character(len=c5), intent(out)           :: method, basis, prog

         logical(kind=4)                          :: finished
         type(SQLITE_STATEMENT)                   :: stmt
         type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
         !Reads in the info file
         sql_vers = db%db_vers
         if (sql_vers .gt. 1) then
            call sqlite3_query_table(db, 'info', cols_p)
            call sqlite3_prepare_select(db, 'info', cols_p, stmt)
            call sqlite3_next_row(stmt, cols_p, finished)
            call sqlite3_get_column(cols_p(1), method)
            call sqlite3_get_column(cols_p(2), basis)
            call sqlite3_get_column(cols_p(3), prog)
         elseif (sql_vers .eq. 1) then
            !Do nothing doesn't exist in version 1
         endif
      
         
      end subroutine dbrdinfo

!-----------------------------------------------------------------------
! DBRDCIDAT
! nsmult: no. of multiplicity blocks
! imult(i): multiplicity of block i
! nsta(i) : no. states in multiplicity i
! nconf(i) : no. CI configurations in multiplicity i
!-----------------------------------------------------------------------

      subroutine dbrdcidat( db, nsmult, imult, nsta, nconf)

      implicit none

      type(SQLITE_DATABASE)                        :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: nsmult
      integer(long_sql)                            :: ns
      integer(long_sql), dimension(nsmult), intent(out) :: imult
      integer(long_sql), dimension(nsmult), intent(out) :: nsta
      integer(long_sql), dimension(nsmult), intent(out) :: nconf

      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      type(SQLITE_STATEMENT)                   :: stmt
      logical(kind=4)                          :: finished
      character(len=c5)                        :: message
      character(len=c3)                        :: routine
 
      sql_vers = db%db_vers
      routine = "DBRDcidat (db_sql.F90)"
 
       if (sql_vers .ge. 4) then

! get no. of configs for each spin block
         call sqlite3_query_table( db, 'civec_dat', cols_p )
         call sqlite3_prepare_select(db, 'civec_dat', cols_p, stmt)
         do ns = 1, nsmult
            call sqlite3_next_row(stmt, cols_p, finished)
            call sqlite3_get_column( cols_p(2), imult(ns) )
            call sqlite3_get_column( cols_p(3), nsta(ns) )
            call sqlite3_get_column( cols_p(4), nconf(ns) )
         enddo
         call sqlite3_finalize(stmt)

      else
         routine = 'DBrdcidat (db_sql.f90)'
         write(message,'(a)') &
              'CIdat table cannot be read for DB version <4'
         call dberrmsg( routine, message )
      endif

      deallocate (cols_p)

      end subroutine dbrdcidat

!-----------------------------------------------------------------------
! DBRDTEMPLATE
!
! reads QC template and writes it to the file on iunit
!-----------------------------------------------------------------------
      subroutine dbrdtemplate(db, iunit)
         type(SQLITE_DATABASE), intent(inout)     :: db
         integer, intent(in)                      :: iunit
         character(len=500)                       :: temp
         integer(long_sql)                        :: sql_vers  !SQL database version

         type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
         type(SQLITE_STATEMENT)                   :: stmt
         logical(kind=4)                          :: finished


         sql_vers = db%db_vers
         call sqlite3_query_table(db,'template',cols_p)
         !only in versions above 1
         if (sql_vers .gt. 1) then
            !Read in as string
            !Set up the columns
            !Set up select statement
            call sqlite3_prepare_select(db, 'template', cols_p, stmt)
            do
               !read each row
               call sqlite3_next_row(stmt, cols_p, finished)
               if (finished) exit
               !Put into string
               call sqlite3_get_column(cols_p(1), temp)

               write(iunit,'(a)') trim(temp)
            enddo
            !End DB transaction
            call sqlite3_finalize(stmt)
         endif         
         deallocate(cols_p)

      end subroutine dbrdtemplate

!-----------------------------------------------------------------------
! DBRDQCFREQ
!
! reads QC frequency file and writes it to the file on iunit
!-----------------------------------------------------------------------

      subroutine dbrdqcfreq(db, iunit)
         type(SQLITE_DATABASE), intent(inout)     :: db
         integer, intent(in)                      :: iunit
         integer                                  :: ilbl,jlbl
         character(len=500)                       :: temp
         integer(long_sql)                        :: sql_vers  !SQL database version

         type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
         type(SQLITE_STATEMENT)                   :: stmt
         logical(kind=4)                          :: finished


         sql_vers = db%db_vers
         !only in versions above 1
         if (sql_vers .gt. 1) then
            !Read in as string
            !Set up the columns
            call sqlite3_query_table(db,'qcfreq',cols_p)
            !Set up select statement
            call sqlite3_prepare_select(db, 'qcfreq', cols_p, stmt)
            do
               !Get results (should be only 1?)
               call sqlite3_next_row(stmt, cols_p, finished)
               if (finished) exit

               !Put into string
               call sqlite3_get_column(cols_p(1), temp)

               !Look for @'' and replace with '
               do
                  ilbl = index(temp,"@''")
                  if (ilbl .eq. 0) exit
                  jlbl = len(temp)
                  temp = temp(1:ilbl-1)//temp(ilbl+2:jlbl)
               enddo

               write(iunit,'(a)') trim(temp)
            enddo
            !End DB transaction
            call sqlite3_finalize(stmt)
         endif         
         deallocate(cols_p)

      end subroutine dbrdqcfreq

!#######################################################################
!
! Routines to OPEN / CLOSE DB and start / stop transactions
!
! modus : 0 open read only
!         1 open read - write
!         2 open read - write and initialise (create)
!
!#######################################################################

      subroutine dbopen(fname,db,sql_vers,quantics_vers,infoarray, &
                 flagarray, mode)

      implicit none

      character(len=c5), intent(in)            :: fname
      character(len=c5+12)                     :: fnamemode
      type(SQLITE_DATABASE), intent(out)       :: db
      integer(long_sql), intent(inout)         :: sql_vers  !SQL db version
      real(dop_sql), intent(in)                :: quantics_vers
      integer(long_sql), intent(in)            :: mode
      !NEWSQL
      integer(long_sql), dimension(:), intent(in) :: infoarray
      logical(kind=4), dimension(:), intent(in)   :: flagarray


      real(dop_sql)                            :: quantics_vers0
      logical(kind=4)                          :: fexists
      character(len=c5)                        :: message
      character(len=c3)                        :: routine

      inquire(file=fname,exist=fexists)

      routine = "DBopen (db_sql.F90)"
      select case (mode)
!     read only
      case (0)
         if (fexists) then
            fnamemode = trim(fname) // '?mode=ro'
            call sqlite3_open(fname, db)
            if (db%error.ne.SQLITE_OK) then
               write(message,'(2a)') 'sql error while opening db(ro): ', &
                                     db%errmsg
               call dberrmsg( routine, message )
            endif

            call sqlite3_do( db, 'PRAGMA foreign_keys=ON' )
            if (db%error.ne.SQLITE_OK) then
               write(message,'(2a)') 'sql error while setting PRAGMA: ', &
                                     db%errmsg
               call dberrmsg( routine, message )
            endif
         else
            write(message,'(2a)') trim(fname),' does not exist'
            call dberrmsg( routine, message )
         endif

!     read write
      case (1)
         if (fexists) then
            fnamemode = trim(fname) // '?mode=rw'
            call sqlite3_open(fname, db)
            if (db%error.ne.SQLITE_OK) then
               write(message,'(2a)') 'sql error while opening db(rw): ', &
                                     db%errmsg
               call dberrmsg( routine, message )
            endif
            call sqlite3_do( db, 'PRAGMA foreign_keys=ON' )
            if (db%error.ne.SQLITE_OK) then
               write(message,'(2a)') 'sql error while setting PRAGMA: ', &
                                     db%errmsg
               call dberrmsg( routine, message )
            endif

         else
            write(message,'(2a)') trim(fname),' does not exist'
            call dberrmsg( routine, message )
         endif

!     read write with create (this mandates creation)
      case (2)
         if (fexists) then
            write(message,'(3a)') 'creating ', trim(fname), &
                      ', but it already exists'
            call dberrmsg( routine, message )
         else
            fnamemode = trim(fname) // '?mode=rwc'
            call sqlite3_open(fname, db)
            if (db%error.ne.SQLITE_OK) then
               write(message,'(2a)') 'sql error while opening db(rwc): ', &
                                     db%errmsg
               call dberrmsg( routine, message )
            endif

            call dbinit(db,sql_vers,quantics_vers,infoarray,flagarray)
         endif

      case default
         write(message,'(2a)') 'invalid mode: ', mode
         call dberrmsg( routine, message )
        stop

      end select

      !Check SQL version and set it for the rest of the programme
      call chkversion( db,quantics_vers0, sql_vers )

      end subroutine dbopen

!-----------------------------------------------------------------------

      subroutine dbclose( db )

      type(SQLITE_DATABASE), intent(inout)     :: db

      call sqlite3_close( db )

      end subroutine dbclose

!-----------------------------------------------------------------------

      subroutine dbstarttrans(db)
!     Use this before any write to the DB.  Changes will not be visible
!     to other processes or permanent until dbcommittrans is called successfully.
!     If dbcommittrans isn't called then the changes will be lost.
!     Other reading processes should be able to read (but may block)
!     and see nothing within this transaction until it is committed.
!     Other writing processes will block until this transaction is committed or
!     aborted.

      logical(kind=4)                          :: busy
      character(len=c5)                        :: message
      character(len=c3)                        :: routine
      type(SQLITE_DATABASE)                    :: db

      busy = .true.
      routine = "DBstarttrans (db_sql.F90)"
      ! might need a sleep in here or something
      do while (busy)
         call sqlite3_begin(db, SQLITE_TRANS_IMMEDIATE)
         busy = db%error.eq.SQLITE_BUSY
      enddo
      if (db%error.ne.SQLITE_OK) then
         write(message,'(2a)') 'sql error while beginning transaction: ', &
                               db%errmsg
         call dberrmsg( routine, message )
      endif

      end subroutine dbstarttrans

!-----------------------------------------------------------------------

      subroutine dbcommittrans(db)
!     Call this to commit a transaction.  All pending transactions will
!     be committed.

      type(SQLITE_DATABASE)                    :: db

      call sqlite3_commit(db)

      end subroutine dbcommittrans

!-----------------------------------------------------------------------

      subroutine dbaborttrans(db)
!     Call this to abort a transaction.  All pending transactions will
!     be discarded.

      type(SQLITE_DATABASE)                    :: db

      call sqlite3_rollback(db)

      end subroutine dbaborttrans

!#######################################################################
!
! Following routines are to WRITE to the TABLES
!
!     For all dbwr* routines you should be inside a transaction.
!     Use dbstarttrans, dbwr*, ..., dbcommittrans
!
!     If irecno is 0 then a new entry is created, "appending" to the DB.
!     The new record number will be returned in irecno.
!     Otherwise we'll try to overwrite the entry at irecno, and fail if
!     it doesn't exist.
!#######################################################################

      subroutine dbwrentry(db, irecno, interp, timestamp)
!     interp is optional if the row already exists, in which case no update
!     will be made.

      type(SQLITE_DATABASE), intent(inout) :: db
      integer(long_sql)                        :: sql_vers  !SQL database version
      integer(long_sql), intent(in), optional  :: interp
      integer(long_sql), intent(in)            :: timestamp
      integer(long_sql), intent(inout)         :: irecno
      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      type(SQLITE_STATEMENT)               :: stmt
      character(len=250)                   :: cmd
      character(len=c5)                    :: message
      character(len=c3)                    :: routine
      logical(kind=4)                      :: finished
   
      sql_vers = db%db_vers
      routine = "DBWRentry (db_sql.F90)"

      if (sql_vers .ge. 2) then
         if (irecno .eq. 0) then
            if (.not. present(interp)) then
               write(message,'(a)') 'tried to insert new row with no interp'
               call dberrmsg( routine, message )
            endif
            ! write dbentry table for new entry
            cmd = 'INSERT INTO dbentry DEFAULT VALUES'
            call sqlite3_do( db, cmd )
            ! get id of this newly inserted row
            ! dbentry(id) is INTEGER primary key, so is alias for rowid
            call sqlite3_query_table( db, 'dbentry', cols_p )
            call sqlite3_column_props( cols_p(1), 'id', SQLITE_INT )
            ! Simply count no. of entries to date and add 1 to get new rowid
            call sqlite3_prepare( db, 'SELECT count(id) FROM dbentry', stmt,&
                 cols_p)
            call sqlite3_next_row( stmt, cols_p, finished )
            irecno = cols_p(1)%int_value + 1

         else
            ! overwrite existing dbentry table
            ! check entry exists
            call sqlite3_query_table( db, 'dbentry', cols_p )
            write(cmd,'(a9,i0)') 'WHERE id=', irecno
            call sqlite3_prepare_select( db, 'dbentry', cols_p, stmt,&
               trim(cmd) )
            call sqlite3_next_row( stmt, cols_p, finished )
            if (finished) then
               write(message,'(2a)') 'tried to overwrite nonexistent entry', &
                                     db%errmsg
               call dberrmsg( routine, message )
            endif
         endif

         ! set the interp value
         if (present(interp)) then
            ! write(cmd,'(a26,i0,a10,i0)') 'UPDATE dbentry SET status=', interp,&
            !    ' WHERE id=', irecno
            write(cmd,'(a45,i0,a1,i0,a1)') &
                'INSERT INTO dbentry(status, created) VALUES (', interp ,',&
                &', timestamp,')'
            call sqlite3_do( db, cmd) 
         endif

      elseif (sql_vers .eq. 1) then
         routine= 'dbwrentry'
         if (irecno .eq. 0) then
            if (.not. present(interp)) then
               write(message,'(a)') 'tried to insert new row with no interp'
               call dberrmsg( routine, message )
            endif

            ! write dbentry table for new entry
            cmd = 'INSERT INTO dbentry DEFAULT VALUES'
            call sqlite3_do( db, cmd )
            ! get id of this newly inserted row
            !   dbentry(id) is INTEGER primary key, so is alias for rowid
            call sqlite3_query_table( db, 'dbentry', cols_p )
            call sqlite3_column_props( cols_p(1), 'id', SQLITE_INT )
            call sqlite3_prepare( db, 'SELECT last_insert_rowid()', stmt,&
                                  cols_p)
            call sqlite3_next_row( stmt, cols_p, finished )
            irecno = cols_p(1)%int_value
         else
            ! overwrite existing dbentry table
            ! check entry exists
            call sqlite3_query_table( db, 'dbentry', cols_p )
            write(cmd,'(a9,i0)') 'WHERE id=', irecno
            call sqlite3_prepare_select( db, 'dbentry', cols_p, stmt,&
               trim(cmd) )
            call sqlite3_next_row( stmt, cols_p, finished )
            if (finished) then
               write(message,'(2a)') 'tried to overwrite nonexistent entry', &
                                     db%errmsg
               call dberrmsg( routine, message )
            endif
         endif

         ! set the interp value
         if (present(interp)) then
            write(cmd,'(a26,i0,a10,i0)') 'UPDATE dbentry SET status=', interp,&
               ' WHERE id=', irecno
            call sqlite3_do( db, cmd )
         endif
      endif 

      call sqlite3_finalize( stmt )
      deallocate (cols_p)

      end subroutine dbwrentry

!-----------------------------------------------------------------------
! Write out Geometry into geo table
! Requires 
! ndof - number of degrees of freedom (3 * natm)
!
      subroutine dbwrgeo(db,irecno, natm, geom)

      type(SQLITE_DATABASE)                    :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: natm, irecno
      real(dop_sql), dimension(:), intent(in)      :: geom
      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      integer(long_sql)                            :: ndof,dof1,i,iat
      character(len=16)                        :: label
      
      sql_vers = db%db_vers
      ndof = natm * 3   !degrees of freedom are 3 * number of atoms

      if (sql_vers .eq. 3) then
         call sqlite3_query_table( db, 'geo', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )
         dof1 = 0
         do iat = 1, natm
            do i=1,3
               if (i .eq. 1) then
                  write(label,"(a2,i0)") "x_",iat
               else if (i .eq. 2) then
                  write(label,"(a2,i0)") "y_",iat
               else if (i .eq. 3) then
                  write(label,"(a2,i0)") "z_",iat
               endif

               dof1 = dof1+1
               call sqlite3_set_column( cols_p(2), dof1 )
               call sqlite3_set_column( cols_p(3), trim(label) )
               call sqlite3_set_column( cols_p(4), geom(dof1) )
               call sqlite3_insert( db, 'geo', cols_p )
            enddo
         enddo

      else if ((sql_vers .eq. 2) .or. (sql_vers .ge. 4)) then
         call sqlite3_query_table( db, 'geo', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )
         do dof1 = 1, ndof
            call sqlite3_set_column( cols_p(dof1+1), geom(dof1) )
         enddo
         call sqlite3_insert( db, 'geo', cols_p )

      else if (sql_vers .eq. 1) then
         call sqlite3_query_table( db, 'geo', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )
         do dof1 = 1, ndof
            call sqlite3_set_column( cols_p(2), dof1 )
            call sqlite3_set_column( cols_p(3), geom(dof1) )
            call sqlite3_insert( db, 'geo', cols_p )
         enddo

      end if

      deallocate (cols_p)

      end subroutine dbwrgeo

!-----------------------------------------------------------------------

      subroutine dbwrpes(db, irecno, nddstate, pes)

      type(SQLITE_DATABASE)                    :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: nddstate,irecno
      real(dop_sql), dimension(:,:), intent(in)    :: pes
      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      integer(long_sql)                            :: i,j,index

      sql_vers = db%db_vers

      if (sql_vers .eq. 3) then
         call sqlite3_query_table( db, 'pes', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )
         do i = 1, nddstate
            call sqlite3_set_column( cols_p(2), i )
            do j = i, nddstate
               call sqlite3_set_column( cols_p(3), j )
               call sqlite3_set_column( cols_p(4), pes(j,i) )
               call sqlite3_insert( db, 'pes', cols_p )
            enddo
         enddo

      else if ((sql_vers .eq. 2) .or. (sql_vers .ge. 4)) then
         !Write out to one line
         call sqlite3_query_table( db, 'pes', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )
         index = 0
         do i =1, nddstate
            do j=i, nddstate
               call sqlite3_set_column( cols_p(2+index), pes(j,i) )
               index = index + 1
            enddo
         enddo
         call sqlite3_insert( db, 'pes', cols_p )
         
      else if (sql_vers .eq. 1) then
         call sqlite3_query_table( db, 'pes', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )
         do i = 1, nddstate
            call sqlite3_set_column( cols_p(2), i )
            do j = i, nddstate
               call sqlite3_set_column( cols_p(3), j )
               call sqlite3_set_column( cols_p(4), 1 )
               call sqlite3_set_column( cols_p(5), pes(j,i) )
               call sqlite3_insert( db, 'pes', cols_p )
            enddo
         enddo

      endif

      deallocate (cols_p)

      end subroutine dbwrpes

!-----------------------------------------------------------------------

      subroutine dbwrcpes(db, irecno, nddstate, cpes)

      type(SQLITE_DATABASE)                    :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: nddstate,irecno
      real(dop_sql), dimension(:,:), intent(in)    :: cpes
      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      integer(long_sql)                            :: i,j,index

      sql_vers = db%db_vers

      if (sql_vers .eq. 3) then
         call sqlite3_query_table( db, 'cpes', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )
         do i = 1, nddstate
            call sqlite3_set_column( cols_p(2), i )
            do j = i, nddstate
               call sqlite3_set_column( cols_p(3), j )
               call sqlite3_set_column( cols_p(4), cpes(j,i) )
               call sqlite3_insert( db, 'cpes', cols_p )
            enddo
         enddo

      else if ((sql_vers .eq. 2) .or. (sql_vers .ge. 4)) then
         !Write out to one line
         call sqlite3_query_table( db, 'cpes', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )
         index = 0
         do i =1, nddstate
            do j=i, nddstate
               call sqlite3_set_column( cols_p(2+index), cpes(j,i) )
               index = index + 1
            enddo
         enddo
         call sqlite3_insert( db, 'cpes', cols_p )
         
      else if (sql_vers .eq. 1) then
         call sqlite3_query_table( db, 'cpes', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )
         do i = 1, nddstate
            call sqlite3_set_column( cols_p(2), i )
            do j = i, nddstate
               call sqlite3_set_column( cols_p(3), j )
               call sqlite3_set_column( cols_p(4), 1 )
               call sqlite3_set_column( cols_p(5), cpes(j,i) )
               call sqlite3_insert( db, 'cpes', cols_p )
            enddo
         enddo

      endif

      deallocate (cols_p)

      end subroutine dbwrcpes

!-----------------------------------------------------------------------

      subroutine dbwrgra(db,irecno, natm, nddstate, grad)

      type(SQLITE_DATABASE)                    :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: natm,nddstate,irecno
      real(dop_sql), dimension(:,:,:), intent(in)  :: grad
      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      integer(long_sql)                            :: ndof,i,j,dof1,index,i1,iat
      character(len=16)                        :: label

      sql_vers = db%db_vers
      ndof = natm * 3   !degrees of freedom are 3 * number of atoms
      
      if (sql_vers .ge. 4) then
         call sqlite3_query_table( db, 'gra', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )
         do i = 1,nddstate
            call sqlite3_set_column( cols_p(2), i )
            do j = i, nddstate
               call sqlite3_set_column( cols_p(3), j )
               index = 0
               do dof1 = 1 ,ndof
                  call sqlite3_set_column( cols_p(4+index), grad(dof1,j,i) )
                  index = index + 1
               enddo
               call sqlite3_insert( db, 'gra', cols_p )
            enddo
         enddo

      else if (sql_vers .eq. 3) then
         call sqlite3_query_table( db, 'gra', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )
         do i = 1, nddstate
            call sqlite3_set_column( cols_p(2), i )
            do j = i, nddstate
               call sqlite3_set_column( cols_p(3), j )
               dof1 = 0
               do iat = 1, natm
                  do i1=1,3
                     if (i1 .eq. 1) then
                        write(label,"(a2,i0)") "x_",iat
                     else if (i1 .eq. 2) then
                        write(label,"(a2,i0)") "y_",iat
                     else if (i1 .eq. 3) then
                        write(label,"(a2,i0)") "z_",iat
                     endif

                     dof1 = dof1+1
                     call sqlite3_set_column( cols_p(4), dof1 )
                     call sqlite3_set_column( cols_p(5), trim(label) )
                     call sqlite3_set_column( cols_p(6), grad(dof1,j,i) )
                     call sqlite3_insert( db, 'gra', cols_p )
                  enddo
               enddo
            enddo
         enddo

      else if (sql_vers .eq. 2) then
         call sqlite3_query_table( db, 'gra', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )
         index = 0
         do i = 1,nddstate
            do j = i, nddstate
               do dof1 = 1 ,ndof
                  call sqlite3_set_column( cols_p(2+index), grad(dof1,j,i) )
                  index = index + 1
               enddo
            enddo
         enddo
         call sqlite3_insert( db, 'gra', cols_p )

      else if (sql_vers .eq. 1) then
         call sqlite3_query_table( db, 'gra', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )
         do i = 1, nddstate
            call sqlite3_set_column( cols_p(2), i )
            do j = i, nddstate
               call sqlite3_set_column( cols_p(3), j )
               do dof1 = 1, ndof
                  call sqlite3_set_column( cols_p(4), dof1 )
                  call sqlite3_set_column( cols_p(5), grad(dof1,j,i) )
                  call sqlite3_insert( db, 'gra', cols_p )
               enddo
            enddo
         enddo
      endif 

      deallocate (cols_p)

      end subroutine dbwrgra

!-----------------------------------------------------------------------

      subroutine dbwrhes(db, irecno, natm, nddstate, hess)

      type(SQLITE_DATABASE)                    :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: natm,nddstate,irecno
      real(dop_sql), dimension(:,:,:,:), intent(in):: hess
      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      integer(long_sql)                            :: ndof,i,j,dof1,dof2, index
      integer(long_sql)                            :: i1,j1,iat,jat
      character(len=250)                       :: cmd, tabname
      character(len=16)                        :: label,label1
      
      sql_vers = db%db_vers
      ndof = natm * 3   !degrees of freedom are 3 * number of atoms

      if (sql_vers .ge. 4) then
         !One table
         !Each row is a single electronic state pair
         call sqlite3_query_table(db, 'hes', cols_p)
         call sqlite3_set_column(cols_p(1), irecno)
         do i=1, nddstate
            do j=i, nddstate
               !Set the states columns
               call sqlite3_set_column(cols_p(2), i)
               call sqlite3_set_column(cols_p(3), j)
               dof1 = 0
               do iat = 1, natm
                  write(label,"(a2,i0)") "x_",iat
                  call sqlite3_set_column( cols_p(4), trim(label) )
                  dof1 = dof1+1
                  index = 5
                  do dof2 = 1, ndof
                     !Can this be sliced??
                     call sqlite3_set_column(cols_p(index), &
                          hess(dof2,dof1,j,i))
                     index = index + 1
                  enddo
                  call sqlite3_insert( db, 'hes', cols_p )

                  write(label,"(a2,i0)") "y_",iat
                  call sqlite3_set_column( cols_p(4), trim(label) )
                  dof1 = dof1+1
                  index = 5
                  do dof2 = 1, ndof
                     !Can this be sliced??
                     call sqlite3_set_column(cols_p(index), &
                          hess(dof2,dof1,j,i))
                     index = index + 1
                  enddo
                  call sqlite3_insert( db, 'hes', cols_p )

                  write(label,"(a2,i0)") "z_",iat
                  call sqlite3_set_column( cols_p(4), trim(label) )
                  dof1 = dof1+1
                  index = 5
                  do dof2 = 1, ndof
                     !Can this be sliced??
                     call sqlite3_set_column(cols_p(index), &
                          hess(dof2,dof1,j,i))
                     index = index + 1
                  enddo
                  call sqlite3_insert( db, 'hes', cols_p )
               enddo

            end do      
         end do
         deallocate(cols_p)

      else if (sql_vers .eq. 3) then
         call sqlite3_do(db, cmd)
         call sqlite3_query_table( db, 'hes', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )
         do i = 1, nddstate
            call sqlite3_set_column( cols_p(2), i )
            do j = i, nddstate
               call sqlite3_set_column( cols_p(3), j )

               dof1 = 0
               do iat = 1, natm
                  do i1=1,3
                     if (i1 .eq. 1) then
                        write(label,"(a2,i0)") "x_",iat
                     else if (i1 .eq. 2) then
                        write(label,"(a2,i0)") "y_",iat
                     else if (i1 .eq. 3) then
                        write(label,"(a2,i0)") "z_",iat
                     endif

                     dof1 = dof1+1
                     call sqlite3_set_column( cols_p(4), dof1 )
                     call sqlite3_set_column( cols_p(6), trim(label) )

                     dof2 = 0
                     do jat = 1, natm
                        do j1=1,3
                           if (j1 .eq. 1) then
                              write(label1,"(a2,i0)") "x_",jat
                           else if (j1 .eq. 2) then
                              write(label1,"(a2,i0)") "y_",jat
                           else if (j1 .eq. 3) then
                              write(label1,"(a2,i0)") "z_",jat
                           endif

                           dof2 = dof2+1
                           call sqlite3_set_column( cols_p(5), dof2 )
                           call sqlite3_set_column( cols_p(7), trim(label1) )
                           call sqlite3_set_column( cols_p(8), hess(dof2,dof1,j,i) )
                           call sqlite3_insert( db, 'hes', cols_p )
                        enddo
                     enddo
                  enddo
               enddo
            enddo
         enddo

         deallocate (cols_p)

      else if (sql_vers .eq. 2) then
         !Need to split up and write into separate tables
         do i=1, nddstate
            do j=i, nddstate
               !Get table name, is hes_st1_st2 and query table
               write(tabname,'(A4,I0,a1,i0)') "hes_", j,"_",i
               !Get table info to set up columns
               call sqlite3_query_table( db, trim(tabname), cols_p )
               !set record number
               call sqlite3_set_column( cols_p(1), irecno )
               !Now loop over other state and coordinates
               index = 0
               do dof1 = 1, ndof
                  do dof2 = 1, ndof
                     call sqlite3_set_column( cols_p(2+index), &
                          hess(dof2,dof1,j,i) )
                     index = index + 1
                  enddo
               enddo
               call sqlite3_insert( db, trim(tabname), cols_p )
               deallocate (cols_p)
            enddo
         enddo

      else if (sql_vers .eq. 1) then
         call sqlite3_query_table( db, 'hes', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )
         do i = 1, nddstate
            call sqlite3_set_column( cols_p(2), i )
            do j = i, nddstate
               call sqlite3_set_column( cols_p(3), j )
               do dof1 = 1, ndof
                  call sqlite3_set_column( cols_p(4), dof1 )
                  do dof2 = 1, ndof
                     call sqlite3_set_column( cols_p(5), dof2 )
                     call sqlite3_set_column( cols_p(6), hess(dof2,dof1,j,i) )
                     call sqlite3_insert( db, 'hes', cols_p )
                  enddo
               enddo
            enddo
         enddo

         deallocate (cols_p)
      endif
     

      end subroutine dbwrhes

!-----------------------------------------------------------------------

      subroutine dbwrapes(db, irecno, nddstate, apes)

      type(SQLITE_DATABASE)                    :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: nddstate,irecno
      real(dop_sql), dimension(:,:), intent(in)    :: apes
      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      integer(long_sql)                            :: i

      sql_vers = db%db_vers

      if ((sql_vers .eq. 2) .or. (sql_vers .ge. 4)) then
         call sqlite3_query_table( db, 'apes', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )
         do i = 1, nddstate
            call sqlite3_set_column( cols_p(1+i), apes(i,i) )
         enddo
         call sqlite3_insert( db, 'apes', cols_p )

      else
         call sqlite3_query_table( db, 'apes', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )
         do i = 1, nddstate
            call sqlite3_set_column( cols_p(2), i )
            call sqlite3_set_column( cols_p(3), i )
            call sqlite3_set_column( cols_p(4), apes(i,i) )
            call sqlite3_insert( db, 'apes', cols_p )
         enddo

      endif

      deallocate (cols_p)

      end subroutine dbwrapes

!-----------------------------------------------------------------------

      subroutine dbwragra(db,irecno, natm, nddstate, agrad)

      type(SQLITE_DATABASE)                    :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: natm,nddstate,irecno
      real(dop_sql), dimension(:,:,:), intent(in)  :: agrad
      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      integer(long_sql)                            :: ndof,i,dof1,index,i1,iat
      character(len=16)                        :: label
      
      sql_vers = db%db_vers
      ndof = natm * 3   !degrees of freedom are 3 * number of atoms


      if (sql_vers .ge. 4) then
         call sqlite3_query_table( db, 'agra', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )
         do i = 1,nddstate
            call sqlite3_set_column( cols_p(2), i )
            index = 0
            do dof1 = 1 ,ndof
               call sqlite3_set_column( cols_p(3+index), agrad(dof1,i,i) )
               index = index + 1
            enddo
            call sqlite3_insert( db, 'agra', cols_p )
         enddo

      else if (sql_vers .eq. 3) then
         call sqlite3_query_table( db, 'agra', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )
         do i = 1, nddstate
            call sqlite3_set_column( cols_p(2), i )
            dof1 = 0
            do iat = 1, natm
               do i1=1,3
                  if (i1 .eq. 1) then
                     write(label,"(a2,i0)") "x_",iat
                  else if (i1 .eq. 2) then
                     write(label,"(a2,i0)") "y_",iat
                  else if (i1 .eq. 3) then
                     write(label,"(a2,i0)") "z_",iat
                  endif

                  dof1 = dof1+1
                  call sqlite3_set_column( cols_p(3), dof1 )
                  call sqlite3_set_column( cols_p(4), trim(label) )
                  call sqlite3_set_column( cols_p(5), agrad(dof1,i,i) )
                  call sqlite3_insert( db, 'agra', cols_p )
               enddo
            enddo
         enddo

      else if (sql_vers .eq. 2) then
         call sqlite3_query_table( db, 'agra', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )
         index = 0
         do i = 1,nddstate
            do dof1 = 1 ,ndof
               call sqlite3_set_column( cols_p(2+index), agrad(dof1,i,i) )
               index = index + 1
            enddo
         enddo
         call sqlite3_insert( db, 'agra', cols_p )

      else if (sql_vers .eq. 1) then
         call sqlite3_query_table( db, 'agra', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )
         do i = 1, nddstate
            call sqlite3_set_column( cols_p(2), i )
            do dof1 = 1, ndof
               call sqlite3_set_column( cols_p(3), dof1 )
               call sqlite3_set_column( cols_p(4), agrad(dof1,i,i) )
               call sqlite3_insert( db, 'agra', cols_p )
            enddo
         enddo

      endif

      deallocate (cols_p)

      end subroutine dbwragra

!-----------------------------------------------------------------------

      subroutine dbwrahes(db,irecno, natm, nddstate, ahess)

      type(SQLITE_DATABASE)                    :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: natm,nddstate,irecno
      real(dop_sql), dimension(:,:,:,:), intent(in):: ahess
      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      integer(long_sql)                            :: ndof,i,dof1,dof2, index
      integer(long_sql)                            :: i1,j1,iat,jat
      character(len=250)                       :: tabname
      character(len=16)                        :: label,label1
      
      sql_vers = db%db_vers
      ndof = natm * 3   !degrees of freedom are 3 * number of atoms

      if (sql_vers .ge. 4) then
         !rows for each first coord for each state
         call sqlite3_query_table(db, 'ahes', cols_p)
         call sqlite3_set_column(cols_p(1), irecno)
         do i = 1, nddstate
            !set state column
            call sqlite3_set_column( cols_p(2), i )

            dof1 = 0
            do iat = 1, natm
               write(label,"(a2,i0)") "x_",iat
               call sqlite3_set_column( cols_p(3), trim(label) )
               dof1 = dof1+1
               index = 4
               do dof2 = 1, ndof
                  !Can this be sliced??
                  call sqlite3_set_column(cols_p(index), &
                       ahess(dof1,dof2,i,i))   
                  index = index + 1
               enddo
               call sqlite3_insert( db, 'ahes', cols_p )

               write(label,"(a2,i0)") "y_",iat
               call sqlite3_set_column( cols_p(3), trim(label) )
               dof1 = dof1+1
               index = 4
               do dof2 = 1, ndof
                  !Can this be sliced??
                  call sqlite3_set_column(cols_p(index), &
                       ahess(dof1,dof2,i,i))   
                  index = index + 1
               enddo
               call sqlite3_insert( db, 'ahes', cols_p )

               write(label,"(a2,i0)") "z_",iat
               call sqlite3_set_column( cols_p(3), trim(label) )
               dof1 = dof1+1
               index = 4
               do dof2 = 1, ndof
                  !Can this be sliced??
                  call sqlite3_set_column(cols_p(index), &
                       ahess(dof1,dof2,i,i))   
                  index = index + 1
               enddo
               call sqlite3_insert( db, 'ahes', cols_p )
            enddo
         end do
         deallocate(cols_p)

      else if (sql_vers .eq. 3) then
         call sqlite3_query_table( db, 'ahes', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )
         do i = 1, nddstate
            call sqlite3_set_column( cols_p(2), i )

            dof1 = 0
            do iat = 1, natm
               do i1=1,3
                  if (i1 .eq. 1) then
                     write(label,"(a2,i0)") "x_",iat
                  else if (i1 .eq. 2) then
                     write(label,"(a2,i0)") "y_",iat
                  else if (i1 .eq. 3) then
                     write(label,"(a2,i0)") "z_",iat
                  endif

                  dof1 = dof1+1
                  call sqlite3_set_column( cols_p(3), dof1 )
                  call sqlite3_set_column( cols_p(5), trim(label) )

                  dof2 = 0
                  do jat = 1, natm
                     do j1=1,3
                        if (j1 .eq. 1) then
                           write(label1,"(a2,i0)") "x_",jat
                        else if (j1 .eq. 2) then
                           write(label1,"(a2,i0)") "y_",jat
                        else if (j1 .eq. 3) then
                           write(label1,"(a2,i0)") "z_",jat
                        endif

                        dof2 = dof2+1
                        call sqlite3_set_column( cols_p(4), dof2 )
                        call sqlite3_set_column( cols_p(6), trim(label1) )
                        call sqlite3_set_column( cols_p(7), ahess(dof2,dof1,i,i) )
                        call sqlite3_insert( db, 'ahes', cols_p )
                     enddo
                  enddo
               enddo
            enddo
         enddo

         deallocate (cols_p)

      else if (sql_vers .eq. 2) then
         !Need to split up and write into separate tables
         do i=1, nddstate
            !Get table name, is ahes_st1 and query table
            write(tabname,'(A5,I0)') "ahes_", i
            !Get table info to set up columns
            call sqlite3_query_table( db, trim(tabname), cols_p )
            !set record number
            call sqlite3_set_column( cols_p(1), irecno )
            !Now loop over other state and coordinates
            index = 0
            do dof1 = 1, ndof
               do dof2 = 1, ndof
                  call sqlite3_set_column( cols_p(2+index), ahess(dof2,dof1,i,i) )
                  index = index + 1
               enddo
            enddo
            call sqlite3_insert( db, trim(tabname), cols_p )
            deallocate (cols_p)
         enddo

      else if (sql_vers .eq. 1) then
         call sqlite3_query_table( db, 'ahes', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )
         do i = 1, nddstate
            call sqlite3_set_column( cols_p(2), i )
            do dof1 = 1, ndof
               call sqlite3_set_column( cols_p(3), dof1 )
               do dof2 = 1, ndof
                  call sqlite3_set_column( cols_p(4), dof2 )
                  call sqlite3_set_column( cols_p(5), ahess(dof2,dof1,i,i) )
                  call sqlite3_insert( db, 'ahes', cols_p )
               enddo
            enddo
         enddo

         deallocate (cols_p)
      endif

      end subroutine dbwrahes

!-----------------------------------------------------------------------
!
! Need to write off-diagonal dipoles (TDips). Write lower triangle 
! and complete in dbrddip
!-----------------------------------------------------------------------

      subroutine dbwrdip(db, irecno, nddstate, dipole)

      type(SQLITE_DATABASE)                    :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: nddstate,irecno
      real(dop_sql), dimension(:,:,:), intent(in)  :: dipole
      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      integer(long_sql)                            :: i

      sql_vers = db%db_vers

      call sqlite3_query_table( db, 'dip', cols_p )
      call sqlite3_set_column( cols_p(1), irecno )
      do i = 1, nddstate
         call sqlite3_set_column( cols_p(2), i )
         call sqlite3_set_column( cols_p(3), i )
         call sqlite3_set_column( cols_p(4), dipole(1,i,i) )
         call sqlite3_set_column( cols_p(5), dipole(2,i,i) )
         call sqlite3_set_column( cols_p(6), dipole(3,i,i) )
         call sqlite3_insert( db, 'dip', cols_p )
      enddo

      deallocate (cols_p)

      end subroutine dbwrdip

!-----------------------------------------------------------------------

      subroutine dbwradip(db, irecno, nddstate, adipole)

      type(SQLITE_DATABASE)                    :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: nddstate,irecno
      real(dop_sql), dimension(:,:,:), intent(in)  :: adipole
      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      integer(long_sql)                            :: i

      sql_vers = db%db_vers

      call sqlite3_query_table( db, 'adip', cols_p )
      call sqlite3_set_column( cols_p(1), irecno )
      do i = 1, nddstate
         call sqlite3_set_column( cols_p(2), i )
         call sqlite3_set_column( cols_p(3), i )
         call sqlite3_set_column( cols_p(4), adipole(1,i,i) )
         call sqlite3_set_column( cols_p(5), adipole(2,i,i) )
         call sqlite3_set_column( cols_p(6), adipole(3,i,i) )
         call sqlite3_insert( db, 'adip', cols_p )
      enddo

      deallocate (cols_p)

      end subroutine dbwradip

!-----------------------------------------------------------------------

      subroutine dbwrmoco(db, irecno, nbasis, mocoeff)

      type(SQLITE_DATABASE)                    :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: nbasis,irecno
      real(sip_sql), dimension(:,:), intent(in)    :: mocoeff
      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      integer(long_sql)                            :: i,j
      character(len=250)                       :: tabname
      
      sql_vers = db%db_vers

      if (sql_vers .ge. 4) then
         !One row per MO 
         call sqlite3_query_table( db, 'moco', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )
         do i = 1, nbasis
            call sqlite3_set_column( cols_p(2), i )
            do j = 1, nbasis
               !Can we slice this?
               call sqlite3_set_column( cols_p(j+2), mocoeff(j,i) )
            enddo
            call sqlite3_insert( db, 'moco', cols_p )
         enddo

         deallocate (cols_p)

      else if (sql_vers .eq. 2) then
         !Get table name
         do i = 1, nbasis
            write(tabname,'(A5,I0)') "moco_", i
            call sqlite3_query_table( db, trim(tabname), cols_p )
            !set record number
            call sqlite3_set_column( cols_p(1), irecno )
            !Now loop over other basis functions
            do j = 1, nbasis
               call sqlite3_set_column( cols_p(1+j), mocoeff(j,i) )
            enddo
            call sqlite3_insert( db, trim(tabname), cols_p )
            deallocate (cols_p)
         enddo

      else
         call sqlite3_query_table( db, 'moco', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )
         do i = 1, nbasis
            call sqlite3_set_column( cols_p(2), i )
            do j = 1, nbasis
               call sqlite3_set_column( cols_p(3), j )
               call sqlite3_set_column( cols_p(4), mocoeff(j,i) )
               call sqlite3_insert( db, 'moco', cols_p )
            enddo
         enddo

         deallocate (cols_p)
      endif 

      end subroutine dbwrmoco

!-----------------------------------------------------------------------

      subroutine dbwrnact(db, irecno, natm, nddstate, nact)

      type(SQLITE_DATABASE)                    :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: natm,nddstate,irecno
      real(dop_sql), dimension(:,:), intent(in)    :: nact
      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      integer(long_sql)                            :: dercpdim,i,j
      integer(long_sql)                            :: ndof,c,dof1,i1,iat
      character(len=16)                        :: label

      sql_vers = db%db_vers
      ndof = natm * 3   !degrees of freedom are 3 * number of atoms
      dercpdim=nddstate*(nddstate-1)/2  ! No. of pairs of states

      if (sql_vers .ge. 4) then
         call sqlite3_query_table( db, 'nact', cols_p )
         call sqlite3_set_column( cols_p(1), irecno)
         do j = 2,nddstate   ! upper triangle sequentially
            do i = 1,j-1
               c = (j-1)*(j-2)/2+i
               call sqlite3_set_column( cols_p(2), c )
               call sqlite3_set_column( cols_p(3), i )
               call sqlite3_set_column( cols_p(4), j )
               do dof1 = 1, ndof
                  call sqlite3_set_column( cols_p(dof1+4), nact(dof1,c) )
               enddo
               call sqlite3_insert( db, 'nact', cols_p )
            enddo
         enddo

      else if (sql_vers .eq. 3) then
         call sqlite3_query_table( db, 'nact', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )
         do i = 1,nddstate-1
            do j = i+1,nddstate
               c = (j-1)*(j-2)/2+i

               call sqlite3_set_column( cols_p(2), c )
               call sqlite3_set_column( cols_p(4), i )
               call sqlite3_set_column( cols_p(5), j )
            
               dof1=0
               do iat = 1, natm
                  do i1=1,3
                     if (i1 .eq. 1) then
                        write(label,"(a2,i0)") "x_",iat
                     else if (i1 .eq. 2) then
                        write(label,"(a2,i0)") "y_",iat
                     else if (i1 .eq. 3) then
                        write(label,"(a2,i0)") "z_",iat
                     endif

                     dof1 = dof1+1
                     call sqlite3_set_column( cols_p(3), dof1 )
                     call sqlite3_set_column( cols_p(6), trim(label) )
                     call sqlite3_set_column( cols_p(7), nact(dof1,c) )
                     call sqlite3_insert( db, 'nact', cols_p )
                  enddo
               enddo
            enddo
         enddo

      else if (sql_vers .eq. 2) then
         call sqlite3_query_table( db, 'nact', cols_p )
         call sqlite3_set_column( cols_p(1), irecno)
         do c = 1, dercpdim
            call sqlite3_set_column( cols_p(2), c )
            do dof1 = 1, ndof
               call sqlite3_set_column( cols_p(dof1+2), nact(dof1,c) )
            enddo
            call sqlite3_insert( db, 'nact', cols_p )
         enddo

      else if (sql_vers .eq. 1) then
         call sqlite3_query_table( db, 'nact', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )
         do c = 1, dercpdim
            call sqlite3_set_column( cols_p(2), c )
            do dof1 = 1, ndof
               call sqlite3_set_column( cols_p(3), dof1 )
               call sqlite3_set_column( cols_p(4), nact(dof1,c) )
               call sqlite3_insert( db, 'nact', cols_p )
            enddo
         enddo

      end if

      deallocate (cols_p)

      end subroutine dbwrnact

!-----------------------------------------------------------------------
      subroutine dbwrtrans(db,irecno, nddstate, diabtrans)

      type(SQLITE_DATABASE)                    :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: nddstate,irecno
      real(dop_sql), dimension(:,:), intent(in)    :: diabtrans
      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      integer(long_sql)                            :: i,j, index

      sql_vers = db%db_vers

      if (sql_vers .eq. 3) then
         call sqlite3_query_table( db, 'trans', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )
         do i = 1, nddstate
            call sqlite3_set_column( cols_p(2), i )
            do j = 1, nddstate
               call sqlite3_set_column( cols_p(3), j )
               call sqlite3_set_column( cols_p(4), diabtrans(j,i) )
               call sqlite3_insert( db, 'trans', cols_p )
            enddo
         enddo

      else if (sql_vers .eq. 2 .or. sql_vers .ge. 4) then
         !Write out to one line
         call sqlite3_query_table( db, 'trans', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )
         index = 0
         do i =1, nddstate
            do j=1, nddstate
               call sqlite3_set_column( cols_p(2+index), diabtrans(j,i) )
               index = index + 1
            enddo
         enddo
         call sqlite3_insert( db, 'trans', cols_p )

      else if (sql_vers .eq. 1) then
         call sqlite3_query_table( db, 'trans', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )
         do i = 1, nddstate
            call sqlite3_set_column( cols_p(2), i )
            do j = 1, nddstate
               call sqlite3_set_column( cols_p(3), j )
               ! if diabtrans(:,:) then no complex energies
               call sqlite3_set_column( cols_p(4), 1 )
               call sqlite3_set_column( cols_p(5), diabtrans(j,i) )
               call sqlite3_insert( db, 'trans', cols_p )
            enddo
         enddo

      end if   

      deallocate (cols_p)

      end subroutine dbwrtrans

!-----------------------------------------------------------------------
      subroutine dbwrctrans(db,irecno, nddstate, diabctrans)

      type(SQLITE_DATABASE)                    :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: nddstate,irecno
      real(dop_sql), dimension(:,:), intent(in)    :: diabctrans
      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      integer(long_sql)                            :: i,j, index

      sql_vers = db%db_vers

      if (sql_vers .eq. 3) then
         call sqlite3_query_table( db, 'ctrans', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )
         do i = 1, nddstate
            call sqlite3_set_column( cols_p(2), i )
            do j = 1, nddstate
               call sqlite3_set_column( cols_p(3), j )
               call sqlite3_set_column( cols_p(4), diabctrans(j,i) )
               call sqlite3_insert( db, 'ctrans', cols_p )
            enddo
         enddo

      else if (sql_vers .eq. 2 .or. sql_vers .ge. 4) then
         !Write out to one line
         call sqlite3_query_table( db, 'ctrans', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )
         index = 0
         do i =1, nddstate
            do j=1, nddstate
               call sqlite3_set_column( cols_p(2+index), diabctrans(j,i) )
               index = index + 1
            enddo
         enddo
         call sqlite3_insert( db, 'ctrans', cols_p )

      else if (sql_vers .eq. 1) then
         call sqlite3_query_table( db, 'ctrans', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )
         do i = 1, nddstate
            call sqlite3_set_column( cols_p(2), i )
            do j = 1, nddstate
               call sqlite3_set_column( cols_p(3), j )
               ! if diabctrans(:,:) then no complex energies
               call sqlite3_set_column( cols_p(4), 1 )
               call sqlite3_set_column( cols_p(5), diabctrans(j,i) )
               call sqlite3_insert( db, 'ctrans', cols_p )
            enddo
         enddo

      end if   

      deallocate (cols_p)

      end subroutine dbwrctrans

!-----------------------------------------------------------------------
!  Write out normal modes
!  SQL version 1 row per coordinate
!  SQL version 2 each row is a single geometry
!
!  nmdof is number of normals modes in molecule
!----------------

      subroutine dbwrnm(db,irecno, nmdof, nm)
      type(SQLITE_DATABASE)                    :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: nmdof,irecno
      real(dop_sql), dimension(:), intent(in)      :: nm
      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      integer(long_sql)                            :: i

      sql_vers = db%db_vers

      if ((sql_vers .eq. 2) .or. (sql_vers .ge. 4)) then
         !Write out all as one row
         call sqlite3_query_table( db, 'nmode', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )
         do i = 1, nmdof
            call sqlite3_set_column( cols_p(i+1), nm(i) )
         enddo
         call sqlite3_insert( db, 'nmode', cols_p )

      else
         call sqlite3_query_table( db, 'nmode', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )
         do i = 1, nmdof
            call sqlite3_set_column( cols_p(2), i )
            call sqlite3_set_column( cols_p(3), nm(i) )
            call sqlite3_insert( db, 'nmode', cols_p )
         enddo
      endif
      deallocate (cols_p)

      end subroutine dbwrnm

!-----------------------------------------------------------------------

      subroutine dbwrcasoc(db, irecno, nsocstate, asoc)

      type(SQLITE_DATABASE)                    :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: nsocstate,irecno
      integer(long_sql)                            :: index
      complex(dop_sql), dimension(:,:)             :: asoc
      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      integer(long_sql)                            :: i,j,c

      sql_vers = db%db_vers

      if (sql_vers .ge. 4) then
         !Write out to one line
         call sqlite3_query_table( db, 'casoc', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )

! write out lower triangle - real then imaginary parts - note change from v1.
         call sqlite3_set_column( cols_p(2), 'real' )
         index = 0
         do i =1, nsocstate-1
            do j=i+1, nsocstate
               call sqlite3_set_column( cols_p(3+index), dble(asoc(j,i)) )
               index = index + 1
            enddo
         enddo
         call sqlite3_insert( db, 'casoc', cols_p )

         call sqlite3_set_column( cols_p(2), 'imag' )
         index = 0
         do i =1, nsocstate-1
            do j=i+1, nsocstate
               call sqlite3_set_column( cols_p(3+index), dimag(asoc(j,i)) )
               index = index + 1
            enddo
         enddo
         call sqlite3_insert( db, 'casoc', cols_p )

      else
         call sqlite3_query_table( db, 'casoc', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )
         do j = 2,nsocstate   ! upper triangle sequentially
            do i = 1,j-1
               c = (j-1)*(j-2)/2+i  ! index
               call sqlite3_set_column( cols_p(2), c )
               call sqlite3_set_column( cols_p(3), dble(asoc(i,j)) )
               call sqlite3_set_column( cols_p(4), dimag(asoc(i,j)) )
               call sqlite3_insert( db, 'casoc', cols_p )
            enddo
         enddo

      endif

      deallocate (cols_p)

      end subroutine dbwrcasoc

!-----------------------------------------------------------------------
! nsmult : No. multiplicities
! imultblk(s) : No. states in spin block s
! imult(s) : multiplicity of spin block s
!-----------------------------------------------------------------------

      subroutine dbwrcidat(db, nsmult, imult, nsta, nconf)

      type(SQLITE_DATABASE)                       :: db
      integer(long_sql)                           :: sql_vers  !SQL database version
      integer(long_sql), intent(in)               :: nsmult
      integer(long_sql), dimension(nsmult), intent(in) ::  imult, nsta, nconf
      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      integer(long_sql)                           :: ns
      character(len=c5)                           :: message
      character(len=c3)                           :: routine

      sql_vers = db%db_vers
      routine = "DBWRcidat (db_sql.F90)"

      if (sql_vers .ge. 4) then
         call sqlite3_query_table( db, 'civec_dat', cols_p )
         do ns=1,nsmult
            call sqlite3_set_column( cols_p(1), ns )
            call sqlite3_set_column( cols_p(2), imult(ns) )
            call sqlite3_set_column( cols_p(3), nsta(ns) )
            call sqlite3_set_column( cols_p(4), nconf(ns) )
            call sqlite3_insert( db, 'civec_dat', cols_p )
         enddo
         deallocate (cols_p)
      else
         write(message,'(a)') &
                'CIdat table cannot be written for DB version <4'
         call dberrmsg( routine, message )
      endif

      end subroutine dbwrcidat

!-----------------------------------------------------------------------
! nsmult : No. multiplicities
! imultblk(s) : No. states in spin block s
! imult(s) : multiplicity of spin block s
!-----------------------------------------------------------------------

      subroutine dbwrcivec(db, irecno, nsmult, nci, nsta, iconf, ista, &
                 ciconfig, cicoeff)

      type(SQLITE_DATABASE)                        :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: irecno
      integer(long_sql), intent(in)                :: nsmult, nci, nsta
      integer(long_sql), dimension(nsmult), intent(in) :: iconf, ista
      integer(long_sql)                            :: index
      character(len=*), dimension(nci,nsmult)      :: ciconfig
      real(dop_sql), dimension(nci,nsta,nsmult)    :: cicoeff     
      type(SQLITE_COLUMN), dimension(:), pointer   :: cols_p
      integer(long_sql)                            :: j,k,ns
      character(len=c5)                        :: message
      character(len=c3)                        :: routine

      sql_vers = db%db_vers
      routine = "DBWRcivec (db_sql.F90)"

      if (sql_vers .ge. 4) then
         !Write out CI vectors
         call sqlite3_query_table( db, 'civec', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )

         do ns=1,nsmult
            call sqlite3_set_column( cols_p(2), ns)
            do j = 1,iconf(ns)
               call sqlite3_set_column( cols_p(3), j )
               call sqlite3_set_column( cols_p(4), ciconfig(j,ns) )
               index=4
               do k = 1,ista(ns)
                  index = index+1
                  call sqlite3_set_column( cols_p(index), cicoeff(j,k,ns) )
               enddo
               call sqlite3_insert( db, 'civec', cols_p )
            enddo
         enddo

      else
         routine = 'DBwrcivec (db_sql.f90)'
         write(message,'(a)') &
                'CIvec table cannot be written for DB version <4'
         call dberrmsg( routine, message )
      endif

      end subroutine dbwrcivec

!-----------------------------------------------------------------------

      subroutine dbwrsymrep(db, irecno, nsymop, symrep)

      type(SQLITE_DATABASE)                    :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: nsymop, irecno
      integer(long_sql), dimension(:), intent(in)  :: symrep
      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      integer(long_sql)                            :: n

      sql_vers = db%db_vers

      if ((sql_vers .eq. 2) .or. (sql_vers .ge. 4)) then
         call sqlite3_query_table( db, 'symrep', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )
         do n = 1, nsymop
            call sqlite3_set_column( cols_p(1+n), symrep(n) )
         enddo
         call sqlite3_insert( db, 'symrep', cols_p )

      else
         call sqlite3_query_table( db, 'symrep', cols_p )
         call sqlite3_set_column( cols_p(1), irecno )
         do n = 1, nsymop
            call sqlite3_set_column( cols_p(2), n )
            call sqlite3_set_column( cols_p(3), symrep(n) )
            call sqlite3_insert( db, 'symrep', cols_p )
         enddo
      endif
      deallocate (cols_p)

      end subroutine dbwrsymrep


!-----------------------------------------------------------------------
!
!  Subroutine to write info table
!  Provides info on calculation that was performed
!
!  Doesn't exist in SQL v1
!--------------------------------------
      subroutine dbwrinfo(db, method, basis, prog)
         type(SQLITE_DATABASE), intent(inout)     :: db
         integer(long_sql)                            :: sql_vers  !SQL database version
         character(len=c2), intent(in)            :: basis
         character(len=c2), intent(in)            :: method, prog
      
         type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p

         sql_vers = db%db_vers
         if (sql_vers .gt. 1) then
            call sqlite3_query_table( db, 'info', cols_p )
            call sqlite3_set_column( cols_p(1), method )
            call sqlite3_set_column( cols_p(2), basis )
            call sqlite3_set_column( cols_p(3), prog )

            call sqlite3_insert( db, 'info', cols_p )

            
            deallocate (cols_p)
         elseif (sql_vers .eq. 1) then
            !Not in this version
         endif


      end subroutine dbwrinfo

!-----------------------------------------------------------------------
!  Subroutine to write the reference data tables

      subroutine dbwrrefdb(db, natm, nroot, nddstate, nbasis, dde0, &
                 dimref, nfam, mref, atnam, xref, dercpdim, dbpntgp, &
                 nsymop, nsocstate, dbsymmap, nddmult, qcprog, qcmethod, &
                 qcbasis, ddqcfile, ddname,ddlaenge)

      implicit none

      type(SQLITE_DATABASE), intent(inout)    :: db
      integer(long_sql)                       :: sql_vers  !SQL database version
      integer(long_sql)                       :: i
      integer(long_sql), intent(in)           :: natm,nroot,nddstate,nbasis
      real(kind=dop_sql), intent(in)          :: dde0
      integer(long_sql), intent(in)           :: dimref,nfam,dercpdim,&
                                                 dbpntgp,nsymop,nsocstate
      integer(long_sql), dimension(:,:), intent(in) :: dbsymmap
      integer(long_sql), dimension(:), intent(in)  :: nddmult
      real(kind=dop_sql), dimension(:), intent(in) :: mref
      character(len=c1), dimension(:), intent(in) :: atnam
      real(kind=dop_sql), dimension(:,:), intent(in) :: xref
      character(len=c2), intent(in)              :: qcprog,qcmethod, qcbasis

      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      logical(kind=4)               :: busy, check
      character(len=c5)             :: message
      character(len=c3)             :: routine
      
      character(len=250)            :: cmd, filename
      integer(long_sql)             :: at,n
      character(len=*)              :: ddqcfile
      character(len=c5)             :: ddname
      integer(long_sql)             :: ddlaenge

      !Variables for the template file

      sql_vers = db%db_vers

      busy = .true.
      routine = "DBWRrefdb (db_sql.F90)"
      ! might need a sleep in here or something
      do while (busy)
         call sqlite3_begin(db, SQLITE_TRANS_IMMEDIATE)
         busy = db%error.eq.SQLITE_BUSY
      enddo
      if (db%error.ne.SQLITE_OK) then
         write(message,'(2a)') 'sql error while beginning transaction: ', &
                               db%errmsg
         call dberrmsg( routine, message )
      endif

      ! overwrite anything already there
      write(cmd,'(a)') 'DELETE FROM refdb'
      call sqlite3_do( db, cmd )
      write(cmd,'(a)') 'DELETE FROM refdbrefgeom'
      call sqlite3_do( db, cmd )
      write(cmd,'(a)') 'DELETE FROM refdbsymmaps'
      call sqlite3_do( db, cmd )
      call sqlite3_query_table(db, 'refdb', cols_p)
      call sqlite3_set_column(cols_p(1), natm)
      call sqlite3_set_column(cols_p(2), nroot)
      call sqlite3_set_column(cols_p(3), nddstate)
      call sqlite3_set_column(cols_p(4), nbasis)
      call sqlite3_set_column(cols_p(5), dde0)
      call sqlite3_set_column(cols_p(6), dimref)
      call sqlite3_set_column(cols_p(7), nfam)
      call sqlite3_set_column(cols_p(8), dercpdim)
      call sqlite3_set_column(cols_p(9), dbpntgp)
      call sqlite3_set_column(cols_p(10), nsymop)
      call sqlite3_set_column(cols_p(11), nsocstate)
      do i=1,7
         call sqlite3_set_column(cols_p(11+i), nddmult(i))
      enddo

      call sqlite3_insert(db, 'refdb', cols_p)
      if (db%error.ne.SQLITE_OK) then
         write(message,'(2a)') 'sql error while inserting: ', &
                               db%errmsg
         call dberrmsg( routine, message )
      endif

      !Do refdbgeom
      !This should be its own subroutine
      call dbwrrefdbgeom(db,natm,atnam,mref,xref)

      if (dbpntgp .gt. 0) then 
         deallocate(cols_p)
         call sqlite3_query_table(db, 'refdbsymmaps', cols_p)
         if (sql_vers .eq. 2) then
         !Now refdbsymmaps
            do n =1,nsymop
               call sqlite3_set_column(cols_p(1), n)
               do at = 1, natm
                  call sqlite3_set_column(cols_p(1+at), dbsymmap(at,n))
               enddo
               call sqlite3_insert(db, 'refdbsymmaps', cols_p)
               if (db%error.ne.SQLITE_OK) then
                  write(message,'(2a)') 'sql error while writing symmaps: ', &
                                       db%errmsg
                  call dberrmsg( routine, message )
               endif
            enddo

         else
            do n = 1,nsymop
               do at = 1,natm
                  call sqlite3_set_column(cols_p(1), n)
                  call sqlite3_set_column(cols_p(2), at)
                  call sqlite3_set_column(cols_p(3), dbsymmap(at,n))
                  call sqlite3_insert(db, 'refdbsymmaps', cols_p)
                  if (db%error.ne.SQLITE_OK) then
                     write(message,'(2a)') 'sql error while writing symmaps: ', &
                                          db%errmsg
                     call dberrmsg( routine, message )
                  endif
               enddo
            enddo

         endif

      endif

      !Enter stuff into the info table
      if (db%db_vers .gt. 1) then
         call dbwrinfo(db,qcmethod,qcbasis,qcprog)
      endif
      !If needed load template and QC files into database
      if (db%db_vers .ge. 4) then   !Should be other flag to check needed
         filename=ddname(1:ddlaenge)
         filename(ddlaenge+1:ddlaenge+13)='/template.dat'
         inquire(file=filename,exist=check)
         if (.not. check) then
            message = ' There is no direct dynamics template.dat file! '// &
                  '                      '//filename(1:ddlaenge+13)
            !call warnmsg
         else
            !Attempt to write into database
            call dbwrtemplate(db, filename)
         end if
         !Get QC filename
         check = .false.
         if (ddqcfile .ne. ' ') then
            filename=ddname(1:ddlaenge)
            filename=trim(filename) // "/" // trim(ddqcfile)
            inquire(file=filename,exist=check)
         endif

! check there is a transformation file to read
         if (ddqcfile .eq. ' ' .or. .not. check) then
            continue
         else
            !Attempt to write into database
            call dbwrqcfreq(db, filename, ddqcfile)
         end if
      end if
!     Actually commit the transaction
      call sqlite3_commit(db)
      if (db%error.ne.SQLITE_OK) then
         write(message,'(2a)') 'sql error while committing transaction: ', &
                               db%errmsg
         call dberrmsg( routine, message )
      endif
         
      deallocate(cols_p)

      end subroutine dbwrrefdb

!-----------------------------------------------------------------------
!  Subroutine to write the refddgeom
      subroutine dbwrrefdbgeom(db, natm, atnam, mref, xref)

      type(SQLITE_DATABASE), intent(inout)     :: db
      integer(long_sql)                            :: sql_vers  !SQL database version
      integer(long_sql), intent(in)                :: natm
      real(kind=dop_sql), dimension(:), intent(in) :: mref
      character(len=c1), dimension(:), intent(in) :: atnam
      real(kind=dop_sql), dimension(:,:), intent(in) :: xref

      type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p
      character(len=c5)                        :: message
      character(len=c3)                        :: routine
      integer(long_sql)                            :: at
      
      sql_vers = db%db_vers
      routine = "DBWRrefdbgeom (db_sql.F90)"

      call sqlite3_query_table(db, 'refdbrefgeom', cols_p)
      do at = 1, natm
         call sqlite3_set_column(cols_p(1), at)
         call sqlite3_set_column(cols_p(2), mref(at))
         call sqlite3_set_column(cols_p(3), atnam(at))
         call sqlite3_set_column(cols_p(4), xref(1,at))
         call sqlite3_set_column(cols_p(5), xref(2,at))
         call sqlite3_set_column(cols_p(6), xref(3,at))
         call sqlite3_insert(db, 'refdbrefgeom', cols_p)
         if (db%error.ne.SQLITE_OK) then
            write(message,'(2a)') 'sql error while writing geom: ', &
                                 db%errmsg
            call dberrmsg( routine, message )
         endif
      enddo

      deallocate(cols_p)
         
      end subroutine dbwrrefdbgeom

!#######################################################################
! UTILITY ROUTINES:
! chkversion, wrversion, dberrmsg, dbdelallentries
!#######################################################################

      subroutine chkversion(db, quantics_vers, sql_vers)
      implicit none
      type(SQLITE_DATABASE), intent(inout)     :: db
      real(dop_sql), intent(inout)             :: quantics_vers 
      integer(long_sql), intent(inout)         :: sql_vers  !SQL database version
      integer(long_sql)                        :: dbvers

      type(SQLITE_STATEMENT)                   :: stmt
      type(SQLITE_COLUMN), dimension(:),pointer:: cols_p
      character(len=c5)                        :: message
      character(len=c3)                        :: routine
      logical(kind=4)                          :: finished

      routine = "CHKVERSION (db_sql.F90)"

      call sqlite3_query_table(db, 'versions', cols_p)
      call sqlite3_column_props(cols_p(1), 'quanticsversion', SQLITE_INT)
      call sqlite3_column_props(cols_p(2), 'dbversion', SQLITE_INT)
      call sqlite3_set_column(cols_p(1), quantics_vers)
      call sqlite3_set_column(cols_p(2), sql_vers)
      
      !This is true for sql vers greater than 1
      if (size(cols_p) .gt. 2) then
         call sqlite3_column_props(cols_p(3), 'dbentry', SQLITE_INT)
         call sqlite3_set_column(cols_p(3), 0)
      endif

      !Now get some values
      call sqlite3_prepare(db, 'SELECT * FROM versions',&
                           stmt, cols_p)

      call sqlite3_next_row(stmt, cols_p, finished)
      if (finished) then
         !No row
         dbvers = -1
      else
         quantics_vers = cols_p(1)%double_value
         db%db_vers = cols_p(2)%int_value
         sql_vers = db%db_vers
         if (size(cols_p) .gt. 2) then
            !Can set number of entries if that is of interest
         endif
         if (sql_vers > DBVERSION) then
            write(message,'(2a)') &
                  'database has higher version than we understand'
            call dberrmsg( routine, message )
         endif
      endif

      deallocate(cols_p)

      end subroutine chkversion

!-----------------------------------------------------------------------

      subroutine dbwrversion(db,  quantics_vers, sql_vers)
         implicit none
         type(SQLITE_DATABASE), intent(inout)     :: db
         real(dop_sql), intent(in)                :: quantics_vers  
         integer(long_sql), intent(in)            :: sql_vers  !SQL database version
         type(SQLITE_COLUMN), dimension(:),pointer:: cols_p
   
         call sqlite3_query_table(db, 'versions', cols_p)
         call sqlite3_column_props(cols_p(1), 'quanticsversion', SQLITE_INT)
         call sqlite3_column_props(cols_p(2), 'dbversion', SQLITE_INT)
         call sqlite3_set_column(cols_p(1), quantics_vers)
         call sqlite3_set_column(cols_p(2), sql_vers)
         
         !This is true for sql vers greater than 1
         if (size(cols_p) .gt. 2) then
            call sqlite3_column_props(cols_p(3), 'dbentry', SQLITE_INT)
            call sqlite3_set_column(cols_p(3), 0)
         endif
   
         call sqlite3_insert( db, 'versions', cols_p )
         
         deallocate(cols_p)
         
      end subroutine dbwrversion

      subroutine dbwrtemplate(db, templatefile)
         type(SQLITE_DATABASE), intent(inout)     :: db
         character(len=c5), intent(inout)         :: templatefile
         integer(long_sql)                        :: sql_vers  !SQL database version

         character(len=500)                       :: temp,file
         character(len=500)                       :: cmd
         integer                                  :: error
         character(len=c5)                        :: message
         character(len=c3)                        :: routine
         integer(long_sql)                        :: itmpin = 24

         type(SQLITE_COLUMN), dimension(:),pointer:: cols_p

         sql_vers = db%db_vers
         routine = "DBWRtemplate (db_sql.F90)"


         !only in versions above 1
         if (sql_vers .gt. 1) then
            !Insert template into db
            file = ""
            open(itmpin,file=templatefile,form="formatted",status='old')
            call sqlite3_query_table(db, 'template', cols_p)
            do
               !Read in the template file line by line
               read(itmpin,"(A)",iostat = error) temp
               if (is_iostat_end(error)) exit

               !write line to table
               call sqlite3_set_column(cols_p(1), trim(temp))
               call sqlite3_insert(db, 'template', cols_p)

               if (db%error.ne.SQLITE_OK) then
                  write(message,'(2a)') 'sql error while reading in template: ', &
                                        db%errmsg
                  call dberrmsg( routine, message )
               endif
            end do
            deallocate(cols_p)
            close(itmpin)
         elseif (sql_vers .eq. 1) then
            !Do nothing does not exist
         endif

      end subroutine dbwrtemplate

      subroutine dbwrqcfreq(db, qcfile, qcname)
         type(SQLITE_DATABASE), intent(inout)     :: db
         character(len=*), intent(in)             :: qcfile, qcname
         integer(long_sql)                        :: sql_vers  !SQL database version
         character(len=c5)                        :: message
         character(len=c3)                        :: routine
         character(len=500)                       :: temp, temp1
         character(len=500)                       :: file, cmd
         integer                                  :: error, qidx,oqidx, cnt, pidx
         integer(long_sql)                        :: itmpin = 24
         type(SQLITE_COLUMN), dimension(:), pointer  :: cols_p

         sql_vers = db%db_vers
         routine = "DBWRQCfreq (db_sql.F90)"

         !only in versions above 1
         if (sql_vers .gt. 1) then
            open(itmpin,file=qcfile,form="formatted",status='old')
            call sqlite3_query_table(db, 'qcfreq', cols_p)
            do
               !Read in the QC frequency file line by line
               read(itmpin,"(A)",iostat = error) temp
               if (is_iostat_end(error)) exit

               !Check for escape characters
               cnt = 0
               temp1 = ""
               if (len(temp) .gt. 0) then
                  qidx = 1
                  do 
                     pidx = index(temp(qidx:), "'")
                     if (pidx .eq. 0) then 
                        exit
                     end if
                     cnt = cnt + 1
                     oqidx = qidx
                     qidx = qidx + pidx + len("'") - 1
                     !Escape single quote with an extra quote before
                     !Also an @ so it is easy to find when it gets read back out
                     temp1 = trim(temp1) // temp(oqidx:qidx-2) // "@'" &
                                  // temp(qidx-1:qidx-1)
                  end do
                  temp1 = trim(temp1) // temp(qidx:) 
               end if

               !write line to table
               call sqlite3_set_column(cols_p(1), trim(temp1))
               call sqlite3_insert(db, 'qcfreq', cols_p)

               if (db%error.ne.SQLITE_OK) then
                  write(message,'(2a)') &
                        'sql error while writing out Frequency file: ', &
                        db%errmsg
                  call dberrmsg( routine, message )
               endif
            end do
            deallocate(cols_p)
            close(itmpin)

         elseif (sql_vers .eq. 1) then
            !Do nothing does not exist
         endif
      end subroutine dbwrqcfreq

!----------------
      subroutine dberrmsg( routine, message )

      !use global

      Implicit None
      character(len=c3),intent(in) :: routine
      character(len=c5),intent(in) :: message
      integer(long_sql)                :: routlength,messlength,messl1,messl2
      integer(long_sql), external      :: slen

! --- DETERMINE THE LENGTH OF THE ARGUMENTS ---

      messl1=0
      messl2=0
      routlength = slen(routine)
      messlength = slen(message)
      if(messlength .gt. 80) then
         messl1=index(message(70:messlength),' ')+69
         if(messl1.gt.80) messl1=index(message(60:88),' ')+59
         messl1 = slen(message(1:messl1))
         messl2 = messl1
  30     messl2 = messl2+1
         if(index(message(messl2:messlength),' ').eq.1) goto 30
      endif

! --- WRITE ERROR MESSAGE AND CLOSE FILES---

      write(dbsqllog,*)
      write(dbsqllog,*) 'ERROR in subroutine ', routine(1:routlength),' :'
      if(messlength .gt. 80) then
         write(dbsqllog,*) message(1:messl1)
         write(dbsqllog,*) message(messl2:messlength)
      else
         write(dbsqllog,*) message(1:messlength)
      endif
      write(dbsqllog,*)

      if( .not.test0 ) Stop
      Return
      End Subroutine dberrmsg

!-----------------------------------------------------------------------

      subroutine dbdelallentries( db )

!     For this you should be inside a transaction.
!     Use dbstarttrans,... , dbcommittrans
!-----Remove all entries from the database.  We only need to remove from the
!-----dbentry table and it will automatically cascade to delete the entries
!-----from all other tables.
!-----NOTE: if any other program or routine is using this database it may
!-----have cached entry IDs and this will screw everything up.  Please
!-----treat database entries as immutable once written.

      implicit none

      type(SQLITE_DATABASE), intent(inout)      :: db
      character(len=c3) :: routine
      character(len=c5) :: message

      routine = 'dbdelallentries'
      routine = "DBDELallentrioes (db_sql.F90)"

      call sqlite3_do( db, 'DELETE FROM dbentry' )
      if (db%error.ne.SQLITE_OK) then
         write(message,'(2a)') 'sql error while deleting entries: ', &
                               db%errmsg
         call dberrmsg( routine, message )
      endif

      end subroutine dbdelallentries

!#######################################################################
! 
! Here are the routines to CREATE TABLES
!
!Subroutines below will be moved into db_schema after testing  
! (Comment left by Simon Clifford. To do ??)
!
!#######################################################################

      !Create dbentry
      subroutine credbent(db,cmd)
         type(SQLITE_DATABASE), intent(inout)        :: db
         integer(long_sql)                               :: sql_vers  !SQL database version
         character(len=250), intent(out)             :: cmd

         !Table containing the primary key
         !Column ID is the number of the entry
         !status is how it was made, calculated or estimated
         !created is the time it was entered
         sql_vers = db%db_vers
         if (sql_vers .ge. 2) then
            cmd = "CREATE TABLE dbentry (&
                  &id INTEGER PRIMARY KEY,&
                  &status INTEGER NOT NULL,&
                  &created INTEGER NOT NULL );"
            call sqlite3_do(db,cmd)

            cmd = 'CREATE INDEX dbentryidx ON dbentry(id)'
            call sqlite3_do(db,cmd)

         elseif (sql_vers .eq. 1) then
            cmd = "CREATE TABLE dbentry (&
                  &id INTEGER PRIMARY KEY,&
                  &status INTEGER NOT NULL DEFAULT 0);"
            call sqlite3_do(db,cmd)

            cmd = 'CREATE INDEX dbentryidx ON dbentry(id)'
            call sqlite3_do(db,cmd)

         endif

      end subroutine credbent

!-----------------------------------------------------------------------

      !Create table for info on calculation
      subroutine creinfo(db,cmd)
         type(SQLITE_DATABASE), intent(inout)        :: db
         integer(long_sql)                               :: sql_vers  !SQL database version
         character(len=250), intent(out)             :: cmd
         
         sql_vers = db%db_vers
         if (sql_vers .ge. 2) then
            !Table containing info on how db was calculated
            !method - method used to perform calculation (i.e CCSD, CAS)
            !programme - programme used to perform calculation
            cmd = "CREATE TABLE info (&
                  &method TEXT NOT NULL,&
                  &basis TEXT NOT NULL,&
                  &programme TEXT NOT NULL);"
            call sqlite3_do(db,cmd)

         elseif (sql_vers .eq. 1) then
            !Doesn't exist in first version
         endif
         
      end subroutine creinfo

!-----------------------------------------------------------------------

      !Create version table
      subroutine creversion(db,cmd)
         type(SQLITE_DATABASE), intent(inout)        :: db
         integer(long_sql)                           :: sql_vers  !SQL database version
         character(len=250), intent(out)             :: cmd
      sql_vers = db%db_vers
      if (sql_vers .ge. 2) then
         !Table containing the quantics and db version
         cmd = "CREATE TABLE versions (quanticsversion DOUBLE NOT NULL,&
               &dbversion INTEGER NOT NULL,&
               &dbentry INTEGER NOT NULL);"
         call sqlite3_do(db,cmd)

      elseif (sql_vers .eq. 1) then
              cmd = 'CREATE TABLE versions (quanticsversion DOUBLE NOT NULL,&
               &dbversion INTEGER NOT NULL)'
         call sqlite3_do(db,cmd)

      endif


      end subroutine creversion

!-----------------------------------------------------------------------

      !Create refdb table
      subroutine crerefdb(db,cmd)
         type(SQLITE_DATABASE), intent(inout)        :: db
         character(len=350), intent(out)             :: cmd

      !Currently the same for all versions
      !sql_vers is just here for future proofing
         !refdb has key info on the database and calculation
         ! SAME IN OLD AND NEW FORMAT!
         ! Size, only a single entry
         ! Natm - number of atoms in molecule
         ! Nroot - number of states
         ! Nddstate - number of states
         ! Nbasis - number of basis functions in CAS
         ! E0 - the zero of the energy scale
         ! Dimref -
         ! Nfam -
         ! Dercpdim - Number of derivative coupling dimensions
         ! DBpntgp - point group used in calculation
         ! Nsynop - number of symmetry operations
         ! Nsocstate - No. of states including degeneracies
         ! Nmult1 ...7  - No. of states with multiplicity 1....7

         cmd = 'CREATE TABLE refdb (Natm INTEGER NOT NULL,&
                &Nroot INTEGER NOT NULL,&
                &Nstate INTEGER NOT NULL,&
                &Nbasis INTEGER,&
                &E0 DOUBLE,&
                &Dimref INTEGER,&
                &Nfam INTEGER,&
                &Dercpdim INTEGER,&
                &DBpntgp INTEGER,&
                &Nsymop INTEGER,&
                &Nsocstate INTEGER,&  
                &Nmult1 INTEGER,&  
                &Nmult2 INTEGER,&  
                &Nmult3 INTEGER,&  
                &Nmult4 INTEGER,&  
                &Nmult5 INTEGER,&  
                &Nmult6 INTEGER,&  
                &Nmult7 INTEGER);' 
         call sqlite3_do(db,cmd)

      end subroutine crerefdb

!-----------------------------------------------------------------------

      !Create refdbgeom the reference geom
      subroutine crerefdbgeom(db,cmd)
         type(SQLITE_DATABASE), intent(inout)        :: db
         character(len=250), intent(out)             :: cmd
      !Currently the same for all versions
      !sql_vers is just here for future proofing
         !This contains the reference geometry of the molecule
         !SAME IN OLD AND NEW FORMAT!
         !Size, each row is an atom
         !mass - mass of the atom
         !name - name/label of the atom
         !x,y,z - the x,y and z coordinates of the atom


         cmd = "CREATE TABLE refdbrefgeom(atno INTEGER NOT NULL,&
                              &mass DOUBLE NOT NULL,&
                              &name TEXT NOT NULL,&
                              &x DOUBLE NOT NULL,&
                              &y DOUBLE NOT NULL,&
                              &z DOUBLE NOT NULL);"
         
         call sqlite3_do(db,cmd)
         
      end subroutine crerefdbgeom

!-----------------------------------------------------------------------
      
      !Create the symmetry reference db
      subroutine crerefsym(db,natom,cmd)
         type(SQLITE_DATABASE), intent(inout)        :: db
         integer(long_sql)                               :: sql_vers  !SQL database version
         integer(long_sql), intent(in)                   :: natom
         character(len=250), intent(out)             :: cmd
         character(len=50)                           :: bstr
         integer                                     :: natm
         !create refdbsymmaps table
         !Contains the symmetry relationships
         !Size, a row per symmetry operation
         !Column size changes with number of atoms
         !Symopno - The number of the symmetry operation
         !atmi - the mapping of atom number i for given symopno
         sql_vers = db%db_vers
         if (sql_vers .eq. 2) then
            cmd = "CREATE TABLE refdbsymmaps (&
                  &Symopno INTEGER PRIMARY KEY"
            do natm = 1, natom
               write(bstr,"(A4,I0)") trim(",atm") , natm
               cmd = trim(cmd) // trim(bstr)// " INTEGER"  
            enddo
            cmd = trim(cmd) // ");"
            call sqlite3_do(db,cmd)
       
         else
            cmd = 'CREATE TABLE refdbsymmaps(Symopno INTEGER,&
                  & Natm INTEGER, DBsymmap INTEGER);'
            call sqlite3_do(db,cmd)

         endif

      end subroutine crerefsym

!-----------------------------------------------------------------------
      
      !Create the symmetry reference db
      subroutine cresymrep(db,nsymop,cmd)
         type(SQLITE_DATABASE), intent(inout)        :: db
         integer(long_sql)                               :: sql_vers  !SQL database version
         integer(long_sql), intent(in)                   :: nsymop
         character(len=500), intent(out)             :: cmd
         character(len=50)                           :: bstr
         integer                                     :: nsym
         !create symrep table
         !Contains the symmetry relationships
         !Size, a row per entry
         !Column size changes with number of sym ops
         !op_i the symrep for that sym
         !NOTE UNTESTED WITH REAL DATA

         sql_vers = db%db_vers
         if ((sql_vers .eq. 2) .or. (sql_vers .ge. 4)) then
            cmd = 'CREATE TABLE symrep (&
                  &id INTEGER NOT NULL &
                  & REFERENCES dbentry(id) &
                  & ON UPDATE CASCADE ON DELETE CASCADE'
            do nsym = 1, nsymop
               write(bstr,'(A4,I0,A17)') ",op_", nsym, "INTEGER NOT NULL"
               cmd = trim(cmd) // trim(bstr)
            enddo
            cmd = trim(cmd) // ");" 
            call sqlite3_do(db,cmd)

         else
            cmd = 'CREATE TABLE symrep (id INTEGER NOT NULL REFERENCES dbentry(id)&
                  &ON UPDATE CASCADE ON DELETE CASCADE,&
                  &nsymop INTEGER NOT NULL,&
                  &value INTEGER NOT NULL);'
            call sqlite3_do(db,cmd)

         endif

      end subroutine cresymrep

!-----------------------------------------------------------------------
      
      !Create geometry table
      subroutine cregeo(db,natom,cmd)
         type(SQLITE_DATABASE), intent(inout)        :: db
         integer(long_sql)                               :: sql_vers  !SQL database version
         integer(long_sql), intent(in)                   :: natom
! SQLITE_MAX_COLUMN = 2000 by default
         character(len=20000), intent(out)           :: cmd
         integer                                     :: natm
         character(len=150)                          :: bstr

         sql_vers = db%db_vers
         if (sql_vers .eq. 3) then
            cmd = 'CREATE TABLE geo (id INTEGER NOT NULL REFERENCES dbentry(id)&
                  &ON UPDATE CASCADE ON DELETE CASCADE,&
                  &ndof INTEGER NOT NULL,&
                  &coord TEXT NOT NULL,&
                  &value DOUBLE NOT NULL);'
            call sqlite3_do( db, cmd )
            cmd = 'CREATE INDEX geoidx ON geo(id)'
            call sqlite3_do(db,cmd)

         else if ((sql_vers .eq. 2) .or. (sql_vers .ge. 4)) then
            !Contains geometries for each DB entry in cartesian
            !Size, a row for each DB entry
            !Column number changes based on number of atoms, for each atom have 3 columns
            !x_i,y_i,z_i - the x,y and z coordinates of the atom, i is the atom number
            cmd = 'CREATE TABLE geo (&
                  &id INTEGER NOT NULL &
                  & REFERENCES dbentry(id) &
                  & ON UPDATE CASCADE ON DELETE CASCADE'
                        
            do natm = 1, natom
               write(bstr,"(A4,I0,A20)") ",x_" , natm, "  DOUBLE NOT NULL,"
               cmd = trim(cmd) // trim(bstr)
               write(bstr,"(A4,I0,A20)") "y_" , natm, "  DOUBLE NOT NULL,"
               cmd = trim(cmd) // trim(bstr)
               write(bstr,"(A4,I0,A20)") "z_" , natm, "  DOUBLE NOT NULL"
               cmd = trim(cmd) // trim(bstr)
            enddo
            cmd = trim(cmd) // ");" 
            call sqlite3_do(db,cmd)

         else if (sql_vers .eq. 1) then
            cmd = 'CREATE TABLE geo (id INTEGER NOT NULL REFERENCES dbentry(id)&
                  &ON UPDATE CASCADE ON DELETE CASCADE,&
                  &ndof INTEGER NOT NULL,&
                  &value DOUBLE NOT NULL);'
            call sqlite3_do( db, cmd )
            cmd = 'CREATE INDEX geoidx ON geo(id)'
            call sqlite3_do(db,cmd)

         endif

      end subroutine cregeo

      !Create template file table
      subroutine cretemplate(db,cmd)
         type(SQLITE_DATABASE), intent(inout)        :: db
         integer(long_sql)                               :: sql_vers  !SQL database version
         character(len=1000), intent(out)            :: cmd

         sql_vers = db%db_vers
         if (sql_vers .ge. 4) then
            !Create the table
            cmd ="CREATE TABLE template (&
                  &contents TEXT NOT NULL);"

            call sqlite3_do(db,cmd)
         elseif (sql_vers .eq. 1) then
            !Do nothing doesn't exist
         endif
      end subroutine cretemplate

      !Create the QC file table
      subroutine creqcfreq(db,cmd)
         type(SQLITE_DATABASE), intent(inout)        :: db
         integer(long_sql)                               :: sql_vers  !SQL database version
         character(len=1000), intent(out)            :: cmd
         
         sql_vers = db%db_vers
         if (sql_vers .ge. 4) then
            !Create the table
            cmd ="CREATE TABLE qcfreq (&
                  &contents TEXT NOT NULL);"

            call sqlite3_do(db,cmd)
         elseif (sql_vers .eq. 1) then
            !Do nothing doesn't exist
         endif
      end subroutine creqcfreq

      !Create nact table - this is in cart like GEO
      subroutine crenact(db,natom,cmd)
         type(SQLITE_DATABASE), intent(inout)        :: db
         integer(long_sql)                               :: sql_vers  !SQL database version
         integer(long_sql), intent(in)                   :: natom
! SQLITE_MAX_COLUMN = 2000 by default
         character(len=20000), intent(out)           :: cmd
         integer                                     :: nact
         character(len=150)                          :: bstr
      
         sql_vers =db%db_vers

         if (sql_vers .ge. 4) then
            !Contains geometries for each DB entry in cartesian
            !Size, a row for each DB entry
            !Column number changes based on number of atoms, for each atom have 3 columns
            !x_i,y_i,z_i - the x,y and z coordinates of the atom, i is the atom number
            cmd = 'CREATE TABLE nact (&
                  &id INTEGER NOT NULL &
                  &REFERENCES dbentry(id)&
                  &ON UPDATE CASCADE ON DELETE CASCADE,&
                  &dercp INTEGER NOT NULL,&
                  &state1 INTEGER NOT NULL,& 
                  &state2 INTEGER NOT NULL'
            do nact = 1, natom
               write(bstr,"(A4,I0,A20)") ",x_" , nact, "  DOUBLE NOT NULL,"
               cmd = trim(cmd) // trim(bstr)
               write(bstr,"(A4,I0,A20)") "y_" , nact, "  DOUBLE NOT NULL,"
               cmd = trim(cmd) // trim(bstr)
               write(bstr,"(A4,I0,A20)") "z_" , nact, "  DOUBLE NOT NULL"
               cmd = trim(cmd) // trim(bstr)
            enddo
            cmd = trim(cmd) // ");"
            call sqlite3_do(db,cmd)

         else if (sql_vers .eq. 3) then
            cmd = 'CREATE TABLE nact (id INTEGER NOT NULL REFERENCES dbentry(id)&
                  &ON UPDATE CASCADE ON DELETE CASCADE,&
                  &dercp INTEGER NOT NULL,&
                  &ndof INTEGER NOT NULL,&
                  &state1 INTEGER NOT NULL,&
                  &state2 INTEGER NOT NULL,&
                  &coord TEXT NOT NULL,&
                  &value DOUBLE NOT NULL);'
            call sqlite3_do( db, cmd )
            cmd = 'CREATE INDEX nactidx ON nact(id)'
            call sqlite3_do( db, cmd )

         else if (sql_vers .eq. 2) then
            !Contains geometries for each DB entry in cartesian
            !Size, a row for each DB entry
            !Column number changes based on number of atoms, for each atom have 3 columns
            !x_i,y_i,z_i - the x,y and z coordinates of the atom, i is the atom number
            cmd = 'CREATE TABLE nact (&
                  &id INTEGER NOT NULL &
                  &REFERENCES dbentry(id)&
                  &ON UPDATE CASCADE ON DELETE CASCADE, &
                  &dercp INTEGER NOT NULL'
            do nact = 1, natom
               write(bstr,"(A4,I0,A20)") ",x_" , nact, "  DOUBLE NOT NULL,"
               cmd = trim(cmd) // trim(bstr)
               write(bstr,"(A4,I0,A20)") "y_" , nact, "  DOUBLE NOT NULL,"
               cmd = trim(cmd) // trim(bstr)
               write(bstr,"(A4,I0,A20)") "z_" , nact, "  DOUBLE NOT NULL"
               cmd = trim(cmd) // trim(bstr)
            enddo
            cmd = trim(cmd) // ");"
            call sqlite3_do(db,cmd)

         else
            cmd = 'CREATE TABLE nact (id INTEGER NOT NULL REFERENCES dbentry(id)&
                  &ON UPDATE CASCADE ON DELETE CASCADE,&
                  &dercp INTEGER NOT NULL,&
                  &dof INTEGER NOT NULL,&
                  &value DOUBLE NOT NULL);'
            call sqlite3_do( db, cmd )
            cmd = 'CREATE INDEX nactidx ON nact(id)'
            call sqlite3_do( db, cmd )

         endif

      end subroutine crenact

!-----------------------------------------------------------------------

      !Create PES table
      subroutine crepes(db,nstate,cmd)
         type(SQLITE_DATABASE), intent(inout)        :: db
         integer(long_sql)                :: sql_vers  !SQL database version
         integer(long_sql), intent(in)                   :: nstate
! SQLITE_MAX_COLUMN = 2000 by default
         character(len=20000), intent(out)           :: cmd
         integer                                     :: state1, state2
         character(len=150)                          :: bstr
      
      sql_vers = db%db_vers
      if (sql_vers .eq. 3) then
         cmd = 'CREATE TABLE pes (id INTEGER NOT NULL REFERENCES dbentry(id)&
               &ON UPDATE CASCADE ON DELETE CASCADE,&
               &state1 INTEGER NOT NULL,&
               &state2 INTEGER NOT NULL,&
               &energy DOUBLE NOT NULL)'
         call sqlite3_do( db, cmd )
         cmd = 'CREATE INDEX pesidx ON pes(id)'
         call sqlite3_do( db, cmd )

      else if ((sql_vers .eq. 2) .or. (sql_vers .ge. 4)) then
         !make pes table
         !Contains the PES energies for each DB entry
         !Size, a row for DB entry
         !Column number changes based on number of electronic states
         !Have a column for each state and the coupling between them
         !eng_i_j -   the energy for states i & j (only i & j stored not j & i)
         cmd = "CREATE TABLE pes (&
               &id INTEGER NOT NULL &
               &REFERENCES dbentry(id)&
               &ON UPDATE CASCADE ON DELETE CASCADE"
         do state1 = 1, nstate
            do state2 = state1, nstate
               write(bstr,"(A5,I0,A1,I0,A20)") ",E_" , state1,"_",state2,&
                     "  DOUBLE NOT NULL"
               cmd = trim(cmd) // trim(bstr)
            enddo
         enddo
         cmd = trim(cmd) // ")"
         call sqlite3_do(db,cmd)  

      else
         cmd = 'CREATE TABLE pes (id INTEGER NOT NULL REFERENCES dbentry(id)&
               &ON UPDATE CASCADE ON DELETE CASCADE,&
               &state1 INTEGER NOT NULL,&
               &state2 INTEGER NOT NULL,&
               &compdim INTEGER NOT NULL,&
               &energy DOUBLE NOT NULL)'
         call sqlite3_do( db, cmd )
         cmd = 'CREATE INDEX pesidx ON pes(id)'
         call sqlite3_do( db, cmd )

      end if
      
      end subroutine crepes
      
!-----------------------------------------------------------------------

      !Create complex PES table
      subroutine crecpes(db,nstate,cmd)
         type(SQLITE_DATABASE), intent(inout)        :: db
         integer(long_sql)                               :: sql_vers  !SQL database version
         integer(long_sql), intent(in)                   :: nstate
! SQLITE_MAX_COLUMN = 2000 by default
         character(len=20000), intent(out)           :: cmd
         integer                                     :: state1, state2
         character(len=150)                          :: bstr
      
         sql_vers = db%db_vers

      if (sql_vers .eq. 3) then
         cmd = 'CREATE TABLE cpes (id INTEGER NOT NULL REFERENCES dbentry(id)&
               &ON UPDATE CASCADE ON DELETE CASCADE,&
               &state1 INTEGER NOT NULL,&
               &state2 INTEGER NOT NULL,&
               &energy DOUBLE NOT NULL)'
         call sqlite3_do( db, cmd )
         cmd = 'CREATE INDEX cpesidx ON cpes(id)'
         call sqlite3_do( db, cmd )

      else if ((sql_vers .eq. 2) .or. (sql_vers .ge. 4)) then
         !make complex pes table
         !Contains the PES energies for each DB entry
         !Size, a row for DB entry
         !Column number changes based on number of electronic states
         !Have a column for each state and the coupling between them
         !eng_i_j -   the energy for states i & j (only i & j stored not j & i)
         cmd = 'CREATE TABLE cpes (&
               &id INTEGER NOT NULL &
               &REFERENCES dbentry(id)&
               &ON UPDATE CASCADE ON DELETE CASCADE'
         do state1 = 1, nstate
            do state2 = state1, nstate
               write(bstr,"(A5,I0,A1,I0,A20)") ",CE_" , state1,"_",state2,&
                     "  DOUBLE NOT NULL"
               cmd = trim(cmd) // trim(bstr)
            enddo
         enddo
         cmd = trim(cmd) // ")"

      elseif (sql_vers .eq. 1) then
         continue
         !This is not a separate table in version 1
      end if

      call sqlite3_do(db,cmd)  
      end subroutine crecpes

!-----------------------------------------------------------------------

      !Create adiabatic PES table
      subroutine creapes(db,nstate,cmd)
         type(SQLITE_DATABASE), intent(inout)        :: db
         integer(long_sql)                  :: sql_vers  !SQL database version
         integer(long_sql), intent(in)                   :: nstate
! SQLITE_MAX_COLUMN = 2000 by default
         character(len=20000), intent(out)           :: cmd
         integer                                     :: state1
         character(len=150)                          :: bstr

         sql_vers = db%db_vers

         if ((sql_vers .eq. 2) .or. (sql_vers .ge. 4)) then
            !make pes table
            !Contains the adiabatic PES energies for each DB entry
            !Size, a row for DB entry
            !Column number changes based on number of electronic states
            !Have a column for each state and the coupling between them
            !eng_i -   the energy for state i
            cmd = 'CREATE TABLE apes (&
                  &id INTEGER NOT NULL &
                  &REFERENCES dbentry(id)&
                  &ON UPDATE CASCADE ON DELETE CASCADE'
            do state1 = 1, nstate
               write(bstr,"(A5,I0,A20)") ",E_" , state1,"  DOUBLE NOT NULL"
               cmd = trim(cmd) // trim(bstr)
            enddo
            cmd = trim(cmd) // ")"
            call sqlite3_do(db,cmd) 
   
         else
            cmd = 'CREATE TABLE apes (id INTEGER NOT NULL REFERENCES dbentry(id)&
                  &ON UPDATE CASCADE ON DELETE CASCADE,&
                  &state1 INTEGER NOT NULL,&
                  &state2 INTEGER NOT NULL,&
                  &energy DOUBLE NOT NULL)'
            call sqlite3_do( db, cmd )
            cmd = 'CREATE INDEX apesidx ON apes(id)'
            call sqlite3_do( db, cmd )
         end if

      end subroutine creapes

!-----------------------------------------------------------------------

      !Create normal modes table
      subroutine crenmodes(db,nmodes,cmd)
         type(SQLITE_DATABASE), intent(inout)        :: db
         integer(long_sql)                   :: sql_vers  !SQL database version
         integer(long_sql), intent(in)                   :: nmodes
! SQLITE_MAX_COLUMN = 2000 by default
         character(len=20000), intent(out)           :: cmd
         integer                                     :: nmm
         character(len=150)                          :: bstr
         
         sql_vers = db%db_vers
         
         if ((sql_vers .eq. 2) .or. (sql_vers .ge. 4)) then
            !make Nmode table
            !Contains geometries for each DB entry in normal modes
            !Size, a row for each DB entry
            !Column number changes based on number of atoms,
            !have 3N-6 (for nonlinear molecules) normal modes
            !nmdof_i - the normal mode coordinate i is the atom number
            cmd = 'CREATE TABLE nmode (&
                  &id INTEGER NOT NULL &
                  &REFERENCES dbentry(id)&
                  &ON UPDATE CASCADE ON DELETE CASCADE'
            do nmm = 1, nmodes
               write(bstr,"(A6,I0,A20)") ",nmdof_" , nmm,"  DOUBLE NOT NULL"
               cmd = trim(cmd) // trim(bstr)
            enddo
            cmd = trim(cmd) // ")"
            call sqlite3_do(db,cmd) 

         else
            cmd = 'CREATE TABLE nmode (id INTEGER NOT NULL REFERENCES dbentry(id)&
                  &ON UPDATE CASCADE ON DELETE CASCADE,&
                  &nmdof INTEGER NOT NULL,&
                  &value DOUBLE NOT NULL)'
            call sqlite3_do( db, cmd )
            cmd = 'CREATE INDEX nmidx ON nmode(id)'
            call sqlite3_do( db, cmd )

         endif 

      end subroutine crenmodes

!-----------------------------------------------------------------------

      !Create Trans table
      subroutine cretrans(db,nstate,cmd)
         type(SQLITE_DATABASE), intent(inout)        :: db
         integer(long_sql)                               :: sql_vers  !SQL database version
         integer(long_sql), intent(in)                   :: nstate
! SQLITE_MAX_COLUMN = 2000 by default
         character(len=20000), intent(out)           :: cmd
         integer                                     :: state1, state2
         character(len=150)                          :: bstr
      
         sql_vers = db%db_vers
         if (sql_vers .eq. 3) then
            cmd = 'CREATE TABLE trans(id INTEGER NOT NULL REFERENCES dbentry(id)&
                  &ON UPDATE CASCADE ON DELETE CASCADE,&
                  &state1 INTEGER NOT NULL,&
                  &state2 INTEGER NOT NULL,&
                  &value DOUBLE NOT NULL)'
            call sqlite3_do( db, cmd )
            cmd = 'CREATE INDEX transidx ON trans(id)'
            call sqlite3_do( db, cmd )

         else if ((sql_vers .eq. 2) .or. (sql_vers .ge. 4)) then
            !make trans table
            !Contains trans data
            !Size, a row for each DB entry
            !Column number changes based on number of electronic states
            !Have a column for each state and the coupling between them
            !tran_i_j -   the trans between states i & j (only i & j stored not j & i)
            cmd = 'CREATE TABLE trans (&
                  &id INTEGER NOT NULL &
                  &REFERENCES dbentry(id)&
                  &ON UPDATE CASCADE ON DELETE CASCADE'
            do state1 = 1, nstate
               do state2 = 1, nstate
                  write(bstr,"(A5,I0,A1,I0,A20)") ",tran_" , state2,"_",&
                        state1, "  DOUBLE NOT NULL"
                  cmd = trim(cmd) // trim(bstr)
               enddo
            enddo
            cmd = trim(cmd) // ")"
            call sqlite3_do(db,cmd)

         else
            cmd = 'CREATE TABLE trans(id INTEGER NOT NULL REFERENCES dbentry(id)&
                  &ON UPDATE CASCADE ON DELETE CASCADE,&
                  &state1 INTEGER NOT NULL,&
                  &state2 INTEGER NOT NULL,&
                  &compdim INTEGER NOT NULL,&
                  &value DOUBLE NOT NULL)'
            call sqlite3_do( db, cmd )
            cmd = 'CREATE INDEX transidx ON trans(id)'
            call sqlite3_do( db, cmd )

         end if

      end subroutine cretrans

!-----------------------------------------------------------------------

      !Create Complex Trans table
      subroutine crectrans(db,nstate,cmd)
         type(SQLITE_DATABASE), intent(inout)        :: db
         integer(long_sql)                               :: sql_vers  !SQL database version
         integer(long_sql), intent(in)                   :: nstate
! SQLITE_MAX_COLUMN = 2000 by default
         character(len=20000), intent(out)           :: cmd
         integer                                     :: state1, state2
         character(len=150)                          :: bstr
      
         sql_vers = db%db_vers
         if (sql_vers .eq. 3) then
            cmd = 'CREATE TABLE ctrans(id INTEGER NOT NULL REFERENCES dbentry(id)&
                  &ON UPDATE CASCADE ON DELETE CASCADE,&
                  &state1 INTEGER NOT NULL,&
                  &state2 INTEGER NOT NULL,&
                  &value DOUBLE NOT NULL)'
            call sqlite3_do( db, cmd )
            cmd = 'CREATE INDEX ctransidx ON ctrans(id)'
            call sqlite3_do( db, cmd )

         else if ((sql_vers .eq. 2) .or. (sql_vers .ge. 4)) then
            !make ctrans table
            !Contains ctrans data
            !Size, a row for each DB entry
            !Column number changes based on number of electronic states
            !Have a column for each state and the coupling between them
            !tran_i_j -   the trans between states i & j (only i & j stored not j & i)
            cmd = 'CREATE TABLE ctrans (&
                  &id INTEGER NOT NULL &
                  &REFERENCES dbentry(id)&
                  &ON UPDATE CASCADE ON DELETE CASCADE'
            do state1 = 1, nstate
               do state2 = 1, nstate
                  write(bstr,"(A5,I0,A1,I0,A20)") ",tran_" , state2,"_",&
                        state1, "  DOUBLE NOT NULL"
                  cmd = trim(cmd) // trim(bstr)
               enddo
            enddo
            cmd = trim(cmd) // ")"
            call sqlite3_do(db,cmd)

         else
            cmd = 'CREATE TABLE ctrans(id INTEGER NOT NULL REFERENCES dbentry(id)&
                  &ON UPDATE CASCADE ON DELETE CASCADE,&
                  &state1 INTEGER NOT NULL,&
                  &state2 INTEGER NOT NULL,&
                  &compdim INTEGER NOT NULL,&
                  &value DOUBLE NOT NULL)'
            call sqlite3_do( db, cmd )
            cmd = 'CREATE INDEX ctransidx ON ctrans(id)'
            call sqlite3_do( db, cmd )

         end if

      end subroutine crectrans

!-----------------------------------------------------------------------

      !Create gradient array
      subroutine cregrad(db,nstate,natoms,cmd)
         type(SQLITE_DATABASE), intent(inout)        :: db
         integer(long_sql)                               :: sql_vers  !SQL database version
         integer(long_sql), intent(in)                   :: nstate
         integer(long_sql),intent(in)                    :: natoms
! SQLITE_MAX_COLUMN = 2000 by default
         character(len=20000), intent(out)           :: cmd
         integer                                     :: state1, state2, coo
         character(len=150)                          :: bstr
      
         sql_vers = db%db_vers
         if (sql_vers .ge. 4) then
            !make grad table
            !Contains PES gradient values in cartesian
            !Size, a row for each DB entry
            !Column number changes based on number of electronic states and atoms
            !Have a column for each state and coupling between them for each atom
            !x_i_j_k, y_i_j_k, z_i_j_k - the gradient in cartesian for 
            !states i & j with atom k
            cmd = 'CREATE TABLE gra (&
                   &id INTEGER NOT NULL &
                   &REFERENCES dbentry(id)&
                   &ON UPDATE CASCADE ON DELETE CASCADE,&
                   &state1 INTEGER NOT NULL,&
                   &state2 INTEGER NOT NULL' 
            do coo = 1, natoms
               write(bstr,"(A3,I0,A20)") &
                     ",x_" ,coo, "  DOUBLE NOT NULL,"
               cmd = trim(cmd) // trim(bstr)
               write(bstr,"(A2,I0,A20)") &
                     "y_" ,coo, "  DOUBLE NOT NULL,"
               cmd = trim(cmd) // trim(bstr)
               write(bstr,"(A2,I0,A20)") &
                     "z_" ,coo, "  DOUBLE NOT NULL"
               cmd = trim(cmd) // trim(bstr)
            enddo
            cmd = trim(cmd) // ")"
            call sqlite3_do(db,cmd)

         else if (sql_vers .eq. 3) then
            cmd = 'CREATE TABLE gra (id INTEGER NOT NULL REFERENCES dbentry(id)&
                  &ON UPDATE CASCADE ON DELETE CASCADE,&
                  &state1 INTEGER NOT NULL,&
                  &state2 INTEGER NOT NULL,&
                  &ndof INTEGER NOT NULL,&
                  &coord TEXT NOT NULL,&
                  &value DOUBLE NOT NULL)'
            call sqlite3_do( db, cmd )
            cmd = 'CREATE INDEX graidx ON gra(id)'
            call sqlite3_do( db, cmd )

         else if (sql_vers .eq. 2) then
            !make grad table
            !Contains PES gradient values in cartesian
            !Size, a row for each DB entry
            !Column number changes based on number of electronic states and atoms
            !Have a column for each state and coupling between them for each atom
            !x_i_j_k, y_i_j_k, z_i_j_k - the gradient in cartesian for 
            !states i & j with atom k
            cmd = 'CREATE TABLE gra (&
                  &id INTEGER NOT NULL &
                  &REFERENCES dbentry(id)&
                  &ON UPDATE CASCADE ON DELETE CASCADE'
            do state1 = 1, nstate
               do state2 = state1, nstate
                  do coo = 1, natoms
                     write(bstr,"(A4,I0,A1,I0,A1,I0,A20)") &
                           ",x_" , state1,"_", state2,"_",coo, "  DOUBLE NOT NULL,"
                     cmd = trim(cmd) // trim(bstr)
                     write(bstr,"(A4,I0,A1,I0,A1,I0,A20)") &
                           "y_" , state1,"_", state2,"_",coo, "  DOUBLE NOT NULL,"
                     cmd = trim(cmd) // trim(bstr)
                     write(bstr,"(A4,I0,A1,I0,A1,I0,A20)") &
                           "z_" , state1,"_", state2,"_",coo, "  DOUBLE NOT NULL"
                     cmd = trim(cmd) // trim(bstr)
                  enddo
               enddo 
            enddo
            cmd = trim(cmd) // ")"
            call sqlite3_do(db,cmd)

         else if (sql_vers .eq. 1) then
            cmd = 'CREATE TABLE gra (id INTEGER NOT NULL REFERENCES dbentry(id)&
                  &ON UPDATE CASCADE ON DELETE CASCADE,&
                  &state1 INTEGER NOT NULL,&
                  &state2 INTEGER NOT NULL,&
                  &ndof INTEGER NOT NULL,&
                  &value DOUBLE NOT NULL)'
            call sqlite3_do( db, cmd )
            cmd = 'CREATE INDEX graidx ON gra(id)'
            call sqlite3_do( db, cmd )

         end if
            
      end subroutine cregrad

!-----------------------------------------------------------------------

      !Create adiabatic gradient
      subroutine creagrad(db,nstate,natoms,cmd)
         type(SQLITE_DATABASE), intent(inout)        :: db
         integer(long_sql)                               :: sql_vers  !SQL database version
         integer(long_sql), intent(in)                   :: nstate
         integer(long_sql),intent(in)                    :: natoms
! SQLITE_MAX_COLUMN = 2000 by default
         character(len=20000), intent(out)           :: cmd
         integer                                     :: state1, coo
         character(len=150)                          :: bstr
      
         sql_vers = db%db_vers
         if (sql_vers .ge. 4) then
            !make agrad table
            !Contains adiabatic PES gradient values in cartesian
            !Size, a row for each DB entry
            !Column number changes based on number of electronic states and atoms
            !Have a column for for each atom for each state
            !k  x_i, y_i, z_i - the gradient in cartesian for state i with atom k
            cmd = 'CREATE TABLE agra (&
                  &id INTEGER NOT NULL &
                  &REFERENCES dbentry(id) &
                  &ON UPDATE CASCADE ON DELETE CASCADE,&
                  &state INTEGER NOT NULL'
            do coo = 1, natoms
               write(bstr,"(A3,I0,A20)") &
                     ",x_",coo, "  DOUBLE NOT NULL,"
               cmd = trim(cmd) // trim(bstr)
               write(bstr,"(A2,I0,A20)") &
                     "y_",coo, "  DOUBLE NOT NULL,"
               cmd = trim(cmd) // trim(bstr)
               write(bstr,"(A2,I0,A20)") &
                     "z_",coo, "  DOUBLE NOT NULL"
               cmd = trim(cmd) // trim(bstr)
            enddo 
            cmd = trim(cmd) // ")"
            call sqlite3_do(db,cmd)   

         else if (sql_vers .eq. 3) then
            cmd = 'CREATE TABLE agra (id INTEGER NOT NULL REFERENCES dbentry(id)&
                  &ON UPDATE CASCADE ON DELETE CASCADE,&
                  &state1 INTEGER NOT NULL,&
                  &ndof INTEGER NOT NULL,&
                  &coord TEXT NOT NULL,&
                  &value DOUBLE NOT NULL)'
            call sqlite3_do( db, cmd )
            cmd = 'CREATE INDEX agraidx ON agra(id)'
            call sqlite3_do( db, cmd )

         else if (sql_vers .eq. 2) then
            !make agrad table
            !Contains adiabatic PES gradient values in cartesian
            !Size, a row for each DB entry
            !Column number changes based on number of electronic states and atoms
            !Have a column for each state for each atom
            !x_i_k, y_i_k, z_i_k - the gradient in cartesian for state i with atom k
            cmd = 'CREATE TABLE agra (&
                  &id INTEGER NOT NULL &
                  &REFERENCES dbentry(id)&
                  &ON UPDATE CASCADE ON DELETE CASCADE'
            do state1 = 1, nstate
               do coo = 1, natoms
                  write(bstr,"(A4,I0,A1,I0,A20)") &
                        ",x_" , state1,"_",coo, "  DOUBLE NOT NULL,"
                  cmd = trim(cmd) // trim(bstr)
                  write(bstr,"(A4,I0,A1,I0,A20)") &
                        "y_" , state1,"_",coo, "  DOUBLE NOT NULL,"
                  cmd = trim(cmd) // trim(bstr)
                  write(bstr,"(A4,I0,A1,I0,A20)") &
                        "z_" , state1,"_",coo, "  DOUBLE NOT NULL"
                  cmd = trim(cmd) // trim(bstr)
               enddo 
            enddo
            cmd = trim(cmd) // ")"
            call sqlite3_do(db,cmd)   

         else if (sql_vers .eq. 1) then
            cmd = 'CREATE TABLE agra (id INTEGER NOT NULL REFERENCES dbentry(id)&
                  &ON UPDATE CASCADE ON DELETE CASCADE,&
                  &state1 INTEGER NOT NULL,&
                  &ndof INTEGER NOT NULL,&
                  &value DOUBLE NOT NULL)'
            call sqlite3_do( db, cmd )
            cmd = 'CREATE INDEX agraidx ON agra(id)'
            call sqlite3_do( db, cmd )

         end if

      end subroutine creagrad

!-----------------------------------------------------------------------

      !Create Hessian
      subroutine crehess(db,sql_vers,nstate,natoms,cmd)
         type(SQLITE_DATABASE), intent(inout)        :: db
         integer(long_sql), intent(in)                   :: sql_vers  !SQL database version
         integer(long_sql), intent(in)                   :: nstate
         integer(long_sql),intent(in)                    :: natoms
! SQLITE_MAX_COLUMN = 2000 by default          
         character(len=20000), intent(out)           :: cmd
         integer                                     :: state1, state2, coo1, coo2
         character(len=150)                          :: bstr
      
         if (sql_vers .ge. 4) then
            !Create a single table
            !Each row is all DOFs for a single pair of electronic states
            !Create base for table
            cmd = "CREATE TABLE hes (id INTEGER NOT NULL REFERENCES dbentry(id)&
                  & ON UPDATE CASCADE ON DELETE CASCADE,&
                  & state1 INTEGER NOT NULL,&
                  & state2 INTEGER NOT NULL,&
                  & coord1 TEXT NOT NULL"
                  !Now loop over 2nd coord
                  do coo2 = 1, natoms
                     write(bstr,"(A4,I0,A20)") &
                         ",x_",coo2, "  DOUBLE NOT NULL,"
                     cmd = trim(cmd) // trim(bstr)
               
                     write(bstr,"(A3,I0,A20)") &
                         "y_",coo2,"  DOUBLE NOT NULL,"
                     cmd = trim(cmd) // trim(bstr)

                     write(bstr,"(A3,I0,A20)") &
                         "z_",coo2,"  DOUBLE NOT NULL"
                     cmd = trim(cmd) // trim(bstr)
                  end do
            cmd = trim(cmd) // ")"
            call sqlite3_do(db,cmd) 
            if (db%error.ne.SQLITE_OK) then
               write(*,'(2a)') 'sql error while setting Hess: ', &
                                     db%errmsg
            end if
            cmd = 'CREATE INDEX hesidx ON hes(id)'
            call sqlite3_do( db, cmd )

         else if (sql_vers .eq. 3) then
            cmd = 'CREATE TABLE hes (id INTEGER NOT NULL REFERENCES dbentry(id)&
                  &ON UPDATE CASCADE ON DELETE CASCADE,&
                  &state1 INTEGER NOT NULL,&
                  &state2 INTEGER NOT NULL,&
                  &ndof1 INTEGER NOT NULL,&
                  &ndof2 INTEGER NOT NULL,&
                  &coord1 TEXT NOT NULL,&
                  &coord2 TEXT NOT NULL,&
                  &value DOUBLE NOT NULL)'
                  call sqlite3_do( db, cmd )
            cmd = 'CREATE INDEX hesidx ON hes(id)'
            call sqlite3_do( db, cmd )

         else if (sql_vers .eq. 2) then
            !Also create new hes table with one entry per electronic state pair
            ! (lower triangle). Contains the Hessian values in cartesian
            !Size, a row for each DB entry a table for each electronic state
            !Have a column for each state 
            !x_k_l, y1_i_k_l, z1_i_k_l - first hessian entries for atoms k & l

            !Create table per electronic state

            do state1 = 1, nstate
               !Now loop over states
               !Create the table
               do state2 = state1, nstate
                  write(cmd,"(A17,I0,a1,i0,A80)") &
                 "CREATE TABLE hes_", state2 ,"_",state1,&
                 " (id INTEGER NOT NULL REFERENCES dbentry(id)&
                  &ON UPDATE CASCADE ON DELETE CASCADE"
                  !Now loop over 1st coord
                  do coo1 = 1, natoms
                     !Now loop over 2nd coord
                     do coo2 = 1, natoms
                        write(bstr,"(A4,I0,A3,I0,A20)") &
                            ",x_",coo2,"_x_", coo1, "  DOUBLE NOT NULL,"
                        cmd = trim(cmd) // trim(bstr)
                        write(bstr,"(A4,I0,A3,I0,A20)") &
                            "y_" ,coo2,"_x_", coo1, "  DOUBLE NOT NULL,"
                        cmd = trim(cmd) // trim(bstr)
                        write(bstr,"(A4,I0,A3,I0,A20)") &
                            "z_" ,coo2,"_x_", coo1, "  DOUBLE NOT NULL"
                        cmd = trim(cmd) // trim(bstr)
                     enddo
                     do coo2 = 1, natoms
                        write(bstr,"(A4,I0,A3,I0,A20)") &
                            ",x_",coo2,"_y_", coo1, "  DOUBLE NOT NULL,"
                        cmd = trim(cmd) // trim(bstr)
                        write(bstr,"(A4,I0,A3,I0,A20)") &
                            "y_" ,coo2,"_y_", coo1, "  DOUBLE NOT NULL,"
                        cmd = trim(cmd) // trim(bstr)
                        write(bstr,"(A4,I0,A3,I0,A20)") &
                            "z_" ,coo2,"_y_", coo1, "  DOUBLE NOT NULL"
                        cmd = trim(cmd) // trim(bstr)
                     enddo
                     do coo2 = 1, natoms
                        write(bstr,"(A4,I0,A3,I0,A20)") &
                            ",x_",coo2,"_z_", coo1, "  DOUBLE NOT NULL,"
                        cmd = trim(cmd) // trim(bstr)
                        write(bstr,"(A4,I0,A3,I0,A20)") &
                            "y_" ,coo2,"_z_", coo1, "  DOUBLE NOT NULL,"
                        cmd = trim(cmd) // trim(bstr)
                        write(bstr,"(A4,I0,A3,I0,A20)") &
                            "z_" ,coo2,"_z_", coo1, "  DOUBLE NOT NULL"
                        cmd = trim(cmd) // trim(bstr)
                     enddo
                  enddo
                  cmd = trim(cmd) // ")"
                  call sqlite3_do(db,cmd) 
    
                  write(cmd,"(A20,I0,a1,i0,a8,i0,a1,i0,a4)") &
                       "CREATE INDEX hesidx_",state2,"_",state1, &
                       " ON hes_", state2 ,"_",state1,"(id)"
                  call sqlite3_do( db, cmd )
               enddo
            enddo

         else if (sql_vers .eq. 1) then
            cmd = 'CREATE TABLE hes (id INTEGER NOT NULL REFERENCES dbentry(id)&
                  &ON UPDATE CASCADE ON DELETE CASCADE,&
                  &state1 INTEGER NOT NULL,&
                  &state2 INTEGER NOT NULL,&
                  &ndof1 INTEGER NOT NULL,&
                  &ndof2 INTEGER NOT NULL,&
                  &value DOUBLE NOT NULL)'
                  call sqlite3_do( db, cmd )
            cmd = 'CREATE INDEX hesidx ON hes(id)'
            call sqlite3_do( db, cmd )

         end if
           
      end subroutine crehess
      
!-----------------------------------------------------------------------

      !Create Adiabatic Hessian
      subroutine creahess(db,sql_vers,nstate,natoms,cmd)
         type(SQLITE_DATABASE), intent(inout)        :: db
         integer(long_sql), intent(in)                   :: sql_vers  !SQL database version
         integer(long_sql), intent(in)                   :: nstate
         integer(long_sql),intent(in)                    :: natoms
! SQLITE_MAX_COLUMN = 2000 by default
         character(len=20000), intent(out)           :: cmd
         integer                                     :: state1, coo1, coo2
         character(len=150)                          :: bstr
      
         if (sql_vers .ge. 4) then
            !One row per state then all DOF
            !Create base for table
            cmd = "CREATE TABLE ahes (id INTEGER NOT NULL REFERENCES dbentry(id)&
                  & ON UPDATE CASCADE ON DELETE CASCADE,&
                  & state1 INTEGER NOT NULL,&
                  & coord1 TEXT NOT NULL"
                  !Now loop over 2nd coord
                  do coo2 = 1, natoms
                     write(bstr,"(A4,I0,A20)") &
                         ",x_",coo2, "  DOUBLE NOT NULL,"
                     cmd = trim(cmd) // trim(bstr)

                     write(bstr,"(A3,I0,A20)") &
                         "y_",coo2,"  DOUBLE NOT NULL,"
                     cmd = trim(cmd) // trim(bstr)

                     write(bstr,"(A3,I0,A20)") &
                         "z_",coo2,"  DOUBLE NOT NULL"
                     cmd = trim(cmd) // trim(bstr)
                  end do
            cmd = trim(cmd) // ")"
            call sqlite3_do(db,cmd) 
            if (db%error.ne.SQLITE_OK) then
               write(*,'(2a)') 'sql error while setting aHess: ', &
                                     db%errmsg
            end if
            cmd = 'CREATE INDEX ahesidx ON ahes(id)'
            call sqlite3_do( db, cmd )
            call sqlite3_do( db, cmd )

         else if (sql_vers .eq. 3) then
            cmd = 'CREATE TABLE ahes (id INTEGER NOT NULL REFERENCES dbentry(id) &
                  &ON UPDATE CASCADE ON DELETE CASCADE,&
                  &state1 INTEGER NOT NULL,&
                  &ndof1 INTEGER NOT NULL,&
                  &ndof2 INTEGER NOT NULL,&
                  &coord1 TEXT NOT NULL,&
                  &coord2 TEXT NOT NULL,&
                  &value DOUBLE NOT NULL)'
            call sqlite3_do( db, cmd )
            cmd = 'CREATE INDEX ahesidx ON ahes(id)'
            call sqlite3_do( db, cmd )
            
         else if (sql_vers .eq. 2) then
            !Also create new hes table with one entry per electronic state
            !Contains the Hessian values in cartesian
            !Size, a row for each DB entry a table for each electronic state
            !Column number changes based on number of electronic states and atoms
            !Have a column for each state for each atom (twice as second diff) and coupling
            !x_k_x_l, y_k_x_l, z_k_x_l - first hessian entries for atoms k & l

            !Create table per electronic state

            do state1 = 1, nstate
               !Loop over states
               !Create the table
               write(cmd,"(A18,I0,A80)") "CREATE TABLE ahes_", state1 , &
                    " (id INTEGER NOT NULL &
                    &REFERENCES dbentry(id)&
                    &ON UPDATE CASCADE ON DELETE CASCADE"
               !Now loop over 2nd coord
               do coo1 = 1, natoms
                  !Now loop over 1st coord
                  do coo2 = 1, natoms
                     write(bstr,"(A4,I0,A3,I0,A20)") &
                         ",x_",coo2,"_x_", coo1, "  DOUBLE NOT NULL,"
                     cmd = trim(cmd) // trim(bstr)
                     write(bstr,"(A4,I0,A3,I0,A20)") &
                         "y_" ,coo2,"_x_", coo1, "  DOUBLE NOT NULL,"
                     cmd = trim(cmd) // trim(bstr)
                     write(bstr,"(A4,I0,A3,I0,A20)") &
                         "z_" ,coo2,"_x_", coo1, "  DOUBLE NOT NULL"
                     cmd = trim(cmd) // trim(bstr)
                  enddo
                  do coo2 = 1, natoms
                     write(bstr,"(A4,I0,A3,I0,A20)") &
                         ",x_",coo2,"_y_", coo1, "  DOUBLE NOT NULL,"
                     cmd = trim(cmd) // trim(bstr)
                     write(bstr,"(A4,I0,A3,I0,A20)") &
                         "y_" ,coo2,"_y_", coo1, "  DOUBLE NOT NULL,"
                     cmd = trim(cmd) // trim(bstr)
                     write(bstr,"(A4,I0,A3,I0,A20)") &
                         "z_" ,coo2,"_y_", coo1, "  DOUBLE NOT NULL"
                     cmd = trim(cmd) // trim(bstr)
                  enddo
                  do coo2 = 1, natoms
                     write(bstr,"(A4,I0,A3,I0,A20)") &
                         ",x_",coo2,"_z_", coo1, "  DOUBLE NOT NULL,"
                     cmd = trim(cmd) // trim(bstr)
                     write(bstr,"(A4,I0,A3,I0,A20)") &
                         "y_" ,coo2,"_z_", coo1, "  DOUBLE NOT NULL,"
                     cmd = trim(cmd) // trim(bstr)
                     write(bstr,"(A4,I0,A3,I0,A20)") &
                         "z_" ,coo2,"_z_", coo1, "  DOUBLE NOT NULL"
                     cmd = trim(cmd) // trim(bstr)
                  enddo
               enddo
               cmd = trim(cmd) // ")"
               call sqlite3_do(db,cmd) 
            enddo

         else if (sql_vers .eq. 1) then
            cmd = 'CREATE TABLE ahes (id INTEGER NOT NULL REFERENCES dbentry(id)&
                  &ON UPDATE CASCADE ON DELETE CASCADE,&
                  &state1 INTEGER NOT NULL,&
                  &ndof1 INTEGER NOT NULL,&
                  &ndof2 INTEGER NOT NULL,&
                  &value DOUBLE NOT NULL)'
            call sqlite3_do( db, cmd )
            cmd = 'CREATE INDEX ahesidx ON ahes(id)'
            call sqlite3_do( db, cmd )
            
         end if
         
      end subroutine creahess

!-----------------------------------------------------------------------

      !Create MOCO table
      subroutine cremoco(db,sql_vers,nbasis,cmd)
         type(SQLITE_DATABASE), intent(inout)        :: db
         integer(long_sql), intent(in)                   :: sql_vers  !SQL database version
         integer(long_sql), intent(in)                   :: nbasis
! SQLITE_MAX_COLUMN = 2000 by default
         character(len=20000), intent(out)           :: cmd
         integer(long_sql)                               :: mo1, mo2
         character(len=150)                          :: bstr

         if (sql_vers .ge. 4) then
            !Make MOCO table
            !One row per MO basis, each column is a AO in basis set
            cmd = "CREATE TABLE moco &
                  &(id INTEGER NOT NULL REFERENCES dbentry(id)&
                  & ON UPDATE CASCADE ON DELETE CASCADE,&
                  & AO  INTEGER NOT NULL"
            !loop over basis functions to create columns
            do mo2 = 1, nbasis
               write(bstr,"(A4,I0,A20)") ",mo_" ,mo2, "  DOUBLE NOT NULL"
               cmd = trim(cmd) // trim(bstr)
            enddo
            cmd = trim(cmd) // ")"
            call sqlite3_do( db, cmd )
            cmd = 'CREATE INDEX mocoidx ON moco(id)'
            call sqlite3_do( db, cmd )

         else if (sql_vers .eq. 3) then
            cmd = 'CREATE TABLE moco (id INTEGER NOT NULL REFERENCES dbentry(id)&
                  &ON UPDATE CASCADE ON DELETE CASCADE,&
                  &MO  INTEGER NOT NULL,&
                  &AO  INTEGER NOT NULL,&
                  &value DOUBLE NOT NULL)'
            call sqlite3_do( db, cmd )
            cmd = 'CREATE INDEX mocoidx ON moco(id)'
            call sqlite3_do( db, cmd )

         else if (sql_vers .eq. 2) then
            !make moco table
            !Contains the MO coefficients
            !Size, a row for each DB entry a table for each basis function
            !Column number changes based on number of basis functions
            !Have a column for each basis function
            !mo_i - mo coefficent for mo i within moco_j table

            !Loop of basis functions
            do mo1 = 1, nbasis
               !Create the table
               write(cmd,"(A18,I0,A80)") "CREATE TABLE moco_", mo1 ,&
                     " (id INTEGER NOT NULL REFERENCES dbentry(id)&
                     &ON UPDATE CASCADE ON DELETE CASCADE"
               !loop over basis functions
               do mo2 = 1, nbasis
                  write(bstr,"(A4,I0,A20)") ",mo_" ,mo2, "  DOUBLE NOT NULL"
                  cmd = trim(cmd) // trim(bstr)
               enddo
               cmd = trim(cmd) // ")"
               !write(*,*) trim(cmd)
               call sqlite3_do(db,cmd) 
            enddo

         elseif (sql_vers .eq. 1) then
            cmd = 'CREATE TABLE moco (id INTEGER NOT NULL REFERENCES dbentry(id)&
                  &ON UPDATE CASCADE ON DELETE CASCADE,&
                  &mo1 INTEGER NOT NULL,&
                  &mo2 INTEGER NOT NULL,&
                  &value DOUBLE NOT NULL)'
            call sqlite3_do( db, cmd )
            cmd = 'CREATE INDEX mocoidx ON moco(id)'
            call sqlite3_do( db, cmd )

         endif

      end subroutine cremoco

!-----------------------------------------------------------------------

      !Create Dipole Moment Table (diabatic)
      subroutine credip(db,cmd)
         type(SQLITE_DATABASE), intent(inout)        :: db
         integer(long_sql)                               :: sql_vers  !SQL database version
! SQLITE_MAX_COLUMN = 2000 by default
         character(len=20000), intent(out)           :: cmd

         !Not changed yet

         sql_vers = db%db_vers

         cmd = 'CREATE TABLE dip (id INTEGER NOT NULL REFERENCES dbentry(id)&
               &ON UPDATE CASCADE ON DELETE CASCADE,&
               &state1 INTEGER NOT NULL,&
               &state2 INTEGER NOT NULL,&
               &dipx DOUBLE NOT NULL,&
               &dipy DOUBLE NOT NULL,&
               &dipz DOUBLE NOT NULL)'
          call sqlite3_do( db, cmd )
          cmd = 'CREATE INDEX dipidx ON dip(id)'
          call sqlite3_do( db, cmd )
         
      end subroutine credip

!-----------------------------------------------------------------------

      !Create Dipole Moment Table (adiabatic)
      subroutine creadip(db,cmd)
         type(SQLITE_DATABASE), intent(inout)        :: db
         integer(long_sql)                               :: sql_vers  !SQL database version
! SQLITE_MAX_COLUMN = 2000 by default
         character(len=20000), intent(out)           :: cmd

         !Not changed yet

         sql_vers = db%db_vers

         cmd = 'CREATE TABLE adip (id INTEGER NOT NULL REFERENCES dbentry(id)&
               &ON UPDATE CASCADE ON DELETE CASCADE,&
               &state1 INTEGER NOT NULL,&
               &state2 INTEGER NOT NULL,&
               &dipx DOUBLE NOT NULL,&
               &dipy DOUBLE NOT NULL,&
               &dipz DOUBLE NOT NULL)'
          call sqlite3_do( db, cmd )
          cmd = 'CREATE INDEX adipidx ON adip(id)'
          call sqlite3_do( db, cmd )
         
      end subroutine creadip

!-----------------------------------------------------------------------

      !Create SOC Table (adiabatic)
      subroutine crecasoc(db,nqcstate,cmd)
         type(SQLITE_DATABASE), intent(inout)        :: db
         integer(long_sql), intent(in)                   :: nqcstate
         integer(long_sql)                               :: sql_vers  !SQL database version
! SQLITE_MAX_COLUMN = 2000 by default
         integer(long_sql)                               :: state1,state2
         character(len=20000), intent(out)           :: cmd
         character(len=50)                           :: bstr
         
         sql_vers = db%db_vers

      if (sql_vers .ge. 4) then
         !make SOC table as 2 parts - real and imaginary
         cmd = 'CREATE TABLE casoc (&
               &id INTEGER NOT NULL &
               &REFERENCES dbentry(id)&
               &ON UPDATE CASCADE ON DELETE CASCADE,&
               & part  TEXT NOT NULL'
         do state1 = 1, nqcstate-1
            do state2 = state1+1, nqcstate
               write(bstr,"(A5,I0,A1,I0,A20)") ",SOC_" , state2,"_",state1,&
                     "  DOUBLE NOT NULL"
               cmd = trim(cmd) // trim(bstr)
            enddo
         enddo
         cmd = trim(cmd) // ")"
         call sqlite3_do( db, cmd )

      else
         cmd = 'CREATE TABLE casoc (id INTEGER NOT NULL REFERENCES dbentry(id)&
               &ON UPDATE CASCADE ON DELETE CASCADE,&
               &socindex INTEGER NOT NULL,&
               &realvalue DOUBLE NOT NULL,&
               &complexvalue DOUBLE NOT NULL)'
         call sqlite3_do( db, cmd )
         cmd = 'CREATE INDEX casocidx ON casoc(id)'
         call sqlite3_do( db, cmd )

      endif
         
      end subroutine crecasoc

!-----------------------------------------------------------------------

      !Create civec Table 
      subroutine crecivec(db,maxspblk,cmd)
        type(SQLITE_DATABASE), intent(inout)        :: db
        integer(long_sql)                               :: sql_vers  !SQL database version

! SQLITE_MAX_COLUMN = 2000 by default
         integer(long_sql)                              :: state1
         integer(long_sql), intent(in)                  :: maxspblk
         character(len=20000), intent(out)          :: cmd
         character(len=50)                          :: bstr
         character(len=c5)                          :: message
         character(len=c3)                          :: routine
         
         sql_vers = db%db_vers
         routine = "CREcivec (db_sql.F90)"

      if (sql_vers .ge. 4) then
         cmd = 'CREATE TABLE civec (&
               &id INTEGER NOT NULL &
               &REFERENCES dbentry(id)&
               &ON UPDATE CASCADE ON DELETE CASCADE,&
               & Mult INTEGER NOT NULL,&
               & Iconf INTEGER NOT NULL,&
               & Conf TEXT NOT NULL'
         do state1 = 1, maxspblk
            write(bstr,"(A7,I0,A17)") ",State_" , state1,"  DOUBLE NOT NULL"
            cmd = trim(cmd) // trim(bstr)
         enddo

         cmd = trim(cmd) // ")"
         call sqlite3_do( db, cmd )

         cmd = 'CREATE TABLE civec_dat (&
               & Iblk INTEGER NOT NULL,&
               & Mult INTEGER NOT NULL,&
               & Nsta INTEGER NOT NULL,&
               & Nconf INTEGER NOT NULL'
         cmd = trim(cmd) // ")"
         call sqlite3_do( db, cmd )

      else
         routine = 'Crecivec (db_sql.f90)'
         write(message,'(a)') 'CIVec table not possible for DB version <4'
         call dberrmsg( routine, message )
      endif

      end subroutine crecivec

!-----------------------------------------------------------------------

      !! CHECK if a table is present. teststring comes from DB
      subroutine checktable(teststring,table_list,testflag,dispname, iunit)
         character(len=80), intent(in) :: teststring
         logical(kind=4), intent(inout)  ::  testflag
         character(len=*), intent(in)        ::dispname
         integer(long_sql), intent(in)                :: iunit
            
         character(len=80), dimension(:), allocatable:: table_list !List of tables

         if (ANY(table_list == teststring)) then
            testflag = .true.
            write (iunit,'(a20,a15)') trim(dispname),'     Included'
         else
            testflag = .false.
            write (iunit,'(a20,a20)') trim(dispname),'    Not Included'
         endif
         
      end subroutine checktable


      end module db_sql

      




