!
      module coeffs

use decimal, only:long,dop
use logdat, only: stopnow
use mtlib,   only: mttxdd, mttxdr
use timing

      implicit none
      private
      public :: vcoeffs,ctensor
      contains

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
!     This routine can be used for ALL coefficient calculations.
!
!     vvec (i)
!        exact potential or modified reference potential
!     evec (i)
!     coeff (o)
!     work2 (w)
!
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine vcoeffs(vvec,evec,coeff,work2)

use global
use pglobal
use griddatmod
use operdef
use runpfitmod
use combined
use channels

use zeigpfitmod, only:setv2c

      implicit none

      real(dop), dimension(evecdim), intent(in)   :: evec
      real(dop), dimension(workdim2), intent(out) :: work2
      real(dop), dimension(maxcldim), intent(out) :: coeff
      real(dop), dimension(vpotdim), intent(in)   :: vvec
      integer(long)                               :: mm,m,woffs0,woffs1,swp
      character(len=240)                          :: filename

      integer(long), save                         :: tid=0 ! timer ID

!-----------------------------------------------------------------
! MGPF lextpot => read contracted from ext file (fdcoeff)
!-----------------------------------------------------------------
      if(lextpot)then

         write(filename,'(2a)') name(1:laenge),'/fdcoeff'
         open(ieigvec,file=filename,form='unformatted',status='unknown',err=999)

         call rdvecout(coeff,dimbef,psubdim(modc),dimaft,ieigvec)
         close(ieigvec)

!-----------------------------------------------------------------
! POTFIT
!-----------------------------------------------------------------
      else

         call get_timer(tid, 'Coeffs')
         call start_timer(tid)

         woffs0=1
         woffs1=v2c(1,2)+1
         m=1
         mm=v2c(m,1)

         if(npmode.eq.2) then  ! special case: 2D
            call mttxdd(evec(zevec(mm)),vvec,coeff, &
                        v2c(m,3),v2c(m,4),v2c(m,5),v2c(m,6))
            goto 50
         endif

         call mttxdd(evec(zevec(mm)),vvec,work2(woffs0), &
                    v2c(m,3),v2c(m,4),v2c(m,5),v2c(m,6))

         do m=2,npmode-2
            mm=v2c(m,1)
            call mttxdd(evec(zevec(mm)),work2(woffs0),work2(woffs1), &
                        v2c(m,3),v2c(m,4),v2c(m,5),v2c(m,6))
            swp=woffs0
            woffs0=woffs1
            woffs1=swp
         enddo

         m=npmode-1
         mm=v2c(m,1)
         call mttxdd(evec(zevec(mm)),work2(woffs0),coeff, &
                     v2c(m,3),v2c(m,4),v2c(m,5),v2c(m,6))

      endif

   50 continue
      call stop_timer(tid)

      return
 999  write(6,'(/2a)') 'Cannot open file: ',filename(1:laenge+10)
      call stopnow
 
      end subroutine vcoeffs

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!  MGPF: read contrc from contracted file
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine rdcontr(coeff,potdim,csubdim,modc,modcdim,nmode)

use global
use channels

      implicit none

      integer(long), intent(in)       :: csubdim,modc,modcdim,nmode
      integer(long)                   :: i,m,g
      integer(long), dimension(nmode) :: potdim
      integer(long), dimension(16)    :: indx
      real(dop), dimension(csubdim,modcdim), intent(out) :: coeff !,rms,cmax
      character(len=240)                                 :: filename

      write(filename,'(2a)') name(1:laenge),'/contracted'
      open(ieigvec,file=filename,form='formatted',status='unknown',err=999)

      read(ieigvec,*)

      do m=1,nmode
         indx(m) = 1
      enddo
      indx(modc)=0

      do i=1,modcdim
         read(ieigvec,*)
         read(ieigvec,*)
         read(ieigvec,'(1p5e16.7)')(coeff(g,i),g=1,csubdim)
! --- determine new set of indices
         do m=1,nmode
            if(m.ne.modc) then
               if (indx(m).lt.potdim(m)) then
                  indx(m)=indx(m)+1
                  goto 1
               else
                  indx(m)=1
               endif
            endif
         enddo
 1    continue
      enddo

      close(ieigvec)

      return

 999  write(6,'(/2a)') 'Cannot open file: ',filename(1:laenge+10)
      call stopnow
      end subroutine rdcontr

!----------------------------------------------------------------------
!  C tensor: compute and write
!     DPR 2013
!----------------------------------------------------------------------
! dani 18.03.13
      subroutine ctensor(evec,coeff)

use global
use pglobal
use griddatmod
use operdef
use runpfitmod
use combined
use channels

use wrfitmod, only: vecout

      implicit none

      real(dop), dimension(coeffdim), intent(in) :: coeff
      real(dop), dimension(evecdim), intent(in)  :: evec
      real(dop), dimension(cvecdim)              :: cvec
      character(len=240)                         :: filename

!----------------------------------------------------------------------------
!   Multiply:
!       mttxdd (a,b,c,dim1,dim2,dim3,dim4)
!       Definition: a(j,l)*b(i,j,k) = c(i,l,k) .
!       Dimensions: a(dim2,dim4),b(dim1,dim2,dim3),c(dim1,dim4,dim3)
!     &            v2c(m,3),v2c(m,4),v2c(m,5),v2c(m,6))
!----------------------------------------------------------------------------

      call mttxdd(evec(zevec(modc)),coeff,cvec,dimbef,psubdim(modc),dimaft,mctens)

!----------------------------------------------------------------------------
!  Write file 'ctensor'
!----------------------------------------------------------------------------

      write(filename,'(2a)') name(1:laenge),'/ctensor'
      open(ioutbin,file=filename,form='unformatted',status='unknown',err=999)

      call vecout(cvec,dimbef,mctens,dimaft,ioutbin)
      close(ioutbin)


      return
 999  write(6,'(/2a)') '!annot open file: ',filename(1:laenge+10)
      call stopnow
      end subroutine ctensor

!-----------------------------------------------------------
! MGPF: Subroutine rdvecout
!-----------------------------------------------------------

      subroutine rdvecout(vec,v,gdim,n,unit)

      implicit none

      integer(long), intent(in)                   :: v,gdim,n,unit
      integer(long)                               :: i,j,g
      real(dop), dimension(v,gdim,n), intent(out) :: vec

      do i=1,n
         do j=1,v
           read(unit) (vec(j,g,i),g=1,gdim)
        enddo
      enddo

      return
      end subroutine rdvecout


      end module coeffs


