      module gapfuncsmod

use decimal, only: dop,long,vlong
use dvrdatmod
use psidef
use dirdyn ! the_db
use operdef
use griddatmod
use gapmod
use runpropmod
use rdopermod, only: assocdof
use openmpmod
use mpidata
use globmemmod, only: agmat
use datenmod, only: tfinal
use lctmod
use hphimod, only: hphi1mk
use operutil, only: getvgn

use logdat, only: errormsg
use op1lib,  only: cpvxd,zerovxz,initvxz,cpvxz,sum2vxd,unitqxz,unitqxd,zeromxz
use op2lib, only: addmxxzo
use xvlib, only: xmxxzza
use mmlib,   only: mmaxzzh,mmaxzz,mmaxzzh1
use mtlib,   only: dtxxdz,dtxxyz

use dd_db, only: ddq2x,ddx2q,distdb,ddmo_rd,ddf2q,ddf2x,dddb_rd_db0
use funcqchemmod, only: lhaqchem,molpro_wfover,citrans,molpro_moover
use timing
use funcgapmod
use gwplib, only: gwp_getwidth,center,gwp2gpoint
use db_sql

      public  :: gapdrv,hphigap,phihphigap,mfieldsgap,hlochphigap,summfgap,&
                 calchagap,hphigap1m,hlochphi1mgap,hunphigap1ms,hunphigap,&
                 hphigap1mk,mpihlochphigap,mpiphihphigap,phihunphigap,&
                 calchunagap
      private :: gapdbinit,gapdb,gaphop,zeighpsigap,get_energy,&
                 hcallgap,calchagap1k,calchagap2k,wpgauss,init_gapwf,&
                 tensgaphop

! Unused routines
      public :: optgaphop

!-----------------------------------------------------------------------
!         used by reading to avoid excess memory usage.
! dbcivec : database array of CI vectors used in diabatisation routines. 
!-----------------------------------------------------------------------
      type(civec), dimension(:,:), allocatable, target :: dbcivec

      integer(long) :: imethod

      contains

!#########################################################################
! Subroutine to drive the calculation of the GAP.
! If new DB points are added, the GAP weights are recalculated and the 
! operator values re-calculated at the MCTDH grid points.
! GWR 07/16
!#########################################################################
      subroutine gapdrv(time,psi,ort,laddnew)

      implicit none

      real(dop), intent(in)                          :: time
      real(dop), dimension(ortdim), intent(in)       :: ort
      complex(dop), dimension(dgldim), intent(inout) :: psi
      logical(kind=4), intent(in)                    :: laddnew

      logical(kind=4)       :: lwgt0,lupdated
      logical(kind=4), save :: lfirst=.true.
      integer(long)         :: nrecold,s,hl,h,hl1
      real(dop)             :: wgnorm,delt

! test
!      real(dop), dimension(nddstate,nddstate) :: v
! test
      
      integer(long), save   :: tid = 0 ! timer ID

! Timing
      call get_timer(tid, 'gapdrv')
      call start_timer(tid)

!! Optimisation of GAP parameters at end of propagation if required.
!      if(abs(time-tfinal).lt.1.0e-8_dop .and. .not.laddnew .and. lgapparopt)then
!         call gapparopt
!         call stop_timer(tid)
!         return
!      endif

! TEST
      if(lmalon)then
         call potfit_malon(ort)
         stop
      endif
! TEST

! Check which electronic structure method is being used.
      hl = 0
      do h=1,htmdim
         hl1=hterm(1,h)
         if(hl1.eq.0)cycle
         if (htmform(h) .eq. 1) cycle
         if (hopfile(hl1) .eq. 208 .or. &
              hopfile(hl1) .eq. 209 .or. hopfile(hl1) .eq. 205) then
            hl = hl1
            exit
         endif
      enddo
      imethod = hopipar(7,hl)

! Initialisations on first call.
      if(lfirst)then
         call alloc_gapwfmod
         if (lddmctdh) call init_gapwf
         call agapdiag
         call alloc_gapmisc
         if(ldiabdb.and.(dddiab.eq.4.or.dddiab.eq.6).and.lddwrdb.and.imethod.ne.33) then
! Read the CI vectors of the calculated wavefunctions from file.
            call get_nconfig
            call alloc_dbcivec
         endif
      endif

! If wanting to use an adiabatic initial wavefunction (only possible with a 
! read-only calculation), setup the wavepacket now by diabatising it.
      if(lfirst.and.lgapadiabwf.and.lexact) call gap_diab_wf(psi,ort)

! First, calculate a point at the centre of the wavepacket if DB is empty.
      nrecold = dbnrec
      lupdated = .false.

      if(dbnrec .eq. 0) then
         call gapdbinit(time,lgaptest)
         lupdated = .true.
      endif

! Decide where, if anywhere, to calculate new points.
      if(lddwrdb.and.laddnew.and.(gapupdate.eq.1))then
         delt = abs(time-gapuptim0)
         if((delt.ge.gapuptim) .or. (abs(delt-gapuptim).lt.tiny)) then
            if(lfirst.and..not.lupdated)call acholupd(.true.,dbnrec,dbnrec)

! Print variances for exact calculation if requested.
            if(lgapvarprint)then
               call printvar(time,ort)
               stop
            endif
 
! Rediabatise at the start in case we are doing a run on a pre-existing DB
! in which case gapalpha may have changed (propagation diabatisation only).
            if (lfirst.and.ldiabdb.and.dddiab.eq.4.and.nrecold.ne.0) then
               call diabat_gap
            endif
! Call to routine which builds the DB.
            if(.not.lgapcomp)call gapdb(time,psi,ort,lgaptest)
            lupdated = .true.
         endif
      endif

! Check whether the weight vector is 0 (at start of calculation).
! If so we also construct the Cholesky decomposition of the A-matrix.
      lwgt0 = .false.
      outer: do s = 1,nddstate 
         wgnorm = &
              dot_product(dbgapwgt(s,s,1:dbnrec),dbgapwgt(s,s,1:dbnrec))
         lwgt0 = abs(wgnorm).lt.tiny        
         if(lwgt0)exit outer
      enddo outer

! If zero weights or new points added, calculate the weight vector and 
! allocate memory. Then calculate the value of the potential terms at the 
! gridpoints.
      if(dbnrec.ne.nrecold .or. lwgt0 .or. lgapadiabwf) then
         if(.not.lupdated)call acholupd(.true.,dbnrec,dbnrec)
         if (lsocbas) then
            call ddcmgapwgt(dbgapwgt(1:nddstate,1:nddstate,1:dbnrec),&
                 dbener(1:nddstate,1:nddstate,1:dbnrec),&
                 dbcgapwgt(1:nddstate,1:nddstate,1:dbnrec),&
                 dbcener(1:nddstate,1:nddstate,1:dbnrec),&
                 dbnrec,nddstate,ddmorder)
         else
            call ddmgapwgt(dbgapwgt(1:nddstate,1:nddstate,1:dbnrec),&
                 dbener(1:nddstate,1:nddstate,1:dbnrec),&
                 dbnrec,nddstate,ddmorder)
         endif

         if(lddsoc) call ddmsocwgt(dbsocwgt(1:nddstate,1:nddstate,1:dbnrec),&
              dbsoc(1:nddstate,1:nddstate,1:dbnrec),dbnrec,nddstate,ddmorder)

         if(lgapcomp)then
            call gapcomp(lgaptest)
            stop
         else
! 
! OPT/SVD GAP comes below here.
!
            if(lddmopt)then
!               if(lddtrans)then
!                  call ddmopt(ort,dbnm(1:ndofdd,1:dbnrec),&
!                       dbener(1,1:nddstate,1:nddstate,1:dbnrec),&
!                       dbnrec,nddstate,ddmoptord)
!               else
!                  call ddmopt(ort,dbgeo(1:ndofdd,1:dbnrec),&
!                       dbener(1,1:nddstate,1:nddstate,1:dbnrec),&
!                       dbnrec,nddstate,ddmoptord)
!               endif
            else if(lddgbtendec)then
               if(lddtrans.and.ddcoo.eq.1)then
                  call ddgbtendec(ort,dbnm(1:ndofdd,1:dbnrec))
               else
                  call ddgbtendec(ort,dbgeo(1:ndofdd,1:dbnrec))
               endif
            endif
! 
! OPT/SVD GAP comes above here.
!
! Only require special arrays etc. if using DD-GB, not DD-vMCG

            if(lddmctdh)then
               call alloc_gappntmod
               if(lddmopt)then
!                  call zeighpsioptgap
!                  call alloc_gaparrmod
!                  call optgaphop(ort)
               elseif(lddgbtendec)then
                  call zeighpsioptgap
                  call alloc_gaparrmod
                  call tensgaphop
               else
                  call zeighpsigap
                  call alloc_gaparrmod

                  call gaphop(ort)
#ifdef MPI
                  call gaphop_bcast
#endif
               endif
            endif
         endif
      endif

      if(lupdated) gapuptim0 = time

      lfirst = .false.

      call stop_timer(tid)

      return        
      end subroutine gapdrv

!#########################################################################
! Subroutine to initialise a database for GAP i.e. calculate an 
! initial point at the centre of the wavepacket.
! Especially important for diabatisation.
! GWR 07/16
!#########################################################################
      subroutine gapdbinit(time,ltest)

      implicit none

      real(dop), intent(in)      :: time
      logical(kind=4),intent(in) :: ltest

      logical(kind=4) :: lfail
      logical(kind=4), dimension(1) :: lfaila

      integer(long) :: f,h,hl,hl1

      real(dop), dimension(ndofddpes) :: gpoint,xgpoint
      real(dop), dimension(nddstate,nddstate)            :: v
      real(dop), dimension(nddstate,nddstate)            :: rdum,rdum1,trans
      real(dop), dimension(ndofddpes,nddstate,nddstate)  :: deriv1
      real(dop), dimension(ndofddpes,ndofddpes,nddstate,nddstate) :: deriv2
      real(dop), dimension(3,nddstate,nddstate)          :: dipole
      real(dop), dimension(ndofddpes,nactdim)            :: nacte
      complex(dop), dimension(nddstate,nddstate)         :: soc
      real(sip), dimension(nbasis,nbasis)                :: mocoeff
      character(len=30), dimension(maxcivec,nsmult)      :: ciconfig
      real(dop), dimension(maxcivec,maxmultblk,nsmult)   :: cicoeff

      deriv1=0.0_dop
      deriv2=0.0_dop
      mocoeff=0.0
      nacte=0.0_dop
      dipole=0.0_dop
      soc=cmplx(0.0_dop,0.0_dop,dop)

      if(dbnrec.ne.0)then
         routine = 'gapdbinit'      
         message='Called with non-empty DB'
         call errormsg
      endif

      gpoint = 0.0_dop
      do f = 1,ndof
         if(f.eq.feb)cycle
         if(f.eq.fkoe)cycle
         if(f.eq.ftime)cycle
! Initialise the geometry:
! If a point DVR, coordinate is the coordinate of the DVR
! If a normal DVR in an inactive mode, the coordinate is 0 in normal 
! modes or the coordinate of the reference geometry in Cartesians.
! If a normal DVR in an active mode, the coordinate is chosen to be
! the centre of the initial wavepacket.
         if(basis(f).eq.19)then
            gpoint(f) = rpbaspar(1,f)
         else if(dofspf(f).eq.0)then
            if(ddcoo.eq.2 .or. lgapuseinwf)then
               gpoint(f) = rsbaspar(1,f,1)
            else
               gpoint(f) = 0.0_dop
            endif
         else
            gpoint(f) = rsbaspar(1,f,1)
         endif
      enddo

      if(lddtrans.or.ddcoo.eq.0)then
         call ddq2x(gpoint,xgpoint)
      else
! Why is ddxcoo0 added here?
!         call cpvxd(gpoint,xgpoint,ndofddpes)
         xgpoint(1:ndofddpes) = gpoint(1:ndofddpes) + ddxcoo0(1:ndofddpes)
      endif

! Call to the test routine using various fitted potentials.
      if(ltest)then
         call testdrv(xgpoint,gpoint,v,iddmtest)
      else
         hl = 0
         do h=1,htmdim
            hl1=hterm(1,h)
            if(hl1.eq.0)cycle
            if (htmform(h) .eq. 1) cycle
            if (hopfile(hl1) .eq. 208 .or. hopfile(hl1) .eq. 209 &
                 .or. hopfile(hl1).eq.205) then
               hl = hl1
               exit
            endif
         enddo

         if(hl.eq.0)return        
         hopipar(2,hl)=dddiab
         hopipar(3,hl)=1
         hopipar(4,hl)=1
         hopipar(5,hl)=0

         if(ldiabdb)then
            if(hopilab(hl).eq.1)then
               hopipar(6,hl)=1
            else if (hopilab(hl).eq.3 .or. hopilab(hl).eq.6)then
               hopipar(6,hl)=0
            else
               routine = 'gapdbinit'
               message='Diabatisation not implemented for this QC program'
               call errormsg
            endif
         else
            hopipar(6,hl)=0
         endif         
         call lhaqchem(time,hopilab(hl),xgpoint,v,deriv1,deriv2,dipole,&
              mocoeff,nacte,soc,ndofddpes,hopipar(1,hl),lfail,ciconfig,cicoeff)
         lfaila(1)=lfail       
         if(lfail)then
            routine = 'gapdbinit'
            message='Initial electronic structure calculation failed'
            call errormsg
         endif
         if(v(1,1).gt.gapethresh)then
            routine = 'gapdbinit'
            write(message,'(a,f20.12)')'Energy of initial point above the threshold: '&
                     ,v(1,1)
            call errormsg
         endif
      endif

      call dddb_wradiab_gap_mem(xgpoint,v,dipole,mocoeff,nacte,soc,ndofddpes,ltest,&
           ciconfig,cicoeff)

      call unitqxd(trans,nddstate)
      call dddb_wrdiab_gap_mem(v,rdum,dipole,trans,rdum1,soc,dbnrec)
! Write diabatic quantities if using projection diabatisation.
      if(dddiab.eq.5)then
         call dddb_wr_gap_file(2)
      endif

      call acholupd(.true.,1,1)

      return
      end subroutine gapdbinit

!#########################################################################
! Subroutine to determine whether extra DB points are needed for the GAP 
! expansion and if so to calculate them, then generate the weight vector.
! GWR 07/16
!#########################################################################

      subroutine gapdb(time,psi,ort,ltest)

      implicit none

      logical(kind=4),intent(in) :: ltest
      real(dop), intent(in)      :: time
      real(dop), dimension(ortdim), intent(in) :: ort
      complex(dop), dimension(dgldim), intent(in) :: psi

      logical(kind=4), save :: lfirst=.true.,lvarfst=.true.
      logical(kind=4)       :: lrjct,llast,lcentre
      logical(kind=4), dimension(:), allocatable :: lsyminc

      integer(long) :: naccept,varrjct,erjct
      integer(long) :: npoints,nsymwr,nrectmp
!      integer(long), save :: dbnrecold
      integer(long) :: i,m,f,n,h,hl,hl1,s,j,k,s1,jg,ierr,fdx,d,i1,i2,f1

      real(dop)                                :: xlen,xrand,xstep
      real(dop), dimension(:), allocatable     :: xi,xf,xsobol!,xaxis
      real(dop), dimension(ndofddpes)          :: gpoint,gpointsym,gpoint1,dcxtmp
      real(dop), dimension(ndofdd)             :: dcnm,dcnmref
      real(dop), allocatable, dimension(:)     :: xgpoint
      real(dop), allocatable, dimension(:,:)   :: qcenter
      real(dop), allocatable, dimension(:,:)   :: v
      real(dop), dimension(3,nddstate,nddstate)           :: dipole
      real(dop), allocatable, dimension(:,:)              :: nacte
      real(sip), allocatable, dimension(:,:)              :: mocoeff
      real(dop), dimension(nddstate,nddstate)             :: trans,ctrans
      complex(dop), allocatable, dimension(:,:)           :: soc
      character(len=30), dimension(maxcivec,nsmult)       :: ciconfig
      real(dop), dimension(maxcivec,maxmultblk,nsmult)    :: cicoeff

      complex(dop) :: ag

      character(len=10) :: cerr

! Get the number of sample points.
      if(lgapallgrd)then
         if(actdof.gt.2)then
            routine = 'gapdb'
            message = 'Sampling the entire grid only available for two active DOFs.'
            call errormsg
         endif
         if(lfirst)then
            if(dbnrec.ne.1)return
            npoints=1
            do f = 1,ndof
               if(f.eq.feb .or. f.eq.fkoe .or. f.eq.ftime .or. basis(f).eq.19 &
                    .or. dofspf(f).eq.0)cycle
               npoints = npoints*gdim(f)
            enddo
            lfirst = .false.
         else
            return
         endif
      else
         npoints = gapsample
      endif

      cerr = 'xgpoint'
      allocate(xgpoint(ndofddpes),stat=ierr)
      if(ierr.gt.0)goto 1000
      cerr = 'v'
      allocate(v(nddstate,nddstate),stat=ierr)
      if(ierr.gt.0)goto 1000
      cerr = 'nacte'
      allocate(nacte(ndofddpes,max(1,nactdim)),stat=ierr)
      if(ierr.gt.0)goto 1000
      cerr = 'soc'
      allocate(soc(nddstate,nddstate),stat=ierr)
      if(ierr.gt.0)goto 1000
      cerr = 'mocoeff'
      allocate(mocoeff(nbasis,nbasis),stat=ierr) 
      if(ierr.gt.0)goto 1000

      if(lgapsobol)then
         cerr = 'xsobol'
         allocate(xsobol(actdof),stat=ierr) 
         if(ierr.gt.0)goto 1000
!         cerr = 'xaxis'
!         allocate(xaxis(ndofdd),stat=ierr) 
!         if(ierr.gt.0)goto 1000
      else
         cerr = 'xsobol'
         allocate(xsobol(1),stat=ierr) 
         if(ierr.gt.0)goto 1000
      endif

      hl = 0
      do h=1,htmdim
         hl1=hterm(1,h)
         if(hl1.eq.0)cycle
         if (htmform(h) .eq. 1) cycle
         if (hopfile(hl1) .eq. 208 .or. hopfile(hl1) .eq. 209 .or. hopfile(hl1) .eq. 205) then
            hl = hl1
            exit
         endif
      enddo
      if(hl.eq.0)return

      dbnrecold = dbnrec

!      if(allocated(afac))deallocate(afac)
!      allocate(afac(dbnrecold,dbnrecold))
      lvarfst = .true.

! Initiate the random number generator.
      if(lfirst.and. .not.lgapsobol.and. .not.lgapallgrd)then
         call init_random_seed
         lfirst = .false.
      endif

      naccept = 0
      varrjct = 0
      erjct = 0

      if(ldd)then
         acholtmpdim = dbnrecold + (npoints+1)*nddstate*nmode&
              *maxval(gwpdim(1:nmode,1:nstate))
      else
         acholtmpdim = dbnrecold + npoints*nddstate
      endif
      if(allocated(acholtmp))deallocate(acholtmp)
      allocate(acholtmp(acholtmpdim,acholtmpdim))
      acholtmp(1:dbnrecold,1:dbnrecold) = achol(1:dbnrecold,1:dbnrecold)
      nrectmp = dbnrecold

! Allocate initial/final geometry arrays.
      if(allocated(xi))deallocate(xi)
      if(allocated(xf))deallocate(xf)
      allocate(xi(ndofdd),xf(ndofdd))

! Distribution for DD-vMCG. Distributions around the centres of the GWPs.
      if(ldd)then
         mvmcg: do m = 1,nmode 
            if (m .eq. meb) cycle
            svmcg: do s = 1,nstate
! Get the centres of the GWP and convert to system coordinates.
               if(allocated(qcenter))deallocate(qcenter)
               allocate(qcenter(nspfdof(m),gwpdim(m,s)))
               
               call center(nspfdof(m),gwpdim(m,s),psi(zxi(m,s)),qcenter)
               
               jvmcg: do jg = 1,gwpdim(m,s)
                  call gwp2gpoint(qcenter(1,jg),gpoint,nspfdof(m),ndofdd,m)

! Get energy and write to DB if successful.
                  if(.not.lgapsobol)then
                     call get_energy(gpoint,xgpoint,v,mocoeff,&
                          nacte,soc,naccept,varrjct,ltest,iddmtest,lrjct,&
                          nrectmp,lvarfst,hl,time,.false.,ciconfig,cicoeff)
                     if(.not.lrjct) then 
                        call dddb_wradiab_gap_mem(xgpoint,v,dipole,&
                             mocoeff,nacte,soc,ndofddpes,ltest,ciconfig,cicoeff)
                        call acholupd(.false.,dbnrec,nrectmp)
                        nrectmp = dbnrec
                     endif
                  endif

! Loop to get a scatter of geometries around the centre of the GWP.
                  fdx = 0
                  ivmcg: do i = 1,npoints
                     gpoint1 = gpoint

                     if(lgapsobol)then
                        llast = i.eq.npoints
                        if(i.eq.1) call sobol_init(actdof)
                        call sobol(actdof,xsobol,llast)
                        fdx = 0                       
                     endif

                     do n = 1,nspfdof(m)
                        call gwp_getwidth(agmat(zagmat(m,s)),ag,&
                             nspfdof(m),gwpdim(m,s),n,n,jg)
                  
                        f = spfdof(n,m)
                        xi(1) = gpoint1(f) - 0.5_dop*gapsd/sqrt(abs(dble(ag)))
                        xf(1) = gpoint1(f) + 0.5_dop*gapsd/sqrt(abs(dble(ag)))
                        xlen = xf(1)-xi(1)
                        if(lgapsobol)then
                           fdx = fdx + 1
                           gpoint1(f) = xi(1) + xsobol(fdx)*xlen
                        else
                           call random_number(xrand)
                           gpoint1(f) = xi(1) + xrand*xlen
                        endif
                     enddo

! Get energy and write to DB if successful.
                     call get_energy(gpoint1,xgpoint,v,mocoeff,&
                          nacte,soc,naccept,varrjct,ltest,iddmtest,lrjct,&
                          nrectmp,lvarfst,hl,time,.false.,ciconfig,cicoeff)
                     if(lrjct) cycle ivmcg
                     call dddb_wradiab_gap_mem(xgpoint,v,dipole,mocoeff,&
                          nacte,soc,ndofddpes,ltest,ciconfig,cicoeff)

                     call acholupd(.false.,dbnrec,nrectmp)
                     nrectmp = dbnrec

                  enddo ivmcg
               enddo jvmcg
               if(allocated(qcenter))deallocate(qcenter)
               
            enddo svmcg
         enddo mvmcg

      else

! Distribution for DD-GB.
! Scatter of points around the centres of the wavepacket on each state.
         fdx=0
         sloop: do s = 1,nddstate
            if(lgapallgrd.and.s.gt.1)exit sloop
            iloop: do i = 1,npoints
               gpoint = 0.0_dop
               lsyminc = .false.

! Initialise the Sobol sequence at the first point for each state, then 
! call the routine to generate the sequence.
               if(lgapsobol)then
                  llast = i.eq.npoints
                  if(i.eq.1) call sobol_init(actdof)
                  call sobol(actdof,xsobol,llast)
                  fdx = 0                  
               endif

               i1=0
               i2=0
               f1=0
               lcentre=lgapallgrd
               do f = 1,ndof
                  if(f.eq.feb)cycle
                  if(f.eq.fkoe)cycle
                  if(f.eq.ftime)cycle
! Initialise the geometry:
! If a point DVR, coordinate is the coordinate of the DVR
! If a normal DVR in an inactive mode, the coordinate is 0 in normal 
! modes or the coordinate of the reference geometry in Cartesians.
! If a normal DVR in an active mode, the coordinate is chosen randomly
! or by Sobol sampling according to the centre of the wavepacket.
                  if(basis(f).eq.19)then
                     gpoint(f) = rpbaspar(1,f)
                  else if(dofspf(f).eq.0)then
                     if(ddcoo.eq.2 .or. lgapuseinwf)then
                        gpoint(f) = rsbaspar(1,f,1)
                     else
                        gpoint(f) = 0.0_dop
                     endif
                  else
                     if(.not.lgapallgrd)then
                        xi(f) = max(wfposn(f,s)-gapsd*wfwidth(f,s),ort(zort(f))-1.0_dop)
                        xf(f) = min(wfposn(f,s)+gapsd*wfwidth(f,s),ort(zort(f)+gdim(f)-1)+1.0_dop)
                        xlen = xf(f)-xi(f)
                        if(.not.lgapsobol)then
                           call random_number(xrand)
                           gpoint(f) = xi(f) + xrand*xlen
                        else
                           fdx = fdx + 1
                           xstep = xsobol(fdx)*xlen
                           gpoint(f) = xi(f) + xstep
                        endif

                     else
                        if(i1.eq.0.and.nspfdof(1).gt.1)then
                           i1 = (i-1)/gdim(f)+1
                           f1 = f
                           gpoint(f) = ort(zort(f)+i1-1)
                        else
                           if(nspfdof(1).gt.1)then
                              i2 = i-(i1-1)*gdim(f1)
                           else
                              i2=i
                           endif
                           gpoint(f) = ort(zort(f)+i2-1)
                        endif
                        if(lddtrans.and.ddcoo.eq.1)then
                           lcentre = lcentre.and.(abs(gpoint(f)-dbnm(f,1)).lt.1.0e-8_dop)
                        else
                           lcentre = lcentre.and.(abs(gpoint(f)-dbgeo(f,1)).lt.1.0e-8_dop)
                        endif
                     endif
                  endif
               enddo

               if(lcentre)cycle iloop

! Get energy and write to DB if successful.
               call get_energy(gpoint,xgpoint(1),&
                    v(1,1),mocoeff(1,1),nacte(1,1),soc(1,1),naccept,varrjct,&
                    ltest,iddmtest,lrjct,nrectmp,lvarfst,hl,time,lgapvarmax,&
                    ciconfig,cicoeff)

! Reject points where energy of lowest state is above threshold. Check based on
! extrapolated energies done in get_energy.
               if(v(1,1).gt.gapethresh .and. .not.lrjct) then
                  erjct = erjct+1
                  naccept = naccept-1
                  lrjct=.true.
               endif

               if(lrjct)cycle iloop

! Symmetry equivalent geometries.
               nsymwr = 1

               call dddb_wradiab_gap_mem(xgpoint(1),v(1,1),dipole,&
                    mocoeff(1,1),nacte(1,1),soc(1,1),ndofddpes,ltest,ciconfig,cicoeff)
               if(ldiabdb .and. dddiab.eq.5) call proj_diab(1,dipole)
               call acholupd(.false.,dbnrec,nrectmp)
               nrectmp = dbnrec
            enddo iloop
         enddo sloop

      endif

! Final Cholesky update: copying from tmp to achol.
! Must be before the diabatisation as it is used therein.
      if(dbnrec.ne.dbnrecold)then
         if(allocated(achol))deallocate(achol)
         acholdim = dbnrec
         allocate(achol(acholdim,acholdim))
         achol(1:acholdim,1:acholdim) = acholtmp(1:acholdim,1:acholdim)
      endif
      if(allocated(acholtmp))deallocate(acholtmp)

! If performing a multi-state calculation then call the diabatisation routine.
      if(dbnrec.ne.dbnrecold .or. dbnrec.eq.1)then
         if(ldiabdb.and. (dddiab.eq.4.or.dddiab.eq.6))then
            if(.not.ltest)then
               call diabat_gap
            else
               trans = 0.0_dop
               ctrans = 0.0_dop
               trans(1,1) = 1.0_dop
               trans(2,2) = 1.0_dop
               do i = 1,dbnrec
                  call dddb_wrdiab_gap_mem(dbener(1,1,i),dbcener(1,1,i),&
                       dipole,trans,ctrans,soc,i)
               enddo
               call dddb_wr_gap_file(1)
            endif
         else
            call dddb_wr_gap_file(1)
         endif
      endif

      if(allocated(xgpoint))deallocate(xgpoint)
      if(allocated(v))deallocate(v)
      if(allocated(nacte))deallocate(nacte)
      if(allocated(mocoeff))deallocate(mocoeff)
      if(allocated(lsyminc))deallocate(lsyminc)
      if(allocated(xi))deallocate(xi)
      if(allocated(xf))deallocate(xf)
      if(allocated(xsobol))deallocate(xsobol) 
!      if(allocated(xaxis))deallocate(xaxis) 

      write(ilog,'(3(a,i8))')'GAP: Accepted ',naccept,' Rejected variance ',varrjct,&
           ' Rejected energy ',erjct
      flush(ilog)

      return

1000  write(ilog,'(2a)') 'Error allocating memory in GAPDB :',cerr
      write(ilog,'(a,i5)') 'Error Code :',ierr
      write(ilog,'(a)')'Try turning off symmetry.'
      stop

      end subroutine gapdb

!#########################################################################
! Subroutine to initialise a Sobol sequence of points for use in 
! sampling configuration space.
! GWR 05/18
!#########################################################################
      subroutine sobol_init(ndof1)

      implicit none 

      integer(long), intent(in) :: ndof1

      integer(long) :: i,idxp,j,k,l

      if(ndof1.gt.maxsob)then
         routine = 'sobol_init'
         message = 'Too many DOFs for current implementation'
         call errormsg
      endif

      idxn = 0
      
      if(allocated(dirnno)) deallocate(dirnno)
      allocate(dirnno(ndof1,maxb))
      dirnno = 0
      
      if(dirninit(1+idnpnt(ndof1)) .ne. 1) then
         routine = 'sobol_init'
         message = 'First initialising value not 1'
         call errormsg
      endif

      sobfac = 0.5_dop**maxb
      
      do k = 1,ndof1
         do j = 1, polydeg(ndof1)
            dirnno(k,j) = dirninit(j+idnpnt(k))*2**(maxb-j)
         enddo
         
         do j = polydeg(k)+1,maxb
            idxp = ipoly(k)
            i = dirnno(k,j-polydeg(k))
            i = ieor(i,i/2**polydeg(k))
            do l = polydeg(k)-1,1,-1
               if(iand(idxp,1).ne.0) i = ieor(i,dirnno(k,j-l))
               idxp = idxp/2
            enddo
            dirnno(k,j) = i
         enddo
      enddo

      nchk = 0
      intx = 0

      return

      end subroutine sobol_init

!#########################################################################
! Subroutine to calculate a Sobol sequence of points for use in 
! sampling configuration space.
! GWR 05/18
!#########################################################################
      subroutine sobol(ndof1,xsobol,llast)

      implicit none 

      logical(kind=4), intent(in)              :: llast 
      integer(long), intent(in)                :: ndof1
      real(dop), dimension(ndof1), intent(out) :: xsobol

      logical(kind=4) :: lsmall
      integer(long) :: imin,j,k,jdx

      if(nchk.eq.0)then
         nchk = ndof1
      else if(ndof1.ne.nchk)then
         routine = 'sobol'
         message = 'No. of dimensions changed between initialisations.'
         call errormsg
      endif
      
      imin = idxn
      lsmall = .true.
      jdx=0
      do j = 1,maxb
         if(iand(imin,1).eq.0)lsmall=.false.
         if(.not.lsmall)then
            jdx = j
            exit
         endif
         imin = imin/2
      enddo

      if(lsmall)then
         routine = 'sobol'
         message = 'Maximum no. of bits too small'
         call errormsg
      endif

      imin = (jdx-1)*ndof1
      do k = 1,ndof1
         intx(k) = ieor(intx(k),dirnno(k,jdx))
         xsobol(k) = dble(intx(k))
      enddo
      xsobol = sobfac*xsobol
      idxn = idxn + 1
      
      if(llast)then
         if(allocated(dirnno))deallocate(dirnno)
      endif

      return

      end subroutine sobol

!#########################################################################
! Subroutine to calculate the error in the GAP by evaluating the GAP and
! ab initio energies at randomly selected geometries.
! GWR 02/18
!#########################################################################
      subroutine gapcomp(ltest)

      implicit none

      logical(kind=4),intent(in) :: ltest

      logical(kind=4), save :: lfirst=.true.
      logical(kind=4)       :: lrjct,lvarfst

      integer(long) :: i,f,h,hl,hl1,s,s1,ierr,irec,naccept,varrjct,vcdim

      real(dop)                              :: xlen,xrand,rmsall,diffsq,vtmp
      real(dop), dimension(:), allocatable   :: xi,xf
      real(dop), dimension(1) :: vr,vl
      real(dop), dimension(ndofddpes)        :: gpoint
      real(dop), allocatable, dimension(:)   :: xgpoint,av,avi,work,rmsest,abv
      real(dop), allocatable, dimension(:,:,:) :: vgap
      real(dop), allocatable, dimension(:,:)            :: nacte,vai
      real(sip), allocatable, dimension(:,:)            :: mocoeff
      complex(dop), dimension(nddstate,nddstate)        :: soc
      character(len=30), dimension(maxcivec,nsmult)     :: ciconfig
      real(dop), dimension(maxcivec,maxmultblk,nsmult)  :: cicoeff
 
      character(len=10) :: cerr

      routine = 'gapcomp'

      if(lddsoc.and.lsocbas)then
         vcdim=2
      else
         vcdim=1
      endif

      cerr = 'xgpoint'
      allocate(xgpoint(ndofddpes),stat=ierr)
      if(ierr.gt.0)goto 1000
      cerr = 'vai'
      allocate(vai(nddstate,nddstate),stat=ierr)
      if(ierr.gt.0)goto 1000
      cerr = 'vgap'
      allocate(vgap(vcdim,nddstate,nddstate),stat=ierr)
      if(ierr.gt.0)goto 1000
      cerr = 'av'
      allocate(av(nddstate),stat=ierr)
      if(ierr.gt.0)goto 1000
      cerr = 'avi'
      allocate(avi(nddstate),stat=ierr)
      if(ierr.gt.0)goto 1000
      cerr = 'abv'
      allocate(abv(nddstate),stat=ierr)
      if(ierr.gt.0)goto 1000
      cerr = 'rmsest'
      allocate(rmsest(nddstate),stat=ierr)
      if(ierr.gt.0)goto 1000
      cerr = 'work'
      allocate(work(4*nddstate),stat=ierr)
      if(ierr.gt.0)goto 1000
      cerr = 'nacte'
      allocate(nacte(ndofddpes,nactdim),stat=ierr)
      if(ierr.gt.0)goto 1000
      cerr = 'mocoeff'
      allocate(mocoeff(nbasis,nbasis),stat=ierr) 
      if(ierr.gt.0)goto 1000

      lvarfst = .true.
      naccept = 0
      varrjct = 0
      rmsest = 0.0_dop
      rmsall = 0.0_dop

      hl = 0
      do h=1,htmdim
         hl1=hterm(1,h)
         if(hl1.eq.0)cycle
         if (htmform(h) .eq. 1) cycle
         if (hopfile(hl1) .eq. 208 .or. hopfile(hl1) .eq. 209) then
            hl = hl1
            exit
         endif
      enddo
      if(hl.eq.0)return
      
! Initiate the random number generator.
      if(lfirst)then
         call init_random_seed
         lfirst = .false.
      endif

! Allocate initial/final geometry arrays.
      if(allocated(xi))deallocate(xi)
      if(allocated(xf))deallocate(xf)
      allocate(xi(ndofdd),xf(ndofdd))

! Get the extent of the sample space from the database entries.
      xi = 9.9e10_dop
      xf = -9.9e10_dop
      do irec = 1,dbnrec
         do f = 1,ndofdd
            if(lddtrans.and.ddcoo.eq.1)then
               if(dbnm(f,irec).lt.xi(f)) xi(f)=dbnm(f,irec)
               if(dbnm(f,irec).gt.xf(f)) xf(f)=dbnm(f,irec)
            else
               if(dbgeo(f,irec).lt.xi(f)) xi(f)=dbgeo(f,irec)
               if(dbgeo(f,irec).gt.xf(f)) xf(f)=dbgeo(f,irec)
            endif
         enddo
      enddo

! Distribution for DD-GB.
! Scatter of points around the centres of the wavepacket on each state.
      iloop: do i = 1,ngapcomp
         gpoint = 0.0_dop
            
         do f = 1,ndof
            if(f.eq.feb)cycle
            if(f.eq.fkoe)cycle
            if(f.eq.ftime)cycle
! Initialise the geometry:
! If a point DVR, coordinate is the coordinate of the DVR
! If a normal DVR in an inactive mode, the coordinate is 0 in normal 
! modes or the coordinate of the reference geometry in Cartesians.
! If a normal DVR in an active mode, the coordinate is chosen randomly
! according to the centre of the wavepacket.
            if(basis(f).eq.19)then
               gpoint(f) = rpbaspar(1,f)
            else if(dofspf(f).eq.0)then
               if(ddcoo.eq.2)then
                  gpoint(f) = rsbaspar(1,f,1)
               else
                  gpoint(f) = 0.0_dop
               endif
            else
               xlen = xf(f)-xi(f)
               call random_number(xrand)
               gpoint(f) = xi(f) + xrand*xlen
            endif
         enddo

! Get ab initio energy.
         call get_energy(gpoint,xgpoint,vai,mocoeff,nacte,soc,naccept,&
              varrjct,ltest,iddmtest,lrjct,dbnrec,lvarfst,hl,&
              0.0_dop,lgapvarmax,ciconfig,cicoeff)
         
         if(lrjct)cycle iloop

         do s = 1,nddstate
            abv(s) = vai(s,s)
         enddo

! Put in increasing order as this is what happens in dgeev. Needed for SOC
! calculations as get_energy outputs in multiplicity order (S,T,Q,...).
         do s = 1,nddstate
            do s1 = s+1,nddstate
               if(abv(s).gt.abv(s1))then
                  vtmp = abv(s)
                  abv(s) = abv(s1)
                  abv(s1) = vtmp
               endif
            enddo
         enddo
         

! Calculate GAP energy.
         call potgap(gpoint,vgap,vcdim,1)

! If a multi-state calculation, diagonalise the diabatic energy to get the
! adiabatic for comparison with the ab initio.
         if(nddstate.gt.1)then

            call dgeev('N','N',nddstate,vgap,nddstate,av,avi,vl,1,vr,1,&
                 work,4*nddstate,ierr)
            if(ierr.ne.0)then
               write(message,'(a,i6)')'Error in diagonalisation',ierr
               call errormsg
            endif
            do s = 1,nddstate
               do s1 = s+1,nddstate
                  if(av(s).gt.av(s1))then
                     vtmp = av(s)
                     av(s) = av(s1)
                     av(s1) = vtmp
                  endif
               enddo
            enddo
         else
            av(1) = vgap(1,1,1)
         endif
         
         do s = 1,nddstate
            diffsq = (abv(s) - av(s))**2
            rmsest(s) = rmsest(s) + diffsq
            rmsall = rmsall + diffsq
         enddo

         write(ilog,*)naccept,abv(1:nddstate),av(1:nddstate)
         flush(ilog)

      enddo iloop

      do s = 1,nddstate
         rmsest(s) = sqrt(rmsest(s)/dble(naccept))
      enddo

      rmsall = sqrt(rmsall/dble(naccept*nddstate))

      write(ilog,'(a,i8)')'Total sample size',naccept
      write(ilog,'(a,f16.8)')'Total RMS Error',rmsall
      write(ilog,'(a)')'State RMS Error'
      do s = 1,nddstate
         write(ilog,'(a,i6,f16.8)')'State: ',s,rmsest(s)
      enddo      

      if(allocated(xgpoint))deallocate(xgpoint)
      if(allocated(vai))deallocate(vai)
      if(allocated(vgap))deallocate(vgap)
      if(allocated(av))deallocate(av)
      if(allocated(avi))deallocate(avi)
      if(allocated(abv))deallocate(abv)
      if(allocated(work))deallocate(work)
      if(allocated(rmsest))deallocate(rmsest)
      if(allocated(nacte))deallocate(nacte)
      if(allocated(mocoeff))deallocate(mocoeff)
      if(allocated(xi))deallocate(xi)
      if(allocated(xf))deallocate(xf)

      return

1000  write(ilog,'(2a)') 'Error allocating memory in GAPCOMP :',cerr
      write(ilog,'(a,i5)') 'Error Code :',ierr
      stop

      end subroutine gapcomp

!#########################################################################
! Subroutine called from gapdb which actually calls the electronic 
! structure program and extracts the data for a single geometry.
! GWR 12/17
!#########################################################################
      subroutine get_energy(gpoint,xgpoint,v,mocoeff,nacte,soc,naccept,&
           varrjct,ltest,iddmtest,lrjct,nrec,lvarfst,hl,time,lvarmax,&
           ciconfig,cicoeff)

      implicit none

      logical(kind=4), intent(in)    :: ltest,lvarmax
      logical(kind=4), intent(out)   :: lrjct
      logical(kind=4), intent(inout) :: lvarfst
      integer(long), intent(in)      :: iddmtest,nrec,hl
      integer(long), intent(inout)   :: naccept,varrjct
      real(dop), intent(in)                                 :: time
      real(dop), dimension(ndofdd), intent(inout)           :: gpoint
      real(dop), dimension(ndofddpes), intent(out)          :: xgpoint
      real(dop), dimension(nddstate,nddstate), intent(out)  :: v
      real(dop), dimension(ndofddpes,nactdim), intent(out)  :: nacte
      real(sip), dimension(nbasis,nbasis), intent(out)      :: mocoeff
      complex(dop), dimension(nddstate,nddstate), intent(out) :: soc
      character(len=30), dimension(maxcivec,nsmult), intent(out)    :: ciconfig
      real(dop), dimension(maxcivec,maxmultblk,nsmult), intent(out) :: cicoeff
      logical(kind=4)                                       :: lcas

      integer(long) :: idbloc
      integer(long), save :: modus=0
      real(dop) :: var
      real(dop), dimension(2) :: vtmp
      real(dop), dimension(:), allocatable :: r1
      real(dop)                            :: mindist,mindisp
      real(dop), dimension(ndofddpes,nddstate,nddstate)   :: deriv1
      real(dop), dimension(ndofddpes,ndofddpes,nddstate,nddstate) :: deriv2
      real(dop), dimension(3,nddstate,nddstate)           :: dipole

! Initialisations.
      lrjct = .false.
      v = -9999999_dop
      nacte = 0.0_dop
      mocoeff = 0.0_dop
      soc = dcmplx(0.0_dop,0.0_dop)

!-----------------------------------------------------------------------
! Is this a CAS calculation with MOCOs?
!-----------------------------------------------------------------------
      lcas = .false. 
      lcas = (hopipar(7,hl).eq.12 .or. hopipar(7,hl).eq.13 .or. &
              hopipar(7,hl).eq.15 .or. hopipar(7,hl).eq.32)

! Test the variance at the point.
! If sufficiently low, no need to add a new point so return.
      call gapvarddm(gpoint,var,nrec,lvarfst,ddmorder)
      if(.not.lgapcomp)then
         lvarfst = .false.

! If this is the first of the symmetry equivalent geometries, shift it so that
! the variance is maximised within the sample space. If the variance is still
! less than the tolerance, reject and exit.
         if(lvarmax .and. ndofdd.gt.1)then
!            call varmax(gpoint,xi,xf,var,nrec,lvarfst,ddmorder)
! debug
            message='VarMax not working at the moment'
            call errormsg
! debug
         endif
         if(abs(var).lt.gapvartol .and. .not.lgapallgrd)then
            varrjct = varrjct + 1
            lrjct = .true.
            return
         endif
      else

         if(abs(var).gt.gapvartol)then
            varrjct = varrjct + 1
            lrjct = .true.
            return
         endif
      endif

! If using an energy threshold, predict the GS energy and only allow a new energy to 
! be calculated if within a reasonable range.
      if(lgapethresh .and. .not.lgapallgrd)then
         vtmp = 0.0_dop
         call potgapss1(gpoint,vtmp,2,1,1,modus)
         if(modus.eq.0)modus=1
         if(norm2(vtmp(1:2)).gt.gapethresh*1.1_dop)then
            varrjct = varrjct + 1
            lrjct = .true.
            return
         endif
      endif

! Convert to Cartesians if necessary.
      if(lddtrans.or.ddcoo.eq.0)then
         call ddq2x(gpoint,xgpoint)
      else
         call cpvxd(gpoint,xgpoint,ndofddpes)
!         xgpoint(1:ndofddpes) = gpoint(1:ndofddpes) + ddxcoo0(1:ndofddpes)
      endif

! Are we using the test potentials?
      if(ltest)then
         call testdrv(xgpoint,gpoint,v,iddmtest)
      else
! Set up the parameters which control the electronic structure calculation.
         hopipar(2,hl)=dddiab
         hopipar(3,hl)=1
         hopipar(4,hl)=1
         hopipar(5,hl)=1
! Gaussian or Molpro? Affects requirement for frequencies or not.
         if(ldiabdb)then
            if(hopilab(hl).eq.1)then
               hopipar(6,hl)=1
            else if (hopilab(hl).eq.3 .or. hopilab(hl).eq.6)then
               hopipar(6,hl)=0
            else
               message='Diabatisation not implemented for this QC program'
               call errormsg
            endif
         else
            hopipar(6,hl)=0
         endif

! Get the MO coefficients of the nearest DB point if using CASSCF.
         if (lcas) then
            allocate(r1(dbnrec))
            call distdb(xgpoint,idbloc,r1,mindist,mindisp,dbnrec)
            deallocate(r1)

            mocoeff(1:nbasis,1:nbasis) = dbmoco(1:nbasis,1:nbasis,idbloc)
         endif

! Call the routine which actually calls the electronic structure program.
    !     call lhaqchem(time,hopilab(hl),xgpoint,v,deriv1,deriv2,dipole,&
    !          mocoeff,nacte,soc,ndofddpes,hopipar(1,hl),lrjct)
         if(lrjct)then
            write(ilog,'(a)')'Warning: Failed elec. structure calculation'
            flush(ilog)
         endif

      endif

! New DB point to be added.
      if(.not.lrjct)naccept = naccept + 1

      return
      end subroutine get_energy

!#########################################################################
! Routine which takes a sampled geometry and shifts it so as to maximise
! the GAP variance within the sampled space. This is to ensure poorly 
! described regions have a better chance of being sampled.
! GWR 02/18
!#########################################################################
      subroutine varmax(gpoint,xi,xf,var,nrec,lvarfst,order)

      implicit none

      logical(kind=4), intent(in) :: lvarfst
      integer(long), intent(in)   :: nrec,order
      real(dop), dimension(ndofdd), intent(in)    :: xi,xf
      real(dop), dimension(ndofdd), intent(inout) :: gpoint
      real(dop), intent(inout) :: var

      logical(kind=4) :: ledge
      integer(long) :: m,n,nf,f,i
      integer(long), dimension(actdof) :: grid
      real(dop) :: gamma,varm1,norm2
      real(dop), dimension(ndofdd) :: gpointm1
      real(dop), dimension(actdof) :: grad,gpred,gradm1,gpredm1,dgrad,dgp
     
! Get the mapping between the gradient and the full set of coordinates.
      nf = 0
      do m=1,nmode
         do n = 1,nspfdof(m)
            grid(n+nf) = spfdof(n,m)
         enddo
         nf = nf + nspfdof(m)
      enddo

! Copy the coordinates of the active DOFs to the temporary array.
      do f=1,actdof
         gpred(f) = gpoint(grid(f))
      enddo

! Calculate the initial gradient of the variance.
      call vargrad(grad,gpred,gpoint,grid,nrec,order,lvarfst)

      iloop: do i = 1,100
         varm1 = var
         gpointm1 = gpoint

! Set the step length (Barzilai-Borwein).
         if(i.eq.1)then
            gamma = 0.01_dop
         else
            dgp = gpred-gpredm1
            dgrad = grad - gradm1
            gamma = dot_product(dgp,dgrad)/dot_product(dgrad,dgrad)
         endif

! Perform the steepest ascent step.
         gpredm1 = gpred
         gpred = gpred + gamma*grad

! Copy the resulting geometry to the gpoint array, but check that the edge of 
! the sample space has not been breached; if it has, set the coordinate to the 
! edge and exit once all coordinates have been checked.
         ledge = .false.
         do f = 1,actdof
            gpoint(grid(f)) = gpred(f)
            if(gpoint(grid(f)).gt.xf(grid(f)))then
               gpoint(grid(f)) = xf(grid(f))
               ledge = .true.
            else if(gpoint(grid(f)).lt.xi(grid(f)))then
               gpoint(grid(f)) = xi(grid(f))
               ledge = .true.
            endif
         enddo
! Calculate the gradient and variance at the new geometry.
         gradm1 = grad
         call vargrad(grad,gpred,gpoint,grid,nrec,order,lvarfst)
         call gapvarddm(gpoint,var,nrec,lvarfst,order)

! If at the edge of the sample space, exit.
         if(ledge)then
!            write(*,*)'Edge'
            exit iloop
         endif

! If the gradient is '0', exit.
         call sum2vxd(grad,norm2,actdof)
         if(norm2.lt.1.0e-6_dop)then 
!            write(*,*)'Gradient'
            exit iloop
         else
! If the variance hasn't increased, reset the geometry and exit.
            if(var.le.varm1)then
               gpoint = gpointm1
!               write(*,*)'Decrease',i
               exit iloop
            endif
! If the variance exceeds 0.5, it's very large (?) so exit anyway.
            if(var.gt.0.5_dop)then
!               write(*,*)'Large'
               exit
            endif
         endif
         
      enddo iloop

      return
      end subroutine varmax

!#########################################################################
! Subroutine to calculate the gradient of the GAP variance wrt the space
! coordinates.
! GWR 02/18
!#########################################################################
      subroutine vargrad(grad,gpred,gpoint,grid,nrec,order,lvarfst)

      implicit none

      logical(kind=4), intent(in) :: lvarfst
      integer(long), intent(in) :: nrec,order
      integer(long) ,dimension(actdof), intent(in) :: grid
      real(dop), dimension(actdof),intent(in)  :: gpred
      real(dop), dimension(actdof),intent(out) :: grad
      real(dop), dimension(ndofdd), intent(in) :: gpoint

      integer(long) :: f,irec,ierr
      real(dop), dimension(nrec) :: kvec,kgvec,tmpvec

      routine='vargrad'

! First step is to construct the covariance vector, k.
      do irec = 1,nrec
         if(lddtrans.and.ddcoo.eq.1)then
            call gapcovar(gpoint(1:ndofdd),dbnm(1:ndofdd,irec),&
                 kvec(irec),ndofdd,order,gapalpha,gapkfac)
         else
            call gapcovar(gpoint(1:ndofdd),dbgeo(1:ndofdd,irec),&
                 kvec(irec),ndofdd,order,gapalpha,gapkfac)
         endif
      enddo
      tmpvec = kvec

! Second step is to evaluate A^{-1}k by using the Cholesky decomposition of A.
      if(lvarfst)then
         call dpotrs('L',acholdim,1,achol,acholdim,tmpvec,acholdim,ierr)
      else
         call dpotrs('L',nrec,1,acholtmp,acholtmpdim,tmpvec,nrec,ierr)
      endif
      if(ierr.ne.0)then
         write(message,'(a,i6)') 'Failure to solve linear equations: ',ierr
         call errormsg
      endif

! Third step is to calculate the gradient of k and combine it with the results 
! of the first two steps.
      do f = 1,actdof
         kgvec = kvec
         do irec = 1,nrec
            if(lddtrans.and.ddcoo.eq.1)then 
               kgvec(irec) = kgvec(irec) * (gpred(f)-dbnm(grid(f),irec))
            else
               kgvec(irec) = kgvec(irec) * (gpred(f)-dbgeo(grid(f),irec))
            endif
         enddo
         grad(f) = dot_product(kgvec,tmpvec)
      enddo

! Final multiplication.
!      grad = 2.0_dop*gapalpha*grad

      return
      end subroutine vargrad
        
!#########################################################################
! Function to generate the value of a Gaussian function with the same
! centre and width as the wavepacket.
! GWR 08/16
!#########################################################################
      double precision function wpgauss(gpoint,s)

      implicit none

      integer(long), intent(in) :: s
      real(dop), dimension(ndofdd), intent(in) :: gpoint

      integer(long) :: f,m,nf,n
      real(dop) :: x,xcent,xwidth,gausstmp

      gausstmp = 1.0_dop
      do m = 1,nmode
         if(m.eq.meb)cycle
         nf = nspfdof(m)
         do n = 1,nf
            f = spfdof(n,m)
            x = gpoint(f)
            xcent = wfposn(f,s)
            xwidth = wfwidth(f,s)
            gausstmp = gausstmp * exp(-0.25_dop*((x-xcent)/xwidth)**2)
         enddo
      enddo

      wpgauss = gausstmp

      return
      end function wpgauss

!#########################################################################
! Routine to calculate the actual values of the GAP terms at each grid 
! point. These values are placed in the hops array.
! GWR 07/16
! Modified to allow use of SOCs.
! GWR 02/19
!#########################################################################
      subroutine gaphop(ort)

      implicit none

      real(dop), dimension(ortdim), intent(in):: ort

      logical(kind=4),dimension(nmode,khtot(1)):: kcalc
      logical(kind=4), dimension(khtot(1))    :: firstk
      integer(long) :: m,n,nf,f,gpnt,mgpnt,p,irec,s,s1,k1c,k2c,k,h,h1,k1u,k2u
      integer(long) :: sdm
      integer(long), dimension(maxdim)        :: grid,halias,htmdof1
      real(dop), dimension(sdim)              :: tmpexp
      real(dop), dimension(maxnspfdof,sdim)   :: expon
      real(dop), dimension(maxdim)            :: gpoint

      integer(long), save                     :: tid = 0 ! timer ID
      call get_timer(tid, 'gaphop')
      call start_timer(tid)

      hopsgap(:) = 0.0_dop

! Initialise the hops array:
! For correlated terms this is with the GPR weights (hence the k-loop), 
! for uncorrelated terms we simply initialise to 0.
      k1c = zham(1)
      k2c = zham(1)+kzahl(1)-1
      hopsgap = 0.0_dop
      kcalc = .false.
      if(.not.lexact)then
         firstk = .true.
         do m = 1,nmode
            nf = nspfdof(m)
            sdm = subdim(m)
            do k = k1c,k2c
               if(kinop(m,k).ne.0 .or. diag(m,k))cycle
               do n = 1,nf
                  f = spfdof(n,m)
                  h = hamilton(f,k)
                  kcalc(m,k) = kcalc(m,k) .or. (f.ne.h)
               enddo
               if(.not.kcalc(m,k))cycle

               s  = ksf(k)
               s1 = ksi(k)
               do irec = 1,dbnrec
                  if(firstk(k))then
                     hopsgap(zhopgap(m,k,irec):zhopgap(m,k,irec)+sdm-1) = &
                          dbgapwgt(s,s1,irec)
                     if(lddsoc) then
                        if(lsocbas)then
                           hopsgap(zhopgap(m,k,irec)+sdm:zhopgap(m,k,irec)+2*sdm-1) &
                                = dbcgapwgt(s,s1,irec)
                        else
                           hopsgap(zhopgap(m,k,irec):zhopgap(m,k,irec)+sdm-1) = &
                                hopsgap(zhopgap(m,k,irec):zhopgap(m,k,irec)+sdm-1)&
                                + dble(dbsocwgt(s,s1,irec))
                           hopsgap(zhopgap(m,k,irec)+sdm:zhopgap(m,k,irec)+2*sdm-1) &
                                = aimag(dbsocwgt(s,s1,irec))
                        endif
                     endif
                  else
                     hopsgap(zhopgap(m,k,irec):zhopgap(m,k,irec)+sdm-1)=1.0_dop
                     if(lddsoc) &
                          hopsgap(zhopgap(m,k,irec)+sdm:zhopgap(m,k,irec)+2*sdm-1)=1.0_dop
                  endif
               enddo
               firstk(k) = .false.
            enddo
         enddo
      endif

! Now include the values of the Gaussians.
      do m = 1,nmode
         grid = 0
         nf = nspfdof(m)
         do n = 1,nf
            grid(n) = spfdof(n,m)
         enddo
         sdm = subdim(m)

         do f = 1,ndof
            if(basis(f).eq.19)then
               gpoint(f) = ort(zort(f))
            else
               gpoint(f) = 0.0_dop
            endif
         enddo

         do irec = 1,dbnrec

            do p = 1,subdim(m)
               gpnt=p-1
               do n=1,nf
                  f=grid(n)
                  mgpnt=gpnt
                  gpnt=gpnt/gdim(f)
                  mgpnt=mgpnt-gpnt*gdim(f)
                  gpoint(f)=ort(zort(f)+mgpnt)  
               enddo

               call gapexpeval(irec,expon(1,p),gpoint,grid,nf)
            enddo

! Correlated terms first.
! For MCTDH first.
            if(.not.lexact)then
               do k = k1c,k2c
                  if(kinop(m,k).ne.0 .or. diag(m,k) .or. .not.kcalc(m,k))cycle
                  s  = kf(k)
                  s1 = ki(k)
                  
                  f=grid(1)
                  h=hamilton(f,k)
                  
                  halias = 0
                  if(htmmode(h).ne.0)then
                     do h1 = 1,htmfac(h)
                        halias(h1) = hterm(h1,h)
                     enddo
                  else
                     halias(1) = h
                  endif
                  do h1 = 1,htmfac(h)
                     htmdof1(h1) = htmdof(halias(h1))
                  enddo
                  
                  do n = 1,nf
                     do h1 = 1,htmfac(h)
                        if(grid(n).eq.htmdof1(h1))then   
                           do p = 1,subdim(m)
                              hopsgap(zhopgap(m,k,irec)+p-1) = &
                                   hopsgap(zhopgap(m,k,irec)+p-1) * expon(n,p)
                              if(lddsoc) hopsgap(zhopgap(m,k,irec)+sdm+p-1) = &
                                   hopsgap(zhopgap(m,k,irec)+sdm+p-1) * expon(n,p)

                           enddo
                        endif
                     enddo
                  enddo
               enddo
            else
! Then inter-state operators for the standard method.
               do k = k1c,k2c
                  if(kinop(m,k).ne.0 .or. diag(m,k))cycle
                  s  = kf(k)
                  s1 = ki(k)
                  
                  f=grid(1)
                  h=hamilton(f,k)
                  
                  halias = 0
                  if(htmmode(h).ne.0)then
                     do h1 = 1,htmfac(h)
                        halias(h1) = hterm(h1,h)
                     enddo
                  else
                     halias(1) = h
                  endif
                  do h1 = 1,htmfac(h)
                     htmdof1(h1) = htmdof(halias(h1))
                  enddo
                  
                  tmpexp = 1.0_dop
                  do n = 1,nf
                     do h1 = 1,htmfac(h)
                        if(grid(n).eq.htmdof1(h1))then                       
                           do p = 1,subdim(m)
                              tmpexp(p) = tmpexp(p)*expon(n,p)
                           enddo
                        endif
                     enddo
                  enddo
                  do p = 1,subdim(m)
                     hopsgap(zhopgap(m,k,1)+p-1) = hopsgap(zhopgap(m,k,1)+p-1) &
                          + tmpexp(p)*dbgapwgt(s,s1,irec)
                     if(lddsoc) then
                        if(lsocbas)then
                           hopsgap(zhopgap(m,k,1)+sdm+p-1) &
                                = hopsgap(zhopgap(m,k,1)+sdm+p-1)&
                                +tmpexp(p)*dbcgapwgt(s,s1,irec)
                        else
                           hopsgap(zhopgap(m,k,1)+p-1) &
                                = hopsgap(zhopgap(m,k,1)+p-1)&
                                +tmpexp(p)*dble(dbsocwgt(s,s1,irec))
                           hopsgap(zhopgap(m,k,1)+sdm+p-1) &
                                = hopsgap(zhopgap(m,k,1)+sdm+p-1)&
                                +tmpexp(p)*aimag(dbsocwgt(s,s1,irec))
                        endif
                     endif
                  enddo
               enddo
            endif
! Uncorrelated terms next.
            do s = 1,nddstate
               k1u = zhun(m,s,1)
               k2u = zhun(m,s,1)+khzahl(m,s,1)-1
               do k = k1u,k2u
                  if(kinop(m,k).ne.0 .or. diag(m,k))cycle
                  
                  f=grid(1)
                  h=hamilton(f,k)
                  
                  halias = 0
                  if(htmmode(h).ne.0)then
                     do h1 = 1,htmfac(h)
                        halias(h1) = hterm(h1,h)
                     enddo
                  else
                     halias(1) = h
                  endif
                  do h1 = 1,htmfac(h)
                     htmdof1(h1) = htmdof(halias(h1))
                  enddo
                  
                  tmpexp = 1.0_dop
                  do n = 1,nf
                     do h1 = 1,htmfac(h)
                        if(grid(n).eq.htmdof1(h1))then                       
                           do p = 1,subdim(m)
                              tmpexp(p) = tmpexp(p)*expon(n,p)
                           enddo
                        endif
                     enddo
                  enddo
                  do p = 1,subdim(m)
                     hopsgap(zhopgap(m,k,1)+p-1) = hopsgap(zhopgap(m,k,1)+p-1) &
                          + tmpexp(p)*dbgapwgt(s,s,irec)
! SOC matrix is Hermitian so imaginary part is 0 on the diagonal.
                     if(lddsoc .and. .not. lsocbas)&
                          hopsgap(zhopgap(m,k,1)+p-1) &
                          = hopsgap(zhopgap(m,k,1)+p-1) &
                          + tmpexp(p)*dble(dbsocwgt(s,s,irec))

! Add energy shift (usually 0).
                     if(irec.eq.1) hopsgap(zhopgap(m,k,1)+p-1) &
                          = hopsgap(zhopgap(m,k,1)+p-1) + gapeshift

                  enddo
               enddo              
            enddo
         enddo
      enddo

      call stop_timer(tid)

      return
      end subroutine gaphop

!#########################################################################
! Routine to calculate the actual values of the GAP terms at each grid 
! point. These values are placed in the hops array.
! Variant for use with OPTGAP.
! GWR 05/17
!#########################################################################
      subroutine optgaphop(ort)

      implicit none

      real(dop), dimension(ortdim), intent(in):: ort

      integer(long) :: m,n,nf,f,gpnt,mgpnt,p,k,kidx,g,g1,n1
      integer(long), dimension(maxdim)        :: grid
      real(dop) :: xg,xg1
      real(dop), dimension(maxgdim,ddmoptord) :: opgrid

      integer(long), save                     :: tid = 0 ! timer ID
      call get_timer(tid, 'optgaphop')
      call start_timer(tid)

! Rewind the list of operators.
      call ogparrew(ogpar)

      hopsgap = 1.0_dop

! Loop over the potential operator list.
      do while (associated(ogpar%next))

! First, evaluate the potential terms along the DVR grids (i.e. along 1 DOF each).
         kidx = ogpar%kidx
         k = ogpar%k

         opgrid = 0.0_dop
         do n = 1,ogpar%ndofs
            f = ogpar%dofs(n)
            do g = 1,gdim(f)
               xg = ort(zort(f)+g-1)
               do g1 = 1,ogpar%gdofs(n)
                  xg1 = ogpar%gaport(g1,n)
                  opgrid(g,n) = opgrid(g,n) + ogpar%coeff(1,g1,n)&
                       *exp(-ogpar%alpha(n)*(xg-xg1)*(xg-xg1))
               enddo
            enddo
         enddo                

! Now combine the DOF grid values into mode values depending on whether 
! this is a correlated or uncorrelated operator.
         if(ogpar%lcorr)then
            do m = 1,nmode
               nf = nspfdof(m)
               do n = 1,nf
                  grid(n) = spfdof(n,m)
               enddo
               do p = 1,subdim(m)
                  gpnt=p-1
                  do n=1,nf
                     f = grid(n)
                     mgpnt=gpnt
                     gpnt=gpnt/gdim(f)
                     mgpnt=mgpnt-gpnt*gdim(f)+1
                     do n1 = 1,ogpar%ndofs
                        if(ogpar%dofs(n1).ne.f)cycle
                        hopsgap(zhopgap(m,kidx,1)+p-1) = &
                             hopsgap(zhopgap(m,kidx,1)+p-1) * opgrid(mgpnt,n1)
                     enddo
                  enddo
               enddo
            enddo
         else
            f = ogpar%dofs(1)
            m = dofspf(f)          
            nf = nspfdof(m)
            do n = 1,nf
               grid(n) = spfdof(n,m)
            enddo
            do p = 1,subdim(m)
               gpnt=p-1
               do n=1,nf
                  f = grid(n)
                  mgpnt=gpnt
                  gpnt=gpnt/gdim(f)
                  mgpnt=mgpnt-gpnt*gdim(f)+1
                  do n1 = 1,ogpar%ndofs
                     if(ogpar%dofs(n1).ne.f)cycle
                     hopsgap(zhopgap(m,kidx,1)+p-1) = &
                          hopsgap(zhopgap(m,kidx,1)+p-1) * opgrid(mgpnt,n1)
                  enddo
               enddo
            enddo
         endif

         ogpar => ogpar%next
      enddo

      call stop_timer(tid)

      return
      end subroutine optgaphop

!#########################################################################
! Routine to calculate the actual values of the PES terms at each grid 
! point. These values are placed in the hops array.
! Variant for use with SVD generated approximation to GAP.
! GWR 05/17
!#########################################################################
      subroutine tensgaphop

      implicit none

      integer(long) :: m,n,nf,f,gpnt,mgpnt,p,k,kidx,g,n1,vcdim,sdm
      integer(long), dimension(maxdim)        :: grid
!      real(dop), dimension(maxgdim,ddmoptord) :: opgrid
      real(dop), dimension(:,:,:), allocatable :: opgrid

      integer(long), save                     :: tid = 0 ! timer ID
      call get_timer(tid, 'tensgaphop')
      call start_timer(tid)

      if(lddsoc)then
         vcdim = 2
      else
         vcdim = 1
      endif
      if(allocated(opgrid))deallocate(opgrid)
      allocate(opgrid(vcdim,maxgdim,ddmoptord))

! Rewind the list of operators.
      call ogparrew(ogpar)

! Initialise the potential.
      hopsgap = 1.0_dop

! Loop over the potential operator list.
      do while (associated(ogpar%next))

! First, evaluate the potential terms along the DVR grids (i.e. 
! along 1 DOF each).
         kidx = ogpar%kidx
         k = ogpar%k

         opgrid = 0.0_dop
         do n = 1,ogpar%ndofs
            f = ogpar%dofs(n)
            if(gdim(f).ne.ogpar%gdofs(n))then
               routine = 'tensgaphop'
               message = 'Inconsistent number of gridpoints in DVR and op.'
               call errormsg
            endif
            do g = 1,gdim(f)
               opgrid(1:vcdim,g,n) = ogpar%coeff(1:vcdim,g,n)
            enddo
         enddo                
! Now combine the DOF grid values into mode values depending on whether 
! this is a correlated or uncorrelated operator.
         if(ogpar%lcorr)then
            do m = 1,nmode
               sdm = subdim(m)
               nf = nspfdof(m)
               do n = 1,nf
                  grid(n) = spfdof(n,m)
               enddo
               do p = 1,subdim(m)
                  gpnt=p-1
                  do n=1,nf
                     f = grid(n)
                     mgpnt=gpnt
                     gpnt=gpnt/gdim(f)
                     mgpnt=mgpnt-gpnt*gdim(f)+1
                     do n1 = 1,ogpar%ndofs
                        if(ogpar%dofs(n1).ne.f)cycle
                        hopsgap(zhopgap(m,kidx,1)+p-1) = &
                             hopsgap(zhopgap(m,kidx,1)+p-1) * opgrid(1,mgpnt,n1)
                        if(lddsoc)&
                             hopsgap(zhopgap(m,kidx,1)+sdm+p-1) = &
                             hopsgap(zhopgap(m,kidx,1)+sdm+p-1) * opgrid(2,mgpnt,n1)
                     enddo
                  enddo
               enddo
            enddo
         else
            f = ogpar%dofs(1)
            m = dofspf(f)          
            sdm = subdim(m)
            nf = nspfdof(m)
            do n = 1,nf
               grid(n) = spfdof(n,m)
            enddo
            do p = 1,subdim(m)
               gpnt=p-1
               do n=1,nf
                  f = grid(n)
                  mgpnt=gpnt
                  gpnt=gpnt/gdim(f)
                  mgpnt=mgpnt-gpnt*gdim(f)+1
                  do n1 = 1,ogpar%ndofs
                     if(ogpar%dofs(n1).ne.f)cycle
                     hopsgap(zhopgap(m,kidx,1)+p-1) = &
                          hopsgap(zhopgap(m,kidx,1)+p-1) * opgrid(1,mgpnt,n1)
                     if(lddsoc)&
                          hopsgap(zhopgap(m,kidx,1)+sdm+p-1) = &
                          hopsgap(zhopgap(m,kidx,1)+sdm+p-1) * opgrid(2,mgpnt,n1)
                  enddo
               enddo
            enddo
         endif

         ogpar => ogpar%next
      enddo

      if(allocated(opgrid))deallocate(opgrid)

      call stop_timer(tid)

      return
      end subroutine tensgaphop

!#########################################################################
! Subroutine to pass updated GAP operator to the slave processes. 
!#########################################################################
      subroutine gaphop_bcast
#ifdef MPI
use mpi

      mpitask = 50
      call MPI_Bcast(mpitask,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(dbnrec,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)        
      if (.not. lexact) then
         call MPI_Bcast(pmatgapdim,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
         call MPI_Bcast(hpsigapdim,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
         call MPI_Bcast(hlochgapdim,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
         call MPI_Bcast(hopsgapdim,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
         call MPI_Bcast(zhpsigap(1,zham(1),1),nmode*kzahl(1)*dbnrec,MPI_INTEGER,&
              master,MPI_COMM_WORLD,mpierr)
         call MPI_Bcast(pmatgap(1,zham(1),1),nmode*(kzahl(1)+maxsta)*dbnrec,&
              MPI_INTEGER,&
              master,MPI_COMM_WORLD,mpierr)
         call MPI_Bcast(phlochgap(1,zham(1),1),nmode*kzahl(1)*dbnrec,MPI_INTEGER,&
              master,MPI_COMM_WORLD,mpierr)
         call MPI_Bcast(zhopgap,nmode*khtot(1)*dbnrec,MPI_INTEGER,master,&
              MPI_COMM_WORLD,mpierr)
         call MPI_Bcast(hopsgap,hopsgapdim,MPI_DOUBLE,master,MPI_COMM_WORLD,mpierr)
      else
         call MPI_Bcast(hopsgapdim,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
         call MPI_Bcast(zhopgap,nmode*khtot(1)*1,MPI_INTEGER,master,&
              MPI_COMM_WORLD,mpierr)
         call MPI_Bcast(hopsgap,hopsgapdim,MPI_DOUBLE,master,MPI_COMM_WORLD,mpierr)
      endif

#endif
      end subroutine gaphop_bcast

!#########################################################################
! Subroutine to determine the sizes of intermediate arrays for GAP 
! matrix elements.
! Adapted from zeighpsi.
! GWR 07/16
!#########################################################################
      subroutine zeighpsigap

      implicit none

      integer(long) :: m,irec,s,s1,zeig,sdm1,k1,k2,k3,k,f,dbnrec1

      routine='zeighpsigap'

!-----------------------------------------------------------------------
! determine pointers to hpsi array
!-----------------------------------------------------------------------
      if(.not.lexact)then
         k1=zham(1)
         k2=zham(1)+kzahl(1)-1
         zeig=1
         do m = 1,nmode
            if(gwpm(m))cycle
            do k = k1,k2
               if(kinop(m,k).ne.0.or.diag(m,k))cycle
               if (k .eq. kfirst(m,k)) then
                  s = ki(k)
                  do irec = 1,dbnrec
                     zhpsigap(m,k,irec) = zeig
                     zeig = zeig+subdim(m)*dim(m,s)
                  enddo
               else
                  k3=kfirst(m,k)
                  do irec = 1,dbnrec
                     zhpsigap(m,k,irec) = zhpsigap(m,k3,irec)
                  enddo
               endif
            enddo
         enddo
         hpsigapdim=zeig-1
         
! test mem
         hpsigapdim = sdim*maxspf
! test mem

!-----------------------------------------------------------------------
! pointers for single particle function matrices hloch
!-----------------------------------------------------------------------
         zeig=1
         do m=1,nmode
            if(idmode(m).eq.0) then
               do k = k1,k2
                  if(kinop(m,k).ne.0.or.diag(m,k))cycle
                  s=kf(k)
                  s1=ki(k)
                  do irec=1,dbnrec
                     phlochgap(m,k,irec) = zeig
                     zeig=zeig+dim(m,s)*dim(m,s1)
                  enddo
               enddo
            else
               message='idmode not 0'
               call errormsg
            endif
         enddo
         hlochgapdim = zeig-1

!-----------------------------------------------------------------------
! pointers for single-particle function matrices
!
! If the "proj-h" input option is used or CAPS are present the matrix
! elements <phi|h|phi> are also needed for the uncorrelated Hamiltonian
! terms. Effectively the number of Hamiltonian terms is then
! kzahl+nstate.
!-----------------------------------------------------------------------
         zeig = 1
         do m=1,nmode
            do k =k1,k2
!               if(kinop(m,k).ne.0.or.diag(m,k))cycle
               if(kinop(m,k).eq.0.and..not.diag(m,k))then
!                  if(k.eq.kfirst(m,k))then
                     s=kf(k)
                     s1=ki(k)
                     do irec=1,dbnrec
                        pmatgap(m,k,irec) = zeig
                        zeig = zeig+dim(m,s)*dim(m,s1)
                     enddo
!                  else
!                     k3=kfirst(m,k)
!                     do irec=1,dbnrec
!                        pmatgap(m,k,irec)=pmatgap(m,k3,irec)
!                     enddo
!                  endif
               else
                  pmatgap(m,k,1:dbnrec)=1
               endif
            enddo

            k = zham(1)+kzahl(1)-1
            do s = 1,nddstate
               if( khzahl(m,s,1) .gt. 0 ) then
                  pmatgap(m,k+s,1) = zeig
                  zeig = zeig+dim(m,s)*dim(m,s)
               else
                  pmatgap(m,k+s,1) = 1
               endif
            enddo
               
         enddo
         pmatgapdim = zeig-1

      endif
!------------------------------------------------------------------------
! Pointers for the operator array.
!------------------------------------------------------------------------
      if(lexact)then
         dbnrec1=1
      else
         dbnrec1 = dbnrec
      endif
      k1 = zham(1)
      k2 = zham(1)+kzahl(1)- 1
      zeig = 1
      do m = 1,nmode
         if(nspfdof(m).eq.1)then
            f = spfdof(1,m)
            sdm1 = gdim(f)
         else
            sdm1 = subdim(m)
         endif
         if(lddsoc) sdm1 = 2*sdm1
         do k = k1,k2
            if(kinop(m,k).ne.0.or.diag(m,k))cycle
            do irec = 1,dbnrec1
               zhopgap(m,k,irec)=zeig
               zeig=zeig+sdm1
            enddo
         enddo
      enddo

      do m = 1,nmode
         if(nspfdof(m).eq.1)then
            f = spfdof(1,m)
            sdm1 = gdim(f)
         else
            sdm1 = subdim(m)
         endif

         if(lddsoc) sdm1 = 2*sdm1

         do s = 1,nddstate           
            k1 = zhun(m,s,1)
            k2 = zhun(m,s,1)+khzahl(m,s,1)-1
            do k = k1,k2
               if(kinop(m,k).ne.0.or.diag(m,k))cycle
               zhopgap(m,k,1)=zeig
               zeig=zeig+sdm1
            enddo            
         enddo
      enddo

      hopsgapdim = zeig-1

      return

      end subroutine zeighpsigap

!#########################################################################
! Subroutine to determine the sizes of intermediate arrays for GAP 
! matrix elements - optimised GAP variant.
! Adapted from zeighpsi.
! GWR 05/17
!#########################################################################
      subroutine zeighpsioptgap

      implicit none

      logical(kind=4), dimension(maxdim) :: lf
      integer(long) :: m,s,s1,zeig,k,f,n,kc,vcdim

      routine='zeighpsioptgap'

!-----------------------------------------------------------------------
! determine pointers to hpsi array
!-----------------------------------------------------------------------
      if(.not.lexact)then
         call ogparrew(ogpar)
         zhpsigap = 0
         zeig = 1
         do while(associated(ogpar%next))

            if(ogpar%lcorr)then

               k = ogpar%k
               kc = ogpar%kcorridx
               do n = 1,ogpar%ndofs
                  f = ogpar%dofs(n)
                  m = dofspf(f)
                  s = ogpar%ki
                  zhpsigap(m,kc,1) = zeig
                  zeig = zeig+subdim(m)*dim(m,s)
               enddo
            endif
            ogpar => ogpar%next
         enddo
         hpsigapdim = zeig-1
      endif
      
!-----------------------------------------------------------------------
! pointers for single particle function matrices hloch
!-----------------------------------------------------------------------
      if(.not.lexact)then
         phlochgap = 0
         zeig=1
         call ogparrew(ogpar)
         do while(associated(ogpar%next))
            if(ogpar%lcorr)then
               kc = ogpar%kcorridx
! test
               k = ogpar%k
               do m = 1,nmode
                  if(idmode(m).ne.0)then
                     message='idmode not 0'
                     call errormsg
                  endif
                  if(.not.diag(m,k))then
                     s = ogpar%kf
                     s1 = ogpar%ki
                     phlochgap(m,kc,1) = zeig
                     zeig = zeig+dim(m,s)*dim(m,s1)
                  else
                     phlochgap(m,kc,1) = 1
                  endif
               enddo
! test
!               do n = 1,ogpar%ndofs
!                  f = ogpar%dofs(n)
!                  m = dofspf(f)
!                  if(idmode(m).ne.0)then
!                     message='idmode not 0'
!                     call errormsg
!                  endif
!                  s = ogpar%kf
!                  s1 = ogpar%ki
!                  phlochgap(m,kc,1) = zeig
!                  zeig = zeig + dim(m,s)*dim(m,s1)
!               enddo
! test
            endif
            ogpar => ogpar%next
         enddo
         hlochgapdim = zeig-1
      endif

!-----------------------------------------------------------------------
! pointers for single-particle function matrices
!
! If the "proj-h" input option is used or CAPS are present the matrix
! elements <phi|h|phi> are also needed for the uncorrelated Hamiltonian
! terms. Effectively the number of Hamiltonian terms is then
! kzahl+nstate.
!-----------------------------------------------------------------------
      if(.not.lexact)then
         pmatgap = 0
         zeig = 1
         call ogparrew(ogpar)
         do while(associated(ogpar%next))
            if(ogpar%lcorr)then
               k = ogpar%k
               kc = ogpar%kcorridx
! test
               s = ogpar%kf
               s1 = ogpar%ki
               do m = 1,nmode
                  if(kinop(m,k).eq.0.and..not.diag(m,k))then
                     pmatgap(m,kc,1) = zeig
                     zeig = zeig + dim(m,s)*dim(m,s1)
                  else
                     pmatgap(m,kc,1) = 1
                  endif
               enddo
            endif
            ogpar => ogpar%next
         enddo

         kc = ktotcog
         do m = 1,nmode
            do s = 1,nddstate
               if( khzahl(m,s,1) .gt. 0 ) then
                  pmatgap(m,kc+s,1) = zeig
                  zeig = zeig+dim(m,s)*dim(m,s)
               else
                  pmatgap(m,kc+s,1) = 1
               endif
            enddo
         enddo

         pmatgapdim = zeig-1

      endif

!------------------------------------------------------------------------
! Pointers for the operator array.
!------------------------------------------------------------------------
      if(lddsoc)then
         vcdim = 2
      else
         vcdim=1
      endif
      call ogparrew(ogpar)
      zhopgap = 0
      zeig = 1
      do while (associated(ogpar%next))

         k = ogpar%kidx

         if(ogpar%lcorr)then
            lf = .true.
            do n = 1,ogpar%ndofs
               f = ogpar%dofs(n)
               m = dofspf(f)
               if(lf(m))then
                  zhopgap(m,k,1) = zeig
                  zeig = zeig+subdim(m)*vcdim
                  lf(m) = .false.
               endif
            enddo
         else
            f = ogpar%dofs(1)
            m = dofspf(f)
            zhopgap(m,k,1) = zeig
            zeig = zeig+subdim(m)*vcdim
         endif
         ogpar => ogpar%next

      enddo

      hopsgapdim = zeig-1

      return

      end subroutine zeighpsioptgap

!##########################################################################
! Subroutine to deal with calculating the product of the GAP and the SPF
! coefficients.
! GWR 07/16
!##########################################################################
      subroutine hphigap(psi,hpsi)

      implicit none

      complex(dop), dimension(dgldim), intent(in)      :: psi
      complex(dop), dimension(hpsigapdim), intent(out) :: hpsi

      integer(long) :: m

      integer(long), save                            :: tid = 0 ! timer ID
      call get_timer(tid, 'hphigap')
      call start_timer(tid)

      do m = 1,nmode
         call hphigap1m(psi(zetf(m,1)),hpsi,m,zetf(m,1),phidim(m))
      enddo

      call stop_timer(tid)
      return
      end subroutine hphigap
      

! *** V6.6 GW ***************> SUBROUTINE HUNPHI <**********************
!                             ~~~~~~~~~~~~~~~~~~~
!
! Computes for all modes and all states the action of the uncorrelated
! Hamiltonian on the single-particle functions.
! Result is returned as a sum of all terms in hunpsi.

      subroutine hunphigap (psi,hunpsi,hktype,nham)

      implicit none

      integer(long)                                       :: f,m,s
      integer(long), save                                 :: tid=0
      integer(long), intent(in)                           :: hktype,nham
      complex(dop), dimension(dgldim), intent(in)         :: psi
      complex(dop), dimension(adim+1:dgldim), intent(out) :: hunpsi

      call get_timer(tid,'HunPhiGAP')
      call start_timer(tid)

! --- LOOP OVER EACH MODE AND STATE ---

      do m = 1,nmode

         if (m .eq. mpb) cycle
! ... exclude Gaussian mode :
         if (gwpm(m)) cycle

         f = spfdof(1,m)
         if(idmode(m) .eq. 0) then
            do s = 1,nstate
               call hunphigap1ms(psi(zetf(m,s)),hunpsi(zetf(m,s)), &
                    m,s,dim(m,s),subdim(m),hktype,nham)
            enddo
         endif
      enddo

      call stop_timer(tid)

      return
      end subroutine hunphigap

!##########################################################################
! Subroutine to deal with calculating the product of the GAP and the SPF
! coefficients for a particular mode, m.
! GWR 07/16
!##########################################################################
      subroutine hphigap1m(psi,hpsi,m,zetf1,phidim1)

      implicit none

      integer(long), intent(in)    :: m,zetf1,phidim1
      complex(dop), dimension(zetf1:zetf1+phidim1-1), intent(in) :: psi
      complex(dop), dimension(hpsigapdim), intent(out)           :: hpsi

      integer(long) :: s1,irec,k1,k2,k,idum

      k1=zham(1)
      k2=zham(1)+kzahl(1)-1
      idum = 1

      do k = k1,k2
         if(kinop(m,k).ne.0)cycle

         do irec = 1,dbnrec
            s1 = ki(k)
            call hphigap1mk(psi(zetf(m,s1)),hpsi,m,k,irec,idum, &
                 dim(m,s1),subdim(m))
         enddo
      enddo

      return
      end subroutine hphigap1m

!#######################################################################
! Subroutine to calculate the RHS of the EOMs for an exact calculation
! using a GAP.
! Based on the routine hunphi1ms
! GWR 10/16
!#######################################################################

      subroutine hunphigap1ms (psi,hunpsi,m,s,dim1,sdm,hktype,nham)

!$ use omp_lib

      implicit none

      integer(long)                                  :: k,k1,k2,h,idum
      integer(long), intent(in)                      :: m,s,dim1,sdm,hktype,nham
      complex(dop), dimension(sdm,dim1), intent(in)  :: psi
      complex(dop), dimension(sdm,dim1), intent(out) :: hunpsi
      complex(dop), dimension(sdm,dim1)              :: tmphpsi
      logical(kind=4), dimension(maxkoe)             :: kcalc

! test
      integer(long), save :: tid=0
      call get_timer(tid,'HunPhiGAP1ms')
      call start_timer(tid)
! test

!     sdm = subdim(m), dim1=dim(m,s)
      call zeromxz(hunpsi,sdm,dim1)
      call zeromxz(tmphpsi,sdm,dim1)
      idum = 1

!-----------------------------------------------------------------------
! first find which terms need to be calculated
! autocap = 0 : normal hermitian operator
!         > 0 : non-active CAP 
!         < 0 : active CAP 
! nstatecap(k) : state on which CAP is active. If 0 all states.
!-----------------------------------------------------------------------
      if(khzahl(m,s,nham).eq.0) return
      k1=zhun(m,s,nham)
      k2=zhun(m,s,nham)+khzahl(m,s,nham)-1

      do k=k1,k2
         if (hktype .eq. 0) then
            if (autocap(k) .eq. 0) then
               kcalc(k)=.true.
            else if (autocap(k) .lt. 0 .and. nstatecap(k) .eq. 0) then
               kcalc(k)=.true.
            else if (autocap(k) .lt. 0 .and. nstatecap(k) .eq. s) then
               kcalc(k)=.true.
            endif
         else if (hktype .eq. 1 .and. autocap(k) .eq. 0) then
            kcalc(k)=.true.
         else if (hktype .eq. 2 .and. autocap(k) .lt. 0) then
            if (nstatecap(k) .eq. 0 .or. nstatecap(k) .eq. s) kcalc(k)=.true.
         else
            kcalc(k)=.false.
         endif
      enddo

! --- LOOP OVER EACH UNCORRELATED HAMILTONIAN TERM ---
      do k=k1,k2
         if (kcalc(k)) then
            h=hamilton(fkoe,k)

            if(kinop(m,k).eq.1)then
               call hphi1mk(psi,tmphpsi,m,k,dim1,sdm)
               if (htmtype(h) .eq. 1) then
                  call addmxxzo(tmphpsi,hunpsi,sdm,dim1)
               else
                  call xmxxzza(koeff(k),tmphpsi,hunpsi,sdm,dim1)
               endif

            else if(kinop(m,k).eq.0)then
               if(lddmopt .or. lddgbtendec)then
                  call ogparrew(ogpar)
                  do while (associated(ogpar%next))
                     if(ogpar%k.eq.k)then
                        call hphigap1mk(psi,tmphpsi,m,k,1,ogpar%kidx,dim1,sdm)
                        call xmxxzza(koeff(k),tmphpsi,hunpsi,sdm,dim1)
                     endif
                     ogpar => ogpar%next
                  enddo
               else
!!$omp parallel num_threads(ompthread) if(lompthread) private(irec,tmphpsi)
!!$omp do reduction(+:tmphunpsi)
!               do irec = 1,dbnrec
!                  call hphigap1mk(psi,tmphpsi,m,k,irec,dim1,sdm)
                  call hphigap1mk(psi,tmphpsi,m,k,1,idum,dim1,sdm)
                  call xmxxzza(koeff(k),tmphpsi,hunpsi,sdm,dim1)
!                  call xmxxzza(koeff(k),tmphpsi,tmphunpsi,sdm,dim1)
!               enddo
!!$omp end do
!!$omp end parallel
               endif
            endif
         endif
      enddo

      call stop_timer(tid)

      return
      end subroutine hunphigap1ms
      
!##########################################################################
! Subroutine to deal with calculating the product of the GAP and the SPF
! coefficients, for a particular mode, m, operator, k, and DB point, irec.
! GWR 07/16
!##########################################################################
      subroutine hphigap1mk(psi,hpsi,m,k,irec,kog,dim1,subdim1)

      implicit none

      integer(long), intent(in) :: m,dim1,subdim1,irec,k,kog
      complex(dop), dimension(subdim1,dim1), intent(in)  :: psi
      complex(dop), dimension(subdim1,dim1), intent(out) :: hpsi

      integer(long) :: e

!      integer(long), save                            :: tid = 0 ! timer ID
!      call get_timer(tid, 'hphigap1mk')
!      call start_timer(tid)

      do e = 1,dim1
         call hcallgap(m,k,irec,kog,hpsi(1,e),psi(1,e),subdim1)
      enddo

!      call stop_timer(tid)

      return
      end subroutine hphigap1mk
      
!##########################################################################
! Subroutine to deal with calculating the product of the GAP and the SPF
! coefficients, for a particular mode, m, operator, k, DB point, irec, 
! and SPF.
! GWR 07/16
!##########################################################################
      subroutine hcallgap(m,k,irec,kog,hpsi,psi,subdim1)

      implicit none

      integer(long), intent(in) :: m,subdim1,irec,k,kog
      complex(dop), dimension(subdim1), intent(in)  :: psi
      complex(dop), dimension(subdim1), intent(out) :: hpsi

      integer(long) :: f,h

      f = spfdof(1,m)
      h = hamilton(f,k)

      call hopgap(h,m,k,irec,kog,hpsi,psi,hopsgap,subdim1)

      return
      end subroutine hcallgap
      
!##########################################################################
! Subroutine to actually calculate the product of the GAP and the SPF
! coefficients.
! GWR 07/16
!##########################################################################
      subroutine hopgap(h,m,k,irec,kog,hpsi,psi,hops,subdim1)

      implicit none

      integer(long), intent(in) :: subdim1,irec,h,m,k,kog
      real(dop), dimension(hopsgapdim), intent(in)  :: hops
      complex(dop), dimension(subdim1), intent(in)  :: psi
      complex(dop), dimension(subdim1), intent(out) :: hpsi

      integer(long) :: vd,gd,nd

      call getvgn(h,vd,gd,nd)

! Unit matrix
      if(htmtype(h).eq.1)then
         call cpvxz(psi,hpsi,subdim1)
! Real vector (diagonal matrix).
      else if(htmtype(h).eq.2 .or. htmtype(h).eq.202)then
         if(lddmopt .or. lddgbtendec)then
            call dtxxdz(hops(zhopgap(m,kog,1)),psi,hpsi,vd,gd,nd)
         else
            call dtxxdz(hops(zhopgap(m,k,irec)),psi,hpsi,vd,gd,nd)
         endif
      else if(htmtype(h).eq.4 .or. htmtype(h).eq.204)then
         if(lddmopt .or. lddgbtendec)then
            call dtxxyz(hops(zhopgap(m,kog,1)),psi,hpsi,vd,gd,nd)
!            routine = 'hopgap'
!            write(message,'(a,3i6)') 'SOCs with SVD not implemented yet'
!            call errormsg
         else
            call dtxxyz(hops(zhopgap(m,k,irec)),psi,hpsi,vd,gd,nd)
         endif        
!      else if(htmtype(h).eq.8)then
!         call qtxxdz(hops(zhop(h)),psi,hpsi,vd,gd,nd)
      else
         routine = 'hopgap'
         write(message,'(a,3i6)') 'Unrecognised htmtype (k,h,htmtype)',&
              k,h,htmtype(h)
         call errormsg
      endif

      return
      end subroutine hopgap

!##########################################################################
! Subroutine to calculate the SPF GAP matrix elements.
! GWR 07/16
!##########################################################################
      subroutine phihphigap(psi,hteil)

!$ use omp_lib
#ifdef MPI
use mpi
#endif
      implicit none

      complex(dop), dimension(dgldim), intent(in)      :: psi
      complex(dop), allocatable                        :: hpsi(:)
      complex(dop), dimension(pmatgapdim), intent(out) :: hteil

      integer(long) :: k1,k2,k,m,s,s1,irec,idum,kidx,kcorridx

      integer(long), save :: tid = 0 ! timer ID
      call get_timer(tid, 'phihphigap')
      call start_timer(tid)

      idum = 1
      routine='Phihphigap'
         
      if(lmpi.and.lmpistarted)then
#ifdef MPI
         if(lddmopt .or. lddgbtendec)then
            message='OPT/SVD GAP not implemented with MPI yet'
            call errormsg
         endif
         mpitask=51
         call MPI_Bcast(mpitask,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
         call mpiphihphigap(psi(zetf(1,1)),hteil)
#else
         message='Wants MPI code but MPI not compiled'
         call errormsg
#endif       
      else

         k1=zham(1)
         k2=zham(1)+kzahl(1)-1

! Are we using OPTGAP or normal GAP?
         if(lddmopt .or. lddgbtendec)then
!$omp parallel num_threads(ompthread) if(lompthread) private(m,k,s,s1,hpsi,kidx,kcorridx,ogpar1)
            if(allocated(hpsi))deallocate(hpsi)
            allocate(hpsi(hpsigapdim))
            ogpar1 => ogpar
!$omp do
            do k = k1,k2
               s=kf(k)
               s1=ki(k)

               do m = 1,nmode
!               if(diag(m,k) .or. k.ne.(kfirst(m,k)))cycle
                  if(diag(m,k) .or. kinop(m,k).ne.0)cycle
                  if(abs(hsym(m,k)).gt.1)then
                     routine='phihphigap'
                     message='Wrong symmetry for OPTGAP'
                     call errormsg
                  endif
                  call ogparrew(ogpar1)
                  do while(associated(ogpar1%next))
                     if(ogpar1%k.eq.k)then
                        kidx = ogpar1%kidx
                        kcorridx = ogpar1%kcorridx
                        call hphigap1mk(psi(zetf(m,s1)),hpsi,m,k,idum,&
                             kidx,dim(m,s1),subdim(m))
                        if(hsym(m,k).eq.0 .or. .not. usehsym)then
                           call mmaxzz(psi(zetf(m,s)),hpsi,&
                                hteil(pmatgap(m,kcorridx,idum)),subdim(m),dim(m,s),&
                                dim(m,s1))
                        else if(hsym(m,k).eq.1)then
                           call mmaxzzh(psi(zetf(m,s)),hpsi,&
                                hteil(pmatgap(m,kcorridx,idum)),subdim(m),dim(m,s))
                        else if(hsym(m,k).eq.-1)then
                           call mmaxzzh1(psi(zetf(m,s)),hpsi,&
                                hteil(pmatgap(m,kcorridx,idum)),subdim(m),dim(m,s))
                        endif
                     endif
                     ogpar1 => ogpar1%next
                  enddo
               enddo
            enddo
!$omp enddo
            deallocate(hpsi)
            nullify(ogpar1)
!$omp end parallel
         else
!$omp parallel num_threads(ompthread) if(lompthread) private(m,k,s,s1,hpsi)
            if(allocated(hpsi))deallocate(hpsi)
            allocate(hpsi(hpsigapdim))
!$omp do
            do irec = 1,dbnrec
               do k = k1,k2
                  s=kf(k)
                  s1=ki(k)
                  
                  do m = 1,nmode
!               if(diag(m,k) .or. k.ne.(kfirst(m,k)))cycle
                     if(diag(m,k))cycle
                     if(kinop(m,k).ne.0)cycle
                     if(abs(hsym(m,k)).gt.1)then
                        routine='phihphigap'
                        message='Wrong symmetry for GAP'
                        call errormsg
                     endif
                     call hphigap1mk(psi(zetf(m,s1)),hpsi,m,k,irec,idum,&
                          dim(m,s1),subdim(m))
                     if(hsym(m,k).eq.0 .or. .not. usehsym)then
                        call mmaxzz(psi(zetf(m,s)),hpsi,&
                             hteil(pmatgap(m,k,irec)),subdim(m),&
                             dim(m,s),dim(m,s1))
                     else if(hsym(m,k).eq.1)then
                        call mmaxzzh(psi(zetf(m,s)),hpsi,&
                             hteil(pmatgap(m,k,irec)),subdim(m),dim(m,s))
                     else if(hsym(m,k).eq.-1)then
                        call mmaxzzh1(psi(zetf(m,s)),hpsi,&
                             hteil(pmatgap(m,k,irec)),subdim(m),dim(m,s))
                     endif

                  enddo
               enddo
            enddo
!$omp enddo
            deallocate(hpsi)
!$omp end parallel
         endif
      endif

      call stop_timer(tid)

      return
      end subroutine phihphigap

!##########################################################################
! Subroutine to calculate the SPF GAP matrix elements. MPI version.
! GWR 02/17
!##########################################################################
#ifdef MPI
      subroutine mpiphihphigap(psi,hteil)

!$ use omp_lib
use mpi
use mpiutilsmod

      implicit none

      complex(dop), dimension(zetf(1,1):dgldim), intent(in) :: psi
      complex(dop), allocatable                             :: hpsi(:)
      complex(dop), dimension(pmatgapdim), intent(out)      :: hteil

      integer(long) :: k1,k2,k,m,s,s1,irec,start,end,idum
      complex(dop), dimension(pmatgapdim) :: tmphteil

!      integer(long), save                              :: tid = 0 ! timer ID
!      call get_timer(tid, 'phihphigap')
!      call start_timer(tid)

      k1=zham(1)
      k2=zham(1)+kzahl(1)-1

      idum = 1

      call mpe_decomp1d(1,dbnrec,start,end)
      call MPI_Bcast(psi(zetf(1,1)),totphidim,MPI_DOUBLE_COMPLEX,master,&
           MPI_COMM_WORLD,mpierr)
      call zerovxz(hteil,pmatgapdim)

!$omp parallel num_threads(ompthread) if(lompthread) private(m,k,s,s1,hpsi)
      if(allocated(hpsi))deallocate(hpsi)
      allocate(hpsi(hpsigapdim))
!$omp do
      do irec = start,end
         do k = k1,k2
            s=kf(k)
            s1=ki(k)
            
            do m = 1,nmode
               if(diag(m,k))cycle
               if(kinop(m,k).ne.0)cycle
               if(hsym(m,k).ne.1)then
                  routine='mpiphihphigap'
                  message='Wrong symmetry for GAP'
                  call errormsg
               endif
               call hphigap1mk(psi(zetf(m,s1)),hpsi,m,k,irec,idum,&
                    dim(m,s1),subdim(m))
               call mmaxzzh(psi(zetf(m,s)),hpsi,hteil(pmatgap(m,k,irec)),&
                    subdim(m),dim(m,s))

            enddo
         enddo
      enddo
!$omp enddo
      deallocate(hpsi)
!$omp end parallel

      call MPI_Reduce(hteil,tmphteil,pmatgapdim,MPI_DOUBLE_COMPLEX,MPI_SUM,&
           master,MPI_COMM_WORLD,mpierr)
      if(mpirank.eq.master) &
           call cpvxz(tmphteil,hteil,pmatgapdim)

!      call stop_timer(tid)

      return
      end subroutine mpiphihphigap
#endif

!#######################################################################
! Computes for all modes and states the uncorrelated matrix elements
! <phi|h_uncorr|phi>.
! Modified from phihunphi
! GWR 02/17
!#######################################################################

      subroutine phihunphigap (psi,hunpsi,hteil,nham)

!$ use omp_lib

      implicit none

      integer(long)                                      :: m,s,kz
      integer(long), intent(in)                          :: nham
      complex(dop), dimension(dgldim), intent(in)        :: psi
      complex(dop), dimension(adim+1:dgldim), intent(in) :: hunpsi
      complex(dop), dimension(pmatgapdim), intent(out)   :: hteil

      integer(long), save                                :: tid=0
      call get_timer(tid,'PhiHunPhiGAP')
      call start_timer(tid)

! --- LOOP OVER EACH MODE AND STATE ---

      if(lddmopt .or. lddgbtendec)then
         kz = ktotcog
      else
         kz=zham(nham)+kzahl(nham)-1
      endif

!$omp parallel do num_threads(ompthread) if(lompthread) &
!$omp private(m,s)
      do m = 1,nmode
         do s = 1,nstate
            if(khzahl(m,s,nham) .gt. 0 ) then
               call mmaxzz(psi(zetf(m,s)),hunpsi(zetf(m,s)),&
                    hteil(pmatgap(m,kz+s,1)),subdim(m),dim(m,s),dim(m,s))
            endif
         enddo
      enddo
!$omp end parallel do

      call stop_timer(tid)

      return
      end subroutine phihunphigap

!##########################################################################
! Routine to calculate the mean field matrices and add terms to dtpsi
! using GAP operators.
! Adapted from mfields in mfieldsmod.
! GWR 07/16
!##########################################################################
      subroutine mfieldsgap(psi,dtpsi,lsym)

!$ use omp_lib

      implicit none

      logical(kind=4), intent(in)                    :: lsym
      complex(dop), dimension(dgldim), intent(in)    :: psi
      complex(dop), dimension(dgldim), intent(inout) :: dtpsi

      complex(dop), allocatable                      :: workc(:)

      integer(long) :: k1,k2,k,s,s1,irec,kcorridx,idum

      integer(long), save :: tid = 0 ! timer ID
      call get_timer(tid, 'mfieldsgap')
      call start_timer(tid)

      k1=zham(1)
      k2=zham(1)+kzahl(1)-1

      idum = 1

      if(lddmopt .or. lddgbtendec)then
!$omp parallel num_threads(ompthread) if(lompthread) &
!$omp private(k,s,s1,workc,kcorridx,ogpar1)
         if(allocated(workc))deallocate(workc)
         allocate(workc(3*maxblock))
         ogpar1 => ogpar
!$omp do      
         do k = k1,k2

            if(kinop(1,k).ne.0)cycle
               
            s  = kf(k)
            s1 = ki(k)
            call ogparrew(ogpar1)
            do while(associated(ogpar1%next))
               if(ogpar1%k.eq.k)then
                  kcorridx = ogpar1%kcorridx
                  if(s1.ne.s)then
                     call matrizen2gap(k,kcorridx,idum,s,s1,psi,dtpsi,hteilgap,&
                          hlochgap,workc,psi)
                  else
                     call matrizen1gap(k,kcorridx,idum,s,psi,dtpsi,hteilgap,&
                          hlochgap,workc,psi,lsym)
                  endif
               endif
               ogpar1 => ogpar1%next
            enddo
         enddo
!$omp end do
         deallocate(workc)
         nullify(ogpar1)
!$omp end parallel
      else
!$omp parallel num_threads(ompthread) if(lompthread) &
!$omp private(k,s,s1,workc)
         if(allocated(workc))deallocate(workc)
         allocate(workc(3*maxblock))
!$omp do      
         do irec=1,dbnrec
            
            do k = k1,k2

               if(kinop(1,k).ne.0)cycle
               
               s  = kf(k)
               s1 = ki(k)

               if(s1.ne.s)then
                  call matrizen2gap(k,idum,irec,s,s1,psi,dtpsi,hteilgap,hlochgap,&
                       workc,psi)
               else
                  call matrizen1gap(k,idum,irec,s,psi,dtpsi,hteilgap,hlochgap,&
                       workc,psi,lsym)
               endif
            enddo
         enddo
!$omp end do
         deallocate(workc)
!$omp end parallel
      endif
      call stop_timer(tid)
      return
      end subroutine mfieldsgap
      
!##########################################################################
! Calculates mean field matrices for GAP terms involving only 1 state, s.
! Adapted from matrizen1 in mfieldsmod.
! GWR 07/16
!##########################################################################
      subroutine matrizen1gap(k,kcorridx,irec,s,psi,dtpsi,hteil,hloch,workc,&
           psi1,lsym)

!$use omp_lib
use op1lib, only: cpvxz 
use op2lib, only: swapsxxi,addvxxzo
use mtlib, only: qtxxzz
use xvlib, only: xqxxzzo,xvxxzza
use rmlib, only: rmmxxxzz,rmaxxxzz,rmhxxxzz 

      implicit none

      logical(kind=4), intent(in)                        :: lsym
      integer(long), intent(in)                          :: k,irec,s,kcorridx
      complex(dop), dimension(dgldim), intent(in)        :: psi,psi1
      complex(dop), dimension(adim), intent(inout)       :: dtpsi
      complex(dop), dimension(pmatgapdim), intent(in)    :: hteil
      complex(dop), dimension(hlochgapdim), intent(out)  :: hloch
      complex(dop), dimension(3*maxblock), intent(out)   :: workc

      logical(kind=4) :: work2eq3
      integer(long) :: zeig1,zeig2,zeig3,h,m,m1,kdx,irdx

! --- INITIALISE VARIABLES ---------------------------------------------
! workc is divided into three parts of length block.
! The result of a multiplication is stored after zeig2.
! Before multiplication, the vector to be multiplied is moved to after
! zeig1. After zeig3 is stored the vector multiplied up to mode m-1.
!-----------------------------------------------------------------------
      zeig1 = block(s)+1
      zeig2 = 1
      zeig3 = 2*block(s)+1
      call cpvxz(psi(zpsi(s)),workc(zeig2),block(s))
      work2eq3 = .false.

      h=hamilton(fkoe,k)

! Deal with different indexing used with OPTGAP and standard GAP.
      if(lddmopt .or. lddgbtendec)then
         kdx = kcorridx
         irdx = 1
      else
         kdx = k
         irdx = irec
      endif

!-----------------------------------------------------------------------
!  Loop over modes. 
!  First set initial pointers, then multiply wavefunction 
!  with single-particle hamiltonian matrices corresponding to 
!  dimensions numbers lower than mode 
!-----------------------------------------------------------------------
      do m=1,nmode
         if (m.ne.1) then
            if (.not. work2eq3) then
               call cpvxz(workc(zeig3),workc(zeig2),block(s))
               work2eq3 = .true.
            endif
            if (.not.diag(m-1,k)) then
               call swapsxxi(zeig1,zeig2)
               work2eq3 = .false.
               call qtxxzz(hteil(pmatgap(m-1,kdx,irdx)),workc(zeig1), &
                    workc(zeig2),vdim(m-1,s),dim(m-1,s),ndim(m-1,s))
            endif
         endif
         
!-----------------------------------------------------------------------
!  save wavefunction multiplied up to mode m-1, if it has changed.
!-----------------------------------------------------------------------
         if (.not. work2eq3) then
            call cpvxz(workc(zeig2),workc(zeig3),block(s))
            work2eq3 = .true.
         endif

!-----------------------------------------------------------------------
!  Multiply array with single-particle hamiltonian matrices for
!  modes higher than m.
!  If H is unitoperator for mode m, or basis is complete, do not bother 
!  as hloch is never needed.
!-----------------------------------------------------------------------

         if (.not. diag(m,k) .and. .not. complete(m)) then
            do m1=m+1,nmode
               if (.not.diag(m1,k)) then
                  call swapsxxi(zeig1,zeig2)
                  work2eq3 = .false.
                  call qtxxzz(hteil(pmatgap(m1,kdx,irdx)),workc(zeig1), &
                       workc(zeig2),vdim(m1,s),dim(m1,s),ndim(m1,s))
               endif
            enddo

!-----------------------------------------------------------------------
!  calculate single-hole matrix
!-----------------------------------------------------------------------
            if (hlsym(m,k) .eq. 0 .or. .not. lsym) then
               call rmmxxxzz(psi1(zpsi(s)),workc(zeig2), &
                    hloch(phlochgap(m,kdx,irdx)),vdim(m,s),dim(m,s),ndim(m,s), &
                    dim(m,s))
            else if (hlsym(m,k) .eq. 1) then
               call rmhxxxzz(psi1(zpsi(s)),workc(zeig2), &
                    hloch(phlochgap(m,kdx,irdx)),vdim(m,s),dim(m,s),ndim(m,s))
            else if (hlsym(m,k) .eq. -1) then
               call rmaxxxzz(psi1(zpsi(s)),workc(zeig2), &
                    hloch(phlochgap(m,kdx,irdx)),vdim(m,s),dim(m,s),ndim(m,s))
            endif

            if (htmtype(h) .ne. 1 .and. htmtype(h) .ne. 104)  &
               call xqxxzzo(koeff(k),hloch(phlochgap(m,kdx,irdx)),dim(m,s))

         endif
      enddo

!-----------------------------------------------------------------------
! Now calculate the time derivative of the coefficients.
! calculation of hloch for final mode has left workc(zeig2) with the 
! product of psi(block) and hteil matrices for all, excluding the final 
! mode. 
!-----------------------------------------------------------------------
      m=nmode
      if (.not.diag(m,k)) then
         call swapsxxi(zeig1,zeig2)
         work2eq3 = .false.
         call qtxxzz(hteil(pmatgap(m,kdx,irdx)),workc(zeig1),workc(zeig2), &
              vdim(m,s),dim(m,s),ndim(m,s))
      endif

!-----------------------------------------------------------------------
!  multiply by Hamiltonian coefficient and add to dtpsi
!-----------------------------------------------------------------------
!$omp critical
      if (htmtype(h) .eq. 1 .or. htmtype(h) .eq. 104) then
         call addvxxzo(workc(zeig2),dtpsi(zpsi(s)),block(s))
      else
         call xvxxzza(koeff(k),workc(zeig2),dtpsi(zpsi(s)),block(s))
      endif
!$omp end critical

      return
      end subroutine matrizen1gap

!##########################################################################
! Calculates mean field matrices for GAP terms involving 2 states, 
! s and s1.
! Adapted from matrizen2 in mfieldsmod.
! GWR 07/16
!##########################################################################
      subroutine matrizen2gap(k,kcorridx,irec,s,s1,psi,dtpsi,hteil,hloch,&
           workc,psi1)

!$use omp_lib
use op1lib, only: cpvxz,zerovxz
use op2lib, only: addvxxzo
use mtlib, only: mtxxzz
use xvlib, only: xmxxzzo,xvxxzza
use rmlib, only: rmmxxxzz 

      implicit none

      integer(long), intent(in)                          :: k,irec,s,s1,kcorridx
      complex(dop), dimension(adim), intent(in)          :: psi,psi1
      complex(dop), dimension(adim), intent(inout)       :: dtpsi
      complex(dop), dimension(pmatgapdim), intent(in)    :: hteil
      complex(dop), dimension(hlochgapdim), intent(out)  :: hloch
      complex(dop), dimension(3*maxblock), intent(out)   :: workc
      
      integer(long)                                      :: zeig1,zeig2,&
                                                            zeig3,swapzeig,&
                                                            m,m1,block1,&
                                                            vdim1,h,kdx,irdx

!---- INITIALISE VARIABLES ---------------------------------------------
! workc is divided into three parts of length maxblock, the longest
! possible
! length of the array during the multiplication.
! The result of a multiplication is stored after zeig2.
! Before multiplication, the vector to be multiplied is moved to after
! zeig1.
! After zeig3 is stored the vector multiplied up to mode m-1
!-----------------------------------------------------------------------
      call zerovxz(workc,3*MaxBlock)
      zeig1=maxblock+1
      zeig2=1
      zeig3=2*maxblock+1
      call cpvxz(psi(zpsi(s1)),workc(zeig2),block(s1))
      block1=block(s1)

      h=hamilton(fkoe,k)

! Deal with different indexing used with OPTGAP and standard GAP.
      if(lddmopt .or. lddgbtendec)then
         kdx = kcorridx
         irdx = 1
      else
         kdx = k
         irdx = irec
      endif

!-----------------------------------------------------------------------
!  First set initial pointers, then multiply wavefunction of state s1
!  with single-particle hamiltonian matrices corresponding to 
!  dimensions numbers lower than mode 
!  as the matrix elements are between different states with different
!   number of coefficients, the following pointers are needed:
!  block1: number of elements in multiplied array.
!  vdim1: equivalent of vdim during multiplication over the modes lower
!  than m.
!  vdim2: equivalent of vdim during multiplication over the modes higher
!  then m.
!  before a multiplication, block=vdim1*dim(m,s1)*ndim(m,s1)
!  after a multiplication, block=vdim1*dim(m,s)*ndim(m,s1)
!-----------------------------------------------------------------------
      do m=1,nmode
         if (m.ne.1) then
            call cpvxz(workc(zeig3),workc(zeig2),block1)

            swapzeig=zeig1
            zeig1=zeig2
            zeig2=swapzeig
            call mtxxzz(hteil(pmatgap(m-1,kdx,irdx)),workc(zeig1), &
                 workc(zeig2),vdim(m-1,s),dim(m-1,s1),ndim(m-1,s1), &
                 dim(m-1,s))
            block1=vdim(m-1,s)*dim(m-1,s)*ndim(m-1,s1)
         endif

!-----------------------------------------------------------------------
!  save wavefunction multiplied up to m-1, if it has changed.
!-----------------------------------------------------------------------
         call cpvxz(workc(zeig2),workc(zeig3),block1)

!-----------------------------------------------------------------------
!  Multiply array with single-particle hamiltonian matrices for
!  modes higher than m.
!  If H is unit-operator for mode m, or basis is complete, do not bother 
!  as hloch is never needed.
!-----------------------------------------------------------------------
         if (.not. complete(m)) then
            vdim1=vdim(m,s)*dim(m,s1)
            do m1=m+1,nmode
               swapzeig=zeig1
               zeig1=zeig2
               zeig2=swapzeig

               call mtxxzz(hteil(pmatgap(m1,kdx,irdx)),workc(zeig1), &
                    workc(zeig2),vdim1,dim(m1,s1),ndim(m1,s1),dim(m1,s))
               vdim1=vdim1*dim(m1,s)
            enddo
!-----------------------------------------------------------------------
!  calculate single-hole matrix
!-----------------------------------------------------------------------
            call rmmxxxzz(psi1(zpsi(s)),workc(zeig2),&
                 hloch(phlochgap(m,kdx,irdx)),vdim(m,s),dim(m,s),ndim(m,s),&
                 dim(m,s1))

            if (htmtype(h) .ne. 1 .and. htmtype(h) .ne. 104) &
               call xmxxzzo(koeff(k),hloch(phlochgap(m,kdx,irdx)),dim(m,s),&
                    dim(m,s1))
         endif
      enddo

!-----------------------------------------------------------------------
! now calculate the time derivative of the coefficients.
! calculation of hloch for final mode has left workc(zeig2) with the 
! product of psi(block) and hteil matrices for all, excluding the final 
! mode. 
!-----------------------------------------------------------------------
      m=nmode
      swapzeig=zeig1
      zeig1=zeig2
      zeig2=swapzeig
      call mtxxzz(hteil(pmatgap(m,kdx,irdx)),workc(zeig1),workc(zeig2),&
           vdim(m,s),dim(m,s1),ndim(m,s),dim(m,s))
!-----------------------------------------------------------------------
!  multiply by Hamiltonian coefficient
!-----------------------------------------------------------------------
!$omp critical
      if (htmtype(h) .eq. 1 .or. htmtype(h) .eq. 104) then
         call addvxxzo(workc(zeig2),dtpsi(zpsi(s)),block(s))
      else
         call xvxxzza(koeff(k),workc(zeig2),dtpsi(zpsi(s)),block(s))
      endif
!$omp end critical

      return
      end subroutine matrizen2gap

!##########################################################################
! Routine to multiply the hloch and hpsi arrays for mode, m, and add the 
! result to the SPF part of dtpsi.
! Modified from hlochphi1m in hlochphimod.
! GWR 07/16
!##########################################################################
      subroutine hlochphigap(psi,dtpsi,hloch,ktype)

!$ use omp_lib

use op1lib,  only: zerovxl
use mmlib,   only: mmxtzza
#ifdef MPI
use mpi
#endif

      implicit none

      integer(long), intent(in)                                   :: ktype
      complex(dop), dimension(dgldim), intent(in)                 :: psi
      complex(dop), dimension(hlochgapdim), intent(in)            :: hloch
      complex(dop), allocatable                                   :: hpsi(:)
      complex(dop), dimension(dgldim), intent(inout)              :: dtpsi

      integer(long) :: k,k1,k2,s,s1,irec,m,idum,kidx,kcorridx
      logical(kind=4), dimension(maxdim,maxkoe)                   :: kcalc

      integer(long), save                            :: tid = 0 ! timer ID
      call get_timer(tid, 'hlochphigap')
      call start_timer(tid)

      idum = 1
      routine='Hlochphigap'

! --- IF BASIS IS COMPLETE DO NOT CALCULATE ---

      if(lmpi.and.lmpistarted)then
#ifdef MPI
         if(lddmopt .or. lddgbtendec)then
            message='MPI not implemented for OPT/SVD GAP'
            call errormsg
         endif
         mpitask = 52
         call MPI_Bcast(mpitask,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
         call mpihlochphigap(psi(zetf(1,1)),dtpsi(zetf(1,1)),hloch,ktype)
#else
         message='Wants MPI code but MPI not compiled'
         call errormsg
#endif
      else
         kcalc=.false.
         k1=zham(1)
         k2=zham(1)+kzahl(1)-1

         do m = 1,nmode
            if(complete(m))cycle
            if (ktype .eq. 0) then
               do k=k1,k2
                  if (.not. diag(m,k)) kcalc(m,k)=.true.
                  if (autocap(k) .gt. 0) kcalc(m,k)=.false.
               enddo
            else if (ktype .eq. 1) then
               do k=k1,k2
                  if (kinop(m,k) == 1) kcalc(m,k)=.true.
                  if (autocap(k) > 0) kcalc(m,k)=.false.
               enddo
            endif
         enddo

! --- COMPUTE MEAN-FIELD TIMES H|PHI> FOR ALL STATES AND TERMS ---
         if(lddmopt .or. lddgbtendec)then
!$omp parallel num_threads(ompthread) if(lompthread) private(k,s,s1,hpsi,m,kidx,kcorridx,ogpar1) 
            if(allocated(hpsi))deallocate(hpsi)
            allocate(hpsi(hpsigapdim))
            ogpar1 => ogpar
!$omp do reduction(+:dtpsi)
            do k = k1,k2
               do m = 1,nmode
                  if (.not.kcalc(m,k))cycle
                  s=kf(k)
                  s1=ki(k)
                  call ogparrew(ogpar1)
                  do while(associated(ogpar1%next))
                     if(k.eq.ogpar1%k)then
                        kidx = ogpar1%kidx
                        kcorridx = ogpar1%kcorridx
                        call hphigap1mk(psi(zetf(m,s1)),hpsi,m,k,idum,kidx,&
                             dim(m,s1),subdim(m))
                        call mmxtzza(hpsi,hloch(phlochgap(m,kcorridx,1)), &
                             dtpsi(zetf(m,s)),subdim(m),dim(m,s1),dim(m,s))
                     endif
                     ogpar1 => ogpar1%next
                  enddo
               enddo
            enddo
!$omp end do
            deallocate(hpsi)
            nullify(ogpar1)
!$omp end parallel
         else
!$omp parallel num_threads(ompthread) if(lompthread) private(k,s,s1,hpsi,m) 
            if(allocated(hpsi))deallocate(hpsi)
            allocate(hpsi(hpsigapdim))
!$omp do reduction(+:dtpsi)
            do irec = 1,dbnrec
               do k = k1,k2
                  do m = 1,nmode

                     if (.not.kcalc(m,k))cycle
                  
                     s=kf(k)
                     s1=ki(k)
                     call hphigap1mk(psi(zetf(m,s1)),hpsi,m,k,irec,idum,&
                          dim(m,s1),subdim(m))
                     call mmxtzza(hpsi,hloch(phlochgap(m,k,irec)), &
                          dtpsi(zetf(m,s)),subdim(m),dim(m,s1),dim(m,s))
                  enddo
               enddo
            enddo
!$omp end do
            deallocate(hpsi)
!$omp end parallel
         endif
      endif
! test
!      stop
! test

      call stop_timer(tid)
      return
      end subroutine hlochphigap

!##########################################################################
! Routine to multiply the hloch and hpsi arrays for mode, m, and add the 
! result to the SPF part of dtpsi.
! Modified from hlochphi1m in hlochphimod.
! MPI version.
! GWR 02/17
!##########################################################################
#ifdef MPI
      subroutine mpihlochphigap(psi,dtpsi,hloch,ktype)

!$ use omp_lib

use op1lib,  only: zerovxl
use mmlib,   only: mmxtzza
use mpi
use mpiutilsmod

!+----------------------------------------------------------------------
!     write database for direct dynamics                               
!+----------------------------------------------------------------------
      implicit none                                                     

      integer(long), intent(in)                                   :: ktype
      complex(dop), dimension(zetf(1,1):dgldim), intent(in)       :: psi
      complex(dop), dimension(hlochgapdim), intent(in)            :: hloch
      complex(dop), dimension(zetf(1,1):dgldim), intent(inout)    :: dtpsi

      integer(long) :: k,s,s1,irec,m,start,end,k1,k2,idum
      logical(kind=4), dimension(maxdim,maxkoe)                   :: kcalc
      complex(dop), allocatable                                   :: hpsi(:)
      complex(dop), dimension(zetf(1,1):dgldim)                   :: tmpdtpsi

!      integer(long), save                            :: tid = 0 ! timer ID
!      call get_timer(tid, 'mpihlochphigap')
!      call start_timer(tid)

      kcalc=.false.
      k1=zham(1)
      k2=zham(1)+kzahl(1)-1

      idum = 1

      do m = 1,nmode
         if(complete(m))cycle
         if (ktype .eq. 0) then
            do k=k1,k2
               if (.not. diag(m,k)) kcalc(m,k)=.true.
               if (autocap(k) .gt. 0) kcalc(m,k)=.false.
            enddo
         else if (ktype .eq. 1) then
            do k=k1,k2
               if (kinop(m,k) == 1) kcalc(m,k)=.true.
               if (autocap(k) > 0) kcalc(m,k)=.false.
            enddo
         endif
      enddo

      call mpe_decomp1d(1,dbnrec,start,end)
!      call MPI_Bcast(psi(zetf(1,1)),totphidim,MPI_DOUBLE_COMPLEX,master,&
!           MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(ktype,1,MPI_INTEGER,master,MPI_COMM_WORLD,mpierr)
      call MPI_Bcast(hloch,hlochgapdim,MPI_DOUBLE_COMPLEX,master,MPI_COMM_WORLD,mpierr)

! debug
!      if(mpirank.eq.0)write(*,*)'hello',mpirank,size(zhopgap)
!      call MPI_BARRIER(MPI_COMM_WORLD,mpierr)
!      if(mpirank.eq.1)write(*,*)'hello',mpirank,size(zhopgap)
!      call MPI_BARRIER(MPI_COMM_WORLD,mpierr)
!      stop
! debug

      if (mpirank.ne.master) call zerovxz(dtpsi(zetf(1,1)),totphidim)

! --- COMPUTE MEAN-FIELD TIMES H|PHI> FOR ALL STATES AND TERMS ---
!$omp parallel num_threads(ompthread) if(lompthread) private(k,s,s1,hpsi,m) 
      if(allocated(hpsi))deallocate(hpsi)
      allocate(hpsi(hpsigapdim))
!$omp do reduction(+:dtpsi)
      do irec = start,end
         do k = k1,k2
            do m = 1,nmode
               
               if (.not.kcalc(m,k))cycle
               
               s=kf(k)
               s1=ki(k)
               call hphigap1mk(psi(zetf(m,s1)),hpsi,m,k,irec,idum,&
                    dim(m,s1),subdim(m))
               call mmxtzza(hpsi,hloch(phlochgap(m,k,irec)), &
                    dtpsi(zetf(m,s)),subdim(m),dim(m,s1),dim(m,s))
            enddo
         enddo
      enddo
!$omp end do
      deallocate(hpsi)
!$omp end parallel

      call MPI_Reduce(dtpsi(zetf(1,1)),tmpdtpsi(zetf(1,1)),totphidim, &
                   MPI_DOUBLE_COMPLEX,MPI_SUM,master,MPI_COMM_WORLD,mpierr)
      if(mpirank.eq.master) &
           call cpvxz(tmpdtpsi(zetf(1,1)),dtpsi(zetf(1,1)),totphidim)

!      call stop_timer(tid)
      return

      end subroutine mpihlochphigap
#endif
!##########################################################################
! Routine to multiply the hloch and hpsi arrays for mode, m, and add the 
! result to the SPF part of dtpsi.
! Modified from hlochphi1m in hlochphimod.
! GWR 07/16
!##########################################################################
      subroutine hlochphi1mgap(psi,dtpsi,hloch,m,zetf1,phidim1,ktype)

!$ use omp_lib

use op1lib,  only: zerovxl
use mmlib,   only: mmxtzza

      implicit none                                                     
                                                                        
      integer(long), intent(in)                                   :: m,zetf1, &
                                                                     phidim1, &
                                                                     ktype
      complex(dop), dimension(zetf1:zetf1+phidim1-1), intent(in)  :: psi
      complex(dop), dimension(hlochgapdim), intent(in)            :: hloch
! test mem
!      complex(dop), dimension(hpsigapdim), intent(out)            :: hpsi
      complex(dop), allocatable                                   :: hpsi(:)
! test mem
!      complex(dop), dimension(hpsigapdim), intent(in)             :: hpsi
! test mem
      complex(dop), dimension(zetf1:zetf1+phidim1-1), intent(out) :: dtpsi

      integer(long) :: k,k1,k2,s,s1,irec,idum
      logical(kind=4), dimension(maxkoe)                          :: kcalc

! --- IF BASIS IS COMPLETE DO NOT CALCULATE ---

      if (complete(m)) return

      idum = 1

      call zerovxl(kcalc,ktot)
      k1=zham(1)
      k2=zham(1)+kzahl(1)-1

      if (ktype .eq. 0) then
         do k=k1,k2
            if (.not. diag(m,k)) kcalc(k)=.true.
            if (autocap(k) .gt. 0) kcalc(k)=.false.
         enddo
      else if (ktype .eq. 1) then
         do k=k1,k2
            if (kinop(m,k) == 1) kcalc(k)=.true.
            if (autocap(k) > 0) kcalc(k)=.false.
         enddo
      endif

! --- COMPUTE MEAN-FIELD TIMES H|PHI> FOR ALL STATES AND TERMS ---
!$omp parallel num_threads(ompthread) if(lompthread) private(k,s,s1,hpsi) 
      if(allocated(hpsi))deallocate(hpsi)
      allocate(hpsi(hpsigapdim))
!$omp do reduction(+:dtpsi)
      do irec = 1,dbnrec
         do k = k1,k2
            if (kcalc(k)) then
               s=kf(k)
               s1=ki(k)
! test mem
               call hphigap1mk(psi(zetf(m,s1)),hpsi,m,k,irec,idum,&
                    dim(m,s1),subdim(m))
               call mmxtzza(hpsi,hloch(phlochgap(m,k,irec)), &
                    dtpsi(zetf(m,s)),subdim(m),dim(m,s1),dim(m,s))
! test mem
!               call mmxtzza(hpsi(zhpsigap(m,k,irec)),&
!                    hloch(phlochgap(m,k,irec)), &
!                    dtpsi(zetf(m,s)),subdim(m),dim(m,s1),dim(m,s))
! test mem
            endif
         enddo
      enddo
!$omp end do
      deallocate(hpsi)
!$omp end parallel

      return
      end subroutine hlochphi1mgap

!##########################################################################
! Sums up for all modes and all states the product of the mean-field
! matrices times the single-particle potential operators.
! Based on summf.
! GWR 08/16
!##########################################################################
      subroutine summfgap(mfsum,hloch,nham)

use lalib,   only: mktens

      implicit none

      integer(long), intent(in)     :: nham
      complex(dop), dimension(mfsumdim), intent(out)   :: mfsum
      complex(dop), dimension(hlochgapdim), intent(in) :: hloch

      integer(long) :: k1,k2,k,irec,m,zeig,zeig1,s,s1,idum
      complex(dop), dimension(3*sdim)                  :: workc

      integer(long), save                            :: tid=0 ! timer ID
      call get_timer(tid, 'Summfgap')
      call start_timer(tid)

      k1=zham(nham)
      k2=zham(nham)+kzahl(nham)-1

      idum = 1

      call zerovxz(mfsum,mfsumdim)

      do m = 1,nmode
         if (gwpm(m)) cycle
         if(idmode(m).ne.0) cycle
         do k = k1,k2
            if (kinop(m,k).ne.0 .or. diag(m,k)) cycle
            do irec = 1,dbnrec
               zeig=1
               zeig1=zeig+subdim(m)
               s1=ki(k)
               s=kf(k)
               call initvxz(workc(zeig1),subdim(m))
               call hcallgap(m,k,irec,idum,workc(zeig),workc(zeig1),subdim(m))
               call mktens(mfsum(zmfsum1(m,s1,s,1)), &
                    workc(zeig),hloch(phlochgap(m,k,irec)),subdim(m), &
                    dim(m,s),dim(m,s1),true)             
            enddo
         enddo
      enddo

      call stop_timer(tid)

      return
      end subroutine summfgap

!##########################################################################
! Calcha routine for GAP. 
! GWR 08/16
!##########################################################################
      subroutine calchagap(psi,dtpsi,hteil,nham)

use op2lib, only: addvxxzo
use xvlib, only: xvxxzza

      implicit none

      integer(long), intent(in)                          :: nham
      complex(dop), dimension(adim), intent(in)          :: psi
      complex(dop), dimension(adim), intent(inout)       :: dtpsi
      complex(dop), dimension(pmatgapdim), intent(inout) :: hteil

      integer(long) :: k,s,s1,k1,k2,h,irec
      complex(dop), dimension(maxblock)                  :: tmpdtpsi

      k1=zham(nham)
      k2=zham(nham)+kzahl(nham)-1

      do k = k1,k2
         s = kf(k)
         s1= ki(k)

         h=hamilton(fkoe,k)

         do irec = 1,dbnrec
            if (s.eq.s1) then
               call calchagap1k(psi,hteil,tmpdtpsi,k,s,irec)
            else
               call calchagap2k(psi,hteil,tmpdtpsi,k,s,s1,irec)
            endif

!    --- STORE RIGHT HAND SIDE IN DTPSI ---

            if (htmtype(h) .eq. 1) then
               call addvxxzo(tmpdtpsi,dtpsi(zpsi(s)),block(s))
            else
               call xvxxzza(koeff(k),tmpdtpsi,dtpsi(zpsi(s)),block(s))
            endif
         enddo
      enddo

      return
      end subroutine calchagap

!############################################################################
! Calculates the action of the correlated Hamiltonian on the
! MCTDH-coefficients for a single intra-state term, k.
! Result is returned in dtpsi
! Based on calcha1k
!############################################################################
      subroutine calchagap1k (psi,hteil,dtpsi,k,s,irec)

use mtlib, only: qtxxzz

      implicit none

      integer(long)                                      :: zeig1,zeig2,&
                                                            zeig3,m
      integer(long), intent(in)                          :: k,s,irec
      complex(dop), dimension(adim), intent(in)          :: psi
      complex(dop), dimension(maxblock), intent(out)     :: dtpsi
      complex(dop), dimension(pmatgapdim), intent(in)    :: hteil

      complex(dop), dimension(2*maxblock)                :: workc

! --- COPY PSI VECTOR AND SET UP POINTERS ---

      zeig2 = maxblock+1
      zeig1 = 1
      call cpvxz(psi(zpsi(s)),workc(zeig2),block(s))

! --- LOOP OVER EACH MODE ---
      do m = 1,nmode
         if (.not. diag(m,k)) then

!       --- SWAP POINTERS OR SELECT COEFFICIENTS ---

            zeig3 = zeig1
            zeig1 = zeig2
            zeig2 = zeig3

!       --- MULTIPLY WITH MATRIX ELEMENTS ---

!            if(.not.lompthread)then
               call qtxxzz(hteil(pmatgap(m,k,irec)),workc(zeig1), &
                    workc(zeig2),vdim(m,s),dim(m,s),ndim(m,s))
!            else
!               call qtxxzzomp(hteil(pmatgap(m,k,irec)),workc(zeig1), &
!                    workc(zeig2),vdim(m,s),dim(m,s),ndim(m,s),ompthread)
!            endif
         endif
      enddo

! --- MOVE RESULT TO OUTPUT DTPSI ARRAY ---

      call cpvxz(workc(zeig2),dtpsi,block(s))
     
      return
      end subroutine calchagap1k


!##################################################################################
! Calculates the action of the correlated Hamiltonian on the
! MCTDH-coefficients for a single inter-state term, k.
! Result is returned in dtpsi
! Based on calcha2k
!##################################################################################
      subroutine calchagap2k(psi,hteil,dtpsi,k,s,s1,irec)

use mtlib, only: mtxxzz

      implicit none

      integer(long)                                       :: zeig1,zeig2,zeig3,m,&
                                                             vdim1
      integer(long), intent(in)                           :: k,s,s1,irec
      complex(dop), dimension(adim), intent(in)           :: psi
      complex(dop), dimension(maxblock), intent(out)      :: dtpsi
      complex(dop), dimension(pmatgapdim), intent(in)     :: hteil

      complex(dop), dimension(2*maxblock)                 :: workc

! --- LOOP OVER EACH CORRELATED HAMILTONIAN TERM ---

      zeig2 = maxblock+1
      zeig1 = 1

! --- INITIALIZE WORK ARRAY WITH PSI ARRAY ---

      call cpvxz(psi(zpsi(s1)),workc(zeig2),block(s1))

      vdim1 = vdim(1,s1)
      do m = 1,nmode

!       --- SWAP POINTERS ---

         zeig3 = zeig1
         zeig1 = zeig2
         zeig2 = zeig3
         
!       --- MULTIPLY WITH MATRIX ELEMENTS ---

         call zerovxz(workc(zeig2),maxblock)
!         if(.not.lompthread)then
            call mtxxzz(hteil(pmatgap(m,k,irec)),workc(zeig1), &
                 workc(zeig2),vdim1,dim(m,s1),ndim(m,s1),dim(m,s))
!         else
!            call mtxxzzomp(hteil(pmat(m,k,irec)),workc(zeig1), &
!                 workc(zeig2),vdim1,dim(m,s1),ndim(m,s1),dim(m,s),ompthread)
!         endif
         vdim1 = vdim1*dim(m,s)
      enddo

! --- MOVE RESULT TO OUTPUT DTPSI ARRAY ---

      call cpvxz(workc(zeig2),dtpsi,block(s))

      return
      end subroutine calchagap2k

! **********************************************************************
! Subroutine to Calculate the action of the uncorrelated Hamiltonian on 
! the MCTDH-coefficients.     
! Modified from calchuna
!
! GWR 02/17
! **********************************************************************
      subroutine calchunagap (psi,dtpsi,hteil,nham)

use mtlib, only: qtxxzza

      implicit none

      integer(long)                                      :: m,s,k
      integer(long), intent(in)                          :: nham
      complex(dop), dimension(adim), intent(in)          :: psi
      complex(dop), dimension(adim), intent(inout)       :: dtpsi
      complex(dop), dimension(pmatgapdim), intent(in)    :: hteil

! --- LOOP OVER EACH UNCORRELATED HAMILTONIAN TERM ---

      if(lddmopt .or. lddgbtendec)then
         k= ktotcog
      else
         k=zham(nham)+kzahl(nham)-1
      endif

!$omp parallel do num_threads(ompthread) private(s,m) 
      do s=1,nstate
         do m=1,nmode
            if (khzahl(m,s,nham) .gt. 0) then
               call qtxxzza(hteil(pmatgap(m,k+s,1)),psi(zpsi(s)), &
                    dtpsi(zpsi(s)),vdim(m,s),dim(m,s),ndim(m,s))
            endif
         enddo
      enddo
!$omp end parallel do

      return
      end subroutine calchunagap

!##########################################################################
! Driver routine for the test potentials.
! GWR 07/16
!##########################################################################
      subroutine testdrv(xgpoint,gpoint,v,itest)

      implicit none

      integer(long), intent(in) :: itest
      real(dop), dimension(ndofddpes), intent(in) :: gpoint,xgpoint
      real(dop), intent(out) :: v(nddstate,nddstate)

      integer(long) :: m,f,n,n1,idx
!      logical(kind=4), dimension(ndofddpes) :: lactive
      real(dop), dimension(ndofddpes)       :: testq
      real(dop), allocatable, dimension(:)  :: g,h
      integer(long), dimension(42) :: saldidx
      data saldidx /1,3*0,2,0,3,0,4,5,6,0,7,0,0,8,5*0,9,10,11,7*0,12,3*0,13,6*0/

!      lactive = .false.
      testq = 0.0_dop
      if(itest.eq.1)then
         do m = 1,nmode
            n = nspfdof(m)
            do n1 = 1,n
               f = spfdof(n1,m)
               idx = saldidx(f)
               if(idx.eq.0)then
                  routine='gapdrv'
                  message='DOF used which is not in fitted potential'
                  call errormsg
               else
                  testq(idx) = gpoint(f)
               endif
!               if(f.eq.1) then
!                  lactive(1)=.true.
!                  testq(1) = gpoint(f)
!               endif
!               if(f.eq.18)then
!                  lactive(2)=.true.
!                  testq(2) = gpoint(f)
!               endif
!               if(f.eq.36)then
!                  lactive(3)=.true.
!                  testq(3) = gpoint(f)
!               endif
            enddo
         enddo
!         v(1) = saldpot(testq,lactive) 
         v(1,1) = saldpot13(testq)
      else if(itest.eq.2) then
         allocate(g(ndofddpes),h(ndofddpes*(ndofddpes+1)/2))
         call malonpes(xgpoint,0,v(1,1),g,h)
         deallocate(g,h)
      else if(itest.eq.5)then
         call butpes(gpoint,v)
      else if(itest.eq.6)then
         call formpot(xgpoint,v(1,1))
         if(v(1,1).gt.gapethresh .or. v(1,1).lt.-5e-6_dop) &
              v(1,1)=gapethresh
      else
         call dddb_rd_db0(v,xgpoint)
      endif

      return
      end subroutine testdrv

!##########################################################################
! Subroutine to test DD-GB. It is the relevant part of the linear 5-mode
! model for the butatriene cation.
! GWR 02/18
!##########################################################################
      subroutine butpes(gpoint,v)

      implicit none 

      real(dop), dimension(ndofddpes), intent(in) :: gpoint
      real(dop), intent(out) :: v(nddstate,nddstate)

      real(dop) :: tmpv(nddstate,nddstate)

      tmpv = 0.0_dop

      tmpv(1,1) = 0.5_dop*(0.0912_dop*gpoint(5)*gpoint(5) &
           + 0.25780_dop*gpoint(14)*gpoint(14))
      tmpv(2,2) = tmpv(1,1)

      tmpv(1,1) = tmpv(1,1) + 8.9014_dop
      tmpv(2,2) = tmpv(2,2) + 9.4455_dop

      tmpv(1,1) = tmpv(1,1) + 0.16280_dop*gpoint(14)
      tmpv(2,2) = tmpv(2,2) - 0.34220_dop*gpoint(14)

      tmpv(1,2) = 0.288_dop*gpoint(5)
      tmpv(2,1) = tmpv(1,2)

      v = tmpv/ev

      return
      end subroutine butpes

!##########################################################################
! Function used to test the DD-MCTDH program.
! It is the relevant parts of the vibronic coupling potential for
! salicylaldimine (modes 1, 18 and 36)
! GWR 07/16
!##########################################################################
      double precision function saldpot(testq,lactive)

      implicit none

      logical(kind=4), dimension(3), intent(in) :: lactive
      real(dop), dimension(3), intent(in)       :: testq

      logical(kind=4) :: xact,yact,zact
      real(dop)       :: x,y,z

      real(dop) :: omega1,omega18,omega36,kappa1_1,kappa1_36,gamma1_1_1,&
           gamma1_1_18,gamma1_1_36,gamma1_36_36,explicit_1,explicit_1_18_1,&
           explicit_1_18_2,explicit_1_18_3,EPm1,D_18,A_18,X_18,E_18
      real(dop) :: v1m18
      real(dop) :: xpart,ypart,zpart,xypart,xzpart

      x = testq(1)
      y = testq(2)
      z = testq(3)
      xact = lactive(1) 
      yact = lactive(2) 
      zact = lactive(3) 

      omega1          =  6.9118093e-3_dop
      omega18         =  4.9769584e-3_dop
      omega36         =  9.9087151e-3_dop
      kappa1_1        = -5.93868768e-4_dop
      kappa1_36       = -1.05470505e-3_dop
      gamma1_1_1      = -1.40713089e-2_dop
      gamma1_1_18     = -1.61696942e-4_dop
      gamma1_1_36     = -2.27845691e-4_dop
      gamma1_36_36    = -6.28413115e-4_dop
      explicit_1      = -3.40374267416e-4_dop
      explicit_1_18_1 =  5.9996915263e-6_dop
      explicit_1_18_2 =  1.3246580964e-4_dop
      explicit_1_18_3 = -1.27409840266e-6_dop
      EPm1            =  1.788992914e-2_dop
      D_18            =  9.1873262e-1_dop
      A_18            =  5.46684e-2_dop
      X_18            = -5.0e-6_dop
      E_18            =  0.0_dop

      if(yact)then
         v1m18 = exp(-1.0_dop*A_18*(y-X_18))
         v1m18 = v1m18-1.0_dop
         v1m18 = v1m18**2
         v1m18 = D_18*v1m18
      endif

      xpart  = 0.0_dop
      ypart  = 0.0_dop
      zpart  = 0.0_dop
      xypart = 0.0_dop
      xzpart = 0.0_dop    

      if(xact)then
         xpart = 0.5_dop*(omega1 + gamma1_1_1)*x*x
         xpart = xpart + kappa1_1*x
         xpart = xpart + explicit_1*x*x*x
         xpart = xpart + 0.04116_dop*EPm1*x*x*x*x
      endif

      if(yact)then
         ypart = 0.5_dop*omega18*y*y
         ypart = ypart + v1m18
      endif

      if(zact)then
         zpart = 0.5_dop*(omega36 + gamma1_36_36)*z*z
         zpart = zpart + kappa1_36*z
      endif

      if(xact .and. yact)then
         xypart = gamma1_1_18*x*y
         xypart = xypart + explicit_1_18_1*x*x*y*y
         xypart = xypart + explicit_1_18_2*x*x*x*y
         xypart = xypart + explicit_1_18_3*x*y*y*y
      endif

      if(xact .and. zact)then
         xzpart = 0.5_dop*gamma1_1_36*x*z
      endif

      saldpot = xpart+ypart+zpart+xypart+xzpart

      end function saldpot

!##########################################################################
! Function used to test the DD-MCTDH program.
! It is the relevant parts of the vibronic coupling potential for
! salicylaldimine (modes 1,5,7,9,10,11,13,16,22,23,24,32,36).
! Uses potential from Polyak et al, JCP, 2015, (143), 084121.
! Potential corrected from that used in the paper as the quartic terms 
! were multiplieded by 0.04116 rather than 0.04167 (1/24) due to a bug
! in VCHAM. Does not match correct factorials from Taylor series, again
! due to vagaries in VCHAM. 
! GWR 08/16
!##########################################################################
      double precision function saldpot13(testq)

      implicit none

      real(dop), dimension(13), intent(in) :: testq

      integer(long) :: i,j

      real(dop), parameter :: evau = 1.0_dop/27.211399_dop
      real(dop) :: ompart,kapart,gapart,iopart,eppart,etpart,eta_32_1,gaoffpart
      real(dop), dimension(13) :: omega,kappa,gammad,gamma1,gamma10,gamma11,&
                                  gamma13,gamma32,iotad,epsd,eps1
      real(dop), dimension(13,13) :: iotaoff

      data omega /0.18808,0.05254,0.06644,0.07815,0.07928,0.09815,0.10501,0.12166,&
                  0.15213,0.15954,0.16244,0.20607,0.26963 /

      data kappa /-0.010707,0.00008446,0.00019188,0.00029048,-0.00009656,&
                  -0.00008476,0.00009051,-0.000005,-0.00003752,-0.00027668,&
                   0.00006096,0.00008376,-0.0079318/

      data gammad /-0.38006,-0.00005211,-0.00001328,-0.000041680,-0.00003527,&
                   -0.00004026,-0.00002384,-0.00006204,0.000005,0.00018145, &
                    0.00034101,-0.00007011,-0.00728990/

      data gamma1 /0.0000,0.01020,-0.0034,0.0008,0.0009,-0.0113,0.0022,0.0058,&
                  -0.0088,0.01060,0.0094,0.0036, 0.0206/

      data gamma10 /5*0.0000,-0.0004,0.0004,4*0.0000,0.0003,-0.0012/

      data gamma11 /6*0.0000,0.0004,4*0.0000,0.0003,-0.0012/

      data gamma13 /11*0.0000,-0.0003,0.0012/

      data gamma32 /12*0.0000, 0.0009/

      data iotad / -0.0654,0.0012,-0.0006,0.0012,0.0018,0.0073,0.0031,&
                   -0.0005,-0.0030,0.0114,-0.0044,-0.0060,0.0000/

      data iotaoff / 0.0000,0.0018,-0.0002,0.0005,0.0003,0.0020,0.0016, &
                    -0.0005,0.0005,0.0025,0.0025,-0.0047,-0.0228, &
                    -0.0175,12*0.0000, &
                    -0.0109,12*0.0000, &
                    -0.0164,12*0.0000, &
                     0.0407,4*0.0000,0.0011,0.0011,4*0.0000,0.0006,-0.0006, &
                     0.0445,3*0.0000,0.0011, 0.0000,0.0011,4*0.0000,0.0006,-0.0006,&
                    -0.0450,3*0.0000,-0.0010,-0.0010,5*0.0000,-0.0006,0.0006, &
                    -0.0169,12*0.0000, &
                     0.0183,12*0.0000, &
                    -0.0050,12*0.0000, &
                     0.0074,12*0.0000, &
                     0.0215,3*0.0000, -0.0008, -0.0008, -0.0008, 5*0.0000, 0.0004, &
                    -0.0662,3*0.0000,0.0032,0.0032,0.0032,4*0.0000,0.0018,0.0000/

      data epsd / 0.48662,0.001183,0.00047001,0.00018046,0.000320840,0.00074117,0.0013485, &
                  0.00020035,0.0028139,0.006542,0.0089239,0.002778,0.000005/

      data eps1 / 0.0000,-0.02153664,0.00045576,3*0.0000,0.0446784,-0.00408216, &
                 -0.0110424,-0.0299568,-0.0307416,0.1723224,0.0240/

      eta_32_1 = -0.25764

      ompart = 0.0_dop
      kapart = 0.0_dop
      gapart = 0.0_dop
      gaoffpart = 0.0_dop
      iopart = 0.0_dop
      eppart = 0.0_dop
      etpart = 0.0_dop

      do i = 1, 13
         ompart = ompart + omega(i)*testq(i)*testq(i)
      enddo
      ompart = 0.5_dop*ompart

      do i = 1, 13
         kapart = kapart + kappa(i)*testq(i)
      enddo
      
      do i = 1, 13
         eppart = eppart + epsd(i)*testq(i)**4!/12.0_dop
         eppart = eppart + eps1(i)*testq(1)**2 * testq(i)**2
      enddo
      eppart = eppart/24.0_dop

      do i = 1, 13
         gapart = gapart + gammad(i)*testq(i)*testq(i)
      enddo
      gapart = 0.5_dop*gapart

      do i = 1, 13
         gaoffpart = gaoffpart + gamma1(i)*testq(1)*testq(i)
      enddo

      do i = 1, 13
         gaoffpart = gaoffpart + gamma10(i)*testq(5)*testq(i)
      enddo

      do i = 1, 13
         gaoffpart = gaoffpart + gamma11(i)*testq(6)*testq(i)
      enddo

      do i = 1, 13
         gaoffpart = gaoffpart + gamma13(i)*testq(7)*testq(i)
      enddo

      do i = 1, 13
         gaoffpart = gaoffpart + gamma32(i)*testq(12)*testq(i)
      enddo

      do i = 1, 13
         iopart = iopart + iotad(i)*testq(i)**3
      enddo
      iopart = iopart/6.0_dop

      do i = 1, 13
         do j = 1, 13
            iopart = iopart + 0.5_dop*iotaoff(i,j)*testq(i)*testq(i)*testq(j)
         enddo
      enddo

      etpart = eta_32_1 * testq(12)*testq(1)**3/24.0_dop

      saldpot13 = evau*(ompart + kapart + gapart &
           + gaoffpart + iopart + eppart + etpart)     

      end function saldpot13

!##########################################################################
! Random seed routine.
! From:
! https://gcc.gnu.org/onlinedocs/gfortran/RANDOM_005fSEED.html#RANDOM_005fSEED
!########################################################################## 
      subroutine init_random_seed()
      implicit none
      integer(long), allocatable :: seed(:)
      integer(long) :: i, n, dt(8)
      integer(vlong) :: t
        
      call random_seed(size = n)
      allocate(seed(n))
        ! First try if the OS provides a random number generator
!        open(newunit=un, file="/dev/urandom", access="stream", &
!             form="unformatted", action="read", status="old", iostat=istat)
!        if (istat == 0) then
!           read(un) seed
!           close(un)
!        else
           ! Fallback to XOR:ing the current time and pid. The PID is
           ! useful in case one launches multiple instances of the same
           ! program in parallel.
      call system_clock(t)
      if (t == 0) then
         call date_and_time(values=dt)
         t = (dt(1) - 1970) * 365_vlong * 24 * 60 * 60 * 1000 &
              + dt(2) * 31_vlong * 24 * 60 * 60 * 1000 &
              + dt(3) * 24_vlong * 60 * 60 * 1000 &
              + dt(5) * 60 * 60 * 1000 &
              + dt(6) * 60 * 1000 + dt(7) * 1000 &
              + dt(8)
      end if
!           pid = getpid()
!           t = ieor(t, int(pid, kind(t)))
      do i = 1, n
         seed(i) = lcg(t)
      end do
!        end if
      call random_seed(put=seed)
!        deallocate(seed)
    contains
        ! This simple PRNG might not be good enough for real work, but is
        ! sufficient for seeding a better PRNG.
      function lcg(s)
      integer(long) :: lcg
      integer(vlong) :: s
      if (s == 0) then
         s = 104729
      else
         s = mod(s, 4294967296_vlong)
      end if
      s = mod(s * 279470273_vlong, 4294967291_vlong)
      lcg = int(mod(s, int(huge(0), vlong)), kind(0))
      end function lcg
      end subroutine init_random_seed

!##########################################################################
! Subroutine to initialise the WP position and width arrays on the first
! call to gapdrv (afterwards these values are calculated and updated in 
! calcdatmod.
! GWR 08/16
!##########################################################################
      subroutine init_gapwf

      implicit none

      integer(long) :: f,s

      wfposn = 0.0_dop
      wfwidth = 0.0_dop
      do s = 1,nstate
         do f = 1,ndof
            if(f.eq.feb)cycle
            if(f.eq.fkoe)cycle
            if(f.eq.ftime)cycle
            if(basis(f).eq.19)then
               wfposn(f,s) = rpbaspar(1,f)
            else
               wfposn(f,s)  = rsbaspar(1,f,s)
               wfwidth(f,s) = 1.0_dop/rsbaspar(3,f,s)
            endif
         enddo
      enddo

      return
      end subroutine init_gapwf

!#######################################################################
! 
! Write the new record to the DB arrays.
! Only copies adiabatic quantities when using the propagation 
! diabatisation; the diabatic quantities are added later.
!
!#######################################################################
                                                                        
      subroutine dddb_wradiab_gap_mem(gpoint,av,adip,mocoeff,nacte,soc,&
           ndof1,ltest,ciconfig,cicoeff)    

use channels

!+----------------------------------------------------------------------
!     write database for direct dynamics                               
!+----------------------------------------------------------------------
      implicit none                                                     

      logical(kind=4), intent(in)                                :: ltest
                                                                        
      integer(long)                                              :: s,s1,i,sm,&
                                                                    n,sdx,sdx1
      integer(long)                                              :: nc,ns,nsta
      integer(long), intent(in)                                  :: ndof1
      real(dop), dimension(ndof1), intent(in)                    :: gpoint
      real(dop), dimension(ndof1,nactdim), intent(in)            :: nacte
      complex(dop), dimension(nddstate,nddstate), intent(in)     :: soc
      real(dop), dimension(nddstate,nddstate), intent(in)        :: av
      real(dop), dimension(3,nddstate,nddstate), intent(in)      :: adip
      real(sip), dimension(nbasis,nbasis), intent(in)            :: mocoeff
      real(dop), dimension(ndofdd)                               :: nmpoint
      real(dop), dimension(ndof1,dercpdim)                       :: dercp
      character(len=30), dimension(maxcivec,nsmult), intent(in)  :: ciconfig
      real(dop), dimension(maxcivec,maxmultblk,nsmult), intent(in) :: cicoeff

      if (.not.ldbsave) then
         routine = 'dddb_wradiab_gap_mem' 
         message = 'Trying to write to DB arrays without dbsave option'
         call errormsg
      endif

      dbnrec = dbnrec + 1
      if (dbnrec .gt. dbmemdim) then
         routine = 'dddb_wradiab_gap_mem' 
         message = 'Too many points. Increase DBMEMDIM. '
         call errormsg
      endif

      dbgeo(1:ndof1,dbnrec) = gpoint(1:ndof1)
      if(lddtrans.and.ddcoo.eq.1) then
         call ddx2q(gpoint,nmpoint)
         dbnm(1:ndofdd,dbnrec) = nmpoint(1:ndofdd)
      endif

      if(ddcoo.eq.2)&
           dbcoord(1:ndofdd,dbnrec) = gpoint(1:ndofdd)-ddxcoo0(1:ndofdd)

      if(ldiabdb.and.dddiab.eq.4) then
! Compress derivative couplings if different multiplicities present.
         if(nsmult.gt.1)then
            do i = 1,nsmult
               do s = 2,imultblk(i)
                  do s1 = 1,s-1 
                     sdx = (s-2)*(s-1)/2+s1
                     sdx1 = (s+imultoff(i)-2)*(s+imultoff(i)-1)/2+s1+imultoff(i)
                     dercp(1:ndof1,dercpoff(i)+sdx) = nacte(1:ndof1,sdx1)
                  enddo
               enddo
            enddo
         else
            dercp = nacte
         endif
         dbdercp(1:ndof1,1:dercpdim,dbnrec) = dercp(1:ndof1,1:dercpdim)
      endif

      do s=1,nddstate
         do s1=s,nddstate
            if(.not.ldiabdb.or.ltest) dbener(s1,s,dbnrec) = av(s1,s)
            if (ldipdb) dbdipmom(1:3,s1,s,dbnrec) = adip(1:3,s1,s)
         enddo
      enddo

      do s=1,nddstate-1
         do s1=s+1,nddstate
            if(.not.ldiabdb.or.ltest) dbener(s,s1,dbnrec)=dbener(s1,s,dbnrec)
            if (ldipdb) dbdipmom(:,s,s1,dbnrec)=dbdipmom(:,s1,s,dbnrec)
         enddo
      enddo

      if (lmocodb)&
           dbmoco(1:nbasis,1:nbasis,dbnrec) = mocoeff(1:nbasis,1:nbasis)
      if(lcasocdb)&
           dbadsoc(1:nddstate,1:nddstate,dbnrec) = soc(1:nddstate,1:nddstate)
      if (nddstate .gt. 1 .and. ldiabdb)then
         do s=1,nddstate
            dbadener(s,s,dbnrec) = av(s,s)
         enddo
      endif

      if(ldiabdb .and. (dddiab.eq.3 .or. dddiab.eq.4 .or. dddiab.eq.6) &
           .and. imethod.ne.33)then
         do ns=1,nsmult
            nc = nciconf(ns)
            nsta = imultblk(ns)
            dbciconfig(1:nc,ns,dbnrec) = ciconfig(1:nc,ns)
            dbcicoeff(1:nc,1:nsta,ns,dbnrec) = cicoeff(1:nc,1:nsta,ns)
         enddo
      endif

      return                                                            

      end subroutine dddb_wradiab_gap_mem
                                                                        
!#######################################################################
! 
! Write the new record to the DB.
! For use with the diabatic quantities generated by the propagation
! and Procrustes diabatisations.
!
!#######################################################################
                                                                        
      subroutine dddb_wrdiab_gap_mem(v,cv,dip,trans,ctrans,soc,nrec)    

use channels

!+----------------------------------------------------------------------
!     write database for direct dynamics                               
!+----------------------------------------------------------------------
      implicit none                                                     
                                                                        
      integer(long), intent(in)                             :: nrec
      real(dop), dimension(nddstate,nddstate), intent(in)   :: v,trans
      real(dop), dimension(nddstate,nddstate), intent(in)   :: cv,ctrans
      real(dop), dimension(3,nddstate,nddstate), intent(in) :: dip
      complex(dop), dimension(nddstate,nddstate), intent(in):: soc

      integer(long)    :: s,s1

      dbtrans(1:nddstate,1:nddstate,nrec) &
           = trans(1:nddstate,1:nddstate)
      if (lsocbas) then
         dbctrans(1:nddstate,1:nddstate,nrec) &
              = ctrans(1:nddstate,1:nddstate)
      endif

      do s=1,nddstate
         dbener(s,s,nrec) = v(s,s)
         do s1=s+1,nddstate
            dbener(s1,s,nrec) = v(s1,s)
            if (ldipdb) dbdipmom(1:3,s1,s,nrec) = dip(1:3,s1,s)
         enddo
      enddo
      do s=1,nddstate-1
         do s1=s+1,nddstate
            dbener(s,s1,nrec)=dbener(s1,s,nrec)
            if (ldipdb) dbdipmom(:,s,s1,nrec)=dbdipmom(:,s1,s,nrec)
         enddo
      enddo

      if (lddsoc .and. .not. lsocdegen) then
         do s=1,nddstate
            dbcener(s,s,nrec) = 0.0_dop
            do s1=s+1,nddstate
               dbcener(s1,s,nrec) = cv(s1,s)
            enddo
         enddo
! Account for complex energies being Hermitian.
         do s=1,nddstate-1
            do s1=s+1,nddstate
               dbcener(s,s1,nrec)=-1.0_dop*dbcener(s1,s,nrec)
            enddo
         enddo
      endif

! Save diabatic SOCs
      if (lcsocdb) then
         do s=1,nddstate
            do s1=1,nddstate
               dbsoc(s1,s,nrec) = soc(s1,s)
            enddo
         enddo
      endif

      return                                                            

      end subroutine dddb_wrdiab_gap_mem

!#######################################################################
! 
! Write records to the DB.
! Depending on diabatisation method the DB is completely rewritten
! (propagation and Procrustes) or simply extended one entry at a time
! (projection or no diabatisation).
! Updated to use SQL based on prior effort by another author. 
! GWR 01/23
!
!#######################################################################
                                                                        
      subroutine dddb_wr_gap_file(modus)    

use channels
use db_sql

!+----------------------------------------------------------------------
!     write database for direct dynamics                               
!+----------------------------------------------------------------------
      implicit none                                                     
                                                                        
      integer(long), intent(in) :: modus
      integer(long)             :: irec,initrec,iflag,rowid

      iflag = 1

! Start SQL transaction.
      call dbstarttrans(the_db)
      
! Overwriting or adding to DB.
      if(modus.eq.1)then
         initrec = 1
      else
         initrec = dbnrec
      endif

! Loop to write to files.
      do irec = initrec, dbnrec

! Overwriting or adding to DB.
         if(irec.gt.dbnrecold .or. modus.ne.1 &
              .or. (irec.eq.1 .and. dbnrecold.eq.1))then
            rowid = 0
         else
            rowid = irec
         endif

! Setup the row in the DB.
         call dbwrentry(the_db,rowid,iflag,1000)

! Write geometry.
         call dbwrgeo(the_db,rowid,natm,dbgeo(1:ndofddpes,irec))
! Write normal modes if being used.
         if(lddtrans) &
              call dbwrnm(the_db,rowid,ndofdd,dbnm(1:ndofdd,irec))
! Write diabatic energies (which are the adiabatic ones if no diabatisation used).
         call dbwrpes(the_db,irec,nddstate,dbener(:,:,irec))
         if (lcpesdb) call dbwrcpes(the_db,irec,nddstate,dbcener(:,:,irec))
! Write adiabatic energies if multiple states.
         if(nddstate.gt.1) &
              call dbwrapes(the_db,irec,nddstate,dbadener(:,:,irec))
! Write diabatic dipoles. Adiabatic ones not stored separately.
         if(ldipdb) &
              call dbwrdip(the_db,irec,nddstate,dbdipmom(:,:,:,irec))
! Write MO coefficients if using (SA)-CAS.
         if(lmocodb) &
              call dbwrmoco(the_db,irec,nbasis,dbmoco(:,:,irec))
! Write CI vectors if required (propagation and Procrustes diabatisation).
!         if(lcivecdb) &
!              call dbwrcivec(the_db,irec,nsmult,imultblk,idxmult,dbcivec(:,irec))
! Write adiabatic-diabatic transformation matrices (propagation or
! Procrustes diabatisation calculations).
        if (dddiab.eq.4.or.dddiab.eq.6) then
           call dbwrtrans(the_db,irec,nddstate,dbtrans(:,:,irec))
        endif
        if (lsocbas) then
           call dbwrctrans(the_db,irec,nddstate,dbctrans(:,:,irec))
        endif
! Write non-adiabatic coupling terms (propagation diabatisation only).
        if(dddiab.eq.4) &
             call dbwrnact(the_db,irec,natm,nddstate,dbdercp(:,:,irec))
! Write adiabatic spin-orbit couplings if needed.
        if(lcasocdb) &
             call dbwrcasoc(the_db,irec,nddstate,dbadsoc(:,:,irec))

!         if(ddcoo.eq.2)&
!              write(icoodb) (dbcoord(n,irec),n=1,ndofdd)

      enddo

! Commit the SQL transaction.
      call dbcommittrans(the_db)

      return                                                            

      end subroutine dddb_wr_gap_file

!##########################################################################
! Subroutine to diabatise the PES when performing multi-state calculations,
! using the projection operator methods of Robertson et al.
! GWR 04/18
!##########################################################################
      subroutine proj_diab(pidx,dipole)

      implicit none

      integer(long), intent(in) :: pidx
      real(dop), dimension(3,nddstate,nddstate), intent(in) :: dipole
!      real(sip), dimension(nbasis,nbasis), intent(in) :: mocoeff

      logical(kind=4) :: fexist
      character(len=c5) :: diabinp,refpun,objpun,ddname1,extcmd,diabout
      integer(long) :: ddlaenge1,reflen,dialen,outlen,idiab,s,s1
      real(dop), dimension(istatemap(nddstate))  :: vtmp
      real(dop), dimension(nddstate,nddstate) :: v,rdum,wdum,wdum1
      complex(dop), dimension(nddstate,nddstate) :: cdum

      routine = 'proj_diab'

! Check that the input file which contains information about the 
! diabatisation exists.
      diabinp = ddname(1:ddlaenge)//'/diabci.inp'
      call rmblnk(diabinp)
      dialen = len(trim(diabinp))
      inquire(file=diabinp,exist=fexist)
      if(.not.fexist)then
         message = 'Diabatisation input file does not exist'
         call errormsg
      endif

! Check that the file which contains the CI vectors at the reference 
! geometry exists.
      refpun = ddname(1:ddlaenge)//'/reference/reference.pun'
      call rmblnk(refpun)
      reflen = len(trim(refpun))
      inquire(file=refpun,exist=fexist)
      if(.not.fexist)then
         message = 'Reference punch file does not exist'
         call errormsg
      endif
      
! Check that the file which contains the CI vectors at the new geometry exists.
      write(ddname1,'(a,2(a,i5))')ddname(1:ddlaenge),'/gwp',pidx,'_V',nddstate
      call rmblnk(ddname1)
      ddlaenge1 = len(trim(ddname1))

      objpun = ddname1(1:ddlaenge1)//'/object.pun'
      inquire(file=objpun,exist=fexist)
      if(.not.fexist)then
         message = 'Object punch file does not exist'
         call errormsg
      endif

! Copy the input and reference CI vector files to the directory containing 
! the results of the calculations at the new geometry.
      extcmd = 'cp '//refpun(1:reflen)//' '//ddname1(1:ddlaenge1)
      call excmd(extcmd)
      extcmd = 'cp '//diabinp(1:dialen)//' '//ddname1(1:ddlaenge1)
      call excmd(extcmd)

! Call the python script which does the actual diabatisation.
      extcmd = 'rm -f '//ddname1(1:ddlaenge1)//'/*endiab'
      call excmd(extcmd)
      extcmd = 'proj_diab diabci.inp '//ddname1(1:ddlaenge1)
      call excmd(extcmd)

! Check that the output file, containing the diabatic energies, exists.
      diabout = ddname1(1:ddlaenge1)//'/object.endiab'
      outlen = len(trim(diabout))
      inquire(file=diabout,exist=fexist)
      if(.not.fexist)then
         message = 'Diabatic energies file does not exist'
         call errormsg
      endif

! Open the output file and read the energies in.      
      idiab = 999
      open(unit=idiab,file=diabout,status='old',err=1000)
      rewind(idiab)
      do s = 1,nddstate
         read(idiab,*) (vtmp(s1),s1=1,nddstate)
         do s1 = 1,nddstate
            v(s,s1) = vtmp(s1)
         enddo
      enddo
      close(idiab,status='keep')

! Shift energy origin.
      do s = 1,nddstate
         v(s,s) = v(s,s)-dde0
      enddo

! Write the diabatic quantities to the DB.
      call dddb_wrdiab_gap_mem(v,rdum,dipole,wdum,wdum1,cdum,dbnrec)
      call dddb_wr_gap_file(2)

! Delete the directory of the new geometry.
      extcmd = 'rm -rf '//ddname1(1:ddlaenge1)
      call excmd(extcmd)

      return

1000  write(*,*)'Error opening diabatic output file'
      stop

      end subroutine proj_diab

!##########################################################################
! Subroutine to drive calls to the propagation diabatisation subroutines,
! the choice of which to use depends on whether we are following a 
! diabatise/add SOCs route or a transform to SOC basis/diabatise way.
! If no SOCs then just use the default prop_spin_diab.
! GWR 04/19
!##########################################################################
      subroutine diabat_gap
      implicit none

      integer(long), save   :: tid = 0 ! timer ID

! Timing
      call get_timer(tid, 'diabat_gap')
      call start_timer(tid)

      if(dddiab.eq.4)then
         if(lsocbas)then
            call prop_soc_diab
         else
            call prop_spin_diab
         endif
      else if(dddiab.eq.6)then
         if(lsocbas)then
            routine='diabat_gap'
            message='Procrustes diabatisation scheme not implemented for SOCBAS'
            call errormsg
         else
            call proc_spin_diab
         endif
      else
         routine='diabat_gap'
         message='Diabatisation scheme not implemented for GAP'
         call errormsg
      endif

      call stop_timer(tid)

      return

      end subroutine diabat_gap

!##########################################################################
! Subroutine to diabatise the PES when performing multi-state calculations.
! Based on the diabat3 routine used for DD-vMCG along with the 
! rediabatisation routine in wroper.f90. A reference geometry is defined;
! then the DB points ordered in terms of their distance from the reference.
! The propagation diabatisation proceeds from the reference point outwards.
! GWR 11/16
! Modified to include SOCs when required (resulting PESs are pure 
! spin-states).
! GWR 01/19
!##########################################################################
      subroutine prop_spin_diab

use op1lib, only: unitqxd
use mmlib, only: qqtxdd,qqtxdz,qqxxzd

      implicit none

      integer(long) :: irec,jrec,i,mtmp,idb,iprox,nstep,istep,s,s1,sdx,&
                       ierr,ms,sact,sact1
      integer(long), dimension(dbnrec)   :: map,proxidx
      integer(long), dimension(nddstate) :: pivots

      real(dop)                       :: currdist,mindist,dtmp,distsqdiff,&
                                         avdiff,avreg,fac,sgn!,dprod
      real(dop), dimension(ndofddpes) :: xgpoint
      real(dop), dimension(ndofddpes) :: dr,stepinit,stepend
      real(dop), dimension(dbnrec)    :: distsq
      real(dop), dimension(ndofddpes,dercpdim)  :: dercpdiff,dercpinit,&
                                                   dercpend,nactinit,nactend,&
                                                   integrand
      real(dop), dimension(nddstate,nddstate,dbnrec) :: gapwgt,tmpener
      real(dop), dimension(nddstate,nddstate)   :: trans,avinit,avend,dv,iminusp,&
                                                   iplusp,ipluspinv,tmpmat,tmpmat2,&
                                                   impold,ippold
      real(dop), dimension(nddstate,nddstate)   :: integral
      real(dop), dimension(nddstate,nddstate)   :: rdum,rdum1 
      real(dop), dimension(3,nddstate,nddstate) :: dipdum
      real(dop), dimension(3*nddstate)          :: work
      real(dop), dimension(:), allocatable      :: rsocvec,isocvec,rsocvecp,isocvecp
      complex(dop), dimension(nddstate,nddstate) :: soc,ctmpmat,adsoc

      logical(kind=4), save :: lfirst=.true.

! Initialise the array which keeps track of whether a point needs diabatisation
! during this step. A point only need diabatising if something has changed in the 
! path which leads to it.
      if(lfirst)then
         dbdiabrec(1,:) = 0
         dbdiabrec(2,:) = 1
         lfirst = .false.
      endif

! Allocate the array to hold the DB CI vectors.
!      call alloc_dbcivec

! Number of steps in the integration of the NACTs.
      nstep = 20

! Calculate the square of the distance from the central reference to 
! the other DB points.
      distsq = 0.0_dop
      do irec = 2,dbnrec
         do i = 1,ndofdd
            if(lddtrans.and.ddcoo.eq.1)then
               distsq(irec) = distsq(irec) + (dbnm(i,1)-dbnm(i,irec))**2
            else
               distsq(irec) = distsq(irec) + (dbgeo(i,1)-dbgeo(i,irec))**2
            endif
         enddo
      enddo

! Initialise the map array.
      do irec = 1,dbnrec
         map(irec) = irec
      enddo

! Map the DB entries so we know the order of their proximity to the reference,
! i.e. the order in which they will be diabatised.
      do irec = 1,dbnrec
         do jrec = irec+1,dbnrec
            if(distsq(jrec) .lt. distsq(irec))then
               dtmp = distsq(irec)
               distsq(irec) = distsq(jrec)
               distsq(jrec) = dtmp
               mtmp = map(irec)
               map(irec) = map(jrec)
               map(jrec) = mtmp
            endif
         enddo
      enddo

! Find the nearest other DB entry to each, which is closer to the reference; 
! this will be DB entry from which the propagation diabatisation will occur.
      proxidx = 0
      do irec = 2,dbnrec
         mindist = 99999999_dop
         do jrec = 1,irec-1
            currdist = 0.0_dop
            do i = 1,ndofdd
               if(lddtrans.and.ddcoo.eq.1)then
                  currdist = currdist + (dbnm(i,map(irec))-dbnm(i,map(jrec)))**2
               else
                  currdist = currdist + (dbgeo(i,map(irec))-dbgeo(i,map(jrec)))**2
               endif
            enddo
            distsqdiff = abs(distsq(irec)-distsq(jrec))

            if(currdist.lt.mindist .and. distsqdiff.gt.1.0e-10_dop)then
               mindist = currdist
               proxidx(map(irec)) = map(jrec)
            endif
         enddo
      enddo

! Initialise the transformation matrix, then write the first, reference entry to DB.
      if(dbdiabrec(2,1).eq.1)then
         call unitqxd(trans,nddstate)
         dv = 0.0_dop
         soc = cmplx(0.0_dop,0.0_dop,dop)
         sdx = 0
         do s = 1,nddstate
            dv(s,s) = dbadener(s,s,1)
            if(lddsoc)then
               do s1 = 1,s-1
                  sdx = sdx + 1
                  soc(s1,s) = dbadsoc(s1,s,1)
                  soc(s,s1) = dbadsoc(s,s1,1)
               enddo
            endif
         enddo        
         call dddb_wrdiab_gap_mem(dv,rdum,dipdum,trans,rdum1,soc,1)
      endif

! Check the relative signs of the states by overlapping them at adjacent geometries, 
! and adjust the signs of the derivative and SO couplings as needed.
      if(imethod.eq.12)then
         call state_sign(map,proxidx)
! If using TDDFT we cannot overlap adjacent wavefunctions so we use the overlaps of the NACTs.
      else if(imethod.eq.33)then
         call state_sign_nact(map,proxidx)
      else
         message = 'Cannot propagation diabatise with the current electronic structure method.'
         routine = 'prop_spin_diab'
         call errormsg
      endif

! Get the GPR approximation of the adiabatic energies.
      tmpener = 0.0_dop
      do irec = 1,dbnrec
         do s = 1,nddstate
            tmpener(s,s,irec) = dbadener(s,s,irec)
         enddo
      enddo
      call ddmgapwgt(gapwgt,tmpener,dbnrec,nddstate,ddmorder)

! On with the actual diabatisation.
      do irec = 2, dbnrec
         idb = map(irec)
         iprox = proxidx(idb)

! Check whether we need to diabatise this point.
! Steps from the reference point to the new point. 
         dr(1:ndofddpes) = (dbgeo(1:ndofddpes,idb) - dbgeo(1:ndofddpes,iprox))&
              /dble(nstep)

! Path of the derivative coupling between the reference and the new point.
         dercpdiff(1:ndofddpes,1:dercpdim) = (dbdercp(1:ndofddpes,1:dercpdim,idb) &
              - dbdercp(1:ndofddpes,1:dercpdim,iprox))/dble(nstep)

! Initialise the path, calculating the adiabatic energies and NACTs 
! at the starting geometry.
         stepinit(1:ndofddpes) = dbgeo(1:ndofddpes,iprox)
         dercpinit(1:ndofddpes,1:dercpdim) = dbdercp(1:ndofddpes,1:dercpdim,iprox)
         avinit(1:nddstate,1:nddstate) = dbadener(1:nddstate,1:nddstate,iprox)

         do ms = 1,nsmult
            sdx = 0
            do s = 2,imultblk(ms)
               sact = s+stoff(ms)
               do s1 = 1,s-1
                  sact1 = s1+stoff(ms)
                  sdx = sdx + 1
                  avdiff =  (avinit(sact,sact)-avinit(sact1,sact1))
                  if(abs(avdiff).lt.1.0e-8_dop)then
                     avreg = 1.0e-8_dop
                  else
                     avreg = 0.0_dop
                  endif
                  nactinit(1:ndofddpes,sdx+dercpoff(ms)) = &
                       dercpinit(1:ndofddpes,sdx+dercpoff(ms)) &
                       / (avdiff + sign(avreg,avdiff))
               enddo
            enddo
         enddo

! Loop over the number of integration steps.
         integral  = 0.0_dop

         steps: do istep = 1,nstep

! Evaluate the energies and couplings at the end of the step.
            stepend = stepinit + dr
            xgpoint = stepend
            if(istep.eq.nstep)then
               avend = 0.0_dop
               do s = 1,nddstate
                  avend(s,s) = dbadener(s,s,idb)
               enddo
            else
               call energyeval(xgpoint,gapwgt,avend,ddmorder,dbnrec)
            endif
            dercpend(:,:) = dercpinit(:,:) + dercpdiff(:,:)

! Calculate the non-adiabatic couplings at the end of the step
            do ms = 1,nsmult
               sdx = 0
               do s = 2,imultblk(ms)
                  sact = s+stoff(ms)
                  do s1 = 1,s-1
                     sact1 = s1+stoff(ms)
                     sdx = sdx + 1
                     avdiff =  (avend(sact,sact)-avend(sact1,sact1))
                     if(abs(avdiff).lt.1.0e-8_dop)then
                        avreg = 1.0e-8_dop
                     else
                        avreg = 0.0_dop
                     endif
                     nactend(1:ndofddpes,sdx+dercpoff(ms)) = &
                          dercpend(1:ndofddpes,sdx+dercpoff(ms)) &
                          / (avdiff + sign(avreg,avdiff))

                  enddo
               enddo
            enddo

! Form the actual integral.
            integrand = (nactend + nactinit)
            do ms = 1,nsmult
               sdx = 0
               do s = 2,imultblk(ms)
                  sact = s+imultoff(ms)
                  do s1 = 1,s-1
                     sact1 = s1+imultoff(ms)
                     sdx = sdx + 1
                     integral(sact1,sact) = integral(sact1,sact) + &
                          dot_product(integrand(1:ndofddpes,sdx+dercpoff(ms)),dr)
                  enddo
               enddo
            enddo
         
! The end of this step becomes the start of the next.
            stepinit(:) = stepend(:)
            avinit(:,:) = avend(:,:)
            dercpinit(:,:) = dercpend(:,:)
            nactinit(:,:) = nactend(:,:)
            
         enddo steps

! Final part of the trapezium rule integration.
         integral = 0.5_dop*integral

! Expand the integral array to cover all degenerate states by copying
! values to the relevant places. Also account for the anti-symmetry
! of the NACTs.
         tmpmat = 0.0_dop
         do s = 2,nddstate
            sact = istatemap(s)
            do s1 = 1,s-1
               if(imultmap(s).ne.imultmap(s1)) cycle
               sact1 = istatemap(s1)
               tmpmat(s1,s) = integral(sact1,sact)
               tmpmat(s,s1) = -1.0_dop*tmpmat(s1,s)               
            enddo
         enddo

! Initialise the Crank-Nicholson arrays. Phys. Rev. A, 2003, 68, 042706
         call unitqxd(iminusp,nddstate)
         call unitqxd(iplusp,nddstate)
         tmpmat = 0.5_dop*tmpmat
         tmpmat2 = tmpmat
         iminusp(:,:) = iminusp(:,:) - tmpmat
         iplusp(:,:) = iplusp(:,:) + tmpmat

! Taylor series of the exponentials.
         sgn = 1.0_dop
         fac = 1.0_dop
         do i = 2,20
            impold = iminusp
            ippold = iplusp
            tmpmat2 = matmul(tmpmat,tmpmat2)
            fac = fac/(dble(i))
            iminusp = sgn*fac*tmpmat2+iminusp
            iplusp = fac*tmpmat2+iplusp
            sgn = -sgn
            if(norm2(ippold-iplusp).lt.1.0e-10_dop &
                 .and. norm2(impold-iminusp).lt.1.0e-10_dop) exit
         enddo

! Invert the I+P/2+... matrix.
         ipluspinv=iplusp
         call dgetrf(nddstate,nddstate,ipluspinv,nddstate,pivots,ierr)
         if(ierr.ne.0)then
            routine = 'prop_spin_diab'
            message = 'Singularity in forward transformation prop.'
            call errormsg
         endif
         call dgetri(nddstate,ipluspinv,nddstate,pivots,work,3*nddstate,ierr)

! Form the transformation matrix at the new point, then get the diabatic 
! potential (and SOCs if required).
         trans = matmul(iminusp,dbtrans(1:nddstate,1:nddstate,iprox))
         trans = matmul(ipluspinv,trans)
         avend = 0.0_dop
         adsoc = cmplx(0.0_dop,0.0_dop,dop)
         sdx = 0
         do s = 1,nddstate
            avend(s,s) = dbadener(s,s,idb)
            if(lddsoc)then
               do s1 = 1,s-1
                  adsoc(s1,s) = dbadsoc(s1,s,idb)
                  adsoc(s,s1) = dbadsoc(s,s1,idb)
               enddo
            endif
         enddo

! Energies first.
         tmpmat = matmul(avend,trans)
         call qqtxdd(trans,tmpmat,dv,nddstate)
! Then SOCs.
         if(lddsoc)then
            call qqxxzd(adsoc,trans,ctmpmat,nddstate)
            call qqtxdz(trans,ctmpmat,soc,nddstate)
         endif

! Write the diabatic quantities and transformation matrix to DB.
         call dddb_wrdiab_gap_mem(dv,rdum,dipdum,trans,rdum1,soc,idb)

      enddo ! irec

! Write diabatic quantities to file.
      call dddb_wr_gap_file(1)

! Reset the diabatisation record array.
      dbdiabrec(2,:) = 0

      if(allocated(rsocvec))deallocate(rsocvec)
      if(allocated(isocvec))deallocate(isocvec)
      if(allocated(rsocvecp))deallocate(rsocvecp)
      if(allocated(isocvecp))deallocate(isocvecp)
!      call dealloc_dbcivec

      return

      end subroutine prop_spin_diab

!##########################################################################
! Subroutine to diabatise the PES when performing multi-state calculations.
! Based on the diabat3 routine used for DD-vMCG along with the 
! rediabatisation routine in wroper.f90. A reference geometry is defined;
! then the DB points ordered in terms of their distance from the reference.
! The propagation diabatisation proceeds from the reference point outwards.
! Derived from the original prop_spin_diab, this routine first transforms the 
! adiabatic quantities (energies and NACTs)to the spin-orbit basis before
! carrying out the diabatisation. As a result the diabatic PESs which 
! result are not pure spin-states.
! GWR 04/19
!
! THIS ROUTINE IS BROKEN - GW 11/23
!##########################################################################
      subroutine prop_soc_diab

use mmlib, only: qqtxdd,qqtxdz,qqxxzd,qqxxzz,qqaxzz

      implicit none

      integer(long) :: irec,jrec,i,mtmp,idb,iprox,nstep,istep,s,s1,sdx,&
                       ierr,d,ms,sact,sact1
      integer(long), dimension(dbnrec)   :: map,proxidx
      integer(long), dimension(nddstate) :: pivots

      real(dop)                       :: currdist,mindist,dtmp,distsqdiff,&
                                         avdiff,avreg,fac,sgn,tnorm
      real(dop), dimension(ndofddpes) :: xgpoint
      real(dop), dimension(ndofdd)    :: dr,stepinit,stepend
      real(dop), dimension(ndofdd)    :: dcnmidb,dcnmiprox,nmidb,nmiprox
      real(dop), dimension(dbnrec)    :: distsq
      real(dop), dimension(ndofdd,dercpdim) :: dercpdiff,dercpinit,dercpend,&
                                               nactinit,nactend
      real(dop), dimension(nddstate,nddstate,dbnrec) :: gapwgt,tmpener
      real(dop), dimension(nddstate)            :: vvec
      real(dop), dimension(nddstate,nddstate)   :: avinit,avend
      real(dop), dimension(nddstate,nddstate)   :: rtrans,rener
      real(dop), dimension(nddstate,nddstate)   :: ztrans,zener
      real(dop), dimension(3,nddstate,nddstate) :: dipdum
      real(dop), dimension(3*nddstate)          :: rwork
      real(dop), dimension(:), allocatable      :: rsocvec,isocvec,rsocvecp,isocvecp
      complex(dop), dimension(3*nddstate)       :: cwork
      complex(dop), dimension(nddstate,nddstate):: soc,cener,integral,&
                                                   tmpmat,tmpmat2,iminusp,iplusp,&
                                                   ipluspinv,trans,cdv,ippold,impold
      complex(dop), dimension(nddstate,nddstate,dbnrec) :: tmpsoc,socwgt
      complex(dop), dimension(nddstate,nddstate,ndofdd) :: inactfull,fnactfull, &
                                                           nftmp,integrand

      logical(kind=4), save :: lfirst=.true.

! Initialise the array which keeps track of whether a point needs diabatisation
! during this step. A point only need diabatising if something has changed in the 
! path which leads to it.
      if(lfirst)then
         dbdiabrec(1,:) = 0
         dbdiabrec(2,:) = 1
         lfirst = .false.
      endif

! Allocate the array to hold the DB CI vectors.
!      call alloc_dbcivec

! Number of steps in the integration of the NACTs.
      nstep = 20

! Calculate the square of the distance from the central reference to 
! the other DB points.
      distsq = 0.0_dop
      do irec = 2,dbnrec
         do i = 1,ndofdd
            if(lddtrans.and.ddcoo.eq.1)then
               distsq(irec) = distsq(irec) + (dbnm(i,1)-dbnm(i,irec))**2
            else
               distsq(irec) = distsq(irec) + (dbgeo(i,1)-dbgeo(i,irec))**2
            endif
         enddo
      enddo

! Initialise the map array.
      do irec = 1,dbnrec
         map(irec) = irec
      enddo

! Map the DB entries so we know the order of their proximity to the reference,
! i.e. the order in which they will be diabatised.
      do irec = 1,dbnrec
         do jrec = irec+1,dbnrec
            if(distsq(jrec) .lt. distsq(irec))then
               dtmp = distsq(irec)
               distsq(irec) = distsq(jrec)
               distsq(jrec) = dtmp
               mtmp = map(irec)
               map(irec) = map(jrec)
               map(jrec) = mtmp
            endif
         enddo
      enddo

! Find the nearest other DB entry to each, which is closer to the reference; 
! this will be DB entry from which the propagation diabatisation will occur.
      proxidx = 0
      do irec = 2,dbnrec
         mindist = 99999999_dop
         do jrec = 1,irec-1
            currdist = 0.0_dop
            do i = 1,ndofdd
               if(lddtrans.and.ddcoo.eq.1)then
                  currdist = currdist + (dbnm(i,map(irec))-dbnm(i,map(jrec)))**2
               else
                  currdist = currdist + (dbgeo(i,map(irec))-dbgeo(i,map(jrec)))**2
               endif
            enddo
            distsqdiff = abs(distsq(irec)-distsq(jrec))

            if(currdist.lt.mindist .and. distsqdiff.gt.1.0e-10_dop)then
               mindist = currdist
               proxidx(map(irec)) = map(jrec)
            endif
         enddo
      enddo

! Initialise the ad-diab transformation matrix, diagonalise the Hamiltonian matrix
! in the SOC basis, then write the first, reference entry to DB.
      if(dbdiabrec(2,1).eq.1)then
         cener = cmplx(0.0_dop,0.0_dop,dop)
         rtrans = 0.0_dop
         ztrans = 0.0_dop
         do s = 1,nddstate
            rtrans(s,s) = 1.0_dop
         enddo
         rener = 0.0_dop
         zener = 0.0_dop
         soc = cmplx(0.0_dop,0.0_dop,dop)
         do s = 1,nddstate
            cener(s,s) = dbadener(s,s,1)
            do s1 = 1,s-1
               cener(s1,s) = dbadsoc(s1,s,1)
               soc(s1,s) = dbadsoc(s1,s,1)
               cener(s,s1) = dbadsoc(s,s1,1)
               soc(s,s1) = dbadsoc(s,s1,1)
            enddo
         enddo
         call zheev('N','U',nddstate,cener,nddstate,vvec,cwork,3*nddstate,rwork,ierr)
         if(ierr.ne.0)then
            routine = 'prop_soc_diab'
            message = 'Error diagonalising q_ref energy matrix.'
            call errormsg
         endif
         do s = 1,nddstate
            rener(s,s) = vvec(s)
         enddo
         call dddb_wrdiab_gap_mem(rener,zener,dipdum,rtrans,ztrans,soc,1)
      endif

! Check the relative signs of the states by overlapping them at adjacent geometries, 
! and adjust the signs of the derivative and SO couplings as needed.
      call state_sign(map,proxidx)

! Get the GPR approximation of the SOCs and the adiabatic energies.
      tmpsoc = cmplx(0.0_dop,0.0_dop,dop)
      socwgt = cmplx(0.0_dop,0.0_dop,dop)
      tmpener = 0.0_dop
      do irec = 1,dbnrec
         do s = 1,nddstate
            tmpener(s,s,irec) = dbadener(s,s,irec)
            do s1 = 1,s-1
               tmpsoc(s,s1,irec) = dbadsoc(s,s1,irec)
               tmpsoc(s1,s,irec) = dbadsoc(s1,s,irec)
            enddo
         enddo
      enddo
      call ddmsocwgt(socwgt,tmpsoc,dbnrec,nddstate,ddmorder)
      call ddmgapwgt(gapwgt,tmpener,dbnrec,nddstate,ddmorder)

! On with the actual diabatisation.
      do irec = 2, dbnrec
         idb = map(irec)
         iprox = proxidx(idb)

!! Steps from the reference point to the new point. 
         if(lddtrans.and.ddcoo.eq.1)then
            call ddx2q(dbgeo(1:ndofddpes,idb),nmidb)
            call ddx2q(dbgeo(1:ndofddpes,iprox),nmiprox)
            dr = (nmidb-nmiprox)/dble(nstep)
         else
            dr(1:ndofdd) = (dbgeo(1:ndofdd,idb) - dbgeo(1:ndofdd,iprox))&
                 /dble(nstep)
         endif

! Path of the derivative coupling between the reference and the new point.
         if(lddtrans.and.ddcoo.eq.1)then
            do d = 1,dercpdim
               call ddf2q(dbdercp(1,d,idb),dcnmidb)
               call ddf2q(dbdercp(1,d,iprox),dcnmiprox)
               dercpdiff(1:ndofdd,d) = (dcnmidb - dcnmiprox)/dble(nstep)
            enddo
         else
            dercpdiff(1:ndofdd,1:dercpdim) = (dbdercp(1:ndofdd,1:dercpdim,idb) &
                 - dbdercp(1:ndofdd,1:dercpdim,iprox))/dble(nstep)
         endif

! Initialise the path, calculating the adiabatic energies and NACTs 
! at the starting geometry.
         if(lddtrans.and.ddcoo.eq.1)then
            call ddx2q(dbgeo(1:ndofddpes,iprox),stepinit)
            do d = 1,dercpdim
               call ddf2q(dbdercp(1,d,iprox),dercpinit(1,d))
            enddo
         else
            stepinit(1:ndofdd) = dbgeo(1:ndofddpes,iprox)
            dercpinit(1:ndofdd,1:dercpdim) = dbdercp(1:ndofddpes,1:dercpdim,iprox)
         endif
         avinit(1:nddstate,1:nddstate) = dbadener(1:nddstate,1:nddstate,iprox)
         do ms = 1,nsmult
            sdx = 0
            do s = 2,imultblk(ms)
               sact = s+stoff(ms)
               do s1 = 1,s-1
                  sact1 = s1+stoff(ms)
                  sdx = sdx + 1
                  avdiff =  (avinit(sact,sact)-avinit(sact1,sact1))
                  if(abs(avdiff).lt.1.0e-8_dop)then
                     avreg = 1.0e-8_dop
                  else
                     avreg = 0.0_dop
                  endif
                  nactinit(1:ndofdd,sdx+dercpoff(ms)) = &
                       dercpinit(1:ndofdd,sdx+dercpoff(ms)) &
                       / (avdiff + sign(avreg,avdiff))
               enddo
            enddo
         enddo

! Transform the initial NACT to the SOC basis.
! First get the transformation matrix by diagonalising the energy (in cener).
         cener = cmplx(0.0_dop,0.0_dop,dop)
         do s = 1,nddstate
            cener(s,s) = dbadener(s,s,iprox)
            do s1 = 1,s-1
               cener(s,s1) = dbadsoc(s,s1,iprox)
               cener(s1,s) = dbadsoc(s1,s,iprox)
            enddo
         enddo
         call zheev('V','U',nddstate,cener,nddstate,vvec,cwork,3*nddstate,rwork,ierr)
         if(ierr.ne.0)then
            routine = 'prop_soc_diab'
            write(message,'(a,i6)')'Error diagonalising q_init energy matrix. ',irec
            call errormsg
         endif
! Copy the NACT to a full-dimensioned array (nddstate*nddstate), then transform.
         inactfull = cmplx(0.0_dop,0.0_dop,dop)
         do ms = 1,nsmult
            sdx = 0
            do s = 2,imultblk(ms)
               sact = s+imultoff(ms)
               do s1 = 1,s-1
                  sact1 = s1+imultoff(ms)
                  sdx = sdx + 1
                  inactfull(sact1,sact,1:ndofdd) &
                       = nactinit(1:ndofdd,sdx+dercpoff(ms))
                  inactfull(sact,sact1,1:ndofdd) &
                       = -1.0_dop*inactfull(sact1,sact,1:ndofdd)
               enddo
            enddo
         enddo
         do d = 1,ndofdd
            call qqxxzz(inactfull(1,1,d),cener,nftmp(1,1,d),nddstate)
            call qqaxzz(cener,nftmp(1,1,d),inactfull(1,1,d),nddstate)
         enddo
         
! Loop over the number of integration steps.
         integral  = cmplx(0.0_dop,0.0_dop,dop)

         steps: do istep = 1,nstep

! Evaluate the energies and couplings at the end of the step.
            stepend(1:ndofdd) = stepinit(1:ndofdd) + dr(1:ndofdd)
            if(lddtrans.and.ddcoo.eq.1)then
               call ddq2x(stepend,xgpoint)
            else
               xgpoint = stepend
            endif
            if(istep.eq.nstep)then
               avend = 0.0_dop
               do s = 1,nddstate
                  avend(s,s) = dbadener(s,s,idb)
               enddo
            else
               call energyeval(xgpoint,gapwgt,avend,ddmorder,dbnrec)
!               call energyeval(stepend,gapwgt,avend,ddmorder,dbnrec)
!               avend = avinit + avstep
            endif
            dercpend(:,:) = dercpinit(:,:) + dercpdiff(:,:)

! Calculate the non-adiabatic couplings at the end of the step
            do ms = 1,nsmult
               sdx = 0
               do s = 2,imultblk(ms)
                  sact = s+stoff(ms)
                  do s1 = 1,s-1
                     sact1 = s1+stoff(ms)
                     sdx = sdx + 1
                     avdiff =  (avend(sact,sact)-avend(sact1,sact1))
                     if(abs(avdiff).lt.1.0e-8_dop)then
                        avreg = 1.0e-8_dop
                     else
                        avreg = 0.0_dop
                     endif
                     nactend(1:ndofdd,sdx+dercpoff(ms)) = &
                          dercpend(1:ndofdd,sdx+dercpoff(ms)) &
                          / (avdiff + sign(avreg,avdiff))
                  enddo
               enddo
            enddo

! Transform the end NACT to the SOC basis.
! First get the transformation matrix by diagonalising the energy (in cener).
         cener = cmplx(0.0_dop,0.0_dop,dop)
         if(istep.ne.nstep)&
              call soceval(xgpoint,socwgt,cener,ddmorder,dbnrec)
         do s = 1,nddstate
            cener(s,s) = avend(s,s)
            if(istep .eq. nstep)then
               do s1 = 1,s-1
                  cener(s,s1) = dbadsoc(s,s1,idb)
                  cener(s1,s) = dbadsoc(s1,s,idb)
               enddo
            endif
         enddo
         call zheev('V','U',nddstate,cener,nddstate,vvec,cwork,3*nddstate,rwork,ierr)
         if(ierr.ne.0)then
            routine = 'prop_soc_diab'
            write(message,'(a,i6)')'Error diagonalising q_end energy matrix. ',irec
            call errormsg
         endif
! Copy the NACT to a full-dimensioned array (nddstate*nddstate), then transform.
         fnactfull = cmplx(0.0_dop,0.0_dop,dop)
         do ms = 1,nsmult
            sdx = 0
            do s = 2,imultblk(ms)
               sact = s+imultoff(ms)
               do s1 = 1,s-1
                  sact1 = s1+imultoff(ms)
                  sdx = sdx + 1
                  fnactfull(sact1,sact,1:ndofdd) &
                       = nactend(1:ndofdd,sdx+dercpoff(ms))
                  fnactfull(sact,sact1,1:ndofdd) &
                       = -1.0_dop*fnactfull(sact1,sact,1:ndofdd)
               enddo
            enddo
         enddo
         do d = 1,ndofdd
            call qqxxzz(fnactfull(1,1,d),cener,nftmp(1,1,d),nddstate)
            call qqaxzz(cener,nftmp(1,1,d),fnactfull(1,1,d),nddstate)
         enddo

! Form the actual integral.
         integrand = (inactfull + fnactfull)
         do s = 1,nddstate
            do s1 = 1,nddstate
               integral(s1,s) = integral(s1,s) + &
                    dot_product(integrand(s1,s,1:ndofdd),dr)                 
            enddo
         enddo
         
! The end of this step becomes the start of the next.
            stepinit(:) = stepend(:)
            avinit(:,:) = avend(:,:)
            dercpinit(:,:) = dercpend(:,:)
            inactfull(:,:,:) = fnactfull(:,:,:)
            
         enddo steps

! Final part of the trapezium rule integration.
         integral = 0.5_dop*integral

! Initialise the Crank-Nicholson arrays. Phys. Rev. A, 2003, 68, 042706
         call unitqxz(iminusp,nddstate)
         call unitqxz(iplusp,nddstate)
         tmpmat = 0.5_dop*integral
         tmpmat2 = tmpmat
         iminusp(:,:) = iminusp(:,:) - tmpmat
         iplusp(:,:) = iplusp(:,:) + tmpmat

! Taylor series of the exponentials.
         sgn = 1.0_dop
         fac = 1.0_dop
         do i = 2,20
            impold = iminusp
            ippold = iplusp
            tmpmat2 = matmul(tmpmat,tmpmat2)
            fac = fac/(dble(i))
            iminusp = sgn*fac*tmpmat2+iminusp
            iplusp = fac*tmpmat2+iplusp
            sgn = -sgn
            tnorm  = 0.0_dop
            do s = 1,nddstate
               do s1 = 1,nddstate
                  tnorm = tnorm+abs(ippold(s1,s)-iplusp(s1,s))**2
               enddo
            enddo
            if(sqrt(tnorm).lt.1.0e-10_dop) exit
         enddo

! Invert the I+P/2+... matrix.
         ipluspinv=iplusp
         call zgetrf(nddstate,nddstate,ipluspinv,nddstate,pivots,ierr)
         if(ierr.ne.0)then
            routine = 'prop_soc_diab'
            message = 'Singularity in forward transformation prop.'
            call errormsg
         endif
         call zgetri(nddstate,ipluspinv,nddstate,pivots,cwork,3*nddstate,ierr)

! Form the transformation matrix at the new point, then get the diabatic 
! potential (and SOCs if required).
         trans(1:nddstate,1:nddstate) = cmplx(dbtrans(1:nddstate,1:nddstate,iprox),&
              dbctrans(1:nddstate,1:nddstate,iprox),dop)
         trans = matmul(iminusp,trans)
         trans = matmul(ipluspinv,trans)

         cener = cmplx(0.0_dop,0.0_dop,dop)
         do s = 1,nddstate
            cener(s,s) = dbadener(s,s,idb)
            if(lddsoc)then
               do s1 = 1,s-1
                  cener(s1,s) = dbadsoc(s1,s,idb)
                  soc(s1,s) = dbadsoc(s1,s,idb)
                  cener(s,s1) = conjg(cener(s1,s))
                  soc(s,s1) = conjg(soc(s1,s))
               enddo
            endif
         enddo
         call zheev('N','U',nddstate,cener,nddstate,vvec,cwork,3*nddstate,rwork,ierr)
         if(ierr.ne.0)then
            routine = 'prop_soc_diab'
            write(message,'(a,i6)')'Error diagonalising q_end energy matrix. ',irec
            call errormsg
         endif
         avend = 0.0_dop
         do s = 1,nddstate
            avend(s,s) = vvec(s)
         enddo

! Energies first.
         tmpmat = matmul(avend,trans)
         call qqaxzz(trans,tmpmat,cdv,nddstate)

! Convert to real format.
         do s = 1,nddstate
            do s1 = 1,nddstate
               rener(s,s1) = dble(cdv(s,s1))
               zener(s,s1) = aimag(cdv(s,s1))
               rtrans(s,s1) = dble(trans(s,s1))
               ztrans(s,s1) = aimag(trans(s,s1))
            enddo
         enddo

! Write the diabatic quantities and transformation matrix to DB.
         call dddb_wrdiab_gap_mem(rener,zener,dipdum,rtrans,ztrans,soc,idb)

      enddo ! irec

! Write diabatic quantities to file.
      call dddb_wr_gap_file(1)

! Reset the diabatisation record array.
      dbdiabrec(2,:) = 0

! Final deallocation.
      if(allocated(rsocvec))deallocate(rsocvec)
      if(allocated(isocvec))deallocate(isocvec)
      if(allocated(rsocvecp))deallocate(rsocvecp)
      if(allocated(isocvecp))deallocate(isocvecp)
!      call dealloc_dbcivec

      return

      end subroutine prop_soc_diab

!##########################################################################
! Subroutine to diabatise the PES, when performing multi-state 
! calculations, using the solution to the orthogonal Procrustes problem
! using the SVD wavefunction overlaps.
! Reuses some stuff from the propagation diabatisation routines.
! GWR 08/19
!##########################################################################
      subroutine proc_spin_diab

use op1lib, only: unitqxd
use mmlib, only: qqtxdd,qqtxdz,qqxxzd
use funcqchemmod, only: wrmolpro_geom,rdmolpro_ao,rdmolpro_ao_1,rdmolpro_ci,&
     wrmolpro_orbfile

      implicit none

      logical(kind=4) :: laofail
      logical(kind=4), dimension(:,:), allocatable :: lsign
      integer(long) :: irec,jrec,mtmp,s,s1,s2,sdx,f,idb,iprox,jlbl,msta,ilbl,ncas1,&
           soff,lwork,i,info,ms,send,sact1,sact2,j,k,soff1,irrep,iroffcls,iroffocc,&
           iroffcas,iroffbas,multend
      integer(long), dimension(dbnrec)    :: map,proxidx
      real(dop) :: dtmp,mindist,currdist,distsqdiff
      real(dop), dimension(dbnrec)    :: distsq
      real(dop), dimension(nddstate,nddstate)   :: trans,dv,tmpmat
      real(dop), dimension(nddstate,nddstate)   :: rdum,rdum1
      real(dop), dimension(3,nddstate,nddstate) :: dipdum
      real(dop), dimension(ndofddpes) :: xgpoint,xgpointref
      real(dop), dimension(:), allocatable :: stover,sigma,work,sigmamo
      real(dop), dimension(:,:), allocatable :: sao,smat,dsmat,umat,vmatt,&
           tmptrans,smo,umatmo,vmattmo,moco,assmo,cssmo,motransas,motranscs,tmpdsmat,&
           mocoidb,mocoiprox
      real(sip), dimension(:,:,:), allocatable :: dbmocotmp
      complex(dop), dimension(nddstate,nddstate) :: soc,adsoc,ctmpmat
      character(len=240) :: templdatfileao,templdatfileci,filnamao,inpfileao,&
           outfileao,inpfileci,outfileci,filnamci,orbfileci
      character(len=240) :: extcmd

      logical(kind=4), save :: lfirst=.true.

! Initialise the array which keeps track of whether a point needs diabatisation
! during this step. A point only need diabatising if something has changed in the 
! path which leads to it.
      if(lfirst)then
         dbdiabrec(1,:) = 0
         dbdiabrec(2,:) = 1
         lfirst = .false.
      endif

! Calculate the square of the distance from the central reference to 
! the other DB points.
      distsq = 0.0_dop
      do irec = 2,dbnrec
         do f = 1,ndofdd
            if(lddtrans.and.ddcoo.eq.1)then
               distsq(irec) = distsq(irec) + (dbnm(f,1)-dbnm(f,irec))**2
            else
               distsq(irec) = distsq(irec) + (dbgeo(f,1)-dbgeo(f,irec))**2
            endif
         enddo
      enddo

! Initialise the map array.
      do irec = 1,dbnrec
         map(irec) = irec
      enddo

! Map the DB entries so we know the order of their proximity to the reference,
! i.e. the order in which they will be diabatised.
      do irec = 1,dbnrec
         do jrec = irec+1,dbnrec
            if(distsq(jrec) .lt. distsq(irec))then
               dtmp = distsq(irec)
               distsq(irec) = distsq(jrec)
               distsq(jrec) = dtmp
               mtmp = map(irec)
               map(irec) = map(jrec)
               map(jrec) = mtmp
            endif
         enddo
      enddo

! Find the nearest other DB entry to each, which is closer to the reference; 
! this will be DB entry from which the diabatisation will occur.
      proxidx = 0
      do irec = 2,dbnrec
         mindist = 99999999_dop
         do jrec = 1,irec-1
            currdist = 0.0_dop
            do f = 1,ndofdd
               if(lddtrans.and.ddcoo.eq.1)then
                  currdist = currdist + (dbnm(f,map(irec))-dbnm(f,map(jrec)))**2
               else
                  currdist = currdist + (dbgeo(f,map(irec))-dbgeo(f,map(jrec)))**2
               endif
            enddo
            distsqdiff = abs(distsq(irec)-distsq(jrec))

            if(currdist.lt.mindist .and. distsqdiff.gt.1.0e-10_dop)then
               mindist = currdist
               proxidx(map(irec)) = map(jrec)
            endif
         enddo
      enddo

! Initialise the transformation matrix, then write the first, reference entry to DB.
      if(dbdiabrec(2,1).eq.1)then
         call unitqxd(trans,nddstate)
         dv = 0.0_dop
         soc = cmplx(0.0_dop,0.0_dop,dop)
         do s = 1,nddstate
            dv(s,s) = dbadener(s,s,1)
            if(lddsoc)then
               do s1 = 1,s-1
                  soc(s1,s) = dbadsoc(s1,s,1)
                  soc(s,s1) = dbadsoc(s,s1,1)
               enddo
            endif
         enddo
         call dddb_wrdiab_gap_mem(dv,rdum,dipdum,trans,rdum1,soc,1)
         dbdiabrec(2,1) = 0
      endif

! Allocate the array to hold the DB CI vectors.
!      call alloc_dbcivec

! Read the CI vectors of the calculated wavefunctions from file. Array
! must be allocated before the call to here.
!      call dddb_rd_civec

! Allocate arrays to hold state overlaps, and arrays holding AS occupancies etc.
      msta=maxval(imultblk(1:nsmult))

      if(allocated(stover))deallocate(stover)
      allocate(stover(nsmult*msta*msta))

      if(allocated(smat))deallocate(smat)
      allocate(smat(msta,msta))

      if(allocated(dsmat))deallocate(dsmat)
      allocate(dsmat(msta,msta))

      if(allocated(tmpdsmat))deallocate(tmpdsmat)
      allocate(tmpdsmat(msta,msta))

      if(allocated(umat))deallocate(umat)
      allocate(umat(msta,msta))

      if(allocated(vmatt))deallocate(vmatt)
      allocate(vmatt(msta,msta))

      if(allocated(tmptrans))deallocate(tmptrans)
      allocate(tmptrans(msta,msta))

      if(allocated(sigma))deallocate(sigma)
      allocate(sigma(msta))

      lwork = 5*max(msta,nocc)
      if(allocated(work))deallocate(work)
      allocate(work(lwork))

      if(allocated(sao))deallocate(sao)
      allocate(sao(nbasis,nbasis))

      if(allocated(smo))deallocate(smo)
      allocate(smo(nocc,nocc))

      if(allocated(sigmamo))deallocate(sigmamo)
      allocate(sigmamo(nocc))

      if(allocated(umatmo))deallocate(umatmo)
      allocate(umatmo(nocc,nocc))

      if(allocated(vmattmo))deallocate(vmattmo)
      allocate(vmattmo(nocc,nocc))

      if(allocated(motransas))deallocate(motransas)
      allocate(motransas(ncas,ncas))

      if(allocated(motranscs))deallocate(motranscs)
      allocate(motranscs(nclosed,nclosed))

      if(allocated(moco))deallocate(moco)
      allocate(moco(nocc,nbasis))

      if(allocated(mocoidb))deallocate(mocoidb)
      allocate(mocoidb(nbasis,nbasis))

      if(allocated(mocoiprox))deallocate(mocoiprox)
      allocate(mocoiprox(nbasis,nbasis))

      if(allocated(assmo))deallocate(assmo)
      allocate(assmo(ncas,ncas))

      if(allocated(cssmo))deallocate(cssmo)
      allocate(cssmo(nclosed,nclosed))

      if(allocated(lsign))deallocate(lsign)
      allocate(lsign(msta,nsmult))

      if(allocated(dbmocotmp))deallocate(dbmocotmp)
      allocate(dbmocotmp(nbasis,nbasis,dbnrec))
      do irec = 1,dbnrec
         dbmocotmp(1:nbasis,1:nbasis,irec) = dbmoco(1:nbasis,1:nbasis,irec)
      enddo

      templdatfileao=ddname(1:ddlaenge)//'/template_ao.dat'
      filnamao = ddname(1:ddlaenge)//'/ao_over'
      call rmblnk(filnamao)
      ilbl=index(filnamao,' ')-1
      inpfileao=filnamao(1:ilbl)//'.inp'
      outfileao=filnamao(1:ilbl)//'.out'

      templdatfileci=ddname(1:ddlaenge)//'/template_ci.dat'
      filnamci = ddname(1:ddlaenge)//'/diab_ci'
      call rmblnk(filnamci)
      ilbl=index(filnamci,' ')-1
      inpfileci=filnamci(1:ilbl)//'.inp'
      outfileci=filnamci(1:ilbl)//'.out'
      orbfileci=filnamci(1:ilbl)//'.orb'

! Loop over all bar the reference record (irec=1). Move out in order of
! increasing distance from the reference (idb) with comparisons made to 
! the nearest DB entry which is closed to the reference (iprox).
      do irec = 2,dbnrec
         idb = map(irec)
         iprox = proxidx(idb)

! Check whether we need to diabatise this point.
!         if(dbdiabrec(1,idb).eq.0 .or. &    ! New point.
!            dbdiabrec(1,idb).ne.iprox.or. & ! Nearest point is new.
!            dbdiabrec(2,iprox).eq.1)then    ! Nearest point rediabatised.
!            dbdiabrec(1,idb) = iprox
!            dbdiabrec(2,idb) = 1
!         else
!            cycle
!         endif

! Set up a Molpro input file to get the matrix of AO overlaps at the two
! geometries.
         xgpoint=dbgeo(1:ndofddpes,idb)
         xgpointref=dbgeo(1:ndofddpes,iprox)
         call wrmolpro_geom(templdatfileao,inpfileao,xgpoint,xgpointref,xatnam,natm,2)
! Run Molpro.
         jlbl=index(ddextcmd,' ')
         if (qcsubform .eq. 1) then
            extcmd=ddextcmd(1:jlbl)//filnamao(ddlaenge+2:ilbl+1)//char(zeroi)
         else if (qcsubform .eq. 2) then
            extcmd=ddextcmd(1:jlbl)//filnamao(ddlaenge+2:ilbl+1)// &
                 ddname(1:ddlaenge+1)//char(zeroi)
         endif
         call excmd(extcmd)
!         call execute_command_line(extcmd)

! Extract the AO overlap matrix from the output file.
         call rdmolpro_ao(outfileao,sao,nbasis,laofail)

         if(laofail)then
            call unitqxd(sao,nbasis)
            write(ilog,'(a,2i6,a)')'Failed to get AO overlap between DB entries',&
                 idb,iprox,' Using unit matrix.'
            flush(ilog)
         endif

! First task is to diabatise the orbitals.
! Get the overlaps of the MOs (same for all multiplicities).
         mocoidb(1:nbasis,1:nbasis) = dble(dbmocotmp(1:nbasis,1:nbasis,idb))
         mocoiprox(1:nbasis,1:nbasis) = dble(dbmocotmp(1:nbasis,1:nbasis,iprox))
         call molpro_moover(sao,smo,mocoidb,mocoiprox,nbasis,0)

! Next transform the active space MOs at idb to best match those at iprox using the
! orthogonal Procrustes method.
! Perform the SVD of the transpose of the AS MO overlaps, then use the product of 
! the left and right singular vectors to transform the MO coefficients.
         iroffcas=nclosed
         iroffcls=0

         moco = 0.0_dop
         ncas1 = nocc-nclosed
         assmo=0.0_dop

         do i = 1,ncas1
            do j = 1,ncas1
               assmo(i,j) = smo(j,i)
            enddo
         enddo

         call dgesvd('S','S',ncas1,ncas1,assmo,ncas,sigmamo,umatmo,nocc,vmattmo,&
              nocc,work,lwork,info)
         if(info.ne.0)then
            routine = 'proc_spin_diab'
            write(message,'(a,i10,x,a,2i6)')'Error in SVD for AS MOs:',info,&
                 'Records: ',idb,iprox
            call errormsg
         endif

         do i = 1,ncas1
            do j = 1,ncas1
               motransas(i,j) = umatmo(i,1)*vmattmo(1,j)
               do k = 2,ncas1
                  motransas(i,j) = motransas(i,j) + umatmo(i,k)*vmattmo(k,j)
               enddo
            enddo
         enddo
         iroffcls=iroffcls+nclosed
         do k = 1,nbasis
            do j = 1,ncas1
               do i = 1,ncas1
                  moco(i,k) = moco(i,k) &
                       + motransas(i,j)*mocoidb(iroffcls+j,k)
               enddo
            enddo
         enddo

! Copy back to the DB array.
         do j=1,ncas1
            do k = 1,nbasis
               dbmocotmp(iroffcls+j,k,idb) = real(moco(j,k))
            enddo
         enddo

! Perform the SVD of the transpose of the Closed MO overlaps, then use the product of 
! the left and right singular vectors to transform the MO coefficients.
         iroffcls=0
         iroffocc=0

         moco = 0.0_dop

         cssmo = 0.0_dop
         do i = 1,nclosed
            do j = 1,nclosed
               cssmo(i,j) = smo(j,i)
            enddo
         enddo

         call dgesvd('S','S',nclosed,nclosed,cssmo,nclosed,sigmamo,&
              umatmo,nocc,vmattmo,nocc,work,lwork,info)
         if(info.ne.0)then
            routine = 'proc_spin_diab'
            write(message,'(a,i10,x,a,2i6,x,a,i6)')'Error in SVD for Closed MOs:',info,&
                 'Records: ',idb,iprox,'Irrep: ',irrep
            call errormsg
         endif

         do i = 1,nclosed
            do j = 1,nclosed
               motranscs(i,j) = umatmo(i,1)*vmattmo(1,j)
               do k = 2,nclosed
                  motranscs(i,j) = motranscs(i,j) + umatmo(i,k)*vmattmo(k,j)
               enddo
            enddo
         enddo
         do k = 1,nbasis
            do j = 1,nclosed
               do i = 1,nclosed
                  moco(i,k) = moco(i,k) &
                       + motranscs(i,j)*mocoidb(j,k)
               enddo
            enddo
         enddo

! Copy back to the DB array.
         do j=1,nclosed
            do k = 1,nbasis
               dbmocotmp(j,k,idb) = real(moco(j,k))
            enddo
         enddo

! Check whether we need to diabatise this point.
         if(dbdiabrec(1,idb).eq.0 .or. &    ! New point.
            dbdiabrec(1,idb).ne.iprox.or. & ! Nearest point is new.
            dbdiabrec(2,iprox).eq.1)then    ! Nearest point rediabatised.
            dbdiabrec(1,idb) = iprox
            dbdiabrec(2,idb) = 1
         else
            cycle
         endif

! Transform the CI vectors using the AS and CS MO transformations.
! NB it seems the CI stored in the DB is in diabatic but MOs still in 
! adiabatic??
         call citrans(motransas,nciconf,dbciconfig(1,1,idb),dbcicoeff(1,1,1,idb))

! Get the overlaps of all states of the the adiabatic wavefunctions.
! MOCOs and CIs are in diabatic basis
         mocoidb(1:nbasis,1:nbasis) = dble(dbmocotmp(1:nbasis,1:nbasis,idb))
         mocoiprox(1:nbasis,1:nbasis) = dble(dbmocotmp(1:nbasis,1:nbasis,iprox))       
! Get smo again with new MOCOs
         call molpro_moover(sao,smo,mocoidb,mocoiprox,nbasis,0)

! NB. Having added smo and not calculating it in molpro_wfover
! need to check bases.
         call molpro_wfover(sao,smo,mocoidb,mocoiprox,nbasis,lsign,msta,stover,&
              0,nciconf,dbciconfig(1,1,idb),dbcicoeff(1,1,1,idb),&
              dbciconfig(1,1,iprox),dbcicoeff(1,1,1,iprox))

! Loop over the multiplicities.
         soff = 0
         soff1 = 0
         trans = 0.0_dop
         do ms = 1,nsmult
! Copy the adiabatic overlaps to a temporary array. Reference geometry rows, 
! new geometry columns.
            sdx = 1
            do s = 1,imultblk(ms)
               do s1 = 1,imultblk(ms)
                  smat(s1,s) = stover(sdx)
                  sdx = sdx + 1
               enddo
            enddo
! Transform the reference adiabatic states to the diabatic to get the overlap 
! between the iprox diabatic states and the idb adiabatic.
            do s = 1,imultblk(ms)
               do s1 = 1,imultblk(ms)
                  dsmat(s1,s) = dbtrans(imultoff(ms)+1,imultoff(ms)+s1,iprox)*smat(1,s)
                  do s2 = 2,imultblk(ms)
                     dsmat(s1,s) = dsmat(s1,s) &
                          + dbtrans(imultoff(ms)+s2,imultoff(ms)+s1,iprox)*smat(s2,s)
                  enddo
               enddo
            enddo

            tmpdsmat = dsmat
            dsmat=transpose(dsmat)
! Carry out the SVD of dsmat.
            call dgesvd('S','S',imultblk(ms),imultblk(ms),dsmat,msta,sigma,umat,msta,&
                 vmatt,msta,work,lwork,info)
            if(info.ne.0)then
               routine = 'proc_spin_diab'
               write(message,'(a,i8,a,2i8,a,i8)')'Problem in total WF SVD',info,&
                    ' Records',idb,iprox,' Multiplicity',ms
               call errormsg
            endif
            dsmat = tmpdsmat

! The transformation matrix at idb is just the product of the left and right 
! singular vectors transposed (UV^{T}).
            do s = 1,imultblk(ms)
               do s1 = 1,imultblk(ms)
                  tmptrans(s1,s) = umat(s1,1)*vmatt(1,s)
                  do s2 = 2,imultblk(ms)
                     tmptrans(s1,s) = tmptrans(s1,s) + umat(s1,s2)*vmatt(s2,s)
                  enddo
               enddo
            enddo
! Copy the transformation matrix for this multiplicity to the full transformation 
! matrix, including copies for all spin projections.
            if(ms.eq.nsmult)then
               send = nddstate
            else
               send = stoff(ms+1)
            endif
            multend = 1
            if(lddsoc.and..not.lsocdegen)&
                 multend = idxmult(ms)

!            do s = 1,idxmult(ms)
            do s = 1,multend
               do s1 = 1,imultblk(ms)
                  sact1 = istatemap(s1+stoff(ms))-imultoff(ms)
                  do s2 = 1,imultblk(ms)
                     sact2 = istatemap(s2+stoff(ms))-imultoff(ms)
                     trans(s2+soff1,s1+soff1) = tmptrans(sact2,sact1)
                  enddo
               enddo
               soff1 = soff1 + imultblk(ms)
            enddo
!            endif

! Increment the offset.
            soff = soff + imultblk(ms)*imultblk(ms)
         enddo

! Transform the adiabatic quantities at point idb, then write to DB arrays.
! First the energies.
         tmpmat = matmul(dbadener(1:nddstate,1:nddstate,idb),trans)
         call qqtxdd(trans,tmpmat,dv,nddstate)

! Then SOCs.
         if(lddsoc)then
            adsoc = cmplx(0.0_dop,0.0_dop,dop)
            do s = 1,nddstate
               do s1 = 1,s-1
                  adsoc(s1,s) = dbadsoc(s1,s,idb)
                  adsoc(s,s1) = dbadsoc(s,s1,idb)
               enddo
            enddo
            call qqxxzd(adsoc,trans,ctmpmat,nddstate)
            call qqtxdz(trans,ctmpmat,soc,nddstate)
         endif

         call dddb_wrdiab_gap_mem(dv,rdum,dipdum,trans,rdum1,soc,idb)

      enddo ! irec

! Write diabatic quantities to file.
      call dddb_wr_gap_file(1)

! Reset the diabatisation record array.
      dbdiabrec(2,:) = 0

! Final deallocation.
      deallocate(stover,sao,smo,lsign,smat,dsmat,sigma,umat,vmatt,tmptrans,&
           sigmamo,umatmo,vmattmo,moco,assmo,cssmo,motransas,motranscs,tmpdsmat,&
           mocoidb,mocoiprox,dbmocotmp)
!      call dealloc_dbcivec

      return

      end subroutine proc_spin_diab

!##########################################################################
! Subroutine to overlap a CAS wavefunction at one geometry with another
! at a different geometry (using Molpro) so as to check the consistency 
! of the (arbitrary) phase of the states, and to change the phases if
! required.
! GWR 05/19
!##########################################################################
      subroutine state_sign(map,proxidx)

use funcqchemmod, only: wrmolpro_geom,rdmolpro_ao

      implicit none

      integer(long), dimension(dbnrec), intent(in) :: map,proxidx

      logical(kind=4) :: laofail
      logical(kind=4), dimension(:,:), allocatable :: lsign
      integer(long) :: iprox,idb,irec,ilbl,jlbl,s,s1,msta,nconf1,sm,sdx,i,sm1,&
                       soff,soff1,sact,sact1
      character(len=120) :: templdatfile,filnam,inpfile,outfile
      character(len=240) :: extcmd
      real(dop), dimension(ndofddpes) :: xgpoint,xgpointref
      real(dop), dimension(:), allocatable :: stover
      real(dop), dimension(:,:), allocatable :: sao,smo
      real(dop), dimension(nbasis,nbasis) :: mocoidb,mocoiprox

! Read the CI vectors of the calculated wavefunctions from file. Array
! must be allocated before the call to here.
!      call dddb_rd_civec

      templdatfile=ddname(1:ddlaenge)//'/template_ao.dat'
      filnam = ddname(1:ddlaenge)//'/ao_over'
      call rmblnk(filnam)
      ilbl=index(filnam,' ')-1
      inpfile=filnam(1:ilbl)//'.inp'

      msta=maxval(imultblk(1:nsmult))
      if(allocated(lsign))deallocate(lsign)
      allocate(lsign(msta,nsmult))
      lsign=.false.

! Allocate array to hold state overlaps, and arrays holding AS occupancies.
      if(allocated(stover))deallocate(stover)
      allocate(stover(nsmult*msta*msta))

      if(allocated(sao))deallocate(sao)
      allocate(sao(nbasis,nbasis))
      if(allocated(smo))deallocate(smo)
      allocate(smo(nocc,nocc))
! Loop over all bar the reference record (irec=1). Move out in order of
! increasing distance from the reference (idb) with comparisons made to 
! the nearest DB entry which is closed to the reference (iprox).
      do irec = 2,dbnrec
         idb = map(irec)
         iprox = proxidx(idb)

! Check whether this point is being diabatised, hence whether it needs its sign checking.
         if(dbdiabrec(1,idb).eq.0 .or. &    ! New point.
            dbdiabrec(1,idb).ne.iprox.or. & ! Nearest point is new.
            dbdiabrec(2,iprox).eq.1)then    ! Nearest point rediabatised.
            dbdiabrec(1,idb) = iprox
            dbdiabrec(2,idb) = 1
         else
            cycle
         endif

! Set up a Molpro input file to get the matrix of AO overlaps at the two
! geometries.
         xgpoint=dbgeo(1:ndofddpes,idb)
         xgpointref=dbgeo(1:ndofddpes,iprox)
         call wrmolpro_geom(templdatfile,inpfile,xgpoint,xgpointref,xatnam,natm,2)
! Run Molpro.
         jlbl=index(ddextcmd,' ')
         if (qcsubform .eq. 1) then
            extcmd=ddextcmd(1:jlbl)//filnam(ddlaenge+2:ilbl+1)//char(zeroi)
         else if (qcsubform .eq. 2) then
            extcmd=ddextcmd(1:jlbl)//filnam(ddlaenge+2:ilbl+1)// &
                 ddname(1:ddlaenge+1)//char(zeroi)
         endif
         call excmd(extcmd)
!         call execute_command_line(extcmd)

! Extract the AO overlap matrix from the output file.
         outfile=filnam(1:ilbl)//'.out'
         call rdmolpro_ao(outfile,sao,nbasis,laofail)

! Call the routine which actually overlaps the two wavefunctions using the AO overlaps,
! MO coefficients and CI expansion vectors. The logical array, lsign, indicates which 
! states need to have their signs flipped.
         if(.not.laofail)then
            mocoidb(1:nbasis,1:nbasis) = dble(dbmoco(1:nbasis,1:nbasis,idb))
            mocoiprox(1:nbasis,1:nbasis) = dble(dbmoco(1:nbasis,1:nbasis,iprox))
! Get smo 
            call molpro_moover(sao,smo,mocoidb,mocoiprox,nbasis,0)

            call molpro_wfover(sao,smo,mocoidb,mocoiprox,nbasis,lsign,msta,&
                 stover,0,nciconf,dbciconfig(1,1,idb),dbcicoeff(1,1,1,idb),&
                 dbciconfig(1,1,iprox),dbcicoeff(1,1,1,iprox))
         else
            write(ilog,'(a,2i6)')'Failed to get AO overlap between DB entries',idb,iprox
            flush(ilog)
         endif

! Flip the signs of the CI vectors and derivative couplings if needed.
         do sm = 1,nsmult
            nconf1 = dbcivec(sm,idb)%nconf
            sdx=0
            do s=1,imultblk(sm)
               if(lsign(s,sm)) dbcivec(sm,idb)%coeff(1:nconf1,s) &
                    = -1.0_dop*dbcivec(sm,idb)%coeff(1:nconf1,s)
               do s1 = 1,s-1
                  sdx=sdx+1
                  if(lsign(s,sm)) dbdercp(1:ndofddpes,sdx+dercpoff(sm),idb) &
                       = -1.0_dop*dbdercp(1:ndofddpes,sdx+dercpoff(sm),idb)
                  if(lsign(s1,sm)) dbdercp(1:ndofddpes,sdx+dercpoff(sm),idb) &
                       = -1.0_dop*dbdercp(1:ndofddpes,sdx+dercpoff(sm),idb)
               enddo
            enddo
         enddo

! Flip the SOCs if required.
         if(lddsoc)then
            do s = 1,nddstate
               sact = istatemap(s)
               sm = 1
               do i = 1,nsmult
                  if(imultmap(s) .eq. idxmult(i))exit
                  sm = sm+1
               enddo
               soff = imultoff(sm)
               do s1 = 1,s-1
                  sact1 = istatemap(s1)
                  sm1 = 1
                  do i = 1,nsmult
                     if(imultmap(s1) .eq. idxmult(i))exit
                     sm1 = sm1+1
                  enddo
                  soff1 = imultoff(sm1)

! Oh for .xor. in the Fortran standard.
                  if((lsign(sact-soff,sm).or.lsign(sact1-soff1,sm1)) .and. &
                       .not.(lsign(sact-soff,sm) .and. lsign(sact1-soff1,sm1))) then
                     dbadsoc(s,s1,idb) = -1.0_dop*dbadsoc(s,s1,idb)
                     dbadsoc(s1,s,idb) = -1.0_dop*dbadsoc(s1,s,idb)
                  endif
               enddo
            enddo
         endif

      enddo

      if(allocated(sao))deallocate(sao)
      if(allocated(smo))deallocate(smo)
      if(allocated(stover))deallocate(stover)

      return

      end subroutine state_sign

!##########################################################################
! Subroutine to overlap the derivative coupling at one geometry with 
! another at a different geometry so as to check the consistency of the 
! (arbitrary) phase of the states, and to change the phases if
! required.
! GWR 11/21
!##########################################################################
      subroutine state_sign_nact(map,proxidx)

      implicit none

      integer(long), dimension(dbnrec), intent(in) :: map,proxidx

      integer(long) :: iprox,idb,irec,i
      real(dop) :: dcover

      if(lddsoc)then
         message = 'Use of TDDFT with SOCs not implemented yet'
         routine = 'state_sign_nact'
         call errormsg
      endif

! Loop over DB entries.
      do irec = 2,dbnrec
         idb = map(irec)
         iprox = proxidx(idb)

! Check whether this point is being diabatised, hence whether it needs its sign checking.
         if(dbdiabrec(1,idb).eq.0 .or. &    ! New point.
            dbdiabrec(1,idb).ne.iprox.or. & ! Nearest point is new.
            dbdiabrec(2,iprox).eq.1)then    ! Nearest point rediabatised.
            dbdiabrec(1,idb) = iprox
            dbdiabrec(2,idb) = 1
         else
            cycle
         endif

! Overlap derivative couplings between adjacent points, and change the sign if the overlap
! is negative.
         do i = 1,dercpdim
            dcover = dot_product(dbdercp(1:ndofddpes,i,idb),dbdercp(1:ndofddpes,i,iprox))
            if(dcover.lt.0.0_dop) &
                 dbdercp(1:ndofddpes,i,idb) = -dbdercp(1:ndofddpes,i,idb)
         enddo
      enddo

      return

      end subroutine state_sign_nact

!#############################################################################
! Subroutine to evaluate the PES at a given geometry.
! GWR 11/16
!#############################################################################
      subroutine energyeval(xgpoint,weight,energy,order,nrec)

      implicit none

      integer(long), intent(in) :: order,nrec       
      real(dop), dimension(ndofddpes), intent(in)          :: xgpoint
      real(dop), dimension(nddstate,nddstate,nrec), intent(in) :: weight       
      real(dop), dimension(nddstate,nddstate), intent(out) :: energy

      integer(long) :: irec,s,s1

      real(dop) :: covar
      real(dop), dimension(ndofdd) :: gpoint
      real(dop), dimension(nddstate,nddstate) :: vtmp

! Transform the test geometry Cartesian coordinates to normal modes 
! if necessary.
      if(lddtrans.and.ddcoo.eq.1)then
         call ddx2q(xgpoint,gpoint)
      else
         call cpvxd(xgpoint,gpoint,ndofdd)
      endif

! Loop over the DB records to generate the potential.
      vtmp = 0.0_dop
      do irec = 1,nrec

         if(lddtrans.and.ddcoo.eq.1)then
            call gapcovar(gpoint,dbnm(1:ndofdd,irec),covar,ndofdd,&
                 order,gapalpha,gapkfac)
         else
            call gapcovar(gpoint,dbgeo(1:ndofdd,irec),covar,ndofdd,&
                 order,gapalpha,gapkfac)
         endif

         do s = 1,nddstate
            vtmp(s,s) = vtmp(s,s) + weight(s,s,irec)*covar
            do s1 = s+1,nddstate
               vtmp(s,s1) = vtmp(s,s1) + weight(s,s1,irec)*covar
               vtmp(s1,s) = vtmp(s,s1)
            enddo
         enddo         
      enddo

! Copy the temporary energy array to the output.
      energy = vtmp

      return

      end subroutine energyeval

!#############################################################################
! Subroutine to evaluate the SOCs at a given geometry.
! GWR 04/19
!#############################################################################
      subroutine soceval(xgpoint,weight,soc,order,nrec)

      implicit none

      integer(long), intent(in) :: order,nrec       
      real(dop), dimension(ndofddpes), intent(in)          :: xgpoint
      complex(dop), dimension(nddstate,nddstate,nrec), intent(in) :: weight       
      complex(dop), dimension(nddstate,nddstate), intent(out) :: soc

      integer(long) :: irec,s,s1

      real(dop) :: covar
      real(dop), dimension(ndofdd) :: gpoint
      complex(dop), dimension(nddstate,nddstate) :: soctmp

! Transform the test geometry Cartesian coordinates to normal modes 
! if necessary.
      if(lddtrans.and.ddcoo.eq.1)then
         call ddx2q(xgpoint,gpoint)
      else
         call cpvxd(xgpoint,gpoint,ndofdd)
      endif

! Loop over the DB records to generate the potential.
      soctmp = 0.0_dop
      do irec = 1,nrec

         if(lddtrans.and.ddcoo.eq.1)then
            call gapcovar(gpoint,dbnm(1:ndofdd,irec),covar,ndofdd,&
                 order,gapalpha,gapkfac)
         else
            call gapcovar(gpoint,dbgeo(1:ndofdd,irec),covar,ndofdd,&
                 order,gapalpha,gapkfac)
         endif

         do s = 1,nddstate
            soctmp(s,s) = soctmp(s,s) + weight(s,s,irec)*covar
            do s1 = s+1,nddstate
               soctmp(s,s1) = soctmp(s,s1) + weight(s,s1,irec)*covar
               soctmp(s1,s) = soctmp(s,s1)
            enddo
         enddo         
      enddo

! Copy the temporary energy array to the output.
      soc = soctmp

      return

      end subroutine soceval

!#############################################################################
! Subroutine to evaluate the derivative coupling at a given geometry.
! GWR 02/18
!#############################################################################
      subroutine dceval(xgpoint,weight,dercp,order,nrec,dcdim,ndof1)

      implicit none

      integer(long), intent(in) :: order,nrec,dcdim,ndof1
      real(dop), dimension(ndofddpes), intent(in)        :: xgpoint
      real(dop), dimension(ndof1,dcdim,nrec), intent(in) :: weight       
      real(dop), dimension(ndof1,dcdim), intent(out)     :: dercp

      integer(long) :: irec,d,f
      real(dop)     :: covar
      real(dop), dimension(ndofdd)      :: gpoint
      real(dop), dimension(ndof1,dcdim) :: dctmp

! Transform the test geometry Cartesian coordinates to normal modes 
! if necessary.
      if(lddtrans.and.ddcoo.eq.1)then
         call ddx2q(xgpoint,gpoint)
      else
         call cpvxd(xgpoint,gpoint,ndofdd)
      endif

! Loop over the DB records to generate the couplings.
      dctmp = 0.0_dop
      do irec = 1,nrec

         if(lddtrans.and.ddcoo.eq.1)then
            call gapcovar(gpoint,dbnm(1:ndofdd,irec),covar,ndofdd,&
                 order,gapalpha,gapkfac)
         else
            call gapcovar(gpoint,dbgeo(1:ndofdd,irec),covar,ndofdd,&
                 order,gapalpha,gapkfac)
         endif

         do d = 1,dcdim
            do f = 1,ndof1
               dctmp(f,d) = dctmp(f,d) + weight(f,d,irec)*covar
            enddo
         enddo         
      enddo

! Copy the temporary coupling array to the output.
      dercp = dctmp

      return

      end subroutine dceval

!########################################################################
! Test routine to generate a potfit database for the malonaldehyde PES
! along modes 1/12/18 centred at the TS.
! GWR 09/18
!########################################################################
      subroutine potfit_malon(ort)

      implicit none
      real(dop), dimension(ortdim), intent(in)    :: ort

      integer(long) :: i,j,k,imalon
      real(dop), dimension(1,1)       :: v
      real(dop), dimension(ndofddpes) :: gpoint,xgpoint

      imalon = 250
      open(imalon,file='potfit.dat',status='replace')

      gpoint = 0.0_dop
      do k = 1,gdim(18)
         gpoint(18) = ort(zort(18)+k-1)
         do j = 1,gdim(12)
            gpoint(12) = ort(zort(12)+j-1)
            do i = 1,gdim(1)
               gpoint(1) = ort(zort(1)+i-1)

               call ddq2x(gpoint,xgpoint)
               call testdrv(xgpoint,gpoint,v,2)

!               write(imalon,*) gpoint(1),gpoint(12),gpoint(18),v
               write(imalon,*) v(1,1)
            enddo
         enddo
      enddo
      return

      end subroutine potfit_malon

!#####################################################################
! Routine to allocate a DB array of CI vectors.
! GWR 05/19
!#####################################################################

      subroutine alloc_dbcivec

use channels

      implicit none

      integer(long)      :: ierr,d,sm
      character(len=100) :: cerr

      call dealloc_dbcivec

      cerr = 'dbcivec'
      allocate(dbcivec(nsmult,dbmemdim),stat=ierr)
      if (ierr > 0) go to 1000

      do d = 1,dbmemdim
         do sm = 1,nsmult
            write(cerr,'(a,2i6)')'dbcivec%config',d,sm
            allocate(dbcivec(sm,d)%config(maxcivec),stat=ierr)
            if (ierr > 0) go to 1000
            write(cerr,'(a,2i6)')'dbcivec%coeff',d,sm
            allocate(dbcivec(sm,d)%coeff(maxcivec,maxsta),stat=ierr)
            if (ierr > 0) go to 1000
         enddo
      enddo
      return

 1000 write(ilog,'(a)') 'Error allocating memory in ALLOC_DBCIVEC :',cerr
      write(ilog,'(a,i5)') 'Error Code :',ierr
      stop

      end subroutine alloc_dbcivec

!#####################################################################
! Routine to deallocate a DB array of CI vectors.
! GWR 05/19
!#####################################################################

      subroutine dealloc_dbcivec

      implicit none

      integer(long)      :: d,sm

      if(allocated(dbcivec)) then
         do d = 1,size(dbcivec,2)
            do sm = 1,size(dbcivec,1)
               deallocate(dbcivec(sm,d)%config)
               deallocate(dbcivec(sm,d)%coeff)
            enddo
         enddo
         deallocate(dbcivec)
      endif

      return

      end subroutine dealloc_dbcivec

!########################################################################
      end module gapfuncsmod

