!***********************************************************************
!
!  potfit module  "directutil.F"
!
!  contains a number of auxiliary subroutines supporting the direct
!  version of the potfit program.
!
!  NOTE: only some of these routines are actually used!
!
!
!***********************************************************************
      module directutil

use decimal, only: long,dop
use global
use pglobal
use dvrdatmod
use griddatmod
use runpfitmod
use operdef
use logdat

      implicit none
      private
      public :: index2coord,indexmap,xevec2,rdcpd,getvcpd,igpoint2impoint
      contains
      
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
!     transforms the indices of a grid point into its coordinates.
!
!     igpoint (i)
!        vector of ndof integers representing the indeces of a grid point.
!     ort (i)
!        vector containing the coordinates of the grid points of every
!        degree of freedom
!     gpoint (o)
!        vector of ndof reals representing the coordinates of the grid
!        point described by igpoint.
!
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine index2coord(igpoint,ort,gpoint)

      implicit none

      integer(long)                              :: f
      integer(long), dimension(ndof), intent(in) :: igpoint
      real(dop), dimension(ortdim), intent(in)   :: ort
      real(dop), dimension(ndof), intent(out)    :: gpoint

      do f=1,ndof
         gpoint(f)=ort(zort(f)+igpoint(f)-1)
      enddo

      return
      end subroutine index2coord


     subroutine igpoint2impoint(igpoint,impoint)
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
!     transforms the indices of a grid point into indices of a mode point
!
!     igpoint (i)
!        vector of ndof integers representing the indices of a grid point.
!     impoint (o)
!        vector of nmode integers representing the indices of the
!        mode grid point corresponding to igpoint.
!
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      implicit none

      integer igpoint(ndof),impoint(nmode)
      integer f,m,n,irow

      do m=1,nmode
         impoint(m)=1
         irow=1
         do n=1,nspfdof(m)
            f=spfdof(n,m)
            impoint(m)=impoint(m)+(igpoint(f)-1)*irow
            irow=irow*gdim(f)
         enddo
      enddo

      return
      end subroutine igpoint2impoint



!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
!     copies row `row' of length `rowlength' from the eigenvectormatrix 
!     evec(zevec(f)) into column `f' of evmat. 
!
!     f (i)
!     row (i)
!     evec (i)
!     rowlength (i)
!     evmat (o)
!
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine xevec(f,row,evec,rowlength,evmat)

      implicit none

      integer(long)                                   :: ci,evoffset,inc
      integer(long), intent(in)                       :: f,row,rowlength
      real(dop), dimension(evecdim), intent(in)       :: evec
      real(dop), dimension(maxgdim,ndof), intent(out) :: evmat
      

      inc=gdim(f)
      evoffset=row+zevec(f)-1
      do ci=1,rowlength
         evmat(ci,f)=evec(evoffset)
         evoffset=evoffset+inc
      enddo
         
      return
      end subroutine xevec

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
!     copies row `row' of length `rowlength' from the eigenvectormatrix 
!     evec into vector vec.
!
!     collength (i)
!     row (i)
!     evec (i)
!     rowlength (i)
!     vec (o)
!
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine xevec2(row,evec,collength,rowlength,vec)

      implicit none

      integer(long)                                         :: ci
      integer(long), intent(in)                             :: collength,row,rowlength
      real(dop), dimension(collength), intent(out)          :: vec
      real(dop), dimension(collength,collength), intent(in) :: evec

      do ci=1,rowlength
         vec(ci)=evec(row,ci)
      enddo
         
      return
      end subroutine xevec2


!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
!     maps the indey j to jj, j->jj.
!     j is the 1D index of V in primitive order and where
!     jj is the 1D index of V in mode order.
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine indexmap(j,jj)

      implicit none

      integer(long)              :: k,g,f
      integer(long), intent(in)  :: j
      integer(long), intent(out) :: jj

      k=j-1
      jj=1
      do f=ndof,1,-1
         g=k/vgpdim(f)
         k=k-g*vgpdim(f)
         jj=jj+g*vgmdim(f)
      enddo

      return
      end  subroutine indexmap

!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

!!!!! This subroutine reads the cpd file and write it to the array evec

      subroutine rdcpd(inpot,loned,oned,evec,cpdrank,pevec)

      implicit none

      integer(long) ::  inpot,f,p,n,m,g,ierr,cpdrank
      integer(long) ::  pevec(cpdrank*nmode+1)
      real(dop)     ::  coeff,evec(evecdim),oned(oneddim)
      logical       ::  loned(ndof)
         
!-----------------------------------------------------------------------
! read one dimensional potentials (if any)
!-----------------------------------------------------------------------
      do f=1,ndof
         if (loned(f)) then
            read(inpot,iostat=ierr) (oned(zort(f)+g-1),g=1,gdim(f))
            if (ierr.ne.0) then
               routine='rdcpd'
               message='error on reading one dimensional potential ' &
                  //'from cpd file'
               call errormsg
            endif
         endif
      enddo

!----------------------------------------------------------------------
! read cpd terms
!---------------------------------------------------------------------
      pevec(1)=1
      n=0
      do p=1,cpdrank
         read(inpot,err=40) coeff

         do m=1,nmode
            n=n+1
            read(inpot,err=40) (evec(pevec(n)-1+g),g=1,subdim(m))

            ! multiply the first term with the coeff
            if (m.eq.1) then
               do g=1,subdim(m)
                  evec(pevec(n)-1+g) = evec(pevec(n)-1+g)*coeff
               end do
            end if
            pevec(n+1)=pevec(n)+subdim(m)
         enddo

      enddo

      return

   40 routine='rdcpd'
      write(6,'(a,i6)') 'reading error n =',n
      message = 'Error reading cpdfile: '
      call errormsg

      end subroutine rdcpd

!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

!!!!! This subroutine computes an energy point form a cpd in array evec

      subroutine getvcpd(impoint,evec,pevec,evecdim,cpdrank,nmode,x)

      implicit none

      integer(long) :: evecdim,cpdrank,nmode,p,n,m
      integer(long) :: impoint(nmode),pevec(cpdrank*nmode)
      real(dop)     :: evec(evecdim),x,a

      x=0.d0
      n=0
      do p=1,cpdrank
         a=1.d0
         do m=1,nmode
            n=n+1
            a=a*evec(pevec(n)+impoint(m)-1)
         enddo
         x=x+a
      enddo

      return
      end subroutine getvcpd


      end module directutil


