!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      module vpotpars

use decimal, only:long,dop
use logdat
use op1lib,  only: zerovxi
use versions, only: progname,progver

      implicit none
      private
      public :: writevpotpars,skipvpotpars,readvpotinfo,readvpotpars,&
                cpprojinput,cppinput
      contains

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
!     writes parameters held in memory to the i/o channel `unit' which
!     is connected to the file of name `fname'. p.ne.0 signals that
!     the info to be written corresponds to the vpot file which is
!     created by projector number p, instead of the system's vpot file.
!
!     unit (i)
!     fname (i)
!     p(i)
!
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      
      subroutine writevpotpars(unit,fname,p)

use global
use pglobal
use dvrdatmod
use griddatmod
use operdef
use runpfitmod

use iodvrdef, only: wrdvrdef

      implicit none

      integer(long), external      :: log2int,slen
      integer(long), intent(in)    :: unit,p
      integer(long)                :: flength,hl
      character(len=c5)            :: opts
      character(len=*), intent(in) :: fname
      character(len=80)            :: buffer
      character(len=c2)            :: wrtag

      flength=slen(fname)
      
      filever(unit)=progver
      write(unit,err=999) progver

      call wrdvrdef(unit)

! start section
      wrtag= '<start vpotpars>'
      write(unit) wrtag

      call fpstartup(buffer)
      wrtag= '<creation date>'
      write(unit) wrtag
      write(unit,err=999) buffer

      wrtag= '<vpot title>'
      write(unit) wrtag
      if (p.eq.0) then
         call cppinput(unit,fname)
      else
         call cpprojinput(unit,p)
      endif

      hl=hopdim1+1
      wrtag= '<srflabel>'
      write(unit) wrtag
      write(unit,err=999) hoplab(hl)

      opts=' '
      if (npesopt.eq.1) then
         if (nopts(1).gt.0) then
            opts=pesopts(1)(1:zpesopts(nopts(npesopt),1))
         endif
      endif
      wrtag= '<srfopts>'
      write(unit) wrtag
      write(unit,err=999) opts

      wrtag= '<lsp>'
      write(unit) wrtag
      write(unit,err=999) log2int(lsp)

      wrtag= '<pesmin>'
      write(unit) wrtag
      write(unit,err=999) pesmin
      wrtag= '<pesmax>'
      write(unit) wrtag
      write(unit,err=999) pesmax

      wrtag= '<lcspot>'
      write(unit) wrtag
      write(unit,err=999) log2int(lcspot)
      wrtag= '<jtot>'
      write(unit) wrtag
      write(unit,err=999) jtot
      wrtag= '<jbf>'
      write(unit) wrtag
      write(unit,err=999) jbf
      wrtag= '<csmass>'
      write(unit) wrtag
      write(unit,err=999) csmass
      
! end section
      wrtag= '<end vpotpars>'
      write(unit) wrtag

      return
  999 continue
      routine='writevpotpars'
      message='cannot write '//wrtag(1:slen(wrtag))//' to '//fname(1:flength)
      call errormsg

      end subroutine writevpotpars

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
!     rewinds the file `fname' opened for reading via channel `unit' and
!     reads the head of this file which contains the parameters
!     describing the potential energy surface storred in the file.
!
!     unit (i)
!     fname (i)
!     progver1 (o)
!     label1 (o)
!     opts1 (o)
!     lsp1 (o)
!     lftp1 (o)
!     pdofft1 (o)
!     pesmin1,pesmax1 (o)
!     lcspot1,jtot1,jbf1,csmass1 (o)
!
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine readvpotinfo(unit,progver1,label1,opts1,lsp1,&
                              lftp1,pdofft1,pesmin1,pesmax1,chkdvr,&
                              lcspot1,jtot1,jbf1,csmass1,fdvr)

use global
use maxv, only:maxdim
use channels

use iodvrdef, only:rddvrdef

      implicit none

      character(len=c2), intent(out) :: label1
      character(len=c5), intent(out) :: opts1
      real(dop), intent(out)         :: pesmin1,pesmax1
      real(dop), intent(out)         :: progver1
      real(dop), intent(out)         :: jtot1,jbf1,csmass1
      integer(long), intent(in)      :: unit
      integer(long), intent(out)     :: pdofft1
      integer(long)                  :: ierr,chkdvr,ndof1
      logical(kind=4), intent(out)   :: lsp1,lcspot1,lftp1
      logical(kind=4)                :: lwrout

      integer(long), dimension(maxdim), intent(out) :: fdvr


! attempt to read file version. If not first record in file,
! assume filever of 8.4000 (the file version was incorrectly placed
! after dvrinfo until 8.3017). The true version will be returned
! by readvpotpars.
!
      rewind(unit)
      read(unit,iostat=ierr) filever(unit)
      if (ierr .gt. 0) then
         filever(unit) = 8.4000_dop
         rewind(unit)
      else if (ierr .lt. 0) then
         write(message,'(a,f8.5)') &
           'Vpot file empty'
         call errormsg
      endif

      if (filever(unit) .gt. progver) then
         write(message,'(a,f8.5)') &
           'Vpot file newer than program. Use version :',filever(unit)
         call errormsg
      endif

!     vpot file is open: read dvr and surface parameters

! --- read dvr definition
      chkdvr=6
      call zerovxi(chkerr,nerr)
      call rddvrdef(unit,chkdvr,ndof1,fdvr)
      if(chkerr(1).ne.0) then
         routine='checkvpot rddvrdef'
         message='DVR error on reading from '//vname(1:vlength)
         call errormsg
      endif

! --- read vpot parameters
      lwrout = .false.
      call readvpotpars(unit,vname,progver1,label1,opts1,lsp1,&
                        lftp1,pdofft1,pesmin1,pesmax1,&
                        lcspot1,jtot1,jbf1,csmass1,lwrout)
      filever(unit) = progver1

      return
      end subroutine readvpotinfo

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
!     rewinds the file `fname' opened for reading via channel `unit' and
!     reads the head of this file which contains the parameters
!     describing the potential energy surface storred in the file.
!
!     unit (i)
!     fname (i)
!     progver1 (o)
!     label1 (o)
!     opts1 (o)
!     lsp1 (o)
!     lftp1 (o)
!     pdofft1 (o)
!     pesmin1,pesmax1 (o)
!     lcspot1,jtot1,jbf1,csmass1 (o)
!
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine readvpotpars(unit,fname,progver1,label1,opts1,lsp1,&
                              lftp1,pdofft1,pesmin1,pesmax1,&
                              lcspot1,jtot1,jbf1,csmass1,lwrout)

use global
use channels

      implicit none

      character(len=c2), intent(out) :: label1
      character(len=c5), intent(out) :: opts1
      character(len=*), intent(in)   :: fname
      real(dop), intent(out)         :: pesmin1,pesmax1
      real(dop), intent(out)         :: progver1
      real(dop), intent(out)         :: jtot1,jbf1,csmass1
      integer(long), intent(in)      :: unit
      integer(long), intent(out)     :: pdofft1
      integer(long), external        :: slen
      logical(kind=4), intent(in)    :: lwrout
      logical(kind=4), intent(out)   :: lsp1,lcspot1,lftp1
      logical(kind=4), external      :: int2log

!
! read file depending on version
!
      if (filever(unit) .lt. 11.0_dop) then
         call readvpotpars_pre11(unit,fname,progver1,label1,opts1,lsp1,&
                              lftp1,pdofft1,pesmin1,pesmax1,&
                              lcspot1,jtot1,jbf1,csmass1,lwrout)

      else
         lftp1 = .false.
         pdofft1=0
         call readvpotpars11(unit,fname,progver1,label1,opts1,lsp1,&
                             pesmin1,pesmax1,lcspot1,jtot1,jbf1,csmass1,&
                             lwrout)
      endif

      return
      end subroutine readvpotpars

!#######################################################################

      subroutine readvpotpars11(unit,fname,progver1,label1,opts1,lsp1,&
                              pesmin1,pesmax1,lcspot1,jtot1,jbf1,csmass1,&
                              lwrout)

use global
use channels

      implicit none

      character(len=c2), intent(out) :: label1
      character(len=c5), intent(out) :: opts1
      character(len=*), intent(in)   :: fname
      real(dop), intent(out)         :: pesmin1,pesmax1
      real(dop), intent(out)         :: progver1
      real(dop), intent(out)         :: jtot1,jbf1,csmass1
      integer(long), intent(in)      :: unit
      integer(long)                  :: ierr,ilsp1,ilcspot1,&
                                        flength,lcount,i,ilbl,jlbl
      integer(long), external        :: slen
      logical(kind=4), intent(in)    :: lwrout       ! extra output written 
      logical(kind=4), intent(out)   :: lsp1,lcspot1
      logical(kind=4), external      :: int2log
      character(len=80)              :: buffer
      logical(kind=4), save          :: lwritten = .false.
      logical(kind=4)                :: lfound
      character(len=c2)              :: rdtag

      flength=slen(fname)
      progver1=filever(unit)

      ilbl=0
      jlbl=0
!-----------------------------------------------------------------------
! read down to vpotpars
!-----------------------------------------------------------------------
      rewind(unit)
      lfound=.false.
      do
  10     continue
         read(unit,iostat=chkerr(1),err=10,end=100) rdtag
         if (rdtag(1:1) .ne. '<') cycle

         ilbl=2
         jlbl=index(rdtag,'>')-1
         if (rdtag(ilbl:jlbl) .eq. 'start vpotpars') then
            lfound=.true.
            exit
         endif

      enddo

  100 continue
      if (.not. lfound) then
         routine='readvpotpars11'
         message='cannot find vpotpars in '//fname(1:flength)
         call errormsg
      endif

!-----------------------------------------------------------------------
! read title and write to log file
!-----------------------------------------------------------------------
      do
  20     continue
         read(unit,iostat=chkerr(1),err=20,end=200) rdtag
         if (rdtag(1:1) .ne. '<') cycle

         ilbl=2
         jlbl=index(rdtag,'>')-1
         if (rdtag(ilbl:jlbl) .eq. 'creation date') then
            if (.not. lwritten .and. progname(1:6) .ne. 'chnpot') then
               read(unit,err=999) buffer
               write(ilog,'(/,a)') '**** Reading VPOT file ****'
               write(ilog,'(a,f9.5)') 'File version:', progver1
               write(ilog,'(/,a)') 'Created on:'
               write(ilog,'(a)') trim(buffer)
               call myflush(ilog)
            endif

         else if (rdtag(ilbl:jlbl) .eq. 'vpot title') then
            if (.not. lwritten .and. progname(1:6) .ne. 'chnpot') then
               read(unit,iostat=ierr) lcount
               write(ilog,'(a)') 'vvvvvvvvvvvvvvvvvv'
               do i=1,lcount
                  read(unit,iostat=ierr) buffer
                  write(ilog,'(a)') trim(buffer)
               enddo
               write(ilog,'(a)') '^^^^^^^^^^^^^^^^^^'
               call myflush(ilog)
            endif
            lwritten=.true.
      
         else if (rdtag(ilbl:jlbl) .eq. 'srflabel') then
            read(unit,err=999) label1
         else if (rdtag(ilbl:jlbl) .eq. 'srfopts') then
            read(unit,err=999) opts1
         else if (rdtag(ilbl:jlbl) .eq. 'lsp') then
            read(unit,err=999) ilsp1
            lsp1 = int2log(ilsp1)
         else if (rdtag(ilbl:jlbl) .eq. 'pesmin') then
            read(unit,err=999) pesmin1
         else if (rdtag(ilbl:jlbl) .eq. 'pesmax') then
            read(unit,err=999) pesmax1
         else if (rdtag(ilbl:jlbl) .eq. 'lcspot') then
            read(unit,err=999) ilcspot1
            lcspot1 = int2log(ilcspot1)
         else if (rdtag(ilbl:jlbl) .eq. 'jtot') then
            read(unit,err=999) jtot1
         else if (rdtag(ilbl:jlbl) .eq. 'jbf') then
            read(unit,err=999) jbf1
         else if (rdtag(ilbl:jlbl) .eq. 'csmass') then
            read(unit,err=999) csmass1
         else if (rdtag(ilbl:jlbl) .eq. 'end vpotpars') then
            exit

         endif

      enddo
  200 continue

! Extra writing to log file if asked for (e.g. from rdnatpot)
      if (lwrout) then 
         write(ilog,'(/2a)') ' PES-label = ', trim(label1)
         write(ilog,'(3a)') ' PES-options =  {', trim(opts1),'}'
         write(ilog,'(a,1pe13.5,a,1pe13.5)') &
                    ' pesmin =', pesmin1,' ,  pesmax =',pesmax1
         If(lcspot1) then
            write(ilog,'(/a,l2,a)') ' lcspot =',lcspot1, &
                '  (true if CS centr. pot. included)'
            write(ilog,'(a,2f6.2,1pe13.5)') ' jtot,jbf,csmass:', &
                                         jtot1,jbf1,csmass1
         endif
         write(ilog,*)
      endif

      return
  999 continue
      routine='readvpotpars'
      message='cannot read '//rdtag(ilbl:jlbl)//' from '//fname(1:flength)
      call errormsg

      end subroutine readvpotpars11

!#######################################################################

      subroutine readvpotpars_pre11(unit,fname,progver1,label1,opts1,lsp1,&
                              lftp1,pdofft1,pesmin1,pesmax1,&
                              lcspot1,jtot1,jbf1,csmass1,lwrout)

use global
use channels

      implicit none

      character(len=c2), intent(out) :: label1
      character(len=c5), intent(out) :: opts1
      character(len=*), intent(in)   :: fname
      real(dop), intent(out)         :: pesmin1,pesmax1
      real(dop), intent(out)         :: progver1
      real(dop), intent(out)         :: jtot1,jbf1,csmass1
      integer(long), intent(in)      :: unit
      integer(long), intent(out)     :: pdofft1
      integer(long)                  :: ierr,ilsp1,ilcspot1,liftp,ilinterpol1,&
                                        flength,lcount,i
      integer(long), external        :: slen
      logical(kind=4), intent(in)    :: lwrout       ! extra output written 
      logical(kind=4), intent(out)   :: lsp1,lcspot1,lftp1
      logical(kind=4), external      :: int2log
      character(len=80)              :: buffer
      logical(kind=4), save          :: lwritten = .false.

      flength=slen(fname)

!-----------------------------------------------------------------------
! read program version
! This was in wrong place (after dvrinfo) until 8.3017 and 8.4.4. 
! Hence this fix.
!-----------------------------------------------------------------------
      if ( (filever(unit).lt.8.3017_dop) .or. ( (filever(unit).ge.8.4_dop)&
           .and. (filever(unit).lt.8.4004_dop) ) ) then
         read(unit,iostat=ierr) progver1
         if (ierr.ne.0) then
            routine='readvpotpars'
            message=&
              'cannot read program version from '//fname(1:flength)
            call errormsg
         endif
      else
         progver1=filever(unit)
      endif

      read(unit,iostat=ierr) buffer
      if (ierr.ne.0) then
         routine='readvpotpars'
         message='cannot read date/time from '//fname(1:flength)
         call errormsg
      endif

      if (.not.lwritten) then
         if (progname(1:6) .ne. 'chnpot') then
            write(ilog,'(/,a)') '**** Reading VPOT file ****'
            write(ilog,'(a,f9.5)') 'File version:', progver1
            write(ilog,'(a)') 'Created on:'
            write(ilog,'(a)') trim(buffer)
!CC         write(ilog,'(a)') 'potfit input file:'
!CC         write(ilog,'(a)') 'vvvvvvvvvvvvvvvvvv'
         endif
         read(unit,iostat=ierr) lcount
         if (ierr.ne.0) then
            routine='readvpotpars'
            message='cannot read line count from '//fname(1:flength)
            call errormsg
         endif
         do i=1,lcount
            read(unit,iostat=ierr) buffer
!CC         write(ilog,'(a)') buffer
         enddo
!CC      write(ilog,'(a)') '^^^^^^^^^^^^^^^^^^'
         lwritten=.true.
         call myflush(ilog)
      else
!-----------------------------------------------------------------------
! skip over potfit input file
!-----------------------------------------------------------------------
         read(unit,iostat=ierr) lcount
         if (ierr.ne.0) then
            routine='readvpotpars'
            message='cannot read line count from '//fname(1:flength)
            call errormsg
         endif
         do i=1,lcount
            read(unit,iostat=ierr) buffer
            if (ierr.ne.0) then
               routine='readvpotpars'
               message='cannot read input file from '//fname(1:flength)
               call errormsg
            endif
         enddo
      endif
      
      read(unit,iostat=ierr) label1
      if (ierr.ne.0) then
         routine='readvpotpars'
         message='cannot read surface label from '//fname(1:flength)
         call errormsg
      endif

      read(unit,iostat=ierr) opts1
      if (ierr.ne.0) then
         routine='readvpotpars'
         message='cannot read surface options from '//fname(1:flength)
         call errormsg
      endif

      read(unit,iostat=ierr) ilsp1
      lsp1 = int2log(ilsp1)
      if (ierr.ne.0) then
         routine='readvpotpars'
         message='cannot read lsp from '//fname(1:flength)
         call errormsg
      endif
 
      if( progver1 .ge. 8.3013_dop) then

! --- lftp and pdofft are obsolete, they are kept for concistency only.
! --- lftp=.false., pdofft=0.
         read(unit,iostat=ierr) liftp,pdofft1
         lftp1=int2log(liftp)
         if (ierr.ne.0) then
            routine='readvpotpars'
            message='cannot read lftp/pdofft  from '//&
                     fname(1:flength)
            call errormsg
         endif

      else

         read(unit,iostat=ierr) ilinterpol1
         if (ierr.ne.0) then
            routine='readvpotpars'
            message='cannot read linterpol from '//fname(1:flength)
            call errormsg
         endif

      endif

      read(unit,iostat=ierr) pesmin1,pesmax1
      if (ierr.ne.0) then
         routine='readvpotpars'
         message='cannot read pesmin/pesmax from '//fname(1:flength)
         call errormsg
      endif

      read(unit,iostat=ierr) ilcspot1,jtot1,jbf1,csmass1
      lcspot1 = int2log(ilcspot1)
      if (ierr.ne.0) then
         routine='readvpotpars'
         message='cannot read lcspot/jtot/jbf/csmass from '//&
            fname(1:flength)
         call errormsg
      endif

! Extra writing to log file if asked for (e.g. from rdnatpot)
      if (lwrout) then 
         write(ilog,'(/2a)') ' PES-label = ', trim(label1)
         write(ilog,'(3a)') ' PES-options =  {', trim(opts1),'}'
         write(ilog,'(a,1pe13.5,a,1pe13.5)') &
                    ' pesmin =', pesmin1,' ,  pesmax =',pesmax1
         If(lcspot1) then
            write(ilog,'(/a,l2,a)') ' lcspot =',lcspot1, &
                '  (true if CS centr. pot. included)'
            write(ilog,'(a,2f6.2,1pe13.5)') ' jtot,jbf,csmass:', &
                                         jtot1,jbf1,csmass1
         endif
         write(ilog,*)
      endif

      return
      end subroutine readvpotpars_pre11

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
!     dummy routine used to rewind the vpot file connected to channel
!     `unit' and to skip over the parameters defining the potential
!     energy surface storred in the file. on exit, the vpot file
!     is positioned to the number `nitems' of potential data elements
!     followed by the data themselves.
!
!     unit (i)
!
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine skipvpotpars(unit)

use global
use pglobal
use maxv, only:maxdim
use griddatmod
use combined

      implicit none
      
      integer(long), intent(in)              :: unit
      character(len=c5)                      :: label,opts
      logical(kind=4)                        :: lsp1,lcspot1,lftp1
      real(dop)                              :: pesmin1,pesmax1,progver1,jtot1,jbf1,csmass1
      integer(long)                          :: ierr,pdofft1,chkdvr
      integer(long), dimension(maxdim)       :: fdvr

      rewind(unit,iostat=ierr)
      if (ierr.ne.0) then
         routine='skipvpotpars'
         message='cannot rewind '//vname(1:vlength)
         call errormsg
      endif

      chkdvr=6
      call readvpotinfo(unit,progver1,label,opts,lsp1,&
                        lftp1,pdofft1,pesmin1,pesmax1,chkdvr,&
                        lcspot1,jtot1,jbf1,csmass1,fdvr)

      return
      end subroutine skipvpotpars

!***********************************************************************
!
! SUBROUTINE CPPROJINPUT
!
! Writes some information about the projector #p from the input file
! to the file-channel "unit".
! Called from: writevpotpars
!
!***********************************************************************

      subroutine cpprojinput(unit,p)

use global
use pglobal
use dvrdatmod
use griddatmod

      implicit none

      integer(long), intent(in) :: unit,p
      integer(long)             :: lcount,len,q
      character(len=80)         :: buffer

!     see CPPINPUT for why this format of output is chosen
      if (p.ge.0) then
         lcount = 1+nprojdof(p)
         write(unit) lcount
         len = index(projlab(p),' ')-1
         write(buffer,'(3a)') 'Projector "',projlab(p)(1:len),'"'
         write(unit) buffer
         do q=1,nprojdof(p)
            write(buffer,'(4a)') '  ',modelabel(projdof(p,q))
            write(unit) buffer
         enddo

      else
         lcount = 1+nprojdof(1)
         write(unit) lcount
         write(buffer,'(a)') 'Projection Error'
         write(unit) buffer
         write(buffer,'(a)') '--- This space for rent ---'
         do q=1,nprojdof(1)
            write(unit) buffer
         enddo
      endif

      return
      end subroutine cpprojinput  ! of cpprojinput

!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
!     copies the input file to the unformatted file `outfile' connected to
!     channel `outunit' for documentation purposes. the file is preceded by
!     the number of the input file's lines. we assume that channel `iin' is
!     already connected to the input file.
!     
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine cppinput(outunit,outfile)

use global
use pglobal
use dvrdatmod
use griddatmod
use operdef
use runpfitmod

      implicit none

      integer(long), external      :: slen
      integer(long)                :: lcount,ierr,sl,i
      integer(long), intent(in)    :: outunit
      character(len=200)           :: line
      character(len=*), intent(in) :: outfile

      routine = 'cppinput'
      sl=slen(outfile)

! --- write number of lines

      lcount = 4
      write(outunit,iostat=ierr) lcount
      if (ierr.ne.0) then
         message = 'cannot write line count to '//outfile(1:sl)
         call errormsg
      endif

! --- write information

      write(line,'(a,i3,a,20i4)') ' contracted mode:',modc, &
                 ',  nat.pots:',(potdim(i),i=1,min(ndof,20))
      write(outunit,iostat=ierr) line(1:80)
      write(line,'(a,e12.4,a,18i3)') ' V-cut =', pesmax*27.2114,&
       ' eV,  oned:',(onedipot(i),i=1,min(ndof,18))
      write(outunit,iostat=ierr) line(1:80)
      write(line,'(a,e12.4,a,e12.4,a)') ' C-Weights: V-max =', &
           iwvmax*27.2114,' eV,  V-min =',iwvmin*27.2114,' eV'
      write(outunit,iostat=ierr) line(1:80)
      write(line,'(a,11(1x,a))') ' Modelabels:', &
          (modelabel(i)(1:slen(modelabel(i))),i=1,min(ndof,11))
      write(outunit,iostat=ierr) line(1:80)
      if (ierr.ne.0) then
         message = 'i/o error on writing to '//outfile(1:sl)
         call errormsg
      endif

      return
      end subroutine cppinput

      end module vpotpars


