!#######################################################################
! Note: beginning of what needs to be improved.
! - eckart
! - prewash
!
! contains: dddb_rd,distdb,getdbnrec,eckart,prewash,rdrefdb,rmcom,
!           ddq2x,ddx2q,ddf2q,ddpes2q,ddpes2x,ddfc2q,
!           shiftdd,dddb_wr
!#######################################################################

      module dd_db

use decimal, only: long, sip, dop
use keyunitsmod
use logdat
use op1lib,  only: zerovxd,cpvxd,cpvxz,cpmxd,zerovxi,cpqxd1,zeromxd,cpvxi
use mmlib, only: qqtxdd
use channels
use versions

use global
use dvrdatmod
use griddatmod
use psidef
use operdef
use dirdyn ! the_db
use openmpmod
use symoplib
use timing
use gwplib, only: center
use readqc, only: getinam,getmass
use lalib, only: simtranbd,simtranbdz

use omp_lib
use db_sql
use sqlite
use lineq
#ifdef OMP
use omp_lib
#endif 


      implicit none                                                    

      !integer(long)     :: nrec ! number of records in the database
      real(dop)          :: rad_conf   ! confidence radius used during interpolation
      real(dop)          :: shep_norm  ! Norm used during interpolation
      !$omp threadprivate(shep_norm)               

#ifdef OMP
! to allow timing on threads
      integer(long), dimension(:), allocatable, save :: tid_dbread   
#endif 
      
      private
      public :: ddpes2x,ddpes2q,ddf2q,ddf2x,ddx2q,ddq2x,ddq2x_diff,ddv2p,&
                rdrefdb,prewash,eckart,getdbnrec,distdb,&
                dddb_rd,prewash_reord,eckart_reord,&
                eadiab_rd,hessian_update,ddmo_rd, list_init,list_insert, &
                rdrefdbmem,hessian_update2,dddb_wrmem,dddb_wr1,hessian_update1,&
                hestrans,preparedb,gapwgt,gapvar,ddx2q_diff,&
                dddb_wrmem_gp,dddb_gp,distdb_gp,sort,distdb_nopar,&
                maxatdist,shiftdd,ddfc2q,mksymcent,dbfinalise,&
                dddb_rd_db0,dddb_rd_local,gpdbupdate,gwpcentres,prepsym,distdb1

      public :: rad_conf,shep_norm
!unused
      public :: rmcom, dbwarnmsg, findrad_gp

      contains

!#######################################################################
!
! Prepare DB files for a DD calculation
! Load DB into memory if required
!
! modus : 0   do not read DB into memory
!         1   read into memory for propagation
!         2   read into memory for rediabatisation
!         3   read into memory for analysis
! icheck : 0  Check against system data in memory
!        : 1  Do not check against system data in memory
!
!#######################################################################

      subroutine preparedb(modus,icheck)

      implicit none

      integer(long), intent(in)              :: modus,icheck
      integer(long)                          :: i,nrec,irec,n,n1
      integer(long)                          :: nddstate0,natm0,nbasis0
      integer(long)                          :: iatiso
      integer(long), external                :: slen
      character(len=c5)                      :: filename

      logical(kind=4)                        :: check,checka,checkb,&
                                                checkold
      real(dop), parameter                   :: angst = 0.52917725_dop

      integer(long)                          ::stamp 
      !NEWSQL

      integer(long), save                          :: tid = 0 ! timer ID

!-----------------------------------------------------------------------
! If not using DB, fill atnam and iatnam arrays (needed to
! set up QC input files) and return
!-----------------------------------------------------------------------
      if (.not. lddwrdb .and. .not. lddrddb) then
         i=0
         do n=1,natm
            do n1=1,3
               i=i+1
               call getinam(xatnam(n),iatsym(i),iatiso,ilog)
            enddo
         enddo
         atnam(:) = xatnam(:)
     
         return
      endif

      call get_timer(tid, 'PrepareDB')
      call start_timer(tid)

!-----------------------------------------------------------------------
! name for SQL DB file
!-----------------------------------------------------------------------
      sqldbfile=ddname(1:ddlaenge)//'/database.sql'

      lsqldb = .false.
      inquire(file=sqldbfile,exist=lsqldb)

!-----------------------------------------------------------------------
! look for old style DB
!-----------------------------------------------------------------------
      checka = .false.
      filename=ddname(1:ddlaenge)//'/geo.dba'
      inquire(file=filename,exist=checka)
      checkb = .false.
      filename=ddname(1:ddlaenge)//'/geo.db'
      inquire(file=filename,exist=checkb)
      checkold = checka .or. checkb
      check = .false.
      filename=ddname(1:ddlaenge)//'/refdb.dat'
      inquire(file=filename,exist=check)

!-----------------------------------------------------------------------
! A refdb.dat file is no longer used to initialise a calculation, but 
! may have been left by old versions even if there is no DB. Get user 
! to check and remove it.
!-----------------------------------------------------------------------
      if (check .and. .not. checkold) then
         routine = 'Preparedb'
         message = &
          'There is a refdb.dat file and no old DB. Please delete it.'
         call errormsg
      endif

!-----------------------------------------------------------------------
! Set SQL DB log (error) output to ilog
!-----------------------------------------------------------------------
      dbsqllog = ilog

!-----------------------------------------------------------------------
! Now, we check if there is:
! 1) An old style DB and an SQL DB?  Fail.
! 2) An old style DB?  Convert old style DB to SQL with convdbtosql. 
!
! 3) Nothing. Open new SQL DB and write new refdb entry.
! 4) Already an SQL DB?  Open it.
! 5) Something else.  Fail and cry.  Note fail condition at start of routine.
!
! So we finish with failure or an open SQL DB in global the_db (dirdyn)
!-----------------------------------------------------------------------
      nrec = 0
      if (checkold.and.lsqldb) then
         routine = 'preparedb'
         message = 'Both SQL and old DB present ! Sort out which.'
         call errormsg

      else if (checkold .and. .not. lsqldb) then
         routine = 'preparedb'
         message = 'Flat file DB present. Convert to SQL (use convdbtosql). '
         call errormsg

      endif

      if (.not. lsqldb) then
         ! open new SQL DB for read-write
         write(ilog,'(/,a)') '*** Opening New SQL DB  ***'
         call setsqldbflags    ! set up flags for what needs to be opened
         call prepsym          ! Prepare symmetry info
         call makeflagarray
         call makeinfoarray(nddstate)
         if (.not.the_db%db_open) &
               call dbopen(sqldbfile,the_db,sqlvers,quanticsver, &
                    infoarray,flagarray, 2)
         
         call dbwrrefdb(the_db,natm, nroot, nddstate, nbasis, dde0, &
                        dimref, nfam, mref, xatnam, xref, dercpdim, &
                        dbpntgp, nsymop, nsocstate, dbsymmap, nddmult, &
                        qcprog, qcmethod, qcbasis, ddtrfile,ddname,ddlaenge)

         ! Translate atom name to atomic symbol for each coordinate and 
         ! store in itasym
         i=0
         do n=1,natm
            do n1=1,3
               i=i+1
               call getinam(xatnam(n),iatsym(i),iatiso,ilog)
            enddo
         enddo

! open symmetry map DB if needed
         if (dbpntgp .gt. 0) then
            lsymdb =.true.
         endif

      else if (lsqldb) then
         ! open existing SQL DB for read-write
         write(ilog,'(/,a)') '*** Opening existing SQL DB  ***'
         if (.not.the_db%db_open) &
               call dbopen(sqldbfile,the_db,sqlvers,quanticsver,&
                    infoarray,flagarray, 1)
         ! get no. records from DB
         call getdbnrec(nrec)
         ! see what's in the DB. 
!Geometries always present.
         lgeodb=.true.
         call dbgetentries(the_db, ilog, lpesdb, lgradb, lhesdb, lapesdb, &
              lagradb, lahesdb, ldipdb, ladipdb, lmocodb, lnactdb, ltransdb, &
              lnmdb, lcasocdb, lsymdb, lcpesdb, lctransdb, lcivecdb, &
              ltemplate, lqcfreq)

         ! read refdb entry. For analysis programs only read dimensions
         if (modus .eq. 3) then
            call rdrefdbmem

         else if (icheck .eq. 1) then
            call rdrefdb 

         else
            nddstate0 = nddstate
            natm0 = natm
            nbasis0 = nbasis

            call rdrefdb

            if (nbasis0 .le. 0) nbasis=nbasis0   ! not defined unless CAS
            if ((nddstate0 .ne. nddstate) .or. (natm0 .ne. natm) .or. &
                (nbasis0 .ne. nbasis)) then
               routine = 'Preparedb'
               write(message(1:72),'(a)') &
                         'DB system does not match that in input.'
               write(message(81:160),'(3(a,i0,a,i0))') &
                         'Nddstate (sys/DB) = ',nddstate0,'/',nddstate, &
                         '  Natm (sys/DB) = ',natm0,'/',natm, &
                         '  Nbasis (sys/DB) = ',nbasis0,'/',nbasis
               call errormsg
            endif
         endif

         ! Translate atom name to atomic symbol for each coordinate and 
         ! store in itasym
         i=0
         do n=1,natm
            do n1=1,3
               i=i+1
               call getinam(atnam(n),iatsym(i),iatiso,ilog)
            enddo
         enddo

      else
         ! oopsie
         routine = 'preparedb'
         message = 'Shouldn''t get here.'
         call errormsg
      endif

!-----------------------------------------------------------------------
! Set array sizes and check consistency
!-----------------------------------------------------------------------
      if (nrec .gt. dbmemdim) then
         routine = 'Preparedb'
         write(message(1:72),'(a)') 'Nrec larger than DBMEMDIM.'
         write(message(80:152),'(a,i8)') &
             'Use maxdbsize = I keyword to increase memory. Minimum: ',nrec
         call errormsg
      endif
         
      if (nrec .gt. 0) then
         write(ilog,'(/,2a)')  'Using old DB :',ddname(1:ddlaenge)
         write(ilog,'(a,i10)') 'NREC         :',nrec
      else
         write(ilog,'(/,2a)')  'Using new DB :',ddname(1:ddlaenge)
      endif
      dbnrec = nrec

!-----------------------------------------------------------------------
! Checks for GB DD
!-----------------------------------------------------------------------
      if (lddmctdh .or. lgausspot) then
         if (lcasocdb) lcsocdb=.true. ! Diabatic SOCs - held in memory

         if (lgradb) then
            routine = 'preparedb'
            message = &
         'Gradients in DB but not used or added to in DDMCTDH / GAUSSPOT run.'
            call warnmsg1
            lgradb = .false.
         endif
         if (lhesdb) then
            routine = 'preparedb'
            message = &
          'Hessians in DB but not used or added to in DDMCTDH / GAUSSPOT run.'
            call warnmsg1
            lhesdb = .false.
          endif
      endif

!-----------------------------------------------------------------------
! Check for SOCs
!-----------------------------------------------------------------------
      if (lddsoc) then
         if (.not. (lcasocdb)) then
            routine = 'preparedb'
            message = 'SOCs requested but no SOC DB table present.'
            call errormsg
         endif
      endif

! Size of Derivative coupling array
      nactdim = nddstate*(nddstate-1)/2

      if(lrediab .and. nrec.eq.0)then
         routine = 'preparedb'
         message = 'Trying to re-diabatise an empty DB'
         call errormsg
      endif
      if ((.not. lddwrdb .and. lddrddb) .and. nrec .eq. 0) then
         routine = 'Preparedb'
         message = 'DB only calculation requested, but DB is empty!'
         call errormsg
      endif

!-----------------------------------------------------------------------
! check adiabatic files present for rediabatisation
!-----------------------------------------------------------------------
      if (lrediab) then
         if (.not. lapesdb) go to 1002
         if (.not. lagradb) go to 1003
         if (.not. lahesdb) go to 1004
         if (.not. lnactdb) go to 1005
         if (lddsoc .and. .not. lcasocdb) go to 1006
     endif
  
!-----------------------------------------------------------------------
! load DB into memory if ldbsave
!-----------------------------------------------------------------------
      if(lapesdb .or. lrediab) dbadener(:,:,:) = 0.0_dop

      if (modus .eq. 3) then
         if (.not. ldbsave) then
            routine = 'Preparedb (ddb_db.f90)'
            message = 'Analysis calculations require DB in memory (dbsave option)'
            call errormsg
         endif

         call alloc_dirdyn_rediab(dbnrec,nddstate,ndofddpes)

         call dbrdentry(the_db,1,dbnrec,dbinterp,stamp )
         call dbrdgeo(the_db,natm,1,dbnrec,dbgeo)
         call dbrdpes(the_db,nddstate,1,dbnrec,dbener)
         if (lcpesdb) call dbrdcpes(the_db,nddstate,1,dbnrec,dbcener)
         call dbrdgra(the_db,nddstate,natm,1,dbnrec,dbgrad)
         call dbrdhes(the_db,nddstate,natm,1,dbnrec,dbhess)
         call dbrdagra(the_db,nddstate,natm,1,dbnrec,dbadgrad)
         call dbrdapes(the_db,nddstate,1,dbnrec,dbadener)
         call dbrdahes(the_db,nddstate,natm,1,dbnrec,dbadhess)
         call dbrdnact(the_db,natm,dercpdim,1,dbnrec,dbdercp)
         if (lmocodb) call dbrdmoco(the_db,nbasis,1,dbnrec,dbmoco)
         if (ladipdb) call dbrdadip(the_db,1,dbnrec,dbaddipmom)
         if (lcasocdb) call dbrdcasoc(the_db,nsocstate,1,dbnrec,dbadsoc)
         if (ltransdb) call dbrdtrans(the_db,nddstate,1,dbnrec,dbtrans)
         if (lctransdb) call dbrdctrans(the_db,nddstate,1,dbnrec,dbtrans)
         if (lnmdb) call dbrdnm(the_db,ndofdd,1,dbnrec,dbnm)
         if (lsymdb) call dbrdsymrep(the_db,nsymop,1,dbnrec,dbsymrep)

      else if (modus .eq. 2) then
         if (.not. ldbsave) then
            routine = 'Preparedb (ddb_db.f90)'
            message = 'Rediab calculations require DB in memory (dbsave option)'
            call errormsg
         endif

         call alloc_dirdyn_rediab(dbnrec,nddstate,ndofddpes)

! If doing a re-diabatisation, only read adiabatic data into memory
         call dbrdentry(the_db,1,dbnrec,dbinterp,stamp)
         call dbrdgeo(the_db,natm,1,dbnrec,dbgeo)
         call dbrdapes(the_db,nddstate,1,dbnrec,dbadener)
         if (lgradb) call dbrdagra(the_db,nddstate,natm,1,dbnrec,dbadgrad)
         if (lhesdb) call dbrdahes(the_db,nddstate,natm,1,dbnrec,dbadhess)
         if (lmocodb) call dbrdmoco(the_db,nbasis,1,dbnrec,dbmoco)
         if (lnactdb) call dbrdnact(the_db,natm,dercpdim,1,dbnrec,dbdercp)
         if (ldipdb) call dbrdadip(the_db,1,dbnrec,dbaddipmom)
         if (lcasocdb) call dbrdcasoc(the_db,nsocstate,1,dbnrec,dbadsoc)
         if (lsymdb) call dbrdsymrep(the_db,nsymop,1,dbnrec,dbsymrep)
         if (lcivecdb) call dbrdcivec(the_db, nsmult, imultblk, 1,dbnrec, &
            dbciconfig,dbcicoeff)

         if (lupdhes .and. .not.(lddmctdh.or.lgausspot)) then
            irec=1
            dbgeoref(1:ndofddpes,irec) = dbgeo(1:ndofddpes,irec)
            dbenref(1:nddstate,1:nddstate,irec) = &
                  dbadener(1:nddstate,1:nddstate,irec)
            dbgraref(1:ndofddpes,1:nddstate,1:nddstate,irec) = &
                 dbadgrad(1:ndofddpes,1:nddstate,1:nddstate,irec)
            dbhesref(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,irec) = &
                 dbadhess(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,irec)
         endif

      else if (modus .eq. 1) then
! for normal propagation, do not need to hold adiabatic data in memory, except NACTs and SOCs
         call dbrdentry(the_db,1,dbnrec,dbinterp,stamp)
         call dbrdgeo(the_db,natm,1,dbnrec,dbgeo)
         call dbrdpes(the_db,nddstate,1,dbnrec,dbener)
         if (lcpesdb) call dbrdcpes(the_db,nddstate,1,dbnrec,dbcener)
         if (lgradb) call dbrdgra(the_db,nddstate,natm,1,dbnrec,dbgrad)
         if (lhesdb) call dbrdhes(the_db,nddstate,natm,1,dbnrec,dbhess)
         if (ldiabdb.and.lapesdb) call dbrdapes(the_db,nddstate,1,dbnrec,dbadener)
         if (ldipdb) call dbrddip( the_db,1,dbnrec,dbdipmom)
         if (lmocodb) call dbrdmoco(the_db,nbasis,1,dbnrec,dbmoco)
         if (lnactdb) call dbrdnact(the_db,natm,dercpdim,1,dbnrec,dbdercp)
         if (ltransdb) call dbrdtrans(the_db,nddstate,1,dbnrec,dbtrans)
         if (lctransdb) call dbrdctrans(the_db,nddstate,1,dbnrec,dbtrans)
         if (lcasocdb) call dbrdcasoc(the_db,nsocstate,1,dbnrec,dbadsoc)
         if (lsymdb) call dbrdsymrep(the_db,nsymop,1,dbnrec,dbsymrep)
         if (lcivecdb) call dbrdcivec(the_db, nsmult, imultblk, 1,dbnrec, &
            dbciconfig,dbcicoeff)

         if (lddmctdh .or. lgausspot) then
            if (lnmdb) call dbrdnm(the_db,ndofdd,1,dbnrec,dbnm)
         endif

! If using Hessian update, copy the first entry to the reference arrays. May 
! need generalising if using multiple or different reference geometries.
         if (lupdhes .and. .not.(lddmctdh.or.lgausspot)) then
            irec=1
            dbgeoref(:,irec) = dbgeo(:,irec)
            dbenref(:,:,irec) = dbener(:,:,irec)
            dbgraref(:,:,:,irec) = dbgrad(:,:,:,irec)
            dbhesref(:,:,:,:,irec) = dbhess(:,:,:,:,irec)
         endif

! Form diabatic SOCs if needed
         if (lcsocdb) then
            do irec=1,dbnrec
               call soctrafo(dbadsoc(:,:,irec),&
                    dbtrans(:,:,irec),&
                    dbctrans(:,:,irec),&
                    dbsoc(:,:,irec),nddstate)
            enddo
         endif

      endif

      call stop_timer(tid)

!-----------------------------------------------------------------------
! set up directories for QChem files
!-----------------------------------------------------------------------
      return
 1002 routine='Preparedb (dd_db.f90)'
      write(message,'(2a)') 'No Adiabatic PES data in DB'
      call errormsg
 1003 routine='Preparedb (dd_db.f90)'
      write(message,'(2a)') 'No Adiabatic Gradient data in DB'
      call errormsg
 1004 routine='Preparedb (dd_db.f90)'
      write(message,'(2a)') 'No Adiabatic Hessian data in DB'
      call errormsg
 1005 routine='Preparedb (dd_db.f90)'
      write(message,'(2a)') 'No Non-Adiabatic Coupling data in DB'
      call errormsg
 1006 routine='Preparedb (dd_db.f90)'
      write(message,'(2a)') 'No Spin-Orbit Coupling data in DB'
      call errormsg
      
      end subroutine preparedb

!#######################################################################

      subroutine soctrafo(adsoc,trans,ctrans,dsoc,nddstate)

      implicit none

      integer(long), intent(in)                  :: nddstate
      real(dop), dimension(:,:), intent(in)      :: trans,ctrans
      complex(dop), dimension(:,:), intent(in)   :: adsoc
      complex(dop), dimension(:,:), intent(out)  :: dsoc
      complex(dop), dimension(nddstate,nddstate) :: tmpsoc

      dsoc(:,:) = 0.0_dop
      if (lsocbas) then
         call simtranbdz(adsoc(1:nddstate,1:nddstate),&   ! Real part of trans
              trans(1:nddstate,1:nddstate),dsoc(1:nddstate,1:nddstate),nddstate)
         call simtranbdz(adsoc(1:nddstate,1:nddstate),&   ! Imag part of trans
              ctrans(1:nddstate,1:nddstate),tmpsoc(1:nddstate,1:nddstate),nddstate)
         dsoc(:,:) = dsoc(:,:) + (0,1.0_dop)*tmpsoc(:,:)

      else 
         call simtranbdz(adsoc(1:nddstate,1:nddstate),&
              trans(1:nddstate,1:nddstate),dsoc(1:nddstate,1:nddstate),nddstate)
      endif

      return
      end subroutine soctrafo

!#######################################################################
! if new DB set flags to open files required.
! ldipdb, lnactdb set in eindirdyn
!
! if rediabatisation, diab files may not exist. OPen all 
! and then check for existence in rediabatisation routine.
!#######################################################################

      subroutine setsqldbflags

      if (dddiab .eq. 0) then
         ldiabdb = .false.
      else
         ldiabdb = .true.
      endif

      lgeodb=.true.
      lpesdb=.true.
      if (lddsoc .and. .not. lsocdegen) lcpesdb=.true.
      if (ldiabdb) then
         lapesdb=.true.
         if (dddiab .ne. 5) lnactdb=.true.
         if (dddiab .eq. 3 .or. dddiab .eq. 4 .or. dddiab .eq. 6) then
            ltransdb=.true.
            if (lsocbas) lctransdb=.true.
         endif
      endif

      if (lddtrans) then
         lnmdb = .true.
      endif

      if (ldipdb) ladipdb = .true.

! see if state multiplicities mean that SOC will be calculated
      if (lddsoc) then
         lcasocdb = .true.
         if (lddmctdh.or.lgausspot) lcsocdb = .true.     ! Diabatic SOCs
      endif

! Need to store MOs?
      if (nbasis .gt. 0) lmocodb = .true.

! CI vector for Procustes diabatisation
      if (dddiab.eq.6) lcivecdb=.true.

! Gausspot does not need gradients
      if (.not. (lddmctdh.or.lgausspot)) then
         lgradb=.true.
         lhesdb=.true.
         if (ldiabdb) then
            lagradb=.true.
            lahesdb=.true.            
         endif
      endif

! Adiabatic with multiple states needs adiabatic data
      if (dddiab .eq. 0 .and. nddstate .gt. 1) then
         lapesdb=.true.
         lagradb=.not.(lddmctdh.or.lgausspot)
         lahesdb=.not.(lddmctdh.or.lgausspot)
      endif

!Check that sym table is made
      dbpntgp = dbpntgp0       ! set dbpntgp from input file
      if (dbpntgp .gt. 0) then
         lsymdb =.true.
      endif

! If running in normal modes, save the QC frequency file used for the 
! transformation
      if (lddtrans .and. ddtrfile .ne. ' ' ) lqcfreq=.true.

      return
      end subroutine setsqldbflags

!#######################################################################

      subroutine prepsym

      implicit none

      integer(long)   :: i,n,n1
      integer(long)   :: iat,iatiso,iau
      real(dop), dimension(:,:), allocatable   :: xgp
      logical(kind=4) :: lfound

!-----------------------------------------------------------------------
! get no. of symmetry operations to be performed
!-----------------------------------------------------------------------
      nsymop = fnsymop(dbpntgp)
      if (nsymop .eq. -99) then
         routine = 'Prepsym (dd_db.f90)'
         write(message,'(a,i5)') 'Unknown point group : ',dbpntgp
         call errormsg
      endif

!-----------------------------------------------------------------------
! Look for a reference point specified in DDPOINTS-SECTION in input file
! ELSE
! take initial geometry as reference point
!-----------------------------------------------------------------------
      lfound=.false.
      call rdxref(lfound)

      if (lfound) then
         write(ilog,'(a)') '   Using symmetry reference specified in DDPOINTS'

      else
         write(ilog,'(a)') '   Taking initial WP geometry as symmetry reference'

         xref = xingeo
         atnam = xatnam
      endif

      iatiso=0
      iau=0
      do n=1,natm
         call getinam(atnam(n),iat,iatiso,ilog)
         call getmass(mref(n),iat,iau,ilog)
      enddo

!-----------------------------------------------------------------------
! Symmetrise initial structure
!-----------------------------------------------------------------------
      if (nsymop .gt. 1) then
         call gensymmaps

         allocate(xgp(ndofddpes,nsymop))
         n1 = 0
         do n = 1,natm
            do i =1,3
               n1 = n1+1
               xgp(n1,1) = xref(i,n)
            enddo
         enddo
         call mksymcent(xgp,dbpntgp,nsymop,1,1)
         do n=2,nsymop
            xgp(:,1) = xgp(:,1) + xgp(:,n)
         enddo
         xgp(:,1) = xgp(:,1) / dble(nsymop)
         n1 = 0
         do n = 1,natm
            do i =1,3
               n1 = n1+1
               xref(i,n) = xgp(n1,1)
            enddo
         enddo
         deallocate(xgp)
      endif

      return
      end subroutine prepsym

!#######################################################################
!
! DDDB_RD
! routine to read LHA info from all points (weighted) in a DB
! only diabatic files need to be read.
!
! Calling routine to different versions
!
!#######################################################################

      subroutine dddb_rd(v,deriv1,deriv2,dipole,xgpoint,ltransder,&
                 lvonly,lforce)
                                                                        
!+----------------------------------------------------------------------
!     read LHA PES database for direct dynamics                        
!+----------------------------------------------------------------------
      
      implicit none

      logical(kind=4), intent(in) :: ltransder,lvonly,lforce
      real(dop), dimension(ndofddpes),intent(in)               :: xgpoint
      real(dop), dimension(nddstate,nddstate), intent(out)     :: v
      real(dop), dimension(ndofddpes,nddstate,nddstate), &
                                                   intent(out) :: deriv1
      real(dop), dimension(ndofddpes,ndofddpes,nddstate,nddstate), &
                                                   intent(out) :: deriv2
      real(dop), dimension(3,nddstate,nddstate), intent(out)   :: dipole

#ifdef OMP
      integer(long)          :: ithr
      character(len=20)      :: timstring
#else
      integer(long), save    :: tid = 0 ! timer ID
#endif

#ifdef OMP
      if (.not. allocated(tid_dbread)) then
         allocate(tid_dbread(ompthread))
         tid_dbread(:)=0
      endif

      ithr = omp_get_thread_num()+1
      write(timstring,'(a,i0)') 'DBread_thr',ithr
      call get_timer(tid_dbread(ithr), timstring)
      call start_timer(tid_dbread(ithr))
#else
      call get_timer(tid,'DBread')
      call start_timer(tid)
#endif

!-----------------------------------------------------------------------
! Comparisons are to be done in Cartesians. The extrapolation in
! Cartesians is followed by transformation back to system coordinates.
! xgpoint are original Cartesians, gpoint are rotated to Eckart frame
!-----------------------------------------------------------------------
      if(lshepgrad)then
         call dddb_rd_shepgrad(v,deriv1,deriv2,dipole,xgpoint,&
              ltransder,lvonly)
      else if (ldblocal) then
         call dddb_rd_local(v,deriv1,deriv2,dipole,xgpoint)
      else if (lgausspot) then
         call dddb_rd_gap(v,deriv1,deriv2,xgpoint,ltransder)
      else if (ldbsmall) then
        call dddb_rd_gp(v,deriv1,deriv2,dipole,xgpoint,ltransder,&
             lvonly,lforce)
      else
        call dddb_rd_full(v,deriv1,deriv2,dipole,xgpoint,ltransder,&
             lvonly,lforce)
      endif

#ifdef OMP
      call stop_timer(tid_dbread(ithr))
#else
      call stop_timer(tid)
#endif

      return
      end subroutine dddb_rd

!#######################################################################
!
! routine to read LHA info from all points (weighted) in a DB
! only diabatic files need to be read.
!
! lvonly: just read energy. NB this seems to be wrong as shiftdd
! will not work!!
!
! lforce: ignore last call information and recalculate weighted 
! potential
!
! dbignore(dbnrec): Passed by module. If 1, record ignored in weighting
!#######################################################################

      subroutine dddb_rd_full(v,deriv1,deriv2,dipole,xgpoint,ltransder,&
                 lvonly,lforce)
                                                                        
!+----------------------------------------------------------------------
!     read LHA PES database for direct dynamics                        
!+----------------------------------------------------------------------
      
!+----------------------------------------------------------------------
!     read LHA PES database for direct dynamics                        
! if using lbdsave, DB is in memory not read from file
!+----------------------------------------------------------------------
      implicit none

      logical(kind=4), intent(in) :: ltransder,lvonly,lforce
      logical                     :: linterpol
      integer(long)               :: s,s1,i,f,order,irec,idbloc,t
      real(dop), dimension(ndofddpes),intent(in)               :: xgpoint
      real(dop), dimension(nddstate,nddstate), intent(out)     :: v
      real(dop), dimension(ndofddpes,nddstate,nddstate), &
                                                   intent(out) :: deriv1
      real(dop), dimension(ndofddpes,ndofddpes,nddstate,nddstate), &
                                                   intent(out) :: deriv2
      real(dop), dimension(3,nddstate,nddstate), intent(out)   :: dipole
      real(dop), dimension(:), allocatable   :: r1
      real(dop), dimension(:), allocatable   :: weight
      real(dop)                              :: mindist,mindisp
      real(dop), allocatable        :: gx(:)
      real(dop), allocatable        :: vv(:,:)
      real(dop), allocatable        :: der1(:,:,:)
      real(dop), allocatable        :: der2(:,:,:,:)
      real(dop), allocatable        :: dip(:,:,:)
      real(dop), allocatable        :: vtmp(:,:)
      real(dop), allocatable        :: dertmp1(:,:,:)
      real(dop), allocatable        :: dertmp2(:,:,:,:)
      real(dop), allocatable        :: diptmp(:,:,:)
      real(dop), allocatable        :: tv(:,:)
      real(dop), allocatable        :: tderiv1(:,:,:)
      real(dop), allocatable        :: tderiv2(:,:,:,:)
      real(dop), allocatable        :: tdipole(:,:,:)
      real(dop)                     :: maxwgt,dv,rtmp

      real(dop), allocatable        :: ord_dist(:)
      real(dop), dimension(1,1)     :: tmpmat
      integer(long)                 :: dbnconf1

      logical, save     :: initial = .true.

      real(dop)         :: dnrm2 !BLAS norm function

      v(:,:)=0.0_dop
      deriv1(:,:,:)=0.0_dop
      deriv2(:,:,:,:)=0.0_dop
      dipole(:,:,:)=0.0_dop
      linterpol=.true.
      mindist=0.0_dop
      mindisp=0.0_dop
      idbloc=0
      dv = 0.0_dop
      initial = lforce
      t=0

      allocate(gx(ndofddpes))
      gx=0
      if (dbnrec .gt. 0) then
         allocate(r1(dbnrec))
         allocate(weight(dbnrec))
         allocate(ord_dist(dbnrec))
      else
         allocate(r1(1))
         allocate(weight(1))
         allocate(ord_dist(1))
      endif
      r1=0
      weight=0.0_dop

! Shepard norm (summ or weights) needed in diabat4. 
      shep_norm = 1.0_dop

!-----------------------------------------------------------------------
! see if PES was calculated at this point on last entry
! this happens for multi-state systems
!
! Ignore if forcing recalculation
!-----------------------------------------------------------------------
      if (.not. initial) then
         do f=1,ndofddpes
            gx(f) = xgpold(f)-xgpoint(f)
         enddo
         mindist=dnrm2(ndofddpes,gx,1)
         if (mindist .le. 1.0e-10_dop) then
            v(:,:) = vold(1:nddstate,1:nddstate)
            deriv1(:,:,:) = der1old(1:ndofddpes,1:nddstate,1:nddstate)
            deriv2(:,:,:,:) =  &
                der2old(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate)
            if (ldipdb) dipole(:,:,:) = dipold(:,1:nddstate,1:nddstate)
            if (nddstate .eq. 2 .and. dddiab .eq. 1) then
              dv=dsqrt((v(2,2)-v(1,1))**2+4.0_dop*v(2,1)**2)
              if (dv .lt. dddvmin) then
                v(:,:)=0.0_dop
                deriv1(:,:,:)=0.0_dop
                deriv2(:,:,:,:)=0.0_dop
                if (ldipdb) dipole(:,:,:)=0.0_dop
              else
                return
              endif
            else
              return
            endif
         endif
      endif

!-----------------------------------------------------------------------
! Calculate distances to points in the database
!-----------------------------------------------------------------------
      call distdb(xgpoint,idbloc,r1,mindist,mindisp,dbnrec)

!-----------------------------------------------------------------------
! Determine if the point doesn't need interpolation
! NB If this value is changed, you should also change the value before the call
! to dbuncertain in routine getddpes.
!-----------------------------------------------------------------------
      if (mindisp .lt. dbdisp) then
         allocate(vv(nddstate,nddstate),&
                  der1(ndofddpes,nddstate,nddstate),&
                  der2(ndofddpes,ndofddpes,nddstate,nddstate),&
                  dip(3,nddstate,nddstate))
         vv=0
         der1=0
         der2=0
         dip=0

         linterpol=.false.
         if (ldbsave) then
            gx(1:ndofddpes) = dbgeo(1:ndofddpes,idbloc)
            do s=1,nddstate
               vv(1:nddstate,s) = dbener(1:nddstate,s,idbloc)
               do s1=1,nddstate
                  der1(1:ndofddpes,s1,s) = dbgrad(1:ndofddpes,s1,s,idbloc)
                  do i=1,ndofddpes
                     der2(1:ndofddpes,i,s1,s) = &
                          dbhess(1:ndofddpes,i,s1,s,idbloc)
                  enddo
                  if (ldipdb) dip(1:3,s1,s) = dbdipmom(1:3,s1,s,idbloc)
               enddo
            enddo
         else
            call dbrdgeo(the_db,natm,idbloc,idbloc,dbgeo)
            gx(:) = dbgeo(:,1)
            call dbrdpes(the_db,nddstate,idbloc,-1,dbener)
            vv(1:nddstate,1:nddstate) = dbener(1:nddstate,1:nddstate,1)
            call dbrdgra( the_db,nddstate,natm,idbloc,-1,dbgrad)
            der1(:,:,:) = dbgrad(:,:,:,1)
            call dbrdhes( the_db,nddstate,natm,idbloc,-1,dbhess)
            der2(:,:,:,:) = dbhess(:,:,:,:,1)
            if (ldipdb) then
               call dbrddip(the_db,idbloc,-1,dbdipmom)
               dip(:,:,:) = dbdipmom(:,:,:,1)
            endif
            do s=1,nddstate-1
               vv(s,s+1:)=vv(s+1:,s)
               der1(:,s,s+1:)=der1(:,s+1:,s)
               der2(:,:,s,s+1:)=der2(:,:,s+1:,s)
               if (ldipdb) dip(:,s,s+1:)=dip(:,s+1:,s)
            enddo
         endif

! shift PES to current coordinate
         call shiftdd(v,deriv1,deriv2,dipole,xgpoint,vv, &
                     der1,der2,dip,gx,2)  

         deallocate(vv,der1,der2,dip)
      endif
      deallocate(gx)

!-----------------------------------------------------------------------
! loop over DB - OMP parallelized (if storing DB in memory only)
!-----------------------------------------------------------------------
      if (linterpol) then

!-----------------------------------------------------------------------
! Get confidence radius
!-----------------------------------------------------------------------
         dbnconf1=min(dbnconf,dbnrec)
         ord_dist = r1
         call dbubble(ord_dist,tmpmat,dbnrec,0)
         irec = dbnconf1
         do
            rad_conf = ord_dist(irec)
            if (rad_conf .lt. 90000.0) exit  ! ignored points in DB have 
                                             ! distance of 99999.0
            irec=irec-1
         enddo

!-----------------------------------------------------------------------
! Get Shepard weights
!-----------------------------------------------------------------------
         maxwgt = -999.0_dop
         do irec=1,dbnrec
            if (dbignore(irec) .eq. 1) cycle
            if (dbinterp(irec) .lt. 0) cycle

! Take care of small distance (possible because of mindv)
            rtmp = max(r1(irec)/rad_conf,1.0d-6)
            weight(irec) = 1.0_dop/((rtmp**dbinterord1) + (rtmp**dbinterord))

            if (weight(irec) .lt. dbminwgt) weight(irec) = 0.0_dop
         enddo
         maxwgt = max(maxwgt,maxval(weight))
         weight = weight/maxwgt

         shep_norm=0.0_dop
         do irec=1,dbnrec
            shep_norm = shep_norm + weight(irec)
         enddo
         if (shep_norm.eq.0.0_dop) then
           routine='dddb_rd (dd_db.f90)'
           write(message,'(2a)') 'ERROR: No suitable points in database'
           call dberrormsg
         endif

         if (ldbsave) then

!$omp parallel reduction(+:dipole,v,deriv1,deriv2) default(none) &
!$omp shared(dbignore,dbinterp,weight,dbgeo,dbener,dbgrad,dbhess,dbdipmom,xgpoint) &
!!$omp private(dv,vtmp,dertmp1,dertmp2,diptmp,tdipole,tv,tderiv1,tderiv2,irec,t1,t2,dt,tid) &
!$omp private(dv,vtmp,dertmp1,dertmp2,diptmp,tdipole,tv,tderiv1,tderiv2,irec) &
!$omp firstprivate(dbnrec,dbminwgt,nddstate,dddiab,dddvmin,ldipdb,lvonly,ndofddpes) &
!$omp num_threads(ompthread) if(lompthread)

            allocate(vtmp(nddstate,nddstate),&
                     dertmp1(ndofddpes,nddstate,nddstate),&
                     dertmp2(ndofddpes,ndofddpes,nddstate,nddstate),&
                     diptmp(3,nddstate,nddstate),&
                     tdipole(3,nddstate,nddstate),tv(nddstate,nddstate),&
                     tderiv1(ndofddpes,nddstate,nddstate),&
                     tderiv2(ndofddpes,ndofddpes,nddstate,nddstate))
            dv=0
            vtmp=0
            dertmp1=0
            dertmp2=0
            diptmp=0
            tdipole=0
            tv=0
            tderiv1=0
            tderiv2=0

!$omp do schedule(dynamic)
             do irec=1,dbnrec
 
                !tid = OMP_get_thread_num()
                !t1 = OMP_get_wtime()
 
                if (dbignore(irec) .eq. 1) cycle
                if (dbinterp(irec) .lt. 0) cycle
                if (weight(irec) .lt. dbminwgt) cycle
                   
                ! get adiabatic energy difference and check if too small
                ! do not use point when regularisation (only for two states)
                if (nddstate .eq. 2 .and. dddiab .eq. 1) then
                   dv=dsqrt((dbener(2,2,irec)-dbener(1,1,irec))**2+4.0_dop*dbener(2,1,irec)**2)
                   if (dv .lt. dddvmin) cycle
                endif

                ! shift PES
                !tid = OMP_get_thread_num()
                !t1 = OMP_get_wtime()
                call shiftdd(vtmp,dertmp1,dertmp2,diptmp,xgpoint,&
                     dbener(1:nddstate,1:nddstate,irec), &
                     dbgrad(1:ndofddpes,1:nddstate,1:nddstate,irec), &
                     dbhess(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,irec), &
                     dbdipmom(1:3,1:nddstate,1:nddstate,irec),&
                     dbgeo(1:ndofddpes,irec),2)  

                !t2 = OMP_get_wtime()
                !dt = t2-t1
                !write(ilog,*) 'TID: ',tid,'Time1: ',t1, 'Time2: ',t2,'dT: ',dt

                if (ldipdb .and. .not. lvonly) tdipole = tdipole + weight(irec)*diptmp
                tv = tv + weight(irec)*vtmp

                tderiv1 = tderiv1 + weight(irec)*dertmp1
                !if (.not. lvonly) call hessian_update1(xgpoint,deriv1,deriv2,r1,dbnrec)
                if (.not. lvonly) tderiv2 = tderiv2 + weight(irec)*dertmp2

                !t2 = OMP_get_wtime()
                !dt = t2-t1
                !write(ilog,*) 'TID: ',tid,'Time1: ',t1, 'Time2: ',t2,'dT: ',dt

                ! get next record
             enddo
!$omp enddo

             if (ldipdb) dipole = dipole + tdipole
             v = v + tv
             deriv1 = deriv1 + tderiv1
             if (.not. lvonly) deriv2 = deriv2 + tderiv2
  
            deallocate(vtmp,dertmp1,dertmp2,diptmp,tdipole,tv,tderiv1,tderiv2)

!$omp end parallel

         else
             ! Do interpolation if not reading DB from memory
             !
             allocate(gx(ndofddpes),vv(nddstate,nddstate),&
                  der1(ndofddpes,nddstate,nddstate),&
                  der2(ndofddpes,ndofddpes,nddstate,nddstate),&
                  dip(3,nddstate,nddstate),&
                  vtmp(nddstate,nddstate),dertmp1(ndofddpes,nddstate,nddstate),&
                  dertmp2(ndofddpes,ndofddpes,nddstate,nddstate),&
                  diptmp(3,nddstate,nddstate))
             gx=0
             vv=0
             der1=0
             der2=0
             dip=0
             vtmp=0
             dertmp1=0
             dertmp2=0
             diptmp=0
             !
             do irec=1,dbnrec

                if (dbignore(irec) .eq. 1) cycle
                call dbrdgeo(the_db,natm,irec,-1,dbgeo)
                gx(:) = dbgeo(:,1)
                !
! calculate distance with BLAS norm function
              !  r1(irec) = dnrm2(ndofddpes,gx-xgpoint,1)
                !
                call dbrdpes(the_db,nddstate,irec,-1,dbener)
                vv(1:nddstate,1:nddstate) = dbener(1:nddstate,1:nddstate,1)
                !
                if (nddstate .eq. 2 .and. dddiab .eq. 1) then
                   dv=dsqrt((vv(2,2)-vv(1,1))**2+4.0_dop*vv(2,1)**2)
                else
                   dv = 0.0_dop
                endif
                !
                !if (r1(irec) .gt. ddmaxdb .or. (nddstate .eq. 2 .and. dddiab .eq. 1 &
                !                             .and. dv .lt. dddvmin)) then
                if ((weight(irec) .lt. dbminwgt) .or. &
                         (nddstate .eq. 2 .and. dddiab .eq. 1 .and. dv .lt. dddvmin)) then
                !  skip this one
                else
                   call dbrdgra(the_db,nddstate,natm,irec,-1,dbgrad)
                   der1(:,:,:) = dbgrad(:,:,:,1)
                   call dbrdhes(the_db,nddstate,natm,irec,-1,dbhess)
                   der2(:,:,:,:) = dbhess(:,:,:,:,1)
                   if (ldipdb) then
                      call dbrddip(the_db,irec,-1,dbdipmom)
                      dip(:,:,:) = dbdipmom(:,:,:,1)
                   endif
                   !
                   do s=1,nddstate-1
                      vv(s,s+1:)=vv(s+1:,s)
                      der1(:,s,s+1:)=der1(:,s+1:,s)
                      der2(:,:,s,s+1:)=der2(:,:,s+1:,s)
                      if (ldipdb) dip(:,s,s+1:)=dip(:,s+1:,s)
                   enddo
                   !
                   call shiftdd(vtmp,dertmp1,dertmp2,diptmp,xgpoint,vv, &
                        der1,der2,dip,gx,2)
                   !
                   !
! the scaling below is done through array operations
                   v = v + weight(irec)*vtmp
                   if(.not.lvonly)then
                      deriv1 = deriv1 + weight(irec)*dertmp1
                      deriv2 = deriv2 + weight(irec)*dertmp2
                      if (ldipdb) dipole = dipole + weight(irec)*dip
                   endif
               endif
             enddo
!
! end of file
!
             deallocate(gx,vv,der1,der2,dip,vtmp,dertmp1,dertmp2,diptmp)
             !
         endif

!-----------------------------------------------------------------------
! normalise averaged PES via array operations
! Not yet needed for dipoles.
!-----------------------------------------------------------------------
          v = v / shep_norm
          deriv1 = deriv1 / shep_norm
          deriv2 = deriv2 / shep_norm
          if (ldipdb) dipole = dipole / shep_norm

      endif

! return un-scaled weight norm (needed in diabat4)
      shep_norm=shep_norm*maxwgt

!-----------------------------------------------------------------------
! For derivatives
! 1. Eckart transform to take derivatives to original Cartesian frame
! 2. Transform to dynamics coordinates
!-----------------------------------------------------------------------
      order = 2
!     call ddpes2x(deriv1,deriv2,ddeckart,order)
      if (lddtrans.and.ltransder.and. .not.lvonly) &
           call ddpes2q(deriv1,deriv2,order)

!-----------------------------------------------------------------------
! save result
!-----------------------------------------------------------------------
      xgpold(1:ndofddpes) = xgpoint(:)
      vold(1:nddstate,1:nddstate) = v(:,:)
      der1old(1:ndofddpes,1:nddstate,1:nddstate) = deriv1(:,:,:) 
      der2old(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate) = &
            deriv2(:,:,:,:) 
      if (ldipdb) dipold(:,1:nddstate,1:nddstate) = dipole(:,:,:) 
      initial=.false.

      if (allocated(r1)) deallocate(r1)
      if (allocated(ord_dist)) deallocate(ord_dist)
      if (allocated(weight)) deallocate(weight)

      return
      end subroutine dddb_rd_full
                                                                        
!#######################################################################
!
! Routine to read LHA info from all points (weighted) in a DB.
! This version performs the derivatives of the interpolated energies,
! rather than interpolating the derivatives. GWR 08/15
!
!#######################################################################

      subroutine dddb_rd_shepgrad(v,deriv1,deriv2,dipole,xgpoint,&
           ltransder,lvonly)
                                                                        
!+----------------------------------------------------------------------
!     read LHA PES database for direct dynamics                        
!+----------------------------------------------------------------------
      
!+----------------------------------------------------------------------
!     read LHA PES database for direct dynamics                        
!+----------------------------------------------------------------------
      implicit none

      logical(kind=4), intent(in)                          :: ltransder,lvonly
      logical(kind=4)                                      :: linterpol
      logical(kind=4), allocatable                         :: lincpnt(:)
      integer(long)                                        :: s,s1,i,f,f1,&
                                                              order,irec,&
                                                              idbloc,factor!, &
!                                                              i1,i2,jrec
      real(dop), dimension(ndofddpes),intent(in)           :: xgpoint
      real(dop), dimension(ndofddpes)                      :: tmpvec1
      real(dop), allocatable                               :: gx(:),r1(:), &
                                                              tmpvec2(:),&
                                                              xdiff(:),r1prod(:)

      real(dop), dimension(nddstate,nddstate), intent(out) :: v
      real(dop), dimension(ndofddpes,nddstate,nddstate), &
                                                   intent(out) :: deriv1
      real(dop), dimension(ndofddpes,ndofddpes,nddstate,nddstate), &
                                                   intent(out) :: deriv2
      real(dop), dimension(3,nddstate,nddstate), intent(out)   :: dipole
      real(dop), dimension(nddstate,nddstate)          :: tmpmat2
      real(dop), allocatable                           :: vv(:,:),tmpmat(:,:)
      real(dop), allocatable                           :: der1(:,:,:)
      real(dop), allocatable                           :: der2(:,:,:,:)
      real(dop), allocatable                           :: dip(:,:,:)
      real(dop), allocatable                           :: vtmp(:,:,:)
      real(dop), allocatable                           :: dertmp1(:,:,:)
      real(dop), allocatable                           :: dertmp2(:,:,:,:)
      real(dop), allocatable                           :: diptmp(:,:,:)
      real(dop)                                        :: tmpscal,&
                                                          weight,dv,normp,normp1
      real(dop)                                        :: mindist,mindisp

  !    real(dop)                                        :: dnrm2 !BLAS norm function

      factor = dbinterord/2 ! If this changes, change in directdyn.f90/dbuncertain as well
      normp = 0.0_dop
      normp1 = 0.0_dop
      v(:,:)=0.0_dop
      deriv1(:,:,:)=0.0_dop
      deriv2(:,:,:,:)=0.0_dop
      dipole(:,:,:)=0.0_dop
      linterpol=.true.
      dv = 0.0_dop

!
! Trafo to Cartesians
!
!      if (lddtrans) then
!         call ddq2x(gpoint,xgpoint)
!      else
!         call cpvxd(gpoint,xgpoint,ndofddpes)
!         !call eckart(xmass,xgpoint,gp,ddeckart)
!      endif

      if (dbnrec .gt. 0) then
         allocate(r1(dbnrec))
      else
         allocate(r1(1))
      endif

! Calculate distance to the closest point on the database
      r1=0.0_dop
      call distdb(xgpoint,idbloc,r1,mindist,mindisp,dbnrec)

! Open files if database not stored in memory
!      if(.not.ldbsave) then
!         geofile=ddname(1:ddlaenge)//'/geo.db'
!         pesfile=ddname(1:ddlaenge)//'/pes.db'
!         grafile=ddname(1:ddlaenge)//'/gra.db'
!         hesfile=ddname(1:ddlaenge)//'/hes.db'
!         dipfile=ddname(1:ddlaenge)//'/dip.db'
!         open(igeodb,file=geofile,status='old',form='unformatted',err=1006)
!         open(ipesdb,file=pesfile,status='old',form='unformatted',err=1003)
!         open(igradb,file=grafile,status='old',form='unformatted',err=1004)
!         open(ihesdb,file=hesfile,status='old',form='unformatted',err=1005)
!         if (ldipdb) open(idipdb,file=dipfile,status='old',form='unformatted',err=1008)
!      endif

!
! Determine if the point doesn't need interpolation
! NB If this value is changed, you should also change the value before the call
! to dbuncertain in routine getddpes.
!
      if (mindist .lt. 1.0e-10_dop) then
         linterpol=.false.
         if (ldbsave) then
            do s=1,nddstate
               v(1:nddstate,s) = dbener(1:nddstate,s,idbloc)
               if(.not.lvonly)then
                  do s1=1,nddstate
                     deriv1(1:ndofddpes,s1,s) = dbgrad(1:ndofddpes,s1,s,idbloc)
                     do i=1,ndofddpes
                        deriv2(1:ndofddpes,i,s1,s) = &
                             dbhess(1:ndofddpes,i,s1,s,idbloc)
                     enddo
                     if (ldipdb) dipole(1:3,s1,s) = dbdipmom(1:3,s1,s,idbloc)
                  enddo
               endif
            enddo
         else
!            do i=1,idbloc-1
!               read(ipesdb,err=1000)((v(s1,s),s1=s,nddstate),s=1,nddstate)
!               read(igradb,err=1001)(((deriv1(i1,s1,s),i1=1,ndofddpes), &
!                 s1=s,nddstate),s=1,nddstate)
!               read(ihesdb,err=1002)((((deriv2(i2,i1,s1,s),i2=1,ndofddpes), &
!                 i1=1,ndofddpes),s1=s,nddstate),s=1,nddstate)
!               if (ldipdb) read(idipdb,err=1008)&
!                    (((dipole(i1,s1,s),i1=1,3),s1=s,nddstate),s=1,nddstate)
!            enddo
!            read(ipesdb,err=1000) ((v(s1,s),s1=s,nddstate),s=1,nddstate)
!            read(igradb,err=1001) (((deriv1(i,s1,s),i=1,ndofddpes), &
!                 s1=s,nddstate),s=1,nddstate)
!            read(ihesdb,err=1002) ((((deriv2(i,i1,s1,s),i=1,ndofddpes), &
!                 i1=1,ndofddpes),s1=s,nddstate),s=1,nddstate)
!            if (ldipdb) read(idipdb,err=1008) &
!                 (((dipole(i,s1,s),i=1,3),s1=s,nddstate),s=1,nddstate)
!            do s=1,nddstate-1
!               v(s,s+1:)=v(s+1:,s)
!               deriv1(:,s,s+1:)=deriv1(:,s+1:,s)
!               deriv2(:,:,s,s+1:)=deriv2(:,:,s+1:,s)
!               if (ldipdb) dipole(:,s,s+1:)=dipole(:,s+1:,s)
!            enddo
!            rewind(ipesdb)
!            rewind(igradb)
!            rewind(ihesdb)
         endif
         if (nddstate .eq. 2 .and. dddiab .eq. 1) then
            dv=dsqrt((v(2,2)-v(1,1))**2+4.0_dop*v(2,1)**2)
            if (dv .lt. dddvmin) then
               linterpol=.true.
               v(:,:)=0.0_dop
               deriv1(:,:,:)=0.0_dop
               deriv2(:,:,:,:)=0.0_dop
               if (ldipdb) dipole(:,:,:)=0.0_dop
            endif
         endif
      endif

      if (linterpol) then
!
! loop over DB - OMP parallelized (if storing DB in memory only)
!
         if (ldbsave) then

            allocate(vtmp(nddstate,nddstate,dbnrec),&
                 lincpnt(dbnrec),r1prod(dbnrec))
            lincpnt=.true.

!$omp parallel &
!$omp private(gx,vv,dv,der1,der2,dip,dertmp1,dertmp2,diptmp,weight,f,f1,s,s1,i,tmpvec2,xdiff,tmpmat,tmpscal) &
!$omp num_threads(ompthread) if(lompthread)

            allocate(gx(ndofddpes),vv(nddstate,nddstate),&
                 der1(ndofddpes,nddstate,nddstate),&
                 der2(ndofddpes,ndofddpes,nddstate,nddstate),&
                 dip(3,nddstate,nddstate),&
                 dertmp1(ndofddpes,nddstate,nddstate),&
                 dertmp2(ndofddpes,ndofddpes,nddstate,nddstate),&
                 diptmp(3,nddstate,nddstate),tmpvec2(ndofddpes),&
                 xdiff(ndofddpes),tmpmat(ndofddpes,ndofddpes))

            dv=0

! Get the distances from the current point to all DB points.

!!$omp do reduction(+:normp,normp1)
!            do irec = 1, dbnrec
!               gx(1:ndofddpes) = dbgeo(1:ndofddpes,irec)
!               r1(irec)=dnrm2(ndofddpes,xgpoint-gx,1) 
!            enddo
!!$omp end do

! Determine whether to exclude any points.
!$omp do 
            do irec = 1, dbnrec
               vv(1:nddstate,1:nddstate) = dbener(1:nddstate,1:nddstate,irec)
               if (nddstate .eq. 2 .and. dddiab .eq. 1) then
                  dv=dsqrt((vv(2,2)-vv(1,1))**2+4.0_dop*vv(2,1)**2)
               endif
          !     if (r1(irec) .gt. ddmaxdb .or. (nddstate .eq. 2 &
          !                 .and. dddiab .eq. 1 .and. dv .lt. dddvmin)) &
               if (nddstate .eq. 2 .and. dddiab .eq. 1 .and. dv .lt. dddvmin) &
                    lincpnt(irec)=.false.
            enddo
!$omp enddo

! Get the norms of the distance weights.
!$omp do reduction(+:normp,normp1)
            do irec = 1, dbnrec
               if (dbignore(irec) .eq. 1) cycle
               if (.not.lincpnt(irec))cycle
               weight = 1.0_dop/(r1(irec)**(2*factor))
               normp = normp + weight
               weight = weight/(r1(irec)**2)
               normp1 = normp1 + weight
            enddo
!$omp end do

            tmpvec1 = 0.0_dop
!$omp do reduction(+:tmpvec1)
            do irec = 1, dbnrec
               if (dbignore(irec) .eq. 1) cycle
               if(.not.lincpnt(irec))cycle
               weight = 1.0_dop/(r1(irec)**(2*(factor+1)))
               gx(1:ndofddpes) = xgpoint(1:ndofddpes) - dbgeo(1:ndofddpes,irec)
               gx = gx*weight
               tmpvec1 = tmpvec1 + gx
            enddo
!$omp end do

!$omp do
            do irec=1,dbnrec
               if (dbignore(irec) .eq. 1) cycle
               if(.not.lincpnt(irec))cycle
! read point from DB
               gx(1:ndofddpes) = dbgeo(1:ndofddpes,irec)

! get adiabatic/diabatic energy
               do s=1,nddstate
                  vv(1:nddstate,s) = dbener(1:nddstate,s,irec)
               enddo
! read gradients, Hessian and optionally dipole moment
               do s=1,nddstate
                  do s1=1,nddstate
                     der1(1:ndofddpes,s1,s) = dbgrad(1:ndofddpes,s1,s,irec)
                     do i=1,ndofddpes
                        der2(1:ndofddpes,i,s1,s) = &
                             dbhess(1:ndofddpes,i,s1,s,irec)
                     enddo
                     if (ldipdb) dip(1:3,s1,s) = dbdipmom(1:3,s1,s,irec)
                  enddo
               enddo
! shift PES
               call shiftdd(vtmp(1,1,irec),dertmp1,dertmp2,diptmp,xgpoint,&
                    vv,der1,der2,dip,gx,2)  

!$omp critical

               weight = 1.0_dop/(r1(irec)**(2*factor))
 
               if (ldipdb .and. .not. lvonly) then
                  do s = 1,nddstate
                     do s1 = 1,nddstate
                        do i=1,3
                           dipole(i,s1,s) = &
                                dipole(i,s1,s) + weight*dip(i,s1,s)
                        enddo
                     enddo
                  enddo
               endif
!$omp end critical
!$omp critical
               do s = 1,nddstate
                  do s1 = 1,nddstate
                     v(s1,s) = v(s1,s) + weight*vtmp(s1,s,irec)
                     if(.not.lvonly)then
                        do f=1,ndofddpes
                           deriv1(f,s1,s) = deriv1(f,s1,s) + &
                                weight*dertmp1(f,s1,s)
                        enddo
                     endif
                  enddo
               enddo             
!$omp end critical

               if(.not.lvonly)then
! Extra terms for gradient.
                  xdiff = xgpoint - gx
                  tmpvec2 = tmpvec1/normp
                  tmpvec2 = tmpvec2 -  xdiff/(r1(irec)**2)
                  do s = 1,nddstate
                     do s1 = 1,nddstate
!$omp critical
                        deriv1(1:ndofddpes,s1,s) = &
                             deriv1(1:ndofddpes,s1,s) + tmpvec2(1:ndofddpes)*&
                             weight*vtmp(s1,s,irec)*dble(2*factor)
!$omp end critical
                     enddo
                  enddo

!$omp critical
                  do s = 1,nddstate
                     do s1 = 1,nddstate
                        do f=1,ndofddpes
                           do f1=1,ndofddpes
                              deriv2(f1,f,s1,s) = deriv2(f1,f,s1,s) + &
                                   weight*dertmp2(f1,f,s1,s)
                           enddo
                        enddo
                     enddo
                  enddo
!$omp end critical
               endif
            enddo
!$omp enddo

            if(.not.lvonly)then
! Adding corrections to the Hessian.
! Term 1
!$omp do
               do irec = 1,dbnrec
                  if (dbignore(irec) .eq. 1) cycle
                  if(.not.lincpnt(irec))cycle
                  weight = 1.0_dop/(r1(irec)**(2*factor))
                  do s = 1,nddstate
                     do s1 = 1,nddstate
                        tmpvec2(1:ndofddpes) = &
                             dbgrad(1:ndofddpes,s1,s,irec)*weight
                        do f = 1,ndofddpes
                           do f1 = f,ndofddpes
                              tmpmat(f,f1) = tmpvec2(f)*tmpvec1(f1) &
                                   + tmpvec2(f1)*tmpvec1(f)
                              if(f.ne.f1) tmpmat(f1,f) = tmpmat(f,f1)
                           enddo
                        enddo
                        tmpmat = tmpmat * dble(2*factor)/normp
!$omp critical
                        deriv2(1:ndofddpes,1:ndofddpes,s1,s) = &
                             deriv2(1:ndofddpes,1:ndofddpes,s1,s) &
                             + tmpmat(1:ndofddpes,1:ndofddpes)
!$omp end critical
                     enddo
                  enddo
               enddo
!$omp enddo

! Term 2
!$omp do
               do irec = 1,dbnrec
                  if (dbignore(irec) .eq. 1) cycle
                  if(.not.lincpnt(irec))cycle
                  weight = 1.0_dop/(r1(irec)**(2*factor))
                  gx = xgpoint - dbgeo(1:ndofddpes,irec)
                  do s = 1,nddstate
                     do s1 = 1,nddstate
                        tmpmat = dbhess(1:ndofddpes,1:ndofddpes,s1,s,irec)
                        tmpvec2 = matmul(tmpmat,gx)
                        tmpvec2 = tmpvec2 * weight
                     
                        do f = 1,ndofddpes
                           do f1 = f,ndofddpes
                              tmpmat(f,f1) = tmpvec2(f)*tmpvec1(f1) &
                                   + tmpvec2(f1)*tmpvec1(f)
                              if(f.ne.f1) tmpmat(f1,f) = tmpmat(f,f1)
                           enddo
                        enddo
                        tmpmat = tmpmat*dble(2*factor)/normp
!$omp critical
                        deriv2(1:ndofddpes,1:ndofddpes,s1,s) = &
                             deriv2(1:ndofddpes,1:ndofddpes,s1,s) &
                             + tmpmat(1:ndofddpes,1:ndofddpes)
!$omp end critical
                     enddo
                  enddo
               enddo
!$omp enddo

! Term 3
!$omp do
               do irec = 1,dbnrec
                  if (dbignore(irec) .eq. 1) cycle
                  if(.not.lincpnt(irec))cycle
                  gx = xgpoint - dbgeo(1:ndofddpes,irec)
                  weight = 1.0_dop/(r1(irec)**(2*factor+2))
                  do s = 1,nddstate
                     do s1 = 1,nddstate
                        tmpmat = dbhess(1:ndofddpes,1:ndofddpes,s1,s,irec)
                        tmpvec2 = matmul(tmpmat,gx)
                        do f = 1,ndofddpes
                           do f1 = f,ndofddpes
                              tmpmat(f,f1) = tmpvec2(f)*gx(f1) &
                                   + tmpvec2(f1)*gx(f)
                              if(f.ne.f1) tmpmat(f1,f) = tmpmat(f,f1)
                           enddo
                        enddo
                        tmpmat = tmpmat*dble(2*factor)*weight
!$omp critical
                        deriv2(1:ndofddpes,1:ndofddpes,s1,s) = &
                             deriv2(1:ndofddpes,1:ndofddpes,s1,s) &
                             - tmpmat(1:ndofddpes,1:ndofddpes)
!$omp end critical
                     enddo
                  enddo
               enddo
!$omp enddo

! Term 4
!$omp do
               do irec = 1,dbnrec
                  if (dbignore(irec) .eq. 1) cycle
                  if(.not.lincpnt(irec))cycle
                  weight = 1.0_dop/(r1(irec)**(2*factor))
                  do s = 1,nddstate
                     do s1 = 1,nddstate
                        tmpscal = dble(2*factor)*vtmp(s1,s,irec) &
                             *weight*(normp1 - normp/(r1(irec)**2))
                        tmpscal = tmpscal/normp
                        do f = 1,ndofddpes
!$omp critical
                           deriv2(f,f,s1,s) = deriv2(f,f,s1,s) + tmpscal
!$omp end critical
                        enddo
                     enddo
                  enddo
               enddo
!$omp end do

! Term 5 is the Shepard interpolation of the Hessian, see above.

! Term 6
!$omp do
               do irec = 1,dbnrec
                  if (dbignore(irec) .eq. 1) cycle
                  if(.not.lincpnt(irec))cycle
                  weight = 1.0_dop/(r1(irec)**(2*factor+2))
                  gx = xgpoint - dbgeo(1:ndofddpes,irec)
                  gx = gx * weight
                  do s = 1,nddstate
                     do s1 = 1,nddstate
                        tmpvec2 = gx*vtmp(s1,s,irec)
                        do f = 1,ndofddpes
                           do f1 = f,ndofddpes
                              tmpmat(f,f1) = tmpvec1(f)*tmpvec2(f1) &
                                   + tmpvec1(f1)*tmpvec2(f)
                              if(f.ne.f1)tmpmat(f1,f) = tmpmat(f,f1)
                           enddo
                        enddo
                        tmpmat = tmpmat*dble(4*factor*factor)/normp
!$omp critical
                        deriv2(1:ndofddpes,1:ndofddpes,s1,s) = &
                             deriv2(1:ndofddpes,1:ndofddpes,s1,s) &
                             - tmpmat(1:ndofddpes,1:ndofddpes)
!$omp end critical
                     enddo
                  enddo
               enddo
!$omp end do

! Term 7
!$omp do
               do irec = 1,dbnrec
                  if (dbignore(irec) .eq. 1) cycle
                  if(.not.lincpnt(irec))cycle
                  weight = 1.0_dop/(r1(irec)**(2*factor+2))
                  gx = xgpoint - dbgeo(1:ndofddpes,irec)
                  gx = gx * weight
                  do s = 1,nddstate
                     do s1 = 1,nddstate
                        tmpvec2 = dbgrad(1:ndofddpes,s1,s,irec)
                        do f = 1,ndofddpes
                           do f1 = f,ndofddpes
                              tmpmat(f,f1) = &
                                   gx(f)*tmpvec2(f1) + gx(f1)*tmpvec2(f)
                              if(f.ne.f1)tmpmat(f1,f) = tmpmat(f,f1)
                           enddo
                        enddo
                        tmpmat = tmpmat*dble(2*factor)
!$omp critical
                        deriv2(1:ndofddpes,1:ndofddpes,s1,s) = &
                             deriv2(1:ndofddpes,1:ndofddpes,s1,s) &
                             - tmpmat(1:ndofddpes,1:ndofddpes)
!$omp end critical

                     enddo
                  enddo
               enddo
!$omp end do

! Term 8
!$omp do
               do irec = 1,dbnrec
                  if (dbignore(irec) .eq. 1) cycle
                  if(.not.lincpnt(irec))cycle
                  weight = 1.0_dop/(r1(irec)**(2*factor+4))
                  gx = xgpoint - dbgeo(1:ndofddpes,irec)
                  do s = 1,nddstate
                     do s1 = 1,nddstate
                        do f = 1,ndofddpes
                           do f1 = f,ndofddpes
                              tmpmat(f,f1) = gx(f)*gx(f1)
                              if(f.ne.f1)tmpmat(f1,f) = tmpmat(f,f1)
                           enddo
                        enddo
                        tmpmat = tmpmat*weight*vtmp(s1,s,irec)
                        tmpmat = tmpmat*dble(4*factor*(factor+1))
!$omp critical
                        deriv2(1:ndofddpes,1:ndofddpes,s1,s) = &
                             deriv2(1:ndofddpes,1:ndofddpes,s1,s) &
                             + tmpmat(1:ndofddpes,1:ndofddpes)
!$omp end critical
                     enddo
                  enddo
               enddo
!$omp end do

! Term 9
               tmpmat2 = 0.0_dop
!$omp do reduction(+:tmpmat2)
               do irec = 1,dbnrec
                  if (dbignore(irec) .eq. 1) cycle
                  if(.not.lincpnt(irec))cycle
                  weight = 1.0_dop/(r1(irec)**(2*factor))
                  tmpmat2 = tmpmat2 + vtmp(1:nddstate,1:nddstate,irec)*weight
               enddo
!$omp end do

!$omp do
               do irec = 1,dbnrec
                  if (dbignore(irec) .eq. 1) cycle
                  if(.not.lincpnt(irec))cycle
                  weight = 1.0_dop/(r1(irec)**(2*factor+4))
                  gx = xgpoint - dbgeo(1:ndofddpes,irec)
                  
                  do f = 1,ndofddpes
                     do f1 = f,ndofddpes
                        tmpmat(f,f1) = gx(f)*gx(f1)
                        if(f.ne.f1)tmpmat(f1,f) = tmpmat(f,f1)
                     enddo
                  enddo
                  tmpmat = tmpmat * weight * dble(4*factor*(factor+1))/normp

!$omp critical
                  do s = 1,nddstate
                     do s1 = 1,nddstate
                        do f = 1,ndofddpes
                           do f1 = 1,ndofddpes
                              deriv2(f,f1,s1,s) = deriv2(f,f1,s1,s) &
                                   - tmpmat(f,f1)*tmpmat2(s1,s)
                           enddo
                        enddo
                     enddo
                  enddo
!$omp end critical               
               enddo
!$omp end do

! Term 10
!$omp master
               do s = 1,nddstate
                  do s1 = 1,nddstate
                     do f = 1,ndofddpes
                        do f1 = 1,ndofddpes
                           deriv2(f,f1,s1,s) = deriv2(f,f1,s1,s) &
                                + tmpvec1(f)*tmpmat2(s1,s)*tmpvec1(f1)&
                                *dble(8*factor*factor)/(normp*normp)
                        enddo
                     enddo
                  enddo
               enddo
!$omp end master
            endif

            deallocate(gx,vv,der1,der2,dip,dertmp1,dertmp2,diptmp,&
                 tmpvec2,xdiff,tmpmat)

!$omp end parallel

         else
            routine = "dddb_rd_shepgrad"
            message = "Exact grad./Hess. of Shepard interp. only for DB in mem"
            call errormsg
         endif
      
! normalise averaged PES via array operations
! Not yet needed for dipoles.

         v = v / normp
         if(.not.lvonly)then
            deriv1 = deriv1 / normp
            deriv2 = deriv2 / normp
            if (ldipdb) dipole = dipole / normp
         endif
         
         if (allocated(r1)) deallocate(r1)
         if (allocated(r1prod)) deallocate(r1prod)
         if (allocated(vtmp)) deallocate(vtmp)
         if (allocated(lincpnt)) deallocate(lincpnt)
      endif
      
!-----------------------------------------------------------------------
! For derivatives
! 1. Eckart transform to take derivatives to original Cartesian frame
! 2. Transform to dynamics coordinates
!-----------------------------------------------------------------------
      order = 2
      if (lddtrans.and.ltransder .and. .not.lvonly) &
           call ddpes2q(deriv1,deriv2,order)

      return
               
      end subroutine dddb_rd_shepgrad
                                                                        
!#######################################################################
!
! routine to read energy + derivs from nearest point (used to e.g. plot 
! DB energies). No interpolation done, so not suited for dynamics.
!
!#######################################################################

      subroutine dddb_rd_local(v,deriv1,deriv2,dipole,xgpoint)
                                                                        
!+----------------------------------------------------------------------
!     read LHA PES database for direct dynamics                        
!+----------------------------------------------------------------------
      implicit none

      integer(long)       :: i,s,s1,idbloc
      real(dop), dimension(ndofddpes),intent(in)               :: xgpoint
      real(dop), dimension(nddstate,nddstate), intent(out)     :: v
      real(dop), dimension(ndofddpes,nddstate,nddstate), &
                                                   intent(out) :: deriv1
      real(dop), dimension(ndofddpes,ndofddpes,nddstate,nddstate), &
                                                   intent(out) :: deriv2
      real(dop), dimension(3,nddstate,nddstate), intent(out)   :: dipole
      real(dop), dimension(:), allocatable      :: r1
      real(dop)                                 :: mindist, mindisp
      real(dop), allocatable        :: gx(:)
      real(dop), allocatable        :: vv(:,:)
      real(dop), allocatable        :: der1(:,:,:)
      real(dop), allocatable        :: der2(:,:,:,:)
      real(dop), allocatable        :: dip(:,:,:)

      logical, save     :: initial = .true.

      real(dop)         :: dnrm2 !BLAS norm function

      v(:,:)=0.0_dop
      deriv1(:,:,:)=0.0_dop
      deriv2(:,:,:,:)=0.0_dop
      dipole(:,:,:)=0.0_dop
      mindist=0.0_dop
      idbloc=0

      allocate(gx(ndofddpes))
      gx=0
      if (dbnrec .gt. 0) then
         allocate(r1(dbnrec))
         r1=0
      endif

!-----------------------------------------------------------------------
! see if PES was calculated at this point on last entry
! this happens for multi-state systems
!-----------------------------------------------------------------------
      if (.not. initial) then
         gx(:) = xgpold(1:ndofddpes)-xgpoint(:)
         mindist=dnrm2(ndofddpes,gx,1)
         if (mindist .le. tiny) then
            v(:,:) = vold(1:nddstate,1:nddstate)
            deriv1(:,:,:) = der1old(1:ndofddpes,1:nddstate,1:nddstate)
            deriv2(:,:,:,:) =  &
                der2old(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate)
            if (ldipdb) dipole(:,:,:) = dipold(:,1:nddstate,1:nddstate)
            return
         endif
      endif

!-----------------------------------------------------------------------
! Allocate temporary arrays
!-----------------------------------------------------------------------
      allocate(vv(nddstate,nddstate),&
               der1(ndofddpes,nddstate,nddstate),&
               der2(ndofddpes,ndofddpes,nddstate,nddstate),&
               dip(3,nddstate,nddstate))
      vv=0
      der1=0
      der2=0
      dip=0

!-----------------------------------------------------------------------
! Calculate distances to points in the database
!-----------------------------------------------------------------------
      call distdb(xgpoint,idbloc,r1,mindist,mindisp,dbnrec)

!-----------------------------------------------------------------------
! Read DB info from nearest point
!-----------------------------------------------------------------------
      if (ldbsave) then
         do s=1,nddstate
            gx(1:ndofddpes) = dbgeo(1:ndofddpes,idbloc)
            vv(1:nddstate,s) = dbener(1:nddstate,s,idbloc)
            do s1=1,nddstate
               der1(1:ndofddpes,s1,s) = dbgrad(1:ndofddpes,s1,s,idbloc)
               do i=1,ndofddpes
                  der2(1:ndofddpes,i,s1,s) = &
                       dbhess(1:ndofddpes,i,s1,s,idbloc)
               enddo
               if (ldipdb) dip(1:3,s1,s) = dbdipmom(1:3,s1,s,idbloc)
            enddo
         enddo
      else
         call dbrdgeo( the_db,natm,idbloc,-1,dbgeo)
         gx(:) = dbgeo(:,1)
         call dbrdpes(the_db,nddstate,idbloc,-1,dbener)
         vv(1:nddstate,1:nddstate) = dbener(1:nddstate,1:nddstate,1)
         call dbrdgra(the_db,nddstate,natm,idbloc,-1,dbgrad)
         der1(:,:,:) = dbgrad(:,:,:,1)
         call dbrdhes(the_db,nddstate,natm,idbloc,-1,dbhess)
         der2(:,:,:,:) = dbhess(:,:,:,:,1)
         if (ldipdb) then
            call dbrddip(the_db,idbloc,-1,dbdipmom)
            dip(:,:,:) = dbdipmom(:,:,:,1)
         endif

         do s=1,nddstate-1
            vv(s,s+1:)=vv(s+1:,s)
            der1(:,s,s+1:)=der1(:,s+1:,s)
            der2(:,:,s,s+1:)=der2(:,:,s+1:,s)
            if (ldipdb) dip(:,s,s+1:)=dip(:,s+1:,s)
         enddo
      endif

!-----------------------------------------------------------------------
! shift PES to current coordinate
!-----------------------------------------------------------------------
      call shiftdd(v,deriv1,deriv2,dipole,xgpoint,vv, &
                   der1,der2,dip,gx,2)  

!-----------------------------------------------------------------------
! save result
!-----------------------------------------------------------------------
      xgpold(1:ndofddpes) = xgpoint(:)
      vold(1:nddstate,1:nddstate) = v(:,:)
      if (ldipdb) dipold(:,1:nddstate,1:nddstate) = dipole(:,:,:) 
      initial=.false.

      if (allocated(r1)) deallocate(r1)
      if (allocated(gx)) deallocate(gx)
      if (allocated(vv)) deallocate(vv)
      if (allocated(der1)) deallocate(der1)
      if (allocated(der2)) deallocate(der2)
      if (allocated(dip)) deallocate(dip)

      return
               
      end subroutine dddb_rd_local

!#######################################################################
!
! routine to read energy only from DB 
!
!#######################################################################

      subroutine dddb_rd_db0(v,xgpoint)
                                                                        
!+----------------------------------------------------------------------
!     read LHA PES database for direct dynamics                        
!+----------------------------------------------------------------------
      
!+----------------------------------------------------------------------
!     read LHA PES database for direct dynamics                        
!+----------------------------------------------------------------------
      implicit none

      integer(long)       :: i,s,s1,idbloc,irec
      real(dop), dimension(ndofddpes),intent(in)               :: xgpoint
      real(dop), dimension(ndofddpes)                          :: gx
      real(dop), dimension(ndofddpes)                          :: distvectmp
      real(dop), dimension(nddstate,nddstate), intent(out)     :: v
      real(dop), dimension(ndofddpes,nddstate,nddstate)        :: deriv1
      real(dop), dimension(ndofddpes,ndofddpes,nddstate,nddstate) :: deriv2
      real(dop), dimension(3,nddstate,nddstate)                :: dipole
      real(dop), dimension(:), allocatable      :: r1
      real(dop), dimension(:), allocatable      :: weight
      real(dop)                     :: mindist,rtmp,natnrm
      real(dop), allocatable        :: ord_dist(:)
      real(dop), dimension(1,1)     :: tmpmat
      integer(long)                 :: dbnconf1,natom
      real(dop), allocatable        :: vv(:,:)
      real(dop), allocatable        :: der1(:,:,:)
      real(dop), allocatable        :: der2(:,:,:,:)
      real(dop), allocatable        :: dip(:,:,:)
      real(dop), allocatable        :: tv(:,:)

      real(dop)         :: dnrm2 !BLAS norm function

      shep_norm = 0.0_dop
      v(:,:)=0.0_dop
      deriv1(:,:,:)=0.0_dop
      deriv2(:,:,:,:)=0.0_dop
      dipole(:,:,:)=0.0_dop
      mindist=0.0_dop
      idbloc=0

      gx=0
      if (db0nrec .gt. 0) then
         allocate(r1(db0nrec))
         allocate(weight(db0nrec))
         allocate(ord_dist(db0nrec))
      else
         allocate(r1(1))
         allocate(weight(1))
         allocate(ord_dist(1))
      endif
      r1=0
      weight=0.0_dop

!-----------------------------------------------------------------------
! Calculate distances to points in the database
!-----------------------------------------------------------------------
      natom = ndofddpes/3
      natnrm = dsqrt(dble(natom))
!$omp parallel do schedule(static) default(none) &
!$omp shared(db0geo,r1,natnrm,ndofdd,db0nrec,ndofddpes,xgpoint,vv) &
!$omp private(irec,gx,distvectmp,routine,message,tv) &
!$omp num_threads(ompthread) if(lompthread)
      do irec=1,db0nrec
         gx(1:ndofddpes) = db0geo(1:ndofddpes,irec)
         distvectmp=xgpoint-gx
         ! calculate the distance using BLAS norm function
         r1(irec) = dnrm2(ndofddpes,distvectmp,1)
         r1(irec) = r1(irec)/natnrm
      enddo
!$omp end parallel do
      mindist = minval(r1)
      idbloc = minloc(r1,DIM=1)

      allocate(vv(nddstate,nddstate),&
               der1(ndofddpes,nddstate,nddstate),&
               der2(ndofddpes,ndofddpes,nddstate,nddstate),&
               dip(3,nddstate,nddstate))

!-----------------------------------------------------------------------
! Determine if the point doesn't need interpolation
! NB If this value is changed, you should also change the value before the call
! to dbuncertain in routine getddpes.
!-----------------------------------------------------------------------
      if (mindist .lt. dbdisp) then
         vv=0
         der1=0
         der2=0
         dip=0

         gx(1:ndofddpes) = db0geo(1:ndofddpes,idbloc)
         do s=1,nddstate
            vv(1:nddstate,s) = db0ener(1:nddstate,s,idbloc)
            do s1=1,nddstate
               der1(1:ndofddpes,s1,s) = db0grad(1:ndofddpes,s1,s,idbloc)
               do i=1,ndofddpes
                  der2(1:ndofddpes,i,s1,s) = &
                       db0hess(1:ndofddpes,i,s1,s,idbloc)
               enddo
            enddo
         enddo

! shift PES to current coordinate
         call shiftdd(v,deriv1,deriv2,dipole,xgpoint,vv, &
                  der1,der2,dip,gx,2)  

!-----------------------------------------------------------------------
! loop over DB - OMP parallelized (if storing DB in memory only)
!-----------------------------------------------------------------------
      else
!-----------------------------------------------------------------------
! Get confidence radius
!-----------------------------------------------------------------------
         dbnconf1=min(dbnconf,db0nrec)
         ord_dist = r1
         call dbubble(ord_dist,tmpmat,db0nrec,0)
         rad_conf = ord_dist(dbnconf1)

!$omp parallel do schedule(static) &
!$omp shared(weight) &
!$omp private(irec,rtmp) &
!$omp firstprivate(db0nrec,dbinterord,r1,rad_conf,dbinterord1) &
!$omp num_threads(ompthread) if(lompthread)
         do irec=1,db0nrec
! Take care of small distance 
            rtmp = max(r1(irec)/rad_conf,1.0d-6)
            weight(irec) = 1.0_dop/((rtmp**dbinterord1) + (rtmp**dbinterord))
         enddo
!$omp end parallel do
         do irec=1,db0nrec
            if (weight(irec) .ge. dbminwgt) shep_norm = shep_norm + weight(irec)
         enddo
         if (shep_norm.eq.0.0_dop) then
           routine='dddb_rd_db1 (dd_db.f90)'
           write(message,'(2a)') 'ERROR: No suitable points in database'
           call dberrormsg
         endif

!$omp parallel reduction(+:v)  &
!$omp private(vv,der1,der2,dip,tv,irec) &
!$omp firstprivate(db0nrec,dbminwgt,nddstate,ndofddpes) &
!$omp num_threads(ompthread) if(lompthread)

         allocate(tv(nddstate,nddstate))

         v=0
         tv=0

!$omp do schedule(dynamic)
          do irec=1,db0nrec
             if (weight(irec) .lt. dbminwgt) cycle
                   
             ! shift PES
             call shiftdd(vv,der1,der2,dip,xgpoint,&
                  db0ener(1:nddstate,1:nddstate,irec), &
                  db0grad(1:ndofddpes,1:nddstate,1:nddstate,irec), &
                  db0hess(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,irec), &
                  db0dipmom(1:3,1:nddstate,1:nddstate,irec),&
                  db0geo(1:ndofddpes,irec),2)  

             tv = tv + weight(irec)*vv
          enddo
!$omp enddo

          v = v + tv
  
         deallocate(tv)

!$omp end parallel

          v = v / shep_norm
      endif

      deallocate(vv,der1,der2,dip)

      if (allocated(r1)) deallocate(r1)
      if (allocated(ord_dist)) deallocate(ord_dist)
      if (allocated(weight)) deallocate(weight)

      return
               
      end subroutine dddb_rd_db0

!#######################################################################
! Routine to interpolate potential using Gaussian approximation.
! GWR 06/16
!#######################################################################
!main subroutine for GAP project

      subroutine dddb_rd_gap(v,deriv1,deriv2,xgpoint,ltransder)

use global
use griddatmod
use dirdyn
use channels
use gapmod

      implicit none

      real(dop), dimension(nddstate,nddstate), intent(out) :: v
      real(dop), dimension(ndofddpes,nddstate,nddstate), &
           intent(out) :: deriv1
      real(dop), dimension(ndofddpes,ndofddpes,nddstate,nddstate), &
           intent(out) :: deriv2
      real(dop), dimension(ndofddpes), intent(in)          :: xgpoint
      logical(kind=4), intent(in) :: ltransder

      integer(long)                        :: i,j,k,order,f
      real(dop)                            :: expon,wgnorm,distsq
      real(dop), dimension(:), allocatable :: r1
      real(dop), dimension(ndofdd)         :: gpoint,dbgpoint,gpdiff

      if (dbnrec .eq. 0) return

      allocate(r1(dbnrec))
      r1=0.0_dop

      outer: do i = 1,nddstate 
         inner: do j = 1,nddstate
            wgnorm = dot_product(dbgapwgt(i,j,1:dbnrec),dbgapwgt(i,j,1:dbnrec))

            if(abs(wgnorm).lt.tiny) then
               call gapwgt(dbgeo(1:ndofddpes,1:dbnrec),&
                    dbener(1:nddstate,1:nddstate,1:dbnrec),&
                    dbgapwgt(1:nddstate,1:nddstate,1:dbnrec),&
                    dbnrec,nddstate,ndofddpes)
               exit outer
            endif
         enddo inner
      enddo outer

      gpoint = 0.0_dop
      if(lddtrans)then
         call ddx2q(xgpoint,gpoint)
      else
         call cpvxd(xgpoint,gpoint,ndofdd)
      endif
      
!      call distdb(xgpoint,idum,r1,mindist,mindisp,dbnrec)

      v = 0.0_dop
      deriv1 = 0.0_dop
      deriv2 = 0.0_dop

      do i = 1, dbnrec

         dbgpoint = 0.0_dop
         if(lddtrans)then
            call ddx2q(dbgeo(1:ndofddpes,i),dbgpoint)
         else
            call cpvxd(dbgeo(1:ndofddpes,i),dbgpoint,ndofdd)
         endif

         do f = 1,ndofdd
            gpdiff(f) = (gpoint(f)-dbgpoint(f))*sqrt(gapalpha(f))
         enddo
         distsq = dot_product(gpdiff,gpdiff)

!         expon = gapkfac**2*exp(-gapalpha*distsq)
         expon = gapkfac**2*exp(-distsq)

         v(1:nddstate,1:nddstate) = v(1:nddstate,1:nddstate) &
              + dbgapwgt(1:nddstate,1:nddstate,i)*expon

         do j = 1,ndofddpes
            deriv1(j,1:nddstate,1:nddstate) = deriv1(j,1:nddstate,1:nddstate) &
                 + dbgapwgt(1:nddstate,1:nddstate,i)&
                 * expon * (xgpoint(j)-dbgeo(i,j)) * gapalpha(j)

            deriv2(j,j,1:nddstate,1:nddstate) = &
                 deriv2(j,j,1:nddstate,1:nddstate) &
                 + (1.0_dop - 2.0_dop*gapalpha(j)*(xgpoint(j)-dbgeo(j,i))**2) &
                 * expon * gapalpha(j)

            do k = j+1, ndofddpes
               deriv2(j,k,1:nddstate,1:nddstate) = &
                    deriv2(j,k,1:nddstate,1:nddstate) &
                    - 2.0_dop * gapalpha(j) * (xgpoint(j)-dbgeo(j,i)) &
                    * (xgpoint(k)-dbgeo(k,i)) * expon * gapalpha(k)
            enddo
         enddo
      enddo

      do j = 1, ndofddpes
         do k = j+1,ndofddpes
            deriv2(k,j,1:nddstate,1:nddstate) = &
                 deriv2(j,k,1:nddstate,1:nddstate)
         enddo
      enddo

      deriv1 = -2.0_dop*gapkfac**2*deriv1
      deriv2 = -2.0_dop*gapkfac**2*deriv2

      order = 2
      if (lddtrans.and.ltransder) call ddpes2q(deriv1,deriv2,order)

      deallocate(r1)

    end subroutine dddb_rd_gap
                                                                        
!#######################################################################
! Calculate the GAP weight vector after new DB points have been 
! calculated.
! GWR 06/16
!#######################################################################
      subroutine gapwgt(geo,ener,weight,nrec,nst,ncoord)

use dirdyn
use gapmod
!use lineqmod, only: lineqd
use timing

      implicit none

      integer(long), intent(in) :: nrec,nst,ncoord
      real(dop), dimension(ncoord,nrec), intent(in)   :: geo
      real(dop), dimension(nst,nst,nrec), intent(in)  :: ener
      real(dop), dimension(nst,nst,nrec), intent(out) :: weight

      integer(long)                      :: i,j,k,ierr
      real(dop), dimension(nrec,nrec)    :: amat,distsq
      real(dop), dimension(nrec,nst,nst) :: tmppot
      real(dop), dimension(ndofdd)       :: distvec
      real(dop), dimension(ndofdd,nrec)  :: gpoint

      integer(long), save                :: tid = 0 ! timer ID
      call get_timer(tid, 'gapwgt')
      call start_timer(tid)

      gpoint = 0.0_dop
      do i = 1,nrec
         if(lddtrans)then
            call ddx2q(geo(1:ncoord,i),gpoint(1,i))
         else
            call cpvxd(geo(1:ncoord,i),gpoint(1,i),ncoord)
         endif
      enddo

! Calculate the distance squared between all pairs of DB points (in terms
! of the coordinate system of choice, not necessarily Cartesian).
      distsq = 0.0_dop
      do i = 1,nrec
         do j = i+1,nrec
            distvec = 0.0_dop
            do k = 1,ndofdd
               distvec(k) = (gpoint(k,i) - gpoint(k,j))*sqrt(gapalpha(k))
            enddo
            distsq(i,j) = dot_product(distvec,distvec)
         enddo
      enddo  

! Construct A-matrix i.e. matrix of covariances.
      do i = 1,nrec
         amat(i,i) = gapkfac**2 + gapsigma
         do j = i+1, nrec
!            amat(i,j) = gapkfac**2*exp(-gapalpha*distsq(i,j)) 
            amat(i,j) = gapkfac**2*exp(-distsq(i,j)) 
         enddo
      enddo
      do i = 1,nrec
         do j = i+1,nrec
            amat(j,i) = amat(i,j)
         enddo
      enddo

! Call the inversion routine to get the Gaussian weights.
      do i = 1,nrec
         tmppot(i,1:nst,1:nst) = ener(1:nst,1:nst,i)
      enddo

!      call lineqd(nrec,nrec,amat,tmppot(1,1,1),weight(1,1,1),ierr)
      call dposv('L',nrec,nst*nst,amat,nrec,tmppot(1,1,1),nrec,ierr)
      if(ierr.ne.0)then
         routine = 'gapwgt'
         write(message,'(a,i4)') 'Failure to invert A-matrix',ierr
         call errormsg
      endif

      do i = 1,nrec
         weight(1:nst,1:nst,i) = tmppot(i,1:nst,1:nst)
      enddo

! Use the resulting inverse for all other states.
!      do i = 1,nst
!         do j = 1,nst
!            if(i.eq.1 .and. j.eq.1)cycle
!            weight(1:nrec,i,j) = matmul(amat,tmppot(1:nrec,i,j))            
!         enddo
!      enddo

      call stop_timer(tid)
      return
      end subroutine gapwgt

!#######################################################################
! Subroutine to calculate the GAP variance measure for a set of 
! molecular geoemtries in order to determine whether to calculate a new
! DB point.
! GWR 06/16
!#######################################################################

      subroutine gapvar(xgp,lrddb,ngp,ncoord,nrec)

use dirdyn
use gapmod
use op1lib, only: unitqxd
use mmlib, only: qqxtdd

      implicit none

      integer(long), intent(in) :: ngp,ncoord,nrec
      real(dop), dimension(ncoord,ngp), intent(in) :: xgp
      logical(kind=4), dimension(ngp), intent(out) :: lrddb

      integer(long) :: i,j,k,ierr,e,lwork
      real(dop) :: distsq,var
      real(dop), dimension(nrec,nrec)    :: amat,distsqmat,aeigen,&
                                            ainv,tmpmat
      real(dop), dimension(nrec)         :: tmpvec,covar
      real(dop), dimension(ncoord)       :: distvec
      real(dop), dimension(3*nrec)       :: work
      
      lrddb = .true.
      lwork = 3*nrec

! Calculate the distance squared between all pairs of DB points.
      distsqmat = 0.0_dop
      do i = 1,nrec
         do j = i+1,nrec
            distvec = 0.0_dop
            do k = 1,ncoord
               distvec(k) = (dbgeo(i,k) - dbgeo(j,k))*sqrt(gapalpha(k))
            enddo
            distsqmat(i,j) = dot_product(distvec,distvec)
         enddo
      enddo  

! Construct A-matrix i.e. matrix of covariances.
      do i = 1,nrec
         amat(i,i) = gapkfac**2 + gapsigma
         do j = i+1, nrec
!            amat(i,j) = gapkfac**2*exp(-gapalpha*distsqmat(i,j)) 
            amat(i,j) = gapkfac**2*exp(-distsqmat(i,j)) 
         enddo
      enddo
      do i = 1,nrec
         do j = i+1,nrec
            amat(j,i) = amat(i,j)
         enddo
      enddo

! Call the inversion routine.
      tmpvec = 0.0_dop      

      aeigen = amat
      call dsyev('V','U',nrec,aeigen,nrec,tmpvec,work,lwork,ierr)
      if(ierr.ne.0)then
         routine = 'gapvar'
         write(message,'(a,i4)') 'Failure to diagonalise A-matrix',ierr
         call errormsg
      endif

      tmpmat = 0.0_dop
      do i = 1,nrec
         tmpmat(i,i) = 1.0_dop/tmpvec(i)
      enddo
      call qqxtdd(tmpmat,aeigen,ainv,nrec)
      ainv = matmul(aeigen,ainv)

! Calculate the covariance vector for each geometry, then the variance.
! If the variance is too large, set the flag to calclate a new point.
      do e = 1,ngp
         do i = 1,nrec
            do k = 1,ncoord
               distvec(k) = (xgp(k,e) - dbgeo(i,k))*sqrt(gapalpha(k))
            enddo
            distsq = dot_product(distvec,distvec)
!            covar(i) = gapkfac**2*exp(-gapalpha*distsq)
            covar(i) = gapkfac**2*exp(-distsq)
         enddo
         tmpvec = matmul(ainv,covar)
         var = 1.0_dop+gapsigma - dot_product(covar,tmpvec)

         if(var.gt.1.0e-4_dop)lrddb(e) = .false.

      enddo

      return
      end subroutine gapvar

!#######################################################################
!
! DISTDB
!
! gpoint:       system coordinates
! iloc: DB location of nearest structure
! dist: Distances to all points in DB in Cartesian (Euclidean norm)
! mindist: Minimum distance from any point in DB
! mindisp: Minimum Max atom displacement from any point in DB
!
!#######################################################################

      subroutine distdb(xgpoint,iloc,dist,mindist,mindisp,nrec)  

!+----------------------------------------------------------------------
!     find the two closest geometries in database for direct dynamics  
!+----------------------------------------------------------------------
      implicit none                                                     
                                                                        
      integer(long)                             :: irec
      integer(long), intent(in)                 :: nrec
      integer(long), intent(out)                :: iloc
      real(dop), dimension(nrec)                :: atdisp
      real(dop), dimension(ndofddpes), intent(in) :: xgpoint
      real(dop), dimension(ndofdd)              :: qdist
      real(dop), dimension(nrec), intent(out)   :: dist
      real(dop), intent(out)                    :: mindisp,mindist
      real(dop), dimension(ndofddpes)           :: gx,xcoord,xpoint,xdist

      if (nrec .eq. 0) return

!----------------------------------------------------------------------- 
! initialise variables 
!----------------------------------------------------------------------- 
      irec=0
      iloc = 0

      mindisp = 99999.0_dop
      mindist = 99999.0_dop
      xcoord(:)=0.0_dop
      xpoint(:)=0.0_dop
      qdist(:)=0.0_dop
      xdist(:)=0.0_dop
      dist(:)=99999.0_dop

!-----------------------------------------------------------------------  
! loop over DB 
!----------------------------------------------------------------------- 
      ! If in memory 
      if (ldbsave) then
         do irec=1,nrec
            if (dbignore(irec) .eq. 1) cycle

            call distdb1(xgpoint,dbgeo(1:ndofddpes,irec),dist(irec),&
                 atdisp(irec))
         
            mindist = min(mindist,dist(irec))
            mindisp = min(mindisp,atdisp(irec))
         enddo

      ! If from file 
      else
         do irec=1,nrec
            if (dbignore(irec) .eq. 1) cycle
            call dbrdgeo(the_db,natm,irec,irec,dbgeo)
            gx(:) = dbgeo(:,1)

            call distdb1(xgpoint,gx,dist(irec),atdisp(irec))
            mindist = min(mindist,dist(irec))
            mindisp = min(mindisp,atdisp(irec))
         enddo

      endif

!-----------------------------------------------------------------------
! get minimum distance
!-----------------------------------------------------------------------
      iloc = minloc(dist,DIM=1)
                                                                        
      return                                                            
      end subroutine distdb      

!######################################################################
!
! Two structures: xpnt,dbpnt
! dist: RMSD (Euclidean norm)
! disp: max atom displacement
!
!######################################################################

      subroutine distdb1(xpnt,dbpnt,dist,disp)

      implicit none

      real(dop), dimension(ndofddpes), intent(in) :: xpnt,dbpnt
      real(dop), dimension(ndofddpes)             :: vec
      real(dop), intent(out)                      :: disp,dist
      real(dop)                                   :: natnrm
      real(dop)                                   :: dnrm2 !BLAS norm function
                                                           !sqrt(vec^2)

      natnrm = dsqrt(dble(natm))

      vec=xpnt-dbpnt
      ! calculate the distance using BLAS norm function
      dist = dnrm2(ndofddpes,vec,1)
      dist = dist/natnrm

      ! calculate max atom displacement
      call maxatdist(disp,vec)

      return
      end subroutine distdb1

!######################################################################

      subroutine maxatdist(rx,distvec)

      implicit none

      integer(long)                            :: i,n
      real(dop), dimension(3,natm), intent(in) :: distvec
      real(dop), intent(out)                   :: rx
      real(dop)                                :: atdist

      rx = 0.0_dop
      do n=1,natm
         atdist = 0.0_dop
         do i=1,3
            atdist = atdist+distvec(i,n)*distvec(i,n)
         enddo
         atdist=dsqrt(atdist)
         rx=max(rx,atdist)
      enddo

      return
      end subroutine maxatdist

!######################################################################
!
! Calculate the updated Hessian
!
!######################################################################

      subroutine hessian_update(geo,grad,hess)

      implicit none

      integer(long)                      :: irec,jrec,krec,lrec,f,&
                                            nrec_int,nrec_ext,mtmp
      integer(long), dimension(dbnrec)   :: map
      real(dop)                          :: norm,dtmp,dpt,weight
      real(dop), dimension(dbnrec)       :: d1,d2,d3
      real(dop), dimension(ndofddpes)    :: r,gpoint,dbgpointref,dbgpoint,&
                                            dbgpoint2
      real(dop), dimension(ndofddpes,ndofddpes,nddstate,nddstate), intent(out)&
           :: hess
      real(dop), dimension(ndofddpes), intent(in) :: geo
      real(dop), dimension(ndofddpes,nddstate,nddstate), intent(in) :: grad

      hess(:,:,:,:) = 0.0_dop
      r(:) = 0.0_dop
      d1(:) = 0.0_dop
      d2(:) = 0.0_dop
      d3(:) = 0.0_dop
      dpt = 0.0_dop
      map(:) = 0
      nrec_int = 0
      nrec_ext = 0
      norm = 0.0_dop
      !
      ! Get the distance between the current point and the (closest) reference point
      !
      if(lddtrans)then
         call ddx2q(geo,gpoint)
         call ddx2q(dbgeoref(1:ndofddpes,1),dbgpointref)
         do f=1,ndofdd
            r(f) = gpoint(f) - dbgpointref(f)
            dpt = dpt + r(f)*r(f)
         enddo
      else
         do f=1,ndofddpes
            r(f) = geo(f) - dbgeoref(f,1)
            dpt = dpt + r(f)*r(f)
         enddo
      endif
      dpt = dsqrt(dpt)

      !
      ! Loop over all database points
      !
      do irec=1,dbnrec
         if (dbignore(irec) .eq. 1) cycle
         !
         ! Get the distance between each database point and (closest) reference point
         !
         if(lddtrans)then
            call ddx2q(dbgeo(1:ndofddpes,irec),dbgpoint)
            do f=1,ndofdd
               r(f) = dbgpoint(f) - dbgpointref(f)
               d1(irec) = d1(irec) + r(f)*r(f)
            enddo
         else
            do f=1,ndofddpes
               r(f) = dbgeo(f,irec) - dbgeoref(f,1)
               d1(irec) = d1(irec) + r(f)*r(f)
            enddo
         endif
         d1(irec) = dsqrt(d1(irec))

         !
         ! Get the distance between each database point and the current point
         !
         if(lddtrans)then
            call ddx2q(dbgeo(1:ndofddpes,irec),dbgpoint)
            do f=1,ndofdd
               r(f) = dbgpoint(f) - gpoint(f)
               d2(irec) = d2(irec) + r(f)*r(f)
            enddo
         else
            do f=1,ndofddpes
               r(f) = dbgeo(f,irec) - geo(f)
               d2(irec) = d2(irec) + r(f)*r(f)
            enddo
         endif
         d2(irec) = dsqrt(d2(irec))

         !
         ! Perform a weighted Hessian update for the current point outward from the "inner circle" points
         !
         if (d1(irec) .lt. dpt) then
            nrec_int = nrec_int + 1
            !
            ! Assign weight
            !
            weight = 1.0_dop/(d2(irec)**4)
            norm = norm + weight
            !
            ! Hessian update algorithm
            !
            call powell(weight,geo,dbgeo(1:ndofddpes,irec),grad,&
                 dbgrad(1:ndofddpes,1:nddstate,1:nddstate,irec),hess,&
                 dbhess(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,irec),&
                 ndofddpes,nddstate)
         else
            nrec_ext = nrec_ext + 1
         endif
      enddo
      !

      if(nrec_int.gt.0)hess = hess/norm
      !
      ! Now we need to re-update the Hessians of the "outer region" points
      !
      ! First, sort the d1-array and create a mapping array
      !
      do irec=1,dbnrec
         map(irec) = irec
      enddo
      do irec=1,dbnrec
         do jrec=irec+1,dbnrec
            if (d1(jrec) .lt. d1(irec)) then
               dtmp = d1(irec)
               d1(irec) = d1(jrec)
               d1(jrec) = dtmp
               mtmp = map(irec)
               map(irec) = map(jrec)
               map(jrec) = mtmp
            endif
         enddo
      enddo
      !
      ! Then, iteratively update remaining, "outer region" points Hessians
      !
      do irec = nrec_int+1,dbnrec
         jrec = map(irec)
         norm = 0.0_dop
         dbhess(:,:,:,:,jrec) = 0.0_dop
         d3(:) = 0.0_dop
         !
         do krec = 1,irec-1
            lrec = map(krec)

            if(lddtrans)then
               call ddx2q(dbgeo(1:ndofddpes,jrec),dbgpoint)
               call ddx2q(dbgeo(1:ndofddpes,lrec),dbgpoint2)

               do f=1,ndofdd
                  r(f) = dbgpoint2(f) - dbgpoint(f)
                  d3(krec) = d2(krec) + r(f)*r(f)
               enddo
            else
               do f=1,ndofddpes
                  r(f) = dbgeo(f,lrec) - dbgeo(f,jrec)
                  d3(krec) = d2(krec) + r(f)*r(f)
               enddo
            endif

            d3(krec) = dsqrt(d3(krec))
            weight = 1/(d3(krec)**4)
            norm = norm + weight
!
! Hessian update algorithm for previously stored "inner circle" points
!
            call powell(weight,dbgeo(1:ndofddpes,jrec),&
                 dbgeo(1:ndofddpes,lrec),&
                 dbgrad(1:ndofddpes,1:nddstate,1:nddstate,jrec),&
                 dbgrad(1:ndofddpes,1:nddstate,1:nddstate,lrec),&
                 dbhess(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,jrec),&
                 dbhess(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,lrec),&
                 ndofddpes,nddstate)
         enddo
         !
         weight = 1/(d2(jrec)**4)
         norm = norm + weight
         !
         ! Hessian update algorithm for the current point
         !
         call powell(weight,dbgeo(1:ndofddpes,jrec),geo,&
              dbgrad(1:ndofddpes,1:nddstate,1:nddstate,jrec),grad,&
              dbhess(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,jrec),&
              hess,ndofddpes,nddstate)
            
         !
         dbhess(:,:,:,:,jrec) = dbhess(:,:,:,:,jrec)/norm
         !
      enddo
      !
      return
      end subroutine hessian_update

!######################################################################
!
! Calculate the updated Hessian
!
!######################################################################

      subroutine hessian_update1(geo,grad,hess,dbdist,nrec)

use global
use griddatmod
use dirdyn

      implicit none

      integer(long), intent(in)          :: nrec
      integer(long)                      :: irec,jrec,nrec_int,nrec_ext,f
      integer(long), dimension(nrec)     :: map
      integer(long), dimension(nrec)     :: loc
      real(dop), dimension(nrec), intent(in) :: dbdist
      real(dop)                          :: norm,dpt
      real(dop), dimension(nrec)       :: d1,d2,d3,weight
      real(dop), dimension(ndofddpes)    :: r
      real(dop), dimension(ndofddpes,ndofddpes,nddstate,nddstate), intent(out)&
           :: hess
      real(dop), dimension(ndofddpes), intent(in) :: geo
      real(dop), dimension(ndofddpes,nddstate,nddstate), intent(in) :: grad
      type(nextptr), pointer :: current

      hess(:,:,:,:) = 0.0_dop
      r(:) = 0.0_dop
      d1(:) = 0.0_dop
      d2(:) = 0.0_dop
      d3(:) = 0.0_dop
      dpt = 0.0_dop
      map(:) = 0
      nrec_int = 0
      nrec_ext = 0
      norm = 0.0_dop

! Local DBs set up array with locations in full DB
      if (ldbsmall) then
         call list_init(Current)
         current => ngp_loc(num_gp)%locpt
         do irec = 1,nrec
            loc(irec) = current%locDB
            current => current%Next
         enddo
      else
         do irec = 1,nrec
            loc(irec) = irec
         enddo
      endif

! dist of point from initial reference
      do f=1,ndofddpes
         r(f) = geo(f) - dbgeo(f,1)
         dpt = dpt + r(f)*r(f)
      enddo
      dpt = dsqrt(dpt)

!-----------------------------------------------------------------------
! Assign weight
!-----------------------------------------------------------------------
      norm=0.0_dop
      do irec=1,nrec
         jrec=loc(irec)
         if (dbignore(jrec) .eq. 1) cycle
         if (dbinterp(jrec) .lt. 0) cycle

         weight(irec) = 1.0_dop/(dbdist(irec)**dbinterord)

         norm = norm + weight(irec)
      enddo
      weight = weight/norm

      do irec=1,nrec
         jrec=loc(irec)
         if (dbignore(jrec) .eq. 1) cycle
         if (dbinterp(jrec) .lt. 0) cycle

         ! Perform a weighted Hessian update for the current point outward from 
         ! the "inner circle" points
         nrec_int = nrec_int + 1
         !
         ! Hessian update algorithm
         !
         call powell(weight(irec),geo,dbgeo(1:ndofddpes,jrec),grad,&
              dbgrad(1:ndofddpes,1:nddstate,1:nddstate,jrec),hess,&
              dbhess(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,jrec),&
              ndofddpes,nddstate)
      enddo
      
      return
      end subroutine hessian_update1

!######################################################################
!
! Calculate the updated Hessian.
! Updates all points in the DB, starting from the reference outwards.
!
!######################################################################

      subroutine hessian_update2

      implicit none

      integer(long)                      :: irec,jrec,f,mtmp
      integer(long), dimension(dbnrec)   :: map
      real(dop)                          :: norm,dpt,weight,d2,r
      real(dop), dimension(dbnrec)       :: d1
      real(dop), dimension(ndofddpes)    :: gpoint,dbgpointref,dbgpoint

!
! Get the distance between each database point and (closest) reference point
!
      if(lddtrans)&
         call ddx2q(dbgeoref(1:ndofddpes,1),dbgpointref)
      d1 = 0.0_dop
      do irec = 1, dbnrec
         if(lddtrans)then
            call ddx2q(dbgeo(1:ndofddpes,irec),dbgpoint)
            do f=1,ndofdd
               r = dbgpoint(f) - dbgpointref(f)
               d1(irec) = d1(irec) + r*r
            enddo
         else
            do f=1,ndofddpes
               r = dbgeo(f,irec) - dbgeoref(f,1)
               d1(irec) = d1(irec) + r*r
            enddo
         endif
         d1(irec) = dsqrt(d1(irec))
      enddo

! Generate a map array to indicate the relative proximity of each point to 
! the reference point.      
      do irec=1,dbnrec
         map(irec) = irec
      enddo
      do irec=1,dbnrec
         do jrec=irec+1,dbnrec
            if (d1(map(jrec)) .lt. d1(map(irec))) then
               mtmp = map(irec)
               map(irec) = map(jrec)
               map(jrec) = mtmp
            endif
         enddo
      enddo
         
! Zero the Hessian.
      dbhess = 0.0_dop
      dbhess(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,1) = &
           dbhesref(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,1)           

      do irec = 1, dbnrec
! Get the distance from the current point to the reference point.         
         if(lddtrans)&
              call ddx2q(dbgeo(1:ndofddpes,map(irec)),gpoint)
         dpt = d1(map(irec))

! Cycle if irec is the reference point.
         if(abs(dpt).lt.1.0e-8_dop)cycle

! Initialise.
         norm = 0.0_dop

! Loop over the other members of the DB.
         do jrec=1,irec-1

! Cycle if points are the same distance from the reference. 
! For symmetry reasons.
            if(abs(d1(map(jrec))-dpt).lt.1.0e-8_dop)cycle

!
! Get the distance between each database point and the current point
!
            d2 = 0.0_dop
            if(lddtrans)then
               call ddx2q(dbgeo(1:ndofddpes,map(jrec)),dbgpoint)
               do f=1,ndofdd
                  r = dbgpoint(f) - gpoint(f)
                  d2 = d2 + r*r
               enddo
            else
               do f=1,ndofddpes
                  r = dbgeo(f,map(jrec)) - dbgeo(f,map(irec))
                  d2 = d2 + r*r
               enddo
            endif
            d2 = dsqrt(d2)

            if(abs(d2).lt.1.0e-8_dop)cycle

! Assign weight
            weight = 1.0_dop/(d2**4)
            norm = norm + weight

! Hessian update algorithm
            call powell(weight,dbgeo(1:ndofddpes,map(irec)),&
                 dbgeo(1:ndofddpes,map(jrec)),&
                 dbgrad(1:ndofddpes,1:nddstate,1:nddstate,map(irec)),&
                 dbgrad(1:ndofddpes,1:nddstate,1:nddstate,map(jrec)),&
                 dbhess(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,map(irec)),&
                 dbhess(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,map(jrec)),&
                 ndofddpes,nddstate)
         enddo ! DB loop

! Normalise the result.
         dbhess(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,map(irec)) = &
              dbhess(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,map(irec))&
              /norm

      enddo

      return
      end subroutine hessian_update2

!#######################################################################
! Routine to do the Powell Hessian update. Separated out from routine
! hessian_update to tidy things up a bit.
! See Frankcombe, JCP, 2014, 140/ 114108
!####################################################################### 

      subroutine powell(weight,geo,dbgeo,grad,dbgrad,hess,dbhess,&
                 ndofddpes,nddstate)

      implicit none

      integer(long), intent(in) :: ndofddpes,nddstate
      real(dop), intent(in)     :: weight
      real(dop), dimension(ndofddpes), intent(in) :: geo,dbgeo
      real(dop), dimension(ndofddpes,nddstate,nddstate), &
           intent(in) :: grad,dbgrad
      real(dop), dimension(ndofddpes,ndofddpes,nddstate,nddstate),&
           intent(in) :: dbhess
      real(dop), dimension(ndofddpes,ndofddpes,nddstate,nddstate),&
           intent(inout) :: hess

      integer(long)                             :: s,s1,i,j
      real(dop)                                 :: dds,ddtd,temp
      real(dop), dimension(ndofddpes)           :: del,gamm,eps
      real(dop), dimension(ndofddpes,ndofddpes) :: amat

      del = 0.0_dop
      gamm = 0.0_dop
      eps = 0.0_dop

      do s=1,nddstate
         s1=s
  !       do s1=1,nddstate
            del(1:ndofddpes) = geo(1:ndofddpes) - dbgeo(1:ndofddpes)
            gamm(1:ndofddpes) = grad(1:ndofddpes,s,s1) &
                 - dbgrad(1:ndofddpes,s,s1)
            amat(1:ndofddpes,1:ndofddpes) = dbhess(1:ndofddpes,1:ndofddpes,s,s1)
            eps(:) = gamm(:)
            call dgemm('N','N',ndofddpes,1,ndofddpes,-1.0_dop,amat,&
                 ndofddpes,del,ndofddpes,1.0_dop,eps,ndofddpes)
            dds = sum(del(:)*del(:))
            ddtd = sum(eps(:)*del(:))/dds
            do i = 2, ndofddpes
               do j = 1, i - 1
                  temp = eps(i)*del(j) + del(i)*eps(j) &
                       - del(i)*ddtd*del(j)
                  hess(i,j,s,s1) = hess(i,j,s,s1) &
                       + weight*(dbhess(i,j,s,s1) + temp/dds)
                  hess(j,i,s,s1) = hess(i,j,s,s1)
               enddo
            enddo
            do i = 1, ndofddpes
               temp = del(i)*(2.0_dop*eps(i)-del(i)*ddtd)
               hess(i,i,s,s1) = hess(i,i,s,s1) &
                    + weight*(dbhess(i,i,s,s1) + temp/dds)
            enddo
  !       enddo
      enddo
      
      return
      end subroutine powell

!-----------------------------------------------------------------------
! transform diabatic Hessian to adiabatic Hessian
! (required after Hessian update as no adiabatic Hessian known)
!-----------------------------------------------------------------------

      subroutine hestrans(transf,aderiv2,deriv2,ndofddpes,nddstate)

      implicit none

      integer(long), intent(in)                    :: ndofddpes,nddstate
      real(dop), dimension(nddstate,nddstate), intent(in) :: transf
      real(dop), dimension(nddstate,nddstate)             :: transb
      real(dop), dimension(ndofddpes,ndofddpes,nddstate,nddstate),&
                                              intent(out) :: aderiv2
      real(dop), dimension(ndofddpes,ndofddpes,nddstate,nddstate), &
                                               intent(in) :: deriv2

      integer(long)                              :: idx,jdx,kdx,ldx
      real(dop), dimension(nddstate,nddstate)    :: tmpmat,tmpmat2

      do jdx = 1, nddstate
         transb(jdx,jdx) = transf(jdx,jdx)
         do kdx = jdx+1, nddstate
            transb(kdx,jdx) = transf(jdx,kdx)
            transb(jdx,kdx) = transf(kdx,jdx)
         enddo
      enddo

! the Hessian terms, DOF by DOF:
      do idx = 1, ndofddpes
         do ldx = idx, ndofddpes
            do jdx = 1, nddstate
               do kdx = jdx, nddstate
                  tmpmat2(kdx,jdx) = deriv2(idx,ldx,kdx,jdx)
                  tmpmat2(jdx,kdx) = deriv2(idx,ldx,jdx,kdx)
               enddo
            enddo

            tmpmat = matmul(tmpmat2,transb)
            call qqtxdd(transb,tmpmat,tmpmat2,nddstate)
            do jdx = 1, nddstate
               aderiv2(idx,ldx,jdx,jdx) = tmpmat2(jdx,jdx)
               aderiv2(ldx,idx,jdx,jdx) = tmpmat2(jdx,jdx)
            enddo
         enddo
      enddo

      return
      end subroutine hestrans

!#######################################################################
!
! Get the no. of records in a DB
!
!#######################################################################

      subroutine getdbnrec(nrec)

      implicit none
                                        
      integer(long), intent(out)     :: nrec

      call dbgetnrec( the_db, nrec )

      end subroutine getdbnrec

!#######################################################################
!
! Eckart
!
! INPUT
! natm: no. of atoms
! m(a): mass of atom a
! xcur(i): starting coordinates 
!
! On OUTPUT
! ddeckart(i,f): Eckart transformation matrix
! rnew(i): contains transformed coordinates
!#######################################################################

      subroutine eckart(m,xcur,rnew,ddeckart)                      

      implicit none                                                     
                                                                        
      integer(long), parameter                       :: lwork=9
      real(dop), parameter                           :: tol=1.0e-6_dop

      integer(long)                                  :: a,b,i,j,k,& 
                                                        dimold,icase,info  
      integer(long), dimension(3)                    :: jj,aa              

      real(dop)                                      :: pro,norm 
      real(dop), dimension(lwork)                    :: work
      real(dop), dimension(natm), intent(in)         :: m
      real(dop), dimension(3)                        :: w3
      real(dop), dimension(2)                        :: w2
      real(dop), dimension(3,3)                      :: v,u3,e3,f
      real(dop), dimension(2,2)                      :: u2,e2
      real(dop), dimension(3,natm), intent(in)       :: xcur
      real(dop), dimension(3,natm)                   :: rold
      real(dop), dimension(3,natm), intent(out)      :: rnew
      real(dop), dimension(ndofddpes,ndofddpes), intent(out)  :: ddeckart
      real(dop), dimension(3,natm,3,natm)            :: t,u

!-----------------------------------------------------------------------
! Call prewash to get coordinates centered at the center of mass and           
! aligned with the principal axes of inertia.                           
! rold: rotated coordinates with atoms ordered according to mass
!
! check m is correct
!-----------------------------------------------------------------------
      call prewash(xcur,rold,natm,m,dimold,t) 

!-----------------------------------------------------------------------
! Evaluate the Eckart vectors (reciprocal vectors F_i's).               
!-----------------------------------------------------------------------
      call zerovxd(v,3*3)
      do j=1,3                                                          
        do i=1,3                                                        
          do a=1,natm                                                   
            v(i,j)=v(i,j)+mref(a)*xref(j,a)*rold(i,a)          
          enddo                                                         
        enddo                                                           
      enddo                                                             
                                                                        
!-----------------------------------------------------------------------
! Evaluate the number of independent vectors among the F_i's            
! and determine to which case they belong (3D, 2D, 1D or undefined).    
!-----------------------------------------------------------------------
      if (dimref .eq. 3 .and. dimold .eq. 3) then                       
        icase=3                                                         
      elseif (dimref .eq. 1 .or. dimold .eq. 1) then                    
        icase=1                                                         
      else                                                              
        icase=2                                                         
      endif                                                             
                                                                        
      if (icase .eq. 3) then                                            
        pro=v(1,1)*(v(2,2)*v(3,3)-v(3,2)*v(2,3)) &           
           +v(2,1)*(v(3,2)*v(1,3)-v(1,2)*v(3,3)) &                
           +v(3,1)*(v(1,2)*v(2,3)-v(2,2)*v(1,3))                        
        if (pro .le. tol**3) then                                       
          icase=2                                                       
          dimref=2                                                      
        endif                                                           
      endif                                                             
      if (icase .eq. 2) then                                            
        jj(1)=1                                                         
        jj(2)=2                                                         
        jj(3)=3                                                         
        pro=(v(2,1)*v(3,2)-v(3,1)*v(2,2))**2  &            
           +(v(3,1)*v(1,2)-v(1,1)*v(3,2))**2  &               
           +(v(1,1)*v(2,2)-v(2,1)*v(1,2))**2                            
        if (dabs(pro) .le. tol**4) then                                 
          jj(1)=2                                                       
          jj(2)=3                                                       
          jj(3)=1                                                       
          pro=(v(2,2)*v(3,3)-v(3,2)*v(2,3))**2 &          
             +(v(3,2)*v(1,3)-v(1,2)*v(3,3))**2 &         
             +(v(1,2)*v(2,3)-v(2,2)*v(1,3))**2                          
          if (dabs(pro) .le. tol**4) then                               
            jj(1)=3                                                     
            jj(2)=1                                                     
            jj(3)=2                                                     
            pro=(v(2,3)*v(3,1)-v(3,3)*v(2,1))**2 &          
               +(v(3,3)*v(1,1)-v(1,3)*v(3,1))**2 &           
               +(v(1,3)*v(2,1)-v(2,3)*v(1,1))**2                        
            if (dabs(pro) .le. tol**4) then                             
              icase=1                                                   
              dimref=1                                                  
            endif                                                       
          endif                                                         
        endif                                                           
      endif                                                             
      if (icase .eq. 1) then                                            
        jj(1)=1                                                         
        jj(2)=2                                                         
        jj(3)=3                                                         
        norm=0.0_dop                                                       
        do i=1,3                                                        
          norm=norm+v(i,1)**2                                           
        enddo                                                           
        if (norm .le. tol**2) then                                      
          jj(1)=2                                                       
          jj(2)=3                                                       
          jj(3)=1                                                       
          norm=0.0_dop                                                     
          do i=1,3                                                      
            norm=norm+v(i,2)**2                                         
          enddo                                                         
          if (norm .le. tol**2) then                                    
            jj(1)=3                                                     
            jj(2)=1                                                     
            jj(3)=2                                                     
            norm=0.0_dop                                                   
            do i=1,3                                                    
              norm=norm+v(i,3)**2                                       
            enddo                                                       
            if (norm .le. tol**2) then                                  
              icase=0                                                   
              dimref=0                                                  
            endif                                                       
          endif                                                         
        endif                                                           
      endif                                                             
                                                                        
!-----------------------------------------------------------------------
! Evaluate the Eckart frame (direct vectors f_i's)                      
! depending on the case (3D, 2D, 1D or undefined).                      
!-----------------------------------------------------------------------
      if (icase .eq. 3) then                                            
        call zerovxd(u3,3*3)
        do i=1,3                                                        
          do j=1,3                                                      
            do k=1,3                                                    
              u3(i,j)=u3(i,j)+v(k,i)*v(k,j)                             
            enddo                                                       
          enddo                                                         
        enddo                                                           
                                                                        
        call dsyev('V','U',3,u3,3,w3,work,lwork,info)                   
        if (info .ne. 0) go to 999                                      
                                                                        
        call zerovxd(e3,3*3)
        do i=1,3                                                        
          do j=1,3                                                      
            do k=1,3                                                    
              e3(i,j)=e3(i,j)+u3(i,k)*u3(j,k)/dsqrt(w3(k))              
            enddo                                                       
          enddo                                                         
        enddo                                                           
                                                                        
        call zerovxd(f,3*3)
        do i=1,3                                                        
          do j=1,3                                                      
            do k=1,3                                                    
              f(i,j)=f(i,j)+v(i,k)*e3(k,j)                              
            enddo                                                       
          enddo                                                         
        enddo                                                           
                                                                        
      elseif (icase .eq. 2) then                                        
        call zerovxd(u2,2*2)
        do i=1,2                                                        
          do j=1,2                                                      
            do k=1,3                                                    
              u2(i,j)=u2(i,j)+v(k,jj(i))*v(k,jj(j))                     
            enddo                                                       
          enddo                                                         
        enddo                                                           
                                                                        
        call dsyev('V','U',2,u2,2,w2,work,lwork,info)                   
        if (info .ne. 0) go to 999                                      
                                                                        
        call zerovxd(e2,2*2)
        do i=1,2                                                        
          do j=1,2                                                      
            do k=1,2                                                    
              e2(i,j)=e2(i,j)+u2(i,k)*u2(j,k)/dsqrt(w2(k))              
            enddo                                                       
          enddo                                                         
        enddo                                                           
                                                                        
        call zerovxd(f,3*3)
        do i=1,3                                                        
          do j=1,2                                                      
            do k=1,2                                                    
              f(i,jj(j))=f(i,jj(j))+v(i,jj(k))*e2(k,j)                  
            enddo                                                       
          enddo                                                         
        enddo                                                           
        f(1,jj(3))=f(2,jj(1))*f(3,jj(2))-f(3,jj(1))*f(2,jj(2))          
        f(2,jj(3))=f(3,jj(1))*f(1,jj(2))-f(1,jj(1))*f(3,jj(2))          
        f(3,jj(3))=f(1,jj(1))*f(2,jj(2))-f(2,jj(1))*f(1,jj(2))          
                                                                        
      elseif (icase .eq. 1) then                                        
        norm=0.0_dop                                                       
        do i=1,3                                                        
          norm=norm+v(i,jj(1))**2                                       
        enddo                                                           
        do i=1,3                                                        
          f(i,jj(1))=v(i,jj(1))/dsqrt(norm)                             
        enddo                                                           
        if (1.0_dop-f(3,jj(1))**2 .gt. tol**2) then                        
          f(1,jj(2))=-f(2,jj(1))/dsqrt(1.0_dop-f(3,jj(1))**2)              
          f(2,jj(2))=f(1,jj(1))/dsqrt(1.0_dop-f(3,jj(1))**2)               
        else                                                            
          f(1,jj(2))=1.0_dop                                               
          f(2,jj(2))=0.0_dop                                               
        endif                                                           
        f(3,jj(2))=0.0_dop                                                 
        f(1,jj(3))=-f(3,jj(1))*f(2,jj(2))                               
        f(2,jj(3))=f(3,jj(1))*f(1,jj(2))                                
        f(3,jj(3))=f(1,jj(1))*f(2,jj(2))-f(2,jj(1))*f(1,jj(2))          
                                                                        
      else                                                              
        call zerovxd(f,3*3)
        do i=1,3                                                        
          f(i,i)=1.0_dop                                                   
        enddo                                                           
      endif                                                             
                                                                        
!-----------------------------------------------------------------------
! When dim(ref) < dim(old), or for undefined case, undetermined axes    
! must be sign-oriented following certain conventions.                  
!-----------------------------------------------------------------------
      call zerovxd(rnew,3*natm)
      do a=1,natm                                                       
        do i=1,3                                                        
          do j=1,3                                                      
            rnew(i,a)=rnew(i,a)+rold(j,a)*f(j,i)                        
          enddo                                                         
        enddo                                                           
      enddo   
                                                                        
      do k=1,3                                                          
        if ((icase .eq. 0) .or. &                  
            (k .eq. jj(2) .and. dimref .eq. 1 .and. dimold .ge. 2) .or. &
            (k .eq. jj(3) .and. dimold .eq. 3 .and. dimref .le. 2)) then
          aa(k)=1                                                       
 70       continue                                                      
          if (dabs(rnew(k,aa(k))) .lt. tol) then                        
            if (aa(k) .lt. natm) then                                   
              aa(k)=aa(k)+1                                             
              go to 70                                                  
            endif                                                       
          endif                                                         
          if (rnew(k,aa(k)) .lt. 0.0_dop) then                             
            do i=1,3                                                    
              f(i,k)=-f(i,k)                                            
            enddo                                                       
            do a=1,natm                                                 
              rnew(k,a)=-rnew(k,a)                                      
            enddo                                                       
          endif                                                         
        endif                                                           
      enddo                                                             
                                                                        
!-----------------------------------------------------------------------
! Final transformation defined by                                       
!   geo_new(k,b) = sum_a,i geo_old(i,a) * u(i,a,k,b) and                
!   u(i,a,k,b) = sum_j t(i,a,j,b) * f(j,k)                              
!-----------------------------------------------------------------------
      call zerovxd(u,3*(ndofddpes/3)*3*(ndofddpes/3))
      do b=1,natm                                                       
        do k=1,3                                                        
          do a=1,natm                                                   
            do i=1,3                                                    
              do j=1,3                                                  
                u(i,a,k,b)=u(i,a,k,b)+t(i,a,j,b)*f(j,k)         
              enddo                                                     
            enddo                                                       
          enddo                                                         
        enddo                                                           
      enddo                                                             
      call zerovxd(ddeckart,ndofddpes*ndofddpes)
      do b=1,natm
        do k=1,3
          do a=1,natm
            do i=1,3
              ddeckart(3*(a-1)+i,3*(b-1)+k)=u(i,a,k,b)
            enddo
          enddo
        enddo
      enddo

      return                                                            
  999 routine='Eckart'                                                  
      message = &   
      'Matrix diagonalisation (dsyev) failure'                          
      call dberrormsg                                                     
      end subroutine eckart         
                                                                        
!-----------------------------------------------------------------------
! PREWASH
!
! translates molecule to COM and rotates molecule to lie along
! inertial axes.
!
! natm: no. atoms
! rold(3,natm): initial coordinates 
! m(a): mass of atom a
!
! rnew(3,natm): on output rotated coordinates
! transfo(3,natm,3,natm): transformation matrix rnew = rold*transfo
!-----------------------------------------------------------------------

      subroutine prewash(rold,rnew,natm,m,dimold,transfo)

      implicit none                                                     
                                                                        
      integer(long), parameter                    :: lwork = 9
      real(dop), parameter                        :: tol = 1.0e-6_dop     

      integer(long)                               :: a,i,j,info,j1,j2,j3,s3
      integer(long), intent(in)                   :: natm
      integer(long), intent(out)                  :: dimold

      real(dop)                                                :: mtot,wtmp,ip,im
      real(dop), dimension(lwork)                              :: work
      real(dop), dimension(natm), intent(in)                   :: m
      real(dop), dimension(3)                                  :: utmp,bold,wold
      real(dop), dimension(3,3)                                :: uold
      real(dop), dimension(3,natm), intent(in)                 :: rold
      real(dop), dimension(3,natm), intent(out)                :: rnew
      real(dop), dimension(3,natm)                         :: rtmp
      real(dop), dimension(3,natm,3,natm), intent(out)     :: transfo
                                                                        
!-----------------------------------------------------------------------
! Center the old geometry at its center of mass.                        
! bold is COM
!-----------------------------------------------------------------------
      mtot=0.0_dop                                                         
      do a=1,natm                                                       
        mtot=mtot+m(a)                                                  
      enddo                                                             
      call zerovxd(bold,3)
      do i=1,3                                                          
        bold(i)=0.0_dop                                                    
        do a=1,natm                                                     
          bold(i)=bold(i)+m(a)*rold(i,a)                                
        enddo                                                           
        bold(i)=bold(i)/mtot                                            
      enddo                                                             
                                                                        
      do a=1,natm                                                       
        do i=1,3                                                        
          rtmp(i,a)=rold(i,a)-bold(i)                                   
        enddo                                                           
      enddo                                                             
                                                                        
!-----------------------------------------------------------------------
! Check if linear, planar or not.                                       
! Evaluate its principal axes of inertia and align it with them.        
!-----------------------------------------------------------------------
      call zerovxd(uold,3*3)
      do a=1,natm                                                       
        uold(1,1)=uold(1,1)+m(a)*(rtmp(2,a)**2+rtmp(3,a)**2)            
        uold(2,2)=uold(2,2)+m(a)*(rtmp(3,a)**2+rtmp(1,a)**2)            
        uold(3,3)=uold(3,3)+m(a)*(rtmp(1,a)**2+rtmp(2,a)**2)            
        uold(1,2)=uold(1,2)-m(a)*(rtmp(1,a)*rtmp(2,a))                  
        uold(2,3)=uold(2,3)-m(a)*(rtmp(2,a)*rtmp(3,a))                  
        uold(3,1)=uold(3,1)-m(a)*(rtmp(3,a)*rtmp(1,a))                  
      enddo                                                             
      uold(2,1)=uold(1,2)                                               
      uold(3,2)=uold(2,3)                                               
      uold(1,3)=uold(3,1)                                               
                                                                        
      call dsyev('V','U',3,uold,3,wold,work,lwork,info)                 
      if (info .ne. 0) go to 999                                        
!
! sort vectors (lowest first)
!
      if (wold(2) .lt. wold(1)) then                                    
        wtmp=wold(1)                                                    
        wold(1)=wold(2)                                                 
        wold(2)=wtmp                                                    
        do i=1,3                                                        
          utmp(i)=uold(i,1)                                             
          uold(i,1)=uold(i,2)                                           
          uold(i,2)=-utmp(i)                                            
        enddo                                                           
      endif                                                             
      if (wold(3) .lt. wold(1)) then                                    
        wtmp=wold(2)                                                    
        wold(2)=wold(1)                                                 
        wold(1)=wold(3)                                                 
        wold(3)=wtmp                                                    
        do i=1,3                                                        
          utmp(i)=uold(i,2)                                             
          uold(i,2)=uold(i,1)                                           
          uold(i,1)=uold(i,3)                                           
          uold(i,3)=utmp(i)                                             
        enddo                                                           
      endif                                                             
      if (wold(3) .lt. wold(2)) then                                    
        wtmp=wold(2)                                                    
        wold(2)=wold(3)                                                 
        wold(3)=wtmp                                                    
        do i=1,3                                                        
          utmp(i)=uold(i,2)                                             
          uold(i,2)=uold(i,3)                                           
          uold(i,3)=-utmp(i)                                            
        enddo                                                           
      endif                                                             
                                                                        
      if (dabs(wold(1)) .le. tol**2) then                               
        dimold=1                                                        
      elseif (dabs(wold(3)-wold(2)-wold(1)) .le. tol**2) then           
        dimold=2                                                        
      else                                                              
        dimold=3                                                        
      endif                                                             
                                                                        
      call zerovxd(rnew,3*natm)
      do a=1,natm                                                       
        do j=1,3                                                        
          do i=1,3                                                      
            rnew(j,a)=rnew(j,a)+rtmp(i,a)*uold(i,j)                     
          enddo                                                         
        enddo                                                           
        if (dimold .eq. 2) then                                         
          rnew(3,a)=0.0_dop                                                
        elseif (dimold .eq. 1) then                                     
          rnew(2,a)=0.0_dop                                                
          rnew(3,a)=0.0_dop                                                
        endif                                                           
      enddo                                                             
                                                                        
!-----------------------------------------------------------------------
! Define left-right orientation by convention:                          
! Larger part of inertia is on negative side of axis
!-----------------------------------------------------------------------
      if (dimold .gt. 1) then                                           
        s3=1                                                            
        j1=1                                                            
        j2=2                                                            
        j3=3                                                            
 10     continue                                                        
        ip=0.0_dop                                                         
        im=0.0_dop                                                         
        do a=1,natm                                                     
          if (rnew(j1,a) .ge. tol) then                                 
            ip=ip+m(a)*(rnew(j2,a)**2+rnew(j3,a)**2)                    
          else
            im=im+m(a)*(rnew(j2,a)**2+rnew(j3,a)**2)                    
          endif                                                         
        enddo                                                           
        if (ip-im .gt. tol**2) then                                     
          do i=1,3                                                      
            uold(i,j1)=-uold(i,j1)                                      
          enddo                                                         
          do a=1,natm                                                   
            rnew(j1,a)=-rnew(j1,a)                                      
          enddo                                                         
          s3=-s3                                                        
        elseif (dabs(ip-im) .le. tol**2) then                           
          ip=0.0_dop                                                       
          im=0.0_dop                                                       
          do a=1,natm                                                   
            if (rnew(j1,a) .ge. tol) then                               
              ip=ip+rnew(j2,a)**2+rnew(j3,a)**2                         
            else
              im=im+rnew(j2,a)**2+rnew(j3,a)**2                         
            endif                                                       
          enddo                                                         
          if (ip-im .gt. tol**2) then                                   
            do i=1,3                                                    
              uold(i,j1)=-uold(i,j1)                                    
            enddo                                                       
            do a=1,natm                                                 
              rnew(j1,a)=-rnew(j1,a)                                    
            enddo                                                       
            s3=-s3                                                      
          elseif (dabs(ip-im) .le. tol**2) then                         
            ip=0.0_dop                                                     
            im=0.0_dop                                                     
            do a=1,natm                                                 
              if (rnew(j1,a) .ge. tol) then                             
                ip=ip+m(a)*(rnew(j1,a)**2)                              
              else
                im=im+m(a)*(rnew(j1,a)**2)                              
              endif                                                     
            enddo                                                       
            if (ip-im .gt. tol**2) then                                 
              do i=1,3                                                  
                uold(i,j1)=-uold(i,j1)                                  
              enddo                                                     
              do a=1,natm                                               
                rnew(j1,a)=-rnew(j1,a)                                  
              enddo                                                     
              s3=-s3                                                    
            elseif (dabs(ip-im) .le. tol**2) then                       
              ip=0.0_dop                                                   
              im=0.0_dop                                                   
              do a=1,natm                                               
                if (rnew(j1,a) .ge. tol) then                           
                  ip=ip+rnew(j1,a)**2                                   
                else
                  im=im+rnew(j1,a)**2                                   
                endif                                                   
              enddo                                                     
              if (ip-im .gt. tol**2) then                               
                do i=1,3                                                
                  uold(i,j1)=-uold(i,j1)                                
                enddo                                                   
                do a=1,natm                                             
                  rnew(j1,a)=-rnew(j1,a)                                
                enddo                                                   
                s3=-s3                                                  
              endif                                                     
            endif                                                       
          endif                                                         
        endif                                                           
        if (j1 .eq. 1) then                                             
          j1=2                                                          
          j2=3                                                          
          j3=1                                                          
          go to 10                                                      
        elseif (j1 .eq. 2) then                                         
          if (dimold .eq. 3) then                                       
            j1=3                                                        
            j2=1                                                        
            j3=2                                                        
            go to 10                                                    
          elseif (dimold .eq. 2) then                                   
            if (s3 .eq. -1) then                                        
              do i=1,3                                                  
                uold(i,3)=-uold(i,3)                                    
              enddo                                                     
              do a=1,natm                                               
                rnew(3,a)=-rnew(3,a)                                    
              enddo                                                     
            endif                                                       
          endif                                                         
        endif                                                           
      elseif (dimold .eq. 1) then                                       
        s3=1                                                            
        j1=1                                                            
        j2=2                                                            
        j3=3                                                            
        ip=0.0_dop                                                         
        im=0.0_dop                                                         
        do a=1,natm                                                     
          if (rnew(j1,a) .gt. tol) then                                 
            ip=ip+m(a)*(rnew(j1,a)**2)                                  
          else
            im=im+m(a)*(rnew(j1,a)**2)                                  
          endif                                                         
        enddo                                                           
        if (ip-im .gt. tol**2) then                                     
          do i=1,3                                                      
            uold(i,j1)=-uold(i,j1)                                      
          enddo                                                         
          do a=1,natm                                                   
            rnew(j1,a)=-rnew(j1,a)                                      
          enddo                                                         
          s3=-s3                                                        
        elseif (dabs(ip-im) .le. tol**2) then                           
          ip=0.0_dop                                                       
          im=0.0_dop                                                       
          do a=1,natm                                                   
            if (rnew(j1,a) .gt. tol) then                               
              ip=ip+rnew(j1,a)**2                                       
            else
              im=im+rnew(j1,a)**2                                       
            endif                                                       
          enddo                                                         
          if (ip-im .gt. tol**2) then                                   
            do i=1,3                                                    
              uold(i,j1)=-uold(i,j1)                                    
            enddo                                                       
            do a=1,natm                                                 
              rnew(j1,a)=-rnew(j1,a)                                    
            enddo                                                       
            s3=-s3                                                      
          endif                                                         
        endif                                                           
        if (s3 .eq. -1) then                                            
          do i=1,3                                                      
            uold(i,3)=-uold(i,3)                                        
          enddo                                                         
          do a=1,natm                                                   
            rnew(3,a)=-rnew(3,a)                                        
          enddo                                                         
        endif                                                           
      endif                                                             
      
!-----------------------------------------------------------------------
! Transformation defined by                                             
!   geo_new(j,b) = sum_a,i geo_old(i,a) * transfo(i,a,j,b)              
!-----------------------------------------------------------------------
      call zerovxd(transfo,3*natm*3*natm)
      do j=1,3                                                          
        do i=1,3                                                        
          do a=1,natm                                                   
            transfo(i,a,j,a)=uold(i,j)                              
          enddo                                                         
        enddo                                                           
      enddo                                                             
                                                                        
      return                                                            
  999 routine='Prewash'                                                 
      message = &   
      'Matrix diagonalisation (dsyev) failure'                          
      call dberrormsg                                                     
      end subroutine prewash                   

!#######################################################################
!
! Eckart_reord
!
! INPUT
! natm: no. of atoms
! m(a): mass of atom a
! xcur(i): starting coordinates 
!
! On OUTPUT
! ddeckart(i,f): Eckart transformation matrix
! ddamap(i): mapping of atoms to reference frame
! rnew(i): contains transformed coordinates
!
! Atoms are ordered according to families
!#######################################################################

      subroutine eckart_reord(m,xcur,rnew,ddeckart)                      

      implicit none                                                     
                                                                        
      integer(long), parameter                       :: lwork=9
      real(dop), parameter                           :: tol=1.0e-6_dop

      integer(long)                                  :: a,b,ai,aj,afam,imaxi,&
                                                        jmaxi,imax,nval,i,j,k,& 
                                                        icount,istep,nstep,nm,&
                                                        jsum,dimold,icase,info  
      integer(long), dimension(3)                    :: jj,aa              
      integer(long), dimension(natm)                 :: ifam,jmax,j0,&
                                                        jsav,ival,jval,&
                                                        ipb,i0,ipb0,bb,&
                                                        jf,ddamap
      integer(long), dimension(natm,natm)            :: jfree,jm

      real(dop)                                      :: dsmax,pro,norm 
      real(dop), dimension(lwork)                    :: work
      real(dop), dimension(natm), intent(in)         :: m
      real(dop), dimension(natm)                     :: smax,ds,ssum,ssum0
      real(dop), dimension(3)                        :: w3
      real(dop), dimension(2)                        :: w2
      real(dop), dimension(3,3)                      :: v,u3,e3,f
      real(dop), dimension(2,2)                      :: u2,e2
      real(dop), dimension(3,natm), intent(in)       :: xcur
      real(dop), dimension(3,natm)                   :: rold
      real(dop), dimension(3,natm), intent(out)      :: rnew
      real(dop), dimension(natm,natm)                :: s
      real(dop), dimension(ndofddpes,ndofddpes), intent(out)  :: ddeckart
      real(dop), dimension(3,natm,3,natm)            :: t,u

!-----------------------------------------------------------------------
! Call prewash to get coordinates centered at the center of mass and           
! aligned with the principal axes of inertia.                           
! rold: rotated coordinates with atoms ordered according to mass
!
! check m is correct
!-----------------------------------------------------------------------
      call prewash_reord(xcur,rold,natm,m,dimold,t,nfam,ifam,ddamap) 
!-----------------------------------------------------------------------
! Sort equivalent nuclei through permutation                            
! and reduce the equivalence classes to single representatives.         
!-----------------------------------------------------------------------
!!!! Ignore until sure this works  GW 9/10 !!!
      go to 777

      b=0                                                               
      do afam=1,nfam                                                    
!-----------------------------------------------------------------------
! 0a. If a family is made up of more than one atom, find the best order.
!-----------------------------------------------------------------------
        if (ifam(afam) .gt. 1) then                                     
!-----------------------------------------------------------------------
! 1. Calculate the overlap with the reference geometry:                 
!-----------------------------------------------------------------------
          call zerovxd(s,(ndofddpes/3)*(ndofddpes/3))
          call zerovxi(jfree,(ndofddpes/3)*(ndofddpes/3))
          call zerovxi(jmax,natm)
          call zerovxd(smax,natm)
          ai=b                                                          
          do i=1,ifam(afam)                                             
            ai=ai+1                                                     
            aj=b                                                        
            do j=1,ifam(afam)                                           
              aj=aj+1                                                   
              do k=1,3                                                  
                s(j,i)=s(j,i)+rold(k,aj)*xref(k,ai)                     
              enddo                                                     
              s(j,i)=1.0e-12_dop*anint(s(j,i)*1.0e12_dop)                         
              jfree(j,i)=1                                              
            enddo                                                       
          enddo                                                         
!-----------------------------------------------------------------------
! 1. End.                                                               
! 2. Find the closest atom to each atom of the reference geometry:      
!-----------------------------------------------------------------------
          do i=1,ifam(afam)                                             
            jmax(i)=1                                                   
            smax(i)=s(1,i)                                              
            do j=2,ifam(afam)                                           
              if (s(j,i) .gt. smax(i)) then                             
                jmax(i)=j                                               
                smax(i)=s(j,i)                                          
              endif                                                     
            enddo                                                       
            jfree(jmax(i),i)=0                                          
          enddo                                                         
          call cpvxi(jmax,j0,natm)
          icount=0                                                      
          istep=0                                                       
          call zerovxi(i0,natm)
          call zerovxd(ssum0,natm)
!-----------------------------------------------------------------------
! 2. End.                                                               
! 3. Check whether they are redundant or not:                           
!-----------------------------------------------------------------------
 10       continue                                                      
          call zerovxi(jval,natm)
          call zerovxi(ival,natm)
          nval=1                                                        
          ival(1)=1                                                     
          jval(1)=jmax(1)                                               
          do a=2,ifam(afam)                                             
            do i=1,nval                                                 
              if (jmax(a) .eq. jval(i)) then                            
                ival(i)=ival(i)+1                                       
                go to 20                                                
              endif                                                     
              ipb(a)=0                                                  
            enddo                                                       
            nval=nval+1                                                 
            ival(nval)=1                                                
            jval(nval)=jmax(a)                                          
 20         continue                                                    
          enddo                                                         
          call zerovxi(ipb,natm)
          do a=1,ifam(afam)                                             
            do i=1,nval                                                 
              if (jmax(a) .eq. jval(i)) then                            
                if (ival(i) .gt. 1) ipb(a)=1                            
              endif                                                     
            enddo                                                       
          enddo                                                         
          if (istep .eq. 0) then                                        
            nstep=0                                                     
            call cpvxi(ipb,ipb0,natm)
            do a=1,ifam(afam)                                           
              nstep=nstep+ipb(a)                                        
            enddo                                                       
            if (nstep .gt. 0) istep=1                                   
          endif                                                         
!-----------------------------------------------------------------------
! 3. End.                                                               
! 4a. Yes, they are: change an atom and come back to test 3:            
!-----------------------------------------------------------------------
          if (nval .lt. ifam(afam)) then                                
!-----------------------------------------------------------------------
! 5a. Start with the best change (step 1), then the 2nd best (step 2),  
!     for each of the redundant atoms fund at the beginning:            
!-----------------------------------------------------------------------
            if (istep .lt. nstep) then                                  
!-----------------------------------------------------------------------
! 6a. Change atom by atom until they aren't redundant any longer:       
!-----------------------------------------------------------------------
              if (icount .lt. ifam(afam)*(ifam(afam)-1)) then           
                icount=icount+1                                         
!-----------------------------------------------------------------------
! 6b. If they are still redundant, change step:                         
!-----------------------------------------------------------------------
              elseif (icount .eq. ifam(afam)*(ifam(afam)-1)) then       
                call cpvxi(j0,jmax,natm)
                call cpvxi(ipb0,ipb,natm)
                a=0                                                     
                do i=1,ifam(afam)                                       
                  do j=1,ifam(afam)                                     
                    jfree(j,i)=1                                        
                  enddo                                                 
                  jfree(jmax(i),i)=0                                    
                  smax(i)=s(jmax(i),i)                                  
                  if (ipb(i) .eq. 1) a=a+1                              
                  if (a .eq. istep) i0(i)=1                             
                enddo                                                   
                icount=1                                                
                istep=istep+1                                           
!-----------------------------------------------------------------------
! 6. End.                                                               
!-----------------------------------------------------------------------
              endif                                                     
!-----------------------------------------------------------------------
! 5b. Last step. If after that they are still redundant, stop:          
!-----------------------------------------------------------------------
            elseif (istep .eq. nstep) then                              
              if (icount .lt. ifam(afam)*(ifam(afam)-1)) then           
                icount=icount+1                                         
              elseif (icount .eq. ifam(afam)*(ifam(afam)-1)) then       
                print*,'Impossible case'                                
                stop                                                    
              endif                                                     
!-----------------------------------------------------------------------
! 5. End.                                                               
!-----------------------------------------------------------------------
            endif                                                       
            if (icount .eq. 1) then                                     
              do i=1,ifam(afam)                                         
                if (i0(i) .eq. 1) ipb(i)=0                              
              enddo                                                     
            endif                                                       
            call zerovxd(ds,natm)
            call cpvxi(jmax,jsav,natm)
            do i=1,ifam(afam)                                           
              if (ipb(i) .eq. 1) then                                   
                jmaxi=1                                                 
                do j=1,ifam(afam)                                       
                  if (jfree(j,i) .eq. 1) then                           
                    jmaxi=j                                             
                    go to 30                                            
                  endif                                                 
                enddo                                                   
 30             continue                                                
                jmax(i)=jmaxi                                           
                smax(i)=s(jmaxi,i)                                      
                do j=jmaxi,ifam(afam)                                   
                  if (jfree(j,i) .eq. 1) then                           
                    if (s(j,i) .gt. smax(i)) then                       
                      jmax(i)=j                                         
                      smax(i)=s(j,i)                                    
                    endif                                               
                  endif                                                 
                enddo                                                   
              endif                                                     
              ds(i)=s(jmax(i),i)-s(j0(i),i)                             
            enddo                                                       
            imaxi=1                                                     
            do i=1,ifam(afam)                                           
              if (ipb(i) .eq. 1) then                                   
                imaxi=i                                                 
                go to 40                                                
              endif                                                     
            enddo                                                       
 40         continue                                                    
            imax=imaxi                                                  
            dsmax=ds(imaxi)                                             
            do i=imaxi,ifam(afam)                                       
              if (ipb(i) .eq. 1) then                                   
                if (ds(i) .gt. dsmax) then                              
                  imax=i                                                
                  dsmax=ds(i)                                           
                endif                                                   
              endif                                                     
            enddo                                                       
            jsav(imax)=jmax(imax)                                       
            call cpvxi(jsav,jmax,natm)
            do i=1,ifam(afam)                                           
              smax(i)=s(jmax(i),i)                                      
            enddo                                                       
            jfree(jmax(imax),imax)=0                                    
            go to 10                                                    
!-----------------------------------------------------------------------
! 4b. No, they aren't: check whether the order is to be kept:           
!-----------------------------------------------------------------------
          elseif (nval .eq. ifam(afam)) then                            
!-----------------------------------------------------------------------
! 7a. If they weren't from the beginning, the order is good.            
!-----------------------------------------------------------------------
            if (nstep .eq. 0) then                                      
              do i=1,ifam(afam)                                         
                bb(b+i)=b+jmax(i)                                       
              enddo                                                     
!-----------------------------------------------------------------------
! 7b. If there has been a change, try other possibilities, maybe better:
!-----------------------------------------------------------------------
            else                                                        
              do a=1,ifam(afam)                                         
                ssum(a)=0.0_dop                                            
                do i=1,ifam(afam)                                       
                  ssum(a)=ssum(a) &                       
                         +s(jmax(i),mod(i+a-2,ifam(afam))+1)            
                enddo                                                   
              enddo                                                     
              do a=1,ifam(afam)                                         
                if (ssum(a) .lt. ssum0(a)) then                         
                  go to 50                                              
                elseif (ssum(a) .gt. ssum0(a)) then                     
                  call cpvxd(ssum,ssum0,natm)
                  do i=1,ifam(afam)                                     
                    bb(b+i)=b+jmax(i)                                   
                  enddo                                                 
                  call cpvxi(jmax,jf,natm)
                  go to 50                                              
                endif                                                   
              enddo                                                     
 50           continue                                                  
!-----------------------------------------------------------------------
! 8a. Change step and come back to test 3:                              
!-----------------------------------------------------------------------
              if (istep .lt. nstep) then                                
                call cpvxi(j0,jmax,natm)
                call cpvxi(ipb0,ipb,natm)
                a=0                                                     
                do i=1,ifam(afam)                                       
                  do j=1,ifam(afam)                                     
                    jfree(j,i)=1                                        
                  enddo                                                 
                  jfree(jmax(i),i)=0                                    
                  smax(i)=s(jmax(i),i)                                  
                  if (ipb(i) .eq. 1) a=a+1                              
                  if (a .eq. istep) i0(i)=1                             
                enddo                                                   
                icount=0                                                
                istep=istep+1                                           
                go to 10                                                
!-----------------------------------------------------------------------
! 8b. Last step. If the order is still not good, stop:                  
!-----------------------------------------------------------------------
              elseif (istep .eq. nstep) then                            
                if (ssum0(1) .ne. 0.0_dop) then                            
!-----------------------------------------------------------------------
! 9. If there is degeneracy by symmetry, choose the best order possible:
!-----------------------------------------------------------------------
                  call cpvxi(jf,jmax,natm)
                  call zerovxi(jf,natm)
                  call zerovxi(jm,(ndofddpes/3)*(ndofddpes/3))
                  nm=0                                                  
                  do i=1,ifam(afam)                                     
                    do j=1,ifam(afam)                                   
                      if (s(j,i) .eq. s(jmax(i),i)) then                
                        jm(j,i)=1                                       
                        nm=nm+1                                         
                      endif                                             
                    enddo                                               
                  enddo                                                 
                  if (nm .gt. ifam(afam)) then                          
                    do k=1,ifam(afam)**ifam(afam)                   
                      j=k                                               
                      do i=1,ifam(afam)                                 
                        jf(jmax(i))=1                                   
                      enddo                                             
                      do i=ifam(afam),1,-1                              
                        jmax(i)=(j-1)/ifam(afam)**(i-1)+1             
                        j=mod(j-1,ifam(afam)**(i-1))+1                
                        jf(jmax(i))=0                                   
                      enddo                                             
                      jsum=0                                            
                      do i=1,ifam(afam)                                 
                        jsum=jsum+jf(i)                                 
                        if (jm(jmax(i),i) .eq. 0) go to 60              
                      enddo                                             
                      if (jsum .eq. 0) then                             
                        do a=2,ifam(afam)                               
                          ssum(a)=0.0_dop                                  
                          do i=1,ifam(afam)                             
                            ssum(a)=ssum(a) &               
                                   +s(jmax(i),mod(i+a-2,ifam(afam))+1)  
                          enddo                                         
                        enddo                                           
                        do a=2,ifam(afam)                               
                          if (ssum(a) .lt. ssum0(a)) then               
                            go to 60                                    
                          elseif (ssum(a) .gt. ssum0(a)) then           
                            call cpvxd(ssum,ssum0,natm)
                            do i=1,ifam(afam)                           
                              bb(b+i)=b+jmax(i)                         
                            enddo                                       
                            go to 60                                    
                          endif                                         
                        enddo                                           
                      endif                                             
 60                   continue                                          
                    enddo                                               
                  endif                                                 
!-----------------------------------------------------------------------
! 9. End.                                                               
!-----------------------------------------------------------------------
                else                                                    
                  print*,'Impossible case'                              
                  do i=1,ifam(afam)                           
                     bb(b+i)=b+i
                  enddo                                       
!                 stop                                                  
                endif                                                   
!-----------------------------------------------------------------------
! 8. End.                                                               
!-----------------------------------------------------------------------
              endif                                                     
!-----------------------------------------------------------------------
! 7. End.                                                               
!-----------------------------------------------------------------------
            endif                                                       
!-----------------------------------------------------------------------
! 4. End.                                                               
!-----------------------------------------------------------------------
          endif                                                         
!-----------------------------------------------------------------------
! 0b. If there is only one atom in the family, do nothing:              
!-----------------------------------------------------------------------
        else                                                            
          bb(b+1)=b+1                                                   
!-----------------------------------------------------------------------
! 0. End.                                                               
!-----------------------------------------------------------------------
        endif                                                           
        b=b+ifam(afam)                                                  
      enddo                                                             
                                                                        
      do a=1,natm                                                       
        do i=1,3                                                        
          rnew(i,a)=rold(i,bb(a))                                       
        enddo                                                           
      enddo                                                             
      call cpvxd(rnew,rold,3*natm)

  777 continue
      do a=1,natm
         bb(a)=a
      enddo
                                                                        
!-----------------------------------------------------------------------
! Evaluate the Eckart vectors (reciprocal vectors F_i's).               
!-----------------------------------------------------------------------
      call zerovxd(v,3*3)
      do j=1,3                                                          
        do i=1,3                                                        
          do a=1,natm                                                   
            v(i,j)=v(i,j)+mref(a)*xref(j,a)*rold(i,a)          
          enddo                                                         
        enddo                                                           
      enddo                                                             
                                                                        
!-----------------------------------------------------------------------
! Evaluate the number of independent vectors among the F_i's            
! and determine to which case they belong (3D, 2D, 1D or undefined).    
!-----------------------------------------------------------------------
      if (dimref .eq. 3 .and. dimold .eq. 3) then                       
        icase=3                                                         
      elseif (dimref .eq. 1 .or. dimold .eq. 1) then                    
        icase=1                                                         
      else                                                              
        icase=2                                                         
      endif                                                             
                                                                        
      if (icase .eq. 3) then                                            
        pro=v(1,1)*(v(2,2)*v(3,3)-v(3,2)*v(2,3)) &           
           +v(2,1)*(v(3,2)*v(1,3)-v(1,2)*v(3,3)) &                
           +v(3,1)*(v(1,2)*v(2,3)-v(2,2)*v(1,3))                        
        if (pro .le. tol**3) then                                       
          icase=2                                                       
          dimref=2                                                      
        endif                                                           
      endif                                                             
      if (icase .eq. 2) then                                            
        jj(1)=1                                                         
        jj(2)=2                                                         
        jj(3)=3                                                         
        pro=(v(2,1)*v(3,2)-v(3,1)*v(2,2))**2  &            
           +(v(3,1)*v(1,2)-v(1,1)*v(3,2))**2  &               
           +(v(1,1)*v(2,2)-v(2,1)*v(1,2))**2                            
        if (dabs(pro) .le. tol**4) then                                 
          jj(1)=2                                                       
          jj(2)=3                                                       
          jj(3)=1                                                       
          pro=(v(2,2)*v(3,3)-v(3,2)*v(2,3))**2 &          
             +(v(3,2)*v(1,3)-v(1,2)*v(3,3))**2 &         
             +(v(1,2)*v(2,3)-v(2,2)*v(1,3))**2                          
          if (dabs(pro) .le. tol**4) then                               
            jj(1)=3                                                     
            jj(2)=1                                                     
            jj(3)=2                                                     
            pro=(v(2,3)*v(3,1)-v(3,3)*v(2,1))**2 &          
               +(v(3,3)*v(1,1)-v(1,3)*v(3,1))**2 &           
               +(v(1,3)*v(2,1)-v(2,3)*v(1,1))**2                        
            if (dabs(pro) .le. tol**4) then                             
              icase=1                                                   
              dimref=1                                                  
            endif                                                       
          endif                                                         
        endif                                                           
      endif                                                             
      if (icase .eq. 1) then                                            
        jj(1)=1                                                         
        jj(2)=2                                                         
        jj(3)=3                                                         
        norm=0.0_dop                                                       
        do i=1,3                                                        
          norm=norm+v(i,1)**2                                           
        enddo                                                           
        if (norm .le. tol**2) then                                      
          jj(1)=2                                                       
          jj(2)=3                                                       
          jj(3)=1                                                       
          norm=0.0_dop                                                     
          do i=1,3                                                      
            norm=norm+v(i,2)**2                                         
          enddo                                                         
          if (norm .le. tol**2) then                                    
            jj(1)=3                                                     
            jj(2)=1                                                     
            jj(3)=2                                                     
            norm=0.0_dop                                                   
            do i=1,3                                                    
              norm=norm+v(i,3)**2                                       
            enddo                                                       
            if (norm .le. tol**2) then                                  
              icase=0                                                   
              dimref=0                                                  
            endif                                                       
          endif                                                         
        endif                                                           
      endif                                                             
                                                                        
!-----------------------------------------------------------------------
! Evaluate the Eckart frame (direct vectors f_i's)                      
! depending on the case (3D, 2D, 1D or undefined).                      
!-----------------------------------------------------------------------
      if (icase .eq. 3) then                                            
        call zerovxd(u3,3*3)
        do i=1,3                                                        
          do j=1,3                                                      
            do k=1,3                                                    
              u3(i,j)=u3(i,j)+v(k,i)*v(k,j)                             
            enddo                                                       
          enddo                                                         
        enddo                                                           
                                                                        
        call dsyev('V','U',3,u3,3,w3,work,lwork,info)                   
        if (info .ne. 0) go to 999                                      
                                                                        
        call zerovxd(e3,3*3)
        do i=1,3                                                        
          do j=1,3                                                      
            do k=1,3                                                    
              e3(i,j)=e3(i,j)+u3(i,k)*u3(j,k)/dsqrt(w3(k))              
            enddo                                                       
          enddo                                                         
        enddo                                                           
                                                                        
        call zerovxd(f,3*3)
        do i=1,3                                                        
          do j=1,3                                                      
            do k=1,3                                                    
              f(i,j)=f(i,j)+v(i,k)*e3(k,j)                              
            enddo                                                       
          enddo                                                         
        enddo                                                           
                                                                        
      elseif (icase .eq. 2) then                                        
        call zerovxd(u2,2*2)
        do i=1,2                                                        
          do j=1,2                                                      
            do k=1,3                                                    
              u2(i,j)=u2(i,j)+v(k,jj(i))*v(k,jj(j))                     
            enddo                                                       
          enddo                                                         
        enddo                                                           
                                                                        
        call dsyev('V','U',2,u2,2,w2,work,lwork,info)                   
        if (info .ne. 0) go to 999                                      
                                                                        
        call zerovxd(e2,2*2)
        do i=1,2                                                        
          do j=1,2                                                      
            do k=1,2                                                    
              e2(i,j)=e2(i,j)+u2(i,k)*u2(j,k)/dsqrt(w2(k))              
            enddo                                                       
          enddo                                                         
        enddo                                                           
                                                                        
        call zerovxd(f,3*3)
        do i=1,3                                                        
          do j=1,2                                                      
            do k=1,2                                                    
              f(i,jj(j))=f(i,jj(j))+v(i,jj(k))*e2(k,j)                  
            enddo                                                       
          enddo                                                         
        enddo                                                           
        f(1,jj(3))=f(2,jj(1))*f(3,jj(2))-f(3,jj(1))*f(2,jj(2))          
        f(2,jj(3))=f(3,jj(1))*f(1,jj(2))-f(1,jj(1))*f(3,jj(2))          
        f(3,jj(3))=f(1,jj(1))*f(2,jj(2))-f(2,jj(1))*f(1,jj(2))          
                                                                        
      elseif (icase .eq. 1) then                                        
        norm=0.0_dop                                                       
        do i=1,3                                                        
          norm=norm+v(i,jj(1))**2                                       
        enddo                                                           
        do i=1,3                                                        
          f(i,jj(1))=v(i,jj(1))/dsqrt(norm)                             
        enddo                                                           
        if (1.0_dop-f(3,jj(1))**2 .gt. tol**2) then                        
          f(1,jj(2))=-f(2,jj(1))/dsqrt(1.0_dop-f(3,jj(1))**2)              
          f(2,jj(2))=f(1,jj(1))/dsqrt(1.0_dop-f(3,jj(1))**2)               
        else                                                            
          f(1,jj(2))=1.0_dop                                               
          f(2,jj(2))=0.0_dop                                               
        endif                                                           
        f(3,jj(2))=0.0_dop                                                 
        f(1,jj(3))=-f(3,jj(1))*f(2,jj(2))                               
        f(2,jj(3))=f(3,jj(1))*f(1,jj(2))                                
        f(3,jj(3))=f(1,jj(1))*f(2,jj(2))-f(2,jj(1))*f(1,jj(2))          
                                                                        
      else                                                              
        call zerovxd(f,3*3)
        do i=1,3                                                        
          f(i,i)=1.0_dop                                                   
        enddo                                                           
      endif                                                             
                                                                        
!-----------------------------------------------------------------------
! When dim(ref) < dim(old), or for undefined case, undetermined axes    
! must be sign-oriented following certain conventions.                  
!-----------------------------------------------------------------------
      call zerovxd(rnew,3*natm)
      do a=1,natm                                                       
        do i=1,3                                                        
          do j=1,3                                                      
            rnew(i,a)=rnew(i,a)+rold(j,a)*f(j,i)                        
          enddo                                                         
        enddo                                                           
      enddo   
                                                                        
      do k=1,3                                                          
        if ((icase .eq. 0) .or. &                  
            (k .eq. jj(2) .and. dimref .eq. 1 .and. dimold .ge. 2) .or. &
            (k .eq. jj(3) .and. dimold .eq. 3 .and. dimref .le. 2)) then
          aa(k)=1                                                       
 70       continue                                                      
          if (dabs(rnew(k,aa(k))) .lt. tol) then                        
            if (aa(k) .lt. natm) then                                   
              aa(k)=aa(k)+1                                             
              go to 70                                                  
            endif                                                       
          endif                                                         
          if (rnew(k,aa(k)) .lt. 0.0_dop) then                             
            do i=1,3                                                    
              f(i,k)=-f(i,k)                                            
            enddo                                                       
            do a=1,natm                                                 
              rnew(k,a)=-rnew(k,a)                                      
            enddo                                                       
          endif                                                         
        endif                                                           
      enddo                                                             
                                                                        
!-----------------------------------------------------------------------
! Final transformation defined by                                       
!   geo_new(k,b) = sum_a,i geo_old(i,a) * u(i,a,k,b) and                
!   u(i,a,k,b) = sum_j t(i,a,j,b) * f(j,k)                              
!-----------------------------------------------------------------------
      call zerovxd(u,3*(ndofddpes/3)*3*(ndofddpes/3))
      do b=1,natm                                                       
        do k=1,3                                                        
          do a=1,natm                                                   
            do i=1,3                                                    
              do j=1,3                                                  
                u(i,a,k,bb(b))=u(i,a,k,bb(b))+t(i,a,j,b)*f(j,k)         
              enddo                                                     
            enddo                                                       
          enddo                                                         
        enddo                                                           
      enddo                                                             
      call zerovxd(ddeckart,ndofddpes*ndofddpes)
      do b=1,natm
        do k=1,3
          do a=1,natm
            do i=1,3
              ddeckart(3*(a-1)+i,3*(b-1)+k)=u(i,a,k,b)
            enddo
          enddo
        enddo
      enddo

      return                                                            
  999 routine='Eckart_reord'                                                  
      message = &   
      'Matrix diagonalisation (dsyev) failure'                          
      call dberrormsg                                                     
      end subroutine eckart_reord   
                                                                        
!-----------------------------------------------------------------------
! PREWASH_REORD
!
! translates molecule to COM and rotates molecule to lie along
! inertial axes.
!
! natm: no. atoms
! rold(3,natm): initial coordinates 
! m(a): mass of atom a
!
! rnew(3,natm): on output rotated coordinates
! transfo(3,natm,3,natm): transformation matrix rnew = rold*transfo
!
! on output atoms have been ordered into families by mass.
!-----------------------------------------------------------------------

      subroutine prewash_reord(rold,rnew,natm,m,dimold,transfo,nfam,ifam,aa)

      implicit none                                                     
                                                                        
      integer(long), parameter                    :: lwork = 9
      real(dop), parameter                        :: tol = 1.0e-6_dop     

      integer(long)                               :: a,i,j,info,j1,j2,j3,s3,itmp,ai,afam
      integer(long), intent(in)                   :: natm
      integer(long), intent(out)                  :: dimold,nfam
      integer(long), dimension(natm), intent(out) :: ifam,aa

      real(dop)                                                :: mtot,wtmp,ip,im,mtmp
      real(dop), dimension(lwork)                              :: work
      real(dop), dimension(natm), intent(in)                   :: m
      real(dop), dimension(natm)                           :: mfam
      real(dop), dimension(3)                                  :: utmp,bold,wold
      real(dop), dimension(3,3)                                :: uold
      real(dop), dimension(3,natm), intent(in)                 :: rold
      real(dop), dimension(3,natm), intent(out)                :: rnew
      real(dop), dimension(3,natm)                         :: rtmp
      real(dop), dimension(3,natm,3,natm), intent(out)     :: transfo
                                                                        
!-----------------------------------------------------------------------
! Center the old geometry at its center of mass.                        
! bold is COM
!-----------------------------------------------------------------------
      mtot=0.0_dop                                                         
      do a=1,natm                                                       
        mtot=mtot+m(a)                                                  
      enddo                                                             
      call zerovxd(bold,3)
      do i=1,3                                                          
        bold(i)=0.0_dop                                                    
        do a=1,natm                                                     
          bold(i)=bold(i)+m(a)*rold(i,a)                                
        enddo                                                           
        bold(i)=bold(i)/mtot                                            
      enddo                                                             
                                                                        
      do a=1,natm                                                       
        do i=1,3                                                        
          rtmp(i,a)=rold(i,a)-bold(i)                                   
        enddo                                                           
      enddo                                                             
                                                                        
!-----------------------------------------------------------------------
! Check if linear, planar or not.                                       
! Evaluate its principal axes of inertia and align it with them.        
!-----------------------------------------------------------------------
      call zerovxd(uold,3*3)
      do a=1,natm                                                       
        uold(1,1)=uold(1,1)+m(a)*(rtmp(2,a)**2+rtmp(3,a)**2)            
        uold(2,2)=uold(2,2)+m(a)*(rtmp(3,a)**2+rtmp(1,a)**2)            
        uold(3,3)=uold(3,3)+m(a)*(rtmp(1,a)**2+rtmp(2,a)**2)            
        uold(1,2)=uold(1,2)-m(a)*(rtmp(1,a)*rtmp(2,a))                  
        uold(2,3)=uold(2,3)-m(a)*(rtmp(2,a)*rtmp(3,a))                  
        uold(3,1)=uold(3,1)-m(a)*(rtmp(3,a)*rtmp(1,a))                  
      enddo                                                             
      uold(2,1)=uold(1,2)                                               
      uold(3,2)=uold(2,3)                                               
      uold(1,3)=uold(3,1)                                               
                                                                        
      call dsyev('V','U',3,uold,3,wold,work,lwork,info)                 
      if (info .ne. 0) go to 999                                        
!
! sort vectors (lowest first)
!
      if (wold(2) .lt. wold(1)) then                                    
        wtmp=wold(1)                                                    
        wold(1)=wold(2)                                                 
        wold(2)=wtmp                                                    
        do i=1,3                                                        
          utmp(i)=uold(i,1)                                             
          uold(i,1)=uold(i,2)                                           
          uold(i,2)=-utmp(i)                                            
        enddo                                                           
      endif                                                             
      if (wold(3) .lt. wold(1)) then                                    
        wtmp=wold(2)                                                    
        wold(2)=wold(1)                                                 
        wold(1)=wold(3)                                                 
        wold(3)=wtmp                                                    
        do i=1,3                                                        
          utmp(i)=uold(i,2)                                             
          uold(i,2)=uold(i,1)                                           
          uold(i,1)=uold(i,3)                                           
          uold(i,3)=utmp(i)                                             
        enddo                                                           
      endif                                                             
      if (wold(3) .lt. wold(2)) then                                    
        wtmp=wold(2)                                                    
        wold(2)=wold(3)                                                 
        wold(3)=wtmp                                                    
        do i=1,3                                                        
          utmp(i)=uold(i,2)                                             
          uold(i,2)=uold(i,3)                                           
          uold(i,3)=-utmp(i)                                            
        enddo                                                           
      endif                                                             
                                                                        
      if (dabs(wold(1)) .le. tol**2) then                               
        dimold=1                                                        
      elseif (dabs(wold(3)-wold(2)-wold(1)) .le. tol**2) then           
        dimold=2                                                        
      else                                                              
        dimold=3                                                        
      endif                                                             
                                                                        
      call zerovxd(rnew,3*natm)
      do a=1,natm                                                       
        do j=1,3                                                        
          do i=1,3                                                      
            rnew(j,a)=rnew(j,a)+rtmp(i,a)*uold(i,j)                     
          enddo                                                         
        enddo                                                           
        if (dimold .eq. 2) then                                         
          rnew(3,a)=0.0_dop                                                
        elseif (dimold .eq. 1) then                                     
          rnew(2,a)=0.0_dop                                                
          rnew(3,a)=0.0_dop                                                
        endif                                                           
      enddo                                                             
                                                                        
!-----------------------------------------------------------------------
! Define left-right orientation by convention:                          
! Larger part of inertia is on negative side of axis
!-----------------------------------------------------------------------
      if (dimold .gt. 1) then                                           
        s3=1                                                            
        j1=1                                                            
        j2=2                                                            
        j3=3                                                            
 10     continue                                                        
        ip=0.0_dop                                                         
        im=0.0_dop                                                         
        do a=1,natm                                                     
          if (rnew(j1,a) .ge. tol) then                                 
            ip=ip+m(a)*(rnew(j2,a)**2+rnew(j3,a)**2)                    
          else
            im=im+m(a)*(rnew(j2,a)**2+rnew(j3,a)**2)                    
          endif                                                         
        enddo                                                           
        if (ip-im .gt. tol**2) then                                     
          do i=1,3                                                      
            uold(i,j1)=-uold(i,j1)                                      
          enddo                                                         
          do a=1,natm                                                   
            rnew(j1,a)=-rnew(j1,a)                                      
          enddo                                                         
          s3=-s3                                                        
        elseif (dabs(ip-im) .le. tol**2) then                           
          ip=0.0_dop                                                       
          im=0.0_dop                                                       
          do a=1,natm                                                   
            if (rnew(j1,a) .ge. tol) then                               
              ip=ip+rnew(j2,a)**2+rnew(j3,a)**2                         
            else
              im=im+rnew(j2,a)**2+rnew(j3,a)**2                         
            endif                                                       
          enddo                                                         
          if (ip-im .gt. tol**2) then                                   
            do i=1,3                                                    
              uold(i,j1)=-uold(i,j1)                                    
            enddo                                                       
            do a=1,natm                                                 
              rnew(j1,a)=-rnew(j1,a)                                    
            enddo                                                       
            s3=-s3                                                      
          elseif (dabs(ip-im) .le. tol**2) then                         
            ip=0.0_dop                                                     
            im=0.0_dop                                                     
            do a=1,natm                                                 
              if (rnew(j1,a) .ge. tol) then                             
                ip=ip+m(a)*(rnew(j1,a)**2)                              
              else
                im=im+m(a)*(rnew(j1,a)**2)                              
              endif                                                     
            enddo                                                       
            if (ip-im .gt. tol**2) then                                 
              do i=1,3                                                  
                uold(i,j1)=-uold(i,j1)                                  
              enddo                                                     
              do a=1,natm                                               
                rnew(j1,a)=-rnew(j1,a)                                  
              enddo                                                     
              s3=-s3                                                    
            elseif (dabs(ip-im) .le. tol**2) then                       
              ip=0.0_dop                                                   
              im=0.0_dop                                                   
              do a=1,natm                                               
                if (rnew(j1,a) .ge. tol) then                           
                  ip=ip+rnew(j1,a)**2                                   
                else
                  im=im+rnew(j1,a)**2                                   
                endif                                                   
              enddo                                                     
              if (ip-im .gt. tol**2) then                               
                do i=1,3                                                
                  uold(i,j1)=-uold(i,j1)                                
                enddo                                                   
                do a=1,natm                                             
                  rnew(j1,a)=-rnew(j1,a)                                
                enddo                                                   
                s3=-s3                                                  
              endif                                                     
            endif                                                       
          endif                                                         
        endif                                                           
        if (j1 .eq. 1) then                                             
          j1=2                                                          
          j2=3                                                          
          j3=1                                                          
          go to 10                                                      
        elseif (j1 .eq. 2) then                                         
          if (dimold .eq. 3) then                                       
            j1=3                                                        
            j2=1                                                        
            j3=2                                                        
            go to 10                                                    
          elseif (dimold .eq. 2) then                                   
            if (s3 .eq. -1) then                                        
              do i=1,3                                                  
                uold(i,3)=-uold(i,3)                                    
              enddo                                                     
              do a=1,natm                                               
                rnew(3,a)=-rnew(3,a)                                    
              enddo                                                     
            endif                                                       
          endif                                                         
        endif                                                           
      elseif (dimold .eq. 1) then                                       
        s3=1                                                            
        j1=1                                                            
        j2=2                                                            
        j3=3                                                            
        ip=0.0_dop                                                         
        im=0.0_dop                                                         
        do a=1,natm                                                     
          if (rnew(j1,a) .gt. tol) then                                 
            ip=ip+m(a)*(rnew(j1,a)**2)                                  
          else
            im=im+m(a)*(rnew(j1,a)**2)                                  
          endif                                                         
        enddo                                                           
        if (ip-im .gt. tol**2) then                                     
          do i=1,3                                                      
            uold(i,j1)=-uold(i,j1)                                      
          enddo                                                         
          do a=1,natm                                                   
            rnew(j1,a)=-rnew(j1,a)                                      
          enddo                                                         
          s3=-s3                                                        
        elseif (dabs(ip-im) .le. tol**2) then                           
          ip=0.0_dop                                                       
          im=0.0_dop                                                       
          do a=1,natm                                                   
            if (rnew(j1,a) .gt. tol) then                               
              ip=ip+rnew(j1,a)**2                                       
            else
              im=im+rnew(j1,a)**2                                       
            endif                                                       
          enddo                                                         
          if (ip-im .gt. tol**2) then                                   
            do i=1,3                                                    
              uold(i,j1)=-uold(i,j1)                                    
            enddo                                                       
            do a=1,natm                                                 
              rnew(j1,a)=-rnew(j1,a)                                    
            enddo                                                       
            s3=-s3                                                      
          endif                                                         
        endif                                                           
        if (s3 .eq. -1) then                                            
          do i=1,3                                                      
            uold(i,3)=-uold(i,3)                                        
          enddo                                                         
          do a=1,natm                                                   
            rnew(3,a)=-rnew(3,a)                                        
          enddo                                                         
        endif                                                           
      endif                                                             
      
      call cpmxd(rnew,rtmp,3,natm)
                                                                        
!-----------------------------------------------------------------------
! Sort nuclei by masses.                                                
!-----------------------------------------------------------------------
      call zerovxd(mfam,natm)
      call zerovxi(ifam,natm)
      nfam=1                                                            
      ifam(1)=1                                                         
      mfam(1)=m(1)                                                      
      if (natm .gt. 1) then                                             
        do a=2,natm                                                     
          do afam=1,nfam                                                
            if (m(a) .eq. mfam(afam)) then                              
              ifam(afam)=ifam(afam)+1                                   
              go to 20                                                  
            endif                                                       
          enddo                                                         
          nfam=nfam+1                                                   
          ifam(nfam)=1                                                  
          mfam(nfam)=m(a)                                               
 20       continue                                                      
        enddo                                                           
      endif                                                             
                                                                        
      if (nfam .gt. 1) then                                             
        do afam=1,nfam-1                                                
          do a=afam+1,nfam                                              
            if (mfam(a) .gt. mfam(afam)) then                           
              mtmp=mfam(a)                                              
              mfam(a)=mfam(afam)                                        
              mfam(afam)=mtmp                                           
              itmp=ifam(a)                                              
              ifam(a)=ifam(afam)                                        
              ifam(afam)=itmp                                           
            endif                                                       
          enddo                                                         
        enddo                                                           
      endif                                                             
                                                                        
      ai=0                                                              
      do afam=1,nfam                                                    
        do a=1,natm                                                     
          if (m(a) .eq. mfam(afam)) then                                
            ai=ai+1                                                     
            aa(ai)=a                                                    
          endif                                                         
        enddo                                                           
      enddo                                                             
                                                                        
      do a=1,natm                                                       
        do i=1,3                                                        
          rnew(i,a)=rtmp(i,aa(a))                                       
        enddo                                                           
      enddo                                                             
                                                                        
!-----------------------------------------------------------------------
! Transformation defined by                                             
!   geo_new(j,b) = sum_a,i geo_old(i,a) * transfo(i,a,j,b)              
!-----------------------------------------------------------------------
      call zerovxd(transfo,3*natm*3*natm)
      do j=1,3                                                          
        do i=1,3                                                        
          do a=1,natm                                                   
            transfo(i,a,j,aa(a))=uold(i,j)                              
          enddo                                                         
        enddo                                                           
      enddo                                                             
                                                                        
      return                                                            
  999 routine='Prewash_reord'                                                 
      message = &   
      'Matrix diagonalisation (dsyev) failure'                          
      call dberrormsg                                                     
      end subroutine prewash_reord             

!#######################################################################

      subroutine rdrefdb

use griddatmod, only: nddstate
use dirdyn, only: natm, nroot, nbasis, dde0, dimref, nfam, &
                  mref, atnam, xref, dercpdim, dbpntgp, nsymop, &
                  dbsymmap, nsocstate

      implicit none

! loads all these into the globals from dirdyn and griddatmod
      call dbrdrefdb(the_db, natm, nroot, nddstate, nbasis, dde0, dimref, &
           nfam, dercpdim,dbpntgp,nsymop,nsocstate,nddmult)
      call dbrdrefdbgeom(the_db, natm, mref, atnam, xref)
      if (dbpntgp .gt. 0) then
         call dbrdrefdbmaps(the_db,natm,nsymop,dbsymmap)
      endif

      end subroutine rdrefdb

!#######################################################################
! read refdb.dat file for system sizes that can be used to allocate memory
!#######################################################################

      subroutine rdrefdbmem

      implicit none

      call dbrdrefdb(the_db,natm,nroot,nddstate,nbasis,dde0,dimref, &
           nfam,dercpdim,dbpntgp,nsymop,nsocstate,nddmult)

      end subroutine rdrefdbmem

!#######################################################################
! Note: end of what needs to be improved.
!#######################################################################

      subroutine rmcom(xgpoint,xmass,natm)

      implicit none

      integer(long)                               :: a,i
      integer(long), intent(in)                   :: natm
      real(dop)                                   :: mtot
      real(dop), dimension(3)                     :: com
      real(dop), dimension(natm), intent(in)      :: xmass
      real(dop), dimension(3,natm), intent(inout) :: xgpoint

!-----------------------------------------------------------------------
! Center the old geometry at its center of mass.
!-----------------------------------------------------------------------
      mtot=0.0_dop
      do a=1,natm
        mtot=mtot+xmass(a)
      enddo
      call zerovxd(com,3)
      do i=1,3
        com(i)=0.0_dop
        do a=1,natm
          com(i)=com(i)+xmass(a)*xgpoint(i,a)
        enddo
        com(i)=com(i)/mtot
      enddo

      do a=1,natm
        do i=1,3
          xgpoint(i,a)=xgpoint(i,a)-com(i)
        enddo
      enddo

      return
      end subroutine rmcom

!#######################################################################

      subroutine ddq2x(gpoint,xgpoint)

      implicit none

      integer(long)                                :: i,n
      real(dop), dimension(ndofddpes), intent(out) :: xgpoint
      real(dop), dimension(ndofdd), intent(in)     :: gpoint

      do i=1,ndofddpes
         xgpoint(i)=ddxcoo0(i)
         do n=1,ndofdd
            xgpoint(i)=xgpoint(i)+ddtransf(i,n)*gpoint(n)
         enddo
      enddo

      return
      end subroutine ddq2x

!#######################################################################

      subroutine ddq2x_diff(gpoint,xgpoint)

      implicit none

      integer(long)                                :: i,n,gdof
      real(dop), dimension(ndofddpes), intent(out) :: xgpoint
      real(dop), dimension(ndofdd), intent(in)     :: gpoint

      gdof = nspfdof(1)

      do i=1,ndofddpes
         xgpoint(i)=0.0_dop
         do n=1,ndofdd
            xgpoint(i)=xgpoint(i)+ddtransf(i,n)*gpoint(n)
         enddo
      enddo

      return
      end subroutine ddq2x_diff

!#######################################################################

      subroutine ddx2q(xgpoint,gpoint)

      implicit none

      integer(long)                                :: i,n
      real(dop), dimension(ndofddpes), intent(in)  :: xgpoint
      real(dop), dimension(ndofdd), intent(out)    :: gpoint
      real(dop), dimension(ndofddpes)              :: work

      do i=1,ndofddpes
         work(i) = xgpoint(i)-ddxcoo0(i)
      enddo

      do i=1,ndofdd
         gpoint(i)=0.0_dop
         do n=1,ndofddpes
            gpoint(i)=gpoint(i)+ddtransb(i,n)*work(n)
         enddo
      enddo

      return
      end subroutine ddx2q

!#######################################################################
! Transforms a vector between two DB points in Cartesian coordinates 
! into a difference vector in normal modes.
!#######################################################################

      subroutine ddx2q_diff(xgpoint,gpoint)

use global
use griddatmod
use operdef
use dirdyn

      implicit none

      integer(long)                                :: i,n
      real(dop), dimension(ndofddpes), intent(in)  :: xgpoint
      real(dop), dimension(ndofdd), intent(out)    :: gpoint

      gpoint=0.0_dop
      do i=1,ndofdd
         do n=1,ndofddpes
            gpoint(i)=gpoint(i)+ddtransb(i,n)*xgpoint(n)
         enddo
      enddo

      return
      end subroutine ddx2q_diff

!#######################################################################
! DDPES2Q
!
! transforms derivatives from cartesian to normal modes
! ORDER parameter redundant??
!#######################################################################

      subroutine ddpes2q(deriv1,deriv2,order)

      implicit none

      integer(long)                                                 :: g,g1,s,s1
      integer(long), intent(in)                                     :: order
      real(dop), dimension(ndofddpes)                               :: temp1
      real(dop), dimension(ndofddpes,ndofddpes)                     :: temp2
      real(dop), dimension(ndofddpes,nddstate,nddstate), &
                                                       intent(inout) :: deriv1
      real(dop), dimension(ndofddpes,ndofddpes,nddstate,nddstate), &
                                                       intent(inout) :: deriv2

      if (order .lt. 0 .or. order .gt. 2) then
         routine = 'ddpes2q'
         message = 'Order parameter undefined!!'
         call errormsg
      endif

      if (order .ge. 1) then
         do s=1,nddstate
            do s1=s,nddstate
               call cpvxd(deriv1(1,s1,s),temp1,ndofddpes) 
               call zerovxd(deriv1(1,s1,s),ndofddpes) 

               call ddf2q(temp1,deriv1(1,s1,s))

               do g=1,ndofdd
                  if (dofspf(g) > nmode) deriv1(g,s1,s)=0.0_dop
               enddo

               if (s .ne. s1) &
                  call cpvxd(deriv1(1,s1,s),deriv1(1,s,s1),ndofddpes) 
            enddo
         enddo
      endif

      if (order .ge. 2) then
         do s=1,nddstate
            do s1=s,nddstate
               call cpqxd1(deriv2(1,1,s1,s),temp2,ndofddpes,ndofddpes) 
               call zeromxd(deriv2(1,1,s1,s),ndofddpes,ndofddpes) 

               call ddfc2q(temp2,deriv2(1,1,s1,s))

               do g=1,ndofdd
                  if (dofspf(g) > nmode) then  
                     do g1=1,ndofdd
                        deriv2(g1,g,s1,s)=0.0_dop
                     enddo
                  else
                     do g1=1,ndofdd
                        if (dofspf(g1) > nmode) deriv2(g1,g,s1,s)=0.0_dop
                     enddo
                  endif
               enddo

               if (s .ne. s1) &
                  call cpqxd1(deriv2(1,1,s1,s),deriv2(1,1,s,s1),ndofddpes,&
                       ndofddpes) 

            enddo
         enddo
      endif

      return
      end subroutine ddpes2q

!#######################################################################
! Routine which transforms a force from Cartesian to normal mode coords.
!#######################################################################

      subroutine ddf2q(xforce,force)

      implicit none

      integer(long)                                   :: i,n
      real(dop), dimension(ndofddpes), intent(in)     :: xforce
      real(dop), dimension(ndofdd), intent(out)       :: force

      do i=1,ndofdd
         force(i)=0.0_dop
         do n=1,ndofddpes
            force(i)=force(i)+ddtransf(n,i)*xforce(n)
         enddo
      enddo

      return
      end subroutine ddf2q

!#######################################################################
! Routine which transforms a force from normal mode to Cartesian coords.
!#######################################################################

      subroutine ddf2x(force,xforce)

      implicit none

      integer(long)                                   :: i,n
      real(dop), dimension(ndofddpes), intent(out)    :: xforce
      real(dop), dimension(ndofdd), intent(in)        :: force

      do i=1,ndofddpes
         xforce(i)=0.0_dop
         do n=1,ndofdd
            xforce(i)=xforce(i)+ddtransb(n,i)*force(n)
         enddo
      enddo

      return
      end subroutine ddf2x

!#######################################################################
! 
! routine that transformd xdot to qdot
!
!#######################################################################

      subroutine ddv2p(xmom,qmom)

      implicit none

      integer(long)                                   :: i,n
      real(dop), dimension(ndofddpes), intent(in)     :: xmom
      real(dop), dimension(ndofdd), intent(out)       :: qmom

      do i=1,ndofdd
         qmom(i)=0.0_dop
         do n=1,ndofddpes
            qmom(i)=qmom(i)+ddtransb(i,n)*xmom(n)
         enddo
      enddo

      return
      end subroutine ddv2p

!#######################################################################
!
! DDfc2q
!
! transforms force constants from Cartesian to internal
!
!#######################################################################

      subroutine ddfc2q(xderiv2,deriv2)

      implicit none

      integer(long)                                    :: i,j,k,l
      real(dop), dimension(ndofddpes,ndofddpes), intent(in)  :: xderiv2
      real(dop), dimension(ndofddpes,ndofddpes), intent(out) :: deriv2
      real(dop), dimension(ndofddpes,ndofdd) :: tmp

      do l=1,ndofddpes
        do i=1,ndofdd
          tmp(l,i) = 0.0_dop
          do k=1,ndofddpes
            tmp(l,i) = tmp(l,i) + xderiv2(l,k)*ddtransf(k,i)
          enddo
        enddo
      enddo
      !
      do i=1,ndofdd
        do j=i,ndofdd
          deriv2(j,i) = 0.0_dop
          do l=1,ndofddpes
            deriv2(j,i) = deriv2(j,i) + ddtransf(l,j)*tmp(l,i)
          enddo
        enddo
      enddo
      !
      do i=1,ndofdd
         do j=i+1,ndofdd
            deriv2(i,j)= deriv2(j,i)
         enddo
      enddo

      return
      end subroutine ddfc2q

!#######################################################################

      subroutine ddpes2x(deriv1,deriv2,ddeckart,order)

      implicit none

      integer(long)                                         :: i,f,f1,i1,s,s1
      integer(long), intent(in)                             :: order
      real(dop), dimension(ndofddpes,ndofddpes), intent(in) :: ddeckart
      real(dop), dimension(ndofddpes,nddstate,nddstate)     :: der1
      real(dop), dimension(ndofddpes,nddstate,nddstate), intent(inout) :: deriv1
      real(dop), dimension(ndofddpes,ndofddpes,nddstate,nddstate)      :: der2
      real(dop), dimension(ndofddpes,ndofddpes,nddstate,nddstate), intent(inout) :: deriv2

      if (order .ge. 1) then
         do s=1,nddstate
            do s1=s,nddstate
               do f=1,ndofddpes
                  der1(f,s1,s)=0.0_dop
                  do i=1,ndofddpes
                     der1(f,s1,s)=der1(f,s1,s)+ &
                          deriv1(i,s1,s)*ddeckart(f,i)
                  enddo
               enddo
            enddo
         enddo
! now copy result back to deriv1 and fill in upper diagonal
         do s=1,nddstate
            do f=1,ndofddpes
               deriv1(f,s,s)=der1(f,s,s)
            enddo

            do s1=s+1,nddstate
               do f=1,ndofddpes
                  deriv1(f,s1,s)=der1(f,s1,s)
                  deriv1(f,s,s1)=der1(f,s1,s)
               enddo
            enddo
         enddo
      endif

      if (order .ge. 2) then
         do s=1,nddstate
            do s1=s,nddstate
               do f=1,ndofddpes
                  do f1=f,ndofddpes
                     der2(f1,f,s1,s)=0.0_dop
                     do i=1,ndofddpes
                        do i1=1,ndofddpes
                           der2(f1,f,s1,s)=der2(f1,f,s1,s) &
                     +deriv2(i1,i,s1,s)*ddeckart(f1,i1)*ddeckart(f,i)
                        enddo
                     enddo
                  enddo
               enddo
            enddo
         enddo
! now copy result back to deriv2 and fill in upper diagonal
         do s=1,nddstate
            do f=1,ndofddpes
               deriv2(f,f,s,s)=der2(f,f,s,s)
               do f1=f+1,ndofddpes
                  deriv2(f1,f,s,s)=der2(f1,f,s,s)
                  deriv2(f,f1,s,s)=der2(f1,f,s,s)
               enddo
            enddo

            do s1=s+1,nddstate
               do f=1,ndofddpes
                  deriv2(f,f,s1,s)=der2(f,f,s1,s)
                  deriv2(f,f,s,s1)=der2(f,f,s1,s)
                  do f1=f+1,ndofddpes
                     deriv2(f1,f,s1,s)=der2(f1,f,s1,s)
                     deriv2(f,f1,s1,s)=der2(f1,f,s1,s)
                     deriv2(f1,f,s,s1)=der2(f1,f,s1,s)
                     deriv2(f,f1,s,s1)=der2(f1,f,s1,s)
                  enddo
               enddo
            enddo
         enddo

      endif

      return
      end subroutine ddpes2x

!#######################################################################
!
! qcenter(gdof) is in Cartesian coordinates
!
!#######################################################################

      subroutine shiftdd(v,deriv1,deriv2,dip,qcenter,vo,deriv1o, &
                         deriv2o,dipo,qcentero,order)

      implicit none


      integer(long), intent(in)                                   :: order
      integer(long)                                               :: s,s1,g,&
                                                                     g1,idx
      real(dop)                                                   :: delg,delg1
      real(dop), dimension(ndofddpes), intent(in)                 :: qcenter, &
                                                                     qcentero
      real(dop), dimension(nddstate,nddstate), intent(out)           :: v
      real(dop), dimension(nddstate,nddstate), intent(in)            :: vo
      real(dop), dimension(ndofddpes,nddstate,nddstate), intent(out) :: deriv1
      real(dop), dimension(ndofddpes,nddstate,nddstate), intent(in)  :: deriv1o
      real(dop), dimension(ndofddpes,ndofddpes,nddstate,nddstate), &
                                                         intent(in)  :: deriv2o
      real(dop), dimension(ndofddpes,ndofddpes,nddstate,nddstate), &
                                                         intent(out) :: deriv2
      real(dop), dimension(3,nddstate,nddstate), intent(out) :: dip
      real(dop), dimension(3,nddstate,nddstate), intent(in)  :: dipo
!
! calculate potential matrix at new gpoint 
!

      do s=1,nddstate
         do s1=s,nddstate
            v(s1,s) = vo(s1,s)
            do idx = 1,3
               dip(idx,s1,s) = dipo(idx,s1,s)
            enddo
            do g=1,ndofddpes
               delg = qcenter(g) - qcentero(g)
               v(s1,s) = v(s1,s)+deriv1o(g,s1,s)*delg

               deriv1(g,s1,s) = deriv1o(g,s1,s)
            enddo
         enddo
      enddo

      if (order .ge. 2) then
         do s=1,nddstate
            do s1=s,nddstate
               do g=1,ndofddpes
                  delg = qcenter(g) - qcentero(g)                 
                  do g1=1,ndofddpes
                     delg1 = qcenter(g1) - qcentero(g1)
                     v(s1,s) = v(s1,s)+ &
                           0.5*deriv2o(g1,g,s1,s)*delg*delg1
                     deriv1(g,s1,s) = deriv1(g,s1,s)+ &
                           deriv2o(g1,g,s1,s)*delg1
                     deriv2(g1,g,s1,s) = deriv2o(g1,g,s1,s)
                     deriv2(g,g1,s1,s) = deriv2o(g1,g,s1,s)
                  enddo
               enddo
            enddo
         enddo
      endif

      do s=1,nddstate-1
         do s1=s+1,nddstate
            v(s,s1) = v(s1,s)
            do g=1,ndofddpes
               deriv1(g,s,s1) = deriv1(g,s1,s)
               deriv2(g,g,s,s1) = deriv2(g,g,s1,s)
               do g1=g+1,ndofddpes
                  deriv2(g1,g,s,s1) = deriv2(g1,g,s1,s)
                  deriv2(g,g1,s,s1) = deriv2(g,g1,s1,s)
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine shiftdd

!#######################################################################
!
!               DDMO_RD
!
! In this routine the MO coefficients from the geometry nearest to the 
! relevant GWP are read from the database. For use when performing DD
! with CAS of any variety.
!
! GWR 04/14
!#######################################################################

      subroutine ddmo_rd(mocoeff,nbasis,id)

      implicit none

      integer(long), intent(in) :: nbasis,id
      real(sip), dimension(nbasis,nbasis), intent(out) :: mocoeff
      real(sip), dimension(nbasis,nbasis,1)            :: tmpmo

      if(lmocodb) call dbrdmoco( the_db,1, id, -1, tmpmo )
      mocoeff(:,:) = tmpmo(:,:,1)

      end subroutine ddmo_rd

!#######################################################################
!
!               EADIAB_RD
!
! In this routine the adiabatic energies from the geometry nearest
! to the relevant GWP are read from the database. For use when performing DD
! with CAS of any variety.
!
! GWR 08/14
!#######################################################################

      subroutine eadiab_rd(av,nstate,id)

      implicit none

      integer(long), intent(in) :: nstate,id
      real(dop), dimension(nstate,nstate), intent(out) :: av
      real(dop), dimension(nstate,nstate,1) :: tmpav

      if(ldiabdb) call dbrdapes(the_db,nstate,id,-1,tmpav)
      av(:,:) = tmpav(:,:,1)

      end subroutine eadiab_rd

!#######################################################################
! 
! Write the new record to the DB memory arrays.
!
!#######################################################################
                                                                        
      subroutine dddb_wrmem(gpoint,v,deriv1,deriv2,av,dip,trans,dercp,&
                 moco,cv,ctrans,ciconfig,cicoeff,ndof1,nrec,interp,&
                 gwpsymrep)

use global
use griddatmod
use operdef
use dirdyn

!+----------------------------------------------------------------------
!     write database for direct dynamics                               
!+----------------------------------------------------------------------
      implicit none                                                     
                                                                        
      integer(long), intent(in)                                  :: ndof1
      integer(long), intent(in)                                  :: nrec,interp
      integer(long)                                              :: ns,nc,nsta
      integer(long), dimension(maxsymop),intent(in)              :: gwpsymrep
      real(dop), dimension(ndof1), intent(in)                    :: gpoint
      real(dop), dimension(nddstate,nddstate), intent(in)        :: v,cv
      real(dop), dimension(nddstate,nddstate), intent(in)        :: av
      real(dop), dimension(ndof1,nddstate,nddstate), intent(in)  :: deriv1
      real(dop), dimension(ndof1,dercpdim), intent(in)            :: dercp
      real(dop), dimension(ndof1,ndof1,nddstate,nddstate), intent(in) :: deriv2
      real(dop), dimension(3,nddstate,nddstate), intent(in)      :: dip
      real(dop), dimension(nddstate,nddstate), intent(in)        :: trans
      real(dop), dimension(nddstate,nddstate), intent(in)        :: ctrans
      real(sip), dimension(nbasis,nbasis), intent(in)            :: moco
      character(len=*), dimension(maxcivec,nsmult), intent(in)     :: ciconfig
      real(dop), dimension(maxcivec,maxmultblk,nsmult), intent(in) :: cicoeff

      if (.not. ldbsave) return
      if (nrec .gt. dbmemdim) then
         routine='DDDB_WRMEM (dd_db.f90)'
         write(message,'(a,i10)') &
               'Increase size of DB. Present size : ',dbmemdim
         call errormsg
      endif

!-----------------------------------------------------------------------
! save data in dbmem arrays
! Update record nrec 
!-----------------------------------------------------------------------
      dbinterp(nrec) = interp
      dbgeo(1:ndof1,nrec) = gpoint(1:ndof1)
      dbener(1:nddstate,1:nddstate,nrec) = v(1:nddstate,1:nddstate)
      if (lcpesdb) dbcener(1:nddstate,1:nddstate,nrec) = &
              cv(1:nddstate,1:nddstate)
      dbgrad(1:ndof1,1:nddstate,1:nddstate,nrec) = &
              deriv1(1:ndof1,1:nddstate,1:nddstate)
      dbhess(1:ndof1,1:ndof1,1:nddstate,1:nddstate,nrec) = &
              deriv2(1:ndof1,1:ndof1,1:nddstate,1:nddstate)
      if (ldiabdb) dbadener(1:nddstate,1:nddstate,nrec) = &
              av(1:nddstate,1:nddstate)
      if (lmocodb) dbmoco(1:nbasis,1:nbasis,nrec) = moco(1:nbasis,1:nbasis)
      if (lnactdb) dbdercp(1:ndof1,1:dercpdim,nrec) = &
              dercp(1:ndof1,1:dercpdim)
      if (ltransdb) dbtrans(1:nddstate,1:nddstate,nrec) = &
              trans(1:nddstate,1:nddstate)
      if (lctransdb) dbctrans(1:nddstate,1:nddstate,nrec) = &
              ctrans(1:nddstate,1:nddstate)
      if (ldipdb) dbdipmom(1:3,1:nddstate,1:nddstate,nrec) = &
              dip(1:3,1:nddstate,1:nddstate)
      if (lsymdb) dbsymrep(1:nsymop,nrec) = gwpsymrep(1:nsymop)
      if (lcivecdb) then
         do ns=1,nsmult
            nc = nciconf(ns)
            nsta = imultblk(ns)
            dbciconfig(1:nc,ns,nrec) = ciconfig(1:nc,ns)
            dbcicoeff(1:nc,1:nsta,ns,nrec) = cicoeff(1:nc,1:nsta,ns)
         enddo
      endif
   
      return
      end subroutine dddb_wrmem       
                                                                     
!#######################################################################
! 
! Write the new record to the DB.
!
!#######################################################################
                                                                        
      subroutine dddb_wr1(gpoint,av,aderiv1,aderiv2,adip,v,deriv1, &
                 deriv2,dip,mocoeff,trans,dercp,cv,ctrans,ciconfig, &
                 cicoeff,ndof1,nddstate1,interp,dercpdim1,gwpsymrep, &
                 asoc,nsocstate1)    

use global
use griddatmod
use operdef
use dirdyn

!+----------------------------------------------------------------------
!     write database for direct dynamics                               
!+----------------------------------------------------------------------
      implicit none                                                     
                                                                        
      integer(long), intent(in)                                  :: ndof1,&
                                                                    nddstate1,&
                                                                    interp,&
                                                                    dercpdim1,&
                                                                    nsocstate1
      integer(long), dimension(nsymop),intent(in)                :: gwpsymrep
      real(dop), dimension(ndof1), intent(in)                    :: gpoint
      real(dop), dimension(nddstate1,nddstate1), intent(in)      :: v,cv
      real(dop), dimension(ndof1,nddstate1,nddstate1), intent(in) :: deriv1
      real(dop), dimension(ndof1,dercpdim1), intent(in)          :: dercp
      real(dop), dimension(ndof1,ndof1,nddstate1,nddstate1), intent(in) :: deriv2
      real(dop), dimension(3,nddstate1,nddstate1), intent(in)    :: dip
      real(dop), dimension(nddstate1,nddstate1), intent(in)      :: av
      real(dop), dimension(ndof1,nddstate1,nddstate1), intent(in)       :: aderiv1
      real(dop), dimension(ndof1,ndof1,nddstate1,nddstate1), intent(in) :: aderiv2
      real(dop), dimension(3,nddstate1,nddstate1), intent(in)    :: adip
      real(sip), dimension(nbasis,nbasis), intent(in)            :: mocoeff
      real(dop), dimension(nddstate1,nddstate1), intent(in)      :: trans,ctrans
      real(dop), dimension(:),allocatable                        :: nmpoint
      character(len=*), dimension(maxcivec,nsmult)               :: ciconfig
      real(dop), dimension(maxcivec,maxmultblk,nsmult)           :: cicoeff
      complex(dop), dimension(nsocstate1,nsocstate1), intent(in) :: asoc
      integer(long)                                              :: rowid

!-----------------------------------------------------------------------
! open database files                                         
!-----------------------------------------------------------------------
      ! START TRANSACTION
      call dbstarttrans ( the_db )
      
!-----------------------------------------------------------------------
! write various quantities in database files                           
!-----------------------------------------------------------------------
      ! get new rowid from dbwrentry
      rowid = 0
      call dbwrentry(the_db,rowid,interp,0)
      call dbwrgeo(the_db,rowid,natm,gpoint)
      call dbwrpes(the_db,rowid,nddstate,v)
      if (lcpesdb) call dbwrcpes(the_db,rowid,nddstate,cv)
      if (lgradb) call dbwrgra(the_db, rowid, natm, nddstate, deriv1 )
      if (lhesdb) call dbwrhes(the_db,rowid, natm, nddstate, deriv2 )
      if (lddtrans) then
         allocate(nmpoint(ndofdd))
         call ddx2q(gpoint,nmpoint)
         call dbwrnm(the_db, rowid, ndofdd, nmpoint )
         deallocate(nmpoint)
      endif
      if (ldipdb) call dbwrdip(the_db, rowid, nddstate, dip)
      if (lmocodb) call dbwrmoco(the_db, rowid, nbasis, mocoeff)
      if (ltransdb) call dbwrtrans(the_db, rowid, nddstate, trans)
      if (lctransdb) call dbwrctrans(the_db, rowid, nddstate, ctrans)
      if (lnactdb) call dbwrnact(the_db, rowid, natm, nddstate, dercp)
      if (lapesdb) call dbwrapes(the_db, rowid, nddstate, av)
      if (lagradb) call dbwragra(the_db, rowid, natm, nddstate, aderiv1)
      if (lahesdb) call dbwrahes(the_db, rowid, natm, nddstate, aderiv2 )
      if (ladipdb) call dbwradip(the_db, rowid, nddstate, adip )
      if (lcasocdb) call dbwrcasoc(the_db, rowid, nsocstate, asoc )
      if (lsymdb) call dbwrsymrep(the_db, rowid, nsymop, gwpsymrep )
      if (lcivecdb) call dbwrcivec(the_db, rowid, nsmult, maxcivec, &
                         maxmultblk, nciconf, imultblk, ciconfig, cicoeff)

      ! COMMIT TRANSACTION
      call dbcommittrans ( the_db )

      return                                                            
      end subroutine dddb_wr1          
                                                                        
!#######################################################################
! *** V5.0 MB ********> SUBROUTINE ERROR MESSAGE <**********************
!                      ~~~~~~~~~~~~~~~~~~~~~~~~~~
! Writes an error message to the log file and to the screen.
! The calculation is then terminated.
!
! repeat of errormsg: required due to archive structure
!#######################################################################

      Subroutine dberrormsg

      Implicit None

      integer(long)           :: routlength,messlength,messl1,messl2
      integer(long), external :: slen

! --- DETERMINE THE LENGTH OF THE ARGUMENTS ---

      messl1=0
      messl2=0
      routlength = slen(routine)
      messlength = slen(message)
      if(messlength .gt. 80) then
         messl1=index(message(70:messlength),' ')+69
         if(messl1.gt.80) messl1=index(message(60:88),' ')+59
         messl1 = slen(message(1:messl1))
         messl2 = messl1
  30     messl2 = messl2+1
         if(index(message(messl2:messlength),' ').eq.1) goto 30
      endif

! --- WRITE ERROR MESSAGE AND CLOSE FILES---

      if (logisopen) Then
         Write(ilog,*)
         Write(ilog,*) 'ERROR in subroutine ',&
                       routine(1:routlength),' :'
         if(messlength .gt. 80) then
            write(ilog,*) message(1:messl1)
            write(ilog,*) message(messl2:messlength)
         else
            write(ilog,*) message(1:messlength)
         endif
         write(ilog,*)
      endif
      write(6,*)
      write(6,*) 'ERROR in subroutine ', routine(1:routlength),' :'
      if(messlength .gt. 80) then
         write(6,*) message(1:messl1)
         write(6,*) message(messl2:messlength)
      else
         write(6,*) message(1:messlength)
      endif
      write(6,*)

      if( .not.test0 ) Stop
      Return
      End Subroutine dberrormsg

!#######################################################################
! *** V6.3 GW ********> SUBROUTINE WARNING MESSAGE 1 <******************
!                       ~~~~~~~~~~~~~~~~~~~~~~~~~~
!
! Writes a warning message to the log file.
!
! copy of warnmsg1, needed due to archive structure
!#######################################################################

      subroutine dbwarnmsg

      implicit None

      integer(long)           :: routlength,messlength,messl1
      integer(long), external :: slen

! --- DETERMINE THE LENGTH OF THE ARGUMENTS ---

      routlength = slen(routine)
      messlength = slen(message)
      if(messlength .gt. 80) then
         messl1=index(message(70:messlength),' ')+69
         if(messl1.gt.80) messl1=index(message(60:88),' ')+59
      else
         messl1=0
      endif

! --- WRITE WARNING MESSAGE TO LOG FILE OR SCREEN ---

      if (logisopen) then
         write(ilog,*)
         write(ilog,*) 'WARNING in subroutine ',&
                       routine(1:routlength),' :'
         if(messlength .gt. 80) then
            write(ilog,*) message(1:messl1-1)
            write(ilog,*) message(messl1+1:messlength)
         else
            write(ilog,*) message(1:messlength)
         endif
         call myflush(ilog)
      else
         write(6,*)
         write(6,*) 'WARNING in subroutine ',&
                       routine(1:routlength),' :'
         if(messlength .gt. 80) then
            write(6,*) message(1:messl1-1)
            write(6,*) message(messl1+1:messlength)
         else
            write(6,*) message(1:messlength)
         endif
      endif

      return
      end subroutine dbwarnmsg

!#######################################################################
! 
! Write the new record to the small DB memory arrays.
!
!#######################################################################

      subroutine dddb_wrmem_gp(loc,e)

use global
use griddatmod
use operdef
use dirdyn

!+----------------------------------------------------------------------
!     write database for direct dynamics                               
!+----------------------------------------------------------------------
      implicit none

      integer(long), intent(in)                                  :: loc
      integer(long), intent(in)                                  :: e
      
      if (.not. ldbsave) return

!-----------------------------------------------------------------------
! save new location in the linked list
!-----------------------------------------------------------------------
      
       call list_insert(loc,ngp_loc(e)%locpt)

      return
      end subroutine dddb_wrmem_gp

!#######################################################################

      subroutine gpdbupdate(qcenter)

      implicit none

      integer(long) :: m,s,e,f
      real(dop), dimension(ndofdd,totgp) :: allcent
      real(dop), dimension(ndofddpes)    :: xgpoint
      real(dop), dimension(qcentdim), intent(in) :: qcenter

      integer(long), save                :: tid = 0 ! timer ID

      call get_timer(tid, 'DBupdate')
      call start_timer(tid)

      allcent = 0.0_dop

      do s=1,nstate
         do m=1,nmode
            if (.not. gwpm(m)) cycle
            call gwpcentres(m,allcent(1,zgp(s)),qcenter(zcent(m,s)),ndofdd,&
                 nspfdof(m),vdimgp(m,s),dimgp(m,s),ndimgp(m,s))
         enddo
      enddo

! add coordinates of point DVRs
      if (lpntdvr) then
         do f=1,ndof
            if (basis(f) .eq. 19) allcent(f,:) = rpbaspar(1,f)
         enddo
      endif

      do e=1,totgp
         if (lddtrans) then
            call ddq2x(allcent(1,e),xgpoint)
         else
            xgpoint(1:ndofddpes) = allcent(1:ndofddpes,e)
         endif

         call dddb_gp(dbnrec,xgpoint,e)
      enddo

      call stop_timer(tid)

      return
      end subroutine gpdbupdate
  
!#######################################################################

      subroutine gwpcentres(m,allcent,qcenter,ndof1,gdof,vdim1,dim1,ndim1)

      implicit none

      integer(long), intent(in)                   :: m,ndof1,gdof, &
                                                     vdim1,dim1,ndim1
      real(dop), dimension(ndof1,vdim1,dim1,ndim1), intent(out) :: allcent
      real(dop), dimension(gdof,dim1), intent(in)               :: qcenter
      integer(long)                               :: n,e,v,g,f

      do n=1,ndim1
         do e=1,dim1
            do v=1,vdim1
               do g=1,gdof
                  f=spfdof(g,m)
                  allcent(f,v,e,n) = qcenter(g,e)
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine gwpcentres

!###########################################################################
!
! DDDB_GP
! Routine to create the dynamic version of database for each GWP
!
! numdb : No. points in full DB
! xgp : system coordinates for each GWP
! e : GWP index for small DB (i.e. which GWP small DB belongs to)
!
! loc_gp : DB location matrix of nearest structures for each GWP
! dist_gp : Distances to all points in DB for each GWP
! dbnrec_gp(e) : number of records in each dynamic version of the big DB  
!
!###########################################################################

      subroutine dddb_gp(numdb,xgp,e)

      use dirdyn
         
      implicit none
         
      integer(long)                                :: i,iloc,nrec,j,k,l
      integer(long), intent(in)                    :: e,numdb
      real(dop), dimension(ndofddpes), intent(in)  :: xgp
      real(dop), dimension(numdb)                  :: dist_gpt
      integer(long), dimension(:,:), allocatable   :: loc_gp 
      integer(long), dimension(:), allocatable     :: indx
      real(dop), dimension(:), allocatable         :: dist_gp 
      real(dop)                                    :: mindist,mindisp
         
      if (numdb .eq. 0) return
      
!-----------------------------------------------------------------------
! initialise variables 
!----------------------------------------------------------------------- 
      mindist = 0.0_dop

!----------------------------------------------------------------------------- 
! find the closest geometries in database for direct dynamics for each GWP   
!-----------------------------------------------------------------------------
      call distdb_nopar(xgp(:),iloc,dist_gpt(:),mindist,mindisp,numdb)

!----------------------------------------------------------------------------- 
! get size of local DB
!-----------------------------------------------------------------------------
      nrec = min(dbnrec,numrec)

!----------------------------------------------------------------------------- 
! Set up initial local DB
!-----------------------------------------------------------------------------
      allocate(dist_gp(nrec))
      allocate(loc_gp(nrec,2),indx(nrec))
      do i=1,nrec
         dist_gp(i) = dist_gpt(i)
         loc_gp(i,1) = i
      enddo

!-----------------------------------------------------------------------------
! sort a(nrec) array into ascending order and get indices of locations (loc_gp)
!-----------------------------------------------------------------------------
      call ibubble_up(dist_gp,loc_gp(1,1),nrec)   ! sort initial list
   
!-----------------------------------------------------------------------------
! Swap in points if closer 
! sort a(nrec) array into ascending order and get indices of locations (loc_gp)
! ibubble returns place in initial vector so need to sort loc_gp index too
!-----------------------------------------------------------------------------
      k=1
      l=2
      do i=nrec+1,numdb
         if (dist_gpt(i) .lt. dist_gp(nrec)) then
            dist_gp(nrec) = dist_gpt(i)    ! add to list
            loc_gp(nrec,k) = i              ! add to list
            call ibubble_up(dist_gp,indx,nrec)
            do j=1,nrec
               loc_gp(j,l) = loc_gp(indx(j),k)
            enddo

           j=k
           k=l
           l=j
         endif
      enddo

!----------------------------------------------------------------------------- 
! create the dynamic database for each GWP
!-----------------------------------------------------------------------------
      call list_init(ngp_loc(e)%locpt)

      do i=1,nrec
        call list_insert(loc_gp(i,k),ngp_loc(e)%locpt)
      enddo
   
      dbnrec_gp(e) = nrec

      deallocate(dist_gp,loc_gp,indx)

      return
      end subroutine dddb_gp

!#######################################################################
!
! DISTDB_GP
!
! gpoint: system coordinates
! iloc: DB location of nearest structures. In full DB.
! dist: Distances to all points in DB (max atom displacement). In local DB.
! mindist: Max atom displacement from any point in DB
! dist: Distances to all points in DB in Cartesian (Euclidean norm)i.
!       Ordered in local DB order.
!
!#######################################################################

      subroutine distdb_gp(xgpoint,iloc,dist,mindist,mindisp,nrec)

!+----------------------------------------------------------------------
!     find the two closest geometries in database for direct dynamics  
!+----------------------------------------------------------------------
      implicit none
         
      integer(long)                             :: irec,cnt,jrec
      integer(long), intent(in)                 :: nrec
      integer(long), intent(out)                :: iloc
      real(dop)                                 :: atdisp
      real(dop), dimension(ndofddpes), intent(in) :: xgpoint
      real(dop), dimension(nrec), intent(out)   :: dist
      real(dop), intent(out)                    :: mindist,mindisp

      type(nextptr), pointer :: current

      if (nrec .eq. 0) return
         
!----------------------------------------------------------------------- 
! initialise variables 
! r1 will be min distance. iloc is location of minimum distance
!----------------------------------------------------------------------- 
      irec = 0
      iloc = 0
      cnt = 0

      mindist = 99999.0_dop
      mindisp = 99999.0_dop

      dist(:) = 99999.0_dop

!-----------------------------------------------------------------------  
! loop over DB 
!----------------------------------------------------------------------- 
 
      call list_init(current)
      current=> ngp_loc(num_gp)%locpt

      do irec=1,nrec
         if (dbignore(current%locDB) .eq. 1) then
            current => current%next
            cycle
         endif   
! set cnt to first non-ignored DB record in local DB
         if (cnt .eq. 0) cnt=irec

         jrec = current%locDB
         call distdb1(xgpoint,dbgeo(1:ndofddpes,jrec),dist(irec),atdisp)
         mindist = min(mindist,dist(irec))
         mindisp = min(mindisp,atdisp)
 
         if (irec .eq. cnt) then
            iloc = current%locDB
!find the location of the minimum distance inside the big DB
         else if ( dist(irec) .lt. dist(cnt) ) then
            iloc = current%locDB
            cnt=irec
         endif
         current => current%next

      enddo

      if (cnt .eq. 0) then
         routine = 'Distdb_GP (dd_db.F90)'
         message = 'No usable points found in Local DB '
         call errormsg
      endif
!
      return
      end subroutine distdb_gp


!------------------------------------------------------------------------
!findrad: finds the confidence radius
! 
!------------------------------------------------------------------------

      subroutine findrad_gp(r1,locmin,dist,numin,iloc)
       
      implicit none
       
      integer(long)                             :: irec,natom,cnt,locmax
      integer(long), intent(in)                 :: locmin,numin
      integer(long), dimension(dbnrec_gp(num_gp)), intent(in) :: iloc         
      real(dop), dimension(dbnrec_gp(num_gp)), intent(in)     :: r1
      real(dop), dimension(dbnrec_gp(num_gp))   :: r1_cp
      real(dop)                                 :: r2,natnrm
      real(dop), dimension(ndofddpes)           :: xgpoint
      real(dop), dimension(ndofdd)              :: qdist
      real(dop), intent(out)                    :: dist
      real(dop), dimension(ndofddpes)           :: gx,xcoord,xpoint,xdist
      real(dop), dimension(ndofddpes)           :: distvectmp

      real(dop)                                 :: dnrm2 !BLAS norm function
      
      if (dbnrec .eq. 0) return
         
      natom = ndofddpes/3
      natnrm = dsqrt(dble(natom))
              
!----------------------------------------------------------------------- 
! initialise variables 
!-----------------------------------------------------------------------          
      irec = 0
      r2 = 990.0_dop
      xcoord(:)=0.0_dop
      xpoint(:)=0.0_dop
      qdist(:)=0.0_dop
      xdist(:)=0.0_dop

      r1_cp = r1
  
      locmax=0
      do irec=1,numin
         if (irec .lt. numin) then   
            cnt=minloc(r1_cp,dim=1)
            r1_cp(cnt)=maxval(r1_cp)  
         else
            locmax=minloc(r1_cp,dim=1)
         endif
      enddo 
                  
      xgpoint(1:ndofddpes) = dbgeo(1:ndofddpes,locmin)
      gx(1:ndofddpes) = dbgeo(1:ndofddpes,iloc(locmax))
      distvectmp=xgpoint-gx
      dist = dnrm2(ndofddpes,distvectmp,1)
      dist = dist/natnrm
      
      if (dist .le. 0.0 ) dist=minval(r1)

      return
      end subroutine findrad_gp     

!-----------------------------------------------------------------------
! 
! Sort from lower bound to the highter bound.
!
!-----------------------------------------------------------------------     
   
      subroutine sort(a)
   
      implicit none
   
      real(dop), intent(inout), dimension(:)   :: a
      real(dop)                                :: temp
      integer(long)                            :: i,j
      logical                                  :: swapped
   
       do j = SIZE(a)-1, 1, -1
         swapped = .FALSE.
          do i = 1, j
            if (a(i) > a(i+1)) then
               temp = a(i)
               a(i) = a(i+1)
               a(i+1) = temp
               swapped = .TRUE.
            endif
          enddo
         if (.NOT. swapped) exit
        enddo

      end subroutine sort

!#######################################################################
!
! DISTDB_nopar
! non parallel version of distdb 
!      
!#######################################################################

      subroutine distdb_nopar(xgpoint,iloc,dist,mindist,mindisp,nrec)  

!+----------------------------------------------------------------------
!     find the two closest geometries in database for direct dynamics  
!+----------------------------------------------------------------------
               
      implicit none                                                     
                                                                                          
      integer(long)                             :: irec
      integer(long), intent(in)                 :: nrec
      integer(long), intent(out)                :: iloc
      real(dop)                                 :: atdisp
      real(dop), dimension(ndofddpes), intent(in) :: xgpoint
      real(dop), dimension(nrec), intent(out)   :: dist
      real(dop), intent(out)                    :: mindist,mindisp
                  
      if (nrec .eq. 0) return
                  
!----------------------------------------------------------------------- 
! initialise variables 
! r1 will be min distance. iloc is location of minimum distance
!----------------------------------------------------------------------- 
      irec=0
      iloc = 0
      mindist = 99999.0_dop
      mindisp = 99999.0_dop
      dist(:) = 99999.0_dop
                  
!-----------------------------------------------------------------------  
! loop over DB 
!----------------------------------------------------------------------- 
      do irec=1,nrec
         if (dbignore(irec) .eq. 1) cycle

         call distdb1(xgpoint,dbgeo(1:ndofddpes,irec),dist(irec),atdisp)
         mindist = min(mindist,dist(irec))
         mindisp = min(mindisp,atdisp)
      enddo
               
      iloc = minloc(dist,DIM=1)

      return                                                            
      end subroutine distdb_nopar  


!#######################################################################
!
! routine to read LHA info from all points (weighted) in a  small DB
! only diabatic files need to be read.
!
! num_gp denotes DB to be read - passed via dirdyn module and set in
!        gh_elements
!
! lvonly: just read energy. NB this seems to be wrong as shiftdd
! will not work!!
!
! lforce: ignore last call information and recalculate weighted 
! potential
!
! dbignore(dbnrec): Passed by module. If 1, record ignored in weighting
!#######################################################################

      subroutine dddb_rd_gp(v,deriv1,deriv2,dipole,xgpoint,ltransder,&
                 lvonly,lforce)
                                                                
!+----------------------------------------------------------------------
!     read LHA PES database for direct dynamics                        
! if using lbdsave, DB is in memory not read from file
!+----------------------------------------------------------------------
      implicit none

      logical(kind=4), intent(in) :: ltransder,lvonly,lforce
      logical                     :: linterpol
      integer(long)               :: s,s1,i,order,irec,&
                              idbloc,t
      real(dop), dimension(ndofddpes),intent(in)               :: xgpoint
      real(dop), dimension(nddstate,nddstate), intent(out)     :: v
      real(dop), dimension(ndofddpes,nddstate,nddstate), &
                                            intent(out) :: deriv1
      real(dop), dimension(ndofddpes,ndofddpes,nddstate,nddstate), &
                                            intent(out) :: deriv2
      real(dop), dimension(3,nddstate,nddstate), intent(out)   :: dipole
      real(dop), dimension(:), allocatable   :: r1
      integer(long), dimension(:), allocatable   :: loc
      real(dop), dimension(:), allocatable   :: weight
      real(dop)                              :: mindist,mindisp
      real(dop), allocatable        :: gx(:)
      real(dop), allocatable        :: vv(:,:)
      real(dop), allocatable        :: der1(:,:,:)
      real(dop), allocatable        :: der2(:,:,:,:)
      real(dop), allocatable        :: dip(:,:,:)
      real(dop), allocatable        :: vtmp(:,:)
      real(dop), allocatable        :: dertmp1(:,:,:)
      real(dop), allocatable        :: dertmp2(:,:,:,:)
      real(dop), allocatable        :: diptmp(:,:,:)
      real(dop), allocatable        :: tv(:,:)
      real(dop), allocatable        :: tderiv1(:,:,:)
      real(dop), allocatable        :: tderiv2(:,:,:,:)
      real(dop), allocatable        :: tdipole(:,:,:)
      real(dop)                     :: maxwgt,dv,rtmp

      real(dop), allocatable        :: ord_dist(:)
      real(dop), dimension(1,1)     :: tmpmat
      integer(long)                 :: dbnconf1

      logical, save     :: initial = .true.

      type(nextptr), pointer :: current

      v(:,:)=0.0_dop
      deriv1(:,:,:)=0.0_dop
      deriv2(:,:,:,:)=0.0_dop
      dipole(:,:,:)=0.0_dop
      linterpol=.true.
      mindist=0.0_dop
      idbloc=0
      dv = 0.0_dop
      initial = lforce
      t=0

!----------------------------------------------------------------------- 
! allocate variables
!----------------------------------------------------------------------- 
      allocate(gx(ndofddpes))
      gx=0
      if (dbnrec .gt. 0) then
         allocate(r1(dbnrec_gp(num_gp)))
         allocate(loc(dbnrec_gp(num_gp)))
         allocate(weight(dbnrec_gp(num_gp)))
         allocate(ord_dist(dbnrec_gp(num_gp)))
      else
         allocate(r1(1))
         allocate(loc(1))
         allocate(weight(1))
         allocate(ord_dist(1))
      endif
      r1=0
      loc=0
      weight=0.0_dop

      call list_init(Current)
      current => ngp_loc(num_gp)%locpt
      do irec = 1,dbnrec_gp(num_gp)
         loc(irec) = current%locDB
            current => current%Next
      enddo

!-----------------------------------------------------------------------
! Calculate distances to points in the database
!-----------------------------------------------------------------------
      call distdb_gp(xgpoint,idbloc,r1,mindist,mindisp,dbnrec_gp(num_gp))

!-----------------------------------------------------------------------
! Determine if the point doesn't need interpolation
! NB If this value is changed, you should also change the value before the call
! to dbuncertain in routine getddpes.
!-----------------------------------------------------------------------
      if (mindist .lt. dbdisp) then
         allocate(vv(nddstate,nddstate),&
                 der1(ndofddpes,nddstate,nddstate),&
                 der2(ndofddpes,ndofddpes,nddstate,nddstate),&
                 dip(3,nddstate,nddstate))
         vv=0
         der1=0
         der2=0
         dip=0

         linterpol=.false.
     
         gx(1:ndofddpes) = dbgeo(1:ndofddpes,idbloc)
         do s=1,nddstate
            vv(1:nddstate,s) = dbener(1:nddstate,s,idbloc)
            do s1=1,nddstate
               der1(1:ndofddpes,s1,s) = dbgrad(1:ndofddpes,s1,s,idbloc)
               do i=1,ndofddpes
                  der2(1:ndofddpes,i,s1,s) = &
                      dbhess(1:ndofddpes,i,s1,s,idbloc)
               enddo
               if (ldipdb) dip(1:3,s1,s) = dbdipmom(1:3,s1,s,idbloc)
            enddo
         enddo

! shift PES to current coordinate
         call shiftdd(v,deriv1,deriv2,dipole,xgpoint,vv, &
                     der1,der2,dip,gx,2)  

         deallocate(vv,der1,der2,dip)
      endif
      deallocate(gx)

!-----------------------------------------------------------------------
! loop over DB - OMP parallelized (if storing DB in memory only)
!-----------------------------------------------------------------------
      if (linterpol) then

!-----------------------------------------------------------------------
! Get confidence radius
!-----------------------------------------------------------------------
         dbnconf1=min(dbnconf,dbnrec_gp(num_gp))
         ord_dist = r1
         call dbubble(ord_dist,tmpmat,dbnrec_gp(num_gp),0)
         irec = dbnconf1
         do
            rad_conf = ord_dist(irec)
            if (rad_conf .lt. 90000.0) exit  ! ignored points in DB 
                                             ! have distance of 99999.0
            irec=irec-1
         enddo

!-----------------------------------------------------------------------
! Get weights
!-----------------------------------------------------------------------
         maxwgt = -999.0_dop
         weight(:) = 0.0_dop
         do irec=1,dbnrec_gp(num_gp)
            if (dbignore(loc(irec)) .eq. 1) cycle
            if (dbinterp(loc(irec)) .lt. 0) cycle
            rtmp = max(r1(irec)/rad_conf,1.0d-6)
            weight(irec) = 1.0_dop/((rtmp**dbinterord1) + (rtmp**dbinterord))

            if (weight(irec) .lt. dbminwgt) weight(irec) = 0.0_dop
         enddo
         maxwgt = max(maxwgt,maxval(weight))
         weight = weight/maxwgt

         shep_norm=0.0_dop
         do irec=1,dbnrec_gp(num_gp)
            shep_norm = shep_norm + weight(irec)
         enddo
         if (shep_norm.eq.0.0_dop) then
           routine='dddb_rd_gp (dd_db.f90)'
           write(message,'(2a)') 'ERROR: No suitable points in database'
           call dberrormsg
         endif

         allocate(vtmp(nddstate,nddstate),&
                 dertmp1(ndofddpes,nddstate,nddstate),&
                 dertmp2(ndofddpes,ndofddpes,nddstate,nddstate),&
                 diptmp(3,nddstate,nddstate),&
                 tdipole(3,nddstate,nddstate),tv(nddstate,nddstate),&
                 tderiv1(ndofddpes,nddstate,nddstate),&
                 tderiv2(ndofddpes,ndofddpes,nddstate,nddstate))
         dv=0
         vtmp=0
         dertmp1=0
         dertmp2=0
         diptmp=0
         tdipole=0
         tv=0
         tderiv1=0
         tderiv2=0

         do irec=1,dbnrec_gp(num_gp)
            if (dbignore(loc(irec)) .eq. 1) cycle
            if (dbinterp(loc(irec)) .lt. 0) cycle
            if (weight(irec) .lt. dbminwgt)  cycle
       
! get adiabatic energy difference and check if too small
! do not use point when regularisation (only for two states)
           if (nddstate .eq. 2 .and. dddiab .eq. 1) then
              dv=dsqrt((dbener(2,2,loc(irec))-dbener(1,1,loc(irec)))**2+&
              4.0_dop*dbener(2,1,loc(irec))**2)
              if (dv .lt. dddvmin) cycle
           endif

           call shiftdd(vtmp,dertmp1,dertmp2,diptmp,xgpoint,&
                dbener(1:nddstate,1:nddstate,loc(irec)), &
                dbgrad(1:ndofddpes,1:nddstate,1:nddstate,loc(irec)), &
                dbhess(1:ndofddpes,1:ndofddpes,1:nddstate,1:nddstate,loc(irec)), &
                dbdipmom(1:3,1:nddstate,1:nddstate,loc(irec)), &
                dbgeo(1:ndofddpes,loc(irec)),2)

           if (ldipdb) tdipole = tdipole + weight(irec)*diptmp
           tv = tv + weight(irec)*vtmp
           tderiv1 = tderiv1 + weight(irec)*dertmp1
           tderiv2 = tderiv2 + weight(irec)*dertmp2
         enddo

         if (ldipdb) dipole = dipole + tdipole
         v = v + tv
         deriv1 = deriv1 + tderiv1
         deriv2 = deriv2 + tderiv2

         deallocate(vtmp,dertmp1,dertmp2,diptmp,tdipole,tv,tderiv1,tderiv2)

!-----------------------------------------------------------------------
! normalise averaged PES via array operations
! Not yet needed for dipoles.
!-----------------------------------------------------------------------
          v = v / shep_norm
          deriv1 = deriv1 / shep_norm
          deriv2 = deriv2 / shep_norm
          if (ldipdb) dipole = dipole / shep_norm

      endif

! return un-scaled weight norm (needed in diabat4)
      shep_norm=shep_norm*maxwgt

!-----------------------------------------------------------------------
! For derivatives
! 1. Eckart transform to take derivatives to original Cartesian frame
! 2. Transform to dynamics coordinates
!-----------------------------------------------------------------------
      order = 2
      if (lddtrans.and.ltransder.and. .not.lvonly) &
         call ddpes2q(deriv1,deriv2,order)

      if (allocated(r1)) deallocate(r1)
      if (allocated(loc)) deallocate(loc)
      if (allocated(ord_dist)) deallocate(ord_dist)
      if (allocated(weight)) deallocate(weight)

      return
      end subroutine dddb_rd_gp   

!#######################################################################
!-----------------------------------------------------------------------
! finish DD calculations
!-----------------------------------------------------------------------

      subroutine dbfinalise

      implicit none

      if (lddwrdb) then
         write(ilog,'(/,a)') '*** DD DB Written ***'
         write(ilog,'(a,i7)') 'No. Entries: ',dbnrec
      endif

      return
      end subroutine dbfinalise
   
!###########################################################################
!
! Subroutines for linked list arrays
! list_init : Nullify pointers
! list_insert : Add a new item to the end of the list 
!  
!###########################################################################

      subroutine list_init(head)
      implicit none
      type(nextptr), pointer :: head

      nullify(Head)     ! No successors to them yet.

      end subroutine list_init

!-----------------------------------------------------------------------

      subroutine list_insert(loc,head)     ! Push a string on the list
      implicit none
      integer(long), intent(in)  :: loc
      type(nextptr), pointer     :: new_node,head    ! The node we will add

      allocate(new_node)                             ! Allocate new node

      if(.not.(associated(new_node))) then     ! Check the allocation worked
         routine = 'List_Insert (dd_db.F90)'
         message = 'Could not allocate node, Error in list_insert'
         call errormsg
      endif

      new_node%locDB = loc          ! Copy the input string into our new node

      if(associated(head)) then     ! If the list is not empty
        new_node%next => head       ! we set the next node to the previous 
                                    ! first node
      else
        new_node%next => null()     ! else we set it to NULL
      endif

      head => new_node              ! Link the new node at the top of the list

      return
      end subroutine list_insert
               
!###########################################################################
!
! Generate symmetry maps. After each symmetry operation, find the nearest 
! atom to the original structure and provide mapping to reorder replica
! to match original
!
! ANTONIA
!###########################################################################

      subroutine gensymmaps

      implicit none

      integer(long)                    :: n,n1,nop
      integer(long), dimension(natm)   :: imass
      real(dop), dimension(3)          :: xdiff
      real(dop), dimension(3,natm)     :: xcoo
      real(dop)                        :: rdist, mindist

! first operation is unit mapping for identity operation
       do n=1,natm
          dbsymmap(n,1) = n
       enddo

       do n=1,natm
          imass(n) = nint(xmass(n))
       enddo

! loop over symmetry operations
       do nop=2,nsymop

! generate structure by symmetry operations
! standard structure is in xref(3,natm) read from refdb.dat file
          call dosymop(xref,xcoo,dbpntgp,nop,natm,ilog)

! Find which atom maps from original to new and store in dbsymmap(n,nop)=n1
! Calculate distance matrix r = xcoo(i)-xref(j)
          rdist = 0.0_dop
          do n=1,natm
             mindist = 999.0

             do n1 = 1,natm
! only look at equivalent atoms
                if (imass(n) .ne. imass(n1)) cycle

                xdiff(:) = xcoo(:,n1) - xref(:,n)
                rdist = xdiff(1)*xdiff(1) + xdiff(2)*xdiff(2) + &
                                            xdiff(3)*xdiff(3) 
                if (rdist .lt. mindist) then
                   dbsymmap(n,nop) = n1
                   mindist = rdist
                endif
             enddo
             n1 = dbsymmap(n,nop)
          enddo
       enddo

      return
      end subroutine gensymmaps

!#######################################################################
!
! read reference point for symmetry operations
!
!#######################################################################

      subroutine rdxref(lfound)

      implicit none

      integer(long)                              :: i,iz,iz0,j,n,ic,ierr,ilbl
      real(dop)                                  :: xunit
      logical(kind=4)                            :: check,lfound
      character(len=c5)                          :: filename
      character(len=20)                          :: cunit

      inquire(iin,opened=check)
      if (.not. check) then
         filename=inputfile(1:inplaenge)
         open(iin,file=filename,status='old',form='formatted',err=900)
      endif

      ierr=0
      cunit='au'
      xunit=1.0_dop

      rewind(iin)
      iz0=0
!-----------------------------------------------------------------------
! FIND SECTION
!-----------------------------------------------------------------------
      iz=0
   10 continue

      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      if (ierr .eq. 1) then
         message = inptit
         call errormsg
      endif

      if (keyword(1).eq.'ddpoints-section') then
         go to 12
      elseif (keyword(1).eq.'end-input') then
         return
      else
         go to 10
      endif

!-----------------------------------------------------------------------
! READ KEYWORDS:
!-----------------------------------------------------------------------
   12 continue
      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      message = inptit
      call cntrl(keyword(1),lc(1),ierr)

      i=1
   15 continue
      if (keyword(1).eq.'symmetry_reference') then
         lfound = .true.
         xunit = 1.0_dop/0.52917225_dop
         if (keyword(2) .eq. '=') then
            cunit = keyword(3)(1:20)
! test unit keyword
            call keylunits(xunit,cunit,ierr)
            if (ierr .eq. 1) then
               routine = 'Rdxref (dd_db.f90)'
               message = 'Length unit given not recognized'
               call wrline(zeroi,iz,iz0)
            endif
         endif

         j=0
         do n=1,natm
            read(iin,*,err=999) atnam(n),(xref(i,n),i=1,3)
            if (atnam(n) .ne. xatnam(n)) then
               routine = 'Rdxref (dd_db.f90)'
               message='Symmetry_reference point in different order from input'
               call errormsg
            endif
            xref(:,n)=xref(:,n)*xunit
         enddo

         go to 20

      else if (keyword(1).eq.'end-ddpoints-section') then
         go to 20

      end if

!-----------------------------------------------------------------------
! check if more keywords are to be read, else read new line
!-----------------------------------------------------------------------
      if (i.lt.ic) then
         i=i+1
         go to 15
      else
         go to 12
      endif

 20   continue
      close(iin)

      return
  900 routine='Rdxref (dd_db.f90)'
      ilbl=index(filename,' ')-1
      message = 'Cannot open file: '//filename(1:ilbl)
      call errormsg
  999 routine='Rdxref (dd_db.f90)'
      message='Error reading input file'
      call errormsg
      end subroutine rdxref

!#######################################################################
! MKSYMCENT
!
! use symmetry operations to generate a set of replicas of a structure
! e is the index of the structure
!
!#######################################################################

      subroutine mksymcent(xgp,dbpntgp,nsymop,ngp,e)

      implicit none

      integer(long), intent(in)   :: dbpntgp, nsymop,ngp,e
      real(dop), dimension(ndofddpes,nsymop,ngp), intent(inout) :: xgp
      real(dop), dimension(ndofddpes) :: xgpoint

      integer(long)  :: n,natm

      natm = ndofddpes/3
      do n=2,nsymop
         call dosymop(xgp(1,1,e),xgpoint,dbpntgp,n,natm,ilog)

! ANTONIA
! Use dbsymmap to permute atoms back to original orientation in new configuration
         call dosymperm(xgpoint,xgp(1,n,e),dbsymmap(1,n),natm)

      enddo

      return
      end subroutine mksymcent


      end module dd_db

