        module h2copot
!-----------------------------------------------------------------------
! H2CO by Bowman J. Phys. Chem. A 2004, 108, 8980-8986
!-----------------------------------------------------------------------
        implicit double precision (a-h,o-z)
        private
        save
        public::Getpot
        contains

C**********************************************************************
C this getpot() subroutine use cartesian coordinates as input (in bohr)
C it will return potential value in wavenumber, combining CCSD(T) fit with MRCI
C fit with switch range 1.5 - 2.0
C**********************************************************************

      subroutine Getpot (natom,xmass,xx,v,v1,v2,s)

      implicit double precision (a-h,o-z)
      dimension xx(36,3),rr(36,36),r(6),rmrci(6)
      dimension xmass(36)
      data rech,reoh,rehh / 2.15d0 , 1.85d0, 1.45d0 /

      hcm=219474.63d0

c Switching range

      rl=1.5d0
      ru=2.0d0

      xmab=(xmass(1)*xx(1,1)+xmass(4)*xx(4,1))/(xmass(1)+xmass(4))
      ymab=(xmass(1)*xx(1,2)+xmass(4)*xx(4,2))/(xmass(1)+xmass(4))
      zmab=(xmass(1)*xx(1,3)+xmass(4)*xx(4,3))/(xmass(1)+xmass(4))
c      write(*,*) xmab,ymab,zmab

      xmcd=(xmass(2)*xx(2,1)+xmass(3)*xx(3,1))/(xmass(2)+xmass(3))
      ymcd=(xmass(2)*xx(2,2)+xmass(3)*xx(3,2))/(xmass(2)+xmass(3))
      zmcd=(xmass(2)*xx(2,3)+xmass(3)*xx(3,3))/(xmass(2)+xmass(3))
c      write(*,*) xmcd,ymcd,zmcd

      rbig=sqrt((xmcd-xmab)*(xmcd-xmab)+
     $          (ymcd-ymab)*(ymcd-ymab)+
     $          (zmcd-zmab)*(zmcd-zmab))
c      write(*,*) "rbig",rbig

c     do i=1,4
c     write(*,*) (xx(i,j),j=1,3)
c     end do

c Calculating the internuclear distances

      Call BONDS (natom,xx,rr) 

      r(1)=rr(1,2)
      r(2)=rr(3,4)
      r(3)=rr(2,3)
      r(4)=rr(1,3)
      r(5)=rr(2,4)
      r(6)=rr(1,4)

      rmrci(1)=rr(2,3)
      rmrci(2)=rr(1,3)
      rmrci(3)=rr(3,4)
      rmrci(4)=rr(1,2)
      rmrci(5)=rr(2,4)
      rmrci(6)=rr(1,4)

c     do i=1,6
c      write(*,*) r(i)
c     end do
c      write(*,*)'rcha',rr(1,3)
c      write(*,*)'rchb',rr(3,4)
c      write(*,*)'roha',rr(1,2)
c      write(*,*)'rohb',rr(2,4)
c      write(*,*)'rhh ',rr(1,4)

c calculating the switch function 

      scha = rr(1,3)/rech
      schb = rr(3,4)/rech
      soha = rr(1,2)/reoh
      sohb = rr(2,4)/reoh
      shh  = rr(1,4)/rehh

      Ra=dmin1(scha,soha,shh)
      Rb=dmin1(schb,sohb,shh)
      S0=dmax1(Ra,Rb)

      x=(S0-rl)/(ru-rl)

      if (x.LT.0.) then
        s=1.0
      end if
      if (x.GT.1.) then
        s=0.0
      end if
      if ((x.GE.0.).and.(x.LE.1.)) then
        s=1.0d0-(10.0d0*x**3-15.0d0*x**4+6.0d0*x**5)
      end if

c Getting potential from MRCI fit

        Call h2co_mrci(rmrci,en) 
        v1=(en+114.341634d0)

c Getting potential from CCSC(T) fit

        Call h2co_ccsdt(r,rbig,v2)
        v2=v2/hcm

c Combining the two fit with switch function s

      v=(v2*s+v1*(1-s))*hcm

!      write(*,'(3f10.2)') v1,v2,v
!      write(*,*) v1,v2,v
      return
      end subroutine

C*************************************************************************
C*************************************************************************


C**************************************************************
C  For calculating bond length
C**************************************************************
      SUBROUTINE BONDS (NATOM,XX,RR)
      
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION RR(36,36),XX(36,3)
      
      DO I=1,NATOM
        DO J=1,I
          RR(J,I)=0
        END DO
      END DO

      DO K=1,3
        DO I=1,NATOM
          DO J=1,I
            RR(J,I)=RR(J,I)+(XX(I,K)-XX(J,K))*(XX(I,K)-XX(J,K))
          END DO
        END DO
      END DO
      DO I=1,NATOM
        DO J=1,I
          RR(J,I)=SQRT(RR(J,I))
          RR(I,J)=RR(J,I)
        END DO
      END DO

9200  FORMAT (f8.0,7f10.5)

      RETURN
      END SUBROUTINE
C**************************************************************
C**************************************************************


      subroutine h2co_mrci(rij,en)
c
c     note h1 and h2 are not equivalent, it is assumed that
c     h2 is further away from the other atoms than h1
c
c     rij(1) = r(c-o) (au)
c     rij(2) = r(c-h1)
c     rij(3) = r(c-h2)
c     rij(4) = r(o-h1)
c     rij(5) = r(o-h2)
c     rij(6) = r(h1-h2)
c
c     en = energy (hartree)
c
c
      implicit real*8 (a-h,o-z)
      dimension rij(6),tij(6)
      data rech,reoh,rehh / 2.15d0 , 1.85d0, 1.45d0 /

      ru1 = 3.0d0
      rl1 = 2.0d0

      ru2 = 4.0d0
      rl2 = 3.0d0

      scha = rij(2)/rech
      schb = rij(3)/rech
      soha = rij(4)/reoh
      sohb = rij(5)/reoh
      shh  = rij(6)/rehh

      Ra=dmin1(scha,soha,shh)
      Rb=dmin1(schb,sohb,shh)
      S0=dmax1(Ra,Rb)

      if(Ra .gt. Rb) then
       tij(1) = rij(1)
       tij(2) = rij(3)
       tij(3) = rij(2)
       tij(4) = rij(5)
       tij(5) = rij(4)
       tij(6) = rij(6)
      else
       tij(1) = rij(1)
       tij(2) = rij(2)
       tij(3) = rij(3)
       tij(4) = rij(4)
       tij(5) = rij(5)
       tij(6) = rij(6)
      endif

      x1=(S0-rl1)/(ru1-rl1)
      x2=(S0-rl2)/(ru2-rl2)

      if (x1.LE.0.) then
        call f27367(tij,en)
      else if ((x1.GT.0.).and.(x1.LT.1.)) then
        call f27367(tij,v1)
        call f33757(tij,v2)
        s=1.0-(10.0d0*x1**3-15.0d0*x1**4+6.0d0*x1**5)
        en=v1*s+v2*(1-s)
      else if ((x2.GE.0.).and.(x2.LT.1.)) then
        call f33757(tij,v1)
        call f15158(tij,v2)
        s=1.0d0-(10.0d0*x2**3-15.0d0*x2**4+6.0d0*x2**5)
        en=v1*s+v2*(1-s)
      else if (x2.GE.1.) then
        call f15158(tij,en)
      endif

      return
      end subroutine

      subroutine f27367(rij,en)
      implicit real*8 (a-h,o-z)
      dimension fk(896),fmr(6,6), shift(6),rij(6),alpham(6)

      data shift  / 2.25d0, 2.12d0, 2.12d0, 3.85d0, 3.85d0, 3.57d0 /
      data alpham / 6*0.5d0 /
      data nmax,nvar /6,6/

      data fk(   1), fk(   2) / -1.1363726888D+02 , -1.6959432998D+00 /
      data fk(   3), fk(   4) / -3.2239021260D+00 , -7.4596063866D+00 /
      data fk(   5), fk(   6) /  4.2008718464D+00 ,  4.9133127360D-01 /
      data fk(   7), fk(   8) /  1.3572193324D+00 ,  5.1566979271D-01 /
      data fk(   9), fk(  10) /  7.0546500473D+00 ,  2.9459156303D+01 /
      data fk(  11), fk(  12) / -9.2045095311D-01 ,  1.9313128176D+00 /
      data fk(  13), fk(  14) /  1.4389682978D+00 ,  3.3131973535D+00 /
      data fk(  15), fk(  16) /  6.5403722800D+00 ,  2.7552799159D-01 /
      data fk(  17), fk(  18) /  2.9629886975D+00 ,  2.5282201402D+00 /
      data fk(  19), fk(  20) /  3.1860766488D+01 , -5.7065426421D-01 /
      data fk(  21), fk(  22) / -9.7406652955D+00 , -2.5646817109D+00 /
      data fk(  23), fk(  24) / -2.8404583674D+01 , -2.9740758433D+00 /
      data fk(  25), fk(  26) / -8.7919348109D+00 , -2.9408664885D+00 /
      data fk(  27), fk(  28) /  5.7525347379D-01 , -1.8039548149D+00 /
      data fk(  29), fk(  30) / -3.9343197382D+00 , -1.2312654480D+00 /
      data fk(  31), fk(  32) / -3.6297948985D+01 , -1.2302433247D+00 /
      data fk(  33), fk(  34) /  2.3658244300D+00 ,  7.6867062143D-01 /
      data fk(  35), fk(  36) / -9.5253966930D-01 ,  8.5151814599D+00 /
      data fk(  37), fk(  38) / -5.0218612451D+00 , -1.1193430158D+00 /
      data fk(  39), fk(  40) / -4.0059019555D+00 ,  2.3067850999D+00 /
      data fk(  41), fk(  42) / -3.8997462827D+01 , -6.3044565415D+00 /
      data fk(  43), fk(  44) /  4.0586701010D+00 ,  5.4147295636D+00 /
      data fk(  45), fk(  46) /  1.0174710829D+01 , -1.1323439944D+02 /
      data fk(  47), fk(  48) /  7.1270349553D+01 , -1.2961289548D+01 /
      data fk(  49), fk(  50) /  5.4852042857D+00 ,  6.2700879349D+00 /
      data fk(  51), fk(  52) /  4.8648848662D+00 ,  4.2636002856D+00 /
      data fk(  53), fk(  54) / -3.0609884396D+00 ,  1.1887084521D+00 /
      data fk(  55), fk(  56) /  4.4066666095D+00 ,  5.2706797472D+00 /
      data fk(  57), fk(  58) / -8.8343306321D+00 , -5.2732130457D-01 /
      data fk(  59), fk(  60) / -4.5410450427D+00 ,  4.2731047247D-01 /
      data fk(  61), fk(  62) / -3.5288101249D+00 , -7.5521624189D+00 /
      data fk(  63), fk(  64) /  1.6915783475D+00 ,  4.7370941896D-01 /
      data fk(  65), fk(  66) / -2.5355991377D+01 , -5.3437813126D+00 /
      data fk(  67), fk(  68) / -7.7098983010D+00 ,  2.7465487483D+00 /
      data fk(  69), fk(  70) / -2.3246692276D+00 , -2.5223030545D+01 /
      data fk(  71), fk(  72) / -2.1357445352D+01 ,  4.9040497556D+00 /
      data fk(  73), fk(  74) /  2.5869700504D+00 ,  5.9881417990D+00 /
      data fk(  75), fk(  76) /  1.5443652574D+01 ,  5.6379587996D+01 /
      data fk(  77), fk(  78) /  2.9656881243D+01 , -2.9145996283D+00 /
      data fk(  79), fk(  80) / -2.6001689205D+00 , -1.6652207687D+01 /
      data fk(  81), fk(  82) /  1.4115246201D+01 , -3.8597761097D+00 /
      data fk(  83), fk(  84) /  2.6436831952D+01 , -3.3286344082D-01 /
      data fk(  85), fk(  86) / -3.6073121502D+00 , -2.5167034428D+01 /
      data fk(  87), fk(  88) / -1.3839136441D-01 , -5.0020521945D-02 /
      data fk(  89), fk(  90) /  1.9431106492D-01 , -4.4264799969D+00 /
      data fk(  91), fk(  92) /  1.1735813736D+01 , -3.9230942073D-01 /
      data fk(  93), fk(  94) /  2.5864643759D+00 , -6.2376300076D-01 /
      data fk(  95), fk(  96) / -8.3779782705D+00 ,  5.2555550447D+00 /
      data fk(  97), fk(  98) /  6.4486025961D+00 , -2.2490641537D-01 /
      data fk(  99), fk( 100) /  1.4248975773D+00 , -7.5960776362D+01 /
      data fk( 101), fk( 102) /  1.9538984856D+02 , -8.1637377616D+01 /
      data fk( 103), fk( 104) /  9.2593459648D+01 ,  5.2629353306D+01 /
      data fk( 105), fk( 106) /  1.6590332486D+00 ,  1.5110657180D+00 /
      data fk( 107), fk( 108) /  4.3466283079D+00 , -2.3921907886D-01 /
      data fk( 109), fk( 110) /  1.1960050710D-01 ,  2.8923476909D+00 /
      data fk( 111), fk( 112) /  1.1884954460D+00 , -6.4284412265D+00 /
      data fk( 113), fk( 114) / -2.2516460351D+00 , -4.1420273810D+00 /
      data fk( 115), fk( 116) /  6.0277103449D-02 , -5.9337430415D-01 /
      data fk( 117), fk( 118) / -2.6682378456D+00 ,  4.5561933157D-01 /
      data fk( 119), fk( 120) / -5.0737613183D-02 , -2.0451766202D+01 /
      data fk( 121), fk( 122) / -2.1725204409D+01 , -7.6712810248D-01 /
      data fk( 123), fk( 124) / -1.0598310986D+00 , -4.0883361637D-01 /
      data fk( 125), fk( 126) /  8.7626760736D+01 , -6.6295779525D+00 /
      data fk( 127), fk( 128) / -3.1770815161D-01 , -3.6643924124D-02 /
      data fk( 129), fk( 130) / -4.2417188168D+00 , -1.5281076817D+00 /
      data fk( 131), fk( 132) /  9.3093882995D+00 ,  6.4033097676D-01 /
      data fk( 133), fk( 134) / -8.4966027433D-01 , -1.2418328591D-01 /
      data fk( 135), fk( 136) /  1.6367873410D+01 ,  5.0302920836D-01 /
      data fk( 137), fk( 138) /  1.8813176767D+01 , -2.0962930102D-01 /
      data fk( 139), fk( 140) /  2.0990823569D+01 , -3.5586246596D+00 /
      data fk( 141), fk( 142) /  1.6483684466D+01 , -6.9983505009D+00 /
      data fk( 143), fk( 144) / -2.9405940902D+00 , -1.9831097866D+00 /
      data fk( 145), fk( 146) / -8.9110387859D+00 ,  8.6455681245D+00 /
      data fk( 147), fk( 148) /  2.3001842517D+01 ,  2.7857852946D+00 /
      data fk( 149), fk( 150) /  1.7470759458D+01 , -2.0892874157D+01 /
      data fk( 151), fk( 152) / -1.9972425907D+01 , -1.4299315133D+00 /
      data fk( 153), fk( 154) / -3.5545582340D+00 , -6.9219942698D-01 /
      data fk( 155), fk( 156) /  7.8044482905D+01 ,  5.4153002602D+00 /
      data fk( 157), fk( 158) /  8.0867744326D+01 ,  6.6478163546D+01 /
      data fk( 159), fk( 160) / -5.6766777956D+01 , -1.3322469924D+02 /
      data fk( 161), fk( 162) / -1.0178675214D+02 , -3.0353264541D+01 /
      data fk( 163), fk( 164) /  1.1769296220D+01 , -1.1600673906D+02 /
      data fk( 165), fk( 166) / -2.5321081514D+00 , -1.8132848415D+01 /
      data fk( 167), fk( 168) / -8.1774211588D-01 , -1.1385654356D+00 /
      data fk( 169), fk( 170) / -1.1963425882D+01 ,  6.6588651349D-01 /
      data fk( 171), fk( 172) /  4.1404581153D-01 ,  6.5305179563D+00 /
      data fk( 173), fk( 174) / -5.4034290710D+00 ,  1.3787334740D+00 /
      data fk( 175), fk( 176) / -3.3059169729D+00 , -2.2403720967D+01 /
      data fk( 177), fk( 178) /  9.4981970974D-01 ,  3.8867698882D+00 /
      data fk( 179), fk( 180) / -1.2151053192D+01 ,  3.4484455387D+00 /
      data fk( 181), fk( 182) / -5.1380426156D+00 ,  6.7549652143D+00 /
      data fk( 183), fk( 184) /  3.5212426519D+01 , -1.8067046866D+00 /
      data fk( 185), fk( 186) /  1.4144533739D+00 , -3.2919305304D+00 /
      data fk( 187), fk( 188) /  1.3416895004D+00 ,  1.4332050309D-01 /
      data fk( 189), fk( 190) /  1.9848884725D+01 ,  2.3282491844D+00 /
      data fk( 191), fk( 192) / -6.9529006542D+00 , -9.9841439244D+00 /
      data fk( 193), fk( 194) /  2.4457138820D+00 ,  2.5464217074D+00 /
      data fk( 195), fk( 196) / -2.0485455180D+00 , -3.4072968924D-02 /
      data fk( 197), fk( 198) /  8.7238951257D+01 ,  1.2472302228D+01 /
      data fk( 199), fk( 200) /  1.4468226635D+01 ,  4.6866036738D+00 /
      data fk( 201), fk( 202) / -2.9252099701D+01 , -1.2023674052D+01 /
      data fk( 203), fk( 204) / -1.0887319506D+01 , -1.0123268416D+00 /
      data fk( 205), fk( 206) /  9.4632295388D-01 , -1.7470662357D+01 /
      data fk( 207), fk( 208) / -2.0921972580D+01 ,  1.0217956663D+01 /
      data fk( 209), fk( 210) /  1.1937132959D+01 , -9.3922944966D-01 /
      data fk( 211), fk( 212) /  8.7475657600D+01 ,  1.8257392382D-01 /
      data fk( 213), fk( 214) / -1.4822013134D-01 ,  1.4075482468D-03 /
      data fk( 215), fk( 216) / -2.2789016133D+00 ,  1.1298371690D+01 /
      data fk( 217), fk( 218) /  2.7012762167D+00 , -1.4946431423D+00 /
      data fk( 219), fk( 220) / -8.1026076276D-01 ,  1.1233090254D+02 /
      data fk( 221), fk( 222) / -1.7167575224D+02 ,  4.3731788026D+01 /
      data fk( 223), fk( 224) / -1.6548468571D+02 , -1.1076595221D+02 /
      data fk( 225), fk( 226) /  1.4940570646D-01 , -8.8603820213D-01 /
      data fk( 227), fk( 228) /  1.4176874411D+00 , -5.7781447737D-02 /
      data fk( 229), fk( 230) /  1.5590934458D-01 ,  9.5506030430D-02 /
      data fk( 231), fk( 232) / -8.1054815750D-02 ,  1.0359162469D+00 /
      data fk( 233), fk( 234) / -5.8964982731D-01 , -1.2080498245D+00 /
      data fk( 235), fk( 236) / -3.7536964744D-02 ,  6.9130999543D-02 /
      data fk( 237), fk( 238) / -3.0625084148D-01 ,  2.6256245715D-02 /
      data fk( 239), fk( 240) / -1.6027398242D-01 ,  4.0403034957D+00 /
      data fk( 241), fk( 242) / -2.6934738967D+01 ,  4.4299033499D+00 /
      data fk( 243), fk( 244) / -8.2173352978D+00 , -7.9155499986D-01 /
      data fk( 245), fk( 246) /  1.7810716152D+00 , -9.5705336913D+00 /
      data fk( 247), fk( 248) / -1.3511049114D-01 ,  5.1487779918D-01 /
      data fk( 249), fk( 250) /  1.0734865281D+00 ,  2.5086948192D+01 /
      data fk( 251), fk( 252) / -8.9061347557D+01 ,  6.3353409202D-01 /
      data fk( 253), fk( 254) /  4.3880632729D+01 ,  2.8291125745D+00 /
      data fk( 255), fk( 256) / -5.4826392059D-01 ,  5.0778792107D-01 /
      data fk( 257), fk( 258) / -3.3761578914D+00 ,  3.0810062270D-01 /
      data fk( 259), fk( 260) / -6.0267580420D-02 ,  3.1017950416D+00 /
      data fk( 261), fk( 262) / -2.9049930841D+00 , -9.6932205581D-01 /
      data fk( 263), fk( 264) /  1.5753141327D+00 , -1.0543717504D+00 /
      data fk( 265), fk( 266) /  5.5036972329D-01 , -9.3014584357D-02 /
      data fk( 267), fk( 268) /  3.0855530446D+00 , -2.0753131357D-01 /
      data fk( 269), fk( 270) /  4.5800379946D-01 ,  1.4676627179D+00 /
      data fk( 271), fk( 272) / -1.2621619310D+01 , -1.6179270058D+00 /
      data fk( 273), fk( 274) / -4.7455555164D+00 ,  8.2578895291D+00 /
      data fk( 275), fk( 276) /  6.9602996312D+00 ,  5.4336218718D+00 /
      data fk( 277), fk( 278) /  5.7066088669D+00 ,  7.5220581045D-01 /
      data fk( 279), fk( 280) / -1.3276809527D+00 ,  2.3749682901D+01 /
      data fk( 281), fk( 282) /  1.6250135080D+00 , -2.8009294052D+00 /
      data fk( 283), fk( 284) / -2.9756225708D-01 , -1.9049354369D+01 /
      data fk( 285), fk( 286) / -2.4907588821D+00 , -6.7267563974D+00 /
      data fk( 287), fk( 288) /  3.2545145044D+00 , -1.5686652232D-01 /
      data fk( 289), fk( 290) /  3.3766477843D+00 , -1.0328051349D+02 /
      data fk( 291), fk( 292) / -6.3904346147D+00 , -1.0670401605D+02 /
      data fk( 293), fk( 294) / -8.9709224459D+01 ,  6.8716024700D+01 /
      data fk( 295), fk( 296) /  1.5641594325D+02 ,  1.4151753347D+02 /
      data fk( 297), fk( 298) /  2.8241689038D+01 , -1.8581169388D+01 /
      data fk( 299), fk( 300) /  1.9842165019D+02 ,  1.4805133660D+00 /
      data fk( 301), fk( 302) / -4.9518857333D+00 , -2.4845356261D-01 /
      data fk( 303), fk( 304) / -6.5559677291D-01 , -5.7547012783D+00 /
      data fk( 305), fk( 306) / -4.2479041342D-01 , -1.0707088173D+00 /
      data fk( 307), fk( 308) /  3.6762212495D-01 ,  8.3487384760D-01 /
      data fk( 309), fk( 310) / -4.4217877850D-01 , -8.7215694608D-01 /
      data fk( 311), fk( 312) / -6.3492739214D+00 , -3.5544130467D+00 /
      data fk( 313), fk( 314) / -4.5660229499D-01 ,  9.2816677949D-01 /
      data fk( 315), fk( 316) /  5.8405627908D-01 , -2.8384399951D+00 /
      data fk( 317), fk( 318) /  7.5431462940D+00 ,  1.8006932920D+01 /
      data fk( 319), fk( 320) / -1.3577909532D+00 ,  3.9269337389D-01 /
      data fk( 321), fk( 322) /  1.2285960564D-02 ,  1.5330573429D-01 /
      data fk( 323), fk( 324) / -5.9276467437D-01 ,  1.8892975708D+00 /
      data fk( 325), fk( 326) /  5.5064454659D-01 , -5.5293166024D-01 /
      data fk( 327), fk( 328) / -1.8985564908D+00 , -3.6770816369D-01 /
      data fk( 329), fk( 330) /  5.3889650327D-01 ,  1.1490166407D+01 /
      data fk( 331), fk( 332) / -2.1250396056D+00 , -2.4621368314D-01 /
      data fk( 333), fk( 334) /  2.0261935529D+00 ,  1.4048831786D+01 /
      data fk( 335), fk( 336) /  3.3496401131D+01 ,  9.7910362907D+00 /
      data fk( 337), fk( 338) / -1.4130513179D+00 , -4.4000410546D-01 /
      data fk( 339), fk( 340) /  1.0662384509D+00 , -4.5758257014D+01 /
      data fk( 341), fk( 342) /  9.4905728497D-01 , -1.1082053675D+01 /
      data fk( 343), fk( 344) / -4.4387195376D-01 ,  3.9364931989D+00 /
      data fk( 345), fk( 346) / -5.1616409311D+00 , -3.5242852103D+01 /
      data fk( 347), fk( 348) / -3.7823967898D+00 , -4.6870077367D-02 /
      data fk( 349), fk( 350) / -1.6179164251D+00 ,  4.8385490144D+01 /
      data fk( 351), fk( 352) /  4.6815452478D+00 , -8.1117666851D-01 /
      data fk( 353), fk( 354) /  1.0387858699D+00 ,  1.6531704577D+01 /
      data fk( 355), fk( 356) /  2.2863267578D+00 , -1.2167696773D+00 /
      data fk( 357), fk( 358) / -2.6508732420D+00 ,  2.0103267807D+00 /
      data fk( 359), fk( 360) /  3.8445933137D+00 , -2.7765082686D+00 /
      data fk( 361), fk( 362) / -2.5488959380D+01 ,  7.9902957873D+00 /
      data fk( 363), fk( 364) /  1.3916798449D+00 , -1.1283962991D+01 /
      data fk( 365), fk( 366) /  3.3214179026D+00 , -5.2198884737D-01 /
      data fk( 367), fk( 368) / -1.4516671590D+01 ,  1.7858881207D+01 /
      data fk( 369), fk( 370) / -4.4075542616D-02 , -3.2130191386D-01 /
      data fk( 371), fk( 372) /  1.4081230351D+01 , -4.9763435762D+00 /
      data fk( 373), fk( 374) / -5.7108997348D-01 ,  3.6282657632D+01 /
      data fk( 375), fk( 376) / -6.2261152181D-02 , -9.8026735286D-02 /
      data fk( 377), fk( 378) / -3.4952603722D+00 , -1.3613309383D+01 /
      data fk( 379), fk( 380) /  1.7690795102D-01 , -7.8264055679D+00 /
      data fk( 381), fk( 382) / -2.1977822060D+01 ,  1.7733910646D+00 /
      data fk( 383), fk( 384) / -2.5443612527D+00 ,  2.9106115464D+01 /
      data fk( 385), fk( 386) /  9.8608359001D-01 ,  1.3679763246D+00 /
      data fk( 387), fk( 388) /  8.4545178143D+00 , -8.5709860044D+01 /
      data fk( 389), fk( 390) / -5.2664395971D-01 ,  2.0196060994D-01 /
      data fk( 391), fk( 392) / -2.7273071220D+01 ,  2.7127436849D-01 /
      data fk( 393), fk( 394) /  5.4674969945D+00 , -1.6641009524D+00 /
      data fk( 395), fk( 396) /  9.7362111835D+00 ,  3.9705456151D+00 /
      data fk( 397), fk( 398) /  1.0142271717D+01 ,  4.0355617804D+00 /
      data fk( 399), fk( 400) / -2.5305527109D+00 , -2.7298088239D+01 /
      data fk( 401), fk( 402) / -5.3958175395D+01 , -1.5083763018D+01 /
      data fk( 403), fk( 404) /  3.8138022230D+00 , -2.9813025126D+00 /
      data fk( 405), fk( 406) /  9.4200310989D+00 , -2.0981288035D+00 /
      data fk( 407), fk( 408) /  1.2653544785D+01 , -3.0144102599D+00 /
      data fk( 409), fk( 410) / -3.9545494402D+00 ,  2.4257268986D+00 /
      data fk( 411), fk( 412) /  1.6625345184D+01 ,  4.9833987223D+01 /
      data fk( 413), fk( 414) /  4.3481984341D+00 ,  1.4851605704D+01 /
      data fk( 415), fk( 416) /  4.6557175285D+01 , -1.8187454385D+01 /
      data fk( 417), fk( 418) / -1.9334941252D+01 , -1.5661993247D+02 /
      data fk( 419), fk( 420) /  1.1486529875D+01 ,  1.1669295631D+01 /
      data fk( 421), fk( 422) / -2.8987652292D+00 ,  3.8764489768D+00 /
      data fk( 423), fk( 424) /  3.3549232562D+00 , -1.7399213788D+00 /
      data fk( 425), fk( 426) /  2.6018576127D+00 ,  2.2874638337D+00 /
      data fk( 427), fk( 428) / -4.0033178890D+00 ,  2.7191389733D+00 /
      data fk( 429), fk( 430) / -3.5247104269D+00 ,  5.8633562452D+00 /
      data fk( 431), fk( 432) /  9.9774772906D+00 , -3.9661390001D+00 /
      data fk( 433), fk( 434) /  1.3369844164D+00 , -6.3357727995D+00 /
      data fk( 435), fk( 436) /  2.0417307457D+00 , -9.4261726611D+00 /
      data fk( 437), fk( 438) /  2.7015991025D+01 ,  4.4062943839D-01 /
      data fk( 439), fk( 440) /  3.5360791891D+00 , -8.7042332441D+00 /
      data fk( 441), fk( 442) /  2.3050151546D-01 ,  9.4806187665D-01 /
      data fk( 443), fk( 444) / -3.0673199629D+00 , -2.9225505610D+00 /
      data fk( 445), fk( 446) /  3.8843958585D-01 , -5.8170520304D+00 /
      data fk( 447), fk( 448) /  2.2108022180D+01 ,  7.4812781207D-01 /
      data fk( 449), fk( 450) / -7.6332504262D+00 ,  2.8620245886D+01 /
      data fk( 451), fk( 452) /  6.3903239114D+00 , -4.9789702364D+00 /
      data fk( 453), fk( 454) / -7.4626237463D+00 , -1.0622683062D+01 /
      data fk( 455), fk( 456) / -3.5191754601D-01 ,  6.9802202384D-01 /
      data fk( 457), fk( 458) / -5.0749095727D+01 ,  3.1672691486D-02 /
      data fk( 459), fk( 460) / -3.9357247688D-02 , -5.0789889489D-03 /
      data fk( 461), fk( 462) /  1.5594997305D+00 ,  1.9837228996D-01 /
      data fk( 463), fk( 464) / -3.2065789259D+00 ,  4.2067562723D-01 /
      data fk( 465), fk( 466) /  1.6811410192D-01 , -5.1509490413D+01 /
      data fk( 467), fk( 468) /  6.3369843895D+01 , -1.1141591493D+01 /
      data fk( 469), fk( 470) /  9.4921576364D+01 ,  6.3417143749D+01 /
      data fk( 471), fk( 472) /  4.1489926912D-02 , -3.1188333262D-01 /
      data fk( 473), fk( 474) /  1.2869794273D-01 , -3.4223766366D-02 /
      data fk( 475), fk( 476) /  2.0481813877D-02 , -6.0793366971D-02 /
      data fk( 477), fk( 478) /  6.8244477705D-02 ,  3.3443502073D-01 /
      data fk( 479), fk( 480) /  1.8475404119D-01 , -1.4021605743D-01 /
      data fk( 481), fk( 482) / -2.8294905977D-02 ,  6.5636845573D-03 /
      data fk( 483), fk( 484) /  5.1200048299D-03 , -1.6247659774D-03 /
      data fk( 485), fk( 486) / -7.7824473815D-03 ,  1.5348257836D+00 /
      data fk( 487), fk( 488) / -7.3830263119D+00 ,  1.3411126614D+00 /
      data fk( 489), fk( 490) /  2.6074043038D-01 ,  2.2106136650D-02 /
      data fk( 491), fk( 492) / -1.1946757188D+01 ,  3.4677904094D+01 /
      data fk( 493), fk( 494) / -9.3828917513D-01 , -4.2969197358D+01 /
      data fk( 495), fk( 496) / -6.8807415686D+00 , -2.2748425810D-01 /
      data fk( 497), fk( 498) /  6.9797757856D-01 , -5.3221618590D-01 /
      data fk( 499), fk( 500) /  1.0419139665D-01 , -3.8801308928D-03 /
      data fk( 501), fk( 502) /  7.2806416899D-01 , -3.3021300592D-01 /
      data fk( 503), fk( 504) / -1.9341450184D+00 , -8.7489792652D-02 /
      data fk( 505), fk( 506) / -2.5202213571D-01 ,  5.5774024662D-02 /
      data fk( 507), fk( 508) / -3.7954198827D-03 ,  9.7589721674D-02 /
      data fk( 509), fk( 510) / -2.1470305638D-03 , -9.6116815624D-02 /
      data fk( 511), fk( 512) /  5.4747351214D+00 ,  1.8617327608D+01 /
      data fk( 513), fk( 514) /  2.3475884375D-01 ,  1.2384676886D+00 /
      data fk( 515), fk( 516) /  2.8276454934D-01 ,  3.6394562976D+00 /
      data fk( 517), fk( 518) / -1.9414539015D-01 , -5.6651500398D-02 /
      data fk( 519), fk( 520) /  2.3559408123D-01 ,  3.0225323579D-01 /
      data fk( 521), fk( 522) /  8.9647388703D+00 , -1.3819457202D+00 /
      data fk( 523), fk( 524) /  1.1159112479D-01 , -5.2260723882D-03 /
      data fk( 525), fk( 526) /  3.1629989527D-01 ,  5.2748546774D-02 /
      data fk( 527), fk( 528) / -5.2036495542D+00 , -1.3552392717D+00 /
      data fk( 529), fk( 530) / -8.8310866091D-02 ,  5.3746715626D-01 /
      data fk( 531), fk( 532) /  6.5253459714D-02 ,  1.8112852319D+00 /
      data fk( 533), fk( 534) /  2.4902384434D+00 , -1.6085499550D+00 /
      data fk( 535), fk( 536) / -6.3471762215D+00 , -9.3854067635D-02 /
      data fk( 537), fk( 538) /  5.1084716541D-02 ,  1.5023433230D+00 /
      data fk( 539), fk( 540) / -1.2670469655D+00 ,  1.8743062640D+00 /
      data fk( 541), fk( 542) / -2.7397258131D+00 , -1.3391597341D+00 /
      data fk( 543), fk( 544) /  2.7398780099D+00 ,  1.5323886764D+00 /
      data fk( 545), fk( 546) / -2.9561899415D+01 , -4.8082560840D-01 /
      data fk( 547), fk( 548) / -4.9566483122D-01 , -7.2663408566D-01 /
      data fk( 549), fk( 550) / -7.0508518820D-01 ,  3.7761639655D+00 /
      data fk( 551), fk( 552) /  9.6334387428D-01 ,  6.7646706616D-01 /
      data fk( 553), fk( 554) / -1.4611791758D+00 , -5.8513531929D-01 /
      data fk( 555), fk( 556) / -6.1033042731D-02 ,  4.8277593513D+01 /
      data fk( 557), fk( 558) /  4.5180353360D+00 ,  4.6962174756D+01 /
      data fk( 559), fk( 560) /  4.2987521996D+01 , -2.4489427620D+01 /
      data fk( 561), fk( 562) / -7.6470361214D+01 , -7.0177313232D+01 /
      data fk( 563), fk( 564) / -4.5133977837D+00 ,  1.2864677469D+01 /
      data fk( 565), fk( 566) / -1.1643170306D+02 ,  3.4142416700D-01 /
      data fk( 567), fk( 568) / -4.0100275896D-01 , -9.2853629323D-02 /
      data fk( 569), fk( 570) / -5.7290471398D-02 , -6.5190183732D-01 /
      data fk( 571), fk( 572) / -9.2261084700D-03 , -2.1649296910D-01 /
      data fk( 573), fk( 574) / -7.2894056934D-02 ,  1.0387284446D-01 /
      data fk( 575), fk( 576) / -1.1998580546D-01 ,  6.7685948515D-01 /
      data fk( 577), fk( 578) /  2.1565883762D-01 , -8.9526984966D-01 /
      data fk( 579), fk( 580) / -2.4035810727D-01 ,  3.1418447553D-01 /
      data fk( 581), fk( 582) / -3.4412233632D-01 , -3.2384883008D-01 /
      data fk( 583), fk( 584) /  4.7509777891D-01 ,  2.5490250110D+00 /
      data fk( 585), fk( 586) / -1.1038152613D-01 ,  1.1506567484D-01 /
      data fk( 587), fk( 588) / -3.3099600505D-03 , -5.5986188325D-02 /
      data fk( 589), fk( 590) / -1.9832039811D-02 , -6.4884469222D-02 /
      data fk( 591), fk( 592) /  1.7930104600D-02 , -4.0557437419D-02 /
      data fk( 593), fk( 594) / -8.0580817929D-02 ,  3.0406854317D-01 /
      data fk( 595), fk( 596) /  5.8242028270D-02 , -1.1064799945D+01 /
      data fk( 597), fk( 598) /  4.6753253768D-01 ,  1.0649654663D+00 /
      data fk( 599), fk( 600) / -3.3860834372D+00 ,  2.9407093806D+00 /
      data fk( 601), fk( 602) / -9.8864403650D+00 , -1.4380873945D+01 /
      data fk( 603), fk( 604) / -8.4972277132D+00 , -4.7716068364D+00 /
      data fk( 605), fk( 606) / -4.2680032732D-01 , -1.4006016590D-01 /
      data fk( 607), fk( 608) /  4.9994958973D-01 , -7.5316546826D-01 /
      data fk( 609), fk( 610) /  9.6160930106D+00 , -7.7386942689D-01 /
      data fk( 611), fk( 612) / -1.3865197439D+00 , -1.4603277861D-01 /
      data fk( 613), fk( 614) /  1.8201264400D+00 , -3.5678821854D-02 /
      data fk( 615), fk( 616) / -5.9440299249D-01 , -3.0822491656D+00 /
      data fk( 617), fk( 618) / -3.7660882347D+00 , -2.3635832705D+00 /
      data fk( 619), fk( 620) / -1.4627174528D+00 , -1.4157434244D+01 /
      data fk( 621), fk( 622) /  7.5101333649D+00 ,  5.5850340969D+00 /
      data fk( 623), fk( 624) /  6.6138205182D+00 ,  1.2542505050D+00 /
      data fk( 625), fk( 626) / -3.1655901335D+00 ,  4.7023260370D-01 /
      data fk( 627), fk( 628) /  3.3215352130D-01 ,  3.0301134145D+00 /
      data fk( 629), fk( 630) / -2.2186914732D+00 , -1.7424441426D+00 /
      data fk( 631), fk( 632) /  1.6402166924D+00 ,  4.7106650871D-01 /
      data fk( 633), fk( 634) / -1.6045876645D+00 , -2.0813234279D-01 /
      data fk( 635), fk( 636) /  1.7783871985D-01 ,  3.4126973776D+01 /
      data fk( 637), fk( 638) /  1.0463552215D+01 , -8.3362602687D+00 /
      data fk( 639), fk( 640) /  1.1716254626D+01 , -5.4198883152D+00 /
      data fk( 641), fk( 642) /  1.2219978552D+00 , -2.1423110408D+01 /
      data fk( 643), fk( 644) / -1.5753174788D+01 , -2.9559467286D+00 /
      data fk( 645), fk( 646) /  2.9816491362D+00 ,  2.6728947253D+00 /
      data fk( 647), fk( 648) / -2.7283836885D+00 , -1.2240191387D+01 /
      data fk( 649), fk( 650) /  1.8874024541D+01 ,  4.4905083326D+00 /
      data fk( 651), fk( 652) /  6.5554339086D+01 , -8.2259307672D+00 /
      data fk( 653), fk( 654) /  1.9191178768D+01 , -1.0305785800D+01 /
      data fk( 655), fk( 656) /  1.2326704560D+01 ,  9.9247266272D-01 /
      data fk( 657), fk( 658) / -2.9954311955D-01 , -3.7436334405D-01 /
      data fk( 659), fk( 660) /  1.6700431665D-01 , -2.0921327681D+00 /
      data fk( 661), fk( 662) /  7.3745858604D-01 ,  5.0652535890D-01 /
      data fk( 663), fk( 664) /  4.4066473713D-01 ,  1.4194185042D+00 /
      data fk( 665), fk( 666) /  2.9772502779D+00 ,  3.8666235610D-01 /
      data fk( 667), fk( 668) /  3.3661814429D-02 , -1.2613288381D-01 /
      data fk( 669), fk( 670) / -4.7232009421D-01 , -3.4554330213D-01 /
      data fk( 671), fk( 672) /  1.2928922859D-01 ,  4.0232341783D-03 /
      data fk( 673), fk( 674) /  5.5023574860D-02 ,  6.4842982241D-03 /
      data fk( 675), fk( 676) / -1.5437259371D-02 , -1.8125454852D+00 /
      data fk( 677), fk( 678) / -4.1967952681D+00 , -1.0191447567D+00 /
      data fk( 679), fk( 680) / -3.5057997078D-01 , -2.6388588570D+00 /
      data fk( 681), fk( 682) /  4.1074604555D+00 ,  1.4702772934D+00 /
      data fk( 683), fk( 684) / -3.2273532731D-03 ,  1.2721514751D+00 /
      data fk( 685), fk( 686) / -5.1237216656D+00 , -4.1375756874D+00 /
      data fk( 687), fk( 688) / -1.9560402187D+01 ,  8.3358824912D-01 /
      data fk( 689), fk( 690) / -8.4241567118D-01 , -7.4483403413D-01 /
      data fk( 691), fk( 692) / -3.2370338548D-01 ,  4.5853112085D-02 /
      data fk( 693), fk( 694) / -4.2034726415D-01 ,  2.0481596300D+00 /
      data fk( 695), fk( 696) /  1.6976803952D-01 ,  5.7662204175D-01 /
      data fk( 697), fk( 698) / -8.5076337959D-01 , -2.4674958874D-01 /
      data fk( 699), fk( 700) / -6.6599909764D-02 ,  3.5724910527D-01 /
      data fk( 701), fk( 702) / -4.8189306374D-01 , -1.7343304453D-01 /
      data fk( 703), fk( 704) /  2.8215048283D-01 ,  2.9562312915D-01 /
      data fk( 705), fk( 706) / -8.7834185007D-01 ,  1.4479901494D+00 /
      data fk( 707), fk( 708) /  1.1203920254D+00 ,  6.2553760173D-02 /
      data fk( 709), fk( 710) /  5.1693747238D-02 ,  1.9905766456D-01 /
      data fk( 711), fk( 712) /  7.5721333758D-03 , -4.9222718868D-02 /
      data fk( 713), fk( 714) /  4.2589142664D-01 , -1.3251109180D+00 /
      data fk( 715), fk( 716) / -2.1745283463D-01 ,  1.5625059578D+01 /
      data fk( 717), fk( 718) /  4.5519761842D+00 ,  1.0763054950D+01 /
      data fk( 719), fk( 720) / -3.4947416324D+00 ,  4.7152702577D-01 /
      data fk( 721), fk( 722) / -2.9857273666D+00 , -3.8725869238D+00 /
      data fk( 723), fk( 724) / -1.6960944219D+00 ,  5.4528028049D+00 /
      data fk( 725), fk( 726) /  1.3451032905D+00 ,  6.7052714575D-01 /
      data fk( 727), fk( 728) /  1.5453638484D+00 ,  3.0485609690D+00 /
      data fk( 729), fk( 730) /  1.5501355361D+00 ,  2.2474182122D+00 /
      data fk( 731), fk( 732) / -2.6316174882D+00 , -1.6278012353D-02 /
      data fk( 733), fk( 734) /  7.5451369478D-01 , -6.2969261825D+00 /
      data fk( 735), fk( 736) /  3.3354770226D-01 , -1.0538724279D+01 /
      data fk( 737), fk( 738) / -1.9095498775D+01 , -3.3830051918D+01 /
      data fk( 739), fk( 740) / -8.6574558922D-01 , -7.7249598283D+00 /
      data fk( 741), fk( 742) / -2.2918254290D+01 ,  1.4971877466D+00 /
      data fk( 743), fk( 744) /  6.0783778579D+00 ,  9.4533838435D+01 /
      data fk( 745), fk( 746) / -1.4775377981D+01 ,  2.5811101980D+00 /
      data fk( 747), fk( 748) / -4.1248644048D-01 ,  8.5344951723D-01 /
      data fk( 749), fk( 750) /  5.4274542405D-01 , -3.6669049103D-02 /
      data fk( 751), fk( 752) /  1.6943124427D-01 ,  4.3905895314D-02 /
      data fk( 753), fk( 754) / -3.6281245597D-01 ,  7.7329155339D-01 /
      data fk( 755), fk( 756) / -3.8220012130D-01 ,  3.2307622769D+00 /
      data fk( 757), fk( 758) /  1.2972585171D+00 , -2.8109760840D+00 /
      data fk( 759), fk( 760) /  4.5513976856D+00 ,  3.0241639134D+00 /
      data fk( 761), fk( 762) /  2.9769935470D-01 , -2.4047257221D+00 /
      data fk( 763), fk( 764) /  6.3923356034D+00 ,  1.3530477259D+00 /
      data fk( 765), fk( 766) / -8.2178959333D-02 , -1.0474456882D+00 /
      data fk( 767), fk( 768) / -2.6983627108D-01 , -5.8153907607D-02 /
      data fk( 769), fk( 770) / -5.1877662443D-02 ,  6.2868638275D-01 /
      data fk( 771), fk( 772) /  1.3228525358D-01 , -3.7709394900D-01 /
      data fk( 773), fk( 774) / -1.5335443823D-01 , -1.1384672003D-01 /
      data fk( 775), fk( 776) / -3.0707902342D-01 ,  9.7667994792D+00 /
      data fk( 777), fk( 778) /  2.3318090754D+00 ,  1.6374867153D+01 /
      data fk( 779), fk( 780) / -1.4514201074D+00 ,  2.4917019534D+00 /
      data fk( 781), fk( 782) / -3.8429768806D+00 , -5.0523195221D+00 /
      data fk( 783), fk( 784) /  3.5411183539D+00 , -7.8208539639D+00 /
      data fk( 785), fk( 786) /  3.0117496701D+00 , -4.6414302322D+00 /
      data fk( 787), fk( 788) / -3.4666131229D+00 , -3.9068302394D+00 /
      data fk( 789), fk( 790) /  2.0222824783D+00 , -9.4313962587D-01 /
      data fk( 791), fk( 792) /  3.1916391470D+00 , -7.8974890725D-01 /
      data fk( 793), fk( 794) / -8.6563154233D-01 ,  1.3988972479D+01 /
      data fk( 795), fk( 796) / -2.7994951500D+00 ,  1.4583056232D+00 /
      data fk( 797), fk( 798) / -6.4157811541D+00 ,  3.1708001911D+00 /
      data fk( 799), fk( 800) /  3.1677005859D-01 ,  2.8331344870D+00 /
      data fk( 801), fk( 802) /  9.3533979994D-01 , -1.2640141431D+00 /
      data fk( 803), fk( 804) / -2.7613922706D+00 ,  1.1591501720D+00 /
      data fk( 805), fk( 806) /  6.9056887934D-01 ,  1.6619650921D+01 /
      data fk( 807), fk( 808) /  3.2500148788D+01 ,  1.2458330323D+01 /
      data fk( 809), fk( 810) /  3.0762846072D+00 , -8.4201306767D+00 /
      data fk( 811), fk( 812) /  6.9424196717D+00 , -1.7102041716D+00 /
      data fk( 813), fk( 814) / -8.8321042030D-01 , -2.6180102919D+00 /
      data fk( 815), fk( 816) / -2.1284258718D+00 , -2.8619712990D-01 /
      data fk( 817), fk( 818) / -7.5800524739D-01 ,  9.1159293330D-01 /
      data fk( 819), fk( 820) / -3.7973369005D+00 ,  2.9481136072D+00 /
      data fk( 821), fk( 822) / -3.6548156507D+00 , -3.4884769040D+00 /
      data fk( 823), fk( 824) / -1.5847778013D+00 , -3.2152308861D+00 /
      data fk( 825), fk( 826) / -3.7010743489D-03 ,  9.7220246194D-02 /
      data fk( 827), fk( 828) /  4.0641635598D-01 , -1.2216412134D+00 /
      data fk( 829), fk( 830) /  4.1196900099D-01 , -6.3397849883D+00 /
      data fk( 831), fk( 832) / -1.8407402991D+00 ,  5.6488677669D-01 /
      data fk( 833), fk( 834) / -3.9910272118D+00 ,  1.4370309682D+00 /
      data fk( 835), fk( 836) /  3.0150428027D-01 , -6.3152877279D+00 /
      data fk( 837), fk( 838) / -1.3326617136D+00 , -1.0733683011D+00 /
      data fk( 839), fk( 840) /  1.0472383203D+01 , -3.0033780310D+01 /
      data fk( 841), fk( 842) /  2.7482629803D+00 ,  9.3887074606D+00 /
      data fk( 843), fk( 844) /  2.3813322165D+00 ,  4.3591781952D+00 /
      data fk( 845), fk( 846) /  4.3059522496D+00 , -1.5769948971D+01 /
      data fk( 847), fk( 848) /  5.7733819086D+00 ,  1.6600753246D+00 /
      data fk( 849), fk( 850) / -9.6247319820D-01 , -4.4045967520D-01 /
      data fk( 851), fk( 852) /  4.4517560252D+00 , -2.9285458260D-02 /
      data fk( 853), fk( 854) /  1.6581139537D+00 , -2.8179366767D-01 /
      data fk( 855), fk( 856) /  8.4251944839D-01 , -9.6797919487D-02 /
      data fk( 857), fk( 858) /  2.2652680317D-01 , -6.0840181118D-02 /
      data fk( 859), fk( 860) / -2.6105771287D-01 ,  9.3131865174D-01 /
      data fk( 861), fk( 862) / -1.7978145056D+00 , -2.9571422119D-02 /
      data fk( 863), fk( 864) /  2.1160623140D+00 , -1.3571670720D-01 /
      data fk( 865), fk( 866) / -1.3111603672D-01 , -3.0096326474D+00 /
      data fk( 867), fk( 868) / -3.8489683516D+00 , -7.5819043872D+00 /
      data fk( 869), fk( 870) /  2.1711780881D+00 ,  1.0013199460D+00 /
      data fk( 871), fk( 872) / -4.3585436643D+00 , -4.3444301269D+00 /
      data fk( 873), fk( 874) / -1.1985195136D+01 ,  4.4380018849D+00 /
      data fk( 875), fk( 876) /  4.5992794094D+00 , -7.2593305175D+00 /
      data fk( 877), fk( 878) / -8.0255886515D-01 ,  8.1239606741D+00 /
      data fk( 879), fk( 880) /  8.2552003818D+00 ,  4.6896493727D+00 /
      data fk( 881), fk( 882) /  3.9537139284D+00 ,  8.7714761495D-01 /
      data fk( 883), fk( 884) / -1.6567286356D+00 , -1.0645664109D+00 /
      data fk( 885), fk( 886) / -1.2688758902D+00 , -1.1271637565D+01 /
      data fk( 887), fk( 888) /  6.5008881488D+00 , -3.4064284297D-01 /
      data fk( 889), fk( 890) / -2.7379840545D+00 , -2.8827389013D+00 /
      data fk( 891), fk( 892) / -9.8743816222D-01 , -2.9195822428D+00 /
      data fk( 893), fk( 894) /  7.9502205965D-01 , -1.0850919396D+00 /
      data fk( 895), fk( 896) / -1.3253810174D+00 ,  3.0064124408D+00 /
   
c
      en=0.0d0
      do i=1,nvar
       fmr(i,1) = 1.0d0- exp(-alpham(i)*(rij(i)-shift(i)))
       do j=2,nmax
       fmr(i,j) = fmr(i,1)**j
       enddo
      enddo
c
      en=en+fk(1) 
      en=en+fk(2)*fmr(1,1) 
      en=en+fk(3)*fmr(2,1) 
      en=en+fk(4)*fmr(3,1) 
      en=en+fk(5)*fmr(4,1) 
      en=en+fk(6)*fmr(5,1) 
      en=en+fk(7)*fmr(6,1) 
      en=en+fk(8)*fmr(1,2) 
      en=en+fk(9)*fmr(2,2) 
      en=en+fk(10)*fmr(3,2) 
      en=en+fk(11)*fmr(4,2) 
      en=en+fk(12)*fmr(5,2) 
      en=en+fk(13)*fmr(6,2) 
      en=en+fk(14)*fmr(1,1)*fmr(2,1) 
      en=en+fk(15)*fmr(1,1)*fmr(3,1) 
      en=en+fk(16)*fmr(1,1)*fmr(4,1) 
      en=en+fk(17)*fmr(1,1)*fmr(5,1) 
      en=en+fk(18)*fmr(1,1)*fmr(6,1) 
      en=en+fk(19)*fmr(2,1)*fmr(3,1) 
      en=en+fk(20)*fmr(2,1)*fmr(4,1) 
      en=en+fk(21)*fmr(2,1)*fmr(5,1) 
      en=en+fk(22)*fmr(2,1)*fmr(6,1) 
      en=en+fk(23)*fmr(3,1)*fmr(4,1) 
      en=en+fk(24)*fmr(3,1)*fmr(5,1) 
      en=en+fk(25)*fmr(3,1)*fmr(6,1) 
      en=en+fk(26)*fmr(4,1)*fmr(5,1) 
      en=en+fk(27)*fmr(4,1)*fmr(6,1) 
      en=en+fk(28)*fmr(5,1)*fmr(6,1) 
      en=en+fk(29)*fmr(1,3) 
      en=en+fk(30)*fmr(2,3) 
      en=en+fk(31)*fmr(3,3) 
      en=en+fk(32)*fmr(4,3) 
      en=en+fk(33)*fmr(5,3) 
      en=en+fk(34)*fmr(6,3) 
      en=en+fk(35)*fmr(1,2)*fmr(2,1) 
      en=en+fk(36)*fmr(1,2)*fmr(3,1) 
      en=en+fk(37)*fmr(1,2)*fmr(4,1) 
      en=en+fk(38)*fmr(1,2)*fmr(5,1) 
      en=en+fk(39)*fmr(1,2)*fmr(6,1) 
      en=en+fk(40)*fmr(1,1)*fmr(2,2) 
      en=en+fk(41)*fmr(2,2)*fmr(3,1) 
      en=en+fk(42)*fmr(2,2)*fmr(4,1) 
      en=en+fk(43)*fmr(2,2)*fmr(5,1) 
      en=en+fk(44)*fmr(2,2)*fmr(6,1) 
      en=en+fk(45)*fmr(1,1)*fmr(3,2) 
      en=en+fk(46)*fmr(2,1)*fmr(3,2) 
      en=en+fk(47)*fmr(3,2)*fmr(4,1) 
      en=en+fk(48)*fmr(3,2)*fmr(5,1) 
      en=en+fk(49)*fmr(3,2)*fmr(6,1) 
      en=en+fk(50)*fmr(1,1)*fmr(4,2) 
      en=en+fk(51)*fmr(2,1)*fmr(4,2) 
      en=en+fk(52)*fmr(3,1)*fmr(4,2) 
      en=en+fk(53)*fmr(4,2)*fmr(5,1) 
      en=en+fk(54)*fmr(4,2)*fmr(6,1) 
      en=en+fk(55)*fmr(1,1)*fmr(5,2) 
      en=en+fk(56)*fmr(2,1)*fmr(5,2) 
      en=en+fk(57)*fmr(3,1)*fmr(5,2) 
      en=en+fk(58)*fmr(4,1)*fmr(5,2) 
      en=en+fk(59)*fmr(5,2)*fmr(6,1) 
      en=en+fk(60)*fmr(1,1)*fmr(6,2) 
      en=en+fk(61)*fmr(2,1)*fmr(6,2) 
      en=en+fk(62)*fmr(3,1)*fmr(6,2) 
      en=en+fk(63)*fmr(4,1)*fmr(6,2) 
      en=en+fk(64)*fmr(5,1)*fmr(6,2) 
      en=en+fk(65)*fmr(1,1)*fmr(2,1)*fmr(3,1) 
      en=en+fk(66)*fmr(1,1)*fmr(2,1)*fmr(4,1) 
      en=en+fk(67)*fmr(1,1)*fmr(2,1)*fmr(5,1) 
      en=en+fk(68)*fmr(1,1)*fmr(2,1)*fmr(6,1) 
      en=en+fk(69)*fmr(1,1)*fmr(3,1)*fmr(4,1) 
      en=en+fk(70)*fmr(1,1)*fmr(3,1)*fmr(5,1) 
      en=en+fk(71)*fmr(1,1)*fmr(3,1)*fmr(6,1) 
      en=en+fk(72)*fmr(1,1)*fmr(4,1)*fmr(5,1) 
      en=en+fk(73)*fmr(1,1)*fmr(4,1)*fmr(6,1) 
      en=en+fk(74)*fmr(1,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(75)*fmr(2,1)*fmr(3,1)*fmr(4,1) 
      en=en+fk(76)*fmr(2,1)*fmr(3,1)*fmr(5,1) 
      en=en+fk(77)*fmr(2,1)*fmr(3,1)*fmr(6,1) 
      en=en+fk(78)*fmr(2,1)*fmr(4,1)*fmr(5,1) 
      en=en+fk(79)*fmr(2,1)*fmr(4,1)*fmr(6,1) 
      en=en+fk(80)*fmr(2,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(81)*fmr(3,1)*fmr(4,1)*fmr(5,1) 
      en=en+fk(82)*fmr(3,1)*fmr(4,1)*fmr(6,1) 
      en=en+fk(83)*fmr(3,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(84)*fmr(4,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(85)*fmr(2,4) 
      en=en+fk(86)*fmr(3,4) 
      en=en+fk(87)*fmr(4,4) 
      en=en+fk(88)*fmr(5,4) 
      en=en+fk(89)*fmr(6,4) 
      en=en+fk(90)*fmr(1,3)*fmr(2,1) 
      en=en+fk(91)*fmr(1,3)*fmr(3,1) 
      en=en+fk(92)*fmr(1,3)*fmr(4,1) 
      en=en+fk(93)*fmr(1,3)*fmr(5,1) 
      en=en+fk(94)*fmr(1,3)*fmr(6,1) 
      en=en+fk(95)*fmr(1,1)*fmr(2,3) 
      en=en+fk(96)*fmr(2,3)*fmr(3,1) 
      en=en+fk(97)*fmr(2,3)*fmr(4,1) 
      en=en+fk(98)*fmr(2,3)*fmr(5,1) 
      en=en+fk(99)*fmr(2,3)*fmr(6,1) 
      en=en+fk(100)*fmr(1,1)*fmr(3,3) 
      en=en+fk(101)*fmr(2,1)*fmr(3,3) 
      en=en+fk(102)*fmr(3,3)*fmr(4,1) 
      en=en+fk(103)*fmr(3,3)*fmr(5,1) 
      en=en+fk(104)*fmr(3,3)*fmr(6,1) 
      en=en+fk(105)*fmr(1,1)*fmr(4,3) 
      en=en+fk(106)*fmr(2,1)*fmr(4,3) 
      en=en+fk(107)*fmr(3,1)*fmr(4,3) 
      en=en+fk(108)*fmr(4,3)*fmr(5,1) 
      en=en+fk(109)*fmr(4,3)*fmr(6,1) 
      en=en+fk(110)*fmr(1,1)*fmr(5,3) 
      en=en+fk(111)*fmr(2,1)*fmr(5,3) 
      en=en+fk(112)*fmr(3,1)*fmr(5,3) 
      en=en+fk(113)*fmr(4,1)*fmr(5,3) 
      en=en+fk(114)*fmr(5,3)*fmr(6,1) 
      en=en+fk(115)*fmr(1,1)*fmr(6,3) 
      en=en+fk(116)*fmr(2,1)*fmr(6,3) 
      en=en+fk(117)*fmr(3,1)*fmr(6,3) 
      en=en+fk(118)*fmr(4,1)*fmr(6,3) 
      en=en+fk(119)*fmr(5,1)*fmr(6,3) 
      en=en+fk(120)*fmr(1,2)*fmr(2,2) 
      en=en+fk(121)*fmr(1,2)*fmr(3,2) 
      en=en+fk(122)*fmr(1,2)*fmr(4,2) 
      en=en+fk(123)*fmr(1,2)*fmr(5,2) 
      en=en+fk(124)*fmr(1,2)*fmr(6,2) 
      en=en+fk(125)*fmr(2,2)*fmr(3,2) 
      en=en+fk(126)*fmr(2,2)*fmr(4,2) 
      en=en+fk(127)*fmr(2,2)*fmr(5,2) 
      en=en+fk(128)*fmr(2,2)*fmr(6,2) 
      en=en+fk(129)*fmr(3,2)*fmr(4,2) 
      en=en+fk(130)*fmr(3,2)*fmr(5,2) 
      en=en+fk(131)*fmr(3,2)*fmr(6,2) 
      en=en+fk(132)*fmr(4,2)*fmr(5,2) 
      en=en+fk(133)*fmr(4,2)*fmr(6,2) 
      en=en+fk(134)*fmr(5,2)*fmr(6,2) 
      en=en+fk(135)*fmr(1,2)*fmr(2,1)*fmr(3,1) 
      en=en+fk(136)*fmr(1,2)*fmr(2,1)*fmr(4,1) 
      en=en+fk(137)*fmr(1,2)*fmr(2,1)*fmr(5,1) 
      en=en+fk(138)*fmr(1,2)*fmr(2,1)*fmr(6,1) 
      en=en+fk(139)*fmr(1,2)*fmr(3,1)*fmr(4,1) 
      en=en+fk(140)*fmr(1,2)*fmr(3,1)*fmr(5,1) 
      en=en+fk(141)*fmr(1,2)*fmr(3,1)*fmr(6,1) 
      en=en+fk(142)*fmr(1,2)*fmr(4,1)*fmr(5,1) 
      en=en+fk(143)*fmr(1,2)*fmr(4,1)*fmr(6,1) 
      en=en+fk(144)*fmr(1,2)*fmr(5,1)*fmr(6,1) 
      en=en+fk(145)*fmr(1,1)*fmr(2,2)*fmr(3,1) 
      en=en+fk(146)*fmr(1,1)*fmr(2,2)*fmr(4,1) 
      en=en+fk(147)*fmr(1,1)*fmr(2,2)*fmr(5,1) 
      en=en+fk(148)*fmr(1,1)*fmr(2,2)*fmr(6,1) 
      en=en+fk(149)*fmr(2,2)*fmr(3,1)*fmr(4,1) 
      en=en+fk(150)*fmr(2,2)*fmr(3,1)*fmr(5,1) 
      en=en+fk(151)*fmr(2,2)*fmr(3,1)*fmr(6,1) 
      en=en+fk(152)*fmr(2,2)*fmr(4,1)*fmr(5,1) 
      en=en+fk(153)*fmr(2,2)*fmr(4,1)*fmr(6,1) 
      en=en+fk(154)*fmr(2,2)*fmr(5,1)*fmr(6,1) 
      en=en+fk(155)*fmr(1,1)*fmr(2,1)*fmr(3,2) 
      en=en+fk(156)*fmr(1,1)*fmr(3,2)*fmr(4,1) 
      en=en+fk(157)*fmr(1,1)*fmr(3,2)*fmr(5,1) 
      en=en+fk(158)*fmr(1,1)*fmr(3,2)*fmr(6,1) 
      en=en+fk(159)*fmr(2,1)*fmr(3,2)*fmr(4,1) 
      en=en+fk(160)*fmr(2,1)*fmr(3,2)*fmr(5,1) 
      en=en+fk(161)*fmr(2,1)*fmr(3,2)*fmr(6,1) 
      en=en+fk(162)*fmr(3,2)*fmr(4,1)*fmr(5,1) 
      en=en+fk(163)*fmr(3,2)*fmr(4,1)*fmr(6,1) 
      en=en+fk(164)*fmr(3,2)*fmr(5,1)*fmr(6,1) 
      en=en+fk(165)*fmr(1,1)*fmr(2,1)*fmr(4,2) 
      en=en+fk(166)*fmr(1,1)*fmr(3,1)*fmr(4,2) 
      en=en+fk(167)*fmr(1,1)*fmr(4,2)*fmr(5,1) 
      en=en+fk(168)*fmr(1,1)*fmr(4,2)*fmr(6,1) 
      en=en+fk(169)*fmr(2,1)*fmr(3,1)*fmr(4,2) 
      en=en+fk(170)*fmr(2,1)*fmr(4,2)*fmr(5,1) 
      en=en+fk(171)*fmr(2,1)*fmr(4,2)*fmr(6,1) 
      en=en+fk(172)*fmr(3,1)*fmr(4,2)*fmr(5,1) 
      en=en+fk(173)*fmr(3,1)*fmr(4,2)*fmr(6,1) 
      en=en+fk(174)*fmr(4,2)*fmr(5,1)*fmr(6,1) 
      en=en+fk(175)*fmr(1,1)*fmr(2,1)*fmr(5,2) 
      en=en+fk(176)*fmr(1,1)*fmr(3,1)*fmr(5,2) 
      en=en+fk(177)*fmr(1,1)*fmr(4,1)*fmr(5,2) 
      en=en+fk(178)*fmr(1,1)*fmr(5,2)*fmr(6,1) 
      en=en+fk(179)*fmr(2,1)*fmr(3,1)*fmr(5,2) 
      en=en+fk(180)*fmr(2,1)*fmr(4,1)*fmr(5,2) 
      en=en+fk(181)*fmr(2,1)*fmr(5,2)*fmr(6,1) 
      en=en+fk(182)*fmr(3,1)*fmr(4,1)*fmr(5,2) 
      en=en+fk(183)*fmr(3,1)*fmr(5,2)*fmr(6,1) 
      en=en+fk(184)*fmr(4,1)*fmr(5,2)*fmr(6,1) 
      en=en+fk(185)*fmr(1,1)*fmr(2,1)*fmr(6,2) 
      en=en+fk(186)*fmr(1,1)*fmr(3,1)*fmr(6,2) 
      en=en+fk(187)*fmr(1,1)*fmr(4,1)*fmr(6,2) 
      en=en+fk(188)*fmr(1,1)*fmr(5,1)*fmr(6,2) 
      en=en+fk(189)*fmr(2,1)*fmr(3,1)*fmr(6,2) 
      en=en+fk(190)*fmr(2,1)*fmr(4,1)*fmr(6,2) 
      en=en+fk(191)*fmr(2,1)*fmr(5,1)*fmr(6,2) 
      en=en+fk(192)*fmr(3,1)*fmr(4,1)*fmr(6,2) 
      en=en+fk(193)*fmr(3,1)*fmr(5,1)*fmr(6,2) 
      en=en+fk(194)*fmr(4,1)*fmr(5,1)*fmr(6,2) 
      en=en+fk(195)*fmr(3,1)*fmr(4,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(196)*fmr(2,1)*fmr(4,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(197)*fmr(2,1)*fmr(3,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(198)*fmr(2,1)*fmr(3,1)*fmr(4,1)*fmr(6,1) 
      en=en+fk(199)*fmr(2,1)*fmr(3,1)*fmr(4,1)*fmr(5,1) 
      en=en+fk(200)*fmr(1,1)*fmr(4,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(201)*fmr(1,1)*fmr(3,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(202)*fmr(1,1)*fmr(3,1)*fmr(4,1)*fmr(6,1) 
      en=en+fk(203)*fmr(1,1)*fmr(3,1)*fmr(4,1)*fmr(5,1) 
      en=en+fk(204)*fmr(1,1)*fmr(2,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(205)*fmr(1,1)*fmr(2,1)*fmr(4,1)*fmr(6,1) 
      en=en+fk(206)*fmr(1,1)*fmr(2,1)*fmr(4,1)*fmr(5,1) 
      en=en+fk(207)*fmr(1,1)*fmr(2,1)*fmr(3,1)*fmr(6,1) 
      en=en+fk(208)*fmr(1,1)*fmr(2,1)*fmr(3,1)*fmr(5,1) 
      en=en+fk(209)*fmr(1,1)*fmr(2,1)*fmr(3,1)*fmr(4,1) 
      en=en+fk(210)*fmr(2,5) 
      en=en+fk(211)*fmr(3,5) 
      en=en+fk(212)*fmr(4,5) 
      en=en+fk(213)*fmr(5,5) 
      en=en+fk(214)*fmr(6,5) 
      en=en+fk(215)*fmr(1,1)*fmr(2,4) 
      en=en+fk(216)*fmr(2,4)*fmr(3,1) 
      en=en+fk(217)*fmr(2,4)*fmr(4,1) 
      en=en+fk(218)*fmr(2,4)*fmr(5,1) 
      en=en+fk(219)*fmr(2,4)*fmr(6,1) 
      en=en+fk(220)*fmr(1,1)*fmr(3,4) 
      en=en+fk(221)*fmr(2,1)*fmr(3,4) 
      en=en+fk(222)*fmr(3,4)*fmr(4,1) 
      en=en+fk(223)*fmr(3,4)*fmr(5,1) 
      en=en+fk(224)*fmr(3,4)*fmr(6,1) 
      en=en+fk(225)*fmr(1,1)*fmr(4,4) 
      en=en+fk(226)*fmr(2,1)*fmr(4,4) 
      en=en+fk(227)*fmr(3,1)*fmr(4,4) 
      en=en+fk(228)*fmr(4,4)*fmr(5,1) 
      en=en+fk(229)*fmr(4,4)*fmr(6,1) 
      en=en+fk(230)*fmr(1,1)*fmr(5,4) 
      en=en+fk(231)*fmr(2,1)*fmr(5,4) 
      en=en+fk(232)*fmr(3,1)*fmr(5,4) 
      en=en+fk(233)*fmr(4,1)*fmr(5,4) 
      en=en+fk(234)*fmr(5,4)*fmr(6,1) 
      en=en+fk(235)*fmr(1,1)*fmr(6,4) 
      en=en+fk(236)*fmr(2,1)*fmr(6,4) 
      en=en+fk(237)*fmr(3,1)*fmr(6,4) 
      en=en+fk(238)*fmr(4,1)*fmr(6,4) 
      en=en+fk(239)*fmr(5,1)*fmr(6,4) 
      en=en+fk(240)*fmr(1,3)*fmr(2,2) 
      en=en+fk(241)*fmr(1,3)*fmr(3,2) 
      en=en+fk(242)*fmr(1,3)*fmr(4,2) 
      en=en+fk(243)*fmr(1,3)*fmr(5,2) 
      en=en+fk(244)*fmr(1,3)*fmr(6,2) 
      en=en+fk(245)*fmr(1,2)*fmr(2,3) 
      en=en+fk(246)*fmr(2,3)*fmr(3,2) 
      en=en+fk(247)*fmr(2,3)*fmr(4,2) 
      en=en+fk(248)*fmr(2,3)*fmr(5,2) 
      en=en+fk(249)*fmr(2,3)*fmr(6,2) 
      en=en+fk(250)*fmr(1,2)*fmr(3,3) 
      en=en+fk(251)*fmr(2,2)*fmr(3,3) 
      en=en+fk(252)*fmr(3,3)*fmr(4,2) 
      en=en+fk(253)*fmr(3,3)*fmr(5,2) 
      en=en+fk(254)*fmr(3,3)*fmr(6,2) 
      en=en+fk(255)*fmr(1,2)*fmr(4,3) 
      en=en+fk(256)*fmr(2,2)*fmr(4,3) 
      en=en+fk(257)*fmr(3,2)*fmr(4,3) 
      en=en+fk(258)*fmr(4,3)*fmr(5,2) 
      en=en+fk(259)*fmr(4,3)*fmr(6,2) 
      en=en+fk(260)*fmr(1,2)*fmr(5,3) 
      en=en+fk(261)*fmr(2,2)*fmr(5,3) 
      en=en+fk(262)*fmr(3,2)*fmr(5,3) 
      en=en+fk(263)*fmr(4,2)*fmr(5,3) 
      en=en+fk(264)*fmr(5,3)*fmr(6,2) 
      en=en+fk(265)*fmr(1,2)*fmr(6,3) 
      en=en+fk(266)*fmr(2,2)*fmr(6,3) 
      en=en+fk(267)*fmr(3,2)*fmr(6,3) 
      en=en+fk(268)*fmr(4,2)*fmr(6,3) 
      en=en+fk(269)*fmr(5,2)*fmr(6,3) 
      en=en+fk(270)*fmr(1,3)*fmr(2,1)*fmr(3,1) 
      en=en+fk(271)*fmr(1,3)*fmr(2,1)*fmr(4,1) 
      en=en+fk(272)*fmr(1,3)*fmr(2,1)*fmr(5,1) 
      en=en+fk(273)*fmr(1,3)*fmr(2,1)*fmr(6,1) 
      en=en+fk(274)*fmr(1,3)*fmr(3,1)*fmr(4,1) 
      en=en+fk(275)*fmr(1,3)*fmr(3,1)*fmr(5,1) 
      en=en+fk(276)*fmr(1,3)*fmr(3,1)*fmr(6,1) 
      en=en+fk(277)*fmr(1,3)*fmr(4,1)*fmr(5,1) 
      en=en+fk(278)*fmr(1,3)*fmr(4,1)*fmr(6,1) 
      en=en+fk(279)*fmr(1,3)*fmr(5,1)*fmr(6,1) 
      en=en+fk(280)*fmr(1,1)*fmr(2,3)*fmr(3,1) 
      en=en+fk(281)*fmr(1,1)*fmr(2,3)*fmr(4,1) 
      en=en+fk(282)*fmr(1,1)*fmr(2,3)*fmr(5,1) 
      en=en+fk(283)*fmr(1,1)*fmr(2,3)*fmr(6,1) 
      en=en+fk(284)*fmr(2,3)*fmr(3,1)*fmr(4,1) 
      en=en+fk(285)*fmr(2,3)*fmr(3,1)*fmr(5,1) 
      en=en+fk(286)*fmr(2,3)*fmr(3,1)*fmr(6,1) 
      en=en+fk(287)*fmr(2,3)*fmr(4,1)*fmr(5,1) 
      en=en+fk(288)*fmr(2,3)*fmr(4,1)*fmr(6,1) 
      en=en+fk(289)*fmr(2,3)*fmr(5,1)*fmr(6,1) 
      en=en+fk(290)*fmr(1,1)*fmr(2,1)*fmr(3,3) 
      en=en+fk(291)*fmr(1,1)*fmr(3,3)*fmr(4,1) 
      en=en+fk(292)*fmr(1,1)*fmr(3,3)*fmr(5,1) 
      en=en+fk(293)*fmr(1,1)*fmr(3,3)*fmr(6,1) 
      en=en+fk(294)*fmr(2,1)*fmr(3,3)*fmr(4,1) 
      en=en+fk(295)*fmr(2,1)*fmr(3,3)*fmr(5,1) 
      en=en+fk(296)*fmr(2,1)*fmr(3,3)*fmr(6,1) 
      en=en+fk(297)*fmr(3,3)*fmr(4,1)*fmr(5,1) 
      en=en+fk(298)*fmr(3,3)*fmr(4,1)*fmr(6,1) 
      en=en+fk(299)*fmr(3,3)*fmr(5,1)*fmr(6,1) 
      en=en+fk(300)*fmr(1,1)*fmr(2,1)*fmr(4,3) 
      en=en+fk(301)*fmr(1,1)*fmr(3,1)*fmr(4,3) 
      en=en+fk(302)*fmr(1,1)*fmr(4,3)*fmr(5,1) 
      en=en+fk(303)*fmr(1,1)*fmr(4,3)*fmr(6,1) 
      en=en+fk(304)*fmr(2,1)*fmr(3,1)*fmr(4,3) 
      en=en+fk(305)*fmr(2,1)*fmr(4,3)*fmr(5,1) 
      en=en+fk(306)*fmr(2,1)*fmr(4,3)*fmr(6,1) 
      en=en+fk(307)*fmr(3,1)*fmr(4,3)*fmr(5,1) 
      en=en+fk(308)*fmr(3,1)*fmr(4,3)*fmr(6,1) 
      en=en+fk(309)*fmr(4,3)*fmr(5,1)*fmr(6,1) 
      en=en+fk(310)*fmr(1,1)*fmr(2,1)*fmr(5,3) 
      en=en+fk(311)*fmr(1,1)*fmr(3,1)*fmr(5,3) 
      en=en+fk(312)*fmr(1,1)*fmr(4,1)*fmr(5,3) 
      en=en+fk(313)*fmr(1,1)*fmr(5,3)*fmr(6,1) 
      en=en+fk(314)*fmr(2,1)*fmr(3,1)*fmr(5,3) 
      en=en+fk(315)*fmr(2,1)*fmr(4,1)*fmr(5,3) 
      en=en+fk(316)*fmr(2,1)*fmr(5,3)*fmr(6,1) 
      en=en+fk(317)*fmr(3,1)*fmr(4,1)*fmr(5,3) 
      en=en+fk(318)*fmr(3,1)*fmr(5,3)*fmr(6,1) 
      en=en+fk(319)*fmr(4,1)*fmr(5,3)*fmr(6,1) 
      en=en+fk(320)*fmr(1,1)*fmr(2,1)*fmr(6,3) 
      en=en+fk(321)*fmr(1,1)*fmr(3,1)*fmr(6,3) 
      en=en+fk(322)*fmr(1,1)*fmr(4,1)*fmr(6,3) 
      en=en+fk(323)*fmr(1,1)*fmr(5,1)*fmr(6,3) 
      en=en+fk(324)*fmr(2,1)*fmr(3,1)*fmr(6,3) 
      en=en+fk(325)*fmr(2,1)*fmr(4,1)*fmr(6,3) 
      en=en+fk(326)*fmr(2,1)*fmr(5,1)*fmr(6,3) 
      en=en+fk(327)*fmr(3,1)*fmr(4,1)*fmr(6,3) 
      en=en+fk(328)*fmr(3,1)*fmr(5,1)*fmr(6,3) 
      en=en+fk(329)*fmr(4,1)*fmr(5,1)*fmr(6,3) 
      en=en+fk(330)*fmr(1,1)*fmr(2,2)*fmr(3,2) 
      en=en+fk(331)*fmr(1,1)*fmr(2,2)*fmr(4,2) 
      en=en+fk(332)*fmr(1,1)*fmr(2,2)*fmr(5,2) 
      en=en+fk(333)*fmr(1,1)*fmr(2,2)*fmr(6,2) 
      en=en+fk(334)*fmr(1,1)*fmr(3,2)*fmr(4,2) 
      en=en+fk(335)*fmr(1,1)*fmr(3,2)*fmr(5,2) 
      en=en+fk(336)*fmr(1,1)*fmr(3,2)*fmr(6,2) 
      en=en+fk(337)*fmr(1,1)*fmr(4,2)*fmr(5,2) 
      en=en+fk(338)*fmr(1,1)*fmr(4,2)*fmr(6,2) 
      en=en+fk(339)*fmr(1,1)*fmr(5,2)*fmr(6,2) 
      en=en+fk(340)*fmr(1,2)*fmr(2,1)*fmr(3,2) 
      en=en+fk(341)*fmr(1,2)*fmr(2,1)*fmr(4,2) 
      en=en+fk(342)*fmr(1,2)*fmr(2,1)*fmr(5,2) 
      en=en+fk(343)*fmr(1,2)*fmr(2,1)*fmr(6,2) 
      en=en+fk(344)*fmr(2,1)*fmr(3,2)*fmr(4,2) 
      en=en+fk(345)*fmr(2,1)*fmr(3,2)*fmr(5,2) 
      en=en+fk(346)*fmr(2,1)*fmr(3,2)*fmr(6,2) 
      en=en+fk(347)*fmr(2,1)*fmr(4,2)*fmr(5,2) 
      en=en+fk(348)*fmr(2,1)*fmr(4,2)*fmr(6,2) 
      en=en+fk(349)*fmr(2,1)*fmr(5,2)*fmr(6,2) 
      en=en+fk(350)*fmr(1,2)*fmr(2,2)*fmr(3,1) 
      en=en+fk(351)*fmr(1,2)*fmr(3,1)*fmr(4,2) 
      en=en+fk(352)*fmr(1,2)*fmr(3,1)*fmr(5,2) 
      en=en+fk(353)*fmr(1,2)*fmr(3,1)*fmr(6,2) 
      en=en+fk(354)*fmr(2,2)*fmr(3,1)*fmr(4,2) 
      en=en+fk(355)*fmr(2,2)*fmr(3,1)*fmr(5,2) 
      en=en+fk(356)*fmr(2,2)*fmr(3,1)*fmr(6,2) 
      en=en+fk(357)*fmr(3,1)*fmr(4,2)*fmr(5,2) 
      en=en+fk(358)*fmr(3,1)*fmr(4,2)*fmr(6,2) 
      en=en+fk(359)*fmr(3,1)*fmr(5,2)*fmr(6,2) 
      en=en+fk(360)*fmr(1,2)*fmr(2,2)*fmr(4,1) 
      en=en+fk(361)*fmr(1,2)*fmr(3,2)*fmr(4,1) 
      en=en+fk(362)*fmr(1,2)*fmr(4,1)*fmr(5,2) 
      en=en+fk(363)*fmr(1,2)*fmr(4,1)*fmr(6,2) 
      en=en+fk(364)*fmr(2,2)*fmr(3,2)*fmr(4,1) 
      en=en+fk(365)*fmr(2,2)*fmr(4,1)*fmr(5,2) 
      en=en+fk(366)*fmr(2,2)*fmr(4,1)*fmr(6,2) 
      en=en+fk(367)*fmr(3,2)*fmr(4,1)*fmr(5,2) 
      en=en+fk(368)*fmr(3,2)*fmr(4,1)*fmr(6,2) 
      en=en+fk(369)*fmr(4,1)*fmr(5,2)*fmr(6,2) 
      en=en+fk(370)*fmr(1,2)*fmr(2,2)*fmr(5,1) 
      en=en+fk(371)*fmr(1,2)*fmr(3,2)*fmr(5,1) 
      en=en+fk(372)*fmr(1,2)*fmr(4,2)*fmr(5,1) 
      en=en+fk(373)*fmr(1,2)*fmr(5,1)*fmr(6,2) 
      en=en+fk(374)*fmr(2,2)*fmr(3,2)*fmr(5,1) 
      en=en+fk(375)*fmr(2,2)*fmr(4,2)*fmr(5,1) 
      en=en+fk(376)*fmr(2,2)*fmr(5,1)*fmr(6,2) 
      en=en+fk(377)*fmr(3,2)*fmr(4,2)*fmr(5,1) 
      en=en+fk(378)*fmr(3,2)*fmr(5,1)*fmr(6,2) 
      en=en+fk(379)*fmr(4,2)*fmr(5,1)*fmr(6,2) 
      en=en+fk(380)*fmr(1,2)*fmr(2,2)*fmr(6,1) 
      en=en+fk(381)*fmr(1,2)*fmr(3,2)*fmr(6,1) 
      en=en+fk(382)*fmr(1,2)*fmr(4,2)*fmr(6,1) 
      en=en+fk(383)*fmr(1,2)*fmr(5,2)*fmr(6,1) 
      en=en+fk(384)*fmr(2,2)*fmr(3,2)*fmr(6,1) 
      en=en+fk(385)*fmr(2,2)*fmr(4,2)*fmr(6,1) 
      en=en+fk(386)*fmr(2,2)*fmr(5,2)*fmr(6,1) 
      en=en+fk(387)*fmr(3,2)*fmr(4,2)*fmr(6,1) 
      en=en+fk(388)*fmr(3,2)*fmr(5,2)*fmr(6,1) 
      en=en+fk(389)*fmr(4,2)*fmr(5,2)*fmr(6,1) 
      en=en+fk(390)*fmr(1,2)*fmr(2,1)*fmr(3,1)*fmr(4,1) 
      en=en+fk(391)*fmr(1,2)*fmr(2,1)*fmr(3,1)*fmr(5,1) 
      en=en+fk(392)*fmr(1,2)*fmr(2,1)*fmr(3,1)*fmr(6,1) 
      en=en+fk(393)*fmr(1,2)*fmr(2,1)*fmr(4,1)*fmr(5,1) 
      en=en+fk(394)*fmr(1,2)*fmr(2,1)*fmr(4,1)*fmr(6,1) 
      en=en+fk(395)*fmr(1,2)*fmr(2,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(396)*fmr(1,2)*fmr(3,1)*fmr(4,1)*fmr(5,1) 
      en=en+fk(397)*fmr(1,2)*fmr(3,1)*fmr(4,1)*fmr(6,1) 
      en=en+fk(398)*fmr(1,2)*fmr(3,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(399)*fmr(1,2)*fmr(4,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(400)*fmr(1,1)*fmr(2,2)*fmr(3,1)*fmr(4,1) 
      en=en+fk(401)*fmr(1,1)*fmr(2,2)*fmr(3,1)*fmr(5,1) 
      en=en+fk(402)*fmr(1,1)*fmr(2,2)*fmr(3,1)*fmr(6,1) 
      en=en+fk(403)*fmr(1,1)*fmr(2,2)*fmr(4,1)*fmr(5,1) 
      en=en+fk(404)*fmr(1,1)*fmr(2,2)*fmr(4,1)*fmr(6,1) 
      en=en+fk(405)*fmr(1,1)*fmr(2,2)*fmr(5,1)*fmr(6,1) 
      en=en+fk(406)*fmr(2,2)*fmr(3,1)*fmr(4,1)*fmr(5,1) 
      en=en+fk(407)*fmr(2,2)*fmr(3,1)*fmr(4,1)*fmr(6,1) 
      en=en+fk(408)*fmr(2,2)*fmr(3,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(409)*fmr(2,2)*fmr(4,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(410)*fmr(1,1)*fmr(2,1)*fmr(3,2)*fmr(4,1) 
      en=en+fk(411)*fmr(1,1)*fmr(2,1)*fmr(3,2)*fmr(5,1) 
      en=en+fk(412)*fmr(1,1)*fmr(2,1)*fmr(3,2)*fmr(6,1) 
      en=en+fk(413)*fmr(1,1)*fmr(3,2)*fmr(4,1)*fmr(5,1) 
      en=en+fk(414)*fmr(1,1)*fmr(3,2)*fmr(4,1)*fmr(6,1) 
      en=en+fk(415)*fmr(1,1)*fmr(3,2)*fmr(5,1)*fmr(6,1) 
      en=en+fk(416)*fmr(2,1)*fmr(3,2)*fmr(4,1)*fmr(5,1) 
      en=en+fk(417)*fmr(2,1)*fmr(3,2)*fmr(4,1)*fmr(6,1) 
      en=en+fk(418)*fmr(2,1)*fmr(3,2)*fmr(5,1)*fmr(6,1) 
      en=en+fk(419)*fmr(3,2)*fmr(4,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(420)*fmr(1,1)*fmr(2,1)*fmr(3,1)*fmr(4,2) 
      en=en+fk(421)*fmr(1,1)*fmr(2,1)*fmr(4,2)*fmr(5,1) 
      en=en+fk(422)*fmr(1,1)*fmr(2,1)*fmr(4,2)*fmr(6,1) 
      en=en+fk(423)*fmr(1,1)*fmr(3,1)*fmr(4,2)*fmr(5,1) 
      en=en+fk(424)*fmr(1,1)*fmr(3,1)*fmr(4,2)*fmr(6,1) 
      en=en+fk(425)*fmr(1,1)*fmr(4,2)*fmr(5,1)*fmr(6,1) 
      en=en+fk(426)*fmr(2,1)*fmr(3,1)*fmr(4,2)*fmr(5,1) 
      en=en+fk(427)*fmr(2,1)*fmr(3,1)*fmr(4,2)*fmr(6,1) 
      en=en+fk(428)*fmr(2,1)*fmr(4,2)*fmr(5,1)*fmr(6,1) 
      en=en+fk(429)*fmr(3,1)*fmr(4,2)*fmr(5,1)*fmr(6,1) 
      en=en+fk(430)*fmr(1,1)*fmr(2,1)*fmr(3,1)*fmr(5,2) 
      en=en+fk(431)*fmr(1,1)*fmr(2,1)*fmr(4,1)*fmr(5,2) 
      en=en+fk(432)*fmr(1,1)*fmr(2,1)*fmr(5,2)*fmr(6,1) 
      en=en+fk(433)*fmr(1,1)*fmr(3,1)*fmr(4,1)*fmr(5,2) 
      en=en+fk(434)*fmr(1,1)*fmr(3,1)*fmr(5,2)*fmr(6,1) 
      en=en+fk(435)*fmr(1,1)*fmr(4,1)*fmr(5,2)*fmr(6,1) 
      en=en+fk(436)*fmr(2,1)*fmr(3,1)*fmr(4,1)*fmr(5,2) 
      en=en+fk(437)*fmr(2,1)*fmr(3,1)*fmr(5,2)*fmr(6,1) 
      en=en+fk(438)*fmr(2,1)*fmr(4,1)*fmr(5,2)*fmr(6,1) 
      en=en+fk(439)*fmr(3,1)*fmr(4,1)*fmr(5,2)*fmr(6,1) 
      en=en+fk(440)*fmr(1,1)*fmr(2,1)*fmr(3,1)*fmr(6,2) 
      en=en+fk(441)*fmr(1,1)*fmr(2,1)*fmr(4,1)*fmr(6,2) 
      en=en+fk(442)*fmr(1,1)*fmr(2,1)*fmr(5,1)*fmr(6,2) 
      en=en+fk(443)*fmr(1,1)*fmr(3,1)*fmr(4,1)*fmr(6,2) 
      en=en+fk(444)*fmr(1,1)*fmr(3,1)*fmr(5,1)*fmr(6,2) 
      en=en+fk(445)*fmr(1,1)*fmr(4,1)*fmr(5,1)*fmr(6,2) 
      en=en+fk(446)*fmr(2,1)*fmr(3,1)*fmr(4,1)*fmr(6,2) 
      en=en+fk(447)*fmr(2,1)*fmr(3,1)*fmr(5,1)*fmr(6,2) 
      en=en+fk(448)*fmr(2,1)*fmr(4,1)*fmr(5,1)*fmr(6,2) 
      en=en+fk(449)*fmr(3,1)*fmr(4,1)*fmr(5,1)*fmr(6,2) 
      en=en+fk(450)*fmr(1,1)*fmr(2,1)*fmr(3,1)*fmr(4,1)*fmr(5,1) 
      en=en+fk(451)*fmr(1,1)*fmr(2,1)*fmr(3,1)*fmr(4,1)*fmr(6,1) 
      en=en+fk(452)*fmr(1,1)*fmr(2,1)*fmr(3,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(453)*fmr(1,1)*fmr(2,1)*fmr(4,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(454)*fmr(1,1)*fmr(3,1)*fmr(4,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(455)*fmr(2,1)*fmr(3,1)*fmr(4,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(456)*fmr(2,6) 
      en=en+fk(457)*fmr(3,6) 
      en=en+fk(458)*fmr(4,6) 
      en=en+fk(459)*fmr(5,6) 
      en=en+fk(460)*fmr(6,6) 
      en=en+fk(461)*fmr(1,1)*fmr(2,5) 
      en=en+fk(462)*fmr(2,5)*fmr(3,1) 
      en=en+fk(463)*fmr(2,5)*fmr(4,1) 
      en=en+fk(464)*fmr(2,5)*fmr(5,1) 
      en=en+fk(465)*fmr(2,5)*fmr(6,1) 
      en=en+fk(466)*fmr(1,1)*fmr(3,5) 
      en=en+fk(467)*fmr(2,1)*fmr(3,5) 
      en=en+fk(468)*fmr(3,5)*fmr(4,1) 
      en=en+fk(469)*fmr(3,5)*fmr(5,1) 
      en=en+fk(470)*fmr(3,5)*fmr(6,1) 
      en=en+fk(471)*fmr(1,1)*fmr(4,5) 
      en=en+fk(472)*fmr(2,1)*fmr(4,5) 
      en=en+fk(473)*fmr(3,1)*fmr(4,5) 
      en=en+fk(474)*fmr(4,5)*fmr(5,1) 
      en=en+fk(475)*fmr(4,5)*fmr(6,1) 
      en=en+fk(476)*fmr(1,1)*fmr(5,5) 
      en=en+fk(477)*fmr(2,1)*fmr(5,5) 
      en=en+fk(478)*fmr(3,1)*fmr(5,5) 
      en=en+fk(479)*fmr(4,1)*fmr(5,5) 
      en=en+fk(480)*fmr(5,5)*fmr(6,1) 
      en=en+fk(481)*fmr(1,1)*fmr(6,5) 
      en=en+fk(482)*fmr(2,1)*fmr(6,5) 
      en=en+fk(483)*fmr(3,1)*fmr(6,5) 
      en=en+fk(484)*fmr(4,1)*fmr(6,5) 
      en=en+fk(485)*fmr(5,1)*fmr(6,5) 
      en=en+fk(486)*fmr(1,2)*fmr(2,4) 
      en=en+fk(487)*fmr(2,4)*fmr(3,2) 
      en=en+fk(488)*fmr(2,4)*fmr(4,2) 
      en=en+fk(489)*fmr(2,4)*fmr(5,2) 
      en=en+fk(490)*fmr(2,4)*fmr(6,2) 
      en=en+fk(491)*fmr(1,2)*fmr(3,4) 
      en=en+fk(492)*fmr(2,2)*fmr(3,4) 
      en=en+fk(493)*fmr(3,4)*fmr(4,2) 
      en=en+fk(494)*fmr(3,4)*fmr(5,2) 
      en=en+fk(495)*fmr(3,4)*fmr(6,2) 
      en=en+fk(496)*fmr(1,2)*fmr(4,4) 
      en=en+fk(497)*fmr(2,2)*fmr(4,4) 
      en=en+fk(498)*fmr(3,2)*fmr(4,4) 
      en=en+fk(499)*fmr(4,4)*fmr(5,2) 
      en=en+fk(500)*fmr(4,4)*fmr(6,2) 
      en=en+fk(501)*fmr(1,2)*fmr(5,4) 
      en=en+fk(502)*fmr(2,2)*fmr(5,4) 
      en=en+fk(503)*fmr(3,2)*fmr(5,4) 
      en=en+fk(504)*fmr(4,2)*fmr(5,4) 
      en=en+fk(505)*fmr(5,4)*fmr(6,2) 
      en=en+fk(506)*fmr(1,2)*fmr(6,4) 
      en=en+fk(507)*fmr(2,2)*fmr(6,4) 
      en=en+fk(508)*fmr(3,2)*fmr(6,4) 
      en=en+fk(509)*fmr(4,2)*fmr(6,4) 
      en=en+fk(510)*fmr(5,2)*fmr(6,4) 
      en=en+fk(511)*fmr(1,3)*fmr(2,3) 
      en=en+fk(512)*fmr(1,3)*fmr(3,3) 
      en=en+fk(513)*fmr(1,3)*fmr(4,3) 
      en=en+fk(514)*fmr(1,3)*fmr(5,3) 
      en=en+fk(515)*fmr(1,3)*fmr(6,3) 
      en=en+fk(516)*fmr(2,3)*fmr(3,3) 
      en=en+fk(517)*fmr(2,3)*fmr(4,3) 
      en=en+fk(518)*fmr(2,3)*fmr(5,3) 
      en=en+fk(519)*fmr(2,3)*fmr(6,3) 
      en=en+fk(520)*fmr(3,3)*fmr(4,3) 
      en=en+fk(521)*fmr(3,3)*fmr(5,3) 
      en=en+fk(522)*fmr(3,3)*fmr(6,3) 
      en=en+fk(523)*fmr(4,3)*fmr(5,3) 
      en=en+fk(524)*fmr(4,3)*fmr(6,3) 
      en=en+fk(525)*fmr(5,3)*fmr(6,3) 
      en=en+fk(526)*fmr(4,2)*fmr(5,2)*fmr(6,2) 
      en=en+fk(527)*fmr(3,2)*fmr(5,2)*fmr(6,2) 
      en=en+fk(528)*fmr(3,2)*fmr(4,2)*fmr(6,2) 
      en=en+fk(529)*fmr(3,2)*fmr(4,2)*fmr(5,2) 
      en=en+fk(530)*fmr(2,2)*fmr(5,2)*fmr(6,2) 
      en=en+fk(531)*fmr(2,2)*fmr(4,2)*fmr(6,2) 
      en=en+fk(532)*fmr(2,2)*fmr(4,2)*fmr(5,2) 
      en=en+fk(533)*fmr(2,2)*fmr(3,2)*fmr(6,2) 
      en=en+fk(534)*fmr(2,2)*fmr(3,2)*fmr(5,2) 
      en=en+fk(535)*fmr(2,2)*fmr(3,2)*fmr(4,2) 
      en=en+fk(536)*fmr(1,2)*fmr(5,2)*fmr(6,2) 
      en=en+fk(537)*fmr(1,2)*fmr(4,2)*fmr(6,2) 
      en=en+fk(538)*fmr(1,2)*fmr(4,2)*fmr(5,2) 
      en=en+fk(539)*fmr(1,2)*fmr(3,2)*fmr(6,2) 
      en=en+fk(540)*fmr(1,2)*fmr(3,2)*fmr(5,2) 
      en=en+fk(541)*fmr(1,2)*fmr(3,2)*fmr(4,2) 
      en=en+fk(542)*fmr(1,2)*fmr(2,2)*fmr(6,2) 
      en=en+fk(543)*fmr(1,2)*fmr(2,2)*fmr(5,2) 
      en=en+fk(544)*fmr(1,2)*fmr(2,2)*fmr(4,2) 
      en=en+fk(545)*fmr(1,2)*fmr(2,2)*fmr(3,2) 
      en=en+fk(546)*fmr(1,1)*fmr(2,4)*fmr(3,1) 
      en=en+fk(547)*fmr(1,1)*fmr(2,4)*fmr(4,1) 
      en=en+fk(548)*fmr(1,1)*fmr(2,4)*fmr(5,1) 
      en=en+fk(549)*fmr(1,1)*fmr(2,4)*fmr(6,1) 
      en=en+fk(550)*fmr(2,4)*fmr(3,1)*fmr(4,1) 
      en=en+fk(551)*fmr(2,4)*fmr(3,1)*fmr(5,1) 
      en=en+fk(552)*fmr(2,4)*fmr(3,1)*fmr(6,1) 
      en=en+fk(553)*fmr(2,4)*fmr(4,1)*fmr(5,1) 
      en=en+fk(554)*fmr(2,4)*fmr(4,1)*fmr(6,1) 
      en=en+fk(555)*fmr(2,4)*fmr(5,1)*fmr(6,1) 
      en=en+fk(556)*fmr(1,1)*fmr(2,1)*fmr(3,4) 
      en=en+fk(557)*fmr(1,1)*fmr(3,4)*fmr(4,1) 
      en=en+fk(558)*fmr(1,1)*fmr(3,4)*fmr(5,1) 
      en=en+fk(559)*fmr(1,1)*fmr(3,4)*fmr(6,1) 
      en=en+fk(560)*fmr(2,1)*fmr(3,4)*fmr(4,1) 
      en=en+fk(561)*fmr(2,1)*fmr(3,4)*fmr(5,1) 
      en=en+fk(562)*fmr(2,1)*fmr(3,4)*fmr(6,1) 
      en=en+fk(563)*fmr(3,4)*fmr(4,1)*fmr(5,1) 
      en=en+fk(564)*fmr(3,4)*fmr(4,1)*fmr(6,1) 
      en=en+fk(565)*fmr(3,4)*fmr(5,1)*fmr(6,1) 
      en=en+fk(566)*fmr(1,1)*fmr(2,1)*fmr(4,4) 
      en=en+fk(567)*fmr(1,1)*fmr(3,1)*fmr(4,4) 
      en=en+fk(568)*fmr(1,1)*fmr(4,4)*fmr(5,1) 
      en=en+fk(569)*fmr(1,1)*fmr(4,4)*fmr(6,1) 
      en=en+fk(570)*fmr(2,1)*fmr(3,1)*fmr(4,4) 
      en=en+fk(571)*fmr(2,1)*fmr(4,4)*fmr(5,1) 
      en=en+fk(572)*fmr(2,1)*fmr(4,4)*fmr(6,1) 
      en=en+fk(573)*fmr(3,1)*fmr(4,4)*fmr(5,1) 
      en=en+fk(574)*fmr(3,1)*fmr(4,4)*fmr(6,1) 
      en=en+fk(575)*fmr(4,4)*fmr(5,1)*fmr(6,1) 
      en=en+fk(576)*fmr(1,1)*fmr(2,1)*fmr(5,4) 
      en=en+fk(577)*fmr(1,1)*fmr(3,1)*fmr(5,4) 
      en=en+fk(578)*fmr(1,1)*fmr(4,1)*fmr(5,4) 
      en=en+fk(579)*fmr(1,1)*fmr(5,4)*fmr(6,1) 
      en=en+fk(580)*fmr(2,1)*fmr(3,1)*fmr(5,4) 
      en=en+fk(581)*fmr(2,1)*fmr(4,1)*fmr(5,4) 
      en=en+fk(582)*fmr(2,1)*fmr(5,4)*fmr(6,1) 
      en=en+fk(583)*fmr(3,1)*fmr(4,1)*fmr(5,4) 
      en=en+fk(584)*fmr(3,1)*fmr(5,4)*fmr(6,1) 
      en=en+fk(585)*fmr(4,1)*fmr(5,4)*fmr(6,1) 
      en=en+fk(586)*fmr(1,1)*fmr(2,1)*fmr(6,4) 
      en=en+fk(587)*fmr(1,1)*fmr(3,1)*fmr(6,4) 
      en=en+fk(588)*fmr(1,1)*fmr(4,1)*fmr(6,4) 
      en=en+fk(589)*fmr(1,1)*fmr(5,1)*fmr(6,4) 
      en=en+fk(590)*fmr(2,1)*fmr(3,1)*fmr(6,4) 
      en=en+fk(591)*fmr(2,1)*fmr(4,1)*fmr(6,4) 
      en=en+fk(592)*fmr(2,1)*fmr(5,1)*fmr(6,4) 
      en=en+fk(593)*fmr(3,1)*fmr(4,1)*fmr(6,4) 
      en=en+fk(594)*fmr(3,1)*fmr(5,1)*fmr(6,4) 
      en=en+fk(595)*fmr(4,1)*fmr(5,1)*fmr(6,4) 
      en=en+fk(596)*fmr(1,3)*fmr(2,2)*fmr(3,1) 
      en=en+fk(597)*fmr(1,3)*fmr(2,2)*fmr(4,1) 
      en=en+fk(598)*fmr(1,3)*fmr(2,2)*fmr(5,1) 
      en=en+fk(599)*fmr(1,3)*fmr(2,2)*fmr(6,1) 
      en=en+fk(600)*fmr(1,3)*fmr(2,1)*fmr(3,2) 
      en=en+fk(601)*fmr(1,3)*fmr(3,2)*fmr(4,1) 
      en=en+fk(602)*fmr(1,3)*fmr(3,2)*fmr(5,1) 
      en=en+fk(603)*fmr(1,3)*fmr(3,2)*fmr(6,1) 
      en=en+fk(604)*fmr(1,3)*fmr(2,1)*fmr(4,2) 
      en=en+fk(605)*fmr(1,3)*fmr(3,1)*fmr(4,2) 
      en=en+fk(606)*fmr(1,3)*fmr(4,2)*fmr(5,1) 
      en=en+fk(607)*fmr(1,3)*fmr(4,2)*fmr(6,1) 
      en=en+fk(608)*fmr(1,3)*fmr(2,1)*fmr(5,2) 
      en=en+fk(609)*fmr(1,3)*fmr(3,1)*fmr(5,2) 
      en=en+fk(610)*fmr(1,3)*fmr(4,1)*fmr(5,2) 
      en=en+fk(611)*fmr(1,3)*fmr(5,2)*fmr(6,1) 
      en=en+fk(612)*fmr(1,3)*fmr(2,1)*fmr(6,2) 
      en=en+fk(613)*fmr(1,3)*fmr(3,1)*fmr(6,2) 
      en=en+fk(614)*fmr(1,3)*fmr(4,1)*fmr(6,2) 
      en=en+fk(615)*fmr(1,3)*fmr(5,1)*fmr(6,2) 
      en=en+fk(616)*fmr(1,2)*fmr(2,3)*fmr(3,1) 
      en=en+fk(617)*fmr(1,2)*fmr(2,3)*fmr(4,1) 
      en=en+fk(618)*fmr(1,2)*fmr(2,3)*fmr(5,1) 
      en=en+fk(619)*fmr(1,2)*fmr(2,3)*fmr(6,1) 
      en=en+fk(620)*fmr(1,1)*fmr(2,3)*fmr(3,2) 
      en=en+fk(621)*fmr(2,3)*fmr(3,2)*fmr(4,1) 
      en=en+fk(622)*fmr(2,3)*fmr(3,2)*fmr(5,1) 
      en=en+fk(623)*fmr(2,3)*fmr(3,2)*fmr(6,1) 
      en=en+fk(624)*fmr(1,1)*fmr(2,3)*fmr(4,2) 
      en=en+fk(625)*fmr(2,3)*fmr(3,1)*fmr(4,2) 
      en=en+fk(626)*fmr(2,3)*fmr(4,2)*fmr(5,1) 
      en=en+fk(627)*fmr(2,3)*fmr(4,2)*fmr(6,1) 
      en=en+fk(628)*fmr(1,1)*fmr(2,3)*fmr(5,2) 
      en=en+fk(629)*fmr(2,3)*fmr(3,1)*fmr(5,2) 
      en=en+fk(630)*fmr(2,3)*fmr(4,1)*fmr(5,2) 
      en=en+fk(631)*fmr(2,3)*fmr(5,2)*fmr(6,1) 
      en=en+fk(632)*fmr(1,1)*fmr(2,3)*fmr(6,2) 
      en=en+fk(633)*fmr(2,3)*fmr(3,1)*fmr(6,2) 
      en=en+fk(634)*fmr(2,3)*fmr(4,1)*fmr(6,2) 
      en=en+fk(635)*fmr(2,3)*fmr(5,1)*fmr(6,2) 
      en=en+fk(636)*fmr(1,2)*fmr(2,1)*fmr(3,3) 
      en=en+fk(637)*fmr(1,2)*fmr(3,3)*fmr(4,1) 
      en=en+fk(638)*fmr(1,2)*fmr(3,3)*fmr(5,1) 
      en=en+fk(639)*fmr(1,2)*fmr(3,3)*fmr(6,1) 
      en=en+fk(640)*fmr(1,1)*fmr(2,2)*fmr(3,3) 
      en=en+fk(641)*fmr(2,2)*fmr(3,3)*fmr(4,1) 
      en=en+fk(642)*fmr(2,2)*fmr(3,3)*fmr(5,1) 
      en=en+fk(643)*fmr(2,2)*fmr(3,3)*fmr(6,1) 
      en=en+fk(644)*fmr(1,1)*fmr(3,3)*fmr(4,2) 
      en=en+fk(645)*fmr(2,1)*fmr(3,3)*fmr(4,2) 
      en=en+fk(646)*fmr(3,3)*fmr(4,2)*fmr(5,1) 
      en=en+fk(647)*fmr(3,3)*fmr(4,2)*fmr(6,1) 
      en=en+fk(648)*fmr(1,1)*fmr(3,3)*fmr(5,2) 
      en=en+fk(649)*fmr(2,1)*fmr(3,3)*fmr(5,2) 
      en=en+fk(650)*fmr(3,3)*fmr(4,1)*fmr(5,2) 
      en=en+fk(651)*fmr(3,3)*fmr(5,2)*fmr(6,1) 
      en=en+fk(652)*fmr(1,1)*fmr(3,3)*fmr(6,2) 
      en=en+fk(653)*fmr(2,1)*fmr(3,3)*fmr(6,2) 
      en=en+fk(654)*fmr(3,3)*fmr(4,1)*fmr(6,2) 
      en=en+fk(655)*fmr(3,3)*fmr(5,1)*fmr(6,2) 
      en=en+fk(656)*fmr(1,2)*fmr(2,1)*fmr(4,3) 
      en=en+fk(657)*fmr(1,2)*fmr(3,1)*fmr(4,3) 
      en=en+fk(658)*fmr(1,2)*fmr(4,3)*fmr(5,1) 
      en=en+fk(659)*fmr(1,2)*fmr(4,3)*fmr(6,1) 
      en=en+fk(660)*fmr(1,1)*fmr(2,2)*fmr(4,3) 
      en=en+fk(661)*fmr(2,2)*fmr(3,1)*fmr(4,3) 
      en=en+fk(662)*fmr(2,2)*fmr(4,3)*fmr(5,1) 
      en=en+fk(663)*fmr(2,2)*fmr(4,3)*fmr(6,1) 
      en=en+fk(664)*fmr(1,1)*fmr(3,2)*fmr(4,3) 
      en=en+fk(665)*fmr(2,1)*fmr(3,2)*fmr(4,3) 
      en=en+fk(666)*fmr(3,2)*fmr(4,3)*fmr(5,1) 
      en=en+fk(667)*fmr(3,2)*fmr(4,3)*fmr(6,1) 
      en=en+fk(668)*fmr(1,1)*fmr(4,3)*fmr(5,2) 
      en=en+fk(669)*fmr(2,1)*fmr(4,3)*fmr(5,2) 
      en=en+fk(670)*fmr(3,1)*fmr(4,3)*fmr(5,2) 
      en=en+fk(671)*fmr(4,3)*fmr(5,2)*fmr(6,1) 
      en=en+fk(672)*fmr(1,1)*fmr(4,3)*fmr(6,2) 
      en=en+fk(673)*fmr(2,1)*fmr(4,3)*fmr(6,2) 
      en=en+fk(674)*fmr(3,1)*fmr(4,3)*fmr(6,2) 
      en=en+fk(675)*fmr(4,3)*fmr(5,1)*fmr(6,2) 
      en=en+fk(676)*fmr(1,2)*fmr(2,1)*fmr(5,3) 
      en=en+fk(677)*fmr(1,2)*fmr(3,1)*fmr(5,3) 
      en=en+fk(678)*fmr(1,2)*fmr(4,1)*fmr(5,3) 
      en=en+fk(679)*fmr(1,2)*fmr(5,3)*fmr(6,1) 
      en=en+fk(680)*fmr(1,1)*fmr(2,2)*fmr(5,3) 
      en=en+fk(681)*fmr(2,2)*fmr(3,1)*fmr(5,3) 
      en=en+fk(682)*fmr(2,2)*fmr(4,1)*fmr(5,3) 
      en=en+fk(683)*fmr(2,2)*fmr(5,3)*fmr(6,1) 
      en=en+fk(684)*fmr(1,1)*fmr(3,2)*fmr(5,3) 
      en=en+fk(685)*fmr(2,1)*fmr(3,2)*fmr(5,3) 
      en=en+fk(686)*fmr(3,2)*fmr(4,1)*fmr(5,3) 
      en=en+fk(687)*fmr(3,2)*fmr(5,3)*fmr(6,1) 
      en=en+fk(688)*fmr(1,1)*fmr(4,2)*fmr(5,3) 
      en=en+fk(689)*fmr(2,1)*fmr(4,2)*fmr(5,3) 
      en=en+fk(690)*fmr(3,1)*fmr(4,2)*fmr(5,3) 
      en=en+fk(691)*fmr(4,2)*fmr(5,3)*fmr(6,1) 
      en=en+fk(692)*fmr(1,1)*fmr(5,3)*fmr(6,2) 
      en=en+fk(693)*fmr(2,1)*fmr(5,3)*fmr(6,2) 
      en=en+fk(694)*fmr(3,1)*fmr(5,3)*fmr(6,2) 
      en=en+fk(695)*fmr(4,1)*fmr(5,3)*fmr(6,2) 
      en=en+fk(696)*fmr(1,2)*fmr(2,1)*fmr(6,3) 
      en=en+fk(697)*fmr(1,2)*fmr(3,1)*fmr(6,3) 
      en=en+fk(698)*fmr(1,2)*fmr(4,1)*fmr(6,3) 
      en=en+fk(699)*fmr(1,2)*fmr(5,1)*fmr(6,3) 
      en=en+fk(700)*fmr(1,1)*fmr(2,2)*fmr(6,3) 
      en=en+fk(701)*fmr(2,2)*fmr(3,1)*fmr(6,3) 
      en=en+fk(702)*fmr(2,2)*fmr(4,1)*fmr(6,3) 
      en=en+fk(703)*fmr(2,2)*fmr(5,1)*fmr(6,3) 
      en=en+fk(704)*fmr(1,1)*fmr(3,2)*fmr(6,3) 
      en=en+fk(705)*fmr(2,1)*fmr(3,2)*fmr(6,3) 
      en=en+fk(706)*fmr(3,2)*fmr(4,1)*fmr(6,3) 
      en=en+fk(707)*fmr(3,2)*fmr(5,1)*fmr(6,3) 
      en=en+fk(708)*fmr(1,1)*fmr(4,2)*fmr(6,3) 
      en=en+fk(709)*fmr(2,1)*fmr(4,2)*fmr(6,3) 
      en=en+fk(710)*fmr(3,1)*fmr(4,2)*fmr(6,3) 
      en=en+fk(711)*fmr(4,2)*fmr(5,1)*fmr(6,3) 
      en=en+fk(712)*fmr(1,1)*fmr(5,2)*fmr(6,3) 
      en=en+fk(713)*fmr(2,1)*fmr(5,2)*fmr(6,3) 
      en=en+fk(714)*fmr(3,1)*fmr(5,2)*fmr(6,3) 
      en=en+fk(715)*fmr(4,1)*fmr(5,2)*fmr(6,3) 
      en=en+fk(716)*fmr(1,3)*fmr(2,1)*fmr(3,1)*fmr(4,1) 
      en=en+fk(717)*fmr(1,3)*fmr(2,1)*fmr(3,1)*fmr(5,1) 
      en=en+fk(718)*fmr(1,3)*fmr(2,1)*fmr(3,1)*fmr(6,1) 
      en=en+fk(719)*fmr(1,3)*fmr(2,1)*fmr(4,1)*fmr(5,1) 
      en=en+fk(720)*fmr(1,3)*fmr(2,1)*fmr(4,1)*fmr(6,1) 
      en=en+fk(721)*fmr(1,3)*fmr(2,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(722)*fmr(1,3)*fmr(3,1)*fmr(4,1)*fmr(5,1) 
      en=en+fk(723)*fmr(1,3)*fmr(3,1)*fmr(4,1)*fmr(6,1) 
      en=en+fk(724)*fmr(1,3)*fmr(3,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(725)*fmr(1,3)*fmr(4,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(726)*fmr(1,1)*fmr(2,3)*fmr(3,1)*fmr(4,1) 
      en=en+fk(727)*fmr(1,1)*fmr(2,3)*fmr(3,1)*fmr(5,1) 
      en=en+fk(728)*fmr(1,1)*fmr(2,3)*fmr(3,1)*fmr(6,1) 
      en=en+fk(729)*fmr(1,1)*fmr(2,3)*fmr(4,1)*fmr(5,1) 
      en=en+fk(730)*fmr(1,1)*fmr(2,3)*fmr(4,1)*fmr(6,1) 
      en=en+fk(731)*fmr(1,1)*fmr(2,3)*fmr(5,1)*fmr(6,1) 
      en=en+fk(732)*fmr(2,3)*fmr(3,1)*fmr(4,1)*fmr(5,1) 
      en=en+fk(733)*fmr(2,3)*fmr(3,1)*fmr(4,1)*fmr(6,1) 
      en=en+fk(734)*fmr(2,3)*fmr(3,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(735)*fmr(2,3)*fmr(4,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(736)*fmr(1,1)*fmr(2,1)*fmr(3,3)*fmr(4,1) 
      en=en+fk(737)*fmr(1,1)*fmr(2,1)*fmr(3,3)*fmr(5,1) 
      en=en+fk(738)*fmr(1,1)*fmr(2,1)*fmr(3,3)*fmr(6,1) 
      en=en+fk(739)*fmr(1,1)*fmr(3,3)*fmr(4,1)*fmr(5,1) 
      en=en+fk(740)*fmr(1,1)*fmr(3,3)*fmr(4,1)*fmr(6,1) 
      en=en+fk(741)*fmr(1,1)*fmr(3,3)*fmr(5,1)*fmr(6,1) 
      en=en+fk(742)*fmr(2,1)*fmr(3,3)*fmr(4,1)*fmr(5,1) 
      en=en+fk(743)*fmr(2,1)*fmr(3,3)*fmr(4,1)*fmr(6,1) 
      en=en+fk(744)*fmr(2,1)*fmr(3,3)*fmr(5,1)*fmr(6,1) 
      en=en+fk(745)*fmr(3,3)*fmr(4,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(746)*fmr(1,1)*fmr(2,1)*fmr(3,1)*fmr(4,3) 
      en=en+fk(747)*fmr(1,1)*fmr(2,1)*fmr(4,3)*fmr(5,1) 
      en=en+fk(748)*fmr(1,1)*fmr(2,1)*fmr(4,3)*fmr(6,1) 
      en=en+fk(749)*fmr(1,1)*fmr(3,1)*fmr(4,3)*fmr(5,1) 
      en=en+fk(750)*fmr(1,1)*fmr(3,1)*fmr(4,3)*fmr(6,1) 
      en=en+fk(751)*fmr(1,1)*fmr(4,3)*fmr(5,1)*fmr(6,1) 
      en=en+fk(752)*fmr(2,1)*fmr(3,1)*fmr(4,3)*fmr(5,1) 
      en=en+fk(753)*fmr(2,1)*fmr(3,1)*fmr(4,3)*fmr(6,1) 
      en=en+fk(754)*fmr(2,1)*fmr(4,3)*fmr(5,1)*fmr(6,1) 
      en=en+fk(755)*fmr(3,1)*fmr(4,3)*fmr(5,1)*fmr(6,1) 
      en=en+fk(756)*fmr(1,1)*fmr(2,1)*fmr(3,1)*fmr(5,3) 
      en=en+fk(757)*fmr(1,1)*fmr(2,1)*fmr(4,1)*fmr(5,3) 
      en=en+fk(758)*fmr(1,1)*fmr(2,1)*fmr(5,3)*fmr(6,1) 
      en=en+fk(759)*fmr(1,1)*fmr(3,1)*fmr(4,1)*fmr(5,3) 
      en=en+fk(760)*fmr(1,1)*fmr(3,1)*fmr(5,3)*fmr(6,1) 
      en=en+fk(761)*fmr(1,1)*fmr(4,1)*fmr(5,3)*fmr(6,1) 
      en=en+fk(762)*fmr(2,1)*fmr(3,1)*fmr(4,1)*fmr(5,3) 
      en=en+fk(763)*fmr(2,1)*fmr(3,1)*fmr(5,3)*fmr(6,1) 
      en=en+fk(764)*fmr(2,1)*fmr(4,1)*fmr(5,3)*fmr(6,1) 
      en=en+fk(765)*fmr(3,1)*fmr(4,1)*fmr(5,3)*fmr(6,1) 
      en=en+fk(766)*fmr(1,1)*fmr(2,1)*fmr(3,1)*fmr(6,3) 
      en=en+fk(767)*fmr(1,1)*fmr(2,1)*fmr(4,1)*fmr(6,3) 
      en=en+fk(768)*fmr(1,1)*fmr(2,1)*fmr(5,1)*fmr(6,3) 
      en=en+fk(769)*fmr(1,1)*fmr(3,1)*fmr(4,1)*fmr(6,3) 
      en=en+fk(770)*fmr(1,1)*fmr(3,1)*fmr(5,1)*fmr(6,3) 
      en=en+fk(771)*fmr(1,1)*fmr(4,1)*fmr(5,1)*fmr(6,3) 
      en=en+fk(772)*fmr(2,1)*fmr(3,1)*fmr(4,1)*fmr(6,3) 
      en=en+fk(773)*fmr(2,1)*fmr(3,1)*fmr(5,1)*fmr(6,3) 
      en=en+fk(774)*fmr(2,1)*fmr(4,1)*fmr(5,1)*fmr(6,3) 
      en=en+fk(775)*fmr(3,1)*fmr(4,1)*fmr(5,1)*fmr(6,3) 
      en=en+fk(776)*fmr(1,2)*fmr(2,2)*fmr(3,1)*fmr(4,1) 
      en=en+fk(777)*fmr(1,2)*fmr(2,2)*fmr(3,1)*fmr(5,1) 
      en=en+fk(778)*fmr(1,2)*fmr(2,2)*fmr(3,1)*fmr(6,1) 
      en=en+fk(779)*fmr(1,2)*fmr(2,2)*fmr(4,1)*fmr(5,1) 
      en=en+fk(780)*fmr(1,2)*fmr(2,2)*fmr(4,1)*fmr(6,1) 
      en=en+fk(781)*fmr(1,2)*fmr(2,2)*fmr(5,1)*fmr(6,1) 
      en=en+fk(782)*fmr(1,2)*fmr(2,1)*fmr(3,2)*fmr(4,1) 
      en=en+fk(783)*fmr(1,2)*fmr(2,1)*fmr(3,2)*fmr(5,1) 
      en=en+fk(784)*fmr(1,2)*fmr(2,1)*fmr(3,2)*fmr(6,1) 
      en=en+fk(785)*fmr(1,2)*fmr(3,2)*fmr(4,1)*fmr(5,1) 
      en=en+fk(786)*fmr(1,2)*fmr(3,2)*fmr(4,1)*fmr(6,1) 
      en=en+fk(787)*fmr(1,2)*fmr(3,2)*fmr(5,1)*fmr(6,1) 
      en=en+fk(788)*fmr(1,2)*fmr(2,1)*fmr(3,1)*fmr(4,2) 
      en=en+fk(789)*fmr(1,2)*fmr(2,1)*fmr(4,2)*fmr(5,1) 
      en=en+fk(790)*fmr(1,2)*fmr(2,1)*fmr(4,2)*fmr(6,1) 
      en=en+fk(791)*fmr(1,2)*fmr(3,1)*fmr(4,2)*fmr(5,1) 
      en=en+fk(792)*fmr(1,2)*fmr(3,1)*fmr(4,2)*fmr(6,1) 
      en=en+fk(793)*fmr(1,2)*fmr(4,2)*fmr(5,1)*fmr(6,1) 
      en=en+fk(794)*fmr(1,2)*fmr(2,1)*fmr(3,1)*fmr(5,2) 
      en=en+fk(795)*fmr(1,2)*fmr(2,1)*fmr(4,1)*fmr(5,2) 
      en=en+fk(796)*fmr(1,2)*fmr(2,1)*fmr(5,2)*fmr(6,1) 
      en=en+fk(797)*fmr(1,2)*fmr(3,1)*fmr(4,1)*fmr(5,2) 
      en=en+fk(798)*fmr(1,2)*fmr(3,1)*fmr(5,2)*fmr(6,1) 
      en=en+fk(799)*fmr(1,2)*fmr(4,1)*fmr(5,2)*fmr(6,1) 
      en=en+fk(800)*fmr(1,2)*fmr(2,1)*fmr(3,1)*fmr(6,2) 
      en=en+fk(801)*fmr(1,2)*fmr(2,1)*fmr(4,1)*fmr(6,2) 
      en=en+fk(802)*fmr(1,2)*fmr(2,1)*fmr(5,1)*fmr(6,2) 
      en=en+fk(803)*fmr(1,2)*fmr(3,1)*fmr(4,1)*fmr(6,2) 
      en=en+fk(804)*fmr(1,2)*fmr(3,1)*fmr(5,1)*fmr(6,2) 
      en=en+fk(805)*fmr(1,2)*fmr(4,1)*fmr(5,1)*fmr(6,2) 
      en=en+fk(806)*fmr(1,1)*fmr(2,2)*fmr(3,2)*fmr(4,1) 
      en=en+fk(807)*fmr(1,1)*fmr(2,2)*fmr(3,2)*fmr(5,1) 
      en=en+fk(808)*fmr(1,1)*fmr(2,2)*fmr(3,2)*fmr(6,1) 
      en=en+fk(809)*fmr(2,2)*fmr(3,2)*fmr(4,1)*fmr(5,1) 
      en=en+fk(810)*fmr(2,2)*fmr(3,2)*fmr(4,1)*fmr(6,1) 
      en=en+fk(811)*fmr(2,2)*fmr(3,2)*fmr(5,1)*fmr(6,1) 
      en=en+fk(812)*fmr(1,1)*fmr(2,2)*fmr(3,1)*fmr(4,2) 
      en=en+fk(813)*fmr(1,1)*fmr(2,2)*fmr(4,2)*fmr(5,1) 
      en=en+fk(814)*fmr(1,1)*fmr(2,2)*fmr(4,2)*fmr(6,1) 
      en=en+fk(815)*fmr(2,2)*fmr(3,1)*fmr(4,2)*fmr(5,1) 
      en=en+fk(816)*fmr(2,2)*fmr(3,1)*fmr(4,2)*fmr(6,1) 
      en=en+fk(817)*fmr(2,2)*fmr(4,2)*fmr(5,1)*fmr(6,1) 
      en=en+fk(818)*fmr(1,1)*fmr(2,2)*fmr(3,1)*fmr(5,2) 
      en=en+fk(819)*fmr(1,1)*fmr(2,2)*fmr(4,1)*fmr(5,2) 
      en=en+fk(820)*fmr(1,1)*fmr(2,2)*fmr(5,2)*fmr(6,1) 
      en=en+fk(821)*fmr(2,2)*fmr(3,1)*fmr(4,1)*fmr(5,2) 
      en=en+fk(822)*fmr(2,2)*fmr(3,1)*fmr(5,2)*fmr(6,1) 
      en=en+fk(823)*fmr(2,2)*fmr(4,1)*fmr(5,2)*fmr(6,1) 
      en=en+fk(824)*fmr(1,1)*fmr(2,2)*fmr(3,1)*fmr(6,2) 
      en=en+fk(825)*fmr(1,1)*fmr(2,2)*fmr(4,1)*fmr(6,2) 
      en=en+fk(826)*fmr(1,1)*fmr(2,2)*fmr(5,1)*fmr(6,2) 
      en=en+fk(827)*fmr(2,2)*fmr(3,1)*fmr(4,1)*fmr(6,2) 
      en=en+fk(828)*fmr(2,2)*fmr(3,1)*fmr(5,1)*fmr(6,2) 
      en=en+fk(829)*fmr(2,2)*fmr(4,1)*fmr(5,1)*fmr(6,2) 
      en=en+fk(830)*fmr(1,1)*fmr(2,1)*fmr(3,2)*fmr(4,2) 
      en=en+fk(831)*fmr(1,1)*fmr(3,2)*fmr(4,2)*fmr(5,1) 
      en=en+fk(832)*fmr(1,1)*fmr(3,2)*fmr(4,2)*fmr(6,1) 
      en=en+fk(833)*fmr(2,1)*fmr(3,2)*fmr(4,2)*fmr(5,1) 
      en=en+fk(834)*fmr(2,1)*fmr(3,2)*fmr(4,2)*fmr(6,1) 
      en=en+fk(835)*fmr(3,2)*fmr(4,2)*fmr(5,1)*fmr(6,1) 
      en=en+fk(836)*fmr(1,1)*fmr(2,1)*fmr(3,2)*fmr(5,2) 
      en=en+fk(837)*fmr(1,1)*fmr(3,2)*fmr(4,1)*fmr(5,2) 
      en=en+fk(838)*fmr(1,1)*fmr(3,2)*fmr(5,2)*fmr(6,1) 
      en=en+fk(839)*fmr(2,1)*fmr(3,2)*fmr(4,1)*fmr(5,2) 
      en=en+fk(840)*fmr(2,1)*fmr(3,2)*fmr(5,2)*fmr(6,1) 
      en=en+fk(841)*fmr(3,2)*fmr(4,1)*fmr(5,2)*fmr(6,1) 
      en=en+fk(842)*fmr(1,1)*fmr(2,1)*fmr(3,2)*fmr(6,2) 
      en=en+fk(843)*fmr(1,1)*fmr(3,2)*fmr(4,1)*fmr(6,2) 
      en=en+fk(844)*fmr(1,1)*fmr(3,2)*fmr(5,1)*fmr(6,2) 
      en=en+fk(845)*fmr(2,1)*fmr(3,2)*fmr(4,1)*fmr(6,2) 
      en=en+fk(846)*fmr(2,1)*fmr(3,2)*fmr(5,1)*fmr(6,2) 
      en=en+fk(847)*fmr(3,2)*fmr(4,1)*fmr(5,1)*fmr(6,2) 
      en=en+fk(848)*fmr(1,1)*fmr(2,1)*fmr(4,2)*fmr(5,2) 
      en=en+fk(849)*fmr(1,1)*fmr(3,1)*fmr(4,2)*fmr(5,2) 
      en=en+fk(850)*fmr(1,1)*fmr(4,2)*fmr(5,2)*fmr(6,1) 
      en=en+fk(851)*fmr(2,1)*fmr(3,1)*fmr(4,2)*fmr(5,2) 
      en=en+fk(852)*fmr(2,1)*fmr(4,2)*fmr(5,2)*fmr(6,1) 
      en=en+fk(853)*fmr(3,1)*fmr(4,2)*fmr(5,2)*fmr(6,1) 
      en=en+fk(854)*fmr(1,1)*fmr(2,1)*fmr(4,2)*fmr(6,2) 
      en=en+fk(855)*fmr(1,1)*fmr(3,1)*fmr(4,2)*fmr(6,2) 
      en=en+fk(856)*fmr(1,1)*fmr(4,2)*fmr(5,1)*fmr(6,2) 
      en=en+fk(857)*fmr(2,1)*fmr(3,1)*fmr(4,2)*fmr(6,2) 
      en=en+fk(858)*fmr(2,1)*fmr(4,2)*fmr(5,1)*fmr(6,2) 
      en=en+fk(859)*fmr(3,1)*fmr(4,2)*fmr(5,1)*fmr(6,2) 
      en=en+fk(860)*fmr(1,1)*fmr(2,1)*fmr(5,2)*fmr(6,2) 
      en=en+fk(861)*fmr(1,1)*fmr(3,1)*fmr(5,2)*fmr(6,2) 
      en=en+fk(862)*fmr(1,1)*fmr(4,1)*fmr(5,2)*fmr(6,2) 
      en=en+fk(863)*fmr(2,1)*fmr(3,1)*fmr(5,2)*fmr(6,2) 
      en=en+fk(864)*fmr(2,1)*fmr(4,1)*fmr(5,2)*fmr(6,2) 
      en=en+fk(865)*fmr(3,1)*fmr(4,1)*fmr(5,2)*fmr(6,2) 
      en=en+fk(866)*fmr(1,2)*fmr(2,1)*fmr(3,1)*fmr(4,1)*fmr(5,1) 
      en=en+fk(867)*fmr(1,2)*fmr(2,1)*fmr(3,1)*fmr(4,1)*fmr(6,1) 
      en=en+fk(868)*fmr(1,2)*fmr(2,1)*fmr(3,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(869)*fmr(1,2)*fmr(2,1)*fmr(4,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(870)*fmr(1,2)*fmr(3,1)*fmr(4,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(871)*fmr(1,1)*fmr(2,2)*fmr(3,1)*fmr(4,1)*fmr(5,1) 
      en=en+fk(872)*fmr(1,1)*fmr(2,2)*fmr(3,1)*fmr(4,1)*fmr(6,1) 
      en=en+fk(873)*fmr(1,1)*fmr(2,2)*fmr(3,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(874)*fmr(1,1)*fmr(2,2)*fmr(4,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(875)*fmr(2,2)*fmr(3,1)*fmr(4,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(876)*fmr(1,1)*fmr(2,1)*fmr(3,2)*fmr(4,1)*fmr(5,1) 
      en=en+fk(877)*fmr(1,1)*fmr(2,1)*fmr(3,2)*fmr(4,1)*fmr(6,1) 
      en=en+fk(878)*fmr(1,1)*fmr(2,1)*fmr(3,2)*fmr(5,1)*fmr(6,1) 
      en=en+fk(879)*fmr(1,1)*fmr(3,2)*fmr(4,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(880)*fmr(2,1)*fmr(3,2)*fmr(4,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(881)*fmr(1,1)*fmr(2,1)*fmr(3,1)*fmr(4,2)*fmr(5,1) 
      en=en+fk(882)*fmr(1,1)*fmr(2,1)*fmr(3,1)*fmr(4,2)*fmr(6,1) 
      en=en+fk(883)*fmr(1,1)*fmr(2,1)*fmr(4,2)*fmr(5,1)*fmr(6,1) 
      en=en+fk(884)*fmr(1,1)*fmr(3,1)*fmr(4,2)*fmr(5,1)*fmr(6,1) 
      en=en+fk(885)*fmr(2,1)*fmr(3,1)*fmr(4,2)*fmr(5,1)*fmr(6,1) 
      en=en+fk(886)*fmr(1,1)*fmr(2,1)*fmr(3,1)*fmr(4,1)*fmr(5,2) 
      en=en+fk(887)*fmr(1,1)*fmr(2,1)*fmr(3,1)*fmr(5,2)*fmr(6,1) 
      en=en+fk(888)*fmr(1,1)*fmr(2,1)*fmr(4,1)*fmr(5,2)*fmr(6,1) 
      en=en+fk(889)*fmr(1,1)*fmr(3,1)*fmr(4,1)*fmr(5,2)*fmr(6,1) 
      en=en+fk(890)*fmr(2,1)*fmr(3,1)*fmr(4,1)*fmr(5,2)*fmr(6,1) 
      en=en+fk(891)*fmr(1,1)*fmr(2,1)*fmr(3,1)*fmr(4,1)*fmr(6,2) 
      en=en+fk(892)*fmr(1,1)*fmr(2,1)*fmr(3,1)*fmr(5,1)*fmr(6,2) 
      en=en+fk(893)*fmr(1,1)*fmr(2,1)*fmr(4,1)*fmr(5,1)*fmr(6,2) 
      en=en+fk(894)*fmr(1,1)*fmr(3,1)*fmr(4,1)*fmr(5,1)*fmr(6,2) 
      en=en+fk(895)*fmr(2,1)*fmr(3,1)*fmr(4,1)*fmr(5,1)*fmr(6,2) 
      en=en+fk(896)*fmr(1,1)*fmr(2,1)*fmr(3,1)*fmr(4,1)*fmr(5,1)
     #  *fmr(6,1) 

      return
      end subroutine

      subroutine f33757(rij,en)
      implicit real*8 (a-h,o-z)
      dimension fk(505),fmr(6,6), shift(6),rij(6),alpham(6)

      data shift  / 2.25d0, 2.12d0, 2.12d0, 3.85d0, 3.85d0, 3.57d0 /
      data alpham / 6*0.5d0 /
      data nmax,nvar /6,6/

      data fk(   1), fk(   2) / -9.9050798129D+01 ,  2.8636697745D+00 /
      data fk(   3), fk(   4) / -1.6930829775D-01 , -1.4099814672D+02 /
      data fk(   5), fk(   6) /  4.0731840211D-01 ,  2.8630717003D+01 /
      data fk(   7), fk(   8) /  6.3411270597D+00 ,  7.7931144075D-01 /
      data fk(   9), fk(  10) / -1.2641369442D+00 ,  5.4970412350D+02 /
      data fk(  11), fk(  12) / -6.2986360492D-01 ,  1.5182797924D+01 /
      data fk(  13), fk(  14) / -1.5968934068D+00 , -1.1427828228D+01 /
      data fk(  15), fk(  16) / -1.1714745200D+01 ,  5.3775638021D+00 /
      data fk(  17), fk(  18) /  2.9098642934D+00 ,  5.7484208544D-01 /
      data fk(  19), fk(  20) / -3.3580728555D-01 ,  2.9523327150D+00 /
      data fk(  21), fk(  22) /  5.6435866676D+00 ,  1.2275270350D+00 /
      data fk(  23), fk(  24) / -4.6387429512D+00 , -2.2573658371D+02 /
      data fk(  25), fk(  26) / -5.7124875749D+01 , -2.1073840521D+00 /
      data fk(  27), fk(  28) /  1.8141449809D+00 ,  2.3726670224D+01 /
      data fk(  29), fk(  30) /  2.9659616085D+00 , -6.2990636947D-01 /
      data fk(  31), fk(  32) / -1.1367432970D+03 ,  5.5791790089D-01 /
      data fk(  33), fk(  34) /  6.1254734579D+00 , -1.5505338363D+00 /
      data fk(  35), fk(  36) /  1.1036235347D+01 , -5.3347984711D+00 /
      data fk(  37), fk(  38) / -3.6997766827D+00 ,  3.2006925175D+00 /
      data fk(  39), fk(  40) /  8.5930156393D-01 ,  8.6574703220D+00 /
      data fk(  41), fk(  42) /  7.7491575162D+00 , -2.6027552670D+00 /
      data fk(  43), fk(  44) / -4.1755625368D+00 ,  9.5790433698D-01 /
      data fk(  45), fk(  46) /  1.2699083944D+01 ,  2.5345495624D+00 /
      data fk(  47), fk(  48) /  1.5371698416D+01 ,  6.9194748688D+02 /
      data fk(  49), fk(  50) /  1.9216712104D+02 , -1.3151764312D+00 /
      data fk(  51), fk(  52) /  1.2109263545D+00 ,  1.4477639132D+00 /
      data fk(  53), fk(  54) /  4.1138567969D-01 , -6.0494718075D-01 /
      data fk(  55), fk(  56) / -9.7004211858D-02 ,  6.7373412460D-02 /
      data fk(  57), fk(  58) / -9.2818192408D+01 , -3.7240802517D-02 /
      data fk(  59), fk(  60) /  6.2746030564D-01 , -9.9630892716D-02 /
      data fk(  61), fk(  62) /  3.5448448326D-01 ,  2.0924922144D+00 /
      data fk(  63), fk(  64) /  4.5244805576D-01 ,  8.9488770170D+00 /
      data fk(  65), fk(  66) /  4.3655236161D+01 , -6.4172711601D+00 /
      data fk(  67), fk(  68) / -9.1257771356D+00 , -3.9121673334D+00 /
      data fk(  69), fk(  70) / -1.4065222106D+01 , -4.7147228727D+00 /
      data fk(  71), fk(  72) /  5.0653766652D-01 , -1.2127644024D+00 /
      data fk(  73), fk(  74) /  9.5502535505D-01 ,  5.8805725571D-01 /
      data fk(  75), fk(  76) / -1.3782316346D+01 , -1.8692884609D+01 /
      data fk(  77), fk(  78) / -7.6732557729D+00 ,  5.7475133656D+00 /
      data fk(  79), fk(  80) /  1.9616211539D+00 ,  4.9913521914D+00 /
      data fk(  81), fk(  82) /  5.0731969536D+00 , -7.3446227072D+00 /
      data fk(  83), fk(  84) / -1.3206043416D+02 , -1.7640656305D+00 /
      data fk(  85), fk(  86) / -1.4646585752D-01 ,  1.3060603370D+03 /
      data fk(  87), fk(  88) /  9.8266305172D-01 ,  4.4702490075D-01 /
      data fk(  89), fk(  90) / -3.7964803762D-01 , -3.1483393131D+00 /
      data fk(  91), fk(  92) / -1.4743509991D+01 ,  3.2002839179D-01 /
      data fk(  93), fk(  94) /  2.2781409900D+00 ,  4.9241822219D-01 /
      data fk(  95), fk(  96) /  1.1062870916D+00 , -6.1349203336D-01 /
      data fk(  97), fk(  98) / -1.5112933748D+00 , -2.7415747385D-02 /
      data fk(  99), fk( 100) /  9.2340830049D-02 ,  5.3170860163D-01 /
      data fk( 101), fk( 102) / -5.4479426765D+00 , -1.7800181812D+01 /
      data fk( 103), fk( 104) / -1.0291385154D+03 , -3.0896234717D+02 /
      data fk( 105), fk( 106) / -2.4294171204D+00 , -2.9586858362D+00 /
      data fk( 107), fk( 108) /  6.4892877619D-01 ,  4.7203259310D-02 /
      data fk( 109), fk( 110) /  4.1326283022D-02 , -5.9156773796D-01 /
      data fk( 111), fk( 112) / -6.0016820305D-01 , -2.2563905623D+01 /
      data fk( 113), fk( 114) /  6.6619957724D-01 , -4.6621705197D+00 /
      data fk( 115), fk( 116) / -2.2849282572D-01 , -9.0933520033D-03 /
      data fk( 117), fk( 118) /  7.0772514557D+00 ,  1.3318629569D-01 /
      data fk( 119), fk( 120) / -5.9596697266D-01 , -8.4963186799D-01 /
      data fk( 121), fk( 122) /  1.8507399163D+01 ,  4.8676005642D-01 /
      data fk( 123), fk( 124) / -1.0622331519D-01 , -1.2895323202D-01 /
      data fk( 125), fk( 126) / -9.1724895821D+00 ,  2.5941469749D+00 /
      data fk( 127), fk( 128) / -3.6165287326D-01 , -1.8097357977D-01 /
      data fk( 129), fk( 130) /  6.4443611676D-01 ,  1.9988165975D+02 /
      data fk( 131), fk( 132) /  1.0887421519D+01 , -5.4990862124D-01 /
      data fk( 133), fk( 134) / -9.8417525364D-02 ,  8.5131538277D+00 /
      data fk( 135), fk( 136) / -2.9175819596D+01 , -3.7106344570D+00 /
      data fk( 137), fk( 138) /  2.1542464048D+00 ,  1.8916062120D+00 /
      data fk( 139), fk( 140) /  1.1314177271D+01 , -8.3116638401D+00 /
      data fk( 141), fk( 142) / -2.1675066933D+00 , -1.4673877260D+00 /
      data fk( 143), fk( 144) /  7.3713402523D-01 , -1.4030140587D+00 /
      data fk( 145), fk( 146) / -2.3145836226D+01 , -4.3862678371D+00 /
      data fk( 147), fk( 148) /  2.4081853791D+00 ,  1.9613678340D+00 /
      data fk( 149), fk( 150) /  8.7113908912D+00 ,  8.6443328303D+00 /
      data fk( 151), fk( 152) / -3.0217089234D+00 , -7.4026603825D-01 /
      data fk( 153), fk( 154) / -6.2931854909D-01 ,  1.0765997740D+00 /
      data fk( 155), fk( 156) / -5.2460201836D+01 ,  8.7371526980D+00 /
      data fk( 157), fk( 158) / -1.8814701056D+00 , -4.9922815680D+00 /
      data fk( 159), fk( 160) /  1.6998220431D+01 ,  2.3060904858D+01 /
      data fk( 161), fk( 162) /  1.5433495787D+01 , -5.8935599911D+00 /
      data fk( 163), fk( 164) /  8.4826529202D+00 ,  2.7340342850D+02 /
      data fk( 165), fk( 166) /  6.7736811334D+00 , -6.8645621346D-01 /
      data fk( 167), fk( 168) /  1.4877453460D-01 , -5.5873354731D-01 /
      data fk( 169), fk( 170) / -6.4575974986D+00 , -1.7131391240D-01 /
      data fk( 171), fk( 172) /  3.1341758329D-01 , -5.9582321832D-01 /
      data fk( 173), fk( 174) /  8.0880767384D-01 ,  5.2937921345D-01 /
      data fk( 175), fk( 176) / -9.2000562197D-01 ,  1.1169948853D+00 /
      data fk( 177), fk( 178) /  1.1756021633D-01 ,  2.3331965829D+00 /
      data fk( 179), fk( 180) / -1.7705108046D-01 ,  5.2492830706D-01 /
      data fk( 181), fk( 182) /  2.7833269806D+00 ,  3.1006165109D-01 /
      data fk( 183), fk( 184) / -1.7839908086D-01 , -2.1645411688D+00 /
      data fk( 185), fk( 186) / -1.0631879100D+00 ,  2.0183459219D+00 /
      data fk( 187), fk( 188) /  6.7766141108D-01 , -8.9005793401D-01 /
      data fk( 189), fk( 190) / -9.8329175976D-01 ,  3.7620353732D-01 /
      data fk( 191), fk( 192) /  4.3564380197D-01 , -1.6040998844D+00 /
      data fk( 193), fk( 194) / -4.7758071664D+01 , -3.8618922313D-02 /
      data fk( 195), fk( 196) /  7.6032036128D+00 , -5.2786558685D-01 /
      data fk( 197), fk( 198) / -1.6571973244D+01 , -3.9271197817D+00 /
      data fk( 199), fk( 200) / -1.1981263561D+01 ,  6.2351798199D-02 /
      data fk( 201), fk( 202) / -1.8936412178D+00 , -2.5322734106D+00 /
      data fk( 203), fk( 204) /  3.5285263653D+00 , -2.3834171093D+00 /
      data fk( 205), fk( 206) / -5.7981254301D-01 , -1.6710582535D+00 /
      data fk( 207), fk( 208) /  1.0231644892D+01 ,  2.1400077504D+01 /
      data fk( 209), fk( 210) /  2.1573576713D+01 , -3.2067005375D-01 /
      data fk( 211), fk( 212) / -7.8563645945D+02 ,  3.2856581345D-01 /
      data fk( 213), fk( 214) /  3.5719348601D-01 , -1.9254788059D-02 /
      data fk( 215), fk( 216) / -3.4250534227D+00 ,  5.3067714449D-01 /
      data fk( 217), fk( 218) /  4.2948969962D+00 , -7.8047638474D-02 /
      data fk( 219), fk( 220) / -1.8481097724D-01 , -4.1887481563D+00 /
      data fk( 221), fk( 222) /  4.2637974590D+00 ,  5.7863142970D+00 /
      data fk( 223), fk( 224) /  7.3742142534D+02 ,  2.3992281392D+02 /
      data fk( 225), fk( 226) / -5.8103645977D-01 , -2.4939458614D+00 /
      data fk( 227), fk( 228) /  1.6821749138D-02 ,  7.6055364566D-03 /
      data fk( 229), fk( 230) /  2.9510156237D-02 , -8.8775440203D-02 /
      data fk( 231), fk( 232) / -8.6181002234D-02 , -9.1899930537D-01 /
      data fk( 233), fk( 234) /  3.2109688509D-02 , -2.2089232722D+00 /
      data fk( 235), fk( 236) / -6.3402451660D-02 ,  1.4653706615D-02 /
      data fk( 237), fk( 238) /  1.2502421189D+00 ,  3.0171717077D-02 /
      data fk( 239), fk( 240) / -2.1980124509D-01 , -4.4645336779D+00 /
      data fk( 241), fk( 242) /  8.4834324867D+00 ,  3.1453081996D+00 /
      data fk( 243), fk( 244) / -7.3734409662D-01 , -1.2419272919D-01 /
      data fk( 245), fk( 246) / -1.4609811474D+00 ,  3.5860172705D-02 /
      data fk( 247), fk( 248) / -2.4836322918D+00 , -9.5693792831D-02 /
      data fk( 249), fk( 250) /  1.3241739282D-01 , -1.2969057755D+01 /
      data fk( 251), fk( 252) /  2.8635407608D+00 , -1.7559795856D+00 /
      data fk( 253), fk( 254) / -1.7545082553D+02 , -2.6453673366D+01 /
      data fk( 255), fk( 256) / -1.5735685957D+00 ,  3.2328214122D+00 /
      data fk( 257), fk( 258) / -4.6311995932D-01 , -3.9565288801D-02 /
      data fk( 259), fk( 260) / -2.0534305722D-02 ,  2.1038451288D-01 /
      data fk( 261), fk( 262) /  9.7389594454D-03 ,  2.3885308727D+01 /
      data fk( 263), fk( 264) /  1.2089494257D-01 ,  3.6264366231D+00 /
      data fk( 265), fk( 266) / -1.1155907747D-01 , -6.8633194801D-02 /
      data fk( 267), fk( 268) / -9.6541380077D+00 ,  4.8660096435D-03 /
      data fk( 269), fk( 270) /  1.1685704720D+00 ,  6.7834995753D+00 /
      data fk( 271), fk( 272) / -1.9901310703D+00 , -1.1018069261D+00 /
      data fk( 273), fk( 274) /  2.1743181971D-01 ,  1.9868222963D-01 /
      data fk( 275), fk( 276) / -1.4792667034D+00 , -2.0467321802D-01 /
      data fk( 277), fk( 278) /  6.0444110729D-01 , -4.8741624818D-02 /
      data fk( 279), fk( 280) / -1.0616277046D-01 ,  2.4312566872D+00 /
      data fk( 281), fk( 282) /  4.6281061508D+00 , -3.1881005874D-01 /
      data fk( 283), fk( 284) / -1.9695786980D-01 , -1.8706272509D+00 /
      data fk( 285), fk( 286) /  2.3614972007D-01 , -2.1551979985D-01 /
      data fk( 287), fk( 288) /  5.9770806421D-02 ,  2.0546837560D-01 /
      data fk( 289), fk( 290) /  4.6970572573D-02 ,  2.0097555321D+01 /
      data fk( 291), fk( 292) /  3.0600968940D-01 ,  3.3372916374D+00 /
      data fk( 293), fk( 294) /  4.0162910863D+00 , -5.5107639674D+00 /
      data fk( 295), fk( 296) / -1.1767911639D+01 , -9.9686564994D+00 /
      data fk( 297), fk( 298) /  4.9451692991D+00 , -2.0606297648D+00 /
      data fk( 299), fk( 300) / -2.4730424552D+02 ,  6.6331963085D+00 /
      data fk( 301), fk( 302) / -3.3795277579D-02 , -4.2982775634D-02 /
      data fk( 303), fk( 304) / -5.3100359527D-02 , -2.4542068571D-01 /
      data fk( 305), fk( 306) /  7.5143268723D-02 , -2.1600286020D-01 /
      data fk( 307), fk( 308) /  5.4621105770D-02 ,  2.2578201596D-01 /
      data fk( 309), fk( 310) / -7.7048291960D-02 ,  2.6150215411D-01 /
      data fk( 311), fk( 312) /  8.0884147836D-02 , -5.7850840463D-01 /
      data fk( 313), fk( 314) /  1.0041340734D+00 ,  1.4388215395D-01 /
      data fk( 315), fk( 316) / -2.2856619312D-01 ,  8.7151399367D-01 /
      data fk( 317), fk( 318) /  4.5402403811D-02 ,  1.4654897416D+01 /
      data fk( 319), fk( 320) / -8.4367366296D-01 ,  5.7017559346D-03 /
      data fk( 321), fk( 322) /  4.7295247611D-01 ,  9.6335643485D-03 /
      data fk( 323), fk( 324) / -1.0266591902D-01 , -2.8469302836D-02 /
      data fk( 325), fk( 326) /  8.1581904145D-03 ,  8.6203284795D-02 /
      data fk( 327), fk( 328) / -2.1032673430D-01 , -6.0613164279D-01 /
      data fk( 329), fk( 330) / -1.9207330853D-02 ,  1.3846184563D+01 /
      data fk( 331), fk( 332) / -9.3161391753D+00 ,  4.0565098381D-02 /
      data fk( 333), fk( 334) /  1.2716514406D-01 ,  7.7914504580D-02 /
      data fk( 335), fk( 336) / -1.0161331124D-01 , -2.5341951756D+00 /
      data fk( 337), fk( 338) /  2.3809472714D-01 ,  1.3384586182D-02 /
      data fk( 339), fk( 340) / -6.5195120995D-01 ,  1.6709584328D+01 /
      data fk( 341), fk( 342) / -1.9118828037D+00 , -4.5418378096D-01 /
      data fk( 343), fk( 344) /  3.0687842046D-01 ,  3.7046467781D+00 /
      data fk( 345), fk( 346) /  1.5794576172D+00 ,  6.6116131969D-01 /
      data fk( 347), fk( 348) /  1.9683475784D-03 ,  1.8407883297D-01 /
      data fk( 349), fk( 350) / -3.4388288744D-01 ,  4.9732148069D+00 /
      data fk( 351), fk( 352) /  1.1067501042D+00 , -6.3963496313D-02 /
      data fk( 353), fk( 354) /  1.9168224765D-01 ,  1.2042743738D+00 /
      data fk( 355), fk( 356) /  4.6319064929D-01 , -5.1826354620D-02 /
      data fk( 357), fk( 358) /  6.7717803483D-02 ,  1.2132464289D-02 /
      data fk( 359), fk( 360) / -2.9809796264D+01 ,  8.9506893424D+00 /
      data fk( 361), fk( 362) / -5.6606957658D+00 ,  6.1067252756D-01 /
      data fk( 363), fk( 364) /  8.6079008272D-02 , -4.8013648232D+00 /
      data fk( 365), fk( 366) /  1.1845010824D-01 , -1.8517591347D-01 /
      data fk( 367), fk( 368) / -2.1693264649D+00 ,  1.1933789047D+00 /
      data fk( 369), fk( 370) /  3.8355962601D-01 , -5.5749269675D-01 /
      data fk( 371), fk( 372) /  5.6130217062D+00 , -4.4231430743D-01 /
      data fk( 373), fk( 374) / -1.7325854885D-01 , -4.1729261541D+00 /
      data fk( 375), fk( 376) / -1.2261434712D-01 ,  1.8949339610D-01 /
      data fk( 377), fk( 378) /  9.6991542457D-01 ,  8.3583631262D+01 /
      data fk( 379), fk( 380) / -9.4899191736D-02 ,  3.4738458093D-01 /
      data fk( 381), fk( 382) /  1.9961140876D+00 ,  7.4120524531D-02 /
      data fk( 383), fk( 384) / -3.4032717913D-01 ,  2.9568967425D+00 /
      data fk( 385), fk( 386) /  1.8949675202D-01 , -1.0406934810D-01 /
      data fk( 387), fk( 388) /  5.3960473861D-01 , -5.1898220955D+00 /
      data fk( 389), fk( 390) /  2.6843992381D-01 , -2.8448812089D+00 /
      data fk( 391), fk( 392) / -1.7732894764D+00 , -3.5196769497D+00 /
      data fk( 393), fk( 394) /  7.6208144258D-01 , -2.0749559718D-01 /
      data fk( 395), fk( 396) /  5.8414940474D-01 ,  5.0291059133D-01 /
      data fk( 397), fk( 398) / -8.6599970996D-01 ,  1.6933473194D+00 /
      data fk( 399), fk( 400) /  1.3855857134D-01 , -3.8867972881D+00 /
      data fk( 401), fk( 402) / -2.8615194933D+00 , -2.7907772951D+00 /
      data fk( 403), fk( 404) /  3.1389427715D-01 , -6.2173481743D-01 /
      data fk( 405), fk( 406) /  5.7693982644D-01 ,  9.3212394495D-01 /
      data fk( 407), fk( 408) /  1.3055602462D+00 , -1.6446057904D+00 /
      data fk( 409), fk( 410) / -3.6315205433D-01 , -1.1861042146D+01 /
      data fk( 411), fk( 412) / -1.2436186728D+01 , -5.7878217405D+00 /
      data fk( 413), fk( 414) / -3.5450887350D+00 ,  1.1376328296D+00 /
      data fk( 415), fk( 416) /  1.1315443727D+00 ,  5.0163448498D+00 /
      data fk( 417), fk( 418) /  4.8180055686D-01 ,  1.3553129274D+01 /
      data fk( 419), fk( 420) / -7.6113643376D+00 ,  1.3101552182D+00 /
      data fk( 421), fk( 422) / -2.0791926974D-01 ,  5.5981693561D-01 /
      data fk( 423), fk( 424) / -4.2651541192D-01 ,  1.0289772984D-01 /
      data fk( 425), fk( 426) /  6.7640713951D-03 , -2.2446916900D-02 /
      data fk( 427), fk( 428) / -1.2639857848D+00 ,  2.8734410460D-01 /
      data fk( 429), fk( 430) / -1.1364453999D+00 ,  1.1108316989D+00 /
      data fk( 431), fk( 432) /  1.3264829149D-01 , -9.3141480312D-01 /
      data fk( 433), fk( 434) /  1.1741129023D+00 , -3.5064936964D+00 /
      data fk( 435), fk( 436) / -1.4545315313D-01 , -8.8887809542D-03 /
      data fk( 437), fk( 438) / -4.6045774046D+00 , -5.2637644043D-02 /
      data fk( 439), fk( 440) /  3.8727010011D+00 ,  9.5450568813D-01 /
      data fk( 441), fk( 442) / -1.7595663328D-01 , -1.7141345351D-01 /
      data fk( 443), fk( 444) / -9.0224358066D-01 ,  2.1855974196D+00 /
      data fk( 445), fk( 446) /  3.2831595809D-01 ,  1.3256102606D-02 /
      data fk( 447), fk( 448) / -2.3896499122D-01 , -1.5112066279D-01 /
      data fk( 449), fk( 450) / -2.9935186459D-01 ,  1.5352455820D+00 /
      data fk( 451), fk( 452) /  1.9229844349D+00 ,  3.3890745139D+00 /
      data fk( 453), fk( 454) / -1.0475788246D-01 ,  6.3945964507D-02 /
      data fk( 455), fk( 456) /  1.5711286052D+00 ,  6.6974992284D-01 /
      data fk( 457), fk( 458) /  1.9188640567D+02 ,  3.0978160972D-02 /
      data fk( 459), fk( 460) /  2.2118692376D-02 , -1.2272615446D-03 /
      data fk( 461), fk( 462) /  1.1789417081D+00 , -2.7015216048D+00 /
      data fk( 463), fk( 464) / -2.0064567094D+02 , -7.2233752435D+01 /
      data fk( 465), fk( 466) / -3.9197272607D-02 , -4.1543354678D-01 /
      data fk( 467), fk( 468) / -2.4291876472D-01 , -3.1380305749D-01 /
      data fk( 469), fk( 470) / -9.1771623429D-02 ,  1.2429487334D-01 /
      data fk( 471), fk( 472) / -2.5688658196D-01 ,  6.5103729320D-01 /
      data fk( 473), fk( 474) /  4.9203607399D+01 ,  1.5829112266D+01 /
      data fk( 475), fk( 476) / -3.8033571020D-01 ,  1.4077594948D+00 /
      data fk( 477), fk( 478) / -5.0761105322D-01 ,  3.3970014021D-01 /
      data fk( 479), fk( 480) / -9.5462115251D-01 , -4.9632922101D-01 /
      data fk( 481), fk( 482) /  2.3672237551D+00 , -2.0102438622D-01 /
      data fk( 483), fk( 484) / -7.3033105829D-01 , -4.5238926981D+00 /
      data fk( 485), fk( 486) /  4.1986887507D+00 ,  8.9618477831D-01 /
      data fk( 487), fk( 488) /  2.6352528676D+01 ,  8.1962049403D-01 /
      data fk( 489), fk( 490) /  4.5671030587D-01 ,  8.1604317739D+01 /
      data fk( 491), fk( 492) /  7.3836312267D-01 ,  3.5152468277D+00 /
      data fk( 493), fk( 494) /  7.5484990135D-01 ,  1.5841534590D+00 /
      data fk( 495), fk( 496) / -4.1776594451D+00 , -5.1479157339D+00 /
      data fk( 497), fk( 498) /  2.9906071568D+00 ,  6.6522590151D+00 /
      data fk( 499), fk( 500) / -4.8313454164D+01 ,  2.1106069946D+00 /
      data fk( 501), fk( 502) / -4.1728281820D+00 , -1.2290306322D+01 /
      data fk( 503), fk( 504) / -6.3444876210D+00 ,  1.5498205312D+00 /
      data fk( 505) / -2.4034530950D+00 /
   
c
      en=0.0d0
      do i=1,nvar
       fmr(i,1) = 1.0d0- exp(-alpham(i)*(rij(i)-shift(i)))
       do j=2,nmax
       fmr(i,j) = fmr(i,1)**j
       enddo
      enddo
c
      en=en+fk(1) 
      en=en+fk(2)*fmr(1,1) 
      en=en+fk(3)*fmr(2,1) 
      en=en+fk(4)*fmr(3,1) 
      en=en+fk(5)*fmr(4,1) 
      en=en+fk(6)*fmr(5,1) 
      en=en+fk(7)*fmr(6,1) 
      en=en+fk(8)*fmr(1,2) 
      en=en+fk(9)*fmr(2,2) 
      en=en+fk(10)*fmr(3,2) 
      en=en+fk(11)*fmr(4,2) 
      en=en+fk(12)*fmr(5,2) 
      en=en+fk(13)*fmr(6,2) 
      en=en+fk(14)*fmr(1,1)*fmr(2,1) 
      en=en+fk(15)*fmr(1,1)*fmr(3,1) 
      en=en+fk(16)*fmr(1,1)*fmr(4,1) 
      en=en+fk(17)*fmr(1,1)*fmr(5,1) 
      en=en+fk(18)*fmr(1,1)*fmr(6,1) 
      en=en+fk(19)*fmr(2,1)*fmr(3,1) 
      en=en+fk(20)*fmr(2,1)*fmr(4,1) 
      en=en+fk(21)*fmr(2,1)*fmr(5,1) 
      en=en+fk(22)*fmr(2,1)*fmr(6,1) 
      en=en+fk(23)*fmr(3,1)*fmr(4,1) 
      en=en+fk(24)*fmr(3,1)*fmr(5,1) 
      en=en+fk(25)*fmr(3,1)*fmr(6,1) 
      en=en+fk(26)*fmr(4,1)*fmr(5,1) 
      en=en+fk(27)*fmr(4,1)*fmr(6,1) 
      en=en+fk(28)*fmr(5,1)*fmr(6,1) 
      en=en+fk(29)*fmr(1,3) 
      en=en+fk(30)*fmr(2,3) 
      en=en+fk(31)*fmr(3,3) 
      en=en+fk(32)*fmr(4,3) 
      en=en+fk(33)*fmr(5,3) 
      en=en+fk(34)*fmr(6,3) 
      en=en+fk(35)*fmr(1,2)*fmr(2,1) 
      en=en+fk(36)*fmr(1,2)*fmr(3,1) 
      en=en+fk(37)*fmr(1,2)*fmr(4,1) 
      en=en+fk(38)*fmr(1,2)*fmr(5,1) 
      en=en+fk(39)*fmr(1,2)*fmr(6,1) 
      en=en+fk(40)*fmr(1,1)*fmr(2,2) 
      en=en+fk(41)*fmr(2,2)*fmr(3,1) 
      en=en+fk(42)*fmr(2,2)*fmr(4,1) 
      en=en+fk(43)*fmr(2,2)*fmr(5,1) 
      en=en+fk(44)*fmr(2,2)*fmr(6,1) 
      en=en+fk(45)*fmr(1,1)*fmr(3,2) 
      en=en+fk(46)*fmr(2,1)*fmr(3,2) 
      en=en+fk(47)*fmr(3,2)*fmr(4,1) 
      en=en+fk(48)*fmr(3,2)*fmr(5,1) 
      en=en+fk(49)*fmr(3,2)*fmr(6,1) 
      en=en+fk(50)*fmr(1,1)*fmr(4,2) 
      en=en+fk(51)*fmr(2,1)*fmr(4,2) 
      en=en+fk(52)*fmr(3,1)*fmr(4,2) 
      en=en+fk(53)*fmr(4,2)*fmr(5,1) 
      en=en+fk(54)*fmr(4,2)*fmr(6,1) 
      en=en+fk(55)*fmr(1,1)*fmr(5,2) 
      en=en+fk(56)*fmr(2,1)*fmr(5,2) 
      en=en+fk(57)*fmr(3,1)*fmr(5,2) 
      en=en+fk(58)*fmr(4,1)*fmr(5,2) 
      en=en+fk(59)*fmr(5,2)*fmr(6,1) 
      en=en+fk(60)*fmr(1,1)*fmr(6,2) 
      en=en+fk(61)*fmr(2,1)*fmr(6,2) 
      en=en+fk(62)*fmr(3,1)*fmr(6,2) 
      en=en+fk(63)*fmr(4,1)*fmr(6,2) 
      en=en+fk(64)*fmr(5,1)*fmr(6,2) 
      en=en+fk(65)*fmr(1,1)*fmr(2,1)*fmr(3,1) 
      en=en+fk(66)*fmr(1,1)*fmr(2,1)*fmr(4,1) 
      en=en+fk(67)*fmr(1,1)*fmr(2,1)*fmr(5,1) 
      en=en+fk(68)*fmr(1,1)*fmr(2,1)*fmr(6,1) 
      en=en+fk(69)*fmr(1,1)*fmr(3,1)*fmr(4,1) 
      en=en+fk(70)*fmr(1,1)*fmr(3,1)*fmr(5,1) 
      en=en+fk(71)*fmr(1,1)*fmr(3,1)*fmr(6,1) 
      en=en+fk(72)*fmr(1,1)*fmr(4,1)*fmr(5,1) 
      en=en+fk(73)*fmr(1,1)*fmr(4,1)*fmr(6,1) 
      en=en+fk(74)*fmr(1,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(75)*fmr(2,1)*fmr(3,1)*fmr(4,1) 
      en=en+fk(76)*fmr(2,1)*fmr(3,1)*fmr(5,1) 
      en=en+fk(77)*fmr(2,1)*fmr(3,1)*fmr(6,1) 
      en=en+fk(78)*fmr(2,1)*fmr(4,1)*fmr(5,1) 
      en=en+fk(79)*fmr(2,1)*fmr(4,1)*fmr(6,1) 
      en=en+fk(80)*fmr(2,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(81)*fmr(3,1)*fmr(4,1)*fmr(5,1) 
      en=en+fk(82)*fmr(3,1)*fmr(4,1)*fmr(6,1) 
      en=en+fk(83)*fmr(3,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(84)*fmr(4,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(85)*fmr(2,4) 
      en=en+fk(86)*fmr(3,4) 
      en=en+fk(87)*fmr(4,4) 
      en=en+fk(88)*fmr(5,4) 
      en=en+fk(89)*fmr(6,4) 
      en=en+fk(90)*fmr(1,3)*fmr(2,1) 
      en=en+fk(91)*fmr(1,3)*fmr(3,1) 
      en=en+fk(92)*fmr(1,3)*fmr(4,1) 
      en=en+fk(93)*fmr(1,3)*fmr(5,1) 
      en=en+fk(94)*fmr(1,3)*fmr(6,1) 
      en=en+fk(95)*fmr(1,1)*fmr(2,3) 
      en=en+fk(96)*fmr(2,3)*fmr(3,1) 
      en=en+fk(97)*fmr(2,3)*fmr(4,1) 
      en=en+fk(98)*fmr(2,3)*fmr(5,1) 
      en=en+fk(99)*fmr(2,3)*fmr(6,1) 
      en=en+fk(100)*fmr(1,1)*fmr(3,3) 
      en=en+fk(101)*fmr(2,1)*fmr(3,3) 
      en=en+fk(102)*fmr(3,3)*fmr(4,1) 
      en=en+fk(103)*fmr(3,3)*fmr(5,1) 
      en=en+fk(104)*fmr(3,3)*fmr(6,1) 
      en=en+fk(105)*fmr(1,1)*fmr(4,3) 
      en=en+fk(106)*fmr(2,1)*fmr(4,3) 
      en=en+fk(107)*fmr(3,1)*fmr(4,3) 
      en=en+fk(108)*fmr(4,3)*fmr(5,1) 
      en=en+fk(109)*fmr(4,3)*fmr(6,1) 
      en=en+fk(110)*fmr(1,1)*fmr(5,3) 
      en=en+fk(111)*fmr(2,1)*fmr(5,3) 
      en=en+fk(112)*fmr(3,1)*fmr(5,3) 
      en=en+fk(113)*fmr(4,1)*fmr(5,3) 
      en=en+fk(114)*fmr(5,3)*fmr(6,1) 
      en=en+fk(115)*fmr(1,1)*fmr(6,3) 
      en=en+fk(116)*fmr(2,1)*fmr(6,3) 
      en=en+fk(117)*fmr(3,1)*fmr(6,3) 
      en=en+fk(118)*fmr(4,1)*fmr(6,3) 
      en=en+fk(119)*fmr(5,1)*fmr(6,3) 
      en=en+fk(120)*fmr(1,2)*fmr(2,2) 
      en=en+fk(121)*fmr(1,2)*fmr(3,2) 
      en=en+fk(122)*fmr(1,2)*fmr(4,2) 
      en=en+fk(123)*fmr(1,2)*fmr(5,2) 
      en=en+fk(124)*fmr(1,2)*fmr(6,2) 
      en=en+fk(125)*fmr(2,2)*fmr(3,2) 
      en=en+fk(126)*fmr(2,2)*fmr(4,2) 
      en=en+fk(127)*fmr(2,2)*fmr(5,2) 
      en=en+fk(128)*fmr(2,2)*fmr(6,2) 
      en=en+fk(129)*fmr(3,2)*fmr(4,2) 
      en=en+fk(130)*fmr(3,2)*fmr(5,2) 
      en=en+fk(131)*fmr(3,2)*fmr(6,2) 
      en=en+fk(132)*fmr(4,2)*fmr(5,2) 
      en=en+fk(133)*fmr(4,2)*fmr(6,2) 
      en=en+fk(134)*fmr(5,2)*fmr(6,2) 
      en=en+fk(135)*fmr(1,2)*fmr(2,1)*fmr(3,1) 
      en=en+fk(136)*fmr(1,2)*fmr(2,1)*fmr(4,1) 
      en=en+fk(137)*fmr(1,2)*fmr(2,1)*fmr(5,1) 
      en=en+fk(138)*fmr(1,2)*fmr(2,1)*fmr(6,1) 
      en=en+fk(139)*fmr(1,2)*fmr(3,1)*fmr(4,1) 
      en=en+fk(140)*fmr(1,2)*fmr(3,1)*fmr(5,1) 
      en=en+fk(141)*fmr(1,2)*fmr(3,1)*fmr(6,1) 
      en=en+fk(142)*fmr(1,2)*fmr(4,1)*fmr(5,1) 
      en=en+fk(143)*fmr(1,2)*fmr(4,1)*fmr(6,1) 
      en=en+fk(144)*fmr(1,2)*fmr(5,1)*fmr(6,1) 
      en=en+fk(145)*fmr(1,1)*fmr(2,2)*fmr(3,1) 
      en=en+fk(146)*fmr(1,1)*fmr(2,2)*fmr(4,1) 
      en=en+fk(147)*fmr(1,1)*fmr(2,2)*fmr(5,1) 
      en=en+fk(148)*fmr(1,1)*fmr(2,2)*fmr(6,1) 
      en=en+fk(149)*fmr(2,2)*fmr(3,1)*fmr(4,1) 
      en=en+fk(150)*fmr(2,2)*fmr(3,1)*fmr(5,1) 
      en=en+fk(151)*fmr(2,2)*fmr(3,1)*fmr(6,1) 
      en=en+fk(152)*fmr(2,2)*fmr(4,1)*fmr(5,1) 
      en=en+fk(153)*fmr(2,2)*fmr(4,1)*fmr(6,1) 
      en=en+fk(154)*fmr(2,2)*fmr(5,1)*fmr(6,1) 
      en=en+fk(155)*fmr(1,1)*fmr(2,1)*fmr(3,2) 
      en=en+fk(156)*fmr(1,1)*fmr(3,2)*fmr(4,1) 
      en=en+fk(157)*fmr(1,1)*fmr(3,2)*fmr(5,1) 
      en=en+fk(158)*fmr(1,1)*fmr(3,2)*fmr(6,1) 
      en=en+fk(159)*fmr(2,1)*fmr(3,2)*fmr(4,1) 
      en=en+fk(160)*fmr(2,1)*fmr(3,2)*fmr(5,1) 
      en=en+fk(161)*fmr(2,1)*fmr(3,2)*fmr(6,1) 
      en=en+fk(162)*fmr(3,2)*fmr(4,1)*fmr(5,1) 
      en=en+fk(163)*fmr(3,2)*fmr(4,1)*fmr(6,1) 
      en=en+fk(164)*fmr(3,2)*fmr(5,1)*fmr(6,1) 
      en=en+fk(165)*fmr(1,1)*fmr(2,1)*fmr(4,2) 
      en=en+fk(166)*fmr(1,1)*fmr(3,1)*fmr(4,2) 
      en=en+fk(167)*fmr(1,1)*fmr(4,2)*fmr(5,1) 
      en=en+fk(168)*fmr(1,1)*fmr(4,2)*fmr(6,1) 
      en=en+fk(169)*fmr(2,1)*fmr(3,1)*fmr(4,2) 
      en=en+fk(170)*fmr(2,1)*fmr(4,2)*fmr(5,1) 
      en=en+fk(171)*fmr(2,1)*fmr(4,2)*fmr(6,1) 
      en=en+fk(172)*fmr(3,1)*fmr(4,2)*fmr(5,1) 
      en=en+fk(173)*fmr(3,1)*fmr(4,2)*fmr(6,1) 
      en=en+fk(174)*fmr(4,2)*fmr(5,1)*fmr(6,1) 
      en=en+fk(175)*fmr(1,1)*fmr(2,1)*fmr(5,2) 
      en=en+fk(176)*fmr(1,1)*fmr(3,1)*fmr(5,2) 
      en=en+fk(177)*fmr(1,1)*fmr(4,1)*fmr(5,2) 
      en=en+fk(178)*fmr(1,1)*fmr(5,2)*fmr(6,1) 
      en=en+fk(179)*fmr(2,1)*fmr(3,1)*fmr(5,2) 
      en=en+fk(180)*fmr(2,1)*fmr(4,1)*fmr(5,2) 
      en=en+fk(181)*fmr(2,1)*fmr(5,2)*fmr(6,1) 
      en=en+fk(182)*fmr(3,1)*fmr(4,1)*fmr(5,2) 
      en=en+fk(183)*fmr(3,1)*fmr(5,2)*fmr(6,1) 
      en=en+fk(184)*fmr(4,1)*fmr(5,2)*fmr(6,1) 
      en=en+fk(185)*fmr(1,1)*fmr(2,1)*fmr(6,2) 
      en=en+fk(186)*fmr(1,1)*fmr(3,1)*fmr(6,2) 
      en=en+fk(187)*fmr(1,1)*fmr(4,1)*fmr(6,2) 
      en=en+fk(188)*fmr(1,1)*fmr(5,1)*fmr(6,2) 
      en=en+fk(189)*fmr(2,1)*fmr(3,1)*fmr(6,2) 
      en=en+fk(190)*fmr(2,1)*fmr(4,1)*fmr(6,2) 
      en=en+fk(191)*fmr(2,1)*fmr(5,1)*fmr(6,2) 
      en=en+fk(192)*fmr(3,1)*fmr(4,1)*fmr(6,2) 
      en=en+fk(193)*fmr(3,1)*fmr(5,1)*fmr(6,2) 
      en=en+fk(194)*fmr(4,1)*fmr(5,1)*fmr(6,2) 
      en=en+fk(195)*fmr(3,1)*fmr(4,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(196)*fmr(2,1)*fmr(4,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(197)*fmr(2,1)*fmr(3,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(198)*fmr(2,1)*fmr(3,1)*fmr(4,1)*fmr(6,1) 
      en=en+fk(199)*fmr(2,1)*fmr(3,1)*fmr(4,1)*fmr(5,1) 
      en=en+fk(200)*fmr(1,1)*fmr(4,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(201)*fmr(1,1)*fmr(3,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(202)*fmr(1,1)*fmr(3,1)*fmr(4,1)*fmr(6,1) 
      en=en+fk(203)*fmr(1,1)*fmr(3,1)*fmr(4,1)*fmr(5,1) 
      en=en+fk(204)*fmr(1,1)*fmr(2,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(205)*fmr(1,1)*fmr(2,1)*fmr(4,1)*fmr(6,1) 
      en=en+fk(206)*fmr(1,1)*fmr(2,1)*fmr(4,1)*fmr(5,1) 
      en=en+fk(207)*fmr(1,1)*fmr(2,1)*fmr(3,1)*fmr(6,1) 
      en=en+fk(208)*fmr(1,1)*fmr(2,1)*fmr(3,1)*fmr(5,1) 
      en=en+fk(209)*fmr(1,1)*fmr(2,1)*fmr(3,1)*fmr(4,1) 
      en=en+fk(210)*fmr(2,5) 
      en=en+fk(211)*fmr(3,5) 
      en=en+fk(212)*fmr(4,5) 
      en=en+fk(213)*fmr(5,5) 
      en=en+fk(214)*fmr(6,5) 
      en=en+fk(215)*fmr(1,1)*fmr(2,4) 
      en=en+fk(216)*fmr(2,4)*fmr(3,1) 
      en=en+fk(217)*fmr(2,4)*fmr(4,1) 
      en=en+fk(218)*fmr(2,4)*fmr(5,1) 
      en=en+fk(219)*fmr(2,4)*fmr(6,1) 
      en=en+fk(220)*fmr(1,1)*fmr(3,4) 
      en=en+fk(221)*fmr(2,1)*fmr(3,4) 
      en=en+fk(222)*fmr(3,4)*fmr(4,1) 
      en=en+fk(223)*fmr(3,4)*fmr(5,1) 
      en=en+fk(224)*fmr(3,4)*fmr(6,1) 
      en=en+fk(225)*fmr(1,1)*fmr(4,4) 
      en=en+fk(226)*fmr(2,1)*fmr(4,4) 
      en=en+fk(227)*fmr(3,1)*fmr(4,4) 
      en=en+fk(228)*fmr(4,4)*fmr(5,1) 
      en=en+fk(229)*fmr(4,4)*fmr(6,1) 
      en=en+fk(230)*fmr(1,1)*fmr(5,4) 
      en=en+fk(231)*fmr(2,1)*fmr(5,4) 
      en=en+fk(232)*fmr(3,1)*fmr(5,4) 
      en=en+fk(233)*fmr(4,1)*fmr(5,4) 
      en=en+fk(234)*fmr(5,4)*fmr(6,1) 
      en=en+fk(235)*fmr(1,1)*fmr(6,4) 
      en=en+fk(236)*fmr(2,1)*fmr(6,4) 
      en=en+fk(237)*fmr(3,1)*fmr(6,4) 
      en=en+fk(238)*fmr(4,1)*fmr(6,4) 
      en=en+fk(239)*fmr(5,1)*fmr(6,4) 
      en=en+fk(240)*fmr(1,3)*fmr(2,2) 
      en=en+fk(241)*fmr(1,3)*fmr(3,2) 
      en=en+fk(242)*fmr(1,3)*fmr(4,2) 
      en=en+fk(243)*fmr(1,3)*fmr(5,2) 
      en=en+fk(244)*fmr(1,3)*fmr(6,2) 
      en=en+fk(245)*fmr(1,2)*fmr(2,3) 
      en=en+fk(246)*fmr(2,3)*fmr(3,2) 
      en=en+fk(247)*fmr(2,3)*fmr(4,2) 
      en=en+fk(248)*fmr(2,3)*fmr(5,2) 
      en=en+fk(249)*fmr(2,3)*fmr(6,2) 
      en=en+fk(250)*fmr(1,2)*fmr(3,3) 
      en=en+fk(251)*fmr(2,2)*fmr(3,3) 
      en=en+fk(252)*fmr(3,3)*fmr(4,2) 
      en=en+fk(253)*fmr(3,3)*fmr(5,2) 
      en=en+fk(254)*fmr(3,3)*fmr(6,2) 
      en=en+fk(255)*fmr(1,2)*fmr(4,3) 
      en=en+fk(256)*fmr(2,2)*fmr(4,3) 
      en=en+fk(257)*fmr(3,2)*fmr(4,3) 
      en=en+fk(258)*fmr(4,3)*fmr(5,2) 
      en=en+fk(259)*fmr(4,3)*fmr(6,2) 
      en=en+fk(260)*fmr(1,2)*fmr(5,3) 
      en=en+fk(261)*fmr(2,2)*fmr(5,3) 
      en=en+fk(262)*fmr(3,2)*fmr(5,3) 
      en=en+fk(263)*fmr(4,2)*fmr(5,3) 
      en=en+fk(264)*fmr(5,3)*fmr(6,2) 
      en=en+fk(265)*fmr(1,2)*fmr(6,3) 
      en=en+fk(266)*fmr(2,2)*fmr(6,3) 
      en=en+fk(267)*fmr(3,2)*fmr(6,3) 
      en=en+fk(268)*fmr(4,2)*fmr(6,3) 
      en=en+fk(269)*fmr(5,2)*fmr(6,3) 
      en=en+fk(270)*fmr(1,3)*fmr(2,1)*fmr(3,1) 
      en=en+fk(271)*fmr(1,3)*fmr(2,1)*fmr(4,1) 
      en=en+fk(272)*fmr(1,3)*fmr(2,1)*fmr(5,1) 
      en=en+fk(273)*fmr(1,3)*fmr(2,1)*fmr(6,1) 
      en=en+fk(274)*fmr(1,3)*fmr(3,1)*fmr(4,1) 
      en=en+fk(275)*fmr(1,3)*fmr(3,1)*fmr(5,1) 
      en=en+fk(276)*fmr(1,3)*fmr(3,1)*fmr(6,1) 
      en=en+fk(277)*fmr(1,3)*fmr(4,1)*fmr(5,1) 
      en=en+fk(278)*fmr(1,3)*fmr(4,1)*fmr(6,1) 
      en=en+fk(279)*fmr(1,3)*fmr(5,1)*fmr(6,1) 
      en=en+fk(280)*fmr(1,1)*fmr(2,3)*fmr(3,1) 
      en=en+fk(281)*fmr(1,1)*fmr(2,3)*fmr(4,1) 
      en=en+fk(282)*fmr(1,1)*fmr(2,3)*fmr(5,1) 
      en=en+fk(283)*fmr(1,1)*fmr(2,3)*fmr(6,1) 
      en=en+fk(284)*fmr(2,3)*fmr(3,1)*fmr(4,1) 
      en=en+fk(285)*fmr(2,3)*fmr(3,1)*fmr(5,1) 
      en=en+fk(286)*fmr(2,3)*fmr(3,1)*fmr(6,1) 
      en=en+fk(287)*fmr(2,3)*fmr(4,1)*fmr(5,1) 
      en=en+fk(288)*fmr(2,3)*fmr(4,1)*fmr(6,1) 
      en=en+fk(289)*fmr(2,3)*fmr(5,1)*fmr(6,1) 
      en=en+fk(290)*fmr(1,1)*fmr(2,1)*fmr(3,3) 
      en=en+fk(291)*fmr(1,1)*fmr(3,3)*fmr(4,1) 
      en=en+fk(292)*fmr(1,1)*fmr(3,3)*fmr(5,1) 
      en=en+fk(293)*fmr(1,1)*fmr(3,3)*fmr(6,1) 
      en=en+fk(294)*fmr(2,1)*fmr(3,3)*fmr(4,1) 
      en=en+fk(295)*fmr(2,1)*fmr(3,3)*fmr(5,1) 
      en=en+fk(296)*fmr(2,1)*fmr(3,3)*fmr(6,1) 
      en=en+fk(297)*fmr(3,3)*fmr(4,1)*fmr(5,1) 
      en=en+fk(298)*fmr(3,3)*fmr(4,1)*fmr(6,1) 
      en=en+fk(299)*fmr(3,3)*fmr(5,1)*fmr(6,1) 
      en=en+fk(300)*fmr(1,1)*fmr(2,1)*fmr(4,3) 
      en=en+fk(301)*fmr(1,1)*fmr(3,1)*fmr(4,3) 
      en=en+fk(302)*fmr(1,1)*fmr(4,3)*fmr(5,1) 
      en=en+fk(303)*fmr(1,1)*fmr(4,3)*fmr(6,1) 
      en=en+fk(304)*fmr(2,1)*fmr(3,1)*fmr(4,3) 
      en=en+fk(305)*fmr(2,1)*fmr(4,3)*fmr(5,1) 
      en=en+fk(306)*fmr(2,1)*fmr(4,3)*fmr(6,1) 
      en=en+fk(307)*fmr(3,1)*fmr(4,3)*fmr(5,1) 
      en=en+fk(308)*fmr(3,1)*fmr(4,3)*fmr(6,1) 
      en=en+fk(309)*fmr(4,3)*fmr(5,1)*fmr(6,1) 
      en=en+fk(310)*fmr(1,1)*fmr(2,1)*fmr(5,3) 
      en=en+fk(311)*fmr(1,1)*fmr(3,1)*fmr(5,3) 
      en=en+fk(312)*fmr(1,1)*fmr(4,1)*fmr(5,3) 
      en=en+fk(313)*fmr(1,1)*fmr(5,3)*fmr(6,1) 
      en=en+fk(314)*fmr(2,1)*fmr(3,1)*fmr(5,3) 
      en=en+fk(315)*fmr(2,1)*fmr(4,1)*fmr(5,3) 
      en=en+fk(316)*fmr(2,1)*fmr(5,3)*fmr(6,1) 
      en=en+fk(317)*fmr(3,1)*fmr(4,1)*fmr(5,3) 
      en=en+fk(318)*fmr(3,1)*fmr(5,3)*fmr(6,1) 
      en=en+fk(319)*fmr(4,1)*fmr(5,3)*fmr(6,1) 
      en=en+fk(320)*fmr(1,1)*fmr(2,1)*fmr(6,3) 
      en=en+fk(321)*fmr(1,1)*fmr(3,1)*fmr(6,3) 
      en=en+fk(322)*fmr(1,1)*fmr(4,1)*fmr(6,3) 
      en=en+fk(323)*fmr(1,1)*fmr(5,1)*fmr(6,3) 
      en=en+fk(324)*fmr(2,1)*fmr(3,1)*fmr(6,3) 
      en=en+fk(325)*fmr(2,1)*fmr(4,1)*fmr(6,3) 
      en=en+fk(326)*fmr(2,1)*fmr(5,1)*fmr(6,3) 
      en=en+fk(327)*fmr(3,1)*fmr(4,1)*fmr(6,3) 
      en=en+fk(328)*fmr(3,1)*fmr(5,1)*fmr(6,3) 
      en=en+fk(329)*fmr(4,1)*fmr(5,1)*fmr(6,3) 
      en=en+fk(330)*fmr(1,1)*fmr(2,2)*fmr(3,2) 
      en=en+fk(331)*fmr(1,1)*fmr(2,2)*fmr(4,2) 
      en=en+fk(332)*fmr(1,1)*fmr(2,2)*fmr(5,2) 
      en=en+fk(333)*fmr(1,1)*fmr(2,2)*fmr(6,2) 
      en=en+fk(334)*fmr(1,1)*fmr(3,2)*fmr(4,2) 
      en=en+fk(335)*fmr(1,1)*fmr(3,2)*fmr(5,2) 
      en=en+fk(336)*fmr(1,1)*fmr(3,2)*fmr(6,2) 
      en=en+fk(337)*fmr(1,1)*fmr(4,2)*fmr(5,2) 
      en=en+fk(338)*fmr(1,1)*fmr(4,2)*fmr(6,2) 
      en=en+fk(339)*fmr(1,1)*fmr(5,2)*fmr(6,2) 
      en=en+fk(340)*fmr(1,2)*fmr(2,1)*fmr(3,2) 
      en=en+fk(341)*fmr(1,2)*fmr(2,1)*fmr(4,2) 
      en=en+fk(342)*fmr(1,2)*fmr(2,1)*fmr(5,2) 
      en=en+fk(343)*fmr(1,2)*fmr(2,1)*fmr(6,2) 
      en=en+fk(344)*fmr(2,1)*fmr(3,2)*fmr(4,2) 
      en=en+fk(345)*fmr(2,1)*fmr(3,2)*fmr(5,2) 
      en=en+fk(346)*fmr(2,1)*fmr(3,2)*fmr(6,2) 
      en=en+fk(347)*fmr(2,1)*fmr(4,2)*fmr(5,2) 
      en=en+fk(348)*fmr(2,1)*fmr(4,2)*fmr(6,2) 
      en=en+fk(349)*fmr(2,1)*fmr(5,2)*fmr(6,2) 
      en=en+fk(350)*fmr(1,2)*fmr(2,2)*fmr(3,1) 
      en=en+fk(351)*fmr(1,2)*fmr(3,1)*fmr(4,2) 
      en=en+fk(352)*fmr(1,2)*fmr(3,1)*fmr(5,2) 
      en=en+fk(353)*fmr(1,2)*fmr(3,1)*fmr(6,2) 
      en=en+fk(354)*fmr(2,2)*fmr(3,1)*fmr(4,2) 
      en=en+fk(355)*fmr(2,2)*fmr(3,1)*fmr(5,2) 
      en=en+fk(356)*fmr(2,2)*fmr(3,1)*fmr(6,2) 
      en=en+fk(357)*fmr(3,1)*fmr(4,2)*fmr(5,2) 
      en=en+fk(358)*fmr(3,1)*fmr(4,2)*fmr(6,2) 
      en=en+fk(359)*fmr(3,1)*fmr(5,2)*fmr(6,2) 
      en=en+fk(360)*fmr(1,2)*fmr(2,2)*fmr(4,1) 
      en=en+fk(361)*fmr(1,2)*fmr(3,2)*fmr(4,1) 
      en=en+fk(362)*fmr(1,2)*fmr(4,1)*fmr(5,2) 
      en=en+fk(363)*fmr(1,2)*fmr(4,1)*fmr(6,2) 
      en=en+fk(364)*fmr(2,2)*fmr(3,2)*fmr(4,1) 
      en=en+fk(365)*fmr(2,2)*fmr(4,1)*fmr(5,2) 
      en=en+fk(366)*fmr(2,2)*fmr(4,1)*fmr(6,2) 
      en=en+fk(367)*fmr(3,2)*fmr(4,1)*fmr(5,2) 
      en=en+fk(368)*fmr(3,2)*fmr(4,1)*fmr(6,2) 
      en=en+fk(369)*fmr(4,1)*fmr(5,2)*fmr(6,2) 
      en=en+fk(370)*fmr(1,2)*fmr(2,2)*fmr(5,1) 
      en=en+fk(371)*fmr(1,2)*fmr(3,2)*fmr(5,1) 
      en=en+fk(372)*fmr(1,2)*fmr(4,2)*fmr(5,1) 
      en=en+fk(373)*fmr(1,2)*fmr(5,1)*fmr(6,2) 
      en=en+fk(374)*fmr(2,2)*fmr(3,2)*fmr(5,1) 
      en=en+fk(375)*fmr(2,2)*fmr(4,2)*fmr(5,1) 
      en=en+fk(376)*fmr(2,2)*fmr(5,1)*fmr(6,2) 
      en=en+fk(377)*fmr(3,2)*fmr(4,2)*fmr(5,1) 
      en=en+fk(378)*fmr(3,2)*fmr(5,1)*fmr(6,2) 
      en=en+fk(379)*fmr(4,2)*fmr(5,1)*fmr(6,2) 
      en=en+fk(380)*fmr(1,2)*fmr(2,2)*fmr(6,1) 
      en=en+fk(381)*fmr(1,2)*fmr(3,2)*fmr(6,1) 
      en=en+fk(382)*fmr(1,2)*fmr(4,2)*fmr(6,1) 
      en=en+fk(383)*fmr(1,2)*fmr(5,2)*fmr(6,1) 
      en=en+fk(384)*fmr(2,2)*fmr(3,2)*fmr(6,1) 
      en=en+fk(385)*fmr(2,2)*fmr(4,2)*fmr(6,1) 
      en=en+fk(386)*fmr(2,2)*fmr(5,2)*fmr(6,1) 
      en=en+fk(387)*fmr(3,2)*fmr(4,2)*fmr(6,1) 
      en=en+fk(388)*fmr(3,2)*fmr(5,2)*fmr(6,1) 
      en=en+fk(389)*fmr(4,2)*fmr(5,2)*fmr(6,1) 
      en=en+fk(390)*fmr(1,2)*fmr(2,1)*fmr(3,1)*fmr(4,1) 
      en=en+fk(391)*fmr(1,2)*fmr(2,1)*fmr(3,1)*fmr(5,1) 
      en=en+fk(392)*fmr(1,2)*fmr(2,1)*fmr(3,1)*fmr(6,1) 
      en=en+fk(393)*fmr(1,2)*fmr(2,1)*fmr(4,1)*fmr(5,1) 
      en=en+fk(394)*fmr(1,2)*fmr(2,1)*fmr(4,1)*fmr(6,1) 
      en=en+fk(395)*fmr(1,2)*fmr(2,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(396)*fmr(1,2)*fmr(3,1)*fmr(4,1)*fmr(5,1) 
      en=en+fk(397)*fmr(1,2)*fmr(3,1)*fmr(4,1)*fmr(6,1) 
      en=en+fk(398)*fmr(1,2)*fmr(3,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(399)*fmr(1,2)*fmr(4,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(400)*fmr(1,1)*fmr(2,2)*fmr(3,1)*fmr(4,1) 
      en=en+fk(401)*fmr(1,1)*fmr(2,2)*fmr(3,1)*fmr(5,1) 
      en=en+fk(402)*fmr(1,1)*fmr(2,2)*fmr(3,1)*fmr(6,1) 
      en=en+fk(403)*fmr(1,1)*fmr(2,2)*fmr(4,1)*fmr(5,1) 
      en=en+fk(404)*fmr(1,1)*fmr(2,2)*fmr(4,1)*fmr(6,1) 
      en=en+fk(405)*fmr(1,1)*fmr(2,2)*fmr(5,1)*fmr(6,1) 
      en=en+fk(406)*fmr(2,2)*fmr(3,1)*fmr(4,1)*fmr(5,1) 
      en=en+fk(407)*fmr(2,2)*fmr(3,1)*fmr(4,1)*fmr(6,1) 
      en=en+fk(408)*fmr(2,2)*fmr(3,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(409)*fmr(2,2)*fmr(4,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(410)*fmr(1,1)*fmr(2,1)*fmr(3,2)*fmr(4,1) 
      en=en+fk(411)*fmr(1,1)*fmr(2,1)*fmr(3,2)*fmr(5,1) 
      en=en+fk(412)*fmr(1,1)*fmr(2,1)*fmr(3,2)*fmr(6,1) 
      en=en+fk(413)*fmr(1,1)*fmr(3,2)*fmr(4,1)*fmr(5,1) 
      en=en+fk(414)*fmr(1,1)*fmr(3,2)*fmr(4,1)*fmr(6,1) 
      en=en+fk(415)*fmr(1,1)*fmr(3,2)*fmr(5,1)*fmr(6,1) 
      en=en+fk(416)*fmr(2,1)*fmr(3,2)*fmr(4,1)*fmr(5,1) 
      en=en+fk(417)*fmr(2,1)*fmr(3,2)*fmr(4,1)*fmr(6,1) 
      en=en+fk(418)*fmr(2,1)*fmr(3,2)*fmr(5,1)*fmr(6,1) 
      en=en+fk(419)*fmr(3,2)*fmr(4,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(420)*fmr(1,1)*fmr(2,1)*fmr(3,1)*fmr(4,2) 
      en=en+fk(421)*fmr(1,1)*fmr(2,1)*fmr(4,2)*fmr(5,1) 
      en=en+fk(422)*fmr(1,1)*fmr(2,1)*fmr(4,2)*fmr(6,1) 
      en=en+fk(423)*fmr(1,1)*fmr(3,1)*fmr(4,2)*fmr(5,1) 
      en=en+fk(424)*fmr(1,1)*fmr(3,1)*fmr(4,2)*fmr(6,1) 
      en=en+fk(425)*fmr(1,1)*fmr(4,2)*fmr(5,1)*fmr(6,1) 
      en=en+fk(426)*fmr(2,1)*fmr(3,1)*fmr(4,2)*fmr(5,1) 
      en=en+fk(427)*fmr(2,1)*fmr(3,1)*fmr(4,2)*fmr(6,1) 
      en=en+fk(428)*fmr(2,1)*fmr(4,2)*fmr(5,1)*fmr(6,1) 
      en=en+fk(429)*fmr(3,1)*fmr(4,2)*fmr(5,1)*fmr(6,1) 
      en=en+fk(430)*fmr(1,1)*fmr(2,1)*fmr(3,1)*fmr(5,2) 
      en=en+fk(431)*fmr(1,1)*fmr(2,1)*fmr(4,1)*fmr(5,2) 
      en=en+fk(432)*fmr(1,1)*fmr(2,1)*fmr(5,2)*fmr(6,1) 
      en=en+fk(433)*fmr(1,1)*fmr(3,1)*fmr(4,1)*fmr(5,2) 
      en=en+fk(434)*fmr(1,1)*fmr(3,1)*fmr(5,2)*fmr(6,1) 
      en=en+fk(435)*fmr(1,1)*fmr(4,1)*fmr(5,2)*fmr(6,1) 
      en=en+fk(436)*fmr(2,1)*fmr(3,1)*fmr(4,1)*fmr(5,2) 
      en=en+fk(437)*fmr(2,1)*fmr(3,1)*fmr(5,2)*fmr(6,1) 
      en=en+fk(438)*fmr(2,1)*fmr(4,1)*fmr(5,2)*fmr(6,1) 
      en=en+fk(439)*fmr(3,1)*fmr(4,1)*fmr(5,2)*fmr(6,1) 
      en=en+fk(440)*fmr(1,1)*fmr(2,1)*fmr(3,1)*fmr(6,2) 
      en=en+fk(441)*fmr(1,1)*fmr(2,1)*fmr(4,1)*fmr(6,2) 
      en=en+fk(442)*fmr(1,1)*fmr(2,1)*fmr(5,1)*fmr(6,2) 
      en=en+fk(443)*fmr(1,1)*fmr(3,1)*fmr(4,1)*fmr(6,2) 
      en=en+fk(444)*fmr(1,1)*fmr(3,1)*fmr(5,1)*fmr(6,2) 
      en=en+fk(445)*fmr(1,1)*fmr(4,1)*fmr(5,1)*fmr(6,2) 
      en=en+fk(446)*fmr(2,1)*fmr(3,1)*fmr(4,1)*fmr(6,2) 
      en=en+fk(447)*fmr(2,1)*fmr(3,1)*fmr(5,1)*fmr(6,2) 
      en=en+fk(448)*fmr(2,1)*fmr(4,1)*fmr(5,1)*fmr(6,2) 
      en=en+fk(449)*fmr(3,1)*fmr(4,1)*fmr(5,1)*fmr(6,2) 
      en=en+fk(450)*fmr(1,1)*fmr(2,1)*fmr(3,1)*fmr(4,1)*fmr(5,1) 
      en=en+fk(451)*fmr(1,1)*fmr(2,1)*fmr(3,1)*fmr(4,1)*fmr(6,1) 
      en=en+fk(452)*fmr(1,1)*fmr(2,1)*fmr(3,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(453)*fmr(1,1)*fmr(2,1)*fmr(4,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(454)*fmr(1,1)*fmr(3,1)*fmr(4,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(455)*fmr(2,1)*fmr(3,1)*fmr(4,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(456)*fmr(2,6) 
      en=en+fk(457)*fmr(3,6) 
      en=en+fk(458)*fmr(4,6) 
      en=en+fk(459)*fmr(5,6) 
      en=en+fk(460)*fmr(6,6) 
      en=en+fk(461)*fmr(1,1)*fmr(2,5) 
      en=en+fk(462)*fmr(2,5)*fmr(4,1) 
      en=en+fk(463)*fmr(3,5)*fmr(5,1) 
      en=en+fk(464)*fmr(3,5)*fmr(6,1) 
      en=en+fk(465)*fmr(1,1)*fmr(4,5) 
      en=en+fk(466)*fmr(2,1)*fmr(4,5) 
      en=en+fk(467)*fmr(3,1)*fmr(5,5) 
      en=en+fk(468)*fmr(5,5)*fmr(6,1) 
      en=en+fk(469)*fmr(3,1)*fmr(6,5) 
      en=en+fk(470)*fmr(5,1)*fmr(6,5) 
      en=en+fk(471)*fmr(1,2)*fmr(2,4) 
      en=en+fk(472)*fmr(2,4)*fmr(4,2) 
      en=en+fk(473)*fmr(3,4)*fmr(5,2) 
      en=en+fk(474)*fmr(3,4)*fmr(6,2) 
      en=en+fk(475)*fmr(1,2)*fmr(4,4) 
      en=en+fk(476)*fmr(2,2)*fmr(4,4) 
      en=en+fk(477)*fmr(3,2)*fmr(5,4) 
      en=en+fk(478)*fmr(5,4)*fmr(6,2) 
      en=en+fk(479)*fmr(3,2)*fmr(6,4) 
      en=en+fk(480)*fmr(5,2)*fmr(6,4) 
      en=en+fk(481)*fmr(1,3)*fmr(2,3) 
      en=en+fk(482)*fmr(1,3)*fmr(4,3) 
      en=en+fk(483)*fmr(2,3)*fmr(4,3) 
      en=en+fk(484)*fmr(3,3)*fmr(5,3) 
      en=en+fk(485)*fmr(3,3)*fmr(6,3) 
      en=en+fk(486)*fmr(5,3)*fmr(6,3) 
      en=en+fk(487)*fmr(3,2)*fmr(5,2)*fmr(6,2) 
      en=en+fk(488)*fmr(1,2)*fmr(2,2)*fmr(4,2) 
      en=en+fk(489)*fmr(1,1)*fmr(2,4)*fmr(4,1) 
      en=en+fk(490)*fmr(3,4)*fmr(5,1)*fmr(6,1) 
      en=en+fk(491)*fmr(1,1)*fmr(2,1)*fmr(4,4) 
      en=en+fk(492)*fmr(3,1)*fmr(5,4)*fmr(6,1) 
      en=en+fk(493)*fmr(3,1)*fmr(5,1)*fmr(6,4) 
      en=en+fk(494)*fmr(1,3)*fmr(2,2)*fmr(4,1) 
      en=en+fk(495)*fmr(1,3)*fmr(2,1)*fmr(4,2) 
      en=en+fk(496)*fmr(1,2)*fmr(2,3)*fmr(4,1) 
      en=en+fk(497)*fmr(1,1)*fmr(2,3)*fmr(4,2) 
      en=en+fk(498)*fmr(3,3)*fmr(5,2)*fmr(6,1) 
      en=en+fk(499)*fmr(3,3)*fmr(5,1)*fmr(6,2) 
      en=en+fk(500)*fmr(1,2)*fmr(2,1)*fmr(4,3) 
      en=en+fk(501)*fmr(1,1)*fmr(2,2)*fmr(4,3) 
      en=en+fk(502)*fmr(3,2)*fmr(5,3)*fmr(6,1) 
      en=en+fk(503)*fmr(3,1)*fmr(5,3)*fmr(6,2) 
      en=en+fk(504)*fmr(3,2)*fmr(5,1)*fmr(6,3) 
      en=en+fk(505)*fmr(3,1)*fmr(5,2)*fmr(6,3) 

      return
      end subroutine

      subroutine f15158(rij,en)
      implicit real*8 (a-h,o-z)
      dimension fk(129),fmr(6,6), shift(6),rij(6),alpham(6)

      data shift  / 2.25d0, 2.12d0, 2.12d0, 3.85d0, 3.85d0, 3.57d0 /
      data alpham / 6*0.5d0 /
      data nmax,nvar /6,6/

      data fk(   1), fk(   2) / -2.9390689416D+02 ,  3.0721455678D-02 /
      data fk(   3), fk(   4) /  6.6239213627D-03 ,  1.6964139575D+03 /
      data fk(   5), fk(   6) / -6.5171941181D-03 , -9.3841011748D+02 /
      data fk(   7), fk(   8) /  1.8988605324D+02 ,  1.9408812478D+00 /
      data fk(   9), fk(  10) /  7.1152334306D-01 , -5.7338430094D+03 /
      data fk(  11), fk(  12) /  3.7444961204D-01 , -3.3228874185D+02 /
      data fk(  13), fk(  14) / -8.6348727176D+01 ,  4.9862216017D-01 /
      data fk(  15), fk(  16) / -4.9990285305D-01 , -5.1514895375D-01 /
      data fk(  17), fk(  18) /  4.8077945422D+03 , -4.4975675702D+02 /
      data fk(  19), fk(  20) / -2.0632615779D+02 , -3.1926662980D+00 /
      data fk(  21), fk(  22) / -1.0857568025D+00 ,  8.9558910791D+03 /
      data fk(  23), fk(  24) /  1.0570458477D+00 ,  1.1280706998D+01 /
      data fk(  25), fk(  26) / -1.8539285905D+01 , -2.1428460333D+00 /
      data fk(  27), fk(  28) /  1.9637429514D+00 , -1.1396881506D+00 /
      data fk(  29), fk(  30) /  1.5934207593D+00 , -8.9841597881D+03 /
      data fk(  31), fk(  32) /  1.3925258791D+02 , -2.5371193981D+00 /
      data fk(  33), fk(  34) / -2.2426739982D+00 ,  1.0653589275D+03 /
      data fk(  35), fk(  36) / -2.3175568110D+01 ,  2.9074496925D+02 /
      data fk(  37), fk(  38) /  3.7479104155D+01 ,  4.2267042428D+00 /
      data fk(  39), fk(  40) /  6.3826151344D+02 , -5.1332032404D-02 /
      data fk(  41), fk(  42) / -6.5973981293D+03 ,  1.1060363306D+00 /
      data fk(  43), fk(  44) /  7.9325676073D+00 ,  4.5863091982D-02 /
      data fk(  45), fk(  46) /  3.5819226273D+00 , -3.7798732737D-02 /
      data fk(  47), fk(  48) /  1.4822516440D+00 , -1.5225016214D+00 /
      data fk(  49), fk(  50) /  7.2738422553D+03 ,  3.0930412967D+02 /
      data fk(  51), fk(  52) / -3.8855184251D+00 , -3.2295346952D+00 /
      data fk(  53), fk(  54) / -5.5913933619D+01 ,  9.0770107962D+00 /
      data fk(  55), fk(  56) /  4.2734288986D+01 , -2.5682939554D+00 /
      data fk(  57), fk(  58) /  2.5129381213D+00 ,  3.1914687995D+00 /
      data fk(  59), fk(  60) /  2.5138440494D+00 , -1.0864439053D+03 /
      data fk(  61), fk(  62) / -3.2918699201D+02 ,  6.4137933088D+00 /
      data fk(  63), fk(  64) / -8.5424163808D+00 , -6.5770268756D+00 /
      data fk(  65), fk(  66) / -6.2361003385D+02 ,  1.0054127619D+01 /
      data fk(  67), fk(  68) /  1.4417260944D+01 , -8.4111811938D+01 /
      data fk(  69), fk(  70) /  4.6703015985D-01 ,  1.8550186268D+03 /
      data fk(  71), fk(  72) /  2.7707127059D-01 ,  2.5435920846D+00 /
      data fk(  73), fk(  74) /  3.6135428141D-01 ,  9.6788948530D-01 /
      data fk(  75), fk(  76) / -1.5443085318D-01 , -2.1406541123D+03 /
      data fk(  77), fk(  78) / -1.8463368229D+02 , -1.6878042416D+00 /
      data fk(  79), fk(  80) / -2.7565645894D+00 , -2.4301706481D+01 /
      data fk(  81), fk(  82) /  5.7726339492D+00 , -2.0286549464D+00 /
      data fk(  83), fk(  84) /  7.1609776371D-01 ,  1.6972482785D-01 /
      data fk(  85), fk(  86) / -1.6813126541D+00 ,  6.0823005048D+00 /
      data fk(  87), fk(  88) /  2.3103784873D+00 ,  3.1623463384D+02 /
      data fk(  89), fk(  90) /  1.2098338429D+02 ,  1.2226281002D+00 /
      data fk(  91), fk(  92) /  2.2039790963D+00 ,  8.0799180264D+01 /
      data fk(  93), fk(  94) / -1.6333741982D+00 , -1.9897620272D+01 /
      data fk(  95), fk(  96) /  9.1500271537D-01 , -3.8764817578D+00 /
      data fk(  97), fk(  98) / -3.7840366537D+00 ,  1.7855183292D+02 /
      data fk(  99), fk( 100) /  1.0384341973D+01 , -2.6278905502D+01 /
      data fk( 101), fk( 102) / -9.6184164594D-01 , -8.9055923083D+00 /
      data fk( 103), fk( 104) / -2.7729579789D+00 , -4.7126917442D+00 /
      data fk( 105), fk( 106) /  2.2654640751D+00 ,  4.9350066770D+01 /
      data fk( 107), fk( 108) /  2.9523790501D+01 ,  1.8507733524D-02 /
      data fk( 109), fk( 110) /  1.8129695894D-02 , -2.0233222621D+00 /
      data fk( 111), fk( 112) /  6.8353321205D-01 , -2.2596933924D-01 /
      data fk( 113), fk( 114) / -3.8111882977D-01 , -8.0402754975D+00 /
      data fk( 115), fk( 116) / -3.4079177740D+00 ,  2.4421983466D-01 /
      data fk( 117), fk( 118) /  1.6115379072D+00 , -5.6549466714D+00 /
      data fk( 119), fk( 120) /  6.1208351473D-01 ,  4.3579117973D-01 /
      data fk( 121), fk( 122) / -3.6302832819D+00 ,  8.3646415760D+00 /
      data fk( 123), fk( 124) /  2.1864169161D+00 ,  9.3229859157D+00 /
      data fk( 125), fk( 126) / -2.3490255791D+00 ,  8.0888334379D+00 /
      data fk( 127), fk( 128) / -8.1716336009D-01 , -8.0334030314D-01 /
      data fk( 129) / -5.9037836207D+00 /
   
c
      en=0.0d0
      do i=1,nvar
       fmr(i,1) = 1.0d0- exp(-alpham(i)*(rij(i)-shift(i)))
       do j=2,nmax
       fmr(i,j) = fmr(i,1)**j
       enddo
      enddo
c
      en=en+fk(1) 
      en=en+fk(2)*fmr(1,1) 
      en=en+fk(3)*fmr(2,1) 
      en=en+fk(4)*fmr(3,1) 
      en=en+fk(5)*fmr(4,1) 
      en=en+fk(6)*fmr(5,1) 
      en=en+fk(7)*fmr(6,1) 
      en=en+fk(8)*fmr(1,2) 
      en=en+fk(9)*fmr(2,2) 
      en=en+fk(10)*fmr(3,2) 
      en=en+fk(11)*fmr(4,2) 
      en=en+fk(12)*fmr(5,2) 
      en=en+fk(13)*fmr(6,2) 
      en=en+fk(14)*fmr(1,1)*fmr(2,1) 
      en=en+fk(15)*fmr(1,1)*fmr(4,1) 
      en=en+fk(16)*fmr(2,1)*fmr(4,1) 
      en=en+fk(17)*fmr(3,1)*fmr(5,1) 
      en=en+fk(18)*fmr(3,1)*fmr(6,1) 
      en=en+fk(19)*fmr(5,1)*fmr(6,1) 
      en=en+fk(20)*fmr(1,3) 
      en=en+fk(21)*fmr(2,3) 
      en=en+fk(22)*fmr(3,3) 
      en=en+fk(23)*fmr(4,3) 
      en=en+fk(24)*fmr(5,3) 
      en=en+fk(25)*fmr(6,3) 
      en=en+fk(26)*fmr(1,2)*fmr(2,1) 
      en=en+fk(27)*fmr(1,2)*fmr(4,1) 
      en=en+fk(28)*fmr(1,1)*fmr(2,2) 
      en=en+fk(29)*fmr(2,2)*fmr(4,1) 
      en=en+fk(30)*fmr(3,2)*fmr(5,1) 
      en=en+fk(31)*fmr(3,2)*fmr(6,1) 
      en=en+fk(32)*fmr(1,1)*fmr(4,2) 
      en=en+fk(33)*fmr(2,1)*fmr(4,2) 
      en=en+fk(34)*fmr(3,1)*fmr(5,2) 
      en=en+fk(35)*fmr(5,2)*fmr(6,1) 
      en=en+fk(36)*fmr(3,1)*fmr(6,2) 
      en=en+fk(37)*fmr(5,1)*fmr(6,2) 
      en=en+fk(38)*fmr(1,1)*fmr(2,1)*fmr(4,1) 
      en=en+fk(39)*fmr(3,1)*fmr(5,1)*fmr(6,1) 
      en=en+fk(40)*fmr(2,4) 
      en=en+fk(41)*fmr(3,4) 
      en=en+fk(42)*fmr(4,4) 
      en=en+fk(43)*fmr(5,4) 
      en=en+fk(44)*fmr(6,4) 
      en=en+fk(45)*fmr(1,3)*fmr(2,1) 
      en=en+fk(46)*fmr(1,3)*fmr(4,1) 
      en=en+fk(47)*fmr(1,1)*fmr(2,3) 
      en=en+fk(48)*fmr(2,3)*fmr(4,1) 
      en=en+fk(49)*fmr(3,3)*fmr(5,1) 
      en=en+fk(50)*fmr(3,3)*fmr(6,1) 
      en=en+fk(51)*fmr(1,1)*fmr(4,3) 
      en=en+fk(52)*fmr(2,1)*fmr(4,3) 
      en=en+fk(53)*fmr(3,1)*fmr(5,3) 
      en=en+fk(54)*fmr(5,3)*fmr(6,1) 
      en=en+fk(55)*fmr(3,1)*fmr(6,3) 
      en=en+fk(56)*fmr(5,1)*fmr(6,3) 
      en=en+fk(57)*fmr(1,2)*fmr(2,2) 
      en=en+fk(58)*fmr(1,2)*fmr(4,2) 
      en=en+fk(59)*fmr(2,2)*fmr(4,2) 
      en=en+fk(60)*fmr(3,2)*fmr(5,2) 
      en=en+fk(61)*fmr(3,2)*fmr(6,2) 
      en=en+fk(62)*fmr(5,2)*fmr(6,2) 
      en=en+fk(63)*fmr(1,2)*fmr(2,1)*fmr(4,1) 
      en=en+fk(64)*fmr(1,1)*fmr(2,2)*fmr(4,1) 
      en=en+fk(65)*fmr(3,2)*fmr(5,1)*fmr(6,1) 
      en=en+fk(66)*fmr(1,1)*fmr(2,1)*fmr(4,2) 
      en=en+fk(67)*fmr(3,1)*fmr(5,2)*fmr(6,1) 
      en=en+fk(68)*fmr(3,1)*fmr(5,1)*fmr(6,2) 
      en=en+fk(69)*fmr(2,5) 
      en=en+fk(70)*fmr(3,5) 
      en=en+fk(71)*fmr(4,5) 
      en=en+fk(72)*fmr(5,5) 
      en=en+fk(73)*fmr(6,5) 
      en=en+fk(74)*fmr(1,1)*fmr(2,4) 
      en=en+fk(75)*fmr(2,4)*fmr(4,1) 
      en=en+fk(76)*fmr(3,4)*fmr(5,1) 
      en=en+fk(77)*fmr(3,4)*fmr(6,1) 
      en=en+fk(78)*fmr(1,1)*fmr(4,4) 
      en=en+fk(79)*fmr(2,1)*fmr(4,4) 
      en=en+fk(80)*fmr(3,1)*fmr(5,4) 
      en=en+fk(81)*fmr(5,4)*fmr(6,1) 
      en=en+fk(82)*fmr(3,1)*fmr(6,4) 
      en=en+fk(83)*fmr(5,1)*fmr(6,4) 
      en=en+fk(84)*fmr(1,3)*fmr(2,2) 
      en=en+fk(85)*fmr(1,3)*fmr(4,2) 
      en=en+fk(86)*fmr(1,2)*fmr(2,3) 
      en=en+fk(87)*fmr(2,3)*fmr(4,2) 
      en=en+fk(88)*fmr(3,3)*fmr(5,2) 
      en=en+fk(89)*fmr(3,3)*fmr(6,2) 
      en=en+fk(90)*fmr(1,2)*fmr(4,3) 
      en=en+fk(91)*fmr(2,2)*fmr(4,3) 
      en=en+fk(92)*fmr(3,2)*fmr(5,3) 
      en=en+fk(93)*fmr(5,3)*fmr(6,2) 
      en=en+fk(94)*fmr(3,2)*fmr(6,3) 
      en=en+fk(95)*fmr(5,2)*fmr(6,3) 
      en=en+fk(96)*fmr(1,3)*fmr(2,1)*fmr(4,1) 
      en=en+fk(97)*fmr(1,1)*fmr(2,3)*fmr(4,1) 
      en=en+fk(98)*fmr(3,3)*fmr(5,1)*fmr(6,1) 
      en=en+fk(99)*fmr(1,1)*fmr(2,1)*fmr(4,3) 
      en=en+fk(100)*fmr(3,1)*fmr(5,3)*fmr(6,1) 
      en=en+fk(101)*fmr(3,1)*fmr(5,1)*fmr(6,3) 
      en=en+fk(102)*fmr(1,1)*fmr(2,2)*fmr(4,2) 
      en=en+fk(103)*fmr(1,2)*fmr(2,1)*fmr(4,2) 
      en=en+fk(104)*fmr(3,1)*fmr(5,2)*fmr(6,2) 
      en=en+fk(105)*fmr(1,2)*fmr(2,2)*fmr(4,1) 
      en=en+fk(106)*fmr(3,2)*fmr(5,1)*fmr(6,2) 
      en=en+fk(107)*fmr(3,2)*fmr(5,2)*fmr(6,1) 
      en=en+fk(108)*fmr(2,6) 
      en=en+fk(109)*fmr(4,6) 
      en=en+fk(110)*fmr(1,1)*fmr(2,5) 
      en=en+fk(111)*fmr(2,5)*fmr(4,1) 
      en=en+fk(112)*fmr(1,1)*fmr(4,5) 
      en=en+fk(113)*fmr(2,1)*fmr(4,5) 
      en=en+fk(114)*fmr(1,2)*fmr(2,4) 
      en=en+fk(115)*fmr(2,4)*fmr(4,2) 
      en=en+fk(116)*fmr(1,2)*fmr(4,4) 
      en=en+fk(117)*fmr(2,2)*fmr(4,4) 
      en=en+fk(118)*fmr(1,3)*fmr(2,3) 
      en=en+fk(119)*fmr(1,3)*fmr(4,3) 
      en=en+fk(120)*fmr(2,3)*fmr(4,3) 
      en=en+fk(121)*fmr(1,2)*fmr(2,2)*fmr(4,2) 
      en=en+fk(122)*fmr(1,1)*fmr(2,4)*fmr(4,1) 
      en=en+fk(123)*fmr(1,1)*fmr(2,1)*fmr(4,4) 
      en=en+fk(124)*fmr(1,3)*fmr(2,2)*fmr(4,1) 
      en=en+fk(125)*fmr(1,3)*fmr(2,1)*fmr(4,2) 
      en=en+fk(126)*fmr(1,2)*fmr(2,3)*fmr(4,1) 
      en=en+fk(127)*fmr(1,1)*fmr(2,3)*fmr(4,2) 
      en=en+fk(128)*fmr(1,2)*fmr(2,1)*fmr(4,3) 
      en=en+fk(129)*fmr(1,1)*fmr(2,2)*fmr(4,3) 

      return
      end subroutine


C This CCSD(T) fit swithes between three fits. rbig is the distance 
C between the centers of mass of CO and HH.

      subroutine h2co_ccsdt(r,rbig,v)

      implicit real*8 (a-h,o-z)
      dimension r(6)

      bohr=0.529177d0
      hcm=219474.63d0

      rl1=4.5d0
      ru1=5.5d0
      rl2=9.5d0
      ru2=10.0d0

      xs1=(rbig-rl1)/(ru1-rl1)
      xs2=(rbig-rl2)/(ru2-rl2)
c     write(*,*) xs1,xs2

      if (xs1.LT.0.) then
         Call h2co_inter(r,v)
         v=v*hcm+234.653256d0
      else if ((xs1.GE.0.).and.(xs1.LT.1.)) then 
         Call h2co_inter(r,v1)
         v1=v1*hcm+234.653256d0
         Call h2co_asym(r,v2)
         v2=v2*hcm+234.653256d0
         s=1.0d0-(10.0d0*xs1**3-15.0d0*xs1**4+6.0d0*xs1**5)
         v=v1*s+v2*(1-s)
      else if (xs2.LT.0.) then
         Call h2co_asym(r,v)
         v=v*hcm+234.653256d0
c        write(*,*)v
      else if ((xs2.GE.0.).and.(xs2.LT.1.)) then
         Call h2co_asym(r,v1)
         v1=v1*hcm+234.653256d0
         Call hh_pot(r(6),vhh)
         Call co_pot(r(3),vco)
c        write(*,*) vhh*hcm,vco*hcm 
         v2=vhh*hcm+vco*hcm+1931.628493d0
         s=1.0d0-(10.0d0*xs2**3-15.0d0*xs2**4+6.0d0*xs2**5)
         v=v1*s+v2*(1-s)
      else if (xs2.GE.1) then
         Call hh_pot(r(6),vhh)
         Call co_pot(r(3),vco)
c        write(*,*) vhh*hcm,vco*hcm
         v=vhh*hcm+vco*hcm+1931.628493d0
      end if
  
      return
      end subroutine

C*************************************************************************
C*************************************************************************
      subroutine h2co_inter(r,pot)
 
      implicit double precision (a-h, o-z)
      parameter (nfun=896)
      dimension r(6),re(6),dr(6),coeff(nfun)
      dimension terms(nfun)
      hcm=219474.63
      data re/
     &  3.8215337d0,
     &  2.0845219d0,
     &  2.289493d0,
     &  2.0845219d0,
     &  3.8215337d0,
     &  3.54666986d0/
       do i=1,6
           dr(i)=1-exp(-.5d0*(r(i)-re(i)))
       enddo
       data coeff/
     & -0.1543032646D+03, -0.2807830558D+05, 0.2287926146D+05, 
     & 0.3127263433D+05, 0.1560041041D+05, 0.3415380909D+04, 
     & 0.2691017057D+03, -0.3017000846D+05, -0.5944163463D+04, 
     & 0.1003476920D+05, 0.6140349001D+04, 0.9118260609D+03, 
     & 0.1329258947D+03, 0.2000136194D+05, 0.6224769077D+04, 
     & -0.6261386778D+04, -0.1025387862D+05, -0.4649215291D+04, 
     & 0.2260538690D+05, 0.1169447682D+05, 0.1557515896D+04, 
     & 0.3092730176D+04, 0.4721082626D+04, 0.4828824172D+04, 
     & 0.4029509574D+03, -0.9376544252D+03, 0.3268696623D+03, 
     & -0.2041858708D+03, 0.4512431552D+05, -0.7139542280D+05, 
     & -0.1479643730D+06, -0.6164890338D+05, -0.1277528552D+05, 
     & -0.1484365682D+04, 0.9820350557D+04, -0.1103472393D+06, 
     & -0.1426891892D+06, -0.8108239892D+05, -0.2013141347D+05, 
     & -0.3875766023D+05, -0.7800876043D+04, 0.1474474677D+05, 
     & 0.1342639184D+05, -0.1981168237D+05, 0.1817150300D+05, 
     & 0.5975551926D+04, -0.4650671307D+04, 0.3357536676D+04, 
     & -0.1003285011D+04, 0.2301241828D+06, 0.3117249508D+06, 
     & 0.8009257993D+05, -0.4048249071D+05, -0.1539127702D+05, 
     & 0.1740422551D+06, 0.3905312391D+06, 0.1829431796D+06, 
     & 0.3404141908D+05, 0.6846526868D+05, 0.1371447223D+06, 
     & 0.2100006082D+05, -0.2268621454D+05, 0.1779946330D+05, 
     & -0.1121807037D+05, -0.2734360016D+06, -0.4952758682D+04, 
     & 0.5137325199D+05, 0.3396803411D+05, -0.1670297519D+06, 
     & -0.1142071100D+06, -0.6450972847D+04, -0.1076121997D+06, 
     & -0.1794670063D+05, -0.2506302570D+05, -0.1019742303D+06, 
     & -0.3885301743D+06, 0.4431625108D+05, -0.1778785503D+06, 
     & 0.5662019547D+05, -0.6215928748D+05, -0.2464491911D+05, 
     & 0.2929203301D+06, -0.3883733557D+05, 0.6069580602D+05, 
     & 0.5296503317D+05, 0.4164624770D+05, 0.1840015243D+05, 
     & 0.6885856543D+04, 0.4498746438D+04, -0.2828861747D+03, 
     & 0.7298034198D+05, 0.3608829053D+05, -0.5823683658D+05, 
     & -0.1025602607D+05, 0.6484598642D+03, -0.2952908986D+05, 
     & -0.3810089459D+04, -0.5041697340D+04, 0.6118224176D+04, 
     & -0.4459887679D+04, -0.1858742610D+05, -0.7971411987D+04, 
     & 0.1397839385D+05, -0.4025478250D+04, 0.4513430511D+04, 
     & -0.4962833550D+05, 0.4401527772D+05, 0.3060529805D+05, 
     & 0.1972483255D+05, -0.4889653487D+04, -0.1011322451D+06, 
     & -0.1470878659D+06, 0.1392402950D+06, 0.3101255543D+05, 
     & -0.4551707893D+05, -0.1373100492D+06, 0.1362168681D+05, 
     & -0.1202263919D+05, -0.4524101413D+05, -0.5571799313D+04, 
     & -0.1730700398D+06, -0.2620218788D+06, 0.1157854397D+06, 
     & 0.1030144302D+05, -0.3203293215D+04, -0.4214807489D+05, 
     & -0.2109780906D+05, 0.2041503261D+06, -0.9581742473D+05, 
     & 0.7476399748D+05, 0.8114803624D+05, -0.1802819335D+06, 
     & -0.1812289856D+06, 0.1588634635D+06, -0.1500305216D+06, 
     & -0.2532757746D+04, 0.7182378290D+06, 0.9692780601D+05, 
     & 0.9535200854D+05, -0.3955916054D+06, 0.2479957696D+06, 
     & -0.4987979981D+05, 0.5488929390D+05, 0.4185500997D+04, 
     & -0.1535888234D+04, -0.1102927297D+06, 0.1873399991D+05, 
     & 0.2969566866D+05, -0.1168692378D+04, -0.8852758081D+05, 
     & 0.6398618601D+05, 0.5237194006D+04, -0.4199216273D+05, 
     & -0.3224983284D+05, -0.6421235754D+04, 0.1547453554D+06, 
     & 0.1494554048D+06, -0.1133812540D+06, -0.7499817709D+04, 
     & 0.3342415369D+06, -0.4855556978D+05, 0.4273980461D+05, 
     & 0.2066063506D+06, -0.2319522911D+06, -0.1351245363D+05, 
     & -0.1952282647D+06, -0.2681616284D+06, 0.1272631995D+06, 
     & -0.5606042870D+06, 0.3541021263D+05, -0.2090605266D+06, 
     & -0.4239076240D+06, 0.1014863345D+06, 0.3746567139D+06, 
     & 0.5319354435D+06, -0.3442813493D+06, -0.9174018956D+05, 
     & 0.2093882827D+05, -0.3138680249D+05, 0.1475308874D+06, 
     & -0.2485598834D+06, -0.1358830686D+05, 0.3275115817D+06, 
     & -0.1174264569D+06, 0.9412289800D+05, -0.2863399993D+06, 
     & 0.4776720128D+06, -0.8395743745D+05, -0.1496693987D+06, 
     & 0.1469286798D+06, -0.3862157111D+06, 0.3702492824D+05, 
     & -0.3840223933D+05, 0.5749497712D+06, -0.2612134730D+06, 
     & 0.4512431552D+05, -0.7139542280D+05, -0.1479643730D+06, 
     & -0.6164890338D+05, -0.1277528552D+05, -0.1484365682D+04, 
     & -0.5941070027D+05, -0.1232364856D+06, 0.5971413028D+05, 
     & 0.6455283334D+05, 0.1967038019D+05, -0.5730427573D+05, 
     & -0.3482395361D+05, 0.1632169124D+05, -0.1227273126D+05, 
     & 0.2814760565D+05, -0.2919155296D+05, -0.3340319995D+04, 
     & 0.1742024152D+05, -0.7426937315D+04, 0.1698001746D+04, 
     & 0.1089484237D+06, 0.6309689750D+06, 0.5858654694D+06, 
     & 0.2221188960D+06, 0.3871477040D+05, 0.3721026855D+06, 
     & 0.4551117995D+06, 0.8821708762D+05, 0.2359047651D+03, 
     & 0.1954236781D+06, -0.1249119520D+05, -0.4234270006D+05, 
     & 0.3323662159D+05, -0.3078663879D+05, 0.1002644904D+04, 
     & -0.7329961934D+06, -0.1231345016D+07, -0.4473376770D+06, 
     & -0.6505914414D+05, -0.8100804104D+06, -0.9413248367D+06, 
     & -0.1905042658D+06, -0.2086373266D+06, -0.8294364915D+05, 
     & -0.7056907749D+04, 0.5418711842D+06, 0.5338412924D+06, 
     & -0.8387886735D+05, 0.6624272342D+06, 0.1387021041D+06, 
     & 0.1942642631D+06, 0.4519374722D+06, -0.1741220407D+06, 
     & 0.2601838914D+06, -0.1279856249D+06, -0.4962833550D+05, 
     & 0.4401527772D+05, 0.3060529805D+05, 0.1972483255D+05, 
     & -0.4889653487D+04, 0.9372123621D+05, -0.1825536562D+05, 
     & -0.3020969611D+05, -0.1717419785D+05, -0.9333143240D+05, 
     & 0.8636181313D+05, -0.6745271977D+04, -0.5473426048D+05, 
     & 0.8560288259D+05, -0.2366119527D+03, -0.3392653750D+06, 
     & -0.5346028925D+06, -0.3189807397D+06, -0.9811163364D+05, 
     & -0.1339470575D+06, 0.2318951931D+06, -0.1833033066D+06, 
     & 0.2824916636D+05, 0.5212300970D+06, -0.8722621491D+04, 
     & 0.1340759554D+07, 0.1125030101D+07, 0.1545373720D+06, 
     & -0.4792473588D+05, 0.6876030451D+06, -0.9602842199D+05, 
     & -0.1172099413D+07, 0.2653041639D+05, -0.5302618309D+06, 
     & -0.1896354197D+06, 0.1547453554D+06, 0.1494554048D+06, 
     & -0.1133812540D+06, -0.7499817709D+04, 0.3877725222D+06, 
     & 0.3545754772D+05, -0.1132122863D+06, -0.5424885723D+04, 
     & 0.1229212459D+06, 0.4126751829D+05, -0.2819394269D+06, 
     & -0.4821825062D+06, -0.8349178391D+05, -0.8520762834D+06, 
     & 0.5566193954D+05, 0.1763537486D+04, 0.5609085841D+06, 
     & 0.3197526231D+06, 0.1469150140D+06, -0.8836746671D+06, 
     & -0.2863399993D+06, 0.4776720128D+06, -0.8395743745D+05, 
     & -0.9085966350D+06, 0.1085339508D+06, -0.1911382168D+06, 
     & 0.1340614206D+07, -0.1137677882D+07, -0.2356938072D+06, 
     & -0.3899478002D+06, 0.2301241828D+06, 0.3117249508D+06, 
     & 0.8009257993D+05, -0.4048249071D+05, -0.1539127702D+05, 
     & 0.1417473933D+06, 0.1681019590D+06, -0.7525409651D+05, 
     & -0.1994750720D+05, -0.5429391252D+05, -0.4905675089D+05, 
     & 0.4533936329D+03, -0.9847072621D+05, -0.1005546451D+05, 
     & -0.1998106256D+05, -0.7329961934D+06, -0.1231345016D+07, 
     & -0.4473376770D+06, -0.6505914414D+05, -0.8797439103D+06, 
     & -0.3656021921D+06, 0.1331105891D+06, -0.1429591982D+06, 
     & -0.1695858600D+06, 0.1434163863D+05, 0.1457032085D+07, 
     & 0.1593719380D+07, 0.3836479139D+06, 0.1187045864D+07, 
     & 0.2095796616D+06, 0.1155428685D+06, -0.1021794490D+07, 
     & -0.3550229903D+06, -0.5753432650D+06, -0.4449122723D+05, 
     & -0.1730700398D+06, -0.2620218788D+06, 0.1157854397D+06, 
     & 0.1030144302D+05, -0.5307637518D+05, 0.1340086412D+06, 
     & 0.1534283064D+06, 0.3355236359D+06, -0.3661190539D+06, 
     & 0.4707256914D+05, 0.1340759554D+07, 0.1125030101D+07, 
     & 0.1545373720D+06, 0.1025157908D+07, -0.1048241041D+07, 
     & -0.3046113478D+06, -0.2283043269D+07, -0.1252810761D+07, 
     & -0.1627591043D+06, 0.1152500757D+07, -0.1952282647D+06, 
     & -0.2681616284D+06, 0.1272631995D+06, 0.3225304750D+06, 
     & -0.1903075966D+06, 0.3348651295D+05, 0.5609085841D+06, 
     & 0.3197526231D+06, 0.4547058194D+06, 0.9754431959D+06, 
     & 0.3702492824D+05, -0.3840223933D+05, 0.7195853854D+06, 
     & -0.3899478002D+06, -0.2734360016D+06, -0.4952758682D+04, 
     & 0.5137325199D+05, 0.3396803411D+05, 0.5488994672D+05, 
     & 0.1368285443D+06, -0.5892886990D+05, 0.1141850043D+06, 
     & 0.2517424353D+06, 0.5297977556D+05, 0.5418711842D+06, 
     & 0.5338412924D+06, -0.8387886735D+05, 0.4227610710D+06, 
     & 0.5361952716D+06, 0.1310954103D+06, -0.1021794490D+07, 
     & -0.3550229903D+06, -0.3096961558D+06, 0.4788645512D+06, 
     & 0.8114803624D+05, -0.1802819335D+06, -0.1812289856D+06, 
     & -0.9980274314D+06, 0.2793761970D+06, -0.7892661525D+05, 
     & -0.1172099413D+07, 0.2653041639D+05, -0.1106520627D+05, 
     & 0.1152500757D+07, -0.4239076240D+06, 0.1014863345D+06, 
     & -0.5492777796D+06, -0.8836746671D+06, -0.2612134730D+06, 
     & -0.1019742303D+06, -0.3885301743D+06, 0.4431625108D+05, 
     & -0.8150731558D+05, -0.4350905968D+06, -0.1208203509D+06, 
     & 0.4519374722D+06, -0.1741220407D+06, -0.3509180860D+06, 
     & -0.4449122723D+05, 0.7182378290D+06, 0.9692780601D+05, 
     & 0.5755447075D+06, -0.1896354197D+06, 0.5319354435D+06, 
     & -0.2464491911D+05, 0.2929203301D+06, 0.1626602189D+06, 
     & -0.1279856249D+06, -0.3955916054D+06, 0.6069580602D+05, 
     & -0.3017000846D+05, -0.5944163463D+04, 0.1003476920D+05, 
     & 0.6140349002D+04, 0.9118260609D+03, 0.1329258947D+03, 
     & -0.4567779762D+05, 0.1388121467D+05, 0.6667044045D+05, 
     & 0.2868951926D+05, 0.9723764308D+04, -0.2085554515D+05, 
     & -0.2634864331D+05, -0.2189995180D+04, -0.3723294597D+04, 
     & -0.2130604198D+05, -0.2613900324D+05, -0.1052453916D+05, 
     & -0.1080458451D+05, -0.4538567568D+04, -0.1653841881D+04, 
     & -0.5941070027D+05, -0.1232364856D+06, 0.5971413028D+05, 
     & 0.6455283334D+05, 0.1967038019D+05, -0.2007918016D+05, 
     & -0.9819811657D+05, -0.1200876185D+06, -0.5702912951D+04, 
     & 0.1366314809D+05, -0.8655623578D+05, -0.6967607523D+05, 
     & 0.4285571140D+05, -0.2934770748D+04, 0.1363195424D+05, 
     & 0.1417473933D+06, 0.1681019590D+06, -0.7525409651D+05, 
     & -0.1994750720D+05, 0.2469859762D+06, -0.1699069400D+06, 
     & -0.2201376269D+05, 0.1828628482D+06, -0.2241902486D+05, 
     & 0.3713531200D+05, 0.5488994672D+05, 0.1368285443D+06, 
     & -0.5892886990D+05, 0.1841586412D+05, -0.1045646680D+05, 
     & 0.4325495409D+05, -0.8150731558D+05, -0.4350905968D+06, 
     & 0.1141488789D+06, 0.1626602189D+06, 0.7298034198D+05, 
     & 0.3608829053D+05, -0.5823683658D+05, -0.1025602607D+05, 
     & 0.6484598642D+03, 0.1333051910D+06, 0.2216166045D+06, 
     & 0.3884983084D+05, -0.7772687240D+04, 0.8995940976D+05, 
     & 0.2238480005D+05, 0.6437634212D+04, 0.2480432529D+05, 
     & 0.4077616826D+05, 0.1753959535D+04, 0.9372123621D+05, 
     & -0.1825536561D+05, -0.3020969611D+05, -0.1717419785D+05, 
     & -0.4388453426D+06, -0.2319693584D+06, -0.1346457822D+05, 
     & -0.2797867235D+06, 0.1661055490D+06, -0.3370443842D+05, 
     & -0.5307637518D+05, 0.1340086412D+06, 0.1534283064D+06, 
     & 0.7217293720D+05, 0.4001920546D+06, -0.5416987931D+05, 
     & -0.9980274314D+06, 0.2793761970D+06, -0.2312207115D+06, 
     & 0.5755447075D+06, -0.1102927297D+06, 0.1873399991D+05, 
     & 0.2969566866D+05, -0.1168692378D+04, -0.1450991397D+06, 
     & 0.5605918661D+05, 0.2081059116D+05, -0.1895254207D+06, 
     & 0.8224681305D+05, -0.1120198817D+05, 0.3877725222D+06, 
     & 0.3545754772D+05, -0.1132122863D+06, 0.4557557960D+06, 
     & -0.1619861862D+05, 0.1976138682D+06, 0.3225304750D+06, 
     & -0.1903075966D+06, -0.1213463084D+06, -0.5492777796D+06, 
     & 0.1475308874D+06, -0.2485598834D+06, -0.1358830686D+05, 
     & 0.1808583345D+06, 0.1340453275D+06, 0.7447382737D+05, 
     & -0.9085966350D+06, 0.1085339508D+06, -0.2248762807D+06, 
     & 0.7195853854D+06, 0.9820350557D+04, -0.1103472393D+06, 
     & -0.1426891892D+06, -0.8108239892D+05, -0.2013141347D+05, 
     & -0.2007918016D+05, -0.9819811657D+05, -0.1200876185D+06, 
     & -0.5702912951D+04, 0.3076734911D+05, 0.1402909052D+06, 
     & 0.6219944446D+05, 0.3740209543D+05, 0.2538128525D+05, 
     & 0.7687137399D+04, 0.3721026855D+06, 0.4551117995D+06, 
     & 0.8821708762D+05, 0.2359047651D+03, 0.2815797517D+06, 
     & 0.5996241027D+06, 0.2235264347D+06, -0.4099084000D+05, 
     & 0.4644198605D+05, -0.4129220010D+05, -0.8797439103D+06, 
     & -0.3656021921D+06, 0.1331105891D+06, -0.7341756645D+06, 
     & -0.3848407695D+05, -0.2128144866D+06, 0.4227610710D+06, 
     & 0.5361952716D+06, -0.8583085150D+05, -0.3509180860D+06, 
     & -0.1011322451D+06, -0.1470878659D+06, 0.1392402950D+06, 
     & 0.3101255543D+05, -0.4388453426D+06, -0.2319693584D+06, 
     & -0.1346457822D+05, -0.1331585984D+06, -0.2152991036D+06, 
     & -0.7763492950D+05, -0.1339470575D+06, 0.2318951931D+06, 
     & -0.1833033066D+06, 0.1242544157D+07, -0.5071619888D+06, 
     & 0.1753102494D+06, 0.1025157908D+07, -0.1048241041D+07, 
     & -0.3936388586D+05, -0.1106520627D+05, 0.3342415369D+06, 
     & -0.4855556978D+05, 0.4273980461D+05, 0.4557557960D+06, 
     & -0.1619861862D+05, 0.1226359731D+06, -0.8520762834D+06, 
     & 0.5566193954D+05, -0.6841115204D+06, 0.4547058194D+06, 
     & -0.1496693987D+06, 0.1469286798D+06, -0.2248762807D+06, 
     & -0.2356938072D+06, 0.1740422551D+06, 0.3905312391D+06, 
     & 0.1829431796D+06, 0.3404141908D+05, 0.2469859762D+06, 
     & -0.1699069400D+06, -0.2201376269D+05, -0.6302185744D+05, 
     & -0.3743609292D+05, -0.6112123778D+04, -0.8100804104D+06, 
     & -0.9413248367D+06, -0.1905042658D+06, -0.7341756645D+06, 
     & -0.3848407695D+05, 0.9655053307D+05, 0.1187045864D+07, 
     & 0.2095796616D+06, 0.7136967870D+06, -0.3096961558D+06, 
     & -0.3203293215D+04, -0.4214807489D+05, -0.2109780906D+05, 
     & 0.7217293720D+05, 0.4001920546D+06, 0.2279496534D+06, 
     & -0.4792473588D+05, 0.6876030451D+06, -0.3936388586D+05, 
     & -0.1627591043D+06, -0.5606042870D+06, 0.3541021263D+05, 
     & -0.1213463084D+06, 0.1469150140D+06, 0.5749497712D+06, 
     & -0.1670297519D+06, -0.1142071100D+06, -0.6450972847D+04, 
     & 0.1841586412D+05, -0.1045646680D+05, -0.5405620088D+05, 
     & 0.6624272342D+06, 0.1387021041D+06, -0.8583085150D+05, 
     & -0.5753432650D+06, 0.1588634635D+06, -0.1500305216D+06, 
     & -0.2312207115D+06, -0.5302618309D+06, 0.3746567139D+06, 
     & -0.1778785503D+06, 0.5662019547D+05, 0.1141488789D+06, 
     & 0.2601838914D+06, 0.9535200854D+05, -0.3883733557D+05, 
     & 0.2000136194D+05, 0.6224769077D+04, -0.6261386778D+04, 
     & -0.1025387862D+05, -0.4649215291D+04, -0.2085554515D+05, 
     & -0.2634864331D+05, -0.2189995180D+04, -0.3723294597D+04, 
     & 0.4130747936D+04, 0.2739341773D+05, 0.2337156519D+05, 
     & -0.4099582349D+04, 0.1719514743D+04, -0.2197945223D+04, 
     & -0.5730427573D+05, -0.3482395361D+05, 0.1632169124D+05, 
     & -0.1227273126D+05, 0.3076734911D+05, 0.1402909052D+06, 
     & 0.6219944446D+05, -0.4893850334D+05, -0.1638440371D+04, 
     & -0.1870655068D+05, -0.5429391252D+05, -0.4905675089D+05, 
     & 0.4533936329D+03, -0.6302185744D+05, -0.3743609292D+05, 
     & -0.1335029011D+05, 0.1141850043D+06, 0.2517424353D+06, 
     & -0.5405620088D+05, -0.1208203509D+06, -0.2952908986D+05, 
     & -0.3810089459D+04, -0.5041697340D+04, 0.6118224176D+04, 
     & 0.8995940976D+05, 0.2238480005D+05, 0.6437634212D+04, 
     & 0.1397490036D+06, -0.7212862759D+05, 0.2051935916D+05, 
     & -0.9333143240D+05, 0.8636181313D+05, -0.6745271977D+04, 
     & -0.1331585984D+06, -0.2152991036D+06, -0.3808688500D+05, 
     & 0.3355236359D+06, -0.3661190539D+06, 0.2279496534D+06, 
     & -0.7892661525D+05, -0.8852758080D+05, 0.6398618601D+05, 
     & 0.5237194006D+04, -0.1895254207D+06, 0.8224681305D+05, 
     & 0.4784856975D+05, -0.5424885723D+04, 0.1229212459D+06, 
     & 0.1226359731D+06, 0.3348651295D+05, 0.3275115817D+06, 
     & -0.1174264569D+06, 0.7447382737D+05, -0.1911382168D+06, 
     & -0.3875766023D+05, -0.7800876043D+04, 0.1474474677D+05, 
     & 0.1342639184D+05, 0.1366314809D+05, -0.8655623578D+05, 
     & -0.6967607523D+05, -0.4893850334D+05, -0.1638440371D+04, 
     & 0.1070849594D+05, 0.1954236781D+06, -0.1249119520D+05, 
     & -0.4234270006D+05, -0.4099084000D+05, 0.4644198605D+05, 
     & 0.8647625243D+05, -0.1429591982D+06, -0.1695858600D+06, 
     & 0.9655053307D+05, 0.1310954103D+06, -0.4551707893D+05, 
     & -0.1373100492D+06, 0.1362168681D+05, -0.2797867235D+06, 
     & 0.1661055490D+06, -0.3808688500D+05, 0.2824916636D+05, 
     & 0.5212300970D+06, 0.1753102494D+06, -0.3046113478D+06, 
     & 0.2066063506D+06, -0.2319522911D+06, 0.1976138682D+06, 
     & 0.1763537486D+04, -0.3862157111D+06, 0.6846526868D+05, 
     & 0.1371447223D+06, 0.2100006082D+05, 0.1828628482D+06, 
     & -0.2241902486D+05, -0.1335029011D+05, -0.2086373266D+06, 
     & -0.8294364915D+05, -0.2128144866D+06, 0.1155428685D+06, 
     & 0.2041503261D+06, -0.9581742473D+05, -0.5416987931D+05, 
     & -0.9602842199D+05, -0.2090605266D+06, -0.1076121997D+06, 
     & -0.1794670063D+05, 0.4325495409D+05, 0.1942642631D+06, 
     & -0.2532757746D+04, -0.6215928748D+05, 0.2260538690D+05, 
     & 0.1169447682D+05, 0.1557515896D+04, 0.3092730176D+04, 
     & -0.2130604198D+05, -0.2613900324D+05, -0.1052453916D+05, 
     & -0.4099582349D+04, 0.1719514743D+04, 0.3747515096D+04, 
     & 0.2814760565D+05, -0.2919155296D+05, -0.3340319995D+04, 
     & 0.3740209543D+05, 0.2538128525D+05, 0.1070849594D+05, 
     & -0.9847072621D+05, -0.1005546451D+05, -0.6112123778D+04, 
     & 0.5297977556D+05, -0.4459887680D+04, -0.1858742610D+05, 
     & -0.7971411987D+04, 0.2480432529D+05, 0.4077616826D+05, 
     & 0.2051935916D+05, -0.5473426048D+05, 0.8560288259D+05, 
     & -0.7763492950D+05, 0.4707256914D+05, -0.4199216273D+05, 
     & -0.3224983284D+05, -0.1120198817D+05, 0.4126751829D+05, 
     & 0.9412289800D+05, -0.1981168237D+05, 0.1817150300D+05, 
     & 0.5975551926D+04, 0.4285571140D+05, -0.2934770748D+04, 
     & -0.1870655068D+05, 0.3323662159D+05, -0.3078663879D+05, 
     & -0.4129220010D+05, 0.1434163863D+05, -0.1202263919D+05, 
     & -0.4524101413D+05, -0.3370443842D+05, -0.8722621491D+04, 
     & -0.1351245363D+05, -0.2268621454D+05, 0.1779946330D+05, 
     & 0.3713531200D+05, -0.7056907749D+04, 0.7476399748D+05, 
     & -0.2506302570D+05, 0.4721082626D+04, 0.4828824172D+04, 
     & 0.4029509574D+03, -0.1080458451D+05, -0.4538567568D+04, 
     & -0.2197945223D+04, 0.1742024152D+05, -0.7426937315D+04, 
     & 0.7687137399D+04, -0.1998106256D+05, 0.1397839385D+05, 
     & -0.4025478250D+04, 0.1753959535D+04, -0.2366119527D+03, 
     & -0.6421235754D+04, -0.4650671307D+04, 0.3357536676D+04, 
     & 0.1363195424D+05, 0.1002644904D+04, -0.5571799313D+04, 
     & -0.1121807037D+05, -0.9376544252D+03, 0.3268696623D+03, 
     & -0.1653841881D+04, 0.1698001746D+04, 0.4513430511D+04, 
     & -0.1003285011D+04, -0.2041858708D+03/ 
      terms(1)=coeff(1) 
      terms(2)=coeff(2) *dr(6) 
      terms(3)=coeff(3) *dr(6) *dr(6) 
      terms(4)=coeff(4) *dr(6) *dr(6) *dr(6) 
      terms(5)=coeff(5) *dr(6) *dr(6) *dr(6) *dr(6) 
      terms(6)=coeff(6) *dr(6) *dr(6) *dr(6) *dr(6) *dr(6) 
      terms(7)=coeff(7) *dr(6) *dr(6) *dr(6) *dr(6) *dr(6) *dr(6) 
      terms(8)=coeff(8) *dr(5) 
      terms(9)=coeff(9) *dr(5) *dr(6) 
      terms(10)=coeff(10) *dr(5) *dr(6) *dr(6) 
      terms(11)=coeff(11) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(12)=coeff(12) *dr(5) *dr(6) *dr(6) *dr(6) *dr(6) 
      terms(13)=coeff(13) *dr(5) *dr(6) *dr(6) *dr(6) *dr(6) *dr(6) 
      terms(14)=coeff(14) *dr(5) *dr(5) 
      terms(15)=coeff(15) *dr(5) *dr(5) *dr(6) 
      terms(16)=coeff(16) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(17)=coeff(17) *dr(5) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(18)=coeff(18) *dr(5) *dr(5) *dr(6) *dr(6) *dr(6) *dr(6) 
      terms(19)=coeff(19) *dr(5) *dr(5) *dr(5) 
      terms(20)=coeff(20) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(21)=coeff(21) *dr(5) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(22)=coeff(22) *dr(5) *dr(5) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(23)=coeff(23) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(24)=coeff(24) *dr(5) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(25)=coeff(25) *dr(5) *dr(5) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(26)=coeff(26) *dr(5) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(27)=coeff(27) *dr(5) *dr(5) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(28)=coeff(28) *dr(5) *dr(5) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(29)=coeff(29) *dr(4) 
      terms(30)=coeff(30) *dr(4) *dr(6) 
      terms(31)=coeff(31) *dr(4) *dr(6) *dr(6) 
      terms(32)=coeff(32) *dr(4) *dr(6) *dr(6) *dr(6) 
      terms(33)=coeff(33) *dr(4) *dr(6) *dr(6) *dr(6) *dr(6) 
      terms(34)=coeff(34) *dr(4) *dr(6) *dr(6) *dr(6) *dr(6) *dr(6) 
      terms(35)=coeff(35) *dr(4) *dr(5) 
      terms(36)=coeff(36) *dr(4) *dr(5) *dr(6) 
      terms(37)=coeff(37) *dr(4) *dr(5) *dr(6) *dr(6) 
      terms(38)=coeff(38) *dr(4) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(39)=coeff(39) *dr(4) *dr(5) *dr(6) *dr(6) *dr(6) *dr(6) 
      terms(40)=coeff(40) *dr(4) *dr(5) *dr(5) 
      terms(41)=coeff(41) *dr(4) *dr(5) *dr(5) *dr(6) 
      terms(42)=coeff(42) *dr(4) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(43)=coeff(43) *dr(4) *dr(5) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(44)=coeff(44) *dr(4) *dr(5) *dr(5) *dr(5) 
      terms(45)=coeff(45) *dr(4) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(46)=coeff(46) *dr(4) *dr(5) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(47)=coeff(47) *dr(4) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(48)=coeff(48) *dr(4) *dr(5) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(49)=coeff(49) *dr(4) *dr(5) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(50)=coeff(50) *dr(4) *dr(4) 
      terms(51)=coeff(51) *dr(4) *dr(4) *dr(6) 
      terms(52)=coeff(52) *dr(4) *dr(4) *dr(6) *dr(6) 
      terms(53)=coeff(53) *dr(4) *dr(4) *dr(6) *dr(6) *dr(6) 
      terms(54)=coeff(54) *dr(4) *dr(4) *dr(6) *dr(6) *dr(6) *dr(6) 
      terms(55)=coeff(55) *dr(4) *dr(4) *dr(5) 
      terms(56)=coeff(56) *dr(4) *dr(4) *dr(5) *dr(6) 
      terms(57)=coeff(57) *dr(4) *dr(4) *dr(5) *dr(6) *dr(6) 
      terms(58)=coeff(58) *dr(4) *dr(4) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(59)=coeff(59) *dr(4) *dr(4) *dr(5) *dr(5) 
      terms(60)=coeff(60) *dr(4) *dr(4) *dr(5) *dr(5) *dr(6) 
      terms(61)=coeff(61) *dr(4) *dr(4) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(62)=coeff(62) *dr(4) *dr(4) *dr(5) *dr(5) *dr(5) 
      terms(63)=coeff(63) *dr(4) *dr(4) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(64)=coeff(64) *dr(4) *dr(4) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(65)=coeff(65) *dr(4) *dr(4) *dr(4) 
      terms(66)=coeff(66) *dr(4) *dr(4) *dr(4) *dr(6) 
      terms(67)=coeff(67) *dr(4) *dr(4) *dr(4) *dr(6) *dr(6) 
      terms(68)=coeff(68) *dr(4) *dr(4) *dr(4) *dr(6) *dr(6) *dr(6) 
      terms(69)=coeff(69) *dr(4) *dr(4) *dr(4) *dr(5) 
      terms(70)=coeff(70) *dr(4) *dr(4) *dr(4) *dr(5) *dr(6) 
      terms(71)=coeff(71) *dr(4) *dr(4) *dr(4) *dr(5) *dr(6) *dr(6) 
      terms(72)=coeff(72) *dr(4) *dr(4) *dr(4) *dr(5) *dr(5) 
      terms(73)=coeff(73) *dr(4) *dr(4) *dr(4) *dr(5) *dr(5) *dr(6) 
      terms(74)=coeff(74) *dr(4) *dr(4) *dr(4) *dr(5) *dr(5) *dr(5) 
      terms(75)=coeff(75) *dr(4) *dr(4) *dr(4) *dr(4) 
      terms(76)=coeff(76) *dr(4) *dr(4) *dr(4) *dr(4) *dr(6) 
      terms(77)=coeff(77) *dr(4) *dr(4) *dr(4) *dr(4) *dr(6) *dr(6) 
      terms(78)=coeff(78) *dr(4) *dr(4) *dr(4) *dr(4) *dr(5) 
      terms(79)=coeff(79) *dr(4) *dr(4) *dr(4) *dr(4) *dr(5) *dr(6) 
      terms(80)=coeff(80) *dr(4) *dr(4) *dr(4) *dr(4) *dr(5) *dr(5) 
      terms(81)=coeff(81) *dr(4) *dr(4) *dr(4) *dr(4) *dr(4) 
      terms(82)=coeff(82) *dr(4) *dr(4) *dr(4) *dr(4) *dr(4) *dr(6) 
      terms(83)=coeff(83) *dr(4) *dr(4) *dr(4) *dr(4) *dr(4) *dr(5) 
      terms(84)=coeff(84) *dr(4) *dr(4) *dr(4) *dr(4) *dr(4) *dr(4) 
      terms(85)=coeff(85) *dr(3) 
      terms(86)=coeff(86) *dr(3) *dr(6) 
      terms(87)=coeff(87) *dr(3) *dr(6) *dr(6) 
      terms(88)=coeff(88) *dr(3) *dr(6) *dr(6) *dr(6) 
      terms(89)=coeff(89) *dr(3) *dr(6) *dr(6) *dr(6) *dr(6) 
      terms(90)=coeff(90) *dr(3) *dr(6) *dr(6) *dr(6) *dr(6) *dr(6) 
      terms(91)=coeff(91) *dr(3) *dr(5) 
      terms(92)=coeff(92) *dr(3) *dr(5) *dr(6) 
      terms(93)=coeff(93) *dr(3) *dr(5) *dr(6) *dr(6) 
      terms(94)=coeff(94) *dr(3) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(95)=coeff(95) *dr(3) *dr(5) *dr(6) *dr(6) *dr(6) *dr(6) 
      terms(96)=coeff(96) *dr(3) *dr(5) *dr(5) 
      terms(97)=coeff(97) *dr(3) *dr(5) *dr(5) *dr(6) 
      terms(98)=coeff(98) *dr(3) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(99)=coeff(99) *dr(3) *dr(5) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(100)=coeff(100) *dr(3) *dr(5) *dr(5) *dr(5) 
      terms(101)=coeff(101) *dr(3) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(102)=coeff(102) *dr(3) *dr(5) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(103)=coeff(103) *dr(3) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(104)=coeff(104) *dr(3) *dr(5) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(105)=coeff(105) *dr(3) *dr(5) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(106)=coeff(106) *dr(3) *dr(4) 
      terms(107)=coeff(107) *dr(3) *dr(4) *dr(6) 
      terms(108)=coeff(108) *dr(3) *dr(4) *dr(6) *dr(6) 
      terms(109)=coeff(109) *dr(3) *dr(4) *dr(6) *dr(6) *dr(6) 
      terms(110)=coeff(110) *dr(3) *dr(4) *dr(6) *dr(6) *dr(6) *dr(6) 
      terms(111)=coeff(111) *dr(3) *dr(4) *dr(5) 
      terms(112)=coeff(112) *dr(3) *dr(4) *dr(5) *dr(6) 
      terms(113)=coeff(113) *dr(3) *dr(4) *dr(5) *dr(6) *dr(6) 
      terms(114)=coeff(114) *dr(3) *dr(4) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(115)=coeff(115) *dr(3) *dr(4) *dr(5) *dr(5) 
      terms(116)=coeff(116) *dr(3) *dr(4) *dr(5) *dr(5) *dr(6) 
      terms(117)=coeff(117) *dr(3) *dr(4) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(118)=coeff(118) *dr(3) *dr(4) *dr(5) *dr(5) *dr(5) 
      terms(119)=coeff(119) *dr(3) *dr(4) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(120)=coeff(120) *dr(3) *dr(4) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(121)=coeff(121) *dr(3) *dr(4) *dr(4) 
      terms(122)=coeff(122) *dr(3) *dr(4) *dr(4) *dr(6) 
      terms(123)=coeff(123) *dr(3) *dr(4) *dr(4) *dr(6) *dr(6) 
      terms(124)=coeff(124) *dr(3) *dr(4) *dr(4) *dr(6) *dr(6) *dr(6) 
      terms(125)=coeff(125) *dr(3) *dr(4) *dr(4) *dr(5) 
      terms(126)=coeff(126) *dr(3) *dr(4) *dr(4) *dr(5) *dr(6) 
      terms(127)=coeff(127) *dr(3) *dr(4) *dr(4) *dr(5) *dr(6) *dr(6) 
      terms(128)=coeff(128) *dr(3) *dr(4) *dr(4) *dr(5) *dr(5) 
      terms(129)=coeff(129) *dr(3) *dr(4) *dr(4) *dr(5) *dr(5) *dr(6) 
      terms(130)=coeff(130) *dr(3) *dr(4) *dr(4) *dr(5) *dr(5) *dr(5) 
      terms(131)=coeff(131) *dr(3) *dr(4) *dr(4) *dr(4) 
      terms(132)=coeff(132) *dr(3) *dr(4) *dr(4) *dr(4) *dr(6) 
      terms(133)=coeff(133) *dr(3) *dr(4) *dr(4) *dr(4) *dr(6) *dr(6) 
      terms(134)=coeff(134) *dr(3) *dr(4) *dr(4) *dr(4) *dr(5) 
      terms(135)=coeff(135) *dr(3) *dr(4) *dr(4) *dr(4) *dr(5) *dr(6) 
      terms(136)=coeff(136) *dr(3) *dr(4) *dr(4) *dr(4) *dr(5) *dr(5) 
      terms(137)=coeff(137) *dr(3) *dr(4) *dr(4) *dr(4) *dr(4) 
      terms(138)=coeff(138) *dr(3) *dr(4) *dr(4) *dr(4) *dr(4) *dr(6) 
      terms(139)=coeff(139) *dr(3) *dr(4) *dr(4) *dr(4) *dr(4) *dr(5) 
      terms(140)=coeff(140) *dr(3) *dr(4) *dr(4) *dr(4) *dr(4) *dr(4) 
      terms(141)=coeff(141) *dr(3) *dr(3) 
      terms(142)=coeff(142) *dr(3) *dr(3) *dr(6) 
      terms(143)=coeff(143) *dr(3) *dr(3) *dr(6) *dr(6) 
      terms(144)=coeff(144) *dr(3) *dr(3) *dr(6) *dr(6) *dr(6) 
      terms(145)=coeff(145) *dr(3) *dr(3) *dr(6) *dr(6) *dr(6) *dr(6) 
      terms(146)=coeff(146) *dr(3) *dr(3) *dr(5) 
      terms(147)=coeff(147) *dr(3) *dr(3) *dr(5) *dr(6) 
      terms(148)=coeff(148) *dr(3) *dr(3) *dr(5) *dr(6) *dr(6) 
      terms(149)=coeff(149) *dr(3) *dr(3) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(150)=coeff(150) *dr(3) *dr(3) *dr(5) *dr(5) 
      terms(151)=coeff(151) *dr(3) *dr(3) *dr(5) *dr(5) *dr(6) 
      terms(152)=coeff(152) *dr(3) *dr(3) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(153)=coeff(153) *dr(3) *dr(3) *dr(5) *dr(5) *dr(5) 
      terms(154)=coeff(154) *dr(3) *dr(3) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(155)=coeff(155) *dr(3) *dr(3) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(156)=coeff(156) *dr(3) *dr(3) *dr(4) 
      terms(157)=coeff(157) *dr(3) *dr(3) *dr(4) *dr(6) 
      terms(158)=coeff(158) *dr(3) *dr(3) *dr(4) *dr(6) *dr(6) 
      terms(159)=coeff(159) *dr(3) *dr(3) *dr(4) *dr(6) *dr(6) *dr(6) 
      terms(160)=coeff(160) *dr(3) *dr(3) *dr(4) *dr(5) 
      terms(161)=coeff(161) *dr(3) *dr(3) *dr(4) *dr(5) *dr(6) 
      terms(162)=coeff(162) *dr(3) *dr(3) *dr(4) *dr(5) *dr(6) *dr(6) 
      terms(163)=coeff(163) *dr(3) *dr(3) *dr(4) *dr(5) *dr(5) 
      terms(164)=coeff(164) *dr(3) *dr(3) *dr(4) *dr(5) *dr(5) *dr(6) 
      terms(165)=coeff(165) *dr(3) *dr(3) *dr(4) *dr(5) *dr(5) *dr(5) 
      terms(166)=coeff(166) *dr(3) *dr(3) *dr(4) *dr(4) 
      terms(167)=coeff(167) *dr(3) *dr(3) *dr(4) *dr(4) *dr(6) 
      terms(168)=coeff(168) *dr(3) *dr(3) *dr(4) *dr(4) *dr(6) *dr(6) 
      terms(169)=coeff(169) *dr(3) *dr(3) *dr(4) *dr(4) *dr(5) 
      terms(170)=coeff(170) *dr(3) *dr(3) *dr(4) *dr(4) *dr(5) *dr(6) 
      terms(171)=coeff(171) *dr(3) *dr(3) *dr(4) *dr(4) *dr(5) *dr(5) 
      terms(172)=coeff(172) *dr(3) *dr(3) *dr(4) *dr(4) *dr(4) 
      terms(173)=coeff(173) *dr(3) *dr(3) *dr(4) *dr(4) *dr(4) *dr(6) 
      terms(174)=coeff(174) *dr(3) *dr(3) *dr(4) *dr(4) *dr(4) *dr(5) 
      terms(175)=coeff(175) *dr(3) *dr(3) *dr(4) *dr(4) *dr(4) *dr(4) 
      terms(176)=coeff(176) *dr(3) *dr(3) *dr(3) 
      terms(177)=coeff(177) *dr(3) *dr(3) *dr(3) *dr(6) 
      terms(178)=coeff(178) *dr(3) *dr(3) *dr(3) *dr(6) *dr(6) 
      terms(179)=coeff(179) *dr(3) *dr(3) *dr(3) *dr(6) *dr(6) *dr(6) 
      terms(180)=coeff(180) *dr(3) *dr(3) *dr(3) *dr(5) 
      terms(181)=coeff(181) *dr(3) *dr(3) *dr(3) *dr(5) *dr(6) 
      terms(182)=coeff(182) *dr(3) *dr(3) *dr(3) *dr(5) *dr(6) *dr(6) 
      terms(183)=coeff(183) *dr(3) *dr(3) *dr(3) *dr(5) *dr(5) 
      terms(184)=coeff(184) *dr(3) *dr(3) *dr(3) *dr(5) *dr(5) *dr(6) 
      terms(185)=coeff(185) *dr(3) *dr(3) *dr(3) *dr(5) *dr(5) *dr(5) 
      terms(186)=coeff(186) *dr(3) *dr(3) *dr(3) *dr(4) 
      terms(187)=coeff(187) *dr(3) *dr(3) *dr(3) *dr(4) *dr(6) 
      terms(188)=coeff(188) *dr(3) *dr(3) *dr(3) *dr(4) *dr(6) *dr(6) 
      terms(189)=coeff(189) *dr(3) *dr(3) *dr(3) *dr(4) *dr(5) 
      terms(190)=coeff(190) *dr(3) *dr(3) *dr(3) *dr(4) *dr(5) *dr(6) 
      terms(191)=coeff(191) *dr(3) *dr(3) *dr(3) *dr(4) *dr(5) *dr(5) 
      terms(192)=coeff(192) *dr(3) *dr(3) *dr(3) *dr(4) *dr(4) 
      terms(193)=coeff(193) *dr(3) *dr(3) *dr(3) *dr(4) *dr(4) *dr(6) 
      terms(194)=coeff(194) *dr(3) *dr(3) *dr(3) *dr(4) *dr(4) *dr(5) 
      terms(195)=coeff(195) *dr(3) *dr(3) *dr(3) *dr(4) *dr(4) *dr(4) 
      terms(196)=coeff(196) *dr(2) 
      terms(197)=coeff(197) *dr(2) *dr(6) 
      terms(198)=coeff(198) *dr(2) *dr(6) *dr(6) 
      terms(199)=coeff(199) *dr(2) *dr(6) *dr(6) *dr(6) 
      terms(200)=coeff(200) *dr(2) *dr(6) *dr(6) *dr(6) *dr(6) 
      terms(201)=coeff(201) *dr(2) *dr(6) *dr(6) *dr(6) *dr(6) *dr(6) 
      terms(202)=coeff(202) *dr(2) *dr(5) 
      terms(203)=coeff(203) *dr(2) *dr(5) *dr(6) 
      terms(204)=coeff(204) *dr(2) *dr(5) *dr(6) *dr(6) 
      terms(205)=coeff(205) *dr(2) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(206)=coeff(206) *dr(2) *dr(5) *dr(6) *dr(6) *dr(6) *dr(6) 
      terms(207)=coeff(207) *dr(2) *dr(5) *dr(5) 
      terms(208)=coeff(208) *dr(2) *dr(5) *dr(5) *dr(6) 
      terms(209)=coeff(209) *dr(2) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(210)=coeff(210) *dr(2) *dr(5) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(211)=coeff(211) *dr(2) *dr(5) *dr(5) *dr(5) 
      terms(212)=coeff(212) *dr(2) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(213)=coeff(213) *dr(2) *dr(5) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(214)=coeff(214) *dr(2) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(215)=coeff(215) *dr(2) *dr(5) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(216)=coeff(216) *dr(2) *dr(5) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(217)=coeff(217) *dr(2) *dr(4) 
      terms(218)=coeff(218) *dr(2) *dr(4) *dr(6) 
      terms(219)=coeff(219) *dr(2) *dr(4) *dr(6) *dr(6) 
      terms(220)=coeff(220) *dr(2) *dr(4) *dr(6) *dr(6) *dr(6) 
      terms(221)=coeff(221) *dr(2) *dr(4) *dr(6) *dr(6) *dr(6) *dr(6) 
      terms(222)=coeff(222) *dr(2) *dr(4) *dr(5) 
      terms(223)=coeff(223) *dr(2) *dr(4) *dr(5) *dr(6) 
      terms(224)=coeff(224) *dr(2) *dr(4) *dr(5) *dr(6) *dr(6) 
      terms(225)=coeff(225) *dr(2) *dr(4) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(226)=coeff(226) *dr(2) *dr(4) *dr(5) *dr(5) 
      terms(227)=coeff(227) *dr(2) *dr(4) *dr(5) *dr(5) *dr(6) 
      terms(228)=coeff(228) *dr(2) *dr(4) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(229)=coeff(229) *dr(2) *dr(4) *dr(5) *dr(5) *dr(5) 
      terms(230)=coeff(230) *dr(2) *dr(4) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(231)=coeff(231) *dr(2) *dr(4) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(232)=coeff(232) *dr(2) *dr(4) *dr(4) 
      terms(233)=coeff(233) *dr(2) *dr(4) *dr(4) *dr(6) 
      terms(234)=coeff(234) *dr(2) *dr(4) *dr(4) *dr(6) *dr(6) 
      terms(235)=coeff(235) *dr(2) *dr(4) *dr(4) *dr(6) *dr(6) *dr(6) 
      terms(236)=coeff(236) *dr(2) *dr(4) *dr(4) *dr(5) 
      terms(237)=coeff(237) *dr(2) *dr(4) *dr(4) *dr(5) *dr(6) 
      terms(238)=coeff(238) *dr(2) *dr(4) *dr(4) *dr(5) *dr(6) *dr(6) 
      terms(239)=coeff(239) *dr(2) *dr(4) *dr(4) *dr(5) *dr(5) 
      terms(240)=coeff(240) *dr(2) *dr(4) *dr(4) *dr(5) *dr(5) *dr(6) 
      terms(241)=coeff(241) *dr(2) *dr(4) *dr(4) *dr(5) *dr(5) *dr(5) 
      terms(242)=coeff(242) *dr(2) *dr(4) *dr(4) *dr(4) 
      terms(243)=coeff(243) *dr(2) *dr(4) *dr(4) *dr(4) *dr(6) 
      terms(244)=coeff(244) *dr(2) *dr(4) *dr(4) *dr(4) *dr(6) *dr(6) 
      terms(245)=coeff(245) *dr(2) *dr(4) *dr(4) *dr(4) *dr(5) 
      terms(246)=coeff(246) *dr(2) *dr(4) *dr(4) *dr(4) *dr(5) *dr(6) 
      terms(247)=coeff(247) *dr(2) *dr(4) *dr(4) *dr(4) *dr(5) *dr(5) 
      terms(248)=coeff(248) *dr(2) *dr(4) *dr(4) *dr(4) *dr(4) 
      terms(249)=coeff(249) *dr(2) *dr(4) *dr(4) *dr(4) *dr(4) *dr(6) 
      terms(250)=coeff(250) *dr(2) *dr(4) *dr(4) *dr(4) *dr(4) *dr(5) 
      terms(251)=coeff(251) *dr(2) *dr(4) *dr(4) *dr(4) *dr(4) *dr(4) 
      terms(252)=coeff(252) *dr(2) *dr(3) 
      terms(253)=coeff(253) *dr(2) *dr(3) *dr(6) 
      terms(254)=coeff(254) *dr(2) *dr(3) *dr(6) *dr(6) 
      terms(255)=coeff(255) *dr(2) *dr(3) *dr(6) *dr(6) *dr(6) 
      terms(256)=coeff(256) *dr(2) *dr(3) *dr(6) *dr(6) *dr(6) *dr(6) 
      terms(257)=coeff(257) *dr(2) *dr(3) *dr(5) 
      terms(258)=coeff(258) *dr(2) *dr(3) *dr(5) *dr(6) 
      terms(259)=coeff(259) *dr(2) *dr(3) *dr(5) *dr(6) *dr(6) 
      terms(260)=coeff(260) *dr(2) *dr(3) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(261)=coeff(261) *dr(2) *dr(3) *dr(5) *dr(5) 
      terms(262)=coeff(262) *dr(2) *dr(3) *dr(5) *dr(5) *dr(6) 
      terms(263)=coeff(263) *dr(2) *dr(3) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(264)=coeff(264) *dr(2) *dr(3) *dr(5) *dr(5) *dr(5) 
      terms(265)=coeff(265) *dr(2) *dr(3) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(266)=coeff(266) *dr(2) *dr(3) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(267)=coeff(267) *dr(2) *dr(3) *dr(4) 
      terms(268)=coeff(268) *dr(2) *dr(3) *dr(4) *dr(6) 
      terms(269)=coeff(269) *dr(2) *dr(3) *dr(4) *dr(6) *dr(6) 
      terms(270)=coeff(270) *dr(2) *dr(3) *dr(4) *dr(6) *dr(6) *dr(6) 
      terms(271)=coeff(271) *dr(2) *dr(3) *dr(4) *dr(5) 
      terms(272)=coeff(272) *dr(2) *dr(3) *dr(4) *dr(5) *dr(6) 
      terms(273)=coeff(273) *dr(2) *dr(3) *dr(4) *dr(5) *dr(6) *dr(6) 
      terms(274)=coeff(274) *dr(2) *dr(3) *dr(4) *dr(5) *dr(5) 
      terms(275)=coeff(275) *dr(2) *dr(3) *dr(4) *dr(5) *dr(5) *dr(6) 
      terms(276)=coeff(276) *dr(2) *dr(3) *dr(4) *dr(5) *dr(5) *dr(5) 
      terms(277)=coeff(277) *dr(2) *dr(3) *dr(4) *dr(4) 
      terms(278)=coeff(278) *dr(2) *dr(3) *dr(4) *dr(4) *dr(6) 
      terms(279)=coeff(279) *dr(2) *dr(3) *dr(4) *dr(4) *dr(6) *dr(6) 
      terms(280)=coeff(280) *dr(2) *dr(3) *dr(4) *dr(4) *dr(5) 
      terms(281)=coeff(281) *dr(2) *dr(3) *dr(4) *dr(4) *dr(5) *dr(6) 
      terms(282)=coeff(282) *dr(2) *dr(3) *dr(4) *dr(4) *dr(5) *dr(5) 
      terms(283)=coeff(283) *dr(2) *dr(3) *dr(4) *dr(4) *dr(4) 
      terms(284)=coeff(284) *dr(2) *dr(3) *dr(4) *dr(4) *dr(4) *dr(6) 
      terms(285)=coeff(285) *dr(2) *dr(3) *dr(4) *dr(4) *dr(4) *dr(5) 
      terms(286)=coeff(286) *dr(2) *dr(3) *dr(4) *dr(4) *dr(4) *dr(4) 
      terms(287)=coeff(287) *dr(2) *dr(3) *dr(3) 
      terms(288)=coeff(288) *dr(2) *dr(3) *dr(3) *dr(6) 
      terms(289)=coeff(289) *dr(2) *dr(3) *dr(3) *dr(6) *dr(6) 
      terms(290)=coeff(290) *dr(2) *dr(3) *dr(3) *dr(6) *dr(6) *dr(6) 
      terms(291)=coeff(291) *dr(2) *dr(3) *dr(3) *dr(5) 
      terms(292)=coeff(292) *dr(2) *dr(3) *dr(3) *dr(5) *dr(6) 
      terms(293)=coeff(293) *dr(2) *dr(3) *dr(3) *dr(5) *dr(6) *dr(6) 
      terms(294)=coeff(294) *dr(2) *dr(3) *dr(3) *dr(5) *dr(5) 
      terms(295)=coeff(295) *dr(2) *dr(3) *dr(3) *dr(5) *dr(5) *dr(6) 
      terms(296)=coeff(296) *dr(2) *dr(3) *dr(3) *dr(5) *dr(5) *dr(5) 
      terms(297)=coeff(297) *dr(2) *dr(3) *dr(3) *dr(4) 
      terms(298)=coeff(298) *dr(2) *dr(3) *dr(3) *dr(4) *dr(6) 
      terms(299)=coeff(299) *dr(2) *dr(3) *dr(3) *dr(4) *dr(6) *dr(6) 
      terms(300)=coeff(300) *dr(2) *dr(3) *dr(3) *dr(4) *dr(5) 
      terms(301)=coeff(301) *dr(2) *dr(3) *dr(3) *dr(4) *dr(5) *dr(6) 
      terms(302)=coeff(302) *dr(2) *dr(3) *dr(3) *dr(4) *dr(5) *dr(5) 
      terms(303)=coeff(303) *dr(2) *dr(3) *dr(3) *dr(4) *dr(4) 
      terms(304)=coeff(304) *dr(2) *dr(3) *dr(3) *dr(4) *dr(4) *dr(6) 
      terms(305)=coeff(305) *dr(2) *dr(3) *dr(3) *dr(4) *dr(4) *dr(5) 
      terms(306)=coeff(306) *dr(2) *dr(3) *dr(3) *dr(4) *dr(4) *dr(4) 
      terms(307)=coeff(307) *dr(2) *dr(3) *dr(3) *dr(3) 
      terms(308)=coeff(308) *dr(2) *dr(3) *dr(3) *dr(3) *dr(6) 
      terms(309)=coeff(309) *dr(2) *dr(3) *dr(3) *dr(3) *dr(6) *dr(6) 
      terms(310)=coeff(310) *dr(2) *dr(3) *dr(3) *dr(3) *dr(5) 
      terms(311)=coeff(311) *dr(2) *dr(3) *dr(3) *dr(3) *dr(5) *dr(6) 
      terms(312)=coeff(312) *dr(2) *dr(3) *dr(3) *dr(3) *dr(5) *dr(5) 
      terms(313)=coeff(313) *dr(2) *dr(3) *dr(3) *dr(3) *dr(4) 
      terms(314)=coeff(314) *dr(2) *dr(3) *dr(3) *dr(3) *dr(4) *dr(6) 
      terms(315)=coeff(315) *dr(2) *dr(3) *dr(3) *dr(3) *dr(4) *dr(5) 
      terms(316)=coeff(316) *dr(2) *dr(3) *dr(3) *dr(3) *dr(4) *dr(4) 
      terms(317)=coeff(317) *dr(2) *dr(2) 
      terms(318)=coeff(318) *dr(2) *dr(2) *dr(6) 
      terms(319)=coeff(319) *dr(2) *dr(2) *dr(6) *dr(6) 
      terms(320)=coeff(320) *dr(2) *dr(2) *dr(6) *dr(6) *dr(6) 
      terms(321)=coeff(321) *dr(2) *dr(2) *dr(6) *dr(6) *dr(6) *dr(6) 
      terms(322)=coeff(322) *dr(2) *dr(2) *dr(5) 
      terms(323)=coeff(323) *dr(2) *dr(2) *dr(5) *dr(6) 
      terms(324)=coeff(324) *dr(2) *dr(2) *dr(5) *dr(6) *dr(6) 
      terms(325)=coeff(325) *dr(2) *dr(2) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(326)=coeff(326) *dr(2) *dr(2) *dr(5) *dr(5) 
      terms(327)=coeff(327) *dr(2) *dr(2) *dr(5) *dr(5) *dr(6) 
      terms(328)=coeff(328) *dr(2) *dr(2) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(329)=coeff(329) *dr(2) *dr(2) *dr(5) *dr(5) *dr(5) 
      terms(330)=coeff(330) *dr(2) *dr(2) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(331)=coeff(331) *dr(2) *dr(2) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(332)=coeff(332) *dr(2) *dr(2) *dr(4) 
      terms(333)=coeff(333) *dr(2) *dr(2) *dr(4) *dr(6) 
      terms(334)=coeff(334) *dr(2) *dr(2) *dr(4) *dr(6) *dr(6) 
      terms(335)=coeff(335) *dr(2) *dr(2) *dr(4) *dr(6) *dr(6) *dr(6) 
      terms(336)=coeff(336) *dr(2) *dr(2) *dr(4) *dr(5) 
      terms(337)=coeff(337) *dr(2) *dr(2) *dr(4) *dr(5) *dr(6) 
      terms(338)=coeff(338) *dr(2) *dr(2) *dr(4) *dr(5) *dr(6) *dr(6) 
      terms(339)=coeff(339) *dr(2) *dr(2) *dr(4) *dr(5) *dr(5) 
      terms(340)=coeff(340) *dr(2) *dr(2) *dr(4) *dr(5) *dr(5) *dr(6) 
      terms(341)=coeff(341) *dr(2) *dr(2) *dr(4) *dr(5) *dr(5) *dr(5) 
      terms(342)=coeff(342) *dr(2) *dr(2) *dr(4) *dr(4) 
      terms(343)=coeff(343) *dr(2) *dr(2) *dr(4) *dr(4) *dr(6) 
      terms(344)=coeff(344) *dr(2) *dr(2) *dr(4) *dr(4) *dr(6) *dr(6) 
      terms(345)=coeff(345) *dr(2) *dr(2) *dr(4) *dr(4) *dr(5) 
      terms(346)=coeff(346) *dr(2) *dr(2) *dr(4) *dr(4) *dr(5) *dr(6) 
      terms(347)=coeff(347) *dr(2) *dr(2) *dr(4) *dr(4) *dr(5) *dr(5) 
      terms(348)=coeff(348) *dr(2) *dr(2) *dr(4) *dr(4) *dr(4) 
      terms(349)=coeff(349) *dr(2) *dr(2) *dr(4) *dr(4) *dr(4) *dr(6) 
      terms(350)=coeff(350) *dr(2) *dr(2) *dr(4) *dr(4) *dr(4) *dr(5) 
      terms(351)=coeff(351) *dr(2) *dr(2) *dr(4) *dr(4) *dr(4) *dr(4) 
      terms(352)=coeff(352) *dr(2) *dr(2) *dr(3) 
      terms(353)=coeff(353) *dr(2) *dr(2) *dr(3) *dr(6) 
      terms(354)=coeff(354) *dr(2) *dr(2) *dr(3) *dr(6) *dr(6) 
      terms(355)=coeff(355) *dr(2) *dr(2) *dr(3) *dr(6) *dr(6) *dr(6) 
      terms(356)=coeff(356) *dr(2) *dr(2) *dr(3) *dr(5) 
      terms(357)=coeff(357) *dr(2) *dr(2) *dr(3) *dr(5) *dr(6) 
      terms(358)=coeff(358) *dr(2) *dr(2) *dr(3) *dr(5) *dr(6) *dr(6) 
      terms(359)=coeff(359) *dr(2) *dr(2) *dr(3) *dr(5) *dr(5) 
      terms(360)=coeff(360) *dr(2) *dr(2) *dr(3) *dr(5) *dr(5) *dr(6) 
      terms(361)=coeff(361) *dr(2) *dr(2) *dr(3) *dr(5) *dr(5) *dr(5) 
      terms(362)=coeff(362) *dr(2) *dr(2) *dr(3) *dr(4) 
      terms(363)=coeff(363) *dr(2) *dr(2) *dr(3) *dr(4) *dr(6) 
      terms(364)=coeff(364) *dr(2) *dr(2) *dr(3) *dr(4) *dr(6) *dr(6) 
      terms(365)=coeff(365) *dr(2) *dr(2) *dr(3) *dr(4) *dr(5) 
      terms(366)=coeff(366) *dr(2) *dr(2) *dr(3) *dr(4) *dr(5) *dr(6) 
      terms(367)=coeff(367) *dr(2) *dr(2) *dr(3) *dr(4) *dr(5) *dr(5) 
      terms(368)=coeff(368) *dr(2) *dr(2) *dr(3) *dr(4) *dr(4) 
      terms(369)=coeff(369) *dr(2) *dr(2) *dr(3) *dr(4) *dr(4) *dr(6) 
      terms(370)=coeff(370) *dr(2) *dr(2) *dr(3) *dr(4) *dr(4) *dr(5) 
      terms(371)=coeff(371) *dr(2) *dr(2) *dr(3) *dr(4) *dr(4) *dr(4) 
      terms(372)=coeff(372) *dr(2) *dr(2) *dr(3) *dr(3) 
      terms(373)=coeff(373) *dr(2) *dr(2) *dr(3) *dr(3) *dr(6) 
      terms(374)=coeff(374) *dr(2) *dr(2) *dr(3) *dr(3) *dr(6) *dr(6) 
      terms(375)=coeff(375) *dr(2) *dr(2) *dr(3) *dr(3) *dr(5) 
      terms(376)=coeff(376) *dr(2) *dr(2) *dr(3) *dr(3) *dr(5) *dr(6) 
      terms(377)=coeff(377) *dr(2) *dr(2) *dr(3) *dr(3) *dr(5) *dr(5) 
      terms(378)=coeff(378) *dr(2) *dr(2) *dr(3) *dr(3) *dr(4) 
      terms(379)=coeff(379) *dr(2) *dr(2) *dr(3) *dr(3) *dr(4) *dr(6) 
      terms(380)=coeff(380) *dr(2) *dr(2) *dr(3) *dr(3) *dr(4) *dr(5) 
      terms(381)=coeff(381) *dr(2) *dr(2) *dr(3) *dr(3) *dr(4) *dr(4) 
      terms(382)=coeff(382) *dr(2) *dr(2) *dr(3) *dr(3) *dr(3) 
      terms(383)=coeff(383) *dr(2) *dr(2) *dr(3) *dr(3) *dr(3) *dr(6) 
      terms(384)=coeff(384) *dr(2) *dr(2) *dr(3) *dr(3) *dr(3) *dr(5) 
      terms(385)=coeff(385) *dr(2) *dr(2) *dr(3) *dr(3) *dr(3) *dr(4) 
      terms(386)=coeff(386) *dr(2) *dr(2) *dr(2) 
      terms(387)=coeff(387) *dr(2) *dr(2) *dr(2) *dr(6) 
      terms(388)=coeff(388) *dr(2) *dr(2) *dr(2) *dr(6) *dr(6) 
      terms(389)=coeff(389) *dr(2) *dr(2) *dr(2) *dr(6) *dr(6) *dr(6) 
      terms(390)=coeff(390) *dr(2) *dr(2) *dr(2) *dr(5) 
      terms(391)=coeff(391) *dr(2) *dr(2) *dr(2) *dr(5) *dr(6) 
      terms(392)=coeff(392) *dr(2) *dr(2) *dr(2) *dr(5) *dr(6) *dr(6) 
      terms(393)=coeff(393) *dr(2) *dr(2) *dr(2) *dr(5) *dr(5) 
      terms(394)=coeff(394) *dr(2) *dr(2) *dr(2) *dr(5) *dr(5) *dr(6) 
      terms(395)=coeff(395) *dr(2) *dr(2) *dr(2) *dr(5) *dr(5) *dr(5) 
      terms(396)=coeff(396) *dr(2) *dr(2) *dr(2) *dr(4) 
      terms(397)=coeff(397) *dr(2) *dr(2) *dr(2) *dr(4) *dr(6) 
      terms(398)=coeff(398) *dr(2) *dr(2) *dr(2) *dr(4) *dr(6) *dr(6) 
      terms(399)=coeff(399) *dr(2) *dr(2) *dr(2) *dr(4) *dr(5) 
      terms(400)=coeff(400) *dr(2) *dr(2) *dr(2) *dr(4) *dr(5) *dr(6) 
      terms(401)=coeff(401) *dr(2) *dr(2) *dr(2) *dr(4) *dr(5) *dr(5) 
      terms(402)=coeff(402) *dr(2) *dr(2) *dr(2) *dr(4) *dr(4) 
      terms(403)=coeff(403) *dr(2) *dr(2) *dr(2) *dr(4) *dr(4) *dr(6) 
      terms(404)=coeff(404) *dr(2) *dr(2) *dr(2) *dr(4) *dr(4) *dr(5) 
      terms(405)=coeff(405) *dr(2) *dr(2) *dr(2) *dr(4) *dr(4) *dr(4) 
      terms(406)=coeff(406) *dr(2) *dr(2) *dr(2) *dr(3) 
      terms(407)=coeff(407) *dr(2) *dr(2) *dr(2) *dr(3) *dr(6) 
      terms(408)=coeff(408) *dr(2) *dr(2) *dr(2) *dr(3) *dr(6) *dr(6) 
      terms(409)=coeff(409) *dr(2) *dr(2) *dr(2) *dr(3) *dr(5) 
      terms(410)=coeff(410) *dr(2) *dr(2) *dr(2) *dr(3) *dr(5) *dr(6) 
      terms(411)=coeff(411) *dr(2) *dr(2) *dr(2) *dr(3) *dr(5) *dr(5) 
      terms(412)=coeff(412) *dr(2) *dr(2) *dr(2) *dr(3) *dr(4) 
      terms(413)=coeff(413) *dr(2) *dr(2) *dr(2) *dr(3) *dr(4) *dr(6) 
      terms(414)=coeff(414) *dr(2) *dr(2) *dr(2) *dr(3) *dr(4) *dr(5) 
      terms(415)=coeff(415) *dr(2) *dr(2) *dr(2) *dr(3) *dr(4) *dr(4) 
      terms(416)=coeff(416) *dr(2) *dr(2) *dr(2) *dr(3) *dr(3) 
      terms(417)=coeff(417) *dr(2) *dr(2) *dr(2) *dr(3) *dr(3) *dr(6) 
      terms(418)=coeff(418) *dr(2) *dr(2) *dr(2) *dr(3) *dr(3) *dr(5) 
      terms(419)=coeff(419) *dr(2) *dr(2) *dr(2) *dr(3) *dr(3) *dr(4) 
      terms(420)=coeff(420) *dr(2) *dr(2) *dr(2) *dr(3) *dr(3) *dr(3) 
      terms(421)=coeff(421) *dr(2) *dr(2) *dr(2) *dr(2) 
      terms(422)=coeff(422) *dr(2) *dr(2) *dr(2) *dr(2) *dr(6) 
      terms(423)=coeff(423) *dr(2) *dr(2) *dr(2) *dr(2) *dr(6) *dr(6) 
      terms(424)=coeff(424) *dr(2) *dr(2) *dr(2) *dr(2) *dr(5) 
      terms(425)=coeff(425) *dr(2) *dr(2) *dr(2) *dr(2) *dr(5) *dr(6) 
      terms(426)=coeff(426) *dr(2) *dr(2) *dr(2) *dr(2) *dr(5) *dr(5) 
      terms(427)=coeff(427) *dr(2) *dr(2) *dr(2) *dr(2) *dr(4) 
      terms(428)=coeff(428) *dr(2) *dr(2) *dr(2) *dr(2) *dr(4) *dr(6) 
      terms(429)=coeff(429) *dr(2) *dr(2) *dr(2) *dr(2) *dr(4) *dr(5) 
      terms(430)=coeff(430) *dr(2) *dr(2) *dr(2) *dr(2) *dr(4) *dr(4) 
      terms(431)=coeff(431) *dr(2) *dr(2) *dr(2) *dr(2) *dr(3) 
      terms(432)=coeff(432) *dr(2) *dr(2) *dr(2) *dr(2) *dr(3) *dr(6) 
      terms(433)=coeff(433) *dr(2) *dr(2) *dr(2) *dr(2) *dr(3) *dr(5) 
      terms(434)=coeff(434) *dr(2) *dr(2) *dr(2) *dr(2) *dr(3) *dr(4) 
      terms(435)=coeff(435) *dr(2) *dr(2) *dr(2) *dr(2) *dr(3) *dr(3) 
      terms(436)=coeff(436) *dr(2) *dr(2) *dr(2) *dr(2) *dr(2) 
      terms(437)=coeff(437) *dr(2) *dr(2) *dr(2) *dr(2) *dr(2) *dr(6) 
      terms(438)=coeff(438) *dr(2) *dr(2) *dr(2) *dr(2) *dr(2) *dr(5) 
      terms(439)=coeff(439) *dr(2) *dr(2) *dr(2) *dr(2) *dr(2) *dr(4) 
      terms(440)=coeff(440) *dr(2) *dr(2) *dr(2) *dr(2) *dr(2) *dr(3) 
      terms(441)=coeff(441) *dr(2) *dr(2) *dr(2) *dr(2) *dr(2) *dr(2) 
      terms(442)=coeff(442) *dr(1) 
      terms(443)=coeff(443) *dr(1) *dr(6) 
      terms(444)=coeff(444) *dr(1) *dr(6) *dr(6) 
      terms(445)=coeff(445) *dr(1) *dr(6) *dr(6) *dr(6) 
      terms(446)=coeff(446) *dr(1) *dr(6) *dr(6) *dr(6) *dr(6) 
      terms(447)=coeff(447) *dr(1) *dr(6) *dr(6) *dr(6) *dr(6) *dr(6) 
      terms(448)=coeff(448) *dr(1) *dr(5) 
      terms(449)=coeff(449) *dr(1) *dr(5) *dr(6) 
      terms(450)=coeff(450) *dr(1) *dr(5) *dr(6) *dr(6) 
      terms(451)=coeff(451) *dr(1) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(452)=coeff(452) *dr(1) *dr(5) *dr(6) *dr(6) *dr(6) *dr(6) 
      terms(453)=coeff(453) *dr(1) *dr(5) *dr(5) 
      terms(454)=coeff(454) *dr(1) *dr(5) *dr(5) *dr(6) 
      terms(455)=coeff(455) *dr(1) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(456)=coeff(456) *dr(1) *dr(5) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(457)=coeff(457) *dr(1) *dr(5) *dr(5) *dr(5) 
      terms(458)=coeff(458) *dr(1) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(459)=coeff(459) *dr(1) *dr(5) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(460)=coeff(460) *dr(1) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(461)=coeff(461) *dr(1) *dr(5) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(462)=coeff(462) *dr(1) *dr(5) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(463)=coeff(463) *dr(1) *dr(4) 
      terms(464)=coeff(464) *dr(1) *dr(4) *dr(6) 
      terms(465)=coeff(465) *dr(1) *dr(4) *dr(6) *dr(6) 
      terms(466)=coeff(466) *dr(1) *dr(4) *dr(6) *dr(6) *dr(6) 
      terms(467)=coeff(467) *dr(1) *dr(4) *dr(6) *dr(6) *dr(6) *dr(6) 
      terms(468)=coeff(468) *dr(1) *dr(4) *dr(5) 
      terms(469)=coeff(469) *dr(1) *dr(4) *dr(5) *dr(6) 
      terms(470)=coeff(470) *dr(1) *dr(4) *dr(5) *dr(6) *dr(6) 
      terms(471)=coeff(471) *dr(1) *dr(4) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(472)=coeff(472) *dr(1) *dr(4) *dr(5) *dr(5) 
      terms(473)=coeff(473) *dr(1) *dr(4) *dr(5) *dr(5) *dr(6) 
      terms(474)=coeff(474) *dr(1) *dr(4) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(475)=coeff(475) *dr(1) *dr(4) *dr(5) *dr(5) *dr(5) 
      terms(476)=coeff(476) *dr(1) *dr(4) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(477)=coeff(477) *dr(1) *dr(4) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(478)=coeff(478) *dr(1) *dr(4) *dr(4) 
      terms(479)=coeff(479) *dr(1) *dr(4) *dr(4) *dr(6) 
      terms(480)=coeff(480) *dr(1) *dr(4) *dr(4) *dr(6) *dr(6) 
      terms(481)=coeff(481) *dr(1) *dr(4) *dr(4) *dr(6) *dr(6) *dr(6) 
      terms(482)=coeff(482) *dr(1) *dr(4) *dr(4) *dr(5) 
      terms(483)=coeff(483) *dr(1) *dr(4) *dr(4) *dr(5) *dr(6) 
      terms(484)=coeff(484) *dr(1) *dr(4) *dr(4) *dr(5) *dr(6) *dr(6) 
      terms(485)=coeff(485) *dr(1) *dr(4) *dr(4) *dr(5) *dr(5) 
      terms(486)=coeff(486) *dr(1) *dr(4) *dr(4) *dr(5) *dr(5) *dr(6) 
      terms(487)=coeff(487) *dr(1) *dr(4) *dr(4) *dr(5) *dr(5) *dr(5) 
      terms(488)=coeff(488) *dr(1) *dr(4) *dr(4) *dr(4) 
      terms(489)=coeff(489) *dr(1) *dr(4) *dr(4) *dr(4) *dr(6) 
      terms(490)=coeff(490) *dr(1) *dr(4) *dr(4) *dr(4) *dr(6) *dr(6) 
      terms(491)=coeff(491) *dr(1) *dr(4) *dr(4) *dr(4) *dr(5) 
      terms(492)=coeff(492) *dr(1) *dr(4) *dr(4) *dr(4) *dr(5) *dr(6) 
      terms(493)=coeff(493) *dr(1) *dr(4) *dr(4) *dr(4) *dr(5) *dr(5) 
      terms(494)=coeff(494) *dr(1) *dr(4) *dr(4) *dr(4) *dr(4) 
      terms(495)=coeff(495) *dr(1) *dr(4) *dr(4) *dr(4) *dr(4) *dr(6) 
      terms(496)=coeff(496) *dr(1) *dr(4) *dr(4) *dr(4) *dr(4) *dr(5) 
      terms(497)=coeff(497) *dr(1) *dr(4) *dr(4) *dr(4) *dr(4) *dr(4) 
      terms(498)=coeff(498) *dr(1) *dr(3) 
      terms(499)=coeff(499) *dr(1) *dr(3) *dr(6) 
      terms(500)=coeff(500) *dr(1) *dr(3) *dr(6) *dr(6) 
      terms(501)=coeff(501) *dr(1) *dr(3) *dr(6) *dr(6) *dr(6) 
      terms(502)=coeff(502) *dr(1) *dr(3) *dr(6) *dr(6) *dr(6) *dr(6) 
      terms(503)=coeff(503) *dr(1) *dr(3) *dr(5) 
      terms(504)=coeff(504) *dr(1) *dr(3) *dr(5) *dr(6) 
      terms(505)=coeff(505) *dr(1) *dr(3) *dr(5) *dr(6) *dr(6) 
      terms(506)=coeff(506) *dr(1) *dr(3) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(507)=coeff(507) *dr(1) *dr(3) *dr(5) *dr(5) 
      terms(508)=coeff(508) *dr(1) *dr(3) *dr(5) *dr(5) *dr(6) 
      terms(509)=coeff(509) *dr(1) *dr(3) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(510)=coeff(510) *dr(1) *dr(3) *dr(5) *dr(5) *dr(5) 
      terms(511)=coeff(511) *dr(1) *dr(3) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(512)=coeff(512) *dr(1) *dr(3) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(513)=coeff(513) *dr(1) *dr(3) *dr(4) 
      terms(514)=coeff(514) *dr(1) *dr(3) *dr(4) *dr(6) 
      terms(515)=coeff(515) *dr(1) *dr(3) *dr(4) *dr(6) *dr(6) 
      terms(516)=coeff(516) *dr(1) *dr(3) *dr(4) *dr(6) *dr(6) *dr(6) 
      terms(517)=coeff(517) *dr(1) *dr(3) *dr(4) *dr(5) 
      terms(518)=coeff(518) *dr(1) *dr(3) *dr(4) *dr(5) *dr(6) 
      terms(519)=coeff(519) *dr(1) *dr(3) *dr(4) *dr(5) *dr(6) *dr(6) 
      terms(520)=coeff(520) *dr(1) *dr(3) *dr(4) *dr(5) *dr(5) 
      terms(521)=coeff(521) *dr(1) *dr(3) *dr(4) *dr(5) *dr(5) *dr(6) 
      terms(522)=coeff(522) *dr(1) *dr(3) *dr(4) *dr(5) *dr(5) *dr(5) 
      terms(523)=coeff(523) *dr(1) *dr(3) *dr(4) *dr(4) 
      terms(524)=coeff(524) *dr(1) *dr(3) *dr(4) *dr(4) *dr(6) 
      terms(525)=coeff(525) *dr(1) *dr(3) *dr(4) *dr(4) *dr(6) *dr(6) 
      terms(526)=coeff(526) *dr(1) *dr(3) *dr(4) *dr(4) *dr(5) 
      terms(527)=coeff(527) *dr(1) *dr(3) *dr(4) *dr(4) *dr(5) *dr(6) 
      terms(528)=coeff(528) *dr(1) *dr(3) *dr(4) *dr(4) *dr(5) *dr(5) 
      terms(529)=coeff(529) *dr(1) *dr(3) *dr(4) *dr(4) *dr(4) 
      terms(530)=coeff(530) *dr(1) *dr(3) *dr(4) *dr(4) *dr(4) *dr(6) 
      terms(531)=coeff(531) *dr(1) *dr(3) *dr(4) *dr(4) *dr(4) *dr(5) 
      terms(532)=coeff(532) *dr(1) *dr(3) *dr(4) *dr(4) *dr(4) *dr(4) 
      terms(533)=coeff(533) *dr(1) *dr(3) *dr(3) 
      terms(534)=coeff(534) *dr(1) *dr(3) *dr(3) *dr(6) 
      terms(535)=coeff(535) *dr(1) *dr(3) *dr(3) *dr(6) *dr(6) 
      terms(536)=coeff(536) *dr(1) *dr(3) *dr(3) *dr(6) *dr(6) *dr(6) 
      terms(537)=coeff(537) *dr(1) *dr(3) *dr(3) *dr(5) 
      terms(538)=coeff(538) *dr(1) *dr(3) *dr(3) *dr(5) *dr(6) 
      terms(539)=coeff(539) *dr(1) *dr(3) *dr(3) *dr(5) *dr(6) *dr(6) 
      terms(540)=coeff(540) *dr(1) *dr(3) *dr(3) *dr(5) *dr(5) 
      terms(541)=coeff(541) *dr(1) *dr(3) *dr(3) *dr(5) *dr(5) *dr(6) 
      terms(542)=coeff(542) *dr(1) *dr(3) *dr(3) *dr(5) *dr(5) *dr(5) 
      terms(543)=coeff(543) *dr(1) *dr(3) *dr(3) *dr(4) 
      terms(544)=coeff(544) *dr(1) *dr(3) *dr(3) *dr(4) *dr(6) 
      terms(545)=coeff(545) *dr(1) *dr(3) *dr(3) *dr(4) *dr(6) *dr(6) 
      terms(546)=coeff(546) *dr(1) *dr(3) *dr(3) *dr(4) *dr(5) 
      terms(547)=coeff(547) *dr(1) *dr(3) *dr(3) *dr(4) *dr(5) *dr(6) 
      terms(548)=coeff(548) *dr(1) *dr(3) *dr(3) *dr(4) *dr(5) *dr(5) 
      terms(549)=coeff(549) *dr(1) *dr(3) *dr(3) *dr(4) *dr(4) 
      terms(550)=coeff(550) *dr(1) *dr(3) *dr(3) *dr(4) *dr(4) *dr(6) 
      terms(551)=coeff(551) *dr(1) *dr(3) *dr(3) *dr(4) *dr(4) *dr(5) 
      terms(552)=coeff(552) *dr(1) *dr(3) *dr(3) *dr(4) *dr(4) *dr(4) 
      terms(553)=coeff(553) *dr(1) *dr(3) *dr(3) *dr(3) 
      terms(554)=coeff(554) *dr(1) *dr(3) *dr(3) *dr(3) *dr(6) 
      terms(555)=coeff(555) *dr(1) *dr(3) *dr(3) *dr(3) *dr(6) *dr(6) 
      terms(556)=coeff(556) *dr(1) *dr(3) *dr(3) *dr(3) *dr(5) 
      terms(557)=coeff(557) *dr(1) *dr(3) *dr(3) *dr(3) *dr(5) *dr(6) 
      terms(558)=coeff(558) *dr(1) *dr(3) *dr(3) *dr(3) *dr(5) *dr(5) 
      terms(559)=coeff(559) *dr(1) *dr(3) *dr(3) *dr(3) *dr(4) 
      terms(560)=coeff(560) *dr(1) *dr(3) *dr(3) *dr(3) *dr(4) *dr(6) 
      terms(561)=coeff(561) *dr(1) *dr(3) *dr(3) *dr(3) *dr(4) *dr(5) 
      terms(562)=coeff(562) *dr(1) *dr(3) *dr(3) *dr(3) *dr(4) *dr(4) 
      terms(563)=coeff(563) *dr(1) *dr(2) 
      terms(564)=coeff(564) *dr(1) *dr(2) *dr(6) 
      terms(565)=coeff(565) *dr(1) *dr(2) *dr(6) *dr(6) 
      terms(566)=coeff(566) *dr(1) *dr(2) *dr(6) *dr(6) *dr(6) 
      terms(567)=coeff(567) *dr(1) *dr(2) *dr(6) *dr(6) *dr(6) *dr(6) 
      terms(568)=coeff(568) *dr(1) *dr(2) *dr(5) 
      terms(569)=coeff(569) *dr(1) *dr(2) *dr(5) *dr(6) 
      terms(570)=coeff(570) *dr(1) *dr(2) *dr(5) *dr(6) *dr(6) 
      terms(571)=coeff(571) *dr(1) *dr(2) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(572)=coeff(572) *dr(1) *dr(2) *dr(5) *dr(5) 
      terms(573)=coeff(573) *dr(1) *dr(2) *dr(5) *dr(5) *dr(6) 
      terms(574)=coeff(574) *dr(1) *dr(2) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(575)=coeff(575) *dr(1) *dr(2) *dr(5) *dr(5) *dr(5) 
      terms(576)=coeff(576) *dr(1) *dr(2) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(577)=coeff(577) *dr(1) *dr(2) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(578)=coeff(578) *dr(1) *dr(2) *dr(4) 
      terms(579)=coeff(579) *dr(1) *dr(2) *dr(4) *dr(6) 
      terms(580)=coeff(580) *dr(1) *dr(2) *dr(4) *dr(6) *dr(6) 
      terms(581)=coeff(581) *dr(1) *dr(2) *dr(4) *dr(6) *dr(6) *dr(6) 
      terms(582)=coeff(582) *dr(1) *dr(2) *dr(4) *dr(5) 
      terms(583)=coeff(583) *dr(1) *dr(2) *dr(4) *dr(5) *dr(6) 
      terms(584)=coeff(584) *dr(1) *dr(2) *dr(4) *dr(5) *dr(6) *dr(6) 
      terms(585)=coeff(585) *dr(1) *dr(2) *dr(4) *dr(5) *dr(5) 
      terms(586)=coeff(586) *dr(1) *dr(2) *dr(4) *dr(5) *dr(5) *dr(6) 
      terms(587)=coeff(587) *dr(1) *dr(2) *dr(4) *dr(5) *dr(5) *dr(5) 
      terms(588)=coeff(588) *dr(1) *dr(2) *dr(4) *dr(4) 
      terms(589)=coeff(589) *dr(1) *dr(2) *dr(4) *dr(4) *dr(6) 
      terms(590)=coeff(590) *dr(1) *dr(2) *dr(4) *dr(4) *dr(6) *dr(6) 
      terms(591)=coeff(591) *dr(1) *dr(2) *dr(4) *dr(4) *dr(5) 
      terms(592)=coeff(592) *dr(1) *dr(2) *dr(4) *dr(4) *dr(5) *dr(6) 
      terms(593)=coeff(593) *dr(1) *dr(2) *dr(4) *dr(4) *dr(5) *dr(5) 
      terms(594)=coeff(594) *dr(1) *dr(2) *dr(4) *dr(4) *dr(4) 
      terms(595)=coeff(595) *dr(1) *dr(2) *dr(4) *dr(4) *dr(4) *dr(6) 
      terms(596)=coeff(596) *dr(1) *dr(2) *dr(4) *dr(4) *dr(4) *dr(5) 
      terms(597)=coeff(597) *dr(1) *dr(2) *dr(4) *dr(4) *dr(4) *dr(4) 
      terms(598)=coeff(598) *dr(1) *dr(2) *dr(3) 
      terms(599)=coeff(599) *dr(1) *dr(2) *dr(3) *dr(6) 
      terms(600)=coeff(600) *dr(1) *dr(2) *dr(3) *dr(6) *dr(6) 
      terms(601)=coeff(601) *dr(1) *dr(2) *dr(3) *dr(6) *dr(6) *dr(6) 
      terms(602)=coeff(602) *dr(1) *dr(2) *dr(3) *dr(5) 
      terms(603)=coeff(603) *dr(1) *dr(2) *dr(3) *dr(5) *dr(6) 
      terms(604)=coeff(604) *dr(1) *dr(2) *dr(3) *dr(5) *dr(6) *dr(6) 
      terms(605)=coeff(605) *dr(1) *dr(2) *dr(3) *dr(5) *dr(5) 
      terms(606)=coeff(606) *dr(1) *dr(2) *dr(3) *dr(5) *dr(5) *dr(6) 
      terms(607)=coeff(607) *dr(1) *dr(2) *dr(3) *dr(5) *dr(5) *dr(5) 
      terms(608)=coeff(608) *dr(1) *dr(2) *dr(3) *dr(4) 
      terms(609)=coeff(609) *dr(1) *dr(2) *dr(3) *dr(4) *dr(6) 
      terms(610)=coeff(610) *dr(1) *dr(2) *dr(3) *dr(4) *dr(6) *dr(6) 
      terms(611)=coeff(611) *dr(1) *dr(2) *dr(3) *dr(4) *dr(5) 
      terms(612)=coeff(612) *dr(1) *dr(2) *dr(3) *dr(4) *dr(5) *dr(6) 
      terms(613)=coeff(613) *dr(1) *dr(2) *dr(3) *dr(4) *dr(5) *dr(5) 
      terms(614)=coeff(614) *dr(1) *dr(2) *dr(3) *dr(4) *dr(4) 
      terms(615)=coeff(615) *dr(1) *dr(2) *dr(3) *dr(4) *dr(4) *dr(6) 
      terms(616)=coeff(616) *dr(1) *dr(2) *dr(3) *dr(4) *dr(4) *dr(5) 
      terms(617)=coeff(617) *dr(1) *dr(2) *dr(3) *dr(4) *dr(4) *dr(4) 
      terms(618)=coeff(618) *dr(1) *dr(2) *dr(3) *dr(3) 
      terms(619)=coeff(619) *dr(1) *dr(2) *dr(3) *dr(3) *dr(6) 
      terms(620)=coeff(620) *dr(1) *dr(2) *dr(3) *dr(3) *dr(6) *dr(6) 
      terms(621)=coeff(621) *dr(1) *dr(2) *dr(3) *dr(3) *dr(5) 
      terms(622)=coeff(622) *dr(1) *dr(2) *dr(3) *dr(3) *dr(5) *dr(6) 
      terms(623)=coeff(623) *dr(1) *dr(2) *dr(3) *dr(3) *dr(5) *dr(5) 
      terms(624)=coeff(624) *dr(1) *dr(2) *dr(3) *dr(3) *dr(4) 
      terms(625)=coeff(625) *dr(1) *dr(2) *dr(3) *dr(3) *dr(4) *dr(6) 
      terms(626)=coeff(626) *dr(1) *dr(2) *dr(3) *dr(3) *dr(4) *dr(5) 
      terms(627)=coeff(627) *dr(1) *dr(2) *dr(3) *dr(3) *dr(4) *dr(4) 
      terms(628)=coeff(628) *dr(1) *dr(2) *dr(3) *dr(3) *dr(3) 
      terms(629)=coeff(629) *dr(1) *dr(2) *dr(3) *dr(3) *dr(3) *dr(6) 
      terms(630)=coeff(630) *dr(1) *dr(2) *dr(3) *dr(3) *dr(3) *dr(5) 
      terms(631)=coeff(631) *dr(1) *dr(2) *dr(3) *dr(3) *dr(3) *dr(4) 
      terms(632)=coeff(632) *dr(1) *dr(2) *dr(2) 
      terms(633)=coeff(633) *dr(1) *dr(2) *dr(2) *dr(6) 
      terms(634)=coeff(634) *dr(1) *dr(2) *dr(2) *dr(6) *dr(6) 
      terms(635)=coeff(635) *dr(1) *dr(2) *dr(2) *dr(6) *dr(6) *dr(6) 
      terms(636)=coeff(636) *dr(1) *dr(2) *dr(2) *dr(5) 
      terms(637)=coeff(637) *dr(1) *dr(2) *dr(2) *dr(5) *dr(6) 
      terms(638)=coeff(638) *dr(1) *dr(2) *dr(2) *dr(5) *dr(6) *dr(6) 
      terms(639)=coeff(639) *dr(1) *dr(2) *dr(2) *dr(5) *dr(5) 
      terms(640)=coeff(640) *dr(1) *dr(2) *dr(2) *dr(5) *dr(5) *dr(6) 
      terms(641)=coeff(641) *dr(1) *dr(2) *dr(2) *dr(5) *dr(5) *dr(5) 
      terms(642)=coeff(642) *dr(1) *dr(2) *dr(2) *dr(4) 
      terms(643)=coeff(643) *dr(1) *dr(2) *dr(2) *dr(4) *dr(6) 
      terms(644)=coeff(644) *dr(1) *dr(2) *dr(2) *dr(4) *dr(6) *dr(6) 
      terms(645)=coeff(645) *dr(1) *dr(2) *dr(2) *dr(4) *dr(5) 
      terms(646)=coeff(646) *dr(1) *dr(2) *dr(2) *dr(4) *dr(5) *dr(6) 
      terms(647)=coeff(647) *dr(1) *dr(2) *dr(2) *dr(4) *dr(5) *dr(5) 
      terms(648)=coeff(648) *dr(1) *dr(2) *dr(2) *dr(4) *dr(4) 
      terms(649)=coeff(649) *dr(1) *dr(2) *dr(2) *dr(4) *dr(4) *dr(6) 
      terms(650)=coeff(650) *dr(1) *dr(2) *dr(2) *dr(4) *dr(4) *dr(5) 
      terms(651)=coeff(651) *dr(1) *dr(2) *dr(2) *dr(4) *dr(4) *dr(4) 
      terms(652)=coeff(652) *dr(1) *dr(2) *dr(2) *dr(3) 
      terms(653)=coeff(653) *dr(1) *dr(2) *dr(2) *dr(3) *dr(6) 
      terms(654)=coeff(654) *dr(1) *dr(2) *dr(2) *dr(3) *dr(6) *dr(6) 
      terms(655)=coeff(655) *dr(1) *dr(2) *dr(2) *dr(3) *dr(5) 
      terms(656)=coeff(656) *dr(1) *dr(2) *dr(2) *dr(3) *dr(5) *dr(6) 
      terms(657)=coeff(657) *dr(1) *dr(2) *dr(2) *dr(3) *dr(5) *dr(5) 
      terms(658)=coeff(658) *dr(1) *dr(2) *dr(2) *dr(3) *dr(4) 
      terms(659)=coeff(659) *dr(1) *dr(2) *dr(2) *dr(3) *dr(4) *dr(6) 
      terms(660)=coeff(660) *dr(1) *dr(2) *dr(2) *dr(3) *dr(4) *dr(5) 
      terms(661)=coeff(661) *dr(1) *dr(2) *dr(2) *dr(3) *dr(4) *dr(4) 
      terms(662)=coeff(662) *dr(1) *dr(2) *dr(2) *dr(3) *dr(3) 
      terms(663)=coeff(663) *dr(1) *dr(2) *dr(2) *dr(3) *dr(3) *dr(6) 
      terms(664)=coeff(664) *dr(1) *dr(2) *dr(2) *dr(3) *dr(3) *dr(5) 
      terms(665)=coeff(665) *dr(1) *dr(2) *dr(2) *dr(3) *dr(3) *dr(4) 
      terms(666)=coeff(666) *dr(1) *dr(2) *dr(2) *dr(3) *dr(3) *dr(3) 
      terms(667)=coeff(667) *dr(1) *dr(2) *dr(2) *dr(2) 
      terms(668)=coeff(668) *dr(1) *dr(2) *dr(2) *dr(2) *dr(6) 
      terms(669)=coeff(669) *dr(1) *dr(2) *dr(2) *dr(2) *dr(6) *dr(6) 
      terms(670)=coeff(670) *dr(1) *dr(2) *dr(2) *dr(2) *dr(5) 
      terms(671)=coeff(671) *dr(1) *dr(2) *dr(2) *dr(2) *dr(5) *dr(6) 
      terms(672)=coeff(672) *dr(1) *dr(2) *dr(2) *dr(2) *dr(5) *dr(5) 
      terms(673)=coeff(673) *dr(1) *dr(2) *dr(2) *dr(2) *dr(4) 
      terms(674)=coeff(674) *dr(1) *dr(2) *dr(2) *dr(2) *dr(4) *dr(6) 
      terms(675)=coeff(675) *dr(1) *dr(2) *dr(2) *dr(2) *dr(4) *dr(5) 
      terms(676)=coeff(676) *dr(1) *dr(2) *dr(2) *dr(2) *dr(4) *dr(4) 
      terms(677)=coeff(677) *dr(1) *dr(2) *dr(2) *dr(2) *dr(3) 
      terms(678)=coeff(678) *dr(1) *dr(2) *dr(2) *dr(2) *dr(3) *dr(6) 
      terms(679)=coeff(679) *dr(1) *dr(2) *dr(2) *dr(2) *dr(3) *dr(5) 
      terms(680)=coeff(680) *dr(1) *dr(2) *dr(2) *dr(2) *dr(3) *dr(4) 
      terms(681)=coeff(681) *dr(1) *dr(2) *dr(2) *dr(2) *dr(3) *dr(3) 
      terms(682)=coeff(682) *dr(1) *dr(2) *dr(2) *dr(2) *dr(2) 
      terms(683)=coeff(683) *dr(1) *dr(2) *dr(2) *dr(2) *dr(2) *dr(6) 
      terms(684)=coeff(684) *dr(1) *dr(2) *dr(2) *dr(2) *dr(2) *dr(5) 
      terms(685)=coeff(685) *dr(1) *dr(2) *dr(2) *dr(2) *dr(2) *dr(4) 
      terms(686)=coeff(686) *dr(1) *dr(2) *dr(2) *dr(2) *dr(2) *dr(3) 
      terms(687)=coeff(687) *dr(1) *dr(2) *dr(2) *dr(2) *dr(2) *dr(2) 
      terms(688)=coeff(688) *dr(1) *dr(1) 
      terms(689)=coeff(689) *dr(1) *dr(1) *dr(6) 
      terms(690)=coeff(690) *dr(1) *dr(1) *dr(6) *dr(6) 
      terms(691)=coeff(691) *dr(1) *dr(1) *dr(6) *dr(6) *dr(6) 
      terms(692)=coeff(692) *dr(1) *dr(1) *dr(6) *dr(6) *dr(6) *dr(6) 
      terms(693)=coeff(693) *dr(1) *dr(1) *dr(5) 
      terms(694)=coeff(694) *dr(1) *dr(1) *dr(5) *dr(6) 
      terms(695)=coeff(695) *dr(1) *dr(1) *dr(5) *dr(6) *dr(6) 
      terms(696)=coeff(696) *dr(1) *dr(1) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(697)=coeff(697) *dr(1) *dr(1) *dr(5) *dr(5) 
      terms(698)=coeff(698) *dr(1) *dr(1) *dr(5) *dr(5) *dr(6) 
      terms(699)=coeff(699) *dr(1) *dr(1) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(700)=coeff(700) *dr(1) *dr(1) *dr(5) *dr(5) *dr(5) 
      terms(701)=coeff(701) *dr(1) *dr(1) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(702)=coeff(702) *dr(1) *dr(1) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(703)=coeff(703) *dr(1) *dr(1) *dr(4) 
      terms(704)=coeff(704) *dr(1) *dr(1) *dr(4) *dr(6) 
      terms(705)=coeff(705) *dr(1) *dr(1) *dr(4) *dr(6) *dr(6) 
      terms(706)=coeff(706) *dr(1) *dr(1) *dr(4) *dr(6) *dr(6) *dr(6) 
      terms(707)=coeff(707) *dr(1) *dr(1) *dr(4) *dr(5) 
      terms(708)=coeff(708) *dr(1) *dr(1) *dr(4) *dr(5) *dr(6) 
      terms(709)=coeff(709) *dr(1) *dr(1) *dr(4) *dr(5) *dr(6) *dr(6) 
      terms(710)=coeff(710) *dr(1) *dr(1) *dr(4) *dr(5) *dr(5) 
      terms(711)=coeff(711) *dr(1) *dr(1) *dr(4) *dr(5) *dr(5) *dr(6) 
      terms(712)=coeff(712) *dr(1) *dr(1) *dr(4) *dr(5) *dr(5) *dr(5) 
      terms(713)=coeff(713) *dr(1) *dr(1) *dr(4) *dr(4) 
      terms(714)=coeff(714) *dr(1) *dr(1) *dr(4) *dr(4) *dr(6) 
      terms(715)=coeff(715) *dr(1) *dr(1) *dr(4) *dr(4) *dr(6) *dr(6) 
      terms(716)=coeff(716) *dr(1) *dr(1) *dr(4) *dr(4) *dr(5) 
      terms(717)=coeff(717) *dr(1) *dr(1) *dr(4) *dr(4) *dr(5) *dr(6) 
      terms(718)=coeff(718) *dr(1) *dr(1) *dr(4) *dr(4) *dr(5) *dr(5) 
      terms(719)=coeff(719) *dr(1) *dr(1) *dr(4) *dr(4) *dr(4) 
      terms(720)=coeff(720) *dr(1) *dr(1) *dr(4) *dr(4) *dr(4) *dr(6) 
      terms(721)=coeff(721) *dr(1) *dr(1) *dr(4) *dr(4) *dr(4) *dr(5) 
      terms(722)=coeff(722) *dr(1) *dr(1) *dr(4) *dr(4) *dr(4) *dr(4) 
      terms(723)=coeff(723) *dr(1) *dr(1) *dr(3) 
      terms(724)=coeff(724) *dr(1) *dr(1) *dr(3) *dr(6) 
      terms(725)=coeff(725) *dr(1) *dr(1) *dr(3) *dr(6) *dr(6) 
      terms(726)=coeff(726) *dr(1) *dr(1) *dr(3) *dr(6) *dr(6) *dr(6) 
      terms(727)=coeff(727) *dr(1) *dr(1) *dr(3) *dr(5) 
      terms(728)=coeff(728) *dr(1) *dr(1) *dr(3) *dr(5) *dr(6) 
      terms(729)=coeff(729) *dr(1) *dr(1) *dr(3) *dr(5) *dr(6) *dr(6) 
      terms(730)=coeff(730) *dr(1) *dr(1) *dr(3) *dr(5) *dr(5) 
      terms(731)=coeff(731) *dr(1) *dr(1) *dr(3) *dr(5) *dr(5) *dr(6) 
      terms(732)=coeff(732) *dr(1) *dr(1) *dr(3) *dr(5) *dr(5) *dr(5) 
      terms(733)=coeff(733) *dr(1) *dr(1) *dr(3) *dr(4) 
      terms(734)=coeff(734) *dr(1) *dr(1) *dr(3) *dr(4) *dr(6) 
      terms(735)=coeff(735) *dr(1) *dr(1) *dr(3) *dr(4) *dr(6) *dr(6) 
      terms(736)=coeff(736) *dr(1) *dr(1) *dr(3) *dr(4) *dr(5) 
      terms(737)=coeff(737) *dr(1) *dr(1) *dr(3) *dr(4) *dr(5) *dr(6) 
      terms(738)=coeff(738) *dr(1) *dr(1) *dr(3) *dr(4) *dr(5) *dr(5) 
      terms(739)=coeff(739) *dr(1) *dr(1) *dr(3) *dr(4) *dr(4) 
      terms(740)=coeff(740) *dr(1) *dr(1) *dr(3) *dr(4) *dr(4) *dr(6) 
      terms(741)=coeff(741) *dr(1) *dr(1) *dr(3) *dr(4) *dr(4) *dr(5) 
      terms(742)=coeff(742) *dr(1) *dr(1) *dr(3) *dr(4) *dr(4) *dr(4) 
      terms(743)=coeff(743) *dr(1) *dr(1) *dr(3) *dr(3) 
      terms(744)=coeff(744) *dr(1) *dr(1) *dr(3) *dr(3) *dr(6) 
      terms(745)=coeff(745) *dr(1) *dr(1) *dr(3) *dr(3) *dr(6) *dr(6) 
      terms(746)=coeff(746) *dr(1) *dr(1) *dr(3) *dr(3) *dr(5) 
      terms(747)=coeff(747) *dr(1) *dr(1) *dr(3) *dr(3) *dr(5) *dr(6) 
      terms(748)=coeff(748) *dr(1) *dr(1) *dr(3) *dr(3) *dr(5) *dr(5) 
      terms(749)=coeff(749) *dr(1) *dr(1) *dr(3) *dr(3) *dr(4) 
      terms(750)=coeff(750) *dr(1) *dr(1) *dr(3) *dr(3) *dr(4) *dr(6) 
      terms(751)=coeff(751) *dr(1) *dr(1) *dr(3) *dr(3) *dr(4) *dr(5) 
      terms(752)=coeff(752) *dr(1) *dr(1) *dr(3) *dr(3) *dr(4) *dr(4) 
      terms(753)=coeff(753) *dr(1) *dr(1) *dr(3) *dr(3) *dr(3) 
      terms(754)=coeff(754) *dr(1) *dr(1) *dr(3) *dr(3) *dr(3) *dr(6) 
      terms(755)=coeff(755) *dr(1) *dr(1) *dr(3) *dr(3) *dr(3) *dr(5) 
      terms(756)=coeff(756) *dr(1) *dr(1) *dr(3) *dr(3) *dr(3) *dr(4) 
      terms(757)=coeff(757) *dr(1) *dr(1) *dr(2) 
      terms(758)=coeff(758) *dr(1) *dr(1) *dr(2) *dr(6) 
      terms(759)=coeff(759) *dr(1) *dr(1) *dr(2) *dr(6) *dr(6) 
      terms(760)=coeff(760) *dr(1) *dr(1) *dr(2) *dr(6) *dr(6) *dr(6) 
      terms(761)=coeff(761) *dr(1) *dr(1) *dr(2) *dr(5) 
      terms(762)=coeff(762) *dr(1) *dr(1) *dr(2) *dr(5) *dr(6) 
      terms(763)=coeff(763) *dr(1) *dr(1) *dr(2) *dr(5) *dr(6) *dr(6) 
      terms(764)=coeff(764) *dr(1) *dr(1) *dr(2) *dr(5) *dr(5) 
      terms(765)=coeff(765) *dr(1) *dr(1) *dr(2) *dr(5) *dr(5) *dr(6) 
      terms(766)=coeff(766) *dr(1) *dr(1) *dr(2) *dr(5) *dr(5) *dr(5) 
      terms(767)=coeff(767) *dr(1) *dr(1) *dr(2) *dr(4) 
      terms(768)=coeff(768) *dr(1) *dr(1) *dr(2) *dr(4) *dr(6) 
      terms(769)=coeff(769) *dr(1) *dr(1) *dr(2) *dr(4) *dr(6) *dr(6) 
      terms(770)=coeff(770) *dr(1) *dr(1) *dr(2) *dr(4) *dr(5) 
      terms(771)=coeff(771) *dr(1) *dr(1) *dr(2) *dr(4) *dr(5) *dr(6) 
      terms(772)=coeff(772) *dr(1) *dr(1) *dr(2) *dr(4) *dr(5) *dr(5) 
      terms(773)=coeff(773) *dr(1) *dr(1) *dr(2) *dr(4) *dr(4) 
      terms(774)=coeff(774) *dr(1) *dr(1) *dr(2) *dr(4) *dr(4) *dr(6) 
      terms(775)=coeff(775) *dr(1) *dr(1) *dr(2) *dr(4) *dr(4) *dr(5) 
      terms(776)=coeff(776) *dr(1) *dr(1) *dr(2) *dr(4) *dr(4) *dr(4) 
      terms(777)=coeff(777) *dr(1) *dr(1) *dr(2) *dr(3) 
      terms(778)=coeff(778) *dr(1) *dr(1) *dr(2) *dr(3) *dr(6) 
      terms(779)=coeff(779) *dr(1) *dr(1) *dr(2) *dr(3) *dr(6) *dr(6) 
      terms(780)=coeff(780) *dr(1) *dr(1) *dr(2) *dr(3) *dr(5) 
      terms(781)=coeff(781) *dr(1) *dr(1) *dr(2) *dr(3) *dr(5) *dr(6) 
      terms(782)=coeff(782) *dr(1) *dr(1) *dr(2) *dr(3) *dr(5) *dr(5) 
      terms(783)=coeff(783) *dr(1) *dr(1) *dr(2) *dr(3) *dr(4) 
      terms(784)=coeff(784) *dr(1) *dr(1) *dr(2) *dr(3) *dr(4) *dr(6) 
      terms(785)=coeff(785) *dr(1) *dr(1) *dr(2) *dr(3) *dr(4) *dr(5) 
      terms(786)=coeff(786) *dr(1) *dr(1) *dr(2) *dr(3) *dr(4) *dr(4) 
      terms(787)=coeff(787) *dr(1) *dr(1) *dr(2) *dr(3) *dr(3) 
      terms(788)=coeff(788) *dr(1) *dr(1) *dr(2) *dr(3) *dr(3) *dr(6) 
      terms(789)=coeff(789) *dr(1) *dr(1) *dr(2) *dr(3) *dr(3) *dr(5) 
      terms(790)=coeff(790) *dr(1) *dr(1) *dr(2) *dr(3) *dr(3) *dr(4) 
      terms(791)=coeff(791) *dr(1) *dr(1) *dr(2) *dr(3) *dr(3) *dr(3) 
      terms(792)=coeff(792) *dr(1) *dr(1) *dr(2) *dr(2) 
      terms(793)=coeff(793) *dr(1) *dr(1) *dr(2) *dr(2) *dr(6) 
      terms(794)=coeff(794) *dr(1) *dr(1) *dr(2) *dr(2) *dr(6) *dr(6) 
      terms(795)=coeff(795) *dr(1) *dr(1) *dr(2) *dr(2) *dr(5) 
      terms(796)=coeff(796) *dr(1) *dr(1) *dr(2) *dr(2) *dr(5) *dr(6) 
      terms(797)=coeff(797) *dr(1) *dr(1) *dr(2) *dr(2) *dr(5) *dr(5) 
      terms(798)=coeff(798) *dr(1) *dr(1) *dr(2) *dr(2) *dr(4) 
      terms(799)=coeff(799) *dr(1) *dr(1) *dr(2) *dr(2) *dr(4) *dr(6) 
      terms(800)=coeff(800) *dr(1) *dr(1) *dr(2) *dr(2) *dr(4) *dr(5) 
      terms(801)=coeff(801) *dr(1) *dr(1) *dr(2) *dr(2) *dr(4) *dr(4) 
      terms(802)=coeff(802) *dr(1) *dr(1) *dr(2) *dr(2) *dr(3) 
      terms(803)=coeff(803) *dr(1) *dr(1) *dr(2) *dr(2) *dr(3) *dr(6) 
      terms(804)=coeff(804) *dr(1) *dr(1) *dr(2) *dr(2) *dr(3) *dr(5) 
      terms(805)=coeff(805) *dr(1) *dr(1) *dr(2) *dr(2) *dr(3) *dr(4) 
      terms(806)=coeff(806) *dr(1) *dr(1) *dr(2) *dr(2) *dr(3) *dr(3) 
      terms(807)=coeff(807) *dr(1) *dr(1) *dr(2) *dr(2) *dr(2) 
      terms(808)=coeff(808) *dr(1) *dr(1) *dr(2) *dr(2) *dr(2) *dr(6) 
      terms(809)=coeff(809) *dr(1) *dr(1) *dr(2) *dr(2) *dr(2) *dr(5) 
      terms(810)=coeff(810) *dr(1) *dr(1) *dr(2) *dr(2) *dr(2) *dr(4) 
      terms(811)=coeff(811) *dr(1) *dr(1) *dr(2) *dr(2) *dr(2) *dr(3) 
      terms(812)=coeff(812) *dr(1) *dr(1) *dr(2) *dr(2) *dr(2) *dr(2) 
      terms(813)=coeff(813) *dr(1) *dr(1) *dr(1) 
      terms(814)=coeff(814) *dr(1) *dr(1) *dr(1) *dr(6) 
      terms(815)=coeff(815) *dr(1) *dr(1) *dr(1) *dr(6) *dr(6) 
      terms(816)=coeff(816) *dr(1) *dr(1) *dr(1) *dr(6) *dr(6) *dr(6) 
      terms(817)=coeff(817) *dr(1) *dr(1) *dr(1) *dr(5) 
      terms(818)=coeff(818) *dr(1) *dr(1) *dr(1) *dr(5) *dr(6) 
      terms(819)=coeff(819) *dr(1) *dr(1) *dr(1) *dr(5) *dr(6) *dr(6) 
      terms(820)=coeff(820) *dr(1) *dr(1) *dr(1) *dr(5) *dr(5) 
      terms(821)=coeff(821) *dr(1) *dr(1) *dr(1) *dr(5) *dr(5) *dr(6) 
      terms(822)=coeff(822) *dr(1) *dr(1) *dr(1) *dr(5) *dr(5) *dr(5) 
      terms(823)=coeff(823) *dr(1) *dr(1) *dr(1) *dr(4) 
      terms(824)=coeff(824) *dr(1) *dr(1) *dr(1) *dr(4) *dr(6) 
      terms(825)=coeff(825) *dr(1) *dr(1) *dr(1) *dr(4) *dr(6) *dr(6) 
      terms(826)=coeff(826) *dr(1) *dr(1) *dr(1) *dr(4) *dr(5) 
      terms(827)=coeff(827) *dr(1) *dr(1) *dr(1) *dr(4) *dr(5) *dr(6) 
      terms(828)=coeff(828) *dr(1) *dr(1) *dr(1) *dr(4) *dr(5) *dr(5) 
      terms(829)=coeff(829) *dr(1) *dr(1) *dr(1) *dr(4) *dr(4) 
      terms(830)=coeff(830) *dr(1) *dr(1) *dr(1) *dr(4) *dr(4) *dr(6) 
      terms(831)=coeff(831) *dr(1) *dr(1) *dr(1) *dr(4) *dr(4) *dr(5) 
      terms(832)=coeff(832) *dr(1) *dr(1) *dr(1) *dr(4) *dr(4) *dr(4) 
      terms(833)=coeff(833) *dr(1) *dr(1) *dr(1) *dr(3) 
      terms(834)=coeff(834) *dr(1) *dr(1) *dr(1) *dr(3) *dr(6) 
      terms(835)=coeff(835) *dr(1) *dr(1) *dr(1) *dr(3) *dr(6) *dr(6) 
      terms(836)=coeff(836) *dr(1) *dr(1) *dr(1) *dr(3) *dr(5) 
      terms(837)=coeff(837) *dr(1) *dr(1) *dr(1) *dr(3) *dr(5) *dr(6) 
      terms(838)=coeff(838) *dr(1) *dr(1) *dr(1) *dr(3) *dr(5) *dr(5) 
      terms(839)=coeff(839) *dr(1) *dr(1) *dr(1) *dr(3) *dr(4) 
      terms(840)=coeff(840) *dr(1) *dr(1) *dr(1) *dr(3) *dr(4) *dr(6) 
      terms(841)=coeff(841) *dr(1) *dr(1) *dr(1) *dr(3) *dr(4) *dr(5) 
      terms(842)=coeff(842) *dr(1) *dr(1) *dr(1) *dr(3) *dr(4) *dr(4) 
      terms(843)=coeff(843) *dr(1) *dr(1) *dr(1) *dr(3) *dr(3) 
      terms(844)=coeff(844) *dr(1) *dr(1) *dr(1) *dr(3) *dr(3) *dr(6) 
      terms(845)=coeff(845) *dr(1) *dr(1) *dr(1) *dr(3) *dr(3) *dr(5) 
      terms(846)=coeff(846) *dr(1) *dr(1) *dr(1) *dr(3) *dr(3) *dr(4) 
      terms(847)=coeff(847) *dr(1) *dr(1) *dr(1) *dr(3) *dr(3) *dr(3) 
      terms(848)=coeff(848) *dr(1) *dr(1) *dr(1) *dr(2) 
      terms(849)=coeff(849) *dr(1) *dr(1) *dr(1) *dr(2) *dr(6) 
      terms(850)=coeff(850) *dr(1) *dr(1) *dr(1) *dr(2) *dr(6) *dr(6) 
      terms(851)=coeff(851) *dr(1) *dr(1) *dr(1) *dr(2) *dr(5) 
      terms(852)=coeff(852) *dr(1) *dr(1) *dr(1) *dr(2) *dr(5) *dr(6) 
      terms(853)=coeff(853) *dr(1) *dr(1) *dr(1) *dr(2) *dr(5) *dr(5) 
      terms(854)=coeff(854) *dr(1) *dr(1) *dr(1) *dr(2) *dr(4) 
      terms(855)=coeff(855) *dr(1) *dr(1) *dr(1) *dr(2) *dr(4) *dr(6) 
      terms(856)=coeff(856) *dr(1) *dr(1) *dr(1) *dr(2) *dr(4) *dr(5) 
      terms(857)=coeff(857) *dr(1) *dr(1) *dr(1) *dr(2) *dr(4) *dr(4) 
      terms(858)=coeff(858) *dr(1) *dr(1) *dr(1) *dr(2) *dr(3) 
      terms(859)=coeff(859) *dr(1) *dr(1) *dr(1) *dr(2) *dr(3) *dr(6) 
      terms(860)=coeff(860) *dr(1) *dr(1) *dr(1) *dr(2) *dr(3) *dr(5) 
      terms(861)=coeff(861) *dr(1) *dr(1) *dr(1) *dr(2) *dr(3) *dr(4) 
      terms(862)=coeff(862) *dr(1) *dr(1) *dr(1) *dr(2) *dr(3) *dr(3) 
      terms(863)=coeff(863) *dr(1) *dr(1) *dr(1) *dr(2) *dr(2) 
      terms(864)=coeff(864) *dr(1) *dr(1) *dr(1) *dr(2) *dr(2) *dr(6) 
      terms(865)=coeff(865) *dr(1) *dr(1) *dr(1) *dr(2) *dr(2) *dr(5) 
      terms(866)=coeff(866) *dr(1) *dr(1) *dr(1) *dr(2) *dr(2) *dr(4) 
      terms(867)=coeff(867) *dr(1) *dr(1) *dr(1) *dr(2) *dr(2) *dr(3) 
      terms(868)=coeff(868) *dr(1) *dr(1) *dr(1) *dr(2) *dr(2) *dr(2) 
      terms(869)=coeff(869) *dr(1) *dr(1) *dr(1) *dr(1) 
      terms(870)=coeff(870) *dr(1) *dr(1) *dr(1) *dr(1) *dr(6) 
      terms(871)=coeff(871) *dr(1) *dr(1) *dr(1) *dr(1) *dr(6) *dr(6) 
      terms(872)=coeff(872) *dr(1) *dr(1) *dr(1) *dr(1) *dr(5) 
      terms(873)=coeff(873) *dr(1) *dr(1) *dr(1) *dr(1) *dr(5) *dr(6) 
      terms(874)=coeff(874) *dr(1) *dr(1) *dr(1) *dr(1) *dr(5) *dr(5) 
      terms(875)=coeff(875) *dr(1) *dr(1) *dr(1) *dr(1) *dr(4) 
      terms(876)=coeff(876) *dr(1) *dr(1) *dr(1) *dr(1) *dr(4) *dr(6) 
      terms(877)=coeff(877) *dr(1) *dr(1) *dr(1) *dr(1) *dr(4) *dr(5) 
      terms(878)=coeff(878) *dr(1) *dr(1) *dr(1) *dr(1) *dr(4) *dr(4) 
      terms(879)=coeff(879) *dr(1) *dr(1) *dr(1) *dr(1) *dr(3) 
      terms(880)=coeff(880) *dr(1) *dr(1) *dr(1) *dr(1) *dr(3) *dr(6) 
      terms(881)=coeff(881) *dr(1) *dr(1) *dr(1) *dr(1) *dr(3) *dr(5) 
      terms(882)=coeff(882) *dr(1) *dr(1) *dr(1) *dr(1) *dr(3) *dr(4) 
      terms(883)=coeff(883) *dr(1) *dr(1) *dr(1) *dr(1) *dr(3) *dr(3) 
      terms(884)=coeff(884) *dr(1) *dr(1) *dr(1) *dr(1) *dr(2) 
      terms(885)=coeff(885) *dr(1) *dr(1) *dr(1) *dr(1) *dr(2) *dr(6) 
      terms(886)=coeff(886) *dr(1) *dr(1) *dr(1) *dr(1) *dr(2) *dr(5) 
      terms(887)=coeff(887) *dr(1) *dr(1) *dr(1) *dr(1) *dr(2) *dr(4) 
      terms(888)=coeff(888) *dr(1) *dr(1) *dr(1) *dr(1) *dr(2) *dr(3) 
      terms(889)=coeff(889) *dr(1) *dr(1) *dr(1) *dr(1) *dr(2) *dr(2) 
      terms(890)=coeff(890) *dr(1) *dr(1) *dr(1) *dr(1) *dr(1) 
      terms(891)=coeff(891) *dr(1) *dr(1) *dr(1) *dr(1) *dr(1) *dr(6) 
      terms(892)=coeff(892) *dr(1) *dr(1) *dr(1) *dr(1) *dr(1) *dr(5) 
      terms(893)=coeff(893) *dr(1) *dr(1) *dr(1) *dr(1) *dr(1) *dr(4) 
      terms(894)=coeff(894) *dr(1) *dr(1) *dr(1) *dr(1) *dr(1) *dr(3) 
      terms(895)=coeff(895) *dr(1) *dr(1) *dr(1) *dr(1) *dr(1) *dr(2) 
      terms(896)=coeff(896) *dr(1) *dr(1) *dr(1) *dr(1) *dr(1) *dr(1) 
       pot=0.d0
       do i=1,nfun
         pot=pot+terms(i)
       enddo
       pot=pot/hcm
       return
       end subroutine
C*************************************************************************
C*************************************************************************
      subroutine h2co_asym(r,pot)
 
      implicit double precision (a-h, o-z)
      parameter (nfun=868)
      dimension r(6),re(6),dr(6),coeff(nfun)
      dimension terms(nfun)
      hcm=219474.63d0
      data re/
     &  3.8215337d0,
     &  2.0845219d0,
     &  2.289493d0,
     &  2.0845219d0,
     &  3.8215337d0,
     &  3.54666986d0/
       do i=1,6
           dr(i)=1-exp(-.5d0*(r(i)-re(i)))
       enddo
       data coeff/
     & -0.1885172721D+06, -0.2593913141D+05, -0.2466896771D+05, 
     & -0.8730915468D+04, 0.1546727329D+06, 0.3271662239D+06, 
     & 0.4387847667D+05, 0.3696272065D+03, -0.1968358854D+06, 
     & 0.3167968349D+06, 0.1704630288D+06, 0.2822377414D+05, 
     & -0.8280585672D+06, -0.1623683851D+06, 0.1792209177D+05, 
     & 0.3774561135D+04, -0.9219575523D+05, -0.1542263978D+05, 
     & -0.1533322867D+05, 0.7024908832D+05, -0.2099586712D+03, 
     & 0.2652807021D+04, 0.9247660477D+06, -0.3081235946D+06, 
     & -0.8467334979D+05, -0.4607682502D+04, -0.5242576538D+06, 
     & 0.1493998506D+06, 0.4139249476D+06, 0.8723286448D+05, 
     & -0.3960800358D+07, -0.1242041503D+07, -0.1783971213D+06, 
     & -0.1805606305D+05, 0.1066215329D+07, 0.6284583445D+05, 
     & -0.1709830364D+06, 0.1010760698D+07, -0.1832663997D+05, 
     & -0.6596365444D+05, -0.4800612535D+07, -0.8299114160D+06, 
     & -0.7476271061D+05, 0.1119170772D+05, 0.2192638211D+07, 
     & 0.4169900464D+06, -0.3091583311D+05, -0.3075886345D+05, 
     & 0.9884298517D+07, 0.8263074391D+06, -0.6135194054D+06, 
     & 0.3558726615D+07, -0.2905716549D+06, -0.1053530726D+07, 
     & 0.1168812948D+08, 0.2713475966D+07, 0.4325241497D+06, 
     & 0.4328889772D+05, -0.2898976970D+07, -0.2364132835D+07, 
     & -0.1335498895D+07, 0.2005179676D+06, -0.8932047010D+06, 
     & -0.4321778222D+07, -0.9456979397D+07, -0.2390291842D+07, 
     & -0.6586200933D+06, 0.5118691746D+06, -0.1478608864D+07, 
     & -0.8633000274D+07, -0.2319896514D+07, -0.6449331835D+06, 
     & -0.6273939294D+07, -0.4019839514D+06, -0.2462109705D+05, 
     & -0.3192300671D+06, -0.6205978838D+05, -0.4121066824D+04, 
     & 0.2527196798D+05, 0.1003175018D+06, 0.2337503389D+05, 
     & 0.3819095445D+04, 0.2588944993D+06, 0.4009525545D+05, 
     & 0.8291045482D+04, 0.3664810542D+04, 0.9646243580D+05, 
     & -0.2138269292D+05, 0.9888596472D+03, -0.1456009526D+06, 
     & -0.7544587116D+04, -0.1358547840D+05, -0.5661381732D+06, 
     & 0.4542237425D+06, 0.5384772388D+05, -0.2150332636D+04, 
     & 0.2752684860D+07, -0.1122092314D+06, -0.7272522810D+05, 
     & -0.7933114133D+04, 0.6634937501D+05, -0.1021984596D+05, 
     & 0.3312078325D+04, -0.1644250640D+07, -0.5261580691D+05, 
     & -0.9847839108D+04, 0.5191358035D+07, 0.2771273339D+06, 
     & 0.5055045026D+05, 0.1969488920D+05, -0.4385960312D+07, 
     & 0.3415034371D+05, 0.9696939691D+04, -0.5456275597D+07, 
     & -0.2422440633D+06, 0.9871044072D+06, -0.6266733220D+07, 
     & -0.1999217496D+06, 0.6342761753D+05, -0.6944049180D+07, 
     & -0.4887869603D+06, 0.3799235375D+07, -0.3489101615D+07, 
     & -0.1639308835D+06, 0.6088063493D+07, 0.2966986833D+07, 
     & 0.3948746038D+06, 0.1680083272D+06, 0.2015209682D+06, 
     & 0.4609420253D+05, -0.4107297288D+06, -0.2997442308D+05, 
     & -0.2227557865D+05, -0.2632287991D+04, 0.8658002677D+05, 
     & -0.1788071789D+05, -0.2116271678D+05, 0.7217705020D+05, 
     & 0.2015253610D+04, -0.3900613660D+05, 0.6357245312D+06, 
     & 0.4559635452D+06, 0.1014019795D+06, 0.5786249963D+04, 
     & 0.2430748507D+07, -0.1400204926D+06, -0.4190360782D+05, 
     & 0.9077733880D+05, -0.8858195580D+05, -0.4333017318D+06, 
     & 0.7197731977D+06, -0.7812986791D+03, -0.5188050979D+04, 
     & -0.2947862479D+07, -0.9076154053D+05, -0.1249450731D+07, 
     & -0.1178084529D+07, -0.5487044081D+05, 0.2156074721D+06, 
     & 0.4272463199D+03, 0.1888994719D+07, 0.3398417081D+07, 
     & 0.1788335757D+07, 0.3094794722D+06, -0.2182966423D+06, 
     & -0.1221324530D+06, -0.4710685891D+05, -0.1767841312D+06, 
     & -0.1569685039D+06, 0.6648919827D+05, -0.1056227909D+07, 
     & -0.3733066877D+06, -0.6079471324D+05, -0.5453449445D+06, 
     & -0.8784701853D+06, 0.2808761547D+06, -0.1468449869D+07, 
     & -0.8637653373D+06, -0.1539510217D+06, 0.6293563965D+06, 
     & 0.9247660428D+06, -0.3081235953D+06, -0.8467334973D+05, 
     & -0.4607682483D+04, 0.1066847707D+07, -0.1315458683D+07, 
     & -0.5510403122D+06, -0.8482284820D+05, 0.5905698732D+07, 
     & 0.8881027687D+06, -0.1324192114D+05, -0.4608619864D+04, 
     & 0.1912790844D+07, 0.3721538093D+06, 0.1625078663D+06, 
     & -0.6108345315D+06, -0.2563235585D+05, 0.4987628816D+04, 
     & 0.1482936871D+07, 0.4276425951D+07, 0.7052910319D+06, 
     & 0.4395737091D+03, 0.7359539799D+07, 0.1899359383D+07, 
     & 0.6916253994D+05, 0.2478642646D+05, -0.6282603298D+07, 
     & 0.4477203247D+06, 0.1280695744D+07, -0.8627334788D+07, 
     & -0.1305002207D+06, 0.9139574784D+06, -0.2174107834D+07, 
     & -0.4551795637D+07, -0.7435164112D+06, -0.5003476132D+05, 
     & -0.1968417749D+08, 0.4393204194D+07, 0.3697952112D+07, 
     & -0.2518309399D+08, -0.8152491603D+06, 0.8082355875D+07, 
     & -0.1776446218D+08, 0.2004227707D+07, 0.2332314855D+07, 
     & 0.7788579469D+07, 0.2815797966D+07, 0.3582239442D+08, 
     & 0.3081215970D+08, 0.4079867700D+07, 0.3221892162D+08, 
     & 0.9934355582D+07, -0.5661381611D+06, 0.4542237445D+06, 
     & 0.5384772432D+05, -0.2150332588D+04, -0.2465592531D+07, 
     & -0.6573087123D+05, 0.5320880419D+05, 0.3173981765D+04, 
     & -0.1112375570D+07, 0.5242457758D+05, 0.1577892237D+05, 
     & 0.1358492936D+07, 0.8374904255D+05, 0.1944827404D+06, 
     & -0.4326474558D+07, -0.1556837962D+07, -0.2116796250D+06, 
     & -0.3251489071D+05, 0.6810621452D+06, 0.2752664637D+06, 
     & 0.6281792914D+05, 0.1195636262D+08, 0.4670323885D+06, 
     & 0.1838045059D+06, 0.4404587609D+06, 0.3968487380D+06, 
     & -0.3477197272D+05, 0.2834142910D+08, 0.1289511596D+07, 
     & -0.5362502543D+07, 0.2717056270D+08, 0.1220069176D+07, 
     & -0.1497900776D+08, -0.1055935439D+08, 0.6357245286D+06, 
     & 0.4559635458D+06, 0.1014019796D+06, 0.5786249967D+04, 
     & -0.1421091559D+06, 0.1075087313D+06, 0.4343880902D+05, 
     & -0.3662621400D+06, 0.2153187039D+05, 0.3540284917D+06, 
     & -0.5020386343D+07, -0.6987706791D+06, -0.8489360921D+05, 
     & -0.1386635017D+07, 0.1592295702D+06, 0.2333293290D+07, 
     & 0.3361479879D+07, 0.1303857394D+06, 0.3019179721D+07, 
     & 0.1026962020D+07, -0.1056227909D+07, -0.3733066875D+06, 
     & -0.6079471317D+05, 0.1210760837D+07, 0.7822366291D+06, 
     & -0.2383639792D+06, 0.3993353398D+07, 0.1951468003D+07, 
     & -0.6092688256D+06, -0.5271911017D+06, -0.4800612509D+07, 
     & -0.8299114140D+06, -0.7476271078D+05, 0.1119170768D+05, 
     & -0.1203439350D+08, -0.8577992573D+06, 0.1462159859D+06, 
     & 0.4889100874D+04, -0.8068969659D+07, -0.1407506434D+07, 
     & -0.6187195349D+06, 0.1564442114D+07, 0.1286567756D+06, 
     & -0.2451870133D+06, -0.2174107906D+07, -0.4551795615D+07, 
     & -0.7435164040D+06, -0.5003476077D+05, 0.5711012344D+07, 
     & -0.6619340778D+07, -0.3612724900D+07, 0.3259209966D+08, 
     & 0.3154034176D+07, -0.3254920591D+07, 0.3189513806D+08, 
     & 0.2716319163D+07, -0.3113336755D+07, 0.3756077214D+08, 
     & 0.2254806437D+07, -0.4064620150D+08, -0.2243571812D+08, 
     & -0.3556129700D+07, -0.7899764569D+08, -0.4997444637D+08, 
     & 0.5191357996D+07, 0.2771273322D+06, 0.5055045011D+05, 
     & 0.1969488919D+05, 0.3836303553D+07, -0.2114407472D+06, 
     & -0.8210208676D+05, -0.4592897052D+07, -0.2641630542D+06, 
     & -0.9499552069D+06, 0.4404589716D+06, 0.3968487798D+06, 
     & -0.3477196958D+05, -0.3049963290D+08, -0.1225401592D+07, 
     & -0.2449346799D+07, -0.3689917015D+08, -0.2109168998D+07, 
     & 0.6758878850D+07, 0.5725832390D+07, 0.7197732035D+06, 
     & -0.7812987302D+03, -0.5188050993D+04, 0.6657419549D+06, 
     & -0.6358739336D+05, -0.9149253571D+06, 0.3361479880D+07, 
     & 0.1303857401D+06, -0.1887898032D+07, -0.3859595323D+07, 
     & -0.1468449867D+07, -0.8637653371D+06, -0.5562429977D+05, 
     & -0.5271911156D+06, 0.1168812943D+08, 0.2713475956D+07, 
     & 0.4325241476D+06, 0.4328889757D+05, 0.1718882532D+08, 
     & 0.4020424002D+07, 0.1167256211D+07, -0.4017888873D+07, 
     & -0.1206848542D+07, 0.8080009656D+06, -0.1776446198D+08, 
     & 0.2004227704D+07, 0.2332314852D+07, -0.4359797227D+08, 
     & -0.4676222838D+07, 0.1626116908D+08, -0.2243571770D+08, 
     & -0.3556129601D+07, 0.6617369809D+08, 0.7881545881D+08, 
     & -0.6266733226D+07, -0.1999217622D+06, 0.6342761649D+05, 
     & 0.6965917123D+07, 0.4046566284D+06, 0.2775771680D+07, 
     & 0.2717056251D+08, 0.1220069164D+07, 0.8109681981D+07, 
     & 0.5725832586D+07, -0.1178084533D+07, -0.5487044121D+05, 
     & 0.3292306469D+06, 0.1026962017D+07, 0.6293564004D+06, 
     & -0.9456979403D+07, -0.2390291839D+07, -0.6586200921D+06, 
     & 0.2473731877D+07, 0.1037859324D+07, -0.3556377314D+07, 
     & 0.3081215932D+08, 0.4079867649D+07, -0.1911729616D+08, 
     & -0.4997444637D+08, -0.3489101575D+07, -0.1639308779D+06, 
     & -0.4101529702D+07, -0.1055935442D+08, 0.4272479077D+03, 
     & -0.2319896432D+07, -0.6449331732D+06, 0.2635025271D+07, 
     & 0.9934355639D+07, 0.2966986829D+07, -0.4019839698D+06, 
     & 0.1546727306D+06, 0.3271662237D+06, 0.4387847676D+05, 
     & 0.3696272228D+03, -0.4762022963D+06, -0.1100008047D+07, 
     & -0.3891834546D+06, -0.5843266595D+05, 0.2434814529D+07, 
     & 0.3266234209D+06, -0.1066400942D+05, -0.4711297156D+04, 
     & 0.6197950613D+06, 0.2035365962D+06, 0.6220374794D+05, 
     & -0.3642070007D+06, 0.2756566317D+05, -0.1017935928D+05, 
     & 0.1066847727D+07, -0.1315458682D+07, -0.5510403126D+06, 
     & -0.8482284826D+05, -0.3362945478D+07, 0.8755967404D+06, 
     & 0.2224970007D+06, 0.2412779691D+05, -0.4086213427D+07, 
     & 0.6241093811D+06, 0.5606986380D+06, -0.3722872151D+07, 
     & 0.1469099126D+05, 0.3687649646D+06, -0.1203439355D+08, 
     & -0.8577992701D+06, 0.1462159831D+06, 0.4889100662D+04, 
     & 0.6990739144D+07, 0.1093207446D+07, 0.1341676312D+07, 
     & -0.2315946469D+07, 0.4928398666D+06, 0.3507381310D+07, 
     & 0.1718882531D+08, 0.4020424004D+07, 0.1167256213D+07, 
     & -0.1520263930D+07, 0.2225067399D+07, 0.5598070908D+07, 
     & 0.2473732070D+07, 0.1037859345D+07, 0.9964851038D+07, 
     & 0.2635025225D+07, 0.2527197364D+05, 0.1003175031D+06, 
     & 0.2337503424D+05, 0.3819095483D+04, -0.1877610917D+07, 
     & -0.2579777493D+06, -0.3564984658D+05, -0.7339552401D+04, 
     & 0.2433407655D+06, 0.1250408302D+05, -0.1329016612D+04, 
     & 0.6154488748D+06, 0.3267901500D+05, 0.3733731196D+05, 
     & -0.2465592558D+07, -0.6573087214D+05, 0.5320880407D+05, 
     & 0.3173981753D+04, 0.2069983776D+07, 0.1311878630D+06, 
     & -0.7912156110D+04, 0.2730475972D+07, 0.1102750087D+06, 
     & -0.2554408516D+06, 0.3836303533D+07, -0.2114407649D+06, 
     & -0.8210208827D+05, 0.1075990335D+08, 0.4160330023D+06, 
     & -0.1702489221D+07, 0.6965917168D+07, 0.4046566347D+06, 
     & -0.7048787081D+07, -0.4101529714D+07, -0.4107297293D+06, 
     & -0.2997442252D+05, -0.2227557857D+05, -0.2632287983D+04, 
     & -0.1316068846D+07, 0.1954633472D+05, 0.4097479415D+05, 
     & -0.1818929538D+06, -0.1244178180D+04, 0.1952149344D+06, 
     & -0.1421091539D+06, 0.1075087314D+06, 0.4343880901D+05, 
     & 0.1683307107D+07, 0.7509207182D+05, 0.1408116023D+07, 
     & 0.6657419451D+06, -0.6358739386D+05, 0.1914532274D+07, 
     & 0.3292306557D+06, -0.2182966432D+06, -0.1221324528D+06, 
     & -0.4710685887D+05, -0.8036003780D+05, 0.3338628983D+06, 
     & -0.1361587942D+06, 0.1210760840D+07, 0.7822366292D+06, 
     & 0.3592971528D+06, -0.5562429713D+05, -0.5242576491D+06, 
     & 0.1493998517D+06, 0.4139249477D+06, 0.8723286448D+05, 
     & -0.3362945481D+07, 0.8755967518D+06, 0.2224970032D+06, 
     & 0.2412779712D+05, -0.5723290729D+07, -0.1387572024D+07, 
     & -0.5703251629D+06, 0.1949818753D+07, -0.1107301806D+05, 
     & 0.5882413441D+05, 0.7359539746D+07, 0.1899359406D+07, 
     & 0.6916254645D+05, 0.2478642699D+05, 0.1539353911D+08, 
     & -0.2729049438D+07, -0.2784238434D+07, 0.1793837992D+08, 
     & 0.5118759836D+06, -0.2500019291D+07, 0.5711012561D+07, 
     & -0.6619340772D+07, -0.3612724902D+07, -0.1797776347D+08, 
     & -0.2322949421D+07, -0.1568193909D+08, -0.4359797301D+08, 
     & -0.4676222917D+07, -0.3601644803D+08, -0.1911729611D+08, 
     & 0.2752684851D+07, -0.1122092335D+06, -0.7272522856D+05, 
     & -0.7933114165D+04, 0.2069983838D+07, 0.1311878793D+06, 
     & -0.7912154743D+04, -0.4167056759D+07, -0.1194549197D+06, 
     & -0.5383137415D+06, 0.6810623234D+06, 0.2752665024D+06, 
     & 0.6281793233D+05, -0.2243466400D+08, -0.9485658386D+06, 
     & 0.7934243136D+06, -0.3049963304D+08, -0.1225401596D+07, 
     & 0.6100864211D+07, 0.8109681975D+07, 0.2430748507D+07, 
     & -0.1400204935D+06, -0.4190360785D+05, 0.1683307125D+07, 
     & 0.7509207187D+05, -0.1295066770D+07, -0.1386634991D+07, 
     & 0.1592295708D+06, -0.5626439686D+07, -0.1887898047D+07, 
     & -0.5453449441D+06, -0.8784701854D+06, 0.3592971517D+06, 
     & -0.6092688344D+06, 0.2192638217D+07, 0.4169900319D+06, 
     & -0.3091583670D+05, -0.3075886372D+05, 0.6990739199D+07, 
     & 0.1093207440D+07, 0.1341676311D+07, -0.1871665966D+07, 
     & -0.5884320611D+06, 0.4066599214D+06, -0.1968417745D+08, 
     & 0.4393204182D+07, 0.3697952112D+07, -0.1797776258D+08, 
     & -0.2322949327D+07, 0.5268797103D+07, 0.3756077289D+08, 
     & 0.2254806553D+07, 0.6912898428D+08, 0.6617369834D+08, 
     & -0.4385960383D+07, 0.3415032504D+05, 0.9696938224D+04, 
     & 0.1075990329D+08, 0.4160329950D+06, 0.2494928363D+07, 
     & 0.2834142903D+08, 0.1289511581D+07, 0.6100864212D+07, 
     & 0.6758878957D+07, -0.2947862491D+07, -0.9076154039D+05, 
     & 0.1914532261D+07, 0.3019179713D+07, -0.1539510180D+06, 
     & -0.2898977023D+07, -0.2364132826D+07, -0.1335498894D+07, 
     & -0.1520264305D+07, 0.2225067367D+07, -0.1816247244D+07, 
     & 0.7788578877D+07, 0.2815797887D+07, -0.3601644828D+08, 
     & -0.7899764608D+08, -0.6944049103D+07, -0.4887869494D+06, 
     & -0.7048787063D+07, -0.1497900781D+08, 0.2156074817D+06, 
     & 0.5118693848D+06, -0.1478608843D+07, 0.9964851198D+07, 
     & 0.3221892192D+08, 0.6088063486D+07, -0.6273939396D+07, 
     & -0.1968358889D+06, 0.3167968349D+06, 0.1704630289D+06, 
     & 0.2822377416D+05, 0.2434814529D+07, 0.3266234230D+06, 
     & -0.1066400910D+05, -0.4711297123D+04, -0.1068621314D+06, 
     & -0.3596791900D+06, -0.8702049620D+05, 0.5019461681D+06, 
     & -0.2869567770D+05, -0.3938802901D+05, 0.5905698754D+07, 
     & 0.8881027740D+06, -0.1324192011D+05, -0.4608619780D+04, 
     & -0.5723290706D+07, -0.1387572023D+07, -0.5703251630D+06, 
     & -0.3666485841D+06, 0.5023997744D+05, -0.7829241874D+06, 
     & -0.8068969656D+07, -0.1407506430D+07, -0.6187195354D+06, 
     & -0.1871666216D+07, -0.5884320788D+06, -0.1132034927D+07, 
     & -0.4017889055D+07, -0.1206848559D+07, -0.1816247160D+07, 
     & -0.3556377276D+07, 0.2588945043D+06, 0.4009525538D+05, 
     & 0.8291045500D+04, 0.3664810547D+04, 0.2433407783D+06, 
     & 0.1250408581D+05, -0.1329016401D+04, -0.3940331277D+06, 
     & -0.5782883946D+05, 0.2830849421D+05, -0.1112375558D+07, 
     & 0.5242458511D+05, 0.1577892300D+05, -0.4167056765D+07, 
     & -0.1194549201D+06, 0.1033483130D+06, -0.4592897062D+07, 
     & -0.2641630556D+06, 0.2494928368D+07, 0.2775771686D+07, 
     & 0.8658002724D+05, -0.1788071800D+05, -0.2116271678D+05, 
     & -0.1818929482D+06, -0.1244178154D+04, -0.2697261831D+06, 
     & -0.3662621358D+06, 0.2153187053D+05, -0.1295066774D+07, 
     & -0.9149253624D+06, -0.1767841319D+06, -0.1569685039D+06, 
     & -0.1361587938D+06, -0.2383639799D+06, -0.3960800350D+07, 
     & -0.1242041508D+07, -0.1783971224D+06, -0.1805606315D+05, 
     & -0.4086213425D+07, 0.6241093769D+06, 0.5606986379D+06, 
     & -0.3666484929D+06, 0.5023998327D+05, 0.1086279503D+06, 
     & -0.6282603389D+07, 0.4477203123D+06, 0.1280695744D+07, 
     & 0.1793838034D+08, 0.5118760169D+06, 0.3194340673D+07, 
     & 0.3259210020D+08, 0.3154034220D+07, 0.5268797130D+07, 
     & 0.1626116912D+08, 0.6634933927D+05, -0.1021985348D+05, 
     & 0.3312077685D+04, 0.2730475956D+07, 0.1102750065D+06, 
     & 0.1033483145D+06, 0.1195636263D+08, 0.4670323847D+06, 
     & 0.7934243050D+06, -0.2449346809D+07, 0.9077732973D+05, 
     & -0.8858195580D+05, 0.1408116020D+07, 0.2333293296D+07, 
     & 0.2808761559D+06, 0.9884298512D+07, 0.8263074478D+06, 
     & -0.6135194051D+06, -0.2315946690D+07, 0.4928398520D+06, 
     & -0.1132034977D+07, -0.2518309440D+08, -0.8152491985D+06, 
     & -0.1568193933D+08, -0.4064620187D+08, -0.5456275562D+07, 
     & -0.2422440580D+06, -0.1702489208D+07, -0.5362502540D+07, 
     & -0.1249450726D+07, 0.2005181305D+06, -0.8932046905D+06, 
     & 0.5598071066D+07, 0.3582239479D+08, 0.3799235362D+07, 
     & -0.8633000412D+07, -0.8280585716D+06, -0.1623683859D+06, 
     & 0.1792209167D+05, 0.3774561126D+04, 0.6197950584D+06, 
     & 0.2035365955D+06, 0.6220374792D+05, 0.5019461797D+06, 
     & -0.2869567700D+05, 0.1322338248D+06, 0.1912790846D+07, 
     & 0.3721538072D+06, 0.1625078663D+06, 0.1949818823D+07, 
     & -0.1107301345D+05, 0.1086279448D+06, 0.1564442196D+07, 
     & 0.1286567827D+06, 0.4066599012D+06, 0.8080009515D+06, 
     & 0.9646243399D+05, -0.2138269381D+05, 0.9888595769D+03, 
     & 0.6154488731D+06, 0.3267901482D+05, 0.2830849515D+05, 
     & 0.1358492933D+07, 0.8374904208D+05, -0.5383137384D+06, 
     & -0.9499552065D+06, 0.7217704972D+05, 0.2015253586D+04, 
     & 0.1952149345D+06, 0.3540284926D+06, 0.6648919832D+05, 
     & 0.1066215344D+07, 0.6284583784D+05, -0.1709830363D+06, 
     & -0.3722872212D+07, 0.1469098734D+05, -0.7829241946D+06, 
     & -0.8627334963D+07, -0.1305002324D+06, -0.2500019330D+07, 
     & -0.3254920641D+07, -0.1644250634D+07, -0.5261580566D+05, 
     & -0.2554408521D+06, 0.1838045080D+06, -0.4333017321D+06, 
     & 0.3558726684D+07, -0.2905716508D+06, 0.3507381362D+07, 
     & 0.8082356026D+07, 0.9871044030D+06, -0.4321778298D+07, 
     & -0.9219575584D+05, -0.1542263947D+05, -0.1533322866D+05, 
     & -0.3642070075D+06, 0.2756566274D+05, -0.3938802878D+05, 
     & -0.6108345486D+06, -0.2563235736D+05, 0.5882413758D+05, 
     & -0.2451870095D+06, -0.1456009520D+06, -0.7544587015D+04, 
     & 0.3733731121D+05, 0.1944827393D+06, -0.3900613660D+05, 
     & 0.1010760717D+07, -0.1832663880D+05, 0.3687649691D+06, 
     & 0.9139574896D+06, -0.9847838591D+04, -0.1053530743D+07, 
     & 0.7024908969D+05, -0.2099585508D+03, -0.1017935956D+05, 
     & 0.4987628012D+04, -0.1358547822D+05, -0.6596365507D+05, 
     & 0.2652807099D+04/ 
      terms(1)=coeff(1) 
      terms(2)=coeff(2) *dr(6) 
      terms(3)=coeff(3) *dr(6) *dr(6) 
      terms(4)=coeff(4) *dr(6) *dr(6) *dr(6) 
      terms(5)=coeff(5) *dr(5) 
      terms(6)=coeff(6) *dr(5) *dr(6) 
      terms(7)=coeff(7) *dr(5) *dr(6) *dr(6) 
      terms(8)=coeff(8) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(9)=coeff(9) *dr(5) *dr(5) 
      terms(10)=coeff(10) *dr(5) *dr(5) *dr(6) 
      terms(11)=coeff(11) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(12)=coeff(12) *dr(5) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(13)=coeff(13) *dr(5) *dr(5) *dr(5) 
      terms(14)=coeff(14) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(15)=coeff(15) *dr(5) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(16)=coeff(16) *dr(5) *dr(5) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(17)=coeff(17) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(18)=coeff(18) *dr(5) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(19)=coeff(19) *dr(5) *dr(5) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(20)=coeff(20) *dr(5) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(21)=coeff(21) *dr(5) *dr(5) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(22)=coeff(22) *dr(5) *dr(5) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(23)=coeff(23) *dr(4) 
      terms(24)=coeff(24) *dr(4) *dr(6) 
      terms(25)=coeff(25) *dr(4) *dr(6) *dr(6) 
      terms(26)=coeff(26) *dr(4) *dr(6) *dr(6) *dr(6) 
      terms(27)=coeff(27) *dr(4) *dr(5) 
      terms(28)=coeff(28) *dr(4) *dr(5) *dr(6) 
      terms(29)=coeff(29) *dr(4) *dr(5) *dr(6) *dr(6) 
      terms(30)=coeff(30) *dr(4) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(31)=coeff(31) *dr(4) *dr(5) *dr(5) 
      terms(32)=coeff(32) *dr(4) *dr(5) *dr(5) *dr(6) 
      terms(33)=coeff(33) *dr(4) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(34)=coeff(34) *dr(4) *dr(5) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(35)=coeff(35) *dr(4) *dr(5) *dr(5) *dr(5) 
      terms(36)=coeff(36) *dr(4) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(37)=coeff(37) *dr(4) *dr(5) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(38)=coeff(38) *dr(4) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(39)=coeff(39) *dr(4) *dr(5) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(40)=coeff(40) *dr(4) *dr(5) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(41)=coeff(41) *dr(4) *dr(4) 
      terms(42)=coeff(42) *dr(4) *dr(4) *dr(6) 
      terms(43)=coeff(43) *dr(4) *dr(4) *dr(6) *dr(6) 
      terms(44)=coeff(44) *dr(4) *dr(4) *dr(6) *dr(6) *dr(6) 
      terms(45)=coeff(45) *dr(4) *dr(4) *dr(5) 
      terms(46)=coeff(46) *dr(4) *dr(4) *dr(5) *dr(6) 
      terms(47)=coeff(47) *dr(4) *dr(4) *dr(5) *dr(6) *dr(6) 
      terms(48)=coeff(48) *dr(4) *dr(4) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(49)=coeff(49) *dr(4) *dr(4) *dr(5) *dr(5) 
      terms(50)=coeff(50) *dr(4) *dr(4) *dr(5) *dr(5) *dr(6) 
      terms(51)=coeff(51) *dr(4) *dr(4) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(52)=coeff(52) *dr(4) *dr(4) *dr(5) *dr(5) *dr(5) 
      terms(53)=coeff(53) *dr(4) *dr(4) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(54)=coeff(54) *dr(4) *dr(4) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(55)=coeff(55) *dr(4) *dr(4) *dr(4) 
      terms(56)=coeff(56) *dr(4) *dr(4) *dr(4) *dr(6) 
      terms(57)=coeff(57) *dr(4) *dr(4) *dr(4) *dr(6) *dr(6) 
      terms(58)=coeff(58) *dr(4) *dr(4) *dr(4) *dr(6) *dr(6) *dr(6) 
      terms(59)=coeff(59) *dr(4) *dr(4) *dr(4) *dr(5) 
      terms(60)=coeff(60) *dr(4) *dr(4) *dr(4) *dr(5) *dr(6) 
      terms(61)=coeff(61) *dr(4) *dr(4) *dr(4) *dr(5) *dr(6) *dr(6) 
      terms(62)=coeff(62) *dr(4) *dr(4) *dr(4) *dr(5) *dr(5) 
      terms(63)=coeff(63) *dr(4) *dr(4) *dr(4) *dr(5) *dr(5) *dr(6) 
      terms(64)=coeff(64) *dr(4) *dr(4) *dr(4) *dr(5) *dr(5) *dr(5) 
      terms(65)=coeff(65) *dr(4) *dr(4) *dr(4) *dr(4) 
      terms(66)=coeff(66) *dr(4) *dr(4) *dr(4) *dr(4) *dr(6) 
      terms(67)=coeff(67) *dr(4) *dr(4) *dr(4) *dr(4) *dr(6) *dr(6) 
      terms(68)=coeff(68) *dr(4) *dr(4) *dr(4) *dr(4) *dr(5) 
      terms(69)=coeff(69) *dr(4) *dr(4) *dr(4) *dr(4) *dr(5) *dr(6) 
      terms(70)=coeff(70) *dr(4) *dr(4) *dr(4) *dr(4) *dr(5) *dr(5) 
      terms(71)=coeff(71) *dr(4) *dr(4) *dr(4) *dr(4) *dr(4) 
      terms(72)=coeff(72) *dr(4) *dr(4) *dr(4) *dr(4) *dr(4) *dr(6) 
      terms(73)=coeff(73) *dr(4) *dr(4) *dr(4) *dr(4) *dr(4) *dr(5) 
      terms(74)=coeff(74) *dr(4) *dr(4) *dr(4) *dr(4) *dr(4) *dr(4) 
      terms(75)=coeff(75) *dr(3) 
      terms(76)=coeff(76) *dr(3) *dr(6) 
      terms(77)=coeff(77) *dr(3) *dr(6) *dr(6) 
      terms(78)=coeff(78) *dr(3) *dr(6) *dr(6) *dr(6) 
      terms(79)=coeff(79) *dr(3) *dr(5) 
      terms(80)=coeff(80) *dr(3) *dr(5) *dr(6) 
      terms(81)=coeff(81) *dr(3) *dr(5) *dr(6) *dr(6) 
      terms(82)=coeff(82) *dr(3) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(83)=coeff(83) *dr(3) *dr(5) *dr(5) 
      terms(84)=coeff(84) *dr(3) *dr(5) *dr(5) *dr(6) 
      terms(85)=coeff(85) *dr(3) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(86)=coeff(86) *dr(3) *dr(5) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(87)=coeff(87) *dr(3) *dr(5) *dr(5) *dr(5) 
      terms(88)=coeff(88) *dr(3) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(89)=coeff(89) *dr(3) *dr(5) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(90)=coeff(90) *dr(3) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(91)=coeff(91) *dr(3) *dr(5) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(92)=coeff(92) *dr(3) *dr(5) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(93)=coeff(93) *dr(3) *dr(4) 
      terms(94)=coeff(94) *dr(3) *dr(4) *dr(6) 
      terms(95)=coeff(95) *dr(3) *dr(4) *dr(6) *dr(6) 
      terms(96)=coeff(96) *dr(3) *dr(4) *dr(6) *dr(6) *dr(6) 
      terms(97)=coeff(97) *dr(3) *dr(4) *dr(5) 
      terms(98)=coeff(98) *dr(3) *dr(4) *dr(5) *dr(6) 
      terms(99)=coeff(99) *dr(3) *dr(4) *dr(5) *dr(6) *dr(6) 
      terms(100)=coeff(100) *dr(3) *dr(4) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(101)=coeff(101) *dr(3) *dr(4) *dr(5) *dr(5) 
      terms(102)=coeff(102) *dr(3) *dr(4) *dr(5) *dr(5) *dr(6) 
      terms(103)=coeff(103) *dr(3) *dr(4) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(104)=coeff(104) *dr(3) *dr(4) *dr(5) *dr(5) *dr(5) 
      terms(105)=coeff(105) *dr(3) *dr(4) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(106)=coeff(106) *dr(3) *dr(4) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(107)=coeff(107) *dr(3) *dr(4) *dr(4) 
      terms(108)=coeff(108) *dr(3) *dr(4) *dr(4) *dr(6) 
      terms(109)=coeff(109) *dr(3) *dr(4) *dr(4) *dr(6) *dr(6) 
      terms(110)=coeff(110) *dr(3) *dr(4) *dr(4) *dr(6) *dr(6) *dr(6) 
      terms(111)=coeff(111) *dr(3) *dr(4) *dr(4) *dr(5) 
      terms(112)=coeff(112) *dr(3) *dr(4) *dr(4) *dr(5) *dr(6) 
      terms(113)=coeff(113) *dr(3) *dr(4) *dr(4) *dr(5) *dr(6) *dr(6) 
      terms(114)=coeff(114) *dr(3) *dr(4) *dr(4) *dr(5) *dr(5) 
      terms(115)=coeff(115) *dr(3) *dr(4) *dr(4) *dr(5) *dr(5) *dr(6) 
      terms(116)=coeff(116) *dr(3) *dr(4) *dr(4) *dr(5) *dr(5) *dr(5) 
      terms(117)=coeff(117) *dr(3) *dr(4) *dr(4) *dr(4) 
      terms(118)=coeff(118) *dr(3) *dr(4) *dr(4) *dr(4) *dr(6) 
      terms(119)=coeff(119) *dr(3) *dr(4) *dr(4) *dr(4) *dr(6) *dr(6) 
      terms(120)=coeff(120) *dr(3) *dr(4) *dr(4) *dr(4) *dr(5) 
      terms(121)=coeff(121) *dr(3) *dr(4) *dr(4) *dr(4) *dr(5) *dr(6) 
      terms(122)=coeff(122) *dr(3) *dr(4) *dr(4) *dr(4) *dr(5) *dr(5) 
      terms(123)=coeff(123) *dr(3) *dr(4) *dr(4) *dr(4) *dr(4) 
      terms(124)=coeff(124) *dr(3) *dr(4) *dr(4) *dr(4) *dr(4) *dr(6) 
      terms(125)=coeff(125) *dr(3) *dr(4) *dr(4) *dr(4) *dr(4) *dr(5) 
      terms(126)=coeff(126) *dr(3) *dr(4) *dr(4) *dr(4) *dr(4) *dr(4) 
      terms(127)=coeff(127) *dr(3) *dr(3) 
      terms(128)=coeff(128) *dr(3) *dr(3) *dr(6) 
      terms(129)=coeff(129) *dr(3) *dr(3) *dr(6) *dr(6) 
      terms(130)=coeff(130) *dr(3) *dr(3) *dr(6) *dr(6) *dr(6) 
      terms(131)=coeff(131) *dr(3) *dr(3) *dr(5) 
      terms(132)=coeff(132) *dr(3) *dr(3) *dr(5) *dr(6) 
      terms(133)=coeff(133) *dr(3) *dr(3) *dr(5) *dr(6) *dr(6) 
      terms(134)=coeff(134) *dr(3) *dr(3) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(135)=coeff(135) *dr(3) *dr(3) *dr(5) *dr(5) 
      terms(136)=coeff(136) *dr(3) *dr(3) *dr(5) *dr(5) *dr(6) 
      terms(137)=coeff(137) *dr(3) *dr(3) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(138)=coeff(138) *dr(3) *dr(3) *dr(5) *dr(5) *dr(5) 
      terms(139)=coeff(139) *dr(3) *dr(3) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(140)=coeff(140) *dr(3) *dr(3) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(141)=coeff(141) *dr(3) *dr(3) *dr(4) 
      terms(142)=coeff(142) *dr(3) *dr(3) *dr(4) *dr(6) 
      terms(143)=coeff(143) *dr(3) *dr(3) *dr(4) *dr(6) *dr(6) 
      terms(144)=coeff(144) *dr(3) *dr(3) *dr(4) *dr(6) *dr(6) *dr(6) 
      terms(145)=coeff(145) *dr(3) *dr(3) *dr(4) *dr(5) 
      terms(146)=coeff(146) *dr(3) *dr(3) *dr(4) *dr(5) *dr(6) 
      terms(147)=coeff(147) *dr(3) *dr(3) *dr(4) *dr(5) *dr(6) *dr(6) 
      terms(148)=coeff(148) *dr(3) *dr(3) *dr(4) *dr(5) *dr(5) 
      terms(149)=coeff(149) *dr(3) *dr(3) *dr(4) *dr(5) *dr(5) *dr(6) 
      terms(150)=coeff(150) *dr(3) *dr(3) *dr(4) *dr(5) *dr(5) *dr(5) 
      terms(151)=coeff(151) *dr(3) *dr(3) *dr(4) *dr(4) 
      terms(152)=coeff(152) *dr(3) *dr(3) *dr(4) *dr(4) *dr(6) 
      terms(153)=coeff(153) *dr(3) *dr(3) *dr(4) *dr(4) *dr(6) *dr(6) 
      terms(154)=coeff(154) *dr(3) *dr(3) *dr(4) *dr(4) *dr(5) 
      terms(155)=coeff(155) *dr(3) *dr(3) *dr(4) *dr(4) *dr(5) *dr(6) 
      terms(156)=coeff(156) *dr(3) *dr(3) *dr(4) *dr(4) *dr(5) *dr(5) 
      terms(157)=coeff(157) *dr(3) *dr(3) *dr(4) *dr(4) *dr(4) 
      terms(158)=coeff(158) *dr(3) *dr(3) *dr(4) *dr(4) *dr(4) *dr(6) 
      terms(159)=coeff(159) *dr(3) *dr(3) *dr(4) *dr(4) *dr(4) *dr(5) 
      terms(160)=coeff(160) *dr(3) *dr(3) *dr(4) *dr(4) *dr(4) *dr(4) 
      terms(161)=coeff(161) *dr(3) *dr(3) *dr(3) 
      terms(162)=coeff(162) *dr(3) *dr(3) *dr(3) *dr(6) 
      terms(163)=coeff(163) *dr(3) *dr(3) *dr(3) *dr(6) *dr(6) 
      terms(164)=coeff(164) *dr(3) *dr(3) *dr(3) *dr(6) *dr(6) *dr(6) 
      terms(165)=coeff(165) *dr(3) *dr(3) *dr(3) *dr(5) 
      terms(166)=coeff(166) *dr(3) *dr(3) *dr(3) *dr(5) *dr(6) 
      terms(167)=coeff(167) *dr(3) *dr(3) *dr(3) *dr(5) *dr(6) *dr(6) 
      terms(168)=coeff(168) *dr(3) *dr(3) *dr(3) *dr(5) *dr(5) 
      terms(169)=coeff(169) *dr(3) *dr(3) *dr(3) *dr(5) *dr(5) *dr(6) 
      terms(170)=coeff(170) *dr(3) *dr(3) *dr(3) *dr(5) *dr(5) *dr(5) 
      terms(171)=coeff(171) *dr(3) *dr(3) *dr(3) *dr(4) 
      terms(172)=coeff(172) *dr(3) *dr(3) *dr(3) *dr(4) *dr(6) 
      terms(173)=coeff(173) *dr(3) *dr(3) *dr(3) *dr(4) *dr(6) *dr(6) 
      terms(174)=coeff(174) *dr(3) *dr(3) *dr(3) *dr(4) *dr(5) 
      terms(175)=coeff(175) *dr(3) *dr(3) *dr(3) *dr(4) *dr(5) *dr(6) 
      terms(176)=coeff(176) *dr(3) *dr(3) *dr(3) *dr(4) *dr(5) *dr(5) 
      terms(177)=coeff(177) *dr(3) *dr(3) *dr(3) *dr(4) *dr(4) 
      terms(178)=coeff(178) *dr(3) *dr(3) *dr(3) *dr(4) *dr(4) *dr(6) 
      terms(179)=coeff(179) *dr(3) *dr(3) *dr(3) *dr(4) *dr(4) *dr(5) 
      terms(180)=coeff(180) *dr(3) *dr(3) *dr(3) *dr(4) *dr(4) *dr(4) 
      terms(181)=coeff(181) *dr(2) 
      terms(182)=coeff(182) *dr(2) *dr(6) 
      terms(183)=coeff(183) *dr(2) *dr(6) *dr(6) 
      terms(184)=coeff(184) *dr(2) *dr(6) *dr(6) *dr(6) 
      terms(185)=coeff(185) *dr(2) *dr(5) 
      terms(186)=coeff(186) *dr(2) *dr(5) *dr(6) 
      terms(187)=coeff(187) *dr(2) *dr(5) *dr(6) *dr(6) 
      terms(188)=coeff(188) *dr(2) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(189)=coeff(189) *dr(2) *dr(5) *dr(5) 
      terms(190)=coeff(190) *dr(2) *dr(5) *dr(5) *dr(6) 
      terms(191)=coeff(191) *dr(2) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(192)=coeff(192) *dr(2) *dr(5) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(193)=coeff(193) *dr(2) *dr(5) *dr(5) *dr(5) 
      terms(194)=coeff(194) *dr(2) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(195)=coeff(195) *dr(2) *dr(5) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(196)=coeff(196) *dr(2) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(197)=coeff(197) *dr(2) *dr(5) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(198)=coeff(198) *dr(2) *dr(5) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(199)=coeff(199) *dr(2) *dr(4) 
      terms(200)=coeff(200) *dr(2) *dr(4) *dr(6) 
      terms(201)=coeff(201) *dr(2) *dr(4) *dr(6) *dr(6) 
      terms(202)=coeff(202) *dr(2) *dr(4) *dr(6) *dr(6) *dr(6) 
      terms(203)=coeff(203) *dr(2) *dr(4) *dr(5) 
      terms(204)=coeff(204) *dr(2) *dr(4) *dr(5) *dr(6) 
      terms(205)=coeff(205) *dr(2) *dr(4) *dr(5) *dr(6) *dr(6) 
      terms(206)=coeff(206) *dr(2) *dr(4) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(207)=coeff(207) *dr(2) *dr(4) *dr(5) *dr(5) 
      terms(208)=coeff(208) *dr(2) *dr(4) *dr(5) *dr(5) *dr(6) 
      terms(209)=coeff(209) *dr(2) *dr(4) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(210)=coeff(210) *dr(2) *dr(4) *dr(5) *dr(5) *dr(5) 
      terms(211)=coeff(211) *dr(2) *dr(4) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(212)=coeff(212) *dr(2) *dr(4) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(213)=coeff(213) *dr(2) *dr(4) *dr(4) 
      terms(214)=coeff(214) *dr(2) *dr(4) *dr(4) *dr(6) 
      terms(215)=coeff(215) *dr(2) *dr(4) *dr(4) *dr(6) *dr(6) 
      terms(216)=coeff(216) *dr(2) *dr(4) *dr(4) *dr(6) *dr(6) *dr(6) 
      terms(217)=coeff(217) *dr(2) *dr(4) *dr(4) *dr(5) 
      terms(218)=coeff(218) *dr(2) *dr(4) *dr(4) *dr(5) *dr(6) 
      terms(219)=coeff(219) *dr(2) *dr(4) *dr(4) *dr(5) *dr(6) *dr(6) 
      terms(220)=coeff(220) *dr(2) *dr(4) *dr(4) *dr(5) *dr(5) 
      terms(221)=coeff(221) *dr(2) *dr(4) *dr(4) *dr(5) *dr(5) *dr(6) 
      terms(222)=coeff(222) *dr(2) *dr(4) *dr(4) *dr(5) *dr(5) *dr(5) 
      terms(223)=coeff(223) *dr(2) *dr(4) *dr(4) *dr(4) 
      terms(224)=coeff(224) *dr(2) *dr(4) *dr(4) *dr(4) *dr(6) 
      terms(225)=coeff(225) *dr(2) *dr(4) *dr(4) *dr(4) *dr(6) *dr(6) 
      terms(226)=coeff(226) *dr(2) *dr(4) *dr(4) *dr(4) *dr(5) 
      terms(227)=coeff(227) *dr(2) *dr(4) *dr(4) *dr(4) *dr(5) *dr(6) 
      terms(228)=coeff(228) *dr(2) *dr(4) *dr(4) *dr(4) *dr(5) *dr(5) 
      terms(229)=coeff(229) *dr(2) *dr(4) *dr(4) *dr(4) *dr(4) 
      terms(230)=coeff(230) *dr(2) *dr(4) *dr(4) *dr(4) *dr(4) *dr(6) 
      terms(231)=coeff(231) *dr(2) *dr(4) *dr(4) *dr(4) *dr(4) *dr(5) 
      terms(232)=coeff(232) *dr(2) *dr(4) *dr(4) *dr(4) *dr(4) *dr(4) 
      terms(233)=coeff(233) *dr(2) *dr(3) 
      terms(234)=coeff(234) *dr(2) *dr(3) *dr(6) 
      terms(235)=coeff(235) *dr(2) *dr(3) *dr(6) *dr(6) 
      terms(236)=coeff(236) *dr(2) *dr(3) *dr(6) *dr(6) *dr(6) 
      terms(237)=coeff(237) *dr(2) *dr(3) *dr(5) 
      terms(238)=coeff(238) *dr(2) *dr(3) *dr(5) *dr(6) 
      terms(239)=coeff(239) *dr(2) *dr(3) *dr(5) *dr(6) *dr(6) 
      terms(240)=coeff(240) *dr(2) *dr(3) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(241)=coeff(241) *dr(2) *dr(3) *dr(5) *dr(5) 
      terms(242)=coeff(242) *dr(2) *dr(3) *dr(5) *dr(5) *dr(6) 
      terms(243)=coeff(243) *dr(2) *dr(3) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(244)=coeff(244) *dr(2) *dr(3) *dr(5) *dr(5) *dr(5) 
      terms(245)=coeff(245) *dr(2) *dr(3) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(246)=coeff(246) *dr(2) *dr(3) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(247)=coeff(247) *dr(2) *dr(3) *dr(4) 
      terms(248)=coeff(248) *dr(2) *dr(3) *dr(4) *dr(6) 
      terms(249)=coeff(249) *dr(2) *dr(3) *dr(4) *dr(6) *dr(6) 
      terms(250)=coeff(250) *dr(2) *dr(3) *dr(4) *dr(6) *dr(6) *dr(6) 
      terms(251)=coeff(251) *dr(2) *dr(3) *dr(4) *dr(5) 
      terms(252)=coeff(252) *dr(2) *dr(3) *dr(4) *dr(5) *dr(6) 
      terms(253)=coeff(253) *dr(2) *dr(3) *dr(4) *dr(5) *dr(6) *dr(6) 
      terms(254)=coeff(254) *dr(2) *dr(3) *dr(4) *dr(5) *dr(5) 
      terms(255)=coeff(255) *dr(2) *dr(3) *dr(4) *dr(5) *dr(5) *dr(6) 
      terms(256)=coeff(256) *dr(2) *dr(3) *dr(4) *dr(5) *dr(5) *dr(5) 
      terms(257)=coeff(257) *dr(2) *dr(3) *dr(4) *dr(4) 
      terms(258)=coeff(258) *dr(2) *dr(3) *dr(4) *dr(4) *dr(6) 
      terms(259)=coeff(259) *dr(2) *dr(3) *dr(4) *dr(4) *dr(6) *dr(6) 
      terms(260)=coeff(260) *dr(2) *dr(3) *dr(4) *dr(4) *dr(5) 
      terms(261)=coeff(261) *dr(2) *dr(3) *dr(4) *dr(4) *dr(5) *dr(6) 
      terms(262)=coeff(262) *dr(2) *dr(3) *dr(4) *dr(4) *dr(5) *dr(5) 
      terms(263)=coeff(263) *dr(2) *dr(3) *dr(4) *dr(4) *dr(4) 
      terms(264)=coeff(264) *dr(2) *dr(3) *dr(4) *dr(4) *dr(4) *dr(6) 
      terms(265)=coeff(265) *dr(2) *dr(3) *dr(4) *dr(4) *dr(4) *dr(5) 
      terms(266)=coeff(266) *dr(2) *dr(3) *dr(4) *dr(4) *dr(4) *dr(4) 
      terms(267)=coeff(267) *dr(2) *dr(3) *dr(3) 
      terms(268)=coeff(268) *dr(2) *dr(3) *dr(3) *dr(6) 
      terms(269)=coeff(269) *dr(2) *dr(3) *dr(3) *dr(6) *dr(6) 
      terms(270)=coeff(270) *dr(2) *dr(3) *dr(3) *dr(6) *dr(6) *dr(6) 
      terms(271)=coeff(271) *dr(2) *dr(3) *dr(3) *dr(5) 
      terms(272)=coeff(272) *dr(2) *dr(3) *dr(3) *dr(5) *dr(6) 
      terms(273)=coeff(273) *dr(2) *dr(3) *dr(3) *dr(5) *dr(6) *dr(6) 
      terms(274)=coeff(274) *dr(2) *dr(3) *dr(3) *dr(5) *dr(5) 
      terms(275)=coeff(275) *dr(2) *dr(3) *dr(3) *dr(5) *dr(5) *dr(6) 
      terms(276)=coeff(276) *dr(2) *dr(3) *dr(3) *dr(5) *dr(5) *dr(5) 
      terms(277)=coeff(277) *dr(2) *dr(3) *dr(3) *dr(4) 
      terms(278)=coeff(278) *dr(2) *dr(3) *dr(3) *dr(4) *dr(6) 
      terms(279)=coeff(279) *dr(2) *dr(3) *dr(3) *dr(4) *dr(6) *dr(6) 
      terms(280)=coeff(280) *dr(2) *dr(3) *dr(3) *dr(4) *dr(5) 
      terms(281)=coeff(281) *dr(2) *dr(3) *dr(3) *dr(4) *dr(5) *dr(6) 
      terms(282)=coeff(282) *dr(2) *dr(3) *dr(3) *dr(4) *dr(5) *dr(5) 
      terms(283)=coeff(283) *dr(2) *dr(3) *dr(3) *dr(4) *dr(4) 
      terms(284)=coeff(284) *dr(2) *dr(3) *dr(3) *dr(4) *dr(4) *dr(6) 
      terms(285)=coeff(285) *dr(2) *dr(3) *dr(3) *dr(4) *dr(4) *dr(5) 
      terms(286)=coeff(286) *dr(2) *dr(3) *dr(3) *dr(4) *dr(4) *dr(4) 
      terms(287)=coeff(287) *dr(2) *dr(3) *dr(3) *dr(3) 
      terms(288)=coeff(288) *dr(2) *dr(3) *dr(3) *dr(3) *dr(6) 
      terms(289)=coeff(289) *dr(2) *dr(3) *dr(3) *dr(3) *dr(6) *dr(6) 
      terms(290)=coeff(290) *dr(2) *dr(3) *dr(3) *dr(3) *dr(5) 
      terms(291)=coeff(291) *dr(2) *dr(3) *dr(3) *dr(3) *dr(5) *dr(6) 
      terms(292)=coeff(292) *dr(2) *dr(3) *dr(3) *dr(3) *dr(5) *dr(5) 
      terms(293)=coeff(293) *dr(2) *dr(3) *dr(3) *dr(3) *dr(4) 
      terms(294)=coeff(294) *dr(2) *dr(3) *dr(3) *dr(3) *dr(4) *dr(6) 
      terms(295)=coeff(295) *dr(2) *dr(3) *dr(3) *dr(3) *dr(4) *dr(5) 
      terms(296)=coeff(296) *dr(2) *dr(3) *dr(3) *dr(3) *dr(4) *dr(4) 
      terms(297)=coeff(297) *dr(2) *dr(2) 
      terms(298)=coeff(298) *dr(2) *dr(2) *dr(6) 
      terms(299)=coeff(299) *dr(2) *dr(2) *dr(6) *dr(6) 
      terms(300)=coeff(300) *dr(2) *dr(2) *dr(6) *dr(6) *dr(6) 
      terms(301)=coeff(301) *dr(2) *dr(2) *dr(5) 
      terms(302)=coeff(302) *dr(2) *dr(2) *dr(5) *dr(6) 
      terms(303)=coeff(303) *dr(2) *dr(2) *dr(5) *dr(6) *dr(6) 
      terms(304)=coeff(304) *dr(2) *dr(2) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(305)=coeff(305) *dr(2) *dr(2) *dr(5) *dr(5) 
      terms(306)=coeff(306) *dr(2) *dr(2) *dr(5) *dr(5) *dr(6) 
      terms(307)=coeff(307) *dr(2) *dr(2) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(308)=coeff(308) *dr(2) *dr(2) *dr(5) *dr(5) *dr(5) 
      terms(309)=coeff(309) *dr(2) *dr(2) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(310)=coeff(310) *dr(2) *dr(2) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(311)=coeff(311) *dr(2) *dr(2) *dr(4) 
      terms(312)=coeff(312) *dr(2) *dr(2) *dr(4) *dr(6) 
      terms(313)=coeff(313) *dr(2) *dr(2) *dr(4) *dr(6) *dr(6) 
      terms(314)=coeff(314) *dr(2) *dr(2) *dr(4) *dr(6) *dr(6) *dr(6) 
      terms(315)=coeff(315) *dr(2) *dr(2) *dr(4) *dr(5) 
      terms(316)=coeff(316) *dr(2) *dr(2) *dr(4) *dr(5) *dr(6) 
      terms(317)=coeff(317) *dr(2) *dr(2) *dr(4) *dr(5) *dr(6) *dr(6) 
      terms(318)=coeff(318) *dr(2) *dr(2) *dr(4) *dr(5) *dr(5) 
      terms(319)=coeff(319) *dr(2) *dr(2) *dr(4) *dr(5) *dr(5) *dr(6) 
      terms(320)=coeff(320) *dr(2) *dr(2) *dr(4) *dr(5) *dr(5) *dr(5) 
      terms(321)=coeff(321) *dr(2) *dr(2) *dr(4) *dr(4) 
      terms(322)=coeff(322) *dr(2) *dr(2) *dr(4) *dr(4) *dr(6) 
      terms(323)=coeff(323) *dr(2) *dr(2) *dr(4) *dr(4) *dr(6) *dr(6) 
      terms(324)=coeff(324) *dr(2) *dr(2) *dr(4) *dr(4) *dr(5) 
      terms(325)=coeff(325) *dr(2) *dr(2) *dr(4) *dr(4) *dr(5) *dr(6) 
      terms(326)=coeff(326) *dr(2) *dr(2) *dr(4) *dr(4) *dr(5) *dr(5) 
      terms(327)=coeff(327) *dr(2) *dr(2) *dr(4) *dr(4) *dr(4) 
      terms(328)=coeff(328) *dr(2) *dr(2) *dr(4) *dr(4) *dr(4) *dr(6) 
      terms(329)=coeff(329) *dr(2) *dr(2) *dr(4) *dr(4) *dr(4) *dr(5) 
      terms(330)=coeff(330) *dr(2) *dr(2) *dr(4) *dr(4) *dr(4) *dr(4) 
      terms(331)=coeff(331) *dr(2) *dr(2) *dr(3) 
      terms(332)=coeff(332) *dr(2) *dr(2) *dr(3) *dr(6) 
      terms(333)=coeff(333) *dr(2) *dr(2) *dr(3) *dr(6) *dr(6) 
      terms(334)=coeff(334) *dr(2) *dr(2) *dr(3) *dr(6) *dr(6) *dr(6) 
      terms(335)=coeff(335) *dr(2) *dr(2) *dr(3) *dr(5) 
      terms(336)=coeff(336) *dr(2) *dr(2) *dr(3) *dr(5) *dr(6) 
      terms(337)=coeff(337) *dr(2) *dr(2) *dr(3) *dr(5) *dr(6) *dr(6) 
      terms(338)=coeff(338) *dr(2) *dr(2) *dr(3) *dr(5) *dr(5) 
      terms(339)=coeff(339) *dr(2) *dr(2) *dr(3) *dr(5) *dr(5) *dr(6) 
      terms(340)=coeff(340) *dr(2) *dr(2) *dr(3) *dr(5) *dr(5) *dr(5) 
      terms(341)=coeff(341) *dr(2) *dr(2) *dr(3) *dr(4) 
      terms(342)=coeff(342) *dr(2) *dr(2) *dr(3) *dr(4) *dr(6) 
      terms(343)=coeff(343) *dr(2) *dr(2) *dr(3) *dr(4) *dr(6) *dr(6) 
      terms(344)=coeff(344) *dr(2) *dr(2) *dr(3) *dr(4) *dr(5) 
      terms(345)=coeff(345) *dr(2) *dr(2) *dr(3) *dr(4) *dr(5) *dr(6) 
      terms(346)=coeff(346) *dr(2) *dr(2) *dr(3) *dr(4) *dr(5) *dr(5) 
      terms(347)=coeff(347) *dr(2) *dr(2) *dr(3) *dr(4) *dr(4) 
      terms(348)=coeff(348) *dr(2) *dr(2) *dr(3) *dr(4) *dr(4) *dr(6) 
      terms(349)=coeff(349) *dr(2) *dr(2) *dr(3) *dr(4) *dr(4) *dr(5) 
      terms(350)=coeff(350) *dr(2) *dr(2) *dr(3) *dr(4) *dr(4) *dr(4) 
      terms(351)=coeff(351) *dr(2) *dr(2) *dr(3) *dr(3) 
      terms(352)=coeff(352) *dr(2) *dr(2) *dr(3) *dr(3) *dr(6) 
      terms(353)=coeff(353) *dr(2) *dr(2) *dr(3) *dr(3) *dr(6) *dr(6) 
      terms(354)=coeff(354) *dr(2) *dr(2) *dr(3) *dr(3) *dr(5) 
      terms(355)=coeff(355) *dr(2) *dr(2) *dr(3) *dr(3) *dr(5) *dr(6) 
      terms(356)=coeff(356) *dr(2) *dr(2) *dr(3) *dr(3) *dr(5) *dr(5) 
      terms(357)=coeff(357) *dr(2) *dr(2) *dr(3) *dr(3) *dr(4) 
      terms(358)=coeff(358) *dr(2) *dr(2) *dr(3) *dr(3) *dr(4) *dr(6) 
      terms(359)=coeff(359) *dr(2) *dr(2) *dr(3) *dr(3) *dr(4) *dr(5) 
      terms(360)=coeff(360) *dr(2) *dr(2) *dr(3) *dr(3) *dr(4) *dr(4) 
      terms(361)=coeff(361) *dr(2) *dr(2) *dr(3) *dr(3) *dr(3) 
      terms(362)=coeff(362) *dr(2) *dr(2) *dr(3) *dr(3) *dr(3) *dr(6) 
      terms(363)=coeff(363) *dr(2) *dr(2) *dr(3) *dr(3) *dr(3) *dr(5) 
      terms(364)=coeff(364) *dr(2) *dr(2) *dr(3) *dr(3) *dr(3) *dr(4) 
      terms(365)=coeff(365) *dr(2) *dr(2) *dr(2) 
      terms(366)=coeff(366) *dr(2) *dr(2) *dr(2) *dr(6) 
      terms(367)=coeff(367) *dr(2) *dr(2) *dr(2) *dr(6) *dr(6) 
      terms(368)=coeff(368) *dr(2) *dr(2) *dr(2) *dr(6) *dr(6) *dr(6) 
      terms(369)=coeff(369) *dr(2) *dr(2) *dr(2) *dr(5) 
      terms(370)=coeff(370) *dr(2) *dr(2) *dr(2) *dr(5) *dr(6) 
      terms(371)=coeff(371) *dr(2) *dr(2) *dr(2) *dr(5) *dr(6) *dr(6) 
      terms(372)=coeff(372) *dr(2) *dr(2) *dr(2) *dr(5) *dr(5) 
      terms(373)=coeff(373) *dr(2) *dr(2) *dr(2) *dr(5) *dr(5) *dr(6) 
      terms(374)=coeff(374) *dr(2) *dr(2) *dr(2) *dr(5) *dr(5) *dr(5) 
      terms(375)=coeff(375) *dr(2) *dr(2) *dr(2) *dr(4) 
      terms(376)=coeff(376) *dr(2) *dr(2) *dr(2) *dr(4) *dr(6) 
      terms(377)=coeff(377) *dr(2) *dr(2) *dr(2) *dr(4) *dr(6) *dr(6) 
      terms(378)=coeff(378) *dr(2) *dr(2) *dr(2) *dr(4) *dr(5) 
      terms(379)=coeff(379) *dr(2) *dr(2) *dr(2) *dr(4) *dr(5) *dr(6) 
      terms(380)=coeff(380) *dr(2) *dr(2) *dr(2) *dr(4) *dr(5) *dr(5) 
      terms(381)=coeff(381) *dr(2) *dr(2) *dr(2) *dr(4) *dr(4) 
      terms(382)=coeff(382) *dr(2) *dr(2) *dr(2) *dr(4) *dr(4) *dr(6) 
      terms(383)=coeff(383) *dr(2) *dr(2) *dr(2) *dr(4) *dr(4) *dr(5) 
      terms(384)=coeff(384) *dr(2) *dr(2) *dr(2) *dr(4) *dr(4) *dr(4) 
      terms(385)=coeff(385) *dr(2) *dr(2) *dr(2) *dr(3) 
      terms(386)=coeff(386) *dr(2) *dr(2) *dr(2) *dr(3) *dr(6) 
      terms(387)=coeff(387) *dr(2) *dr(2) *dr(2) *dr(3) *dr(6) *dr(6) 
      terms(388)=coeff(388) *dr(2) *dr(2) *dr(2) *dr(3) *dr(5) 
      terms(389)=coeff(389) *dr(2) *dr(2) *dr(2) *dr(3) *dr(5) *dr(6) 
      terms(390)=coeff(390) *dr(2) *dr(2) *dr(2) *dr(3) *dr(5) *dr(5) 
      terms(391)=coeff(391) *dr(2) *dr(2) *dr(2) *dr(3) *dr(4) 
      terms(392)=coeff(392) *dr(2) *dr(2) *dr(2) *dr(3) *dr(4) *dr(6) 
      terms(393)=coeff(393) *dr(2) *dr(2) *dr(2) *dr(3) *dr(4) *dr(5) 
      terms(394)=coeff(394) *dr(2) *dr(2) *dr(2) *dr(3) *dr(4) *dr(4) 
      terms(395)=coeff(395) *dr(2) *dr(2) *dr(2) *dr(3) *dr(3) 
      terms(396)=coeff(396) *dr(2) *dr(2) *dr(2) *dr(3) *dr(3) *dr(6) 
      terms(397)=coeff(397) *dr(2) *dr(2) *dr(2) *dr(3) *dr(3) *dr(5) 
      terms(398)=coeff(398) *dr(2) *dr(2) *dr(2) *dr(3) *dr(3) *dr(4) 
      terms(399)=coeff(399) *dr(2) *dr(2) *dr(2) *dr(3) *dr(3) *dr(3) 
      terms(400)=coeff(400) *dr(2) *dr(2) *dr(2) *dr(2) 
      terms(401)=coeff(401) *dr(2) *dr(2) *dr(2) *dr(2) *dr(6) 
      terms(402)=coeff(402) *dr(2) *dr(2) *dr(2) *dr(2) *dr(6) *dr(6) 
      terms(403)=coeff(403) *dr(2) *dr(2) *dr(2) *dr(2) *dr(5) 
      terms(404)=coeff(404) *dr(2) *dr(2) *dr(2) *dr(2) *dr(5) *dr(6) 
      terms(405)=coeff(405) *dr(2) *dr(2) *dr(2) *dr(2) *dr(5) *dr(5) 
      terms(406)=coeff(406) *dr(2) *dr(2) *dr(2) *dr(2) *dr(4) 
      terms(407)=coeff(407) *dr(2) *dr(2) *dr(2) *dr(2) *dr(4) *dr(6) 
      terms(408)=coeff(408) *dr(2) *dr(2) *dr(2) *dr(2) *dr(4) *dr(5) 
      terms(409)=coeff(409) *dr(2) *dr(2) *dr(2) *dr(2) *dr(4) *dr(4) 
      terms(410)=coeff(410) *dr(2) *dr(2) *dr(2) *dr(2) *dr(3) 
      terms(411)=coeff(411) *dr(2) *dr(2) *dr(2) *dr(2) *dr(3) *dr(6) 
      terms(412)=coeff(412) *dr(2) *dr(2) *dr(2) *dr(2) *dr(3) *dr(5) 
      terms(413)=coeff(413) *dr(2) *dr(2) *dr(2) *dr(2) *dr(3) *dr(4) 
      terms(414)=coeff(414) *dr(2) *dr(2) *dr(2) *dr(2) *dr(3) *dr(3) 
      terms(415)=coeff(415) *dr(2) *dr(2) *dr(2) *dr(2) *dr(2) 
      terms(416)=coeff(416) *dr(2) *dr(2) *dr(2) *dr(2) *dr(2) *dr(6) 
      terms(417)=coeff(417) *dr(2) *dr(2) *dr(2) *dr(2) *dr(2) *dr(5) 
      terms(418)=coeff(418) *dr(2) *dr(2) *dr(2) *dr(2) *dr(2) *dr(4) 
      terms(419)=coeff(419) *dr(2) *dr(2) *dr(2) *dr(2) *dr(2) *dr(3) 
      terms(420)=coeff(420) *dr(2) *dr(2) *dr(2) *dr(2) *dr(2) *dr(2) 
      terms(421)=coeff(421) *dr(1) 
      terms(422)=coeff(422) *dr(1) *dr(6) 
      terms(423)=coeff(423) *dr(1) *dr(6) *dr(6) 
      terms(424)=coeff(424) *dr(1) *dr(6) *dr(6) *dr(6) 
      terms(425)=coeff(425) *dr(1) *dr(5) 
      terms(426)=coeff(426) *dr(1) *dr(5) *dr(6) 
      terms(427)=coeff(427) *dr(1) *dr(5) *dr(6) *dr(6) 
      terms(428)=coeff(428) *dr(1) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(429)=coeff(429) *dr(1) *dr(5) *dr(5) 
      terms(430)=coeff(430) *dr(1) *dr(5) *dr(5) *dr(6) 
      terms(431)=coeff(431) *dr(1) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(432)=coeff(432) *dr(1) *dr(5) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(433)=coeff(433) *dr(1) *dr(5) *dr(5) *dr(5) 
      terms(434)=coeff(434) *dr(1) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(435)=coeff(435) *dr(1) *dr(5) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(436)=coeff(436) *dr(1) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(437)=coeff(437) *dr(1) *dr(5) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(438)=coeff(438) *dr(1) *dr(5) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(439)=coeff(439) *dr(1) *dr(4) 
      terms(440)=coeff(440) *dr(1) *dr(4) *dr(6) 
      terms(441)=coeff(441) *dr(1) *dr(4) *dr(6) *dr(6) 
      terms(442)=coeff(442) *dr(1) *dr(4) *dr(6) *dr(6) *dr(6) 
      terms(443)=coeff(443) *dr(1) *dr(4) *dr(5) 
      terms(444)=coeff(444) *dr(1) *dr(4) *dr(5) *dr(6) 
      terms(445)=coeff(445) *dr(1) *dr(4) *dr(5) *dr(6) *dr(6) 
      terms(446)=coeff(446) *dr(1) *dr(4) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(447)=coeff(447) *dr(1) *dr(4) *dr(5) *dr(5) 
      terms(448)=coeff(448) *dr(1) *dr(4) *dr(5) *dr(5) *dr(6) 
      terms(449)=coeff(449) *dr(1) *dr(4) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(450)=coeff(450) *dr(1) *dr(4) *dr(5) *dr(5) *dr(5) 
      terms(451)=coeff(451) *dr(1) *dr(4) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(452)=coeff(452) *dr(1) *dr(4) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(453)=coeff(453) *dr(1) *dr(4) *dr(4) 
      terms(454)=coeff(454) *dr(1) *dr(4) *dr(4) *dr(6) 
      terms(455)=coeff(455) *dr(1) *dr(4) *dr(4) *dr(6) *dr(6) 
      terms(456)=coeff(456) *dr(1) *dr(4) *dr(4) *dr(6) *dr(6) *dr(6) 
      terms(457)=coeff(457) *dr(1) *dr(4) *dr(4) *dr(5) 
      terms(458)=coeff(458) *dr(1) *dr(4) *dr(4) *dr(5) *dr(6) 
      terms(459)=coeff(459) *dr(1) *dr(4) *dr(4) *dr(5) *dr(6) *dr(6) 
      terms(460)=coeff(460) *dr(1) *dr(4) *dr(4) *dr(5) *dr(5) 
      terms(461)=coeff(461) *dr(1) *dr(4) *dr(4) *dr(5) *dr(5) *dr(6) 
      terms(462)=coeff(462) *dr(1) *dr(4) *dr(4) *dr(5) *dr(5) *dr(5) 
      terms(463)=coeff(463) *dr(1) *dr(4) *dr(4) *dr(4) 
      terms(464)=coeff(464) *dr(1) *dr(4) *dr(4) *dr(4) *dr(6) 
      terms(465)=coeff(465) *dr(1) *dr(4) *dr(4) *dr(4) *dr(6) *dr(6) 
      terms(466)=coeff(466) *dr(1) *dr(4) *dr(4) *dr(4) *dr(5) 
      terms(467)=coeff(467) *dr(1) *dr(4) *dr(4) *dr(4) *dr(5) *dr(6) 
      terms(468)=coeff(468) *dr(1) *dr(4) *dr(4) *dr(4) *dr(5) *dr(5) 
      terms(469)=coeff(469) *dr(1) *dr(4) *dr(4) *dr(4) *dr(4) 
      terms(470)=coeff(470) *dr(1) *dr(4) *dr(4) *dr(4) *dr(4) *dr(6) 
      terms(471)=coeff(471) *dr(1) *dr(4) *dr(4) *dr(4) *dr(4) *dr(5) 
      terms(472)=coeff(472) *dr(1) *dr(4) *dr(4) *dr(4) *dr(4) *dr(4) 
      terms(473)=coeff(473) *dr(1) *dr(3) 
      terms(474)=coeff(474) *dr(1) *dr(3) *dr(6) 
      terms(475)=coeff(475) *dr(1) *dr(3) *dr(6) *dr(6) 
      terms(476)=coeff(476) *dr(1) *dr(3) *dr(6) *dr(6) *dr(6) 
      terms(477)=coeff(477) *dr(1) *dr(3) *dr(5) 
      terms(478)=coeff(478) *dr(1) *dr(3) *dr(5) *dr(6) 
      terms(479)=coeff(479) *dr(1) *dr(3) *dr(5) *dr(6) *dr(6) 
      terms(480)=coeff(480) *dr(1) *dr(3) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(481)=coeff(481) *dr(1) *dr(3) *dr(5) *dr(5) 
      terms(482)=coeff(482) *dr(1) *dr(3) *dr(5) *dr(5) *dr(6) 
      terms(483)=coeff(483) *dr(1) *dr(3) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(484)=coeff(484) *dr(1) *dr(3) *dr(5) *dr(5) *dr(5) 
      terms(485)=coeff(485) *dr(1) *dr(3) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(486)=coeff(486) *dr(1) *dr(3) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(487)=coeff(487) *dr(1) *dr(3) *dr(4) 
      terms(488)=coeff(488) *dr(1) *dr(3) *dr(4) *dr(6) 
      terms(489)=coeff(489) *dr(1) *dr(3) *dr(4) *dr(6) *dr(6) 
      terms(490)=coeff(490) *dr(1) *dr(3) *dr(4) *dr(6) *dr(6) *dr(6) 
      terms(491)=coeff(491) *dr(1) *dr(3) *dr(4) *dr(5) 
      terms(492)=coeff(492) *dr(1) *dr(3) *dr(4) *dr(5) *dr(6) 
      terms(493)=coeff(493) *dr(1) *dr(3) *dr(4) *dr(5) *dr(6) *dr(6) 
      terms(494)=coeff(494) *dr(1) *dr(3) *dr(4) *dr(5) *dr(5) 
      terms(495)=coeff(495) *dr(1) *dr(3) *dr(4) *dr(5) *dr(5) *dr(6) 
      terms(496)=coeff(496) *dr(1) *dr(3) *dr(4) *dr(5) *dr(5) *dr(5) 
      terms(497)=coeff(497) *dr(1) *dr(3) *dr(4) *dr(4) 
      terms(498)=coeff(498) *dr(1) *dr(3) *dr(4) *dr(4) *dr(6) 
      terms(499)=coeff(499) *dr(1) *dr(3) *dr(4) *dr(4) *dr(6) *dr(6) 
      terms(500)=coeff(500) *dr(1) *dr(3) *dr(4) *dr(4) *dr(5) 
      terms(501)=coeff(501) *dr(1) *dr(3) *dr(4) *dr(4) *dr(5) *dr(6) 
      terms(502)=coeff(502) *dr(1) *dr(3) *dr(4) *dr(4) *dr(5) *dr(5) 
      terms(503)=coeff(503) *dr(1) *dr(3) *dr(4) *dr(4) *dr(4) 
      terms(504)=coeff(504) *dr(1) *dr(3) *dr(4) *dr(4) *dr(4) *dr(6) 
      terms(505)=coeff(505) *dr(1) *dr(3) *dr(4) *dr(4) *dr(4) *dr(5) 
      terms(506)=coeff(506) *dr(1) *dr(3) *dr(4) *dr(4) *dr(4) *dr(4) 
      terms(507)=coeff(507) *dr(1) *dr(3) *dr(3) 
      terms(508)=coeff(508) *dr(1) *dr(3) *dr(3) *dr(6) 
      terms(509)=coeff(509) *dr(1) *dr(3) *dr(3) *dr(6) *dr(6) 
      terms(510)=coeff(510) *dr(1) *dr(3) *dr(3) *dr(6) *dr(6) *dr(6) 
      terms(511)=coeff(511) *dr(1) *dr(3) *dr(3) *dr(5) 
      terms(512)=coeff(512) *dr(1) *dr(3) *dr(3) *dr(5) *dr(6) 
      terms(513)=coeff(513) *dr(1) *dr(3) *dr(3) *dr(5) *dr(6) *dr(6) 
      terms(514)=coeff(514) *dr(1) *dr(3) *dr(3) *dr(5) *dr(5) 
      terms(515)=coeff(515) *dr(1) *dr(3) *dr(3) *dr(5) *dr(5) *dr(6) 
      terms(516)=coeff(516) *dr(1) *dr(3) *dr(3) *dr(5) *dr(5) *dr(5) 
      terms(517)=coeff(517) *dr(1) *dr(3) *dr(3) *dr(4) 
      terms(518)=coeff(518) *dr(1) *dr(3) *dr(3) *dr(4) *dr(6) 
      terms(519)=coeff(519) *dr(1) *dr(3) *dr(3) *dr(4) *dr(6) *dr(6) 
      terms(520)=coeff(520) *dr(1) *dr(3) *dr(3) *dr(4) *dr(5) 
      terms(521)=coeff(521) *dr(1) *dr(3) *dr(3) *dr(4) *dr(5) *dr(6) 
      terms(522)=coeff(522) *dr(1) *dr(3) *dr(3) *dr(4) *dr(5) *dr(5) 
      terms(523)=coeff(523) *dr(1) *dr(3) *dr(3) *dr(4) *dr(4) 
      terms(524)=coeff(524) *dr(1) *dr(3) *dr(3) *dr(4) *dr(4) *dr(6) 
      terms(525)=coeff(525) *dr(1) *dr(3) *dr(3) *dr(4) *dr(4) *dr(5) 
      terms(526)=coeff(526) *dr(1) *dr(3) *dr(3) *dr(4) *dr(4) *dr(4) 
      terms(527)=coeff(527) *dr(1) *dr(3) *dr(3) *dr(3) 
      terms(528)=coeff(528) *dr(1) *dr(3) *dr(3) *dr(3) *dr(6) 
      terms(529)=coeff(529) *dr(1) *dr(3) *dr(3) *dr(3) *dr(6) *dr(6) 
      terms(530)=coeff(530) *dr(1) *dr(3) *dr(3) *dr(3) *dr(5) 
      terms(531)=coeff(531) *dr(1) *dr(3) *dr(3) *dr(3) *dr(5) *dr(6) 
      terms(532)=coeff(532) *dr(1) *dr(3) *dr(3) *dr(3) *dr(5) *dr(5) 
      terms(533)=coeff(533) *dr(1) *dr(3) *dr(3) *dr(3) *dr(4) 
      terms(534)=coeff(534) *dr(1) *dr(3) *dr(3) *dr(3) *dr(4) *dr(6) 
      terms(535)=coeff(535) *dr(1) *dr(3) *dr(3) *dr(3) *dr(4) *dr(5) 
      terms(536)=coeff(536) *dr(1) *dr(3) *dr(3) *dr(3) *dr(4) *dr(4) 
      terms(537)=coeff(537) *dr(1) *dr(2) 
      terms(538)=coeff(538) *dr(1) *dr(2) *dr(6) 
      terms(539)=coeff(539) *dr(1) *dr(2) *dr(6) *dr(6) 
      terms(540)=coeff(540) *dr(1) *dr(2) *dr(6) *dr(6) *dr(6) 
      terms(541)=coeff(541) *dr(1) *dr(2) *dr(5) 
      terms(542)=coeff(542) *dr(1) *dr(2) *dr(5) *dr(6) 
      terms(543)=coeff(543) *dr(1) *dr(2) *dr(5) *dr(6) *dr(6) 
      terms(544)=coeff(544) *dr(1) *dr(2) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(545)=coeff(545) *dr(1) *dr(2) *dr(5) *dr(5) 
      terms(546)=coeff(546) *dr(1) *dr(2) *dr(5) *dr(5) *dr(6) 
      terms(547)=coeff(547) *dr(1) *dr(2) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(548)=coeff(548) *dr(1) *dr(2) *dr(5) *dr(5) *dr(5) 
      terms(549)=coeff(549) *dr(1) *dr(2) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(550)=coeff(550) *dr(1) *dr(2) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(551)=coeff(551) *dr(1) *dr(2) *dr(4) 
      terms(552)=coeff(552) *dr(1) *dr(2) *dr(4) *dr(6) 
      terms(553)=coeff(553) *dr(1) *dr(2) *dr(4) *dr(6) *dr(6) 
      terms(554)=coeff(554) *dr(1) *dr(2) *dr(4) *dr(6) *dr(6) *dr(6) 
      terms(555)=coeff(555) *dr(1) *dr(2) *dr(4) *dr(5) 
      terms(556)=coeff(556) *dr(1) *dr(2) *dr(4) *dr(5) *dr(6) 
      terms(557)=coeff(557) *dr(1) *dr(2) *dr(4) *dr(5) *dr(6) *dr(6) 
      terms(558)=coeff(558) *dr(1) *dr(2) *dr(4) *dr(5) *dr(5) 
      terms(559)=coeff(559) *dr(1) *dr(2) *dr(4) *dr(5) *dr(5) *dr(6) 
      terms(560)=coeff(560) *dr(1) *dr(2) *dr(4) *dr(5) *dr(5) *dr(5) 
      terms(561)=coeff(561) *dr(1) *dr(2) *dr(4) *dr(4) 
      terms(562)=coeff(562) *dr(1) *dr(2) *dr(4) *dr(4) *dr(6) 
      terms(563)=coeff(563) *dr(1) *dr(2) *dr(4) *dr(4) *dr(6) *dr(6) 
      terms(564)=coeff(564) *dr(1) *dr(2) *dr(4) *dr(4) *dr(5) 
      terms(565)=coeff(565) *dr(1) *dr(2) *dr(4) *dr(4) *dr(5) *dr(6) 
      terms(566)=coeff(566) *dr(1) *dr(2) *dr(4) *dr(4) *dr(5) *dr(5) 
      terms(567)=coeff(567) *dr(1) *dr(2) *dr(4) *dr(4) *dr(4) 
      terms(568)=coeff(568) *dr(1) *dr(2) *dr(4) *dr(4) *dr(4) *dr(6) 
      terms(569)=coeff(569) *dr(1) *dr(2) *dr(4) *dr(4) *dr(4) *dr(5) 
      terms(570)=coeff(570) *dr(1) *dr(2) *dr(4) *dr(4) *dr(4) *dr(4) 
      terms(571)=coeff(571) *dr(1) *dr(2) *dr(3) 
      terms(572)=coeff(572) *dr(1) *dr(2) *dr(3) *dr(6) 
      terms(573)=coeff(573) *dr(1) *dr(2) *dr(3) *dr(6) *dr(6) 
      terms(574)=coeff(574) *dr(1) *dr(2) *dr(3) *dr(6) *dr(6) *dr(6) 
      terms(575)=coeff(575) *dr(1) *dr(2) *dr(3) *dr(5) 
      terms(576)=coeff(576) *dr(1) *dr(2) *dr(3) *dr(5) *dr(6) 
      terms(577)=coeff(577) *dr(1) *dr(2) *dr(3) *dr(5) *dr(6) *dr(6) 
      terms(578)=coeff(578) *dr(1) *dr(2) *dr(3) *dr(5) *dr(5) 
      terms(579)=coeff(579) *dr(1) *dr(2) *dr(3) *dr(5) *dr(5) *dr(6) 
      terms(580)=coeff(580) *dr(1) *dr(2) *dr(3) *dr(5) *dr(5) *dr(5) 
      terms(581)=coeff(581) *dr(1) *dr(2) *dr(3) *dr(4) 
      terms(582)=coeff(582) *dr(1) *dr(2) *dr(3) *dr(4) *dr(6) 
      terms(583)=coeff(583) *dr(1) *dr(2) *dr(3) *dr(4) *dr(6) *dr(6) 
      terms(584)=coeff(584) *dr(1) *dr(2) *dr(3) *dr(4) *dr(5) 
      terms(585)=coeff(585) *dr(1) *dr(2) *dr(3) *dr(4) *dr(5) *dr(6) 
      terms(586)=coeff(586) *dr(1) *dr(2) *dr(3) *dr(4) *dr(5) *dr(5) 
      terms(587)=coeff(587) *dr(1) *dr(2) *dr(3) *dr(4) *dr(4) 
      terms(588)=coeff(588) *dr(1) *dr(2) *dr(3) *dr(4) *dr(4) *dr(6) 
      terms(589)=coeff(589) *dr(1) *dr(2) *dr(3) *dr(4) *dr(4) *dr(5) 
      terms(590)=coeff(590) *dr(1) *dr(2) *dr(3) *dr(4) *dr(4) *dr(4) 
      terms(591)=coeff(591) *dr(1) *dr(2) *dr(3) *dr(3) 
      terms(592)=coeff(592) *dr(1) *dr(2) *dr(3) *dr(3) *dr(6) 
      terms(593)=coeff(593) *dr(1) *dr(2) *dr(3) *dr(3) *dr(6) *dr(6) 
      terms(594)=coeff(594) *dr(1) *dr(2) *dr(3) *dr(3) *dr(5) 
      terms(595)=coeff(595) *dr(1) *dr(2) *dr(3) *dr(3) *dr(5) *dr(6) 
      terms(596)=coeff(596) *dr(1) *dr(2) *dr(3) *dr(3) *dr(5) *dr(5) 
      terms(597)=coeff(597) *dr(1) *dr(2) *dr(3) *dr(3) *dr(4) 
      terms(598)=coeff(598) *dr(1) *dr(2) *dr(3) *dr(3) *dr(4) *dr(6) 
      terms(599)=coeff(599) *dr(1) *dr(2) *dr(3) *dr(3) *dr(4) *dr(5) 
      terms(600)=coeff(600) *dr(1) *dr(2) *dr(3) *dr(3) *dr(4) *dr(4) 
      terms(601)=coeff(601) *dr(1) *dr(2) *dr(3) *dr(3) *dr(3) 
      terms(602)=coeff(602) *dr(1) *dr(2) *dr(3) *dr(3) *dr(3) *dr(6) 
      terms(603)=coeff(603) *dr(1) *dr(2) *dr(3) *dr(3) *dr(3) *dr(5) 
      terms(604)=coeff(604) *dr(1) *dr(2) *dr(3) *dr(3) *dr(3) *dr(4) 
      terms(605)=coeff(605) *dr(1) *dr(2) *dr(2) 
      terms(606)=coeff(606) *dr(1) *dr(2) *dr(2) *dr(6) 
      terms(607)=coeff(607) *dr(1) *dr(2) *dr(2) *dr(6) *dr(6) 
      terms(608)=coeff(608) *dr(1) *dr(2) *dr(2) *dr(6) *dr(6) *dr(6) 
      terms(609)=coeff(609) *dr(1) *dr(2) *dr(2) *dr(5) 
      terms(610)=coeff(610) *dr(1) *dr(2) *dr(2) *dr(5) *dr(6) 
      terms(611)=coeff(611) *dr(1) *dr(2) *dr(2) *dr(5) *dr(6) *dr(6) 
      terms(612)=coeff(612) *dr(1) *dr(2) *dr(2) *dr(5) *dr(5) 
      terms(613)=coeff(613) *dr(1) *dr(2) *dr(2) *dr(5) *dr(5) *dr(6) 
      terms(614)=coeff(614) *dr(1) *dr(2) *dr(2) *dr(5) *dr(5) *dr(5) 
      terms(615)=coeff(615) *dr(1) *dr(2) *dr(2) *dr(4) 
      terms(616)=coeff(616) *dr(1) *dr(2) *dr(2) *dr(4) *dr(6) 
      terms(617)=coeff(617) *dr(1) *dr(2) *dr(2) *dr(4) *dr(6) *dr(6) 
      terms(618)=coeff(618) *dr(1) *dr(2) *dr(2) *dr(4) *dr(5) 
      terms(619)=coeff(619) *dr(1) *dr(2) *dr(2) *dr(4) *dr(5) *dr(6) 
      terms(620)=coeff(620) *dr(1) *dr(2) *dr(2) *dr(4) *dr(5) *dr(5) 
      terms(621)=coeff(621) *dr(1) *dr(2) *dr(2) *dr(4) *dr(4) 
      terms(622)=coeff(622) *dr(1) *dr(2) *dr(2) *dr(4) *dr(4) *dr(6) 
      terms(623)=coeff(623) *dr(1) *dr(2) *dr(2) *dr(4) *dr(4) *dr(5) 
      terms(624)=coeff(624) *dr(1) *dr(2) *dr(2) *dr(4) *dr(4) *dr(4) 
      terms(625)=coeff(625) *dr(1) *dr(2) *dr(2) *dr(3) 
      terms(626)=coeff(626) *dr(1) *dr(2) *dr(2) *dr(3) *dr(6) 
      terms(627)=coeff(627) *dr(1) *dr(2) *dr(2) *dr(3) *dr(6) *dr(6) 
      terms(628)=coeff(628) *dr(1) *dr(2) *dr(2) *dr(3) *dr(5) 
      terms(629)=coeff(629) *dr(1) *dr(2) *dr(2) *dr(3) *dr(5) *dr(6) 
      terms(630)=coeff(630) *dr(1) *dr(2) *dr(2) *dr(3) *dr(5) *dr(5) 
      terms(631)=coeff(631) *dr(1) *dr(2) *dr(2) *dr(3) *dr(4) 
      terms(632)=coeff(632) *dr(1) *dr(2) *dr(2) *dr(3) *dr(4) *dr(6) 
      terms(633)=coeff(633) *dr(1) *dr(2) *dr(2) *dr(3) *dr(4) *dr(5) 
      terms(634)=coeff(634) *dr(1) *dr(2) *dr(2) *dr(3) *dr(4) *dr(4) 
      terms(635)=coeff(635) *dr(1) *dr(2) *dr(2) *dr(3) *dr(3) 
      terms(636)=coeff(636) *dr(1) *dr(2) *dr(2) *dr(3) *dr(3) *dr(6) 
      terms(637)=coeff(637) *dr(1) *dr(2) *dr(2) *dr(3) *dr(3) *dr(5) 
      terms(638)=coeff(638) *dr(1) *dr(2) *dr(2) *dr(3) *dr(3) *dr(4) 
      terms(639)=coeff(639) *dr(1) *dr(2) *dr(2) *dr(3) *dr(3) *dr(3) 
      terms(640)=coeff(640) *dr(1) *dr(2) *dr(2) *dr(2) 
      terms(641)=coeff(641) *dr(1) *dr(2) *dr(2) *dr(2) *dr(6) 
      terms(642)=coeff(642) *dr(1) *dr(2) *dr(2) *dr(2) *dr(6) *dr(6) 
      terms(643)=coeff(643) *dr(1) *dr(2) *dr(2) *dr(2) *dr(5) 
      terms(644)=coeff(644) *dr(1) *dr(2) *dr(2) *dr(2) *dr(5) *dr(6) 
      terms(645)=coeff(645) *dr(1) *dr(2) *dr(2) *dr(2) *dr(5) *dr(5) 
      terms(646)=coeff(646) *dr(1) *dr(2) *dr(2) *dr(2) *dr(4) 
      terms(647)=coeff(647) *dr(1) *dr(2) *dr(2) *dr(2) *dr(4) *dr(6) 
      terms(648)=coeff(648) *dr(1) *dr(2) *dr(2) *dr(2) *dr(4) *dr(5) 
      terms(649)=coeff(649) *dr(1) *dr(2) *dr(2) *dr(2) *dr(4) *dr(4) 
      terms(650)=coeff(650) *dr(1) *dr(2) *dr(2) *dr(2) *dr(3) 
      terms(651)=coeff(651) *dr(1) *dr(2) *dr(2) *dr(2) *dr(3) *dr(6) 
      terms(652)=coeff(652) *dr(1) *dr(2) *dr(2) *dr(2) *dr(3) *dr(5) 
      terms(653)=coeff(653) *dr(1) *dr(2) *dr(2) *dr(2) *dr(3) *dr(4) 
      terms(654)=coeff(654) *dr(1) *dr(2) *dr(2) *dr(2) *dr(3) *dr(3) 
      terms(655)=coeff(655) *dr(1) *dr(2) *dr(2) *dr(2) *dr(2) 
      terms(656)=coeff(656) *dr(1) *dr(2) *dr(2) *dr(2) *dr(2) *dr(6) 
      terms(657)=coeff(657) *dr(1) *dr(2) *dr(2) *dr(2) *dr(2) *dr(5) 
      terms(658)=coeff(658) *dr(1) *dr(2) *dr(2) *dr(2) *dr(2) *dr(4) 
      terms(659)=coeff(659) *dr(1) *dr(2) *dr(2) *dr(2) *dr(2) *dr(3) 
      terms(660)=coeff(660) *dr(1) *dr(2) *dr(2) *dr(2) *dr(2) *dr(2) 
      terms(661)=coeff(661) *dr(1) *dr(1) 
      terms(662)=coeff(662) *dr(1) *dr(1) *dr(6) 
      terms(663)=coeff(663) *dr(1) *dr(1) *dr(6) *dr(6) 
      terms(664)=coeff(664) *dr(1) *dr(1) *dr(6) *dr(6) *dr(6) 
      terms(665)=coeff(665) *dr(1) *dr(1) *dr(5) 
      terms(666)=coeff(666) *dr(1) *dr(1) *dr(5) *dr(6) 
      terms(667)=coeff(667) *dr(1) *dr(1) *dr(5) *dr(6) *dr(6) 
      terms(668)=coeff(668) *dr(1) *dr(1) *dr(5) *dr(6) *dr(6) *dr(6) 
      terms(669)=coeff(669) *dr(1) *dr(1) *dr(5) *dr(5) 
      terms(670)=coeff(670) *dr(1) *dr(1) *dr(5) *dr(5) *dr(6) 
      terms(671)=coeff(671) *dr(1) *dr(1) *dr(5) *dr(5) *dr(6) *dr(6) 
      terms(672)=coeff(672) *dr(1) *dr(1) *dr(5) *dr(5) *dr(5) 
      terms(673)=coeff(673) *dr(1) *dr(1) *dr(5) *dr(5) *dr(5) *dr(6) 
      terms(674)=coeff(674) *dr(1) *dr(1) *dr(5) *dr(5) *dr(5) *dr(5) 
      terms(675)=coeff(675) *dr(1) *dr(1) *dr(4) 
      terms(676)=coeff(676) *dr(1) *dr(1) *dr(4) *dr(6) 
      terms(677)=coeff(677) *dr(1) *dr(1) *dr(4) *dr(6) *dr(6) 
      terms(678)=coeff(678) *dr(1) *dr(1) *dr(4) *dr(6) *dr(6) *dr(6) 
      terms(679)=coeff(679) *dr(1) *dr(1) *dr(4) *dr(5) 
      terms(680)=coeff(680) *dr(1) *dr(1) *dr(4) *dr(5) *dr(6) 
      terms(681)=coeff(681) *dr(1) *dr(1) *dr(4) *dr(5) *dr(6) *dr(6) 
      terms(682)=coeff(682) *dr(1) *dr(1) *dr(4) *dr(5) *dr(5) 
      terms(683)=coeff(683) *dr(1) *dr(1) *dr(4) *dr(5) *dr(5) *dr(6) 
      terms(684)=coeff(684) *dr(1) *dr(1) *dr(4) *dr(5) *dr(5) *dr(5) 
      terms(685)=coeff(685) *dr(1) *dr(1) *dr(4) *dr(4) 
      terms(686)=coeff(686) *dr(1) *dr(1) *dr(4) *dr(4) *dr(6) 
      terms(687)=coeff(687) *dr(1) *dr(1) *dr(4) *dr(4) *dr(6) *dr(6) 
      terms(688)=coeff(688) *dr(1) *dr(1) *dr(4) *dr(4) *dr(5) 
      terms(689)=coeff(689) *dr(1) *dr(1) *dr(4) *dr(4) *dr(5) *dr(6) 
      terms(690)=coeff(690) *dr(1) *dr(1) *dr(4) *dr(4) *dr(5) *dr(5) 
      terms(691)=coeff(691) *dr(1) *dr(1) *dr(4) *dr(4) *dr(4) 
      terms(692)=coeff(692) *dr(1) *dr(1) *dr(4) *dr(4) *dr(4) *dr(6) 
      terms(693)=coeff(693) *dr(1) *dr(1) *dr(4) *dr(4) *dr(4) *dr(5) 
      terms(694)=coeff(694) *dr(1) *dr(1) *dr(4) *dr(4) *dr(4) *dr(4) 
      terms(695)=coeff(695) *dr(1) *dr(1) *dr(3) 
      terms(696)=coeff(696) *dr(1) *dr(1) *dr(3) *dr(6) 
      terms(697)=coeff(697) *dr(1) *dr(1) *dr(3) *dr(6) *dr(6) 
      terms(698)=coeff(698) *dr(1) *dr(1) *dr(3) *dr(6) *dr(6) *dr(6) 
      terms(699)=coeff(699) *dr(1) *dr(1) *dr(3) *dr(5) 
      terms(700)=coeff(700) *dr(1) *dr(1) *dr(3) *dr(5) *dr(6) 
      terms(701)=coeff(701) *dr(1) *dr(1) *dr(3) *dr(5) *dr(6) *dr(6) 
      terms(702)=coeff(702) *dr(1) *dr(1) *dr(3) *dr(5) *dr(5) 
      terms(703)=coeff(703) *dr(1) *dr(1) *dr(3) *dr(5) *dr(5) *dr(6) 
      terms(704)=coeff(704) *dr(1) *dr(1) *dr(3) *dr(5) *dr(5) *dr(5) 
      terms(705)=coeff(705) *dr(1) *dr(1) *dr(3) *dr(4) 
      terms(706)=coeff(706) *dr(1) *dr(1) *dr(3) *dr(4) *dr(6) 
      terms(707)=coeff(707) *dr(1) *dr(1) *dr(3) *dr(4) *dr(6) *dr(6) 
      terms(708)=coeff(708) *dr(1) *dr(1) *dr(3) *dr(4) *dr(5) 
      terms(709)=coeff(709) *dr(1) *dr(1) *dr(3) *dr(4) *dr(5) *dr(6) 
      terms(710)=coeff(710) *dr(1) *dr(1) *dr(3) *dr(4) *dr(5) *dr(5) 
      terms(711)=coeff(711) *dr(1) *dr(1) *dr(3) *dr(4) *dr(4) 
      terms(712)=coeff(712) *dr(1) *dr(1) *dr(3) *dr(4) *dr(4) *dr(6) 
      terms(713)=coeff(713) *dr(1) *dr(1) *dr(3) *dr(4) *dr(4) *dr(5) 
      terms(714)=coeff(714) *dr(1) *dr(1) *dr(3) *dr(4) *dr(4) *dr(4) 
      terms(715)=coeff(715) *dr(1) *dr(1) *dr(3) *dr(3) 
      terms(716)=coeff(716) *dr(1) *dr(1) *dr(3) *dr(3) *dr(6) 
      terms(717)=coeff(717) *dr(1) *dr(1) *dr(3) *dr(3) *dr(6) *dr(6) 
      terms(718)=coeff(718) *dr(1) *dr(1) *dr(3) *dr(3) *dr(5) 
      terms(719)=coeff(719) *dr(1) *dr(1) *dr(3) *dr(3) *dr(5) *dr(6) 
      terms(720)=coeff(720) *dr(1) *dr(1) *dr(3) *dr(3) *dr(5) *dr(5) 
      terms(721)=coeff(721) *dr(1) *dr(1) *dr(3) *dr(3) *dr(4) 
      terms(722)=coeff(722) *dr(1) *dr(1) *dr(3) *dr(3) *dr(4) *dr(6) 
      terms(723)=coeff(723) *dr(1) *dr(1) *dr(3) *dr(3) *dr(4) *dr(5) 
      terms(724)=coeff(724) *dr(1) *dr(1) *dr(3) *dr(3) *dr(4) *dr(4) 
      terms(725)=coeff(725) *dr(1) *dr(1) *dr(3) *dr(3) *dr(3) 
      terms(726)=coeff(726) *dr(1) *dr(1) *dr(3) *dr(3) *dr(3) *dr(6) 
      terms(727)=coeff(727) *dr(1) *dr(1) *dr(3) *dr(3) *dr(3) *dr(5) 
      terms(728)=coeff(728) *dr(1) *dr(1) *dr(3) *dr(3) *dr(3) *dr(4) 
      terms(729)=coeff(729) *dr(1) *dr(1) *dr(2) 
      terms(730)=coeff(730) *dr(1) *dr(1) *dr(2) *dr(6) 
      terms(731)=coeff(731) *dr(1) *dr(1) *dr(2) *dr(6) *dr(6) 
      terms(732)=coeff(732) *dr(1) *dr(1) *dr(2) *dr(6) *dr(6) *dr(6) 
      terms(733)=coeff(733) *dr(1) *dr(1) *dr(2) *dr(5) 
      terms(734)=coeff(734) *dr(1) *dr(1) *dr(2) *dr(5) *dr(6) 
      terms(735)=coeff(735) *dr(1) *dr(1) *dr(2) *dr(5) *dr(6) *dr(6) 
      terms(736)=coeff(736) *dr(1) *dr(1) *dr(2) *dr(5) *dr(5) 
      terms(737)=coeff(737) *dr(1) *dr(1) *dr(2) *dr(5) *dr(5) *dr(6) 
      terms(738)=coeff(738) *dr(1) *dr(1) *dr(2) *dr(5) *dr(5) *dr(5) 
      terms(739)=coeff(739) *dr(1) *dr(1) *dr(2) *dr(4) 
      terms(740)=coeff(740) *dr(1) *dr(1) *dr(2) *dr(4) *dr(6) 
      terms(741)=coeff(741) *dr(1) *dr(1) *dr(2) *dr(4) *dr(6) *dr(6) 
      terms(742)=coeff(742) *dr(1) *dr(1) *dr(2) *dr(4) *dr(5) 
      terms(743)=coeff(743) *dr(1) *dr(1) *dr(2) *dr(4) *dr(5) *dr(6) 
      terms(744)=coeff(744) *dr(1) *dr(1) *dr(2) *dr(4) *dr(5) *dr(5) 
      terms(745)=coeff(745) *dr(1) *dr(1) *dr(2) *dr(4) *dr(4) 
      terms(746)=coeff(746) *dr(1) *dr(1) *dr(2) *dr(4) *dr(4) *dr(6) 
      terms(747)=coeff(747) *dr(1) *dr(1) *dr(2) *dr(4) *dr(4) *dr(5) 
      terms(748)=coeff(748) *dr(1) *dr(1) *dr(2) *dr(4) *dr(4) *dr(4) 
      terms(749)=coeff(749) *dr(1) *dr(1) *dr(2) *dr(3) 
      terms(750)=coeff(750) *dr(1) *dr(1) *dr(2) *dr(3) *dr(6) 
      terms(751)=coeff(751) *dr(1) *dr(1) *dr(2) *dr(3) *dr(6) *dr(6) 
      terms(752)=coeff(752) *dr(1) *dr(1) *dr(2) *dr(3) *dr(5) 
      terms(753)=coeff(753) *dr(1) *dr(1) *dr(2) *dr(3) *dr(5) *dr(6) 
      terms(754)=coeff(754) *dr(1) *dr(1) *dr(2) *dr(3) *dr(5) *dr(5) 
      terms(755)=coeff(755) *dr(1) *dr(1) *dr(2) *dr(3) *dr(4) 
      terms(756)=coeff(756) *dr(1) *dr(1) *dr(2) *dr(3) *dr(4) *dr(6) 
      terms(757)=coeff(757) *dr(1) *dr(1) *dr(2) *dr(3) *dr(4) *dr(5) 
      terms(758)=coeff(758) *dr(1) *dr(1) *dr(2) *dr(3) *dr(4) *dr(4) 
      terms(759)=coeff(759) *dr(1) *dr(1) *dr(2) *dr(3) *dr(3) 
      terms(760)=coeff(760) *dr(1) *dr(1) *dr(2) *dr(3) *dr(3) *dr(6) 
      terms(761)=coeff(761) *dr(1) *dr(1) *dr(2) *dr(3) *dr(3) *dr(5) 
      terms(762)=coeff(762) *dr(1) *dr(1) *dr(2) *dr(3) *dr(3) *dr(4) 
      terms(763)=coeff(763) *dr(1) *dr(1) *dr(2) *dr(3) *dr(3) *dr(3) 
      terms(764)=coeff(764) *dr(1) *dr(1) *dr(2) *dr(2) 
      terms(765)=coeff(765) *dr(1) *dr(1) *dr(2) *dr(2) *dr(6) 
      terms(766)=coeff(766) *dr(1) *dr(1) *dr(2) *dr(2) *dr(6) *dr(6) 
      terms(767)=coeff(767) *dr(1) *dr(1) *dr(2) *dr(2) *dr(5) 
      terms(768)=coeff(768) *dr(1) *dr(1) *dr(2) *dr(2) *dr(5) *dr(6) 
      terms(769)=coeff(769) *dr(1) *dr(1) *dr(2) *dr(2) *dr(5) *dr(5) 
      terms(770)=coeff(770) *dr(1) *dr(1) *dr(2) *dr(2) *dr(4) 
      terms(771)=coeff(771) *dr(1) *dr(1) *dr(2) *dr(2) *dr(4) *dr(6) 
      terms(772)=coeff(772) *dr(1) *dr(1) *dr(2) *dr(2) *dr(4) *dr(5) 
      terms(773)=coeff(773) *dr(1) *dr(1) *dr(2) *dr(2) *dr(4) *dr(4) 
      terms(774)=coeff(774) *dr(1) *dr(1) *dr(2) *dr(2) *dr(3) 
      terms(775)=coeff(775) *dr(1) *dr(1) *dr(2) *dr(2) *dr(3) *dr(6) 
      terms(776)=coeff(776) *dr(1) *dr(1) *dr(2) *dr(2) *dr(3) *dr(5) 
      terms(777)=coeff(777) *dr(1) *dr(1) *dr(2) *dr(2) *dr(3) *dr(4) 
      terms(778)=coeff(778) *dr(1) *dr(1) *dr(2) *dr(2) *dr(3) *dr(3) 
      terms(779)=coeff(779) *dr(1) *dr(1) *dr(2) *dr(2) *dr(2) 
      terms(780)=coeff(780) *dr(1) *dr(1) *dr(2) *dr(2) *dr(2) *dr(6) 
      terms(781)=coeff(781) *dr(1) *dr(1) *dr(2) *dr(2) *dr(2) *dr(5) 
      terms(782)=coeff(782) *dr(1) *dr(1) *dr(2) *dr(2) *dr(2) *dr(4) 
      terms(783)=coeff(783) *dr(1) *dr(1) *dr(2) *dr(2) *dr(2) *dr(3) 
      terms(784)=coeff(784) *dr(1) *dr(1) *dr(2) *dr(2) *dr(2) *dr(2) 
      terms(785)=coeff(785) *dr(1) *dr(1) *dr(1) 
      terms(786)=coeff(786) *dr(1) *dr(1) *dr(1) *dr(6) 
      terms(787)=coeff(787) *dr(1) *dr(1) *dr(1) *dr(6) *dr(6) 
      terms(788)=coeff(788) *dr(1) *dr(1) *dr(1) *dr(6) *dr(6) *dr(6) 
      terms(789)=coeff(789) *dr(1) *dr(1) *dr(1) *dr(5) 
      terms(790)=coeff(790) *dr(1) *dr(1) *dr(1) *dr(5) *dr(6) 
      terms(791)=coeff(791) *dr(1) *dr(1) *dr(1) *dr(5) *dr(6) *dr(6) 
      terms(792)=coeff(792) *dr(1) *dr(1) *dr(1) *dr(5) *dr(5) 
      terms(793)=coeff(793) *dr(1) *dr(1) *dr(1) *dr(5) *dr(5) *dr(6) 
      terms(794)=coeff(794) *dr(1) *dr(1) *dr(1) *dr(5) *dr(5) *dr(5) 
      terms(795)=coeff(795) *dr(1) *dr(1) *dr(1) *dr(4) 
      terms(796)=coeff(796) *dr(1) *dr(1) *dr(1) *dr(4) *dr(6) 
      terms(797)=coeff(797) *dr(1) *dr(1) *dr(1) *dr(4) *dr(6) *dr(6) 
      terms(798)=coeff(798) *dr(1) *dr(1) *dr(1) *dr(4) *dr(5) 
      terms(799)=coeff(799) *dr(1) *dr(1) *dr(1) *dr(4) *dr(5) *dr(6) 
      terms(800)=coeff(800) *dr(1) *dr(1) *dr(1) *dr(4) *dr(5) *dr(5) 
      terms(801)=coeff(801) *dr(1) *dr(1) *dr(1) *dr(4) *dr(4) 
      terms(802)=coeff(802) *dr(1) *dr(1) *dr(1) *dr(4) *dr(4) *dr(6) 
      terms(803)=coeff(803) *dr(1) *dr(1) *dr(1) *dr(4) *dr(4) *dr(5) 
      terms(804)=coeff(804) *dr(1) *dr(1) *dr(1) *dr(4) *dr(4) *dr(4) 
      terms(805)=coeff(805) *dr(1) *dr(1) *dr(1) *dr(3) 
      terms(806)=coeff(806) *dr(1) *dr(1) *dr(1) *dr(3) *dr(6) 
      terms(807)=coeff(807) *dr(1) *dr(1) *dr(1) *dr(3) *dr(6) *dr(6) 
      terms(808)=coeff(808) *dr(1) *dr(1) *dr(1) *dr(3) *dr(5) 
      terms(809)=coeff(809) *dr(1) *dr(1) *dr(1) *dr(3) *dr(5) *dr(6) 
      terms(810)=coeff(810) *dr(1) *dr(1) *dr(1) *dr(3) *dr(5) *dr(5) 
      terms(811)=coeff(811) *dr(1) *dr(1) *dr(1) *dr(3) *dr(4) 
      terms(812)=coeff(812) *dr(1) *dr(1) *dr(1) *dr(3) *dr(4) *dr(6) 
      terms(813)=coeff(813) *dr(1) *dr(1) *dr(1) *dr(3) *dr(4) *dr(5) 
      terms(814)=coeff(814) *dr(1) *dr(1) *dr(1) *dr(3) *dr(4) *dr(4) 
      terms(815)=coeff(815) *dr(1) *dr(1) *dr(1) *dr(3) *dr(3) 
      terms(816)=coeff(816) *dr(1) *dr(1) *dr(1) *dr(3) *dr(3) *dr(6) 
      terms(817)=coeff(817) *dr(1) *dr(1) *dr(1) *dr(3) *dr(3) *dr(5) 
      terms(818)=coeff(818) *dr(1) *dr(1) *dr(1) *dr(3) *dr(3) *dr(4) 
      terms(819)=coeff(819) *dr(1) *dr(1) *dr(1) *dr(3) *dr(3) *dr(3) 
      terms(820)=coeff(820) *dr(1) *dr(1) *dr(1) *dr(2) 
      terms(821)=coeff(821) *dr(1) *dr(1) *dr(1) *dr(2) *dr(6) 
      terms(822)=coeff(822) *dr(1) *dr(1) *dr(1) *dr(2) *dr(6) *dr(6) 
      terms(823)=coeff(823) *dr(1) *dr(1) *dr(1) *dr(2) *dr(5) 
      terms(824)=coeff(824) *dr(1) *dr(1) *dr(1) *dr(2) *dr(5) *dr(6) 
      terms(825)=coeff(825) *dr(1) *dr(1) *dr(1) *dr(2) *dr(5) *dr(5) 
      terms(826)=coeff(826) *dr(1) *dr(1) *dr(1) *dr(2) *dr(4) 
      terms(827)=coeff(827) *dr(1) *dr(1) *dr(1) *dr(2) *dr(4) *dr(6) 
      terms(828)=coeff(828) *dr(1) *dr(1) *dr(1) *dr(2) *dr(4) *dr(5) 
      terms(829)=coeff(829) *dr(1) *dr(1) *dr(1) *dr(2) *dr(4) *dr(4) 
      terms(830)=coeff(830) *dr(1) *dr(1) *dr(1) *dr(2) *dr(3) 
      terms(831)=coeff(831) *dr(1) *dr(1) *dr(1) *dr(2) *dr(3) *dr(6) 
      terms(832)=coeff(832) *dr(1) *dr(1) *dr(1) *dr(2) *dr(3) *dr(5) 
      terms(833)=coeff(833) *dr(1) *dr(1) *dr(1) *dr(2) *dr(3) *dr(4) 
      terms(834)=coeff(834) *dr(1) *dr(1) *dr(1) *dr(2) *dr(3) *dr(3) 
      terms(835)=coeff(835) *dr(1) *dr(1) *dr(1) *dr(2) *dr(2) 
      terms(836)=coeff(836) *dr(1) *dr(1) *dr(1) *dr(2) *dr(2) *dr(6) 
      terms(837)=coeff(837) *dr(1) *dr(1) *dr(1) *dr(2) *dr(2) *dr(5) 
      terms(838)=coeff(838) *dr(1) *dr(1) *dr(1) *dr(2) *dr(2) *dr(4) 
      terms(839)=coeff(839) *dr(1) *dr(1) *dr(1) *dr(2) *dr(2) *dr(3) 
      terms(840)=coeff(840) *dr(1) *dr(1) *dr(1) *dr(2) *dr(2) *dr(2) 
      terms(841)=coeff(841) *dr(1) *dr(1) *dr(1) *dr(1) 
      terms(842)=coeff(842) *dr(1) *dr(1) *dr(1) *dr(1) *dr(6) 
      terms(843)=coeff(843) *dr(1) *dr(1) *dr(1) *dr(1) *dr(6) *dr(6) 
      terms(844)=coeff(844) *dr(1) *dr(1) *dr(1) *dr(1) *dr(5) 
      terms(845)=coeff(845) *dr(1) *dr(1) *dr(1) *dr(1) *dr(5) *dr(6) 
      terms(846)=coeff(846) *dr(1) *dr(1) *dr(1) *dr(1) *dr(5) *dr(5) 
      terms(847)=coeff(847) *dr(1) *dr(1) *dr(1) *dr(1) *dr(4) 
      terms(848)=coeff(848) *dr(1) *dr(1) *dr(1) *dr(1) *dr(4) *dr(6) 
      terms(849)=coeff(849) *dr(1) *dr(1) *dr(1) *dr(1) *dr(4) *dr(5) 
      terms(850)=coeff(850) *dr(1) *dr(1) *dr(1) *dr(1) *dr(4) *dr(4) 
      terms(851)=coeff(851) *dr(1) *dr(1) *dr(1) *dr(1) *dr(3) 
      terms(852)=coeff(852) *dr(1) *dr(1) *dr(1) *dr(1) *dr(3) *dr(6) 
      terms(853)=coeff(853) *dr(1) *dr(1) *dr(1) *dr(1) *dr(3) *dr(5) 
      terms(854)=coeff(854) *dr(1) *dr(1) *dr(1) *dr(1) *dr(3) *dr(4) 
      terms(855)=coeff(855) *dr(1) *dr(1) *dr(1) *dr(1) *dr(3) *dr(3) 
      terms(856)=coeff(856) *dr(1) *dr(1) *dr(1) *dr(1) *dr(2) 
      terms(857)=coeff(857) *dr(1) *dr(1) *dr(1) *dr(1) *dr(2) *dr(6) 
      terms(858)=coeff(858) *dr(1) *dr(1) *dr(1) *dr(1) *dr(2) *dr(5) 
      terms(859)=coeff(859) *dr(1) *dr(1) *dr(1) *dr(1) *dr(2) *dr(4) 
      terms(860)=coeff(860) *dr(1) *dr(1) *dr(1) *dr(1) *dr(2) *dr(3) 
      terms(861)=coeff(861) *dr(1) *dr(1) *dr(1) *dr(1) *dr(2) *dr(2) 
      terms(862)=coeff(862) *dr(1) *dr(1) *dr(1) *dr(1) *dr(1) 
      terms(863)=coeff(863) *dr(1) *dr(1) *dr(1) *dr(1) *dr(1) *dr(6) 
      terms(864)=coeff(864) *dr(1) *dr(1) *dr(1) *dr(1) *dr(1) *dr(5) 
      terms(865)=coeff(865) *dr(1) *dr(1) *dr(1) *dr(1) *dr(1) *dr(4) 
      terms(866)=coeff(866) *dr(1) *dr(1) *dr(1) *dr(1) *dr(1) *dr(3) 
      terms(867)=coeff(867) *dr(1) *dr(1) *dr(1) *dr(1) *dr(1) *dr(2) 
      terms(868)=coeff(868) *dr(1) *dr(1) *dr(1) *dr(1) *dr(1) *dr(1) 
       pot=0.d0
       do i=1,nfun
         pot=pot+terms(i)
       enddo
       pot=pot/hcm
       return
       end subroutine
C*************************************************************************
C This subroutine is to produce the poetential of CO from the fitting 
C*************************************************************************
      subroutine co_pot(rco,pot)

      implicit double precision (a-h, o-z)
      parameter (nfun=4)
      dimension coeff(nfun)
      dimension terms(nfun)
      hcm=219474.63d0
      data re/
     & 2.14849d0/
       dr=1-exp(-0.5d0*(rco-re))
       data coeff/
     & 0.1888275871D+04,0.8969607716D+03,0.5327536425D+06,
     & -0.7724719319D+06/
      terms(1)=coeff(1)
      terms(2)=coeff(2)*dr
      terms(3)=coeff(3)*dr*dr
      terms(4)=coeff(4)*dr*dr*dr
       pot=0.d0
       do i=1,nfun
         pot=pot+terms(i)
       enddo
       pot=(pot-1888.245133d0)/hcm
       return
       end subroutine
C*************************************************************************
C This subroutine is to produce the poetential of HH from the fitting
C*************************************************************************
      subroutine hh_pot(rhh,pot)

      implicit double precision (a-h, o-z)
      parameter (nfun=4)
      dimension coeff(nfun)
      dimension terms(nfun)
      hcm=219474.63d0
      data re/
     & 1.42732d0/
       dr=1-exp(-0.5d0*(rhh-re))
       data coeff/
     & 0.1932688614D+04,-0.1673235288D+03,0.1623562958D+06,
     & -0.1386661265D+06/
      terms(1)=coeff(1)
      terms(2)=coeff(2)*dr
      terms(3)=coeff(3)*dr*dr
      terms(4)=coeff(4)*dr*dr*dr
       pot=0.d0
       do i=1,nfun
         pot=pot+terms(i)
       enddo
       pot=(pot-1932.688567d0)/hcm
       return
       end subroutine
C*************************************************************************
C*************************************************************************
        end module h2copot
