module BEL

  use decimal, only: long, dop
!  use decimal, only: long, dop
  use timing
  use hpsimod, only: gwpintordm

    implicit none

  private
  complex(dop), dimension(:),allocatable    :: CoeffSave
  integer(long)::CoeffsSaved
  logical::NoMoreNonorth

  public :: resetGwp,restoreMaxDim,renormalize,checknorm,alloc_beldat,restoreRedDim
  contains

  function sech(x)
    real(dop) :: sech
    real(dop),intent(in) :: x
    sech=1.0_dop/cosh(x)
  end function sech

  ! This checks for applicability of the LHA in a barrier region and then assigns xi,phi and a
  ! The check can be fairly generic and could be replaced with anything that results in a narrow
  ! width in complicated potential regions.
  subroutine ValidateLHA(ex1,gdof,dim1,anew,PQS,iflag,istep,s)
    use global
    use runpropmod, only: BELflags

    use operdef
    use dvrdatmod
    use griddatmod
    use runpropmod
    use potevalmod
    use gh_elementsmod
    use BELglobals, only: MaxWidth

    implicit none

    integer(long),intent(in) :: ex1,dim1,gdof,s
    complex(dop), dimension(gdof,gdof,dim1), intent(out) :: anew

!GAW altered PQS(3,gdof)-> PQS(3,ndof) to allow elk_test to run.
    real(dop),intent(inout) :: PQS(3,ndof)

    integer(long) :: f
    integer(long) :: iflag,istep
    real(dop)     :: v0
    real(dop) :: width,dev

    integer(long) :: k,pottyp
    integer(long),save::h=-1
    integer(long),dimension(:),allocatable,save::hs
    integer(long),save::icount=0
    real(dop), dimension(maxdim)  :: ppoint,q0,der1
    complex(dop)                                            :: vz
    real(dop), dimension(ndof,ndof)  :: der2
    real(dop), dimension(ndof,ndof,ndof)      :: der3
    real(dop), dimension(ndof,ndof,ndof,ndof)    :: der4
    real(dop)     :: Ts,Vs

    character(len=15) :: la,pt
    character(len=11) :: arg
    real(dop),dimension(gdof)::Q

    integer(long), save                             :: tid=0 

    call get_timer(tid, 'ValidateLHA')
    call start_timer(tid)

!    call getarg(2,arg)
!    la(1:2)="la"
!    call int2char(istep,tmp)
!    la(3:6)=tmp
!    la(7:7)="_"
!    call int2char(ex1,tmp)
!    la(8:11)=tmp
!    la(12:15)=".dat"
!    pt=la
!    pt(1:2)="pt"

     ! Find the potential term
!write(*,*)"h",h
    if (h<0) then
      allocate(hs(nstate))
!      write(1000,*) nstate
!write(*,*)gdof,ktot
!write(*,*)hamilton(1:gdof,1:ktot)
!write(*,*)(hamilton(1:10,1)), hamilton(2,1), hamilton(3,1), (hamilton(1,2))
!write(*,*)htmtype(hamilton(1,1)), htmtype(hamilton(1,2))
!write(*,*)hoplab(1:9)
!      if (count(htmtype(pack(hamilton(1:gdof,1:ktot),.true.))==2) &
!          +count(htmtype(pack(hamilton(1:gdof,1:ktot),.true.))==101)/=1) &

!      if (count(htmtype(pack(hamilton(1:gdof,1:ktot),.true.))==2) &
!          +count(htmtype(pack(hamilton(1:gdof,1:ktot),.true.))==105)/=1) &
!                stop 'No single potential found in ValidateLHA'

      do k=1,ktot
!        if (htmtype(hamilton(1,k))==101) exit
        if (htmtype(hamilton(1,k))==105) then
            icount=icount+1
            hs(icount)=hamilton(1,k)
            write(1000,*)"hs",hs(icount),s
            if (icount==nstate) exit
        endif
!        if (htmtype(hamilton(1,k))==105) exit
!        if (htmtype(hamilton(1,k))==2) exit
      enddo
!      if (k==ktot+1) stop 'Missed potential term!'
      if (k==ktot+1) then
        deallocate(hs)
        go to 545
      endif
!      h=hamilton(1,k)
!      if (gwpintordm(1)/=2) stop 'gwpintord is not 2 in ValidateLHA'
q0=1
!h=52
call poteval (hs(s),v0,q0,vz,ppoint,pottyp)
!call poteval (h,v0,q0,vz,ppoint,pottyp)
write(1000,*)"gwpintord",gwpintordm(1)
write(1000,*)hs(s),q0(1),v0,der1(1),der2(1,1),der3(1,1,1),der4(1,1,1,1)
!stop
    endif
      h=hs(s)
!      write(1000,*)"ktot",ktot
!      write(1000,*)"hs",hs(1:nstate)
!      write(1000,*)"h",h
!      stop
        ! Construct a 2nd order Taylor expansion
       q0(1:ndof)=PQS(1,1:ndof)
       call lhaderivs(h,q0,v0,der1,der2,der3,der4)
call poteval (h,Vs,q0,vz,ppoint,pottyp)
call poteval (h,v0,q0,vz,ppoint,pottyp)
!write(*,*)"Vs",Vs
!$omp do
       do f=1,gdof
!write(*,*)"gwpintord",gwpintordm(1)
!write(*,*)h,q0(1),v0,der1(f),der2(f,f),der3(f,f,f),der4(f,f,f,f)
!stop


!write(*,*)h,q0(f),v0,der1(f),der2(f,f)
          anew(f,f,ex1)=-0.25_dop/PQS(3,f)**2
!write(*,*)'w',width,anew(f,f,ex1),ex1,f,PQS(3,f)

          if (BELflags(3) > 0.0_dop) then
             ! width has an extra factor of 2 here to check 2*sigma LHA acceptibility
             ! this factor has to be removed below the loop
             width=1.0/sqrt(-4.0_dop*real(anew(f,f,ex1)))*2.0_dop
             ! initialize deviation to go through the loop at least once
             dev=BELflags(3)*2.0_dop
!write(*,*)'w',width,anew(f,f,ex1),ex1,f

             do while (dev>BELflags(3)/dble(iflag))
               if (gwpintordm(1) ==2) then
                 Ts=v0+der1(f)*width+der2(f,f)*width**2*0.5_dop !2nd
               elseif (gwpintordm(1) ==3) then
                 Ts=v0+der1(f)*width+der2(f,f)*width**2*0.5_dop+der3(f,f,f)*width**3/(6_dop) !3rd
               elseif (gwpintordm(1) ==4) then
                 Ts=v0+der1(f)*width+der2(f,f)*width**2*0.5_dop+der3(f,f,f)*width**3/(6_dop) &
                    +der4(f,f,f,f)*width**4/(24_dop) !4th
               endif
               q0(f)=PQS(1,f)+width
               call poteval (h,Vs,q0,vz,ppoint,pottyp)
               dev=abs(Ts-Vs)
!write(*,*)f,q0(f),width,Ts,Vs
               if (gwpintordm(1) ==2) then
                 Ts=Ts-2*der1(f)*width ! 2nd
               elseif (gwpintordm(1) ==3) then
                 Ts=Ts-2*der1(f)*width-2*(der3(f,f,f)*width**3/(6_dop)) ! 3rd
               elseif (gwpintordm(1) ==4) then
                 Ts=Ts-2*der1(f)*width-2*(der3(f,f,f)*width**3/(6_dop)) ! 4th
               endif
               q0(f)=PQS(1,f)-width
               call poteval (h,Vs,q0,vz,ppoint,pottyp)
               dev=dev+abs(Ts-Vs)
!write(*,*)f,q0(f),width,Ts,Vs
               width=width*0.75_dop
             end do
             ! compensate for reduction in the last run of the loop and the prefactor of 2 (see above)
             width=width/0.75_dop/2.0_dop
!             anew(f,f,ex1)=min(dble(anew(f,f,ex1)),-0.25_dop/width**2)
             PQS(3,f)=width
!write(*,*)'w',width,anew(f,f,ex1),ex1,f,dev
!             MaxWidth(f)=width
!             anew(f,f,ex1)=-0.25_dop/width**2
          end if
          q0(f)=PQS(1,f)

          ! enforce a minimum and maximum width to avoid tiny or huge gaussians disrupting the calculation
!          anew(f,f,ex1)=min(max(dble(anew(f,f,ex1)),-0.25/BELflags(5)**2),-0.25/BELflags(6)**2)
          PQS(3,f)=min(max(dble(PQS(3,f)),BELflags(5)),BELflags(6))
          width=PQS(3,f)
             MaxWidth(f)=width
!write(*,*)'w',width,anew(f,f,ex1),ex1,f
!           open(13,file=arg//"/"//la)
!           do i=-1000,1000
!             Q=0.01d0*dble(i)
!             Ts=der2(f,f)/2.0d0*(Q(f)-PQS(1,f))**2.0d0
!             write(*,*)Q(f),Ts
!           enddo
!           open(17,file=arg//"/"//pt)
!           q0(f)=PQS(1,f)+width
!           call poteval (h,Vs,q0,vz,ppoint,pottyp)
!           write(*,*)q0(f),Vs
!           q0(f)=PQS(1,f)-width
!           call poteval (h,Vs,q0,vz,ppoint,pottyp)
!           write(*,*)q0(f),Vs
!           close(13)
!           close(17)
       end do
!$omp end do
545 continue 
      do f=1,gdof
          PQS(3,f)=min(max(dble(PQS(3,f)),BELflags(5)),BELflags(6))
          width=PQS(3,f)
          MaxWidth(f)=width
      enddo
  call stop_timer(tid)
  end subroutine ValidateLHA

  ! This xi,phi and a
  subroutine PQStoMOM(ex1,ex2,gdof,dim1,tau,zeta,eta,PQS)
    use global
    use runpropmod, only: BELflags

    use operdef
    use griddatmod
    use runpropmod
    use potevalmod
    use gh_elementsmod

    implicit none

    integer(long),intent(in) :: ex1,ex2,dim1,gdof

    complex(dop), dimension(gdof,dim1)                   :: ap
    complex(dop), dimension(gdof,gdof,dim1), intent(out)      :: tau
    complex(dop), dimension(gdof,dim1), intent(out)      :: zeta
    complex(dop), dimension(dim1), intent(out)           :: eta

    real(dop),intent(in) :: PQS(3,gdof,ex1:ex2)

    integer(long) :: e,f
    real(dop)     :: norm, norm1
    integer(long), save                             :: tid=0

    call get_timer(tid, 'PQStoMOM')
    call start_timer(tid)

    do e=ex1,ex2
       eta(e)=0.0_dop
       do f=1,gdof
          ap(f,e)=-0.25_dop/PQS(3,f,e)**2
          tau(f,f,e)=-PQS(3,f,e)**2
          zeta(f,e)=-0.5_dop*PQS(2,f,e)/ap(f,e)-dcmplx(0.0_dop,PQS(1,f,e))
          norm=dsqrt(pi/(-2.0_dop*dble(ap(f,e))))
          norm1=dsqrt((-2.0_dop*dble(ap(f,e))))
          norm=-0.5_dop*dlog(norm)-dlog(norm1)+0.25_dop*PQS(2,f,e)**2/(ap(f,e))
          eta(e)=eta(e)+dcmplx(norm,PQS(2,f,e)*PQS(1,f,e))
       end do
    end do
    call stop_timer(tid)
  end subroutine PQStoMOM

  ! This xi,phi and a
  subroutine PQStoGWP(ex1,ex2,gdof,dim1,phinew,xinew,xivecnew,anew,PQS)
    use global
    use runpropmod, only: BELflags

    use operdef
    use griddatmod
    use runpropmod
    use potevalmod
    use gh_elementsmod

    implicit none

    integer(long),intent(in) :: ex1,ex2,dim1,gdof
    complex(dop), dimension(dim1), intent(out)           :: phinew
    complex(dop), dimension(gdof,dim1), intent(out)      :: xinew
    complex(dop), dimension(gdof,dim1), intent(out)      :: xivecnew
    complex(dop), dimension(gdof,gdof,dim1), intent(out) :: anew

    real(dop),intent(in) :: PQS(3,gdof,ex1:ex2)

    integer(long) :: e,f
    real(dop)     :: norm
    integer(long), save                             :: tid=0 

    call get_timer(tid, 'PQStoGWP')
    call start_timer(tid)

    do e=ex1,ex2
       phinew(e)=0.0_dop
       do f=1,gdof
          anew(f,f,e)=-0.25_dop/PQS(3,f,e)**2
          xivecnew(f,e)=-2.0_dop*anew(f,f,e)*PQS(1,f,e)+dcmplx(0.0_dop,PQS(2,f,e))
          xinew(f,e)=PQS(1,f,e)+dcmplx(0.0_dop,PQS(2,f,e))
          norm=dsqrt(pi/(-2.0_dop*dble(anew(f,f,e))))
! write(1000,*)"sqrt(pi/(-2*a))",norm,PQS(1,f,e), dble(anew(f,f,e))
          norm=-0.5_dop*dlog(norm)+1.0_dop*dble(anew(f,f,e))*PQS(1,f,e)**2
! write(1000,*)"-0.5*log(norm)+a*(q)**2",norm
          phinew(e)=phinew(e)+dcmplx(norm,0.0_dop)
       end do
    end do
    call stop_timer(tid)
  end subroutine PQStoGWP

  subroutine PQStoGWP2(ex1,ex2,gdof,dim1,phinew,xinew,xivecnew,anew,PQS)
    use global
    use runpropmod, only: BELflags

    use operdef
    use griddatmod
    use runpropmod
    use potevalmod
    use gh_elementsmod

    implicit none

    integer(long),intent(in) :: ex1,ex2,dim1,gdof
    complex(dop), dimension(dim1), intent(out)           :: phinew
    complex(dop), dimension(gdof,dim1), intent(out)      :: xinew
    complex(dop), dimension(gdof,dim1), intent(out)      :: xivecnew
    complex(dop), dimension(gdof,gdof,dim1), intent(out) :: anew

    real(dop),intent(in) :: PQS(3,gdof,ex1)

    integer(long) :: e,f
    real(dop)     :: norm

!$omp do
    do e=ex1,ex2
       phinew(e)=0.0_dop
       do f=1,gdof
          anew(f,f,e)=-0.25_dop/PQS(3,f,e)**2
          xivecnew(f,e)=-2.0_dop*anew(f,f,e)*PQS(1,f,e)+dcmplx(0.0_dop,PQS(2,f,e))
          xinew(f,e)=PQS(1,f,e)+dcmplx(0.0_dop,PQS(2,f,e))
          norm=dsqrt(pi/(-2.0_dop*dble(anew(f,f,e))))
          norm=-0.5_dop*dlog(norm)+1.0_dop*dble(anew(f,f,e))*PQS(1,f,e)**2
          phinew(e)=phinew(e)+dcmplx(norm,0.0_dop)
       end do
    end do
!$omp end do
  end subroutine PQStoGWP2

  subroutine povl(PQS,PQfix,ex1,ex2,gdof,res)

  implicit none

  integer(long),intent(in) :: ex1,ex2,gdof
  real(dop),intent(in) :: PQS(3,gdof,ex1:ex2)
  real(dop),intent(in) :: PQfix(3,gdof,ex1)
  complex(dop),intent(out) :: res 
  complex(dop)  :: alpha 
  real(dop)     :: beta,gamma,coef

  coef=dsqrt(2.0_dop*PQS(3,1,ex1)*PQfix(3,1,1)/(PQS(3,1,ex1)**2.0_dop+PQfix(3,1,1)**2.0_dop))
  gamma=4.0_dop*(PQS(3,1,ex1)**2.0_dop+PQfix(3,1,1)**2.0_dop)
  alpha=2.0_dop*(PQS(3,1,ex1)**2.0_dop*PQS(2,1,ex1)+PQfix(3,1,1)**2.0_dop*PQfix(2,1,1)) &
        +dcmplx(0.0_dop,PQS(1,1,ex1)-PQfix(1,1,1))
  beta=gamma*(PQS(3,1,ex1)**2.0_dop*PQS(2,1,ex1)**2.0_dop+PQfix(3,1,1)**2.0_dop*PQfix(2,1,1)**2.0_dop)
  res=coef*cdexp((alpha**2.0_dop-beta)/gamma)
!  write(*,*)alpha,beta,gamma,coef
!  gamma=(PQS(3,1,ex1)**2.0_dop+PQfix(3,1,1)**2.0_dop)
!  alpha=(PQS(3,1,ex1)**2.0_dop*PQS(2,1,ex1)+PQfix(3,1,1)**2.0_dop*PQfix(2,1,1)) &
!        +dcmplx(0.0_dop,0.0_dop)
!  beta=(PQS(3,1,ex1)**2.0_dop*PQS(2,1,ex1)**2.0_dop+PQfix(3,1,1)**2.0_dop*PQfix(2,1,1)**2.0_dop)
!  res=coef*cdexp((alpha**2.0_dop/gamma)-beta)
!  write(*,*)dble(alpha**2.0d0)/gamma,dble(alpha**2.0d0),beta,gamma,coef
  end subroutine povl

! wrapper for funcP to be used in minim
! necessary to allow for optional parameter in funcP
  subroutine funcPcall(PQ,out)
    use BELglobals

    implicit none
    real(dop),intent(in)    :: PQ(PQsize,gdof,ex1:ex2)
    ! real(dop),intent(in) :: PQ(PQsize*gdof)
    real(dop),intent(inout) :: out 
    call funcP(PQ,out,PQfix,PQsize,PQset,gdof,ex1,ex2,allprod,allnewprod,newcoef,psinew,anew,psiold,aold,&
         dim1,dim1red,norm0,norma,m,s)
  end subroutine funcPcall

  ! compute Eq.(7) in WK,TJF, PRL 2012
  ! return f'_i(P) without gimmedev or
  !    <\delta\Psi_i|\delta_Psi_i> if gimmedev==.True.
  subroutine funcP(PQ,out,PQfix,PQsize,PQset,gdof,ex1,ex2,allprod,allnewprod,newcoef,psinew,anew,psiold,aold,&
         dim1,dim1red,norm0,norma,m,s,gimmedev)
    use griddatmod
    use psidef
    use gwplib, only: gwpproj
    use BELglobals, only: MaxWidth,tauold,zetaold,etaold,taunew,zetanew,etanew,aoverlap,ssdim,ssblock
    use runpropmod, only: BELflags

    implicit none

    integer(long),intent(in) :: PQsize
    integer(long),intent(in) :: PQset(3)
    real(dop),intent(in) :: PQfix(:,:,:)

    integer(long),intent(in) :: ex1,ex2,gdof,dim1,dim1red,m,s

    complex(dop),intent(inout)    :: psinew(dgldim),anew(agmatdim)
    complex(dop),intent(in)    :: psiold(dgldim),aold(agmatdim)

    complex(dop),intent(inout) :: allprod(:,:),allnewprod(:,:),newcoef(:,:)

    complex(dop),intent(inout) :: norma
    complex(dop),intent(in) :: norm0


    real(dop),intent(in)    :: PQ(PQsize,gdof,ex1:ex2)
    real(dop),intent(inout) :: out 

    logical(kind=4),intent(in),optional :: gimmedev


!    complex(dop)  :: newprod(ex1:ex2),oldprod(ex1:ex2)
    complex(dop)  :: newprod(1:ex2),oldprod(1:ex2)

    real(dop) :: PQS(3,gdof,ex1:ex2)
!    complex(dop) :: aoverlap
    logical(kind=4) :: temp_gd
    integer(long) :: i,e,k,j
    real(dop)                              :: normnew
    complex(dop) :: res
    integer(long), save                             :: tid=0 

    integer(long) :: itype

    complex(dop)          :: phio,phin
    complex(dop)          :: kappa(1:ex1)

    itype=0 ! <g|d_Psi>
!    itype=1 ! d_Phi(P)

    call get_timer(tid, 'funcP')
    call start_timer(tid)

    if (ex1/=ex2) stop 'Terry broke funcP for multi-e'

    temp_gd=.false.
    if (ex1==1) then
!       temp_gd=.true.
       if (present(gimmedev)) then
          temp_gd=gimmedev
!write(*,*)"tmp_gd",temp_gd
       end if
    else
       temp_gd=.false.
       if (present(gimmedev)) then
          temp_gd=gimmedev
!write(*,*)"tmp_gd",temp_gd
       end if
    end if
    
    PQS(:,1:gdof,ex1:ex2)=PQfix(:,1:gdof,ex1:ex2)
    do i=1,PQsize
       PQS(PQset(i),1:gdof,ex1:ex2)=PQ(i,1:gdof,ex1:ex2)
    end do

     ! Quick return if possible
!    if (any(PQS(3,1:gdof,ex1)>MaxWidth(1:gdof)) &
!            .or.any(PQS(3,1:gdof,ex1)<=0)) then
!      out=0
!write(*,*)real(PQS(:,1,ex1)),real(out)
!      return
!    endif

!if(ex1>1) then
!write(*,*)"MaxWidth",MaxWidth(1:gdof)
!     if (any(PQS(3,1:gdof,ex1)>MaxWidth(1:gdof))) then
!       PQS(3,1:gdof,ex1)=MaxWidth(1:gdof)
!     elseif (any(PQS(3,1:gdof,ex1)<BELflags(5))) then
!       PQS(3,1:gdof,ex1)=BELflags(5)
!     endif

!     if (any(PQS(2,1:gdof,ex1)>50_dop)) then
!       PQS(2,1:gdof,ex1)=50_dop
!     elseif (any(PQS(2,1:gdof,ex1)<-50_dop)) then
!       PQS(2,1:gdof,ex1)=-50_dop
!     endif
!endif

!write(*,*)psinew(zphi(m,s))
!write(*,*)psinew(zxi(m,s))
    ! initialize a,xi,phi
    ! Modification by yapol
!    if (lgwpphase) then
!       call PQStoGWP(ex1,ex2,gdof,dim1,psinew(zphi(m,s)),psinew(zxi(m,s)),&
!            anew(zxivec(m,s)),anew(zagmat(m,s)),PQS)
!    else
!       call PQStoGWP(ex1,ex2,gdof,dim1,anew(zphase(m,s)),psinew(zxi(m,s)),&
!            anew(zxivec(m,s)),anew(zagmat(m,s)),PQS)
!    endif
!    ! End of modification

       call PQStoGWP(ex1,ex2,gdof,dim1,anew(zphase(m,s)),psinew(zxi(m,s)),&
            anew(zxivec(m,s)),anew(zagmat(m,s)),PQS) ! modified by TM

    ! compute Eq.(7) in WK,TJF, PRL 2012
    ! i.e. <g_e|\Delta\Psi>
    ! Need <g|Psi> and <g|Psi'> in oldprod and newprod
    ! Recall that the expansion vector changes in each iteration
    ! We have a Psi' with ex1-1 Gaussians, are adding ex1.
    ! <g|Psi>
!    call gwpproj(ex1,ex1,gdof,&
!         dim1,psinew(zphi(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)), &
!         dim1red,psiold(zphi(m,s)),aold(zxivec(m,s)),aold(zagmat(m,s)),psiold(zpsi(s)),&
!         oldprod)
!    call gwpproj(ex1,ex1,gdof,&
!         dim1,anew(zphase(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)), &
!         dim1red,aold(zphase(m,s)),aold(zxivec(m,s)),aold(zagmat(m,s)),psiold(zpsi(s)),& ! modified by TM
!         oldprod)
    ! <g|Psi'>
!    call gwpproj(ex1,ex1,gdof,&
!         dim1,psinew(zphi(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)), &
!         ex1-1,psinew(zphi(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)),psinew(zpsi(s)),&
!         newprod)
!if (ex1>1) then
!    call gwpproj(ex1,ex1,gdof,&
!         dim1,anew(zphase(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)), &
!         ex1-1,anew(zphase(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)),psinew(zpsi(s)),& ! modified by TM
!         newprod)
!elseif (ex1==1) then
!    newprod=(0.0d0,0.0d0)
!endif
     ! save new values of \sum_j \psi_old_j <g_new_ex1..ex2|g_old_j>
     !                   (that's <g_new_i|Psi>)
!    allprod(ex1:ex2)=oldprod(ex1:ex2)

    if (temp_gd) then
!write(1000,*)ex1,anew(zphase(m,s)+ex1-1)
    do j=0,ssdim-1
          if (abs(psiold(zpsi(s)+j*dim1red))/=0.0d0) then
          else
          oldprod=(0.0d0,0.0d0)
          newprod=(0.0d0,0.0d0)
          psinew(zpsi(s)+j*dim1red+ex1-1)=(0.0d0,0.0d0)
            go to 213
          endif
    call gwpproj(ex1,ex1,gdof,&
         dim1,anew(zphase(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)), &
         dim1red,aold(zphase(m,s)),aold(zxivec(m,s)),aold(zagmat(m,s)),psiold(zpsi(s)+j*dim1red),& ! modified by TM
         oldprod)
    if (ex1>1) then
    call gwpproj(ex1,ex1,gdof,&
         dim1,anew(zphase(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)), &
!         ex1-1,anew(zphase(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)),newcoef(1,j+1),& ! modified by TM
!         ex1-1,anew(zphase(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)),psinew(zpsi(s)+j*dim1red),& 
         ex1-1,anew(zphase(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)),psinew(zpsi(s)+j*dim1),& 
         newprod)
    elseif (ex1==1) then
        newprod=(0.0d0,0.0d0)
    endif
!       if (ssdim>1) then
         psinew(zpsi(s)+j*dim1+ex1-1)=(oldprod(ex1)-newprod(ex1))
!       else
!         psinew(zpsi(s)+j*dim1red+ex1-1)=(oldprod(ex1)-newprod(ex1))
!       endif
!       write(1000,*)"psinew",psinew(zpsi(s)+j*dim1+ex1-1),ex1,j
       newcoef(ex1,j+1)=(oldprod(ex1)-newprod(ex1))
!       write(1000,*)"newcoef",newcoef(ex1,j+1),ex1,j
       allprod(ex1:ex2,j+1)=oldprod(ex1:ex2)
       allnewprod(ex1:ex2,j+1)=newprod(ex1:ex2)
213 continue
    enddo

!      allprod(ex1:ex2)=oldprod(ex1:ex2)
!      allnewprod(ex1:ex2)=newprod(ex1:ex2)

      if (itype==0) then

        out=-abs(oldprod(ex1)-newprod(ex1))
!        psinew(zpsi(s+ss*dim1red)+ex1-1)=(oldprod(ex1)-newprod(ex1))
!        if (abs(aoverlap)<BELflags(1)) then
!          psinew(zpsi(s)+ex1-1)=(0.0d0,0.0d0)
!        endif

      elseif (itype==1) then
!        call PQStoMOM(ex1,ex2,gdof,dim1,taunew,zetanew,etanew,PQS)

!        call GwpValueP(gdof,dim1red,&
!             etaold,zetaold,tauold,psiold(zpsi(s)),PQS(2,:,ex1),&
!             phio)
!        if (ex1>1) then
!          call GwpValueP(gdof,ex1-1,&
!               etanew,zetanew,taunew,psinew(zpsi(s)),PQS(2,:,ex1),&
!               phin)
!        elseif (ex1==1) then
!            phin=(0.0d0,0.0d0)
!        endif
!        out=-(abs(phio-phin)**2)
!        psinew(zpsi(s)+ex1-1)=(phio-phin) 
!        call gwpproj(ex1,ex1,gdof,&
!             dim1,etanew,zetanew,taunew, &
!             dim1red,etaold,zetaold,tauold,psiold(zpsi(s)),&
!             oldprod)
!         write(1000,*)"oldprod",oldprod(1:ex1),ex1
!!stop
!        if (ex1>1) then
!            call gwpproj(ex1,ex1,gdof,&
!                 dim1,etanew,zetanew,taunew, &
!                 ex1-1,etanew,zetanew,taunew,psinew(zpsi(s)),&
!                 newprod)
!       elseif (ex1==1) then
!            newprod=(0.0d0,0.0d0)
!        endif
!        out=-abs(oldprod(ex1)-newprod(ex1))
!        psinew(zpsi(s)+ex1-1)=(oldprod(ex1)-newprod(ex1))
     endif


!      psinew(zpsi(s)+ex1-1)=oldprod(ex1)
!write(*,*)ex1,psinew(zpsi(s)+ex1-1)

!      if (abs(psinew(zpsi(s)+ex1-1))<=BELflags(1)) then
!!        PQS(2,1:gdof,ex1)=0.0d0
!        PQS(2,1:gdof,ex1)=PQfix(2,1:gdof,ex1)
!        call PQStoGWP(ex1,ex2,gdof,dim1,anew(zphase(m,s)),psinew(zxi(m,s)),&
!             anew(zxivec(m,s)),anew(zagmat(m,s)),PQS) ! modified by TM
!    call gwpproj(ex1,ex1,gdof,&
!         dim1,anew(zphase(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)), &
!         dim1red,aold(zphase(m,s)),aold(zxivec(m,s)),aold(zagmat(m,s)),psiold(zpsi(s)),& ! modified by TM
!         oldprod)
!    call gwpproj(ex1,ex1,gdof,&
!         dim1,anew(zphase(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)), &
!         ex1-1,anew(zphase(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)),psinew(zpsi(s)),& ! modified by TM
!         newprod)
!      allprod(ex1:ex2)=oldprod(ex1:ex2)
!      psinew(zpsi(s)+ex1-1)=(oldprod(ex1)-newprod(ex1))
!      endif

       ! compute  <\delta\Psi_i|\delta_Psi_i>

       ! compute \sum_i,j=1..ex2 \psi_new_j* \psi_new_i <g_new_j|g_new_i>
       ! norma is \sum_i,j=1..ex1-1 \psi_new_j* \psi_new_i <g_new_j|g_new_i>
       ! from previous run
!       if (ex1==1) then
!          normnew=0.0_dop
!       else
!          normnew=abs(norma)
!       end if
!       do e=ex1,ex2
    ! fill in the earlier <g|Psi'> that are missing due to the changed coeffs
!       if (ex1>1) call gwpproj(1,ex1-1,gdof,&
!             dim1,psinew(zphi(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)), &
!             ex1-1,psinew(zphi(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)),psinew(zpsi(s)),&
!             newprod)
!       if (ex1>1) call gwpproj(ex1,ex1,gdof,&
!             dim1,anew(zphase(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)), &
!             ex1,aold(zphase(m,s)),aold(zxivec(m,s)),aold(zagmat(m,s)),psiold(zpsi(s)),&
!             oldprod)
!       if (ex1>1) call gwpproj(1,ex1,gdof,&
!             dim1,anew(zphase(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)), &
!             ex1-1,anew(zphase(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)),psinew(zpsi(s)),&
!             newprod)
!       normnew=0.0_dop
!       do e=1,ex2
!          normnew=normnew+2.0_dop*dble(dconjg(psinew(zpsi(s)-1+e))*newprod(e))
!          normnew=normnew+abs(psinew(e))**2
!write(*,*)e,abs(psinew(e))**2,dconjg(psinew(zpsi(s)-1+e))*newprod(e),newprod(e)
!       enddo
!!write(*,*)
!       ! save new value for next run
!       norma=normnew
!
!        ! norm0 has been set to 1 before the call.
!       aoverlap=norm0+normnew
!write(*,*)"norm0 normnew",norm0,normnew
!       ! compute \sum_i=1..ex2 \sum_j \psi_new_i* \psi_old_j <g_new_i|g_old_j>
!       aoverlap=aoverlap-sum(2.0_dop*dble(dconjg(psinew(zpsi(s):zpsi(s)-1+ex2))*allprod(1:ex2)))
!!       aoverlap=-sum(dconjg(allprod(1:ex2))*allprod(1:ex2))+sum(dconjg(newprod(1:ex2))*newprod(1:ex2))
!write(*,*)"sum",-sum(2.0_dop*dble(dconjg(psinew(zpsi(s):zpsi(s)-1+ex2))*allprod(1:ex2)))
!write(*,*)"aoverlap",aoverlap
!      do e=1,ex2
!       write(*,*)"allprod newprod",e, (dconjg(allprod(e))*allprod(e)),(dconjg(newprod(e))*newprod(e))
!      enddo
!      out=abs(aoverlap)
!      out=-abs(oldprod(ex1)-newprod(ex1))
!!write(*,*)"oldprod newprod",oldprod(ex1),newprod(ex1)
!!write(*,*)real(PQS(:,1,ex1)),real(out)
    else

out=0.0d0
      if (itype==0) then
        do j=0,ssdim-1
          if (abs(psiold(zpsi(s)+j*dim1red))/=0.0d0) then
          else
          oldprod=(0.0d0,0.0d0)
          newprod=(0.0d0,0.0d0)
            go to 321
          endif
          call gwpproj(ex1,ex1,gdof,&
             dim1,anew(zphase(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)), &
             dim1red,aold(zphase(m,s)),aold(zxivec(m,s)),aold(zagmat(m,s)),psiold(zpsi(s)+j*dim1red),& ! modified by TM
!             dim1red,aold(zphase(m,s)),aold(zxivec(m,s)),aold(zagmat(m,s)),psiold(zpsi(s)),& ! modified by TM
             oldprod)
          if (ex1>1) then
              call gwpproj(ex1,ex1,gdof,&
                 dim1,anew(zphase(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)), &
                 ex1-1,anew(zphase(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)),psinew(zpsi(s)+j*dim1),& ! modified by TM
!                 ex1-1,anew(zphase(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)),psinew(zpsi(s)),& ! modified by TM
                 newprod)
          elseif (ex1==1) then
              newprod=(0.0d0,0.0d0)
          endif
321 continue 
          out=out+abs(oldprod(ex1)-newprod(ex1))
!          out=out+abs(oldprod(ex1)-newprod(ex1))**2
!write(1000,*)abs(oldprod(ex1)-newprod(ex1))**2,ex1,j,PQS(2,1:gdof,ex1)
        enddo
        out=-out

      elseif (itype==1) then
        call PQStoMOM(ex1,ex2,gdof,dim1,taunew,zetanew,etanew,PQS)
        call GwpValueP(gdof,dim1red,&
             etaold,zetaold,tauold,psiold(zpsi(s)),PQS(2,:,ex1),&
             phio)
        if (ex1>1) then
          call GwpValueP(gdof,ex1-1,&
               etanew,zetanew,taunew,psinew(zpsi(s)),PQS(2,:,ex1),&
               phin)
        elseif (ex1==1) then
            phin=(0.0d0,0.0d0)
        endif
        out=-(abs(phio-phin)**2)
!        do k=1,ex1-1
!          out=out*(1-exp(-sum((PQS(2,:,ex1)-PQS(2,1:gdof,k))**2*0.05d0)))
!        enddo
      endif
! write(1000,*)"funcP",ex1,(PQS(2,:,ex1)),abs(out),temp_gd

!       out=-abs(psinew(zpsi(s)-1+ex1))
!     out=-abs(oldprod(ex1)-newprod(ex1))
!     out=-abs(oldprod(ex1))
!     out=-abs(newprod(ex1))
!write(*,*)real(PQS(:,1,ex1)),real(out)
!write(*,*)"PQS out",real(PQS(:,1,ex1)),real(out)
!     call povl(PQS,PQfix,ex1,ex2,gdof,res)
!     out=-abs(res)
!write(*,*)"PQS out",ex1,real(PQS(:,1,ex1)),real(out)
    end if
    call stop_timer(tid)
  end subroutine funcP

  subroutine getdevdist(gdof,dim1,phi1,xi1,a1,psi1,aoverlap)

    use global
!    use psidef, only: rsbaspar,isbaspar
    implicit none
    integer(long),intent(in) :: gdof,dim1

    complex(dop), dimension(dim1), intent(in)           :: phi1
    complex(dop), dimension(gdof,dim1), intent(in)      :: xi1
    complex(dop), dimension(gdof,gdof,dim1), intent(in) :: a1
    complex(dop), dimension(dim1), intent(in)           :: psi1

    complex(dop),intent(out)                           :: aoverlap
    real(dop) :: aq(gdof,dim1),ap(gdof,dim1)
    integer(long) :: f
!$omp do
    do f=1,gdof
       ap(f,1:dim1-1)=0.5_dop*(sqrt(-0.25/dble(a1(f,f,1:dim1-1)))+sqrt(-0.25/dble(a1(f,f,dim1))))
       aq(f,:)=dble(xi1(f,:))/(-2.0_dop*dble(a1(f,f,:)))
       aq(f,1:dim1-1)=(aq(f,1:dim1-1)-aq(f,dim1))/ap(f,1:dim1-1)
    end do
!$omp end do
    aoverlap=-minval(sum(aq(:,1:dim1-1)**2,1))
  end subroutine getdevdist

!  subroutine funcQcall(PQ,out)
!    use BELglobals
!
!    implicit none
!!    real(dop),intent(in)    :: PQ(PQsize,gdof,ex1:ex2)
!     real(dop),intent(in) :: PQ(PQsize*gdof)
!    real(dop),intent(inout) :: out 
!    call funcQ(PQ,aoverlap,out)
!  end subroutine funcQcall

! caller routine for Q selection to be used in minim
! this corresponds to Eq.(4) in WK,TJF, PRL 2012
  subroutine funcQ(Q,out)
    use psidef
    use global
    use runpropmod, only : BELflags
    use BELglobals

    use randommod, only: randuniform

    implicit none

!    real(dop),intent(in)  :: Q(gdof)
    real(dop)  :: Q(gdof)
!    complex(dop) :: aoverlap
    real(dop),intent(out) :: out 

    real(dop)             :: PQS(3,gdof,ex1:ex2)
    complex(dop)          :: wf0,wfk
    integer::k,j
    real(dop)  :: absvalue
    integer(long), save                             :: tid=0

    logical(kind=4) :: Hea

    call get_timer(tid, 'funcQ')
    call start_timer(tid)

!    Hea=.true.
   Hea=.false.
    PQS(2:3,1:gdof,ex1)=PQfix(2:3,1:gdof,ex1)
    PQS(1,1:gdof,ex1)=Q(1:gdof)
!    call GwpValue(gdof,dim1,&
!         psiold(zphi(1,1)),aold(zxivec(1,1)),aold(zagmat(1,1)),psiold(zpsi(1)),Q,&
!         wf0)
!    call GwpValue(gdof,ex1-1,&
!         psinew(zphi(1,1)),anew(zxivec(1,1)),anew(zagmat(1,1)),psinew(zpsi(1)),Q,&
!         wfk)

!    call GwpValue(gdof,dim1,&
!    if (abs(aoverlap).ge.BELflags(1)) then
!    endif
out=0.0d0
    do j=0,ssdim-1
    call GwpValue(gdof,dim1red,&
         aold(zphase(m,s)),aold(zxivec(m,s)),aold(zagmat(m,s)),psiold(zpsi(s)+j*dim1red),Q,& ! modified by TM
!         aold(zphase(m,s)),aold(zxivec(m,s)),aold(zagmat(m,s)),psiold(zpsi(s)),Q,& ! modified by TM
         wf0)
    call GwpValue(gdof,ex1-1,&
         anew(zphase(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)),psinew(zpsi(s)+j*dim1),Q,& ! modified by TM
!         anew(zphase(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)),psinew(zpsi(s)),Q,& ! modified by TM
         wfk)
    out=out+abs(wf0-wfk)**2
    enddo
!write(*,*)"wf0 wfk ex1 Q",wf0,wfk,ex1,Q
!write(*,*)"wf0 Q",wf0,Q
!write(*,*)"abs(wf0-wfk)**2",abs(wf0-wfk)**2
!$omp do
    do k=1,ex1-1
      do j=1,gdof
!      if (sqrt((Q-PQfix(1,j,k))**2)<0.01_dop) then
!      if (abs(Q-PQfix(1,j,k))<0.01_dop) then
      absvalue=abs(Q(j)-PQfix(1,j,k))
!      if (absvalue<0.03_dop) then
!write(*,*)"random k Q ",k,Q
!        Q=Q-randuniform()
!write(*,*)" k Q ",k,Q
!      endif
      enddo
    enddo
!$omp end do
!write(*,*)" iloop ",iloop
!if ( abs(aoverlap)>BELflags(1) .and. (iloop==0) .and. (Hea) ) then
if ( (Hea) ) then
     ! include Heaviside
!    out=0
!    if (abs(wf0)**2>BELflags(1)) then
    if (abs(wf0)**2>BELflags(1)) then
       ! Pessimistic domain restriction
      where (PQS(1,1:gdof,ex1)<DomainMid(1:gdof))
        DomainLower(1:gdof)=min(PQS(1,1:gdof,ex1),DomainLower(1:gdof))
      elsewhere
        DomainUpper(1:gdof)=max(PQS(1,1:gdof,ex1),DomainUpper(1:gdof))
      end where
       ! product
!      out=1
!      do k=1,ex1-1
!        out=out*sqrt(sum((Q-PQfix(1,1:gdof,k))**2))
!      enddo
       ! sum
      do k=1,ex1-1
        out=out+sqrt(sum((Q-PQfix(1,1:gdof,k))**2))
!        out=out+(sum((Q-PQfix(1,1:gdof,k))**2))
!write(*,*)"sqrt(sum((Q-PQfix(1,1:gdof,k))**2))",sqrt(sum((Q-PQfix(1,1:gdof,k))**2))
      enddo
!      out=out+10d0**10*abs(wf0-wfk)**2
    else
!      out=abs(wf0-wfk)**2
    endif

!    out=out+abs(wf0-wfk)**2
!write(*,*)" Hea "
else
     ! No Heaviside
!    if (abs(wf0)**2>BELflags(1)) then
!    if (abs(wf0)**2>BELflags(1)) then
!       ! Pessimistic domain restriction
!      where (PQS(1,1:gdof,ex1)<DomainMid(1:gdof))
!        DomainLower(1:gdof)=min(PQS(1,1:gdof,ex1),DomainLower(1:gdof))
!      elsewhere
!        DomainUpper(1:gdof)=max(PQS(1,1:gdof,ex1),DomainUpper(1:gdof))
!      end where
!    endif
!    ! Gaussians
!    out=1
!    if (abs(aoverlap)<BELflags(1) .or. (iloop==1) ) then
!    if (abs(aoverlap)<BELflags(1)  ) then
!!    if ((iloop==1) ) then
!    do k=1,ex1-1
!!      out=out*(1-exp(-sum((Q-PQfix(1,1:gdof,k))**2/2/PQfix(3,1:gdof,k)**2)))
!      out=out*(1-exp(-sum((Q-PQfix(1,1:gdof,k))**2/(2*PQfix(3,1:gdof,k)**2))))
!    enddo
!    endif
!!     ! Product of distances
!!    out=1
!!    do k=1,ex1-1
!!      out=out*sum((Q-PQfix(1,1:gdof,k))**2)
!!    enddo
!     ! weight by density
!    out=out*abs(wf0)**2

!    out=out+BELflags(12)*abs(wf0-wfk)**2
!    out=out*abs(wf0-wfk)**2
!    out=out*abs(wf0)**2
!    out=out*abs(wf0-wfk)

!    out=abs(wf0-wfk)**2
!    out=1
!    if ( (abs(aoverlap).ge.BELflags(1)) .or. &
!         ((abs(aoverlap)<BELflags(1)) .and. (abs(nwf1)**2*BELflags(1)-abs(wfk)**2 .ge. 0.0d0)) ) then
      do k=1,ex1-1
!        out=out*(1-exp(-sum((Q-PQfix(1,1:gdof,k))**2/(0.5d0*2*PQfix(3,1:gdof,k)**2))))
!        out=out*(1-exp(-sum((Q-PQfix(1,1:gdof,k))**2/(2*PQfix(3,1:gdof,k)**2))))
!        out=out*(1-exp(-sum((Q-PQfix(1,1:gdof,k))**2/(0.25d0*2*PQfix(3,1:gdof,k)**2))))
!        out=out*(1-exp(-sum((Q-PQfix(1,1:gdof,k))**2/(0.15d0*2*PQfix(3,1:gdof,k)**2))))
!        out=out*(1-exp(-sum((Q-PQfix(1,1:gdof,k))**2/(0.1d0*2*PQfix(3,1:gdof,k)**2))))
!        out=out*(1-exp(-sum((Q-PQfix(1,1:gdof,k))**2/(0.09d0*2*PQfix(3,1:gdof,k)**2)))) 
!        out=out*(1-exp(-sum((Q-PQfix(1,1:gdof,k))**2/(0.08d0*2*PQfix(3,1:gdof,k)**2)))) 
        out=out*(1-exp(-sum((Q-PQfix(1,1:gdof,k))**2/(0.05d0*2*PQfix(3,1:gdof,k)**2)))) !dw2d
!        out=out*(1-exp(-sum((Q-PQfix(1,1:gdof,k))**2/(0.01d0*2*PQfix(3,1:gdof,k)**2)))) 
!        out=out*(1-exp(-sum((Q-PQfix(1,1:gdof,k))**2/(0.005d0*2*PQfix(3,1:gdof,k)**2)))) 
!        out=out*(1-exp(-sum((Q-PQfix(1,1:gdof,k))**2/(0.1d0**2))))
      enddo
!    endif
!    if (abs(aoverlap).ge.BELflags(1)) then
!      out=out*(abs(wf0-wfk)**2)
!    elseif (abs(aoverlap)<BELflags(1)) then
!      if (abs(nwf1)**2*BELflags(1)-abs(wfk)**2 .ge. 0.0d0) then
!        out=out*(abs(wf0-wfk)**2)
!        out=out*abs(wfk)**2
!      else
!        out=0.0d0
!      endif
!    endif

!write(*,*)" NoHea "
endif

!write(1300+ex1-1,*)Q,abs(wf0-wfk)**2,out
!write(*,*)Q,abs(wf0-wfk)**2,out
    out=-out
!write(*,*)"PQS out",real(PQS(:,1,ex1)),real(out)
  call stop_timer(tid)
  end subroutine funcQ

! caller routine for GwpValue to be used in minim
  subroutine funcValue(Q,out)
    use psidef 
    use global
    use BELglobals, only : psiold,aold,gdof,dim1red,m,s

    implicit none

    real(dop),intent(in)    :: Q(gdof)
    real(dop),intent(inout) :: out 

    complex(dop)            :: value

    call GwpValue(gdof,dim1red,&
!         aold(zphi(m,s)),aold(zxivec(m,s)),aold(zagmat(m,s)),psiold(zpsi(s)),Q,& 
         aold(zphase(m,s)),aold(zxivec(m,s)),aold(zagmat(m,s)),psiold(zpsi(s)),Q,& ! phase mod yapol
         value)
    out=-abs(value)**2

  end subroutine funcValue

! compute wave packet value at coordinate Q
  subroutine GwpValue(gdof,dim1,phi,xi,a,psi,Q,value)
    implicit none
    integer(long),intent(in) :: gdof,dim1
    complex(dop), dimension(dim1), intent(in)           :: phi
    complex(dop), dimension(gdof,dim1), intent(in)      :: xi
    complex(dop), dimension(gdof,gdof,dim1), intent(in) :: a
    complex(dop), dimension(dim1), intent(in)           :: psi
    real(dop), dimension(gdof), intent(in)              :: Q
    complex(dop),intent(out)                            :: value
    complex(dop)  :: ap(gdof,dim1)
    integer(long) :: f
    integer(long), save                             :: tid=0 

    call get_timer(tid, 'GwpValue')
    call start_timer(tid)
    do f=1,gdof
       ap(f,:)=a(f,f,:)
    end do
    value=sum(exp(MATMUL(Q(:)**2,ap(:,:))+MATMUL(Q(:),xi(:,:))+phi(:))*psi(:))
    call stop_timer(tid)
  end subroutine GwpValue

! compute wave packet value at coordinate P
  subroutine GwpValueP(gdof,dim1,phi,xi,a,psi,Q,value)
    implicit none
    integer(long),intent(in) :: gdof,dim1
    complex(dop), dimension(dim1), intent(in)           :: phi
    complex(dop), dimension(gdof,dim1), intent(in)      :: xi
    complex(dop), dimension(gdof,gdof,dim1), intent(in) :: a
    complex(dop), dimension(dim1), intent(in)           :: psi
    real(dop), dimension(gdof), intent(in)              :: Q
    complex(dop),intent(out)                            :: value
    complex(dop)  :: ap(gdof,dim1)
    integer(long) :: f
    integer(long), save                             :: tid=0 

    call get_timer(tid, 'GwpValueP')
    call start_timer(tid)
    do f=1,gdof
       ap(f,:)=a(f,f,:)
    end do
    value=sum(exp(MATMUL(Q(:)**2,ap(:,:))+MATMUL(Q(:),xi(:,:))+phi(:))*psi(:))
    call stop_timer(tid)
  end subroutine GwpValueP

! compute values of individual Gaussians at position Q
  function GaussianValuesP(gdof,dim1,phi,xi,a,Q) result (G)
    implicit none
    integer(long),intent(in) :: gdof,dim1
    complex(dop), dimension(dim1), intent(in)           :: phi
    complex(dop), dimension(gdof,dim1), intent(in)      :: xi
    complex(dop), dimension(gdof,gdof,dim1), intent(in) :: a
    real(dop), dimension(gdof), intent(in)              :: Q
    complex(dop),dimension(dim1)                            :: G
    complex(dop)  :: ap(gdof,dim1)
    integer(long) :: f
!$omp do
    do f=1,gdof
       ap(f,:)=a(f,f,:)
    end do
!$omp end do
    G=exp(MATMUL(Q(:)**2,ap(:,:))+MATMUL(Q(:),xi(:,:))+phi(:))
  end function GaussianValuesP

! compute values of individual Gaussians at position Q
  function GaussianValues(gdof,dim1,phi,xi,a,Q) result (G)
    implicit none
    integer(long),intent(in) :: gdof,dim1
    complex(dop), dimension(dim1), intent(in)           :: phi
    complex(dop), dimension(gdof,dim1), intent(in)      :: xi
    complex(dop), dimension(gdof,gdof,dim1), intent(in) :: a
    real(dop), dimension(gdof), intent(in)              :: Q
    complex(dop),dimension(dim1)                            :: G
    complex(dop)  :: ap(gdof,dim1)
    integer(long) :: f
!$omp do
    do f=1,gdof
       ap(f,:)=a(f,f,:)
    end do
!$omp end do
    G=exp(MATMUL(Q(:)**2,ap(:,:))+MATMUL(Q(:),xi(:,:))+phi(:))
  end function GaussianValues


  ! compute norm, and q,sigma,p expectation values and spread for the first
  ! dim1red GWPs
  subroutine gimmenorm(gdof,dim1,dim1red,phi,xi,a,psi,norm,pqISall,qexp,sigexp,pexp)
    
    use gwplib, only: fgaover1
    use BELglobals, only: ssdim

    integer(long),intent(in)                               :: gdof,dim1,dim1red
    complex(dop), dimension(dim1), intent(in)              :: phi
    complex(dop), dimension(dim1*ssdim), intent(in)        :: psi
    complex(dop), dimension(gdof,dim1), intent(in)         :: xi
    complex(dop), dimension(gdof,gdof,dim1), intent(in)    :: a

    complex(dop), intent(out)                              :: norm
    real(dop),intent(out),optional :: pqISall(:,:,:)
    real(dop),intent(out),optional :: qexp(:),sigexp(:),pexp(:)

    real(dop) :: qmin,qmax,pmin,pmax,sigma

    integer(long) :: e,e1,f,j,i
    complex(dop)  :: c,ax(gdof),ap(gdof),s
    real(dop)     :: qe,pe

    ! compute norm and q,sigma,p expectation values
    norm=0.0_dop
    if (present(pexp)) then
       pexp(:)=0.0_dop
    end if
    if (present(qexp)) then
       qexp(:)=0.0_dop
    end if
    if (present(sigexp)) then
       sigexp(:)=0.0_dop
    end if
!write(*,*)qexp(:),pexp(:),sigexp(:)
!$omp do
!do j=1,ssdim
!  do i=1,dim1*ssdim
!    write(1000,*)"psi in gimm",psi(i),i
!  enddo
!enddo
    do e=1,dim1red
!write(*,*)"psi",psi(e)
!write(*,*)"phi",phi(e)
!write(*,*)"xivec",xi(:,e)
!write(*,*)"a",a(:,:,e)
       do e1=1,dim1red
         s=(0.0d0,0.0d0)
!write(1000,*)fgaover1(phi(e1),phi(e),xi(:,e1),xi(:,e),&
!               a(:,:,e1),a(:,:,e),gdof)
          !this is \psi*_e1<g_e1|g_e>\psi_i
!          c=conjg(psi(e1))*fgaover1(phi(e1),phi(e),xi(:,e1),xi(:,e),&
!               a(:,:,e1),a(:,:,e),gdof)*psi(e)
!          c=fgaover1(phi(e1),phi(e),xi(:,e1),xi(:,e),&
!               a(:,:,e1),a(:,:,e),gdof)
          do j=0,ssdim-1
            s=s+conjg(psi(e1+j*dim1red))*fgaover1(phi(e1),phi(e),xi(:,e1),xi(:,e),&
               a(:,:,e1),a(:,:,e),gdof)*psi(e+j*dim1red)
!            write(1000,*)e1+j*dim1red
          enddo
          c=s
!          c=c*s
!    write(*,*)fgaover1(phi(e1),phi(e),xi(:,e1),xi(:,e),&
!               a(:,:,e1),a(:,:,e),gdof)
!write(1000,*)"norm",norm,e,e1,c,s
          norm=norm+c
!    write(1000,*)"norm",abs(norm)
          if (present(qexp).or.present(pexp).or.present(sigexp)) then
            do f=1,gdof
              ap(f)=-0.5_dop/(conjg(a(f,f,e1))+a(f,f,e))
              ax(f)=(conjg(a(f,f,e1))*xi(f,e)-a(f,f,e)*conjg(xi(f,e1)))
            end do
          endif
          if (present(qexp)) then
             qexp(:)=qexp(:)+dble(c*(conjg(xi(:,e1))+xi(:,e))*ap(:))
          end if
          if (present(pexp)) then
             pexp(:)=pexp(:)+dimag(-2.0_dop*c*ap(:)*ax(:))
          end if
          if (present(sigexp)) then
             sigexp(:)=sigexp(:)+dble(c*(ap(:)+((conjg(xi(:,e1))+xi(:,e))*ap(:))**2))
          end if
       end do
    enddo
!$omp end do
    norm=norm+1d-16
!write(1000,*)"norm",norm
    if (present(qexp)) then
       qexp(:)=qexp(:)/abs(norm)
       if (present(sigexp)) then
          sigexp(:)=dble(sqrt(sigexp(:)/abs(norm)-(qexp(:))**2))
       end if
    end if
    if (present(pexp)) then
       pexp(:)=pexp(:)/abs(norm)
    end if
    
!write(*,*)qexp(:),pexp(:),sigexp(:)

    ! compute q,sigma,p spread
    if (present(qexp) .and. present(pexp) .and. present(sigexp) .and. present(pqISall)) then
!$omp do
       do f=1,gdof
          qmin=qexp(f)
          qmax=qexp(f)
          pmin=pexp(f)
          pmax=pexp(f)
          do e=1,dim1red
             sigma=1.0/sqrt(-4.0_dop*dble(a(f,f,e)))
             pe=dimag(xi(f,e))-dble(xi(f,e))/dble(a(f,f,e))*dimag(a(f,f,e))
             qe=dble(xi(f,e))/(-2.0_dop*dble(a(f,f,e)))
             pmin=min(pmin,(pe)-1/sigma)
             pmax=max(pmax,(pe)+1/sigma)
             qmin=min(qmin,(qe)-sigma)
             qmax=max(qmax,(qe)+sigma)
          end do
          pqISall(1,f,1)=(qmax-qmin)/2.0_dop
          pqISall(2,f,1)=(pmax-pmin)/2.0_dop
          pqISall(3,f,1)=(qmax-qmin)/4.0_dop
!        write(*,*)qexp(f),pexp(f),sigexp(f)
       end do
!$omp end do
    end if
  end subroutine gimmenorm

  subroutine changePQfix(gdof,m,dim1,ex1,xi,a,PQfix)

    use runpropmod, only : BELint
    use griddatmod, only : spfdof
    use operdef,    only : mass
    use runpropmod, only : BELflags

    integer(long),intent(in) :: ex1,gdof,dim1,m
    complex(dop), dimension(gdof,dim1), intent(in)         :: xi
    complex(dop), dimension(gdof,gdof,dim1), intent(in)    :: a

    real(dop),intent(out) :: PQfix(3,gdof,*)

    integer(long) :: f,f1

!$omp do
    do f=1,gdof
       f1=spfdof(f,m)
       PQfix(1,f,ex1)=dble(xi(f,ex1))
       PQfix(2,f,ex1)=dimag(xi(f,ex1))
       PQfix(3,f,ex1)=1.0/sqrt(-4.0_dop*dble(a(f,f,ex1)))
!       write(*,*)  PQfix(1:3,f,ex1)
    end do
!$omp end do

  end subroutine changePQfix

  subroutine momspace(gdof,dim1,eta,zeta,tau,phi,xi,a)

    use global
    use runpropmod, only: BELflags

    use operdef
    use griddatmod
    use runpropmod
    use potevalmod
    use gh_elementsmod

    implicit none

    integer(long),intent(in) :: gdof,dim1

    complex(dop), dimension(gdof,gdof,dim1), intent(in) :: a
    complex(dop), dimension(gdof,dim1), intent(in)      :: xi
    complex(dop), dimension(dim1), intent(in)           :: phi
    complex(dop), dimension(gdof,gdof,dim1), intent(out) :: tau
    complex(dop), dimension(gdof,dim1), intent(out)      :: zeta
    complex(dop), dimension(dim1), intent(out)           :: eta

    integer(long) :: e,f
    real(dop)     :: norm, norm1, q, p

  do e=1,dim1
    do f=1,gdof
      q=dble(xi(f,e))
      p=dimag(xi(f,e))
      zeta(f,e)=dcmplx(-0.5_dop*p,-q)
      tau(f,f,e)=0.25_dop/dble(a(f,f,e))
      norm=dsqrt(pi/(-2.0_dop*dble(a(f,f,e))))
      norm1=dsqrt((-2.0_dop*dble(a(f,f,e))))
      norm=-0.5_dop*dlog(norm)-dlog(norm1)+0.25_dop*p**2/(a(f,f,e))
      eta(e)=eta(e)+dcmplx(norm,p*q)
    enddo
  enddo

  end subroutine momspace

  ! initialize GWP parameters from expectation values (sigma,p) 
  ! and (time propagated) values of the last position
  subroutine initPQfix(gdof,m,dim1,ex1,xi,a,PQfix,qexp,sigexp,pexp,aini,ipnz)
    
    use runpropmod, only : BELint
    use griddatmod, only : spfdof
    use operdef,    only : mass
    use runpropmod, only : BELflags

    integer(long),intent(in) :: ex1,gdof,dim1,m,ipnz
    complex(dop), dimension(gdof,dim1), intent(in)         :: xi
    complex(dop), dimension(gdof,gdof,dim1), intent(in)    :: a

    complex(dop) :: aini
    real(dop),intent(in) :: qexp(:),sigexp(:),pexp(:)
    real(dop),intent(out) :: PQfix(3,gdof,*)

    integer(long) :: f,f1
    real(dop) :: qtmp

!$omp do
    do f=1,gdof
       f1=spfdof(f,m)
       qtmp=dble(xi(f,ex1))
!write(1000,*)"qtmp",f,qtmp
       if (0.0d0*qtmp /= 0.0d0) then
       PQfix(1,f,ex1)=qexp(f)
       else
       PQfix(1,f,ex1)=dble(xi(f,ex1))
       endif
!    write(*,*)"dble(xi)",dble(xi(f,ex1))
!20161017
!       if (ex1>1) then
!          PQfix(1,f,ex1)=PQfix(1,f,ex1) + &
!              (dimag(xi(f,ex1))-dble(xi(f,ex1))*dimag(a(f,f,ex1)))*abs(BELint)/mass(f1)
!       end if
!20161017
       PQfix(2,f,ex1)=pexp(f)
       if (BELflags(10)==-1) then
!          PQfix(3,f,ex1)=1.0/sqrt(-4.0_dop*dble(a(f,f,2)))
          PQfix(3,f,ex1)=1.0/sqrt(-4.0_dop*dble(a(f,f,ex1)))
       else
!          PQfix(3,f,ex1)=1.0/sqrt(-4.0_dop*dble(aini)) !dw2d
!          PQfix(3,f,ex1)=1.0/sqrt(-4.0_dop*dble(aini))*0.5d0
!          PQfix(3,f,ex1)=0.5d0*sigexp(f)*BELflags(10) !NH3_4D
!          PQfix(3,f,ex1)=sigexp(f)*BELflags(10) !NH3_4D
          PQfix(3,f,ex1)=sigexp(f)
!          PQfix(3,f,ex1)=BELflags(6)
!20161017
!         if (ex1==1) then
!          PQfix(3,f,ex1)=sigexp(f)*BELflags(10)
!          PQfix(3,f,ex1)=1.0/sqrt(-4.0_dop*dble(aini))
!         else
!          PQfix(3,f,ex1)=sigexp(f)*BELflags(10)*1.0d0
!         endif
!20161017
!          PQfix(3,f,ex1)=sigexp(f)*BELflags(10)*1.01d0
       end if
       if (ipnz==0) PQfix(3,f,ex1)=1.0/sqrt(-4.0_dop*dble(a(f,f,ex1)))
!    write(*,*)BELflags(10),pexp(f),sigexp(f)
!    write(*,*)"PQfix",PQfix(3,f,ex1)
    end do
!$omp end do
!write(1000,*)"Pexp(f)",pexp(1:gdof)
!write(1000,*)"PQfix(2)",PQfix(2,1:gdof,ex1)
!stop

  end subroutine initPQfix

  subroutine shiftGwpSpf(m,s,ex1,gdof,dim1,dim1red,psiold,aold,newcoef,psinew,anew,&
       GSR,aoverlap,time,norm0,norma,pqISall,PQfix,PQsize,PQset,allprod,newprod,&
       allnewprod,istep,ierr,ierr_aovl,ierr_eigval,thr,ipnz,ierrp,re, &
       tempdimred)
    use global
    use channels, only : iBEL
    ! use maxv, only: maxspf,maxsta
    use griddatmod
    use psidef
    use runinwfmod
!    use runpropmod, only : BELprint,BELflags,BELint
!    use runpropmod, only : BELflags
    use runpropmod, only : BELprint,BELplotlims,BELflags
    use gwplib, only: gwptophi,renormdgwp,diagonal_moments, &
                        transahelp,transavec,transavec1,fgaover, &
                        gwpproj,printGWPSPF
    use randommod, only: randuniform

    use Nelder_Mead
    use BELglobals, only:DomainUpper,DomainLower,DomainMid,iloop,nwf1,ssdim,ssblock


    integer(long),intent(in) :: m,s,ex1,gdof,dim1,dim1red,ipnz,ierrp,re
    integer(long)                                  :: tempdimred

    complex(dop), dimension(dgldim), intent(in)              :: psiold
    complex(dop), dimension(agmatdim), intent(in)    :: aold
    complex(dop), dimension(dgldim), intent(inout)              :: psinew
    complex(dop), dimension(agmatdim), intent(inout)    :: anew
    complex(dop), dimension(ssblock,ssdim), intent(inout)              :: newcoef

    complex(dop),intent(in)  :: norm0
    complex(dop),intent(inout)  :: norma
    complex(dop)                :: normnew
    real(dop),intent(in) :: pqISall(:,:,:)

    complex(dop), dimension(npg(m)*dim1,npg(m)*dim1),intent(inout) :: GSR
    complex(dop),intent(out) :: aoverlap
    real(dop), intent(in)  :: time

    integer(long),intent(in) :: PQsize
    integer(long),intent(in) :: PQset(3)
    real(dop),intent(inout) :: PQfix(:,:,:)
    real(dop)               :: PQfixsave(3,gdof,dim1)
    real(dop)               :: PQS(3,gdof,ex1:ex1) !TM
    real(dop)               :: temp3(gdof)

    complex(dop),intent(inout) :: allprod(:,:),allnewprod(:,:)

    integer(long) :: f,e
    integer(long) :: ex3
    real(dop) :: PQ(gdof*PQsize),step(gdof*PQsize),vec(gdof*PQsize)
    real(dop) ::PQSt(3,gdof,ex1)

    complex(dop),dimension(npg(m)*ex1,npg(m)*ex1) :: gs2
    complex(dop),dimension(npg(m)*ex1,npg(m)*ex1) :: gs
    complex(dop),dimension(npg(m)*ex1,npg(m)*ex1) :: gs3
    complex(dop),dimension(npg(m)*ex1,npg(m)*ex1) :: gs4
    complex(dop),dimension(npg(m)*ex1*32)::work
    complex(dop),dimension(npg(m)*ex1)::b
    complex(dop),dimension(npg(m)*ex1)::b2
    complex(dop),dimension(npg(m)*ex1)::b3
    complex(dop)                       ::temp1
    complex(dop)                       ::temp2
    complex(dop),dimension(ex1)  :: newprod
    complex(dop),dimension(ex1)  :: oldprod
    integer,dimension(npg(m)*dim1)::ip

    complex(dop),dimension(dim1)  :: G

    real(dop) :: res
!    integer(long) :: iloop !TM
    real(dop) ::gsmax,gsmin
    integer(long) :: ierr,ierr_aovl,ierr_eigval
    integer(long) :: istep
    real(dop)     :: cnum,snum
    real(dop)     :: thr
    integer(long) :: itr,ifuncp
    integer(long),save :: ispr=2
    integer(long) :: ireg
    real(dop)               :: sortQ(gdof,dim1)
    real(dop)               :: deltaQsave
    real(dop)               :: beta,alpha,gamma,eta
    real(dop),save          :: theta=0.0d0

!      real(dop), dimension(3*dim1)                         :: work
!      complex(dop), dimension(dim1*dim1)                   :: workc
!      integer(long)                                        :: info,lda,lwork,ngwpact
!      real(dop), dimension(dim1), intent(out)              :: gsval
!      complex(dop), dimension(:,:), allocatable            :: evek

integer::i,i1,j,k
integer::iflag,icount
integer, save ::l=0
real(dop),dimension(gdof)::Q,dQ
complex(dop)::wf,wf1,wf2,tmpprod
real(8),dimension(3*ex1)::rw,eig
!    complex(dop)  :: amat(gdof,dim1)
logical::ro

    real(dop) :: out !TM
    real(dop) :: out1,out2,out3
    complex(dop)          :: wf0,wfk !TM
    complex(dop)            :: value !TM
    character(len=100) :: arg
    character(len=15) :: fQ,fQmax
    character(len=15) :: fP,fPmax
    character(len=15) :: dpsi,coef,pnew
    character(len=4) :: tmp
    real(dop) :: MAXfQ,MAXfP !TM
    character(len=15) :: la,pt

    integer::ival(dim1)
    integer::imax
    real(dop) :: mdat
    real(dop) :: qvec(dim1)
    real(dop) :: rnd
    real(dop) :: wmin

    integer(long), save                             :: tid=0

    call get_timer(tid, 'shiftGwpSpf')
    call start_timer(tid)

    ! find new position, either maximum of the function (ex1==1)
    ! or according to Eq.(4) from WK,TJF, PRL 2012

iloop=0
ierr=0
PQfixsave(1:3,1:gdof,ex1)=PQfix(1:3,1:gdof,ex1)
!10 continue
MAXfQ=0.0_dop
MAXfP=0.0_dop
!    write(*,*) "iloop", iloop
!    step(:)=0.0_dop
    step(:)=0.001_dop
!    step(1:gdof)=pqISall(1,1:gdof,1)*0.1_dop
    if (iloop == 1) PQfix(1:3,1:gdof,ex1)=PQfixsave(1:3,1:gdof,ex1)
    PQ(1:gdof)=PQfix(1,1:gdof,ex1)
!    write(*,*) "PQ ex1", PQ(1:gdof),ex1
!write(*,*)"PQS",PQfix(:,1:gdof,ex1:ex1)

    
!----------------------------------------
    call getarg(2,arg)
    fQ(1:2)="fQ"
    call int2char(istep,tmp)
    fQ(3:6)=tmp
    fQ(7:7)="_"
    call int2char(ex1,tmp)
    fQ(8:11)=tmp
    fQ(12:15)=".dat"
    fQmax=fQ
    dpsi=fQ
    pnew=fQ
    coef=fQ
    fP=fQ
    la=fQ
    pt=fQ
    dpsi(1:2)="dp"
    pnew(1:2)="pn"
    coef(1:2)="cf"
    fP(1:2)="fP"
    fPmax=fP
    fQmax(12:15)=".dta"
    fPmax(12:15)=".dta"
    la(1:2)="la"
    pt(1:2)="pt"

!20161017 
!    open(10,file=trim(arg)//"/"//fQ)
!    open(12,file=trim(arg)//"/"//dpsi)
!    open(15,file=trim(arg)//"/"//pnew)
!    open(13,file=trim(arg)//"/"//pt)
!!    if (abs(aoverlap)<BELflags(1) .or. (iloop==1) ) then
!!        open(14,file=trim(arg)//"/"//coef)
!!    endif
!    write(10,*)"# Q funcQ istep ex1",istep, ex1
!    do i=-1000,1000
!      Q=0.01d0*dble(i)
!      if (ex1==1) then
!      call GwpValue(gdof,dim1red,&
!         aold(zphase(m,s)),aold(zxivec(m,s)),aold(zagmat(m,s)),psiold(zpsi(s)),Q,& ! phase mod yapol
!         value)
!      out=abs(value)**2
!      write(12,*) Q,abs(value)**2
!      else
!      out=1
!      call GwpValue(gdof,dim1red,&
!         aold(zphase(1,1)),aold(zxivec(1,1)),aold(zagmat(1,1)),psiold(zpsi(1)),Q,& ! modified by TM
!         wf0)
!      call GwpValue(gdof,ex1-1,&
!         anew(zphase(1,1)),anew(zxivec(1,1)),anew(zagmat(1,1)),psinew(zpsi(1)),Q,& ! modified by TM
!         wfk)
!!      if (abs(aoverlap)<BELflags(1) .or. (iloop==1) ) then
!!!      if ( (iloop==1) ) then
!        do k=1,ex1-1
!          out=out*(1-exp(-sum((Q-PQfix(1,1:gdof,k))**2/(2*PQfix(3,1:gdof,k)**2))))
!!          out=out*(1-exp(-sum((Q-PQfix(1,1:gdof,k))**2/(0.05d0*2*PQfix(3,1:gdof,k)**2))))
!        enddo
!!        write(14,*) Q,out
!!      endif
!      out=out*abs(wf0-wfk)**2
!!!      out=out*abs(wf0)**2
!!        if (MAXfQ<out) then
!!          MAXfQ=out
!!          PQ(1:gdof)=Q
!!        endif
!      write(12,*) Q,abs(wf0-wfk)**2
!      write(15,*) Q,abs(wfk)**2
!!      endif
!      write(10,*) Q,out
!    enddo
!    close(10)
!    close(12)
!    close(14)
!    close(15)
!20161017

!if ( (istep==10) .and. (ex1==104) ) then
!                call printGWPSPF(BELprint,nspfdof(m),DomainLower,DomainUpper,&
!!                     dim(m,s),psiold(zpsi(s)),psiold(zphi(m,s)),psiold(zxi(m,s)),aold(zagmat(m,s)),&
!!                     dim(m,s),psinew(zpsi(s)),psinew(zphi(m,s)),psinew(zxi(m,s)),anew(zagmat(m,s)),&
!!                     dim(m,s),psiold(zpsi(s)),aold(zphi(m,s)),aold(zxivec(m,s)),aold(zagmat(m,s)),&
!!                     dim(m,s),psinew(zpsi(s)),anew(zphi(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)),&
!                     dim(m,s),psiold(zpsi(s)),aold(zphase(m,s)),aold(zxivec(m,s)),aold(zagmat(m,s)),&
!                     dim(m,s),psinew(zpsi(s)),anew(zphase(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)),&
!                     xplotlims=BELplotlims,channel=iBEL)
!endif

!----------------------------------------

!----------------------------------------
!20161017
!    open(11,file=trim(arg)//"/"//fQmax)
!    if (ex1 == 1) then
!       call minim(PQ(1:gdof),step(1:gdof),gdof,res,int(BELflags(9)),&
!            -1,1e-5_dop,gdof*3,&
!            0,1e-9_dop,vec(1:gdof),funcValue,e)
!!      PQ(1:gdof)=0.0d0
!      Q=PQ(1:gdof)
!      call GwpValue(gdof,dim1red,&
!         aold(zphase(m,s)),aold(zxivec(m,s)),aold(zagmat(m,s)),psiold(zpsi(s)),Q,& ! phase mod yapol
!         value)
!      out=abs(value)**2
!      write(11,*) Q,out
!    else
!       ! randomize direction of initial search
!       do f=1,gdof
!         if (randuniform()>0.5) then
!           step(f)=step(f)*(-1)
!         end if
!          ! To check domain
!!    write(*,*)PQ(1:gdof)
!
!         call funcQ(PQ,res)
!!    write(*,*)PQ(1:gdof)
!!         if (PQ(f)<DomainLower(f)) then
!!           PQ(f)=DomainLower(f)+(DomainMid(f)-DomainLower(f)) &
!!                                             *(2*randuniform()-0.5_dop)
!!         elseif (PQ(f)>DomainUpper(f)) then
!!           PQ(f)=DomainUpper(f)-(DomainUpper(f)-DomainMid(f)) &
!!                                             *(2*randuniform()-0.5_dop)
!!         endif
!!    write(*,*)PQ(1:gdof)
!       end do
!!write(997,*)ex1,real(PQ(1:gdof)),real(DomainLower(1:gdof)),real(DomainUpper(1:gdof))
!!write(*,*)ex1,real(PQ(1:gdof)),real(DomainLower(1:gdof)),real(DomainUpper(1:gdof))
!!      if (iloop==0) then
!       call minim(PQ(1:gdof),step(1:gdof),gdof,res,int(BELflags(9)),&
!            -1,1e-5_dop,gdof*3,&
!            0,1e-9_dop,vec(1:gdof),funcQ,e)
!!      else
!!         PQ(1:gdof)=PQ+randuniform()
!!      endif
!
!!      if(mod(ex1,2)/=0) then
!!          PQ(1:gdof)=(PQfix(1,1:gdof,ex1-1)-PQfix(1,1:gdof,1))
!!!        if ((PQfix(1,1:gdof,ex1-1)-PQfix(1,1:gdof,1))>0.0d0) then
!!        if ( PQ(1)>0.0d0) then
!!          PQ(1:gdof)=PQfix(1,1:gdof,1)-abs(PQfix(1,1:gdof,ex1-1)-PQfix(1,1:gdof,1))
!!        else
!!          PQ(1:gdof)=PQfix(1,1:gdof,1)+abs(PQfix(1,1:gdof,ex1-1)-PQfix(1,1:gdof,1))
!!        endif
!!      endif
!      Q=PQ(1:gdof)
!      out=1
!      call GwpValue(gdof,dim1red,&
!         aold(zphase(1,1)),aold(zxivec(1,1)),aold(zagmat(1,1)),psiold(zpsi(1)),Q,& ! modified by TM
!         wf0)
!      call GwpValue(gdof,ex1-1,&
!         anew(zphase(1,1)),anew(zxivec(1,1)),anew(zagmat(1,1)),psinew(zpsi(1)),Q,& ! modified by TM
!         wfk)
!      if (abs(aoverlap)<BELflags(1) .or. (iloop==1) ) then
!!      if ( (iloop==1) ) then
!       do k=1,ex1-1
!         out=out*(1-exp(-sum((Q-PQfix(1,1:gdof,k))**2/(2*PQfix(3,1:gdof,k)**2))))
!       enddo 
!      endif
!      out=out*abs(wf0-wfk)**2
!!      out=out*abs(wf0)**2
!      write(11,*) Q,out
!
!    end if
!
!    close(11)
!20161017

!----------------------------------------

!write(1300+ex1-1,*)
    ! save optimum value
    PQfix(1,1:gdof,ex1)=PQ(1:gdof)
! write(*,*)'Q(PQfix(1))',PQfix(1,1,ex1)
! write(*,*)'P(PQfix(2))',PQfix(2,1,ex1)
!if (ex1==55) write(*,*)'Q',PQfix(1,1,ex1)
!if (time>0.05) write(*,*)'Q',PQfix(1,1,ex1),ex1
!write(*,*)'Q',PQfix(1,1,ex1)

     ! Step 2: a (initially) from LHA
!if (ex1==2 .and. istep==1) then
!  deltaQsave=abs(PQfix(1,1,2)-PQfix(1,1,1))
!endif

i=1
k=0
iflag=1
!123 continue
    if (ex1==1) then
      PQ(1:gdof)=PQfix(1,1:gdof,ex1)
!     if (ipnz==1) then
     if ((ipnz==1) .and. (re<0))then
      call minim(PQ(1:gdof),step(1:gdof),gdof,res,int(BELflags(9)),&
              -1,1e-5_dop,gdof*3,&
              0,1e-9_dop,vec(1:gdof),funcValue,e)
!    open(11,file=trim(arg)//"/"//fQmax)
!      Q=PQ(1:1gdof)
!      call funcValue(Q,out)
!      write(11,*) Q,out
!    close(11)
    elseif ((ipnz==1) .and. (re>0))then
      Q(1:gdof)=PQfix(1,1:gdof,1)
    call GwpValue(gdof,ex1-1,&
         anew(zphase(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)),psinew(zpsi(s)),Q,&
         nwf1)
    endif
      PQfix(1,1:gdof,ex1)=PQ(1:gdof)
      call ValidateLHA(ex1,gdof,dim1,anew(zagmat(m,s)),PQfix(:,1:gdof,ex1),iflag,istep,s)
      PQfix(3,1:gdof,ex1)=PQfix(3,1:gdof,ex1)*BELflags(10)
!      PQfix(3,1:gdof,ex1)=PQfix(3,1:gdof,ex1)*0.5d0
        PQS(:,1:gdof,ex1:ex1)=PQfix(:,1:gdof,ex1:ex1)
!            write(*,*)'PQS',PQS(:,1:gdof,ex1:ex1)
!        call PQStoGWP(ex1,ex1,gdof,dim1,anew(zphase(m,s)),psinew(zxi(m,s)),&
!                     anew(zxivec(m,s)),anew(zagmat(m,s)),PQS)
!      call fgaover(gs3(1:ex1,1:ex1),anew(zphase(m,s)),anew(zphase(m,s)), &
!                  psinew(zxi(m,s)),psinew(zxi(m,s)), &
!                  anew(zagmat(m,s)),anew(zagmat(m,s)),ex1,ex1,gdof)
!        write(*,*) "ex1",ex1, PQfix(1,1:gdof,ex1),PQfix(2,1:gdof,ex1), & 
!                    PQfix(3,1:gdof,ex1), abs(gs3(ex1,ex1))
      PQ(1:gdof)=PQfix(2,1:gdof,ex1)
    else
!      step(1:gdof)=pqISall(1,1:gdof,1)*0.5_dop
      wmin=10000_dop
      step(1:gdof)=pqISall(1,1:gdof,1)*0.5_dop !dw2d
!      step(1:gdof)=pqISall(1,1:gdof,1)*0.8d0
!      step(1:gdof)=pqISall(1,1:gdof,1)*(0.5_dop+dble(iflag/10d0))
      PQ(1:gdof)=(PQfix(1,1:gdof,ex1))
     if (ipnz==1) then !p0copy
      do f=1,gdof
        if (randuniform()>0.5) then
          step(f)=step(f)*(-1)
        end if
      enddo
!        write(*,*) "ex1",ex1, PQfix(1,1:gdof,ex1),PQfix(2,1:gdof,ex1), & 
!                    PQfix(3,1:gdof,ex1), abs(gs3(ex1,ex1-1))
      call minim(PQ(1:gdof),step(1:gdof),gdof,res,int(BELflags(9)),&
              -1,1e-5_dop,gdof*3,&
              0,1e-9_dop,vec(1:gdof),funcQ,e)
!    open(11,file=trim(arg)//"/"//fQmax)
!      Q=PQ(1:1gdof)
!      call funcQ(Q,out)
!      write(11,*) Q,out
!    close(11)
     endif
      PQfix(1,1:gdof,ex1)=PQ(1:gdof)
!        write(*,*) "ex1",ex1, PQfix(1,1:gdof,ex1),PQfix(2,1:gdof,ex1), & 
!                    PQfix(3,1:gdof,ex1), abs(gs3(ex1,ex1-1))
      call ValidateLHA(ex1,gdof,dim1,anew(zagmat(m,s)),PQfix(:,1:gdof,ex1),iflag,istep,s)
      PQfix(3,1:gdof,ex1)=PQfix(3,1:gdof,ex1)*BELflags(10)
!      PQfix(3,1:gdof,ex1)=PQfix(3,1:gdof,ex1)*0.5d0
      PQS(:,1:gdof,ex1:ex1)=PQfix(:,1:gdof,ex1:ex1)
!      call PQStoGWP(ex1,ex1,gdof,dim1,anew(zphase(m,s)),psinew(zxi(m,s)),&
!                   anew(zxivec(m,s)),anew(zagmat(m,s)),PQS)
!      call fgaover(gs3(1:ex1,1:ex1),anew(zphase(m,s)),anew(zphase(m,s)), &
!                  psinew(zxi(m,s)),psinew(zxi(m,s)), &
!                  anew(zagmat(m,s)),anew(zagmat(m,s)),ex1,ex1,gdof)
!124 continue
      do e=1,PQsize
         do f=1,gdof
!            step(e+(f-1)*PQsize)=pqISall(PQset(e),f,1)*0.1_dop !dw2d
           step(e+(f-1)*PQsize)=pqISall(PQset(e),f,1)*0.5_dop
            !step(e+(f-1)*PQsize)=pqISall(PQset(e),f,1)*2.0_dop
            PQ(e+(f-1)*PQsize)=PQfix(PQset(e),f,ex1)
         end do
      end do
!     if (ipnz==1) then
     if  ( (ipnz==1) .and. (re<0) ) then
      call minim(PQ(1:gdof*PQsize),step(1:gdof*PQsize),PQsize*gdof,res,int(BELflags(13)),&
            -1,1e-5_dop,gdof*3*PQsize,&
             0,1e-9_dop,vec(1:gdof*PQsize),funcPcall,e)
     endif
      PQfix(2,1:gdof,ex1)=PQ(1:gdof)
      PQS(:,1:gdof,ex1:ex1)=PQfix(:,1:gdof,ex1:ex1)
!      call PQStoGWP(ex1,ex1,gdof,dim1,anew(zphase(m,s)),psinew(zxi(m,s)),&
!                   anew(zxivec(m,s)),anew(zagmat(m,s)),PQS)
!      call fgaover(gs3(1:ex1,1:ex1),anew(zphase(m,s)),anew(zphase(m,s)), &
!                  psinew(zxi(m,s)),psinew(zxi(m,s)), &
!                  anew(zagmat(m,s)),anew(zagmat(m,s)),ex1,ex1,gdof)
!        write(*,*) "ex1",ex1, PQfix(1,1:gdof,ex1),PQfix(2,1:gdof,ex1), & 
!                    PQfix(3,1:gdof,ex1), abs(gs3(ex1,ex1-1))

!      do i=1,ex1-1
!        if (abs(gs3(ex1,i))>thr) then
!          write(*,*)time/fs,ex1,i,abs(gs3(ex1,i))
!          do f=1,gdof
!            PQfix(1,f,ex1)=PQfix(1,f,ex1)*randuniform()
!          enddo
!          if (k==i) then
!!            write(*,*)time/fs,ex1,i,anew(zagmat(m,s)+i*2**2-1)
!            call changePQfix(gdof,m,dim1,i,psinew(zxi(m,s)),anew(zagmat(m,s)),PQSt)
!            do f=1,gdof
!              PQSt(3,f,i)=PQSt(3,f,i)*2d0
!!           write(*,*) PQSt(1:3,f,i)
!            enddo
!!           write(*,*)""
!!           write(*,*) PQSt(1:3,1:gdof,i)
!!           write(*,*)""
!            call PQStoGWP2(i,i,gdof,dim1,anew(zphase(m,s)),psinew(zxi(m,s)),&
!                   anew(zxivec(m,s)),anew(zagmat(m,s)),PQSt)
!            call fgaover(gs3(1:ex1-1,1:ex1-1),anew(zphase(m,s)),anew(zphase(m,s)), &
!                  psinew(zxi(m,s)),psinew(zxi(m,s)), &
!                  anew(zagmat(m,s)),anew(zagmat(m,s)),ex1-1,ex1-1,gdof)
!!           write(*,*)time/fs,ex1,i,anew(zagmat(m,s)+i*2**2-1)
!!           write(*,*)""
!             gs(1:ex1-1,1:ex1-1)=gs3(1:ex1-1,1:ex1-1)
!             PQ(1:gdof)=PQSt(2,1:gdof,i)
!             PQfix(3,1:gdof,i)=PQSt(3,1:gdof,i)
!!           write(*,*) PQfix(1:3,1:gdof,i)
!             call funcP(PQ,res,PQfix,PQsize,PQset,gdof,i,i,allprod,psinew,anew,psiold,aold,&
!                dim1,dim1red,norm0,norma,m,s,gimmedev=.true.)
!!           write(*,*)time/fs,ex1,i,anew(zagmat(m,s)+i*2**2-1)
!              b(1:ex1-1)=allprod(1:ex1-1)
!             call zhesv('U',ex1-1,1,gs,npg(m)*ex1,ip,b,npg(m)*ex1,work,npg(m)*ex1*32,j)
!             if (j/=0) then
!               stop 'zhesv failed in shiftGwpSpf'
!             endif
!             psinew(zpsi(s):zpsi(s)+ex1-1-1)=b(1:ex1-1)
!!             go to 124
!!          endif
!          k=i
!          goto 123
!        endif
!      enddo
!      do i=1,ex1-1
!        if (abs(gs3(ex1,i))>thr) then
!          write(*,*)time/fs,ex1,i,abs(gs3(ex1,i))
!          do f=1,gdof
!            if (wmin>PQfix(3,f,ex1)) then
!              wmin=PQfix(3,f,ex1)
!            endif
!          enddo
!          do f=1,gdof
!            PQfix(3,f,ex1)=wmin
!          enddo
!!          write(*,*)time/fs,ex1,i,PQfix(3,1:gdof,ex1)
!          iflag=iflag+1
!          goto 123
!        endif
!      enddo

!      do i=1,ex1-1
!        if (abs(gs3(ex1,i))>thr) then
!          write(*,*)time/fs,ex1,i,abs(gs3(ex1,i))
!          iflag=iflag+1
!          goto 123
!        endif
!      enddo
!      j=0
!      do i=1,ex1-1
!!          write(*,*)time/fs,ex1,i,abs(gs3(ex1,i))
!        if (abs(gs3(ex1,i))>thr) then
!          j=j+1
!          ival(j)=i
!!          write(*,*)time/fs,ex1,ival(j),abs(gs3(ex1,ival(j)))
!        endif
!      enddo
!      if (j>0) then
!        do i=1,j
!          qvec(i)=0.0d0
!        enddo
!        do i=1,j
!          do f=1,gdof
!            qvec(i)=qvec(i)+(PQfix(1,f,ival(i))-PQfix(1,f,1))**2.0d0
!          enddo
!          qvec(i)=sqrt(qvec(i))
!        enddo
!        do i=1,j
!          if (i==1) then
!            imax=1
!            mdat=qvec(i)
!          elseif (i>1) then
!            if (qvec(i)>mdat) then
!              mdat=qvec(i)
!              imax=i
!            endif
!          endif
!        enddo
!!            write(*,*) "ex1",ex1, PQfix(1,1:gdof,ex1),PQfix(2,1:gdof,ex1), & 
!!                        PQfix(3,1:gdof,ex1), abs(gs3(ex1,ival(imax)))
!            do f=1,gdof
!              call random_number(rnd)
!              if (ival(imax)==1) then
!                PQfix(1,f,ex1)=rnd*(PQfix(1,f,ival(imax))-PQfix(1,f,2))+PQfix(1,f,ival(imax))
!              else
!                PQfix(1,f,ex1)=rnd*(PQfix(1,f,ival(imax))-PQfix(1,f,1))+PQfix(1,f,ival(imax))
!              endif
!              step(1:gdof)=pqISall(1,1:gdof,1)*rnd
!            enddo
!!            write(iBEL,*)"fq",time/fs,ex1,ival(imax),abs(gs3(ex1,ival(imax)))
!!            write(*,*) "ex1",ex1, PQfix(1,1:gdof,ex1),PQfix(2,1:gdof,ex1), & 
!!                        PQfix(3,1:gdof,ex1), abs(gs3(ex1,ival(imax)))
!!            write(*,*)""
!!            call myflush(iBEL)
!            go to 123
!      endif
    endif
    PQ(1:gdof)=PQfix(2,1:gdof,ex1)
    if  (ipnz==1)  then
    call funcP(PQ,res,PQfix,PQsize,PQset,gdof,ex1,ex1,allprod,allnewprod,newcoef,psinew,anew,psiold,aold,&
         dim1,dim1red,norm0,norma,m,s,gimmedev=.true.)
    elseif (ipnz==0) then
      PQS(:,1:gdof,ex1:ex1)=PQfix(:,1:gdof,ex1:ex1)
      call PQStoGWP(ex1,ex1,gdof,dim1,anew(zphase(m,s)),psinew(zxi(m,s)),&
                   anew(zxivec(m,s)),anew(zagmat(m,s)),PQS)
    endif

!    open(11,file=trim(arg)//"/"//fPmax)
!      write(11,*) PQ(1:gdof),res
!    close(11)

!      Q(1:gdof)=PQfix(1,1:gdof,ex1)
!      call GwpValue(gdof,dim1red,&
!         aold(zphase(1,1)),aold(zxivec(1,1)),aold(zagmat(1,1)),psiold(zpsi(1)),Q,&
!         wf0)
!      G(1:ex1)=GaussianValues(gdof,ex1, &
!           anew(zphase(1,1)),anew(zxivec(1,1)),anew(zagmat(1,1)),Q)
!    if(ex1>1) then
!      call GwpValue(gdof,ex1-1,&
!         anew(zphase(1,1)),anew(zxivec(1,1)),anew(zagmat(1,1)),psinew(zpsi(1)),Q,&
!         wfk)
!      psinew(zpsi(s)+ex1-1)=(wf0-wfk)/G(ex1)
!      write(*,*)ex1, psinew(zpsi(s)+ex1-1)
!      call GwpValue(gdof,ex1,&
!         anew(zphase(1,1)),anew(zxivec(1,1)),anew(zagmat(1,1)),psinew(zpsi(1)),Q,&
!         wfk)
!      write(*,*)ex1, abs(wf0-wfk)
!     else
!      psinew(zpsi(s)+ex1-1)=(wf0)/G(ex1)
!!      write(*,*)ex1, wf0,wfk,G(ex1)
!      write(*,*)ex1, psinew(zpsi(s)+ex1-1)
!      call GwpValue(gdof,ex1,&
!         anew(zphase(1,1)),anew(zxivec(1,1)),anew(zagmat(1,1)),psinew(zpsi(1)),Q,&
!         wfk)
!      write(*,*)ex1, abs(wf0-wfk)
!     endif
!if ((istep==1) .and. (ex1==1) )then
!  call getarg(2,arg)
!  open(101,file=trim(arg)//".101")
!            write(101,*) ex1, psinew(zpsi(s)+ex1-1),PQfix(1,1:gdof,ex1),PQfix(2,1:gdof,ex1), & 
!                        PQfix(3,1:gdof,ex1),s
!  close(101)
!else
!  open(101,file=trim(arg)//".101",position="append")
!            write(101,*) ex1, psinew(zpsi(s)+ex1-1),PQfix(1,1:gdof,ex1),PQfix(2,1:gdof,ex1), & 
!                        PQfix(3,1:gdof,ex1),s
!  close(101)
!endif
!----------------------------------------
!if (ex1>1) then
!  !  PQfix(3,1:gdof,ex1)=0.9d0*PQfix(3,1:gdof,1) !BEL_v03_v13
!  !  PQfix(3,1:gdof,ex1)=0.8d0*PQfix(3,1:gdof,1) !BEL_v03_v14
!  !  PQfix(3,1:gdof,ex1)=0.77d0*PQfix(3,1:gdof,1) !BEL_v03_v15
!
!  !if (istep>=2) then
!  !  thr=0.8d0
!  !  PQfix(3,1:gdof,ex1)=0.5d0*PQfix(3,1:gdof,1) 
!  !endif
!
!  if (ex1>3) then
!    PQ(1:gdof)=PQfix(1,1:gdof,ex1)
!    step(1:gdof)=pqISall(1,1:gdof,1)*0.1_dop
!    do f=1,gdof
!      if (randuniform()>0.5) then
!        step(f)=step(f)*(-1)
!      end if
!    enddo
!    call minim(PQ(1:gdof),step(1:gdof),gdof,res,int(BELflags(9)),&
!            -1,1e-5_dop,gdof*3,&
!            0,1e-9_dop,vec(1:gdof),funcQ,e)
!    PQfix(1,1:gdof,ex1)=PQ(1:gdof)
!
!    ireg=1
!!    sortQ(1:gdof,1:dim1,1:ispr**gdof)=0.0d0
!!    sortQ(1:gdof,1:ex1-1)=PQfix(1,1:gdof,1:ex1-1)
!    do f=1,gdof
!      if (PQfix(1,f,ex1)-PQfix(1,f,1)>0.0d0) then
!        ireg=ireg*2-1
!        if (f=gdof) then
!         iregdim(ireg)=iregdim(ireg)+1
!         sortQ(1:gdof,iregdim(ireg),ireg)=PQfix(1,1:gdof,ex1)
!        endif
!      else
!        ireg=ireg*2
!        if (f=gdof) then
!         iregdim(ireg)=iregdim(ireg)+1
!         sortQ(1:gdof,iregdim(ireg),ireg)=PQfix(1,1:gdof,ex1)
!        endif
!      endif
!    enddo
!    do i=1,iregdim(ireg)-1
!      do j=i+1,iregdim(ireg)
!        if (sortQ(1,i,ireg) .gt. sortQ(1,j,ireg) ) then
!          temp3(1:gdof)=sortQ(1:gdof,i,ireg)
!          sortQ(1:gdof,i,ireg)=sortQ(1:gdof,j,ireg)
!          sortQ(1:gdof,j,ireg)=temp3(1:gdof)
!        endif
!      enddo
!    enddo
!    do i=1,ex1-2
!      do j=i+1,ex1-1
!        if (sortQ(1,i) .gt. sortQ(1,j) ) then
!          temp3(1:gdof)=sortQ(1:gdof,i)
!          sortQ(1:gdof,i)=sortQ(1:gdof,j)
!          sortQ(1:gdof,j)=temp3(1:gdof)
!        endif
!      enddo
!    enddo
!
!    if (PQfix(1,1,ex1)-PQfix(1,1,1)>0.0d0 ) then
!      cnum=1.0d0
!      do i=1,ex1-1
!        if (PQfix(1,1,i)==sortQ(1,ex1-1)) then
!          ex3=i
!          do j=ex1-1,1,-1
!            if (abs(sortQ(1,j)-sortQ(1,j-1)) >= 1.0d-3) then
!              snum=0.5d0*abs(sortQ(1,j)-sortQ(1,j-1))
!              go to 222
!            endif
!          enddo
!222 continue
!!          snum=0.5d0*abs(sortQ(1,ex1-1)-sortQ(1,ex1-2))
!          PQfix(1,1,ex1)=PQfix(1,1,ex3)+sign(snum,cnum)
!        endif
!      enddo
!    else
!      cnum=-1.0d0
!      do i=1,ex1-1
!        if (PQfix(1,1,i)==sortQ(1,1)) then
!          ex3=i
!          do j=1,ex1-1
!            if (abs(sortQ(1,j)-sortQ(1,j+1)) >= 1.0d-3) then
!              snum=0.5d0*abs(sortQ(1,j)-sortQ(1,j+1))
!              go to 333
!            endif
!          enddo
!333 continue
!!          snum=0.5d0*abs(sortQ(1,1)-sortQ(1,2))
!          PQfix(1,1,ex1)=PQfix(1,1,ex3)+sign(snum,cnum)
!        endif
!      enddo
!    endif
!  elseif (ex1==2) then
!    do i=1,gdof**ispr
!      iregdim(i)=0
!    enddo
!    do f=1,gdof
!      if (PQfix(1,f,ex1)-PQfix(1,f,1)>0.0d0) then
!        ireg=ireg*2-1
!        if (f=gdof) then
!         iregdim(ireg)=iregdim(ireg)+1
!         sortQ(1:gdof,iregdim(ireg),ireg)=PQfix(1,1:gdof,ex1)
!        endif
!      else
!        ireg=ireg*2
!        if (f=gdof) then
!         iregdim(ireg)=iregdim(ireg)+1
!         sortQ(1:gdof,iregdim(ireg),ireg)=PQfix(1,1:gdof,ex1)
!        endif
!      endif  
!    enddo
!    ex3=ex1-1
!    cnum=1.0d0
!    snum=0.0d0
!    do f=1,gdof
!    snum=snum+0.5d0*abs(PQfix(1,f,2)-PQfix(1,f,1))
!    deltaQsave=abs(PQfix(1,f,2)-PQfix(1,f,1))
!    enddo
!    snum=snum/gdof
!!    PQfix(1,1,ex1)=PQfix(1,1,ex3)+deltaQsave
!    PQfix(1,1:gdof,ex1)=PQfix(1,1:gdof,ex3)+sign(snum,cnum)
!!  else
!  elseif (ex1==3) then
!    do f=1,gdof
!      if (PQfix(1,f,ex1)-PQfix(1,f,1)>0.0d0) then
!        ireg=ireg*2-1
!        if (f=gdof) then
!         iregdim(ireg)=iregdim(ireg)+1
!         sortQ(1:gdof,iregdim(ireg),ireg)=PQfix(1,1:gdof,ex1)
!        endif
!      else
!        ireg=ireg*2
!        if (f=gdof) then
!         iregdim(ireg)=iregdim(ireg)+1
!         sortQ(1:gdof,iregdim(ireg),ireg)=PQfix(1,1:gdof,ex1)
!        endif
!      endif  
!    enddo
!    ex3=ex1-2
!    cnum=-1.0d0
!!    snum=0.5d0*abs(PQfix(1,1,3)-PQfix(1,1,1))
!    snum=0.5d0*abs(PQfix(1,1,2)-PQfix(1,1,1))
!    deltaQsave=abs(PQfix(1,1,3)-PQfix(1,1,1))
!!    PQfix(1,1,ex1)=PQfix(1,1,ex3)-deltaQsave
!    PQfix(1,1:gdof,ex1)=PQfix(1,1:gdof,ex3)+sign(snum,cnum)
!  endif
!
!  gs3(1:ex1,1:ex1)=(0.0d0,0.0d0)
!  gs4(1:ex1,1:ex1)=(0.0d0,0.0d0)
!
!  if (istep>=2) then
!  !  gamma=0.5d0
!    gamma=1.0d0
!  !  gamma=sqrt(0.5d0)
!    beta=1.0d0/(gamma*gamma)
!    eta=-1.0d0/(2.0d0*PQfix(3,1,1))**2.0d0
!    PQfix(3,1:gdof,ex1)=gamma*PQfix(3,1:gdof,1) 
!!    write(*,*)"width",ex1,PQfix(3,1:gdof,ex1)
!    alpha=1.7d0*PQfix(3,1,ex1)
!  !  alpha=0.05d0
!  !  eta=-1.0d0/(2.0d0*PQfix(3,1,ex3))**2.0d0
!  !  PQfix(3,1:gdof,ex1)=gamma*PQfix(3,1:gdof,ex3) 
!  !  if (PQfix(3,1,ex1)<5.0d-2) then
!  !    PQfix(3,1:gdof,ex1)=PQfix(3,1:gdof,ex3) 
!  !  endif
!  !  if (ex1==2) then
!
!
!  !  thr=(4.0d0*beta/(1.0d0+beta)**2.0d0)**(0.25d0) & 
!  !       *abs(exp(beta*alpha*alpha &
!  !       /(-(2.0d0*PQfix(3,1,ex3))**2.0d0*(1.0d0+beta))))
!  !  thr=(4.0d0*beta/(1.0d0+beta)**2.0d0)**(0.25d0) & 
!  !       *abs(exp(beta*alpha*alpha &
!  !       /(-(2.0d0*PQfix(3,1,1))**2.0d0*(1.0d0+beta))) &
!  !       *exp((PQfix(2,1,2)-PQfix(2,1,1))**2.0d0/(4.0d0*eta*(1.0d0+beta))) &
!  !       *cdexp((0.0d0,1.0d0)*(PQfix(2,1,2)-PQfix(2,1,1)) &
!  !       *(eta*PQfix(1,1,1)+beta*eta*(PQfix(1,1,1)-alpha))/(eta*(1.0d0+beta))))
!  !  endif
!!  write(*,*)"thr",thr
!  !stop
!  endif
!
!!  if (ex1>4) then
!!!    snum=0.5d0*abs(PQfix(1,1,ex3)-PQfix(1,1,ex3-2))
!!    snum=0.5d0*abs(PQfix(1,1,ex3)-PQfix(1,1,ex3-2))
!!  else
!!    snum=0.5d0*abs(PQfix(1,1,2)-PQfix(1,1,1))
!!  endif  
!!  if (ex1>3) then
!!  !  if (PQfix(1,1,ex3)>0.0d0) then
!!    if (PQfix(1,1,ex3)-PQfix(1,1,1)>0.0d0) then 
!!      cnum=1.0d0
!!    else
!!      cnum=-1.0d0
!!    endif
!!    PQfix(1,1,ex1)=PQfix(1,1,ex3)+sign(snum,cnum)
!!  elseif (ex1==2) then
!!!    PQfix(1,1,ex1)=PQfix(1,1,ex3)+abs(PQfix(1,1,2)-PQfix(1,1,1))
!!    PQfix(1,1,ex1)=PQfix(1,1,ex3)+deltaQsave
!!  elseif (ex1==3) then
!!!    PQfix(1,1,ex1)=PQfix(1,1,ex3)-abs(PQfix(1,1,2)-PQfix(1,1,1))
!!    PQfix(1,1,ex1)=PQfix(1,1,ex3)-deltaQsave
!!  endif
!
!!      write(*,*) "itr",itr,"before",PQfix(1,1,ex1),PQfix(3,1,ex1)
!  call ValidateLHA(ex1,gdof,dim1,anew(zagmat(m,s)),PQfix(:,1:gdof,ex1))
!!      write(*,*) "itr",itr,"after",PQfix(1,1,ex1),PQfix(3,1,ex1)
!  do e=1,PQsize
!     do f=1,gdof
!        step(e+(f-1)*PQsize)=pqISall(PQset(e),f,1)*0.1_dop
!        PQ(e+(f-1)*PQsize)=PQfix(PQset(e),f,ex1)
!     end do
!  end do
!  call minim(PQ(1:gdof*PQsize),step(1:gdof*PQsize),PQsize*gdof,res,int(BELflags(9)),&
!          -1,1e-5_dop,gdof*3*PQsize,&
!           0,1e-9_dop,vec(1:gdof*PQsize),funcPcall,e)
!  PQfix(2,1:gdof,ex1)=PQ(1:gdof)
!  PQS(:,1:gdof,ex1:ex1)=PQfix(:,1:gdof,ex1:ex1)
!  call PQStoGWP(ex1,ex1,gdof,dim1,anew(zphase(m,s)),psinew(zxi(m,s)),&
!               anew(zxivec(m,s)),anew(zagmat(m,s)),PQS)
!  call fgaover(gs3(1:ex1,1:ex1),anew(zphase(m,s)),anew(zphase(m,s)), &
!               psinew(zxi(m,s)),psinew(zxi(m,s)), &
!               anew(zagmat(m,s)),anew(zagmat(m,s)),ex1,ex1,gdof)
!  gs4(ex1,ex3)=gs3(ex1,ex3) 
!
!!  if (PQfix(1,1,ex3)>0.0d0) then
!!    cnum=1.0d0
!!  else
!!    cnum=-1.0d0
!!  endif
!
!  itr=1
!  ifuncp=0
!555  continue
!
!
!  do while (abs(abs(gs3(ex1,ex3))/thr-1.0d0) .gt. 1.0d-3)
!777 continue
!      itr=itr+1
!      if (itr==100) then
!!        itr=1
!!        thr=thr-0.1d0
!!        if (thr==0.1) stop"AAA"
!!        go to 555
!!        ierr=1
!!        go to 666
!         ifuncp=1
!         if (ex1>3) then
!           if (PQfix(1,1,ex3)==sortQ(1,ex1-1)) then
!             do j=ex1-1,1,-1
!               if (abs(sortQ(1,j)-sortQ(1,j-1)) >= 1.0d-3) then
!                 snum=0.5d0*abs(sortQ(1,j)-sortQ(1,j-1))
!                 go to 22
!               endif
!             enddo
!22 continue
!!             snum=0.5d0*abs(sortQ(1,ex1-1)-sortQ(1,ex1-2))
!           elseif (PQfix(1,1,ex3)==sortQ(1,1)) then
!             do j=1,ex1-1
!               if (abs(sortQ(1,j)-sortQ(1,j+1)) >= 1.0d-3) then
!                 snum=0.5d0*abs(sortQ(1,j)-sortQ(1,j+1))
!                 go to 33
!               endif
!             enddo
!33 continue
!!             snum=0.5d0*abs(sortQ(1,1)-sortQ(1,2))
!           endif
!         elseif (ex1==2) then
!!           snum=0.5d0*abs(PQfix(1,1,2)-PQfix(1,1,1))
!           snum=0.5d0*deltaQsave
!         elseif (ex1==3) then
!!           snum=0.5d0*abs(PQfix(1,1,2)-PQfix(1,1,1))
!           snum=0.5d0*deltaQsave
!!           snum=0.5d0*abs(PQfix(1,1,3)-PQfix(1,1,1))
!         endif
!!         if (ex1>4) then
!!              snum=0.5d0*abs(PQfix(1,1,ex3)-PQfix(1,1,ex3-2))
!!              snum=abs(PQfix(1,1,ex3)-PQfix(1,1,ex3-2))
!!         else
!!           snum=0.5d0*abs(PQfix(1,1,2)-PQfix(1,1,1))
!!         endif  
!          PQfix(1,1:gdof,ex1)=PQfix(1,1:gdof,ex3)+sign(snum,cnum)
!      elseif (itr==1000) then
!!        ierr=1
!        go to 666
!      endif
!      PQfix(1,1:gdof,ex1)=PQfix(1,1:gdof,ex1)+sign(snum,cnum)
!      if (ex1>3) then
!        if (PQfix(1,1,ex3)-PQfix(1,1,1)>0.0d0) then 
!          if (PQfix(1,1,ex1)-PQfix(1,1,ex3)<0.0d0) then 
!            cnum=1.0d0
!!            if (ex1>4) then
!!              snum=0.5d0*abs(PQfix(1,1,ex3)-PQfix(1,1,ex3-2))
!!              snum=abs(PQfix(1,1,ex3)-PQfix(1,1,ex3-2))
!!            else
!!              snum=0.5d0*abs(PQfix(1,1,2)-PQfix(1,1,1))
!!            endif  
!!            PQfix(1,1,ex1)=PQfix(1,1,ex3)+sign(snum,cnum)
!          endif
!        else
!          if (PQfix(1,1,ex1)-PQfix(1,1,ex3)>0.0d0) then 
!            cnum=-1.0d0
!!            if (ex1>4) then
!!              snum=0.5d0*abs(PQfix(1,1,ex3)-PQfix(1,1,ex3-2))
!!              snum=abs(PQfix(1,1,ex3)-PQfix(1,1,ex3-2))
!!            else
!!              snum=0.5d0*abs(PQfix(1,1,2)-PQfix(1,1,1))
!!            endif  
!            PQfix(1,1:gdof,ex1)=PQfix(1,1:gdof,ex3)+sign(snum,cnum)
!          endif
!        endif
!      elseif (ex1==2) then
!        if (PQfix(1,1,ex1)-PQfix(1,1,ex3)<=0.0d0) then
!          cnum=1.0d0
!!          snum=abs(PQfixsave(1,1,ex1)-PQfix(1,1,ex3))
!          PQfix(1,1:gdof,ex1)=PQfix(1,1:gdof,ex3)+sign(snum,cnum)
!!          PQfix(1,1,ex1)=PQfix(1,1,ex3)+deltaQsave
!        endif
!      elseif (ex1==3) then
!        if (PQfix(1,1,ex1)-PQfix(1,1,ex3)>=0.0d0) then
!          cnum=-1.0d0
!!          snum=abs(PQfixsave(1,1,ex1)-PQfix(1,1,ex3))
!          PQfix(1,1:gdof,ex1)=PQfix(1,1:gdof,ex3)+sign(snum,cnum)
!!          PQfix(1,1,ex1)=PQfix(1,1,ex3)-deltaQsave
!        endif
!      endif
!!        if (abs(PQfix(1,1,ex1))-abs(PQfix(1,1,ex3))<0.0d0) then !init center 0
!!          if (PQfix(1,1,ex3)>0.0d0) then
!!            cnum=1.0d0
!!          else
!!            cnum=-1.0d0
!!          endif
!!         if (ex1>4) then
!!           snum=0.5d0*abs(PQfix(1,1,ex3)-PQfix(1,1,ex3-2))
!!         else
!!           snum=0.5d0*abs(PQfix(1,1,ex1)-PQfix(1,1,ex3))
!!         endif  
!!          PQfix(1,1,ex1)=PQfix(1,1,ex3)+sign(snum,cnum)
!!        endif
!!      if ( (itr<100) .or. (itr>110))then
!      if ( (itr<100) )then
!        PQfix(3,1:gdof,ex1)=gamma*PQfix(3,1:gdof,1) 
!      endif
!      call ValidateLHA(ex1,gdof,dim1,anew(zagmat(m,s)),PQfix(:,1:gdof,ex1))
!      PQ(1:gdof)=PQfix(2,1:gdof,ex1)
!      if (ifuncp==0) then
!      call minim(PQ(1:gdof*PQsize),step(1:gdof*PQsize),PQsize*gdof,res,int(BELflags(9)),&
!           -1,1e-5_dop,gdof*3*PQsize,&
!           0,1e-9_dop,vec(1:gdof*PQsize),funcPcall,e)
!      PQfix(2,1:gdof,ex1)=PQ(1:gdof)
!      else
!!      call minim(PQ(1:gdof*PQsize),step(1:gdof*PQsize),PQsize*gdof,res,int(BELflags(9)),&
!!           -1,1e-5_dop,gdof*3*PQsize,&
!!           0,1e-9_dop,vec(1:gdof*PQsize),funcP2call,e)
!      PQfix(2,1:gdof,ex1)=PQfix(2,1:gdof,ex3)
!      PQ(1:gdof)=PQfix(2,1:gdof,ex1)
!      endif
!      PQS(:,1:gdof,ex1:ex1)=PQfix(:,1:gdof,ex1:ex1)
!      call PQStoGWP(ex1,ex1,gdof,dim1,anew(zphase(m,s)),psinew(zxi(m,s)),&
!                   anew(zxivec(m,s)),anew(zagmat(m,s)),PQS)
!      call fgaover(gs3(1:ex1,1:ex1),anew(zphase(m,s)),anew(zphase(m,s)), &
!                  psinew(zxi(m,s)),psinew(zxi(m,s)), &
!                  anew(zagmat(m,s)),anew(zagmat(m,s)),ex1,ex1,gdof)
!!      if (istep==1 .and. ex1<6) then
!!      if (istep==1 ) then
!        write(*,*) "ex1",ex1, "itr", itr, PQfix(1,1:gdof,ex1), PQfix(2,1:gdof,ex1), & 
!                    PQfix(3,1:gdof,ex1), abs(gs3(ex1,ex3))
!!      endif
!!      if (istep==2 ) then
!!        write(*,*) "ex1",ex1, PQfix(1,1,ex1), gs3(ex1,ex3)
!!      endif
!      if (abs(abs(gs3(ex1,ex3))/thr-1.0d0) .gt. abs(abs(gs4(ex1,ex3))/thr-1.0d0)) then
!        cnum=-1.0d0*cnum
!        snum=0.5d0*snum
!        gs4(ex1,ex3)=gs3(ex1,ex3)
!      else
!        gs4(ex1,ex3)=gs3(ex1,ex3)
!      endif
!  enddo
!  if ( ((PQfix(1,1,ex3)-PQfix(1,1,1)>0.0d0) .and. &
!       (PQfix(1,1,ex1)-PQfix(1,1,ex3)<0.0d0)) .or. &
!      ((PQfix(1,1,ex3)-PQfix(1,1,1)<0.0d0) .and. &
!      (PQfix(1,1,ex1)-PQfix(1,1,ex3)>0.0d0)) ) then
!    go to 777
!  endif
!!  if (ex1==2) then
!!    open(15,file=trim(arg)//"/"//pnew)
!!    do j=1000,-1000,-1
!!      Q=0.01d0*dble(j)
!!      call GwpValue(gdof,ex1-1,&
!!         anew(zphase(1,1)),anew(zxivec(1,1)),anew(zagmat(1,1)),psinew(zpsi(1)),Q,& ! modified by TM
!!         wfk)
!!      out=abs(wfk)**2
!!      write(15,*) Q,abs(wfk)**2
!!    enddo
!!    close(15)
!!  endif
!else
!    sortQ(1:gdof,1:dim1,1:ispr**gdof)=0.0d0
!    PQ(1:gdof)=PQfix(1,1:gdof,ex1)
!    call minim(PQ(1:gdof),step(1:gdof),gdof,res,int(BELflags(9)),&
!            -1,1e-5_dop,gdof*3,&
!            0,1e-9_dop,vec(1:gdof),funcValue,e)
!    PQfix(1,1:gdof,ex1)=PQ(1:gdof)
!!    write(*,*) "after minim istep",istep,ex1,PQfix(1,1:gdof,ex1)
!    Q=PQ(1:gdof)
!    call ValidateLHA(ex1,gdof,dim1,anew(zagmat(m,s)),PQfix(:,1:gdof,ex1))
!  if ( istep==1) then
!!    PQfix(3,1,1)=5.6313514709472046E-003
!!    PQfix(3,1,1)=1.0d-2
!!    PQfix(3,1,1)=5.0d-2
!    theta=PQfix(3,1,1) 
!!    write(*,*) PQfix(3,1:gdof,1)
!!    write(*,*) theta
!  elseif ( istep>1) then
!!    PQfix(3,1:gdof,1)=sqrt(0.5d0)*PQfix(3,1:gdof,1) 
!    PQfix(3,1,1)=theta 
!!    write(*,*) theta
!!    write(*,*) PQfix(3,1:gdof,1)
!  endif
!!      call GwpValue(gdof,dim1red,&
!!         aold(zphase(m,s)),aold(zxivec(m,s)),aold(zagmat(m,s)),psiold(zpsi(s)),Q,& ! phase mod yapol
!!         value)
!!      out1=abs(value)**2
!      PQS(:,1:gdof,ex1:ex1)=PQfix(:,1:gdof,ex1:ex1)
!!            write(*,*)'PQS',PQS(:,1:gdof,ex1:ex1)
!      call PQStoGWP(ex1,ex1,gdof,dim1,anew(zphase(m,s)),psinew(zxi(m,s)),&
!                   anew(zxivec(m,s)),anew(zagmat(m,s)),PQS)
!!      psinew(zpsi(s)+ex1-1)=(1.0d0,0.0d0)
!!      call GwpValue(gdof,ex1,&
!!         anew(zphase(1,1)),anew(zxivec(1,1)),anew(zagmat(1,1)),psinew(zpsi(1)),Q,& ! modified by TM
!!         wfk)
!!      out2=abs(wfk)**2
!!      out3=out2
!      snum=PQfix(3,1,ex1)*0.01d0
!!       write(*,*)"istep",istep,out1,out2
!!      do while ( abs(out1-out2) > 1.0d-3) 
!!        if (out1-out2>0.0d0) then
!!          PQfix(3,1:gdof,ex1)=PQfix(3,1:gdof,ex1)-snum
!!        else
!!          PQfix(3,1:gdof,ex1)=PQfix(3,1:gdof,ex1)+snum
!!        endif
!!        PQS(3,1:gdof,ex1)=PQfix(3,1:gdof,ex1)
!!        call PQStoGWP(ex1,ex1,gdof,dim1,anew(zphase(m,s)),psinew(zxi(m,s)),&
!!                     anew(zxivec(m,s)),anew(zagmat(m,s)),PQS)
!!        call GwpValue(gdof,ex1,&
!!           anew(zphase(1,1)),anew(zxivec(1,1)),anew(zagmat(1,1)),psinew(zpsi(1)),Q,& ! modified by TM
!!           wfk)
!!        out2=abs(wfk)**2
!!        if (abs(out1-out2)>abs(out1-out3)) then
!!          snum=0.5d0*snum
!!          out3=out2
!!        else
!!          out3=out2
!!        endif
!!      enddo
!
!    PQ(1:gdof)=PQfix(2,1:gdof,ex1)
!endif

! write(*,*)'a',anew(zagmat(1,1)+ex1-1)
!if (ex1==55) write(*,*)'a',anew(zagmat(m,s)+ex1-1)
!if (time>0.27) write(*,*)'a',anew(zagmat(m,s)+ex1-1)

     ! Step 3: P from overlaps
     ! find optimum parameters p (PQsize==1) or p,sigma (PQsize==2)

!----------------------------------------
!    open(10,file=trim(arg)//"/"//fP)
!    write(10,*)"# P funcP istep ex1",istep, ex1
!    do i=-100,100
!     PQfix(2,1:gdof,ex1)=1.0d0*dble(i)
!    PQS(:,1:gdof,ex1:ex1)=PQfix(:,1:gdof,ex1:ex1)
!       call PQStoGWP(ex1,ex1,gdof,dim1,anew(zphase(m,s)),psinew(zxi(m,s)),&
!            anew(zxivec(m,s)),anew(zagmat(m,s)),PQS) ! modified by TM
!    ! <g|Psi>
!    call gwpproj(ex1,ex1,gdof,&
!         dim1,anew(zphase(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)), &
!         dim1red,aold(zphase(m,s)),aold(zxivec(m,s)),aold(zagmat(m,s)),psiold(zpsi(s)),& ! modified by TM
!         oldprod)
!    ! <g|Psi'>
!    call gwpproj(ex1,ex1,gdof,&
!         dim1,anew(zphase(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)), &
!         ex1-1,anew(zphase(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)),psinew(zpsi(s)),& ! modified by TM
!         newprod)
!      out=abs(oldprod(ex1)-newprod(ex1))
!        if (MAXfP<out) then
!          MAXfP=out
!          PQ(1:gdof)=PQfix(2,1:gdof,ex1)
!        endif
!    write(10,*) PQfix(2,1:gdof,ex1),out, abs(oldprod(ex1)),abs(newprod(ex1)), PQfix(3,1:gdof,ex1)
!    enddo
!    close(10)
!!PQfix(2,1:gdof,ex1)=PQfixsave(2,1:gdof,ex1)
!----------------------------------------

    do e=1,PQsize
       do f=1,gdof
          step(e+(f-1)*PQsize)=pqISall(PQset(e),f,1)*0.1_dop
!          step(e+(f-1)*PQsize)=0.01_dop
!          PQ(e+(f-1)*PQsize)=PQfix(PQset(e),f,ex1)
! write(*,*)'PQ',PQ(e+(f-1)*PQsize),PQfix(PQset(e),f,ex1)
       end do
    end do
! write(*,*)'q,p',psinew(zxi(1,1)+ex1-1)
!if (ex1>1) then
!201610107
!    call minim(PQ(1:gdof*PQsize),step(1:gdof*PQsize),PQsize*gdof,res,int(BELflags(9)),&
!         -1,1e-5_dop,gdof*3*PQsize,&
!         0,1e-9_dop,vec(1:gdof*PQsize),funcPcall,e)
! PQ(1:gdof)=PQfix(2,1:gdof,ex1)
!if (ex1>1) then
! PQ(1:gdof)=0d0
! PQ(1:gdof)=(0.5d0/(abs(PQfix(1,1,1)-PQfix(1,1,2)))**2.0d0)*PQfix(1,1:gdof,ex1)
! PQ(1:gdof)=50d0*PQfix(1,1:gdof,ex1)+155d0
! PQ(1:gdof)=(0.5d0/(abs(PQfix(1,1,1)-PQfix(1,1,2))))/(80d0*abs(PQfix(1,1,1)-PQfix(1,1,2)))*PQfix(1,1:gdof,ex1) &
! +(PQfix(2,1,1)-(0.5d0/(abs(PQfix(1,1,1)-PQfix(1,1,2))))/(80d0*abs(PQfix(1,1,1)-PQfix(1,1,2)))*PQfix(1,1,1))
! PQ(1:gdof)=-20d0*PQfix(1,1:gdof,ex1)-55d0
!+cosh(PQfix(1,1:gdof,ex1))**(-2.0d0)-cosh(PQfix(1,1,1))**(-2.0d0)+5d0
! PQ(1:gdof)=-20d0
! PQ(1:gdof)=10d0*PQfix(1,1:gdof,ex1)+35d0
!if (ex1>=1) then
!if (PQfix(1,1,ex1) <-3d0  ) then
! PQ(1:gdof)=0d0
!! PQ(1:gdof)=40d0*PQfix(1,1:gdof,ex1)+125d0
! PQ(1:gdof)=sqrt(exp(-1.0d0*(PQfix(1,1:gdof,ex1)-PQfix(1,1,1))**2.0d0))*5.0d0
!else
! PQ(1:gdof)=sqrt(exp(-1.0d0*(PQfix(1,1:gdof,ex1)-PQfix(1,1,1))**2.0d0))*5.0d0
! PQ(1:gdof)=10d0*PQfix(1,1:gdof,ex1)+35d0
!!! PQ(1:gdof)=40d0*PQfix(1,1:gdof,ex1)+125d0
!! PQ(1:gdof)=20d0*PQfix(1,1:gdof,ex1)+65d0
!endif
! PQ(1:gdof)=20d0*PQfix(1,1:gdof,ex1)+80d0
! PQ(1:gdof)=20d0*PQfix(1,1:gdof,ex1)+65d0
! PQ(1:gdof)=-4d0*PQfix(2,1:gdof,ex1)
! PQ(1:gdof)=PQfix(2,1:gdof,ex1)-5d0
! PQ(1:gdof)=10d0*PQfix(1,1:gdof,ex1)+35d0
! PQ(1:gdof)=-20d0*PQfix(1,1:gdof,ex1)-55d0
! PQ(1:gdof)=20d0*tanh(PQfix(1,1:gdof,ex1)-PQfix(1,1,1))+5d0
! PQ(1:gdof)=80d0*tanh(PQfix(1,1:gdof,ex1)-PQfix(1,1,1))+5d0
! PQ(1:gdof)=40d0*tanh(PQfix(1,1:gdof,ex1)-PQfix(1,1,1))+5d0
! PQ(1:gdof)=3d0*sinh(PQfix(1,1:gdof,ex1)-PQfix(1,1,1))+5d0
! PQ(1:gdof)=5d0*sinh(PQfix(1,1:gdof,ex1)-PQfix(1,1,1))+5d0
!if (PQfix(1,1,ex1) <-5  ) then
! PQ(1:gdof)=-55d0
!elseif (PQfix(1,1,ex1) >-5 .and. PQfix(1,1,ex1) <-4 ) then
!! PQ(1:gdof)=25d0
! PQ(1:gdof)=40d0*PQfix(1,1:gdof,ex1)+145d0
!elseif (PQfix(1,1,ex1) >-4 .and. PQfix(1,1,ex1) <-2 ) then
! PQ(1:gdof)=20d0*PQfix(1,1:gdof,ex1)+65d0
!elseif (PQfix(1,1,ex1) >-2 .and. PQfix(1,1,ex1) <-1 ) then
! PQ(1:gdof)=40d0*PQfix(1,1:gdof,ex1)+105d0
!elseif (PQfix(1,1,ex1) >-1  ) then
! PQ(1:gdof)=65d0
!endif
!!if (PQfix(1,1,ex1) <-4 .or. PQfix(1,1,ex1) >-2) then
!if (PQfix(1,1,ex1) <-4 ) then
!! PQ(1:gdof)=-15d0
! PQ(1:gdof)=40d0*PQfix(1,1:gdof,ex1)+145d0
!elseif (PQfix(1,1,ex1) >-2 ) then
!! PQ(1:gdof)=25d0
! PQ(1:gdof)=40d0*PQfix(1,1:gdof,ex1)+105d0
!else
! PQ(1:gdof)=20d0*PQfix(1,1:gdof,ex1)+65d0
!endif
!if (PQfix(1,1,ex1) <-3) then
! PQ(1:gdof)=20d0*PQfix(1,1:gdof,ex1)+65d0
!! PQ(1:gdof)=-20d0*PQfix(1,1:gdof,ex1)-55d0
!else
! PQ(1:gdof)=-20d0*PQfix(1,1:gdof,ex1)-55d0
!! PQ(1:gdof)=20d0*PQfix(1,1:gdof,ex1)+65d0
!endif
! PQ(1:gdof)=100d0*PQfix(1,1:gdof,ex1)+305d0
!  l=l+1
! PQ(1:gdof)=5d0
! if(l==3) then
!   PQ(1:gdof)=-5d0
! elseif(l==4) then
!   PQ(1:gdof)=-5d0
!   l=0
! endif
! PQ(1:gdof)=100d0*PQfix(1,1:gdof,ex1)**5.0d0+3d0*PQfix(1,1:gdof,ex1)**4.0d0 &
!           +80d0*PQfix(1,1:gdof,ex1)**3.0d0+60d0*PQfix(1,1:gdof,ex1)**2.0d0+10d0*PQfix(1,1:gdof,ex1)
! PQ(1:gdof)=5d0*PQfix(1,1:gdof,ex1)**5.0d0+3d0*PQfix(1,1:gdof,ex1)**4.0d0 &
!           +8d0*PQfix(1,1:gdof,ex1)**3.0d0+6d0*PQfix(1,1:gdof,ex1)**2.0d0+10d0*PQfix(1,1:gdof,ex1)
! PQ(1:gdof)=exp(abs(PQfix(1,1:gdof,ex1)))*sin(PQfix(1,1:gdof,ex1))
! PQ(1:gdof)=(abs(PQfix(1,1:gdof,ex1)))*sin(PQfix(1,1:gdof,ex1))
! if(mod(ex1,2)/=0 .and. ex1>1) then
!   PQ(1:gdof)=-1d0*PQ(1:gdof)
! endif
!endif
!20161017

!      if(mod(ex1,2)/=0 .and. ex1>1) then
!        PQ(1:gdof)=-1.0d0*PQfix(2,1:gdof,ex1-1)
!      endif

!    PQfix(2,1:gdof,ex1)=PQ(1)
!    PQS(:,1:gdof,ex1:ex1)=PQfix(:,1:gdof,ex1:ex1)
!    open(11,file=trim(arg)//"/"//fPmax)
!       call PQStoGWP(ex1,ex1,gdof,dim1,anew(zphase(m,s)),psinew(zxi(m,s)),&
!            anew(zxivec(m,s)),anew(zagmat(m,s)),PQS) ! modified by TM
!    ! <g|Psi>
!    call gwpproj(ex1,ex1,gdof,&
!         dim1,anew(zphase(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)), &
!         dim1red,aold(zphase(m,s)),aold(zxivec(m,s)),aold(zagmat(m,s)),psiold(zpsi(s)),& ! modified by TM
!         oldprod)
!    ! <g|Psi'>
!    call gwpproj(ex1,ex1,gdof,&
!         dim1,anew(zphase(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)), &
!         ex1-1,anew(zphase(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)),psinew(zpsi(s)),& ! modified by TM
!         newprod)
!      out=abs(oldprod(ex1)-newprod(ex1))
!    write(11,*) PQfix(2,1:gdof,ex1),out
!    close(11)

!stop"funcP"
!endif

!if (ex1==55) write(*,*)'P',PQ(1)
!if (time>0.27) write(*,*)'P',PQ(1)
!write(*,*)'P',PQ(1)

! write(*,*)'a',anew(zagmat(1,1)+ex1-1)
! write(*,*)'q,p',psinew(zxi(1,1)+ex1-1)
! write(*,*)'psi',psinew(zpsi(1)+ex1-1)
! write(*,*)'ex1',ex1

!      call fgaover(gs2(1:ex1,1:ex1),anew(zphase(m,s)),anew(zphase(m,s)), &
!                  psinew(zxi(m,s)),psinew(zxi(m,s)), &
!                  anew(zagmat(m,s)),anew(zagmat(m,s)),ex1,ex1,gdof)
!      if (ex1>1)  write(*,*) "ex1",ex1, PQfix(1,1:gdof,ex1),PQfix(2,1:gdof,ex1), &
!                    PQfix(3,1:gdof,ex1), abs(gs2(ex1,ex1-1))
!      if (ex1>1)  write(*,*) "ex1",ex1,psinew(zxi(m,s)+ex1*gdof-gdof:zxi(m,s)+ex1*gdof-1)
!      if (ex1>1)  write(*,*) "ex1",ex1,anew(zagmat(m,s)+ex1*gdof**2-gdof**2:zagmat(m,s)+ex1*gdof**2-1)
!      if (ex1>1)  write(*,*) "ex1",ex1,anew(zphase(m,s)+ex1*gdof-gdof:zphase(m,s)+ex1*gdof-1)
    ! compute phi,xi,a,psi from best parameters and compute <\delta\Psi_ex1|\delta_Psi_ex1>
!    call funcP(PQ,res,PQfix,PQsize,PQset,gdof,ex1,ex1,allprod,psinew,anew,psiold,aold,&
!         dim1,dim1red,norm0,norma,m,s,gimmedev=.true.)
!      call fgaover(gs2(1:ex1,1:ex1),anew(zphase(m,s)),anew(zphase(m,s)), &
!                  psinew(zxi(m,s)),psinew(zxi(m,s)), &
!                  anew(zagmat(m,s)),anew(zagmat(m,s)),ex1,ex1,gdof)
!      if (ex1>1)  write(*,*) "ex1",ex1, PQfix(1,1:gdof,ex1),PQfix(2,1:gdof,ex1), &
!                    PQfix(3,1:gdof,ex1), abs(gs2(ex1,ex1-1))
!      if (ex1>1)  write(*,*) "ex1",ex1,psinew(zxi(m,s)+ex1*gdof-gdof:zxi(m,s)+ex1*gdof-1)
!      if (ex1>1)  write(*,*) "ex1",ex1,anew(zagmat(m,s)+ex1*gdof**2-gdof**2:zagmat(m,s)+ex1*gdof**2-1)
!      if (ex1>1)  write(*,*) "ex1",ex1,anew(zphase(m,s)+ex1*gdof-gdof:zphase(m,s)+ex1*gdof-1)
            

!      if(mod(ex1,2)/=0 .and. ex1>1) then
!        psinew(zpsi(1)+ex1)=-1.0d0*psinew(zpsi(1)+ex1-1)
!      endif

!if (ex1==1) write(*,*)'n',psinew(zphi(m,s)+ex1-1)
!if (ex1==1) write(*,*)'n',psinew(zphase(m,s)+ex1-1)
!write(*,*)psinew(zphi(m,s))
!write(*,*)"psinew(zphase(m,s)",psinew(zphase(m,s))
!write(*,*)psinew(zxi(m,s))
!write(*,*)"psinew(zxi(m,s)",psinew(zxi(m,s))
! write(*,*)'q,p',psinew(zxi(1,1)+ex1-1)
! write(*,*)'a',anew(zagmat(1,1)+ex1-1)
! write(*,*)'x',anew(zxivec(1,1)+ex1-1)
! write(*,*)'phi',anew(zphase(1,1)+ex1-1)
! write(*,*)'psi',psinew(zpsi(1)+ex1-1)
!if (ex1==55) write(*,*)'p',allprod(55)
!if (time>0.27) write(*,*)'p',allprod(ex1)

! write(*,*)'ex1',ex1
    ! allprod contains all the <g_i|Psi>.  Need to form new expansion as A=inv(S).<g_i|Psi>

!!    if ((ex1>1).and..not.NoMoreNonorth) then
if (.not.NoMoreNonorth) then
!!      call fgaover(gs2(1:ex1,1:ex1),psinew(zphi(m,s)),psinew(zphi(m,s)), &
!!                  psinew(zxi(m,s)),psinew(zxi(m,s)), &
!!                  anew(zagmat(m,s)),anew(zagmat(m,s)),ex1,ex1,gdof)
!      call fgaover(gs2(1:ex1,1:ex1),anew(zphase(m,s)),anew(zphase(m,s)), &
!                  psinew(zxi(m,s)),psinew(zxi(m,s)), &
!                  anew(zagmat(m,s)),anew(zagmat(m,s)),ex1,ex1,gdof)
!      if (ex1>1) write(*,*)abs(gs2(ex1,ex1-1))
!      gs2(ex1,1:ex1-1)=conjg(gs2(1:ex1-1,ex1))
!      gs(1:ex1,1:ex1)=gs2(1:ex1,1:ex1)
!      if (ex1==dim(1,1)) then
!        call zheev('N','U',ex1,gs,npg(m)*ex1,eig,work,npg(m)*ex1*32,rw,i)
!        if (i/=0) then
!          write(*,*)i,ex1
!          stop 'zheev failed in shiftGwpSpf'
!        endif
!        !write(998,*)real(eig(1)),real(eig(ex1))
!      endif
!!      if (eig(1)>BELflags(11)) then
!        gs(1:ex1,1:ex1)=gs2(1:ex1,1:ex1)
!        gs3(1:ex1,1:ex1)=gs2(1:ex1,1:ex1)
!        b(1:ex1)=allprod(1:ex1)
!        b2(1:ex1)=b(1:ex1)
!        if (ex1==1) write(*,*)time,gs(1,1)
!        do j=1,ex1
!          do k=1,ex1
!            write(*,*)"j k gs2 gs3",j,k,gs2(j,k),gs3(j,k)
!          enddo
!        enddo

!!!!
!        call zhesv('U',ex1,1,gs,npg(m)*ex1,ip,b,npg(m)*ex1,work,npg(m)*ex1*32,i)
!!!!

!        call  ml_MatrixInverse(ex1,npg(m)*dim1, gs3)
!        do j=1,ex1
!          b3(j)=(0.0d0,0.0d0)
!          do k=1,ex1
!            b3(j)=b3(j)+gs3(j,k)*b2(k)
!          enddo
!        enddo
!if (istep==23 ) then
!if (istep==2 .or. istep==13 .or. istep==23) then
!if ( istep==13 .or. istep==23) then
!        do j=1,ex1
!          do k=1,ex1
!            write(*,*)"j k gs2 gs3",j,k,gs2(j,k),gs3(j,k)
!          enddo
!        enddo

!!        do j=1,ex1
!!          temp2=(0.0d0,0.0d0)
!!          temp1=(0.0d0,0.0d0)
!!          do k=1,ex1
!!            temp1=temp1+abs(gs2(j,k)*b(k))        
!!          enddo
!!        temp2=abs(b2(j))-abs(temp1)
!!        write(*,*)"time ex1 b-Sx info",time/fs,j,temp2,i
!!        enddo
!!        write(*,*)" "
!        do j=1,ex1
!          write(*,*)"time ex1 b3 b ",time/fs,j,b3(j),b(j)
!        enddo
!        write(*,*)" "
!endif
i=0
        if (i/=0) then
!          psinew(zpsi(s):zpsi(s)+ex1-1)=b(1:ex1)
          write(*,*)ex1,i
          write(*,*) "time",time/fs
          do j=1,ex1
            write(*,*)'psi',psinew(zpsi(s)+j-1)
            write(*,*)'phi',anew(zphase(m,s)+j-1)
            write(*,*)'xivec',anew(zxivec(m,s)+j-1)
            write(*,*)'xi',psinew(zxi(m,s)+j-1)
            write(*,*)'a',anew(zagmat(m,s)+j-1)
          enddo
          stop 'zhesv failed in shiftGwpSpf'
        endif
!!!!
!        psinew(zpsi(s):zpsi(s)+ex1-1)=b(1:ex1)
!!!!

!        if (istep==19) then
!          write(999,*)ex1
!          do i=1,ex1
!            write(999,*)psinew(zpsi(s)+i-1)
!          enddo
!          write(999,*)" "
!        endif

!        psinew(zpsi(s):zpsi(s)+ex1-1)=b3(1:ex1)
!------------------------------------------------------------------------------------

if (mod(dble(ex1)-1.0d0,100d0) == 0.0d0) then
      call fgaover(gs2(1:ex1,1:ex1),anew(zphase(m,s)),anew(zphase(m,s)), &
                  psinew(zxi(m,s)),psinew(zxi(m,s)), &
                  anew(zagmat(m,s)),anew(zagmat(m,s)),ex1,ex1,gdof)
      gs(1:ex1,1:ex1)=gs2(1:ex1,1:ex1)
      call zheev('N','U',ex1,gs,npg(m)*ex1,eig,work,npg(m)*ex1*32,rw,i)
      gsmax=eig(1)
      gsmin=eig(1)
      do i=2,ex1
        if (abs(eig(i)) > abs(gsmax)) gsmax=eig(i)
        if (abs(eig(i)) < abs(gsmin)) gsmin=eig(i)
      enddo
!      gsmax=1.0d0
!      gsmin=1.0d0
!if (istep==22 .or. istep==23 ) then
!if (istep==45 .or. istep==47 ) then
!if (istep==13 .or. istep==23) then
!if (istep==2 .or. istep==13 .or. istep==23) then
!if (istep==1) then
!        write(*,*)"ex1",ex1
!        write(*,*)"max",gsmax
!        write(*,*)"min",gsmin
!        write(*,*)"min/max",abs(gsmin/gsmax)
!        write(*,*)
  if ((istep==1) .and. (ex1==1) )then
    call getarg(2,arg)
    open(100,file=trim(arg)//".100")
          write(100,*)time/fs,s,ex1,abs(gsmin/gsmax)
    close(100)
  else
    open(100,file=trim(arg)//".100",position="append")
          write(100,*)time/fs,s,ex1,abs(gsmin/gsmax)
    close(100)
  endif
endif

!endif
!      if (abs(gsmin/gsmax)<1.0d-6 .and. (iloop==0) .and. abs(aoverlap)>BELflags(1)) then
!      if (abs(gsmin/gsmax)<1.0d-6 .and. (iloop==0) ) then
!         write(*,*) "re-reset time ex1",time/fs,ex1
!         iloop=1
!         go to 10
!      elseif (abs(gsmin/gsmax)<1.0d-8 .and. (iloop==1)) then
!      elseif (abs(gsmin/gsmax)<1.0d-10 ) then
!      elseif (abs(gsmin/gsmax)<1.39d-10 ) then
!      elseif (abs(gsmin/gsmax)<1.0d-9 ) then
!      elseif (abs(gsmin/gsmax)<1.0d-8 ) then
!      elseif (abs(gsmin/gsmax)<5.0d-7 ) then
!      elseif (abs(gsmin/gsmax)<1.0d-7 ) then
!      elseif (abs(gsmin/gsmax)<1.0d-12 ) then
!      elseif (abs(gsmin/gsmax)<1.0d-12 ) then
!      elseif (abs(gsmin/gsmax)<1.0d-13 ) then
!      elseif (abs(gsmin/gsmax)<1.0d-14 ) then
!      elseif (abs(gsmin/gsmax)<1.0d-15 ) then
!      elseif (abs(gsmin/gsmax)<1.0d-16 ) then
!      elseif (abs(gsmin/gsmax)<1.0d-17 ) then
!      elseif (abs(gsmin/gsmax)<1.0d-18 ) then
!      elseif (abs(gsmin/gsmax)<1.0d-19 ) then
!      elseif (abs(gsmin/gsmax)<1.0d-20 ) then
!      elseif (abs(gsmin/gsmax)<1.0d-21 ) then
!      elseif (abs(gsmin/gsmax)<1.0d-41 ) then
      if ((abs(gsmin/gsmax)<1.0d-10 ) .and. (re>0) )then
         ierr=0
         tempdimred=ex1
      elseif ((abs(gsmin/gsmax)<1.0d-11 ) .and. (re>0) )then
         ierr=0
         tempdimred=ex1
      elseif ((abs(gsmin/gsmax)<1.0d-12 ) .and. (re>0) )then
         ierr=0
         tempdimred=ex1
      elseif ((abs(gsmin/gsmax)<1.0d-13 ) .and. (re>0) )then
         ierr=0
         tempdimred=ex1
      elseif ((abs(gsmin/gsmax)<1.0d-14 ) .and. (re>0) )then
         ierr=0
         tempdimred=ex1
      elseif ((abs(gsmin/gsmax)<1.0d-15 ) .and. (re>0) )then
         ierr=0
         tempdimred=ex1
      elseif (abs(gsmin/gsmax)<1.0d-16 ) then
         ierr_eigval=ierr_eigval+1
         ierr=1
         go to 100
!      elseif (abs(gsmin/gsmax)<1.0d-16 .and. (iloop=0) ) then
!         ierr=1
!         go to 100
      else
         iloop=0
         ierr=0
      endif
!------------------------------------------------------------------------------------
        if (.not.allocated(CoeffSave)) then
          allocate(CoeffSave(dim(1,1)))
        endif
!        CoeffSave(1:ex1)=b(1:ex1)
!        CoeffsSaved=ex1
if (.false.) then
  write(999,*)time,ex1,eig(1),eig(ex1)
  do i=1,ex1
!    write(999,*)cmplx(allprod(i)),cmplx(b(i))
!    write(999,*)real(allprod(i)),imag(allprod(i)),real(b(i)),imag(b(i))
  enddo
  write(999,*)
endif
!      else
!        write(*,*)'Changed to matching at',time,ex1,eig(1)
!!do i=1,CoeffsSaved
!!  write(*,*)psinew(zpsi(s)+i-1)
!!enddo
!        NoMoreNonorth=.true.
!!        do i=1,dim1
!!          do f=1,gdof
!!!          amat(f,i)=a(f,f,i)     ! The anew rep is (gdof,gdof,dim1)
!!            amat(f,i)=anew(zagmat(1,1)+(i-1)*gdof**2+(f-1)*gdof+f-1)
!!          enddo
!!        end do
!!      endif
    endif

    if (NoMoreNonorth) then
       ! Our overlap matrix is getting to be un-invertable.
       ! I don't want to bugger about with regularisation.
if (.false.) then
         ! Here is the <g|dPsi> version
      psinew(zpsi(s):zpsi(s)+CoeffsSaved-1)=CoeffSave(1:CoeffsSaved)
!      psinew(zpsi(s)+CoeffsSaved:zpsi(s)+ex1-1)=allprod(CoeffsSaved+1:ex1)*0.1_dop
!write(*,*)allprod(ex1),psinew(zpsi(s)+ex1-1)
elseif (.true.) then
         ! Werner's original idea of simply using <g|delta Psi> has stability issues.
         ! Let's zero dPsi at the basis centres.
      psinew(zpsi(s):zpsi(s)+CoeffsSaved-1)=CoeffSave(1:CoeffsSaved)
      do i=1,ex1-CoeffsSaved
        i1=i+CoeffsSaved
        GSR(i,1:ex1)=GaussianValues(gdof,ex1, &
           psinew(zphi(1,1)),anew(zxivec(1,1)),anew(zagmat(1,1)), &
           dble(psinew(zxi(m,s)+(i1-1)*gdof:zxi(m,s)+i1*gdof-1)))   ! this is xi(:,i1)
        call GwpValue(1,dim1,&
           psiold(zphi(1,1)),aold(zxivec(1,1)),aold(zagmat(1,1)),psiold(zpsi(1)), &
           dble(psinew(zxi(m,s)+(i1-1)*gdof:zxi(m,s)+i1*gdof-1)),b(i))
        b(i)=b(i)-sum(GSR(i,1:CoeffsSaved)*psinew(zpsi(1):zpsi(1)+CoeffsSaved-1))
      enddo
       ! GSR(i,j) now holds g_j(x_i), b(i) holds dPsi(x_i) (from the saved coeffs)
      GSR(1:ex1-CoeffsSaved,1:ex1-CoeffsSaved)=GSR(1:ex1-CoeffsSaved,CoeffsSaved+1:ex1)
!write(*,*)GSR(1,1)
      call zgesv(ex1-CoeffsSaved,1,GSR,npg(m)*dim1,ip,b,npg(m)*dim1,i)
      if (i/=0) then
        write(*,*)i,ex1,CoeffsSaved
        stop 'zgesv failed in shiftGwpSpf'
      endif
      psinew(zpsi(s)+CoeffsSaved:zpsi(s)+ex1-1)=b(1:ex1-CoeffsSaved)*0.1_dop
!write(*,*)allprod(ex1),psinew(zpsi(s)+ex1-1)
!psinew(zpsi(s)+CoeffsSaved:zpsi(s)+ex1-1)=0
else
   ! whole thing by zeroing
      do i=1,ex1
        GSR(i,1:ex1)=GaussianValues(gdof,ex1, &
           psinew(zphi(1,1)),anew(zxivec(1,1)),anew(zagmat(1,1)), &
           dble(psinew(zxi(m,s)+(i-1)*gdof:zxi(m,s)+i*gdof-1)))   ! this is xi(:,i1)
        call GwpValue(1,ex1,&
           psinew(zphi(1,1)),anew(zxivec(1,1)),anew(zagmat(1,1)),psinew(zpsi(1)), &
           dble(psinew(zxi(m,s)+(i-1)*gdof:zxi(m,s)+i*gdof-1)),b(i))
      enddo
       ! GSR(i,j) now holds g_j(x_i), b(i) holds Psi(x_i)
      call zgesv(ex1,1,GSR,npg(m)*dim1,ip,b,npg(m)*dim1,i)
      if (i/=0) then
        write(*,*)i
        stop 'zgesv failed in shiftGwpSpf'
      endif
      psinew(zpsi(s):zpsi(s)+ex1-1)=b(1:ex1)
endif

 ! write dPsi(x_ex1)
!call GwpValue(1,ex1,&
!           psiold(zphi(1,1)),aold(zxivec(1,1)),aold(zagmat(1,1)),psiold(zpsi(1)), &
!           dble(psinew(zxi(m,s)+(i1-1)*gdof:zxi(m,s)+i1*gdof-1)),b(i))
!write(*,*)ex1,

if (.false.) then
  write(999,*)time,ex1,eig(1),eig(ex1)
  do i=1,ex1
!    write(999,*)cmplx(allprod(i)),cmplx(b(i)),cmplx(psinew(zpsi(s)+i-1))
  enddo
  write(999,*)
endif
    endif



     ! we need the norm of the new wavefunction
     ! actually, norma is <dPsi|dPsi>-<Psi|Psi>
!!    call gwpproj(1,ex1,gdof,&
!!           dim1,psinew(zphi(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)), &
!!           ex1,psinew(zphi(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)),psinew(zpsi(s)),&
!!           newprod)
!    call gwpproj(1,ex1,gdof,&
!           dim1,anew(zphase(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)), &
!           ex1,anew(zphase(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)),psinew(zpsi(s)),& ! modified by TM
!           newprod)
!    norma=0
!!    do i=1,ex1
!!        norma=norma+2.0_dop*dble(dconjg(psinew(zpsi(s)-1+i))*newprod(i))
!!        norma=norma+abs(psinew(i))**2
!!      norma=norma+(dconjg(psinew(zpsi(s)-1+i))*newprod(i))
!    enddo
!!    norma=norma+2.0_dop*dble(dconjg(psinew(zpsi(s)-1+ex1))*newprod(ex1))
!!    norma=norma+abs(psinew(ex1))**2
!!    do i=1,ex1
!!    norma=norma+(dconjg(newprod(i))*newprod(i))
!!    enddo
!     ! <dPsi|dPsi>
!    aoverlap=norm0+norma
!     ! compute \sum_i=1..ex1 \sum_j \psi_new_i* \psi_old_j <g_new_i|g_old_j>
!     ! i.e. <Psi_old|Psi_new>
!!    aoverlap=aoverlap-sum(2.0_dop*dble(dconjg(psinew(zpsi(s):zpsi(s)-1+ex1))*allprod(1:ex1)))
!    do i=1,ex1
!      aoverlap=aoverlap-2.0_dop*dble(dconjg(psinew(zpsi(s)-1+i))*allprod(i))
!    enddo

       if (ex1==1) then
          normnew=0.0_dop
       else
          normnew=norma
       end if

!tmpprod=(0.0d0,0.0d0)
    do e=ex1,ex1
      do j=0,ssdim-1
!        normnew=normnew+2.0_dop*dble(dconjg(psinew(zpsi(s)+j*dim1red-1+e))*allnewprod(e,j+1))
!        normnew=normnew+abs(psinew(zpsi(s)+j*dim1red-1+e))**2
        normnew=normnew+2.0_dop*dble(dconjg(psinew(zpsi(s)+j*dim1-1+e))*allnewprod(e,j+1))
        normnew=normnew+abs(psinew(zpsi(s)+j*dim1-1+e))**2
      enddo
!          normnew=normnew+2.0_dop*dble(dconjg(psinew(zpsi(s)-1+e))*allnewprod(e))
!        normnew=normnew+abs(psinew(zpsi(s+j*dim1red)-1+e))**2
    enddo
    norma=normnew
!write(1000,*)ex1,"normnew",normnew
     ! <dPsi|dPsi>
!tmpprod=(0.0d0,0.0d0)
    aoverlap=norm0+norma
    do i=1,ex1
     do j=0,ssdim-1
!        aoverlap=aoverlap-2.0_dop*dble(dconjg(psinew(zpsi(s)+j*dim1red-1+i))*allprod(i,j+1))
        aoverlap=aoverlap-2.0_dop*dble(dconjg(psinew(zpsi(s)+j*dim1-1+i))*allprod(i,j+1))
     enddo
!      aoverlap=aoverlap-2.0_dop*dble(dconjg(psinew(zpsi(s)-1+i))*allprod(i))
    enddo

!if (mod(dble(ex1)-1.0d0,100d0) == 0.0d0) then
  if ((istep==1) .and. (ex1==1) )then
    call getarg(2,arg)
    open(102,file=trim(arg)//".102")
          write(102,*)time/fs,s,ex1,abs(aoverlap)
    close(102)
  else
    open(102,file=trim(arg)//".102",position="append")
          write(102,*)time/fs,s,ex1,abs(aoverlap)
    close(102)
  endif
!endif

! write(1000,*)"norm0,norma",norm0,norma,ex1,s,abs(aoverlap),abs(allnewprod(ex1)),abs(psinew((zpsi(s)-1+e)))**2
! write(*,*)"abs(psinew(ex1))**2",abs(psinew(ex1))**2
! write(*,*)"newprod",newprod(ex1)
! write(*,*)"newprod**2",sum(dconjg(newprod(1:ex1-1))*newprod(1:ex1-1))
!if (ex1>14 ) then
!  write(*,*)"norma",norma
!endif
! write(*,*)"sum",-sum(2.0_dop*dble(dconjg(psinew(zpsi(s):zpsi(s)-1+ex1))*allprod(1:ex1)))
!if (istep==23) then
!if (istep==13 .or. istep==23) then
!if (istep==2 .or. istep==13 .or. istep==23) then
! write(*,*)"ex1 aoverlap",ex1,abs(aoverlap)
!endif

!if (ex1>=76) write(*,*)ex1,psinew(zpsi(s)+75)

!    aoverlap=res

 ! 1D version:
if (.false.) then
!  do i=-100,0
    Q=0.01*i
!if (NoMoreNonorth.and.(time>0.01)) then
!if (NoMoreNonorth) then
!Q=dble(psinew(zxi(m,s)+ex1*gdof-1))
    call GwpValue(1,ex1,&
           anew(zphase(1,1)),anew(zxivec(1,1)),anew(zagmat(1,1)),psinew(zpsi(1)),Q,&
           wf1)
!write(*,*)anew(zphase(1,1)-1+ex1),anew(zxivec(1,1)-1+ex1),anew(zagmat(1,1)-1+ex1),psinew(zpsi(1)-1+ex1)
    call GwpValue(1,dim1red,&
           aold(zphase(1,1)),aold(zxivec(1,1)),aold(zagmat(1,1)),psiold(zpsi(1)),Q,&
           wf2)
!write(*,*)aold(zphase(1,1)-1+ex1),aold(zxivec(1,1)-1+ex1),aold(zagmat(1,1)-1+ex1),psiold(zpsi(1)-1+ex1)
!write(*,*)time,ex1
!write(*,*)"Q",Q
!write(*,*)"wf1",wf1
!write(*,*)"wf2",wf2
!stop
    wf=wf2-wf1
!    write(300+ex1,*)Q,real(wf1),abs(wf1)**2,real(wf),imag(wf),abs(wf)**2,&
!!        product(abs(PQfix(1,1,1:ex1)-Q(1))),abs(wf)**2+product(abs(PQfix(1,1,1:ex1)-Q(1)))
!!        sum(abs(PQfix(1,1,1:ex1)-Q(1))),abs(wf)**2+sum(abs(PQfix(1,1,1:ex1)-Q(1)))
!!        product(1-exp(-(Q(1)-PQfix(1,1,1:ex1))**2/2/PQfix(3,1,1:ex1)**2))*abs(wf2)**2, &
!        product(1-exp(-(Q(1)-PQfix(1,1,1:ex1))**2/2/PQfix(3,1,1:ex1)**2)),&
!        product(1-exp(-(Q(1)-PQfix(1,1,1:ex1))**2/2/PQfix(3,1,1:ex1)**2))*abs(wf)**2
!!                BELflags(12)*abs(wf)**2 &
!!                +product(1-exp(-(Q(1)-PQfix(1,1,1:ex1))**2/2/PQfix(3,1,1:ex1)**2))*abs(wf2)**2, &
!!             product(1-exp(-(Q(1)-PQfix(1,1,1:ex1))**2/2/PQfix(3,1,1:ex1)**2))
!!    write(55,*)Q,psinew(zpsi(1)+54)*exp(Q**2*anew(zagmat(1,1)+54)+dble(anew(zxivec(1,1)+54))*Q &
!!                                +(0.0_8,1.0_8)*dimag(anew(zxivec(1,1)+54))*10*Q+psinew(zphi(1,1)+54))
!  enddo
endif

 ! 2D version:
if (.false.) then
  do i=-60,60
    Q(1)=0.015*i
    do j=-60,60
      Q(2)=0.015*j
      call GwpValue(gdof,ex1,&
           psinew(zphi(1,1)),anew(zxivec(1,1)),anew(zagmat(1,1)),psinew(zpsi(1)),Q,&
           wf1)
      call GwpValue(gdof,dim1,&
           psiold(zphi(1,1)),aold(zxivec(1,1)),aold(zagmat(1,1)),psiold(zpsi(1)),Q,&
           wf2)
      wf=wf2-wf1
      write(300+ex1,*)Q,real(wf1),abs(wf1)**2,real(wf),imag(wf),abs(wf)**2, &
        product(1-exp(-sum((spread(Q,2,ex1)-PQfix(1,1:gdof,1:ex1))**2/2/PQfix(3,1:gdof,1:ex1)**2,1))), &
        product(1-exp(-sum((spread(Q,2,ex1)-PQfix(1,1:gdof,1:ex1))**2/2/PQfix(3,1:gdof,1:ex1)**2,1)))*abs(wf),&
        product(1-exp(-sum((spread(Q,2,ex1)-PQfix(1,1:gdof,1:ex1))**2/2/PQfix(3,1:gdof,1:ex1)**2,1)))*abs(wf)**2
!        product(sum((spread(Q,2,ex1)-PQfix(1,1:gdof,1:ex1))**2,dim=1)),&
!        product(sum((spread(Q,2,ex1)-PQfix(1,1:gdof,1:ex1))**2,dim=1))*abs(wf)**2
    enddo
    write(300+ex1,'(a)')
  enddo
endif
!if (ex1==50) stop 'q'

 ! nD version:
if (.false.) then
  Q=-0.9_8
  dQ=0.015_8
  do
    call GwpValue(gdof,ex1,&
           psinew(zphi(1,1)),anew(zxivec(1,1)),anew(zagmat(1,1)),psinew(zpsi(1)),Q,&
           wf1)
    call GwpValue(gdof,dim1,&
           psiold(zphi(1,1)),aold(zxivec(1,1)),aold(zagmat(1,1)),psiold(zpsi(1)),Q,&
           wf2)
    wf=wf2-wf1
    write(300+ex1,*)Q,real(wf1),abs(wf1)**2,real(wf),imag(wf),abs(wf)**2, &
        product(1-exp(-sum((spread(Q,2,ex1)-PQfix(1,1:gdof,1:ex1))**2/2/PQfix(3,1:gdof,1:ex1)**2,1))), &
        product(1-exp(-sum((spread(Q,2,ex1)-PQfix(1,1:gdof,1:ex1))**2/2/PQfix(3,1:gdof,1:ex1)**2,1)))*abs(wf),&
        product(1-exp(-sum((spread(Q,2,ex1)-PQfix(1,1:gdof,1:ex1))**2/2/PQfix(3,1:gdof,1:ex1)**2,1)))*abs(wf)**2
!        product(sum((spread(Q,2,ex1)-PQfix(1,1:gdof,1:ex1))**2,dim=1)),&
!        product(sum((spread(Q,2,ex1)-PQfix(1,1:gdof,1:ex1))**2,dim=1))*abs(wf)**2
    Q(1)=Q(1)+dQ(1)
    ro=.false.
    do i=1,gdof-1
      if (Q(i)>0.9_8) then
        Q(i)=-0.9_8
        Q(i+1)=Q(i+1)+dQ(i+1)
        ro=.true.
      endif
    enddo
    if (Q(gdof)>0.9_8) exit
    if (ro) write(300+ex1,'(a)')
  enddo
endif
!if (ex1==50) stop 'q'

100 continue
!666 continue
    if (ierr == 1) then
      ex3=ex1-1
      call fgaover(gs2(1:ex3,1:ex3),anew(zphase(m,s)),anew(zphase(m,s)), &
                  psinew(zxi(m,s)),psinew(zxi(m,s)), &
                  anew(zagmat(m,s)),anew(zagmat(m,s)),ex3,ex3,gdof)
      gs2(ex3,1:ex3-1)=conjg(gs2(1:ex3-1,ex3))
      gs(1:ex3,1:ex3)=gs2(1:ex3,1:ex3)
      if (ex3==dim(1,1)) then
        call zheev('N','U',ex3,gs,npg(m)*ex1,eig,work,npg(m)*ex1*32,rw,i)
        if (i/=0) then
          write(*,*)i,ex3
          stop 'zheev failed in shiftGwpSpf'
        endif
      endif
        gs(1:ex3,1:ex3)=gs2(1:ex3,1:ex3)
!        b(1:ex3)=allprod(1:ex3)
        call zhesv('U',ex3,1,gs,npg(m)*ex1,ip,b,npg(m)*ex1,work,npg(m)*ex1*32,i)
        if (i/=0) then
          write(*,*)ex3,i
          stop 'zhesv failed in shiftGwpSpf'
        endif
        psinew(zpsi(s):zpsi(s)+ex3-1)=b(1:ex3)

    endif
!if ( ex1==14)  then
!                call printGWPSPF(BELprint,nspfdof(m),DomainLower,DomainUpper,&
!                     dim(m,s),psiold(zpsi(s)),aold(zphase(m,s)),aold(zxivec(m,s)),aold(zagmat(m,s)),&
!                     dim(m,s),psinew(zpsi(s)),anew(zphase(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)),&
!                     xplotlims=BELplotlims,channel=iBEL)
!call myflush(iBEL)
!endif
!  write(*,*)istep,ex1
!   if (ex1==14) stop
  call stop_timer(tid)
  return
  end subroutine shiftGwpSpf

  subroutine copyPsi(psi0,a0,zpsi0,zphi0,zphase0,zxi0,zxivec0,zagmat0,zetf0,dim0,block0,&
       psi1,a1,zpsi1,zphi1,zphase1,zxi1,zxivec1,zagmat1,zetf1,dim1,block1)

    use griddatmod, only : nspfdof,gwpm,nstate,nmode
    use maxv, only: maxdim,maxsta
    use BELglobals, only: dgldimmax,agmatdimmax

    integer(long), dimension(maxsta), intent(in)        :: block0,block1,&
                                                           zpsi0,zpsi1
    integer(long), dimension(maxdim,maxsta), intent(in) :: dim0,zphi0,zphase0,zxi0,&
                                                           zxivec0,zagmat0,zetf0
    integer(long), dimension(maxdim,maxsta), intent(in) :: dim1,zphi1,zphase1,zxi1,&
                                                           zxivec1,zagmat1,zetf1
    complex(dop), dimension(dgldimmax), intent(in)      :: psi0
    complex(dop), dimension(agmatdimmax), intent(in)              :: a0
    complex(dop), dimension(dgldimmax), intent(out)     :: psi1
    complex(dop), dimension(agmatdimmax), intent(out)             :: a1

    integer(long) :: m,s

    do s=1,nstate
       psi1(zpsi1(s):zpsi1(s)+min(block1(s),block0(s))-1) = &
                   psi0(zpsi0(s):zpsi0(s)+min(block1(s),block0(s))-1)
!write(*,*)"psi(zpsi)",psi1(zpsi1(s):zpsi1(s)+min(block1(s),block0(s))-1)
!write(1000,*)"block1(s),block0(s)",block1(s),block0(s)
       if (block1(s)>block0(s)) then
          psi1(zpsi1(s)+block0(s):zpsi1(s)+block1(s)-1)=0.0_dop
       end if
       do m=1,nmode
!write(*,*)"nspfdof(m)",nspfdof(m)
!write(*,*)"psi(zphi)",zphi1(m,s),zphi1(m,s)+min(dim1(m,s),dim0(m,s))*nspfdof(m)-1
!write(*,*)"psi(zphi)",zphi1(m,s),zphi1(m,s)+min(block1(s),block0(s))-1
!write(*,*) "a0(zphase)",zphase0(m,s),zphase0(m,s)+min(dim1(m,s),dim0(m,s))*nspfdof(m)-1
!write(*,*) "a0(zphase)",zphase0(m,s),zphase0(m,s)+min(block1(s),block0(s))-1
!write(*,*) "a1(zphase)",zphase1(m,s),zphase1(m,s)+min(block1(s),block0(s))-1
!write(*,*) "psi(zphi)",zphi1(m,s),zphi1(m,s)+min(block1(s),block0(s))-1
!stop
          if (gwpm(m)) then
!             psi1(zphi1(m,s):zphi1(m,s)+min(block1(s),block0(s))-1)=&
!                  psi0(zphi0(m,s):zphi0(m,s)+min(block1(s),block0(s))-1)
             psi1(zphi1(m,s):zphi1(m,s)+min(dim1(m,s),dim0(m,s))-1)=&
             a0(zphase0(m,s):zphase0(m,s)+min(dim1(m,s),dim0(m,s))-1)
!             psi1(zphi1(m,s):zphi1(m,s)+min(dim1(m,s),dim0(m,s))*nspfdof(m)-1)=&
!             a0(zphase0(m,s):zphase0(m,s)+min(dim1(m,s),dim0(m,s))*nspfdof(m)-1)
!write(*,*)"psi(zphi)",psi1(zphi1(m,s):zphi1(m,s)+min(dim1(m,s),dim0(m,s))-1)

!write(*,*)"psi0(zxi)",zxi0(m,s),zxi0(m,s)+min(dim1(m,s),dim0(m,s))*nspfdof(m)-1
!write(*,*)"psi1(zxi)",zxi1(m,s),zxi1(m,s)+min(dim1(m,s),dim0(m,s))*nspfdof(m)-1
             psi1(zxi1(m,s):zxi1(m,s)+min(dim1(m,s),dim0(m,s))*nspfdof(m)-1)=&
                 psi0(zxi0(m,s):zxi0(m,s)+min(dim1(m,s),dim0(m,s))*nspfdof(m)-1)

!             a1(zagmat1(m,s):zagmat1(m,s)+min(block1(s),block0(s))*nspfdof(m)**2-1)=&
!                  a0(zagmat0(m,s):zagmat0(m,s)+min(block1(s),block0(s))*nspfdof(m)**2-1)
             a1(zagmat1(m,s):zagmat1(m,s)+min(dim1(m,s),dim0(m,s))*nspfdof(m)**2-1)=&
                  a0(zagmat0(m,s):zagmat0(m,s)+min(dim1(m,s),dim0(m,s))*nspfdof(m)**2-1)
!write(*,*)"a1(zagmat)",a1(zagmat1(m,s):zagmat1(m,s)+min(dim1(m,s),dim0(m,s))-1)

             a1(zphase1(m,s):zphase1(m,s)+min(dim1(m,s),dim0(m,s))-1)=&
             a0(zphase0(m,s):zphase0(m,s)+min(dim1(m,s),dim0(m,s))-1)
!             a1(zphase1(m,s):zphase1(m,s)+min(dim1(m,s),dim0(m,s))*nspfdof(m)-1)=&
!             a0(zphase0(m,s):zphase0(m,s)+min(dim1(m,s),dim0(m,s))*nspfdof(m)-1)
!write(*,*)"a0(zphase)",a0(zphase0(m,s):zphase0(m,s)+min(block1(s),block0(s))-1)
!write(*,*)"a1(zphase)",a1(zphase1(m,s):zphase1(m,s)+min(dim1(m,s),dim0(m,s))*nspfdof(m)-1)
!stop
!write(*,*)(zphi0(m,s)+min(block1(s),block0(s))-1)

!write(*,*)"m (zxivec0)",m,(zxivec0(m,s))
!write(*,*)"m (zxivec1)",m,(zxivec1(m,s))
!write(*,*)"a0(zxivec)",a0(zxivec0(m,s):zxivec0(m,s)+min(dim1(m,s),dim0(m,s))*nspfdof(m)-1)
             a1(zxivec1(m,s):zxivec1(m,s)+min(dim1(m,s),dim0(m,s))*nspfdof(m)-1)=&
             a0(zxivec0(m,s):zxivec0(m,s)+min(dim1(m,s),dim0(m,s))*nspfdof(m)-1)
!write(1000,*)"a1(zxivec)",a1(zxivec1(m,s):zxivec1(m,s)+min(dim1(m,s),dim0(m,s))*nspfdof(m)-1)
!write(1000,*)"a1(zxivec)",a1(zxivec1(m,s)),s
          else
!            write(1000,*)"m",m,psi1(zetf1(m,s):zetf1(m,s)+dim1(m,s)**2-1)
            psi1(zetf1(m,s):zetf1(m,s)+dim1(m,s)**2-1)=psi0(zetf0(m,s):zetf0(m,s)+dim0(m,s)**2-1)
          end if
       end do
    end do
  end subroutine copyPsi

  subroutine copyPsi_v2(psi0,a0,zpsi0,zphi0,zphase0,zxi0,zxivec0,zagmat0,dim0,block0,&
       psi1,a1,zpsi1,zphi1,zphase1,zxi1,zxivec1,zagmat1,dim1,block1)

    use griddatmod, only : nspfdof,gwpm,nstate,nmode
    use maxv, only: maxdim,maxsta

    integer(long), dimension(maxsta), intent(in)        :: block0,block1,&
                                                           zpsi0,zpsi1
    integer(long), dimension(maxdim,maxsta), intent(in) :: dim0,zphi0,zphase0,zxi0,&
                                                           zxivec0,zagmat0
    integer(long), dimension(maxdim,maxsta), intent(in) :: dim1,zphi1,zphase1,zxi1,&
                                                           zxivec1,zagmat1
    complex(dop), dimension(:), intent(in)              :: psi0,a0
    complex(dop), dimension(:), intent(out)              :: psi1,a1

    integer(long) :: m,s

    do s=1,nstate
          psi1=0.0_dop
       psi1(zpsi1(s):zpsi1(s)+min(block1(s),block0(s))-1) = &
                   psi0(zpsi0(s):zpsi0(s)+min(block1(s),block0(s))-1)
!write(*,*)"psi(zpsi)",psi1(zpsi1(s):zpsi1(s)+min(block1(s),block0(s))-1)
!write(*,*)"block1(s)",block1(s)
!write(*,*)"block0(s)",block0(s)
       if (block1(s)>block0(s)) then
          psi1(zpsi1(s)+block0(s):zpsi1(s)+block1(s)-1)=0.0_dop
       end if
       do m=1,nmode
!write(*,*)"nspfdof(m)",nspfdof(m)
          if (gwpm(m)) then
             psi1(zxi1(m,s):zxi1(m,s)+min(dim1(m,s),dim0(m,s))*nspfdof(m)-1)=&
!             psi1(zphase1(m,s):zphase1(m,s)+min(dim1(m,s),dim0(m,s))*nspfdof(m)-1)=&
                 psi0(zxi0(m,s):zxi0(m,s)+min(dim1(m,s),dim0(m,s))*nspfdof(m)-1)
!write(*,*)"psi(zxi)",psi1(zxi1(m,s):zxi1(m,s)+min(dim1(m,s),dim0(m,s))*nspfdof(m)-1)
!             psi1(zphi1(m,s):zphi1(m,s)+min(block1(s),block0(s))-1)=&
!                  psi0(zphi0(m,s):zphi0(m,s)+min(block1(s),block0(s))-1)
             psi1(zphi1(m,s):zphi1(m,s)+min(dim1(m,s),dim0(m,s))-1)=&
!             a0(zphase0(m,s):zphase0(m,s)+min(block1(s),block0(s))-1)
             a0(zphase0(m,s):zphase0(m,s)+min(dim1(m,s),dim0(m,s))-1)
!write(*,*)"psi(zphi)",psi1(zphi1(m,s):zphi1(m,s)+min(block1(s),block0(s))-1)
             a1(zagmat1(m,s):zagmat1(m,s)+min(block1(s),block0(s))*nspfdof(m)**2-1)=&
                  a0(zagmat0(m,s):zagmat0(m,s)+min(block1(s),block0(s))*nspfdof(m)**2-1)
!write(*,*)"a1(zagmat)",a1(zagmat1(m,s):zagmat1(m,s)+min(block1(s),block0(s))*nspfdof(m)**2-1)
             a1(zxivec1(m,s):zxivec1(m,s)+min(dim1(m,s),dim0(m,s))*nspfdof(m)-1)=&
             a0(zxivec0(m,s):zxivec0(m,s)+min(dim1(m,s),dim0(m,s))*nspfdof(m)-1)
!write(*,*)"a1(xivec)",a1(zxivec1(m,s):zxivec1(m,s)+min(dim1(m,s),dim0(m,s))*nspfdof(m)-1)
             a1(zphase1(m,s):zphase1(m,s)+min(block1(s),block0(s))-1)=&
             a0(zphase0(m,s):zphase0(m,s)+min(block1(s),block0(s))-1)
!write(*,*)"a1(zphase)",a1(zphase1(m,s):zphase1(m,s)+min(block1(s),block0(s))-1)
!write(*,*)(zphi0(m,s)+min(block1(s),block0(s))-1)
          end if
       end do
    end do
  end subroutine copyPsi_v2

!  subroutine copyPsi(psi0,a0,zpsi0,zphi0,zxi0,zxivec0,zagmat0,dim0,block0,&
!       psi1,a1,zpsi1,zphi1,zxi1,zxivec1,zagmat1,dim1,block1)
!
!    use griddatmod, only : nspfdof,gwpm,nstate,nmode
!    use maxv, only: maxdim,maxsta
!    
!    integer(long), dimension(maxsta), intent(in)        :: block0,block1,&
!                                                           zpsi0,zpsi1
!    integer(long), dimension(maxdim,maxsta), intent(in) :: dim0,zphi0,zxi0,&
!                                                           zxivec0,zagmat0
!    integer(long), dimension(maxdim,maxsta), intent(in) :: dim1,zphi1,zxi1,&
!                                                           zxivec1,zagmat1
!    complex(dop), dimension(:), intent(in)              :: psi0,a0
!    complex(dop), dimension(:), intent(out)              :: psi1,a1
!
!    integer(long) :: m,s
!
!    do s=1,nstate
!       psi1(zpsi1(s):zpsi1(s)+min(block1(s),block0(s))-1) = &
!                   psi0(zpsi0(s):zpsi0(s)+min(block1(s),block0(s))-1)
!!write(*,*)"psi(zpsi)",psi1(zpsi1(s):zpsi1(s)+min(block1(s),block0(s))-1)
!       if (block1(s)>block0(s)) then
!          psi1(zpsi1(s)+block0(s):zpsi1(s)+block1(s)-1)=0.0_dop
!       end if
!       do m=1,nmode
!write(*,*)"nspfdof(m)",nspfdof(m)
!          if (gwpm(m)) then
!             psi1(zxi1(m,s):zxi1(m,s)+min(dim1(m,s),dim0(m,s))*nspfdof(m)-1)=&
!                 psi0(zxi0(m,s):zxi0(m,s)+min(dim1(m,s),dim0(m,s))*nspfdof(m)-1)
!write(*,*)"psi(zxi)",psi1(zxi1(m,s):zxi1(m,s)+min(dim1(m,s),dim0(m,s))*nspfdof(m)-1)
!             psi1(zphi1(m,s):zphi1(m,s)+min(block1(s),block0(s))-1)=&
!                  psi0(zphi0(m,s):zphi0(m,s)+min(block1(s),block0(s))-1)
!!write(*,*)"psi(zphase)",psi1(zphi1(m,s):zphi1(m,s)+min(block1(s),block0(s))-1)
!             a1(zagmat1(m,s):zagmat1(m,s)+min(block1(s),block0(s))*nspfdof(m)**2-1)=&
!                  a0(zagmat0(m,s):zagmat0(m,s)+min(block1(s),block0(s))*nspfdof(m)**2-1)
!write(*,*)"a1(zagmat)",a1(zagmat1(m,s):zagmat1(m,s)+min(block1(s),block0(s))*nspfdof(m)**2-1)
!             a1(zxivec1(m,s):zxivec1(m,s)+min(dim1(m,s),dim0(m,s))*nspfdof(m)-1)=&
!             a0(zxivec0(m,s):zxivec0(m,s)+min(dim1(m,s),dim0(m,s))*nspfdof(m)-1)
!write(*,*)"a1(xivec)",a1(zxivec1(m,s):zxivec1(m,s)+min(dim1(m,s),dim0(m,s))*nspfdof(m)-1)
!             a1(zphi1(m,s):zphi1(m,s)+min(block1(s),block0(s))-1)=&
!             a0(zphi0(m,s):zphi0(m,s)+min(block1(s),block0(s))-1)
!write(*,*)"a1(zphase)",a1(zphi1(m,s):zphi1(m,s)+min(block1(s),block0(s))-1)
!write(*,*)(zphi0(m,s)+min(block1(s),block0(s))-1)
!          end if
!       end do
!    end do
!  end subroutine copyPsi

  subroutine renormalize(psi,agmat)
    use psidef
    use griddatmod, only : nstate,nmode,gwpm,nspfdof
    use BELglobals, only :agmatdimmax,dgldimmax
    complex(dop), dimension(dgldimmax), intent(inout) :: psi
    complex(dop), dimension(agmatdimmax), intent(inout)   :: agmat
!    complex(dop), dimension(dgldim), intent(inout) :: psi
!    complex(dop), dimension(agmatdim), intent(inout)   :: agmat
    complex(dop) :: norm0
    integer(long) :: m,s
    do s=1,nstate
       do m=1,nmode
          if (gwpm(m)) then
             call gimmenorm(nspfdof(m),dim(m,s),dimred(m,s),&
!                  agmat(zphi(m,s)),agmat(zxivec(m,s)),agmat(zagmat(m,s)),& 
                  agmat(zphase(m,s)),agmat(zxivec(m,s)),agmat(zagmat(m,s)),& ! phase mod yapol
                  psi(zpsi(s)),norm0)
             psi(zpsi(s):zpsi(s)+block(s)-1)=psi(zpsi(s):zpsi(s)+block(s)-1)*norm0**(-0.5)
          end if
       end do
    end do
  end subroutine renormalize

  subroutine checknorm(psi,agmat,norm0)
    use psidef
    use griddatmod, only : nstate,nmode,gwpm,nspfdof
    complex(dop), dimension(dgldim), intent(in) :: psi
    complex(dop), dimension(agmatdim), intent(in)   :: agmat
    complex(dop), intent(out) :: norm0
    integer(long) :: m,s
    do s=1,nstate
       do m=1,nmode
          if (gwpm(m)) then
             call gimmenorm(nspfdof(m),dim(m,s),dimred(m,s),&
!                  agmat(zphi(m,s)),agmat(zxivec(m,s)),agmat(zagmat(m,s)),& 
                  agmat(zphase(m,s)),agmat(zxivec(m,s)),agmat(zagmat(m,s)),& 
                  psi(zpsi(s)),norm0)
          end if
       end do
    end do
  end subroutine checknorm

  subroutine resetGwp(time,psi,agmat,quality,iflag,ierrp,cinteg)

    use global
    use channels, only : iBEL
    use griddatmod
    use psidef
    use runinwfmod
    use runpropmod, only : BELprint,BELplotlims,BELflags
    use gwplib, only: gwptophi,renormdgwp,diagonal_moments,transahelp,transavec,transavec1,printGWPSPF, &
                      fgaover
    use logdat

    use iopsidef, only : psidat

    use BELglobals

    use Nelder_Mead


    real(dop),intent(in) :: time
    complex(dop), dimension(dgldimmax), intent(inout) :: psi
    complex(dop), dimension(agmatdimmax), intent(inout)   :: agmat
!    complex(dop), dimension(dgldim), intent(inout) :: psi
!    complex(dop), dimension(agmatdim), intent(inout)   :: agmat

    real(dop),intent(out),optional :: quality


!    complex(dop)                                :: roverlap,aoverlap,normp
    complex(dop)                                :: roverlap,normp
    complex(dop)                                :: tmp
    integer(long)                               :: re
    integer(long) :: exte

    logical(kind=4), save                          :: initialized = .false.

    integer(long),save                             :: lcwork
    integer(long)                                  :: tempdimred

    integer(long) :: i,j
!    integer(long) :: lmaxspf,lmaxgdof,i,j
    integer(long), save                             :: tid=0 ! timer ID


    real(dop),allocatable,save :: pqISall(:,:,:)
    real(dop),allocatable,save :: qexp(:),sigexp(:),pexp(:)
    complex(dop), allocatable,save   :: cwork(:)
real(dop),dimension(nspfdof(1))::Q
complex(dop)::wf
complex(dop),dimension(:,:),allocatable,save::GSR
  
    logical(kind=4) ,optional :: cinteg
    integer(long)   :: iflag,ierrp
    integer(long)                                  :: ierr,ierr_aovl,ierr_eigval
    integer(long), save                                  :: istep=0
    character(len=100) :: arg

    real(dop),save :: thr=0.95d0
    real(dop) :: beta
    complex(dop)   :: aovl,aovl_org
    integer(long),save   :: ired=0
    complex(dop),save:: aini

    real(dop),dimension(nstate) :: spop
    real(dop) ::normt
    integer(long) :: ipnz
    integer(long) :: ns !p0copy

    complex(dop),dimension(:,:),allocatable :: gs2
    complex(dop),dimension(:,:),allocatable :: gs
    real(dop),dimension(:), allocatable ::rw,eig
    real(dop),dimension(:), allocatable :: work

    real(dop) ::gsmax,gsmin
    real(dop) :: thr1 !zetapop

    integer(long),save :: icount=0
    icount=icount+1
     beta=abs(BELflags(1))*abs(BELflags(1))
    call get_timer(tid, 'BEL')
    call start_timer(tid)

    if ( psitype == 0 ) then
       call transavec1(psi)
    end if

      !     is this the first call?
      if (.not. initialized) then
!         if (.not. allocated(dimmax)) allocate(dimmax(maxdim,maxsta))
!         if (.not. allocated(gwpdimmax)) allocate(gwpdimmax(maxdim,maxsta))
!         if (.not. allocated(zphimax)) allocate(zphimax(maxdim,maxsta))
!         if (.not. allocated(zphasemax)) allocate(zphasemax(maxdim,maxsta))
!         if (.not. allocated(zximax)) allocate(zximax(maxdim,maxsta))
!         if (.not. allocated(zxivecmax)) allocate(zxivecmax(maxdim,maxsta))
!         if (.not. allocated(zagmatmax)) allocate(zagmatmax(maxdim,maxsta))
!         if (.not. allocated(blockmax)) allocate(blockmax(maxsta))
!         if (.not. allocated(zpsimax)) allocate(zpsimax(maxsta))
!         if (.not. allocated(predim)) allocate(predim(maxdim,maxsta)) !TM
!         if (.not. allocated(predimred)) allocate(predimred(maxdim,maxsta)) !TM
!         dimmax(:,:)=dim(:,:)
!         ! Modification by yapol
!!         if (lgwpphase) then
!!            zphimax(:,:)=zphi(:,:)
!!         else
!!            zphimax(:,:)=zphase(:,:)
!!         endif
!         ! End of modification
!!         zximax(:,:)=zxi(:,:)
!!         zpsimax(:)=zpsi(:)
!!         zxivecmax(:,:)=zxivec(:,:)
!!         zagmatmax(:,:)=zagmat(:,:)
!!         blockmax(:)=block(:)
!!
!         lmaxspf=maxval(dim(1:nmode,1:nstate),mask=spread(gwpm(1:nmode),2,nstate))
!         lmaxgdof=maxval(nspfdof(1:nmode),mask=gwpm(1:nmode))
!
         if (.not. allocated(PQfix)) allocate(PQfix(3,lmaxgdof,lmaxspf))
         if (.not. allocated(pqISall)) allocate(pqISall(3,lmaxgdof,lmaxspf))
!
         write(1000,*)'lmaxgdof,lmaxspf',lmaxgdof,lmaxspf
         if (allocated(DomainUpper)) then
            if (size(DomainUpper) < (lmaxgdof)) then
              deallocate(DomainUpper)
              deallocate(DomainLower)
              deallocate(DomainMid)
              deallocate(MaxWidth)
            endif
         endif
         if (.not. allocated(DomainUpper)) then
            allocate(DomainUpper(lmaxgdof))
            allocate(DomainLower(lmaxgdof))
            allocate(DomainMid(lmaxgdof))
            allocate(MaxWidth(lmaxgdof))
         end if

         if (allocated(qexp)) then
            if (size(qexp) < (lmaxgdof)) deallocate(qexp)
         endif
         if (allocated(sigexp)) then
            if (size(sigexp) < (lmaxgdof)) deallocate(sigexp)
         endif
         if (allocated(pexp)) then
           if (size(pexp) < (lmaxgdof)) deallocate(pexp)
         endif
         if (.not. allocated(qexp)) allocate(qexp(lmaxgdof))
         if (.not. allocated(sigexp)) allocate(sigexp(lmaxgdof))
         if (.not. allocated(pexp)) allocate(pexp(lmaxgdof))

         lcwork=lmaxspf*lmaxspf*2
         if (.not. allocated(cwork)) allocate(cwork(lcwork))

         if (.not. allocated(allprod)) allocate(allprod(lmaxspf,ssdim))
         if (.not. allocated(newprod)) allocate(newprod(lmaxspf))
         if (.not. allocated(allnewprod)) allocate(allnewprod(lmaxspf,ssdim))
!         if (.not. allocated(psinew)) allocate(psinew(dgldim))
!         if (.not. allocated(psiold)) allocate(psiold(dgldim))
!         if (.not. allocated(anew)) allocate(anew(agmatdim))
!         if (.not. allocated(aold)) allocate(aold(agmatdim))
!         if (.not. allocated(GSR)) allocate(GSR(npg(1)*dim(1,1),npg(1)*dim(1,1)))
!!write(*,*)"dgldim",dgldim
!!write(*,*)"dim",dim
!!write(*,*)"gwpdim",gwpdim
!         dimmax(:,:)=dim(:,:)
!         gwpdimmax(:,:)=gwpdim(:,:)
!         zphimax(:,:)=zphi(:,:)
!         zximax(:,:)=zxi(:,:)
!         zpsimax(:)=zpsi(:)
!         zxivecmax(:,:)=zxivec(:,:)
!         zagmatmax(:,:)=zagmat(:,:)
!         blockmax(:)=block(:)
!         zphasemax(:,:)=zphase(:,:) ! modyfied by TM
!
!    dimred(:,:)=dim(:,:) !TM
!    predim(:,:)=dim(:,:) !TM
!!write(*,*)"predim",predim
!ssdim=1
!         if (gwpm(2)) then
!         else
!            ssdim=dim(2,1)
!         endif
!write(1000,*)"ssdim",ssdim
!stop
         initialized = .true.
      end if

!    write(*,*)"dimred(:,:)",dimred(:,:)

    if (.not.lgwpphase) then
      write(*,*)'Check whether we''re copying psi and agmat correctly'
      stop
    endif
!    call copyPsi(psi,agmat,zpsi,zphi,zxi,zxivec,zagmat,dim,block,&
!          psinew,anew,zpsimax,zphimax,zximax,zxivecmax,zagmatmax,dimmax,blockmax)
!write(*,*)"block",block,"blockmax",blockmax
!write(*,*)"psi",psi
!write(*,*)"agmatdim",agmatdim
!write(*,*)"gwpdim(m,s)",gwpdim(1,1)
!write(*,*)"npg(m)",npg(1)
!write(*,*)"first copy"
    call copyPsi(psi,agmat,zpsi,zphi,zphase,zxi,zxivec,zagmat,zetf,dim,block,&
          psinew,anew,zpsimax,zphimax,zphasemax,zximax,zxivecmax,zagmatmax,zetfmax,dimmax,blockmax) ! modyfied by TM

if (.false.) then
  do i=-60,60
    Q(1)=0.015*i
    do j=-60,60
      Q(2)=0.015*j
      call GwpValue(nspfdof(1),dimred(1,1),&
         psinew(zphi(1,1)),anew(zxivec(1,1)),anew(zagmat(1,1)),psinew(zpsi(1)),Q,&
         wf)
      write(300,*)Q,real(wf),abs(wf)**2
    enddo
    write(300,'(a)')
  enddo
endif
!write(*,*)zphi(1,1),zxi(1,1),zagmat(1,1),zpsi(1),dimred(1,1)
!stop

   if (ierrp==0) predim(:,:)=dim(:,:) !TM
    dim(:,:)=dimmax(:,:)
    gwpdim(:,:)=gwpdimmax(:,:)

    call psidat

!    write(*,*)'dgldim',dgldim
    aold(1:agmatdim)=anew(1:agmatdim)
    psiold(1:dgldim)=psinew(1:dgldim)


!    DomainLower=-huge(1.0)
!    DomainUpper=huge(1.0)
    NoMoreNonorth=.false.


    istep=istep+1
    do s=1,nstate
       do m=1,nmode
          if (gwpm(m)) then
             
             ipnz=1
             norma=0.0_dop
             gdof=nspfdof(m)
             dim1=dim(m,s)
             dim1red=dimred(m,s)
                
!write(1000,*)"time",time/fs
!write(1000,*)"dim1red",dim1red
!write(*,*)"gdof",gdof
!      Q=-3.0d0
!      call GwpValue(gdof,dim1red,&
!         aold(zphase(m,s)),aold(zxivec(m,s)),aold(zagmat(m,s)),psiold(zpsi(s)),Q,&
!         wf)
!      write(*,*)Q,real(wf),abs(wf)**2
!      write(*,*)"aold(1:agmatdim)",aold(1:agmatdim)
!      write(*,*)"psiold(1:dgldim)",psiold(1:dgldim)

             
             roverlap=100000.0_dop
             PQsize=int(BELflags(8))
!write(1000,*)"psiold",psiold(zpsi(s):zpsi(s)+block(s)-1)
!write(1000,*)"psinew",psinew(zpsi(s):zpsi(s)+block(s)-1)
!write(1000,*)"psinew",psinew(22)
             if (ssdim>1) then
               do j=0,ssdim-1
                 do i=1,dim1red
                 newcoef(i,j+1)=psinew(zpsi(s)+j*dim1red+i-1)
!                 psinew(zpsi(s)+j*dim1+i-1)=newcoef(i,j+1)
! write(1000,*)"newcoef",i,j,newcoef(i,j+1),psinew(zpsi(s)+j*dim1red+i-1), &
!              zpsi(s)+j*dim1red+i-1,zpsi(s)+j*dim1+i-1
                 enddo
               enddo
               do j=0,ssdim-1
                 do i=1,dim1red
                    psinew(zpsi(s)+j*dim1+i-1)=newcoef(i,j+1)
                 enddo
               psinew(zpsi(s)+j*dim1+dim1red:zpsi(s)+(j+1)*dim1-1)=(0.0d0,0.0d0)
               enddo 
             endif
!write(1000,*)"psinew",psinew(zpsi(s):zpsi(s)+block(s)-1)
             call gimmenorm(nspfdof(m),dim(m,s),dimred(m,s),&
!                  aold(zphi(m,s)),aold(zxivec(m,s)),aold(zagmat(m,s)),psiold(zpsi(s)),& 
                  aold(zphase(m,s)),aold(zxivec(m,s)),aold(zagmat(m,s)),psiold(zpsi(s)),& ! phase mod yapol
                  norm0,pqISall,qexp,sigexp,pexp)
!             psiold(zpsi(s):zpsi(s)+block(s)-1)=psiold(zpsi(s):zpsi(s)+block(s)-1)/sqrt(norm0)
!if (icount==2) stop
!write(1000,*)"norm0",norm0
!             call momspace(nspfdof(m),dimred(m,s),etaold,zetaold,tauold,&
!                           aold(zphase(m,s)),psiold(zxi(m,s)),aold(zagmat(m,s))) !Fourier transform Phi(P)<=Psi(Q)
             thr1=abs(BELflags(1)) ! zetapop
             normp=norm0
             spop(s)=dble(norm0)
!             if (psiold(zpsi(s))/=0.0d0) norm0=1.0_dop
             DomainMid=qexp
             DomainUpper=qexp
             DomainLower=qexp
             if (istep==1)  then
                  aini=aold(zagmat(m,s))
             endif
             ierr_aovl=0
             ierr_eigval=0
111 continue
             tempdimred=dim(m,s)
             roverlap=100000.0_dop
             aoverlap=100000.0_dop
             re=-1
             exte=1
             if (psiold(zpsi(s))==0.0d0) then
!               tempdimred=1
!               go to 66
               ipnz=0
!!!!! zetapop
               thr1=abs(BELflags(1))
                write(1000,*)"time and s",time/fs,s
               tempdimred=int(BELflags(4))
             else
               if (abs(BELflags(2))==0.0d0) then
                 thr1=spop(s)*abs(BELflags(1))
               elseif (abs(BELflags(2))==1.0d0) then
                 thr1=dsqrt(spop(s))*abs(BELflags(1))
               else
                 thr1=abs(BELflags(1))
               endif
!!!!! zetapop
             endif
             do while (exte<=tempdimred)
                ex1=exte
                ex2=exte

!    if(ex1==1) then
!      PQsize=1
!    else
!      PQsize=2
!    endif

!exte=dimred(1,1)+1
                call initPQfix(nspfdof(m),m,dim(m,s),exte,psiold(zxi(m,s)),aold(zagmat(m,s)),&
                     PQfix,qexp,sigexp,pexp,aini,ipnz)
!    write(*,*)"PQfix",PQfix(1:3,1:gdof,ex1)
!    write(*,*)"qexp",qexp(1),"pexp",pexp(1),"sigexp",sigexp(1)
                call shiftGwpSpf(m,s,exte,nspfdof(m),dim(m,s),dimred(m,s),&
                     psiold,aold,newcoef,psinew,anew,&
                     GSR,aoverlap,time,&
                     norm0,norma,pqISall,PQfix,PQsize,PQset,allprod,newprod,&
                     allnewprod,istep,ierr,ierr_aovl,ierr_eigval,thr,ipnz,ierrp,re, &
                     tempdimred)
!    write(*,*)"PQfix after shiftGwpSpf",PQfix(1:3,1,ex1)
!    write(*,*)"qexp",qexp(1),"pexp",pexp(1),"sigexp",sigexp(1)
!    write(1000,*)"aoverlap",aoverlap,ex1
                if  ((abs(aoverlap)<abs(thr1)) .and. (re==-1))then
!                if ((abs(aoverlap)<abs(BELflags(1))) .and. (re==-1) .and.  &
!                    (exte>int(BELflags(4))) .or. (ierr==1) ) then
                   re=exte
                   roverlap=aoverlap
                   tempdimred=min(dim(m,s),re+int(BELflags(4)))
!                   if (tempdimred < 30) tempdimred=30
!                   if (istep==1) tempdimred=1
!                   tempdimred=min(dim(m,s),int(BELflags(4)))
!                   if (re>=int(BELflags(4))) then
!                    tempdimred=re
!                   endif
!                   tempdimred=min(dim(m,s),re+0)
                end if

               if (ierr==1 ) then
                 tempdimred=exte
                 goto 999
               endif
                exte=exte+1
!    write(*,*)"exte,tempdimred",exte,tempdimred
!    write(*,*)" "
             end do
!stop 'e'
999 continue
            if ((ierr==1) .and. (ierr_eigval/=0) ) then
                  write(iBEL,'(g14.7,i5.0,g16.8,i5.0,2g16.8,f12.8,i3,x,A,i3,x,A,f5.2)') &
                  time/fs,re,abs(roverlap),&
                  tempdimred-1,abs(aoverlap),abs(normp),abs(norma),&
                  ierr,"ierr_eigval",ierr_eigval,"thr",thr
                  call myflush(iBEL)
                  thr=thr-0.01d0
                  goto 111
            endif
            if ((abs(aoverlap)>abs(BELflags(1))) ) then
                  ierr_aovl=ierr_aovl+1
                  write(iBEL,'(g14.7,i5.0,g16.8,i5.0,2g16.8,f12.8,i3,x,A,i3,x,A,f5.2)') &
                  time/fs,re,abs(roverlap),&
                  tempdimred,abs(aoverlap),abs(normp), abs(norma),&
                  ierr,"ierr_aovl",ierr_aovl,"thr",thr
                  call myflush(iBEL)
                  thr=thr+0.01d0
!                  dimred(m,s)=dimred(m,s)+10
!                  if (dimred(m,s)>dimmax(m,s)) then
!                    stop"oversize dim"
!                  endif
                  goto 111
            endif
!             if (ierr==1) then
!               dimred(m,s)=tempdimred-1
!               gwpdim(m,s)=tempdimred-1 !TM
!             endif
!66 continue
!             if (allocated(gs2)) deallocate(gs2)
!             if (allocated(gs)) deallocate(gs)
!             if (allocated(rw)) deallocate(rw)
!             if (allocated(eig)) deallocate(eig)
!             if (allocated(work)) deallocate(work)
!             if (.not. allocated(gs2)) allocate(gs2(npg(m)*ex1,npg(m)*ex1))
!             if (.not. allocated(gs)) allocate(gs(npg(m)*ex1,npg(m)*ex1))
!             if (.not. allocated(rw)) allocate(rw(3*ex1))
!             if (.not. allocated(eig)) allocate(eig(3*ex1))
!             if (.not. allocated(work)) allocate(work(npg(m)*ex1*32))
             call getarg(2,arg)
!             if (ex1>1) then
!               call fgaover(gs2(1:ex1,1:ex1),anew(zphase(m,s)),anew(zphase(m,s)), &
!                    psinew(zxi(m,s)),psinew(zxi(m,s)), &
!                    anew(zagmat(m,s)),anew(zagmat(m,s)),ex1,ex1,gdof)
!                 gs(1:ex1,1:ex1)=gs2(1:ex1,1:ex1)
!                i=0
!                call zheev('N','U',ex1,gs,npg(m)*ex1,eig,work,npg(m)*ex1*32,rw,i)
!                 if (i/=0) then
!                 write(*,*)i,ex1
!                   stop 'zheev failed in shiftGwpSpf'
!                 endif
!                 gsmax=eig(1)
!                 gsmin=eig(1)
!                 gsmax=1.0d0
!                 gsmin=1.0d0
!                 do i=2,ex1
!                   if (abs(eig(i)) > abs(gsmax)) gsmax=eig(i)
!                   if (abs(eig(i)) < abs(gsmin)) gsmin=eig(i)
!                 enddo
!                 if ((istep==1))then
!                 call getarg(2,arg)
!                 open(100,file=trim(arg)//".100")
!                       write(100,*)time/fs,s,ex1,abs(gsmin/gsmax)
!                 close(100)
!                 else
                 open(100,file=trim(arg)//".100",position="append")
!                   write(100,*)time/fs,s,ex1,abs(gsmin/gsmax)
                   write(100,*) " "
                   write(100,*) " "
                 close(100)
                 open(102,file=trim(arg)//".102",position="append")
                   write(102,*) " "
                   write(102,*) " "
                 close(102)
!             endif
!       endif
!             open(100,file=trim(arg)//".100",position="append")
!               write(100,*) " "
!               write(100,*) " "
!             close(100)
!             open(101,file=trim(arg)//".101",position="append")
!               write(101,*) " "
!               write(101,*) " "
!             close(101)
             predimred(m,s)=dimred(m,s)
             dimred(m,s)=tempdimred
             gwpdim(m,s)=tempdimred !TM
!    write(*,*)"dimred(m,s)",dimred(m,s)
             if (re==-1) then
                re=dimred(m,s)
                roverlap=aoverlap
             end if
             aovl_org=aoverlap
          if (ired==1) then
             do i=1,tempdimred
               if (dimred(m,s)==int(BELflags(4))+1) then
                 go to 55
               endif
!               aovl=aoverlap+2.0_dop*dble(dconjg(psinew(zpsi(s)-1+i))*allprod(i))&
!                    -(dconjg(psinew(zpsi(s)-1+i))*newprod(i))
               if (abs(aovl)<beta) then
                 aoverlap=aovl
                 psinew(zpsi(s)+i-1)=(0.0d0,0.0d0)
                 dimred(m,s)=dimred(m,s)-1
                 gwpdim(m,s)=gwpdim(m,s)-1
               endif
             enddo
55 continue
            predimred(m,s)=dimred(m,s)
            if (tempdimred/=dimred(m,s) ) then
             do i=1,tempdimred-1
               do j=i+1,tempdimred
                 if (abs(psinew(zpsi(s)+i-1)) .lt. abs(psinew(zpsi(s)+j-1))) then
                    tmp=(0.0d0,0.0d0)
                    tmp=psinew(zpsi(s)+i-1)
                    psinew(zpsi(s)+i-1)=psinew(zpsi(s)+j-1)
                    psinew(zpsi(s)+j-1)=tmp
                    tmp=(0.0d0,0.0d0)
                    tmp=psinew(zxi(m,s)+i-1)
                    psinew(zxi(m,s)+i-1)=psinew(zxi(m,s)+j-1)
                    psinew(zxi(m,s)+j-1)=tmp
                    tmp=(0.0d0,0.0d0)
                    tmp=psinew(zphase(m,s)+i-1)
                    psinew(zphi(m,s)+i-1)=psinew(zphi(m,s)+j-1)
                    psinew(zphi(m,s)+j-1)=tmp
                    tmp=(0.0d0,0.0d0)
                    tmp=anew(zxivec(m,s)+i-1)
                    anew(zxivec(m,s)+i-1)=anew(zxivec(m,s)+j-1)
                    anew(zxivec(m,s)+j-1)=tmp
                    tmp=(0.0d0,0.0d0)
                    tmp=anew(zagmat(m,s)+i-1)
                    anew(zagmat(m,s)+i-1)=anew(zagmat(m,s)+j-1)
                    anew(zagmat(m,s)+j-1)=tmp
                    tmp=(0.0d0,0.0d0)
                    tmp=anew(zphase(m,s)+i-1)
                    anew(zphase(m,s)+i-1)=anew(zphase(m,s)+j-1)
                    anew(zphase(m,s)+j-1)=tmp
                 endif
                enddo
             enddo
            endif
          endif
!             write(iBEL,'(A,g14.7,i6.0,i6.0,g16.8,i6.0,g16.8,i6.0,g16.8,f12.8,f12.8,i3,i3,i3,f5.2,i3)') &
!                  'R',time/fs,predimred(m,s),re,abs(roverlap),&
!                  tempdimred,abs(aovl_org),&
!                  dimred(m,s),abs(aoverlap),abs(normp),abs(norma), & 
!                  ierr,ierr_aovl,ierr_eigval,thr,s
!                  call myflush(iBEL)
             
             if (present(quality)) then
                quality=abs(roverlap)
             end if

             if (ssdim>1) then
               do j=0,ssdim-1
                 do i=1,dimred(m,s)
                 psinew(zpsi(s)+j*dimred(m,s)+i-1)=newcoef(i,j+1)
!                 write(1000,*)"newcoef after re",i,j,newcoef(i,j+1)
                 enddo
               enddo
             endif
             call redblock(s)
!             psinew(zpsi(s)+dimred(m,s):zpsi(s)-1+dim(m,s))=0.0_dop
             psinew(zpsi(s)+blockred(s):zpsi(s)+block(s)-1)=0.0_dop

!write(1000,*)"after re and sort psinew",psinew(zpsi(s):zpsi(s)+block(s)-1)

              do j=0,ssdim-1
             write(iBEL,'(A,g14.7,i6.0,i6.0,g16.8,i6.0,g16.8,i6.0,g16.8,f12.8,f12.8,i3,i3,i3,f5.2,i3,i3)') &
                  'R',time/fs,predimred(m,s),re,abs(roverlap),&
                  tempdimred,abs(aovl_org),&
                  dimred(m,s),abs(aoverlap),abs(normp),abs(norma), & 
                  ierr,ierr_aovl,ierr_eigval,thr,s,j
                call printGWPSPF(BELprint,nspfdof(m),DomainLower,DomainUpper,&
                     predimred(m,s),psiold(zpsi(s)+j*predimred(m,s)),aold(zphase(m,s)),aold(zxivec(m,s)),aold(zagmat(m,s)),&
                     dimred(m,s),psinew(zpsi(s)+j*dimred(m,s)),anew(zphase(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)),&
                     xplotlims=BELplotlims,channel=iBEL)
              enddo
!                call printGWPSPF(BELprint,nspfdof(m),DomainLower,DomainUpper,&
!                     predimred(m,s),psiold(zpsi(s)),aold(zphase(m,s)),aold(zxivec(m,s)),aold(zagmat(m,s)),&
!                     dimred(m,s),psinew(zpsi(s)),anew(zphase(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)),&
!                     xplotlims=BELplotlims,channel=iBEL)
               call myflush(iBEL)
          end if
       end do
    end do
!write(1000,*)'leap'
!TerryFlag=.true.
!stop

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! p0copy
    ns=2
    do s=1,nstate
       do m=1,nmode
          if (gwpm(m)) then
            if (psiold(zpsi(s))==0.0d0) then
            predimred(m,s)=predimred(m,ns)
            dimred(m,s)=dimred(m,ns)
            dim(m,s)=dim(m,ns)
            gwpdim(m,s)=gwpdim(m,ns)
            block(s)=block(ns)
            blockred(s)=blockred(ns)
            anew(zphase(m,s):zphase(m,s)+blockred(s)-1)=anew(zphase(m,ns):zphase(m,ns)+blockred(ns)-1)
            anew(zxivec(m,s):zxivec(m,s)+dimred(m,s)*nspfdof(m)-1)=anew(zxivec(m,ns):zxivec(m,ns)+dimred(m,ns)*nspfdof(m)-1)
            anew(zagmat(m,s):zagmat(m,s)+dimred(m,s)*nspfdof(m)**2-1)=anew(zagmat(m,ns):zagmat(m,ns)+dimred(m,ns)*nspfdof(m)**2-1)
            psinew(zphi(m,s):zphi(m,s)+dimred(m,s)*nspfdof(m)-1)=psinew(zphi(m,ns):zphi(m,ns)+dimred(m,ns)*nspfdof(m)-1)
            psinew(zxi(m,s):zxi(m,s)+dimred(m,s)*nspfdof(m)-1)=psinew(zxi(m,ns):zxi(m,ns)+dimred(m,ns)*nspfdof(m)-1)
            normp=0.0d0
            norma=0.0d0
            roverlap=0.0d0
            aoverlap=0.0d0
            aovl_org=0.0d0
              do j=0,ssdim-1
             write(iBEL,'(A,g14.7,i6.0,i6.0,g16.8,i6.0,g16.8,i6.0,g16.8,f12.8,f12.8,i3,i3,i3,f5.2,i3,i3)')&
                  'R',time/fs,predimred(m,s),re,abs(roverlap),&
                  tempdimred,abs(aovl_org),&
                  dimred(m,s),abs(aoverlap),abs(normp),abs(norma), &
                  ierr,ierr_aovl,ierr_eigval,thr,s,j
                call printGWPSPF(BELprint,nspfdof(m),DomainLower,DomainUpper,&
                     predimred(m,s),psiold(zpsi(s)+j*predimred(m,s)),aold(zphase(m,s)),aold(zxivec(m,s)),aold(zagmat(m,s)),&
                     dimred(m,s),psinew(zpsi(s)+j*dimred(m,s)),anew(zphase(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)),&
                     xplotlims=BELplotlims,channel=iBEL)
              enddo

            endif
          endif
        enddo
     enddo
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! p0copy

normt=0.0d0
    do s=1,nstate
!!       psinew(zpsi(s):zpsi(s)+block(s)-1)=psinew(zpsi(s):zpsi(s)+block(s)-1)*norma**(-0.5)
       do m=1,nmode
          if (gwpm(m)) then
!             if ( BELprint /= 0 ) then
!write(*,*)
!write(*,*)psiold(1),psiold(51),psiold(101)
!write(*,*)psiold(zpsi(s)),psiold(zxi(m,s)),aold(zagmat(m,s)),'r'
!write(*,*)'rg',psiold(zxi(m,s)),zpsi(s),zxi(m,s)
!write(*,*)
!write(*,*)psinew(1),psinew(51),psinew(101)
!write(*,*)psinew(zpsi(s)),psinew(zxi(m,s)),anew(zagmat(m,s)),'r'
!write(*,*)aold(zxivec(m,s)),anew(zxivec(m,s))
!write(*,*)'rg',psinew(zxi(m,s)),zpsi(s),zxi(m,s)
!write(*,*)
        
           normt=normt+spop(s)
!write(1000,*)"spop(s)",spop(s)
          
!                call printGWPSPF(BELprint,nspfdof(m),DomainLower,DomainUpper,&
!!                     dim(m,s),psiold(zpsi(s)),psiold(zphi(m,s)),psiold(zxi(m,s)),aold(zagmat(m,s)),&
!!                     dim(m,s),psinew(zpsi(s)),psinew(zphi(m,s)),psinew(zxi(m,s)),anew(zagmat(m,s)),&
!!                     dim(m,s),psiold(zpsi(s)),aold(zphi(m,s)),aold(zxivec(m,s)),aold(zagmat(m,s)),&
!!                     dim(m,s),psinew(zpsi(s)),anew(zphi(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)),&
!                     predimred(m,s),psiold(zpsi(s)),aold(zphase(m,s)),aold(zxivec(m,s)),aold(zagmat(m,s)),&
!                     dimred(m,s),psinew(zpsi(s)),anew(zphase(m,s)),anew(zxivec(m,s)),anew(zagmat(m,s)),&
!                     xplotlims=BELplotlims,channel=iBEL)
!               call myflush(iBEL)
!             end if
          end if
!write(*,*)"m",m,"aold",agmat(zxivec(m,s)),anew(zxivec(m,s))
!write(*,*)"m",m,"aold",agmat(zagmat(m,s)),anew(zagmat(m,s))
       end do
    end do
!stop

    write(iBEL,'(A,g14.7,f12.8)') 'N',time/fs,sqrt(normt)
    call myflush(iBEL)

    dim(:,:)=dimred(:,:) !TM
    gwpdim(:,:)=dimred(:,:) !TM

!if (time>0.02) write(*,*)psinew(zpsi(1))
!if (time>0.02) TerryFlag=.true.
!if (time>0.02) stop
    call psidat

!    do s=1,nstate
!       do m=1,nmode
!write(*,*)"zpsi(s)",(zpsi(s))
!write(*,*)"zxi(m,s)","m",m,(zxi(m,s))
!write(*,*)"zphi(m,s)","m",m,(zphi(m,s))
!write(*,*)"zagmat(m,s)","m",m,(zagmat(m,s))
!write(*,*)"zxivec(m,s)","m",m,(zxivec(m,s))
!write(*,*)"zphase(m,s)","m",m,(zphase(m,s))
!       enddo
!    enddo
!    ! Modification by yapol
!    if (lgwpphase) then
!       call copyPsi(psinew,anew,zpsimax,zphimax,zximax,zxivecmax,zagmatmax,&
!            dimmax,blockmax,psi,agmat,zpsi,zphi,zxi,zxivec,zagmat,dim,block)
!    else
!       call copyPsi(psinew,anew,zpsimax,zphimax,zximax,zxivecmax,zagmatmax,&
!            dimmax,blockmax,psi,agmat,zpsi,zphase,zxi,zxivec,zagmat,dim,block)
!    endif
!    ! End of modification

!write(*,*)"second copy"
       call copyPsi(psinew,anew,zpsimax,zphimax,zphasemax,zximax,zxivecmax,zagmatmax,zetfmax,&
            dimmax,blockmax,psi,agmat,zpsi,zphi,zphase,zxi,zxivec,zagmat,zetf,dim,block) ! modified by TM
!write(*,*)""
!       call copyPsi_v2(psinew,anew,zpsimax,zphimax,zphasemax,zximax,zxivecmax,zagmatmax,&
!            dimmax,blockmax,psi,agmat,zpsi,zphi,zphase,zxi,zxivec,zagmat,dim,block) ! modified by TM
!do s=1,nstate
!do m=1,nmode
!write(*,*)"agmat(zxivec)",agmat(zxivec(m,s):zxivec(m,s)+min(dimmax(m,s),dim(m,s))*nspfdof(m)-1)
!enddo
!enddo
!write(*,*)"aold",agmat(zxivec(1,1)),anew(zxivecmax(1,1)),zxivec(1,1),zxivecmax(1,1)
!write(*,*)"aold",agmat(zagmat(1,1)),anew(zagmatmax(1,1))
!write(*,*)"aold",dble(agmat(zxivec(1,1)))/(-2.0d0*dble(agmat(zagmat(1,1)))), &
!          dble(aold(zxivecmax(1,1)))/(-2.0d0*dble(aold(zagmatmax(1,1)))), &
!          dble(anew(zxivecmax(1,1)))/(-2.0d0*dble(anew(zagmatmax(1,1))))
!write(*,*)""

    if ( psitype == 0 ) then
       routine= "resetGWP"
       message = "BEL method does not work with ML-GMCTDH"
       call errormsg
    end if

    call myflush(iBEL)
    call stop_timer(tid)
!if (time>0.0085) stop 'f'
!if (time>0.08) stop 'f'
!stop 'f'
!   write(1000,*)psi(1:dgldim)

  end subroutine resetGwp

  subroutine redblock(s)

    use griddatmod
    use psidef
    use BELglobals, only : blockred

     integer(long),intent(in) :: s
     integer(long) :: zeig,m

               zeig=1
               do m=1,nmode
                  if (gwpm(m)) then
                    zeig=zeig*dimred(m,s)
                  else
                    zeig=zeig*dim(m,s)
                  endif
               enddo
               blockred(s)=zeig
  return
  end subroutine redblock

  subroutine restoreMaxDim(psi,agmat)

    use griddatmod
    use psidef
    use iopsidef, only : psidat

    use BELglobals


    complex(dop), dimension(:), intent(inout) :: psi
    complex(dop), dimension(agmatdimmax), intent(inout)   :: agmat
!    complex(dop), dimension(agmatdim), intent(inout)   :: agmat

    ! Modification by yapol
!    if (lgwpphase) then
!       call copyPsi(psi,agmat,zpsi,zphi,zxi,zxivec,zagmat,dim,block,&
!          psinew,anew,zpsimax,zphimax,zximax,zxivecmax,zagmatmax,dimmax,blockmax)
!    else
!       call copyPsi(psi,agmat,zpsi,zphase,zxi,zxivec,zagmat,dim,block,&
!          psinew,anew,zpsimax,zphimax,zximax,zxivecmax,zagmatmax,dimmax,blockmax)
!    endif
!    ! End of modification

!write(*,*)"copyPsi in restoreMaxDim"
       call copyPsi(psi,agmat,zpsi,zphi,zphase,zxi,zxivec,zagmat,zetf,dim,block,&
          psinew,anew,zpsimax,zphimax,zphasemax,zximax,zxivecmax,zagmatmax,zetfmax,dimmax,blockmax) ! modyfied by TM

    dim(:,:)=dimmax(:,:)
    gwpdim(:,:)=gwpdimmax(:,:)

    call psidat

    agmat(1:agmatdim)=anew(1:agmatdim)
    psi(1:dgldim)=psinew(1:dgldim)

  return

  end subroutine restoreMaxDim

 subroutine restoreRedDim(psi,agmat)

    use griddatmod
    use psidef
    use iopsidef, only : psidat

    use BELglobals


    complex(dop), dimension(:), intent(inout) :: psi
    complex(dop), dimension(agmatdimmax), intent(inout)   :: agmat

    dim(:,:)=dimred(:,:) 
    gwpdim(:,:)=dimred(:,:) 

    call psidat

       call copyPsi(psinew,anew,zpsimax,zphimax,zphasemax,zximax,zxivecmax,zagmatmax,zetfmax,&
            dimmax,blockmax,psi,agmat,zpsi,zphi,zphase,zxi,zxivec,zagmat,zetf,dim,block)

 end subroutine restoreRedDim

  subroutine alloc_beldat

use global
use psidef
use griddatmod
use BELglobals


         if (.not. allocated(dimmax)) allocate(dimmax(maxdim,maxsta))
         if (.not. allocated(gwpdimmax)) allocate(gwpdimmax(maxdim,maxsta))
         if (.not. allocated(zphimax)) allocate(zphimax(maxdim,maxsta))
         if (.not. allocated(zphasemax)) allocate(zphasemax(maxdim,maxsta))
         if (.not. allocated(zximax)) allocate(zximax(maxdim,maxsta))
         if (.not. allocated(zxivecmax)) allocate(zxivecmax(maxdim,maxsta))
         if (.not. allocated(zagmatmax)) allocate(zagmatmax(maxdim,maxsta))
         if (.not. allocated(zetfmax)) allocate(zetfmax(maxdim,maxsta))
         if (.not. allocated(blockmax)) allocate(blockmax(maxsta))
         if (.not. allocated(blockred)) allocate(blockred(maxsta))
         if (.not. allocated(zpsimax)) allocate(zpsimax(maxsta))
         if (.not. allocated(predim)) allocate(predim(maxdim,maxsta)) !TM
         if (.not. allocated(predimred)) allocate(predimred(maxdim,maxsta))

         lmaxspf=maxval(dim(1:nmode,1:nstate),mask=spread(gwpm(1:nmode),2,nstate))
write(1000,*)"lmaxspf",lmaxspf
         lmaxgdof=maxval(nspfdof(1:nmode),mask=gwpm(1:nmode))

         dimmax(:,:)=dim(:,:)
write(1000,*)"dimmax(1,1)",dimmax(1,1)
         gwpdimmax(:,:)=gwpdim(:,:)
         zphimax(:,:)=zphi(:,:)
         zximax(:,:)=zxi(:,:)
         zpsimax(:)=zpsi(:)
         zxivecmax(:,:)=zxivec(:,:)
         zagmatmax(:,:)=zagmat(:,:)
         zetfmax(:,:)=zetf(:,:)
         blockmax(:)=block(:)
         zphasemax(:,:)=zphase(:,:) ! modyfied by TM

         dgldimmax=dgldim
         agmatdimmax=agmatdim

         ssdim=1
         do s=1,nstate
         do m=1,nmode
         if (gwpm(m)) then
         else
            ssdim=dim(m,s)
         endif
         enddo
         enddo
         write(1000,*)"ssdim",ssdim

         ssblock=int(blockmax(1)/ssdim)

         if (.not. allocated(psinew)) allocate(psinew(dgldimmax))
         if (.not. allocated(psiold)) allocate(psiold(dgldimmax))
         if (.not. allocated(anew)) allocate(anew(agmatdimmax))
         if (.not. allocated(aold)) allocate(aold(agmatdimmax))
         if (.not. allocated(newcoef)) allocate(newcoef(ssblock,ssdim))
         newcoef(:,:)=(0.0d0,0.0d0)
         if (.not. allocated(tauold)) allocate(tauold(lmaxgdof,lmaxgdof,lmaxspf))
         if (.not. allocated(taunew)) allocate(taunew(lmaxgdof,lmaxgdof,lmaxspf))
         if (.not. allocated(zetaold)) allocate(zetaold(lmaxgdof,lmaxspf))
         if (.not. allocated(zetanew)) allocate(zetanew(lmaxgdof,lmaxspf))
         if (.not. allocated(etaold)) allocate(etaold(lmaxspf))
         if (.not. allocated(etanew)) allocate(etanew(lmaxspf))
    return
    end subroutine alloc_beldat

subroutine int2char(num, cwrk)
integer num, j
character(len=4) cwrk

cwrk=' '
write(cwrk,'(i4)') num
do j=1,4
  if(cwrk(j:j).eq.' ') cwrk(j:j)= '0'
enddo
return
end subroutine  int2char


!----------------------------------------------------------------------------
! Calculate invercs matrix
!------------------------------------------------------------------------------
!subroutine ml_MatrixInverse(n,m,a)
!    
!
!    implicit none
!
!    integer :: n                    ! Dimensionality of the matrix
!    integer :: m                    ! Dimensionality of the matrix
!    complex(dop),dimension(m,m)::a
!    integer ::  i, icol, irow, j, k, l, ll, indxc(n), indxr(n), ipiv(n)
!    real(8) :: big
!    complex(8) :: tmp, dum, pivinv
!    
!!    do i=1,n
!!      do j=1,n
!!        write(*,*)a(i,j)
!!      enddo
!!    enddo
!
!    if (n == 1) then 
!        tmp = a(1,1) 
!        a(1,1) = (1.0d0, 0.0d0) / tmp
!    endif
!
!    if (n == 2) then
!        dum = a(1,1) * a(2,2) - a(1,2) * a(2,1)
!        tmp = a(1,1)
!        a(1,1) = a(2,2) / dum
!        a(2,2) = tmp / dum
!        a(1,2) = -a(1,2) / dum
!        a(2,1) = -a(2,1) / dum
!    endif
!
!    if (n > 2) then
!        do j = 1, n
!            ipiv(j) = 0
!        enddo
!        
!        do i = 1, n
!            big = 0.0d0
!            do j = 1, n
!                if (ipiv(j) /= 1) then
!                    do k = 1, n
!                        if (ipiv(k) == 0) then
!                            if ( cdabs(a(j, k)) >= big) then
!                                big = cdabs( a(j,k) )
!                                irow = j
!                                icol = k
!                            endif
!                        endif
!                    enddo
!                endif
!            enddo
!            
!            ipiv(icol) = ipiv(icol) + 1
!            
!            if (irow /= icol) then
!                do l = 1, n
!                    dum = a(irow, l)
!                    a(irow, l) = a(icol, l)
!                    a(icol, l) = dum
!                enddo
!            endif
!            
!            indxr(i) = irow
!            indxc(i) = icol
!            
!            if ( cdabs( a(icol,icol) ) == 0.0d0 ) then
!                open(9998, file = 'error.txt')
!                write(9998,*) 'singular matrix in ml_MatrixInverse'
!                close(9998)
!                stop
!            endif
!
!            pivinv = (1.0d0, 0.0d0) / a(icol, icol)
!            a(icol, icol) = (1.0d0, 0.0d0)
!            
!            do l = 1, n
!                a(icol, l) = a(icol, l) * pivinv
!            enddo
!            
!            do ll = 1, n
!                if (ll /= icol) then
!                    dum = a(ll, icol)
!                    a(ll, icol) = (0.0d0, 0.0d0)
!                    do l = 1, n
!                        a(ll, l) = a(ll, l) - a(icol, l) * dum
!                    enddo
!                endif
!            enddo
!            
!        enddo
!
!        do l = n, 1, -1
!            if ( indxr(l) /= indxc(l) ) then
!                do k = 1, n
!                    dum = a(k, indxr(l))
!                    a(k, indxr(l)) = a(k, indxc(l))
!                    a(k, indxc(l)) = dum
!                enddo
!            endif
!        enddo
!        
!    endif
!    
!    return
!end subroutine ml_MatrixInverse

end module BEL
