!   -*- f90 -*-




subroutine zundel(coord,v)
  implicit None
  integer, parameter           :: ndof=15
  double precision, intent(IN) :: coord(ndof)
  double precision, intent(OUT):: v

!  double precision             :: bohr_in_ang
  double precision             :: cart(7,3)
  double precision             :: coord1(0:ndof-1)

  coord1(0:ndof-1) = coord(1:ndof)
  call q2x(coord1,cart)
  call calcpot_zundel(V,cart)

  ! some default value when we get negative
  if(V .lt. 0.d0) then
     V = 2.48d0/27.21138386d0
  end if

end subroutine zundel


subroutine dipzundel(coord,dip)
  implicit None
  integer, parameter           :: ndof=15
  double precision, intent(IN) :: coord(ndof)
  double precision, intent(OUT):: dip

  double precision             :: dip3(3)
!  double precision             :: bohr_in_ang
  double precision             :: cart(7,3)
  double precision             :: coord1(0:ndof-1)

  integer                      :: dip_comp
  common /dipolecomponent/ dip_comp


  coord1(0:ndof-1) = coord(1:ndof)
  call q2x(coord1,cart)
  call calcdip_zundel(dip3,cart)

  dip = dip3(dip_comp)

end subroutine dipzundel




!********************************************************************************
!------------Note by Xinchuan Huang on 2004-02-16----------------
! Potential and dipole moment surface for H5O2+
! version 4B, capable of H2O...H3O+ dissociation
!
!  Fitting codes were written by Bastiaan J. Bramms on 2004-01-03, 2004-01-04 and 2004-01-09
!  Codes were modified and applied to fit by Xinchuan Huang    ---- Finished on 2004-01-14
!
!  pgf90 & ifc compatible
!  Two data file are required :
!    h5o2.dms4B.coeff.com.dat  h5o2.pes4B.coeff.dat
!
!  The C2-symmetry minimum geometry on PES-4B in the order of O O H H H H H is: (in bohr)
!          X                  Y                 Z
! O  2.254868053895938 0.000000000000000 0.000000000000000
! O -2.254868053895938 0.000000000000000 0.000000000000000
! H  0.000000000000000 0.000000000000000 0.1235103451781876
! H -3.013194720920241 1.489183788836780 -0.7465980020137635
! H -3.201587564645578 -0.4544810799466731 1.498136372134866
! H  3.013194720920241 -1.489183788836780 -0.7465980020137635
! H  3.201587564645578 0.4544810799466731 1.498136372134866
!
!  prepot_zundel() should be called before first calling calcpot_zundel(V,xx)
!    xx(7,3) is cartesian array containing O O H H H H H coor, in bohr
!    returned V is potential in hartree, taking C2-sym minimum as zero potential reference
!
!  predip() should be calld before first calling calcdip_zundel(dip,xx)
!    xx(7,3) is cartesian array containing O O H H H H H coor, in bohr
!    returned dip(1),dip(2),dip(3) are the x-, y-, z- dipole components
!              relative to the center-of-mass of H5O2+, in a.u.
!
!  For D5O2+ calculations, you may need to change calcdip_zundel() to user D5O2+ center-of-mass frame
!
!
!  Permanent contact:  Joel M. Bowman    E-mail: bowman@euch4e.chem.emory.edu
!------------End of Note----------------------------------------------
!********************************************************************************


SUBROUTINE q2x(qq,xm)
  IMPLICIT NONE
  real (kind=8), intent(in)      :: qq(0:14)
  real (kind=8), intent(out)     :: xm(0:6,0:2)
  real (kind=8)                  :: q(0:14)
  real (kind=8), dimension (0:2) :: xHp,xOa,xH1a
  real (kind=8), dimension (0:2) :: xH2a,xCMa,xOb,xH1b,xH2b,xCMb
  real (kind=8)                  :: R(0:2,0:2)

  logical                   :: zz,rval
  real (kind=8)             :: dmin,kmin,mh,mo,mwat
  common/trans/zz,rval,dmin,kmin,mh,mo,mwat

!   Coordinate transformation, from internal to cartesian

!        Internals are given in the following order:

!        x:        q(0)
!        y:        q(1)
!        z:        q(2)

!        R:        q(3)
!        alpha:    q(4)

!        R1_a:     q(5)
!        R2_a:     q(6)
!        theta_a:  q(7)

!        beta_a:   q(8)
!        lambda_a: q(9)

!        R1_b:     q(10)
!        R2_b:     q(11)
!        theta_b:  q(12)

!        beta_b:   q(13)
!        lambda_b: q(14)
!----------------------------------------------------------

  !   # We do not want to mess with the input vector qq
  q(:)=qq(:)
  !   # Transform angles given as cosines back to radians
  q(7)  = acos(q(7))
  q(8)  = acos(q(8))
  q(12) = acos(q(12))
  q(13) = acos(q(13))
  !     Transform z -> z/(R-2d)
  if (zz) then
     q(2) = q(2)*(q(3)-kmin)
  end if
  !   # Initialize empty array for the final cartesian coordinates
  xm = 0.d0
  !   # Initialize xyz arrays for the cartesians of each atom
  xHp = 0.d0
  xOa = 0.d0
  xH1a= 0.d0
  xH2a= 0.d0
  xCMa= 0.d0
  xOb = 0.d0
  xH1b= 0.d0
  xH2b= 0.d0
  xCMb= 0.d0
  !   # Central proton in BFA
  xHp(0) = q(0)
  xHp(1) = q(1)
  xHp(2) = q(2)
  !     CALL eulerMatrix(R,-0.5d0*q(4),0.d0,0.d0)
  !     xHp = matmul(R,xHp)
  !   # Wat A
  !   # Set watA on the XZ_BFA plane, R2 parallel to Z_BFA
  !   # and calculate the position of the oxygen in relation
  !   # to the R2 vector
  xH1a(2) = 0.5d0*q(6)
  xH2a(2) =-0.5d0*q(6)
  xOa(0) = sin(q(7))*q(5)
  xOa(2) = cos(q(7))*q(5)
  !   # Shift atoms so that the oxygen lies at (0,0,0)_BFA
  if (rval) then
     xCMa = xOa
  else
     xCMa = (mo*xOa + mh*xH1a + mh*xH2a)/(mwat)
  end if
  xOa = xOa - xCMa
  xH1a = xH1a - xCMa
  xH2a = xH2a - xCMa
  !   # Apply Euler rotations
  CALL eulerMatrix(R,0.d0,q(8),q(9))
  xOa = matmul(R,xOa)
  xH1a = matmul(R,xH1a)
  xH2a = matmul(R,xH2a)
  !   # Displace watA by 0.5*R to the left
  xOa(2)  = xOa(2)  - 0.5d0*q(3)
  xH1a(2) = xH1a(2) - 0.5d0*q(3)
  xH2a(2) = xH2a(2) - 0.5d0*q(3)
  !   # Wat B
  !   # Set watB on the XZ_BFB plane, R2 parallel to Z_BFB
  !   # and calculate the position of the oxygen in relation
  !   # to the R2 vector
  xH1b(2) = 0.5d0*q(11)
  xH2b(2) =-0.5d0*q(11)
  xOb(0) = sin(q(12))*q(10)
  xOb(2) = cos(q(12))*q(10)
  !   # Shift atoms so that the oxygen lies at (0,0,0)_BFB
  if (rval) then
     xCMb = xOb
  else
     xCMb = (mo*xOb + mh*xH1b + mh*xH2b)/(mwat)
  end if
  xOb = xOb - xCMb
  xH1b = xH1b - xCMb
  xH2b = xH2b - xCMb
  !   # Apply Euler rotations
  CALL eulerMatrix(R,q(4),q(13),q(14))
  xOb = matmul(R,xOb)
  xH1b = matmul(R,xH1b)
  xH2b = matmul(R,xH2b)
  !   # Displace watB by 0.5*R to the right
  xOb(2)  = xOb(2)  + 0.5d0*q(3)
  xH1b(2) = xH1b(2) + 0.5d0*q(3)
  xH2b(2) = xH2b(2) + 0.5d0*q(3)
  !   # Set the total x vector
  xm(0,:)  = xOa
  xm(1,:)  = xOb
  xm(2,:)  = xHp
  xm(3,:)  = xH1b
  xm(4,:)  = xH2b
  xm(5,:)  = xH1a
  xm(6,:)  = xH2a
  return
END SUBROUTINE q2x

SUBROUTINE eulerMatrix(r,a,b,c)
  IMPLICIT NONE
  real (kind=8), intent(in)  :: a,b,c
  real (kind=8), intent(out) :: r(0:2,0:2)
  real (kind=8) :: Am(0:2,0:2)
  real (kind=8) :: Bm(0:2,0:2)
  real (kind=8) :: Cm(0:2,0:2)

  Am = 0.d0
  Bm = 0.d0
  Cm = 0.d0

  Am(0,0) = cos(a)
  Am(1,1) = cos(a)
  Am(2,2) = 1.0d0
  Bm(0,0) = cos(b)
  Bm(1,1) = 1.0d0
  Bm(2,2) = cos(b)
  Cm(0,0) = cos(c)
  Cm(1,1) = cos(c)
  Cm(2,2) = 1.0d0
  Am(0,1) = -sin(a)
  Am(1,0) = sin(a)
  Bm(0,2) = sin(b)
  Bm(2,0) = -sin(b)
  Cm(0,1) = -sin(c)
  Cm(1,0) = sin(c)
  r=matmul(matmul(Am,Bm),Cm)
  return
END SUBROUTINE eulerMatrix




subroutine h5o2_init(zlog,rlog,m,d,ilog,mpirank)
  !        zlog : true if transform z -> z/(R-2d)
  !        zlog : true if R is valence coordinate
  !        d    : minimium distance of the proton to edges of R
  !        m    : mass of the external hydrogens

  implicit none

  logical, intent(in)       :: zlog
  logical, intent(in)       :: rlog
  real (kind=8), intent(in) :: d
  real (kind=8), intent(in) :: m
  logical                   :: zz,rval
  real (kind=8)             :: dmin,kmin,mh,mo,mwat
  logical, save             :: ZundelInitialized = .false.
  integer,intent(in) :: ilog,mpirank

  common/trans/zz,rval,dmin,kmin,mh,mo,mwat

  ! only one thread does the initialization:
  !$omp critical

  if (.not. ZundelInitialized) then

     zz   = zlog
     rval = rlog
     dmin = d
     mh   = m

     kmin = 2.d0*dmin
     mo = 16.d0
     mwat = mo + 2.d0*mh
     ! call prepot_zundel
     call prepot1
     !call predip
     call predip1


     if (mpirank.eq.0) then
        write(ilog,*) " "
        write(ilog,*) "*** H5O2+ Potential Initialized ***"
        if (zz) then
           write(ilog,*) "   - Z is transformed to Z/(R-2d)"
        end if
        if (rval) then
           write(ilog,*) "   - R is Valence vector"
        else
           write(ilog,*) "   - R is Jacobi vector"
        end if
        write(ilog,*) "   -         Dmin= ",dmin
        write(ilog,*) "   - Proton mass = ",mh
        write(ilog,*) " "
        write(ilog,*) " "
     end if

     ZundelInitialized = .true.

  end if
  !$omp end critical


  return
end subroutine h5o2_init


subroutine h5o2_set_dip_component(comp,ilog)
  implicit none
  integer, intent(IN) :: comp
  integer             :: dip_comp
  integer,intent(in) :: ilog

  common /dipolecomponent/ dip_comp

  !$omp critical

  if (comp .gt. 3 .or. comp .lt. 1) then
     write(ilog,'(a)') "Routine: set_dip_component (zundel.F90)"
     write(ilog,'(a,i4)') &
         'dipole componetnt must be one of (1,2,3)==("x","y","z"), not: ', comp
     stop
  end if

  dip_comp = comp

  !$omp end critical

end subroutine h5o2_set_dip_component


!!$subroutine prepot_zundel()
!!$
!!$  implicit double precision (a-h,o-z)
!!$  implicit integer (i-n)
!!$
!!$  double precision V, cart_in(7,3), v0
!!$  double precision dc0(0:6,0:6),dw0(0:6,0:6), coef(0:7961)
!!$
!!$  integer i,j,k,i1,j1,k1
!!$
!!$  common/NCOE/ms,mr
!!$  common/h5o2coef/dc0,dw0,coef
!!$
!!$  ms=7785 ; mr=59
!!$
!!$  open(20,file='h5o2.pes4B.coeff.dat',status='old')
!!$  read(20,*)
!!$  read(20,*)dc0
!!$  read(20,*)
!!$  read(20,*)dw0
!!$  read(20,*)
!!$  read(20,*)
!!$  read(20,*)(coef(i1),i1=0,ms+3*mr-1)
!!$  close(20)
!!$
!!$  return
!!$end subroutine prepot_zundel

!***************************************************************

subroutine calcpot_zundel(V,cart_in)

  implicit none

  integer ms,mr
  double precision dc0(0:6,0:6),dw0(0:6,0:6),coef(0:7961)

  common/NCOE/ms,mr
  common/h5o2coef/dc0,dw0,coef
  double precision V, cart_in(7,3)

  integer j


  double precision vec(0:ms+3*mr-1)
  double precision xnuc(0:2,0:6)

  do j=1,3
     xnuc(j-1,5:6)=cart_in(1:2,j)
     xnuc(j-1,0:4)=cart_in(3:7,j)
  end do

  call getvec_zundel (ms, mr, xnuc(0:2,0:6), dc0, dw0, vec)

  V = dot_product(coef,vec)

! set C1 min as potential zero-point   ---- PES-4B
  V = V + 153.012245695813d0

! set C2 linear SP as potential zero-point  --- PES-4B  (OH not equal, 7 variable)
!        V = V + 153.0119260317857d0

  return
end subroutine calcpot_zundel

!********************************************************
subroutine getvec_zundel (ms, mr, xn, dc0, dw0, vec)
  implicit none
  integer, parameter :: wp=selected_real_kind(12,300)
! version for X5Y2
  integer nk, ms, mr
  parameter (nk=7)
  double precision  xn(0:2,0:nk-1), dc0(0:nk-1,0:nk-1),dw0(0:nk-1,0:nk-1), vec(0:ms+3*mr-1)
  integer k, l
  double precision  rvec(0:3),d0(0:nk-1,0:nk-1),r0(0:nk-1,0:nk-1)
!-----------------------------------------------------------------------
  vec=0
  call getr0_zundel (nk, xn, r0)
  call getd0_zundel (nk, r0, dc0, dw0, d0)
  call getv_x5y2 (ms, d0, vec(0:ms-1))
  call getrvec_zundel (4, r0, rvec)
  do l=0, mr-1
     do k=0, 2
        vec(ms+3*l+k)=rvec(k+1)*vec(l)
     enddo
  enddo
  return
end subroutine getvec_zundel


subroutine getv_x5y2 (m, d, vec)
  implicit none
  integer, parameter :: wp=selected_real_kind(12,300)
  integer, parameter :: nk=7, mxd=12
  integer m
  double precision  d(0:nk-1,0:nk-1), vec(0:m-1)
! version for molecule X5Y2.
! MolienSeries(0:*): 1 3 12 43 159 554 1879 6066 18755 ...
! #Primaries(1:*):   3 5 3 4 3 2 0 0 0 1
! #Secondaries(1:*): 0 1 12 39 113 338 932 2402 ...
  integer, parameter :: l0=1, l1=l0+3, l2=l1+12, l3=l2+43, l4=l3+159,l5=l4+554, l6=l5+1879, l7=l6+6066-932
!! We haven't incorporated the secondaries at degree 7
  integer, parameter :: np1=3, np2=5, np3=3, np4=4, np5=3, np6=2, np7=0
  integer, parameter :: ns1=0, ns2=1, ns3=12, ns4=39, ns5=113, ns6=338, ns7=932
  double precision  x(0:np1-1), y(0:np2-1),z(0:np3-1),u(0:np4-1), v(0:np5-1), w(0:np6-1)
  double precision  x2(0:np1-1), x3(0:np1-1), x4(0:np1-1), &
       x5(0:np1-1),x6(0:np1-1), x7(0:np1-1), y2(0:np2-1), y3(0:np2-1), &
       z2(0:np2-1), ys(0:ns2-1), zs(0:ns3-1), us(0:ns4-1), &
       vs(0:ns5-1), ws(0:ns6-1), w7s(0:ns7-1)

  integer mdeg, i, j, i0, i1, i2, i3, i4, j0, j1
  double precision  d2(0:nk-1,0:nk-1), d3(0:nk-1,0:nk-1), &
       d4(0:nk-1,0:nk-1), d5(0:nk-1,0:nk-1), d6(0:nk-1,0:nk-1), &
       d7(0:nk-1,0:nk-1)
  double precision  t0, di0j1
  double precision  her2_zundel, her3_zundel, her4_zundel, her5_zundel, her6_zundel, her7_zundel
!        her2_zundel(t0)=(4*t0**2-2)/dsqrt(dble(8*2))
!        her3_zundel(t0)=(8*t0**2-12)*t0/dsqrt(dble(16*6))
!        her4_zundel(t0)=((16*t0**2-48)*t0**2+12)/dsqrt(dble(32*24))
!        her5_zundel(t0)=((32*t0**2-160)*t0**2+120)*t0/dsqrt(dble(64*120))
!        her6_zundel(t0)=(((64*t0**2-480)*t0**2+720)*t0**2-120)/
!     $     dsqrt(dble(128*720))
!        her7_zundel(t0)=(((128*t0**2-1344)*t0**2+3360)*t0**2-1680)*t0/
!     $     dsqrt(dble(256*5040))
!-----------------------------------------------------------------------
! Test for compatibility, set mdeg
  select case (m)
  case (l0)
     mdeg=0
  case (l1)
     mdeg=1
  case (l2)
     mdeg=2
  case (l3)
     mdeg=3
  case (l4)
     mdeg=4
  case (l5)
     mdeg=5
  case (l6)
     mdeg=6
  case (l7)
     mdeg=7
  case default
     stop 'getv - wrong dimension'
  endselect
! auxiliary distances
  do i=0, nk-1
     do j=i+1, nk-1
        d2(i,j)=her2_zundel(d(i,j))
        d2(j,i)=d2(i,j)
        d3(i,j)=her3_zundel(d(i,j))
        d3(j,i)=d3(i,j)
        d4(i,j)=her4_zundel(d(i,j))
        d4(j,i)=d4(i,j)
        d5(i,j)=her5_zundel(d(i,j))
        d5(j,i)=d5(i,j)
        d6(i,j)=her6_zundel(d(i,j))
        d6(j,i)=d6(i,j)
        d7(i,j)=her7_zundel(d(i,j))
        d7(j,i)=d7(i,j)
     enddo
  enddo
  ! Primary Invariants
  x=0 ; y=0 ; z=0 ; u=0 ; v=0 ; w=0
  do i0=0, 4
     t0=0
     do i1=0, 4
        if (i1.ne.i0) then
           t0=t0+d(i0,i1)/4
           x(0)=x(0)+d(i0,i1)/20
           y(0)=y(0)+d2(i0,i1)/20
           z(0)=z(0)+d3(i0,i1)/20
           u(0)=u(0)+d4(i0,i1)/20
           v(0)=v(0)+d5(i0,i1)/20
           w(0)=w(0)+d6(i0,i1)/20
        endif
     enddo
     y(1)=y(1)+her2_zundel(t0)/5
     z(1)=z(1)+her3_zundel(t0)/5
     u(1)=u(1)+her4_zundel(t0)/5
     v(1)=v(1)+her5_zundel(t0)/5
  enddo
  x(1)=sum(d(0:4,5:6))/10
  do i0=0, 4
     y(2)=y(2)+her2_zundel(sum(d(i0,5:6))/2)/5
     z(2)=z(2)+her3_zundel(sum(d(i0,5:6))/2)/5
     u(2)=u(2)+her4_zundel(sum(d(i0,5:6))/2)/5
     v(2)=v(2)+her5_zundel(sum(d(i0,5:6))/2)/5
  enddo
  do j0=5, 6
     y(3)=y(3)+her2_zundel(sum(d(0:4,j0))/5)/2
  enddo
  do i0=0, 4
     y(4)=y(4)+sum(d2(i0,5:6))/10
     u(3)=u(3)+sum(d4(i0,5:6))/10
     w(1)=w(1)+sum(d6(i0,5:6))/10
  enddo
  x(2)=d(5,6)
  ! Required powers
  do i=0, np1-1
     x2(i)=her2_zundel(x(i))
     x3(i)=her3_zundel(x(i))
     x4(i)=her4_zundel(x(i))
     x5(i)=her5_zundel(x(i))
     x6(i)=her6_zundel(x(i))
     x7(i)=her7_zundel(x(i))
  enddo
  do i=0, np2-1
     y2(i)=her2_zundel(y(i))
     y3(i)=her3_zundel(y(i))
  enddo
  do i=0, np3-1
     z2(i)=her2_zundel(z(i))
  enddo
  ! Secondary Invariants
  !! ys(0:0), zs(0:11), us(0:38), vs(0:112), ws(0:337), w7s(0:931)
  !! reducible: us(0:0), vs(0:11), ws(0:113), w7s(0:541)
  ys=0 ; zs=0 ; us=0 ; vs=0 ; ws=0 ; w7s=0
  ! Irreducible secondaries
  do i0=0, 4
     do i1=0, 4
        if (i1.ne.i0) then
           if (3.le.mdeg) then
              do i2=0, 4
                 if (i2.ne.i1.and.i2.ne.i0) then
                    zs(0)=zs(0)+d2(i0,i1)*d(i0,i2)/60
                    zs(1)=zs(1)+d(i0,i1)*d(i0,i2)*d(i1,i2)/60
                    us(1)=us(1)+d3(i0,i1)*d(i0,i2)/60
                    us(2)=us(2)+d2(i0,i1)*d2(i0,i2)/60
                    us(3)=us(3)+d2(i0,i1)*d(i0,i2)*d(i1,i2)/60
                    if (5.le.mdeg) then
                       vs(12)=vs(12)+d4(i0,i1)*d(i0,i2)/60
                       vs(13)=vs(13)+d3(i0,i1)*d2(i0,i2)/60
                       vs(14)=vs(14)+d3(i0,i1)*d(i0,i2)*d(i1,i2)/60
                       vs(15)=vs(15)+d2(i0,i1)*d2(i0,i2)*d(i1,i2)/60
                    endif
                    if (6.le.mdeg) then
                       ws(114)=ws(114)+d5(i0,i1)*d(i0,i2)/60
                       ws(115)=ws(115)+d4(i0,i1)*d2(i0,i2)/60
                       ws(116)=ws(116)+d3(i0,i1)*d3(i0,i2)/60
                       ws(117)=ws(117)+d4(i0,i1)*d(i0,i2)*d(i1,i2)/60
                       ws(118)=ws(118)+d3(i0,i1)*d2(i0,i2)*d(i1,i2)/60
                       ws(119)=ws(119)+d2(i0,i1)*d2(i0,i2)*d2(i1,i2)/60
                    endif
                    do i3=0, 4
                       if (i3.ne.i0.and.i3.ne.i1.and.i3.ne.i2) then
                          us(4)=us(4)+d2(i0,i1)*d(i0,i2)*d(i0,i3)/120
                          us(5)=us(5)+d2(i0,i1)*d(i1,i2)*d(i0,i3)/120
                          if (5.le.mdeg) then
                             vs(16)=vs(16)+d3(i0,i1)*d(i0,i2)*d(i0,i3)/120
                             vs(17)=vs(17)+d2(i0,i1)*d2(i0,i2)*d(i0,i3)/120
                             vs(18)=vs(18)+d3(i0,i1)*d(i1,i2)*d(i0,i3)/120
                             vs(19)=vs(19)+d2(i0,i1)*d(i0,i2)*d(i1,i2)*d(i0,i3)/120
                          endif
                          if (6.le.mdeg) then
                             ws(120)=ws(120)+d4(i0,i1)*d(i0,i2)*d(i0,i3)/120
                             ws(121)=ws(121)+d3(i0,i1)*d2(i0,i2)*d(i0,i3)/120
                             ws(122)=ws(122)+d4(i0,i1)*d(i1,i2)*d(i0,i3)/120
                             ws(123)=ws(123)+d3(i0,i1)*d(i0,i2)*d(i1,i2)*d(i0,i3)/120
                             ws(124)=ws(124)+d2(i0,i1)*d2(i0,i2)*d(i1,i2)*d(i0,i3)/120
                             ws(125)=ws(125)+d3(i0,i1)*d2(i1,i2)*d(i0,i3)/120
                          endif
                       endif
                    enddo
                 endif
              enddo
           endif
        endif
     enddo
  enddo
  do j0=5, 6
     do i0=0, 4
        zs(2)=zs(2)+d3(i0,j0)/10
        vs(20)=vs(20)+d5(i0,j0)/10
        do i1=0, 4
           if (i1.ne.i0) then
              ys(0)=ys(0)+d(i0,i1)*d(i0,j0)/40
              if (3.le.mdeg) then
                 zs(3)=zs(3)+d2(i0,i1)*d(i0,j0)/40
                 zs(4)=zs(4)+d(i0,i1)*d2(i0,j0)/40
                 zs(5)=zs(5)+d(i0,i1)*d(i0,j0)*d(i1,j0)/40
                 zs(6)=zs(6)+d2(i0,j0)*d(i1,j0)/40
                 us(6)=us(6)+d3(i0,i1)*d(i0,j0)/40
                 us(7)=us(7)+d2(i0,i1)*d2(i0,j0)/40
                 us(8)=us(8)+d(i0,i1)*d3(i0,j0)/40
                 us(9)=us(9)+d3(i0,j0)*d(i1,j0)/40
                 us(10)=us(10)+d2(i0,j0)*d2(i1,j0)/40
                 us(11)=us(11)+d2(i0,i1)*d(i0,j0)*d(i1,j0)/40
                 us(12)=us(12)+d(i0,i1)*d2(i0,j0)*d(i1,j0)/40
                 if (5.le.mdeg) then
                    vs(21)=vs(21)+d4(i0,i1)*d(i0,j0)/40
                    vs(22)=vs(22)+d3(i0,i1)*d2(i0,j0)/40
                    vs(23)=vs(23)+d2(i0,i1)*d3(i0,j0)/40
                    vs(24)=vs(24)+d(i0,i1)*d4(i0,j0)/40
                    vs(25)=vs(25)+d3(i0,i1)*d(i0,j0)*d(i1,j0)/40
                    vs(26)=vs(26)+d2(i0,i1)*d2(i0,j0)*d(i1,j0)/40
                    vs(27)=vs(27)+d(i0,i1)*d3(i0,j0)*d(i1,j0)/40
                    vs(28)=vs(28)+d4(i0,j0)*d(i1,j0)/40
                    vs(29)=vs(29)+d(i0,i1)*d2(i0,j0)*d2(i1,j0)/40
                    vs(30)=vs(30)+d3(i0,j0)*d2(i1,j0)/40
                 endif
                 if (6.le.mdeg) then
                    ws(126)=ws(126)+d5(i0,i1)*d(i0,j0)/40
                    ws(127)=ws(127)+d4(i0,i1)*d2(i0,j0)/40
                    ws(128)=ws(128)+d3(i0,i1)*d3(i0,j0)/40
                    ws(129)=ws(129)+d2(i0,i1)*d4(i0,j0)/40
                    ws(130)=ws(130)+d(i0,i1)*d5(i0,j0)/40
                    ws(131)=ws(131)+d4(i0,i1)*d(i0,j0)*d(i1,j0)/40
                    ws(132)=ws(132)+d3(i0,i1)*d2(i0,j0)*d(i1,j0)/40
                    ws(133)=ws(133)+d2(i0,i1)*d3(i0,j0)*d(i1,j0)/40
                    ws(134)=ws(134)+d(i0,i1)*d4(i0,j0)*d(i1,j0)/40
                    ws(135)=ws(135)+d5(i0,j0)*d(i1,j0)/40
                    ws(136)=ws(136)+d2(i0,i1)*d2(i0,j0)*d2(i1,j0)/40
                    ws(137)=ws(137)+d4(i0,j0)*d2(i1,j0)/40
                    ws(138)=ws(138)+d(i0,i1)*d3(i0,j0)*d2(i1,j0)/40
                 endif
                 do i2=0, 4
                    if (i2.ne.i1.and.i2.ne.i0) then
                       zs(7)=zs(7)+d(i0,i1)*d(i0,i2)*d(i0,j0)/120
                       zs(8)=zs(8)+d(i0,i1)*d(i1,i2)*d(i0,j0)/120
                       zs(9)=zs(9)+d(i0,i2)*d(i0,j0)*d(i1,j0)/120
                       us(13)=us(13)+d2(i0,i1)*d(i0,i2)*d(i0,j0)/120
                       us(14)=us(14)+d2(i0,i1)*d(i1,i2)*d(i0,j0)/120
                       us(15)=us(15)+d(i0,i1)*d(i0,i2)*d(i1,i2)*d(i0,j0)/120
                       us(16)=us(16)+d(i0,i1)*d2(i1,i2)*d(i0,j0)/120
                       us(17)=us(17)+d(i0,i1)*d(i0,i2)*d2(i0,j0)/120
                       us(18)=us(18)+d(i0,i1)*d(i1,i2)*d2(i0,j0)/120
                       us(19)=us(19)+d(i0,i1)*d(i0,i2)*d(i0,j0)*d(i1,j0)/120
                       us(20)=us(20)+d2(i0,i2)*d(i0,j0)*d(i1,j0)/120
                       us(21)=us(21)+d(i0,i2)*d(i1,i2)*d(i0,j0)*d(i1,j0)/120
                       us(22)=us(22)+d(i0,i2)*d2(i0,j0)*d(i1,j0)/120
                       us(23)=us(23)+d(i1,i2)*d2(i0,j0)*d(i1,j0)/120
                       us(24)=us(24)+d(i0,i1)*d(i0,j0)*d(i1,j0)*d(i2,j0)/120
                       if (5.le.mdeg) then
                          vs(31)=vs(31)+d3(i0,i1)*d(i0,i2)*d(i0,j0)/120
                          vs(32)=vs(32)+d2(i0,i1)*d2(i0,i2)*d(i0,j0)/120
                          vs(33)=vs(33)+d3(i0,i1)*d(i1,i2)*d(i0,j0)/120
                          vs(34)=vs(34)+d2(i0,i1)*d(i0,i2)*d(i1,i2)*d(i0,j0)/120
                          vs(35)=vs(35)+d2(i0,i1)*d2(i1,i2)*d(i0,j0)/120
                          vs(36)=vs(36)+d(i0,i1)*d(i0,i2)*d2(i1,i2)*d(i0,j0)/120
                          vs(37)=vs(37)+d(i0,i1)*d3(i1,i2)*d(i0,j0)/120
                          vs(38)=vs(38)+d2(i0,i1)*d(i0,i2)*d2(i0,j0)/120
                          vs(39)=vs(39)+d2(i0,i1)*d(i1,i2)*d2(i0,j0)/120
                          vs(40)=vs(40)+d(i0,i1)*d(i0,i2)*d(i1,i2)*d2(i0,j0)/120
                          vs(41)=vs(41)+d(i0,i1)*d2(i1,i2)*d2(i0,j0)/120
                          vs(42)=vs(42)+d(i0,i1)*d(i0,i2)*d3(i0,j0)/120
                          vs(43)=vs(43)+d(i0,i1)*d(i1,i2)*d3(i0,j0)/120
                          vs(44)=vs(44)+d2(i0,i1)*d(i0,i2)*d(i0,j0)*d(i1,j0)/120
                          vs(45)=vs(45)+d(i0,i1)*d2(i0,i2)*d(i0,j0)*d(i1,j0)/120
                          vs(46)=vs(46)+d3(i0,i2)*d(i0,j0)*d(i1,j0)/120
                          vs(47)=vs(47)+d(i0,i1)*d(i0,i2)*d(i1,i2)*d(i0,j0)*d(i1,j0)/120
                          vs(48)=vs(48)+d2(i0,i2)*d(i1,i2)*d(i0,j0)*d(i1,j0)/120
                          vs(49)=vs(49)+d(i0,i1)*d(i0,i2)*d2(i0,j0)*d(i1,j0)/120
                          vs(50)=vs(50)+d2(i0,i2)*d2(i0,j0)*d(i1,j0)/120
                          vs(51)=vs(51)+d(i0,i1)*d(i1,i2)*d2(i0,j0)*d(i1,j0)/120
                          vs(52)=vs(52)+d(i0,i2)*d(i1,i2)*d2(i0,j0)*d(i1,j0)/120
                          vs(53)=vs(53)+d2(i1,i2)*d2(i0,j0)*d(i1,j0)/120
                          vs(54)=vs(54)+d(i0,i2)*d3(i0,j0)*d(i1,j0)/120
                          vs(55)=vs(55)+d(i1,i2)*d3(i0,j0)*d(i1,j0)/120
                          vs(56)=vs(56)+d(i0,i2)*d2(i0,j0)*d2(i1,j0)/120
                          vs(57)=vs(57)+d2(i0,i1)*d(i0,j0)*d(i1,j0)*d(i2,j0)/120
                          vs(58)=vs(58)+d(i0,i1)*d(i0,i2)*d(i0,j0)*d(i1,j0)*d(i2,j0)/120
                          vs(59)=vs(59)+d(i0,i1)*d2(i0,j0)*d(i1,j0)*d(i2,j0)/120
                       endif
                       if (6.le.mdeg) then
                          ws(139)=ws(139)+d4(i0,i1)*d(i0,i2)*d(i0,j0)/120
                          ws(140)=ws(140)+d3(i0,i1)*d2(i0,i2)*d(i0,j0)/120
                          ws(141)=ws(141)+d4(i0,i1)*d(i1,i2)*d(i0,j0)/120
                          ws(142)=ws(142)+d3(i0,i1)*d(i0,i2)*d(i1,i2)*d(i0,j0)/120
                          ws(143)=ws(143)+d2(i0,i1)*d2(i0,i2)*d(i1,i2)*d(i0,j0)/120
                          ws(144)=ws(144)+d3(i0,i1)*d2(i1,i2)*d(i0,j0)/120
                          ws(145)=ws(145)+d2(i0,i1)*d(i0,i2)*d2(i1,i2)*d(i0,j0)/120
                          ws(146)=ws(146)+d2(i0,i1)*d3(i1,i2)*d(i0,j0)/120
                          ws(147)=ws(147)+d(i0,i1)*d(i0,i2)*d3(i1,i2)*d(i0,j0)/120
                          ws(148)=ws(148)+d(i0,i1)*d4(i1,i2)*d(i0,j0)/120
                          ws(149)=ws(149)+d3(i0,i1)*d(i0,i2)*d2(i0,j0)/120
                          ws(150)=ws(150)+d2(i0,i1)*d2(i0,i2)*d2(i0,j0)/120
                          ws(151)=ws(151)+d3(i0,i1)*d(i1,i2)*d2(i0,j0)/120
                          ws(152)=ws(152)+d2(i0,i1)*d(i0,i2)*d(i1,i2)*d2(i0,j0)/120
                          ws(153)=ws(153)+d2(i0,i1)*d2(i1,i2)*d2(i0,j0)/120
                          ws(154)=ws(154)+d(i0,i1)*d(i0,i2)*d2(i1,i2)*d2(i0,j0)/120
                          ws(155)=ws(155)+d(i0,i1)*d3(i1,i2)*d2(i0,j0)/120
                          ws(156)=ws(156)+d2(i0,i1)*d(i0,i2)*d3(i0,j0)/120
                          ws(157)=ws(157)+d2(i0,i1)*d(i1,i2)*d3(i0,j0)/120
                          ws(156)=ws(156)+d(i0,i1)*d(i0,i2)*d(i1,i2)*d3(i0,j0)/120
                          ws(159)=ws(159)+d(i0,i1)*d2(i1,i2)*d3(i0,j0)/120
                          ws(160)=ws(160)+d(i0,i1)*d(i0,i2)*d4(i0,j0)/120
                          ws(161)=ws(161)+d(i0,i1)*d(i1,i2)*d4(i0,j0)/120
                          ws(162)=ws(162)+d3(i0,i1)*d(i0,i2)*d(i0,j0)*d(i1,j0)/120
                          ws(163)=ws(163)+d2(i0,i1)*d2(i0,i2)*d(i0,j0)*d(i1,j0)/120
                          ws(164)=ws(164)+d(i0,i1)*d3(i0,i2)*d(i0,j0)*d(i1,j0)/120
                          ws(165)=ws(165)+d4(i0,i2)*d(i0,j0)*d(i1,j0)/120
                          ws(166)=ws(166)+d2(i0,i1)*d(i0,i2)*d(i1,i2)*d(i0,j0)*d(i1,j0)/120
                          ws(167)=ws(167)+d(i0,i1)*d2(i0,i2)*d(i1,i2)*d(i0,j0)*d(i1,j0)/120
                          ws(168)=ws(168)+d3(i0,i2)*d(i1,i2)*d(i0,j0)*d(i1,j0)/120
                          ws(169)=ws(169)+d2(i0,i2)*d2(i1,i2)*d(i0,j0)*d(i1,j0)/120
                          ws(170)=ws(170)+d2(i0,i1)*d(i0,i2)*d2(i0,j0)*d(i1,j0)/120
                          ws(171)=ws(171)+d(i0,i1)*d2(i0,i2)*d2(i0,j0)*d(i1,j0)/120
                          ws(172)=ws(172)+d3(i0,i2)*d2(i0,j0)*d(i1,j0)/120
                          ws(173)=ws(173)+d2(i0,i1)*d(i1,i2)*d2(i0,j0)*d(i1,j0)/120
                          ws(174)=ws(174)+d(i0,i1)*d(i0,i2)*d(i1,i2)*d2(i0,j0)*d(i1,j0)/120
                          ws(175)=ws(175)+d2(i0,i2)*d(i1,i2)*d2(i0,j0)*d(i1,j0)/120
                          ws(176)=ws(176)+d(i0,i1)*d2(i1,i2)*d2(i0,j0)*d(i1,j0)/120
                          ws(177)=ws(177)+d(i0,i2)*d2(i1,i2)*d2(i0,j0)*d(i1,j0)/120
                          ws(178)=ws(178)+d3(i1,i2)*d2(i0,j0)*d(i1,j0)/120
                          ws(179)=ws(179)+d(i0,i1)*d(i0,i2)*d3(i0,j0)*d(i1,j0)/120
                          ws(180)=ws(180)+d2(i0,i2)*d3(i0,j0)*d(i1,j0)/120
                          ws(181)=ws(181)+d(i0,i1)*d(i1,i2)*d3(i0,j0)*d(i1,j0)/120
                          ws(182)=ws(182)+d(i0,i2)*d(i1,i2)*d3(i0,j0)*d(i1,j0)/120
                          ws(183)=ws(183)+d2(i1,i2)*d3(i0,j0)*d(i1,j0)/120
                          ws(184)=ws(184)+d(i0,i2)*d4(i0,j0)*d(i1,j0)/120
                          ws(185)=ws(185)+d(i1,i2)*d4(i0,j0)*d(i1,j0)/120
                          ws(186)=ws(186)+d(i0,i1)*d(i0,i2)*d2(i0,j0)*d2(i1,j0)/120
                          ws(187)=ws(187)+d2(i0,i2)*d2(i0,j0)*d2(i1,j0)/120
                          ws(188)=ws(188)+d(i0,i2)*d(i1,i2)*d2(i0,j0)*d2(i1,j0)/120
                          ws(189)=ws(189)+d(i0,i2)*d3(i0,j0)*d2(i1,j0)/120
                          ws(190)=ws(190)+d3(i0,i1)*d(i0,j0)*d(i1,j0)*d(i2,j0)/120
                          ws(191)=ws(191)+d2(i0,i1)*d(i0,i2)*d(i0,j0)*d(i1,j0)*d(i2,j0)/120
                          ws(192)=ws(192)+d(i0,i1)*d(i0,i2)*d(i1,i2)*d(i0,j0)*d(i1,j0)*d(i2,j0)/120
                          ws(193)=ws(193)+d2(i0,i1)*d2(i0,j0)*d(i1,j0)*d(i2,j0)/120
                          ws(194)=ws(194)+d(i0,i1)*d(i0,i2)*d2(i0,j0)*d(i1,j0)*d(i2,j0)/120
                          ws(195)=ws(195)+d(i0,i1)*d(i1,i2)*d2(i0,j0)*d(i1,j0)*d(i2,j0)/120
                          ws(196)=ws(196)+d(i0,i1)*d3(i0,j0)*d(i1,j0)*d(i2,j0)/120
                          ws(197)=ws(197)+d(i1,i2)*d3(i0,j0)*d(i1,j0)*d(i2,j0)/120
                       endif
                       do i3=0, 4
                          if (i3.ne.i0.and.i3.ne.i1.and.i3.ne.i2) then
                             us(25)=us(25)+d(i0,i1)*d(i0,i2)*d(i0,i3)*d(i0,j0)/120
                             us(26)=us(26)+d(i0,i1)*d(i1,i2)*d(i0,i3)*d(i0,j0)/120
                             us(27)=us(27)+d(i0,i2)*d(i0,i3)*d(i0,j0)*d(i1,j0)/120
                             us(28)=us(28)+d(i1,i2)*d(i0,i3)*d(i0,j0)*d(i1,j0)/120
                             if (5.le.mdeg) then
                                vs(60)=vs(60)+d2(i0,i1)*d(i0,i2)*d(i0,i3)*d(i0,j0)/240
                                vs(61)=vs(61)+d2(i0,i1)*d(i1,i2)*d(i0,i3)*d(i0,j0)/240
                                vs(62)=vs(62)+d(i0,i1)*d(i0,i2)*d(i1,i2)*d(i0,i3)*d(i0,j0)/240
                                vs(63)=vs(63)+d(i0,i1)*d2(i1,i2)*d(i0,i3)*d(i0,j0)/240
                                vs(64)=vs(64)+d(i0,i1)*d(i1,i2)*d2(i0,i3)*d(i0,j0)/240
                                vs(65)=vs(65)+d(i0,i1)*d(i0,i2)*d(i0,i3)*d2(i0,j0)/240
                                vs(66)=vs(66)+d(i0,i1)*d(i1,i2)*d(i0,i3)*d2(i0,j0)/240
                                vs(67)=vs(67)+d2(i0,i2)*d(i0,i3)*d(i0,j0)*d(i1,j0)/240
                                vs(68)=vs(68)+d(i0,i1)*d(i0,i2)*d(i0,i3)*d(i0,j0)*d(i1,j0)/240
                                vs(69)=vs(69)+d(i0,i1)*d(i1,i2)*d(i0,i3)*d(i0,j0)*d(i1,j0)/240
                                vs(70)=vs(70)+d(i0,i2)*d(i1,i2)*d(i0,i3)*d(i0,j0)*d(i1,j0)/240
                                vs(71)=vs(71)+d2(i1,i2)*d(i0,i3)*d(i0,j0)*d(i1,j0)/240
                                vs(72)=vs(72)+d2(i0,i2)*d(i2,i3)*d(i0,j0)*d(i1,j0)/240
                                vs(73)=vs(73)+d(i0,i2)*d(i1,i2)*d(i2,i3)*d(i0,j0)*d(i1,j0)/240
                                vs(74)=vs(74)+d(i0,i2)*d(i0,i3)*d(i2,i3)*d(i0,j0)*d(i1,j0)/240
                                vs(75)=vs(75)+d(i0,i2)*d(i0,i3)*d2(i0,j0)*d(i1,j0)/240
                                vs(76)=vs(76)+d(i1,i2)*d(i0,i3)*d2(i0,j0)*d(i1,j0)/240
                                vs(77)=vs(77)+d(i1,i2)*d(i1,i3)*d2(i0,j0)*d(i1,j0)/240
                                vs(78)=vs(78)+d(i0,i2)*d(i2,i3)*d2(i0,j0)*d(i1,j0)/240
                             endif
                             if (6.le.mdeg) then
                                ws(198)=ws(198)+d3(i0,i1)*d(i0,i2)*d(i0,i3)*d(i0,j0)/240
                                ws(199)=ws(199)+d2(i0,i1)*d2(i0,i2)*d(i0,i3)*d(i0,j0)/240
                                ws(200)=ws(200)+d3(i0,i1)*d(i1,i2)*d(i0,i3)*d(i0,j0)/240
                                ws(201)=ws(201)+d2(i0,i1)*d(i0,i2)*d(i1,i2)*d(i0,i3)*d(i0,j0)/240
                                ws(202)=ws(202)+d2(i0,i1)*d2(i1,i2)*d(i0,i3)*d(i0,j0)/240
                                ws(203)=ws(203)+d(i0,i1)*d(i0,i2)*d2(i1,i2)*d(i0,i3)*d(i0,j0)/240
                                ws(204)=ws(204)+d2(i0,i1)*d(i1,i2)*d2(i0,i3)*d(i0,j0)/240
                                ws(205)=ws(205)+d2(i0,i1)*d(i0,i2)*d(i0,i3)*d2(i0,j0)/240
                                ws(206)=ws(206)+d2(i0,i1)*d(i1,i2)*d(i0,i3)*d2(i0,j0)/240
                                ws(207)=ws(207)+d(i0,i1)*d(i0,i2)*d(i1,i2)*d(i0,i3)*d2(i0,j0)/240
                                ws(208)=ws(208)+d(i0,i1)*d2(i1,i2)*d(i0,i3)*d2(i0,j0)/240
                                ws(209)=ws(209)+d(i0,i1)*d(i1,i2)*d2(i0,i3)*d2(i0,j0)/240
                                ws(210)=ws(210)+d(i0,i1)*d(i0,i2)*d(i0,i3)*d3(i0,j0)/240
                                ws(211)=ws(211)+d(i0,i1)*d(i1,i2)*d(i0,i3)*d3(i0,j0)/240
                                ws(212)=ws(212)+d2(i0,i1)*d(i0,i2)*d(i0,i3)*d(i0,j0)*d(i1,j0)/240
                                ws(213)=ws(213)+d(i0,i1)*d2(i0,i2)*d(i0,i3)*d(i0,j0)*d(i1,j0)/240
                                ws(214)=ws(214)+d3(i0,i2)*d(i0,i3)*d(i0,j0)*d(i1,j0)/240
                                ws(215)=ws(215)+d2(i0,i1)*d(i1,i2)*d(i0,i3)*d(i0,j0)*d(i1,j0)/240
                                ws(216)=ws(216)+d(i0,i1)*d(i0,i2)*d(i1,i2)*d(i0,i3)*d(i0,j0)*d(i1,j0)/240
                                ws(217)=ws(217)+d2(i0,i2)*d(i1,i2)*d(i0,i3)*d(i0,j0)*d(i1,j0)/240
                                ws(218)=ws(218)+d(i0,i1)*d2(i1,i2)*d(i0,i3)*d(i0,j0)*d(i1,j0)/240
                                ws(219)=ws(219)+d(i0,i2)*d2(i1,i2)*d(i0,i3)*d(i0,j0)*d(i1,j0)/240
                                ws(220)=ws(220)+d3(i1,i2)*d(i0,i3)*d(i0,j0)*d(i1,j0)/240
                                ws(221)=ws(221)+d2(i0,i2)*d2(i0,i3)*d(i0,j0)*d(i1,j0)/240
                                ws(222)=ws(222)+d(i0,i2)*d(i1,i2)*d2(i0,i3)*d(i0,j0)*d(i1,j0)/240
                                ws(223)=ws(223)+d2(i1,i2)*d2(i0,i3)*d(i0,j0)*d(i1,j0)/240
                                ws(224)=ws(224)+d(i0,i2)*d(i1,i2)*d(i0,i3)*d(i1,i3)*d(i0,j0)*d(i1,j0)/240
                                ws(225)=ws(225)+d(i0,i1)*d2(i0,i2)*d(i2,i3)*d(i0,j0)*d(i1,j0)/240
                                ws(226)=ws(226)+d3(i0,i2)*d(i2,i3)*d(i0,j0)*d(i1,j0)/240
                                ws(227)=ws(227)+d(i0,i1)*d(i0,i2)*d(i1,i2)*d(i2,i3)*d(i0,j0)*d(i1,j0)/240
                                ws(228)=ws(228)+d2(i0,i2)*d(i1,i2)*d(i2,i3)*d(i0,j0)*d(i1,j0)/240
                                ws(229)=ws(229)+d(i0,i1)*d(i0,i2)*d(i0,i3)*d(i2,i3)*d(i0,j0)*d(i1,j0)/240
                                ws(230)=ws(230)+d2(i0,i2)*d(i0,i3)*d(i2,i3)*d(i0,j0)*d(i1,j0)/240
                                ws(231)=ws(231)+d(i0,i1)*d(i0,i2)*d(i0,i3)*d2(i0,j0)*d(i1,j0)/240
                                ws(232)=ws(232)+d2(i0,i2)*d(i0,i3)*d2(i0,j0)*d(i1,j0)/240
                                ws(233)=ws(233)+d(i0,i1)*d(i1,i2)*d(i0,i3)*d2(i0,j0)*d(i1,j0)/240
                                ws(234)=ws(234)+d(i0,i2)*d(i1,i2)*d(i0,i3)*d2(i0,j0)*d(i1,j0)/240
                                ws(235)=ws(235)+d2(i1,i2)*d(i0,i3)*d2(i0,j0)*d(i1,j0)/240
                                ws(236)=ws(236)+d(i1,i2)*d2(i0,i3)*d2(i0,j0)*d(i1,j0)/240
                                ws(237)=ws(237)+d(i0,i1)*d(i1,i2)*d(i1,i3)*d2(i0,j0)*d(i1,j0)/240
                                ws(238)=ws(238)+d(i0,i2)*d(i1,i2)*d(i1,i3)*d2(i0,j0)*d(i1,j0)/240
                                ws(239)=ws(239)+d2(i1,i2)*d(i1,i3)*d2(i0,j0)*d(i1,j0)/240
                                ws(240)=ws(240)+d(i0,i1)*d(i0,i2)*d(i2,i3)*d2(i0,j0)*d(i1,j0)/240
                                ws(241)=ws(241)+d2(i0,i2)*d(i2,i3)*d2(i0,j0)*d(i1,j0)/240
                                ws(242)=ws(242)+d(i0,i2)*d(i1,i2)*d(i2,i3)*d2(i0,j0)*d(i1,j0)/240
                                ws(243)=ws(243)+d2(i1,i2)*d(i2,i3)*d2(i0,j0)*d(i1,j0)/240
                                ws(244)=ws(244)+d(i0,i2)*d(i0,i3)*d(i2,i3)*d2(i0,j0)*d(i1,j0)/240
                                ws(245)=ws(245)+d(i1,i2)*d(i0,i3)*d(i2,i3)*d2(i0,j0)*d(i1,j0)/240
                                ws(246)=ws(246)+d(i0,i2)*d2(i2,i3)*d2(i0,j0)*d(i1,j0)/240
                                ws(248)=ws(248)+d(i0,i2)*d(i0,i3)*d3(i0,j0)*d(i1,j0)/240
                                ws(249)=ws(249)+d(i1,i2)*d(i0,i3)*d3(i0,j0)*d(i1,j0)/240
                                ws(250)=ws(250)+d(i1,i2)*d(i1,i3)*d3(i0,j0)*d(i1,j0)/240
                                ws(251)=ws(251)+d(i0,i2)*d(i2,i3)*d3(i0,j0)*d(i1,j0)/240
                                ws(252)=ws(252)+d(i0,i2)*d(i0,i3)*d2(i0,j0)*d2(i1,j0)/240
                                ws(253)=ws(253)+d2(i0,i1)*d(i0,i3)*d(i0,j0)*d(i1,j0)*d(i2,j0)/240
                                ws(254)=ws(254)+d(i1,i2)*d(i0,i3)*d2(i0,j0)*d(i1,j0)*d(i2,j0)/240
                                do i4=0, 4
                                   if (i4.ne.i0.and.i4.ne.i1.and.i4.ne.i2.and.i4.ne.i3) then
                                      ws(247)=ws(247)+d(i0,i2)*d(i0,i3)*d(i0,i4)*d2(i0,j0)*d(i1,j0)/240
                                   endif
                                enddo
                             endif
                          endif
                       enddo
                    endif
                 enddo
              endif
           endif
        enddo
     enddo
  enddo
  if (3.le.mdeg) then
     do j0=5, 6
        do j1=5, 6
           if (j1.ne.j0) then
              do i0=0, 4
                 if (6.le.mdeg) then
                    ws(255)=ws(255)+d5(i0,j0)*d(i0,j1)/10
                    ws(256)=ws(256)+d4(i0,j0)*d2(i0,j1)/10
                 endif
                 do i1=0, 4
                    if (i1.ne.i0) then
                       zs(10)=zs(10)+d(i0,i1)*d(i0,j0)*d(i0,j1)/40
                       zs(11)=zs(11)+d(i0,i1)*d(i1,j0)*d(i0,j1)/40
                       us(29)=us(29)+d2(i0,i1)*d(i0,j0)*d(i0,j1)/40
                       us(30)=us(30)+d(i0,i1)*d2(i0,j0)*d(i0,j1)/40
                       us(31)=us(31)+d3(i0,j0)*d(i0,j1)/40
                       us(32)=us(32)+d2(i0,i1)*d(i1,j0)*d(i0,j1)/40
                       us(33)=us(33)+d(i0,i1)*d(i0,j0)*d(i1,j0)*d(i0,j1)/40
                       us(34)=us(34)+d2(i0,j0)*d(i1,j0)*d(i0,j1)/40
                       us(35)=us(35)+d(i0,i1)*d2(i1,j0)*d(i0,j1)/40
                       if (5.le.mdeg) then
                          vs(79)=vs(79)+d3(i0,i1)*d(i0,j0)*d(i0,j1)/40
                          vs(80)=vs(80)+d2(i0,i1)*d2(i0,j0)*d(i0,j1)/40
                          vs(81)=vs(81)+d(i0,i1)*d3(i0,j0)*d(i0,j1)/40
                          vs(82)=vs(82)+d4(i0,j0)*d(i0,j1)/40
                          vs(83)=vs(83)+d3(i0,i1)*d(i1,j0)*d(i0,j1)/40
                          vs(84)=vs(84)+d2(i0,i1)*d(i0,j0)*d(i1,j0)*d(i0,j1)/40
                          vs(85)=vs(85)+d(i0,i1)*d2(i0,j0)*d(i1,j0)*d(i0,j1)/40
                          vs(86)=vs(86)+d3(i0,j0)*d(i1,j0)*d(i0,j1)/40
                          vs(87)=vs(87)+d2(i0,i1)*d2(i1,j0)*d(i0,j1)/40
                          vs(88)=vs(88)+d(i0,i1)*d(i0,j0)*d2(i1,j0)*d(i0,j1)/40
                          vs(89)=vs(89)+d2(i0,j0)*d2(i1,j0)*d(i0,j1)/40
                          vs(90)=vs(90)+d(i0,i1)*d3(i1,j0)*d(i0,j1)/40
                          vs(91)=vs(91)+d(i0,i1)*d2(i0,j0)*d2(i0,j1)/40
                          vs(92)=vs(92)+d(i0,i1)*d(i0,j0)*d(i1,j0)*d2(i0,j1)/40
                          vs(93)=vs(93)+d(i0,i1)*d2(i1,j0)*d2(i0,j1)/40
                       endif
                       if (6.le.mdeg) then
                          ws(257)=ws(257)+d4(i0,i1)*d(i0,j0)*d(i0,j1)/40
                          ws(258)=ws(258)+d3(i0,i1)*d2(i0,j0)*d(i0,j1)/40
                          ws(259)=ws(259)+d2(i0,i1)*d3(i0,j0)*d(i0,j1)/40
                          ws(260)=ws(260)+d(i0,i1)*d4(i0,j0)*d(i0,j1)/40
                          ws(261)=ws(261)+d4(i0,i1)*d(i1,j0)*d(i0,j1)/40
                          ws(262)=ws(262)+d3(i0,i1)*d(i0,j0)*d(i1,j0)*d(i0,j1)/40
                          ws(263)=ws(263)+d2(i0,i1)*d2(i0,j0)*d(i1,j0)*d(i0,j1)/40
                          ws(264)=ws(264)+d(i0,i1)*d3(i0,j0)*d(i1,j0)*d(i0,j1)/40
                          ws(265)=ws(265)+d4(i0,j0)*d(i1,j0)*d(i0,j1)/40
                          ws(266)=ws(266)+d3(i0,i1)*d2(i1,j0)*d(i0,j1)/40
                          ws(267)=ws(267)+d2(i0,i1)*d(i0,j0)*d2(i1,j0)*d(i0,j1)/40
                          ws(268)=ws(268)+d(i0,i1)*d2(i0,j0)*d2(i1,j0)*d(i0,j1)/40
                          ws(269)=ws(269)+d3(i0,j0)*d2(i1,j0)*d(i0,j1)/40
                          ws(270)=ws(270)+d2(i0,i1)*d3(i1,j0)*d(i0,j1)/40
                          ws(271)=ws(271)+d(i0,i1)*d(i0,j0)*d3(i1,j0)*d(i0,j1)/40
                          ws(272)=ws(272)+d(i0,i1)*d4(i1,j0)*d(i0,j1)/40
                          ws(273)=ws(273)+d2(i0,i1)*d2(i0,j0)*d2(i0,j1)/40
                          ws(274)=ws(274)+d2(i0,i1)*d(i0,j0)*d(i1,j0)*d2(i0,j1)/40
                          ws(275)=ws(275)+d(i0,i1)*d2(i0,j0)*d(i1,j0)*d2(i0,j1)/40
                          ws(276)=ws(276)+d3(i0,j0)*d(i1,j0)*d2(i0,j1)/40
                          ws(277)=ws(277)+d2(i0,i1)*d2(i1,j0)*d2(i0,j1)/40
                          ws(278)=ws(278)+d(i0,i1)*d(i0,j0)*d2(i1,j0)*d2(i0,j1)/40
                          ws(279)=ws(279)+d(i0,i1)*d3(i1,j0)*d2(i0,j1)/40
                       endif
                       do i2=0, 4
                          if (i2.ne.i1.and.i2.ne.i0) then
                             us(36)=us(36)+d(i0,i1)*d(i0,i2)*d(i0,j0)*d(i0,j1)/120
                             us(37)=us(37)+d(i0,i1)*d(i1,i2)*d(i0,j0)*d(i0,j1)/120
                             us(38)=us(38)+d(i0,i1)*d(i0,i2)*d(i1,j0)*d(i0,j1)/120
                             if (5.le.mdeg) then
                                vs(94)=vs(94)+d2(i0,i1)*d(i0,i2)*d(i0,j0)*d(i0,j1)/120
                                vs(95)=vs(95)+d2(i0,i1)*d(i1,i2)*d(i0,j0)*d(i0,j1)/120
                                vs(96)=vs(96)+d(i0,i1)*d(i0,i2)*d(i1,i2)*d(i0,j0)*d(i0,j1)/120
                                vs(97)=vs(97)+d(i0,i1)*d2(i1,i2)*d(i0,j0)*d(i0,j1)/120
                                vs(98)=vs(98)+d(i0,i1)*d(i0,i2)*d2(i0,j0)*d(i0,j1)/120
                                vs(99)=vs(99)+d(i0,i1)*d(i1,i2)*d2(i0,j0)*d(i0,j1)/120
                                vs(100)=vs(100)+d2(i0,i1)*d(i0,i2)*d(i1,j0)*d(i0,j1)/120
                                vs(101)=vs(101)+d(i0,i1)*d2(i0,i2)*d(i1,j0)*d(i0,j1)/120
                                vs(102)=vs(102)+d(i0,i1)*d(i0,i2)*d(i1,i2)*d(i1,j0)*d(i0,j1)/120
                                vs(103)=vs(103)+d(i0,i1)*d(i0,i2)*d(i0,j0)*d(i1,j0)*d(i0,j1)/120
                                vs(104)=vs(104)+d(i0,i1)*d(i1,i2)*d(i0,j0)*d(i1,j0)*d(i0,j1)/120
                                vs(105)=vs(105)+d(i0,i2)*d2(i0,j0)*d(i1,j0)*d(i0,j1)/120
                                vs(106)=vs(106)+d(i1,i2)*d2(i0,j0)*d(i1,j0)*d(i0,j1)/120
                                vs(107)=vs(107)+d(i0,i1)*d(i0,i2)*d2(i1,j0)*d(i0,j1)/120
                                vs(108)=vs(108)+d(i0,i1)*d(i0,i2)*d(i1,j0)*d(i2,j0)*d(i0,j1)/120
                                vs(109)=vs(109)+d(i0,i1)*d(i0,j0)*d(i1,j0)*d(i2,j0)*d(i0,j1)/120
                             endif
                             if (6.le.mdeg) then

                                di0j1=d(i0,j1)/120

                                ws(280)=ws(280)+d3(i0,i1)*d(i0,i2)*d(i0,j0)*di0j1
                                ws(281)=ws(281)+d2(i0,i1)*d2(i0,i2)*d(i0,j0)*di0j1
                                ws(282)=ws(282)+d3(i0,i1)*d(i1,i2)*d(i0,j0)*di0j1
                                ws(283)=ws(283)+d2(i0,i1)*d(i0,i2)*d(i1,i2)*d(i0,j0)*di0j1
                                ws(284)=ws(284)+d2(i0,i1)*d2(i1,i2)*d(i0,j0)*di0j1
                                ws(285)=ws(285)+d(i0,i1)*d(i0,i2)*d2(i1,i2)*d(i0,j0)*di0j1
                                ws(286)=ws(286)+d(i0,i1)*d3(i1,i2)*d(i0,j0)*di0j1
                                ws(287)=ws(287)+d2(i0,i1)*d(i0,i2)*d2(i0,j0)*di0j1
                                ws(288)=ws(288)+d2(i0,i1)*d(i1,i2)*d2(i0,j0)*di0j1
                                ws(289)=ws(289)+d(i0,i1)*d(i0,i2)*d(i1,i2)*d2(i0,j0)*di0j1
                                ws(290)=ws(290)+d(i0,i1)*d2(i1,i2)*d2(i0,j0)*di0j1
                                ws(291)=ws(291)+d(i0,i1)*d(i0,i2)*d3(i0,j0)*di0j1
                                ws(292)=ws(292)+d(i0,i1)*d(i1,i2)*d3(i0,j0)*di0j1
                                ws(293)=ws(293)+d3(i0,i1)*d(i0,i2)*d(i1,j0)*di0j1
                                ws(294)=ws(294)+d(i0,i1)*d3(i0,i2)*d(i1,j0)*di0j1
                                ws(295)=ws(295)+d2(i0,i1)*d(i0,i2)*d(i0,j0)*d(i1,j0)*di0j1
                                ws(296)=ws(296)+d(i0,i1)*d2(i0,i2)*d(i0,j0)*d(i1,j0)*di0j1
                                ws(297)=ws(297)+d2(i0,i1)*d(i1,i2)*d(i0,j0)*d(i1,j0)*di0j1
                                ws(298)=ws(298)+d(i0,i1)*d(i0,i2)*d(i1,i2)*d(i0,j0)*d(i1,j0)*di0j1
                                ws(299)=ws(299)+d2(i0,i2)*d(i1,i2)*d(i0,j0)*d(i1,j0)*di0j1
                                ws(300)=ws(300)+d(i0,i1)*d2(i1,i2)*d(i0,j0)*d(i1,j0)*di0j1
                                ws(301)=ws(301)+d(i0,i1)*d(i0,i2)*d2(i0,j0)*d(i1,j0)*di0j1
                                ws(302)=ws(302)+d2(i0,i2)*d2(i0,j0)*d(i1,j0)*di0j1
                                ws(303)=ws(303)+d(i0,i1)*d(i1,i2)*d2(i0,j0)*d(i1,j0)*di0j1
                                ws(304)=ws(304)+d(i0,i2)*d(i1,i2)*d2(i0,j0)*d(i1,j0)*di0j1
                                ws(305)=ws(305)+d2(i1,i2)*d2(i0,j0)*d(i1,j0)*di0j1
                                ws(306)=ws(306)+d(i0,i2)*d3(i0,j0)*d(i1,j0)*di0j1
                                ws(307)=ws(307)+d(i1,i2)*d3(i0,j0)*d(i1,j0)*di0j1
                                ws(308)=ws(308)+d2(i0,i1)*d(i0,i2)*d2(i1,j0)*di0j1
                                ws(309)=ws(309)+d(i0,i1)*d2(i0,i2)*d2(i1,j0)*di0j1
                                ws(310)=ws(310)+d(i0,i1)*d(i0,i2)*d(i0,j0)*d2(i1,j0)*di0j1
                                ws(311)=ws(311)+d(i0,i1)*d(i1,i2)*d(i0,j0)*d2(i1,j0)*di0j1
                                ws(312)=ws(312)+d(i0,i2)*d2(i0,j0)*d2(i1,j0)*di0j1
                                ws(313)=ws(313)+d(i0,i1)*d(i0,i2)*d3(i1,j0)*di0j1
                                ws(314)=ws(314)+d(i0,i1)*d(i1,i2)*d3(i1,j0)*di0j1
                                ws(315)=ws(315)+d2(i0,i1)*d(i0,i2)*d(i1,j0)*d(i2,j0)*di0j1
                                ws(316)=ws(316)+d2(i0,i1)*d(i1,i2)*d(i1,j0)*d(i2,j0)*di0j1
                                ws(317)=ws(317)+d2(i0,i1)*d(i0,j0)*d(i1,j0)*d(i2,j0)*di0j1
                                ws(318)=ws(318)+d(i0,i1)*d(i0,i2)*d(i0,j0)*d(i1,j0)*d(i2,j0)*di0j1
                                ws(319)=ws(319)+d(i0,i1)*d(i1,i2)*d(i0,j0)*d(i1,j0)*d(i2,j0)*di0j1
                                ws(320)=ws(320)+d(i0,i1)*d2(i0,j0)*d(i1,j0)*d(i2,j0)*di0j1
                                ws(321)=ws(321)+d(i1,i2)*d2(i0,j0)*d(i1,j0)*d(i2,j0)*di0j1
                                ws(322)=ws(322)+d(i0,i1)*d(i0,i2)*d2(i1,j0)*d(i2,j0)*di0j1
                                ws(323)=ws(323)+d(i0,i1)*d(i0,j0)*d2(i1,j0)*d(i2,j0)*di0j1
                                ws(324)=ws(324)+d(i0,i1)*d(i1,i2)*d2(i0,j0)*d2(i0,j1)/120
                                ws(325)=ws(325)+d(i0,i1)*d(i0,i2)*d(i0,j0)*d(i1,j0)*d2(i0,j1)/120
                             endif
                             do i3=0, 4
                                if (i3.ne.i0.and.i3.ne.i1.and.i3.ne.i2) then
                                   if (5.le.mdeg) then
                                      vs(110)=vs(110)+d(i0,i1)*d(i0,i2)*d(i0,i3)*d(i0,j0)*d(i0,j1)/240
                                      vs(111)=vs(111)+d(i0,i1)*d(i1,i2)*d(i0,i3)*d(i0,j0)*d(i0,j1)/240
                                      vs(112)=vs(112)+d(i0,i1)*d(i0,i3)*d(i1,j0)*d(i2,j0)*d(i0,j1)/240
                                   endif
                                   if (6.le.mdeg) then
                                      ws(326)=ws(326)+d2(i0,i1)*d(i0,i2)*d(i0,i3)*d(i0,j0)*d(i0,j1)/240
                                      ws(327)=ws(327)+d2(i0,i1)*d(i1,i2)*d(i0,i3)*d(i0,j0)*d(i0,j1)/240
                                      ws(328)=ws(328)+d(i0,i1)*d(i0,i2)*d(i1,i2)*d(i0,i3)*d(i0,j0)*d(i0,j1)/240
                                      ws(329)=ws(329)+d(i0,i1)*d2(i1,i2)*d(i0,i3)*d(i0,j0)*d(i0,j1)/240
                                      ws(330)=ws(330)+d(i0,i1)*d(i1,i2)*d(i0,i3)*d2(i0,j0)*d(i0,j1)/240
                                      ws(331)=ws(331)+d(i0,i1)*d(i0,i2)*d(i0,i3)*d(i0,j0)*d(i1,j0)*d(i0,j1)/240
                                      ws(332)=ws(332)+d(i0,i2)*d(i0,i3)*d2(i0,j0)*d(i1,j0)*d(i0,j1)/240
                                      ws(333)=ws(333)+d(i1,i2)*d(i0,i3)*d2(i0,j0)*d(i1,j0)*d(i0,j1)/240
                                      ws(334)=ws(334)+d(i0,i2)*d(i2,i3)*d2(i0,j0)*d(i1,j0)*d(i0,j1)/240
                                      ws(335)=ws(335)+d2(i0,i1)*d(i0,i3)*d(i1,j0)*d(i2,j0)*d(i0,j1)/240
                                      ws(336)=ws(336)+d(i0,i1)*d(i0,i2)*d(i0,i3)*d(i1,j0)*d(i2,j0)*d(i0,j1)/240
                                      ws(337)=ws(337)+d(i0,i1)*d(i0,i3)*d(i0,j0)*d(i1,j0)*d(i2,j0)*d(i0,j1)/240
                                   endif
                                endif
                             enddo
                          endif
                       enddo
                    endif
                 enddo
              enddo
           endif
        enddo
     enddo
  endif
  ! Reducible secondaries
  us(0)=her2_zundel(ys(0))
  vs(0:11)=ys(0)*zs(0:11)
  ws(0)=her3_zundel(ys(0))
  ws(1)=her2_zundel(zs(0))
  ws(2)=zs(0)*zs(1)
  ws(3)=her2_zundel(zs(1))
  ws(4:5)=zs(0:1)*zs(2)
  ws(6)=her2_zundel(zs(2))
  ws(7:9)=zs(0:2)*zs(3)
  ws(10)=her2_zundel(zs(3))
  ws(11:14)=zs(0:3)*zs(4)
  ws(15)=her2_zundel(zs(4))
  ws(16:20)=zs(0:4)*zs(5)
  ws(21)=her2_zundel(zs(5))
  ws(22:27)=zs(0:5)*zs(6)
  ! her2_zundel(zs(6)) is not independent
  ws(28:34)=zs(0:6)*zs(7)
  ws(35)=her2_zundel(zs(7))
  ws(36:43)=zs(0:7)*zs(8)
  ws(44)=her2_zundel(zs(8))
  ws(45:50)=zs(0:5)*zs(9)
  ! zs(6)*zs(9) is not independent
  ws(51:52)=zs(7:8)*zs(9)
  ! her2_zundel(zs(9)) is not independent
  ws(53:62)=zs(0:9)*zs(10)
  ws(63)=her2_zundel(zs(10))
  ws(64:74)=zs(0:10)*zs(11)
  ws(75)=her2_zundel(zs(11))
  ws(76:113)=ys(0)*us(1:38)
! Compute vec(0:*).  The code was created using these parameters:
  ! MolienSeries(0:*): 1 3 12 43 159 554 1879 6066 18755
  ! #Primaries(1:*):   3 5 3 4 3 2 0 0
  ! #Secondaries(1:*): 0 1 12 39 113 338 932 2402
  ! constant term
  vec(0)=1
  ! first degree terms
  if (1.le.mdeg) then
     vec(1)=x(0)
     vec(2)=x(1)
     vec(3)=x(2)
  endif
! second degree terms
  if (2.le.mdeg) then
     vec(4)=x2(0)
     vec(5:6)=x(0)*vec(2:3)
     vec(7)=x2(1)
     vec(8:8)=x(1)*vec(3:3)
     vec(9)=x2(2)
     vec(10)=y(0)
     vec(11)=y(1)
     vec(12)=y(2)
     vec(13)=y(3)
     vec(14)=y(4)
     vec(15:15)=ys(0:0)
  endif
  ! third degree terms
  if (3.le.mdeg) then
     vec(16)=x3(0)
     vec(17:18)=x2(0)*vec(2:3)
     vec(19:27)=x(0)*vec(7:15)
     vec(28)=x3(1)
     vec(29:29)=x2(1)*vec(3:3)
     vec(30:36)=x(1)*vec(9:15)
     vec(37)=x3(2)
     vec(38:43)=x(2)*vec(10:15)
     vec(44)=z(0)
     vec(45)=z(1)
     vec(46)=z(2)
     vec(47:58)=zs(0:11)
  endif
  ! fourth degree terms
  if (4.le.mdeg) then
     vec(59)=x4(0)
     vec(60:61)=x3(0)*vec(2:3)
     vec(62:70)=x2(0)*vec(7:15)
     vec(71:101)=x(0)*vec(28:58)
     vec(102)=x4(1)
     vec(103:103)=x3(1)*vec(3:3)
     vec(104:110)=x2(1)*vec(9:15)
     vec(111:132)=x(1)*vec(37:58)
     vec(133)=x4(2)
     vec(134:139)=x2(2)*vec(10:15)
     vec(140:154)=x(2)*vec(44:58)
     vec(155)=y2(0)
     vec(156:160)=y(0)*vec(11:15)
     vec(161)=y2(1)
     vec(162:165)=y(1)*vec(12:15)
     vec(166)=y2(2)
     vec(167:169)=y(2)*vec(13:15)
     vec(170)=y2(3)
     vec(171:172)=y(3)*vec(14:15)
     vec(173)=y2(4)
     vec(174:174)=y(4)*vec(15:15)
     vec(175)=u(0)
     vec(176)=u(1)
     vec(177)=u(2)
     vec(178)=u(3)
     vec(179:217)=us(0:38)
  endif
  ! fifth degree terms
  if (5.le.mdeg) then
     vec(218)=x5(0)
     vec(219:220)=x4(0)*vec(2:3)
     vec(221:229)=x3(0)*vec(7:15)
     vec(230:260)=x2(0)*vec(28:58)
     vec(261:376)=x(0)*vec(102:217)
     vec(377)=x5(1)
     vec(378:378)=x4(1)*vec(3:3)
     vec(379:385)=x3(1)*vec(9:15)
     vec(386:407)=x2(1)*vec(37:58)
     vec(408:492)=x(1)*vec(133:217)
     vec(493)=x5(2)
     vec(494:499)=x3(2)*vec(10:15)
     vec(500:514)=x2(2)*vec(44:58)
     vec(515:577)=x(2)*vec(155:217)
     vec(578:592)=y(0)*vec(44:58)
     vec(593:607)=y(1)*vec(44:58)
     vec(608:622)=y(2)*vec(44:58)
     vec(623:637)=y(3)*vec(44:58)
     vec(638:652)=y(4)*vec(44:58)
     vec(653:653)=z(0)*ys(0:0)
     vec(654:654)=z(1)*ys(0:0)
     vec(655:655)=z(2)*ys(0:0)
     vec(656)=v(0)
     vec(657)=v(1)
     vec(658)=v(2)
     vec(659:771)=vs(0:112)
  endif
  ! sixth degree terms
  if (6.le.mdeg) then
     vec(772)=x6(0)
     vec(773:774)=x5(0)*vec(2:3)
     vec(775:783)=x4(0)*vec(7:15)
     vec(784:814)=x3(0)*vec(28:58)
     vec(815:930)=x2(0)*vec(102:217)
     vec(931:1325)=x(0)*vec(377:771)
     vec(1326)=x6(1)
     vec(1327:1327)=x5(1)*vec(3:3)
     vec(1328:1334)=x4(1)*vec(9:15)
     vec(1335:1356)=x3(1)*vec(37:58)
     vec(1357:1441)=x2(1)*vec(133:217)
     vec(1442:1720)=x(1)*vec(493:771)
     vec(1721)=x6(2)
     vec(1722:1727)=x4(2)*vec(10:15)
     vec(1728:1742)=x3(2)*vec(44:58)
     vec(1743:1805)=x2(2)*vec(155:217)
     vec(1806:1999)=x(2)*vec(578:771)
     vec(2000)=y3(0)
     vec(2001:2005)=y2(0)*vec(11:15)
     vec(2006:2062)=y(0)*vec(161:217)
     vec(2063)=y3(1)
     vec(2064:2067)=y2(1)*vec(12:15)
     vec(2068:2119)=y(1)*vec(166:217)
     vec(2120)=y3(2)
     vec(2121:2123)=y2(2)*vec(13:15)
     vec(2124:2171)=y(2)*vec(170:217)
     vec(2172)=y3(3)
     vec(2173:2174)=y2(3)*vec(14:15)
     vec(2175:2219)=y(3)*vec(173:217)
     vec(2220)=y3(4)
     vec(2221:2221)=y2(4)*vec(15:15)
     vec(2222:2264)=y(4)*vec(175:217)
     vec(2265)=z2(0)
     vec(2266:2279)=z(0)*vec(45:58)
     vec(2280)=z2(1)
     vec(2281:2293)=z(1)*vec(46:58)
     vec(2294)=z2(2)
     vec(2295:2306)=z(2)*vec(47:58)
     vec(2307:2307)=u(0)*ys(0:0)
     vec(2308:2308)=u(1)*ys(0:0)
     vec(2309:2309)=u(2)*ys(0:0)
     vec(2310:2310)=u(3)*ys(0:0)
     vec(2311)=w(0)
     vec(2312)=w(1)
     vec(2313:2650)=ws(0:337)
  endif
  ! seventh degree terms
  if (7.le.mdeg) then
     vec(2651)=x7(0)
     vec(2652:2653)=x6(0)*vec(2:3)
     vec(2654:2662)=x5(0)*vec(7:15)
     vec(2663:2693)=x4(0)*vec(28:58)
     vec(2694:2809)=x3(0)*vec(102:217)
     vec(2810:3204)=x2(0)*vec(377:771)
     vec(3205:4529)=x(0)*vec(1326:2650)
     vec(4530)=x7(1)
     vec(4531:4531)=x6(1)*vec(3:3)
     vec(4532:4538)=x5(1)*vec(9:15)
     vec(4539:4560)=x4(1)*vec(37:58)
     vec(4561:4645)=x3(1)*vec(133:217)
     vec(4646:4924)=x2(1)*vec(493:771)
     vec(4925:5854)=x(1)*vec(1721:2650)
     vec(5855)=x7(2)
     vec(5856:5855)=x6(2)*vec(4:3)
     vec(5856:5861)=x5(2)*vec(10:15)
     vec(5862:5876)=x4(2)*vec(44:58)
     vec(5877:5939)=x3(2)*vec(155:217)
     vec(5940:6133)=x2(2)*vec(578:771)
     vec(6134:6784)=x(2)*vec(2000:2650)
     vec(6785:6799)=y2(0)*vec(44:58)
     vec(6800:6978)=y(0)*vec(593:771)
     vec(6979:6993)=y2(1)*vec(44:58)
     vec(6994:7157)=y(1)*vec(608:771)
     vec(7158:7172)=y2(2)*vec(44:58)
     vec(7173:7321)=y(2)*vec(623:771)
     vec(7322:7336)=y2(3)*vec(44:58)
     vec(7337:7470)=y(3)*vec(638:771)
     vec(7471:7485)=y2(4)*vec(44:58)
     vec(7486:7604)=y(4)*vec(653:771)
     vec(7605:7647)=z(0)*vec(175:217)
     vec(7648:7690)=z(1)*vec(175:217)
     vec(7691:7733)=z(2)*vec(175:217)
     vec(7734:7745)=u(0)*zs(0:11)
     vec(7746:7757)=u(1)*zs(0:11)
     vec(7758:7769)=u(2)*zs(0:11)
     vec(7770:7781)=u(3)*zs(0:11)
     vec(7782:7782)=v(0)*ys(0:0)
     vec(7783:7783)=v(1)*ys(0:0)
     vec(7784:7784)=v(2)*ys(0:0)
     !! vec(7785:8716)=w7s(0:931)
     !! w7s(*) to follow
  endif
  return
end subroutine getv_x5y2

subroutine getrvec_zundel (m, r, vec)
  implicit none
  integer, parameter :: wp=selected_real_kind(12,300)
  ! version for X5Y2
  integer nk, m
  parameter (nk=7)
  double precision  r(0:nk-1,0:nk-1), vec(0:m-1)
  integer i, j
  double precision  x(0:2), r1(0:nk-1,0:nk-1), t0
  !-----------------------------------------------------------------------
  ! Test for compatibility
  if (.not.(m.eq.1.or.m.eq.4)) then
     stop 'getrvec_zundel - wrong dimension'
  endif
  ! Computation
  x=0
  do i=0, nk-1
     do j=0, nk-1
        if (i.eq.j) then
           r1(i,j)=0
        else
           r1(i,j)=exp(-r(i,j))/r(i,j)
        endif
     enddo
  enddo
  ! XY distance
  x(0)=sum(r1(0:4,5:6))/10
  ! XX distance
  t0=0
  do i=0, 4
     do j=i+1, 4
        t0=t0+r1(i,j)/10
     enddo
  enddo
  x(1)=t0
  ! YY distance
  x(2)=r1(5,6)
  ! set vec
  vec(0)=1
  if (4.le.m) then
     vec(1:3)=x
  endif
  return
end subroutine getrvec_zundel


!!$subroutine predip()
!!$
!!$  !        implicit none
!!$  implicit double precision (a-h,o-z)
!!$  implicit integer (i-n)
!!$
!!$  double precision dip(3), cart_in(7,3), v0, dip_coef(0:3843)
!!$  double precision dip_dc0(0:6,0:6), dip_dw0(0:6,0:6)
!!$
!!$  integer i,j,k,i1,j1,k1
!!$
!!$  common/NDIPCOE/ma,mb
!!$  common/h5o2dipcoef/dip_dc0,dip_dw0,dip_coef
!!$
!!$  ma=2512 ; mb=1332
!!$
!!$  open(20,file='h5o2.dms4B.coeff.com.dat',status='old')
!!$
!!$  read(20,*)
!!$  read(20,*)dip_dc0
!!$  read(20,*)
!!$  read(20,*)dip_dw0
!!$  read(20,*)
!!$  read(20,*)
!!$  read(20,*)(dip_coef(i1),i1=0,ma+mb-1)
!!$  close(20)
!!$
!!$  return
!!$end subroutine predip

!*******************************************************************


subroutine calcdip_zundel(dip,cart_in)

  implicit none

  double precision dip(3), cart_in(7,3), dip_coef(0:3843)
  double precision dip_dc0(0:6,0:6), dip_dw0(0:6,0:6)

  integer ma,mb

  common/NDIPCOE/ma,mb
  common/h5o2dipcoef/dip_dc0,dip_dw0,dip_coef

  integer j,i

  double precision xnuc(0:2,0:6),vec(0:3,0:ma+mb-1)

  double precision dx(0:2),xn(0:2,0:6)

  do j=1,3
     xn(j-1,5:6)=cart_in(1:2,j)
     xn(j-1,0:4)=cart_in(3:7,j)
  end do

  !! shift to center-of-charge frame (version for O2H5)
  !        dx = (8*xn(0:2,0)+8*xn(0:2,1)+xn(0:2,2)+
  !     $   xn(0:2,3)+xn(0:2,4)+xn(0:2,5)+xn(0:2,6))/21

  !! shift to h5o2 C.O.M. frame (O2H5)
  dx=(15.9949146d0*xn(0:2,5)+15.9949146d0*xn(0:2,6)+ &
       1.007825d0*(xn(0:2,0)+xn(0:2,1)+xn(0:2,2)+xn(0:2,3)+ &
       xn(0:2,4)))/(15.9949146d0*2+1.007825d0*5)

  do i = 0, 6
     xnuc(0:2,i) = xn(0:2,i)-dx
  enddo

  call getdvec (ma, mb, xnuc(0:2,0:6),dip_dc0,dip_dw0,vec)
  dip = matmul(vec(0:2,0:ma+mb-1),dip_coef)

  dip = dip + dx

  !        write(2002,*)dip

  return
end subroutine calcdip_zundel
!********************************************************
!*********************************************************************
subroutine getscale_zundel (n, xnuc, dc0, dw0)
  implicit none
  integer, parameter :: wp=selected_real_kind(12,300)
  ! Version for X5Y2
  integer n, nk
  parameter (nk=7)
  double precision xnuc(0:2,0:nk-1,0:n-1), dc0(0:nk-1,0:nk-1), dw0(0:nk-1,0:nk-1)
  integer ip, i, j
  double precision d0(0:nk-1,0:nk-1), dt0(0:nk-1,0:nk-1), &
       r0(0:nk-1,0:nk-1), dc1(0:nk-1,0:nk-1), dw1(0:nk-1,0:nk-1),t0
  ! evaluate dc0
  dt0 = 0 ; dc1 = 0 ; dw1 = 1
  do ip = 0, n-1
     call getr0_zundel (nk, xnuc(0:2,0:nk-1,ip), r0)
     call getd0_zundel (nk, r0, dc1, dw1, d0)
     dt0 = dt0+d0/n
  enddo
  do i = 0, nk-1
     dc0(i,i) = 0
  enddo
  t0 = 0
  do i = 0, 4
     do j = i+1, 4
        t0 = t0+dt0(i,j)/10
     enddo
  enddo
  do i = 0, 4
     do j = i+1, 4
        dc0(i,j) = t0
        dc0(j,i) = t0
     enddo
  enddo
  dc0(5,6) = dt0(5,6)
  dc0(6,5) = dt0(5,6)
  t0 = sum(dt0(0:4,5:6))/10
  dc0(0:4,5:6) = t0
  dc0(5:6,0:4) = t0
  ! evaluate dw0
  dt0 = 0
  do ip = 0, n-1
     call getr0_zundel (nk, xnuc(0:2,0:nk-1,ip), r0)
     call getd0_zundel (nk, r0, dc0, dw1, d0)
     dt0 = dt0+d0**2/n
  enddo
  t0 = 0
  do i = 0, 4
     t0 = t0+dt0(i,i)/5
  enddo
  do i = 0, 4
     dw0(i,i) = dsqrt(t0)
  enddo
  do i = 5, 6
     t0 = t0+dt0(i,i)/2
  enddo
  do i = 5, 6
     dw0(i,i) = dsqrt(t0)
  enddo
  t0 = 0
  do i = 0, 4
     do j = i+1, 4
        t0 = t0+dt0(i,j)/10
     enddo
  enddo
  do i = 0, 4
     do j = i+1, 4
        dw0(i,j) = dsqrt(t0)
        dw0(j,i) = dsqrt(t0)
     enddo
  enddo
  dw0(5,6) = dsqrt(dt0(5,6))
  dw0(6,5) = dsqrt(dt0(5,6))
  t0 = sum(dt0(0:4,5:6))/10
  dw0(0:4,5:6) = dsqrt(t0)
  dw0(5:6,0:4) = dsqrt(t0)
  return
end subroutine getscale_zundel
subroutine getr0_zundel (nk, xn, r0)
  implicit none
  integer, parameter :: wp=selected_real_kind(12,300)
  integer nk
  double precision xn(0:2,0:nk-1), r0(0:nk-1,0:nk-1)
  integer i, j
  do i = 0, nk-1
     r0(i,i) = 0
     do j = i+1, nk-1
        r0(i,j) = dsqrt((xn(0,j)-xn(0,i))**2+(xn(1,j)-xn(1,i))**2+(xn(2,j)-xn(2,i))**2)
        r0(j,i) = r0(i,j)
     enddo
  enddo
  return
end subroutine getr0_zundel
subroutine getd0_zundel (nk, r0, dc0, dw0, d0)
  implicit none
  integer, parameter :: wp=selected_real_kind(12,300)
  integer nk
  double precision r0(0:nk-1,0:nk-1), dc0(0:nk-1,0:nk-1)
  double precision dw0(0:nk-1,0:nk-1), d0(0:nk-1,0:nk-1)
  integer i, j

  do i = 0, nk-1
     d0(i,i) = 0
     do j = i+1, nk-1
        !          d0(i,j) = (dlog(r0(i,j))-dc0(i,j))/dw0(i,j)

        d0(i,j) = (dexp(-r0(i,j)/3.d0)-dc0(i,j))/dw0(i,j)

        d0(j,i) = d0(i,j)
     enddo
  enddo
  return
end subroutine getd0_zundel


subroutine getdvec (ma, mb, xn, dc0, dw0, vec)
  implicit none
  integer, parameter :: wp=selected_real_kind(12,300)
  ! version for H5O2(+)
  integer nk, ma, mb
  parameter (nk=7)
  double precision xn(0:2,0:nk-1), dc0(0:nk-1,0:nk-1), dw0(0:nk-1,0:nk-1),   vec(0:3,0:ma+mb-1)
  integer i, j, k, ind(0:nk-1)
  double precision r0(0:nk-1,0:nk-1), d0(0:nk-1,0:nk-1), s0(0:nk-1,0:nk-1),   v0(0:ma+mb-1)
  !-----------------------------------------------------------------------
  vec = 0
  call getr0_zundel (nk, xn, r0)
  call getd0_zundel (nk, r0, dc0, dw0, d0)
  ! vector factor xn(0:2,0) (species X)
  ind = (/ 1, 2, 3, 4, 5, 6, 0 /)
  do i = 0, nk-1
     do j = 0, nk-1
        s0(i,j) = d0(ind(i),ind(j))
     enddo
  enddo
  call getv_x4y2z (ma, s0, v0)
  do k = 0, ma-1
     vec(0:2,k) = vec(0:2,k)+xn(0:2,0)*v0(k)
     vec(3,k) = vec(3,k)+v0(k)
  enddo
  ! vector factor xn(0:2,1) (species X)
  ind = (/ 0, 2, 3, 4, 5, 6, 1 /)
  do i = 0, nk-1
     do j = 0, nk-1
        s0(i,j) = d0(ind(i),ind(j))
     enddo
  enddo
  call getv_x4y2z (ma, s0, v0)
  do k = 0, ma-1
     vec(0:2,k) = vec(0:2,k)+xn(0:2,1)*v0(k)
     vec(3,k) = vec(3,k)+v0(k)
  enddo
  ! vector factor xn(0:2,2) (species X)
  ind = (/ 0, 1, 3, 4, 5, 6, 2 /)
  do i = 0, nk-1
     do j = 0, nk-1
        s0(i,j) = d0(ind(i),ind(j))
     enddo
  enddo
  call getv_x4y2z (ma, s0, v0)
  do k = 0, ma-1
     vec(0:2,k) = vec(0:2,k)+xn(0:2,2)*v0(k)
     vec(3,k) = vec(3,k)+v0(k)
  enddo
  ! vector factor xn(0:2,3) (species X)
  ind = (/ 0, 1, 2, 4, 5, 6, 3 /)
  do i = 0, nk-1
     do j = 0, nk-1
        s0(i,j) = d0(ind(i),ind(j))
     enddo
  enddo
  call getv_x4y2z (ma, s0, v0)
  do k = 0, ma-1
     vec(0:2,k) = vec(0:2,k)+xn(0:2,3)*v0(k)
     vec(3,k) = vec(3,k)+v0(k)
  enddo
  ! vector factor xn(0:2,4) (species X)
  ind = (/ 0, 1, 2, 3, 5, 6, 4 /)
  do i = 0, nk-1
     do j = 0, nk-1
        s0(i,j) = d0(ind(i),ind(j))
     enddo
  enddo
  call getv_x4y2z (ma, s0, v0)
  do k = 0, ma-1
     vec(0:2,k) = vec(0:2,k)+xn(0:2,4)*v0(k)
     vec(3,k) = vec(3,k)+v0(k)
  enddo
  ! vector factor xn(0:2,5) (species Y)
  ind = (/ 0, 1, 2, 3, 4, 6, 5 /)
  do i = 0, nk-1
     do j = 0, nk-1
        s0(i,j) = d0(ind(i),ind(j))
     enddo
  enddo
  call getv_x5yz (mb, s0, v0)
  do k = 0, mb-1
     vec(0:2,ma+k) = vec(0:2,ma+k)+xn(0:2,5)*v0(k)
     vec(3,ma+k) = vec(3,ma+k)+v0(k)
  enddo
  ! vector factor xn(0:2,6) (species Y)
  s0 = d0
  call getv_x5yz (mb, s0, v0)
  do k = 0, mb-1
     vec(0:2,ma+k) = vec(0:2,ma+k)+xn(0:2,6)*v0(k)
     vec(3,ma+k) = vec(3,ma+k)+v0(k)
  enddo
  return
end subroutine getdvec



subroutine getv_x4y2z (m, d, vec)
  implicit none
  integer, parameter :: wp=selected_real_kind(12,300)
  integer, parameter :: nk=7
  integer m
  double precision d(0:nk-1,0:nk-1), vec(0:m-1)
  ! version for molecule X4Y2Z
  ! MolienSeries(0:*):
  ! #Primaries(1:*):
  ! #Secondaries(1:*):
  integer, parameter :: l0=1, l1=l0+5, l2=l1+26, l3=l2+117, l4=l3+501, l5=l4+1975-113, l6=-1, l7=-2
  !! Note, we have included only 86 of the 199 secondaries at degree 5.
  integer, parameter :: np1=5, np2=7, np3=4, np4=4, np5=0, np6=1, np7=0
  integer, parameter :: ns1=0, ns2=4, ns3=23, ns4=71, ns5=199, ns6=548, ns7=1323
  double precision x(0:np1-1), y(0:np2-1), z(0:np3-1), u(0:np4-1), w(0:np6-1)
  double precision x2(0:np1-1),x3(0:np1-1),x4(0:np1-1),x5(0:np1-1), &
       x6(0:np1-1), x7(0:np1-1),  y2(0:np2-1), y3(0:np2-1),z2(0:np2-1)
  double precision ys(0:ns2-1), zs(0:ns3-1), &
       us(0:ns4-1), vs(0:ns5-1), ws(0:ns6-1), w7s(0:ns7-1)
  integer mdeg, i, j, i0, i1, i2, j0, j1, k0
  double precision d2(0:nk-1,0:nk-1), d3(0:nk-1,0:nk-1), &
       d4(0:nk-1,0:nk-1), d5(0:nk-1,0:nk-1), d6(0:nk-1,0:nk-1), &
       d7(0:nk-1,0:nk-1)
  double precision t0
  double precision her2_zundel, her3_zundel, her4_zundel, her5_zundel, her6_zundel, her7_zundel
  her2_zundel(t0) = (4*t0**2-2)/dsqrt(dble(8*2))
  her3_zundel(t0) = (8*t0**2-12)*t0/dsqrt(dble(16*6))
  her4_zundel(t0) = ((16*t0**2-48)*t0**2+12)/dsqrt(dble(32*24))
  her5_zundel(t0) = ((32*t0**2-160)*t0**2+120)*t0/dsqrt(dble(64*120))
  her6_zundel(t0) = (((64*t0**2-480)*t0**2+720)*t0**2-120)/dsqrt(dble(128*720))
  her7_zundel(t0) = (((128*t0**2-1344)*t0**2+3360)*t0**2-1680)*t0/dsqrt(dble(256*5040))
  !-----------------------------------------------------------------------
  !! We don't have the secondaries at degrees 5 and up yet.
  !-----------------------------------------------------------------------
  ! Test for compatibility, set mdeg
  select case (m)
  case (l0)
     mdeg = 0
  case (l1)
     mdeg = 1
  case (l2)
     mdeg = 2
  case (l3)
     mdeg = 3
  case (l4)
     mdeg = 4
  case (l5)
     mdeg = 5
  case (l6)
     mdeg = 6
  case (l7)
     mdeg = 7
  case default
     stop 'getv - wrong dimension'
  endselect
  ! auxiliary distances
  do i = 0, nk-1
     do j = i+1, nk-1
        d2(i,j) = her2_zundel(d(i,j))
        d2(j,i) = d2(i,j)
        d3(i,j) = her3_zundel(d(i,j))
        d3(j,i) = d3(i,j)
        d4(i,j) = her4_zundel(d(i,j))
        d4(j,i) = d4(i,j)
        d5(i,j) = her5_zundel(d(i,j))
        d5(j,i) = d5(i,j)
        d6(i,j) = her6_zundel(d(i,j))
        d6(j,i) = d6(i,j)
        d7(i,j) = her7_zundel(d(i,j))
        d7(j,i) = d7(i,j)
     enddo
  enddo
  ! Primary Invariants
  x = 0 ; y = 0 ; z = 0 ; u = 0 ;  w = 0
  do i0 = 0, 3
     t0 = 0
     do i1 = 0, 3
        if (i1.ne.i0) then
           t0 = t0+d(i0,i1)/3
           x(0) = x(0)+d(i0,i1)/6
           y(0) = y(0)+d2(i0,i1)/6
           z(0) = z(0)+d3(i0,i1)/6
        endif
     enddo
     y(1) = y(1)+her2_zundel(t0)/4
     z(1) = z(1)+her3_zundel(t0)/4
     u(0) = u(0)+her4_zundel(t0)/4
  enddo
  x(1) = sum(d(0:3,4:5))/8
  do i0 = 0, 3
     y(2) = y(2)+her2_zundel(sum(d(i0,4:5))/2)/4
     z(2) = z(2)+her3_zundel(sum(d(i0,4:5))/2)/4
     u(1) = u(1)+her4_zundel(sum(d(i0,4:5))/2)/4
  enddo
  do j0 = 4, 5
     y(3) = y(3)+her2_zundel(sum(d(0:3,j0))/4)/2
  enddo
  y(4) = sum(d2(0:3,4:5))/8
  u(2) = sum(d4(0:3,4:5))/8
  w(0) = sum(d6(0:3,4:5))/8
  x(2) = d(4,5)
  x(3) = sum(d(0:3,6))/4
  y(5) = sum(d2(0:3,6))/4
  z(3) = sum(d3(0:3,6))/4
  u(3) = sum(d4(0:3,6))/4
  x(4) = sum(d(4:5,6))/2
  y(6) = sum(d2(4:5,6))/2
  ! Required powers
  do i = 0, np1-1
     x2(i) = her2_zundel(x(i))
     x3(i) = her3_zundel(x(i))
     x4(i) = her4_zundel(x(i))
     x5(i) = her5_zundel(x(i))
     x6(i) = her6_zundel(x(i))
     x7(i) = her7_zundel(x(i))
  enddo
  do i = 0, np2-1
     y2(i) = her2_zundel(y(i))
     y3(i) = her3_zundel(y(i))
  enddo
  do i = 0, np3-1
     z2(i) = her2_zundel(z(i))
  enddo
  ! Secondary Invariants
  !! reducible: us(0:8), vs(0:85), ws(0:447)
  !! Note: We haven't included them all yet
  ys = 0 ; zs = 0 ; us = 0 ; vs = 0 ; ws = 0 ; w7s = 0
  ! Irreducible secondaries
  do i0 = 0, 3
     do i1 = 0, 3
        if (i1.ne.i0) then
           us(9) = us(9)+d4(i0,i1)/12
           do i2 = 0, 3
              if (i2.ne.i1.and.i2.ne.i0) then
                 zs(0) = zs(0)+d2(i0,i1)*d(i0,i2)/24
              endif
           enddo
        endif
     enddo
  enddo
  do j0 = 4, 5
     do i0 = 0, 3
        zs(1) = zs(1)+d3(i0,j0)/8
        do i1 = 0, 3
           if (i1.ne.i0) then
              ys(0) = ys(0)+d(i0,i1)*d(i0,j0)/24
              zs(2) = zs(2)+d2(i0,i1)*d(i0,j0)/24
              zs(3) = zs(3)+d(i0,i1)*d2(i0,j0)/24
              zs(4) = zs(4)+d(i0,i1)*d(i0,j0)*d(i1,j0)/24
              zs(5) = zs(5)+d2(i0,j0)*d(i1,j0)/24
              us(10) = us(10)+d3(i0,i1)*d(i0,j0)/24
              us(11) = us(11)+d2(i0,i1)*d2(i0,j0)/24
              us(12) = us(12)+d(i0,i1)*d3(i0,j0)/24
              us(13) = us(13)+d2(i0,i1)*d(i0,j0)*d(i1,j0)/24
              us(14) = us(14)+d(i0,i1)*d2(i0,j0)*d(i1,j0)/24
              us(15) = us(15)+d3(i0,j0)*d(i1,j0)/24
              us(16) = us(16)+d2(i0,j0)*d2(i1,j0)/24
              do i2 = 0, 3
                 if (i2.ne.i1.and.i2.ne.i0) then
                    zs(6) = zs(6)+d(i0,i1)*d(i0,i2)*d(i0,j0)/48
                    zs(7) = zs(7)+d(i0,i2)*d(i0,j0)*d(i1,j0)/48
                    us(17) = us(17)+d2(i0,i1)*d(i0,i2)*d(i0,j0)/48
                    us(18) = us(18)+d2(i0,i1)*d(i1,i2)*d(i0,j0)/48
                    us(19) = us(19)+d(i0,i1)*d(i0,i2)*d2(i0,j0)/48
                    us(20) = us(20)+d(i0,i1)*d(i0,i2)*d(i0,j0)*d(i1,j0)/48
                    us(21) = us(21)+d2(i0,i2)*d(i0,j0)*d(i1,j0)/48
                    us(22) = us(22)+d(i0,i2)*d(i1,i2)*d(i0,j0)*d(i1,j0)/48
                    us(23) = us(23)+d(i0,i2)*d2(i0,j0)*d(i1,j0)/48
                    us(24) = us(24)+d(i1,i2)*d2(i0,j0)*d(i1,j0)/48
                 endif
              enddo
           endif
        enddo
     enddo
  enddo
  do j0 = 4, 5
     do j1 = 4, 5
        if (j1.ne.j0) then
           do i0 = 0, 3
              us(25) = us(25)+d3(i0,j0)*d(i0,j1)/8
              do i1 = 0, 3
                 if (i1.ne.i0) then
                    zs(8) = zs(8)+d(i0,i1)*d(i0,j0)*d(i0,j1)/24
                    zs(9) = zs(9)+d(i0,i1)*d(i1,j0)*d(i0,j1)/24
                    us(26) = us(26)+d2(i0,i1)*d(i0,j0)*d(i0,j1)/24
                    us(27) = us(27)+d(i0,i1)*d2(i0,j0)*d(i0,j1)/24
                    us(28) = us(28)+d2(i0,i1)*d(i1,j0)*d(i0,j1)/24
                    us(29) = us(29)+d(i0,i1)*d(i0,j0)*d(i1,j0)*d(i0,j1)/24
                    us(30) = us(30)+d2(i0,j0)*d(i1,j0)*d(i0,j1)/24
                    do i2 = 0, 3
                       if (i2.ne.i1.and.i2.ne.i0) then
                          us(31) = us(31)+d(i0,i1)*d(i0,i2)*d(i0,j0)*d(i0,j1)/48
                       endif
                    enddo
                 endif
              enddo
           enddo
        endif
     enddo
  enddo
  k0 = 6
  do i0 = 0, 3
     do i1 = 0, 3
        if (i1.ne.i0) then
           ys(1) = ys(1)+d(i0,i1)*d(i0,k0)/12
           zs(10) = zs(10)+d2(i0,i1)*d(i0,k0)/12
           zs(11) = zs(11)+d(i0,i1)*d2(i0,k0)/12
           zs(12) = zs(12)+d(i0,i1)*d(i0,k0)*d(i1,k0)/12
           us(32) = us(32)+d3(i0,i1)*d(i0,k0)/12
           us(33) = us(33)+d2(i0,i1)*d2(i0,k0)/12
           us(34) = us(34)+d(i0,i1)*d3(i0,k0)/12
           us(35) = us(35)+d2(i0,i1)*d(i0,k0)*d(i1,k0)/12
           do i2 = 0, 3
              if (i2.ne.i1.and.i2.ne.i0) then
                 zs(13) = zs(13)+d(i0,i1)*d(i0,i2)*d(i0,k0)/24
                 us(36) = us(36)+d2(i0,i1)*d(i0,i2)*d(i0,k0)/24
                 us(37) = us(37)+d2(i0,i1)*d(i1,i2)*d(i0,k0)/24
                 us(38) = us(38)+d(i0,i1)*d(i0,i2)*d2(i0,k0)/24
              endif
           enddo
        endif
     enddo
  enddo
  do j0 = 4, 5
     do i0 = 0, 3
        ys(2) = ys(2)+d(i0,j0)*d(i0,k0)/8
        ys(3) = ys(3)+d(i0,j0)*d(j0,k0)/8
        zs(14) = zs(14)+d2(i0,j0)*d(i0,k0)/8
        zs(15) = zs(15)+d(i0,j0)*d2(i0,k0)/8
        zs(16) = zs(16)+d(i0,j0)*d(i0,k0)*d(j0,k0)/8
        zs(17) = zs(17)+d2(i0,j0)*d(j0,k0)/8
        us(39) = us(39)+d2(i0,j0)*d2(i0,k0)/8
        us(40) = us(40)+d(i0,j0)*d3(i0,k0)/8
        us(41) = us(41)+d3(i0,j0)*d(i0,k0)/8
        us(42) = us(42)+d3(i0,j0)*d(j0,k0)/8
        us(43) = us(43)+d2(i0,j0)*d(i0,k0)*d(j0,k0)/8
        us(44) = us(44)+d(i0,j0)*d2(i0,k0)*d(j0,k0)/8
        do i1 = 0, 3
           if (i1.ne.i0) then
              zs(18) = zs(18)+d(i0,i1)*d(i0,j0)*d(i0,k0)/24
              zs(19) = zs(19)+d(i0,i1)*d(i1,j0)*d(i0,k0)/24
              zs(20) = zs(20)+d(i0,j0)*d(i1,j0)*d(i0,k0)/24
              zs(21) = zs(21)+d(i0,i1)*d(i0,j0)*d(j0,k0)/24
              us(45) = us(45)+d2(i0,i1)*d(i0,j0)*d(i0,k0)/24
              us(46) = us(46)+d(i0,i1)*d2(i0,j0)*d(i0,k0)/24
              us(47) = us(47)+d2(i0,i1)*d(i1,j0)*d(i0,k0)/24
              us(48) = us(48)+d(i0,i1)*d(i0,j0)*d(i1,j0)*d(i0,k0)/24
              us(49) = us(49)+d2(i0,j0)*d(i1,j0)*d(i0,k0)/24
              us(50) = us(50)+d(i0,i1)*d2(i1,j0)*d(i0,k0)/24
              us(51) = us(51)+d(i0,j0)*d2(i1,j0)*d(i0,k0)/24
              us(52) = us(52)+d(i0,i1)*d(i0,j0)*d2(i0,k0)/24
              us(53) = us(53)+d(i0,i1)*d(i1,j0)*d2(i0,k0)/24
              us(54) = us(54)+d(i0,j0)*d(i1,j0)*d2(i0,k0)/24
              us(55) = us(55)+d(i0,j0)*d(i1,j0)*d(i0,k0)*d(i1,k0)/24
              us(56) = us(56)+d2(i0,i1)*d(i0,j0)*d(j0,k0)/24
              us(57) = us(57)+d(i0,i1)*d2(i0,j0)*d(j0,k0)/24
              us(58) = us(58)+d(i0,i1)*d(i0,j0)*d(i1,j0)*d(j0,k0)/24
              us(59) = us(59)+d(i0,i1)*d(i0,j0)*d(i0,k0)*d(j0,k0)/24
              us(60) = us(60)+d(i0,i1)*d(i1,j0)*d(i0,k0)*d(j0,k0)/24
              do i2 = 0, 3
                 if (i2.ne.i1.and.i2.ne.i0) then
                    us(61) = us(61)+d(i0,i1)*d(i0,i2)*d(i0,j0)*d(i0,k0)/48
                    us(62) = us(62)+d(i0,i1)*d(i0,i2)*d(i1,j0)*d(i0,k0)/48
                    us(63) = us(63)+d(i0,i2)*d(i0,j0)*d(i1,j0)*d(i0,k0)/48
                    us(64) = us(64)+d(i1,i2)*d(i0,j0)*d(i1,j0)*d(i0,k0)/48
                    us(65) = us(65)+d(i0,i1)*d(i0,i2)*d(i0,j0)*d(j0,k0)/48
                 endif
              enddo
           endif
        enddo
     enddo
  enddo
  do j0 = 4, 5
     do j1 = 4, 5
        if (j1.ne.j0) then
           do i0 = 0, 3
              zs(22) = zs(22)+d(i0,j0)*d(i0,j1)*d(i0,k0)/8
              us(66) = us(66)+d2(i0,j0)*d(i0,j1)*d(i0,k0)/8
              us(67) = us(67)+d(i0,j0)*d(i0,j1)*d2(i0,k0)/8
              us(68) = us(68)+d2(i0,j0)*d(i0,j1)*d(j0,k0)/8
              do i1 = 0, 3
                 if (i1.ne.i0) then
                    us(69) = us(69)+d(i0,i1)*d(i0,j0)*d(i0,j1)*d(i0,k0)/24
                    us(70) = us(70)+d(i0,i1)*d(i1,j0)*d(i0,j1)*d(i0,k0)/24
                    do i2 = 0, 3
                       if (i2.ne.i1.and.i2.ne.i0) then
                       endif
                    enddo
                 endif
              enddo
           enddo
        endif
     enddo
  enddo
  ! Reducible secondaries
  !! Should be us(0:8), vs(0:85), ws(0:447)
  !! Note. ws(0:*) is incomplete.
  us(0) = her2_zundel(ys(0))
  us(1) = ys(0)*ys(1)
  us(2) = her2_zundel(ys(1))
  us(3:4) = ys(0:1)*ys(2)
  us(5) = her2_zundel(ys(2))
  us(6:8) = ys(0:2)*ys(3)
  ! her2_zundel(ys(3)) is not independent
  vs(0:3) = ys(0:3)*zs(0)
  vs(4:7) = ys(0:3)*zs(1)
  vs(8:11) = ys(0:3)*zs(2)
  vs(12:15) = ys(0:3)*zs(3)
  vs(16:19) = ys(0:3)*zs(4)
  vs(20:22) = ys(0:2)*zs(5)
  vs(23:26) = ys(0:3)*zs(6)
  vs(27:29) = ys(0:2)*zs(7)
  vs(30:33) = ys(0:3)*zs(8)
  vs(34:37) = ys(0:3)*zs(9)
  vs(38:41) = ys(0:3)*zs(10)
  vs(42:45) = ys(0:3)*zs(11)
  vs(46:49) = ys(0:3)*zs(12)
  vs(50:53) = ys(0:3)*zs(13)
  vs(54:57) = ys(0:3)*zs(14)
  vs(58:61) = ys(0:3)*zs(15)
  vs(62:64) = ys(0:2)*zs(16)
  vs(65:67) = ys(0:2)*zs(17)
  vs(68:71) = ys(0:3)*zs(18)
  vs(72:75) = ys(0:3)*zs(19)
  vs(76:78) = ys(0:2)*zs(20)
  vs(79:81) = ys(0:2)*zs(21)
  vs(82:85) = ys(0:3)*zs(22)
  ! Compute vec(0:*).  The code was created using these parameters:
  ! MolienSeries(0:*): 1 5 26 117 501 1975-113 ...
  ! #Primaries(1:*):   5 7 4 4 0 1 0 0
  ! #Secondaries(1:*): 0 4 23 71 199 548 1323 1
  ! The MolienSeries partial sums (allowed size of vec) are:
  ! Molien Sums(0:*):  1 6 32 149 650 2625-113 ...
! constant term
  vec(0) = 1
  ! first degree terms
  if (1.le.mdeg) then
     vec(1) = x(0)
     vec(2) = x(1)
     vec(3) = x(2)
     vec(4) = x(3)
     vec(5) = x(4)
  endif
  ! second degree terms
  if (2.le.mdeg) then
     vec(6) = x2(0)
     vec(7:10) = x(0)*vec(2:5)
     vec(11) = x2(1)
     vec(12:14) = x(1)*vec(3:5)
     vec(15) = x2(2)
     vec(16:17) = x(2)*vec(4:5)
     vec(18) = x2(3)
     vec(19:19) = x(3)*vec(5:5)
     vec(20) = x2(4)
     vec(21:20) = x(4)*vec(6:5)
     vec(21) = y(0)
     vec(22) = y(1)
     vec(23) = y(2)
     vec(24) = y(3)
     vec(25) = y(4)
     vec(26) = y(5)
     vec(27) = y(6)
     vec(28:31) = ys(0:3)
  endif
  ! third degree terms
  if (3.le.mdeg) then
     vec(32) = x3(0)
     vec(33:36) = x2(0)*vec(2:5)
     vec(37:57) = x(0)*vec(11:31)
     vec(58) = x3(1)
     vec(59:61) = x2(1)*vec(3:5)
     vec(62:78) = x(1)*vec(15:31)
     vec(79) = x3(2)
     vec(80:81) = x2(2)*vec(4:5)
     vec(82:95) = x(2)*vec(18:31)
     vec(96) = x3(3)
     vec(97:97) = x2(3)*vec(5:5)
     vec(98:109) = x(3)*vec(20:31)
     vec(110) = x3(4)
     vec(111:110) = x2(4)*vec(6:5)
     vec(111:121) = x(4)*vec(21:31)
     vec(122) = z(0)
     vec(123) = z(1)
     vec(124) = z(2)
     vec(125) = z(3)
     vec(126:148) = zs(0:22)
  endif
  ! fourth degree terms
  if (4.le.mdeg) then
     vec(149) = x4(0)
     vec(150:153) = x3(0)*vec(2:5)
     vec(154:174) = x2(0)*vec(11:31)
     vec(175:265) = x(0)*vec(58:148)
     vec(266) = x4(1)
     vec(267:269) = x3(1)*vec(3:5)
     vec(270:286) = x2(1)*vec(15:31)
     vec(287:356) = x(1)*vec(79:148)
     vec(357) = x4(2)
     vec(358:359) = x3(2)*vec(4:5)
     vec(360:373) = x2(2)*vec(18:31)
     vec(374:426) = x(2)*vec(96:148)
     vec(427) = x4(3)
     vec(428:428) = x3(3)*vec(5:5)
     vec(429:440) = x2(3)*vec(20:31)
     vec(441:479) = x(3)*vec(110:148)
     vec(480) = x4(4)
     vec(481:480) = x3(4)*vec(6:5)
     vec(481:491) = x2(4)*vec(21:31)
     vec(492:518) = x(4)*vec(122:148)
     vec(519) = y2(0)
     vec(520:529) = y(0)*vec(22:31)
     vec(530) = y2(1)
     vec(531:539) = y(1)*vec(23:31)
     vec(540) = y2(2)
     vec(541:548) = y(2)*vec(24:31)
     vec(549) = y2(3)
     vec(550:556) = y(3)*vec(25:31)
     vec(557) = y2(4)
     vec(558:563) = y(4)*vec(26:31)
     vec(564) = y2(5)
     vec(565:569) = y(5)*vec(27:31)
     vec(570) = y2(6)
     vec(571:574) = y(6)*vec(28:31)
     vec(575) = u(0)
     vec(576) = u(1)
     vec(577) = u(2)
     vec(578) = u(3)
     vec(579:649) = us(0:70)
  endif
  ! fifth degree terms
  if (5.le.mdeg) then
     vec(650) = x5(0)
     vec(651:654) = x4(0)*vec(2:5)
     vec(655:675) = x3(0)*vec(11:31)
     vec(676:766) = x2(0)*vec(58:148)
     vec(767:1150) = x(0)*vec(266:649)
     vec(1151) = x5(1)
     vec(1152:1154) = x4(1)*vec(3:5)
     vec(1155:1171) = x3(1)*vec(15:31)
     vec(1172:1241) = x2(1)*vec(79:148)
     vec(1242:1534) = x(1)*vec(357:649)
     vec(1535) = x5(2)
     vec(1536:1537) = x4(2)*vec(4:5)
     vec(1538:1551) = x3(2)*vec(18:31)
     vec(1552:1604) = x2(2)*vec(96:148)
     vec(1605:1827) = x(2)*vec(427:649)
     vec(1828) = x5(3)
     vec(1829:1829) = x4(3)*vec(5:5)
     vec(1830:1841) = x3(3)*vec(20:31)
     vec(1842:1880) = x2(3)*vec(110:148)
     vec(1881:2050) = x(3)*vec(480:649)
     vec(2051) = x5(4)
     vec(2052:2051) = x4(4)*vec(6:5)
     vec(2052:2062) = x3(4)*vec(21:31)
     vec(2063:2089) = x2(4)*vec(122:148)
     vec(2090:2220) = x(4)*vec(519:649)
     vec(2221:2247) = y(0)*vec(122:148)
     vec(2248:2274) = y(1)*vec(122:148)
     vec(2275:2301) = y(2)*vec(122:148)
     vec(2302:2328) = y(3)*vec(122:148)
     vec(2329:2355) = y(4)*vec(122:148)
     vec(2356:2382) = y(5)*vec(122:148)
     vec(2383:2409) = y(6)*vec(122:148)
     vec(2410:2413) = z(0)*ys(0:3)
     vec(2414:2417) = z(1)*ys(0:3)
     vec(2418:2421) = z(2)*ys(0:3)
     vec(2422:2425) = z(3)*ys(0:3)
     vec(2426:2511) = vs(0:85)
     !! should be vec(2426:2624) = vs(0:198)
  endif
  return
end subroutine getv_x4y2z


subroutine getv_x5yz (m, d, vec)
  implicit none
  integer, parameter :: wp=selected_real_kind(12,300)
  integer, parameter :: nk=7
  integer m
  double precision d(0:nk-1,0:nk-1), vec(0:m-1)
  ! version for molecule X5YZ.
  ! MolienSeries(0:*): 1 4 17 68 265 977 3424 11361
  ! #Primaries(1:*):   4 4 4 4 4 1 0 0 ...
  ! #Secondaries(1:*): 0 3 16 54 165 482 1303
  integer, parameter :: l0=1, l1=l0+4, l2=l1+17, l3=l2+68, l4=l3+265, l5=l4+977, l6=l5+3424-482, l7=-1
  !! Incomplete at degree 6, and we haven't done degree 7.
  integer, parameter :: np1=4, np2=4, np3=4, np4=4, np5=4, np6=1,np7=0
  integer, parameter :: ns1=0, ns2=3, ns3=16, ns4=54, ns5=165, ns6=482, ns7=1303
  double precision x(0:np1-1), y(0:np2-1), z(0:np3-1), u(0:np4-1), v(0:np5-1), w(0:np6-1)
  double precision x2(0:np1-1), x3(0:np1-1), x4(0:np1-1), &
       x5(0:np1-1), x6(0:np1-1), x7(0:np1-1),y2(0:np2-1), y3(0:np2-1),z2(0:np2-1)
  double precision ys(0:ns2-1), zs(0:ns3-1), &
       us(0:ns4-1), vs(0:ns5-1), ws(0:ns6-1), w7s(0:ns7-1)
  integer mdeg, i, j, k, i0, i1, i2, i3, j0, k0
  double precision d2(0:nk-1,0:nk-1),d3(0:nk-1,0:nk-1),d4(0:nk-1,0:nk-1), &
       d5(0:nk-1,0:nk-1), d6(0:nk-1,0:nk-1), d7(0:nk-1,0:nk-1)
  double precision t0
  double precision her2_zundel, her3_zundel, her4_zundel, her5_zundel, her6_zundel, her7_zundel

  her2_zundel(t0) = (4*t0**2-2)/dsqrt(dble(8*2))
  her3_zundel(t0) = (8*t0**2-12)*t0/dsqrt(dble(16*6))
  her4_zundel(t0) = ((16*t0**2-48)*t0**2+12)/dsqrt(dble(32*24))
  her5_zundel(t0) = ((32*t0**2-160)*t0**2+120)*t0/dsqrt(dble(64*120))
  her6_zundel(t0) = (((64*t0**2-480)*t0**2+720)*t0**2-120)/dsqrt(dble(128*720))
  her7_zundel(t0) = (((128*t0**2-1344)*t0**2+3360)*t0**2-1680)*t0/dsqrt(dble(256*5040))
  !-----------------------------------------------------------------------
  !! We don't have the secondaries at degree 6 yet.
  !-----------------------------------------------------------------------
  ! Test for compatibility, set mdeg
  select case (m)
  case (l0)
     mdeg = 0
  case (l1)
     mdeg = 1
  case (l2)
     mdeg = 2
  case (l3)
     mdeg = 3
  case (l4)
     mdeg = 4
  case (l5)
     mdeg = 5
  case (l6)
     mdeg = 6
  case (l7)
     mdeg = 7
  case default
     stop 'getv - wrong dimension'
  endselect
  ! auxiliary distances
  do i = 0, nk-1
     do j = i+1, nk-1
        d2(i,j) = her2_zundel(d(i,j))
        d2(j,i) = d2(i,j)
        d3(i,j) = her3_zundel(d(i,j))
        d3(j,i) = d3(i,j)
        d4(i,j) = her4_zundel(d(i,j))
        d4(j,i) = d4(i,j)
        d5(i,j) = her5_zundel(d(i,j))
        d5(j,i) = d5(i,j)
        d6(i,j) = her6_zundel(d(i,j))
        d6(j,i) = d6(i,j)
        d7(i,j) = her7_zundel(d(i,j))
        d7(j,i) = d7(i,j)
     enddo
  enddo
  ! Primary Invariants
  x = 0 ; y = 0 ; z = 0 ; u = 0 ; v = 0 ; w = 0
  do i0 = 0, 4
     t0 = 0
     do i1 = 0, 4
        if (i1.ne.i0) then
           t0 = t0+d(i0,i1)/4
           x(0) = x(0)+d(i0,i1)/20
           y(0) = y(0)+d2(i0,i1)/20
           z(0) = z(0)+d3(i0,i1)/20
           u(0) = u(0)+d4(i0,i1)/20
           v(0) = v(0)+d5(i0,i1)/20
           w(0) = w(0)+d6(i0,i1)/20
        endif
     enddo
     y(1) = y(1)+her2_zundel(t0)/5
     z(1) = z(1)+her3_zundel(t0)/5
     u(1) = u(1)+her4_zundel(t0)/5
     v(1) = v(1)+her5_zundel(t0)/5
  enddo
  x(1) = sum(d(0:4,5))/5
  y(2) = sum(d2(0:4,5))/5
  z(2) = sum(d3(0:4,5))/5
  u(2) = sum(d4(0:4,5))/5
  v(2) = sum(d5(0:4,5))/5
  x(2) = sum(d(0:4,6))/5
  y(3) = sum(d2(0:4,6))/5
  z(3) = sum(d3(0:4,6))/5
  u(3) = sum(d4(0:4,6))/5
  v(3) = sum(d5(0:4,6))/5
  x(3) = d(5,6)
  ! Required powers
  do i = 0, np1-1
     x2(i) = her2_zundel(x(i))
     x3(i) = her3_zundel(x(i))
     x4(i) = her4_zundel(x(i))
     x5(i) = her5_zundel(x(i))
     x6(i) = her6_zundel(x(i))
     x7(i) = her7_zundel(x(i))
  enddo
  do i = 0, np2-1
     y2(i) = her2_zundel(y(i))
     y3(i) = her3_zundel(y(i))
  enddo
  do i = 0, np3-1
     z2(i) = her2_zundel(z(i))
  enddo
  ! Secondary Invariants
  !! ys(0:2), zs(0:15), us(0:53), vs(0:164), ws(0:481), w7s(0:1302)
  !! reducible: us(0:5), vs(0:47), ws(0:..)
  !! Note: at this time the secondaries at degree 6 are incomplete.
  ys = 0 ; zs = 0 ; us = 0 ; vs = 0 ; ws = 0 ; w7s = 0
  ! Irreducible secondaries
  do i0 = 0, 4
     do i1 = 0, 4
        if (i1.ne.i0) then
           do i2 = 0, 4
              if (i2.ne.i1.and.i2.ne.i0) then
                 zs(0) = zs(0)+d2(i0,i1)*d(i0,i2)/60
                 zs(1) = zs(1)+d(i0,i1)*d(i0,i2)*d(i1,i2)/60
                 us(6) = us(6)+d3(i0,i1)*d(i0,i2)/60
                 us(7) = us(7)+d2(i0,i1)*d2(i0,i2)/60
                 us(8) = us(8)+d2(i0,i1)*d(i0,i2)*d(i1,i2)/60
                 vs(48) = vs(48)+d4(i0,i1)*d(i0,i2)/60
                 vs(49) = vs(49)+d3(i0,i1)*d2(i0,i2)/60
                 vs(50) = vs(50)+d3(i0,i1)*d(i0,i2)*d(i1,i2)/60
                 vs(51) = vs(51)+d2(i0,i1)*d2(i0,i2)*d(i1,i2)/60
                 do i3 = 0, 4
                    if (i3.ne.i0.and.i3.ne.i1.and.i3.ne.i2) then
                       us(9) = us(9)+d2(i0,i1)*d(i0,i2)*d(i0,i3)/120
                       us(10) = us(10)+d2(i0,i1)*d(i1,i2)*d(i0,i3)/120
                       vs(52) = vs(52)+d3(i0,i1)*d(i0,i2)*d(i0,i3)/120
                       vs(53) = vs(53)+d2(i0,i1)*d2(i0,i2)*d(i0,i3)/120
                       vs(54) = vs(54)+d3(i0,i1)*d(i1,i2)*d(i0,i3)/120
                       vs(55) = vs(55)+d2(i0,i1)*d(i0,i2)*d(i1,i2)*d(i0,i3)/120
                    endif
                 enddo
              endif
           enddo
        endif
     enddo
  enddo
  j0 = 5 ; k0 = 6
  do i0 = 0, 4
     do i1 = 0, 4
        if (i1.ne.i0) then
           ys(0) = ys(0)+d(i0,i1)*d(i0,j0)/20
           ys(1) = ys(1)+d(i0,i1)*d(i0,k0)/20
           zs(2) = zs(2)+d2(i0,i1)*d(i0,j0)/20
           zs(3) = zs(3)+d(i0,i1)*d2(i0,j0)/20
           zs(4) = zs(4)+d(i0,i1)*d(i0,j0)*d(i1,j0)/20
           zs(5) = zs(5)+d2(i0,i1)*d(i0,k0)/20
           zs(6) = zs(6)+d(i0,i1)*d2(i0,k0)/20
           zs(7) = zs(7)+d(i0,i1)*d(i0,k0)*d(i1,k0)/20
           us(11) = us(11)+d3(i0,i1)*d(i0,j0)/20
           us(12) = us(12)+d2(i0,i1)*d2(i0,j0)/20
           us(13) = us(13)+d(i0,i1)*d3(i0,j0)/20
           us(14) = us(14)+d2(i0,i1)*d(i0,j0)*d(i1,j0)/20
           us(15) = us(15)+d(i0,i1)*d2(i0,j0)*d(i1,j0)/20
           us(16) = us(16)+d3(i0,i1)*d(i0,k0)/20
           us(17) = us(17)+d2(i0,i1)*d2(i0,k0)/20
           us(18) = us(18)+d(i0,i1)*d3(i0,k0)/20
           us(19) = us(19)+d2(i0,i1)*d(i0,k0)*d(i1,k0)/20
           us(20) = us(20)+d(i0,i1)*d2(i0,k0)*d(i1,k0)/20
           vs(56) = vs(56)+d4(i0,i1)*d(i0,j0)/20
           vs(57) = vs(57)+d3(i0,i1)*d2(i0,j0)/20
           vs(58) = vs(58)+d2(i0,i1)*d3(i0,j0)/20
           vs(59) = vs(59)+d(i0,i1)*d4(i0,j0)/20
           vs(60) = vs(60)+d3(i0,i1)*d(i0,j0)*d(i1,j0)/20
           vs(61) = vs(61)+d2(i0,i1)*d2(i0,j0)*d(i1,j0)/20
           vs(62) = vs(62)+d(i0,i1)*d3(i0,j0)*d(i1,j0)/20
           vs(63) = vs(63)+d4(i0,i1)*d(i0,k0)/20
           vs(64) = vs(64)+d3(i0,i1)*d2(i0,k0)/20
           vs(65) = vs(65)+d2(i0,i1)*d3(i0,k0)/20
           vs(66) = vs(66)+d(i0,i1)*d4(i0,k0)/20
           vs(67) = vs(67)+d3(i0,i1)*d(i0,k0)*d(i1,k0)/20
           vs(68) = vs(68)+d2(i0,i1)*d2(i0,k0)*d(i1,k0)/20
           vs(69) = vs(69)+d(i0,i1)*d3(i0,k0)*d(i1,k0)/20
           do i2 = 0, 4
              if (i2.ne.i1.and.i2.ne.i0) then
                 zs(8) = zs(8)+d(i0,i1)*d(i0,i2)*d(i0,j0)/60
                 zs(9) = zs(9)+d(i0,i1)*d(i1,i2)*d(i0,j0)/60
                 zs(10) = zs(10)+d(i0,i1)*d(i0,i2)*d(i0,k0)/60
                 zs(11) = zs(11)+d(i0,i1)*d(i1,i2)*d(i0,k0)/60
                 us(21) = us(21)+d2(i0,i1)*d(i0,i2)*d(i0,j0)/60
                 us(22) = us(22)+d2(i0,i1)*d(i1,i2)*d(i0,j0)/60
                 us(23) = us(23)+d(i0,i1)*d(i0,i2)*d(i1,i2)*d(i0,j0)/60
                 us(24) = us(24)+d(i0,i1)*d2(i1,i2)*d(i0,j0)/60
                 us(25) = us(25)+d(i0,i1)*d(i0,i2)*d2(i0,j0)/60
                 us(26) = us(26)+d(i0,i1)*d(i1,i2)*d2(i0,j0)/60
                 us(27) = us(27)+d(i0,i1)*d(i0,i2)*d(i0,j0)*d(i1,j0)/60
                 us(28) = us(28)+d2(i0,i1)*d(i0,i2)*d(i0,k0)/60
                 us(29) = us(29)+d2(i0,i1)*d(i1,i2)*d(i0,k0)/60
                 us(30) = us(30)+d(i0,i1)*d(i0,i2)*d(i1,i2)*d(i0,k0)/60
                 us(31) = us(31)+d(i0,i1)*d2(i1,i2)*d(i0,k0)/60
                 us(32) = us(32)+d(i0,i1)*d(i0,i2)*d2(i0,k0)/60
                 us(33) = us(33)+d(i0,i1)*d(i1,i2)*d2(i0,k0)/60
                 us(34) = us(34)+d(i0,i1)*d(i0,i2)*d(i0,k0)*d(i1,k0)/60
                 vs(70) = vs(70)+d3(i0,i1)*d(i0,i2)*d(i0,j0)/60
                 vs(71) = vs(71)+d2(i0,i1)*d2(i0,i2)*d(i0,j0)/60
                 vs(72) = vs(72)+d3(i0,i1)*d(i1,i2)*d(i0,j0)/60
                 vs(73) = vs(73)+d2(i0,i1)*d(i0,i2)*d(i1,i2)*d(i0,j0)/60
                 vs(74) = vs(74)+d2(i0,i1)*d2(i1,i2)*d(i0,j0)/60
                 vs(75) = vs(75)+d(i0,i1)*d(i0,i2)*d2(i1,i2)*d(i0,j0)/60
                 vs(76) = vs(76)+d(i0,i1)*d3(i1,i2)*d(i0,j0)/60
                 vs(77) = vs(77)+d2(i0,i1)*d(i0,i2)*d2(i0,j0)/60
                 vs(78) = vs(78)+d2(i0,i1)*d(i1,i2)*d2(i0,j0)/60
                 vs(79) = vs(79)+d(i0,i1)*d(i0,i2)*d(i1,i2)*d2(i0,j0)/60
                 vs(80) = vs(80)+d(i0,i1)*d2(i1,i2)*d2(i0,j0)/60
                 vs(81) = vs(81)+d(i0,i1)*d(i0,i2)*d3(i0,j0)/60
                 vs(82) = vs(82)+d(i0,i1)*d(i1,i2)*d3(i0,j0)/60
                 vs(83) = vs(83)+d2(i0,i1)*d(i0,i2)*d(i0,j0)*d(i1,j0)/60
                 vs(84) = vs(84)+d(i0,i1)*d2(i0,i2)*d(i0,j0)*d(i1,j0)/60
                 vs(85) = vs(85)+d(i0,i1)*d(i0,i2)*d(i1,i2)*d(i0,j0)*d(i1,j0)/60
                 vs(86) = vs(86)+d(i0,i1)*d(i0,i2)*d2(i0,j0)*d(i1,j0)/60
                 vs(87) = vs(87)+d3(i0,i1)*d(i0,i2)*d(i0,k0)/60
                 vs(88) = vs(88)+d2(i0,i1)*d2(i0,i2)*d(i0,k0)/60
                 vs(89) = vs(89)+d3(i0,i1)*d(i1,i2)*d(i0,k0)/60
                 vs(90) = vs(90)+d2(i0,i1)*d(i0,i2)*d(i1,i2)*d(i0,k0)/60
                 vs(91) = vs(91)+d2(i0,i1)*d2(i1,i2)*d(i0,k0)/60
                 vs(92) = vs(92)+d(i0,i1)*d(i0,i2)*d2(i1,i2)*d(i0,k0)/60
                 vs(93) = vs(93)+d(i0,i1)*d3(i1,i2)*d(i0,k0)/60
                 vs(94) = vs(94)+d2(i0,i1)*d(i0,i2)*d2(i0,k0)/60
                 vs(95) = vs(95)+d2(i0,i1)*d(i1,i2)*d2(i0,k0)/60
                 vs(96) = vs(96)+d(i0,i1)*d(i0,i2)*d(i1,i2)*d2(i0,k0)/60
                 vs(97) = vs(97)+d(i0,i1)*d2(i1,i2)*d2(i0,k0)/60
                 vs(98) = vs(98)+d(i0,i1)*d(i0,i2)*d3(i0,k0)/60
                 vs(99) = vs(99)+d(i0,i1)*d(i1,i2)*d3(i0,k0)/60
                 vs(100) = vs(100)+d2(i0,i1)*d(i0,i2)*d(i0,k0)*d(i1,k0)/60
                 vs(101) = vs(101)+d(i0,i1)*d2(i0,i2)*d(i0,k0)*d(i1,k0)/60
                 vs(102) = vs(102)+d(i0,i1)*d(i0,i2)*d(i1,i2)*d(i0,k0)*d(i1,k0)/60
                 vs(103) = vs(103)+d(i0,i1)*d(i0,i2)*d2(i0,k0)*d(i1,k0)/60
                 do i3 = 0, 4
                    if (i3.ne.i0.and.i3.ne.i1.and.i3.ne.i2) then
                       us(35) = us(35)+d(i0,i1)*d(i0,i2)*d(i0,i3)*d(i0,j0)/120
                       us(36) = us(36)+d(i0,i1)*d(i1,i2)*d(i0,i3)*d(i0,j0)/120
                       us(37) = us(37)+d(i0,i1)*d(i0,i2)*d(i0,i3)*d(i0,k0)/120
                       us(38) = us(38)+d(i0,i1)*d(i1,i2)*d(i0,i3)*d(i0,k0)/120
                       vs(104) = vs(104)+d2(i0,i1)*d(i0,i2)*d(i0,i3)*d(i0,j0)/120
                       vs(105) = vs(105)+d2(i0,i1)*d(i1,i2)*d(i0,i3)*d(i0,j0)/120
                       vs(106) = vs(106)+d(i0,i1)*d(i0,i2)*d(i1,i2)*d(i0,i3)*d(i0,j0)/120
                       vs(107) = vs(107)+d(i0,i1)*d2(i1,i2)*d(i0,i3)*d(i0,j0)/120
                       vs(108) = vs(108)+d(i0,i1)*d(i1,i2)*d2(i0,i3)*d(i0,j0)/120
                       vs(109) = vs(109)+d(i0,i1)*d(i0,i2)*d(i0,i3)*d2(i0,j0)/120
                       vs(110) = vs(110)+d(i0,i1)*d(i1,i2)*d(i0,i3)*d2(i0,j0)/120
                       vs(111) = vs(111)+d2(i0,i1)*d(i0,i2)*d(i0,i3)*d(i0,k0)/120
                       vs(112) = vs(112)+d2(i0,i1)*d(i1,i2)*d(i0,i3)*d(i0,k0)/120
                       vs(113) = vs(113)+d(i0,i1)*d(i0,i2)*d(i1,i2)*d(i0,i3)*d(i0,k0)/120
                       vs(114) = vs(114)+d(i0,i1)*d2(i1,i2)*d(i0,i3)*d(i0,k0)/120
                       vs(115) = vs(115)+d(i0,i1)*d(i1,i2)*d2(i0,i3)*d(i0,k0)/120
                       vs(116) = vs(116)+d(i0,i1)*d(i0,i2)*d(i0,i3)*d2(i0,k0)/120
                       vs(117) = vs(117)+d(i0,i1)*d(i1,i2)*d(i0,i3)*d2(i0,k0)/120
                    endif
                 enddo
              endif
           enddo
        endif
     enddo
  enddo
  do i0 = 0, 4
     ys(2) = ys(2)+d(i0,j0)*d(i0,k0)/5
     zs(12) = zs(12)+d2(i0,j0)*d(i0,k0)/5
     zs(13) = zs(13)+d(i0,j0)*d2(i0,k0)/5
     do i1 = 0, 4
        if (i1.ne.i0) then
           zs(14) = zs(14)+d(i0,i1)*d(i0,j0)*d(i0,k0)/20
           zs(15) = zs(15)+d(i0,i1)*d(i1,j0)*d(i0,k0)/20
           us(39) = us(39)+d2(i0,i1)*d(i0,j0)*d(i0,k0)/20
           us(40) = us(40)+d(i0,i1)*d2(i0,j0)*d(i0,k0)/20
           us(41) = us(41)+d3(i0,j0)*d(i0,k0)/20
           us(42) = us(42)+d2(i0,i1)*d(i1,j0)*d(i0,k0)/20
           us(43) = us(43)+d(i0,i1)*d(i0,j0)*d(i1,j0)*d(i0,k0)/20
           us(44) = us(44)+d(i0,i1)*d2(i1,j0)*d(i0,k0)/20
           us(45) = us(45)+d(i0,i1)*d(i0,j0)*d2(i0,k0)/20
           us(46) = us(46)+d2(i0,j0)*d2(i0,k0)/20
           us(47) = us(47)+d(i0,i1)*d(i1,j0)*d2(i0,k0)/20
           us(48) = us(48)+d(i0,j0)*d3(i0,k0)/20
           us(49) = us(49)+d(i0,i1)*d(i0,j0)*d(i0,k0)*d(i1,k0)/20
           vs(118) = vs(118)+d3(i0,i1)*d(i0,j0)*d(i0,k0)/20
           vs(119) = vs(119)+d2(i0,i1)*d2(i0,j0)*d(i0,k0)/20
           vs(120) = vs(120)+d(i0,i1)*d3(i0,j0)*d(i0,k0)/20
           vs(121) = vs(121)+d4(i0,j0)*d(i0,k0)/20
           vs(122) = vs(122)+d3(i0,i1)*d(i1,j0)*d(i0,k0)/20
           vs(123) = vs(123)+d2(i0,i1)*d(i0,j0)*d(i1,j0)*d(i0,k0)/20
           vs(124) = vs(124)+d(i0,i1)*d2(i0,j0)*d(i1,j0)*d(i0,k0)/20
           vs(125) = vs(125)+d2(i0,i1)*d2(i1,j0)*d(i0,k0)/20
           vs(126) = vs(126)+d(i0,i1)*d(i0,j0)*d2(i1,j0)*d(i0,k0)/20
           vs(127) = vs(127)+d2(i0,i1)*d(i0,j0)*d2(i0,k0)/20
           vs(128) = vs(128)+d(i0,i1)*d2(i0,j0)*d2(i0,k0)/20
           vs(129) = vs(129)+d3(i0,j0)*d2(i0,k0)/20
           vs(130) = vs(130)+d2(i0,i1)*d(i1,j0)*d2(i0,k0)/20
           vs(131) = vs(131)+d(i0,i1)*d(i0,j0)*d(i1,j0)*d2(i0,k0)/20
           vs(132) = vs(132)+d(i0,i1)*d2(i1,j0)*d2(i0,k0)/20
           vs(133) = vs(133)+d(i0,i1)*d(i0,j0)*d3(i0,k0)/20
           vs(134) = vs(134)+d2(i0,j0)*d3(i0,k0)/20
           vs(135) = vs(135)+d(i0,i1)*d(i1,j0)*d3(i0,k0)/20
           vs(136) = vs(136)+d(i0,j0)*d4(i0,k0)/20
           vs(137) = vs(137)+d2(i0,i1)*d(i0,j0)*d(i0,k0)*d(i1,k0)/20
           vs(138) = vs(138)+d(i0,i1)*d2(i0,j0)*d(i0,k0)*d(i1,k0)/20
           vs(139) = vs(139)+d(i0,i1)*d(i0,j0)*d2(i0,k0)*d(i1,k0)/20
           do i2 = 0, 4
              if (i2.ne.i1.and.i2.ne.i0) then
                 us(50) = us(50)+d(i0,i1)*d(i0,i2)*d(i0,j0)*d(i0,k0)/60
                 us(51) = us(51)+d(i0,i1)*d(i1,i2)*d(i0,j0)*d(i0,k0)/60
                 us(52) = us(52)+d(i0,i1)*d(i0,i2)*d(i1,j0)*d(i0,k0)/60
                 us(53) = us(53)+d(i0,i1)*d(i1,i2)*d(i1,j0)*d(i0,k0)/60
                 vs(140) = vs(140)+d2(i0,i1)*d(i0,i2)*d(i0,j0)*d(i0,k0)/60
                 vs(141) = vs(141)+d2(i0,i1)*d(i1,i2)*d(i0,j0)*d(i0,k0)/60
                 vs(142) = vs(142)+d(i0,i1)*d(i0,i2)*d(i1,i2)*d(i0,j0)*d(i0,k0)/60
                 vs(143) = vs(143)+d(i0,i1)*d2(i1,i2)*d(i0,j0)*d(i0,k0)/60
                 vs(144) = vs(144)+d(i0,i1)*d(i0,i2)*d2(i0,j0)*d(i0,k0)/60
                 vs(145) = vs(145)+d(i0,i1)*d(i1,i2)*d2(i0,j0)*d(i0,k0)/60
                 vs(146) = vs(146)+d2(i0,i1)*d(i0,i2)*d(i1,j0)*d(i0,k0)/60
                 vs(147) = vs(147)+d(i0,i1)*d2(i0,i2)*d(i1,j0)*d(i0,k0)/60
                 vs(148) = vs(148)+d2(i0,i1)*d(i1,i2)*d(i1,j0)*d(i0,k0)/60
                 vs(149) = vs(149)+d(i0,i1)*d(i0,i2)*d(i1,i2)*d(i1,j0)*d(i0,k0)/60
                 vs(150) = vs(150)+d2(i0,i2)*d(i1,i2)*d(i1,j0)*d(i0,k0)/60
                 vs(151) = vs(151)+d(i0,i1)*d2(i1,i2)*d(i1,j0)*d(i0,k0)/60
                 vs(152) = vs(152)+d(i0,i1)*d(i0,i2)*d(i0,j0)*d(i1,j0)*d(i0,k0)/60
                 vs(153) = vs(153)+d(i0,i1)*d(i1,i2)*d(i0,j0)*d(i1,j0)*d(i0,k0)/60
                 vs(154) = vs(154)+d(i0,i1)*d(i0,i2)*d2(i1,j0)*d(i0,k0)/60
                 vs(155) = vs(155)+d(i0,i1)*d(i0,i2)*d(i1,j0)*d(i2,j0)*d(i0,k0)/60
                 vs(156) = vs(156)+d(i0,i1)*d(i0,i2)*d(i0,j0)*d2(i0,k0)/60
                 vs(157) = vs(157)+d(i0,i1)*d(i1,i2)*d(i0,j0)*d2(i0,k0)/60
                 vs(158) = vs(158)+d(i0,i1)*d(i0,i2)*d(i1,j0)*d2(i0,k0)/60
                 vs(159) = vs(159)+d(i0,i1)*d(i1,i2)*d(i1,j0)*d2(i0,k0)/60
                 vs(160) = vs(160)+d(i0,i1)*d(i0,i2)*d(i0,j0)*d(i0,k0)*d(i1,k0)/60
                 vs(161) = vs(161)+d(i0,i1)*d(i0,i2)*d(i2,j0)*d(i0,k0)*d(i1,k0)/60
                 do i3 = 0, 4
                    if (i3.ne.i0.and.i3.ne.i1.and.i3.ne.i2) then
                       vs(162) = vs(162)+d(i0,i1)*d(i0,i2)*d(i0,i3)*d(i0,j0)*d(i0,k0)/120
                       vs(163) = vs(163)+d(i0,i1)*d(i1,i2)*d(i0,i3)*d(i0,j0)*d(i0,k0)/120
                       vs(164) = vs(164)+d(i0,i1)*d(i0,i2)*d(i0,i3)*d(i1,j0)*d(i0,k0)/120
                    endif
                 enddo
              endif
           enddo
        endif
     enddo
  enddo
  ! Reducible secondaries
  ! us(0:5)
  k = 0
  do i = 0, 2
     us(k) = her2_zundel(ys(i)) ; k = k+1
     do j = i+1, 2
        us(k) = ys(i)*ys(j) ; k = k+1
     enddo
  enddo
  ! vs(0:47)
  do j = 0, 15
     vs(3*j:3*j+2) = ys(0:2)*zs(j)
  enddo
  ! ws(..).  We don't have this yet.
  ! Compute vec(0:*).  The code was created using these parameters:
  ! MolienSeries(0:*): 1 4 17 68 265 977 2942 4 4
  ! #Primaries(1:*):   4 4 4 4 4 1 0 3
  ! #Secondaries(1:*): 0 3 16 54 165 0 1 5
  ! The MolienSeries partial sums (allowed size of vec) are:
  ! Molien Sums(0:*):  1 5 22 90 355 1332 4274 4278 4282
  ! constant term
  vec(0) = 1
  ! first degree terms
  if (1.le.mdeg) then
     vec(1) = x(0)
     vec(2) = x(1)
     vec(3) = x(2)
     vec(4) = x(3)
  endif
  ! second degree terms
  if (2.le.mdeg) then
     vec(5) = x2(0)
     vec(6:8) = x(0)*vec(2:4)
     vec(9) = x2(1)
     vec(10:11) = x(1)*vec(3:4)
     vec(12) = x2(2)
     vec(13:13) = x(2)*vec(4:4)
     vec(14) = x2(3)
     vec(15) = y(0)
     vec(16) = y(1)
     vec(17) = y(2)
     vec(18) = y(3)
     vec(19:21) = ys(0:2)
  endif
  ! third degree terms
  if (3.le.mdeg) then
     vec(22) = x3(0)
     vec(23:25) = x2(0)*vec(2:4)
     vec(26:38) = x(0)*vec(9:21)
     vec(39) = x3(1)
     vec(40:41) = x2(1)*vec(3:4)
     vec(42:51) = x(1)*vec(12:21)
     vec(52) = x3(2)
     vec(53:53) = x2(2)*vec(4:4)
     vec(54:61) = x(2)*vec(14:21)
     vec(62) = x3(3)
     vec(63:69) = x(3)*vec(15:21)
     vec(70) = z(0)
     vec(71) = z(1)
     vec(72) = z(2)
     vec(73) = z(3)
     vec(74:89) = zs(0:15)
  endif
  ! fourth degree terms
  if (4.le.mdeg) then
     vec(90) = x4(0)
     vec(91:93) = x3(0)*vec(2:4)
     vec(94:106) = x2(0)*vec(9:21)
     vec(107:157) = x(0)*vec(39:89)
     vec(158) = x4(1)
     vec(159:160) = x3(1)*vec(3:4)
     vec(161:170) = x2(1)*vec(12:21)
     vec(171:208) = x(1)*vec(52:89)
     vec(209) = x4(2)
     vec(210:210) = x3(2)*vec(4:4)
     vec(211:218) = x2(2)*vec(14:21)
     vec(219:246) = x(2)*vec(62:89)
     vec(247) = x4(3)
     vec(248:254) = x2(3)*vec(15:21)
     vec(255:274) = x(3)*vec(70:89)
     vec(275) = y2(0)
     vec(276:281) = y(0)*vec(16:21)
     vec(282) = y2(1)
     vec(283:287) = y(1)*vec(17:21)
     vec(288) = y2(2)
     vec(289:292) = y(2)*vec(18:21)
     vec(293) = y2(3)
     vec(294:296) = y(3)*vec(19:21)
     vec(297) = u(0)
     vec(298) = u(1)
     vec(299) = u(2)
     vec(300) = u(3)
     vec(301:354) = us(0:53)
  endif
  ! fifth degree terms
  if (5.le.mdeg) then
     vec(355) = x5(0)
     vec(356:358) = x4(0)*vec(2:4)
     vec(359:371) = x3(0)*vec(9:21)
     vec(372:422) = x2(0)*vec(39:89)
     vec(423:619) = x(0)*vec(158:354)
     vec(620) = x5(1)
     vec(621:622) = x4(1)*vec(3:4)
     vec(623:632) = x3(1)*vec(12:21)
     vec(633:670) = x2(1)*vec(52:89)
     vec(671:816) = x(1)*vec(209:354)
     vec(817) = x5(2)
     vec(818:818) = x4(2)*vec(4:4)
     vec(819:826) = x3(2)*vec(14:21)
     vec(827:854) = x2(2)*vec(62:89)
     vec(855:962) = x(2)*vec(247:354)
     vec(963) = x5(3)
     vec(964:970) = x3(3)*vec(15:21)
     vec(971:990) = x2(3)*vec(70:89)
     vec(991:1070) = x(3)*vec(275:354)
     vec(1071:1090) = y(0)*vec(70:89)
     vec(1091:1110) = y(1)*vec(70:89)
     vec(1111:1130) = y(2)*vec(70:89)
     vec(1131:1150) = y(3)*vec(70:89)
     vec(1151:1153) = z(0)*ys(0:2)
     vec(1154:1156) = z(1)*ys(0:2)
     vec(1157:1159) = z(2)*ys(0:2)
     vec(1160:1162) = z(3)*ys(0:2)
     vec(1163) = v(0)
     vec(1164) = v(1)
     vec(1165) = v(2)
     vec(1166) = v(3)
     vec(1167:1331) = vs(0:164)
  endif
  ! sixth degree terms
  if (6.le.mdeg) then
     vec(1332) = x6(0)
     vec(1333:1335) = x5(0)*vec(2:4)
     vec(1336:1348) = x4(0)*vec(9:21)
     vec(1349:1399) = x3(0)*vec(39:89)
     vec(1400:1596) = x2(0)*vec(158:354)
     vec(1597:2308) = x(0)*vec(620:1331)
     vec(2309) = x6(1)
     vec(2310:2311) = x5(1)*vec(3:4)
     vec(2312:2321) = x4(1)*vec(12:21)
     vec(2322:2359) = x3(1)*vec(52:89)
     vec(2360:2505) = x2(1)*vec(209:354)
     vec(2506:3020) = x(1)*vec(817:1331)
     vec(3021) = x6(2)
     vec(3022:3022) = x5(2)*vec(4:4)
     vec(3023:3030) = x4(2)*vec(14:21)
     vec(3031:3058) = x3(2)*vec(62:89)
     vec(3059:3166) = x2(2)*vec(247:354)
     vec(3167:3535) = x(2)*vec(963:1331)
     vec(3536) = x6(3)
     vec(3537:3543) = x4(3)*vec(15:21)
     vec(3544:3563) = x3(3)*vec(70:89)
     vec(3564:3643) = x2(3)*vec(275:354)
     vec(3644:3904) = x(3)*vec(1071:1331)
     vec(3905) = y3(0)
     vec(3906:3911) = y2(0)*vec(16:21)
     vec(3912:3984) = y(0)*vec(282:354)
     vec(3985) = y3(1)
     vec(3986:3990) = y2(1)*vec(17:21)
     vec(3991:4057) = y(1)*vec(288:354)
     vec(4058) = y3(2)
     vec(4059:4062) = y2(2)*vec(18:21)
     vec(4063:4124) = y(2)*vec(293:354)
     vec(4125) = y3(3)
     vec(4126:4128) = y2(3)*vec(19:21)
     vec(4129:4186) = y(3)*vec(297:354)
     vec(4187) = z2(0)
     vec(4188:4206) = z(0)*vec(71:89)
     vec(4207) = z2(1)
     vec(4208:4225) = z(1)*vec(72:89)
     vec(4226) = z2(2)
     vec(4227:4243) = z(2)*vec(73:89)
     vec(4244) = z2(3)
     vec(4245:4260) = z(3)*vec(74:89)
     vec(4261:4263) = u(0)*ys(0:2)
     vec(4264:4266) = u(1)*ys(0:2)
     vec(4267:4269) = u(2)*ys(0:2)
     vec(4270:4272) = u(3)*ys(0:2)
     vec(4273) = w(0)
     !! ws(0:*) to follow
  endif
  return
end subroutine getv_x5yz

!********************************************************


function her2_zundel(t0)
  implicit none

  double precision   t0,her2_zundel
  her2_zundel = (4*t0**2-2)/dsqrt(dble(8*2 ))
  return
end function her2_zundel

function her3_zundel(t0)
  implicit none

  double precision   t0,her3_zundel
  her3_zundel = (8*t0**2-12)*t0/dsqrt(dble(16*6 ))
  return
end function her3_zundel

function her4_zundel(t0)
  implicit none

  double precision   t0,her4_zundel
  her4_zundel = ((16*t0**2-48)*t0**2+12)/dsqrt(dble(32*24 ))
  return
end function her4_zundel

function her5_zundel(t0)
  implicit none

  double precision   t0,her5_zundel
  her5_zundel = ((32*t0**2-160)*t0**2+120)*t0/dsqrt(dble(64*120 ))
  return
end function her5_zundel

function her6_zundel(t0)
  implicit none

  double precision   t0,her6_zundel
  her6_zundel=(((64*t0**2-480)*t0**2+720)*t0**2-120)/dsqrt(dble(128*720))
  return
end function her6_zundel

function her7_zundel(t0)
  implicit none

  double precision   t0,her7_zundel
  her7_zundel = (((128*t0**2-1344)*t0**2+3360)*t0**2-1680)*t0/dsqrt(dble(256*5040 ))
  return
end function her7_zundel

!*************************************************



subroutine prepot1()
  implicit none
  integer ms,mr

  double precision dc0(0:6,0:6),dw0(0:6,0:6), coef(0:7961)

  common/NCOE/ms,mr
  common/h5o2coef/dc0,dw0,coef

  ms=7785
  mr=59

  dc0(0,0) =       0.0000000000000000
  dc0(1,0) =       0.2310320556220000
  dc0(2,0) =       0.2310320556220000
  dc0(3,0) =       0.2310320556220000
  dc0(4,0) =       0.2310320556220000
  dc0(5,0) =       0.3539038579620000
  dc0(6,0) =       0.3539038579620000
  dc0(0,1) =       0.2310320556220000
  dc0(1,1) =       0.0000000000000000
  dc0(2,1) =       0.2310320556220000
  dc0(3,1) =       0.2310320556220000
  dc0(4,1) =       0.2310320556220000
  dc0(5,1) =       0.3539038579620000
  dc0(6,1) =       0.3539038579620000
  dc0(0,2) =       0.2310320556220000
  dc0(1,2) =       0.2310320556220000
  dc0(2,2) =       0.0000000000000000
  dc0(3,2) =       0.2310320556220000
  dc0(4,2) =       0.2310320556220000
  dc0(5,2) =       0.3539038579620000
  dc0(6,2) =       0.3539038579620000
  dc0(0,3) =       0.2310320556220000
  dc0(1,3) =       0.2310320556220000
  dc0(2,3) =       0.2310320556220000
  dc0(3,3) =       0.0000000000000000
  dc0(4,3) =       0.2310320556220000
  dc0(5,3) =       0.3539038579620000
  dc0(6,3) =       0.3539038579620000
  dc0(0,4) =       0.2310320556220000
  dc0(1,4) =       0.2310320556220000
  dc0(2,4) =       0.2310320556220000
  dc0(3,4) =       0.2310320556220000
  dc0(4,4) =       0.0000000000000000
  dc0(5,4) =       0.3539038579620000
  dc0(6,4) =       0.3539038579620000
  dc0(0,5) =       0.3539038579620000
  dc0(1,5) =       0.3539038579620000
  dc0(2,5) =       0.3539038579620000
  dc0(3,5) =       0.3539038579620000
  dc0(4,5) =       0.3539038579620000
  dc0(5,5) =       0.0000000000000000
  dc0(6,5) =       0.1972449649890000
  dc0(0,6) =       0.3539038579620000
  dc0(1,6) =       0.3539038579620000
  dc0(2,6) =       0.3539038579620000
  dc0(3,6) =       0.3539038579620000
  dc0(4,6) =       0.3539038579620000
  dc0(5,6) =       0.1972449649890000
  dc0(6,6) =       0.0000000000000000
  dw0(0,0) =       0.0000000000000000
  dw0(1,0) =       0.1219097143100000
  dw0(2,0) =       0.1219097143100000
  dw0(3,0) =       0.1219097143100000
  dw0(4,0) =       0.1219097143100000
  dw0(5,0) =       0.1891483192840000
  dw0(6,0) =       0.1891483192840000
  dw0(0,1) =       0.1219097143100000
  dw0(1,1) =       0.0000000000000000
  dw0(2,1) =       0.1219097143100000
  dw0(3,1) =       0.1219097143100000
  dw0(4,1) =       0.1219097143100000
  dw0(5,1) =       0.1891483192840000
  dw0(6,1) =       0.1891483192840000

  dw0(0,2) =       0.1219097143100000
  dw0(1,2) =       0.1219097143100000
  dw0(2,2) =       0.0000000000000000
  dw0(3,2) =       0.1219097143100000
  dw0(4,2) =       0.1219097143100000
  dw0(5,2) =       0.1891483192840000
  dw0(6,2) =       0.1891483192840000
  dw0(0,3) =       0.1219097143100000
  dw0(1,3) =       0.1219097143100000
  dw0(2,3) =       0.1219097143100000
  dw0(3,3) =       0.0000000000000000
  dw0(4,3) =       0.1219097143100000
  dw0(5,3) =       0.1891483192840000
  dw0(6,3) =       0.1891483192840000
  dw0(0,4) =       0.1219097143100000
  dw0(1,4) =       0.1219097143100000
  dw0(2,4) =       0.1219097143100000
  dw0(3,4) =       0.1219097143100000
  dw0(4,4) =       0.0000000000000000
  dw0(5,4) =       0.1891483192840000
  dw0(6,4) =       0.1891483192840000
  dw0(0,5) =       0.1891483192840000
  dw0(1,5) =       0.1891483192840000
  dw0(2,5) =       0.1891483192840000
  dw0(3,5) =       0.1891483192840000
  dw0(4,5) =       0.1891483192840000
  dw0(5,5) =       0.0000000000000000
  dw0(6,5) =       0.0608002980568600
  dw0(0,6) =       0.1891483192840000
  dw0(1,6) =       0.1891483192840000
  dw0(2,6) =       0.1891483192840000
  dw0(3,6) =       0.1891483192840000
  dw0(4,6) =       0.1891483192840000
  dw0(5,6) =       0.0608002980568600
  dw0(6,6) =       0.0000000000000000

  coef(   0) =    6014.4258477200000925
  coef(   1) =  -16998.5888785999995889
  coef(   2) =  -28720.3983509999998205
  coef(   3) =    -130.9094846620000112
  coef(   4) =    6401.6801573599996118
  coef(   5) =  -13294.3798891999995249
  coef(   6) =   -1737.9995544400001108
  coef(   7) =   21030.2811763999998220
  coef(   8) =   -1969.6561404100000345
  coef(   9) =     750.6683854220000285
  coef(  10) =    -251.0101495899999975
  coef(  11) =      59.1057858384999975
  coef(  12) =   -4201.5839576099997430
  coef(  13) =    2327.1499321900000723
  coef(  14) =    -852.0953232629999547
  coef(  15) =    5356.9285864000003130
  coef(  16) =  -27638.2197481000002881
  coef(  17) =  -10268.2657605999993393
  coef(  18) =    6839.7813011700000061
  coef(  19) =    5386.0962403200001063
  coef(  20) =    7470.0824070600001505
  coef(  21) =     269.5884442980000131
  coef(  22) =     440.9168850410000005
  coef(  23) =    5162.9453048599998510
  coef(  24) =   12232.9800926000007166
  coef(  25) =  -30619.6473616000002949
  coef(  26) =  -10951.1189537999998720
  coef(  27) =   -8546.9571192300008988
  coef(  28) =  -74160.2027297000022372
  coef(  29) =  -12528.2171440999991319
  coef(  30) =    -249.4891923600000041
  coef(  31) =   -6939.1471604600001228
  coef(  32) =   34270.9824179999995977
  coef(  33) =   16206.7892174999997223
  coef(  34) =   34310.8914779999977327
  coef(  35) =    3302.8689094900000782
  coef(  36) =   13035.8147764999994251
  coef(  37) =     260.2285526850000110
  coef(  38) =    -260.1696587729999806
  coef(  39) =   -1765.6228017200000977
  coef(  40) =    1041.7706683500000509
  coef(  41) =    -602.3918239399999948
  coef(  42) =     458.0001475619999951
  coef(  43) =    -645.4925234249999448
  coef(  44) =     341.6155259889999911
  coef(  45) =   -1111.8594849099999919
  coef(  46) =   -2747.6155468699998892
  coef(  47) =     -72.3211004188000004
  coef(  48) =     -92.4714132872999954
  coef(  49) =    -573.3599360130000377
  coef(  50) =     290.2593647720000263
  coef(  51) =    1187.3445116299999427
  coef(  52) =    1545.8528542100000323
  coef(  53) =   -2227.4801263099998323
  coef(  54) =   -1642.1470637499999157
  coef(  55) =   -3053.6606686499999341
  coef(  56) =    2370.1837230299997827
  coef(  57) =   -2308.2080204799999592
  coef(  58) =      35.9543609839000027
  coef(  59) =   -1905.5520578099999511
  coef(  60) =   11352.7485099999994418
  coef(  61) =   -2140.0398119199999201
  coef(  62) =   21079.1977262000000337
  coef(  63) =    -881.9635271560000547
  coef(  64) =     120.0798553529999992
  coef(  65) =     -50.1605942722000009
  coef(  66) =    1163.2941819499999383
  coef(  67) =   -5193.7811202100001537
  coef(  68) =    8527.8971678199995949
  coef(  69) =   -2437.1096443500000532
  coef(  70) =    1952.8508010600000944
  coef(  71) =  -27498.5366006000003836
  coef(  72) =   -1176.5143035900000541
  coef(  73) =    -466.2388853979999794
  coef(  74) =   -1724.9998519099999612
  coef(  75) =   -9980.0539702799997031
  coef(  76) =    9762.7227913799997623
  coef(  77) =   -9265.8123977499999455
  coef(  78) =    2410.6540716600002270
  coef(  79) =  -14715.6552967999996326
  coef(  80) =      26.7405799844999983
  coef(  81) =     260.6987396409999747
  coef(  82) =     970.0120510990000184
  coef(  83) =     663.0611630619999914
  coef(  84) =     168.2763279019999914
  coef(  85) =    -190.0808468520000076
  coef(  86) =     582.5980714309999939
  coef(  87) =      88.5282547066999967
  coef(  88) =   -1634.1683229600000686
  coef(  89) =    -702.9103615950000403
  coef(  90) =     395.6369795510000245
  coef(  91) =     -53.9477613640000015
  coef(  92) =      41.4628629544000020
  coef(  93) =     284.1351828479999995
  coef(  94) =    1438.1801682899999832
  coef(  95) =    -640.5516427530000101
  coef(  96) =     973.7680025250000426
  coef(  97) =     613.3449170720000438
  coef(  98) =     701.6529445810000425
  coef(  99) =   -1810.2245561699999143
  coef( 100) =    1153.3087722099999155
  coef( 101) =     753.0743941309999627
  coef( 102) =   15389.5911539999997331
  coef( 103) =       5.3335072541999997
  coef( 104) =     365.6106902980000086
  coef( 105) =   -1556.6057240600000569
  coef( 106) =    2209.3234807300000284
  coef( 107) =   -9981.0625668600005156
  coef( 108) =    1528.4515039699999761
  coef( 109) =     334.5391178820000277
  coef( 110) =   14937.0419330000004265
  coef( 111) =    -178.2029766830000028
  coef( 112) =     -43.0517335142999968
  coef( 113) =     556.4060110609999583
  coef( 114) =      67.8838938294999963
  coef( 115) =    -511.7563698820000013
  coef( 116) =     137.7940569749999895
  coef( 117) =     640.3927208520000249
  coef( 118) =     340.9039192540000158
  coef( 119) =    1480.3435594000000037
  coef( 120) =    2318.5511559999999918
  coef( 121) =    1084.1731558500000574
  coef( 122) =      32.0009851736000002
  coef( 123) =     105.4316759000000019
  coef( 124) =     761.1231122589999813
  coef( 125) =   -2217.0666509799998494
  coef( 126) =     657.1047476109999934
  coef( 127) =    -754.4415989840000520
  coef( 128) =     431.6150844179999808
  coef( 129) =    -950.3839037510000480
  coef( 130) =     806.0286616040000354
  coef( 131) =   -1521.7981373899999653
  coef( 132) =    -369.4127647359999855
  coef( 133) =      66.8858867446000005
  coef( 134) =      15.2033231772999997
  coef( 135) =     -52.2509138754999967
  coef( 136) =    -115.7685833320000057
  coef( 137) =     -43.9945478365999989
  coef( 138) =      -6.7372631770500000
  coef( 139) =     224.6387931440000045
  coef( 140) =     -33.0434113848999971
  coef( 141) =    -172.4560036020000098
  coef( 142) =     -61.5606564574000004
  coef( 143) =     -66.4634563535000069
  coef( 144) =      17.8616648494999986
  coef( 145) =      -2.1847788828499999
  coef( 146) =     -48.0235216527999995
  coef( 147) =     -68.2843129631000068
  coef( 148) =     -72.7134041892000056
  coef( 149) =      38.8588995222999998
  coef( 150) =    -204.7522853639999880
  coef( 151) =     -93.8568820227000060
  coef( 152) =      41.5060545421000029
  coef( 153) =    -160.6189619829999913
  coef( 154) =     -56.4327243839000019
  coef( 155) =     -52.8141124497999996
  coef( 156) =     -49.1474718746000008
  coef( 157) =     111.4763872930000019
  coef( 158) =     -37.1222150720000030
  coef( 159) =     100.2666192450000011
  coef( 160) =     236.1019926969999858
  coef( 161) =     462.5250226140000223
  coef( 162) =     714.9005008939999470
  coef( 163) =   -2632.9275439600000936
  coef( 164) =     469.3078126460000021
  coef( 165) =    -143.4710634520000099
  coef( 166) =     796.1283994530000427
  coef( 167) =     520.8402798219999568
  coef( 168) =     -86.3814006183999936
  coef( 169) =   -2010.7800125500000377
  coef( 170) =     832.7206644740000456
  coef( 171) =    -794.6169712759999584
  coef( 172) =    1404.9610319999999319
  coef( 173) =     126.6304455920000009
  coef( 174) =    -269.4614014900000143
  coef( 175) =      -3.5385985179400001
  coef( 176) =     324.7529373869999745
  coef( 177) =    -365.8512947699999813
  coef( 178) =     -15.0540318735999996
  coef( 179) =     726.1023697069999798
  coef( 180) =     -42.7151846539999980
  coef( 181) =      -9.8462983596899996
  coef( 182) =     -23.4793053365999995
  coef( 183) =     -18.9857328529999982
  coef( 184) =       6.9123176534499997
  coef( 185) =     -80.8709708243000023
  coef( 186) =     -97.2151704715000022
  coef( 187) =      76.3173284667000047
  coef( 188) =      35.3015351985999999
  coef( 189) =      14.1316409096999998
  coef( 190) =      61.7644624945000018
  coef( 191) =    -206.8148643359999994
  coef( 192) =    -223.8851345820000063
  coef( 193) =    -125.6907334859999992
  coef( 194) =     -37.5666205432000027
  coef( 195) =     -86.5948844179000048
  coef( 196) =     -76.7631601819000053
  coef( 197) =      14.6432318423000005
  coef( 198) =     276.5959054179999725
  coef( 199) =     143.4219312790000060
  coef( 200) =     -43.3177901179999978
  coef( 201) =    -152.9222695919999921
  coef( 202) =     -62.9979771909999968
  coef( 203) =      -6.5104434237500000
  coef( 204) =     -31.2451503811000002
  coef( 205) =     -77.2913258346000021
  coef( 206) =      59.4486922712000023
  coef( 207) =      -8.2433504357099991
  coef( 208) =    -112.4572646069999990
  coef( 209) =     268.1930581500000130
  coef( 210) =       8.8854766931399993
  coef( 211) =     -74.6054163969999991
  coef( 212) =     -79.5869898573999990
  coef( 213) =     -24.6414699587999984
  coef( 214) =      23.4821108611000007
  coef( 215) =    -106.5621052749999933
  coef( 216) =      75.3003724385000055
  coef( 217) =    -292.4558989020000013
  coef( 218) =   -6212.9392488399998911
  coef( 219) =    6842.5118126799998208
  coef( 220) =    6839.6751124000002164
  coef( 221) =  -64535.7604843000008259
  coef( 222) =   -3271.6409364299997833
  coef( 223) =     294.4573423840000146
  coef( 224) =   -2609.6331043700001828
  coef( 225) =    5913.0164230099999259
  coef( 226) =   21970.1969271000016306
  coef( 227) =  -24576.4024495999983628
  coef( 228) =   -6885.7281712900003185
  coef( 229) =  -13944.4187753999995039
  coef( 230) =  -55999.9106429000021308
  coef( 231) =   -2110.8388280999997733
  coef( 232) =     287.0723315399999933
  coef( 233) =    6285.1014862199999698
  coef( 234) =    4201.2913329000002705
  coef( 235) =   -6691.9487758599998415
  coef( 236) =   78052.4794551999948453
  coef( 237) =   22466.6160038999987592
  coef( 238) =   58167.5916926999998395
  coef( 239) =      -4.4232739609599996
  coef( 240) =    -535.0640963839999813
  coef( 241) =   -3821.1300795400002244
  coef( 242) =    1034.9310972700000093
  coef( 243) =   -1430.0200367400000232
  coef( 244) =     386.4364711040000202
  coef( 245) =     382.8112721460000216
  coef( 246) =     865.6143642739999677
  coef( 247) =     658.9686182149999922
  coef( 248) =   -5103.6663477800002511
  coef( 249) =    -488.4871997789999796
  coef( 250) =    -254.5653290500000026
  coef( 251) =   -2547.5253629399999227
  coef( 252) =    -871.3678264039999704
  coef( 253) =    -786.3086292409999487
  coef( 254) =    1818.3356132799999614
  coef( 255) =   -4658.1187343499996132
  coef( 256) =    1000.0169576399999869
  coef( 257) =    1051.0939387400001124
  coef( 258) =    4940.0656602700000803
  coef( 259) =   -6357.3414468900000429
  coef( 260) =   -1292.0429698299999473
  coef( 261) =   64937.9792073999997228
  coef( 262) =   18878.3996201000009023
  coef( 263) =     277.6898524659999907
  coef( 264) =    8009.0887724100002742
  coef( 265) =    6278.7592166200001884
  coef( 266) =   -8054.5320837099998244
  coef( 267) =  -46108.9386054999995395
  coef( 268) =  -32247.5370231000015337
  coef( 269) =   32595.8588611000013771
  coef( 270) =     -12.0071373433000002
  coef( 271) =    1989.9774346900001092
  coef( 272) =    -996.3084146090000104
  coef( 273) =   -4801.8976103499999226
  coef( 274) =   -3024.1745267800001784
  coef( 275) =    -838.5906070900000486
  coef( 276) =     360.4388451089999990
  coef( 277) =   -2076.3040310800001862
  coef( 278) =    1370.5883591599999818
  coef( 279) =    7389.5855215100000351
  coef( 280) =    -308.8584772300000054
  coef( 281) =    -402.5053105979999941
  coef( 282) =    6266.2912496899998587
  coef( 283) =  -10246.0034066000007442
  coef( 284) =  -15608.9312336000002688
  coef( 285) =  -11875.9903584999992745
  coef( 286) =   13436.1101230000003852
  coef( 287) =  -10116.7725876999993488
  coef( 288) =    4317.9297827099999267
  coef( 289) =  -29514.2346356000016385
  coef( 290) =    3676.1327676599998995
  coef( 291) =    1713.4982627500000945
  coef( 292) =       1.4411033932899999
  coef( 293) =    -100.2423871270000006
  coef( 294) =     -71.2704747691000051
  coef( 295) =    -119.9120297959999988
  coef( 296) =     -73.8384333311999939
  coef( 297) =      42.0269387597000019
  coef( 298) =    -104.1692427280000004
  coef( 299) =     104.9587804840000018
  coef( 300) =     844.4729271270000481
  coef( 301) =     486.9845644210000160
  coef( 302) =     324.0681478050000237
  coef( 303) =      45.9818460215999991
  coef( 304) =      82.9997083016999966
  coef( 305) =    -316.8547761219999757
  coef( 306) =    -608.4458154779999859
  coef( 307) =     288.5918832099999918
  coef( 308) =     561.4091616780000322
  coef( 309) =     152.7030879309999989
  coef( 310) =     190.7302077800000006
  coef( 311) =      61.5024995435000008
  coef( 312) =    -331.7097344349999730
  coef( 313) =    -106.3291456409999967
  coef( 314) =     488.8657728869999914
  coef( 315) =     896.0181693759999462
  coef( 316) =    -616.5870407720000230
  coef( 317) =    -841.3334190320000516
  coef( 318) =    -223.7662935259999983
  coef( 319) =      33.4423388439999982
  coef( 320) =   -3067.0294837599999482
  coef( 321) =   -5144.4144182899999578
  coef( 322) =    9747.8309192399992753
  coef( 323) =    2779.2511049099998672
  coef( 324) =   -1622.1406616799999938
  coef( 325) =    1239.6048870699999043
  coef( 326) =    3311.6534286500000235
  coef( 327) =    2173.1879345299998931
  coef( 328) =   -1222.3610672699999213
  coef( 329) =   -5151.2437965400004032
  coef( 330) =    6266.6728395000000091
  coef( 331) =  -17588.7625275999998848
  coef( 332) =    -269.6525951660000260
  coef( 333) =   -7840.4281543099996270
  coef( 334) =    -275.2020889089999969
  coef( 335) =    -193.3140750649999973
  coef( 336) =    -612.8127099910000197
  coef( 337) =    -705.1334965529999863
  coef( 338) =   -8083.7735795899998266
  coef( 339) =    -359.1366062069999998
  coef( 340) =    -108.3082181970000022
  coef( 341) =     338.8104641470000047
  coef( 342) =    -298.7933825819999925
  coef( 343) =      41.4849055625000034
  coef( 344) =     631.5912590369999862
  coef( 345) =     619.8932551509999485
  coef( 346) =    3245.8249541199998021
  coef( 347) =   -1986.5406129400000736
  coef( 348) =    -447.1962527910000063
  coef( 349) =     178.3386174800000106
  coef( 350) =    2138.7993006400001832
  coef( 351) =     399.7134298180000087
  coef( 352) =    -373.5473406099999920
  coef( 353) =     201.6853376919999903
  coef( 354) =    -481.2219842019999874
  coef( 355) =     394.9632421140000247
  coef( 356) =    -601.9385081269999773
  coef( 357) =    -678.0145909880000090
  coef( 358) =     118.4128313049999974
  coef( 359) =     204.5784206069999982
  coef( 360) =    2592.3957595599999877
  coef( 361) =     861.2093824949999998
  coef( 362) =    -127.6093604429999999
  coef( 363) =     152.8230730429999937
  coef( 364) =     300.8628840829999831
  coef( 365) =    -355.2365896000000021
  coef( 366) =       2.2194958100300002
  coef( 367) =     997.2355347559999927
  coef( 368) =    2147.3253245800001423
  coef( 369) =    -383.1546799440000086
  coef( 370) =     477.0134144250000077
  coef( 371) =     882.1153189420000444
  coef( 372) =    -326.2966723320000142
  coef( 373) =     360.2856232440000213
  coef( 374) =    1827.9921266599999399
  coef( 375) =    -199.0220417289999943
  coef( 376) =     969.6930776299999479
  coef( 377) =  -60415.0876566000006278
  coef( 378) =  -14111.9706050999993749
  coef( 379) =     404.3656947050000099
  coef( 380) =  -14987.4685353999993822
  coef( 381) =   59013.5511602999977185
  coef( 382) =   29837.8861289000014949
  coef( 383) =   23954.4873024000007717
  coef( 384) =    -258.0237661290000233
  coef( 385) =  -12418.7416742999994312
  coef( 386) =      43.8089902103999975
  coef( 387) =    -454.0223537719999740
  coef( 388) =   -2310.7411060300000827
  coef( 389) =    4396.1025506400001177
  coef( 390) =     956.8842046299999993
  coef( 391) =    1457.8325907200000984
  coef( 392) =   -6876.3441152699997474
  coef( 393) =     579.5977730900000324
  coef( 394) =   -6386.3396449399997437
  coef( 395) =   -6864.1476416200002859
  coef( 396) =     -27.8510157909000000
  coef( 397) =    -380.5765228170000114
  coef( 398) =    -709.2834804950000489
  coef( 399) =    1359.8684592699999030
  coef( 400) =    7544.7056585800000903
  coef( 401) =    4043.4331120400001964
  coef( 402) =   -4593.3819111000002522
  coef( 403) =  -10596.3756195999994816
  coef( 404) =  -14834.8316988000005949
  coef( 405) =    1425.4252213300001131
  coef( 406) =   -1962.4397946399999455
  coef( 407) =    1914.8429753099999289
  coef( 408) =     -19.7046983010000005
  coef( 409) =      27.2677437040999990
  coef( 410) =    -224.1036502180000127
  coef( 411) =    -155.1715059660000122
  coef( 412) =      62.1226113318000017
  coef( 413) =       5.0880035119800002
  coef( 414) =    -292.5542473439999753
  coef( 415) =    -148.3293537970000102
  coef( 416) =    -407.3750337019999961
  coef( 417) =   -1171.9359401300000627
  coef( 418) =    -184.2051641440000083
  coef( 419) =     100.0696407140000019
  coef( 420) =    -242.7257015250000052
  coef( 421) =     799.1978708310000457
  coef( 422) =    1235.0616754800000763
  coef( 423) =      88.5606538523000069
  coef( 424) =    -792.1777227769999854
  coef( 425) =     585.3347230499999796
  coef( 426) =    1076.7876450700000532
  coef( 427) =     441.6174542389999829
  coef( 428) =     733.7841869620000352
  coef( 429) =    -198.6153240099999948
  coef( 430) =    -582.8379559700000527
  coef( 431) =    -937.6747285269999566
  coef( 432) =    -683.0750048260000540
  coef( 433) =    1960.6556683500000418
  coef( 434) =    3053.5603722599998946
  coef( 435) =   -5254.0452750799995556
  coef( 436) =    -548.6882468149999568
  coef( 437) =    1228.3041855700000724
  coef( 438) =  -20224.5299601000006078
  coef( 439) =  -17300.7506979999998293
  coef( 440) =   10861.1262986999990972
  coef( 441) =   -5228.6590630000000601
  coef( 442) =   -7405.7275650700003098
  coef( 443) =     757.5520224900000130
  coef( 444) =    -357.8652827939999952
  coef( 445) =   -1196.9522858600000745
  coef( 446) =    1794.5478104599999369
  coef( 447) =   11253.9060573000006116
  coef( 448) =    -170.3609167310000032
  coef( 449) =   12304.7490527000009024
  coef( 450) =     278.7314066420000245
  coef( 451) =    -141.6838666360000047
  coef( 452) =    1206.6722873999999592
  coef( 453) =     102.6104360589999942
  coef( 454) =   -5091.5291963999998188
  coef( 455) =     311.8846464599999990
  coef( 456) =     173.5451359349999905
  coef( 457) =    -156.2228611140000112
  coef( 458) =     190.0574944610000045
  coef( 459) =     -89.1153571021000062
  coef( 460) =     277.2454456350000100
  coef( 461) =    1062.7485140599999340
  coef( 462) =   -3016.5872066900001300
  coef( 463) =     758.2035350549999748
  coef( 464) =     154.9208199150000098
  coef( 465) =     841.1590878969999494
  coef( 466) =   -2526.1181803700001183
  coef( 467) =    1178.9356562099999337
  coef( 468) =    -427.9668674779999833
  coef( 469) =    -643.3127337680000437
  coef( 470) =     281.0929466140000272
  coef( 471) =    4072.1349495300000854
  coef( 472) =    3558.1632341899999119
  coef( 473) =    6664.0457315900002868
  coef( 474) =    1545.8771786199999951
  coef( 475) =    1676.0570072200000595
  coef( 476) =   -2041.2710321199999726
  coef( 477) =      61.3044133815999999
  coef( 478) =     416.4258079740000085
  coef( 479) =     160.7296965010000065
  coef( 480) =    -676.0367140619999873
  coef( 481) =    1398.7260347700000693
  coef( 482) =     773.9422612549999485
  coef( 483) =    -307.6767455699999800
  coef( 484) =    -738.9080875790000391
  coef( 485) =     469.2353100610000070
  coef( 486) =    -302.8633335100000181
  coef( 487) =    -963.2710700280000538
  coef( 488) =     874.8433061470000212
  coef( 489) =    -752.8487374630000204
  coef( 490) =   -1054.8209622500000933
  coef( 491) =     421.3048278009999876
  coef( 492) =   -1157.4223234999999477
  coef( 493) =       8.4499340398800005
  coef( 494) =       1.3976437352700000
  coef( 495) =       6.9688363707300001
  coef( 496) =     -24.0875642433000010
  coef( 497) =     -25.0713224638999996
  coef( 498) =       4.2907897096600003
  coef( 499) =      16.6224020147999987
  coef( 500) =      10.8838575016999997
  coef( 501) =      13.9801959280000005
  coef( 502) =      41.0211586449999999
  coef( 503) =      30.1239662041000003
  coef( 504) =     -10.6829484841000006
  coef( 505) =      -2.5035686366399998
  coef( 506) =       6.9849219961199998
  coef( 507) =      26.3154993857000008
  coef( 508) =      38.8257509903000013
  coef( 509) =     -18.1419215811000001
  coef( 510) =      38.8957126573999972
  coef( 511) =      24.9862569924999995
  coef( 512) =      13.4444126207999997
  coef( 513) =      36.7328374967000002
  coef( 514) =      24.5118327315999984
  coef( 515) =      -4.8148665818499996
  coef( 516) =     179.3202213849999964
  coef( 517) =     162.9964451680000082
  coef( 518) =      73.6809853608999958
  coef( 519) =     -77.0034717479999955
  coef( 520) =    -538.8618469660000301
  coef( 521) =      88.8114540880000050
  coef( 522) =       3.7134746244399999
  coef( 523) =     719.3330986319999738
  coef( 524) =      93.3218382104000028
  coef( 525) =     958.4585846380000476
  coef( 526) =    -233.6046696919999874
  coef( 527) =      -9.5996192627999992
  coef( 528) =      47.8773491493999970
  coef( 529) =     834.5410945300000094
  coef( 530) =     831.4485116749999634
  coef( 531) =    -524.9740535649999629
  coef( 532) =     886.2050341760000265
  coef( 533) =     -34.7683552144999979
  coef( 534) =      52.0413787722999999
  coef( 535) =      -9.7045351575299996
  coef( 536) =     -84.1750891749999965
  coef( 537) =     130.8934075319999977
  coef( 538) =      22.2446212967000001
  coef( 539) =    -297.9276889710000091
  coef( 540) =     -38.0084912875000001
  coef( 541) =     -14.7811578920999995
  coef( 542) =      -6.3324036294099999
  coef( 543) =     -12.3515255965000001
  coef( 544) =       0.8210153413050000
  coef( 545) =      66.9933745501000004
  coef( 546) =     -24.7164552937999993
  coef( 547) =    -106.3445673049999982
  coef( 548) =     110.6542637689999964
  coef( 549) =      26.7129422155000000
  coef( 550) =     -70.1280660535000067
  coef( 551) =     -77.3096445592999970
  coef( 552) =      34.1339229012000018
  coef( 553) =     -19.2866691277999998
  coef( 554) =     -76.5560982311999965
  coef( 555) =      42.2272065424999994
  coef( 556) =      54.8512129141999978
  coef( 557) =     -50.7761656982000034
  coef( 558) =    -197.8357719470000120
  coef( 559) =     -51.8585033613000022
  coef( 560) =     -56.2641672256999996
  coef( 561) =    -174.8301352759999929
  coef( 562) =     -64.1905408554000019
  coef( 563) =     -22.4604893212000007
  coef( 564) =     -27.5771126180999993
  coef( 565) =     -50.5402796170000030
  coef( 566) =      -0.4073408377430000
  coef( 567) =       2.2122023064300000
  coef( 568) =     -42.1902994618999969
  coef( 569) =    -161.9938797439999973
  coef( 570) =      -4.6317890422000003
  coef( 571) =     -14.1249498744000004
  coef( 572) =      18.7047686774000006
  coef( 573) =       7.6019291086700003
  coef( 574) =      23.1868914408999984
  coef( 575) =      53.4316098433999969
  coef( 576) =     -27.4136679508999990
  coef( 577) =      52.2391145296000019
  coef( 578) =     -84.8379387420999933
  coef( 579) =    -380.2201182960000097
  coef( 580) =     119.8779637229999935
  coef( 581) =    -106.8283506159999945
  coef( 582) =      33.9623934290999969
  coef( 583) =    -230.8995943920000116
  coef( 584) =     321.5762304090000043
  coef( 585) =     404.7792304870000066
  coef( 586) =     238.3510594640000022
  coef( 587) =    -616.9082838589999938
  coef( 588) =     164.9755437629999903
  coef( 589) =    -209.6174247920000084
  coef( 590) =     184.5228392049999968
  coef( 591) =     308.1258301240000037
  coef( 592) =      58.5842227743000024
  coef( 593) =     -85.1719399416000016
  coef( 594) =     451.0569882899999925
  coef( 595) =    1202.3368775700000697
  coef( 596) =     545.5074367790000451
  coef( 597) =     100.7172728169999942
  coef( 598) =    1440.9997115199998916
  coef( 599) =     313.8717562499999758
  coef( 600) =   -1413.2268433399999594
  coef( 601) =    -995.2978795340000033
  coef( 602) =    2028.8500856299999668
  coef( 603) =     232.4678882000000044
  coef( 604) =      33.6772105037000031
  coef( 605) =    -516.1098464990000139
  coef( 606) =   -1468.9837248399999226
  coef( 607) =    -418.0625180170000021
  coef( 608) =     -96.0122683146000071
  coef( 609) =    1151.0723563999999897
  coef( 610) =     842.5237155730000040
  coef( 611) =    -471.1926178310000068
  coef( 612) =      56.3992675092000013
  coef( 613) =     109.6929236929999973
  coef( 614) =     190.9904277969999953
  coef( 615) =    -179.4353285530000051
  coef( 616) =     119.9807560550000005
  coef( 617) =     721.8610886159999609
  coef( 618) =     -27.7537824616000002
  coef( 619) =      93.0886981488999936
  coef( 620) =      18.0732787473999998
  coef( 621) =     323.2238857080000116
  coef( 622) =     252.0381541499999969
  coef( 623) =     -10.8941654446000005
  coef( 624) =    -502.2947303870000155
  coef( 625) =      74.6059378812999938
  coef( 626) =     -20.9357690044999991
  coef( 627) =     121.8172532269999948
  coef( 628) =    -231.6852055120000102
  coef( 629) =     248.1181754170000033
  coef( 630) =   -1215.2646753900000931
  coef( 631) =     455.2016151820000118
  coef( 632) =    -734.8544524229999979
  coef( 633) =    2037.2579690600000504
  coef( 634) =    1264.4239851400000134
  coef( 635) =    1842.4988158000001022
  coef( 636) =    -165.5096888479999961
  coef( 637) =    -312.9200266230000125
  coef( 638) =      11.2290261328999996
  coef( 639) =     -97.6323279170999996
  coef( 640) =    -204.8922779700000092
  coef( 641) =     189.8280632109999999
  coef( 642) =      32.3321370922000000
  coef( 643) =      28.9406068294000001
  coef( 644) =     606.0291678470000534
  coef( 645) =    -970.7870460460000004
  coef( 646) =    -841.6231623160000481
  coef( 647) =     243.3375544950000062
  coef( 648) =    1911.0246800600000370
  coef( 649) =    1551.5804502299999967
  coef( 650) =   -1043.8880834799999775
  coef( 651) =    -794.2916603759999816
  coef( 652) =    -357.6461281159999999
  coef( 653) =     247.3836120930000106
  coef( 654) =    -253.1085418570000058
  coef( 655) =    -280.3684123659999727
  coef( 656) =      40.2002966439999980
  coef( 657) =     139.3481586199999924
  coef( 658) =    -108.6239094689999973
  coef( 659) =    -600.1393153349999920
  coef( 660) =    -110.9973421870000010
  coef( 661) =   -1126.9152023399999507
  coef( 662) =    1539.2561006299999917
  coef( 663) =    2207.7191444499999307
  coef( 664) =    1337.9110551799999485
  coef( 665) =   -2202.1247129799999129
  coef( 666) =     647.3966770719999886
  coef( 667) =   -2528.8625099699997918
  coef( 668) =    3665.6184331600002224
  coef( 669) =    -395.8924540950000051
  coef( 670) =    -130.7220623890000013
  coef( 671) =      80.7069209680000057
  coef( 672) =      47.3763998327999971
  coef( 673) =     -45.4462528374999977
  coef( 674) =     -44.7266609961000015
  coef( 675) =      60.8273334479999974
  coef( 676) =      36.1140362001999975
  coef( 677) =      -3.7223166724999999
  coef( 678) =     -12.8177037849000008
  coef( 679) =      12.1521517367000005
  coef( 680) =    -141.5362848169999950
  coef( 681) =     -58.9940290931000035
  coef( 682) =    -292.4692967330000215
  coef( 683) =     433.2354824189999931
  coef( 684) =     -35.4758185761999982
  coef( 685) =    -203.7757124649999980
  coef( 686) =     433.9531296249999741
  coef( 687) =     -38.3069762157999989
  coef( 688) =     131.6771252210000114
  coef( 689) =     -12.3603690815000000
  coef( 690) =    -132.1007882709999990
  coef( 691) =     -55.3790166662999965
  coef( 692) =      68.6721037392999989
  coef( 693) =     161.5420827529999883
  coef( 694) =      32.2502715476000006
  coef( 695) =      49.6085131358000027
  coef( 696) =      12.4332931712000008
  coef( 697) =    -143.9667553840000096
  coef( 698) =     123.0890216779999946
  coef( 699) =     100.3935705579999933
  coef( 700) =     -21.0346903890999997
  coef( 701) =    -819.4721571229999881
  coef( 702) =    -512.5003813350000428
  coef( 703) =      59.8846800306000020
  coef( 704) =     -53.2880821733999994
  coef( 705) =     -41.3527665586999973
  coef( 706) =      30.1438781304000010
  coef( 707) =     -79.3786092177000029
  coef( 708) =    -802.3270316480000020
  coef( 709) =     -93.0198771869000041
  coef( 710) =    -531.3202785480000330
  coef( 711) =    -244.7172916559999862
  coef( 712) =      27.7649636571999991
  coef( 713) =     584.8233137559999477
  coef( 714) =      84.1807616995999979
  coef( 715) =      94.4472872362999993
  coef( 716) =      13.9462963230000003
  coef( 717) =     -99.8506558464000022
  coef( 718) =      16.5356909541000014
  coef( 719) =     -76.8854169175999971
  coef( 720) =      65.7229555873999942
  coef( 721) =      16.7266895560000002
  coef( 722) =       6.0427593714699999
  coef( 723) =      18.0026002906000002
  coef( 724) =     -20.5781114862999992
  coef( 725) =     215.8559312360000035
  coef( 726) =     -10.6624709145000001
  coef( 727) =      59.7366723283000027
  coef( 728) =      -8.3577116262200004
  coef( 729) =     -47.2599992028000031
  coef( 730) =     -10.8649516911999999
  coef( 731) =     -24.1307890984999993
  coef( 732) =     -44.1165914368000003
  coef( 733) =     -33.5531787462999986
  coef( 734) =    -370.6060762810000142
  coef( 735) =    -123.1670922450000063
  coef( 736) =     -58.2201400547000034
  coef( 737) =     -93.8413747949000054
  coef( 738) =     -20.2404605404000009
  coef( 739) =    -200.6207117890000120
  coef( 740) =     176.3992280150000056
  coef( 741) =     -18.9300801049000000
  coef( 742) =     -34.9865480328000018
  coef( 743) =     -98.5477240156999983
  coef( 744) =      96.1781318600999953
  coef( 745) =    -129.1741574890000095
  coef( 746) =     -42.7123685995999978
  coef( 747) =      74.0736326747999954
  coef( 748) =     -22.4522912898999998
  coef( 749) =      99.9689853908000003
  coef( 750) =      -3.7617499821800000
  coef( 751) =      63.7542806897000034
  coef( 752) =      13.9928821455999994
  coef( 753) =     -86.9994606324000017
  coef( 754) =     181.8024726159999886
  coef( 755) =     136.7732723949999922
  coef( 756) =      23.7332715129999983
  coef( 757) =    -233.7586996420000105
  coef( 758) =    -220.6604353989999936
  coef( 759) =     -50.4249856198999993
  coef( 760) =     -22.8144685988000013
  coef( 761) =      70.8428116856999992
  coef( 762) =     -79.3176654958000000
  coef( 763) =     -99.6531558159000070
  coef( 764) =      37.4175396473000035
  coef( 765) =     -71.7712619967000052
  coef( 766) =     -38.4879241605000004
  coef( 767) =      17.5032564830999995
  coef( 768) =     -90.8862064704999995
  coef( 769) =     -53.1296317567000003
  coef( 770) =     213.3811154830000021
  coef( 771) =      61.6866539274000019
  coef( 772) =   -3130.5895448299997952
  coef( 773) =    6574.8448532700003852
  coef( 774) =    -586.5931334910000032
  coef( 775) =   14078.2007823000003555
  coef( 776) =    -443.2860945720000245
  coef( 777) =      -1.0801153508500001
  coef( 778) =     539.0826280119999865
  coef( 779) =    1807.6992386700001134
  coef( 780) =   -2432.1300140299999839
  coef( 781) =    2504.7021790700000565
  coef( 782) =   -1547.7550660599999901
  coef( 783) =     522.9321534399999791
  coef( 784) =   -2276.7209040799998547
  coef( 785) =   -1645.1160145599999396
  coef( 786) =     -25.1746495924999998
  coef( 787) =   -1353.9033563499999673
  coef( 788) =   -5315.7662996099998054
  coef( 789) =     410.8400052129999835
  coef( 790) =   -7203.6853111199998239
  coef( 791) =    2247.7831847200000084
  coef( 792) =  -15081.9850776999992377
  coef( 793) =       0.4062824726910000
  coef( 794) =      50.6202791804999990
  coef( 795) =     582.7164807980000205
  coef( 796) =     615.1822510160000093
  coef( 797) =      -0.6223921235210000
  coef( 798) =     -71.8250445147999983
  coef( 799) =     442.2661602969999990
  coef( 800) =     -95.5293875385000035
  coef( 801) =   -1149.6644029500000670
  coef( 802) =     130.0659792630000027
  coef( 803) =    -111.8915609489999952
  coef( 804) =      42.4522000014000014
  coef( 805) =      15.9116910678999997
  coef( 806) =      91.1003895805000070
  coef( 807) =    1745.4993869800000539
  coef( 808) =      70.0488653717000034
  coef( 809) =     575.4941409479999948
  coef( 810) =     161.8075492920000045
  coef( 811) =    -337.9714362930000107
  coef( 812) =    -299.4906731199999967
  coef( 813) =    1037.9331841999999142
  coef( 814) =     496.0748211559999845
  coef( 815) =   -2222.3180856600001789
  coef( 816) =    -115.0112398029999952
  coef( 817) =     154.9768119789999901
  coef( 818) =   -1872.0240710400000808
  coef( 819) =   -4749.8186595300003319
  coef( 820) =   -2395.7846100699998715
  coef( 821) =    6161.8794511899995996
  coef( 822) =    1218.7025965099999212
  coef( 823) =    1709.2526498899999297
  coef( 824) =      -5.8517548772400003
  coef( 825) =     -24.5567206288000008
  coef( 826) =     513.8523583099999996
  coef( 827) =    -154.1011177210000085
  coef( 828) =    -396.7491206039999838
  coef( 829) =      16.7363656312000018
  coef( 830) =    2290.4179086399999505
  coef( 831) =     419.9173664830000234
  coef( 832) =    2991.0541434000001573
  coef( 833) =     290.9840856659999986
  coef( 834) =     514.4559311760000355
  coef( 835) =     -63.7373918847999974
  coef( 836) =    -500.5100349909999977
  coef( 837) =    2792.9874059600001601
  coef( 838) =    -259.0266095750000090
  coef( 839) =    1561.5477796699999544
  coef( 840) =   -1819.3800484500000039
  coef( 841) =    2890.2123406000000614
  coef( 842) =     701.9347049799999922
  coef( 843) =    2187.7817854899999475
  coef( 844) =     655.2149344470000187
  coef( 845) =     -97.2844167140000025
  coef( 846) =       0.9038138806590000
  coef( 847) =       1.5680837108700001
  coef( 848) =       1.2614670040000000
  coef( 849) =     -17.4085901553999989
  coef( 850) =       9.9620717615699999
  coef( 851) =      -9.4763351898299995
  coef( 852) =       2.1155971326300000
  coef( 853) =     -22.8791537154999993
  coef( 854) =    -250.5892372179999938
  coef( 855) =      33.3968603937999973
  coef( 856) =     -41.8107427754000014
  coef( 857) =      -1.3770413339100001
  coef( 858) =       4.3611562764400000
  coef( 859) =     -86.2196490052999991
  coef( 860) =    -247.6738028110000016
  coef( 861) =    -107.3672484199999957
  coef( 862) =      76.1137763569000043
  coef( 863) =    -184.6908044989999951
  coef( 864) =    -194.7069864869999947
  coef( 865) =      32.1644747642000013
  coef( 866) =    -330.1055815020000068
  coef( 867) =     -80.3966657837000014
  coef( 868) =     -64.3400405115000069
  coef( 869) =    -332.5888006170000040
  coef( 870) =     212.2327305149999859
  coef( 871) =    -270.0578863549999937
  coef( 872) =     129.3085615539999935
  coef( 873) =     181.8960840240000039
  coef( 874) =     512.5537647099999958
  coef( 875) =    1068.9063449699999637
  coef( 876) =   -1773.0329662999999982
  coef( 877) =    1110.9812369700000545
  coef( 878) =   -1615.4477551899999526
  coef( 879) =     442.5562311200000067
  coef( 880) =     726.1949978989999863
  coef( 881) =     -27.0016527171000007
  coef( 882) =   -1184.9473742000000129
  coef( 883) =    1184.2667110800000501
  coef( 884) =   -1393.6857802500001071
  coef( 885) =    2826.4427239800002098
  coef( 886) =      87.4843346157999946
  coef( 887) =    -547.5179006500000014
  coef( 888) =      23.0097956444999987
  coef( 889) =     525.2895944010000449
  coef( 890) =    -164.4430563329999870
  coef( 891) =      61.2422595159000025
  coef( 892) =    3331.1105354599999373
  coef( 893) =      51.8341737308999981
  coef( 894) =      13.6417395944000006
  coef( 895) =     -29.2138142432000016
  coef( 896) =      -0.0717323328428500
  coef( 897) =       6.4808317895899998
  coef( 898) =     -44.3832762676999977
  coef( 899) =    -404.4109995709999907
  coef( 900) =     -49.7858671707000013
  coef( 901) =     202.5015589029999887
  coef( 902) =      36.5084122956000030
  coef( 903) =     -62.6951079307999990
  coef( 904) =    -322.3548103009999863
  coef( 905) =     -32.8615637241000016
  coef( 906) =      69.6654846208000009
  coef( 907) =      22.2894538482999991
  coef( 908) =      59.6420129908999996
  coef( 909) =    -642.0535435320000488
  coef( 910) =    -126.3845783710000035
  coef( 911) =    -108.7198194660000041
  coef( 912) =      55.0063028877999969
  coef( 913) =     -87.4957296617999987
  coef( 914) =    -296.5564437560000215
  coef( 915) =    -123.7718231409999987
  coef( 916) =     -56.4151908053000000
  coef( 917) =     -34.6711741180000033
  coef( 918) =      -9.4352929930599991
  coef( 919) =      11.2034019113000003
  coef( 920) =     -22.7034835408999989
  coef( 921) =    -274.3477987470000130
  coef( 922) =     -68.7275900077000017
  coef( 923) =       8.2496191938599992
  coef( 924) =    -138.7700027429999921
  coef( 925) =    -243.2145347690000108
  coef( 926) =     -77.9683495167000018
  coef( 927) =     -33.2995941431999967
  coef( 928) =    -395.9304435880000028
  coef( 929) =      -2.4070025246200002
  coef( 930) =    -310.1622371189999967
  coef( 931) =  -10313.6323510999991413
  coef( 932) =     317.3766586660000257
  coef( 933) =     -54.8048125988999999
  coef( 934) =    -177.5188150980000046
  coef( 935) =   -2219.6357859200002167
  coef( 936) =    6929.0958217600000353
  coef( 937) =    -828.3980803890000288
  coef( 938) =    -167.5273808420000137
  coef( 939) =    3572.4899930800002039
  coef( 940) =      20.4008610260999994
  coef( 941) =     426.8714774090000219
  coef( 942) =     379.3842916710000281
  coef( 943) =    -109.9366030580000029
  coef( 944) =      84.5632938002000003
  coef( 945) =     -42.1608532619000016
  coef( 946) =      26.7732408174000014
  coef( 947) =     366.9252369309999722
  coef( 948) =     247.5260744980000140
  coef( 949) =   -1483.9373012799999287
  coef( 950) =     884.0961442330000182
  coef( 951) =    -183.6149104159999865
  coef( 952) =      26.9480737766000011
  coef( 953) =     345.6511536450000222
  coef( 954) =   -1555.2895838199999616
  coef( 955) =   -1101.1942293899999186
  coef( 956) =     532.8899886149999929
  coef( 957) =     675.4870815809999840
  coef( 958) =    2231.4593308100002105
  coef( 959) =   -1256.2198074200000519
  coef( 960) =     -38.4294806687999966
  coef( 961) =     159.1628627750000078
  coef( 962) =      -5.5314429256200004
  coef( 963) =      -5.6174157277800001
  coef( 964) =      24.2613326936999982
  coef( 965) =     -26.7560349144999989
  coef( 966) =     -69.7825309577000041
  coef( 967) =      42.3787370601000006
  coef( 968) =    -210.1758825629999876
  coef( 969) =      -3.4655529227300002
  coef( 970) =      74.4614135317000034
  coef( 971) =      72.9539763561000001
  coef( 972) =    -116.1484935609999951
  coef( 973) =     -31.2709952302000005
  coef( 974) =     -13.1013169053999992
  coef( 975) =    -596.2914714280000226
  coef( 976) =      62.0634175762999973
  coef( 977) =     121.0411701780000016
  coef( 978) =     -13.0408481883000000
  coef( 979) =    -617.9221887850000030
  coef( 980) =    -185.8499112330000003
  coef( 981) =     120.9997729760000027
  coef( 982) =      36.4273346423999982
  coef( 983) =       5.8437838368400001
  coef( 984) =     201.2108397480000122
  coef( 985) =     578.1166894519999460
  coef( 986) =     583.5416074270000308
  coef( 987) =     204.1954635140000107
  coef( 988) =    -370.5557522679999920
  coef( 989) =    1070.5558647899999869
  coef( 990) =     496.0742561609999939
  coef( 991) =     781.7668375460000334
  coef( 992) =    4178.5639404699995794
  coef( 993) =   -1387.7464331599999241
  coef( 994) =    1885.2037095499999850
  coef( 995) =   -1045.1319065400000454
  coef( 996) =     -53.7736200959000001
  coef( 997) =    -270.4623444889999746
  coef( 998) =    1246.7609650800000054
  coef( 999) =    -749.5833984350000492
  coef(1000) =     399.6829126839999731
  coef(1001) =   -4215.9448411299999862
  coef(1002) =     -34.0848456467999981
  coef(1003) =    -361.6313475970000013
  coef(1004) =    -110.6548757300000005
  coef(1005) =   -1421.5543769200000952
  coef(1006) =     498.6203332609999848
  coef(1007) =      -8.2498509820999999
  coef(1008) =   -1842.6890775199999553
  coef(1009) =    -255.5515437139999904
  coef(1010) =     -78.3814439646999972
  coef(1011) =      53.1729423764000018
  coef(1012) =    -118.5605633899999987
  coef(1013) =     -61.8384991619000033
  coef(1014) =    -487.2764235289999988
  coef(1015) =      65.4678676061000004
  coef(1016) =     489.7329366989999926
  coef(1017) =     -43.9964447525999987
  coef(1018) =     -32.4847140195999984
  coef(1019) =    -241.6043923519999908
  coef(1020) =     614.8307877260000396
  coef(1021) =   -1010.5121883100000559
  coef(1022) =    -205.1278164779999997
  coef(1023) =     292.6877891010000212
  coef(1024) =    -142.6933233999999970
  coef(1025) =     163.3244561490000137
  coef(1026) =    -302.6875963489999890
  coef(1027) =   -1108.0596360699998968
  coef(1028) =    -249.2649418480000065
  coef(1029) =       0.8849415926800001
  coef(1030) =     788.4792877060000365
  coef(1031) =     289.3214933030000111
  coef(1032) =      44.3694376571000006
  coef(1033) =     -24.3642866464999983
  coef(1034) =      51.0471543187000023
  coef(1035) =     -88.8124605996000014
  coef(1036) =      -2.6163316264400001
  coef(1037) =    -192.5199747469999920
  coef(1038) =     -22.7303822914000015
  coef(1039) =     -21.2001037102999987
  coef(1040) =       8.9742957289700005
  coef(1041) =     210.0367136069999958
  coef(1042) =    -104.7218654859999987
  coef(1043) =      -9.5694958088300002
  coef(1044) =    -202.5119593160000022
  coef(1045) =    -473.6098533059999909
  coef(1046) =      29.0968594124999989
  coef(1047) =       0.3598338447010000
  coef(1048) =       0.1479831859410000
  coef(1049) =      -0.9121558697470000
  coef(1050) =       3.0634385296700000
  coef(1051) =       4.0483081165700003
  coef(1052) =      -5.5223199157799998
  coef(1053) =       2.1249777426100001
  coef(1054) =      -0.6052818018260000
  coef(1055) =       2.3979728099800002
  coef(1056) =       1.1920865212400000
  coef(1057) =      -3.5341031902000002
  coef(1058) =      -0.8091016045660000
  coef(1059) =       0.0900693440501300
  coef(1060) =       0.2788209428090000
  coef(1061) =      24.0422759136999993
  coef(1062) =       6.0300415271899999
  coef(1063) =       6.3410226832600003
  coef(1064) =       0.6401674754940000
  coef(1065) =      12.0708791762000001
  coef(1066) =      -1.1276138486100000
  coef(1067) =      18.9668680688000002
  coef(1068) =      11.0745765540000001
  coef(1069) =      -0.5006439654580001
  coef(1070) =     -34.7731635394999969
  coef(1071) =     -85.1117666563000057
  coef(1072) =     -11.4581770376000005
  coef(1073) =       5.4042692223200000
  coef(1074) =       5.1751070912500001
  coef(1075) =    -138.0862011380000069
  coef(1076) =    -332.9572459330000243
  coef(1077) =     -28.6847956068000016
  coef(1078) =      62.1792047818999976
  coef(1079) =    -321.1047418429999993
  coef(1080) =      56.4361353828999981
  coef(1081) =      77.0601148179000006
  coef(1082) =     -49.7564043154999993
  coef(1083) =     116.4201006210000031
  coef(1084) =      64.5318238618000066
  coef(1085) =     -81.0353498831999985
  coef(1086) =     286.6191958640000053
  coef(1087) =      24.0964654345000007
  coef(1088) =      17.9151004281999988
  coef(1089) =       5.3698055663100002
  coef(1090) =     119.2281562649999955
  coef(1091) =     -41.3135746187000024
  coef(1092) =       2.8725784775899998
  coef(1093) =    -495.1818508939999788
  coef(1094) =      19.3753283094999986
  coef(1095) =       3.1155905491899998
  coef(1096) =       0.2167520024210000
  coef(1097) =       7.6266474333599996
  coef(1098) =       4.2307746386999998
  coef(1099) =      13.2388515791000003
  coef(1100) =      71.7413564735000051
  coef(1101) =     -20.4329649097000008
  coef(1102) =      -0.6286318649190000
  coef(1103) =       1.5244485029499999
  coef(1104) =      45.0796693114999982
  coef(1105) =     -41.9391084211000020
  coef(1106) =      84.9018806306000045
  coef(1107) =      48.6848346004999968
  coef(1108) =      49.2619937848999996
  coef(1109) =      32.9894023000999965
  coef(1110) =      99.7542424006999937
  coef(1111) =      15.1747872996000002
  coef(1112) =      81.6977980518999942
  coef(1113) =      -0.8971988895700000
  coef(1114) =     -10.5286446035000001
  coef(1115) =     -47.3746456806999987
  coef(1116) =     -20.1517390302000017
  coef(1117) =      -0.1263661558640000
  coef(1118) =       5.9685946318100003
  coef(1119) =      18.5417208060000007
  coef(1120) =      -4.4597943290700002
  coef(1121) =      -1.5818036205499999
  coef(1122) =      87.3359978041000033
  coef(1123) =     -26.4007302219000017
  coef(1124) =       1.6786969068399999
  coef(1125) =      40.8647748988000004
  coef(1126) =     -31.2913855396999985
  coef(1127) =      -4.9722688838400000
  coef(1128) =     -12.3677710928000000
  coef(1129) =     133.2124049280000122
  coef(1130) =      34.3763900654999972
  coef(1131) =      66.2847033743999958
  coef(1132) =      23.2051462105999988
  coef(1133) =     180.6369423900000015
  coef(1134) =     -91.5277220736000032
  coef(1135) =      51.8450607226999978
  coef(1136) =       6.1474844002399998
  coef(1137) =      21.6443056343000002
  coef(1138) =    -117.4794117840000069
  coef(1139) =     -78.9559107264999938
  coef(1140) =      -4.4316593394600003
  coef(1141) =      26.2253719328000017
  coef(1142) =     -26.0542845332999988
  coef(1143) =      78.6410274365999982
  coef(1144) =      31.4871064772000011
  coef(1145) =     -52.1919246216999966
  coef(1146) =     -14.2871269860000005
  coef(1147) =      -8.8547355935700001
  coef(1148) =    -205.0927535369999930
  coef(1149) =    -268.6694744519999745
  coef(1150) =    -128.5879136899999935
  coef(1151) =     -47.1754577701999978
  coef(1152) =     -23.5318057248000017
  coef(1153) =     203.3469754820000048
  coef(1154) =    -185.8416862980000133
  coef(1155) =     143.6473036309999998
  coef(1156) =    -258.6133241809999959
  coef(1157) =     444.8444147330000078
  coef(1158) =     460.6990690119999954
  coef(1159) =      52.3313345942000012
  coef(1160) =      32.9647822989999995
  coef(1161) =      -7.0625182347199997
  coef(1162) =     -48.0012113104999969
  coef(1163) =    -294.6002078270000197
  coef(1164) =     100.8327069049999949
  coef(1165) =    -129.2649632989999873
  coef(1166) =      21.8579086065000006
  coef(1167) =       4.9325312209499996
  coef(1168) =      41.7261996853000028
  coef(1169) =    -238.9278854009999975
  coef(1170) =    -211.6849303129999953
  coef(1171) =     -35.9090568777000030
  coef(1172) =     417.0376086960000066
  coef(1173) =      41.0480754404999999
  coef(1174) =    -427.6690808349999884
  coef(1175) =    -238.9479860319999887
  coef(1176) =     -94.7321180940000005
  coef(1177) =      79.3016096374000057
  coef(1178) =     297.4082624580000243
  coef(1179) =       9.3902201317300005
  coef(1180) =     314.3335263170000076
  coef(1181) =      12.1671348542000004
  coef(1182) =      23.4842283459999983
  coef(1183) =    -390.5382514639999840
  coef(1184) =     511.0520682669999815
  coef(1185) =    -260.7409847079999849
  coef(1186) =     159.5707068060000040
  coef(1187) =    -629.3117909569999711
  coef(1188) =    -631.0538105980000410
  coef(1189) =   -1090.4649929199999860
  coef(1190) =    -327.3320806369999900
  coef(1191) =      70.6946424988999951
  coef(1192) =     -40.6404204449000019
  coef(1193) =    -168.9893459440000072
  coef(1194) =      59.8799584930999984
  coef(1195) =    -191.2601401549999878
  coef(1196) =     -14.2235433787000005
  coef(1197) =      -0.5998273127120000
  coef(1198) =     165.4161603999999954
  coef(1199) =      23.6797580640000014
  coef(1200) =     189.6297846639999989
  coef(1201) =     -46.6380988171000013
  coef(1202) =     126.4633030250000019
  coef(1203) =      45.5774661503999994
  coef(1204) =     447.8911378760000161
  coef(1205) =      -4.4973330157699998
  coef(1206) =     -39.4519318150000018
  coef(1207) =     -36.9235245005000010
  coef(1208) =     122.8872446649999972
  coef(1209) =      37.7319816920999997
  coef(1210) =      -5.9118989755699998
  coef(1211) =    -138.3979638280000017
  coef(1212) =     -22.4229438334000015
  coef(1213) =     267.3755429500000105
  coef(1214) =      55.8212984192999997
  coef(1215) =     192.6071255829999984
  coef(1216) =    -676.2543204390000255
  coef(1217) =     -30.3944407605999984
  coef(1218) =    -366.0208395999999880
  coef(1219) =     472.9033583350000072
  coef(1220) =    -676.6427769619999708
  coef(1221) =      18.6415859216999991
  coef(1222) =    -460.2275142299999970
  coef(1223) =       0.6185739806030000
  coef(1224) =      24.3344897648000007
  coef(1225) =     -17.0408772890999991
  coef(1226) =     -13.4895862271000002
  coef(1227) =       4.7147713903900001
  coef(1228) =       7.1665496787300000
  coef(1229) =     -12.8215263318999995
  coef(1230) =      -4.2336445719400002
  coef(1231) =      -3.3593880464899999
  coef(1232) =      16.0034937389000014
  coef(1233) =       0.0788972555541000
  coef(1234) =      28.5785400178000017
  coef(1235) =      67.2701721556999956
  coef(1236) =      13.2367986948999992
  coef(1237) =     -78.1320619780999976
  coef(1238) =      14.6509920224000005
  coef(1239) =      58.8669348486999979
  coef(1240) =     -81.8029132844999936
  coef(1241) =      -2.7214032860600001
  coef(1242) =     -19.8912425946000013
  coef(1243) =      -1.7787339015999999
  coef(1244) =       6.3784643716999998
  coef(1245) =       7.3758293875799996
  coef(1246) =     -28.5462831468999987
  coef(1247) =     -57.8166537642999998
  coef(1248) =     -15.9479882017999994
  coef(1249) =     -21.6119262178000007
  coef(1250) =     -17.1304620125000007
  coef(1251) =     191.9084088889999862
  coef(1252) =      33.7018353047999994
  coef(1253) =     -19.2119562196000011
  coef(1254) =      17.7847415702000013
  coef(1255) =      29.4627860890999997
  coef(1256) =      42.6031473748999971
  coef(1257) =      -3.3211767844500000
  coef(1258) =      13.8507748314000008
  coef(1259) =     -10.6204518727000004
  coef(1260) =     -22.7535127346999992
  coef(1261) =      29.2884289932999984
  coef(1262) =     162.5969745410000087
  coef(1263) =      37.6003698618999991
  coef(1264) =     117.3307373810000058
  coef(1265) =      27.5939712310999994
  coef(1266) =       1.8423187047500000
  coef(1267) =    -159.7431417070000066
  coef(1268) =     -26.4541871732000011
  coef(1269) =     -22.1396250045999992
  coef(1270) =       9.2250473152399994
  coef(1271) =      16.1207401361999985
  coef(1272) =      -8.1912023557900007
  coef(1273) =       6.9567002468899997
  coef(1274) =     -23.8339893612000004
  coef(1275) =      -9.6190833747299997
  coef(1276) =     -13.7682859697000008
  coef(1277) =     -12.0353359389999994
  coef(1278) =      50.6645164557000030
  coef(1279) =       3.7646563118700000
  coef(1280) =     -11.0072102670999996
  coef(1281) =     -20.0523979202000007
  coef(1282) =       0.6498565334850001
  coef(1283) =      42.6788784438000022
  coef(1284) =      14.0967139013999994
  coef(1285) =      -0.3395639312990000
  coef(1286) =      18.2449612144000000
  coef(1287) =       2.8107122562200000
  coef(1288) =      38.9687196687999986
  coef(1289) =      20.4244640480999990
  coef(1290) =       6.9806107000999997
  coef(1291) =      12.3095911639000004
  coef(1292) =      47.5945555877000004
  coef(1293) =      50.4816916190000029
  coef(1294) =      19.0361190903999997
  coef(1295) =       4.1856388887499998
  coef(1296) =      26.6646046914000010
  coef(1297) =      62.6569654048999993
  coef(1298) =      54.2920776200000006
  coef(1299) =      17.4136810914999991
  coef(1300) =       2.9821047743900002
  coef(1301) =       0.6483497055900000
  coef(1302) =       2.1153341793200000
  coef(1303) =       3.8883379232699999
  coef(1304) =      22.1853981078999993
  coef(1305) =      12.2708030707999995
  coef(1306) =       1.9574828715100001
  coef(1307) =     111.4440390809999997
  coef(1308) =      -1.3194284619600001
  coef(1309) =     -39.5371747849999977
  coef(1310) =       4.7422114039399998
  coef(1311) =      42.9684822585999981
  coef(1312) =      92.0564538169000031
  coef(1313) =      41.2437584067000031
  coef(1314) =      29.3082689507000005
  coef(1315) =     -41.8832937561999969
  coef(1316) =     103.0576337530000046
  coef(1317) =     103.1461569389999937
  coef(1318) =      83.8157495702999995
  coef(1319) =      17.6364662768000002
  coef(1320) =      20.6358332106999995
  coef(1321) =     -12.6504092966999995
  coef(1322) =      23.9305208634000017
  coef(1323) =      21.4826042872999992
  coef(1324) =     -27.6209316676999990
  coef(1325) =     -25.2546574007000011
  coef(1326) =    5613.6462990299996818
  coef(1327) =      22.6808182985000002
  coef(1328) =     -56.1827434962999988
  coef(1329) =    -408.7961389619999863
  coef(1330) =    2951.0279734699997789
  coef(1331) =   -5094.8701717100002497
  coef(1332) =    -632.5115684899999451
  coef(1333) =     179.4983737670000039
  coef(1334) =    7988.2462856599995575
  coef(1335) =      -2.7293500278999998
  coef(1336) =     -20.7369086206000013
  coef(1337) =     261.4440339679999852
  coef(1338) =     228.2642414219999978
  coef(1339) =     -13.4211833005999992
  coef(1340) =     -67.1333507571000041
  coef(1341) =    -613.5596516909999991
  coef(1342) =     159.8698311329999910
  coef(1343) =    -684.5056744739999885
  coef(1344) =    1695.1452074799999536
  coef(1345) =     826.3601405399999749
  coef(1346) =      26.1617534388000017
  coef(1347) =     229.6488831229999903
  coef(1348) =    -551.4488522939999484
  coef(1349) =   -1714.4629931899999065
  coef(1350) =     -27.1927971940000006
  coef(1351) =     -29.4480764708999985
  coef(1352) =    -764.1996101280000175
  coef(1353) =   -1235.9656315600000198
  coef(1354) =    -189.1920547279999880
  coef(1355) =   -1376.2150249399999211
  coef(1356) =    -188.0578335799999934
  coef(1357) =       3.8836117159899999
  coef(1358) =       2.7314900979300001
  coef(1359) =     -84.7604351452000060
  coef(1360) =      49.4505275786999974
  coef(1361) =      79.7879996008999939
  coef(1362) =     -14.3789612927999997
  coef(1363) =      67.4161678211000037
  coef(1364) =     -53.5023510271999996
  coef(1365) =      10.8581349906000000
  coef(1366) =    -145.4679364019999923
  coef(1367) =    -123.1614167540000011
  coef(1368) =      38.3033803172999967
  coef(1369) =      13.6019126357999998
  coef(1370) =      15.7397642978000007
  coef(1371) =     129.6215716869999994
  coef(1372) =     -19.2098015711999999
  coef(1373) =      12.6609560210000005
  coef(1374) =    -195.8715584279999860
  coef(1375) =      12.7034918890000004
  coef(1376) =      65.2778017113000004
  coef(1377) =      53.0859996682999977
  coef(1378) =     -21.7704349501000003
  coef(1379) =     -46.4788551492000011
  coef(1380) =     216.7632357619999937
  coef(1381) =     -37.6504896531999975
  coef(1382) =     184.5030703200000062
  coef(1383) =     110.1647483120000004
  coef(1384) =     323.7290777570000273
  coef(1385) =     522.5169226220000382
  coef(1386) =     472.3027571449999868
  coef(1387) =   -2946.9448134700000992
  coef(1388) =    -601.4358701260000544
  coef(1389) =    1410.3901757099999941
  coef(1390) =    1209.5688664300000710
  coef(1391) =     457.9441891669999904
  coef(1392) =    -158.4451276430000064
  coef(1393) =   -2853.0604726900000969
  coef(1394) =     461.9477030180000270
  coef(1395) =    -306.7358767270000044
  coef(1396) =    -274.7670160590000137
  coef(1397) =      90.6695507209999931
  coef(1398) =      89.1309147399999944
  coef(1399) =     -35.9754620757000012
  coef(1400) =     104.4434453060000010
  coef(1401) =    -644.8829396910000469
  coef(1402) =     -70.9571766109000066
  coef(1403) =   -1575.8219883300000674
  coef(1404) =    -155.7930078169999888
  coef(1405) =     -35.4784892313000029
  coef(1406) =     -15.9693241210999997
  coef(1407) =     -37.0898076309999993
  coef(1408) =      13.4624594264000006
  coef(1409) =    -101.7334448769999966
  coef(1410) =     206.7534429190000083
  coef(1411) =     187.8629383329999882
  coef(1412) =     -85.6750589699999949
  coef(1413) =      -3.1395638687599998
  coef(1414) =     130.3043740599999865
  coef(1415) =     -78.0567952350999974
  coef(1416) =    -398.3076614529999802
  coef(1417) =    -309.5353012520000107
  coef(1418) =    -116.6524721690000064
  coef(1419) =    -235.8257947380000132
  coef(1420) =     492.9533284010000216
  coef(1421) =     230.6423132749999922
  coef(1422) =     307.1182622430000038
  coef(1423) =     132.9479717220000055
  coef(1424) =    -111.5721230350000042
  coef(1425) =      65.3139290280000040
  coef(1426) =      51.8284169646000024
  coef(1427) =      92.3127135335000020
  coef(1428) =     -38.1208968153000001
  coef(1429) =    -167.3310737369999970
  coef(1430) =      21.2815738040000006
  coef(1431) =     -76.8614304642999997
  coef(1432) =      17.7349782189000003
  coef(1433) =     570.5572258979999560
  coef(1434) =      10.3567634996999995
  coef(1435) =      10.2427031703000004
  coef(1436) =      40.5299200696999975
  coef(1437) =       9.8384545554499994
  coef(1438) =      38.1874186983000001
  coef(1439) =     104.8547925409999948
  coef(1440) =     145.0016208970000093
  coef(1441) =    -147.9431682450000096
  coef(1442) =      -1.0286059820100000
  coef(1443) =       0.6186257757240000
  coef(1444) =       1.9609220724200001
  coef(1445) =      -8.7438896892199995
  coef(1446) =     -18.1992017791999992
  coef(1447) =       9.1583640376400002
  coef(1448) =      -8.0569108552300008
  coef(1449) =       0.6748330241830000
  coef(1450) =     -12.9339344049000005
  coef(1451) =      -8.0218908877799997
  coef(1452) =       6.8208137448999997
  coef(1453) =       1.5016378727099999
  coef(1454) =      -0.7830424289340000
  coef(1455) =      22.3755467416999991
  coef(1456) =     -19.4418022408000013
  coef(1457) =       2.3659132294199998
  coef(1458) =     -18.4243960774000008
  coef(1459) =      59.6541182978999984
  coef(1460) =      19.0583693009999990
  coef(1461) =       3.9758795613000002
  coef(1462) =      -6.1818165222400001
  coef(1463) =      -8.5186964018900007
  coef(1464) =       1.9429025487900000
  coef(1465) =      26.2034554505000017
  coef(1466) =      15.3392449315999997
  coef(1467) =     -18.5980829848000013
  coef(1468) =      -5.7697553460100002
  coef(1469) =    -388.8519790909999756
  coef(1470) =    -147.5570067319999907
  coef(1471) =     -54.6316222214000007
  coef(1472) =     159.5694542379999916
  coef(1473) =      27.1635299562999997
  coef(1474) =     252.2759133449999922
  coef(1475) =    -181.2185284290000027
  coef(1476) =    -173.1653621060000035
  coef(1477) =      99.8746462825999970
  coef(1478) =     147.8108187160000000
  coef(1479) =    -175.3957476740000061
  coef(1480) =     160.9348549489999982
  coef(1481) =     191.5186725990000127
  coef(1482) =     -41.5531335708000000
  coef(1483) =     -71.3170679247999999
  coef(1484) =      -0.6332661325409999
  coef(1485) =     -21.5269065853999990
  coef(1486) =      88.3003507697000032
  coef(1487) =      -2.0118486115900001
  coef(1488) =     141.6049393620000103
  coef(1489) =       8.0947598680499997
  coef(1490) =       5.1241657064600004
  coef(1491) =      11.0657138667999995
  coef(1492) =       0.4141506393790000
  coef(1493) =       0.0696097322030600
  coef(1494) =      85.5337436077000035
  coef(1495) =     -10.8551515203999998
  coef(1496) =      -9.9362728013399995
  coef(1497) =       8.8499136132699991
  coef(1498) =      -0.0811080275128900
  coef(1499) =     -18.8344263325000014
  coef(1500) =       1.8673426046699999
  coef(1501) =     153.5919532650000008
  coef(1502) =      22.1189590144999997
  coef(1503) =     -75.2128697340999963
  coef(1504) =      36.6822442890000033
  coef(1505) =       6.2938136263200004
  coef(1506) =      12.1461951849999998
  coef(1507) =       8.4098823813199992
  coef(1508) =      -7.8784420825900003
  coef(1509) =       1.9437436689300001
  coef(1510) =     -29.8037562099999995
  coef(1511) =      -6.9648860423599999
  coef(1512) =     -27.1893223777000017
  coef(1513) =       9.4099997469800005
  coef(1514) =      -4.4427009994100004
  coef(1515) =      -0.4228413008740000
  coef(1516) =      -1.6465103861699999
  coef(1517) =      -0.3855253773870000
  coef(1518) =     -36.6923986258999975
  coef(1519) =      -1.0204802663600001
  coef(1520) =      -4.4733679136399997
  coef(1521) =      16.2172030822999993
  coef(1522) =      12.6343059384000007
  coef(1523) =      13.3406396456999996
  coef(1524) =      34.2591438051999972
  coef(1525) =      18.0572966908000012
  coef(1526) =      68.9888788051000006
  coef(1527) =     -35.0701510605000024
  coef(1528) =    -229.8368039730000021
  coef(1529) =      34.9142328328000033
  coef(1530) =     -65.1435823509000045
  coef(1531) =       1.6935106650500000
  coef(1532) =     -23.4144580096000006
  coef(1533) =      52.5792189882999992
  coef(1534) =     -53.3444064201000003
  coef(1535) =     -32.3376994775000028
  coef(1536) =     -45.9750197624999970
  coef(1537) =      63.3873118153999968
  coef(1538) =     -52.4781180876000022
  coef(1539) =       9.9624713854499998
  coef(1540) =    -153.5266821280000045
  coef(1541) =     -38.3913455691999985
  coef(1542) =     -74.6461568616000051
  coef(1543) =    -208.8978164780000100
  coef(1544) =     -22.7946431374000014
  coef(1545) =      81.4514581763999956
  coef(1546) =      76.8549589339999955
  coef(1547) =     218.9953149570000051
  coef(1548) =    -365.6888516320000235
  coef(1549) =    -148.6140570709999906
  coef(1550) =    -496.1001635900000224
  coef(1551) =     694.1605349650000107
  coef(1552) =    -524.4505673139999544
  coef(1553) =    -249.0547974129999886
  coef(1554) =    -622.2253334150000228
  coef(1555) =    -477.0352662540000210
  coef(1556) =    -121.4644749640000043
  coef(1557) =    -111.3319791400000014
  coef(1558) =     182.3437765120000051
  coef(1559) =    -372.7317291940000246
  coef(1560) =    -431.7689826829999902
  coef(1561) =      69.2590237550000012
  coef(1562) =     -10.5036943605000008
  coef(1563) =     -50.6445702812999983
  coef(1564) =     521.3170682749999969
  coef(1565) =      69.8925116885999955
  coef(1566) =      63.0367642201999985
  coef(1567) =    -307.3782747120000067
  coef(1568) =    -285.1362461329999860
  coef(1569) =      17.8934188769999984
  coef(1570) =     380.3443076969999765
  coef(1571) =     118.1371587099999942
  coef(1572) =     -86.5967512651999982
  coef(1573) =      20.2992554624000014
  coef(1574) =    -117.2138790900000060
  coef(1575) =    -471.5401477309999905
  coef(1576) =     -16.6159782478999993
  coef(1577) =     -43.1354487338999988
  coef(1578) =     448.0375944319999917
  coef(1579) =      48.1952639184000020
  coef(1580) =      75.9151192874000031
  coef(1581) =     -88.1321617573999987
  coef(1582) =     867.6378087289999712
  coef(1583) =     894.8678298209999866
  coef(1584) =     273.6845013750000248
  coef(1585) =      -6.9295027076500002
  coef(1586) =     -91.6765202009000006
  coef(1587) =      64.1283397820999994
  coef(1588) =      55.8783580473999990
  coef(1589) =     -16.9556583100000005
  coef(1590) =     289.5980802369999765
  coef(1591) =      -1.6798696904399999
  coef(1592) =     -23.8387305990999998
  coef(1593) =     -43.5127928674000017
  coef(1594) =      25.2576996308999995
  coef(1595) =     -55.8822102536000003
  coef(1596) =      48.5074485904999975
  coef(1597) =      69.9703719104000044
  coef(1598) =      47.6092018527999983
  coef(1599) =     -42.1750884360999976
  coef(1600) =      97.8082252109000052
  coef(1601) =      50.6783428852999975
  coef(1602) =     -72.6650746045999938
  coef(1603) =     -20.9319128734000017
  coef(1604) =     224.5442657510000117
  coef(1605) =      17.4972087945000006
  coef(1606) =     357.3080975870000202
  coef(1607) =     164.5433013180000046
  coef(1608) =    -141.2669422079999890
  coef(1609) =     -61.4233959457000012
  coef(1610) =    -103.0736413519999957
  coef(1611) =     167.1257063379999863
  coef(1612) =     844.6642967770000041
  coef(1613) =     457.8420231559999820
  coef(1614) =    -277.4717827320000083
  coef(1615) =     239.7569087379999928
  coef(1616) =    -368.8957493240000076
  coef(1617) =     842.8233171379999931
  coef(1618) =     295.9427931320000198
  coef(1619) =      71.5988896252000018
  coef(1620) =      56.3576721437000003
  coef(1621) =      29.8443260786000018
  coef(1622) =     -10.0189900054999992
  coef(1623) =     -15.3966545877000005
  coef(1624) =      37.2852430913999981
  coef(1625) =      15.0815206156000006
  coef(1626) =       7.2828352839399999
  coef(1627) =     -19.4745949474000000
  coef(1628) =      15.3419435677999996
  coef(1629) =      38.6247183103999987
  coef(1630) =      -2.4021894575900000
  coef(1631) =     -65.1244135886000066
  coef(1632) =      -3.7003303155000000
  coef(1633) =      23.9277180073000011
  coef(1634) =     -84.9278415815000045
  coef(1635) =      45.5115452058999992
  coef(1636) =      31.2614976857999984
  coef(1637) =      16.4870463232999995
  coef(1638) =       8.5274533119200004
  coef(1639) =     112.3757039689999999
  coef(1640) =      43.6600044700999987
  coef(1641) =      60.6816986482999994
  coef(1642) =      69.8812006883999999
  coef(1643) =      15.2976403411999993
  coef(1644) =       5.2356568803099997
  coef(1645) =      32.1952402233999990
  coef(1646) =     -33.0683459384000003
  coef(1647) =      29.3936336886999996
  coef(1648) =      -8.6740745140200008
  coef(1649) =      25.3416705819000008
  coef(1650) =    -176.9526452700000050
  coef(1651) =     -47.0149285099000025
  coef(1652) =     158.9551612710000086
  coef(1653) =      96.5137774015999952
  coef(1654) =      23.1335427256000017
  coef(1655) =      22.4449382248000013
  coef(1656) =       4.1547660141499998
  coef(1657) =    -240.6188140989999908
  coef(1658) =     -97.4543870913999939
  coef(1659) =    -135.1676376570000002
  coef(1660) =     -31.9156676055000013
  coef(1661) =     -18.8911193194000013
  coef(1662) =      39.2150929411999982
  coef(1663) =       9.6494588302600004
  coef(1664) =      11.7838774235999999
  coef(1665) =     -27.4381692916000013
  coef(1666) =       1.2477211445400000
  coef(1667) =      10.0823721061999994
  coef(1668) =      74.4652231727999947
  coef(1669) =      51.5521380572999988
  coef(1670) =      27.4578061720000015
  coef(1671) =       9.1419618452400009
  coef(1672) =      46.0952744837999973
  coef(1673) =     -40.2548367253999970
  coef(1674) =       5.4977557983300001
  coef(1675) =      41.9276642236999990
  coef(1676) =      76.7228866195000023
  coef(1677) =      45.1492294279000035
  coef(1678) =     -22.5021290900999986
  coef(1679) =      -6.5789112122200004
  coef(1680) =       8.4928261275800008
  coef(1681) =     -27.9686173905000004
  coef(1682) =      -2.3445717997500002
  coef(1683) =    -129.9024640240000110
  coef(1684) =     -37.2562821105000026
  coef(1685) =     -23.1911798998000016
  coef(1686) =     -32.6041639598999993
  coef(1687) =      46.0576979949999981
  coef(1688) =     -23.7090397197999998
  coef(1689) =    -103.3912324599999977
  coef(1690) =      -1.4736800144300000
  coef(1691) =       5.8835454213200000
  coef(1692) =     -41.7274020739999969
  coef(1693) =      14.5991960341000002
  coef(1694) =     -13.5271581211999994
  coef(1695) =     -10.1957725999999997
  coef(1696) =     -14.2419808385000000
  coef(1697) =      -0.0176241558253600
  coef(1698) =       2.0225347676999998
  coef(1699) =     -65.8570037438999947
  coef(1700) =      -4.4421989828799999
  coef(1701) =      -2.9349677614599998
  coef(1702) =     135.9449675669999920
  coef(1703) =     100.1286908080000018
  coef(1704) =       8.6237076491200000
  coef(1705) =      56.1148318584000023
  coef(1706) =     -66.3887347355999964
  coef(1707) =     -23.9492807642999992
  coef(1708) =      24.9161147422000013
  coef(1709) =      27.7345065208000001
  coef(1710) =      -5.4514873510499999
  coef(1711) =     -77.5483027810000038
  coef(1712) =     -65.0869021418999978
  coef(1713) =      53.6177443447000002
  coef(1714) =       7.2913093023700002
  coef(1715) =      13.6923877810000008
  coef(1716) =       8.6742459212600007
  coef(1717) =      -3.5097545899400000
  coef(1718) =      13.3255363250999999
  coef(1719) =      42.2917871734000030
  coef(1720) =      29.3542164129999996
  coef(1721) =       0.3073586844150000
  coef(1722) =      -0.1132023314480000
  coef(1723) =      -0.4284637756000000
  coef(1724) =      -0.7879352131920000
  coef(1725) =       1.2711460623899999
  coef(1726) =      -0.1519419665020000
  coef(1727) =       1.8845710380800000
  coef(1728) =       0.0079037812958890
  coef(1729) =       0.8633289640539999
  coef(1730) =       1.3056894820700000
  coef(1731) =      -0.2311350814960000
  coef(1732) =      -0.1279509479940000
  coef(1733) =      -0.7944088080960000
  coef(1734) =      -0.3253386983990000
  coef(1735) =       1.6835526259500000
  coef(1736) =       0.1218113460980000
  coef(1737) =      -0.0555103477845600
  coef(1738) =      -1.4092607541000000
  coef(1739) =       1.0275509427300000
  coef(1740) =       0.6475955035130000
  coef(1741) =      -0.2041352613030000
  coef(1742) =       0.7154845718360000
  coef(1743) =      -0.2326588107490000
  coef(1744) =       1.4046312872000000
  coef(1745) =      -1.5349044021200000
  coef(1746) =      -0.0168642271272500
  coef(1747) =       0.1007108786060000
  coef(1748) =       2.3114243874799998
  coef(1749) =      -2.8005023040900001
  coef(1750) =      12.9314767017999994
  coef(1751) =       2.9002373616899999
  coef(1752) =       3.1708319755100001
  coef(1753) =      -4.2764059105000003
  coef(1754) =       1.7874366195300000
  coef(1755) =       2.4640027330900001
  coef(1756) =      -3.0999512947199999
  coef(1757) =      -2.3619798622100001
  coef(1758) =      11.3867913816000002
  coef(1759) =     -14.3248831625000008
  coef(1760) =       5.1346013164300004
  coef(1761) =       2.7164678365300001
  coef(1762) =      -6.2661836400600004
  coef(1763) =       0.0632519399969700
  coef(1764) =       0.0755007312267500
  coef(1765) =      -1.3676731861700000
  coef(1766) =       0.4178904730830000
  coef(1767) =      32.9466244019000030
  coef(1768) =       0.2337638856400000
  coef(1769) =       0.1445939639460000
  coef(1770) =       0.3587464144990000
  coef(1771) =      -0.0946939473650700
  coef(1772) =       0.5010936753340000
  coef(1773) =       0.0261602988620100
  coef(1774) =      -3.5910964202400000
  coef(1775) =       1.7266779141399999
  coef(1776) =       1.6132338323800000
  coef(1777) =       0.3285843198800000
  coef(1778) =      -1.8084342123899999
  coef(1779) =       0.5671772779820000
  coef(1780) =       0.3276735204530000
  coef(1781) =      -0.7020156273510000
  coef(1782) =      -1.2838508651900000
  coef(1783) =      -1.6261168052099999
  coef(1784) =      -9.7771020418799992
  coef(1785) =      -4.1270143267400003
  coef(1786) =      -8.3309149118699999
  coef(1787) =      -0.6515342433400000
  coef(1788) =      -1.5623404017300000
  coef(1789) =       0.2265432555540000
  coef(1790) =       0.7726176167240000
  coef(1791) =       1.5885388031400001
  coef(1792) =       0.3845504386330000
  coef(1793) =      -1.0858855168399999
  coef(1794) =      -0.6008475034050000
  coef(1795) =       0.0195737046710100
  coef(1796) =      -3.1555643912800000
  coef(1797) =       3.3014120345900002
  coef(1798) =       0.4136528490060000
  coef(1799) =      -2.2684969582300001
  coef(1800) =       2.0849613116599999
  coef(1801) =       0.2035545180280000
  coef(1802) =       1.3693038378900000
  coef(1803) =      -8.9662718652399995
  coef(1804) =      -4.0212074945999996
  coef(1805) =      -8.9538962358600003
  coef(1806) =      -0.2265693345810000
  coef(1807) =      11.9235517227999992
  coef(1808) =      -2.5046172078000000
  coef(1809) =      -0.6496066042370000
  coef(1810) =       0.2844715495920000
  coef(1811) =      -0.4614551788940000
  coef(1812) =      -4.9236393650799997
  coef(1813) =      42.6392845474000026
  coef(1814) =      17.0540346947999986
  coef(1815) =       4.4192562300900002
  coef(1816) =     -13.4144905170000008
  coef(1817) =       3.0627214940499998
  coef(1818) =       1.6509488458699999
  coef(1819) =      42.3847330465999974
  coef(1820) =      18.3239373989000001
  coef(1821) =       7.2200250660199998
  coef(1822) =      27.5588210057000005
  coef(1823) =       3.4155366041500002
  coef(1824) =      13.9434419415999997
  coef(1825) =      -1.5903510724400001
  coef(1826) =       0.0373450382590500
  coef(1827) =      31.2217305277000001
  coef(1828) =     -27.3748943577000006
  coef(1829) =      -1.1514537045800000
  coef(1830) =     -48.2391064611000004
  coef(1831) =      98.6873851876000003
  coef(1832) =      74.5842883969000070
  coef(1833) =       4.2033706760499996
  coef(1834) =       3.4952577381799999
  coef(1835) =     -14.0822588523000007
  coef(1836) =      15.9121952088000000
  coef(1837) =      24.9871452679999990
  coef(1838) =      55.9144526338000034
  coef(1839) =      44.8591209344999982
  coef(1840) =      -5.8211962277299998
  coef(1841) =      -4.7767500792200002
  coef(1842) =     -18.6172202224999985
  coef(1843) =     -14.8610641719000007
  coef(1844) =      -2.6195739175999999
  coef(1845) =      17.4055725264999985
  coef(1846) =      16.6578555795999996
  coef(1847) =      17.0927624390000013
  coef(1848) =      -4.9346909718300003
  coef(1849) =     -20.9219665291000005
  coef(1850) =       3.0911512296599999
  coef(1851) =       1.7423896361900000
  coef(1852) =      12.1193447908999996
  coef(1853) =      29.3323348712999987
  coef(1854) =      10.3257661106000000
  coef(1855) =       0.9436698893870000
  coef(1856) =      -7.1119675772199997
  coef(1857) =     -29.3697160271000008
  coef(1858) =      -2.8274528770199998
  coef(1859) =      -6.7160684720399999
  coef(1860) =      25.6231388717000002
  coef(1861) =     -73.4951483938999957
  coef(1862) =     -68.6241455236999940
  coef(1863) =     -13.1362338190999992
  coef(1864) =     -27.0991282337000001
  coef(1865) =       2.2144500473800002
  coef(1866) =      -2.2650731798699999
  coef(1867) =      -4.6419364350399999
  coef(1868) =      -9.8586088265100003
  coef(1869) =      -7.5323276864800004
  coef(1870) =      -0.0844297015333300
  coef(1871) =       2.6458951718899999
  coef(1872) =       0.1553535288080000
  coef(1873) =       2.3480761462499999
  coef(1874) =       7.7690911231499999
  coef(1875) =     -10.2368807551999996
  coef(1876) =       0.8835852535449999
  coef(1877) =     -10.4675723769999998
  coef(1878) =      11.8533146269999996
  coef(1879) =       5.8377834178799999
  coef(1880) =       1.3833953477000001
  coef(1881) =       1.1775207893699999
  coef(1882) =      -9.3066736108699999
  coef(1883) =     -39.5144060342000003
  coef(1884) =      -0.5906239485340000
  coef(1885) =     -32.7016918755000034
  coef(1886) =     -25.2562325845000011
  coef(1887) =      20.4149283867000015
  coef(1888) =      12.9898721892999998
  coef(1889) =       6.5657087226300002
  coef(1890) =     158.3471827719999965
  coef(1891) =     -52.2898445666999976
  coef(1892) =     -51.2873977133000025
  coef(1893) =      -3.9305225193700002
  coef(1894) =      31.0132069098999992
  coef(1895) =    -128.4999095050000051
  coef(1896) =     -80.0356396449000016
  coef(1897) =     -21.9292011604000017
  coef(1898) =     -27.2462137595999998
  coef(1899) =      -3.2261839751000001
  coef(1900) =      -1.1897224838900000
  coef(1901) =      -0.3281387574360000
  coef(1902) =       0.3677006403070000
  coef(1903) =      -2.6002740181499999
  coef(1904) =      -0.6818490559590000
  coef(1905) =      -0.8739572410980000
  coef(1906) =       0.4059458807520000
  coef(1907) =       0.4004520377760000
  coef(1908) =      -1.0089013973600001
  coef(1909) =     -13.0132123185000008
  coef(1910) =       3.7011775024300002
  coef(1911) =       0.5307311950920000
  coef(1912) =      -9.2287529977799991
  coef(1913) =       8.3323002364099992
  coef(1914) =      -1.3046430833600000
  coef(1915) =      -1.6839067033499999
  coef(1916) =      -0.3323072352570000
  coef(1917) =      -0.1161867641920000
  coef(1918) =      -8.4836401616000003
  coef(1919) =      -5.3669098225800003
  coef(1920) =      -6.8154405113300003
  coef(1921) =     -17.4190474114000011
  coef(1922) =      -4.6311080755400003
  coef(1923) =      -4.3998957587099996
  coef(1924) =      -4.2800073528900002
  coef(1925) =     -28.4847115120999987
  coef(1926) =       1.4547953365999999
  coef(1927) =      11.5045869946000003
  coef(1928) =      -3.5315731215800001
  coef(1929) =       5.1587926145800003
  coef(1930) =       0.0548751926071000
  coef(1931) =     -13.9981534014999998
  coef(1932) =     -11.7451237389000003
  coef(1933) =      -0.4264210749810000
  coef(1934) =      10.3399759113999998
  coef(1935) =       0.7381506714279999
  coef(1936) =      19.3004381414000008
  coef(1937) =       8.6250226208199994
  coef(1938) =       5.8352164815299998
  coef(1939) =       7.9995943813799997
  coef(1940) =       2.2473700751800001
  coef(1941) =      -3.3012179062999998
  coef(1942) =      -2.6387487642999998
  coef(1943) =      -1.2695935481600000
  coef(1944) =       2.0785564080899999
  coef(1945) =       2.5574848820699998
  coef(1946) =       0.6790175895310000
  coef(1947) =      -6.8120175055200001
  coef(1948) =     -12.6292417085000004
  coef(1949) =      -6.5544819405499997
  coef(1950) =      -1.8230264548599999
  coef(1951) =      -6.0905531242200004
  coef(1952) =      -3.0931378042000000
  coef(1953) =      12.3922803273000000
  coef(1954) =      -2.1201059463599998
  coef(1955) =      -2.2809307323199999
  coef(1956) =      -0.9747484607000000
  coef(1957) =       2.4922168141399998
  coef(1958) =      -0.7377066824330000
  coef(1959) =      -0.0538881972376300
  coef(1960) =       1.7784251613199999
  coef(1961) =       0.1806165198800000
  coef(1962) =      13.8202756776999998
  coef(1963) =       5.6447697070600000
  coef(1964) =       2.7405338577900000
  coef(1965) =       4.8652573832400003
  coef(1966) =     -15.2137151251000002
  coef(1967) =       3.7535402423300002
  coef(1968) =       9.8468712224200008
  coef(1969) =       0.3976871625780000
  coef(1970) =      -8.8544948542899995
  coef(1971) =       7.2817430562199998
  coef(1972) =       0.0902144528980000
  coef(1973) =      -2.4360401301099999
  coef(1974) =       3.1774468952400001
  coef(1975) =       1.0340391955499999
  coef(1976) =       0.0355977789036200
  coef(1977) =      -1.3063253236600001
  coef(1978) =       7.0688173799199996
  coef(1979) =      -0.5192100296270000
  coef(1980) =      -0.1635841950490000
  coef(1981) =     -36.8507386514000004
  coef(1982) =      -2.7444523385299999
  coef(1983) =       8.6385586970699997
  coef(1984) =      -4.8634320946600003
  coef(1985) =      -0.5909692815299999
  coef(1986) =      -0.1310616674160000
  coef(1987) =     -11.1410518911000000
  coef(1988) =      -9.3294035749800006
  coef(1989) =      10.3149182212999992
  coef(1990) =      -0.1511040982450000
  coef(1991) =      -8.4501284944999995
  coef(1992) =       0.9439253706720000
  coef(1993) =      -0.4176740315910000
  coef(1994) =      -5.6468858792100001
  coef(1995) =       0.2095072761590000
  coef(1996) =       3.0955716758700000
  coef(1997) =      -7.0601893466899996
  coef(1998) =       7.6435381396300004
  coef(1999) =      -2.3340222924899998
  coef(2000) =       0.8390777891170000
  coef(2001) =      11.9734639005000005
  coef(2002) =      -2.5245017150400000
  coef(2003) =      -2.9611389387700000
  coef(2004) =      -3.6146474429300000
  coef(2005) =     -30.4246578399999983
  coef(2006) =      46.4672155722999989
  coef(2007) =      35.2981097461000033
  coef(2008) =      36.4297097558999994
  coef(2009) =      -8.0440429364600003
  coef(2010) =       0.5256170674670000
  coef(2011) =      20.4854828839999996
  coef(2012) =       8.6896480313300000
  coef(2013) =     -20.7027937428999991
  coef(2014) =    -186.2217836259999899
  coef(2015) =     -15.8772026211000004
  coef(2016) =     -31.0745127054999983
  coef(2017) =      27.1315974474000008
  coef(2018) =       4.7740642880399999
  coef(2019) =     102.2894583270000055
  coef(2020) =      -3.0408333223500001
  coef(2021) =     -28.7524091918000018
  coef(2022) =     -12.7764634302999998
  coef(2023) =       3.4092679974400002
  coef(2024) =     116.1513365990000040
  coef(2025) =     -14.0188326415999995
  coef(2026) =      -4.0058671426299997
  coef(2027) =      -2.2977293448200000
  coef(2028) =      -8.8788900167799998
  coef(2029) =      -2.8086023624899998
  coef(2030) =      17.7055645822999992
  coef(2031) =      -4.4117601509700002
  coef(2032) =      -0.7899010400140000
  coef(2033) =      12.8659698004000003
  coef(2034) =       2.1886347655999998
  coef(2035) =      -5.0261469319999996
  coef(2036) =      -5.7386233377800000
  coef(2037) =      25.4713545164999999
  coef(2038) =       3.5549906649100000
  coef(2039) =     -15.2520928935000004
  coef(2040) =       0.8185365901690000
  coef(2041) =     -17.4642206231000010
  coef(2042) =      -2.3192506630200000
  coef(2043) =     -23.2286793628000012
  coef(2044) =       2.1632131911000001
  coef(2045) =      -1.4305928720000001
  coef(2046) =     -14.0620999722000004
  coef(2047) =      -4.7242928719000004
  coef(2048) =       4.2452422321999999
  coef(2049) =       3.0589822125000001
  coef(2050) =      -6.4114451349100001
  coef(2051) =      -0.7029160255120001
  coef(2052) =       1.1529623285799999
  coef(2053) =      -7.5854707232100003
  coef(2054) =      37.1997883188000031
  coef(2055) =       2.6065109975700000
  coef(2056) =      -6.1271913055400002
  coef(2057) =      20.0501432987999983
  coef(2058) =      -2.6999004107600002
  coef(2059) =       2.4776458484399999
  coef(2060) =     -20.4192977460999998
  coef(2061) =      -2.5693406220799999
  coef(2062) =     -19.1238019347999995
  coef(2063) =    -307.1034899429999996
  coef(2064) =    -197.8465248059999908
  coef(2065) =     247.3693353909999928
  coef(2066) =    -116.4107413769999937
  coef(2067) =     314.0963952160000190
  coef(2068) =    -181.6735761639999964
  coef(2069) =    -197.6993546369999990
  coef(2070) =      82.2524963998999965
  coef(2071) =     287.1655286270000147
  coef(2072) =    -248.8621647690000032
  coef(2073) =     471.3782686960000206
  coef(2074) =    -553.4145552249999582
  coef(2075) =     -42.4915308867999997
  coef(2076) =     -31.9856088141999990
  coef(2077) =       4.9418894849499999
  coef(2078) =      89.6826613425999994
  coef(2079) =      65.9452556635000064
  coef(2080) =     -30.3239921406999997
  coef(2081) =    -207.0633278390000100
  coef(2082) =      43.0400453931000015
  coef(2083) =      12.3274278063999994
  coef(2084) =      17.9063929787999996
  coef(2085) =      28.4377209999999998
  coef(2086) =      17.6539160300000013
  coef(2087) =     -17.3552057200999990
  coef(2088) =      22.6403462028000000
  coef(2089) =      56.7327115189999986
  coef(2090) =     -75.9429886807999992
  coef(2091) =      -9.7736859088300001
  coef(2092) =      -4.3814807326100000
  coef(2093) =      76.8130833291999977
  coef(2094) =     -96.7107244669000039
  coef(2095) =     -60.3834671186999969
  coef(2096) =      -4.6282425212099998
  coef(2097) =     -36.6403664807999974
  coef(2098) =      23.0172416181000017
  coef(2099) =       2.7053816342200001
  coef(2100) =      23.4410176461000006
  coef(2101) =      -9.9586965466800006
  coef(2102) =      11.8292149919000007
  coef(2103) =      55.0011823473000021
  coef(2104) =      31.4527213639000003
  coef(2105) =       6.7977038831199996
  coef(2106) =      -5.0057490022800000
  coef(2107) =      -3.9578916257499999
  coef(2108) =       4.4294826803199996
  coef(2109) =       0.6090096186310000
  coef(2110) =      21.0390696002999995
  coef(2111) =      78.0176947031999930
  coef(2112) =     -11.2785271647999998
  coef(2113) =       2.8789889841799998
  coef(2114) =      67.0376175343999989
  coef(2115) =      42.9488987778999984
  coef(2116) =      28.9679459554999994
  coef(2117) =      24.4730854123000015
  coef(2118) =      11.2571739577999992
  coef(2119) =       0.5008786764490000
  coef(2120) =       0.6653673381080000
  coef(2121) =     -48.7733648821999992
  coef(2122) =     -50.3651169886000005
  coef(2123) =     115.1644818190000024
  coef(2124) =       2.3669150839199999
  coef(2125) =     -22.3204144299000014
  coef(2126) =     140.9467576000000122
  coef(2127) =      21.5617022863999992
  coef(2128) =     120.3256200419999971
  coef(2129) =       5.5616253180899999
  coef(2130) =      65.2917097364999961
  coef(2131) =      17.5987573807999986
  coef(2132) =       3.3459134993799999
  coef(2133) =       1.1992527123500001
  coef(2134) =      24.8508621808999983
  coef(2135) =       8.7615641886399995
  coef(2136) =       5.0697143982100004
  coef(2137) =       3.5257586881099998
  coef(2138) =       7.3227506549300001
  coef(2139) =      11.7101143820000004
  coef(2140) =      23.4870247361000004
  coef(2141) =     -38.6399498071999972
  coef(2142) =       1.0797998631400001
  coef(2143) =      -1.3857080591800000
  coef(2144) =      25.8278850085000009
  coef(2145) =     -32.6397877742999967
  coef(2146) =     -25.8115197995999992
  coef(2147) =       1.8500827744799999
  coef(2148) =     -54.2200552496000014
  coef(2149) =      29.6553208895999987
  coef(2150) =      36.8235578931000020
  coef(2151) =      16.4306253469000012
  coef(2152) =     112.4748701890000007
  coef(2153) =      28.0936813072000007
  coef(2154) =      15.6935119692999994
  coef(2155) =     -33.7383593383000004
  coef(2156) =       1.2979348089600000
  coef(2157) =     -16.6343120577000008
  coef(2158) =     -29.3093561218999987
  coef(2159) =      -6.6795434341400002
  coef(2160) =      21.9595387086000002
  coef(2161) =      -1.9937983392500001
  coef(2162) =      27.0064449700999987
  coef(2163) =     -77.6757693863000043
  coef(2164) =       5.3007760758200000
  coef(2165) =      10.6213214875999995
  coef(2166) =     -45.1064610163999973
  coef(2167) =      -6.6156750870399996
  coef(2168) =     -22.2924581892000013
  coef(2169) =      54.3335326030000019
  coef(2170) =      36.5949673583000035
  coef(2171) =      24.0899853963000012
  coef(2172) =      37.6972794909000015
  coef(2173) =     -66.0418792906999954
  coef(2174) =     265.2498924710000097
  coef(2175) =      65.1817485855000029
  coef(2176) =     -91.4420684035000022
  coef(2177) =      -5.9096767857000003
  coef(2178) =     -96.6996210540000050
  coef(2179) =      12.4808992392999993
  coef(2180) =       3.4045657282600001
  coef(2181) =    -268.9310125490000019
  coef(2182) =     -34.5019609144000015
  coef(2183) =     -12.2547546595999997
  coef(2184) =     -11.5475132991000002
  coef(2185) =     -19.4812810994000003
  coef(2186) =     -17.3219341076999989
  coef(2187) =      28.4059049110999986
  coef(2188) =     -41.7257861503999976
  coef(2189) =      -2.2946314059900002
  coef(2190) =       1.4994312009399999
  coef(2191) =       0.0981179041699900
  coef(2192) =      33.1687434948999993
  coef(2193) =     -26.3152428922999988
  coef(2194) =     110.1599935979999998
  coef(2195) =      69.2825347002000029
  coef(2196) =      32.3636887457999975
  coef(2197) =      30.4317448078999995
  coef(2198) =     -97.4061673899999931
  coef(2199) =     -85.6195057720999984
  coef(2200) =     226.1541682870000045
  coef(2201) =      74.8599585235999996
  coef(2202) =      98.6625718691999936
  coef(2203) =     -82.6886095229999967
  coef(2204) =     -72.5611268923999972
  coef(2205) =     -50.9696585595999991
  coef(2206) =      22.8806881359999998
  coef(2207) =      34.8451315253999994
  coef(2208) =      44.5645349563000011
  coef(2209) =      82.5236680861000025
  coef(2210) =      19.3305919806999995
  coef(2211) =      -6.3618532526899996
  coef(2212) =       4.5552016970600002
  coef(2213) =     -11.4690137556000007
  coef(2214) =      -3.2347657135099999
  coef(2215) =       5.4968360126400002
  coef(2216) =      16.8395972970000010
  coef(2217) =      63.1084160561999994
  coef(2218) =       7.5898269562099996
  coef(2219) =     -79.6755460661999990
  coef(2220) =     -16.3566346124999988
  coef(2221) =      10.2422827206000004
  coef(2222) =       2.9660542064800000
  coef(2223) =      52.0991139998999984
  coef(2224) =      28.2691762637999986
  coef(2225) =       0.6708551441660000
  coef(2226) =     -62.3786963494999966
  coef(2227) =      15.8622572564999995
  coef(2228) =       8.9448089036699994
  coef(2229) =       7.0080826129200000
  coef(2230) =      11.6968156460999992
  coef(2231) =       9.0399910335599998
  coef(2232) =     -25.0324411321000007
  coef(2233) =       8.0103453718599997
  coef(2234) =     -14.6394242862000006
  coef(2235) =      -7.2521758891400001
  coef(2236) =       0.1992900059020000
  coef(2237) =     -10.3999338761000004
  coef(2238) =      13.2926085415999999
  coef(2239) =     -69.8903019354000037
  coef(2240) =     -19.2170053450000005
  coef(2241) =      10.9320427540999994
  coef(2242) =     -20.0037051151000007
  coef(2243) =      12.5448607284999998
  coef(2244) =      -1.6044992696200000
  coef(2245) =     -79.2229082016999939
  coef(2246) =     -18.5835697187000015
  coef(2247) =      -5.9987651867400000
  coef(2248) =      22.6260741767000013
  coef(2249) =      -4.6553282238399998
  coef(2250) =      10.2236259794999995
  coef(2251) =      -7.7446504841200001
  coef(2252) =      -0.5394814613470000
  coef(2253) =     -17.2816298287999999
  coef(2254) =       3.8251417274000001
  coef(2255) =       7.5196836299200003
  coef(2256) =     -45.3710917444999993
  coef(2257) =      -1.1812235445699999
  coef(2258) =       7.8845567188499999
  coef(2259) =      -2.5513517387200002
  coef(2260) =      -1.1938452813800000
  coef(2261) =      -8.0000202121699999
  coef(2262) =      13.3952516132999992
  coef(2263) =     -12.6018551246000001
  coef(2264) =      16.1534534906999987
  coef(2265) =      -0.5248843836990000
  coef(2266) =      -4.8771520965199997
  coef(2267) =      14.9910105759000007
  coef(2268) =      -0.1501884684110000
  coef(2269) =       0.2449876531490000
  coef(2270) =      -2.0931695078099999
  coef(2271) =      10.2505351076999993
  coef(2272) =       5.1567568778500004
  coef(2273) =      -0.9687941259870000
  coef(2274) =      -1.0192916661600000
  coef(2275) =       4.5419303774999999
  coef(2276) =      -8.1921206543099991
  coef(2277) =      -0.1505224583480000
  coef(2278) =       2.5583116628600000
  coef(2279) =       0.3227783323980000
  coef(2280) =      -4.9146830442300002
  coef(2281) =       8.8227710940600002
  coef(2282) =      -2.8089018760800002
  coef(2283) =       0.8736620562930000
  coef(2284) =       2.1096676379199999
  coef(2285) =      25.9715769035000008
  coef(2286) =     -14.8692965903999994
  coef(2287) =      -9.0021709736600002
  coef(2288) =      -0.4046980418030000
  coef(2289) =      -3.6696541162699998
  coef(2290) =     -10.7647862032999999
  coef(2291) =     -16.5071768711999987
  coef(2292) =      -9.4447931585799996
  coef(2293) =       0.0377604765769200
  coef(2294) =      -9.7596485063300005
  coef(2295) =      65.0204601064999963
  coef(2296) =     -11.1875239011000005
  coef(2297) =       6.6946149627500002
  coef(2298) =     -18.0145305216000011
  coef(2299) =      14.3315536348000006
  coef(2300) =       7.3978504146699997
  coef(2301) =     -22.2436277117000003
  coef(2302) =     -23.2506346262999983
  coef(2303) =      29.6028501334000005
  coef(2304) =     -29.7804770478999998
  coef(2305) =      24.4271043811999995
  coef(2306) =      -1.8240801280600001
  coef(2307) =       8.1249502095199997
  coef(2308) =     -39.7031039222000004
  coef(2309) =      -3.4441581492200002
  coef(2310) =       6.2246893110999997
  coef(2311) =       0.7567674907760000
  coef(2312) =      -2.0736641463800001
  coef(2313) =     118.1579666020000019
  coef(2314) =      -0.8914866890120000
  coef(2315) =      -1.6432363044900000
  coef(2316) =      -0.3004069335340000
  coef(2317) =      -7.8154442826799997
  coef(2318) =       1.0469588139399999
  coef(2319) =       1.1216848214799999
  coef(2320) =     -10.4446884558999997
  coef(2321) =      -3.9736656641299999
  coef(2322) =     -12.5280399020999997
  coef(2323) =       4.4665157562999998
  coef(2324) =       7.4689619278099997
  coef(2325) =       2.1027673854900000
  coef(2326) =       9.1774112212199999
  coef(2327) =      -6.4626144725500003
  coef(2328) =     -53.5598336027999977
  coef(2329) =      -4.0680839879399997
  coef(2330) =      -0.3462442703890000
  coef(2331) =       7.6555841357999999
  coef(2332) =       8.1995127353099999
  coef(2333) =     -42.9160727766999983
  coef(2334) =      -0.9759516542640000
  coef(2335) =      -0.0909072573888100
  coef(2336) =       1.1738774261300000
  coef(2337) =       2.2795420959000001
  coef(2338) =     -25.4665688145000004
  coef(2339) =      21.3539052914999985
  coef(2340) =       1.9482069570400000
  coef(2341) =     -65.3634963612999940
  coef(2342) =      -4.4156847480400003
  coef(2343) =     -41.3556457351999995
  coef(2344) =       8.2490193591000001
  coef(2345) =      42.2047163553000004
  coef(2346) =      59.3000489049999970
  coef(2347) =     -81.3523191812000022
  coef(2348) =      14.7271694050999997
  coef(2349) =     -51.7291454116000011
  coef(2350) =      -7.7184101994100001
  coef(2351) =     -20.9296864040000017
  coef(2352) =       6.2359157407500003
  coef(2353) =      81.2101593237999992
  coef(2354) =      81.5934176955999959
  coef(2355) =     -38.2832490491000001
  coef(2356) =      24.2386806236999988
  coef(2357) =     -19.1060599324999991
  coef(2358) =       5.8888422769800002
  coef(2359) =       0.1812592297970000
  coef(2360) =       6.6889965116700001
  coef(2361) =      15.6581474923999995
  coef(2362) =    -117.9883644949999990
  coef(2363) =     -25.4301549081999987
  coef(2364) =       7.1950807770100003
  coef(2365) =      69.0767980995999977
  coef(2366) =      -4.1192669660999996
  coef(2367) =       1.1574632009800001
  coef(2368) =       2.6747588612800000
  coef(2369) =      20.2797159441999995
  coef(2370) =     -10.3389282821999995
  coef(2371) =      46.4173317209000018
  coef(2372) =      10.7828638904999998
  coef(2373) =      37.7869683802000011
  coef(2374) =      90.6476446588000044
  coef(2375) =     122.7536967000000061
  coef(2376) =      -3.9039007969299999
  coef(2377) =      -2.8831134752300001
  coef(2378) =      -0.4113177586510000
  coef(2379) =       1.6723260333000001
  coef(2380) =      -6.9142105509200000
  coef(2381) =      10.4284423010000005
  coef(2382) =       2.5752989763800000
  coef(2383) =      -1.3932771385900000
  coef(2384) =      19.1525873036000007
  coef(2385) =      35.3974035349000005
  coef(2386) =      12.5345942941999997
  coef(2387) =       8.2392540937599996
  coef(2388) =      -1.3479158500099999
  coef(2389) =     -15.3496809257999995
  coef(2390) =     -14.2430856451000007
  coef(2391) =     -21.9983595459999997
  coef(2392) =       1.3763547270500001
  coef(2393) =      -9.3484205528200004
  coef(2394) =      11.8518524127999996
  coef(2395) =     -25.9087764331999999
  coef(2396) =     -69.8974241635999931
  coef(2397) =       5.3393135375099998
  coef(2398) =       2.9658588084500002
  coef(2399) =      -7.5302917169599999
  coef(2400) =     -19.1512539854999986
  coef(2401) =      45.8689702352999973
  coef(2402) =      28.1895340792000013
  coef(2403) =      14.3654088758000000
  coef(2404) =     -12.3141927217999996
  coef(2405) =      -7.1686178891800001
  coef(2406) =      84.1872833885000063
  coef(2407) =      88.1581010661000022
  coef(2408) =     -25.2761752358000003
  coef(2409) =      23.1899701424000000
  coef(2410) =     -38.8525189865000016
  coef(2411) =     -22.7915203445999985
  coef(2412) =      10.2023415308000001
  coef(2413) =       7.4301308780999999
  coef(2414) =      10.3821234662999995
  coef(2415) =      -0.0808231169951200
  coef(2416) =       8.9454579895199995
  coef(2417) =      -4.4379097121899997
  coef(2418) =     -16.0104550295000010
  coef(2419) =      -3.8275358110400002
  coef(2420) =     -24.3572900196000006
  coef(2421) =       0.9621601836180000
  coef(2422) =      -7.3064269765200001
  coef(2423) =      -4.2596108119100000
  coef(2424) =     -55.9452099566999976
  coef(2425) =      10.8139274838000006
  coef(2426) =      49.9531466245000004
  coef(2427) =       1.6613995463300000
  coef(2428) =       1.0872410724399999
  coef(2429) =       0.8977394010180000
  coef(2430) =      -0.8436960651210000
  coef(2431) =      -0.8655942956829999
  coef(2432) =      -0.1531226193490000
  coef(2433) =       0.7313818410110000
  coef(2434) =       1.6456027284900001
  coef(2435) =      -0.5124077818090000
  coef(2436) =      -0.3814904807880000
  coef(2437) =      -0.8732422258960000
  coef(2438) =       1.1658870181000001
  coef(2439) =      -8.2379026820700005
  coef(2440) =      -2.7627090803500001
  coef(2441) =      -0.6176199777560000
  coef(2442) =       9.7751952741900006
  coef(2443) =      -3.8335607751300000
  coef(2444) =       1.0761552870200000
  coef(2445) =      -2.6663669970599999
  coef(2446) =       8.2426535669799996
  coef(2447) =      -5.6729054541500004
  coef(2448) =      -2.8668761358100001
  coef(2449) =       2.6513720889100001
  coef(2450) =      -0.8886185157580000
  coef(2451) =      -5.7036292041600003
  coef(2452) =      -8.3622795210799996
  coef(2453) =      -2.9132427518299999
  coef(2454) =      -0.5081761325650000
  coef(2455) =      12.0532254516999995
  coef(2456) =       4.6799249124399998
  coef(2457) =       2.3081045774100000
  coef(2458) =       9.8367756983699994
  coef(2459) =       1.3168421815100000
  coef(2460) =       3.7861118699400000
  coef(2461) =       0.5134304785070000
  coef(2462) =      -6.4682622175200004
  coef(2463) =      -3.7577517584300000
  coef(2464) =      -3.0603798902000001
  coef(2465) =      -3.0495433800199998
  coef(2466) =      -0.8721190861770000
  coef(2467) =       0.7229663019380000
  coef(2468) =      -1.2809835109200001
  coef(2469) =      -5.0487723168400001
  coef(2470) =     -11.2262083765000007
  coef(2471) =      -0.0000015600440718
  coef(2472) =      -2.8372282257600001
  coef(2473) =      30.3934631507999988
  coef(2474) =       8.3795245879400007
  coef(2475) =       3.6312954617400002
  coef(2476) =      -0.5631009015050000
  coef(2477) =       0.3917690468120000
  coef(2478) =      -0.2875342557080000
  coef(2479) =      -2.1929189613500002
  coef(2480) =      -6.3321871098800004
  coef(2481) =      -2.2621279438199999
  coef(2482) =      -1.8006521760300001
  coef(2483) =     -15.5022734475000004
  coef(2484) =      -4.5065305658200003
  coef(2485) =       1.4864972584300000
  coef(2486) =     -17.8584124111999998
  coef(2487) =     -11.6475148379999993
  coef(2488) =      -2.3602284621099998
  coef(2489) =      -3.5783471352700000
  coef(2490) =      -0.2163458685830000
  coef(2491) =       0.5397018728630000
  coef(2492) =      34.5534790538999985
  coef(2493) =      11.8164980327000002
  coef(2494) =      16.3963876781000018
  coef(2495) =       6.0443091800600000
  coef(2496) =      -0.3862507191410000
  coef(2497) =      -2.2787999220200001
  coef(2498) =       1.8492098160799999
  coef(2499) =      11.3730855975999994
  coef(2500) =       0.6599663602630000
  coef(2501) =      -0.1355309129350000
  coef(2502) =      -0.2872663521400000
  coef(2503) =       1.4935358712200000
  coef(2504) =       0.1722475969870000
  coef(2505) =      -0.3759833115490000
  coef(2506) =      -0.5631740809180000
  coef(2507) =      -2.8474940751300002
  coef(2508) =      -4.6397115313399997
  coef(2509) =      -2.9203419971000000
  coef(2510) =       0.2984986981880000
  coef(2511) =      -3.2345256602400001
  coef(2512) =      -0.5596764506910000
  coef(2513) =       8.0630430776100006
  coef(2514) =       1.7864334437000000
  coef(2515) =       4.7926535467300004
  coef(2516) =       1.4805396255700001
  coef(2517) =       0.1521741576170000
  coef(2518) =      -4.5591048460900003
  coef(2519) =      -1.8425948188400001
  coef(2520) =      -0.8809961300000000
  coef(2521) =       1.3661753015100000
  coef(2522) =      -0.6116409248750000
  coef(2523) =      -2.3905233225400000
  coef(2524) =     -14.5250272378999998
  coef(2525) =      -0.4550363572500000
  coef(2526) =      -1.7940377697100001
  coef(2527) =      -0.1959543140270000
  coef(2528) =      -1.8864216497499999
  coef(2529) =      -6.5194993837000004
  coef(2530) =      -4.8588796591500003
  coef(2531) =      -1.2811835614100000
  coef(2532) =      -2.9482932268200002
  coef(2533) =      -0.9136564183870000
  coef(2534) =      -0.8665905236040000
  coef(2535) =      -1.6888142607100001
  coef(2536) =      -0.2012371456500000
  coef(2537) =      -0.6486615056230000
  coef(2538) =      -1.2722825953800001
  coef(2539) =       0.3240697646540000
  coef(2540) =      -3.5000574812799998
  coef(2541) =      -2.5711691444799998
  coef(2542) =      -0.7374495817720000
  coef(2543) =      -0.3113782211520000
  coef(2544) =     -10.7904003611999997
  coef(2545) =       4.8345033657099998
  coef(2546) =     -11.7223421565999999
  coef(2547) =      -4.8295082434200003
  coef(2548) =       1.7209035508099999
  coef(2549) =      -1.2151017541000000
  coef(2550) =      -5.0623696367499997
  coef(2551) =      -2.6582000021100001
  coef(2552) =       1.7225543669100001
  coef(2553) =      -7.3918319406700004
  coef(2554) =      -2.0291297685299998
  coef(2555) =       0.3825539112230000
  coef(2556) =       0.8205142640520000
  coef(2557) =      -1.6163931549599999
  coef(2558) =      -1.5242543325400000
  coef(2559) =      -0.9820299077580000
  coef(2560) =       0.9458558975450000
  coef(2561) =      26.7530243377000012
  coef(2562) =       4.0854933479000000
  coef(2563) =       1.9772166872800001
  coef(2564) =       9.9066910692600008
  coef(2565) =       3.2794290598700000
  coef(2566) =       6.0256960243300002
  coef(2567) =      -0.2909190594220000
  coef(2568) =      -1.0545983858000001
  coef(2569) =      -0.0987391611836400
  coef(2570) =      -1.7673436184400000
  coef(2571) =     -11.4602943288999999
  coef(2572) =       0.8836334731460000
  coef(2573) =      -0.3115227901820000
  coef(2574) =      -0.0814741734252600
  coef(2575) =      -7.7663897194600002
  coef(2576) =     -10.5305289267000006
  coef(2577) =       1.4712181400500000
  coef(2578) =      -0.4286019415830000
  coef(2579) =       1.4703697315599999
  coef(2580) =       0.2053021752950000
  coef(2581) =       0.5882186004360000
  coef(2582) =      -0.4913336731010000
  coef(2583) =      -0.4197480421580000
  coef(2584) =      -1.4894573793799999
  coef(2585) =      -0.3985294732130000
  coef(2586) =      -0.3299042738880000
  coef(2587) =      -1.6847384429400001
  coef(2588) =       1.0847678221799999
  coef(2589) =       0.8535286289130000
  coef(2590) =       0.4623554662910000
  coef(2591) =      -0.9397065568920000
  coef(2592) =      -1.0701792654800000
  coef(2593) =      -3.2672918273699998
  coef(2594) =      -1.7058424545399999
  coef(2595) =      -2.3434319477600001
  coef(2596) =      -1.5743326048799999
  coef(2597) =       0.1892218573690000
  coef(2598) =       0.9120178427920000
  coef(2599) =      -1.3028996582000001
  coef(2600) =     -30.2012692182000002
  coef(2601) =     -26.9966871865000009
  coef(2602) =      -2.9826953618699998
  coef(2603) =     -12.3318979319000004
  coef(2604) =       5.1308849678200001
  coef(2605) =      -1.3096615214300000
  coef(2606) =       0.8072791784609999
  coef(2607) =       0.1392002388240000
  coef(2608) =     -20.7950280328999995
  coef(2609) =     -14.3647187758000001
  coef(2610) =     -25.3140280655000005
  coef(2611) =       3.2068753746300001
  coef(2612) =      -4.5760641585500004
  coef(2613) =      -8.8969442393599998
  coef(2614) =     -42.4419865878000024
  coef(2615) =     -14.6653474923000005
  coef(2616) =     -11.9360657877000005
  coef(2617) =      -8.1660507096499995
  coef(2618) =      -0.1158082503600000
  coef(2619) =       4.5226014867400002
  coef(2620) =       1.3621800640299999
  coef(2621) =      -5.9667594378400004
  coef(2622) =      -1.4615748489800000
  coef(2623) =       0.7277804402360000
  coef(2624) =       3.4408984279500001
  coef(2625) =      -2.0380044499699999
  coef(2626) =       2.0263778770299998
  coef(2627) =      -1.8179753387599999
  coef(2628) =       5.7717249244700000
  coef(2629) =       2.7904779565500002
  coef(2630) =      -8.6123701322699997
  coef(2631) =      -5.8381325211300004
  coef(2632) =      -6.4446532226000004
  coef(2633) =       8.8668815198799997
  coef(2634) =       0.9407294946140000
  coef(2635) =      -1.6231700388500001
  coef(2636) =       5.2144688254300000
  coef(2637) =      -4.9492462110600002
  coef(2638) =      -4.6621936985400003
  coef(2639) =      -2.0706947980099999
  coef(2640) =      -0.4603704403110000
  coef(2641) =       0.0507286683287100
  coef(2642) =       1.9549302718499999
  coef(2643) =     -24.0714907017999984
  coef(2644) =     -11.3429436405999997
  coef(2645) =     -27.2111653414000010
  coef(2646) =      -3.1084719514199999
  coef(2647) =      -4.2144738286500001
  coef(2648) =       4.4575241747199996
  coef(2649) =       1.5119604779000000
  coef(2650) =      -8.8438995480100004
  coef(2651) =    -769.9408593079999719
  coef(2652) =  -18628.7339231000005384
  coef(2653) =    2751.7016478199998346
  coef(2654) =   -9301.9948151699991286
  coef(2655) =   -3423.7463952299999619
  coef(2656) =      23.7010626950999992
  coef(2657) =    -189.9436044930000094
  coef(2658) =    1872.9806710000000294
  coef(2659) =    6006.9356478399995467
  coef(2660) =   -2603.6957953900000575
  coef(2661) =     386.4500374410000063
  coef(2662) =    3981.2159951399999045
  coef(2663) =   44377.4614331999982824
  coef(2664) =   -1021.1478275099999564
  coef(2665) =      18.4552725052000000
  coef(2666) =    7677.5440833299999213
  coef(2667) =    6074.3135157400001845
  coef(2668) =   -6965.5057415199999014
  coef(2669) =   13930.1465669000008347
  coef(2670) =   -4683.6369540200003030
  coef(2671) =   16150.0222809000006237
  coef(2672) =       2.7250499877099998
  coef(2673) =    -235.9571324430000061
  coef(2674) =   -2342.4426829700000781
  coef(2675) =     927.5798390869999821
  coef(2676) =   -1127.9119992799999181
  coef(2677) =     251.7907558230000120
  coef(2678) =     881.0405020900000181
  coef(2679) =     -14.2230575631999994
  coef(2680) =    -115.5649976339999938
  coef(2681) =   -1760.6143497200000638
  coef(2682) =     -81.8567775976000007
  coef(2683) =      -8.5191622485699998
  coef(2684) =     154.0674292379999883
  coef(2685) =   -2525.9042390099998556
  coef(2686) =   -1626.6293151299998954
  coef(2687) =    -405.3307200130000183
  coef(2688) =     644.0417239959999733
  coef(2689) =   -1943.6813748300000952
  coef(2690) =    1887.1112714299999880
  coef(2691) =    -478.6494986449999942
  coef(2692) =   -2104.1536273599999731
  coef(2693) =    -483.3180992449999849
  coef(2694) =  -23046.6154018999986874
  coef(2695) =    4537.0251865999998699
  coef(2696) =     523.5947224620000497
  coef(2697) =   -3952.4580053499998940
  coef(2698) =    4044.2221743099999003
  coef(2699) =   16513.5899190999989514
  coef(2700) =  -30100.6000445999998192
  coef(2701) =  -20107.8908393999990949
  coef(2702) =  -41849.8325610999963828
  coef(2703) =       5.5936410680100002
  coef(2704) =     749.0740015669999821
  coef(2705) =    2588.1141011300001082
  coef(2706) =   -2089.3706299300001774
  coef(2707) =   -1031.1735918300000776
  coef(2708) =     152.6002822279999975
  coef(2709) =     944.6278526730000067
  coef(2710) =   -1838.2660107399999561
  coef(2711) =   -2607.4115400699997736
  coef(2712) =     435.9878144020000263
  coef(2713) =    -947.1230535769999506
  coef(2714) =     496.7008329870000125
  coef(2715) =    3755.8412128999998458
  coef(2716) =   -1100.2520827000000736
  coef(2717) =    2316.7457723900001838
  coef(2718) =   -3627.8977498300000661
  coef(2719) =    8184.1301519299995562
  coef(2720) =   -1381.3623473099999046
  coef(2721) =   -2125.3571603700002015
  coef(2722) =   -5487.5700528199995460
  coef(2723) =    2427.0261670300001242
  coef(2724) =    1112.0835243500000615
  coef(2725) =       0.3494254111910000
  coef(2726) =      -2.7440838106899998
  coef(2727) =     -23.3863170177000015
  coef(2728) =    -130.0290394280000044
  coef(2729) =     -29.7781449366999986
  coef(2730) =      25.0411520400999983
  coef(2731) =     -31.0710454264999996
  coef(2732) =      77.9272784380000019
  coef(2733) =     618.3230620909999971
  coef(2734) =     175.4629591080000068
  coef(2735) =     240.1210217340000099
  coef(2736) =      16.1347702439999985
  coef(2737) =       0.1125162413940000
  coef(2738) =    -240.9494684310000139
  coef(2739) =    -450.5879424949999930
  coef(2740) =     301.3494100369999842
  coef(2741) =     185.3872603039999944
  coef(2742) =    -205.7308670159999906
  coef(2743) =     -69.8724554684000054
  coef(2744) =     182.8559678119999887
  coef(2745) =    -333.5349476669999831
  coef(2746) =     -82.2182726875000043
  coef(2747) =      74.6650046062000001
  coef(2748) =      -7.7931172643600002
  coef(2749) =    -565.2921819980000464
  coef(2750) =     -71.8205107077999969
  coef(2751) =     108.2549483300000048
  coef(2752) =   -1295.8086768699999993
  coef(2753) =   -1315.0344826999998986
  coef(2754) =   -2635.0647597699999096
  coef(2755) =    2676.6632039099999929
  coef(2756) =    -447.6257335290000015
  coef(2757) =    1227.4703047900000001
  coef(2758) =    -369.3487788460000161
  coef(2759) =     893.0833055890000196
  coef(2760) =    2010.9038920600000893
  coef(2761) =    -815.7258914620000496
  coef(2762) =   -2766.0017005999998219
  coef(2763) =    5183.0282448999996632
  coef(2764) =    -771.4044054930000129
  coef(2765) =    -326.1744792169999982
  coef(2766) =    1863.5043432299999040
  coef(2767) =      -9.2200092836900005
  coef(2768) =      27.3186734219000016
  coef(2769) =      73.3755517883000010
  coef(2770) =    -466.9790044900000225
  coef(2771) =   -3486.6862298800001554
  coef(2772) =      15.0886362844999997
  coef(2773) =     -12.9817820739999998
  coef(2774) =      37.0800378973999969
  coef(2775) =     -13.0205439693000002
  coef(2776) =      14.5374801718000004
  coef(2777) =     733.1627140809999901
  coef(2778) =     284.7534140019999995
  coef(2779) =     421.8250755729999923
  coef(2780) =   -1383.9245520800000122
  coef(2781) =    -303.1308081689999767
  coef(2782) =     174.6732633800000087
  coef(2783) =     285.9154429509999886
  coef(2784) =    1146.2245357100000547
  coef(2785) =    -350.5711006450000014
  coef(2786) =    -336.2615783389999820
  coef(2787) =    -374.2816951760000279
  coef(2788) =     337.1806305339999881
  coef(2789) =      31.1499578106000001
  coef(2790) =     344.2640186040000003
  coef(2791) =     183.1454829429999904
  coef(2792) =     154.8729344179999998
  coef(2793) =     391.1963563659999750
  coef(2794) =     131.2587083290000010
  coef(2795) =     -25.1568873770999986
  coef(2796) =     226.1118941080000013
  coef(2797) =      14.5328278570999991
  coef(2798) =      10.0962934890000007
  coef(2799) =      28.7959039680999993
  coef(2800) =     426.3013573800000131
  coef(2801) =    1167.4672415000000001
  coef(2802) =    -163.7766405879999922
  coef(2803) =     173.5345286840000085
  coef(2804) =     600.7131915429999935
  coef(2805) =       4.9030861370599998
  coef(2806) =     135.9604351220000069
  coef(2807) =     518.0041218220000019
  coef(2808) =     170.2522263070000008
  coef(2809) =     285.0839504659999761
  coef(2810) =  -67802.6422269000031520
  coef(2811) =     380.4476730999999745
  coef(2812) =     182.8160213600000077
  coef(2813) =   -4648.5360956399999850
  coef(2814) =   -3145.2768125799998415
  coef(2815) =    6660.8213073599999916
  coef(2816) =   47518.7143892000021879
  coef(2817) =   24724.4914642000003369
  coef(2818) =   21496.2467752000011387
  coef(2819) =     -27.6920648438000008
  coef(2820) =    -689.2080764090000002
  coef(2821) =     321.1948414390000153
  coef(2822) =   -1021.7750193199999558
  coef(2823) =   -3182.4914375399998789
  coef(2824) =     760.2672643600000129
  coef(2825) =   -3724.9280160200000864
  coef(2826) =    1796.1173092999999881
  coef(2827) =     824.5208992049999779
  coef(2828) =   -4514.8121820599999410
  coef(2829) =    -565.2452632600000015
  coef(2830) =    -424.1694495680000045
  coef(2831) =   -5499.6496852899999794
  coef(2832) =    6340.7454762600000322
  coef(2833) =    5616.3071541600002092
  coef(2834) =    4200.8678145800004131
  coef(2835) =  -10687.7411955999996280
  coef(2836) =    7309.1649678900002982
  coef(2837) =   -5230.6220107600001938
  coef(2838) =    8738.5850933700003225
  coef(2839) =   -5242.7722346600003220
  coef(2840) =    -417.9047231280000005
  coef(2841) =      -0.6796865989970000
  coef(2842) =       4.7129528419400000
  coef(2843) =     -24.4174552851000009
  coef(2844) =     -52.2585298680999983
  coef(2845) =      67.4279511607000046
  coef(2846) =     -17.4747659878999997
  coef(2847) =    -675.8012682669999549
  coef(2848) =    -208.7529206579999936
  coef(2849) =    -833.5912669450000294
  coef(2850) =     433.7446809290000260
  coef(2851) =    -523.8299560039999960
  coef(2852) =     -44.3215573191999965
  coef(2853) =      29.6803718554999989
  coef(2854) =     121.9964980489999959
  coef(2855) =     711.4469543229999999
  coef(2856) =      71.6140433286000047
  coef(2857) =    -396.4662873949999948
  coef(2858) =    -121.1791194449999978
  coef(2859) =    -129.5031812169999910
  coef(2860) =     464.5196074899999985
  coef(2861) =     432.4587716170000249
  coef(2862) =     -70.9307102638999964
  coef(2863) =    -534.5739134509999531
  coef(2864) =   -2796.6082869900001242
  coef(2865) =    -535.5645251839999901
  coef(2866) =   -1340.3849540000001070
  coef(2867) =    1284.3543489199998930
  coef(2868) =    -874.5649346090000336
  coef(2869) =     139.0063524729999926
  coef(2870) =     601.8271360720000303
  coef(2871) =   -5279.6590516699998261
  coef(2872) =    2660.1294329500001368
  coef(2873) =   -2210.0800349599999208
  coef(2874) =   -1412.7752063499999622
  coef(2875) =   -5894.5687896600002205
  coef(2876) =   -2649.8672097300000132
  coef(2877) =   -8891.6600291399990965
  coef(2878) =   -3222.8560106499999165
  coef(2879) =   -1905.0756891399998949
  coef(2880) =   22214.2892791999984183
  coef(2881) =     -85.8605637655000038
  coef(2882) =   17156.1664659999987634
  coef(2883) =     415.5262102990000130
  coef(2884) =     196.0432392010000058
  coef(2885) =    1406.4854164500000024
  coef(2886) =     794.5268864339999482
  coef(2887) =   15711.6051112000004650
  coef(2888) =     761.4451598050000030
  coef(2889) =     237.7748382810000010
  coef(2890) =    -341.1674136930000145
  coef(2891) =     529.7511691639999754
  coef(2892) =      -0.3756589816910000
  coef(2893) =    -511.0801338210000040
  coef(2894) =    -551.6703758810000409
  coef(2895) =   -3282.9228496199998517
  coef(2896) =    1891.4285935000000336
  coef(2897) =     522.9703519519999873
  coef(2898) =     535.6724175560000276
  coef(2899) =   -2749.3376426700001502
  coef(2900) =    -298.5098662949999948
  coef(2901) =     765.8092484979999881
  coef(2902) =     -44.0259657340000032
  coef(2903) =     556.6197716279999668
  coef(2904) =   -1200.4076321600000483
  coef(2905) =     803.1668056280000201
  coef(2906) =    2484.5744157300000552
  coef(2907) =     847.1999618290000171
  coef(2908) =     101.9333113349999991
  coef(2909) =   -3513.5787018800001533
  coef(2910) =   -1016.7929103799999666
  coef(2911) =       9.6568852985699998
  coef(2912) =    -194.0847364329999891
  coef(2913) =     -75.2239779583999990
  coef(2914) =     323.4163013070000261
  coef(2915) =     133.0560013580000032
  coef(2916) =    -295.1780110959999774
  coef(2917) =    -105.9333854030000026
  coef(2918) =     541.9181235150000475
  coef(2919) =    -423.7686490300000060
  coef(2920) =    -615.0405183520000492
  coef(2921) =     704.2482953879999741
  coef(2922) =    -274.0214751259999844
  coef(2923) =   -1253.4011663399999179
  coef(2924) =     778.0869762419999915
  coef(2925) =    -804.5373024100000521
  coef(2926) =       0.0483808630200100
  coef(2927) =      -0.5531259369670000
  coef(2928) =      -3.8634508250000001
  coef(2929) =       4.5646964125900000
  coef(2930) =       0.8259522972700000
  coef(2931) =       0.2118063192700000
  coef(2932) =      -2.3155682883600002
  coef(2933) =       0.6101125980310000
  coef(2934) =      10.6101940322000008
  coef(2935) =      13.6987454330999991
  coef(2936) =       3.0365903052699998
  coef(2937) =      -1.1662744336699999
  coef(2938) =       0.2544910111390000
  coef(2939) =       9.1283705188000006
  coef(2940) =      53.1362389355999980
  coef(2941) =      42.8314268768999966
  coef(2942) =     -12.4412881186999993
  coef(2943) =       6.7691197173099997
  coef(2944) =      16.2938467825999993
  coef(2945) =       7.0288475240899997
  coef(2946) =      72.6999971396000007
  coef(2947) =      33.1035565301000005
  coef(2948) =      -1.6538589281200000
  coef(2949) =      85.2301472851000028
  coef(2950) =     -16.6997167486999984
  coef(2951) =     510.4366947799999821
  coef(2952) =     -87.3578836119999949
  coef(2953) =    -287.3746275500000138
  coef(2954) =     589.5451561390000279
  coef(2955) =    -624.9167371519999961
  coef(2956) =     671.1739307779999990
  coef(2957) =    -156.6630256569999915
  coef(2958) =    -123.8296803060000002
  coef(2959) =     142.9986653299999944
  coef(2960) =     707.5035978560000558
  coef(2961) =     177.3025338189999900
  coef(2962) =    1181.1767428600001040
  coef(2963) =    2147.5616289399999914
  coef(2964) =   -1120.9884163100000478
  coef(2965) =     641.3720313430000033
  coef(2966) =     -35.8105923803000010
  coef(2967) =    -264.2063708140000244
  coef(2968) =     -12.0351064640000001
  coef(2969) =    -202.5970216009999945
  coef(2970) =    -120.6160353679999986
  coef(2971) =      -4.3701531683699999
  coef(2972) =    -106.8216990260000046
  coef(2973) =     -58.3742612030999979
  coef(2974) =     -18.3421543904999993
  coef(2975) =     -13.3223033325000006
  coef(2976) =     -27.4180445755999997
  coef(2977) =     -13.0630382618999992
  coef(2978) =      71.8718303859999992
  coef(2979) =      78.3580479101999998
  coef(2980) =     -52.1173291559000020
  coef(2981) =      84.0335278403999979
  coef(2982) =      17.1522260304999996
  coef(2983) =     -99.0771691976000000
  coef(2984) =     -41.9735129784999970
  coef(2985) =     137.1023112469999887
  coef(2986) =      48.4816311232999979
  coef(2987) =      22.3541713052000013
  coef(2988) =      35.8204657082999987
  coef(2989) =     172.6992258809999896
  coef(2990) =     107.2991870670000054
  coef(2991) =    -246.6248153710000111
  coef(2992) =    -137.8796883169999887
  coef(2993) =     -16.2290897602000008
  coef(2994) =    -162.5887604829999873
  coef(2995) =     -56.0781313616000006
  coef(2996) =       8.2770544837000006
  coef(2997) =      13.1276290151000001
  coef(2998) =      -6.5859796357900002
  coef(2999) =      -1.6911287393000001
  coef(3000) =      -1.7816274495500000
  coef(3001) =      58.0192986656999992
  coef(3002) =    -115.6439403539999944
  coef(3003) =     -43.2051806548999977
  coef(3004) =      16.3762867402000012
  coef(3005) =     -22.9269474680999998
  coef(3006) =     -22.1829173986000008
  coef(3007) =       6.9434041817100001
  coef(3008) =     121.6674716520000032
  coef(3009) =      65.2418434333000050
  coef(3010) =      59.0909936592000022
  coef(3011) =     -15.8493423448000001
  coef(3012) =    -130.5118187859999921
  coef(3013) =     185.6050838669999905
  coef(3014) =      -9.7166913043900003
  coef(3015) =       7.6532427616299996
  coef(3016) =     -57.5510368602000000
  coef(3017) =     308.2338933589999783
  coef(3018) =     106.1697867289999948
  coef(3019) =     106.8408503349999989
  coef(3020) =    -282.2100447020000047
  coef(3021) =      31.0109899897999988
  coef(3022) =    -245.0822225539999977
  coef(3023) =      35.2524858474000027
  coef(3024) =     290.2005356569999890
  coef(3025) =      47.7000731178000024
  coef(3026) =      55.5329725041000017
  coef(3027) =     618.7132604600000150
  coef(3028) =    1370.4090293700000984
  coef(3029) =     103.6194723960000061
  coef(3030) =     -21.3485701229999982
  coef(3031) =    -120.2734459680000043
  coef(3032) =     813.2922290919999568
  coef(3033) =    -201.0659948489999920
  coef(3034) =    -276.4974971620000019
  coef(3035) =    -489.8403953549999983
  coef(3036) =      90.8184012473000024
  coef(3037) =    -684.1695586560000493
  coef(3038) =     254.4373566840000080
  coef(3039) =    -131.5729753040000105
  coef(3040) =    -217.8997841810000011
  coef(3041) =       8.8222933694499996
  coef(3042) =     882.9124219789999870
  coef(3043) =     393.0575475649999930
  coef(3044) =     173.9550539859999958
  coef(3045) =      -7.4966173533599996
  coef(3046) =      62.9674199092999984
  coef(3047) =     768.0537766520000105
  coef(3048) =     -42.2096896468000011
  coef(3049) =     -88.8471483330000069
  coef(3050) =     808.4207159830000364
  coef(3051) =     374.3603413650000107
  coef(3052) =    1250.8227825100000246
  coef(3053) =    -139.1069904569999949
  coef(3054) =     301.4462757310000143
  coef(3055) =      -7.2924358219399998
  coef(3056) =     -65.4377568701000030
  coef(3057) =    -904.7816679390000445
  coef(3058) =    -419.7555459929999984
  coef(3059) =     -96.1221549339000063
  coef(3060) =     -17.6840531271999986
  coef(3061) =      -2.7122431546799999
  coef(3062) =    -704.9577442510000083
  coef(3063) =   -2950.0063893699998516
  coef(3064) =     329.6502905250000026
  coef(3065) =    -964.0597579179999457
  coef(3066) =    -521.7931619160000309
  coef(3067) =      68.3724507643999999
  coef(3068) =    1251.8112966599999254
  coef(3069) =    -403.3165597170000183
  coef(3070) =    -400.1699427190000051
  coef(3071) =       8.3672498070499994
  coef(3072) =     132.2352588159999982
  coef(3073) =     218.9310643130000074
  coef(3074) =       7.2808012720499997
  coef(3075) =     -10.1603689638999999
  coef(3076) =      57.8899370660000017
  coef(3077) =    -633.7081776879999779
  coef(3078) =   -1323.0886360500001047
  coef(3079) =    -954.3285317649999797
  coef(3080) =     360.7698989209999922
  coef(3081) =    -658.7261745319999591
  coef(3082) =    -648.5217556689999583
  coef(3083) =   -1317.7532623600000079
  coef(3084) =   -1286.7063578499999039
  coef(3085) =    -339.3362542990000179
  coef(3086) =     279.4243363130000262
  coef(3087) =    -689.7439781809999886
  coef(3088) =    -547.7018307350000441
  coef(3089) =       4.2114706110300002
  coef(3090) =      -0.1790947224880000
  coef(3091) =    -220.7654152599999975
  coef(3092) =   -1031.5804507299999386
  coef(3093) =    -241.3190945510000063
  coef(3094) =   -1369.3982463299998926
  coef(3095) =     931.1764766049999480
  coef(3096) =     -48.7146088117999980
  coef(3097) =     506.2603934179999783
  coef(3098) =   -3363.1567305100002159
  coef(3099) =     664.5700534689999586
  coef(3100) =    -504.2989303869999844
  coef(3101) =     880.4846837469999628
  coef(3102) =    -361.0903899560000241
  coef(3103) =    -223.4599368830000117
  coef(3104) =       3.6369649622500000
  coef(3105) =       9.6486467415499995
  coef(3106) =      -8.4683794386399995
  coef(3107) =      -6.8096884732999996
  coef(3108) =      10.0027082693999994
  coef(3109) =       5.4348306487900002
  coef(3110) =      -3.7183533924000001
  coef(3111) =      -8.8755776339200008
  coef(3112) =     -26.2410301490999984
  coef(3113) =    -196.9916807559999938
  coef(3114) =     -16.7808458356999992
  coef(3115) =     -80.9232478931999992
  coef(3116) =     558.4207708579999689
  coef(3117) =     -29.6497873755999990
  coef(3118) =     -49.1804978242000033
  coef(3119) =     546.4269853150000245
  coef(3120) =     -91.4657731753000007
  coef(3121) =     147.7208337939999865
  coef(3122) =     -46.5952965325000008
  coef(3123) =    -231.9803049649999878
  coef(3124) =     -91.3613205026999964
  coef(3125) =      65.0185948313000068
  coef(3126) =     256.7943259169999806
  coef(3127) =      95.7010198193000008
  coef(3128) =     130.2842568339999900
  coef(3129) =      43.3215899356999969
  coef(3130) =     -43.3299543753999998
  coef(3131) =       5.8706138877400003
  coef(3132) =      68.3226578752999956
  coef(3133) =       4.5437841496600004
  coef(3134) =    -134.9743403559999990
  coef(3135) =    -142.2478623240000104
  coef(3136) =     -92.5472947793000031
  coef(3137) =     -59.4579443773999969
  coef(3138) =     -30.9409204813999992
  coef(3139) =      15.3212996042000000
  coef(3140) =     -70.6335702365000060
  coef(3141) =    -181.6796696799999893
  coef(3142) =      38.0546564687000028
  coef(3143) =    -100.2327135000000027
  coef(3144) =     -87.4950691802000051
  coef(3145) =      42.6595667403000007
  coef(3146) =     914.1902752879999525
  coef(3147) =     177.2217281839999998
  coef(3148) =     172.4207122409999897
  coef(3149) =      36.0052382929000032
  coef(3150) =     -10.3172480797000006
  coef(3151) =      25.9518729700000002
  coef(3152) =    -168.3144644410000126
  coef(3153) =      80.8286301182999978
  coef(3154) =      17.7900036202999985
  coef(3155) =      63.5044671803999989
  coef(3156) =     -21.5754977938999986
  coef(3157) =      -8.7314157748299994
  coef(3158) =      90.8847074523999936
  coef(3159) =     -35.1669079844999999
  coef(3160) =     -11.4103966214000003
  coef(3161) =     -14.3295381988999999
  coef(3162) =     -43.3846976003000009
  coef(3163) =      -9.6667134205499998
  coef(3164) =     -23.2094009947999993
  coef(3165) =     -22.2077015861000007
  coef(3166) =      10.6015059171000008
  coef(3167) =     -38.9831148132999985
  coef(3168) =    -105.2987791829999935
  coef(3169) =       3.5331538512599998
  coef(3170) =     -21.1808124780999982
  coef(3171) =     -51.7206652067999997
  coef(3172) =    -327.6790138019999858
  coef(3173) =     230.4317964340000060
  coef(3174) =     -20.7219850116000011
  coef(3175) =     -29.0722824901999992
  coef(3176) =    -183.0111556850000056
  coef(3177) =     104.7506036829999942
  coef(3178) =    -117.8389919910000003
  coef(3179) =     -32.1407456644000007
  coef(3180) =      73.7240013545000039
  coef(3181) =     -47.4236028827999974
  coef(3182) =      85.1126446293999948
  coef(3183) =      -2.2362543519200000
  coef(3184) =      65.0139277186000015
  coef(3185) =       1.0947151826599999
  coef(3186) =    -140.8920193689999962
  coef(3187) =     -36.1222361463999988
  coef(3188) =      40.4729903590000006
  coef(3189) =     -29.2320432313000005
  coef(3190) =    -313.5788972779999995
  coef(3191) =    -408.0616694409999923
  coef(3192) =     -65.9071536663000046
  coef(3193) =     -23.7463240366999990
  coef(3194) =      46.5157529705999977
  coef(3195) =    -180.4327398210000126
  coef(3196) =    -171.7568717960000129
  coef(3197) =      17.6991127842999987
  coef(3198) =     -59.7173027660000031
  coef(3199) =     -34.8428775711999990
  coef(3200) =      49.8033983995999989
  coef(3201) =     -70.5846964449000041
  coef(3202) =     -24.3489871182999984
  coef(3203) =      59.8320111740000016
  coef(3204) =      61.8995214679000014
  coef(3205) =   36252.7304547000021557
  coef(3206) =    6817.2401934400004393
  coef(3207) =    -161.9120631860000117
  coef(3208) =    7436.7010130999997273
  coef(3209) =    1778.7277266999999483
  coef(3210) =  -17810.7014415000012377
  coef(3211) =   -6458.9699295199998232
  coef(3212) =   -5430.4505152100000487
  coef(3213) =   56521.4396017999970354
  coef(3214) =      14.9903321940000005
  coef(3215) =     907.8471449119999761
  coef(3216) =   -4177.5766541100001632
  coef(3217) =   -2549.6645313200001510
  coef(3218) =   -1568.4746310499999709
  coef(3219) =    -751.4511806900000011
  coef(3220) =   -1912.9592498899999100
  coef(3221) =    -203.5418258999999921
  coef(3222) =    4016.4281583000001774
  coef(3223) =    6525.0264419200002521
  coef(3224) =    -127.0438015919999941
  coef(3225) =    -807.6009356170000046
  coef(3226) =    1559.4506867200000215
  coef(3227) =   -7283.5347131500002433
  coef(3228) =  -16025.6589179000002332
  coef(3229) =   -6600.4727872599996772
  coef(3230) =    3161.2802157900000566
  coef(3231) =   -6775.7592919999997321
  coef(3232) =    5349.7174114199997348
  coef(3233) =  -21730.7740838999998232
  coef(3234) =    -434.7451620169999842
  coef(3235) =     -96.6600076433999931
  coef(3236) =      -2.0676211555599999
  coef(3237) =    -197.8545280730000115
  coef(3238) =     -78.3725858474000034
  coef(3239) =      86.8177625934000048
  coef(3240) =     -41.9311224118999988
  coef(3241) =      -1.6349199009199999
  coef(3242) =    -124.5753995099999969
  coef(3243) =     116.7335798470000015
  coef(3244) =     196.0622843540000133
  coef(3245) =     716.3477261460000136
  coef(3246) =     310.4194829479999953
  coef(3247) =      17.8384654820000002
  coef(3248) =     169.8520433649999859
  coef(3249) =     195.7774872159999973
  coef(3250) =       0.7603320980950000
  coef(3251) =     472.9284045769999807
  coef(3252) =     771.5978934089999939
  coef(3253) =    1342.3643590199999380
  coef(3254) =     987.3099730889999819
  coef(3255) =    2192.8949035000000549
  coef(3256) =     447.1142929019999883
  coef(3257) =     150.2642516939999950
  coef(3258) =     795.0136052509999445
  coef(3259) =    1612.6797109900001033
  coef(3260) =     528.6835982119999926
  coef(3261) =   -1629.2249682100000427
  coef(3262) =    -580.3917058109999516
  coef(3263) =    3138.6890620799999851
  coef(3264) =   -2888.2539658000000600
  coef(3265) =   -4183.5910744900002101
  coef(3266) =    9177.1129176100002951
  coef(3267) =    8468.0634964700002456
  coef(3268) =   -9727.3368844500000705
  coef(3269) =    3663.5407276699997965
  coef(3270) =    4790.9212542699997357
  coef(3271) =    -305.8920402630000126
  coef(3272) =    -432.2927712329999963
  coef(3273) =   -2305.8951537299999472
  coef(3274) =    -690.0794338680000237
  coef(3275) =  -27585.3468255000007048
  coef(3276) =     518.9385281500000247
  coef(3277) =  -22009.9410592999993241
  coef(3278) =    -528.2235471779999898
  coef(3279) =     108.8761120930000033
  coef(3280) =   -1799.9224537800000689
  coef(3281) =    -358.0730491169999823
  coef(3282) =   -7204.3963614399999642
  coef(3283) =    -788.2392784170000368
  coef(3284) =    -209.1871368120000056
  coef(3285) =     559.1137055909999845
  coef(3286) =    -630.3994507129999647
  coef(3287) =      22.0085116276999990
  coef(3288) =    -366.8464313239999797
  coef(3289) =     377.4989384990000190
  coef(3290) =    5479.8880373199999667
  coef(3291) =    -743.8236150330000100
  coef(3292) =    -194.4707539409999981
  coef(3293) =      59.3360882444000026
  coef(3294) =    3204.1067453099999511
  coef(3295) =   -1221.4778001399999994
  coef(3296) =     365.8938546059999908
  coef(3297) =    1549.8644506499999807
  coef(3298) =      91.9487221196000064
  coef(3299) =    -189.3039002049999908
  coef(3300) =   -1395.8951381800000036
  coef(3301) =   -1675.0244397799999660
  coef(3302) =      92.3687401324000064
  coef(3303) =      95.1507901603000050
  coef(3304) =    3791.3316947100001926
  coef(3305) =    1247.8659082100000433
  coef(3306) =    -497.3689377189999732
  coef(3307) =     -86.0683938463999993
  coef(3308) =     720.9501591380000036
  coef(3309) =    -637.5770830249999790
  coef(3310) =     -13.0901854575000005
  coef(3311) =     925.1381690870000511
  coef(3312) =    1513.7219024500000160
  coef(3313) =    -399.4184515860000033
  coef(3314) =     460.8431487990000051
  coef(3315) =     -16.4188093950000003
  coef(3316) =    -646.5966532999999572
  coef(3317) =     368.7649035310000158
  coef(3318) =    2381.1754666199999519
  coef(3319) =    -736.3238933229999930
  coef(3320) =     777.4111619010000140
  coef(3321) =       0.4201189372640000
  coef(3322) =       0.8793657309710000
  coef(3323) =      -3.5455461774399999
  coef(3324) =       2.4242081781999998
  coef(3325) =       6.3383841605500004
  coef(3326) =      -6.1469077973999999
  coef(3327) =      39.1141615085999987
  coef(3328) =       0.7026941595710000
  coef(3329) =      -4.5651868669800004
  coef(3330) =     -38.4845525244000015
  coef(3331) =       0.3013668845320000
  coef(3332) =       3.3405901063000001
  coef(3333) =       4.4138700283999999
  coef(3334) =     200.0608083639999961
  coef(3335) =      32.6420013876999988
  coef(3336) =     -20.4750883819000009
  coef(3337) =      13.0835252757999996
  coef(3338) =     155.8982901560000016
  coef(3339) =      26.9090767088000007
  coef(3340) =     -46.4588725272000005
  coef(3341) =      28.6111871389999983
  coef(3342) =      -2.9354344892599999
  coef(3343) =      -0.7635869775800000
  coef(3344) =    -357.2527902699999913
  coef(3345) =     -48.2875506152999989
  coef(3346) =     138.8942647780000073
  coef(3347) =    -134.2041860859999929
  coef(3348) =     656.5185459279999804
  coef(3349) =    -281.6782170820000033
  coef(3350) =    1318.7378967100000864
  coef(3351) =     183.8879321649999952
  coef(3352) =     305.7850724470000046
  coef(3353) =      14.6621317009999999
  coef(3354) =     321.3450298540000176
  coef(3355) =     817.4043137700000443
  coef(3356) =     -26.0352923234000002
  coef(3357) =     526.9889619040000071
  coef(3358) =    -143.3184068189999891
  coef(3359) =     -11.2209432588000002
  coef(3360) =    2765.4361351100001229
  coef(3361) =      92.2740192291999932
  coef(3362) =      31.1811108365999985
  coef(3363) =      37.6659131608999971
  coef(3364) =     292.6882848390000049
  coef(3365) =    -129.8538669900000002
  coef(3366) =     -36.2649815054000015
  coef(3367) =     643.6433310429999892
  coef(3368) =     149.7858385670000132
  coef(3369) =      44.0181355672999999
  coef(3370) =      25.7750894614000003
  coef(3371) =      54.8478431050999973
  coef(3372) =      15.5569058533000000
  coef(3373) =     -94.5364640294999958
  coef(3374) =     -86.3807301056999961
  coef(3375) =     -94.7171360600000014
  coef(3376) =    -157.6430569169999956
  coef(3377) =     -61.9681966248000009
  coef(3378) =       2.4841006499699998
  coef(3379) =      47.4949140741999969
  coef(3380) =    -198.1744421349999925
  coef(3381) =     -49.0035454661000003
  coef(3382) =     -37.6302354199000035
  coef(3383) =     -19.1398327787000007
  coef(3384) =    -321.3018118090000144
  coef(3385) =    -199.1664199260000032
  coef(3386) =     -66.9225824779999954
  coef(3387) =     -53.0611787794000023
  coef(3388) =     -55.1925623743000031
  coef(3389) =     186.7608229100000017
  coef(3390) =      15.1527067468999999
  coef(3391) =      20.4702045951000002
  coef(3392) =     -22.5149368332999984
  coef(3393) =      -6.1491053830000002
  coef(3394) =     -30.4934259107999992
  coef(3395) =     -35.0819025493999987
  coef(3396) =     -43.4395017698000032
  coef(3397) =    -253.9375101720000032
  coef(3398) =     -29.1519775663999994
  coef(3399) =      26.2919770972999984
  coef(3400) =     -27.7905381499000015
  coef(3401) =    -108.0929547780000064
  coef(3402) =     -51.3126972795000000
  coef(3403) =    -161.9791930469999954
  coef(3404) =     -60.3341834355000017
  coef(3405) =      45.8515429121000011
  coef(3406) =     127.2580994940000068
  coef(3407) =     605.4971008940000274
  coef(3408) =      46.4479598039999999
  coef(3409) =     156.7019752740000058
  coef(3410) =     -42.3561210076999970
  coef(3411) =     209.4429887580000127
  coef(3412) =    -382.4755897989999767
  coef(3413) =    -254.4406924090000075
  coef(3414) =     358.4521396879999884
  coef(3415) =     795.4338481329999695
  coef(3416) =     275.1507040289999964
  coef(3417) =     724.8804317599999649
  coef(3418) =     942.4799030850000463
  coef(3419) =     -83.4212647561000011
  coef(3420) =     -90.9296186583000008
  coef(3421) =     400.8010706480000067
  coef(3422) =    -106.4708371680000027
  coef(3423) =   -1538.1233348499999920
  coef(3424) =     145.3869629589999874
  coef(3425) =      49.8134041141000026
  coef(3426) =   -1321.7564762899999096
  coef(3427) =    -334.1340475609999885
  coef(3428) =    2163.0042899000000034
  coef(3429) =    2130.8632723900000201
  coef(3430) =   -2656.7765857399999732
  coef(3431) =     140.6082154249999974
  coef(3432) =    1279.6558603700000276
  coef(3433) =    -172.5709623850000014
  coef(3434) =    2242.5645515100000011
  coef(3435) =     534.5577260249999654
  coef(3436) =     189.2334413629999972
  coef(3437) =   -1924.4319798399999399
  coef(3438) =   -1474.9143475299999864
  coef(3439) =     634.0166401259999702
  coef(3440) =     -36.8639155011000028
  coef(3441) =    -251.5787462829999868
  coef(3442) =    1368.9230751299999156
  coef(3443) =    1842.2140133299999434
  coef(3444) =     361.4368150089999858
  coef(3445) =   -1082.6497326399999110
  coef(3446) =    2860.2931118000001334
  coef(3447) =     248.6344514040000035
  coef(3448) =    2819.1407917500000622
  coef(3449) =    -199.5555236140000090
  coef(3450) =    -617.1252965080000195
  coef(3451) =     483.4531205389999968
  coef(3452) =     605.0948294150000493
  coef(3453) =    -505.0435727160000283
  coef(3454) =     874.7801977120000174
  coef(3455) =    -349.1781714960000045
  coef(3456) =     370.9539263230000188
  coef(3457) =   -1447.1687652299999627
  coef(3458) =    1931.8794202199999290
  coef(3459) =     688.3670805530000507
  coef(3460) =    1116.3753550100000211
  coef(3461) =   -3999.8228193599998121
  coef(3462) =   -2004.0073139900000569
  coef(3463) =    6669.1012516199998572
  coef(3464) =    1795.5860264299999471
  coef(3465) =     812.4200840739999876
  coef(3466) =    -290.6097258510000074
  coef(3467) =      74.0420363533999932
  coef(3468) =     216.9634714640000084
  coef(3469) =   -1139.8337768900000810
  coef(3470) =     -81.5807609319000022
  coef(3471) =     -50.0998436922999986
  coef(3472) =   -2085.4333789500001330
  coef(3473) =    1838.3545603800000663
  coef(3474) =    1339.7598883200000728
  coef(3475) =    -333.6425739879999810
  coef(3476) =   -4260.3316937700001290
  coef(3477) =   -1777.1686405499999637
  coef(3478) =     121.0586979440000022
  coef(3479) =    1846.2524873100001059
  coef(3480) =     718.5173061049999887
  coef(3481) =    -513.1224381849999645
  coef(3482) =    1199.5659194899999420
  coef(3483) =     112.6528003960000035
  coef(3484) =     -67.6325283527000067
  coef(3485) =    -142.7901366689999918
  coef(3486) =     432.4648972049999998
  coef(3487) =    2236.5851746700000149
  coef(3488) =     260.4843800439999768
  coef(3489) =    2903.3545588000001771
  coef(3490) =   -3268.9875917700001082
  coef(3491) =   -3473.8137032899999213
  coef(3492) =   -2675.8203644700001860
  coef(3493) =    5186.9769633699997939
  coef(3494) =   -3727.2296038399999816
  coef(3495) =    4115.7244198500002312
  coef(3496) =   -5661.5669504300003609
  coef(3497) =    1284.5538626200000181
  coef(3498) =    -233.1143754590000015
  coef(3499) =    -147.9691283680000140
  coef(3500) =     -93.6846316026000068
  coef(3501) =      49.7200303748999985
  coef(3502) =      48.5427565761999986
  coef(3503) =    -101.6517762839999932
  coef(3504) =     -60.9370252574000020
  coef(3505) =       2.8977280523900002
  coef(3506) =       7.7530998392699999
  coef(3507) =      24.4135587584999989
  coef(3508) =     317.5106599719999849
  coef(3509) =      32.3902696425000016
  coef(3510) =     281.6349297880000222
  coef(3511) =   -1073.0758394800000133
  coef(3512) =    -153.8434967960000108
  coef(3513) =     311.6059637280000061
  coef(3514) =    -815.6379904760000272
  coef(3515) =     101.5408932569999934
  coef(3516) =    -234.0053400580000016
  coef(3517) =      36.4130146436000004
  coef(3518) =     285.1082208949999881
  coef(3519) =     110.4524135820000055
  coef(3520) =    -187.8276736429999971
  coef(3521) =    -373.7435427749999803
  coef(3522) =    -147.5231293880000010
  coef(3523) =    -182.5198042680000015
  coef(3524) =     -51.7453844675999974
  coef(3525) =     241.1792015110000023
  coef(3526) =    -203.5886282179999967
  coef(3527) =    -214.9667072469999880
  coef(3528) =       2.6407176962299999
  coef(3529) =     921.9607355959999495
  coef(3530) =     407.5341617700000256
  coef(3531) =    -641.7173770970000533
  coef(3532) =    -506.9311142049999717
  coef(3533) =    -260.9814302060000273
  coef(3534) =    -200.3983502709999982
  coef(3535) =     -29.2149828786000008
  coef(3536) =    1148.0567126799999187
  coef(3537) =     177.8100467069999979
  coef(3538) =     552.2993357289999494
  coef(3539) =     132.7265917439999896
  coef(3540) =     -62.3918749352000006
  coef(3541) =   -1081.9953200200000083
  coef(3542) =    -206.7712378319999971
  coef(3543) =    -232.4447872490000009
  coef(3544) =     -17.9571653787000010
  coef(3545) =      11.6025500939999997
  coef(3546) =     184.2260664890000044
  coef(3547) =     196.6813105189999931
  coef(3548) =    -111.5846569060000064
  coef(3549) =      17.2634315206999993
  coef(3550) =     -97.9181962229999954
  coef(3551) =      -1.8514472364400001
  coef(3552) =      84.9135021688999956
  coef(3553) =    -460.3053827470000101
  coef(3554) =    -201.4356341989999919
  coef(3555) =    -327.2268170990000158
  coef(3556) =    -149.5772540529999901
  coef(3557) =     -41.3440092207000021
  coef(3558) =     -55.6949826772999970
  coef(3559) =     -10.5592911096000002
  coef(3560) =      28.6255655934999993
  coef(3561) =      84.1127408672000030
  coef(3562) =     573.2799420809999447
  coef(3563) =      97.1039516661000022
  coef(3564) =      84.1028684757999940
  coef(3565) =      43.9317946370999977
  coef(3566) =     -21.8332448982000003
  coef(3567) =     -60.7018095836999976
  coef(3568) =    -940.0716540710000118
  coef(3569) =      70.4363258075999994
  coef(3570) =      74.5136331652000052
  coef(3571) =      68.1610324213999945
  coef(3572) =    -245.9134786779999899
  coef(3573) =     214.7508410799999865
  coef(3574) =      -0.3447399293370000
  coef(3575) =    -175.6542679529999873
  coef(3576) =      32.0930884969999966
  coef(3577) =    -207.7054520110000055
  coef(3578) =    -126.2742845869999968
  coef(3579) =     -27.1221810352999988
  coef(3580) =     -14.0284197625000004
  coef(3581) =     104.0268953399999958
  coef(3582) =    -269.2095350690000259
  coef(3583) =    -233.9111530110000103
  coef(3584) =     -35.7108456575999966
  coef(3585) =    -166.3910750089999908
  coef(3586) =     364.2095152900000130
  coef(3587) =     170.5922534419999863
  coef(3588) =      77.2135152489999967
  coef(3589) =    -137.1024725899999908
  coef(3590) =     146.9394661800000108
  coef(3591) =      40.9914617427999985
  coef(3592) =    -473.4948250219999863
  coef(3593) =      40.5991936176000010
  coef(3594) =       4.7133462649700002
  coef(3595) =     -73.2512624671000054
  coef(3596) =     155.6375302270000134
  coef(3597) =      98.1331921878000060
  coef(3598) =    -442.7510629289999997
  coef(3599) =    -130.8017241260000105
  coef(3600) =      -0.0118519738986800
  coef(3601) =       0.0021944370449120
  coef(3602) =      -0.4917407582460000
  coef(3603) =      -0.3461498733860000
  coef(3604) =      -0.0524506093090400
  coef(3605) =       0.2811608354020000
  coef(3606) =       0.6059713633479999
  coef(3607) =       0.2249694036600000
  coef(3608) =       0.3975273211590000
  coef(3609) =      -0.4214085465710000
  coef(3610) =       1.3639511966800000
  coef(3611) =       0.1470848182020000
  coef(3612) =       0.2358215106700000
  coef(3613) =       0.3234622516080000
  coef(3614) =      -4.4318464815300000
  coef(3615) =      -1.5746621582500000
  coef(3616) =      -0.4811704643370000
  coef(3617) =       0.4837505105330000
  coef(3618) =      -0.7121742697860000
  coef(3619) =      -0.7771148190300000
  coef(3620) =      -3.6295811524000001
  coef(3621) =      -2.3293152752799999
  coef(3622) =       0.2873910482570000
  coef(3623) =      -0.7476222784560000
  coef(3624) =      36.4294805708999974
  coef(3625) =      -4.1234704654899996
  coef(3626) =       0.9021241675550000
  coef(3627) =       6.2560752171900003
  coef(3628) =       5.8298806852300000
  coef(3629) =      37.1911631918000012
  coef(3630) =      34.6785121795999984
  coef(3631) =     -23.1110232117999992
  coef(3632) =      29.4027582277999997
  coef(3633) =     -19.0968151305999996
  coef(3634) =     -10.0742179620000005
  coef(3635) =      11.0003062541999999
  coef(3636) =      10.6341521709000002
  coef(3637) =     -16.0500765124999987
  coef(3638) =      22.1294014527999998
  coef(3639) =     -33.0885865009000000
  coef(3640) =      -5.0345529214900004
  coef(3641) =       3.0667420989899998
  coef(3642) =      -0.0620144217166700
  coef(3643) =      -5.6929314840299998
  coef(3644) =      12.3986381847999994
  coef(3645) =      -1.0619536016300000
  coef(3646) =     120.3521753609999934
  coef(3647) =      -0.2407935068440000
  coef(3648) =      -0.2496976907780000
  coef(3649) =       0.8563278104180000
  coef(3650) =      -0.6052988760280000
  coef(3651) =      -0.5405358297530000
  coef(3652) =      -3.0443964547300002
  coef(3653) =     -19.2606473232000006
  coef(3654) =      -3.7340241563199998
  coef(3655) =      -2.4521926576299999
  coef(3656) =      -0.5453846466520000
  coef(3657) =     -15.2291795923999995
  coef(3658) =       5.5204556191199998
  coef(3659) =      -6.3539022138499996
  coef(3660) =      -4.8643388954200004
  coef(3661) =      -4.2160406896299998
  coef(3662) =      -3.9985248082900000
  coef(3663) =     -18.2790855870999991
  coef(3664) =       2.2023718457400001
  coef(3665) =     -35.8370705887000014
  coef(3666) =      -0.5974234969180000
  coef(3667) =       1.0539358295700001
  coef(3668) =       9.3295828067700004
  coef(3669) =       4.0578222754600004
  coef(3670) =       0.7661747712280000
  coef(3671) =       0.5559249646610001
  coef(3672) =       0.1967520232460000
  coef(3673) =       0.0125372228747300
  coef(3674) =       1.7550209702199999
  coef(3675) =     -23.6802014324000005
  coef(3676) =      -8.0995847497100009
  coef(3677) =      -0.0691844949142800
  coef(3678) =     -13.2164769368999995
  coef(3679) =       4.1440013114100003
  coef(3680) =      -0.0676314285778700
  coef(3681) =       0.8989576499950001
  coef(3682) =     -27.6694834635000007
  coef(3683) =      -3.4039312734900000
  coef(3684) =     -26.7852080066999996
  coef(3685) =       2.5428216017100000
  coef(3686) =     -17.9994513992999998
  coef(3687) =       6.4474586624299999
  coef(3688) =       8.1334778172100002
  coef(3689) =      -1.3880013122599999
  coef(3690) =       8.7225539280600000
  coef(3691) =      30.4205289709000013
  coef(3692) =     -25.6493801623000017
  coef(3693) =    -108.7939608089999979
  coef(3694) =      -8.9260558921299999
  coef(3695) =      74.8217324998000066
  coef(3696) =      28.9572297931000016
  coef(3697) =    -176.2918385449999903
  coef(3698) =     -18.6050181119000015
  coef(3699) =      -0.4316071509120000
  coef(3700) =     -22.0282700572999985
  coef(3701) =    -135.9349043770000094
  coef(3702) =     -65.6290476176999960
  coef(3703) =     -77.5524014451000028
  coef(3704) =      -2.4236701442499999
  coef(3705) =     -21.8720208754000005
  coef(3706) =       7.9407539620399996
  coef(3707) =     143.1993210960000056
  coef(3708) =    -126.3706092030000008
  coef(3709) =     -44.9687426142999982
  coef(3710) =     -91.2215030951000045
  coef(3711) =     -94.5104851695000008
  coef(3712) =     164.2062861800000064
  coef(3713) =      77.0340095034000001
  coef(3714) =      10.3248840843000007
  coef(3715) =      26.9988767615000000
  coef(3716) =     -10.2696371827000004
  coef(3717) =     -26.4669227354999990
  coef(3718) =     130.1003739200000098
  coef(3719) =      17.7055397059000015
  coef(3720) =      -7.0116295798200001
  coef(3721) =    -222.6177446640000142
  coef(3722) =    -292.7371945109999842
  coef(3723) =     -77.9557063248000048
  coef(3724) =    -137.5485045469999932
  coef(3725) =    -390.4100565879999749
  coef(3726) =    -193.7891753600000015
  coef(3727) =    -424.2696197820000066
  coef(3728) =     -86.3107188387999997
  coef(3729) =      26.8686065324999994
  coef(3730) =    -117.9596783089999974
  coef(3731) =    -163.7394953419999979
  coef(3732) =    -111.0688033090000033
  coef(3733) =    -333.6080941860000166
  coef(3734) =      19.3249600329000017
  coef(3735) =       4.6744040278199996
  coef(3736) =    -124.5842849250000057
  coef(3737) =       4.6464127348900002
  coef(3738) =    -502.6600052350000283
  coef(3739) =     -19.9660325516000015
  coef(3740) =    -208.4632064280000066
  coef(3741) =    -149.8700076700000068
  coef(3742) =   -2202.9494938699999693
  coef(3743) =    -445.8771436559999870
  coef(3744) =    -142.5578562569999974
  coef(3745) =      22.1081300190000007
  coef(3746) =      17.5580472836000006
  coef(3747) =     -18.4349743789000016
  coef(3748) =      70.1717909733000056
  coef(3749) =       2.9537668402100001
  coef(3750) =      -7.8638787099299998
  coef(3751) =      60.2582868793000017
  coef(3752) =     112.9169277700000009
  coef(3753) =      98.9253976779999959
  coef(3754) =      26.4338743666000013
  coef(3755) =      48.3320748770999984
  coef(3756) =      -4.2848186236799997
  coef(3757) =     499.7562924029999749
  coef(3758) =     -85.4562517528000001
  coef(3759) =     -33.5156437216000000
  coef(3760) =      44.8603001616000014
  coef(3761) =      89.4121182124000029
  coef(3762) =     -87.9663813667000056
  coef(3763) =       0.7781307594600000
  coef(3764) =      49.8388323710999970
  coef(3765) =       0.3932637179510000
  coef(3766) =      59.7155922211999979
  coef(3767) =      14.6269346741999993
  coef(3768) =    -116.3979206159999933
  coef(3769) =     -64.6359478905000060
  coef(3770) =     -71.0326540557999948
  coef(3771) =     -72.6517425953999947
  coef(3772) =     157.7800668780000137
  coef(3773) =     -60.5712530841000003
  coef(3774) =     -58.1592039972999970
  coef(3775) =     -99.0613893737999973
  coef(3776) =     -65.9242695237000049
  coef(3777) =      -1.0885996979599999
  coef(3778) =       6.3631213017899997
  coef(3779) =       4.0146846546199999
  coef(3780) =       2.1107116444999998
  coef(3781) =       0.5941137583580000
  coef(3782) =       4.8936535294099999
  coef(3783) =       1.8549322405699999
  coef(3784) =       1.6986088975500000
  coef(3785) =      -2.0205843861799999
  coef(3786) =       4.0836419226400000
  coef(3787) =     -14.6382922099999995
  coef(3788) =      -8.6438480675200005
  coef(3789) =       6.9418786560900001
  coef(3790) =      28.2795475912999983
  coef(3791) =      32.3629614863000015
  coef(3792) =       8.6726171859899992
  coef(3793) =     -19.6406868460999995
  coef(3794) =      15.5879884145999998
  coef(3795) =      -6.9822456102399997
  coef(3796) =      10.5996282311000005
  coef(3797) =     -37.4697704605000013
  coef(3798) =     -11.9098960944000005
  coef(3799) =     -13.5898156214999997
  coef(3800) =     -11.3319523815000007
  coef(3801) =      -7.2180653300199999
  coef(3802) =      -4.5580990485099999
  coef(3803) =      -7.4642488849399999
  coef(3804) =     -39.0013786558999982
  coef(3805) =     -25.7845351801000007
  coef(3806) =     -18.6916308571999998
  coef(3807) =     -16.7869217605000003
  coef(3808) =      25.1736680672000013
  coef(3809) =      18.0616060133000005
  coef(3810) =     100.8712729429999939
  coef(3811) =      87.2717906121999931
  coef(3812) =      43.1342762829999984
  coef(3813) =      18.9116558268000006
  coef(3814) =      18.1293852459000000
  coef(3815) =      40.2832834012999967
  coef(3816) =      22.3646336376000008
  coef(3817) =      19.9292792733999988
  coef(3818) =      28.7024149074000015
  coef(3819) =       4.7979959758000001
  coef(3820) =     -78.2831776654000038
  coef(3821) =      -6.1022933798599999
  coef(3822) =     -17.1480298602999994
  coef(3823) =      -0.8426222479050000
  coef(3824) =       2.3005774498400000
  coef(3825) =     -32.2441686109999992
  coef(3826) =     -19.0761129226000001
  coef(3827) =       3.6494973903700001
  coef(3828) =      -0.6442585911649999
  coef(3829) =      -3.3284781268299999
  coef(3830) =       2.1663772118100000
  coef(3831) =     -12.6309745755999998
  coef(3832) =     -24.1305122770000011
  coef(3833) =      33.5401608742999997
  coef(3834) =      37.5563771938000031
  coef(3835) =      21.8731665675000002
  coef(3836) =       9.4842691547199998
  coef(3837) =       4.6606468641400003
  coef(3838) =      12.7165258061999999
  coef(3839) =       5.1488792139499999
  coef(3840) =      -7.6072606192599999
  coef(3841) =      29.6729640172000018
  coef(3842) =      25.4008442230999982
  coef(3843) =       6.2116760245400000
  coef(3844) =      15.7240454682999999
  coef(3845) =      -5.6568028590999999
  coef(3846) =      18.7829708894999996
  coef(3847) =     121.9046473890000044
  coef(3848) =       4.7094327394700004
  coef(3849) =      -1.9420552213700000
  coef(3850) =      11.2935147645999994
  coef(3851) =      25.1968596072000004
  coef(3852) =      25.4379004946999991
  coef(3853) =      -0.5435443733699999
  coef(3854) =      23.4732495531999987
  coef(3855) =      12.4283416581000008
  coef(3856) =      19.1449536834000007
  coef(3857) =      28.0731756604999987
  coef(3858) =      -9.6821691836100001
  coef(3859) =       2.4781985274700000
  coef(3860) =     -21.4920079995000002
  coef(3861) =     -14.3414510996000004
  coef(3862) =      -6.6981867844699998
  coef(3863) =     -11.5355095806999994
  coef(3864) =      41.3085512412000000
  coef(3865) =      10.0569230214999994
  coef(3866) =      -6.2731305876999999
  coef(3867) =      -8.8660330864900008
  coef(3868) =       5.4888664233400002
  coef(3869) =      12.7494203557999999
  coef(3870) =      16.1081952922000013
  coef(3871) =      57.9620757790999974
  coef(3872) =       3.6243699195599999
  coef(3873) =      -0.5520732984720000
  coef(3874) =       1.8635285217199999
  coef(3875) =      11.5409054640999997
  coef(3876) =      -7.2860280208399999
  coef(3877) =     -13.0422138412000006
  coef(3878) =       2.2771459253700002
  coef(3879) =       1.9643169406800001
  coef(3880) =     -40.3230090782000019
  coef(3881) =       4.4097319802900001
  coef(3882) =      33.0318306101000019
  coef(3883) =      -8.5678156710600000
  coef(3884) =      28.0748289934999988
  coef(3885) =      98.4913025702000056
  coef(3886) =     151.4037567869999918
  coef(3887) =     -29.0861269853000017
  coef(3888) =      -9.9866485326699994
  coef(3889) =     423.0842372849999720
  coef(3890) =    -106.6636222350000054
  coef(3891) =     -27.3390637191000003
  coef(3892) =    -126.9731325529999992
  coef(3893) =     186.6190919150000127
  coef(3894) =      71.4059187217000044
  coef(3895) =     -13.9138414257999994
  coef(3896) =   -1215.8339717900000778
  coef(3897) =      56.5577350292000034
  coef(3898) =    -455.3157455529999993
  coef(3899) =       3.4835575213299999
  coef(3900) =      11.9570071718000008
  coef(3901) =     -34.5643700314000029
  coef(3902) =     -16.2355594652999997
  coef(3903) =     244.0753024670000002
  coef(3904) =      14.7494955428000001
  coef(3905) =       2.3928954747700000
  coef(3906) =      -3.7502831018200000
  coef(3907) =      10.7470800265000008
  coef(3908) =      -1.8256943790100000
  coef(3909) =     -37.3297489774000013
  coef(3910) =      -4.2608348676299999
  coef(3911) =      48.5525842190000034
  coef(3912) =     -61.3122844473000015
  coef(3913) =     -22.7896002547000016
  coef(3914) =     -18.3696152218000002
  coef(3915) =      80.4375574011999959
  coef(3916) =     -11.3961436348999996
  coef(3917) =      18.7353188181999997
  coef(3918) =      64.7850829642000008
  coef(3919) =       2.7562624179199999
  coef(3920) =     -21.6740357328999984
  coef(3921) =      -2.1078545228499999
  coef(3922) =     -18.9145343176999994
  coef(3923) =      -2.5642728564200001
  coef(3924) =      16.7223200158999994
  coef(3925) =     142.6744388790000073
  coef(3926) =      63.6349247539000018
  coef(3927) =       1.3421269422300000
  coef(3928) =       2.8521238547899999
  coef(3929) =      31.2792630844000001
  coef(3930) =      -3.4045138327100002
  coef(3931) =       3.9594968344599999
  coef(3932) =     -31.2272539689999995
  coef(3933) =      -3.9015229957400002
  coef(3934) =      -0.8825807601070000
  coef(3935) =      -8.4592969990700002
  coef(3936) =     -14.4409341960000006
  coef(3937) =     -34.1195992748000023
  coef(3938) =       3.5768291838700002
  coef(3939) =     -71.0287575289000017
  coef(3940) =     -28.0993869642000007
  coef(3941) =       0.5508284149679999
  coef(3942) =     258.4502826679999998
  coef(3943) =     250.1417222729999992
  coef(3944) =    -821.4916157950000297
  coef(3945) =     126.5227245159999967
  coef(3946) =     485.7135071129999915
  coef(3947) =     243.7519370159999994
  coef(3948) =     409.0954431060000047
  coef(3949) =   -1112.1500704099998984
  coef(3950) =    1242.2629150499999469
  coef(3951) =    1202.6957744700000603
  coef(3952) =   -2216.8016801600001600
  coef(3953) =    -123.9482023150000032
  coef(3954) =     188.7630045809999899
  coef(3955) =    -559.5012026220000507
  coef(3956) =      -7.2436015484800000
  coef(3957) =    -207.6553654360000110
  coef(3958) =     -68.2095191256999982
  coef(3959) =     184.7585169420000000
  coef(3960) =   -1211.1718522199998915
  coef(3961) =     -52.3237699011000004
  coef(3962) =      -4.8058034277499999
  coef(3963) =      -2.8653304279500000
  coef(3964) =     -24.0356581065000015
  coef(3965) =     -11.2276754822000004
  coef(3966) =    -228.2816210290000072
  coef(3967) =      66.1027106855000000
  coef(3968) =    -295.8005492369999843
  coef(3969) =     477.4258294539999952
  coef(3970) =     107.2599801519999971
  coef(3971) =      68.4457980013000054
  coef(3972) =    -138.6037301339999885
  coef(3973) =    -460.4315428580000003
  coef(3974) =     -69.5609238857999941
  coef(3975) =    -136.9429085870000051
  coef(3976) =      21.2103936984999990
  coef(3977) =     178.8320364760000132
  coef(3978) =     -29.5951103836999998
  coef(3979) =      74.4181290757999960
  coef(3980) =     -46.6102216374999969
  coef(3981) =     -60.7518788961999974
  coef(3982) =    -276.8379022909999776
  coef(3983) =    -168.9659483650000027
  coef(3984) =      33.5660064672999994
  coef(3985) =     -99.8711809585000054
  coef(3986) =    -100.6072855160000046
  coef(3987) =      -4.6217676037500004
  coef(3988) =     -16.6182374959999990
  coef(3989) =      61.7708259707000025
  coef(3990) =    -702.6408782629999905
  coef(3991) =      62.9788717455000011
  coef(3992) =      57.6786380992000005
  coef(3993) =    -194.3945442019999916
  coef(3994) =     -36.2476099032000008
  coef(3995) =     -55.3245388476999977
  coef(3996) =     191.7005497879999893
  coef(3997) =     -30.3064284694999984
  coef(3998) =      90.6032215520999955
  coef(3999) =    -424.4724269739999727
  coef(4000) =    -424.0490588430000116
  coef(4001) =     200.9877317769999934
  coef(4002) =     448.7655281149999951
  coef(4003) =    -200.3265708190000112
  coef(4004) =      57.8085038314999977
  coef(4005) =    -863.2550956750000068
  coef(4006) =     -17.5873642050999983
  coef(4007) =     553.6946524600000430
  coef(4008) =       9.3926956477099992
  coef(4009) =    -363.7134045380000202
  coef(4010) =     275.4819558379999762
  coef(4011) =       9.9077050703499996
  coef(4012) =     607.7835922530000516
  coef(4013) =      17.6347271055999997
  coef(4014) =      13.4881266745000001
  coef(4015) =      -4.0040546912500004
  coef(4016) =      34.4095665470999990
  coef(4017) =       3.3267537925499999
  coef(4018) =    -147.0936716330000138
  coef(4019) =     -63.7523294574999966
  coef(4020) =     -77.9778923897000027
  coef(4021) =     130.0651953910000032
  coef(4022) =      37.2505169379999970
  coef(4023) =     -53.7513159275000021
  coef(4024) =     -59.0807906335000013
  coef(4025) =    -350.8713327629999981
  coef(4026) =    -227.5153192390000072
  coef(4027) =     -23.4833941312999990
  coef(4028) =    -125.0233112119999959
  coef(4029) =     -85.6922376411999949
  coef(4030) =     -66.3484374341000063
  coef(4031) =      33.1682319477000007
  coef(4032) =     -36.6029276240999977
  coef(4033) =     -37.0039885186999982
  coef(4034) =    -262.8977544620000231
  coef(4035) =    -109.2462917409999932
  coef(4036) =      94.7747009931999997
  coef(4037) =      16.1430665827999995
  coef(4038) =     -89.5717448727999965
  coef(4039) =      37.2859333812999978
  coef(4040) =      12.0613724459000000
  coef(4041) =     -70.7580466473000058
  coef(4042) =    -114.3665367400000008
  coef(4043) =      -7.1019669125300000
  coef(4044) =     -54.4922992370999992
  coef(4045) =     112.0150962590000034
  coef(4046) =      92.8029994656999975
  coef(4047) =      53.9508717336999979
  coef(4048) =    -110.1606975020000050
  coef(4049) =     -83.2113300691000006
  coef(4050) =     -18.0640848697000003
  coef(4051) =    -632.6732105900000533
  coef(4052) =     881.9936748940000371
  coef(4053) =   -3354.8577891300001284
  coef(4054) =    -288.8967631940000160
  coef(4055) =    2319.6815696200001184
  coef(4056) =      14.6792140336999992
  coef(4057) =     430.9113858279999931
  coef(4058) =     127.7140099380000038
  coef(4059) =     -28.1763345632000011
  coef(4060) =   -1308.5204835699998966
  coef(4061) =      61.1637018508000025
  coef(4062) =       2.2029268550399999
  coef(4063) =      -0.3348776599010000
  coef(4064) =      20.7813982272999986
  coef(4065) =      18.4452162058000013
  coef(4066) =     260.5994861199999946
  coef(4067) =     313.9192767279999998
  coef(4068) =     167.3611556150000013
  coef(4069) =    -169.6161210150000045
  coef(4070) =     -37.2669911098999975
  coef(4071) =     -38.3813606561000000
  coef(4072) =     395.8139744129999826
  coef(4073) =     715.6692032790000439
  coef(4074) =     259.1795155130000126
  coef(4075) =     175.1777111629999979
  coef(4076) =     181.1851567719999991
  coef(4077) =     644.8997047909999765
  coef(4078) =     365.3140171040000155
  coef(4079) =    -235.3848935450000113
  coef(4080) =    -133.1758147319999921
  coef(4081) =      10.0083598713999997
  coef(4082) =     790.6782952139999452
  coef(4083) =     368.4232873270000255
  coef(4084) =     290.7461303300000282
  coef(4085) =      44.3892238928999987
  coef(4086) =      70.4155411050000026
  coef(4087) =     -21.8802163205999989
  coef(4088) =     -34.5521123203999991
  coef(4089) =      22.1971982736000015
  coef(4090) =     471.4850054190000037
  coef(4091) =      16.1011420407999992
  coef(4092) =      61.9473179494999968
  coef(4093) =     334.4162876830000073
  coef(4094) =      -0.1155524251880000
  coef(4095) =      21.5553066844999996
  coef(4096) =     111.2636612000000014
  coef(4097) =      28.6086151953000005
  coef(4098) =     282.9706706939999776
  coef(4099) =      20.9911669813000010
  coef(4100) =    -175.5905045689999895
  coef(4101) =      -5.9256388832000004
  coef(4102) =     -12.4402951846000001
  coef(4103) =     -70.7925971174999944
  coef(4104) =      -0.9069145472720000
  coef(4105) =   -2274.6836774100002003
  coef(4106) =     -24.4468274184000016
  coef(4107) =      -5.6133368465100002
  coef(4108) =      -1.3551470992200001
  coef(4109) =     -21.7594357370999987
  coef(4110) =     -11.7665415739999997
  coef(4111) =     150.5889099949999945
  coef(4112) =     232.8401311249999992
  coef(4113) =    -126.7266778599999952
  coef(4114) =      53.2068138882999975
  coef(4115) =      10.4156476435999998
  coef(4116) =     180.5762657159999947
  coef(4117) =    -240.3620766809999907
  coef(4118) =     435.4362743739999928
  coef(4119) =     195.7912525460000097
  coef(4120) =     113.5416317730000060
  coef(4121) =     164.0495234450000055
  coef(4122) =     475.2891705119999983
  coef(4123) =     145.5306711659999905
  coef(4124) =     537.3680013729999700
  coef(4125) =     120.2547193359999937
  coef(4126) =       5.6363411557500003
  coef(4127) =    -285.7608919169999808
  coef(4128) =     -59.9982609261000022
  coef(4129) =    -111.1987862919999941
  coef(4130) =      25.0608942847000016
  coef(4131) =      47.4203041949999999
  coef(4132) =      43.8089296301000033
  coef(4133) =     -14.2599699449999999
  coef(4134) =     269.1144068589999847
  coef(4135) =    -194.0846655109999972
  coef(4136) =      -2.3492142062600001
  coef(4137) =     112.7941926719999941
  coef(4138) =    -163.4434255179999980
  coef(4139) =      -2.4264384962099999
  coef(4140) =     -36.5948627112000011
  coef(4141) =     489.6722889299999792
  coef(4142) =     171.9142542959999957
  coef(4143) =     238.3661133230000075
  coef(4144) =      -0.5428067521050000
  coef(4145) =       3.7157348348000001
  coef(4146) =     -52.0980223994000013
  coef(4147) =      -5.6734596735300000
  coef(4148) =      -0.7019326799590000
  coef(4149) =       7.7340472181999997
  coef(4150) =     -43.1991746344000020
  coef(4151) =      25.4168952710999996
  coef(4152) =       6.7183114616999999
  coef(4153) =      27.6985015825999987
  coef(4154) =     -12.9403501807999994
  coef(4155) =      50.3566734066999970
  coef(4156) =      -5.3618232782800002
  coef(4157) =       7.8022712706000004
  coef(4158) =       7.1092028735000001
  coef(4159) =     -44.6325261276999967
  coef(4160) =     158.2234619669999915
  coef(4161) =      25.9269921871999998
  coef(4162) =      -0.3792507650790000
  coef(4163) =       9.5248553491900001
  coef(4164) =     -79.7727191180999995
  coef(4165) =    -107.6762691859999990
  coef(4166) =      30.2078312382999989
  coef(4167) =     163.8424139930000081
  coef(4168) =     159.0239995979999890
  coef(4169) =     -50.2889269479999967
  coef(4170) =      48.3999465612999984
  coef(4171) =    -116.5216836750000056
  coef(4172) =      50.4859633494999969
  coef(4173) =       6.1922863049799997
  coef(4174) =    -253.7178104930000018
  coef(4175) =      20.5159210718000011
  coef(4176) =       4.0342737147800003
  coef(4177) =     -85.6820659148000061
  coef(4178) =      92.5681494309999948
  coef(4179) =     146.6364924190000067
  coef(4180) =       6.1952320140700001
  coef(4181) =      93.4928033414000055
  coef(4182) =      81.2955467637000027
  coef(4183) =     275.4941375029999904
  coef(4184) =     102.9576564529999985
  coef(4185) =      80.1114388759000065
  coef(4186) =     -40.9851436822999986
  coef(4187) =     384.6509442949999880
  coef(4188) =    -141.1056686930000126
  coef(4189) =     -41.6234323665999995
  coef(4190) =      -0.9121457364780000
  coef(4191) =       3.1505326258399999
  coef(4192) =   -1659.2220278800000415
  coef(4193) =      -5.3067612861800004
  coef(4194) =       4.7093856714799998
  coef(4195) =       1.1772055207100001
  coef(4196) =      42.4642409909999969
  coef(4197) =       2.1834879317000002
  coef(4198) =      -4.8151321248099999
  coef(4199) =      51.4238995007000028
  coef(4200) =      22.4808508280999995
  coef(4201) =     112.7657515780000068
  coef(4202) =      -8.2858352758299993
  coef(4203) =      18.6411677988999998
  coef(4204) =      11.1982903398999998
  coef(4205) =     -39.8405418193000003
  coef(4206) =     -31.4396083697000002
  coef(4207) =     245.8649978919999910
  coef(4208) =     -33.7316280232999972
  coef(4209) =       7.3962297046599996
  coef(4210) =     -32.5827231522999980
  coef(4211) =     -20.6996837907000000
  coef(4212) =     280.7354845319999868
  coef(4213) =      13.7021753042000007
  coef(4214) =     123.7220624469999990
  coef(4215) =      33.1878301797000006
  coef(4216) =     -14.4601153148999995
  coef(4217) =     204.6664026949999879
  coef(4218) =    -150.0790233079999894
  coef(4219) =     -24.3337356472000010
  coef(4220) =     284.4424687659999904
  coef(4221) =      15.7881154532999997
  coef(4222) =     284.0568996979999952
  coef(4223) =     -92.7422553019000020
  coef(4224) =    -108.6683902230000029
  coef(4225) =    -127.4800138140000030
  coef(4226) =     640.2757645679999996
  coef(4227) =       3.0121945985999998
  coef(4228) =     258.6534623800000077
  coef(4229) =      33.6071362262999997
  coef(4230) =     180.6788688800000102
  coef(4231) =     -57.1629788238000032
  coef(4232) =    -438.8147881300000108
  coef(4233) =    -240.7133641659999910
  coef(4234) =     335.9795439420000207
  coef(4235) =      87.7394208466999999
  coef(4236) =      95.2142394270999972
  coef(4237) =     -72.1104136908000015
  coef(4238) =      13.7075617140000006
  coef(4239) =     -19.3168191526000008
  coef(4240) =     -50.9563553185999965
  coef(4241) =     574.1605743229999916
  coef(4242) =      91.4772925307999998
  coef(4243) =    -164.8793168310000112
  coef(4244) =    -318.5394437269999912
  coef(4245) =     -21.3384026079999991
  coef(4246) =       4.3311000419500001
  coef(4247) =     -18.3830437483999987
  coef(4248) =     -85.6947756035000054
  coef(4249) =     169.0069269040000108
  coef(4250) =      48.2829530735000034
  coef(4251) =    -103.3490495829999958
  coef(4252) =     -49.0976081681999972
  coef(4253) =    -423.3676735999999892
  coef(4254) =      68.9547454957999975
  coef(4255) =      -5.9812638816300003
  coef(4256) =     -23.0290547978999989
  coef(4257) =       5.2896566427699998
  coef(4258) =     -10.1736645398000007
  coef(4259) =       5.6470183370299996
  coef(4260) =      86.0362230994000043
  coef(4261) =      43.3840533500999967
  coef(4262) =     -10.2349357882999996
  coef(4263) =     -18.4889637859999993
  coef(4264) =     -88.7069047548999947
  coef(4265) =      89.9916858600000040
  coef(4266) =      41.7004606571999972
  coef(4267) =      13.4195950985000003
  coef(4268) =       6.4122311227699997
  coef(4269) =      52.1276225024000013
  coef(4270) =     117.6044572119999998
  coef(4271) =     -43.2486742199000034
  coef(4272) =      40.3690290385999973
  coef(4273) =       8.2775599856199999
  coef(4274) =      91.3246199907000005
  coef(4275) =     709.7285685489999878
  coef(4276) =    -183.2393885680000096
  coef(4277) =     -78.1387555163999963
  coef(4278) =      63.8221532020000026
  coef(4279) =     579.0024486959999876
  coef(4280) =    -163.6863032070000088
  coef(4281) =    -102.1924563999999975
  coef(4282) =      40.1319841693999990
  coef(4283) =      -0.6037655391440000
  coef(4284) =     244.7242402219999917
  coef(4285) =    -350.6112235389999796
  coef(4286) =     -64.6468973822999970
  coef(4287) =     116.7688055739999982
  coef(4288) =       6.6579304981399998
  coef(4289) =     792.2222464459999856
  coef(4290) =     326.2419024110000123
  coef(4291) =      64.7170789298999978
  coef(4292) =     -12.6074625051000009
  coef(4293) =      -7.0881433215799996
  coef(4294) =     -31.0416995337000010
  coef(4295) =       4.7222994205699997
  coef(4296) =      23.7499127638999994
  coef(4297) =     237.7024782660000142
  coef(4298) =      18.4272721900999983
  coef(4299) =     105.4024359040000007
  coef(4300) =     178.0970936280000103
  coef(4301) =      -5.4190377181300002
  coef(4302) =     108.2684341639999985
  coef(4303) =     384.4369724339999834
  coef(4304) =    -147.9063066900000081
  coef(4305) =      91.2584176719000055
  coef(4306) =      -1.5243118214700000
  coef(4307) =      -1.2899370531800001
  coef(4308) =      -1.1238668401600000
  coef(4309) =       1.6196318183500000
  coef(4310) =       1.3517755222400001
  coef(4311) =       0.2008963483070000
  coef(4312) =      -1.7635589523299999
  coef(4313) =      -3.2391227882300000
  coef(4314) =       1.2669570233700000
  coef(4315) =      -0.1639400923400000
  coef(4316) =       1.5162360109399999
  coef(4317) =      -1.3356713598900001
  coef(4318) =      37.3494237417000008
  coef(4319) =     -11.0112827945999996
  coef(4320) =       8.9035271401500005
  coef(4321) =     -66.7849416374999976
  coef(4322) =      26.9679666713999993
  coef(4323) =      -3.8318426899900002
  coef(4324) =      -1.1069577317999999
  coef(4325) =     -80.8598891386999981
  coef(4326) =      34.9887219865000034
  coef(4327) =      -2.6058470145400001
  coef(4328) =     -25.0558799544000017
  coef(4329) =       1.6412497184899999
  coef(4330) =      27.4078682194999992
  coef(4331) =      43.3003447003000019
  coef(4332) =      21.2897039461000013
  coef(4333) =      -1.6990290301099999
  coef(4334) =     -56.7923093813000008
  coef(4335) =     -21.5308313156999986
  coef(4336) =     -14.1958810807999996
  coef(4337) =     -50.3974985513999982
  coef(4338) =      -7.1734694614799999
  coef(4339) =     -17.1186504765999992
  coef(4340) =      -5.3195590201699998
  coef(4341) =     -14.3477965671999996
  coef(4342) =      -7.7010615893900001
  coef(4343) =       2.8671451918100002
  coef(4344) =      -8.3320300031999999
  coef(4345) =      -4.7193375501599997
  coef(4346) =      -3.7642599634799998
  coef(4347) =      -5.0768144094799998
  coef(4348) =      26.2589490655999995
  coef(4349) =      44.7797154627999987
  coef(4350) =      -0.0000003233028069
  coef(4351) =      -2.9139565333799999
  coef(4352) =    -204.2081125539999960
  coef(4353) =     -61.2500296356000007
  coef(4354) =      16.4883110709999983
  coef(4355) =       7.4382715110099999
  coef(4356) =      -1.5635547901100000
  coef(4357) =       2.6466097114700000
  coef(4358) =       3.7242031405399998
  coef(4359) =      11.8064557355000002
  coef(4360) =       6.6923912115500004
  coef(4361) =       4.5738698262700002
  coef(4362) =      13.2402959925000001
  coef(4363) =      -6.3180797803599997
  coef(4364) =     -17.2814064449999982
  coef(4365) =      24.7120302585000005
  coef(4366) =      20.2811462847999984
  coef(4367) =       5.4690467125400000
  coef(4368) =      -6.5442570711099997
  coef(4369) =       1.5366639234699999
  coef(4370) =      -6.2930511219799996
  coef(4371) =    -295.3752089129999945
  coef(4372) =     -97.5833205062999980
  coef(4373) =    -117.0327758009999997
  coef(4374) =     -53.8094017658999988
  coef(4375) =      -5.5413521130100003
  coef(4376) =      52.3401947077000003
  coef(4377) =      -5.3755263402100004
  coef(4378) =    -101.5539260259999992
  coef(4379) =      -7.0829053445700003
  coef(4380) =      -6.7100033903699998
  coef(4381) =      14.4996729002000002
  coef(4382) =      -9.3025681085799992
  coef(4383) =      -8.6530602970599997
  coef(4384) =       0.4530187107980000
  coef(4385) =     -14.3614138301000001
  coef(4386) =      -4.0521077748699996
  coef(4387) =     -11.6734166262999999
  coef(4388) =       6.0420431619199997
  coef(4389) =       1.4397966891000000
  coef(4390) =      25.9341857165000000
  coef(4391) =      12.5124902446000004
  coef(4392) =     -28.2735384908999983
  coef(4393) =      -6.8532435658599997
  coef(4394) =     -24.9795159550000001
  coef(4395) =     -12.0039353904000006
  coef(4396) =       0.7721135569490000
  coef(4397) =      -4.8924584526199997
  coef(4398) =     -11.8300821666000004
  coef(4399) =     -10.6140753611999994
  coef(4400) =      -7.9970871771800001
  coef(4401) =      -3.7937481960100001
  coef(4402) =       9.5292043936000006
  coef(4403) =      78.6718852648999984
  coef(4404) =       8.3075429349099998
  coef(4405) =       9.1886013622699991
  coef(4406) =       7.4560810393899999
  coef(4407) =       5.2430537551900001
  coef(4408) =       6.2258818299799996
  coef(4409) =       9.8712041863600000
  coef(4410) =       4.2659288113200002
  coef(4411) =       6.8574477518799997
  coef(4412) =      -0.4508351529910000
  coef(4413) =       2.0240195509599999
  coef(4414) =       1.9826665161000001
  coef(4415) =      -1.1551624861700001
  coef(4416) =       1.0195714998400001
  coef(4417) =       1.9803311352499999
  coef(4418) =       4.9543195019199997
  coef(4419) =       7.6220033264299998
  coef(4420) =       8.9419867758099993
  coef(4421) =       0.4786269994760000
  coef(4422) =       0.1744649798820000
  coef(4423) =      28.8053999060000017
  coef(4424) =     -18.6067574966000002
  coef(4425) =      35.8817728134999996
  coef(4426) =      30.9871923761000012
  coef(4427) =      -0.6295723615950000
  coef(4428) =       3.7192085601699998
  coef(4429) =      10.4482635594000008
  coef(4430) =      32.2204865647000034
  coef(4431) =      -6.4476147617499997
  coef(4432) =      16.3635194967999986
  coef(4433) =       2.5652197557399998
  coef(4434) =      19.6218953125999995
  coef(4435) =       0.6625669685120000
  coef(4436) =      -1.7947865616300001
  coef(4437) =      22.4234061812999990
  coef(4438) =      -2.5082471919699998
  coef(4439) =       3.0633208783399999
  coef(4440) =    -181.6422367069999950
  coef(4441) =     -35.2354571142000026
  coef(4442) =     -14.4014410892000004
  coef(4443) =     -44.6045533594000005
  coef(4444) =     -27.6829612492000017
  coef(4445) =     -15.9880854163000006
  coef(4446) =      -2.3773807820299999
  coef(4447) =      -3.3996112024500000
  coef(4448) =      -1.2571209415300000
  coef(4449) =      -4.8543454785900000
  coef(4450) =      59.8826143284999972
  coef(4451) =     -10.1720107559000006
  coef(4452) =       8.9211865707700007
  coef(4453) =      -3.8272461140399998
  coef(4454) =      32.3307859060999974
  coef(4455) =      -7.8413572151400004
  coef(4456) =      -4.9450090541899998
  coef(4457) =     -25.0047300606000000
  coef(4458) =       2.4687703028399999
  coef(4459) =     -20.8892346171999996
  coef(4460) =      -6.1056819519800003
  coef(4461) =      -2.0745643580799999
  coef(4462) =     -18.3122834455999985
  coef(4463) =       1.8412112137800001
  coef(4464) =       3.9548805458900000
  coef(4465) =      17.1732891053000003
  coef(4466) =      -7.1876385112100003
  coef(4467) =      -9.1692641351800006
  coef(4468) =      -9.9308492360400002
  coef(4469) =      -2.2406378165900001
  coef(4470) =       2.7272211411799998
  coef(4471) =       1.8223069997000001
  coef(4472) =       2.9885748373099998
  coef(4473) =       0.3742653490260000
  coef(4474) =      11.6694054575999999
  coef(4475) =       1.4221521448100001
  coef(4476) =       1.5992903101600000
  coef(4477) =       0.0347388979500300
  coef(4478) =      -0.5157907627850000
  coef(4479) =     130.0765346770000122
  coef(4480) =     126.9138794760000053
  coef(4481) =      52.7083052313999971
  coef(4482) =      23.7177353603000007
  coef(4483) =     -96.3478951438000024
  coef(4484) =      -9.7868510536199995
  coef(4485) =      10.8408026221000000
  coef(4486) =      -6.7150190531699998
  coef(4487) =      41.5417688349999992
  coef(4488) =      33.1626316564000021
  coef(4489) =      64.6538404531000026
  coef(4490) =      -3.4897800653000002
  coef(4491) =      12.3798798853999994
  coef(4492) =      22.5206060870000009
  coef(4493) =     -71.0493798019000025
  coef(4494) =      15.0754226776000007
  coef(4495) =       2.6208253701099999
  coef(4496) =      12.1460970141000004
  coef(4497) =      11.3966966842000001
  coef(4498) =      23.1698200396999994
  coef(4499) =      -8.1070532294100008
  coef(4500) =      15.9989641736999992
  coef(4501) =       3.0293398741900002
  coef(4502) =     -33.8582550580999992
  coef(4503) =     -31.8133397715000008
  coef(4504) =      16.1295055691000009
  coef(4505) =     -21.1522308720000005
  coef(4506) =     -46.7366807599999987
  coef(4507) =     -19.5089806026000012
  coef(4508) =      -7.6463741724799998
  coef(4509) =      15.2919218457999992
  coef(4510) =      -0.2267593039850000
  coef(4511) =       3.8816632632300001
  coef(4512) =     -70.8209798656999965
  coef(4513) =      -6.5492769964299997
  coef(4514) =       2.3722408538400002
  coef(4515) =     -25.8271013158000002
  coef(4516) =      14.0490129487999997
  coef(4517) =     -45.9845208837000001
  coef(4518) =       4.2361204700800004
  coef(4519) =      -5.3754265114499997
  coef(4520) =      -8.5088970713100007
  coef(4521) =      -4.1259780466800002
  coef(4522) =     162.4767489669999918
  coef(4523) =      17.3706624844000004
  coef(4524) =     -22.1787102826999991
  coef(4525) =       5.7570664936300000
  coef(4526) =       4.0614021020999997
  coef(4527) =     -12.0860038034000006
  coef(4528) =      -9.0332646590899994
  coef(4529) =       5.2013674851999996
  coef(4530) =  -11724.3638828000002832
  coef(4531) =   -5164.8095657300000312
  coef(4532) =     142.9534953929999972
  coef(4533) =   -5731.1517247600004339
  coef(4534) =   28371.7454023999998753
  coef(4535) =   12524.0158890000002430
  coef(4536) =    2297.7874018399998022
  coef(4537) =   -3380.8891678999998476
  coef(4538) =  -13035.7282845000008820
  coef(4539) =       3.4076919404999999
  coef(4540) =    -120.4129219419999970
  coef(4541) =   -1404.5567644800000835
  coef(4542) =    2819.8454562400002033
  coef(4543) =    1509.3062466399999266
  coef(4544) =     437.3821004949999747
  coef(4545) =   -3357.7619500599998901
  coef(4546) =    -179.2947495580000066
  coef(4547) =   -4404.5572005100002571
  coef(4548) =   -3117.9716336000001320
  coef(4549) =     -72.9114927453999968
  coef(4550) =    -285.5294101110000042
  coef(4551) =     627.5690756469999769
  coef(4552) =   -1057.5833002500000930
  coef(4553) =    3236.7825045899999168
  coef(4554) =    1254.4082252999999127
  coef(4555) =    -352.6859063439999886
  coef(4556) =   -8879.5242994500003988
  coef(4557) =   -7055.5249412900002426
  coef(4558) =   -1793.1193052000000989
  coef(4559) =    1085.9319605900000170
  coef(4560) =    1402.4224538700000267
  coef(4561) =       1.3847584482899999
  coef(4562) =      19.7957083693999998
  coef(4563) =    -165.7389756510000041
  coef(4564) =     -96.2541373741999990
  coef(4565) =     -16.4876034402000009
  coef(4566) =      21.8675988680000017
  coef(4567) =      82.5761792162000035
  coef(4568) =     -33.6901736494000019
  coef(4569) =     125.4673063740000032
  coef(4570) =   -1100.1595861900000273
  coef(4571) =      95.4502359083000016
  coef(4572) =     110.5331649639999938
  coef(4573) =    -153.2492798810000068
  coef(4574) =     686.5594465489999720
  coef(4575) =     805.6743116290000444
  coef(4576) =      12.3161129712000008
  coef(4577) =    -541.9353148510000437
  coef(4578) =     699.5851885990000483
  coef(4579) =     881.4892743310000469
  coef(4580) =      -6.0625577688299996
  coef(4581) =     526.0198155120000365
  coef(4582) =    -144.2754468569999915
  coef(4583) =    -273.7571109109999838
  coef(4584) =      25.3197086173999999
  coef(4585) =    -591.6853018020000263
  coef(4586) =    1642.2407614399999147
  coef(4587) =    2204.1866464400000041
  coef(4588) =   -3234.1807200399998692
  coef(4589) =    -623.7189194830000361
  coef(4590) =    2103.4873942399999578
  coef(4591) =  -13441.0258274999996502
  coef(4592) =  -16196.8153120000006311
  coef(4593) =    8336.5493852100007643
  coef(4594) =   -4385.7830749100003231
  coef(4595) =   -3441.3957140399998025
  coef(4596) =    1598.2116477800000212
  coef(4597) =    4908.3619724399995903
  coef(4598) =     -94.0179959746999998
  coef(4599) =    1871.0844254999999521
  coef(4600) =     253.0746740460000126
  coef(4601) =    -223.6122640650000051
  coef(4602) =    2673.0589487600000211
  coef(4603) =      95.3141942690999997
  coef(4604) =    -194.0551405820000070
  coef(4605) =     812.9083752239999967
  coef(4606) =    -154.8571071240000094
  coef(4607) =  -13046.9611366000008275
  coef(4608) =     115.7731093200000032
  coef(4609) =     108.9797822919999959
  coef(4610) =      56.2798389722999985
  coef(4611) =      16.9256287199000006
  coef(4612) =     -36.9388676561000011
  coef(4613) =     409.0042383940000263
  coef(4614) =    1103.7373975699999846
  coef(4615) =   -1188.5946328299999095
  coef(4616) =      39.0129683782000001
  coef(4617) =     -70.0729276243999948
  coef(4618) =     523.5635655080000106
  coef(4619) =    -887.2741296950000560
  coef(4620) =    1136.4328766099999939
  coef(4621) =    -537.2794753819999869
  coef(4622) =    -255.6263958140000057
  coef(4623) =     -37.2889030171999991
  coef(4624) =    3924.3244712400000935
  coef(4625) =    2573.0491584400001557
  coef(4626) =    4894.6951893499999642
  coef(4627) =    1192.4690196700000797
  coef(4628) =    1538.9080527300000085
  coef(4629) =    -213.6570397300000081
  coef(4630) =     534.5982153080000217
  coef(4631) =     354.5328227379999930
  coef(4632) =     248.3580322050000007
  coef(4633) =    -432.8339202189999924
  coef(4634) =    1157.4299464799998987
  coef(4635) =     852.2078687140000284
  coef(4636) =    -144.9614102939999896
  coef(4637) =   -1020.6004245799999808
  coef(4638) =     183.9624358050000126
  coef(4639) =     -22.0333712754000004
  coef(4640) =    -661.4497901749999755
  coef(4641) =     346.7661337069999945
  coef(4642) =    -574.4982611010000255
  coef(4643) =    -272.5195153290000007
  coef(4644) =    -211.9055279639999867
  coef(4645) =    -351.3249488399999905
  coef(4646) =      -0.3881329676070000
  coef(4647) =      -1.2087016044400001
  coef(4648) =      16.9409446189000015
  coef(4649) =      -9.1156176292300000
  coef(4650) =     -12.2887403886000008
  coef(4651) =       1.4182788930000001
  coef(4652) =     -11.4434588307999991
  coef(4653) =      21.0958895843000001
  coef(4654) =       0.5001566820920000
  coef(4655) =      56.6104924440000019
  coef(4656) =      59.2199900729999982
  coef(4657) =     -20.1565221261999987
  coef(4658) =      -5.0437213944100003
  coef(4659) =      12.1860877974000008
  coef(4660) =     -27.1345749496999993
  coef(4661) =      14.5795668038000006
  coef(4662) =       1.8903787885800001
  coef(4663) =      89.9184182878999962
  coef(4664) =      46.2632062536999982
  coef(4665) =      14.3062653129000008
  coef(4666) =     -31.4316793320000016
  coef(4667) =       4.2410475372500001
  coef(4668) =      -5.3164142516700004
  coef(4669) =     354.3225222070000200
  coef(4670) =     356.8366487420000226
  coef(4671) =    -110.7398597079999973
  coef(4672) =    -148.1203193650000003
  coef(4673) =    -780.1315825420000465
  coef(4674) =    -322.8494359680000230
  coef(4675) =     366.2002183700000160
  coef(4676) =    1101.9901689900000292
  coef(4677) =     482.5766889779999929
  coef(4678) =    2250.7475485700001627
  coef(4679) =    -556.3235771969999632
  coef(4680) =    -673.7329723450000074
  coef(4681) =     -26.0154347841000018
  coef(4682) =     512.1092571870000256
  coef(4683) =    -179.6761075379999966
  coef(4684) =      -3.5474167478599998
  coef(4685) =     646.7859452520000332
  coef(4686) =     -29.5979061343999987
  coef(4687) =     517.5409919689999469
  coef(4688) =     -16.2106221332000011
  coef(4689) =     -18.9290119566999984
  coef(4690) =     383.6471737350000240
  coef(4691) =      38.2535677687000017
  coef(4692) =     635.3951038339999968
  coef(4693) =     -59.7587699075999979
  coef(4694) =     -27.0392294699000004
  coef(4695) =      -3.3879248608700001
  coef(4696) =     -25.5953936052000017
  coef(4697) =       2.0370880025800000
  coef(4698) =      39.0438105636999992
  coef(4699) =    -168.2827118700000142
  coef(4700) =    -173.6553714380000031
  coef(4701) =     132.8694174869999927
  coef(4702) =      48.0205097922999968
  coef(4703) =    -125.3797560560000051
  coef(4704) =     -74.9892924468999951
  coef(4705) =    -147.2516384509999909
  coef(4706) =    -125.8236114660000027
  coef(4707) =    -203.9135880990000089
  coef(4708) =      21.8831366476000007
  coef(4709) =    -196.8443528749999984
  coef(4710) =    -250.4699896639999963
  coef(4711) =    -613.5382905740000297
  coef(4712) =    -162.3334807589999969
  coef(4713) =    -230.6453299490000006
  coef(4714) =    -127.0951441039999992
  coef(4715) =     -67.9832215690999959
  coef(4716) =      29.6319078128000015
  coef(4717) =     -85.7180989810999989
  coef(4718) =    -122.4904843479999954
  coef(4719) =    -105.5680438100000060
  coef(4720) =    -137.2784471559999986
  coef(4721) =    -175.9636944659999926
  coef(4722) =    -226.5819834870000022
  coef(4723) =      28.4326539549999993
  coef(4724) =     -81.1482375877999971
  coef(4725) =     119.3580556680000058
  coef(4726) =      78.8014683252999930
  coef(4727) =      59.3722709696999971
  coef(4728) =    -119.9939970439999968
  coef(4729) =     -94.0626781168000008
  coef(4730) =     -74.1242725250999968
  coef(4731) =    -149.2843551089999892
  coef(4732) =    -612.6237508750000416
  coef(4733) =      82.0867324997999930
  coef(4734) =    -185.8979273930000033
  coef(4735) =      64.5856784897000011
  coef(4736) =    -403.7288571710000156
  coef(4737) =     348.1219718379999790
  coef(4738) =     505.4008845100000258
  coef(4739) =     276.3937922509999794
  coef(4740) =    -925.0075945080000110
  coef(4741) =     390.0798904449999895
  coef(4742) =    -116.7092939589999929
  coef(4743) =     253.2414940829999921
  coef(4744) =     136.6327751670000055
  coef(4745) =     -28.1601067016000002
  coef(4746) =    -219.2431766660000108
  coef(4747) =     395.9474558109999975
  coef(4748) =     570.8202777479999668
  coef(4749) =     970.1978194810000105
  coef(4750) =     195.0569430079999904
  coef(4751) =    2990.0316429999998036
  coef(4752) =     107.9158867379999975
  coef(4753) =   -2243.7144089200000963
  coef(4754) =   -1496.1134274399998958
  coef(4755) =    4299.2596922599996105
  coef(4756) =     647.9420148680000011
  coef(4757) =     797.9095181310000271
  coef(4758) =    -979.9631373199999871
  coef(4759) =   -2270.9773247100001754
  coef(4760) =    -508.6279922630000101
  coef(4761) =    -238.5286284219999970
  coef(4762) =    1455.8914081299999452
  coef(4763) =    1654.2157745599999998
  coef(4764) =   -1252.2124733600001036
  coef(4765) =     166.7407452440000100
  coef(4766) =     105.9447317930000025
  coef(4767) =    -501.5133456059999730
  coef(4768) =    -403.3031331689999774
  coef(4769) =     294.2786193850000132
  coef(4770) =     574.8488703360000045
  coef(4771) =   -1026.9614184399999885
  coef(4772) =   -1288.0553830900000776
  coef(4773) =     317.7438010860000190
  coef(4774) =     251.4516265940000039
  coef(4775) =     418.5950924119999854
  coef(4776) =      56.7766773760999968
  coef(4777) =     492.9704122549999852
  coef(4778) =     485.7795909409999808
  coef(4779) =     100.8881976159999994
  coef(4780) =     325.4172466250000184
  coef(4781) =    -473.2971201550000160
  coef(4782) =     933.9781665350000139
  coef(4783) =     710.1712668320000148
  coef(4784) =     589.1808841829999892
  coef(4785) =    -351.3404483059999848
  coef(4786) =    4128.6238818199999514
  coef(4787) =    1985.8582922999999028
  coef(4788) =    1440.8452732799999012
  coef(4789) =    -137.3149340450000011
  coef(4790) =    -201.4204840829999910
  coef(4791) =       1.2800245037700000
  coef(4792) =    -404.6623324970000226
  coef(4793) =    -620.2632455220000338
  coef(4794) =     294.9282546939999747
  coef(4795) =      51.4930637526000012
  coef(4796) =       7.5816256293800004
  coef(4797) =    1963.7310369700001047
  coef(4798) =    -533.8578414380000368
  coef(4799) =    -623.2267278140000144
  coef(4800) =      51.6740843135999981
  coef(4801) =    4790.7566540199995870
  coef(4802) =    4050.1582782400000724
  coef(4803) =    -327.6346684270000083
  coef(4804) =    -259.0192395650000208
  coef(4805) =    -361.2751552619999984
  coef(4806) =     307.7333278219999784
  coef(4807) =     -34.6885027053000030
  coef(4808) =    -763.0339044329999751
  coef(4809) =      76.1318389187999998
  coef(4810) =     104.9086289269999952
  coef(4811) =    -234.5797122630000047
  coef(4812) =     223.6486959039999931
  coef(4813) =     132.6569374729999993
  coef(4814) =    -757.5628502690000232
  coef(4815) =    1919.3284648100000140
  coef(4816) =    4322.1847713200004364
  coef(4817) =    2064.8528081300000849
  coef(4818) =    -788.1677265319999606
  coef(4819) =    1531.8849920499999371
  coef(4820) =   -3572.4613842099997782
  coef(4821) =    5935.0067246700000396
  coef(4822) =    -507.4957942209999828
  coef(4823) =       6.9025349706899997
  coef(4824) =     159.9770370239999977
  coef(4825) =      86.4764708010000049
  coef(4826) =     -80.9106175012999955
  coef(4827) =     -80.2393127803999988
  coef(4828) =     113.6557827519999933
  coef(4829) =      68.2906274030000020
  coef(4830) =      -4.2214471867699999
  coef(4831) =      -5.1473350062199996
  coef(4832) =      55.7156613089000032
  coef(4833) =    -127.7627133670000035
  coef(4834) =     -62.5795810974000020
  coef(4835) =    -486.3857639170000198
  coef(4836) =     303.3193772719999970
  coef(4837) =     -18.3534947734000013
  coef(4838) =    -331.6001352239999846
  coef(4839) =     275.9080711889999975
  coef(4840) =      22.5018642574000012
  coef(4841) =      90.2227971867999941
  coef(4842) =      37.8974320422000019
  coef(4843) =    -180.4015877880000005
  coef(4844) =     -79.7804481868000011
  coef(4845) =       1.7378528073299999
  coef(4846) =     -54.8336040277999999
  coef(4847) =     -66.8491085011000052
  coef(4848) =     -59.7362645202999971
  coef(4849) =     -42.7645526017999984
  coef(4850) =    -233.8031892479999954
  coef(4851) =     222.3929155499999979
  coef(4852) =      74.4346424022999997
  coef(4853) =     -18.7993023484000013
  coef(4854) =   -1435.8051154899999347
  coef(4855) =    -860.8710409239999990
  coef(4856) =     183.8908094079999955
  coef(4857) =      -7.4132341815400000
  coef(4858) =     -54.7089052489999972
  coef(4859) =     -29.1974392025999983
  coef(4860) =     -76.3539324454000052
  coef(4861) =   -1329.2656131599999298
  coef(4862) =    -183.9379195859999925
  coef(4863) =    -875.2814310270000533
  coef(4864) =    -377.4528062389999832
  coef(4865) =      16.6645188837999996
  coef(4866) =     146.5022317950000001
  coef(4867) =     -52.4195891984999989
  coef(4868) =     -33.0093763829000011
  coef(4869) =       2.7078980312200001
  coef(4870) =    -187.0992349919999924
  coef(4871) =     -59.9231865974999991
  coef(4872) =     -89.2487713672999945
  coef(4873) =     -25.6336216732999986
  coef(4874) =     -58.3851530088000032
  coef(4875) =     -97.9480239262999959
  coef(4876) =      15.5108739571999994
  coef(4877) =     -93.6771962828999989
  coef(4878) =     291.7102724509999803
  coef(4879) =       6.7194091521699999
  coef(4880) =     107.5876429430000059
  coef(4881) =      -4.0887633335300002
  coef(4882) =     -26.8934328968000003
  coef(4883) =      -2.1888105290800000
  coef(4884) =     -24.1528507151999996
  coef(4885) =     -52.4734233097000029
  coef(4886) =     -82.4921588208000003
  coef(4887) =    -641.9167461739999681
  coef(4888) =    -104.5707171800000026
  coef(4889) =    -106.1001954740000031
  coef(4890) =    -153.2978811060000055
  coef(4891) =      41.0504225506999987
  coef(4892) =     -32.8920795634000029
  coef(4893) =     226.6484291200000030
  coef(4894) =     -13.9271065843000006
  coef(4895) =     -18.5599853713000016
  coef(4896) =       8.7169993432700004
  coef(4897) =     133.0538512599999876
  coef(4898) =    -131.0829381739999917
  coef(4899) =     -44.5337067840000032
  coef(4900) =      72.2561087091000047
  coef(4901) =      10.2591370252999994
  coef(4902) =     132.1875848409999890
  coef(4903) =      57.9780282632999970
  coef(4904) =     104.9841667519999930
  coef(4905) =      24.8005676488000013
  coef(4906) =     -57.9768470627000028
  coef(4907) =     363.1865996669999959
  coef(4908) =     163.0813673469999969
  coef(4909) =      97.9400018783999968
  coef(4910) =       0.5265493714910000
  coef(4911) =      44.9999071351000026
  coef(4912) =     -40.4383985500000023
  coef(4913) =      27.8971828413999994
  coef(4914) =      22.7655237737999983
  coef(4915) =      46.1595047940000001
  coef(4916) =      77.6030780373999960
  coef(4917) =      56.6970847844999994
  coef(4918) =     -84.1227986550999987
  coef(4919) =      -3.8747509487400000
  coef(4920) =     -12.2324855485999997
  coef(4921) =     -85.8000057905999967
  coef(4922) =    -160.0808454520000055
  coef(4923) =     301.4165765779999901
  coef(4924) =      52.9156138844000026
  coef(4925) =       0.0064594124351340
  coef(4926) =      -0.2243420192260000
  coef(4927) =       0.7619181710790000
  coef(4928) =      -0.1286966948380000
  coef(4929) =       0.7760283131830000
  coef(4930) =       0.2805186111430000
  coef(4931) =       0.7100976578800000
  coef(4932) =      -0.0500179957819000
  coef(4933) =       1.3256651156199999
  coef(4934) =       2.2180740989299998
  coef(4935) =      -0.7043159467450000
  coef(4936) =      -0.0526880190973100
  coef(4937) =      -0.0870812410982600
  coef(4938) =      -4.4962904130099997
  coef(4939) =       4.7047549754500002
  coef(4940) =       0.2754860503550000
  coef(4941) =       2.8498935026300001
  coef(4942) =     -12.9034278592000007
  coef(4943) =      -6.3402450051199999
  coef(4944) =       1.4514479179499999
  coef(4945) =       1.5549858392400000
  coef(4946) =       1.9292097309000000
  coef(4947) =      -1.3614239700499999
  coef(4948) =       3.9842037317100001
  coef(4949) =      -9.3297793369599997
  coef(4950) =       5.1306238841100003
  coef(4951) =       0.2873357844220000
  coef(4952) =     178.2750116569999932
  coef(4953) =       3.9119258743200001
  coef(4954) =      52.4805458772000009
  coef(4955) =     -11.0269375592000003
  coef(4956) =       8.5674948241299997
  coef(4957) =    -177.6072865250000063
  coef(4958) =      52.3813765148000030
  coef(4959) =      30.7199146146999986
  coef(4960) =     -16.0764859756999989
  coef(4961) =     -27.3249798782999989
  coef(4962) =      31.5963971125000000
  coef(4963) =     -32.3662372376999983
  coef(4964) =      -0.3958525723660000
  coef(4965) =       3.5957429436399999
  coef(4966) =       2.2604373556200001
  coef(4967) =      -0.0015238554921380
  coef(4968) =       0.3427690661070000
  coef(4969) =     -37.7537631579999982
  coef(4970) =       1.6864257737399999
  coef(4971) =     -19.9766508565999992
  coef(4972) =      -1.1355093513000001
  coef(4973) =       0.2449718504490000
  coef(4974) =      -0.8275435441650000
  coef(4975) =       1.7971359280600001
  coef(4976) =       1.1685634643400000
  coef(4977) =     -32.4724220992999975
  coef(4978) =      -3.5588516673599999
  coef(4979) =       4.6211227136500002
  coef(4980) =      -0.3857762392970000
  coef(4981) =       0.1106047068360000
  coef(4982) =       3.8049133453600001
  coef(4983) =      -6.1470701102499996
  coef(4984) =     -52.8257319558999967
  coef(4985) =      -6.3187385627700001
  coef(4986) =      36.5175393951999965
  coef(4987) =     -19.9429820370000002
  coef(4988) =     -16.6044853888000006
  coef(4989) =     -14.0697741184999998
  coef(4990) =      -6.0886594097600000
  coef(4991) =       2.3639106709500002
  coef(4992) =      -1.1274190647800000
  coef(4993) =      -4.2146561774100002
  coef(4994) =      -1.3780266778700001
  coef(4995) =       0.6057566476810000
  coef(4996) =       1.6286297408799999
  coef(4997) =       6.0028512681699997
  coef(4998) =       1.2364963778600000
  coef(4999) =       0.1355612562450000
  coef(5000) =      -2.7930701210900000
  coef(5001) =      18.3143451733000013
  coef(5002) =       0.9638896944050001
  coef(5003) =       1.6603141698399999
  coef(5004) =      -7.0800003370500004
  coef(5005) =      -3.3747232800300000
  coef(5006) =      -2.0623925933599998
  coef(5007) =     -16.1733952694999985
  coef(5008) =     -10.3873762008000003
  coef(5009) =     -16.6525932226999984
  coef(5010) =      -1.4121354985500001
  coef(5011) =      51.7321298820000024
  coef(5012) =    -135.2259917890000054
  coef(5013) =      -1.5785305714900000
  coef(5014) =       3.6958207937899998
  coef(5015) =      35.0840773798000001
  coef(5016) =     -56.2211733342000031
  coef(5017) =     111.5660812259999943
  coef(5018) =      13.1655776579000001
  coef(5019) =      71.2095202901000022
  coef(5020) =    -189.5571439279999879
  coef(5021) =     -77.5817044412000030
  coef(5022) =     -60.2255849411999975
  coef(5023) =      58.2797695877000024
  coef(5024) =      38.2871004894999984
  coef(5025) =      33.3335755299000027
  coef(5026) =      70.6489723466000044
  coef(5027) =     -98.3914122208999942
  coef(5028) =      54.9139936693999999
  coef(5029) =     -12.3595968537999994
  coef(5030) =    -185.7608562860000063
  coef(5031) =     -86.4073063164999979
  coef(5032) =    -379.3097671659999719
  coef(5033) =      60.0885702188000010
  coef(5034) =    -133.5094824519999861
  coef(5035) =     175.6295726239999908
  coef(5036) =     166.1578618720000122
  coef(5037) =     190.2137290649999954
  coef(5038) =    -150.9238415309999937
  coef(5039) =     -16.4076596268999992
  coef(5040) =      -8.2536789929300003
  coef(5041) =     -51.4420875085000020
  coef(5042) =     207.1574229169999910
  coef(5043) =     -58.2751982736999992
  coef(5044) =      21.6351453434999996
  coef(5045) =      18.7627300042000016
  coef(5046) =    -321.1621697909999966
  coef(5047) =     -68.6373467091000009
  coef(5048) =       7.6355586346399997
  coef(5049) =     177.8982645680000019
  coef(5050) =    -164.8610149190000129
  coef(5051) =     -90.1796026730000051
  coef(5052) =      19.1474984723999988
  coef(5053) =    -116.4986355319999944
  coef(5054) =      28.4381875481999984
  coef(5055) =     -21.1244550549999985
  coef(5056) =      44.3437308861999995
  coef(5057) =     172.5341837800000064
  coef(5058) =     -42.2642546569000004
  coef(5059) =      -9.2468779071600000
  coef(5060) =      14.2155349358999992
  coef(5061) =    -139.3589728240000056
  coef(5062) =    -121.1133951730000007
  coef(5063) =      -2.4473253662499999
  coef(5064) =     109.4324767479999991
  coef(5065) =    -563.5860872189999782
  coef(5066) =    -292.4625074769999742
  coef(5067) =     174.8260383139999874
  coef(5068) =    -151.0147848530000090
  coef(5069) =      13.0540658892000003
  coef(5070) =       5.0738152223200004
  coef(5071) =     -21.1878440437000002
  coef(5072) =      24.0539398658999986
  coef(5073) =     -24.5085182732000000
  coef(5074) =     -25.3533060554999992
  coef(5075) =       0.0872820546389100
  coef(5076) =     -32.5414928189999983
  coef(5077) =    -103.6301788130000006
  coef(5078) =     -26.0946496506999992
  coef(5079) =     -40.6348070813000035
  coef(5080) =     -87.6552315749000002
  coef(5081) =    -237.9073171050000042
  coef(5082) =     -25.2815223766999999
  coef(5083) =     -38.4066859153999971
  coef(5084) =     -14.0170053162000006
  coef(5085) =     -47.3818598464999994
  coef(5086) =     -56.0043266439000007
  coef(5087) =    -109.1715888890000059
  coef(5088) =      -2.9673458808199999
  coef(5089) =     -68.5563402954000054
  coef(5090) =     -90.5524225835000038
  coef(5091) =    -212.5835436739999977
  coef(5092) =      13.0187266322999999
  coef(5093) =     -36.0251325958000024
  coef(5094) =      46.5476412898999996
  coef(5095) =     142.8227970890000051
  coef(5096) =       7.9827464565200001
  coef(5097) =    -324.1494454950000090
  coef(5098) =    -242.3996781959999964
  coef(5099) =    -565.8172074950000479
  coef(5100) =    -328.5125242500000127
  coef(5101) =     176.9673905429999934
  coef(5102) =      72.9796421265999982
  coef(5103) =     -16.1154603698999992
  coef(5104) =      -8.0058031306000004
  coef(5105) =      -2.9455206405599998
  coef(5106) =       0.7068867126890001
  coef(5107) =     -11.0485401216000003
  coef(5108) =      -3.9308339321100001
  coef(5109) =      -3.2934651336599998
  coef(5110) =       4.0838691794899997
  coef(5111) =      -5.6017550220599999
  coef(5112) =      18.3103485502000005
  coef(5113) =      -1.6679296219999999
  coef(5114) =      46.7784372269999977
  coef(5115) =      28.2618815651999995
  coef(5116) =      -6.7865504056799999
  coef(5117) =      22.4628182760000001
  coef(5118) =      21.3885445865000001
  coef(5119) =     -22.7602262282000005
  coef(5120) =      13.7007125250000001
  coef(5121) =     -13.8927706562999997
  coef(5122) =      62.6155337957000029
  coef(5123) =      24.2102068244999984
  coef(5124) =      23.8398673587999994
  coef(5125) =      -0.9749045170130000
  coef(5126) =       9.7943610907499998
  coef(5127) =       4.8864365086800001
  coef(5128) =      -0.2410494490370000
  coef(5129) =      47.5215348531000004
  coef(5130) =      42.8259557446000017
  coef(5131) =      46.1308229200000000
  coef(5132) =       3.8929752791299999
  coef(5133) =      53.4660270358999981
  coef(5134) =      86.8594118389000016
  coef(5135) =     -10.8371263150000008
  coef(5136) =     -15.4817731663000000
  coef(5137) =       4.2571921156599997
  coef(5138) =      15.0890086936000003
  coef(5139) =       5.1438271878400004
  coef(5140) =      56.2982525458999987
  coef(5141) =      -8.4152973943300005
  coef(5142) =      60.5804500397999988
  coef(5143) =      39.3283802147999992
  coef(5144) =      -3.3498254792600002
  coef(5145) =      40.4309226828000021
  coef(5146) =       3.1249909037000001
  coef(5147) =      24.5187190052999995
  coef(5148) =       1.4684852302700000
  coef(5149) =      32.5662453815999982
  coef(5150) =       2.2881688090600001
  coef(5151) =      43.5776701547000016
  coef(5152) =      -4.5220620123600002
  coef(5153) =      -4.2246390872199999
  coef(5154) =      15.1675667365999995
  coef(5155) =       4.4342324169099996
  coef(5156) =      43.8396138531999995
  coef(5157) =      77.7531952882000041
  coef(5158) =      -5.3579150205900001
  coef(5159) =      -4.8152240979599998
  coef(5160) =       5.9065978661400003
  coef(5161) =       9.1562213465099997
  coef(5162) =       7.4588084511100003
  coef(5163) =     -11.6905040630000006
  coef(5164) =      -3.1714979990100001
  coef(5165) =      -2.6108926677099999
  coef(5166) =       9.3950938858999997
  coef(5167) =       3.6882782269400001
  coef(5168) =      -0.4500250098010000
  coef(5169) =       8.6514563399199993
  coef(5170) =      -5.8645398890199996
  coef(5171) =      99.9971234300000020
  coef(5172) =      37.0704989735000012
  coef(5173) =      -7.4285443676299998
  coef(5174) =      -8.2162707192000006
  coef(5175) =      32.9679968498000022
  coef(5176) =     -15.3559023851000003
  coef(5177) =     -24.5917921210000010
  coef(5178) =      21.7182302892999992
  coef(5179) =      -6.8059548539400003
  coef(5180) =     -10.1481825231999991
  coef(5181) =     -11.0863065880999994
  coef(5182) =      15.9217580688999991
  coef(5183) =     -12.4070706304999998
  coef(5184) =       0.2110780185110000
  coef(5185) =      10.1787721267000002
  coef(5186) =      14.7299775273000009
  coef(5187) =       8.0676910959699999
  coef(5188) =      -4.9407258530500000
  coef(5189) =      39.2554529052000021
  coef(5190) =      42.3639666186000028
  coef(5191) =      -1.5107180210200000
  coef(5192) =     -15.4050440850000001
  coef(5193) =      11.1191754129000007
  coef(5194) =      -3.2361058330099999
  coef(5195) =       6.5482126885499996
  coef(5196) =      16.7397784334000015
  coef(5197) =       7.8260719550299997
  coef(5198) =       4.5636160528999996
  coef(5199) =       7.1177668094700000
  coef(5200) =     -24.8868673377999983
  coef(5201) =      24.0769477666999983
  coef(5202) =      45.5265217516000007
  coef(5203) =      -3.9492569416099998
  coef(5204) =      -2.7837007371000002
  coef(5205) =       6.9406187397699997
  coef(5206) =      36.2381059528999998
  coef(5207) =      13.1381627401000003
  coef(5208) =      -1.2613309257500001
  coef(5209) =     -46.3818253199999972
  coef(5210) =     303.6918071509999777
  coef(5211) =     125.8969871639999951
  coef(5212) =     136.1557712139999978
  coef(5213) =    -165.8764517010000077
  coef(5214) =     240.8795756490000031
  coef(5215) =     162.6289736129999994
  coef(5216) =     422.6677921709999737
  coef(5217) =     -40.4454509217000009
  coef(5218) =   -1298.8424866699999711
  coef(5219) =     462.3232834629999957
  coef(5220) =    -609.2258437289999620
  coef(5221) =    -150.0151559829999997
  coef(5222) =     -55.7700437327000031
  coef(5223) =      30.7071949939000000
  coef(5224) =      -0.8283266801350000
  coef(5225) =     -11.6516173436999999
  coef(5226) =     -56.3631668035000004
  coef(5227) =      55.7657178721999998
  coef(5228) =     343.6902845490000118
  coef(5229) =      -0.6444512371100000
  coef(5230) =       0.6912988737600000
  coef(5231) =       2.4200976278900002
  coef(5232) =      -7.2486839825800002
  coef(5233) =       8.0729439813599999
  coef(5234) =      36.8842507175000023
  coef(5235) =     -59.3531239076000006
  coef(5236) =     -61.4736241045999989
  coef(5237) =     158.9243638730000043
  coef(5238) =      50.5178777874999980
  coef(5239) =     -54.9032984959999979
  coef(5240) =    -117.5136289590000018
  coef(5241) =     -13.8369708594999992
  coef(5242) =     -57.8709918341999980
  coef(5243) =     -90.7823029947000038
  coef(5244) =     -12.5277988459999996
  coef(5245) =    -113.9987859350000008
  coef(5246) =     -19.3293804157999993
  coef(5247) =    -243.1038882770000100
  coef(5248) =     -30.6251635602999990
  coef(5249) =     -34.6268439481000030
  coef(5250) =    -223.1650405260000127
  coef(5251) =    -102.9472264369999976
  coef(5252) =     -18.6398573532000000
  coef(5253) =      -1.3428866131799999
  coef(5254) =     -39.7590510826000028
  coef(5255) =     -53.0737973728000014
  coef(5256) =      -4.0626417074200001
  coef(5257) =     -53.1359355125999997
  coef(5258) =      62.4632626885999969
  coef(5259) =      10.3152035644000009
  coef(5260) =     -13.5948449616999998
  coef(5261) =      50.0782251397000024
  coef(5262) =      16.9865045563000017
  coef(5263) =      12.0229078556999998
  coef(5264) =    -113.2031061210000047
  coef(5265) =      -8.2944218544300004
  coef(5266) =     -60.4330057215000025
  coef(5267) =     -82.2346669742999978
  coef(5268) =    -274.0722328200000106
  coef(5269) =     729.3738057939999635
  coef(5270) =    -319.5213123190000033
  coef(5271) =    -140.5463100560000100
  coef(5272) =     171.9028310999999860
  coef(5273) =   -1540.1353723199999877
  coef(5274) =    1517.8404622499999732
  coef(5275) =    1061.9388954700000340
  coef(5276) =    -356.5015118769999845
  coef(5277) =    3402.8568828600000415
  coef(5278) =     -62.1923407534999981
  coef(5279) =     -91.1384640814000022
  coef(5280) =   -1612.2164663999999448
  coef(5281) =     -36.6876378915000032
  coef(5282) =      81.0392497471000013
  coef(5283) =    -349.6379885889999741
  coef(5284) =    -379.3412639630000172
  coef(5285) =     550.2441022059999796
  coef(5286) =    -114.9194152129999935
  coef(5287) =     -48.6813929663999971
  coef(5288) =     -41.9448921625999986
  coef(5289) =     -22.7937272399000008
  coef(5290) =     -11.6357028301999996
  coef(5291) =     152.8785775549999926
  coef(5292) =     -83.3611870185000043
  coef(5293) =     423.0644253890000073
  coef(5294) =    -797.9670903759999874
  coef(5295) =    -150.4541959469999881
  coef(5296) =    -211.7376242059999925
  coef(5297) =     382.8284262610000042
  coef(5298) =      98.7205510409999931
  coef(5299) =    -124.6248465569999979
  coef(5300) =     -24.9328619180999986
  coef(5301) =    -233.9399148560000015
  coef(5302) =    -104.6091563829999984
  coef(5303) =      83.8178009011000000
  coef(5304) =    -462.6788490430000138
  coef(5305) =     -62.2513108703999976
  coef(5306) =     119.3139745990000051
  coef(5307) =     465.0691942490000201
  coef(5308) =     254.3792020710000088
  coef(5309) =      73.0454599648000027
  coef(5310) =      71.3512585578999960
  coef(5311) =       0.0533223198506500
  coef(5312) =     117.5732767949999982
  coef(5313) =      17.3925939187999994
  coef(5314) =     -79.5840399719999994
  coef(5315) =     942.4140312509999831
  coef(5316) =    -170.3335385899999892
  coef(5317) =     -65.2395566491999972
  coef(5318) =     337.1617422159999933
  coef(5319) =     215.8533291999999904
  coef(5320) =     117.1525493850000004
  coef(5321) =    -156.9413277609999966
  coef(5322) =      61.7661688726999998
  coef(5323) =    -141.6444680769999991
  coef(5324) =     996.0440177980000271
  coef(5325) =    1207.1938009000000420
  coef(5326) =    -497.6593039210000029
  coef(5327) =   -1559.8468721199999436
  coef(5328) =      87.2231632334000011
  coef(5329) =    -387.1287823000000117
  coef(5330) =   -2449.6920311300000321
  coef(5331) =     152.1354192119999880
  coef(5332) =     550.0432141459999684
  coef(5333) =     -10.5846918489000004
  coef(5334) =     174.9864081839999983
  coef(5335) =    -517.2664768050000248
  coef(5336) =     -29.2141566702000013
  coef(5337) =    1148.8134943200000180
  coef(5338) =       5.6901111109000002
  coef(5339) =     -19.3276883662999985
  coef(5340) =      -0.8784381635750000
  coef(5341) =      -4.0186549444699997
  coef(5342) =      30.2253438006000010
  coef(5343) =     142.6940617509999925
  coef(5344) =    -197.5671189280000135
  coef(5345) =     155.7452947200000040
  coef(5346) =    -169.4872326189999967
  coef(5347) =     -29.5955333602000010
  coef(5348) =     -15.5943667670000004
  coef(5349) =      -9.1390914704899995
  coef(5350) =     179.0744726290000131
  coef(5351) =      79.4971226987999984
  coef(5352) =    -227.5658435779999991
  coef(5353) =     280.9311229549999780
  coef(5354) =    -397.3053996090000055
  coef(5355) =      12.7020117176999996
  coef(5356) =    -191.5635745630000031
  coef(5357) =    -177.4425477389999912
  coef(5358) =     -11.6634846621000001
  coef(5359) =     264.6523731789999943
  coef(5360) =     -16.8900118391000014
  coef(5361) =     -18.0810647594000002
  coef(5362) =     -97.4062682010000032
  coef(5363) =     -40.2619178055999996
  coef(5364) =    -153.9213637400000039
  coef(5365) =     -94.6406284293999960
  coef(5366) =      11.7512243126999998
  coef(5367) =     167.3794546720000085
  coef(5368) =     -14.0180414895999998
  coef(5369) =      77.3707274342999938
  coef(5370) =    -265.3637632549999807
  coef(5371) =    -158.0573970270000075
  coef(5372) =    -138.0141215349999868
  coef(5373) =     -39.5682435287000018
  coef(5374) =     166.2845347479999987
  coef(5375) =     172.3081817210000111
  coef(5376) =     432.6180820270000140
  coef(5377) =    -546.3358370500000092
  coef(5378) =     941.0061735770000269
  coef(5379) =     132.5294299180000053
  coef(5380) =     830.5888621530000364
  coef(5381) =     -51.5838312484999975
  coef(5382) =    -272.0992617979999864
  coef(5383) =    -233.5454674529999863
  coef(5384) =      96.6423216295000032
  coef(5385) =    5569.4457004699997924
  coef(5386) =    -196.0651286439999978
  coef(5387) =     -46.3315010827000009
  coef(5388) =       6.0337308012799999
  coef(5389) =    -124.4312565160000048
  coef(5390) =     -73.6042806455999994
  coef(5391) =      18.2984483372000000
  coef(5392) =    -196.8705498369999987
  coef(5393) =    -245.4437614050000036
  coef(5394) =     177.2044792899999948
  coef(5395) =      72.6101482040999997
  coef(5396) =    -147.2919245260000025
  coef(5397) =    -341.5815249670000071
  coef(5398) =      79.9451150974999933
  coef(5399) =       4.2708837400900004
  coef(5400) =    -169.2753755569999896
  coef(5401) =      38.9897698127999988
  coef(5402) =    -791.8790923330000169
  coef(5403) =    -541.9518403509999871
  coef(5404) =   -1329.5689712999999301
  coef(5405) =    -661.1738350800000035
  coef(5406) =    -716.6801068989999521
  coef(5407) =    -585.1737947530000383
  coef(5408) =    -400.0890433609999945
  coef(5409) =    -190.8293607500000064
  coef(5410) =      17.6751959710999991
  coef(5411) =       9.7561219598799997
  coef(5412) =    -428.5227598920000105
  coef(5413) =    -648.1578803280000329
  coef(5414) =     -96.3545347241000059
  coef(5415) =      11.3983420078000002
  coef(5416) =      50.6970379186999978
  coef(5417) =    -130.8202007079999873
  coef(5418) =      65.7376962533000011
  coef(5419) =      72.4511928584999936
  coef(5420) =     103.8623579030000030
  coef(5421) =    -342.4242524690000096
  coef(5422) =     379.8552038249999896
  coef(5423) =    -417.8292879200000129
  coef(5424) =      20.6423582462999988
  coef(5425) =    -255.0291935410000121
  coef(5426) =      23.4612689930000009
  coef(5427) =      -9.1386450986599996
  coef(5428) =     212.1965881589999867
  coef(5429) =     -18.7194234530000010
  coef(5430) =    2723.9879340000002230
  coef(5431) =     113.6763733119999955
  coef(5432) =      44.7539062259000033
  coef(5433) =      47.2904449159999984
  coef(5434) =      35.4621170110999984
  coef(5435) =      21.4230095493000015
  coef(5436) =     116.3260780760000017
  coef(5437) =    -212.7457653890000131
  coef(5438) =      80.8042876882000058
  coef(5439) =     -62.5489655679999998
  coef(5440) =     -26.1132627701000004
  coef(5441) =    -194.5034725820000006
  coef(5442) =     181.4333475910000004
  coef(5443) =     337.0230787140000075
  coef(5444) =      75.0966841369999969
  coef(5445) =     -40.4181307752999999
  coef(5446) =     -60.6738593386000034
  coef(5447) =    -539.3007529840000416
  coef(5448) =    -313.8291261989999725
  coef(5449) =    -760.0185567360000505
  coef(5450) =     -46.1146844482999967
  coef(5451) =    -111.0686196529999989
  coef(5452) =     101.1100284820000041
  coef(5453) =      39.8724072438999997
  coef(5454) =      26.1313608200000012
  coef(5455) =      84.9175436915000006
  coef(5456) =      25.9215792795999995
  coef(5457) =      17.5369389126000002
  coef(5458) =      78.8017177594000060
  coef(5459) =    -264.3331603109999719
  coef(5460) =      14.6438756453999996
  coef(5461) =      -6.5378128751500002
  coef(5462) =    -137.2348503000000051
  coef(5463) =     165.9695072240000115
  coef(5464) =      -0.1961620004970000
  coef(5465) =      59.3246859987999997
  coef(5466) =    -494.6631199529999776
  coef(5467) =    -315.2034431150000273
  coef(5468) =    -322.2092239319999862
  coef(5469) =      -0.7765446180590000
  coef(5470) =     -15.7060021908999996
  coef(5471) =      61.8522134709999989
  coef(5472) =       1.2947430266600000
  coef(5473) =      -1.0352538913300000
  coef(5474) =     -12.4930283817000003
  coef(5475) =      69.8615147193000041
  coef(5476) =     -57.8328393051999967
  coef(5477) =     -85.9209363080999964
  coef(5478) =     -41.2717316275999977
  coef(5479) =      48.8061507793999994
  coef(5480) =     -65.2375419719000007
  coef(5481) =    -121.6762027059999980
  coef(5482) =     -42.8221212107999989
  coef(5483) =      -3.1986809025000000
  coef(5484) =     -12.3617116439999997
  coef(5485) =    -169.7209862290000046
  coef(5486) =     -38.2629536087000020
  coef(5487) =     -12.0045578812000002
  coef(5488) =      10.8067264066999993
  coef(5489) =      70.0136825332999990
  coef(5490) =      57.3730628659000033
  coef(5491) =    -130.7471792970000024
  coef(5492) =     -77.7369816533000062
  coef(5493) =    -297.4730126910000081
  coef(5494) =       0.4637225405460000
  coef(5495) =     422.9244799780000221
  coef(5496) =      40.4841655074000002
  coef(5497) =     -70.9284370752999962
  coef(5498) =     -26.5565776135000000
  coef(5499) =     388.8336938099999998
  coef(5500) =       7.0447118072799997
  coef(5501) =      16.0258710857999986
  coef(5502) =      76.0903561893999978
  coef(5503) =     -35.5917621775000015
  coef(5504) =    -147.9750923370000066
  coef(5505) =      -9.5226037467700007
  coef(5506) =     108.9025413829999991
  coef(5507) =     144.6435800319999885
  coef(5508) =    -326.4427595230000065
  coef(5509) =     -36.6765290075000010
  coef(5510) =     -68.1968206731000066
  coef(5511) =      64.1626798924999946
  coef(5512) =    -413.9453102900000090
  coef(5513) =     404.3962598859999957
  coef(5514) =      85.6685948454999959
  coef(5515) =       2.4540688419599999
  coef(5516) =      -9.7311524429199991
  coef(5517) =    1363.5679278699999486
  coef(5518) =      11.8099268089000002
  coef(5519) =      -8.0615998717099995
  coef(5520) =      -2.0276554497900001
  coef(5521) =     -77.9791738524999971
  coef(5522) =      -9.8116927530500000
  coef(5523) =       5.1000577423099998
  coef(5524) =     -84.0164701847999993
  coef(5525) =     -34.4654278041999973
  coef(5526) =    -213.4046845240000039
  coef(5527) =      20.8900959925999992
  coef(5528) =    -101.6149675629999933
  coef(5529) =       2.8444022630200001
  coef(5530) =      73.6325137240999936
  coef(5531) =     -12.4439040883000001
  coef(5532) =    -178.9997960530000114
  coef(5533) =    -168.2555049469999915
  coef(5534) =      21.2941001611999994
  coef(5535) =      69.4746914339000057
  coef(5536) =      24.0733038442999998
  coef(5537) =    -129.2503897960000074
  coef(5538) =      12.3340083930999995
  coef(5539) =    -114.3464980170000018
  coef(5540) =     -37.5871376444999967
  coef(5541) =       4.8043920825299997
  coef(5542) =    -123.8800904269999990
  coef(5543) =      99.3634348343999960
  coef(5544) =       1.6188596131699999
  coef(5545) =    -392.3546146969999882
  coef(5546) =     -30.0769979676000005
  coef(5547) =    -571.9372680559999935
  coef(5548) =     150.8943594300000086
  coef(5549) =     154.4268686089999960
  coef(5550) =     283.0704902240000251
  coef(5551) =    -842.1517712549999715
  coef(5552) =     100.7667887789999952
  coef(5553) =    -388.7006201500000202
  coef(5554) =     -50.1539856317999977
  coef(5555) =    -444.7578840310000032
  coef(5556) =      63.6411727647999967
  coef(5557) =     472.1961728709999875
  coef(5558) =     326.7316643429999772
  coef(5559) =    -662.1363319890000412
  coef(5560) =     -91.9706334977999944
  coef(5561) =    -218.2151468710000017
  coef(5562) =    -268.9573853769999801
  coef(5563) =      65.6897812683999973
  coef(5564) =      60.3561248470999985
  coef(5565) =     117.8992724849999973
  coef(5566) =    -343.8090492219999987
  coef(5567) =     -78.3355004037999976
  coef(5568) =     508.1103445279999846
  coef(5569) =     414.4734450899999842
  coef(5570) =    -105.9935677590000012
  coef(5571) =      -5.3694170570699997
  coef(5572) =      50.1481279866000023
  coef(5573) =      27.7156758723000003
  coef(5574) =    -379.8477580879999778
  coef(5575) =    -130.8569108849999907
  coef(5576) =       2.2619046429399998
  coef(5577) =     111.4156061540000024
  coef(5578) =     650.9789232279999851
  coef(5579) =    -668.6608988289999616
  coef(5580) =      58.0132324218999997
  coef(5581) =      31.3657051128999989
  coef(5582) =      -4.3634948286300004
  coef(5583) =      43.2238514914999996
  coef(5584) =     -13.0817504717999995
  coef(5585) =     -76.1193230499000038
  coef(5586) =     -74.9434839320000066
  coef(5587) =      -4.2547563393800001
  coef(5588) =      54.0116533813000004
  coef(5589) =     120.0285367980000046
  coef(5590) =    -202.5563801879999914
  coef(5591) =      79.2988286591999980
  coef(5592) =      -6.4917807459699999
  coef(5593) =     -37.3130506369000017
  coef(5594) =    -101.5409081530000037
  coef(5595) =    -194.7912399779999930
  coef(5596) =      35.5301436429000006
  coef(5597) =     -96.4172535965000037
  coef(5598) =     -49.5657465892000033
  coef(5599) =    -113.6233464759999947
  coef(5600) =   -1179.2923692399999709
  coef(5601) =      93.3355903257999984
  coef(5602) =      34.4107101867000011
  coef(5603) =     -51.8920903802999973
  coef(5604) =    -774.0415743610000163
  coef(5605) =     236.4763380930000096
  coef(5606) =     159.1114119879999862
  coef(5607) =      21.4741658895999983
  coef(5608) =     -57.8617624233000001
  coef(5609) =    -125.5664388729999956
  coef(5610) =     447.3544201199999861
  coef(5611) =     357.0265865399999825
  coef(5612) =     -64.4618161493000059
  coef(5613) =      15.9892023733999995
  coef(5614) =    -989.4786447339999995
  coef(5615) =    -564.8967843819999644
  coef(5616) =    -242.8556674069999985
  coef(5617) =      39.3747371406999989
  coef(5618) =      95.4562791599000064
  coef(5619) =     131.1180152919999955
  coef(5620) =      -7.0913078812199997
  coef(5621) =     -35.8196262139000012
  coef(5622) =    -104.0165819080000063
  coef(5623) =     -25.4167432140999985
  coef(5624) =    -101.8709063999999955
  coef(5625) =      26.7789231723000007
  coef(5626) =      13.2330834159999995
  coef(5627) =     -66.2248176811000064
  coef(5628) =    -389.1585282400000096
  coef(5629) =     218.3918796959999895
  coef(5630) =      40.6628333900999976
  coef(5631) =       5.9124015239399998
  coef(5632) =       3.8564956840800000
  coef(5633) =       1.5155833601399999
  coef(5634) =       0.5107044287010000
  coef(5635) =       1.3124011452900000
  coef(5636) =       0.1341815443390000
  coef(5637) =       4.1130695905600003
  coef(5638) =       4.3113182380000001
  coef(5639) =      -1.1137404503299999
  coef(5640) =       2.4094383018899999
  coef(5641) =      -1.2953993238999999
  coef(5642) =       0.1766277159880000
  coef(5643) =     -56.2366423934999986
  coef(5644) =      26.4864864022999988
  coef(5645) =      -3.3119159385199999
  coef(5646) =     121.1211679360000062
  coef(5647) =     -46.0698993286999965
  coef(5648) =      41.1917166065999965
  coef(5649) =      -0.7006201291180000
  coef(5650) =     102.5882609309999935
  coef(5651) =     -47.8584704493000004
  coef(5652) =      -1.2799420728799999
  coef(5653) =      30.0926673513000011
  coef(5654) =      -4.1902725455100001
  coef(5655) =     -36.1344586927000009
  coef(5656) =     -71.1491574811000049
  coef(5657) =     -29.8876993792000007
  coef(5658) =       2.2177006200000000
  coef(5659) =      82.5233580231000019
  coef(5660) =      33.4867331578000034
  coef(5661) =      22.6838703913999993
  coef(5662) =      74.9723227323000003
  coef(5663) =      10.9866331340999999
  coef(5664) =      24.9304116076000000
  coef(5665) =       7.7113964284199996
  coef(5666) =      46.3057664959999968
  coef(5667) =      17.7399099376000002
  coef(5668) =       1.8003420778400001
  coef(5669) =      -4.6272161286799998
  coef(5670) =      12.9614660803999993
  coef(5671) =       8.0668618472499993
  coef(5672) =       6.8778198053799997
  coef(5673) =     -22.0946579704000001
  coef(5674) =     -49.3311531892999966
  coef(5675) =      -0.0000000875181803
  coef(5676) =       5.4413318859800004
  coef(5677) =     342.4572673160000136
  coef(5678) =     161.1083524119999879
  coef(5679) =     125.4032988090000060
  coef(5680) =      89.8447419927999960
  coef(5681) =      99.3326566589999942
  coef(5682) =      35.4452752178000026
  coef(5683) =      14.1210863395999997
  coef(5684) =      32.7174711951000035
  coef(5685) =      14.9354165031000008
  coef(5686) =      -1.7960176122400000
  coef(5687) =     -57.6531029475000025
  coef(5688) =     -11.7165901276000000
  coef(5689) =      23.6264642934000015
  coef(5690) =     -63.9744736800000027
  coef(5691) =     -30.6746600391999991
  coef(5692) =      -1.9308031459399999
  coef(5693) =       4.5821808886900000
  coef(5694) =       8.9392954851099997
  coef(5695) =      10.8173867535999992
  coef(5696) =     389.4745905930000163
  coef(5697) =      75.0835954885999968
  coef(5698) =     247.1608487830000058
  coef(5699) =     129.3601751669999942
  coef(5700) =       5.6617876908999998
  coef(5701) =     -34.9400284617000025
  coef(5702) =       4.7207145909300001
  coef(5703) =     174.3584884169999896
  coef(5704) =      -3.1700246857300001
  coef(5705) =      21.2261675292000014
  coef(5706) =      -0.8501149327050000
  coef(5707) =       7.4819160049000004
  coef(5708) =       2.2167020872899998
  coef(5709) =      -0.2738684843420000
  coef(5710) =      -1.6350713670299999
  coef(5711) =      27.7326064475000003
  coef(5712) =      62.6549565787000020
  coef(5713) =      -3.3451892602700002
  coef(5714) =      -1.7182890325800000
  coef(5715) =     -43.0594510545000020
  coef(5716) =     -18.2565476890000014
  coef(5717) =      42.3025720504000020
  coef(5718) =      11.5048700109999995
  coef(5719) =      33.9902027904999997
  coef(5720) =      16.0359994211000014
  coef(5721) =       0.9638446180840000
  coef(5722) =      19.9066661037000010
  coef(5723) =       1.1982973919100000
  coef(5724) =       2.2031724668099999
  coef(5725) =      23.9203719590000006
  coef(5726) =      -0.1016155804850000
  coef(5727) =     -16.5457563848999989
  coef(5728) =     -74.7498077581000047
  coef(5729) =      63.6891819072999965
  coef(5730) =      92.8172878092999980
  coef(5731) =      49.0040279231999989
  coef(5732) =      37.4497285755999982
  coef(5733) =      -6.9808154945999998
  coef(5734) =      -9.5895362823700001
  coef(5735) =      42.6295478811999971
  coef(5736) =       1.7716637469900001
  coef(5737) =       4.3168782829500003
  coef(5738) =       4.8473736644300001
  coef(5739) =      -1.5434129956900000
  coef(5740) =      -4.2183384249099998
  coef(5741) =       2.0172642628899999
  coef(5742) =      39.0497475251999973
  coef(5743) =      22.7752187421999999
  coef(5744) =       0.5606956095440000
  coef(5745) =     -12.6622111396000001
  coef(5746) =     -14.4416287549000000
  coef(5747) =     -17.2423952609999986
  coef(5748) =     -84.6195550519999955
  coef(5749) =     -11.8551315443000007
  coef(5750) =     -56.1398533503000010
  coef(5751) =     -21.2474521779000014
  coef(5752) =      10.3199096655999991
  coef(5753) =       6.5018196867500002
  coef(5754) =     -25.8379551650999986
  coef(5755) =     -29.1947404978999998
  coef(5756) =       4.9337469540400001
  coef(5757) =     -27.9696227871000005
  coef(5758) =     -13.3760440980999995
  coef(5759) =     -13.2133933616999997
  coef(5760) =      -7.6744789560599997
  coef(5761) =       7.8554876629299999
  coef(5762) =     -20.8740012298999993
  coef(5763) =      10.3607821056000002
  coef(5764) =     -16.0032319491999999
  coef(5765) =     204.0942746800000123
  coef(5766) =      47.2119793459999997
  coef(5767) =      19.6921268878999989
  coef(5768) =     106.2677510169999948
  coef(5769) =      31.0365211995999992
  coef(5770) =     -14.5884390949000000
  coef(5771) =       0.8488099675290000
  coef(5772) =      -5.9462526307600001
  coef(5773) =       0.3407849904900000
  coef(5774) =      21.3492490086000011
  coef(5775) =     -58.7021906591999993
  coef(5776) =     122.5160619020000041
  coef(5777) =     -27.2622286731000010
  coef(5778) =       6.2906284859800001
  coef(5779) =     -35.7918188184000030
  coef(5780) =      32.6848622318999986
  coef(5781) =      -3.3649481696899999
  coef(5782) =      21.1299443226000001
  coef(5783) =      -0.6257097205890000
  coef(5784) =      61.0326289424999970
  coef(5785) =     -11.0975031601000005
  coef(5786) =      -7.7685499846999999
  coef(5787) =      44.5012603981000012
  coef(5788) =     -22.5128121724999986
  coef(5789) =     -20.7456181794999992
  coef(5790) =       5.5392128868299997
  coef(5791) =      -7.2018478032299997
  coef(5792) =      12.4015329648999995
  coef(5793) =      14.9897906668999994
  coef(5794) =       5.3864895303100004
  coef(5795) =     -11.1560002169000008
  coef(5796) =     -11.6227011788999999
  coef(5797) =      39.5947338212000020
  coef(5798) =      15.5402100956000009
  coef(5799) =      -0.8366080018390000
  coef(5800) =       2.3383128395099999
  coef(5801) =      10.4169041225000001
  coef(5802) =       9.4081646525200000
  coef(5803) =       3.8240625526600001
  coef(5804) =    -143.2016702950000138
  coef(5805) =    -178.2373464739999918
  coef(5806) =     -86.1033824065000033
  coef(5807) =     -49.1710242197000014
  coef(5808) =     277.8069604560000130
  coef(5809) =      56.6797850673000028
  coef(5810) =      -8.1164529422400005
  coef(5811) =       6.4674057335299997
  coef(5812) =     -65.9814024950999993
  coef(5813) =     -52.7588147598000035
  coef(5814) =    -112.1608274400000056
  coef(5815) =     -26.6051619212999988
  coef(5816) =     -28.3194442214999995
  coef(5817) =     -34.1655103650000029
  coef(5818) =     107.1558358949999956
  coef(5819) =      15.7583956961999991
  coef(5820) =      -2.9061957813800001
  coef(5821) =     -50.9257993491999983
  coef(5822) =     -21.6020762572999985
  coef(5823) =      -2.7530429823200002
  coef(5824) =      35.0317305125999994
  coef(5825) =     -24.6280290951999987
  coef(5826) =      -4.5281594860499998
  coef(5827) =      93.0373171921000051
  coef(5828) =      72.3737619651000017
  coef(5829) =       8.8902004404500001
  coef(5830) =      60.6611802226000023
  coef(5831) =      69.6320051904999957
  coef(5832) =      24.3839623613999983
  coef(5833) =       9.9683019857699993
  coef(5834) =      10.7569535242000001
  coef(5835) =      16.3943469097999994
  coef(5836) =      -5.7076938202900003
  coef(5837) =      43.3012015141000006
  coef(5838) =       4.1109965590200002
  coef(5839) =      -1.9524701470300001
  coef(5840) =      -0.2442149635670000
  coef(5841) =     -15.4929627177999993
  coef(5842) =     -38.6858159796000010
  coef(5843) =      22.2005777349000013
  coef(5844) =       8.2582666948300005
  coef(5845) =       7.6368673861199996
  coef(5846) =      26.1001274792000011
  coef(5847) =    -196.5457550200000014
  coef(5848) =     -29.7546918799999993
  coef(5849) =      97.0214859287000024
  coef(5850) =      26.6940427960000015
  coef(5851) =     -53.8476717269000034
  coef(5852) =       8.2752103949499993
  coef(5853) =      18.2451367936999986
  coef(5854) =      18.9505251781999995
  coef(5855) =      -0.0041070166340910
  coef(5856) =      -0.0092547027036380
  coef(5857) =      -0.0434646030493400
  coef(5858) =       0.2173810351840000
  coef(5859) =       0.2195908774240000
  coef(5860) =      -0.0725215606129200
  coef(5861) =      -0.3322606615700000
  coef(5862) =       0.0089971175819680
  coef(5863) =       0.2016526143180000
  coef(5864) =      -0.0787694067616700
  coef(5865) =       0.0463847165373900
  coef(5866) =       0.0116061227775100
  coef(5867) =      -0.0275759418456100
  coef(5868) =      -0.1116682290810000
  coef(5869) =       0.0518105194751500
  coef(5870) =       0.1126280491010000
  coef(5871) =      -0.3032897639560000
  coef(5872) =      -0.3229032915530000
  coef(5873) =      -0.1791940855790000
  coef(5874) =       0.0365551578390000
  coef(5875) =       0.1261231024820000
  coef(5876) =       0.0049662179218530
  coef(5877) =       0.0992738451969300
  coef(5878) =      -0.2873961898540000
  coef(5879) =       0.4476639854780000
  coef(5880) =       0.4893721206640000
  coef(5881) =      -0.3692368252020000
  coef(5882) =      -0.4664914480560000
  coef(5883) =       1.8174802390400000
  coef(5884) =      -2.3466110415100001
  coef(5885) =      -2.1172135187899999
  coef(5886) =       0.4590596403110000
  coef(5887) =      -1.1880711800299999
  coef(5888) =      -0.4771471420790000
  coef(5889) =       0.1460168697690000
  coef(5890) =      -0.2198304456060000
  coef(5891) =      -0.3460047186040000
  coef(5892) =      -0.9888430199729999
  coef(5893) =       1.5074895410800000
  coef(5894) =      -1.1711977108300000
  coef(5895) =       0.0563460144993700
  coef(5896) =       2.0432481233900002
  coef(5897) =      -0.0392447142334200
  coef(5898) =      -0.0404896538294500
  coef(5899) =       0.6042257217060000
  coef(5900) =      -0.0626746007492000
  coef(5901) =      -4.4526064388500002
  coef(5902) =      -0.1992135615070000
  coef(5903) =      -0.1216812842560000
  coef(5904) =      -0.1578703357430000
  coef(5905) =      -0.1760600840200000
  coef(5906) =      -0.1899073399730000
  coef(5907) =       0.0040032189199540
  coef(5908) =       0.7315609724390000
  coef(5909) =      -0.6636728589340000
  coef(5910) =      -0.1760457345430000
  coef(5911) =      -0.1106116642520000
  coef(5912) =       0.3063335052700000
  coef(5913) =      -0.2106832644520000
  coef(5914) =      -0.0875879120206700
  coef(5915) =       0.2793524157210000
  coef(5916) =       0.2455502859150000
  coef(5917) =       0.2728988792240000
  coef(5918) =       2.0217475028999998
  coef(5919) =       0.5238720664860000
  coef(5920) =       1.9558639520900001
  coef(5921) =       0.1431806987760000
  coef(5922) =       0.3690475217260000
  coef(5923) =      -0.6489463760820000
  coef(5924) =      -0.2213578955640000
  coef(5925) =      -0.3003725442760000
  coef(5926) =      -0.0179428608333100
  coef(5927) =       0.2890373052140000
  coef(5928) =       0.2365449307850000
  coef(5929) =      -0.0449085943199300
  coef(5930) =       0.6453003057209999
  coef(5931) =      -1.1472101915999999
  coef(5932) =      -0.0847175854133300
  coef(5933) =       0.3501051556850000
  coef(5934) =      -0.1796447012490000
  coef(5935) =       0.1240054397880000
  coef(5936) =      -0.2936664227310000
  coef(5937) =       1.8874890161500000
  coef(5938) =       0.6006977058290000
  coef(5939) =       1.8419856769100000
  coef(5940) =       0.0585158388018900
  coef(5941) =      -0.8335197126350000
  coef(5942) =       1.6407808098200001
  coef(5943) =       0.3011735488710000
  coef(5944) =      -0.0759048928054000
  coef(5945) =       0.0786062521432300
  coef(5946) =      -0.2787081446810000
  coef(5947) =     -17.6450060168999983
  coef(5948) =      -7.6481519488899998
  coef(5949) =      -0.6346506871220000
  coef(5950) =       0.0523386740076200
  coef(5951) =      -1.8313078295800000
  coef(5952) =       1.2469134175300001
  coef(5953) =     -17.8215772426000001
  coef(5954) =      -7.9212687730400004
  coef(5955) =      -0.0058070228244110
  coef(5956) =       0.4001462473950000
  coef(5957) =     -13.2854279248000005
  coef(5958) =      -0.1366567873480000
  coef(5959) =       1.0739017166400000
  coef(5960) =      -0.5462156738520000
  coef(5961) =      -5.5856384320999997
  coef(5962) =      24.3007612620000018
  coef(5963) =       1.6444879530900001
  coef(5964) =       5.8033281440100000
  coef(5965) =      -8.4354802780600000
  coef(5966) =      -9.8818447400699991
  coef(5967) =     -11.5560566773000009
  coef(5968) =      18.7946626608999985
  coef(5969) =       6.5722241472000000
  coef(5970) =      -4.5012651949400002
  coef(5971) =      -2.1183437574099999
  coef(5972) =     -17.3675573049000000
  coef(5973) =     -12.5930903033000003
  coef(5974) =       3.3451706954799998
  coef(5975) =       1.1451043745800000
  coef(5976) =      -0.2646761639300000
  coef(5977) =       5.1057124441299999
  coef(5978) =      -0.4635322180990000
  coef(5979) =      -5.1030308063899996
  coef(5980) =     -13.0962814210000005
  coef(5981) =      -6.2040414899299998
  coef(5982) =      -3.8881255724099999
  coef(5983) =       7.0867467651400000
  coef(5984) =      -2.2620622590499999
  coef(5985) =      -0.0120251663940200
  coef(5986) =      -3.6242920503899998
  coef(5987) =      -3.8484056171400001
  coef(5988) =      -2.6814837591199998
  coef(5989) =      -1.2386141408700000
  coef(5990) =       1.9820605156700000
  coef(5991) =       0.4198886897290000
  coef(5992) =      -2.7969428472100000
  coef(5993) =       2.3644418928800000
  coef(5994) =      -7.2216801708000000
  coef(5995) =       4.5552806071500003
  coef(5996) =       3.3852453757799998
  coef(5997) =       5.9810674873599998
  coef(5998) =       4.8373819269700000
  coef(5999) =       0.0516540213269400
  coef(6000) =       0.3153952473320000
  coef(6001) =       1.2222392888300000
  coef(6002) =       2.1985342590700001
  coef(6003) =       2.1036642111099999
  coef(6004) =       0.3423329577910000
  coef(6005) =      -0.3373781106350000
  coef(6006) =      -0.3780307634620000
  coef(6007) =       0.2070431693170000
  coef(6008) =      -1.8164216641499999
  coef(6009) =       3.7158480839700001
  coef(6010) =      -2.2211956884399999
  coef(6011) =       0.7906497616530000
  coef(6012) =      -3.0280308794500002
  coef(6013) =      -0.5120652075760000
  coef(6014) =       0.0583478501089000
  coef(6015) =      -1.3441232032799999
  coef(6016) =      -2.4335409889399999
  coef(6017) =      12.2004563018999992
  coef(6018) =      -0.0052449229043970
  coef(6019) =       1.0301857767700000
  coef(6020) =       8.8037200631000001
  coef(6021) =      -2.3408080491800001
  coef(6022) =      -0.8514852927040000
  coef(6023) =      -1.9454715556500000
  coef(6024) =     -47.1482851028000027
  coef(6025) =       0.6371914464640001
  coef(6026) =       8.1061149485200001
  coef(6027) =      -2.0548995087700002
  coef(6028) =      13.0420938959000008
  coef(6029) =      62.5949737242999973
  coef(6030) =      14.8055778105000009
  coef(6031) =      -1.8127818059400000
  coef(6032) =       6.8325658430400003
  coef(6033) =      -0.0249762052039300
  coef(6034) =      -0.0171280538562800
  coef(6035) =      -0.1857323277810000
  coef(6036) =      -0.2205015220280000
  coef(6037) =       0.0966026378798700
  coef(6038) =       0.1255016112730000
  coef(6039) =      -0.0590988179908700
  coef(6040) =      -0.2994655423750000
  coef(6041) =      -0.1011486932930000
  coef(6042) =       0.5954633031640000
  coef(6043) =       4.1930825938099998
  coef(6044) =       0.3562038312290000
  coef(6045) =      -0.3415080582450000
  coef(6046) =       3.3126259016800002
  coef(6047) =      -1.0834789604999999
  coef(6048) =       1.1605468141399999
  coef(6049) =      -0.1730627034280000
  coef(6050) =       0.1477966304970000
  coef(6051) =       0.0942902480000400
  coef(6052) =       1.7418998554600000
  coef(6053) =       0.5840962539700000
  coef(6054) =       1.5301161314400000
  coef(6055) =       1.9843333859700001
  coef(6056) =       0.7872124662200000
  coef(6057) =       0.5803326436110000
  coef(6058) =       0.9075237045400000
  coef(6059) =       6.7848595967799996
  coef(6060) =      -2.2240200194800002
  coef(6061) =      -5.6189076056299996
  coef(6062) =       1.5289661259300000
  coef(6063) =       1.5218472492399999
  coef(6064) =       1.7595746055499999
  coef(6065) =       4.5639830325000004
  coef(6066) =       5.9333285655800001
  coef(6067) =       0.1542602405970000
  coef(6068) =      -3.9133845017500000
  coef(6069) =       0.1372353492780000
  coef(6070) =      -2.9955533616199999
  coef(6071) =      -1.3952329793500000
  coef(6072) =       0.9021291506569999
  coef(6073) =      -0.7076729252640001
  coef(6074) =      -0.3780831749520000
  coef(6075) =       2.9000010867300001
  coef(6076) =       0.7029225849880000
  coef(6077) =       0.7340941182350000
  coef(6078) =      -0.3975273068660000
  coef(6079) =      -0.2326669613800000
  coef(6080) =       0.6983190669000000
  coef(6081) =       0.2143043531470000
  coef(6082) =       1.3628081410499999
  coef(6083) =      -0.1149906538220000
  coef(6084) =      -0.1237545230380000
  coef(6085) =       0.3704308463670000
  coef(6086) =      -1.5290330308600000
  coef(6087) =      -5.7700264869900000
  coef(6088) =       0.7371056967370000
  coef(6089) =       1.3566410624900000
  coef(6090) =       0.4682870461410000
  coef(6091) =      -0.6636688610100000
  coef(6092) =      -0.4868057659490000
  coef(6093) =       0.4380339741240000
  coef(6094) =      -0.1096238897730000
  coef(6095) =       0.4136482837660000
  coef(6096) =      -2.2637309325100001
  coef(6097) =      -0.6399378268550000
  coef(6098) =      -0.6547572008580000
  coef(6099) =      -0.2456404943730000
  coef(6100) =       4.7667683152900002
  coef(6101) =       0.5759404860210000
  coef(6102) =      -2.6721101081200000
  coef(6103) =      -0.0420460312020000
  coef(6104) =       3.0425520666299999
  coef(6105) =      -1.2211872174000000
  coef(6106) =       0.7878717937100000
  coef(6107) =       0.1781372514880000
  coef(6108) =      -0.3043442919060000
  coef(6109) =      -0.3790938817020000
  coef(6110) =       0.2067059170370000
  coef(6111) =       0.3256198202480000
  coef(6112) =      -1.4878077383599999
  coef(6113) =       0.5428654458980000
  coef(6114) =      -0.1500740740610000
  coef(6115) =       9.1553223609700005
  coef(6116) =      -1.2859283886299999
  coef(6117) =      -4.9985699475600001
  coef(6118) =       1.9124615884300000
  coef(6119) =       3.3991735503900000
  coef(6120) =       3.4073903358300002
  coef(6121) =       2.8357030556799998
  coef(6122) =       4.5725419459100003
  coef(6123) =      -4.4249569024099999
  coef(6124) =      -0.5704568433950000
  coef(6125) =       3.5743628809999999
  coef(6126) =       1.5929758022300000
  coef(6127) =       0.1529344243190000
  coef(6128) =       2.5132814284400000
  coef(6129) =      -0.1384699195570000
  coef(6130) =      -0.1011440080180000
  coef(6131) =      -0.4980214006570000
  coef(6132) =      -4.7291941268600004
  coef(6133) =       0.5170186664350001
  coef(6134) =       0.4357971695640000
  coef(6135) =      -0.4294459535000000
  coef(6136) =       4.1802870935099996
  coef(6137) =      -4.1219366781700000
  coef(6138) =      -0.0533177527147400
  coef(6139) =      -5.6586688823799998
  coef(6140) =       6.6086265370800001
  coef(6141) =     -29.6097956177999997
  coef(6142) =    -100.2356688339999948
  coef(6143) =      21.7701067873999996
  coef(6144) =      47.0705573129000001
  coef(6145) =     -35.0680278955000020
  coef(6146) =     -43.5818974983999965
  coef(6147) =       3.7681235628700001
  coef(6148) =     -26.4843083314999994
  coef(6149) =     -91.8638599790999990
  coef(6150) =      82.1425170921999950
  coef(6151) =     -34.9451511223999987
  coef(6152) =       0.1985049840800000
  coef(6153) =      46.4490063323000015
  coef(6154) =      -0.4996058069060000
  coef(6155) =       0.1646046880040000
  coef(6156) =      30.6276064069000000
  coef(6157) =      -4.3917936898200001
  coef(6158) =     -75.8043736216000070
  coef(6159) =      -2.1055023696099999
  coef(6160) =      -1.0402711642200000
  coef(6161) =      -0.2458536432730000
  coef(6162) =      -1.7162746968700000
  coef(6163) =      -0.9044902899580000
  coef(6164) =       0.3789765564350000
  coef(6165) =       2.7421040274899999
  coef(6166) =      -7.1928034815800004
  coef(6167) =     -12.7754405087999992
  coef(6168) =      -2.6474159624700002
  coef(6169) =      10.5383481850000003
  coef(6170) =       3.0399345667100000
  coef(6171) =      -3.5536784863899999
  coef(6172) =      -1.5445158405199999
  coef(6173) =      -0.6099214549060000
  coef(6174) =      -0.0154509918473400
  coef(6175) =      13.6255247461000000
  coef(6176) =       0.6884064403260000
  coef(6177) =      47.9853472917999966
  coef(6178) =       7.4053022581299999
  coef(6179) =       5.2968245817800002
  coef(6180) =       8.8810021955099998
  coef(6181) =       3.6354422073700001
  coef(6182) =       3.8284566243099998
  coef(6183) =      -1.6650994536199999
  coef(6184) =      -0.0756522818080200
  coef(6185) =      10.4352454178999992
  coef(6186) =       1.1309884407699999
  coef(6187) =       2.3619255835100001
  coef(6188) =      -0.4439859533450000
  coef(6189) =      -0.5278974525880000
  coef(6190) =       0.7749985372570000
  coef(6191) =       4.0716498424600003
  coef(6192) =       4.8101621169800000
  coef(6193) =      -2.7435151156900002
  coef(6194) =      12.5457144453999998
  coef(6195) =       1.0759922237899999
  coef(6196) =       6.2681769554800004
  coef(6197) =     -59.3255184512999989
  coef(6198) =     108.9954517099999975
  coef(6199) =     -79.2595888365000008
  coef(6200) =      -0.6321145942010000
  coef(6201) =     -79.6816247672999936
  coef(6202) =     -74.8334643423000045
  coef(6203) =      70.7493671836000004
  coef(6204) =    -118.0629292189999973
  coef(6205) =    -151.6309509609999964
  coef(6206) =    -325.4857157299999812
  coef(6207) =     -78.8651273113000002
  coef(6208) =     -11.6430107403999994
  coef(6209) =      32.9735782276000009
  coef(6210) =     -67.2690233241000044
  coef(6211) =       0.7386247512310000
  coef(6212) =      10.4490847058000007
  coef(6213) =      35.4426821923999995
  coef(6214) =      23.9336219097999994
  coef(6215) =     -11.8684374205999994
  coef(6216) =       9.0219062724000008
  coef(6217) =       3.5161666373100000
  coef(6218) =       4.4235523063000004
  coef(6219) =       7.4185642110199996
  coef(6220) =       3.5743147524999999
  coef(6221) =      -2.5738259473300000
  coef(6222) =      -1.5936295086700001
  coef(6223) =      18.4131213478000006
  coef(6224) =      32.7429238172000012
  coef(6225) =       5.0365191311000004
  coef(6226) =      21.3851721690999987
  coef(6227) =     -23.2473169159999991
  coef(6228) =      12.1736788301000001
  coef(6229) =       4.8584729122499999
  coef(6230) =       4.3242143812900000
  coef(6231) =       3.7039697145099999
  coef(6232) =     -24.1830496888999988
  coef(6233) =     -13.8859469791999999
  coef(6234) =      22.6790371465000007
  coef(6235) =       7.7649117369200003
  coef(6236) =     -18.6589646402999989
  coef(6237) =       1.2032195890399999
  coef(6238) =      -8.8478400769600007
  coef(6239) =     -23.2594347821000014
  coef(6240) =       4.7495142859600001
  coef(6241) =       8.2420263024100002
  coef(6242) =     -23.1185281214999989
  coef(6243) =      -5.2811519572199996
  coef(6244) =      -0.3062138430850000
  coef(6245) =      -4.2922665789699996
  coef(6246) =      17.4588430870000018
  coef(6247) =       3.1272596156399999
  coef(6248) =     -28.3884863470999989
  coef(6249) =     -41.1859427779999976
  coef(6250) =      -4.5214967260699996
  coef(6251) =     -15.5214150156000006
  coef(6252) =      -9.5344056065699991
  coef(6253) =     -10.4904469778999996
  coef(6254) =      -7.7681326321100004
  coef(6255) =       2.1515246301500000
  coef(6256) =      20.7722680536999995
  coef(6257) =      -0.5814863824880000
  coef(6258) =     -33.3237424006999987
  coef(6259) =      25.8086760965999993
  coef(6260) =      53.0680180385000000
  coef(6261) =      -2.7457397741700000
  coef(6262) =     -48.7622627613000006
  coef(6263) =      -2.5531763185999998
  coef(6264) =      -4.8454151804199999
  coef(6265) =       5.3170934357300004
  coef(6266) =      -3.0146001449800002
  coef(6267) =     105.0288810950000027
  coef(6268) =     -12.8475411418000007
  coef(6269) =      -5.4907235152000000
  coef(6270) =      -8.7516773183499996
  coef(6271) =       0.7227621905770000
  coef(6272) =      -5.4547709298199996
  coef(6273) =      22.1709973170000012
  coef(6274) =      54.3181763625999992
  coef(6275) =       0.6280986668550000
  coef(6276) =     -11.5089688067000004
  coef(6277) =      -7.1111184240999998
  coef(6278) =      13.0080867939000004
  coef(6279) =      -0.4981372326280000
  coef(6280) =      62.2451933556000014
  coef(6281) =      21.9629796735999996
  coef(6282) =     -14.0305274679000007
  coef(6283) =       6.1450154765900002
  coef(6284) =      65.7378408615000041
  coef(6285) =      41.6356587988000015
  coef(6286) =      40.9736229438000024
  coef(6287) =      25.7490940978000005
  coef(6288) =       4.6628823517300004
  coef(6289) =     -13.3035312954999991
  coef(6290) =      10.0608060380000008
  coef(6291) =     -31.0348955884000013
  coef(6292) =      13.7199256668000000
  coef(6293) =      17.0704671786999995
  coef(6294) =      23.0419594630000013
  coef(6295) =      14.5267952668000007
  coef(6296) =      33.7662098615999966
  coef(6297) =      10.6533459021999999
  coef(6298) =      -2.6717823030400001
  coef(6299) =       6.8560658814600002
  coef(6300) =      -2.1047339450000000
  coef(6301) =      -2.6606321516200002
  coef(6302) =      -4.5597812457100000
  coef(6303) =      28.8093604548000002
  coef(6304) =      28.6328001553999982
  coef(6305) =      13.2109427823000001
  coef(6306) =     -27.6372003974999991
  coef(6307) =      42.9472388563999985
  coef(6308) =     173.7011300549999930
  coef(6309) =     -13.7580583972999992
  coef(6310) =    -116.4226219399999991
  coef(6311) =      12.6538630651999995
  coef(6312) =      40.7395302405000024
  coef(6313) =     -28.9878369677000016
  coef(6314) =      -2.4698845037999999
  coef(6315) =    -991.9523296849999952
  coef(6316) =      55.1609051920000013
  coef(6317) =      20.6714070873999987
  coef(6318) =       7.8053843513499999
  coef(6319) =      26.3487843724999991
  coef(6320) =      15.5895565296999994
  coef(6321) =      15.0383077927999995
  coef(6322) =      27.9828268487999985
  coef(6323) =      13.2210219102999993
  coef(6324) =     -11.8237846803999993
  coef(6325) =     -10.2968058755000005
  coef(6326) =      65.3101234304999991
  coef(6327) =       4.1542751120199997
  coef(6328) =      38.8612780051999991
  coef(6329) =      20.1798490900999994
  coef(6330) =      25.9131623166000011
  coef(6331) =      12.9848928689999994
  coef(6332) =     110.3828025029999935
  coef(6333) =      64.7774501258000015
  coef(6334) =     438.7589961749999929
  coef(6335) =     175.7776464190000070
  coef(6336) =     158.3569470909999950
  coef(6337) =     -28.7930327888000015
  coef(6338) =       5.7218484205899998
  coef(6339) =     -30.8090254743000003
  coef(6340) =      10.2169123005000007
  coef(6341) =      13.6010776138999994
  coef(6342) =     100.7333744160000037
  coef(6343) =     137.7993547940000099
  coef(6344) =      43.3298518925999971
  coef(6345) =      28.2725782854999999
  coef(6346) =      -2.0236183579200002
  coef(6347) =      24.5463804715999991
  coef(6348) =     -18.9220427774000015
  coef(6349) =      -2.0951782659100000
  coef(6350) =      -7.2468414224900002
  coef(6351) =     110.5994160519999951
  coef(6352) =     -35.4641246257000020
  coef(6353) =      83.8887624004999992
  coef(6354) =       0.2321875693790000
  coef(6355) =      -0.0526859592986400
  coef(6356) =      -2.1335477014799999
  coef(6357) =      -3.7612431902300001
  coef(6358) =     -12.3315213066999991
  coef(6359) =       1.9191523530500001
  coef(6360) =    -110.6129651490000043
  coef(6361) =      -9.9957873453299992
  coef(6362) =      -3.5010776245500002
  coef(6363) =      -2.2321421985700001
  coef(6364) =      -3.5300616784800001
  coef(6365) =      -2.0834802409600002
  coef(6366) =      -6.9970161075700004
  coef(6367) =      -6.1597098112699999
  coef(6368) =      10.2602182981999999
  coef(6369) =       4.7418252246700003
  coef(6370) =       3.1638188045800000
  coef(6371) =      -2.2933407347000001
  coef(6372) =       9.2012127806899997
  coef(6373) =      -5.2616109474800004
  coef(6374) =       4.3775948268300002
  coef(6375) =      13.9810280445000004
  coef(6376) =      -0.3776130335890000
  coef(6377) =      -7.4233917238500000
  coef(6378) =       6.7707870867200004
  coef(6379) =      -5.0235209220600003
  coef(6380) =     -27.2714390175999988
  coef(6381) =       5.5668577479300003
  coef(6382) =       9.9227586689600003
  coef(6383) =      -5.1472552381299996
  coef(6384) =      16.0185669403999995
  coef(6385) =       0.3683923017190000
  coef(6386) =       1.9872247511800001
  coef(6387) =     -20.9372602981000000
  coef(6388) =     -13.4610663454000008
  coef(6389) =      12.5699682981999992
  coef(6390) =      19.6001798956000002
  coef(6391) =      -1.2563394075500001
  coef(6392) =       6.7027928572400004
  coef(6393) =      11.2096438096999993
  coef(6394) =      -1.6523497044800000
  coef(6395) =       4.7031184763600002
  coef(6396) =      23.1914734592000009
  coef(6397) =      17.4007730245999994
  coef(6398) =      12.3117653261999997
  coef(6399) =      -0.0728228332011100
  coef(6400) =      -0.6278608001310000
  coef(6401) =      -0.3032795539950000
  coef(6402) =      -0.2897235849960000
  coef(6403) =       0.0149332733157500
  coef(6404) =      -0.2391015356030000
  coef(6405) =      -0.9547829714210000
  coef(6406) =      -1.5565466314100000
  coef(6407) =      10.9485208761999999
  coef(6408) =       1.5207333396400000
  coef(6409) =      -3.8423135437600000
  coef(6410) =      -0.8905158300720000
  coef(6411) =      17.6850827121999998
  coef(6412) =      -1.5282049576500001
  coef(6413) =      -1.9242758044899999
  coef(6414) =       4.6349151882999999
  coef(6415) =      -1.7363742337700001
  coef(6416) =       0.5211638691030001
  coef(6417) =       0.8661010195050000
  coef(6418) =       0.2499513012650000
  coef(6419) =      -0.6422809390649999
  coef(6420) =      11.0163118139999998
  coef(6421) =      17.0599314273999987
  coef(6422) =      -6.1768037214099998
  coef(6423) =      -5.7587090568699999
  coef(6424) =      -5.4965061593300000
  coef(6425) =     -68.3020520892000036
  coef(6426) =      14.2912577632000009
  coef(6427) =      -0.2687410070070000
  coef(6428) =      -7.9339660212199998
  coef(6429) =      -5.9463951664700003
  coef(6430) =      -6.0364151338100003
  coef(6431) =      -2.5447599351200001
  coef(6432) =      52.6376973731999982
  coef(6433) =       7.1971717205400001
  coef(6434) =       0.3726421244110000
  coef(6435) =     -12.8043970782999992
  coef(6436) =      18.5093109857000009
  coef(6437) =       2.1591959454500000
  coef(6438) =       7.0741920250700003
  coef(6439) =       6.5453755034999999
  coef(6440) =      -3.0313316077099999
  coef(6441) =      -2.2226667879300002
  coef(6442) =      -1.8463732558999999
  coef(6443) =     -22.1198427166999991
  coef(6444) =       3.3106236036799999
  coef(6445) =       0.1289212981380000
  coef(6446) =      -0.1916261101990000
  coef(6447) =     -14.3876689509000002
  coef(6448) =      -0.4562421048270000
  coef(6449) =      -0.7145995483100001
  coef(6450) =       0.0368135340717300
  coef(6451) =       2.0162330610799999
  coef(6452) =       1.8931539365000001
  coef(6453) =       0.7810847598550000
  coef(6454) =       0.8815354818340000
  coef(6455) =       0.0193934767865900
  coef(6456) =      14.2844610642000003
  coef(6457) =       2.7575493690899999
  coef(6458) =      -0.1917705638350000
  coef(6459) =      -6.5137716556200003
  coef(6460) =       0.8459553121950000
  coef(6461) =       3.4736177041000000
  coef(6462) =     -26.3443729208000015
  coef(6463) =      32.2048042039999984
  coef(6464) =      -7.5172262700500001
  coef(6465) =      -0.9284266265650000
  coef(6466) =       7.4608963940099997
  coef(6467) =     -39.5250138418999981
  coef(6468) =      -1.3340631588300000
  coef(6469) =       3.4354404547400001
  coef(6470) =       2.4443191101499999
  coef(6471) =       4.3580151873000004
  coef(6472) =     -17.5060119275000012
  coef(6473) =      17.6262657139000005
  coef(6474) =       3.6965791234199998
  coef(6475) =      -3.3157156080100001
  coef(6476) =      -1.3285552421500000
  coef(6477) =      37.5015676428999996
  coef(6478) =      18.3273035777000004
  coef(6479) =      18.0790692863000011
  coef(6480) =      -1.6244441194000001
  coef(6481) =       3.5477539952599999
  coef(6482) =      -1.4754641520600000
  coef(6483) =       6.3668573850400003
  coef(6484) =       1.1883782192000001
  coef(6485) =      40.1592977964999989
  coef(6486) =      17.3073785067999992
  coef(6487) =      29.3949662708000012
  coef(6488) =       6.8481917610299998
  coef(6489) =      35.0335391111999996
  coef(6490) =      12.1323103439000004
  coef(6491) =       4.8787670304499997
  coef(6492) =      49.3971277770000015
  coef(6493) =     -15.0595586937999997
  coef(6494) =      -8.2917440871199997
  coef(6495) =       6.4291206264399996
  coef(6496) =     -70.5285462685999960
  coef(6497) =       1.1792691039200001
  coef(6498) =     -14.8673362413000003
  coef(6499) =       2.3775527823100000
  coef(6500) =       4.1480089314299997
  coef(6501) =      -3.9964442614800002
  coef(6502) =       2.0112290462000000
  coef(6503) =      10.9873358496000009
  coef(6504) =      27.3318412542000004
  coef(6505) =      12.7376746504000007
  coef(6506) =      29.7196103612000009
  coef(6507) =     -10.0651097802000002
  coef(6508) =     -26.3738716358999987
  coef(6509) =     109.1604428929999955
  coef(6510) =     -13.4571518856000001
  coef(6511) =       1.8637894846900001
  coef(6512) =      -0.9438585224060000
  coef(6513) =      -1.0963230664600001
  coef(6514) =       3.3435184906300002
  coef(6515) =      -3.2227279700399998
  coef(6516) =       4.4330812786999996
  coef(6517) =       2.7594190149500002
  coef(6518) =      -1.5733574216199999
  coef(6519) =       5.8573446794299997
  coef(6520) =      17.9234971917000010
  coef(6521) =     -21.6186642143000007
  coef(6522) =      -1.4048713728200000
  coef(6523) =      -8.5448039386700003
  coef(6524) =       0.4114129397240000
  coef(6525) =      -2.5415848576600002
  coef(6526) =      -4.1041244408299997
  coef(6527) =       3.8032765128500001
  coef(6528) =      13.4471730340000004
  coef(6529) =       1.2519137359300001
  coef(6530) =      43.9781146313999969
  coef(6531) =      27.8887874075999989
  coef(6532) =      11.3716314085000008
  coef(6533) =       7.9065108133299997
  coef(6534) =       9.3812987680499997
  coef(6535) =      50.3888931832999987
  coef(6536) =      20.9854472783000006
  coef(6537) =       2.8770233439299999
  coef(6538) =      12.4680535013000000
  coef(6539) =       6.4560228950600003
  coef(6540) =      19.1074374441000003
  coef(6541) =      17.2888609994000007
  coef(6542) =      -1.4700270929699999
  coef(6543) =      11.9067541322999997
  coef(6544) =      12.0357607326000000
  coef(6545) =      30.8888202708000001
  coef(6546) =      32.2725084600999992
  coef(6547) =       2.9184132926399999
  coef(6548) =      -1.7965062152500000
  coef(6549) =      -8.9348746582299992
  coef(6550) =       2.4323779351299999
  coef(6551) =       4.8857195810100000
  coef(6552) =     -74.7761435712999969
  coef(6553) =       7.9688869797999997
  coef(6554) =       1.0377856461699999
  coef(6555) =     -67.9723833304000067
  coef(6556) =       5.9695884904999996
  coef(6557) =     -17.9783346827999999
  coef(6558) =      -4.9213029783400000
  coef(6559) =      -7.3818110079199997
  coef(6560) =      -0.7298490730490000
  coef(6561) =       0.3636453058330000
  coef(6562) =       0.1774538367040000
  coef(6563) =       0.0426998092703200
  coef(6564) =      -0.1535710130620000
  coef(6565) =      -0.2258400915410000
  coef(6566) =      -0.0201849233218200
  coef(6567) =       0.1684961230930000
  coef(6568) =       0.0656195162666100
  coef(6569) =       0.0289876373965600
  coef(6570) =       0.0406539987579600
  coef(6571) =      -0.0484044620587400
  coef(6572) =       0.0857360976396100
  coef(6573) =       0.7278295566050000
  coef(6574) =       0.0581981159347700
  coef(6575) =      -0.0521631995836900
  coef(6576) =      -9.6593519957300007
  coef(6577) =      -1.1201812927699999
  coef(6578) =      -5.3115680955300002
  coef(6579) =      -0.7454371162590000
  coef(6580) =      -2.1336506163100002
  coef(6581) =      -2.6727414800800000
  coef(6582) =       0.0406440255300400
  coef(6583) =      -0.8765893833380000
  coef(6584) =       0.0515354610368600
  coef(6585) =      -2.5031736158200002
  coef(6586) =       2.8801478325300001
  coef(6587) =       1.0247187541100000
  coef(6588) =       1.2055785280400000
  coef(6589) =       1.4972070293799999
  coef(6590) =       0.5700241972470000
  coef(6591) =      -0.1639312282410000
  coef(6592) =       0.0971263574288400
  coef(6593) =       0.4067323436110000
  coef(6594) =       0.2411632434140000
  coef(6595) =       0.5357515105030000
  coef(6596) =      -1.6799280609400000
  coef(6597) =      -0.2920829570020000
  coef(6598) =       0.6964857058620000
  coef(6599) =       4.7401179262199999
  coef(6600) =       1.5339545656900000
  coef(6601) =       1.4817423733099999
  coef(6602) =       1.9754741220800001
  coef(6603) =      -3.0723390690099999
  coef(6604) =      -3.0141368535000002
  coef(6605) =      -0.0000000870008208
  coef(6606) =      -0.4578689404390000
  coef(6607) =     -15.0978780384999993
  coef(6608) =     -16.1725920159000012
  coef(6609) =     -23.3501484953999991
  coef(6610) =     -14.9444771021000005
  coef(6611) =     -14.0051001432000000
  coef(6612) =      -5.1999376820599998
  coef(6613) =      -1.6986459763799999
  coef(6614) =      -6.2719720317299998
  coef(6615) =      -2.7332221231900000
  coef(6616) =      -0.3639088052040000
  coef(6617) =      -1.1224822524100000
  coef(6618) =      -1.9661920891400000
  coef(6619) =      -2.1137388336399998
  coef(6620) =       0.5249417551100000
  coef(6621) =      -4.0284610721399998
  coef(6622) =      -2.8404755053500002
  coef(6623) =      -0.0501054947284000
  coef(6624) =      -2.2410860890800000
  coef(6625) =      -0.1420430873100000
  coef(6626) =      -2.9593328162599999
  coef(6627) =       6.4534947348199996
  coef(6628) =     -16.4532794801000009
  coef(6629) =     -10.4895157846000000
  coef(6630) =      -0.1894582407870000
  coef(6631) =      -4.1880361425399997
  coef(6632) =       0.9671872965350000
  coef(6633) =      -8.6760114204299992
  coef(6634) =       0.8521747577360000
  coef(6635) =      -2.0580911739399999
  coef(6636) =      -2.3833742108000000
  coef(6637) =       0.2593203394460000
  coef(6638) =       0.0656759036995800
  coef(6639) =      -0.0380314974183000
  coef(6640) =       1.3324073366400000
  coef(6641) =      -5.0621654496900002
  coef(6642) =      -8.8464259783900001
  coef(6643) =      -0.4150933455170000
  coef(6644) =      -0.1482389994670000
  coef(6645) =       2.2955940839000002
  coef(6646) =       0.9053043324850000
  coef(6647) =       0.3288331714050000
  coef(6648) =       1.2854443166800000
  coef(6649) =      -0.1314523408240000
  coef(6650) =      -0.1029864510250000
  coef(6651) =      -0.3649039869600000
  coef(6652) =      -2.8177758409800000
  coef(6653) =       2.0231987226800001
  coef(6654) =       2.5662527765999998
  coef(6655) =       0.3219934245470000
  coef(6656) =       0.3594986842990000
  coef(6657) =      -2.8073143127300000
  coef(6658) =      -6.6860809391000000
  coef(6659) =     -10.6829793261999999
  coef(6660) =     -15.6748736427999997
  coef(6661) =      -7.8078084479900003
  coef(6662) =      -6.1521338474400000
  coef(6663) =       1.6411115204300000
  coef(6664) =      -0.0382529288040600
  coef(6665) =      -7.6327866700099998
  coef(6666) =      -0.9687726759420000
  coef(6667) =      -0.0928317279532000
  coef(6668) =      -0.8823662848540000
  coef(6669) =       0.2212684262600000
  coef(6670) =       0.9701685971900000
  coef(6671) =      -0.7755988913220000
  coef(6672) =      -5.6816171523300003
  coef(6673) =      -3.8533454670400000
  coef(6674) =      -1.3096350972399999
  coef(6675) =       0.5059785049550000
  coef(6676) =       3.1588950014499999
  coef(6677) =       3.3177043526399999
  coef(6678) =      -1.4872965368800000
  coef(6679) =      -1.9655386028699999
  coef(6680) =      -3.0552978337400001
  coef(6681) =      -4.9604327130900003
  coef(6682) =      -1.2409989771300001
  coef(6683) =      -1.7132062230100000
  coef(6684) =       0.4858520610360000
  coef(6685) =      -1.8207069258799999
  coef(6686) =       0.6196556227780000
  coef(6687) =       0.5050550615470000
  coef(6688) =      -0.3876499286310000
  coef(6689) =      -0.7635065605850000
  coef(6690) =       0.9328142701580000
  coef(6691) =      -1.9903766355700001
  coef(6692) =      -0.7570080347890000
  coef(6693) =      -1.2252421658299999
  coef(6694) =       0.0512409303932900
  coef(6695) =       5.1763266700399999
  coef(6696) =      -0.2424349586780000
  coef(6697) =      -0.4185828257320000
  coef(6698) =      -8.1074740486800003
  coef(6699) =       0.2003829288400000
  coef(6700) =       4.4350868596900002
  coef(6701) =       0.8824294278330000
  coef(6702) =       1.2488621144600001
  coef(6703) =       0.0430389608256200
  coef(6704) =      -0.0330934787946700
  coef(6705) =      -1.2256091643500000
  coef(6706) =     -26.7119366165000010
  coef(6707) =      -2.6982372028600001
  coef(6708) =       0.1441590283780000
  coef(6709) =      -2.1943529536500002
  coef(6710) =      -5.5875114826800001
  coef(6711) =      -1.5719698669400000
  coef(6712) =       2.5693133829199999
  coef(6713) =      -0.0411188565461100
  coef(6714) =      -6.6438063942800003
  coef(6715) =       0.5846559050720000
  coef(6716) =       2.2720225824100000
  coef(6717) =      -5.1501613312699996
  coef(6718) =       0.6997657677460000
  coef(6719) =       0.8281307706590000
  coef(6720) =      -8.8567807116400008
  coef(6721) =       0.9189017200130000
  coef(6722) =       1.4113245438899999
  coef(6723) =       0.7423307490060000
  coef(6724) =      -0.7182857015920000
  coef(6725) =      -0.2886813405710000
  coef(6726) =       0.2335507724100000
  coef(6727) =      -2.8404341875600001
  coef(6728) =      -1.0142450585300000
  coef(6729) =      -0.2101717731780000
  coef(6730) =       2.0238779439200001
  coef(6731) =       0.7949074311240000
  coef(6732) =       0.5884979529450000
  coef(6733) =       1.5652023705800000
  coef(6734) =      -0.1344059684260000
  coef(6735) =       2.7001986539899998
  coef(6736) =       6.8144519808300004
  coef(6737) =       1.4386572964200000
  coef(6738) =     -29.9976370979000002
  coef(6739) =     -14.4108487705999995
  coef(6740) =      -2.3787104903700000
  coef(6741) =       1.3491211078900001
  coef(6742) =       2.2399184367200000
  coef(6743) =       0.5411125280060000
  coef(6744) =       3.0645438915700001
  coef(6745) =       9.2134091353300001
  coef(6746) =       1.8465555115900001
  coef(6747) =       0.0000426936157551
  coef(6748) =      -3.3511008064099999
  coef(6749) =      -6.0719931641900002
  coef(6750) =       1.1116825951900000
  coef(6751) =       7.5381261837200002
  coef(6752) =       1.8946867046100000
  coef(6753) =      -7.5827885834400002
  coef(6754) =      -3.8578382570400001
  coef(6755) =       0.1126950025530000
  coef(6756) =       0.4772820015060000
  coef(6757) =      -9.0721769180099994
  coef(6758) =      -5.3715094722799996
  coef(6759) =      -5.6147868816899997
  coef(6760) =      -6.3135944598100000
  coef(6761) =      -3.2332971196700000
  coef(6762) =      -1.5032205080000001
  coef(6763) =      -0.6166588324710000
  coef(6764) =      -3.4439520524399998
  coef(6765) =      -1.7286941252700001
  coef(6766) =       2.0654782833200001
  coef(6767) =       8.7454487329399999
  coef(6768) =       1.0017183463699999
  coef(6769) =      -0.0512683705159400
  coef(6770) =       5.2786273262399996
  coef(6771) =      -4.2125033271500003
  coef(6772) =      16.3157220894999995
  coef(6773) =      -3.9613965632100001
  coef(6774) =      -0.0027488402297670
  coef(6775) =       1.4837095654800001
  coef(6776) =      -0.4834721345690000
  coef(6777) =      -1.1734839950000000
  coef(6778) =       1.5272849985400001
  coef(6779) =     -13.3425408411999999
  coef(6780) =      -5.8295394171300003
  coef(6781) =       9.0513528735200008
  coef(6782) =       0.7837388733780000
  coef(6783) =      -1.8630450587900000
  coef(6784) =      -3.3654778718100000
  coef(6785) =      -0.2317775651890000
  coef(6786) =       2.3948988857900000
  coef(6787) =      -0.3418278844030000
  coef(6788) =      -0.2392864717440000
  coef(6789) =       0.1489281373090000
  coef(6790) =       0.3460301134460000
  coef(6791) =       7.5442966535200000
  coef(6792) =      -8.8114179083300002
  coef(6793) =      -3.6151647951600001
  coef(6794) =       1.9988153536300000
  coef(6795) =      18.8252755616000016
  coef(6796) =       7.6968235041400002
  coef(6797) =      -8.9800646197899994
  coef(6798) =      -9.5020160956099993
  coef(6799) =      -0.9676616309490000
  coef(6800) =       1.0605217650000001
  coef(6801) =     -24.4830096240999993
  coef(6802) =      18.1355925147999990
  coef(6803) =       4.6239160424200003
  coef(6804) =      -1.9290576322999999
  coef(6805) =      -1.6065186631799999
  coef(6806) =     -49.4126246968999965
  coef(6807) =     -10.2021108330000008
  coef(6808) =     -24.8685222583000005
  coef(6809) =      27.8254096199999985
  coef(6810) =    -160.6457728350000025
  coef(6811) =    -120.1308305329999939
  coef(6812) =      20.4809639570999984
  coef(6813) =     -33.2370499140000035
  coef(6814) =     -13.8522373223000006
  coef(6815) =      -5.9801164128100002
  coef(6816) =     -22.1508447444000005
  coef(6817) =      -8.1629211757199993
  coef(6818) =     -18.5421814595000001
  coef(6819) =       0.9094687425190000
  coef(6820) =      -1.9152621016600000
  coef(6821) =      34.7032679060000007
  coef(6822) =      76.6494945378999972
  coef(6823) =      41.6707455856000024
  coef(6824) =     -18.3542848563999996
  coef(6825) =      67.0462126311000048
  coef(6826) =      34.4656910304999968
  coef(6827) =      27.7427345697000014
  coef(6828) =      72.2136867054999954
  coef(6829) =      37.3642904371000029
  coef(6830) =      -5.1881569540400001
  coef(6831) =      13.1351563106999993
  coef(6832) =     -18.4424288481999987
  coef(6833) =     -20.6832761659999989
  coef(6834) =       1.7514662636100000
  coef(6835) =       6.6356533134899998
  coef(6836) =      93.8582514917000026
  coef(6837) =      64.2707967789000065
  coef(6838) =      41.0159187898999988
  coef(6839) =      -6.9397501373399999
  coef(6840) =     188.7020863449999979
  coef(6841) =     145.1211394690000134
  coef(6842) =      23.8968549165000006
  coef(6843) =      84.3905396846000002
  coef(6844) =      50.3658956115000009
  coef(6845) =       1.3723377097100000
  coef(6846) =     -13.3039841500999998
  coef(6847) =      -4.5842022875800001
  coef(6848) =       6.2335268571200002
  coef(6849) =      -0.6201309338480000
  coef(6850) =       1.5371890242199999
  coef(6851) =      16.5957290577999999
  coef(6852) =      35.5345449837999965
  coef(6853) =       8.4635107594400001
  coef(6854) =       5.6794550190699997
  coef(6855) =      21.7196296472000014
  coef(6856) =      27.1949940949999984
  coef(6857) =      -5.0897421862099996
  coef(6858) =      45.7150504115000018
  coef(6859) =      10.5505063944999993
  coef(6860) =       8.8551921860699991
  coef(6861) =       7.0383963974899997
  coef(6862) =      26.1422650505000007
  coef(6863) =      -0.0582021913384700
  coef(6864) =       2.0405291891699999
  coef(6865) =       0.3153561862430000
  coef(6866) =      15.5006991262000007
  coef(6867) =      -4.7107755560199998
  coef(6868) =      -7.1180062450200001
  coef(6869) =     -85.2700441007000052
  coef(6870) =      29.5465018169999993
  coef(6871) =      41.1837260459999968
  coef(6872) =     -31.6684194193999993
  coef(6873) =     -94.7273978580000033
  coef(6874) =     -33.5348410359000013
  coef(6875) =      75.7915886542999999
  coef(6876) =      31.4632722901999990
  coef(6877) =       0.6500025590990000
  coef(6878) =      -0.6869485808650000
  coef(6879) =      -0.3305491405380000
  coef(6880) =      -0.0690188149899200
  coef(6881) =       0.2162070404880000
  coef(6882) =      -0.3777339438690000
  coef(6883) =      -0.0965003602301200
  coef(6884) =      -0.3190515416660000
  coef(6885) =       0.4081048101230000
  coef(6886) =      -1.3685474089500000
  coef(6887) =      -3.4638632434400001
  coef(6888) =       6.6951226511500002
  coef(6889) =      -4.3411465883700000
  coef(6890) =       2.2621295014500000
  coef(6891) =       5.3452888542999997
  coef(6892) =      -8.1612184108899992
  coef(6893) =       5.6626675409700002
  coef(6894) =      -5.2542457783699996
  coef(6895) =       1.7527148632200000
  coef(6896) =      -2.3669152453300000
  coef(6897) =      -8.0714635297299999
  coef(6898) =      -2.5593538707299999
  coef(6899) =      -2.1101718007399999
  coef(6900) =       2.9539663588099998
  coef(6901) =      -0.7013315499760000
  coef(6902) =       0.2237638346110000
  coef(6903) =      -0.1910337198910000
  coef(6904) =      19.6167987944000011
  coef(6905) =       3.1120297949600002
  coef(6906) =      -1.5259584665399999
  coef(6907) =      -0.0387271156413700
  coef(6908) =      -7.1621396824000003
  coef(6909) =      -1.7828545586100000
  coef(6910) =      15.4002052883000005
  coef(6911) =       5.1250082923000004
  coef(6912) =       1.4697532554799999
  coef(6913) =      -2.9896873152199999
  coef(6914) =      -1.4962631134700000
  coef(6915) =     -19.0778507594999986
  coef(6916) =      -8.9773366114200002
  coef(6917) =      -7.5831023298400000
  coef(6918) =      -4.0155423622099997
  coef(6919) =      -1.9254974009900001
  coef(6920) =      14.3035790953999999
  coef(6921) =       3.0177150744799999
  coef(6922) =       3.3774140255699998
  coef(6923) =      -2.2987719336599999
  coef(6924) =      -2.2505922841200001
  coef(6925) =       2.2877731192100002
  coef(6926) =      -4.0750456830299999
  coef(6927) =       0.3322460350520000
  coef(6928) =       2.5398046448899998
  coef(6929) =      -1.8281119097200000
  coef(6930) =      -1.0806907829200001
  coef(6931) =       6.0211223406199998
  coef(6932) =      -1.0959073208300001
  coef(6933) =       2.0401014320800002
  coef(6934) =       2.0294903339300001
  coef(6935) =      -1.6320889594500001
  coef(6936) =      -5.3987919774100002
  coef(6937) =      -0.5501335343390000
  coef(6938) =      -0.6350682456340000
  coef(6939) =      -2.2098089107100001
  coef(6940) =      -1.8276917836100000
  coef(6941) =     -12.2747193902999996
  coef(6942) =      -4.4142200858000002
  coef(6943) =      -2.0388777775400002
  coef(6944) =      -2.3438991871099999
  coef(6945) =       8.4481223346299998
  coef(6946) =      -9.4069667449400001
  coef(6947) =      -4.0586647767499997
  coef(6948) =      -1.2186581291900001
  coef(6949) =       4.2313347689500000
  coef(6950) =      -7.7072877942700000
  coef(6951) =      -1.9737981746100000
  coef(6952) =      -4.7544768079499997
  coef(6953) =      -1.6638978591500000
  coef(6954) =      -1.6582029480200000
  coef(6955) =      -1.2793887293599999
  coef(6956) =      -1.3782013700100000
  coef(6957) =      -2.2594532984900000
  coef(6958) =      -3.1846950592900001
  coef(6959) =      -0.6285334543010000
  coef(6960) =      26.5217966599000015
  coef(6961) =       6.2611325520800003
  coef(6962) =      -0.4656923233560000
  coef(6963) =       0.9935012838190000
  coef(6964) =     -15.5561301424000007
  coef(6965) =      -5.6061669735199997
  coef(6966) =      11.0871569438000002
  coef(6967) =       1.8932321698100001
  coef(6968) =      -1.0760568345499999
  coef(6969) =      -4.6525657338700004
  coef(6970) =      -5.8580673529900000
  coef(6971) =       2.0385875828000000
  coef(6972) =      -2.0889647737799999
  coef(6973) =       0.1729704643970000
  coef(6974) =       0.5767846168800000
  coef(6975) =      -6.9025691389399997
  coef(6976) =       9.5124065286500006
  coef(6977) =       1.7455681065299999
  coef(6978) =       1.2563670436900001
  coef(6979) =      -9.3791134334499997
  coef(6980) =     -21.4615359400000010
  coef(6981) =    -146.7946056959999908
  coef(6982) =     -11.0892988906000003
  coef(6983) =       7.9229812262600001
  coef(6984) =      15.0239096108000005
  coef(6985) =    -126.4818793270000015
  coef(6986) =      49.2987549325999979
  coef(6987) =      83.0134107299000021
  coef(6988) =     -51.2947682897999968
  coef(6989) =      51.8152470234999996
  coef(6990) =      74.8304853486000070
  coef(6991) =     -15.1884081778999995
  coef(6992) =      77.8929087196000012
  coef(6993) =      53.3093759893000012
  coef(6994) =      12.9633238104000004
  coef(6995) =    -101.2389392770000001
  coef(6996) =    -183.4643764980000071
  coef(6997) =      19.8686964364999987
  coef(6998) =       0.0685401767731300
  coef(6999) =       8.2172996765599997
  coef(7000) =    -141.1926463030000036
  coef(7001) =    -163.2012808970000037
  coef(7002) =    -122.5026481790000048
  coef(7003) =      14.1910962851000004
  coef(7004) =    -331.0275992740000106
  coef(7005) =    -193.3007994230000008
  coef(7006) =    -178.1603537999999958
  coef(7007) =     -75.8479489983999997
  coef(7008) =     -67.7204176555000004
  coef(7009) =      27.4199567271000006
  coef(7010) =     129.1036555070000134
  coef(7011) =     262.7473715529999936
  coef(7012) =     105.0462058890000066
  coef(7013) =     -13.0028677524000003
  coef(7014) =     -24.3546153895999993
  coef(7015) =      89.9390333408999965
  coef(7016) =     -64.1331678689000029
  coef(7017) =    -127.1571323809999967
  coef(7018) =     261.2510582350000163
  coef(7019) =     -38.2408156531000003
  coef(7020) =     -81.0080509959999944
  coef(7021) =    -130.4612265220000040
  coef(7022) =    -257.6088648659999762
  coef(7023) =     -72.7226538160000047
  coef(7024) =     -12.7716754400999992
  coef(7025) =      56.1082112269000035
  coef(7026) =     -55.5466118781000020
  coef(7027) =     -35.3483238182000008
  coef(7028) =       4.1947666661299996
  coef(7029) =      -8.6171614741399996
  coef(7030) =     132.3363244219999899
  coef(7031) =      73.7091406006999961
  coef(7032) =      78.5604334944000016
  coef(7033) =    -158.6417652419999911
  coef(7034) =     250.3900888260000102
  coef(7035) =     128.6530949300000088
  coef(7036) =      63.6856417289999968
  coef(7037) =     151.5921214299999917
  coef(7038) =      24.7524594688000015
  coef(7039) =     -68.8465469836000068
  coef(7040) =      31.1163321247999995
  coef(7041) =     296.0580578760000208
  coef(7042) =       0.5028006457380000
  coef(7043) =      29.0407894016999997
  coef(7044) =     108.5888990290000038
  coef(7045) =    -195.6636838990000058
  coef(7046) =      -8.4742857270900007
  coef(7047) =       4.0167561602099999
  coef(7048) =     314.6521108040000172
  coef(7049) =    -146.4146094920000110
  coef(7050) =     -91.5914914705000029
  coef(7051) =      -0.1572479432900000
  coef(7052) =     451.4644392810000113
  coef(7053) =     182.6133536199999980
  coef(7054) =     -98.4391156324000036
  coef(7055) =    -203.2425096039999914
  coef(7056) =     -38.8592584948000024
  coef(7057) =       2.8363424537199999
  coef(7058) =       1.5332725781000001
  coef(7059) =      -0.0098596703259270
  coef(7060) =      -1.1441634821300000
  coef(7061) =       2.3586771837999998
  coef(7062) =       0.5545669849800000
  coef(7063) =       1.8798039933999999
  coef(7064) =      -1.5862462168900000
  coef(7065) =       1.4033988267099999
  coef(7066) =      32.8277290734000005
  coef(7067) =     -17.9829549809999989
  coef(7068) =      14.8374552760000000
  coef(7069) =     -11.4020970273000000
  coef(7070) =     -15.9619135093000004
  coef(7071) =      19.6092789994999990
  coef(7072) =      -6.0450812650100003
  coef(7073) =       6.6182959258400000
  coef(7074) =       0.6574731873700000
  coef(7075) =      -1.2965071160699999
  coef(7076) =      87.8273687185999989
  coef(7077) =      33.8330873251000028
  coef(7078) =      40.7055895983000013
  coef(7079) =      26.2659817414000010
  coef(7080) =      20.0143559463000003
  coef(7081) =      10.9367961881000006
  coef(7082) =      16.6417247792000005
  coef(7083) =     -58.8396224103999970
  coef(7084) =      -4.9329933155800001
  coef(7085) =       4.9846667605599997
  coef(7086) =       6.8909237544200002
  coef(7087) =      21.3590328145999990
  coef(7088) =       1.2151424191200000
  coef(7089) =     -50.6139664519999997
  coef(7090) =      -9.4215790604300000
  coef(7091) =       0.2828528930450000
  coef(7092) =       1.3550504907400001
  coef(7093) =       3.5997185355000001
  coef(7094) =      34.2854859628999975
  coef(7095) =      14.3112856534000006
  coef(7096) =       9.6388683896600007
  coef(7097) =       5.4005503304199998
  coef(7098) =       1.3526441405500000
  coef(7099) =     -14.9119705947999996
  coef(7100) =      -1.3071826915800000
  coef(7101) =       5.8731921787900001
  coef(7102) =       0.9018937605560000
  coef(7103) =       3.2858749260900000
  coef(7104) =      -0.2877317167900000
  coef(7105) =      46.9347212857000002
  coef(7106) =      16.4070223322999986
  coef(7107) =       0.7251374446480000
  coef(7108) =      16.2775976078000006
  coef(7109) =      19.1186580582000012
  coef(7110) =     -22.5854049136999997
  coef(7111) =       2.8736624676100000
  coef(7112) =      -4.0583780246099996
  coef(7113) =      -4.6606100828399999
  coef(7114) =       0.8486162016800000
  coef(7115) =      10.4223066869000007
  coef(7116) =       2.0780345394699999
  coef(7117) =      -1.8108846856600000
  coef(7118) =       1.6007306182500001
  coef(7119) =      -1.4385715756699999
  coef(7120) =      20.6141905586999989
  coef(7121) =       8.4367973796199998
  coef(7122) =      -1.3626282351100001
  coef(7123) =      12.4843292393999992
  coef(7124) =     -20.9376401926000000
  coef(7125) =      46.5414862059000001
  coef(7126) =     -20.6384508597000007
  coef(7127) =      -0.5942353674750001
  coef(7128) =     -13.1663154996999996
  coef(7129) =      31.5095143749000002
  coef(7130) =      10.9166572295000002
  coef(7131) =      -1.7438562956999999
  coef(7132) =       7.3273658088099998
  coef(7133) =       5.5502623591800004
  coef(7134) =      -2.7373027628000002
  coef(7135) =       2.3186071713800001
  coef(7136) =     -12.6725160159999994
  coef(7137) =       5.8635210454499997
  coef(7138) =       2.1434455691699998
  coef(7139) =     -67.1851259114999948
  coef(7140) =      -8.4941150394200005
  coef(7141) =       5.2139347334700004
  coef(7142) =       6.5735846586799997
  coef(7143) =      74.7214523532999948
  coef(7144) =      16.2501744284999994
  coef(7145) =     -40.6888160864999975
  coef(7146) =      -1.2098874761100000
  coef(7147) =      -0.4612938769430000
  coef(7148) =      28.1074852025999995
  coef(7149) =      24.6057826693000017
  coef(7150) =      23.4948928278999993
  coef(7151) =      16.4355537871000017
  coef(7152) =      -3.0309665205999998
  coef(7153) =      -2.8344678520300000
  coef(7154) =      19.9848674517000013
  coef(7155) =     -26.6252714170999987
  coef(7156) =       1.0813025832000001
  coef(7157) =       1.5578155296900000
  coef(7158) =      13.8619601174000007
  coef(7159) =     -30.0793821498999989
  coef(7160) =    -211.9817744700000048
  coef(7161) =      81.3629085504999949
  coef(7162) =       1.0431697200300001
  coef(7163) =      -7.0540488589899999
  coef(7164) =     -14.2470185565000005
  coef(7165) =     119.1726692270000001
  coef(7166) =      39.6009118569999998
  coef(7167) =     -86.4031076589999998
  coef(7168) =     -39.6937897107999973
  coef(7169) =     -35.7466538740000033
  coef(7170) =      62.4062492325000022
  coef(7171) =     120.5675550840000056
  coef(7172) =      22.7104653769000002
  coef(7173) =      21.3650501338999987
  coef(7174) =    -231.1511662110000032
  coef(7175) =    -192.1661962329999938
  coef(7176) =      72.5389678656000001
  coef(7177) =     -34.6244004310999998
  coef(7178) =      22.1984967305999987
  coef(7179) =      25.2699830869000017
  coef(7180) =     143.7434132019999993
  coef(7181) =     109.7362487329999965
  coef(7182) =      55.7726974778000013
  coef(7183) =     433.2164513320000196
  coef(7184) =     262.6141404140000191
  coef(7185) =      43.7573569620999976
  coef(7186) =     109.6631957329999949
  coef(7187) =     104.7134268530000014
  coef(7188) =      18.0014324445000007
  coef(7189) =     114.6667944499999976
  coef(7190) =      29.5077295078999988
  coef(7191) =      47.4905297895999965
  coef(7192) =      -1.6806874699100001
  coef(7193) =     -19.6597031246000000
  coef(7194) =     -68.3143267732999959
  coef(7195) =     -31.6163693284000011
  coef(7196) =     -24.9157288693000005
  coef(7197) =     -41.2187866840000012
  coef(7198) =    -147.0171622480000053
  coef(7199) =     -45.9696199268000001
  coef(7200) =      14.4488398802999995
  coef(7201) =     -30.1616981795999983
  coef(7202) =     -32.7350286280999967
  coef(7203) =       2.6840463573800002
  coef(7204) =     185.9093853480000007
  coef(7205) =     350.1739250350000248
  coef(7206) =      -0.5736511314920000
  coef(7207) =      82.7243928854999950
  coef(7208) =      69.7184498119000011
  coef(7209) =     -76.9854910139000026
  coef(7210) =     -62.5373160820999985
  coef(7211) =      -1.0052773298800000
  coef(7212) =     -99.2657495976999940
  coef(7213) =    -178.6413964420000013
  coef(7214) =     -81.8503242655999941
  coef(7215) =     109.8164657570000031
  coef(7216) =    -474.4821625320000180
  coef(7217) =    -409.1091852220000078
  coef(7218) =    -243.6483299930000044
  coef(7219) =    -135.8427542359999904
  coef(7220) =      -5.5589661234600003
  coef(7221) =      -4.3658551042499996
  coef(7222) =      -3.9869951032099999
  coef(7223) =      -1.8808352934699999
  coef(7224) =      -2.0271556429199999
  coef(7225) =      -2.7294008881699998
  coef(7226) =      -2.4703453997999998
  coef(7227) =      -0.9098504533130000
  coef(7228) =      -4.9628396973099997
  coef(7229) =       6.2977013878900001
  coef(7230) =       0.8417167175369999
  coef(7231) =      -2.0837579514000000
  coef(7232) =      11.0280732912000001
  coef(7233) =     -18.5314049469999986
  coef(7234) =      -1.1842501975099999
  coef(7235) =      17.8483837210999994
  coef(7236) =     -11.4716141070000006
  coef(7237) =      25.2141726756000004
  coef(7238) =      -5.4082079089199997
  coef(7239) =      15.9846283678999992
  coef(7240) =      17.7666370664000013
  coef(7241) =      12.7867437891000009
  coef(7242) =      11.3798545566999998
  coef(7243) =      49.4578677147000008
  coef(7244) =       9.8102214674999999
  coef(7245) =      10.2650476038999994
  coef(7246) =       3.0144762927900000
  coef(7247) =      25.2677827286999985
  coef(7248) =      16.1764810724000014
  coef(7249) =      23.5790536067000005
  coef(7250) =     -10.4572647724000003
  coef(7251) =      27.2337525262000000
  coef(7252) =      -6.9108201549799997
  coef(7253) =      27.6677586457999993
  coef(7254) =     -13.9136511520999999
  coef(7255) =      -3.0525058211300000
  coef(7256) =      35.0130429051000007
  coef(7257) =       8.8751585585600008
  coef(7258) =      43.2402334447999976
  coef(7259) =      32.9242971393999966
  coef(7260) =      14.5372060405999992
  coef(7261) =       5.6419379156699998
  coef(7262) =      10.7132556990999994
  coef(7263) =     -38.9262482517000024
  coef(7264) =      -6.1749724429499997
  coef(7265) =     -15.6463535846999999
  coef(7266) =      12.5935850427999991
  coef(7267) =       6.6494498505199999
  coef(7268) =      -5.0717139450900000
  coef(7269) =      21.0902764613999985
  coef(7270) =      29.6907625639999999
  coef(7271) =      33.4765047496999983
  coef(7272) =       8.7892553893100001
  coef(7273) =       5.8224993356199999
  coef(7274) =      26.8763273584999993
  coef(7275) =      25.4712255988000003
  coef(7276) =       4.1766421895399999
  coef(7277) =      10.3269595676999995
  coef(7278) =       3.9425732341400002
  coef(7279) =      14.5989572819000006
  coef(7280) =       0.4732859215320000
  coef(7281) =       1.0717944230000001
  coef(7282) =       9.2562006897100009
  coef(7283) =       9.5009890979999998
  coef(7284) =      26.7051954747000018
  coef(7285) =      13.8991878387999996
  coef(7286) =       4.9767937411399998
  coef(7287) =       1.6949587358700000
  coef(7288) =      -8.9328571783000008
  coef(7289) =      17.4508269049000013
  coef(7290) =     -41.6751253735999967
  coef(7291) =       3.4168315918399998
  coef(7292) =       0.8425824299560000
  coef(7293) =       0.9924464171130000
  coef(7294) =     -11.8620434354000004
  coef(7295) =      27.0394806332999984
  coef(7296) =      -4.1126362376100003
  coef(7297) =       5.1072140836999997
  coef(7298) =      10.1366219306000005
  coef(7299) =       8.3405220171599996
  coef(7300) =     -29.8176621847999996
  coef(7301) =       2.1649651952300002
  coef(7302) =       4.3695622122800000
  coef(7303) =      -2.2436761601300002
  coef(7304) =       3.6674606896499999
  coef(7305) =      16.1257398031999983
  coef(7306) =     -16.9372654732000001
  coef(7307) =      54.0286656700999970
  coef(7308) =      -9.1462218816400007
  coef(7309) =      25.9287080068000009
  coef(7310) =     -11.8401012617999992
  coef(7311) =      26.8640870358999990
  coef(7312) =      22.5864329746999992
  coef(7313) =      -7.8256407200700000
  coef(7314) =     -18.9815617586000016
  coef(7315) =      -0.4930296627760000
  coef(7316) =      -4.4546467408100003
  coef(7317) =      -3.0247118625299998
  coef(7318) =      -0.5795354482560000
  coef(7319) =      17.9159571687000003
  coef(7320) =      14.9968539447999998
  coef(7321) =       7.6262951934299998
  coef(7322) =     -14.5597273934999993
  coef(7323) =     -76.8370224573999963
  coef(7324) =      -9.7679606975300004
  coef(7325) =     -64.4437428843999953
  coef(7326) =     -19.1029617770999991
  coef(7327) =      26.7619346677000003
  coef(7328) =     217.0944945169999869
  coef(7329) =    -171.6667553979999923
  coef(7330) =     -18.9784274438999994
  coef(7331) =    -109.4487563109999968
  coef(7332) =     341.8683622519999972
  coef(7333) =     283.5664806720000115
  coef(7334) =     433.3661109910000278
  coef(7335) =     164.0941939250000132
  coef(7336) =      12.3760244002000004
  coef(7337) =      13.5770187169999996
  coef(7338) =      94.5665026841000014
  coef(7339) =      23.3124997560000011
  coef(7340) =      69.1744203532999933
  coef(7341) =      11.1588402802999997
  coef(7342) =       9.7657562939200009
  coef(7343) =    -154.4049962729999947
  coef(7344) =     -38.3837980556000034
  coef(7345) =     -93.6688767045999953
  coef(7346) =     138.2516120040000089
  coef(7347) =    -379.1659031230000210
  coef(7348) =    -303.6899731589999760
  coef(7349) =    -421.7381272989999843
  coef(7350) =     -79.6144140438999983
  coef(7351) =      -8.0748156683300003
  coef(7352) =      80.2564645021000018
  coef(7353) =      39.4648546649999972
  coef(7354) =     251.3041736989999890
  coef(7355) =      -1.6910400681000000
  coef(7356) =     -96.7405580788000066
  coef(7357) =      47.0507385988999971
  coef(7358) =     232.3024304170000107
  coef(7359) =      51.0086387070000029
  coef(7360) =    -101.1939822650000025
  coef(7361) =      87.5384164429000009
  coef(7362) =     246.3077926260000083
  coef(7363) =     168.7000564539999914
  coef(7364) =    -257.4312223090000202
  coef(7365) =    -404.9637360159999844
  coef(7366) =    -442.1152823059999832
  coef(7367) =     624.0207311100000425
  coef(7368) =     202.3859031060000007
  coef(7369) =     -36.3243416615000001
  coef(7370) =     -10.6429230578999992
  coef(7371) =      -3.9477107940799998
  coef(7372) =      -0.5023389351770000
  coef(7373) =       2.3069333620600001
  coef(7374) =      -8.9190200562400008
  coef(7375) =      -1.5680337142800000
  coef(7376) =      -4.6070847640099997
  coef(7377) =       4.2075043730899999
  coef(7378) =     -12.5717286472000005
  coef(7379) =     -34.6391217874000006
  coef(7380) =     -15.7549546684999999
  coef(7381) =     -27.7904995544999984
  coef(7382) =      33.8745281352000021
  coef(7383) =      -3.4137901807500000
  coef(7384) =     -47.9159857173000034
  coef(7385) =      52.4025178666000002
  coef(7386) =     -34.0248655186000022
  coef(7387) =      25.8816349400000014
  coef(7388) =     -26.2161757229000010
  coef(7389) =    -104.5194040359999974
  coef(7390) =     -50.3908828125000028
  coef(7391) =     -57.0037751663999970
  coef(7392) =     -64.5025468981999950
  coef(7393) =     -32.7811645343999984
  coef(7394) =     -19.2048494229999989
  coef(7395) =     -24.3883481915999987
  coef(7396) =      -6.4015366387899997
  coef(7397) =       0.6607887341449999
  coef(7398) =      13.2823746964999998
  coef(7399) =     -18.5251298853000002
  coef(7400) =     -77.5197950745999975
  coef(7401) =     -13.6619415347000004
  coef(7402) =      32.0026227418000033
  coef(7403) =      -9.2051335394099993
  coef(7404) =       2.0909925536800000
  coef(7405) =      17.4808863947999988
  coef(7406) =     -18.0279010353000011
  coef(7407) =    -132.3315140359999873
  coef(7408) =     -66.6955090582000025
  coef(7409) =     -90.3742002003999971
  coef(7410) =     -31.2901717437999984
  coef(7411) =     -14.9612500732000004
  coef(7412) =     122.8482174170000008
  coef(7413) =      40.7365769524000001
  coef(7414) =      50.2869696459000011
  coef(7415) =     -14.8578157708000003
  coef(7416) =     -10.6826927596000001
  coef(7417) =      53.1523568209000032
  coef(7418) =     -64.5902812836000066
  coef(7419) =     -44.1621724649000029
  coef(7420) =     -14.3847326989000006
  coef(7421) =     -12.8117721095999997
  coef(7422) =     -29.1646762017999990
  coef(7423) =      22.6168875691000011
  coef(7424) =      14.9658232655999992
  coef(7425) =       3.4432576829700001
  coef(7426) =      10.8264410417000008
  coef(7427) =      -3.6228596681299998
  coef(7428) =     -32.8116601150999969
  coef(7429) =      -7.0674400524500003
  coef(7430) =       0.9699618403169999
  coef(7431) =     -15.2473337578999999
  coef(7432) =       1.2642502832300000
  coef(7433) =     -82.0290499016000041
  coef(7434) =     -49.6259071651000028
  coef(7435) =     -16.5394529689000009
  coef(7436) =     -27.0912175023000010
  coef(7437) =      -6.6956178561600002
  coef(7438) =     -63.6132114086000016
  coef(7439) =     -21.5919650338999993
  coef(7440) =      -4.7452538856700004
  coef(7441) =      -7.8662675580099997
  coef(7442) =     -32.5920404761999976
  coef(7443) =     -35.6345399689000004
  coef(7444) =     -25.3782827224999998
  coef(7445) =      -6.4912265107099998
  coef(7446) =      -3.7451957299499998
  coef(7447) =     -11.0998687813999997
  coef(7448) =     -16.5973469949999988
  coef(7449) =      -9.3996387683500000
  coef(7450) =     -29.6196603709999984
  coef(7451) =      -0.0542627883193100
  coef(7452) =      28.5205885453000008
  coef(7453) =      15.6719167682999991
  coef(7454) =      21.9658589323000015
  coef(7455) =     -12.6321733580999993
  coef(7456) =    -181.7218146129999923
  coef(7457) =     -61.0030738274000015
  coef(7458) =      11.7617824942000002
  coef(7459) =     -23.8745579875999994
  coef(7460) =      20.4984734875000001
  coef(7461) =     -60.0488445355999971
  coef(7462) =     -84.9049322566000058
  coef(7463) =     -20.6382753008000002
  coef(7464) =     -16.0004540213999995
  coef(7465) =     -24.4533152730999994
  coef(7466) =       8.0628375729300004
  coef(7467) =     -49.1697118697999969
  coef(7468) =      36.6846700022000007
  coef(7469) =      26.4094639949999994
  coef(7470) =       2.8545045876300001
  coef(7471) =      -5.4342037548700004
  coef(7472) =     -15.7450672902999997
  coef(7473) =      12.7187596908000007
  coef(7474) =     -27.8255965970999988
  coef(7475) =      -2.8938925962200002
  coef(7476) =      -5.8095819837200002
  coef(7477) =      20.8222945459999984
  coef(7478) =      -3.6118278446800001
  coef(7479) =      23.9233273070999992
  coef(7480) =     -28.6611738486000007
  coef(7481) =      62.4897832505999986
  coef(7482) =      25.4069775257999986
  coef(7483) =      78.6647811991000054
  coef(7484) =     -12.2535329367999992
  coef(7485) =      -1.2755069520700000
  coef(7486) =      42.2099327390000028
  coef(7487) =    -102.2193074169999960
  coef(7488) =     -75.6038185713999979
  coef(7489) =       1.0377493451399999
  coef(7490) =     -18.6083314259999995
  coef(7491) =     -11.1845157836000002
  coef(7492) =     175.1173716380000087
  coef(7493) =      40.5429090853999980
  coef(7494) =      22.4111700169000017
  coef(7495) =     279.7712373809999917
  coef(7496) =    -115.6739757560000044
  coef(7497) =    -128.4209940730000028
  coef(7498) =      89.9989968672000060
  coef(7499) =     310.3421573439999861
  coef(7500) =       6.8502616541199997
  coef(7501) =    -307.2076672069999859
  coef(7502) =     -98.5855923418999964
  coef(7503) =      -5.1491548179200004
  coef(7504) =       5.3031466217099998
  coef(7505) =       3.2581616030500000
  coef(7506) =       0.6690918106580001
  coef(7507) =      -0.1074939099220000
  coef(7508) =       3.2110224938799998
  coef(7509) =       0.9053116706820000
  coef(7510) =       1.1499000973700000
  coef(7511) =      -0.9672760910790000
  coef(7512) =       2.8655128713900000
  coef(7513) =     -16.6029729510000017
  coef(7514) =     -23.4194105631000014
  coef(7515) =      15.8376852598000006
  coef(7516) =      -5.6143412465000004
  coef(7517) =     -15.8758789072000006
  coef(7518) =      26.1467408285999987
  coef(7519) =     -17.4774645293000006
  coef(7520) =      14.9538590996000007
  coef(7521) =      -3.6060763268299998
  coef(7522) =       6.0510887624500000
  coef(7523) =     -58.9838829426999993
  coef(7524) =     -28.7228898384999987
  coef(7525) =     -32.7016840491999972
  coef(7526) =     -55.3060179574000017
  coef(7527) =     -20.7328432163000009
  coef(7528) =     -15.9394172923999999
  coef(7529) =     -13.4264000362000004
  coef(7530) =     -75.3436739584000037
  coef(7531) =      -6.3027436936200001
  coef(7532) =       8.4462653806199999
  coef(7533) =      -1.6020309779199999
  coef(7534) =      30.3782871378999992
  coef(7535) =       7.9502491239299999
  coef(7536) =     -40.6931612174999984
  coef(7537) =     -17.2125377282000009
  coef(7538) =      -1.7221637061999999
  coef(7539) =      12.0118348125000001
  coef(7540) =       6.5691902424500004
  coef(7541) =      70.5587187344000029
  coef(7542) =      33.7112739143000013
  coef(7543) =      26.8433352258000006
  coef(7544) =      16.3516607940999990
  coef(7545) =       7.7705526944500001
  coef(7546) =     -42.1171202235000024
  coef(7547) =      -7.7552709335100003
  coef(7548) =      -7.4080420917199996
  coef(7549) =       7.5061666822099999
  coef(7550) =       8.2021578577999996
  coef(7551) =      -5.1044875449599996
  coef(7552) =     -40.2522175667999988
  coef(7553) =     -35.7960010541000031
  coef(7554) =     -24.0399348810999989
  coef(7555) =     -14.3573002548000002
  coef(7556) =     -15.8391188919000001
  coef(7557) =     -26.5055638149999986
  coef(7558) =       6.8510031390900004
  coef(7559) =      -5.0629724693900000
  coef(7560) =      -4.2115417193500004
  coef(7561) =       4.5926228823399997
  coef(7562) =      15.9370046886000001
  coef(7563) =       1.2119943090200000
  coef(7564) =       3.8355161193799998
  coef(7565) =       8.5955129063700006
  coef(7566) =       2.5009762847700001
  coef(7567) =      47.4622099488999964
  coef(7568) =      13.0084072639000006
  coef(7569) =       6.3693521856700004
  coef(7570) =       9.5651047668500002
  coef(7571) =     -28.6286444123999999
  coef(7572) =      29.6234983951999986
  coef(7573) =       6.3399244982200003
  coef(7574) =       1.7925242886599999
  coef(7575) =     -13.6877170952000000
  coef(7576) =      18.5413520028000001
  coef(7577) =       4.5908802926499996
  coef(7578) =      13.4453395522000001
  coef(7579) =       4.5052754635700003
  coef(7580) =       4.5920096140800002
  coef(7581) =       2.2628081977900001
  coef(7582) =       2.2524238405100001
  coef(7583) =       2.9686441272700002
  coef(7584) =       3.5937198746900001
  coef(7585) =       2.0053120164300000
  coef(7586) =     -97.4435677422000026
  coef(7587) =     -16.9301364810000017
  coef(7588) =       3.5204167526800001
  coef(7589) =      -5.8134208680299997
  coef(7590) =      54.9721530981000015
  coef(7591) =      16.0179811387000015
  coef(7592) =     -30.5769577316000003
  coef(7593) =     -10.1967933160000008
  coef(7594) =       7.0095700055699997
  coef(7595) =      12.2749263042999992
  coef(7596) =      12.0372705759999992
  coef(7597) =      -1.6631727769000000
  coef(7598) =       5.8114378161999998
  coef(7599) =      -4.4885738416100001
  coef(7600) =      -1.6368645706400000
  coef(7601) =      21.2357996552999992
  coef(7602) =     -36.4259757140000033
  coef(7603) =      -2.3786741145899999
  coef(7604) =      -2.8784812455400002
  coef(7605) =       0.0902591235687500
  coef(7606) =       1.5035321705200000
  coef(7607) =       0.1070258091740000
  coef(7608) =      -0.1150704174260000
  coef(7609) =      -1.0622547631299999
  coef(7610) =       0.4885305661870000
  coef(7611) =       0.2409433796980000
  coef(7612) =       0.3180937626720000
  coef(7613) =       0.3479758738210000
  coef(7614) =       0.3433447949490000
  coef(7615) =      -0.4522765369490000
  coef(7616) =      -0.0431016742129500
  coef(7617) =       0.0492615431899700
  coef(7618) =      -0.2166898769960000
  coef(7619) =      -0.0664736538240600
  coef(7620) =      -0.0110939287254400
  coef(7621) =       0.1625906694440000
  coef(7622) =      -1.6369569015600001
  coef(7623) =      -0.8785038041900000
  coef(7624) =      -0.3776114667350000
  coef(7625) =      -0.5226418506520000
  coef(7626) =      -0.1065534192590000
  coef(7627) =       0.1072727074200000
  coef(7628) =      -0.5621024923290000
  coef(7629) =       0.0729479167861700
  coef(7630) =      -0.1932799829130000
  coef(7631) =       0.2355494515430000
  coef(7632) =       0.0331510286693000
  coef(7633) =       0.3546000640070000
  coef(7634) =      -0.2990120939080000
  coef(7635) =      -0.0790920064691300
  coef(7636) =      -0.0831961505504900
  coef(7637) =       0.0543278359618500
  coef(7638) =       0.1270506259410000
  coef(7639) =      -0.7823314053440000
  coef(7640) =      -0.2168732256650000
  coef(7641) =       0.0534150897978500
  coef(7642) =      -0.3687296300250000
  coef(7643) =       0.2475950676290000
  coef(7644) =      -0.2954790221260000
  coef(7645) =       0.1201013525340000
  coef(7646) =       0.0474741855851300
  coef(7647) =      -0.0729789765165200
  coef(7648) =       0.4121820186900000
  coef(7649) =      12.7952657926000004
  coef(7650) =      14.3904925853000005
  coef(7651) =      -0.2833533750640000
  coef(7652) =     -83.2694985267000050
  coef(7653) =      -0.0298476514731500
  coef(7654) =       0.1652127016730000
  coef(7655) =      -0.5289213659480000
  coef(7656) =      -0.6528659138550000
  coef(7657) =      -0.2748207924160000
  coef(7658) =       1.7775317096800001
  coef(7659) =       7.8173354703599998
  coef(7660) =     -15.7109225214000006
  coef(7661) =      -0.5672365146440000
  coef(7662) =      -2.5843067018800001
  coef(7663) =       1.7605724494199999
  coef(7664) =      -9.9019953475100007
  coef(7665) =       0.9504439175090000
  coef(7666) =       7.0570334586300003
  coef(7667) =       6.3338102626300001
  coef(7668) =       1.2219119370800000
  coef(7669) =      19.0343630433999991
  coef(7670) =      -2.2560301827200000
  coef(7671) =      16.7938382920000002
  coef(7672) =       4.9019231652700004
  coef(7673) =       3.9898385469900002
  coef(7674) =     -19.3068338554000007
  coef(7675) =      -1.3945707447500000
  coef(7676) =     -12.0218869348999995
  coef(7677) =      -0.1484851689540000
  coef(7678) =       3.0422759927200000
  coef(7679) =       4.2438253932299999
  coef(7680) =       0.0675781480452900
  coef(7681) =       8.8188209858899995
  coef(7682) =     -35.1540864894000009
  coef(7683) =       0.3195750854690000
  coef(7684) =       1.3943694948500001
  coef(7685) =      -6.9713837039100000
  coef(7686) =       5.7921926736499998
  coef(7687) =      -3.0887834255300000
  coef(7688) =      23.0643845722999998
  coef(7689) =      -0.2198145400570000
  coef(7690) =      12.2482295278999995
  coef(7691) =      -0.0058312981530850
  coef(7692) =      12.9290097871000000
  coef(7693) =      42.8547177100000027
  coef(7694) =       1.4053375055299999
  coef(7695) =      41.0175863738000004
  coef(7696) =       4.8639288623099999
  coef(7697) =       3.8000670095400002
  coef(7698) =       6.5561498468000003
  coef(7699) =       4.7170145749300003
  coef(7700) =       6.4246137674500003
  coef(7701) =      -6.1811111916200003
  coef(7702) =      -9.6340036883100009
  coef(7703) =      -3.7342354809299998
  coef(7704) =      -1.3358870499600000
  coef(7705) =      -1.5930693419899999
  coef(7706) =     -13.0633700608000005
  coef(7707) =      17.8424712284000009
  coef(7708) =     -54.4774124893000007
  coef(7709) =     -22.6898928763999983
  coef(7710) =     -25.5690132024000008
  coef(7711) =     -14.5824782586000001
  coef(7712) =     -15.4493367532000008
  coef(7713) =     -11.5507289735000001
  coef(7714) =     -47.5598930378999967
  coef(7715) =      -8.0557789078299997
  coef(7716) =      -8.5020722149600001
  coef(7717) =      22.8107866590999997
  coef(7718) =      12.3916995496000002
  coef(7719) =       6.0816575179400001
  coef(7720) =     -13.3879945629999995
  coef(7721) =     -10.9088864385999997
  coef(7722) =      -8.4443883026899993
  coef(7723) =      -3.9804832296699999
  coef(7724) =      -8.5121570410899992
  coef(7725) =     -22.4394096352999988
  coef(7726) =      -0.8060479394050000
  coef(7727) =     -10.0840287710999998
  coef(7728) =      10.1738182019999996
  coef(7729) =     -12.2034576340999994
  coef(7730) =       5.2054012398799996
  coef(7731) =     -13.9257507049000004
  coef(7732) =      -6.5327290688700002
  coef(7733) =     -37.0401253328000024
  coef(7734) =      -0.0914472324662400
  coef(7735) =       0.0073859157651650
  coef(7736) =      -0.0083391242788850
  coef(7737) =      -0.0517477794800800
  coef(7738) =       0.1191866982080000
  coef(7739) =      -0.1349478493910000
  coef(7740) =      -0.1739314845710000
  coef(7741) =      -0.4580716220720000
  coef(7742) =       0.3195379642550000
  coef(7743) =       0.1652783158510000
  coef(7744) =       0.2365091201320000
  coef(7745) =      -0.2031282948910000
  coef(7746) =      -0.7637738549790000
  coef(7747) =       0.4481801597650000
  coef(7748) =      -1.3643743332400000
  coef(7749) =      -5.1147583814199997
  coef(7750) =      15.6012945930000004
  coef(7751) =      -3.8877806956000001
  coef(7752) =      -0.9856492252440000
  coef(7753) =     -23.5144647931000002
  coef(7754) =      -2.8923815017200001
  coef(7755) =      -0.5790449345860000
  coef(7756) =      16.4353733021000004
  coef(7757) =      -1.4542725898200000
  coef(7758) =      -3.3621028025799999
  coef(7759) =      -1.7311378440800000
  coef(7760) =       1.5360018490700000
  coef(7761) =      13.1259265468000006
  coef(7762) =     -23.0015614090000007
  coef(7763) =      -5.0044165705900001
  coef(7764) =       3.6490043422400000
  coef(7765) =      35.8844107760999975
  coef(7766) =      23.6401558579000017
  coef(7767) =     -11.7427405504000006
  coef(7768) =     -24.5985287803999988
  coef(7769) =      -8.2249216952899999
  coef(7770) =       0.2361402886070000
  coef(7771) =       0.1229608366680000
  coef(7772) =       0.2399639683500000
  coef(7773) =      -0.4934511401570000
  coef(7774) =      -1.0851792317700000
  coef(7775) =       0.0267995904540000
  coef(7776) =       0.4357476027290000
  coef(7777) =      -0.1484193310610000
  coef(7778) =      -1.2475386677800000
  coef(7779) =      -0.8591425462390000
  coef(7780) =      -1.3351853774000000
  coef(7781) =       0.0227284809537600
  coef(7782) =      -0.6495763185430000
  coef(7783) =     -23.9406405892000009
  coef(7784) =     -44.6247310915000028
  coef(7785) =     183.8303616329999954
  coef(7786) =     763.4170729149999488
  coef(7787) = -253604.5117310000059661
  coef(7788) =    -370.9296375710000007
  coef(7789) =   -2047.7483808599999975
  coef(7790) =    6917.1074509399995804
  coef(7791) =     920.6268398360000447
  coef(7792) =    5056.2282627100003083
  coef(7793) =  641621.7240880000172183
  coef(7794) =    -178.0568139519999988
  coef(7795) =     117.2930102899999980
  coef(7796) = -210926.1382769999909215
  coef(7797) =     222.4075472179999906
  coef(7798) =     330.9058570639999743
  coef(7799) =  -44515.9098158999986481
  coef(7800) =    -184.7514931979999915
  coef(7801) =   -2515.8272309099997983
  coef(7802) =  476410.9068950000219047
  coef(7803) =      47.8210984191999984
  coef(7804) =      -1.2459379126800001
  coef(7805) =  -19157.9640271999996912
  coef(7806) =      26.9759690536999983
  coef(7807) =    2994.9353873700001714
  coef(7808) = -585151.2414090000092983
  coef(7809) =     -48.7525670440999974
  coef(7810) =    -251.5896788500000127
  coef(7811) =   72988.2691552000032971
  coef(7812) =      -4.5209224514399997
  coef(7813) =      12.4676117536000000
  coef(7814) =   -9212.6375446700003522
  coef(7815) =      14.9150390125999994
  coef(7816) =     -79.2693043830000050
  coef(7817) =  -22867.5035629999983939
  coef(7818) =    -134.0829122060000032
  coef(7819) =     -15.9356145209999998
  coef(7820) =    3375.3814978099999280
  coef(7821) =      84.8839984587999936
  coef(7822) =    -594.8902975460000562
  coef(7823) =  231291.6958380000141915
  coef(7824) =      23.3336344147999988
  coef(7825) =    -175.0935086350000063
  coef(7826) =  201995.4968569999909960
  coef(7827) =      49.0195366015000005
  coef(7828) =     153.2354075290000139
  coef(7829) =  -26920.8500352000010025
  coef(7830) =     -95.5490653731999942
  coef(7831) =    1049.7303892100001121
  coef(7832) = -303668.4744000000064261
  coef(7833) =      34.6552192357000024
  coef(7834) =    -311.3813980049999941
  coef(7835) =   -2573.2657267200002025
  coef(7836) =    -305.6558961320000094
  coef(7837) =    2184.6946889199998623
  coef(7838) =   13351.6717912999993132
  coef(7839) =     -19.4335088883999987
  coef(7840) =      22.7880169391999985
  coef(7841) =    1316.6277398300001096
  coef(7842) =     338.5389701749999745
  coef(7843) =   -4827.3071981800003414
  coef(7844) =  -18129.8411474999993516
  coef(7845) =     174.3454650980000054
  coef(7846) =     160.4199633320000089
  coef(7847) =  -34637.0590102999994997
  coef(7848) =      -2.7805370093500001
  coef(7849) =     -26.2242919731000015
  coef(7850) =    1101.7197464599998966
  coef(7851) =     -58.8139202173000015
  coef(7852) =     -98.8160075053999947
  coef(7853) =      15.1250045897999996
  coef(7854) =     131.9004677229999913
  coef(7855) =     220.4162875689999908
  coef(7856) =    8383.3979758099994797
  coef(7857) =    -415.6505502250000177
  coef(7858) =      73.8910895717999949
  coef(7859) =   -1186.3282553700000790
  coef(7860) =    -503.3725403529999767
  coef(7861) =     307.8817434449999837
  coef(7862) =   -7673.1314921699995466
  coef(7863) =     256.3208826850000150
  coef(7864) =     185.7115760430000080
  coef(7865) =    9477.9135256400004437
  coef(7866) =      34.4424213332000022
  coef(7867) =    -788.0776780659999758
  coef(7868) =   -9905.7802332100000058
  coef(7869) =     197.2833350619999919
  coef(7870) =    4336.3753859500002363
  coef(7871) =  -11666.8138674000001629
  coef(7872) =    -326.1792356029999951
  coef(7873) =     -83.9813555197000028
  coef(7874) =   47738.8205697000012151
  coef(7875) =      28.5696329768000012
  coef(7876) =      -2.7399396635800000
  coef(7877) =   -9073.8280066900006204
  coef(7878) =      66.4488629447999983
  coef(7879) =     172.9179752100000087
  coef(7880) =    2607.3925893199998427
  coef(7881) =     199.9677224710000019
  coef(7882) =    -266.6407267839999804
  coef(7883) =  -12941.3036573999997927
  coef(7884) =     528.3492745099999865
  coef(7885) =    -143.3902381759999969
  coef(7886) =   27029.4505576000010478
  coef(7887) =     689.7766659149999668
  coef(7888) =    -274.1402321499999744
  coef(7889) =   37171.1111279999968247
  coef(7890) =    -414.4193154339999978
  coef(7891) =    -635.1463609779999615
  coef(7892) =  -28002.4180028000009770
  coef(7893) =    -523.8998818380000557
  coef(7894) =     816.0087972520000221
  coef(7895) =   -3565.8132853000001887
  coef(7896) =      -1.6271149092599999
  coef(7897) =       2.2663470532200001
  coef(7898) =    2487.2800865399999566
  coef(7899) =      -4.3973679073199996
  coef(7900) =       3.0924735632600000
  coef(7901) =    2263.6347221599999102
  coef(7902) =     -30.7010613521000018
  coef(7903) =      29.6796972365999991
  coef(7904) =    1051.3292509000000337
  coef(7905) =       3.8096461435600002
  coef(7906) =     -39.1238957038000024
  coef(7907) =  -20357.1207856000000902
  coef(7908) =       9.7628093489100003
  coef(7909) =      80.6299948937000011
  coef(7910) =  -22473.0487924000008206
  coef(7911) =       5.0437243950999999
  coef(7912) =      -1.1159830951600000
  coef(7913) =    3209.2254984599999261
  coef(7914) =      55.4192016727999999
  coef(7915) =     -60.1765481686000001
  coef(7916) =   26125.3668838999983564
  coef(7917) =       1.8631887867500001
  coef(7918) =      31.9326978130999990
  coef(7919) =    -261.4764716820000103
  coef(7920) =     -79.4989264173999999
  coef(7921) =    -172.6724679749999893
  coef(7922) =   -5360.6482876500003840
  coef(7923) =    -150.1725079379999954
  coef(7924) =     -39.7643031773000004
  coef(7925) =   -3330.3106789399998888
  coef(7926) =       0.5274974491300000
  coef(7927) =      57.5748618415999971
  coef(7928) =    -959.0313993490000257
  coef(7929) =      -3.0655153398200001
  coef(7930) =      -8.8706238105500006
  coef(7931) =     203.3573595569999952
  coef(7932) =       0.2539477144660000
  coef(7933) =       0.9964235782260000
  coef(7934) =    2747.7337815000000774
  coef(7935) =      34.7493032029999966
  coef(7936) =    -106.0727878189999984
  coef(7937) =    2962.9321853500000543
  coef(7938) =      40.1276918360999986
  coef(7939) =      99.7598494309000046
  coef(7940) =   -5058.5625024000000849
  coef(7941) =     -20.5361879205999998
  coef(7942) =     103.3259262849999942
  coef(7943) =   -3415.0025027199999386
  coef(7944) =      26.9283882008999988
  coef(7945) =      43.3199196282999992
  coef(7946) =    7127.8406167499997537
  coef(7947) =      38.1012829905000032
  coef(7948) =      82.5654898697000021
  coef(7949) =    9949.3315675199992256
  coef(7950) =      20.2521470432999990
  coef(7951) =     -44.1108684004000011
  coef(7952) =    1799.0816631000000143
  coef(7953) =       2.8872673407999998
  coef(7954) =    -239.3331065609999939
  coef(7955) =   -2891.2584393900001487
  coef(7956) =      66.1992556584000056
  coef(7957) =     103.2705772310000043
  coef(7958) =   -1541.7941008099999181
  coef(7959) =      -0.9670909840600000
  coef(7960) =     101.7149918030000038
  coef(7961) =   -2070.3908249099999921



  return
end subroutine prepot1




subroutine predip1()
  implicit none
  integer ma,mb


  double precision dip_coef(0:3843)
  double precision dip_dc0(0:6,0:6), dip_dw0(0:6,0:6)


  common/NDIPCOE/ma,mb
  common/h5o2dipcoef/dip_dc0,dip_dw0,dip_coef

  ma=2512
  mb=1332

  dip_dc0(0,0) =       0.0000000000000000
  dip_dc0(1,0) =       0.2310320556220000
  dip_dc0(2,0) =       0.2310320556220000
  dip_dc0(3,0) =       0.2310320556220000
  dip_dc0(4,0) =       0.2310320556220000
  dip_dc0(5,0) =       0.3539038579620000
  dip_dc0(6,0) =       0.3539038579620000
  dip_dc0(0,1) =       0.2310320556220000
  dip_dc0(1,1) =       0.0000000000000000
  dip_dc0(2,1) =       0.2310320556220000
  dip_dc0(3,1) =       0.2310320556220000
  dip_dc0(4,1) =       0.2310320556220000
  dip_dc0(5,1) =       0.3539038579620000
  dip_dc0(6,1) =       0.3539038579620000
  dip_dc0(0,2) =       0.2310320556220000
  dip_dc0(1,2) =       0.2310320556220000
  dip_dc0(2,2) =       0.0000000000000000
  dip_dc0(3,2) =       0.2310320556220000
  dip_dc0(4,2) =       0.2310320556220000
  dip_dc0(5,2) =       0.3539038579620000
  dip_dc0(6,2) =       0.3539038579620000
  dip_dc0(0,3) =       0.2310320556220000
  dip_dc0(1,3) =       0.2310320556220000
  dip_dc0(2,3) =       0.2310320556220000
  dip_dc0(3,3) =       0.0000000000000000
  dip_dc0(4,3) =       0.2310320556220000
  dip_dc0(5,3) =       0.3539038579620000
  dip_dc0(6,3) =       0.3539038579620000
  dip_dc0(0,4) =       0.2310320556220000
  dip_dc0(1,4) =       0.2310320556220000
  dip_dc0(2,4) =       0.2310320556220000
  dip_dc0(3,4) =       0.2310320556220000
  dip_dc0(4,4) =       0.0000000000000000
  dip_dc0(5,4) =       0.3539038579620000
  dip_dc0(6,4) =       0.3539038579620000
  dip_dc0(0,5) =       0.3539038579620000
  dip_dc0(1,5) =       0.3539038579620000
  dip_dc0(2,5) =       0.3539038579620000
  dip_dc0(3,5) =       0.3539038579620000
  dip_dc0(4,5) =       0.3539038579620000
  dip_dc0(5,5) =       0.0000000000000000
  dip_dc0(6,5) =       0.1972449649890000
  dip_dc0(0,6) =       0.3539038579620000
  dip_dc0(1,6) =       0.3539038579620000
  dip_dc0(2,6) =       0.3539038579620000
  dip_dc0(3,6) =       0.3539038579620000
  dip_dc0(4,6) =       0.3539038579620000
  dip_dc0(5,6) =       0.1972449649890000
  dip_dc0(6,6) =       0.0000000000000000

  dip_dw0(0,0) =       0.0000000000000000
  dip_dw0(1,0) =       0.1219097143100000
  dip_dw0(2,0) =       0.1219097143100000
  dip_dw0(3,0) =       0.1219097143100000
  dip_dw0(4,0) =       0.1219097143100000
  dip_dw0(5,0) =       0.1891483192840000
  dip_dw0(6,0) =       0.1891483192840000
  dip_dw0(0,1) =       0.1219097143100000
  dip_dw0(1,1) =       0.0000000000000000
  dip_dw0(2,1) =       0.1219097143100000
  dip_dw0(3,1) =       0.1219097143100000
  dip_dw0(4,1) =       0.1219097143100000
  dip_dw0(5,1) =       0.1891483192840000
  dip_dw0(6,1) =       0.1891483192840000
  dip_dw0(0,2) =       0.1219097143100000
  dip_dw0(1,2) =       0.1219097143100000
  dip_dw0(2,2) =       0.0000000000000000
  dip_dw0(3,2) =       0.1219097143100000
  dip_dw0(4,2) =       0.1219097143100000
  dip_dw0(5,2) =       0.1891483192840000
  dip_dw0(6,2) =       0.1891483192840000
  dip_dw0(0,3) =       0.1219097143100000
  dip_dw0(1,3) =       0.1219097143100000
  dip_dw0(2,3) =       0.1219097143100000
  dip_dw0(3,3) =       0.0000000000000000
  dip_dw0(4,3) =       0.1219097143100000
  dip_dw0(5,3) =       0.1891483192840000
  dip_dw0(6,3) =       0.1891483192840000
  dip_dw0(0,4) =       0.1219097143100000
  dip_dw0(1,4) =       0.1219097143100000
  dip_dw0(2,4) =       0.1219097143100000
  dip_dw0(3,4) =       0.1219097143100000
  dip_dw0(4,4) =       0.0000000000000000
  dip_dw0(5,4) =       0.1891483192840000
  dip_dw0(6,4) =       0.1891483192840000
  dip_dw0(0,5) =       0.1891483192840000
  dip_dw0(1,5) =       0.1891483192840000
  dip_dw0(2,5) =       0.1891483192840000
  dip_dw0(3,5) =       0.1891483192840000
  dip_dw0(4,5) =       0.1891483192840000
  dip_dw0(5,5) =       0.0000000000000000
  dip_dw0(6,5) =       0.0608002980568600
  dip_dw0(0,6) =       0.1891483192840000
  dip_dw0(1,6) =       0.1891483192840000
  dip_dw0(2,6) =       0.1891483192840000
  dip_dw0(3,6) =       0.1891483192840000
  dip_dw0(4,6) =       0.1891483192840000
  dip_dw0(5,6) =       0.0608002980568600
  dip_dw0(6,6) =       0.0000000000000000

  dip_coef(   0) =      18.3687247418000013
  dip_coef(   1) =       4.1200341951899997
  dip_coef(   2) =    -110.9581740349999990
  dip_coef(   3) =      -6.3123387534299997
  dip_coef(   4) =      19.3977089261000017
  dip_coef(   5) =      -2.1102746446600000
  dip_coef(   6) =       2.7178248029500001
  dip_coef(   7) =     -19.7581562595999998
  dip_coef(   8) =       0.5342975459770000
  dip_coef(   9) =       0.9855359782590000
  dip_coef(  10) =       1.4263466988700000
  dip_coef(  11) =      39.5581733332999974
  dip_coef(  12) =      -6.7836961537200002
  dip_coef(  13) =      -4.9146770700499998
  dip_coef(  14) =      -6.8074463791800000
  dip_coef(  15) =       0.9470541539530000
  dip_coef(  16) =      -2.3676615333000002
  dip_coef(  17) =       2.0689932600700001
  dip_coef(  18) =       0.2397376575450000
  dip_coef(  19) =       3.7097785033899999
  dip_coef(  20) =       1.8860313286699999
  dip_coef(  21) =       0.2246397981330000
  dip_coef(  22) =      -3.5300498698500000
  dip_coef(  23) =      -0.4752444792920000
  dip_coef(  24) =       2.1065852867800001
  dip_coef(  25) =      -4.2159252702399996
  dip_coef(  26) =      -0.4541462435270000
  dip_coef(  27) =      -1.6994788178100000
  dip_coef(  28) =       7.5493646944900004
  dip_coef(  29) =      -1.3434452329000000
  dip_coef(  30) =       1.8533070203799999
  dip_coef(  31) =       0.8176743386820000
  dip_coef(  32) =       0.5164310939660000
  dip_coef(  33) =       9.4971771457000003
  dip_coef(  34) =      -0.7393630776330000
  dip_coef(  35) =      -2.7432811509700001
  dip_coef(  36) =       1.1370201904999999
  dip_coef(  37) =       5.9599078192099997
  dip_coef(  38) =       8.1516370081100007
  dip_coef(  39) =       0.0449300002867600
  dip_coef(  40) =     -13.1503221542999995
  dip_coef(  41) =      -0.0460186537083700
  dip_coef(  42) =      -1.1653945420800000
  dip_coef(  43) =       0.0003272748483055
  dip_coef(  44) =       4.2701663802500001
  dip_coef(  45) =       0.6728347742680000
  dip_coef(  46) =      -3.2447196921299999
  dip_coef(  47) =      -0.3723813022570000
  dip_coef(  48) =      -0.1347313680500000
  dip_coef(  49) =       1.6399041099900000
  dip_coef(  50) =      -8.8664207087300007
  dip_coef(  51) =      -3.0562869121100000
  dip_coef(  52) =      -0.6310256893090000
  dip_coef(  53) =       0.1618639217270000
  dip_coef(  54) =      -7.3617917683899998
  dip_coef(  55) =       1.8527673445499999
  dip_coef(  56) =       1.5062847937199999
  dip_coef(  57) =       0.2184818467740000
  dip_coef(  58) =    -194.5553378200000054
  dip_coef(  59) =      -8.4217449719099999
  dip_coef(  60) =      67.0589425823999932
  dip_coef(  61) =     -11.1403612635000009
  dip_coef(  62) =       2.0551649669800001
  dip_coef(  63) =       0.6127249118040000
  dip_coef(  64) =      -0.3969978685730000
  dip_coef(  65) =     -17.4769870048000016
  dip_coef(  66) =      -0.4853208987220000
  dip_coef(  67) =      12.0232024236000008
  dip_coef(  68) =      -1.3694305730600000
  dip_coef(  69) =     -11.7287041117000008
  dip_coef(  70) =      69.0377886090000032
  dip_coef(  71) =      33.1162478246000020
  dip_coef(  72) =      16.0892644860999994
  dip_coef(  73) =       7.1842820609500002
  dip_coef(  74) =      -4.1861419160800004
  dip_coef(  75) =      16.0058587155000005
  dip_coef(  76) =      -2.4924735528599999
  dip_coef(  77) =     -41.9756611768999974
  dip_coef(  78) =      13.2405879641999995
  dip_coef(  79) =      -0.2687688470660000
  dip_coef(  80) =       0.3744034176290000
  dip_coef(  81) =      -0.1565378546900000
  dip_coef(  82) =       0.8533028373910000
  dip_coef(  83) =      -1.4567725865000001
  dip_coef(  84) =       0.0030323004983350
  dip_coef(  85) =      -0.1280576693990000
  dip_coef(  86) =       0.5497077472980000
  dip_coef(  87) =      -2.9773571102699998
  dip_coef(  88) =      -1.0730395497100000
  dip_coef(  89) =       1.0480848451900000
  dip_coef(  90) =      -0.2413174926490000
  dip_coef(  91) =       0.0949524209001900
  dip_coef(  92) =      -4.7533592021900004
  dip_coef(  93) =       0.6502377769880000
  dip_coef(  94) =       0.6466290885290000
  dip_coef(  95) =      -0.4787244246920000
  dip_coef(  96) =       0.9061232086270000
  dip_coef(  97) =      -3.5731883256499999
  dip_coef(  98) =       6.9892719229800004
  dip_coef(  99) =       0.8720736579450000
  dip_coef( 100) =       2.1408127617299999
  dip_coef( 101) =     -11.6815592215000006
  dip_coef( 102) =       0.6937130341900000
  dip_coef( 103) =      -0.6430438164259999
  dip_coef( 104) =      -0.4759954453930000
  dip_coef( 105) =      -0.3704620834090000
  dip_coef( 106) =      -5.5032829271499999
  dip_coef( 107) =      -3.6699678559700000
  dip_coef( 108) =       8.5154639074199991
  dip_coef( 109) =      -3.6918419402599998
  dip_coef( 110) =      -5.6393577765799998
  dip_coef( 111) =      -0.2395613880640000
  dip_coef( 112) =       0.9111274628860000
  dip_coef( 113) =      12.0092135156000008
  dip_coef( 114) =       1.0556623030300001
  dip_coef( 115) =       1.7932921322399999
  dip_coef( 116) =       0.0554328644280300
  dip_coef( 117) =       1.4986181033099999
  dip_coef( 118) =       3.2403894098200001
  dip_coef( 119) =       4.4362262761700002
  dip_coef( 120) =      -2.5503130880399998
  dip_coef( 121) =      -2.3970828589600002
  dip_coef( 122) =       0.2003431142140000
  dip_coef( 123) =      -2.0820345359200001
  dip_coef( 124) =      -9.4230766848500007
  dip_coef( 125) =       0.0811796689492100
  dip_coef( 126) =       0.9359021606810000
  dip_coef( 127) =      -2.2519529887599998
  dip_coef( 128) =       2.6239396563900002
  dip_coef( 129) =       0.5752599358020000
  dip_coef( 130) =       0.7843447373010000
  dip_coef( 131) =      -6.8681383733599999
  dip_coef( 132) =       5.4051367283899996
  dip_coef( 133) =       2.5521174930699999
  dip_coef( 134) =      -5.2228961492200003
  dip_coef( 135) =      -0.4964300808340000
  dip_coef( 136) =      -1.0688317492399999
  dip_coef( 137) =       0.0529305597846800
  dip_coef( 138) =      -0.2462344364060000
  dip_coef( 139) =      -0.9564605157070000
  dip_coef( 140) =       2.8969451959199999
  dip_coef( 141) =      -2.0978087726900001
  dip_coef( 142) =       1.1910840377700000
  dip_coef( 143) =      -1.3084357390900001
  dip_coef( 144) =       2.4118077732200001
  dip_coef( 145) =       2.5567342485500002
  dip_coef( 146) =       1.2377085736500000
  dip_coef( 147) =      -0.1776018989010000
  dip_coef( 148) =       1.9886967386000001
  dip_coef( 149) =      -0.2189425335300000
  dip_coef( 150) =      -1.4249512971900000
  dip_coef( 151) =       0.0324482471395300
  dip_coef( 152) =      -0.2317372862900000
  dip_coef( 153) =       0.2171693959300000
  dip_coef( 154) =       6.3660113305000001
  dip_coef( 155) =      -0.3087912911680000
  dip_coef( 156) =      -0.4681977446600000
  dip_coef( 157) =      -1.1314281611100001
  dip_coef( 158) =      -0.0513302644770200
  dip_coef( 159) =       0.1975112146610000
  dip_coef( 160) =       0.2348611127410000
  dip_coef( 161) =      -0.3088268530580000
  dip_coef( 162) =       0.0022927111558540
  dip_coef( 163) =       0.0877980997787300
  dip_coef( 164) =       0.0836584785839600
  dip_coef( 165) =       0.6810901416130000
  dip_coef( 166) =      -1.0919545629500000
  dip_coef( 167) =       0.7278042529510000
  dip_coef( 168) =      -0.0621377652594600
  dip_coef( 169) =       0.0888717388566200
  dip_coef( 170) =      -0.0084665693737810
  dip_coef( 171) =       1.7550445029299999
  dip_coef( 172) =      -0.0460715424904200
  dip_coef( 173) =       0.2724451885360000
  dip_coef( 174) =      -0.2076983099510000
  dip_coef( 175) =     -16.1473933014000011
  dip_coef( 176) =       0.5739776601880000
  dip_coef( 177) =       0.7122230803389999
  dip_coef( 178) =       4.8067531345000001
  dip_coef( 179) =       0.3594456942520000
  dip_coef( 180) =      -0.8562036540230000
  dip_coef( 181) =      -2.0962755714700001
  dip_coef( 182) =      -0.0140993124793400
  dip_coef( 183) =      -0.3846622592760000
  dip_coef( 184) =      -1.0737782261000000
  dip_coef( 185) =      -0.0346636794399500
  dip_coef( 186) =       5.2885061871900003
  dip_coef( 187) =      12.2850707298999993
  dip_coef( 188) =      -0.3418524504070000
  dip_coef( 189) =      -2.1607520789999999
  dip_coef( 190) =       0.0720201174873400
  dip_coef( 191) =       0.3093278191980000
  dip_coef( 192) =     -16.7918096068999994
  dip_coef( 193) =       4.4711614484700002
  dip_coef( 194) =      -4.1741003278599997
  dip_coef( 195) =       0.9810940396690000
  dip_coef( 196) =       0.0318250367614700
  dip_coef( 197) =      -0.0437014991407400
  dip_coef( 198) =      -0.1142158482700000
  dip_coef( 199) =       0.3468016216070000
  dip_coef( 200) =      -0.0144790051360700
  dip_coef( 201) =       0.4571535135010000
  dip_coef( 202) =       0.0230374058780700
  dip_coef( 203) =      -0.2931772837630000
  dip_coef( 204) =      -1.3198091188000001
  dip_coef( 205) =      -0.6677424233520000
  dip_coef( 206) =       0.4341713336940000
  dip_coef( 207) =      -0.0434104664783900
  dip_coef( 208) =      -0.0712063372480900
  dip_coef( 209) =       1.4304040450900000
  dip_coef( 210) =      -0.4069188287880000
  dip_coef( 211) =       0.4915670253850000
  dip_coef( 212) =      -0.1627922025570000
  dip_coef( 213) =       0.4178292163380000
  dip_coef( 214) =      -0.1514475455810000
  dip_coef( 215) =       0.2542614864210000
  dip_coef( 216) =      -0.0609409844696400
  dip_coef( 217) =       0.7573647015990000
  dip_coef( 218) =       0.9397675588810001
  dip_coef( 219) =      -1.2565950671999999
  dip_coef( 220) =       1.0695732574800001
  dip_coef( 221) =      -0.3480456581680000
  dip_coef( 222) =      -0.0830264343948500
  dip_coef( 223) =       0.3782014292260000
  dip_coef( 224) =      -0.1240185669460000
  dip_coef( 225) =      -0.0885099772606500
  dip_coef( 226) =       0.3868963755440000
  dip_coef( 227) =      -0.5431961044830000
  dip_coef( 228) =      -0.0019167850908660
  dip_coef( 229) =      -0.7916015279260000
  dip_coef( 230) =      -0.1393268119970000
  dip_coef( 231) =       2.0582783877600002
  dip_coef( 232) =      -0.9309126455220000
  dip_coef( 233) =       0.1490929605720000
  dip_coef( 234) =       0.2142964611200000
  dip_coef( 235) =       1.1459617038000001
  dip_coef( 236) =      -0.4683480102880000
  dip_coef( 237) =       0.0053581263882180
  dip_coef( 238) =       0.9235223905540000
  dip_coef( 239) =      -0.0543082541484300
  dip_coef( 240) =      -0.5955956842050000
  dip_coef( 241) =      -0.8627690519790000
  dip_coef( 242) =       0.0602117748179100
  dip_coef( 243) =      -0.3587224867610000
  dip_coef( 244) =       0.0181251129960400
  dip_coef( 245) =      -0.7656890992670000
  dip_coef( 246) =       0.7876561942390000
  dip_coef( 247) =       0.0632315374995200
  dip_coef( 248) =      -0.3818283773620000
  dip_coef( 249) =      -0.7954397341580000
  dip_coef( 250) =      -0.2087194329810000
  dip_coef( 251) =       1.1350843764900000
  dip_coef( 252) =       0.0581031332473600
  dip_coef( 253) =       0.1227674918100000
  dip_coef( 254) =       0.0399557056196000
  dip_coef( 255) =       0.1811014830070000
  dip_coef( 256) =      -0.0055148168570950
  dip_coef( 257) =       0.1984115604690000
  dip_coef( 258) =       0.0824480052528800
  dip_coef( 259) =      -0.2373875749750000
  dip_coef( 260) =       0.1380925204900000
  dip_coef( 261) =      -1.3027186731900000
  dip_coef( 262) =      -0.6494843726470000
  dip_coef( 263) =       0.5089802349360000
  dip_coef( 264) =       0.0700528942667900
  dip_coef( 265) =       0.1911566667690000
  dip_coef( 266) =      13.9471341688999999
  dip_coef( 267) =      -1.2070968134200000
  dip_coef( 268) =      -3.3193481863500001
  dip_coef( 269) =     -12.0587716826999998
  dip_coef( 270) =       0.4106988537590000
  dip_coef( 271) =      -2.4085134437800000
  dip_coef( 272) =       7.2488617486000004
  dip_coef( 273) =      -1.0138861701399999
  dip_coef( 274) =       5.7872168218500004
  dip_coef( 275) =      -0.9386001902260001
  dip_coef( 276) =       0.2566048719250000
  dip_coef( 277) =      -8.4951535606400004
  dip_coef( 278) =      -7.7495837923300002
  dip_coef( 279) =       7.6212000589600004
  dip_coef( 280) =      -5.5941224017000000
  dip_coef( 281) =      -0.1224307621820000
  dip_coef( 282) =      -2.5136939094800002
  dip_coef( 283) =      22.2496023723000000
  dip_coef( 284) =      -4.7055930573099998
  dip_coef( 285) =       3.8015129239999998
  dip_coef( 286) =       1.9130788509300001
  dip_coef( 287) =      -0.3231515815340000
  dip_coef( 288) =       0.5315954617210000
  dip_coef( 289) =       0.0265036828093500
  dip_coef( 290) =       1.4141702242200000
  dip_coef( 291) =      -3.2252362209999998
  dip_coef( 292) =      -0.5209432082030000
  dip_coef( 293) =      -0.1289749412850000
  dip_coef( 294) =       1.5389983553100000
  dip_coef( 295) =      -4.0203020347400003
  dip_coef( 296) =       0.2340341946460000
  dip_coef( 297) =       1.8600081068900001
  dip_coef( 298) =      -0.2851126796300000
  dip_coef( 299) =       0.4424092021270000
  dip_coef( 300) =      -1.1998204777100001
  dip_coef( 301) =       0.7095721309980000
  dip_coef( 302) =       0.9860811372190000
  dip_coef( 303) =      -0.7115138124220000
  dip_coef( 304) =      -0.9955281972600000
  dip_coef( 305) =      -2.9641987554000000
  dip_coef( 306) =       3.3985503313800001
  dip_coef( 307) =      -0.0371394181821000
  dip_coef( 308) =      -0.9611330955180000
  dip_coef( 309) =       2.4371266173200001
  dip_coef( 310) =       0.9204320002020000
  dip_coef( 311) =      -1.3987803203300000
  dip_coef( 312) =       1.7612427617899999
  dip_coef( 313) =      -0.1415030626360000
  dip_coef( 314) =       4.9148304295000003
  dip_coef( 315) =       2.5832858061200001
  dip_coef( 316) =      -1.3797131284499999
  dip_coef( 317) =       2.3297189460599999
  dip_coef( 318) =      -0.8457243413190000
  dip_coef( 319) =       0.1514049323520000
  dip_coef( 320) =       0.8309694791590000
  dip_coef( 321) =       8.7788990944100007
  dip_coef( 322) =      -0.8061072161870000
  dip_coef( 323) =      -0.8363883051600000
  dip_coef( 324) =       0.6719625057810000
  dip_coef( 325) =       0.1476583858750000
  dip_coef( 326) =      -8.6424732747299995
  dip_coef( 327) =       2.5196321292100001
  dip_coef( 328) =      -2.2057718216400000
  dip_coef( 329) =      -3.8230141743199999
  dip_coef( 330) =      -0.0430317411309400
  dip_coef( 331) =      -3.7548948933799999
  dip_coef( 332) =       8.2861902384399997
  dip_coef( 333) =      -0.6398286936120000
  dip_coef( 334) =       0.3906446726220000
  dip_coef( 335) =       0.3467008788020000
  dip_coef( 336) =       4.5236781268700001
  dip_coef( 337) =      -7.5249397334000001
  dip_coef( 338) =      -0.3978812468450000
  dip_coef( 339) =      -0.4723310470460000
  dip_coef( 340) =       8.2299538203400004
  dip_coef( 341) =      -1.7864540125499999
  dip_coef( 342) =      -9.4526208450600002
  dip_coef( 343) =       0.3168264928240000
  dip_coef( 344) =      -1.3020790712700001
  dip_coef( 345) =      -0.8637050124670000
  dip_coef( 346) =      -0.6605108387400001
  dip_coef( 347) =      -2.0899625552500001
  dip_coef( 348) =      -2.6554567052200002
  dip_coef( 349) =       0.8567803505930000
  dip_coef( 350) =       0.4739145455700000
  dip_coef( 351) =       0.5390863447860000
  dip_coef( 352) =       7.8359113321300002
  dip_coef( 353) =       0.7362525742379999
  dip_coef( 354) =      -0.3289751572520000
  dip_coef( 355) =      -1.2798941592499999
  dip_coef( 356) =      -2.7910485244799998
  dip_coef( 357) =       0.0232789395231000
  dip_coef( 358) =      -0.0144267496759400
  dip_coef( 359) =      -0.0406639435716900
  dip_coef( 360) =      -0.3396306019100000
  dip_coef( 361) =       0.5330692553360000
  dip_coef( 362) =      -0.1009043737930000
  dip_coef( 363) =       0.0152953889719400
  dip_coef( 364) =      -0.0179987070367000
  dip_coef( 365) =       1.1309786811600000
  dip_coef( 366) =       0.3075910527370000
  dip_coef( 367) =      -0.3448118914830000
  dip_coef( 368) =       0.0501750771106700
  dip_coef( 369) =      -0.0260256709257600
  dip_coef( 370) =      -0.9147805876160000
  dip_coef( 371) =       0.2865111717550000
  dip_coef( 372) =      -0.4353013637870000
  dip_coef( 373) =       0.2485529120300000
  dip_coef( 374) =      -0.1951104965430000
  dip_coef( 375) =       1.7154560255200000
  dip_coef( 376) =      -2.0677048304099999
  dip_coef( 377) =       0.0959329120061000
  dip_coef( 378) =      -0.6310756647000000
  dip_coef( 379) =      -0.4753908451980000
  dip_coef( 380) =      -0.1600886642310000
  dip_coef( 381) =       0.3900327204840000
  dip_coef( 382) =      -0.2217730085360000
  dip_coef( 383) =       0.1996890156760000
  dip_coef( 384) =       1.0708109576200000
  dip_coef( 385) =      -0.6896368511950000
  dip_coef( 386) =       0.4375598524350000
  dip_coef( 387) =      -0.8710886191370000
  dip_coef( 388) =       0.8767113206140000
  dip_coef( 389) =      -0.0925948923996700
  dip_coef( 390) =      -0.0426178340613800
  dip_coef( 391) =      -4.5561996879800004
  dip_coef( 392) =      -1.3615046325200000
  dip_coef( 393) =       0.7639350390720000
  dip_coef( 394) =      -0.2071200670620000
  dip_coef( 395) =      -0.1932707081040000
  dip_coef( 396) =       3.1103158220800000
  dip_coef( 397) =      -0.8202875501010000
  dip_coef( 398) =       1.5554842034200000
  dip_coef( 399) =       0.5003281233470001
  dip_coef( 400) =       0.0020150432878780
  dip_coef( 401) =       0.4242424638640000
  dip_coef( 402) =      -1.2349617222700000
  dip_coef( 403) =       0.1118327139430000
  dip_coef( 404) =      -0.2122101619730000
  dip_coef( 405) =      -0.0303959397045800
  dip_coef( 406) =      -0.6694067935990000
  dip_coef( 407) =       1.1414097506300001
  dip_coef( 408) =       0.3986397803700000
  dip_coef( 409) =      -0.1779715145920000
  dip_coef( 410) =      -0.9997521524940000
  dip_coef( 411) =       0.1052347134410000
  dip_coef( 412) =       1.6490920952899999
  dip_coef( 413) =       0.2662847194900000
  dip_coef( 414) =       0.1458305995550000
  dip_coef( 415) =       0.2551719109080000
  dip_coef( 416) =       0.0677964893322800
  dip_coef( 417) =       0.2886247370860000
  dip_coef( 418) =       0.5087666260020000
  dip_coef( 419) =      -0.3087731306250000
  dip_coef( 420) =       0.0887385056749300
  dip_coef( 421) =      -0.2207150148600000
  dip_coef( 422) =      -1.7007630229699999
  dip_coef( 423) =      -0.3001447134690000
  dip_coef( 424) =       0.3091185060960000
  dip_coef( 425) =       0.4256210305490000
  dip_coef( 426) =       0.4941407178100000
  dip_coef( 427) =       0.4081863400130000
  dip_coef( 428) =       0.0706429080886500
  dip_coef( 429) =      -1.6583191344399999
  dip_coef( 430) =      -0.0316291850413200
  dip_coef( 431) =       1.4357340908500000
  dip_coef( 432) =       0.1817644497700000
  dip_coef( 433) =       1.7542586117900001
  dip_coef( 434) =      -0.3279088155990000
  dip_coef( 435) =      -0.4946303495650000
  dip_coef( 436) =       0.1997507890290000
  dip_coef( 437) =      -2.6301590418799998
  dip_coef( 438) =      -0.7284056686840000
  dip_coef( 439) =       0.8866794844720000
  dip_coef( 440) =      -0.2260117309160000
  dip_coef( 441) =       1.7249271439900000
  dip_coef( 442) =      -0.0076241047083360
  dip_coef( 443) =       0.1379395949290000
  dip_coef( 444) =       1.3555991218500001
  dip_coef( 445) =      -3.0665454463300001
  dip_coef( 446) =      -0.2564904563170000
  dip_coef( 447) =       0.9577361337970000
  dip_coef( 448) =      -0.3033320559560000
  dip_coef( 449) =       0.2527421011760000
  dip_coef( 450) =       0.3910279790310000
  dip_coef( 451) =      -2.1513255590800000
  dip_coef( 452) =       1.3259044793400001
  dip_coef( 453) =      -0.0144526758094300
  dip_coef( 454) =       0.3039227570430000
  dip_coef( 455) =       0.1925240186640000
  dip_coef( 456) =       0.0625818456902300
  dip_coef( 457) =      -0.2354532533560000
  dip_coef( 458) =      -0.0108260894899500
  dip_coef( 459) =       0.1026617990090000
  dip_coef( 460) =      -1.9396791879399999
  dip_coef( 461) =      -0.1336229104310000
  dip_coef( 462) =      -0.0897152819012200
  dip_coef( 463) =       0.0986136465780100
  dip_coef( 464) =       1.0731997694800000
  dip_coef( 465) =      -0.8360061395820000
  dip_coef( 466) =      -0.9007974525460000
  dip_coef( 467) =      -0.2203929666400000
  dip_coef( 468) =       0.3489985724770000
  dip_coef( 469) =       0.0785639349229700
  dip_coef( 470) =      -0.2378741026110000
  dip_coef( 471) =      -0.0598856023893100
  dip_coef( 472) =      -0.0636752904762600
  dip_coef( 473) =      -0.1913649066040000
  dip_coef( 474) =       0.1613841043630000
  dip_coef( 475) =       0.8897882178830000
  dip_coef( 476) =       0.3173940723530000
  dip_coef( 477) =      -0.7878281639710000
  dip_coef( 478) =       0.0929018679693100
  dip_coef( 479) =      -0.2629306302930000
  dip_coef( 480) =      -0.6806020272920000
  dip_coef( 481) =       0.0416918612187100
  dip_coef( 482) =      -0.2318470407160000
  dip_coef( 483) =       4.0434993503900003
  dip_coef( 484) =       4.6244742598600004
  dip_coef( 485) =      -2.7115338421000001
  dip_coef( 486) =      -0.1766247197750000
  dip_coef( 487) =       0.4077253904970000
  dip_coef( 488) =      -0.9936768319740000
  dip_coef( 489) =       0.4007534363500000
  dip_coef( 490) =       0.4287413671020000
  dip_coef( 491) =      -0.6384935668370000
  dip_coef( 492) =      -0.0177304707819100
  dip_coef( 493) =      -0.2180565379520000
  dip_coef( 494) =       2.7304257258400000
  dip_coef( 495) =      -0.1920859907140000
  dip_coef( 496) =       0.5051294894580000
  dip_coef( 497) =       0.2754179571290000
  dip_coef( 498) =       0.1721676101000000
  dip_coef( 499) =      -0.4030291512230000
  dip_coef( 500) =      -0.4197772097200000
  dip_coef( 501) =       0.5995221707970000
  dip_coef( 502) =      -0.2453900229090000
  dip_coef( 503) =      -1.0246888941400001
  dip_coef( 504) =      -1.1586332309200000
  dip_coef( 505) =       0.1320787304320000
  dip_coef( 506) =       0.0267651396994700
  dip_coef( 507) =      -0.6933533824320000
  dip_coef( 508) =       0.1761424564420000
  dip_coef( 509) =       0.0185755980104500
  dip_coef( 510) =      -1.0323462414900000
  dip_coef( 511) =       0.4105172514120000
  dip_coef( 512) =      -0.2996303695660000
  dip_coef( 513) =       0.6671777923950000
  dip_coef( 514) =       1.9465447855600000
  dip_coef( 515) =      -0.4507398236490000
  dip_coef( 516) =      -0.5738837599210000
  dip_coef( 517) =      -1.3875681894400000
  dip_coef( 518) =      -1.4755514680899999
  dip_coef( 519) =       0.0020203768248970
  dip_coef( 520) =      -0.2113050203960000
  dip_coef( 521) =       0.1454521702130000
  dip_coef( 522) =      -0.0636625039465000
  dip_coef( 523) =      -0.0934671605970300
  dip_coef( 524) =       0.0460072735476900
  dip_coef( 525) =      -0.0462936372933800
  dip_coef( 526) =       0.2917152010410000
  dip_coef( 527) =       0.0442202609144200
  dip_coef( 528) =      -0.2167078298690000
  dip_coef( 529) =       0.0589067648646300
  dip_coef( 530) =       0.6253770699109999
  dip_coef( 531) =      -0.5547962221920000
  dip_coef( 532) =      -1.8117563493900000
  dip_coef( 533) =       0.7805831424880000
  dip_coef( 534) =      -0.5345704655290000
  dip_coef( 535) =      -0.0748501789938900
  dip_coef( 536) =      -3.8396202184799999
  dip_coef( 537) =       0.2837781558090000
  dip_coef( 538) =       0.7927273714100000
  dip_coef( 539) =       0.6641974047300000
  dip_coef( 540) =       1.0134793322000000
  dip_coef( 541) =      -4.6415494530999997
  dip_coef( 542) =      -0.9046322258490000
  dip_coef( 543) =       0.0596763902331100
  dip_coef( 544) =      -0.8012877181069999
  dip_coef( 545) =      -1.9145757766600000
  dip_coef( 546) =       1.3328654915200000
  dip_coef( 547) =      -1.6381409737999999
  dip_coef( 548) =       0.0936732082928100
  dip_coef( 549) =      -1.1100263740400000
  dip_coef( 550) =       2.5019077366900002
  dip_coef( 551) =      -0.8773718681330001
  dip_coef( 552) =       0.0357975773157500
  dip_coef( 553) =       2.6284023974299999
  dip_coef( 554) =      -1.1729283479199999
  dip_coef( 555) =       2.6716164756700000
  dip_coef( 556) =      -2.7627709116000001
  dip_coef( 557) =       0.6952733729730000
  dip_coef( 558) =       0.2189786179260000
  dip_coef( 559) =       0.7584562991629999
  dip_coef( 560) =       1.1118799614699999
  dip_coef( 561) =      -0.9414292774780000
  dip_coef( 562) =       0.3167370204860000
  dip_coef( 563) =       0.4796404892410000
  dip_coef( 564) =       0.0862303773408000
  dip_coef( 565) =      -0.1619352973990000
  dip_coef( 566) =       0.5615488031890000
  dip_coef( 567) =       0.1522277246760000
  dip_coef( 568) =      -0.4445954507690000
  dip_coef( 569) =       0.0224308731286000
  dip_coef( 570) =      -0.0196978582336400
  dip_coef( 571) =       0.8432719855860000
  dip_coef( 572) =      -0.1332561617430000
  dip_coef( 573) =       0.3509337398940000
  dip_coef( 574) =      -0.3234745702320000
  dip_coef( 575) =       0.4834212806570000
  dip_coef( 576) =       1.0079430184000000
  dip_coef( 577) =      -0.3227590726370000
  dip_coef( 578) =       0.0013964057544930
  dip_coef( 579) =       2.5372983863399998
  dip_coef( 580) =      -2.6785642453800000
  dip_coef( 581) =       0.4412230837030000
  dip_coef( 582) =       0.9935135807480000
  dip_coef( 583) =      -1.1462297774200001
  dip_coef( 584) =       0.6193338653750000
  dip_coef( 585) =      -1.1060453990500001
  dip_coef( 586) =      -0.5080122609860001
  dip_coef( 587) =      -0.4095007642530000
  dip_coef( 588) =       0.0201363940558300
  dip_coef( 589) =       0.2248373000450000
  dip_coef( 590) =      -0.0754448331742600
  dip_coef( 591) =       0.1251659261230000
  dip_coef( 592) =      -0.0183242238258900
  dip_coef( 593) =       0.1240215995900000
  dip_coef( 594) =      -0.4199927559200000
  dip_coef( 595) =      -0.0304258124263500
  dip_coef( 596) =       0.5901543100750000
  dip_coef( 597) =       0.3605553402950000
  dip_coef( 598) =      -0.0829501825911100
  dip_coef( 599) =      -0.3608141656180000
  dip_coef( 600) =       0.0608276776949300
  dip_coef( 601) =       0.1488843956390000
  dip_coef( 602) =       0.9682721059530000
  dip_coef( 603) =       0.4386602467010000
  dip_coef( 604) =      -0.2329748719500000
  dip_coef( 605) =      -0.0810560723230200
  dip_coef( 606) =      -0.3180557302740000
  dip_coef( 607) =      -0.0636594191404200
  dip_coef( 608) =      -0.3483984760480000
  dip_coef( 609) =       0.3961868626660000
  dip_coef( 610) =      -0.0933410880188400
  dip_coef( 611) =      -0.0214675533357600
  dip_coef( 612) =       0.0091509396408390
  dip_coef( 613) =       0.0674538550413000
  dip_coef( 614) =      -0.0267320517794300
  dip_coef( 615) =      -0.0472581204575900
  dip_coef( 616) =      -0.0647345747165600
  dip_coef( 617) =       0.0424046690849400
  dip_coef( 618) =       0.3103227112770000
  dip_coef( 619) =      -0.1035296198560000
  dip_coef( 620) =       0.0186787853771000
  dip_coef( 621) =       0.0617543637560000
  dip_coef( 622) =      -0.5063728303570000
  dip_coef( 623) =       0.3484692260940000
  dip_coef( 624) =       0.3844715203260000
  dip_coef( 625) =       0.0140509079326900
  dip_coef( 626) =       0.2289229750670000
  dip_coef( 627) =       0.1206219916020000
  dip_coef( 628) =      -0.2571437292220000
  dip_coef( 629) =       0.0981087275952000
  dip_coef( 630) =      -0.0244736871196900
  dip_coef( 631) =      -0.5709132624380000
  dip_coef( 632) =       0.0340888748120600
  dip_coef( 633) =       0.3571207716700000
  dip_coef( 634) =       0.0771509168462400
  dip_coef( 635) =      -0.0108951943812400
  dip_coef( 636) =      -0.1479093731120000
  dip_coef( 637) =      -0.1055510168550000
  dip_coef( 638) =       0.3652382651220000
  dip_coef( 639) =       0.0969457603777600
  dip_coef( 640) =       0.4858974492420000
  dip_coef( 641) =       0.0481479694936300
  dip_coef( 642) =      -0.0269919625293900
  dip_coef( 643) =      -0.2168279743430000
  dip_coef( 644) =       0.1353931767730000
  dip_coef( 645) =      -0.0950393563862500
  dip_coef( 646) =       0.3082400709110000
  dip_coef( 647) =       0.1310351234440000
  dip_coef( 648) =       0.0340426722003300
  dip_coef( 649) =       0.0914238108987000
  dip_coef( 650) =       0.1306712425060000
  dip_coef( 651) =      -0.2924510713840000
  dip_coef( 652) =      -0.0906030721772100
  dip_coef( 653) =       0.3506070586990000
  dip_coef( 654) =      -0.0477473072915600
  dip_coef( 655) =      -0.1047632984370000
  dip_coef( 656) =       0.5889430180650000
  dip_coef( 657) =      -0.0920138424077700
  dip_coef( 658) =      -1.4972660690699999
  dip_coef( 659) =       0.0280349247465300
  dip_coef( 660) =      -0.1037267275400000
  dip_coef( 661) =      -0.0415545018548000
  dip_coef( 662) =      -0.2755964656530000
  dip_coef( 663) =       0.0922049285782000
  dip_coef( 664) =       0.1944817000900000
  dip_coef( 665) =       0.0223442087149500
  dip_coef( 666) =      -0.5300462092140000
  dip_coef( 667) =      -0.2096117180500000
  dip_coef( 668) =       0.6298838813480000
  dip_coef( 669) =       0.0189808578254300
  dip_coef( 670) =       0.0468240876977500
  dip_coef( 671) =      -0.0882133456842900
  dip_coef( 672) =       0.7539399852860000
  dip_coef( 673) =      -0.1973936687610000
  dip_coef( 674) =       0.0304410038864100
  dip_coef( 675) =      -0.0484565184805000
  dip_coef( 676) =       8.0777267532200003
  dip_coef( 677) =      -2.3539572012200001
  dip_coef( 678) =      -6.8444793726300004
  dip_coef( 679) =       5.2830165287500002
  dip_coef( 680) =      -0.0334271295291800
  dip_coef( 681) =       0.0032655754732640
  dip_coef( 682) =       0.0426232218435800
  dip_coef( 683) =      -0.2857356651610000
  dip_coef( 684) =      -0.1154912206730000
  dip_coef( 685) =      -0.0073987440768760
  dip_coef( 686) =       0.1170442825100000
  dip_coef( 687) =       0.7002797007629999
  dip_coef( 688) =      -1.4096416588400000
  dip_coef( 689) =      -0.3769602410490000
  dip_coef( 690) =       0.6878596031770000
  dip_coef( 691) =       0.1598073209080000
  dip_coef( 692) =       0.2886889770520000
  dip_coef( 693) =      -4.0744636508900003
  dip_coef( 694) =       0.1910599084720000
  dip_coef( 695) =      -0.0786375397778200
  dip_coef( 696) =      -0.0529917973327200
  dip_coef( 697) =       0.0130600888222000
  dip_coef( 698) =       0.0173403207811300
  dip_coef( 699) =      -0.0584983073597300
  dip_coef( 700) =       0.2974055449870000
  dip_coef( 701) =       0.0345178492318300
  dip_coef( 702) =       0.0398678283430100
  dip_coef( 703) =      -0.0432219285378900
  dip_coef( 704) =       0.3953287493150000
  dip_coef( 705) =       0.7591580957980000
  dip_coef( 706) =      -0.0899853595714400
  dip_coef( 707) =      -0.0607140461336000
  dip_coef( 708) =      -0.1089650348970000
  dip_coef( 709) =      -0.0428681601979000
  dip_coef( 710) =      -0.9607013432110000
  dip_coef( 711) =       0.0580525317080500
  dip_coef( 712) =      -0.0019217859631050
  dip_coef( 713) =      -0.1237482613140000
  dip_coef( 714) =      -0.4144262894440000
  dip_coef( 715) =      -0.8649824789910000
  dip_coef( 716) =      -0.0918584425366900
  dip_coef( 717) =       0.0125844361532900
  dip_coef( 718) =      -1.1607657693500000
  dip_coef( 719) =       1.8905298762000000
  dip_coef( 720) =       1.1801509429199999
  dip_coef( 721) =      -0.6032833025080000
  dip_coef( 722) =       0.0855934632411500
  dip_coef( 723) =       0.0603063195770300
  dip_coef( 724) =      -0.0962836869912400
  dip_coef( 725) =       0.9972997880409999
  dip_coef( 726) =       0.1834396821960000
  dip_coef( 727) =       0.1301943427820000
  dip_coef( 728) =       0.0293138380434000
  dip_coef( 729) =       0.0394919254289800
  dip_coef( 730) =       0.1702117817980000
  dip_coef( 731) =      -1.1415615928200000
  dip_coef( 732) =      -0.6058424186640000
  dip_coef( 733) =       0.4321942171550000
  dip_coef( 734) =       0.1922191577410000
  dip_coef( 735) =       0.0248321565378500
  dip_coef( 736) =       1.5281456266400000
  dip_coef( 737) =       0.0371546911823100
  dip_coef( 738) =       0.2154070113010000
  dip_coef( 739) =       0.0097889671490920
  dip_coef( 740) =       0.0024937691211480
  dip_coef( 741) =       0.3635675469340000
  dip_coef( 742) =       0.2832813503090000
  dip_coef( 743) =      -0.0005022002153193
  dip_coef( 744) =      -0.0481326023588900
  dip_coef( 745) =      -0.1273391519070000
  dip_coef( 746) =      -0.3460546211070000
  dip_coef( 747) =       0.4509052230430000
  dip_coef( 748) =      -0.0733209072673300
  dip_coef( 749) =      -0.6292704304180000
  dip_coef( 750) =      -0.4053463167340000
  dip_coef( 751) =      -0.2437372889770000
  dip_coef( 752) =       0.4098357805020000
  dip_coef( 753) =       0.2829257263420000
  dip_coef( 754) =       0.0620461370658500
  dip_coef( 755) =      -0.2048307480510000
  dip_coef( 756) =      -0.0958906855963100
  dip_coef( 757) =       0.1292051780490000
  dip_coef( 758) =      -0.1627257377150000
  dip_coef( 759) =       0.0013552620729720
  dip_coef( 760) =      -0.1113280330940000
  dip_coef( 761) =       0.0301361738282500
  dip_coef( 762) =       0.1871602765190000
  dip_coef( 763) =      -0.1485810836120000
  dip_coef( 764) =       0.2290538939620000
  dip_coef( 765) =       0.2457558090810000
  dip_coef( 766) =      -0.2260749732280000
  dip_coef( 767) =       4.1025483153200000
  dip_coef( 768) =       7.6028464216399998
  dip_coef( 769) =       1.3660845745100001
  dip_coef( 770) =     -15.4755062181999996
  dip_coef( 771) =      -0.1527128808430000
  dip_coef( 772) =      -2.7968644425200000
  dip_coef( 773) =       0.6883751445830000
  dip_coef( 774) =      11.4851364984999993
  dip_coef( 775) =       0.1607620762610000
  dip_coef( 776) =      -2.5834936447299999
  dip_coef( 777) =      -0.3492953433890000
  dip_coef( 778) =       3.1807184709600000
  dip_coef( 779) =       0.1770884027970000
  dip_coef( 780) =     -19.8106892280999993
  dip_coef( 781) =      -4.2919412281299998
  dip_coef( 782) =      -1.9053994977700000
  dip_coef( 783) =      -0.3331696394210000
  dip_coef( 784) =     -11.1811568128999994
  dip_coef( 785) =       2.7410052905600000
  dip_coef( 786) =       5.3448903278600000
  dip_coef( 787) =      -0.4943307540330000
  dip_coef( 788) =      -0.0811274799931500
  dip_coef( 789) =      -0.0472311712717100
  dip_coef( 790) =       0.4725951774690000
  dip_coef( 791) =       0.4902197800020000
  dip_coef( 792) =      -1.0813035373900000
  dip_coef( 793) =       0.0239719571693400
  dip_coef( 794) =       0.0713267670675000
  dip_coef( 795) =      -1.7521408831100000
  dip_coef( 796) =      -5.8823840798299996
  dip_coef( 797) =      -0.0859542008317900
  dip_coef( 798) =       0.3712625879740000
  dip_coef( 799) =       0.0116594478438200
  dip_coef( 800) =       0.2374720504460000
  dip_coef( 801) =       6.6367084340100000
  dip_coef( 802) =      -0.1405576468760000
  dip_coef( 803) =       2.3294137994800002
  dip_coef( 804) =       0.0470936728556600
  dip_coef( 805) =      -2.4761937009800001
  dip_coef( 806) =       0.2129900636550000
  dip_coef( 807) =       0.8413813632310000
  dip_coef( 808) =      -0.4630960230000000
  dip_coef( 809) =       2.1425027785899999
  dip_coef( 810) =      -0.1562607348950000
  dip_coef( 811) =       1.1344168173900000
  dip_coef( 812) =       0.5187462846030000
  dip_coef( 813) =       1.0574156345400001
  dip_coef( 814) =      -0.4960068645720000
  dip_coef( 815) =      15.1047154297000006
  dip_coef( 816) =      -0.4146241598460000
  dip_coef( 817) =      -6.8721277558000002
  dip_coef( 818) =       2.0231915280999999
  dip_coef( 819) =       0.3419971059960000
  dip_coef( 820) =       0.4674183823040000
  dip_coef( 821) =       4.3608874067199999
  dip_coef( 822) =       9.7542142855500007
  dip_coef( 823) =       3.4042126007400002
  dip_coef( 824) =      -3.9505170653500001
  dip_coef( 825) =      -0.1250415785050000
  dip_coef( 826) =      -0.3985161231280000
  dip_coef( 827) =     -10.8002647362000008
  dip_coef( 828) =       0.3783714478320000
  dip_coef( 829) =      -1.2010339094100000
  dip_coef( 830) =       0.1678298728850000
  dip_coef( 831) =      -0.0678775892335700
  dip_coef( 832) =      -0.3470001145860000
  dip_coef( 833) =       3.1320347777199999
  dip_coef( 834) =      -0.1456689544200000
  dip_coef( 835) =       0.2415529633380000
  dip_coef( 836) =       1.4644963635099999
  dip_coef( 837) =       1.6352686359799999
  dip_coef( 838) =       0.4120144938130000
  dip_coef( 839) =       1.5824141201899999
  dip_coef( 840) =       6.0712608189999999
  dip_coef( 841) =       0.3195937438710000
  dip_coef( 842) =       1.6090118253600001
  dip_coef( 843) =       1.7859755926800001
  dip_coef( 844) =      -0.1614587495840000
  dip_coef( 845) =       0.1903180786900000
  dip_coef( 846) =      -0.2093783952290000
  dip_coef( 847) =       0.1363174484200000
  dip_coef( 848) =      -0.5480479781480000
  dip_coef( 849) =      -1.1326008277299999
  dip_coef( 850) =       1.1525459721700000
  dip_coef( 851) =       0.4199238853830000
  dip_coef( 852) =       1.0386887053600000
  dip_coef( 853) =      -0.5050477371970000
  dip_coef( 854) =      -0.3651165094110000
  dip_coef( 855) =      -2.2011906286700000
  dip_coef( 856) =      -1.8326101051300001
  dip_coef( 857) =      -0.5659602385900000
  dip_coef( 858) =      -0.0083589327493030
  dip_coef( 859) =       0.0160741288687100
  dip_coef( 860) =       0.0186118022120300
  dip_coef( 861) =      -0.0104193678112800
  dip_coef( 862) =      -0.0497781880881700
  dip_coef( 863) =      -0.0741610394075600
  dip_coef( 864) =      -0.0048652050664240
  dip_coef( 865) =      -0.1058603639540000
  dip_coef( 866) =       0.4015358009070000
  dip_coef( 867) =       0.1940301661990000
  dip_coef( 868) =      -0.0905357978536200
  dip_coef( 869) =      -0.0037959424067350
  dip_coef( 870) =       0.0313934052972100
  dip_coef( 871) =      -0.2112719370900000
  dip_coef( 872) =      -0.0062553406012770
  dip_coef( 873) =      -0.0805437598688000
  dip_coef( 874) =       0.0492100646125900
  dip_coef( 875) =      -0.0901445115784700
  dip_coef( 876) =       0.2259387413460000
  dip_coef( 877) =      -0.0760885647120300
  dip_coef( 878) =      -0.0369621351286400
  dip_coef( 879) =       0.5387639387700000
  dip_coef( 880) =       0.6896962064220000
  dip_coef( 881) =       0.5230590457950000
  dip_coef( 882) =      -0.1765441319020000
  dip_coef( 883) =       0.0369209864374300
  dip_coef( 884) =       0.1114409296970000
  dip_coef( 885) =      -0.0250327694962100
  dip_coef( 886) =      -0.8993324870970000
  dip_coef( 887) =      -0.3099893778280000
  dip_coef( 888) =       0.1884022322670000
  dip_coef( 889) =       0.2031221181030000
  dip_coef( 890) =      -0.0153268439913600
  dip_coef( 891) =       0.2008084003330000
  dip_coef( 892) =      -1.0796133076700001
  dip_coef( 893) =      -1.7271937803899999
  dip_coef( 894) =       0.6175486191300000
  dip_coef( 895) =      -0.0211724078932600
  dip_coef( 896) =      -0.2034430878350000
  dip_coef( 897) =       0.6650935390400000
  dip_coef( 898) =       0.0163272110968800
  dip_coef( 899) =       0.2848843654970000
  dip_coef( 900) =      -0.2329904730410000
  dip_coef( 901) =       0.0098197606595810
  dip_coef( 902) =       0.0056876578959990
  dip_coef( 903) =       0.4259051543070000
  dip_coef( 904) =      -0.0078726051406930
  dip_coef( 905) =       0.0086036895639580
  dip_coef( 906) =       0.0144937412991700
  dip_coef( 907) =       0.3765500861240000
  dip_coef( 908) =      -0.5971365328970000
  dip_coef( 909) =      -0.1867047168270000
  dip_coef( 910) =       0.2342409551710000
  dip_coef( 911) =       0.4569061659120000
  dip_coef( 912) =       0.0554472440683900
  dip_coef( 913) =      -0.5566531668410000
  dip_coef( 914) =      -0.1173924374870000
  dip_coef( 915) =      -0.0331714497972300
  dip_coef( 916) =       0.2621544453720000
  dip_coef( 917) =       0.0927195403598300
  dip_coef( 918) =      -0.0149727437133600
  dip_coef( 919) =      -0.1991810887000000
  dip_coef( 920) =       0.0468332093901100
  dip_coef( 921) =      -0.1473978267600000
  dip_coef( 922) =      -0.0350860006970400
  dip_coef( 923) =       0.0027112885477270
  dip_coef( 924) =       0.0297109339804200
  dip_coef( 925) =      -0.1014584272530000
  dip_coef( 926) =       0.1856916701790000
  dip_coef( 927) =      -0.2233031092400000
  dip_coef( 928) =       0.1212269335330000
  dip_coef( 929) =       0.1591692470280000
  dip_coef( 930) =      -0.2600661900270000
  dip_coef( 931) =       0.0852512005408900
  dip_coef( 932) =      -0.2175947629620000
  dip_coef( 933) =      -2.3643549456900002
  dip_coef( 934) =      -0.9110211647300001
  dip_coef( 935) =       0.4036454115580000
  dip_coef( 936) =      -0.1454834216980000
  dip_coef( 937) =       0.0399433332029500
  dip_coef( 938) =       0.6756630352450000
  dip_coef( 939) =       1.0933778274099999
  dip_coef( 940) =       1.4272291048400001
  dip_coef( 941) =      -0.9731884838660000
  dip_coef( 942) =       0.4326921278070000
  dip_coef( 943) =       0.0341993724350300
  dip_coef( 944) =      -0.6435131945180000
  dip_coef( 945) =      -0.3927947255800000
  dip_coef( 946) =      -0.2286143917140000
  dip_coef( 947) =       0.7895505049420000
  dip_coef( 948) =      -0.0700464711281300
  dip_coef( 949) =      -0.0572531439495000
  dip_coef( 950) =       0.1167029134880000
  dip_coef( 951) =       2.3786184819999998
  dip_coef( 952) =      -0.3907998677540000
  dip_coef( 953) =      -0.1087178670530000
  dip_coef( 954) =       0.0254424881683100
  dip_coef( 955) =       0.2726504689070000
  dip_coef( 956) =       0.1048390212340000
  dip_coef( 957) =       0.0571257925974600
  dip_coef( 958) =       0.2065602023450000
  dip_coef( 959) =      -0.1428229409210000
  dip_coef( 960) =       0.6318177723070000
  dip_coef( 961) =      -2.0722893840999999
  dip_coef( 962) =      -1.5790665448300001
  dip_coef( 963) =       0.4828938322610000
  dip_coef( 964) =       0.2400720545330000
  dip_coef( 965) =      -1.0378900615000000
  dip_coef( 966) =      -2.6580784150699999
  dip_coef( 967) =      -0.7489350070260000
  dip_coef( 968) =      -0.3823740958280000
  dip_coef( 969) =      -0.2987586266880000
  dip_coef( 970) =      -0.1665088327460000
  dip_coef( 971) =      -0.2219362706230000
  dip_coef( 972) =       0.6160617383060000
  dip_coef( 973) =      -0.5157719676950000
  dip_coef( 974) =       0.5170115083100000
  dip_coef( 975) =      -0.3904055901260000
  dip_coef( 976) =      -0.2921899990880000
  dip_coef( 977) =      -0.0879356350062200
  dip_coef( 978) =       0.2542936037110000
  dip_coef( 979) =      -0.1338538998940000
  dip_coef( 980) =       0.7426369149420000
  dip_coef( 981) =      -0.5868475868490000
  dip_coef( 982) =       0.0137200843322500
  dip_coef( 983) =      -1.0315665195499999
  dip_coef( 984) =      -0.5478532506470000
  dip_coef( 985) =       1.0738862313999999
  dip_coef( 986) =      -0.0528612405274900
  dip_coef( 987) =      -0.0624753499911600
  dip_coef( 988) =       0.2908844109010000
  dip_coef( 989) =       0.8425875148480000
  dip_coef( 990) =      -0.1174444192290000
  dip_coef( 991) =       0.2739257234880000
  dip_coef( 992) =       0.2392822804100000
  dip_coef( 993) =      -0.0314398906753800
  dip_coef( 994) =      -0.2741112623850000
  dip_coef( 995) =       0.0131664831048500
  dip_coef( 996) =       0.0021361222188280
  dip_coef( 997) =      -0.1643548750140000
  dip_coef( 998) =      -0.0052653993720430
  dip_coef( 999) =      -1.1293352215300001
  dip_coef(1000) =       0.7096464308790000
  dip_coef(1001) =       0.5617504478410000
  dip_coef(1002) =      -0.6327726517300000
  dip_coef(1003) =      -0.4089091413020000
  dip_coef(1004) =       0.1949932818620000
  dip_coef(1005) =       0.8953412332450000
  dip_coef(1006) =       0.1443398363970000
  dip_coef(1007) =      -0.0046459371883370
  dip_coef(1008) =      -0.5705300983379999
  dip_coef(1009) =      -0.1481609379590000
  dip_coef(1010) =      -0.0735282119578600
  dip_coef(1011) =      -0.0461155175815000
  dip_coef(1012) =       0.1965776170990000
  dip_coef(1013) =      -0.1556350930710000
  dip_coef(1014) =       0.0792177763251300
  dip_coef(1015) =      -0.5893114904580000
  dip_coef(1016) =       0.1694756999180000
  dip_coef(1017) =       0.3331094239810000
  dip_coef(1018) =      -0.0013379169515380
  dip_coef(1019) =       0.0983228351184800
  dip_coef(1020) =      -0.0084019134862590
  dip_coef(1021) =      -0.0771094326642700
  dip_coef(1022) =      -0.1443698438640000
  dip_coef(1023) =      -0.3855930543460000
  dip_coef(1024) =       0.1898282397030000
  dip_coef(1025) =      -0.0379045699623300
  dip_coef(1026) =       0.0062262635048540
  dip_coef(1027) =      -0.0050762341605510
  dip_coef(1028) =       0.0167107590954700
  dip_coef(1029) =       0.0260979235509800
  dip_coef(1030) =      -0.0186095846487500
  dip_coef(1031) =       0.7009004245050000
  dip_coef(1032) =       0.3887137966620000
  dip_coef(1033) =      -0.1104757078430000
  dip_coef(1034) =      -1.1856429560899999
  dip_coef(1035) =       0.1770070981640000
  dip_coef(1036) =       0.3573190908440000
  dip_coef(1037) =      -1.5608264149000000
  dip_coef(1038) =       0.2080166782310000
  dip_coef(1039) =      -0.4936779846320000
  dip_coef(1040) =       0.0793625957741600
  dip_coef(1041) =       4.5743884558000003
  dip_coef(1042) =       3.9213064063399998
  dip_coef(1043) =      -3.3672962174899999
  dip_coef(1044) =       0.3327958656780000
  dip_coef(1045) =       0.1582397150970000
  dip_coef(1046) =      -4.4813418422800000
  dip_coef(1047) =      -0.1783219447450000
  dip_coef(1048) =      -2.5170882171400000
  dip_coef(1049) =       1.5262299691700001
  dip_coef(1050) =      -1.4811235912700000
  dip_coef(1051) =       0.3077322979170000
  dip_coef(1052) =       0.2269548493910000
  dip_coef(1053) =      -0.4135588667930000
  dip_coef(1054) =      -3.0842869443000001
  dip_coef(1055) =       0.5009474882279999
  dip_coef(1056) =      -1.8044563663300000
  dip_coef(1057) =       0.9676540047180000
  dip_coef(1058) =       0.4888277502220000
  dip_coef(1059) =      -0.1769145871950000
  dip_coef(1060) =       0.0833393158019000
  dip_coef(1061) =       2.4808567360199998
  dip_coef(1062) =      -0.3282948528700000
  dip_coef(1063) =       1.1298966793200000
  dip_coef(1064) =      -1.3891047477900000
  dip_coef(1065) =       0.0215808591809700
  dip_coef(1066) =      -0.0187093587765400
  dip_coef(1067) =      -0.2666191562440000
  dip_coef(1068) =       0.1291385660730000
  dip_coef(1069) =      -0.2246052647490000
  dip_coef(1070) =       0.2867493921130000
  dip_coef(1071) =      -0.0106747376559200
  dip_coef(1072) =      -0.2071128008740000
  dip_coef(1073) =      -0.1569847110200000
  dip_coef(1074) =       0.1105980628350000
  dip_coef(1075) =      -0.0866459968661800
  dip_coef(1076) =      -0.1387932143250000
  dip_coef(1077) =      -1.8732249146900000
  dip_coef(1078) =      -0.0222291098753700
  dip_coef(1079) =      -0.0130449760073500
  dip_coef(1080) =       2.8905354592800001
  dip_coef(1081) =       0.1259705117000000
  dip_coef(1082) =      -0.3158301178940000
  dip_coef(1083) =       1.7820485834899999
  dip_coef(1084) =      -0.4679156410810000
  dip_coef(1085) =       0.6141928231670000
  dip_coef(1086) =       0.2844957860710000
  dip_coef(1087) =       0.1022805388650000
  dip_coef(1088) =      -0.5109806889239999
  dip_coef(1089) =      -0.0030595525186210
  dip_coef(1090) =       0.0822546559877200
  dip_coef(1091) =      -0.1747859076010000
  dip_coef(1092) =       0.2344115465000000
  dip_coef(1093) =       0.0511803714976800
  dip_coef(1094) =       0.2517564085510000
  dip_coef(1095) =      -0.5435046629540000
  dip_coef(1096) =      -0.0851500131259700
  dip_coef(1097) =      -0.0335723375349300
  dip_coef(1098) =      -0.1073903979890000
  dip_coef(1099) =      -0.0235942519408500
  dip_coef(1100) =       0.6138823222310000
  dip_coef(1101) =       0.1728329948660000
  dip_coef(1102) =       0.0187200346928000
  dip_coef(1103) =       0.4540932244850000
  dip_coef(1104) =       0.0690105723385000
  dip_coef(1105) =       0.0361670860806000
  dip_coef(1106) =      -0.1483628443640000
  dip_coef(1107) =       0.1214798196190000
  dip_coef(1108) =      -0.0586135492369100
  dip_coef(1109) =      -0.1290094399920000
  dip_coef(1110) =      -0.8254694585180000
  dip_coef(1111) =      -0.0602237604782500
  dip_coef(1112) =      -0.0085649007046080
  dip_coef(1113) =       0.1088119553860000
  dip_coef(1114) =      -0.0090627857300730
  dip_coef(1115) =       0.0735081554751300
  dip_coef(1116) =      -0.0199496466201300
  dip_coef(1117) =      -0.0074714885564220
  dip_coef(1118) =       0.1095185268310000
  dip_coef(1119) =      -0.1325874674420000
  dip_coef(1120) =       0.0787931256681800
  dip_coef(1121) =       0.2020712218060000
  dip_coef(1122) =      -0.0464958374868500
  dip_coef(1123) =       0.0895081416003200
  dip_coef(1124) =      -0.1135191956200000
  dip_coef(1125) =      -0.2146167289250000
  dip_coef(1126) =       0.1348016763360000
  dip_coef(1127) =      -0.0641706962268600
  dip_coef(1128) =      -0.0399137147426500
  dip_coef(1129) =       0.3362077684520000
  dip_coef(1130) =       0.0277851709200100
  dip_coef(1131) =       0.0904889168945400
  dip_coef(1132) =       0.1722114553770000
  dip_coef(1133) =       0.0644720676362200
  dip_coef(1134) =      -0.1235353934640000
  dip_coef(1135) =      -0.0545088061282700
  dip_coef(1136) =      -0.0747926845763200
  dip_coef(1137) =       0.1538258968710000
  dip_coef(1138) =      -0.1096259677500000
  dip_coef(1139) =       0.0997758833646400
  dip_coef(1140) =      -0.0778013373425200
  dip_coef(1141) =      -0.2546221725110000
  dip_coef(1142) =       0.0712492849315400
  dip_coef(1143) =      -0.1420309747830000
  dip_coef(1144) =      -0.0857894321423700
  dip_coef(1145) =      -0.2264976867700000
  dip_coef(1146) =       0.3484581678220000
  dip_coef(1147) =      -0.1609085460160000
  dip_coef(1148) =      -0.2137716824940000
  dip_coef(1149) =       0.2863481663710000
  dip_coef(1150) =       0.0715339265491000
  dip_coef(1151) =     -90.8759095059999993
  dip_coef(1152) =      -1.4139954312600000
  dip_coef(1153) =      33.4336182902999965
  dip_coef(1154) =       4.0557498989700003
  dip_coef(1155) =      -0.1044154663280000
  dip_coef(1156) =       3.1042124404900000
  dip_coef(1157) =       0.7388176074500000
  dip_coef(1158) =     -13.5885723485999996
  dip_coef(1159) =      -2.6993549718000001
  dip_coef(1160) =       3.9837952703799999
  dip_coef(1161) =      -1.9929389468200001
  dip_coef(1162) =      -7.9833270443700002
  dip_coef(1163) =      47.2172224872999990
  dip_coef(1164) =      24.7612309418000009
  dip_coef(1165) =      15.1849760992000000
  dip_coef(1166) =       5.4331407868900001
  dip_coef(1167) =      -1.7264470819700000
  dip_coef(1168) =      17.3023588576000016
  dip_coef(1169) =      -1.7321391464700000
  dip_coef(1170) =     -34.4345549909999988
  dip_coef(1171) =       9.6131435842300004
  dip_coef(1172) =       0.0512095373544100
  dip_coef(1173) =       0.2722514810670000
  dip_coef(1174) =      -1.9528980962900000
  dip_coef(1175) =      -0.3817131509890000
  dip_coef(1176) =       1.3450109629700000
  dip_coef(1177) =       1.9867198504900001
  dip_coef(1178) =      -0.0731458833854000
  dip_coef(1179) =       2.5769284601100000
  dip_coef(1180) =       3.8878039628100001
  dip_coef(1181) =      -2.1825720276000000
  dip_coef(1182) =      -0.2656748004380000
  dip_coef(1183) =       0.1348360670700000
  dip_coef(1184) =      -0.5005888213450000
  dip_coef(1185) =     -14.6107171081999994
  dip_coef(1186) =       2.4821989590200002
  dip_coef(1187) =      -2.4689464914000001
  dip_coef(1188) =       0.7785242985320000
  dip_coef(1189) =       2.8750430413900001
  dip_coef(1190) =      -3.0848840045900001
  dip_coef(1191) =       6.8131465482499998
  dip_coef(1192) =       1.8959575725900000
  dip_coef(1193) =       6.2877309518200004
  dip_coef(1194) =     -18.9076626808000015
  dip_coef(1195) =       0.9332704985950000
  dip_coef(1196) =       2.0837689828500001
  dip_coef(1197) =      -2.2903501639999999
  dip_coef(1198) =      -0.1053568983470000
  dip_coef(1199) =     -14.2702725258999994
  dip_coef(1200) =     -10.8931608696000009
  dip_coef(1201) =      15.3485322454999995
  dip_coef(1202) =      -5.9702066622299998
  dip_coef(1203) =     -10.6394375319000005
  dip_coef(1204) =      -0.7471349443999999
  dip_coef(1205) =      -4.7970325168799999
  dip_coef(1206) =      -4.6073135581100004
  dip_coef(1207) =      -2.6815664082000001
  dip_coef(1208) =       5.1734190042700003
  dip_coef(1209) =      -0.7630652123630000
  dip_coef(1210) =       4.0890779835600002
  dip_coef(1211) =      21.4351463548999988
  dip_coef(1212) =       3.6931840132099998
  dip_coef(1213) =       1.5212963080299999
  dip_coef(1214) =      -5.0208733292199996
  dip_coef(1215) =       0.0818256376945000
  dip_coef(1216) =      -2.6525004477500000
  dip_coef(1217) =     -22.0536487174000015
  dip_coef(1218) =       0.6428927088780000
  dip_coef(1219) =      -1.5555875338300000
  dip_coef(1220) =      -5.2235988206400004
  dip_coef(1221) =       5.4988516206900000
  dip_coef(1222) =       1.5172409762700001
  dip_coef(1223) =      -0.2644888233720000
  dip_coef(1224) =     -11.2699301341000009
  dip_coef(1225) =       6.3764737331900001
  dip_coef(1226) =       7.6474317764600004
  dip_coef(1227) =      -7.0146010768000000
  dip_coef(1228) =      -2.3636448646199999
  dip_coef(1229) =      -1.4185106649699999
  dip_coef(1230) =       2.1709923916600000
  dip_coef(1231) =       0.2770274920260000
  dip_coef(1232) =      -0.8307622526370000
  dip_coef(1233) =       7.9286931909199998
  dip_coef(1234) =      -4.5850750712400004
  dip_coef(1235) =       1.6279070329800001
  dip_coef(1236) =      -3.6995639096700002
  dip_coef(1237) =      -0.2369479497400000
  dip_coef(1238) =       2.9087550102100002
  dip_coef(1239) =       3.8352214527199999
  dip_coef(1240) =       2.2894181340299999
  dip_coef(1241) =       5.8414124634600002
  dip_coef(1242) =       0.0548681222681800
  dip_coef(1243) =      -0.0481144127839600
  dip_coef(1244) =       0.0032230663255480
  dip_coef(1245) =      -0.1341830220750000
  dip_coef(1246) =       0.5565798884040000
  dip_coef(1247) =      -0.0234512764354600
  dip_coef(1248) =       0.0005569461851782
  dip_coef(1249) =       0.0353516374908200
  dip_coef(1250) =       0.7205306969940000
  dip_coef(1251) =      -0.1140251676150000
  dip_coef(1252) =      -0.3675140046290000
  dip_coef(1253) =       0.0123247901874700
  dip_coef(1254) =      -0.0532971067280400
  dip_coef(1255) =       0.7149786488010000
  dip_coef(1256) =      -0.1262069743780000
  dip_coef(1257) =       0.0944650325760800
  dip_coef(1258) =       0.2331727231580000
  dip_coef(1259) =       0.3510958268030000
  dip_coef(1260) =      -0.0195084515493900
  dip_coef(1261) =      -1.1553884938000001
  dip_coef(1262) =       0.3333017320650000
  dip_coef(1263) =      -0.5701577756960000
  dip_coef(1264) =      -2.2424348788700001
  dip_coef(1265) =      -1.2955730191799999
  dip_coef(1266) =       0.9647386131580000
  dip_coef(1267) =      -0.2241206830410000
  dip_coef(1268) =       0.0941002995945000
  dip_coef(1269) =       4.1334662129100002
  dip_coef(1270) =      -0.7738708694290000
  dip_coef(1271) =       0.7647783414490000
  dip_coef(1272) =      -1.8234492057700000
  dip_coef(1273) =       0.6228591327630000
  dip_coef(1274) =       0.0779260644151700
  dip_coef(1275) =      -0.7611970590300000
  dip_coef(1276) =       0.4659530424880000
  dip_coef(1277) =       2.6248518463599999
  dip_coef(1278) =      -1.0771061399199999
  dip_coef(1279) =       0.1497935894070000
  dip_coef(1280) =      -0.1146693499380000
  dip_coef(1281) =      -0.4978548818920000
  dip_coef(1282) =       0.3001576553630000
  dip_coef(1283) =      -1.2873195351200000
  dip_coef(1284) =       1.7884781960300000
  dip_coef(1285) =      -0.0422780650233700
  dip_coef(1286) =       0.7545405537250000
  dip_coef(1287) =      -3.7191262209799998
  dip_coef(1288) =       0.0377354257435400
  dip_coef(1289) =      -0.1572826050580000
  dip_coef(1290) =       0.5679388706260000
  dip_coef(1291) =      -1.0396939102000000
  dip_coef(1292) =       4.2027871321700001
  dip_coef(1293) =       0.7602656950020000
  dip_coef(1294) =       0.5075919915450000
  dip_coef(1295) =      -2.2312746267799999
  dip_coef(1296) =       0.2867740221580000
  dip_coef(1297) =       4.3432316625600000
  dip_coef(1298) =       0.7213279012379999
  dip_coef(1299) =      -0.0387047806120200
  dip_coef(1300) =       0.0666992348915700
  dip_coef(1301) =      -0.0475652702458600
  dip_coef(1302) =       0.2913388085820000
  dip_coef(1303) =       0.4489597930210000
  dip_coef(1304) =      -0.0617218602442900
  dip_coef(1305) =       0.3492107005430000
  dip_coef(1306) =       0.1508614677490000
  dip_coef(1307) =      -3.0994958413500000
  dip_coef(1308) =      -1.1268602835699999
  dip_coef(1309) =       0.1927269212730000
  dip_coef(1310) =      -0.4840051390880000
  dip_coef(1311) =       0.5864548460190000
  dip_coef(1312) =       0.5989892302360000
  dip_coef(1313) =      -0.5956258800200001
  dip_coef(1314) =      -1.7933039584599999
  dip_coef(1315) =      -0.7205707879140000
  dip_coef(1316) =       0.3508973538220000
  dip_coef(1317) =       3.0922530067800000
  dip_coef(1318) =       1.7948617919600001
  dip_coef(1319) =      -0.3087952618930000
  dip_coef(1320) =      -0.0216629024936800
  dip_coef(1321) =       0.5647965470060000
  dip_coef(1322) =     -10.6732654643000000
  dip_coef(1323) =       2.3249838694200000
  dip_coef(1324) =      -3.5207790085899999
  dip_coef(1325) =       4.2592023685699996
  dip_coef(1326) =       2.0613973794999998
  dip_coef(1327) =       0.0337601393561000
  dip_coef(1328) =      -0.0009904138041901
  dip_coef(1329) =       4.1318360467300002
  dip_coef(1330) =      -3.2186275472300001
  dip_coef(1331) =      -3.5315976180300002
  dip_coef(1332) =       0.8260217185310000
  dip_coef(1333) =      -1.2128127733800000
  dip_coef(1334) =       2.7242420370699998
  dip_coef(1335) =       0.9597328088620000
  dip_coef(1336) =       0.7347500233380000
  dip_coef(1337) =      -0.8522227876850000
  dip_coef(1338) =       0.0819987970351600
  dip_coef(1339) =       0.4175810965820000
  dip_coef(1340) =       7.7628407202199998
  dip_coef(1341) =      -0.0796730855622900
  dip_coef(1342) =      -0.0052531600700280
  dip_coef(1343) =      -1.5112355055000000
  dip_coef(1344) =      -6.9097721970599997
  dip_coef(1345) =       2.3634309175900001
  dip_coef(1346) =       2.5352055978300001
  dip_coef(1347) =      -3.4578343884499998
  dip_coef(1348) =      -5.8686258727200000
  dip_coef(1349) =      -0.7992227174960000
  dip_coef(1350) =       3.3153048687900002
  dip_coef(1351) =       0.6313543690800000
  dip_coef(1352) =       1.1676273081399999
  dip_coef(1353) =      -1.1392794066700000
  dip_coef(1354) =       0.0970161652037100
  dip_coef(1355) =       0.2036942348700000
  dip_coef(1356) =      -2.4586161902799999
  dip_coef(1357) =       1.9830005614299999
  dip_coef(1358) =      -2.1523176005300000
  dip_coef(1359) =       1.8548077505900000
  dip_coef(1360) =       8.7370133952800000
  dip_coef(1361) =       0.9345774817630000
  dip_coef(1362) =      -1.8039423826400001
  dip_coef(1363) =      -1.1300852286900001
  dip_coef(1364) =      -3.2817293339000000
  dip_coef(1365) =      -0.5086136998730000
  dip_coef(1366) =       0.0622565460452100
  dip_coef(1367) =       0.7135814561010000
  dip_coef(1368) =       5.5854936796699999
  dip_coef(1369) =       4.5305162740200000
  dip_coef(1370) =      -2.3455132964100001
  dip_coef(1371) =       0.4122392751840000
  dip_coef(1372) =       0.3396568971400000
  dip_coef(1373) =      -4.0582796150600000
  dip_coef(1374) =      -0.6822575215030000
  dip_coef(1375) =      -0.0674803202575200
  dip_coef(1376) =      -0.0713418875201100
  dip_coef(1377) =      -0.1675424321080000
  dip_coef(1378) =      -1.8370982077800000
  dip_coef(1379) =      -1.0535632700899999
  dip_coef(1380) =      -0.3601132771890000
  dip_coef(1381) =      -0.4781252309810000
  dip_coef(1382) =       0.3758698042650000
  dip_coef(1383) =       3.0117366665000000
  dip_coef(1384) =      -4.2351931880400002
  dip_coef(1385) =      -2.5479430234199998
  dip_coef(1386) =       3.3145234852800001
  dip_coef(1387) =       2.2373371534099999
  dip_coef(1388) =      -1.2863448817200001
  dip_coef(1389) =      -6.3362159055599996
  dip_coef(1390) =      -0.0597207752351200
  dip_coef(1391) =      -0.3709363538510000
  dip_coef(1392) =      -0.1617272226780000
  dip_coef(1393) =       0.3551167037580000
  dip_coef(1394) =      -0.3078634862810000
  dip_coef(1395) =      -0.6551578542760000
  dip_coef(1396) =       0.2359799943970000
  dip_coef(1397) =      -0.9554322561710000
  dip_coef(1398) =       0.3061369075010000
  dip_coef(1399) =       0.6648411591830000
  dip_coef(1400) =      -1.8612122715899999
  dip_coef(1401) =       0.5209355053360000
  dip_coef(1402) =       1.2365959486200000
  dip_coef(1403) =      -2.0897328795700001
  dip_coef(1404) =       0.0241486619980400
  dip_coef(1405) =      -0.1944259634140000
  dip_coef(1406) =       1.1785030034299999
  dip_coef(1407) =       1.4022104580100001
  dip_coef(1408) =      -0.7304713734920000
  dip_coef(1409) =       0.1745619660290000
  dip_coef(1410) =       0.0299118760039800
  dip_coef(1411) =       0.0085512658937150
  dip_coef(1412) =      -0.0710812580602500
  dip_coef(1413) =      -0.2540265212700000
  dip_coef(1414) =       0.4222591976930000
  dip_coef(1415) =      -1.5191544357000000
  dip_coef(1416) =      -0.8875915816310000
  dip_coef(1417) =       0.2878176032020000
  dip_coef(1418) =       2.3239436069999999
  dip_coef(1419) =      -0.3579241230170000
  dip_coef(1420) =      -0.8539048487100001
  dip_coef(1421) =       7.8533579949899996
  dip_coef(1422) =      -1.8299427719400001
  dip_coef(1423) =       0.3323420135610000
  dip_coef(1424) =       0.6737962192110000
  dip_coef(1425) =      -8.1835613736199999
  dip_coef(1426) =     -18.3881413691999995
  dip_coef(1427) =       1.4655338386000001
  dip_coef(1428) =      -1.4862403342399999
  dip_coef(1429) =      -1.6229288655100000
  dip_coef(1430) =      -0.5475637207900000
  dip_coef(1431) =      -0.3583013840470000
  dip_coef(1432) =      10.3039417715999999
  dip_coef(1433) =      -6.0323819758299999
  dip_coef(1434) =       7.4787332695200002
  dip_coef(1435) =       1.1132312768299999
  dip_coef(1436) =      -0.6005594240940000
  dip_coef(1437) =      -0.4444961821270000
  dip_coef(1438) =      15.1460225142000002
  dip_coef(1439) =      -4.6250824836700000
  dip_coef(1440) =       7.4639595144299999
  dip_coef(1441) =      -1.1669948293700001
  dip_coef(1442) =      -0.3350231603420000
  dip_coef(1443) =       0.0048891782324430
  dip_coef(1444) =       1.5713251335900000
  dip_coef(1445) =      -2.5063794208400001
  dip_coef(1446) =       2.6772050884900001
  dip_coef(1447) =      -0.3065180186440000
  dip_coef(1448) =       0.6158586347700000
  dip_coef(1449) =      -0.0843068875729500
  dip_coef(1450) =       0.0380962104080300
  dip_coef(1451) =       0.2958243550250000
  dip_coef(1452) =      -0.2953175401890000
  dip_coef(1453) =       1.2424445383500000
  dip_coef(1454) =      -1.2423123324400001
  dip_coef(1455) =      -0.0791447039524900
  dip_coef(1456) =       2.5816773343100001
  dip_coef(1457) =       0.2386686998000000
  dip_coef(1458) =      -0.4382580548710000
  dip_coef(1459) =       0.5450521086130000
  dip_coef(1460) =       0.8876037665680000
  dip_coef(1461) =      12.4523816303999997
  dip_coef(1462) =       0.0923499729931700
  dip_coef(1463) =       0.0777602779640600
  dip_coef(1464) =      -7.8511202535900004
  dip_coef(1465) =      -0.3972574677510000
  dip_coef(1466) =       0.5722425427330000
  dip_coef(1467) =      -2.1864414264300001
  dip_coef(1468) =       1.2352193235200000
  dip_coef(1469) =      -4.1433535477500003
  dip_coef(1470) =      -5.3054069710500000
  dip_coef(1471) =      -2.9871951536900001
  dip_coef(1472) =       3.3876044849200002
  dip_coef(1473) =       0.0400669575684100
  dip_coef(1474) =      -0.6565336755040000
  dip_coef(1475) =      -0.2818897198390000
  dip_coef(1476) =      -1.3374898281800001
  dip_coef(1477) =      -0.4196069820300000
  dip_coef(1478) =      -2.1445037000300000
  dip_coef(1479) =       3.3294247741300000
  dip_coef(1480) =       0.7287104044760000
  dip_coef(1481) =      -0.0687824552540200
  dip_coef(1482) =      -0.0703954512478900
  dip_coef(1483) =      -0.9378096091660000
  dip_coef(1484) =      -2.5754492275700001
  dip_coef(1485) =      -1.1277646190799999
  dip_coef(1486) =      -0.3289739658200000
  dip_coef(1487) =      -3.6586801790300001
  dip_coef(1488) =      -0.7836829929419999
  dip_coef(1489) =      -0.3859647067500000
  dip_coef(1490) =      -0.0075157945826990
  dip_coef(1491) =      -2.9706188670400002
  dip_coef(1492) =       0.0218057906969800
  dip_coef(1493) =      -0.8974282242160000
  dip_coef(1494) =       4.8550427220300003
  dip_coef(1495) =      -0.2311034102790000
  dip_coef(1496) =      -0.0262326686418600
  dip_coef(1497) =      -0.1480940831860000
  dip_coef(1498) =       0.1868263532120000
  dip_coef(1499) =      -0.2586504338280000
  dip_coef(1500) =       0.0260484557511500
  dip_coef(1501) =      -0.0142119362678800
  dip_coef(1502) =      -0.1483135019640000
  dip_coef(1503) =       1.3175956537200000
  dip_coef(1504) =      -0.7878744938859999
  dip_coef(1505) =      -1.5712422377399999
  dip_coef(1506) =       0.5320114774400000
  dip_coef(1507) =      -0.7780227710880000
  dip_coef(1508) =       0.8494045428660000
  dip_coef(1509) =       0.4552618115460000
  dip_coef(1510) =       0.7386676164660000
  dip_coef(1511) =       0.5178349724360000
  dip_coef(1512) =       0.4114479559080000
  dip_coef(1513) =      -1.8307876750000001
  dip_coef(1514) =      -0.0571289970430700
  dip_coef(1515) =      -0.6656543700970000
  dip_coef(1516) =      -1.7486348338600000
  dip_coef(1517) =      -0.0052710115756680
  dip_coef(1518) =       0.3205599015350000
  dip_coef(1519) =       0.2642410411640000
  dip_coef(1520) =       0.2710730012160000
  dip_coef(1521) =      -1.5330024179899999
  dip_coef(1522) =       0.6577679823640000
  dip_coef(1523) =      -0.1863823644250000
  dip_coef(1524) =       0.0549773410667500
  dip_coef(1525) =       0.9329066852490000
  dip_coef(1526) =       0.3861820818220000
  dip_coef(1527) =       1.2341647268400000
  dip_coef(1528) =       0.3007291770960000
  dip_coef(1529) =       1.1112863511200000
  dip_coef(1530) =      -4.2295056026599998
  dip_coef(1531) =       1.6846199976900000
  dip_coef(1532) =       1.3078496254100000
  dip_coef(1533) =       0.9388959290320000
  dip_coef(1534) =      -0.3089458116290000
  dip_coef(1535) =      -0.0025662237914980
  dip_coef(1536) =      -0.0044268153933630
  dip_coef(1537) =       0.0079927298413810
  dip_coef(1538) =       0.0358527067562700
  dip_coef(1539) =      -0.0476236681137900
  dip_coef(1540) =       0.0019714899927390
  dip_coef(1541) =      -0.0021796182768780
  dip_coef(1542) =      -0.0201454193289800
  dip_coef(1543) =      -0.2898631453760000
  dip_coef(1544) =      -0.1192912894010000
  dip_coef(1545) =       0.0629021895117800
  dip_coef(1546) =      -0.0052017400638860
  dip_coef(1547) =      -0.0118405107108100
  dip_coef(1548) =       0.2287938813190000
  dip_coef(1549) =      -0.0503834909745400
  dip_coef(1550) =       0.0809283448594900
  dip_coef(1551) =      -0.0679294020774100
  dip_coef(1552) =       0.0298706567113700
  dip_coef(1553) =      -0.3381352018940000
  dip_coef(1554) =       0.3332556670170000
  dip_coef(1555) =      -0.0044775471664980
  dip_coef(1556) =      -0.0315317534013600
  dip_coef(1557) =       0.1670738480580000
  dip_coef(1558) =       0.0119074413467100
  dip_coef(1559) =      -0.0502833231691700
  dip_coef(1560) =       0.0682461803026000
  dip_coef(1561) =      -0.0690292201955100
  dip_coef(1562) =      -0.1302193028020000
  dip_coef(1563) =       0.0722223759453900
  dip_coef(1564) =      -0.1253013526830000
  dip_coef(1565) =       0.1486774503310000
  dip_coef(1566) =      -0.1015739132120000
  dip_coef(1567) =       0.0108469538135400
  dip_coef(1568) =       0.0960480676596300
  dip_coef(1569) =       1.0212133816100000
  dip_coef(1570) =       0.7837414990200000
  dip_coef(1571) =      -0.1932998239630000
  dip_coef(1572) =       0.0150915240375800
  dip_coef(1573) =       0.1158348255450000
  dip_coef(1574) =      -0.8507002347820000
  dip_coef(1575) =       0.1992207111390000
  dip_coef(1576) =      -0.3442009616030000
  dip_coef(1577) =      -0.0009927386005289
  dip_coef(1578) =       0.0085573460679880
  dip_coef(1579) =      -0.0156647348342400
  dip_coef(1580) =       0.7458801964510000
  dip_coef(1581) =      -0.0170857231274200
  dip_coef(1582) =       0.0838930417989100
  dip_coef(1583) =      -0.0112046803704200
  dip_coef(1584) =       0.0750111704783400
  dip_coef(1585) =      -0.4624985501330000
  dip_coef(1586) =      -0.1141561083990000
  dip_coef(1587) =       0.0772779431126300
  dip_coef(1588) =       0.1439077548380000
  dip_coef(1589) =      -0.1036971626940000
  dip_coef(1590) =      -0.5642234475650000
  dip_coef(1591) =      -0.1212338467790000
  dip_coef(1592) =      -0.0060723280306310
  dip_coef(1593) =      -0.0264891674804400
  dip_coef(1594) =      -0.0245908643224100
  dip_coef(1595) =       0.0247777116446700
  dip_coef(1596) =      -0.1542152935240000
  dip_coef(1597) =       0.0464203868619600
  dip_coef(1598) =       0.0407308218484800
  dip_coef(1599) =      -0.0015728861482770
  dip_coef(1600) =       0.2875091780810000
  dip_coef(1601) =       0.1612081673890000
  dip_coef(1602) =      -0.1754318984750000
  dip_coef(1603) =      -0.1289272221040000
  dip_coef(1604) =      -0.1417132016240000
  dip_coef(1605) =      -0.0730774479640800
  dip_coef(1606) =       0.1331695868390000
  dip_coef(1607) =       0.9184597999770000
  dip_coef(1608) =      -0.0728451780187800
  dip_coef(1609) =      -0.3878376161320000
  dip_coef(1610) =       0.2799583974040000
  dip_coef(1611) =      -0.4269205535400000
  dip_coef(1612) =      -0.1758308514050000
  dip_coef(1613) =       0.0320625717243700
  dip_coef(1614) =      -0.1672424298220000
  dip_coef(1615) =      -0.1137779941300000
  dip_coef(1616) =       0.1619438338050000
  dip_coef(1617) =      -0.3579495551960000
  dip_coef(1618) =       0.1463623019150000
  dip_coef(1619) =      -1.1541491150900001
  dip_coef(1620) =      -0.0723986159042900
  dip_coef(1621) =       0.2533968778060000
  dip_coef(1622) =      -1.3512444692000001
  dip_coef(1623) =       0.2894412831470000
  dip_coef(1624) =       0.9030067496710000
  dip_coef(1625) =      -0.4879722408020000
  dip_coef(1626) =       0.5622420462620000
  dip_coef(1627) =       0.5125684109030000
  dip_coef(1628) =      -0.4652945636800000
  dip_coef(1629) =       1.1767777876000001
  dip_coef(1630) =      -0.6088638095780000
  dip_coef(1631) =       0.0009957990753650
  dip_coef(1632) =      -0.2891798572520000
  dip_coef(1633) =      -0.3072669046660000
  dip_coef(1634) =       0.0257318790093200
  dip_coef(1635) =      -0.0501373026046000
  dip_coef(1636) =      -0.0836534912494200
  dip_coef(1637) =      -0.2333091970600000
  dip_coef(1638) =      -0.2708925600540000
  dip_coef(1639) =      -0.3066757565550000
  dip_coef(1640) =       0.0807969115227600
  dip_coef(1641) =       0.2176829271750000
  dip_coef(1642) =      -0.1191925366590000
  dip_coef(1643) =      -0.3786225321530000
  dip_coef(1644) =       0.0498522826117900
  dip_coef(1645) =       0.3745031982650000
  dip_coef(1646) =      -0.0588745150733200
  dip_coef(1647) =       0.0418717870718200
  dip_coef(1648) =       0.2944703400090000
  dip_coef(1649) =       0.1728383754860000
  dip_coef(1650) =      -0.0532894493642700
  dip_coef(1651) =      -0.0055504254326700
  dip_coef(1652) =      -0.1625339768170000
  dip_coef(1653) =       0.0483174481129000
  dip_coef(1654) =       0.0756458897201700
  dip_coef(1655) =       0.3583584141710000
  dip_coef(1656) =       0.2023562254540000
  dip_coef(1657) =       0.1580943578620000
  dip_coef(1658) =       0.3788921602700000
  dip_coef(1659) =       0.0470314016358200
  dip_coef(1660) =      -0.1130735146260000
  dip_coef(1661) =      -1.8062376250900001
  dip_coef(1662) =      -2.2246154297100000
  dip_coef(1663) =       0.3977058754520000
  dip_coef(1664) =       0.0816773768202100
  dip_coef(1665) =      -0.5281133223600000
  dip_coef(1666) =       1.0210355329900000
  dip_coef(1667) =      -0.1823094481030000
  dip_coef(1668) =       0.2458658151780000
  dip_coef(1669) =      -0.0276468689531000
  dip_coef(1670) =      -0.0098307310757140
  dip_coef(1671) =       0.2092749863050000
  dip_coef(1672) =      -1.8496896059300001
  dip_coef(1673) =       0.1186499066530000
  dip_coef(1674) =      -0.2429807624560000
  dip_coef(1675) =       0.1792870464130000
  dip_coef(1676) =      -0.1300615537890000
  dip_coef(1677) =       0.6451299705340000
  dip_coef(1678) =       0.4988337490500000
  dip_coef(1679) =      -0.6074224157810000
  dip_coef(1680) =      -0.1153304035460000
  dip_coef(1681) =       0.6611643184480001
  dip_coef(1682) =       1.2234400096700000
  dip_coef(1683) =       0.1346721845100000
  dip_coef(1684) =       0.0021813849884350
  dip_coef(1685) =       0.1983675225260000
  dip_coef(1686) =       0.0004528606139281
  dip_coef(1687) =      -0.1374440078400000
  dip_coef(1688) =       0.5583744564310000
  dip_coef(1689) =      -0.3853167489740000
  dip_coef(1690) =       0.0851075751281400
  dip_coef(1691) =      -0.1918522572260000
  dip_coef(1692) =      -0.8014713472000000
  dip_coef(1693) =      -0.2279373994140000
  dip_coef(1694) =       0.2640972484680000
  dip_coef(1695) =       0.2113971550760000
  dip_coef(1696) =       0.7296073393490000
  dip_coef(1697) =       0.0023269957549690
  dip_coef(1698) =       0.1359130078560000
  dip_coef(1699) =       0.0089306815682670
  dip_coef(1700) =       0.0261942945453500
  dip_coef(1701) =       0.0053394750197350
  dip_coef(1702) =      -0.0000466563390906
  dip_coef(1703) =       0.0149451572049300
  dip_coef(1704) =      -0.2003492443980000
  dip_coef(1705) =       0.0510680363571200
  dip_coef(1706) =      -0.0059868861327180
  dip_coef(1707) =      -0.0678240049784600
  dip_coef(1708) =      -0.5677181829220000
  dip_coef(1709) =      -0.4849016407500000
  dip_coef(1710) =       0.3859498127380000
  dip_coef(1711) =      -0.0692380391101400
  dip_coef(1712) =       0.2066753495710000
  dip_coef(1713) =       0.0405429237185600
  dip_coef(1714) =       1.2653247775900001
  dip_coef(1715) =      -0.2473357182180000
  dip_coef(1716) =      -0.0617739127788900
  dip_coef(1717) =       0.3049393822630000
  dip_coef(1718) =      -1.0007264940900000
  dip_coef(1719) =       1.5551956518800001
  dip_coef(1720) =       0.4950831073240000
  dip_coef(1721) =      -0.0125507251407700
  dip_coef(1722) =       0.4821349310980000
  dip_coef(1723) =       1.3204170942800000
  dip_coef(1724) =      -0.4409259859050000
  dip_coef(1725) =       0.6771670614590000
  dip_coef(1726) =      -0.5953533647300000
  dip_coef(1727) =       0.8463838525650000
  dip_coef(1728) =      -0.2378248574860000
  dip_coef(1729) =      -0.1074147968960000
  dip_coef(1730) =       0.6019840925820000
  dip_coef(1731) =      -2.0933408390500001
  dip_coef(1732) =       0.1943747932950000
  dip_coef(1733) =       0.3301612268710000
  dip_coef(1734) =      -0.5709074897930000
  dip_coef(1735) =      -0.0255269133242400
  dip_coef(1736) =       0.0407862120878900
  dip_coef(1737) =      -0.1506940627090000
  dip_coef(1738) =       0.3909122595930000
  dip_coef(1739) =       0.1015184519700000
  dip_coef(1740) =      -0.5737145578780000
  dip_coef(1741) =       0.3983994981550000
  dip_coef(1742) =       0.0160770267942900
  dip_coef(1743) =      -0.0243826348725100
  dip_coef(1744) =       0.0131138045795900
  dip_coef(1745) =      -0.0789426915565800
  dip_coef(1746) =      -0.0302196740574000
  dip_coef(1747) =       0.0318471101651600
  dip_coef(1748) =       0.0871441239188700
  dip_coef(1749) =      -0.6593922100750000
  dip_coef(1750) =       0.0650178441446300
  dip_coef(1751) =      -0.0526403825878600
  dip_coef(1752) =      -0.1124886348680000
  dip_coef(1753) =      -0.2020847400670000
  dip_coef(1754) =      -1.0815744944700001
  dip_coef(1755) =       0.0712217193597100
  dip_coef(1756) =      -0.0218747805821700
  dip_coef(1757) =      -0.4304868875910000
  dip_coef(1758) =      -0.0099225088030530
  dip_coef(1759) =       0.2366820903720000
  dip_coef(1760) =      -1.7141713038099999
  dip_coef(1761) =       0.4475705143090000
  dip_coef(1762) =       0.0155823265173200
  dip_coef(1763) =       0.2037484984190000
  dip_coef(1764) =       0.0451524812313100
  dip_coef(1765) =       0.0955953185893500
  dip_coef(1766) =      -0.0148775284356200
  dip_coef(1767) =       0.0200550438514500
  dip_coef(1768) =       0.0032168738529690
  dip_coef(1769) =      -0.1257207385280000
  dip_coef(1770) =      -0.1341498276600000
  dip_coef(1771) =       0.1820154655900000
  dip_coef(1772) =      -0.2963110098130000
  dip_coef(1773) =      -0.1677551209360000
  dip_coef(1774) =      -0.1852698176800000
  dip_coef(1775) =      -0.0357723799484200
  dip_coef(1776) =       0.1022752346350000
  dip_coef(1777) =      -0.1092595067550000
  dip_coef(1778) =       0.0071549071029950
  dip_coef(1779) =       0.0155220081996200
  dip_coef(1780) =       0.0122693962249100
  dip_coef(1781) =      -0.0685582186573300
  dip_coef(1782) =       0.0884025212073000
  dip_coef(1783) =      -0.0440950076569000
  dip_coef(1784) =       0.3922557672330000
  dip_coef(1785) =      -0.0944255950635400
  dip_coef(1786) =       0.4446038659180000
  dip_coef(1787) =      -0.4803020569400000
  dip_coef(1788) =      -0.0247326509252300
  dip_coef(1789) =       0.0024799908913960
  dip_coef(1790) =      -0.1336597124430000
  dip_coef(1791) =       0.0020234957861560
  dip_coef(1792) =      -0.0419960993346100
  dip_coef(1793) =       0.0254674120472200
  dip_coef(1794) =       0.0207360581205800
  dip_coef(1795) =      -0.1494541876740000
  dip_coef(1796) =      -0.1440292774460000
  dip_coef(1797) =       0.1010949398070000
  dip_coef(1798) =       0.1167053274410000
  dip_coef(1799) =      -0.0874392348747800
  dip_coef(1800) =       0.0329002553998500
  dip_coef(1801) =      -0.0567757921444500
  dip_coef(1802) =       0.0970303307092100
  dip_coef(1803) =       0.2598198007900000
  dip_coef(1804) =       0.0289233683643700
  dip_coef(1805) =       0.1407193522320000
  dip_coef(1806) =       0.1194077156580000
  dip_coef(1807) =       0.0131728895928800
  dip_coef(1808) =       0.1394027619050000
  dip_coef(1809) =       0.1013500722790000
  dip_coef(1810) =      -0.0280240945543000
  dip_coef(1811) =      -0.0430254174339200
  dip_coef(1812) =      -0.0368772797303300
  dip_coef(1813) =      -0.0763808295922500
  dip_coef(1814) =       0.2928413748750000
  dip_coef(1815) =       0.0353997136440000
  dip_coef(1816) =       0.0059704330794020
  dip_coef(1817) =       0.0575983750891400
  dip_coef(1818) =      -0.1393703782800000
  dip_coef(1819) =      -0.1976348846590000
  dip_coef(1820) =       0.1288726694880000
  dip_coef(1821) =       0.0943669502113900
  dip_coef(1822) =      -0.0904729614636000
  dip_coef(1823) =       0.3540490152720000
  dip_coef(1824) =      -0.1789985117830000
  dip_coef(1825) =      -0.1241236515180000
  dip_coef(1826) =       0.2084597705220000
  dip_coef(1827) =       0.1448774529770000
  dip_coef(1828) =      -0.1615684020430000
  dip_coef(1829) =       0.2742709600770000
  dip_coef(1830) =      -0.0824275105406300
  dip_coef(1831) =       0.1535170949220000
  dip_coef(1832) =       0.4346733937990000
  dip_coef(1833) =      -0.0983774505825000
  dip_coef(1834) =      -0.8764900166980000
  dip_coef(1835) =       0.2419612190490000
  dip_coef(1836) =       0.1068718931290000
  dip_coef(1837) =      -0.0911487683800300
  dip_coef(1838) =       0.7020638273930000
  dip_coef(1839) =      -0.3086796267820000
  dip_coef(1840) =      -0.0164866176540900
  dip_coef(1841) =      -1.0472459056400001
  dip_coef(1842) =      -1.2058425928000001
  dip_coef(1843) =       0.1022853288480000
  dip_coef(1844) =       1.1336690668400000
  dip_coef(1845) =      -0.2933222702880000
  dip_coef(1846) =       2.0762194956000002
  dip_coef(1847) =       0.5701627603490000
  dip_coef(1848) =      -0.2802071089650000
  dip_coef(1849) =       0.7210738274580000
  dip_coef(1850) =      -0.3468778392540000
  dip_coef(1851) =      -0.0636131126781500
  dip_coef(1852) =       0.0373486195733600
  dip_coef(1853) =       1.5600847950800001
  dip_coef(1854) =      -0.0141554987199000
  dip_coef(1855) =       0.3189459513970000
  dip_coef(1856) =      -0.3350758441370000
  dip_coef(1857) =      -0.0463225538621300
  dip_coef(1858) =       0.0742794465337600
  dip_coef(1859) =       0.1042209320680000
  dip_coef(1860) =       0.4708364633290000
  dip_coef(1861) =       1.2333448868400001
  dip_coef(1862) =       0.2697768033990000
  dip_coef(1863) =       0.2571213934950000
  dip_coef(1864) =       0.0625090419322800
  dip_coef(1865) =       0.0223873014893100
  dip_coef(1866) =       0.9799043641640000
  dip_coef(1867) =       0.4556534492790000
  dip_coef(1868) =      -0.5578916189460000
  dip_coef(1869) =       0.1213274492130000
  dip_coef(1870) =      -0.0030360363774440
  dip_coef(1871) =      -0.3109973766360000
  dip_coef(1872) =       0.3240838462670000
  dip_coef(1873) =       0.0049534589602620
  dip_coef(1874) =       0.4747560181280000
  dip_coef(1875) =      -0.3313673178100000
  dip_coef(1876) =      -1.0564313551100000
  dip_coef(1877) =       0.1244038875220000
  dip_coef(1878) =       0.7735298159100000
  dip_coef(1879) =       0.3157043584290000
  dip_coef(1880) =       0.3165251578470000
  dip_coef(1881) =       2.1014000764000000
  dip_coef(1882) =       0.0993933223564500
  dip_coef(1883) =      -0.2542120148710000
  dip_coef(1884) =       1.3146141869600001
  dip_coef(1885) =      -4.0035685395499998
  dip_coef(1886) =      -2.2756891826099999
  dip_coef(1887) =       0.6092379415319999
  dip_coef(1888) =      -1.3674236035500000
  dip_coef(1889) =       0.2913105269080000
  dip_coef(1890) =       0.2653979226240000
  dip_coef(1891) =      -0.4798505861030000
  dip_coef(1892) =      -1.0138919300200000
  dip_coef(1893) =       0.0128791719019300
  dip_coef(1894) =       0.2118064885640000
  dip_coef(1895) =      -0.3959577337560000
  dip_coef(1896) =      -0.0276720303296900
  dip_coef(1897) =       0.1886529284560000
  dip_coef(1898) =       0.3911075590200000
  dip_coef(1899) =      -0.0931430043090500
  dip_coef(1900) =      -0.7370663418180000
  dip_coef(1901) =       0.1660880667970000
  dip_coef(1902) =       0.9328888989290000
  dip_coef(1903) =      -0.4082946099150000
  dip_coef(1904) =       0.4494689408660000
  dip_coef(1905) =       0.0685012994949600
  dip_coef(1906) =      -0.1742420007550000
  dip_coef(1907) =      -0.8005664095920000
  dip_coef(1908) =       0.0247669540358400
  dip_coef(1909) =      -0.0390064817958800
  dip_coef(1910) =      -0.8776432317380000
  dip_coef(1911) =      -0.3935422424530000
  dip_coef(1912) =       0.3503451864130000
  dip_coef(1913) =      -0.8474656816930000
  dip_coef(1914) =       0.9084475996950000
  dip_coef(1915) =       0.7373373922360000
  dip_coef(1916) =      -0.6912303412810000
  dip_coef(1917) =      -0.5682383218660000
  dip_coef(1918) =      -0.1406339901180000
  dip_coef(1919) =      -0.3065826204420000
  dip_coef(1920) =       0.0074164408951920
  dip_coef(1921) =      -0.0961642959061400
  dip_coef(1922) =      -0.7351449885370001
  dip_coef(1923) =      -0.0423499765151800
  dip_coef(1924) =       0.1133157606110000
  dip_coef(1925) =      -0.0200204445166200
  dip_coef(1926) =       0.0044295619589430
  dip_coef(1927) =       0.0981646839596600
  dip_coef(1928) =       0.1715666336880000
  dip_coef(1929) =       0.1538866587770000
  dip_coef(1930) =       0.0846064942739600
  dip_coef(1931) =       0.8633024885850000
  dip_coef(1932) =      -1.1944987799100000
  dip_coef(1933) =      -2.6463338052599998
  dip_coef(1934) =       0.5213840317330000
  dip_coef(1935) =      -0.2892888572300000
  dip_coef(1936) =      -0.0695934638020500
  dip_coef(1937) =      -2.3825332772999999
  dip_coef(1938) =      -0.7466663326660000
  dip_coef(1939) =       0.7586171393440000
  dip_coef(1940) =      -0.7965747754230000
  dip_coef(1941) =      -1.4569206854600001
  dip_coef(1942) =      -0.1828860149860000
  dip_coef(1943) =       1.3738684736400000
  dip_coef(1944) =       0.2125040844830000
  dip_coef(1945) =       0.0755155882775200
  dip_coef(1946) =       2.9753839464600000
  dip_coef(1947) =       2.5276462370899999
  dip_coef(1948) =      -0.0271010191376200
  dip_coef(1949) =       2.8508045430400002
  dip_coef(1950) =       1.2554207636500001
  dip_coef(1951) =      -4.1300488480500004
  dip_coef(1952) =       0.1777473609170000
  dip_coef(1953) =       0.3654320651470000
  dip_coef(1954) =       1.0284713338000000
  dip_coef(1955) =       1.5223736956400000
  dip_coef(1956) =       0.2987446738940000
  dip_coef(1957) =      -0.3572922586700000
  dip_coef(1958) =      -0.1442161367550000
  dip_coef(1959) =      -0.2165248409720000
  dip_coef(1960) =       0.2348026674950000
  dip_coef(1961) =      -0.0339725094790500
  dip_coef(1962) =      -1.1144972207399999
  dip_coef(1963) =      -0.0382220183428600
  dip_coef(1964) =      -0.1359912612280000
  dip_coef(1965) =       0.0448112026458700
  dip_coef(1966) =      -0.0540907844750300
  dip_coef(1967) =      -0.0260450016497900
  dip_coef(1968) =       0.2053943268690000
  dip_coef(1969) =      -0.2097663211960000
  dip_coef(1970) =       0.6649699710520000
  dip_coef(1971) =       0.0076947002608890
  dip_coef(1972) =      -0.1529060259820000
  dip_coef(1973) =      -0.1707571305280000
  dip_coef(1974) =      -0.0532477420878800
  dip_coef(1975) =      -0.3943168257860000
  dip_coef(1976) =      -0.1838282814300000
  dip_coef(1977) =      -0.7301392783260000
  dip_coef(1978) =       0.3480992107810000
  dip_coef(1979) =       0.0279865545920700
  dip_coef(1980) =      -0.0719353486430000
  dip_coef(1981) =      -0.8124689013000000
  dip_coef(1982) =      -0.1375727974180000
  dip_coef(1983) =       1.4008739329900000
  dip_coef(1984) =      -1.3352678261699999
  dip_coef(1985) =       0.7283331895690000
  dip_coef(1986) =      -0.9299796759220000
  dip_coef(1987) =       1.5488140189699999
  dip_coef(1988) =      -2.4480532128600001
  dip_coef(1989) =      -0.0008133228213060
  dip_coef(1990) =      -0.2127810839340000
  dip_coef(1991) =       0.8843069089990000
  dip_coef(1992) =      -0.1072565622910000
  dip_coef(1993) =       0.7533168948930000
  dip_coef(1994) =      -0.6872584598510000
  dip_coef(1995) =       1.2074954104600000
  dip_coef(1996) =       0.3912761947670000
  dip_coef(1997) =      -0.2029370496260000
  dip_coef(1998) =      -0.0962693608537300
  dip_coef(1999) =       0.8897195098240001
  dip_coef(2000) =       1.1942031096500001
  dip_coef(2001) =       0.3886079539630000
  dip_coef(2002) =      -0.1125909916980000
  dip_coef(2003) =      -0.3684307263870000
  dip_coef(2004) =       0.0144223124763000
  dip_coef(2005) =       0.0108820034945400
  dip_coef(2006) =       1.0590017494899999
  dip_coef(2007) =      -0.8073480831580000
  dip_coef(2008) =       0.4478300968250000
  dip_coef(2009) =      -0.5545187955300001
  dip_coef(2010) =      -0.1881318941390000
  dip_coef(2011) =       1.1063903474200001
  dip_coef(2012) =       0.0213865627099200
  dip_coef(2013) =       0.1921222398100000
  dip_coef(2014) =      -0.0665174386933200
  dip_coef(2015) =       0.1125728450170000
  dip_coef(2016) =      -0.0127602885613800
  dip_coef(2017) =       0.0403891732033800
  dip_coef(2018) =       0.2089850565660000
  dip_coef(2019) =      -0.2477500974890000
  dip_coef(2020) =       0.0070434029663690
  dip_coef(2021) =       0.1416499775990000
  dip_coef(2022) =      -0.2961498117080000
  dip_coef(2023) =       0.1431408378930000
  dip_coef(2024) =      -0.0956027235410500
  dip_coef(2025) =      -0.2389352042870000
  dip_coef(2026) =      -1.2345766463700001
  dip_coef(2027) =      -0.1966223145690000
  dip_coef(2028) =      -0.3535519122520000
  dip_coef(2029) =      -0.2008046550580000
  dip_coef(2030) =      -0.1067529598200000
  dip_coef(2031) =      -0.1911268522880000
  dip_coef(2032) =       0.5654091993510000
  dip_coef(2033) =       0.0698239566296100
  dip_coef(2034) =      -0.2084271953360000
  dip_coef(2035) =      -0.1457539292170000
  dip_coef(2036) =       0.1184793283900000
  dip_coef(2037) =       0.2492859953860000
  dip_coef(2038) =      -0.0930112588732800
  dip_coef(2039) =      -0.3873078006300000
  dip_coef(2040) =      -0.1482260796200000
  dip_coef(2041) =      -0.0563405200723500
  dip_coef(2042) =       0.1606775709230000
  dip_coef(2043) =      -0.2410700539710000
  dip_coef(2044) =      -0.0520390000351700
  dip_coef(2045) =       0.0949104614728400
  dip_coef(2046) =       0.5516295291590000
  dip_coef(2047) =      -0.1819341478770000
  dip_coef(2048) =      -0.0078932211955320
  dip_coef(2049) =      -1.1583212832100001
  dip_coef(2050) =      -0.5649184177690000
  dip_coef(2051) =      -1.1066981379900001
  dip_coef(2052) =      -0.0648137094076700
  dip_coef(2053) =       0.0012030734737350
  dip_coef(2054) =       1.5539400142399999
  dip_coef(2055) =       5.5573225956999996
  dip_coef(2056) =       0.6354846253350001
  dip_coef(2057) =      -0.1632661444590000
  dip_coef(2058) =       0.9467163894420000
  dip_coef(2059) =      -1.0758911927199999
  dip_coef(2060) =       0.0094230448637170
  dip_coef(2061) =       0.3583194683480000
  dip_coef(2062) =       0.7051549153200000
  dip_coef(2063) =       0.0127868807416900
  dip_coef(2064) =       0.1585249431150000
  dip_coef(2065) =       0.9951405988570000
  dip_coef(2066) =      -0.0748187376143300
  dip_coef(2067) =       0.2673504682710000
  dip_coef(2068) =      -0.4576786174180000
  dip_coef(2069) =      -0.3941547684240000
  dip_coef(2070) =       0.6067044545680000
  dip_coef(2071) =       0.1664631690250000
  dip_coef(2072) =      -0.8323914843040000
  dip_coef(2073) =      -0.1515139267710000
  dip_coef(2074) =      -0.9365065174640000
  dip_coef(2075) =      -0.3403871435310000
  dip_coef(2076) =      -0.0839606267319500
  dip_coef(2077) =       0.0016929591315260
  dip_coef(2078) =      -0.0440729442566000
  dip_coef(2079) =       0.0690458005913200
  dip_coef(2080) =       0.1864022660910000
  dip_coef(2081) =      -0.1042823869540000
  dip_coef(2082) =      -0.1453721063290000
  dip_coef(2083) =       0.1780576108360000
  dip_coef(2084) =      -0.2449281307140000
  dip_coef(2085) =      -0.2004892853250000
  dip_coef(2086) =       0.4780920213060000
  dip_coef(2087) =      -0.1308275118110000
  dip_coef(2088) =      -0.2237274840220000
  dip_coef(2089) =      -0.7577248398420000
  dip_coef(2090) =      -0.0140507263529300
  dip_coef(2091) =      -0.0079245666327260
  dip_coef(2092) =       0.2644478981990000
  dip_coef(2093) =      -0.1104644514940000
  dip_coef(2094) =      -0.0548542901072800
  dip_coef(2095) =      -0.0291293663560000
  dip_coef(2096) =      -0.0337675530944600
  dip_coef(2097) =      -0.0869416763405400
  dip_coef(2098) =      -0.0753246443238500
  dip_coef(2099) =      -0.0774934470467900
  dip_coef(2100) =      -0.1823874494890000
  dip_coef(2101) =       0.1967051047820000
  dip_coef(2102) =       0.8237544203680000
  dip_coef(2103) =       1.6469566171800001
  dip_coef(2104) =      -0.1990684384240000
  dip_coef(2105) =      -0.2364210019310000
  dip_coef(2106) =       0.0826698562127600
  dip_coef(2107) =      -2.9421567385200000
  dip_coef(2108) =       0.6321906560620000
  dip_coef(2109) =      -0.1828637062160000
  dip_coef(2110) =       0.0382830381894000
  dip_coef(2111) =       4.7254248467300002
  dip_coef(2112) =      -4.6602000911600001
  dip_coef(2113) =      -3.1584502520300002
  dip_coef(2114) =       0.1734102647220000
  dip_coef(2115) =      -1.8079841584900000
  dip_coef(2116) =      -4.3180964185199997
  dip_coef(2117) =       1.4461762518200001
  dip_coef(2118) =      -1.7956349693300000
  dip_coef(2119) =      -3.4214105192400002
  dip_coef(2120) =      -5.0624187016100004
  dip_coef(2121) =       4.0054221127899998
  dip_coef(2122) =       0.2317737063410000
  dip_coef(2123) =      -2.4221995706900001
  dip_coef(2124) =       2.3876580886399998
  dip_coef(2125) =       0.3535542165030000
  dip_coef(2126) =      -0.9604785479580000
  dip_coef(2127) =       0.0501254933251200
  dip_coef(2128) =      -0.0765419954734800
  dip_coef(2129) =      -0.0029197822748910
  dip_coef(2130) =       0.4353518682240000
  dip_coef(2131) =       1.6397208029300001
  dip_coef(2132) =      -1.5656628231600001
  dip_coef(2133) =       1.6357510919900000
  dip_coef(2134) =       1.3996145553899999
  dip_coef(2135) =      -0.0344469890545800
  dip_coef(2136) =       0.0244050218467200
  dip_coef(2137) =       0.3545324853810000
  dip_coef(2138) =       0.1252939032960000
  dip_coef(2139) =       0.1357241856250000
  dip_coef(2140) =      -0.4767329949530000
  dip_coef(2141) =      -0.1004412625430000
  dip_coef(2142) =       1.1348836430000000
  dip_coef(2143) =      -0.1485063007360000
  dip_coef(2144) =      -0.0579005045872100
  dip_coef(2145) =       0.5652387146159999
  dip_coef(2146) =       0.2751991157050000
  dip_coef(2147) =       3.1839739169299999
  dip_coef(2148) =      -0.4046408675260000
  dip_coef(2149) =       0.0263968802013400
  dip_coef(2150) =       0.7332368585880000
  dip_coef(2151) =      -1.7671338255800000
  dip_coef(2152) =      -0.6149619929640000
  dip_coef(2153) =       0.4323170258740000
  dip_coef(2154) =      -0.6186045003330000
  dip_coef(2155) =      -0.1584976040540000
  dip_coef(2156) =       2.2378961389300001
  dip_coef(2157) =      -0.5255692787539999
  dip_coef(2158) =       2.4719263035900001
  dip_coef(2159) =       0.0230362852141500
  dip_coef(2160) =       0.4310675240040000
  dip_coef(2161) =      -0.1172905153120000
  dip_coef(2162) =      -0.2458134597000000
  dip_coef(2163) =      -0.0059631269159030
  dip_coef(2164) =      -0.1997048527810000
  dip_coef(2165) =      -0.0557249194044400
  dip_coef(2166) =       0.0182647937985900
  dip_coef(2167) =       0.8045407681470000
  dip_coef(2168) =       0.1983795747390000
  dip_coef(2169) =      -0.1610035948040000
  dip_coef(2170) =      -0.2266580464980000
  dip_coef(2171) =      -0.2293367315020000
  dip_coef(2172) =       0.0613741399104900
  dip_coef(2173) =       0.4467102981150000
  dip_coef(2174) =      -0.0153957126551900
  dip_coef(2175) =      -0.1525446120790000
  dip_coef(2176) =      -0.1161698082980000
  dip_coef(2177) =      -1.2045519227699999
  dip_coef(2178) =      -0.0944091556905900
  dip_coef(2179) =      -0.4001232221790000
  dip_coef(2180) =       1.5562273414200001
  dip_coef(2181) =       0.0558972969798300
  dip_coef(2182) =      -0.0053785783543270
  dip_coef(2183) =       0.2933596638570000
  dip_coef(2184) =       0.0300816297383500
  dip_coef(2185) =       0.0880798573464400
  dip_coef(2186) =      -0.0158575846238200
  dip_coef(2187) =       0.0074218129226160
  dip_coef(2188) =       0.3128369375600000
  dip_coef(2189) =       0.2785157582380000
  dip_coef(2190) =      -0.1128892814490000
  dip_coef(2191) =      -0.2819691868350000
  dip_coef(2192) =       0.5013547091920000
  dip_coef(2193) =      -0.2409260999380000
  dip_coef(2194) =       0.1243315768650000
  dip_coef(2195) =       0.2270861138670000
  dip_coef(2196) =       0.7310818144500000
  dip_coef(2197) =       0.1748882234230000
  dip_coef(2198) =       0.1090249746400000
  dip_coef(2199) =      -0.3133864241270000
  dip_coef(2200) =       0.1142750499080000
  dip_coef(2201) =       0.0888285811401500
  dip_coef(2202) =      -0.5503705580279999
  dip_coef(2203) =      -0.0139178853521700
  dip_coef(2204) =       0.3152521936420000
  dip_coef(2205) =      -0.0611348833403000
  dip_coef(2206) =       0.1109746225130000
  dip_coef(2207) =      -0.7649648511829999
  dip_coef(2208) =      -0.1600839493800000
  dip_coef(2209) =       0.6068865052580000
  dip_coef(2210) =      -0.2458457499860000
  dip_coef(2211) =       0.5319278199500000
  dip_coef(2212) =       0.3724278528500000
  dip_coef(2213) =      -0.4076549056630000
  dip_coef(2214) =      -0.2016839741190000
  dip_coef(2215) =      -0.1836352072000000
  dip_coef(2216) =      -0.7839661697870000
  dip_coef(2217) =       0.3572801721900000
  dip_coef(2218) =       0.4504316133900000
  dip_coef(2219) =       0.5332080404820000
  dip_coef(2220) =       0.2422977992940000
  dip_coef(2221) =       0.0036714610351890
  dip_coef(2222) =       0.0558595431705600
  dip_coef(2223) =      -0.1586212622430000
  dip_coef(2224) =      -0.0071455960906960
  dip_coef(2225) =       0.0113467166551200
  dip_coef(2226) =      -0.0021015551733200
  dip_coef(2227) =      -0.0614137127962900
  dip_coef(2228) =       0.1036351482410000
  dip_coef(2229) =       0.0886542275914200
  dip_coef(2230) =      -0.1056539214460000
  dip_coef(2231) =      -0.0820329771129400
  dip_coef(2232) =       0.1516466670450000
  dip_coef(2233) =       0.0928870030432000
  dip_coef(2234) =       0.0384437591296500
  dip_coef(2235) =      -0.0175092427318600
  dip_coef(2236) =      -0.0248332336123400
  dip_coef(2237) =      -0.0265270654272300
  dip_coef(2238) =      -0.0213957751989400
  dip_coef(2239) =      -0.0186726458308900
  dip_coef(2240) =       0.0048607982399990
  dip_coef(2241) =       0.0001861284025509
  dip_coef(2242) =       0.0421432616851400
  dip_coef(2243) =       0.1321318656440000
  dip_coef(2244) =       0.0575732618581300
  dip_coef(2245) =      -0.0295880198118700
  dip_coef(2246) =      -0.0260145239900700
  dip_coef(2247) =      -0.0170872010642700
  dip_coef(2248) =       0.0106680081729100
  dip_coef(2249) =      -0.7464451045650000
  dip_coef(2250) =      -0.4530520095000000
  dip_coef(2251) =       0.0716183019562600
  dip_coef(2252) =       0.2056356937700000
  dip_coef(2253) =       0.0534799506918700
  dip_coef(2254) =       0.7760762266590000
  dip_coef(2255) =      -0.9247509127220001
  dip_coef(2256) =      -0.8824058100060000
  dip_coef(2257) =       0.7891470671470000
  dip_coef(2258) =       1.3163188561900001
  dip_coef(2259) =      -1.6246533673200001
  dip_coef(2260) =      -1.0431893335700000
  dip_coef(2261) =      -0.6078587531960000
  dip_coef(2262) =      -0.0620468973472900
  dip_coef(2263) =      -0.0451269134085900
  dip_coef(2264) =       0.1232981944150000
  dip_coef(2265) =      -0.0867730999522300
  dip_coef(2266) =       0.4892173091550000
  dip_coef(2267) =      -0.3882948453780000
  dip_coef(2268) =       0.2918483766660000
  dip_coef(2269) =      -0.5242548611059999
  dip_coef(2270) =       0.4943819366370000
  dip_coef(2271) =       0.8002342771460000
  dip_coef(2272) =      -0.2774052804860000
  dip_coef(2273) =       0.0665791876835900
  dip_coef(2274) =       0.4897465949940000
  dip_coef(2275) =       0.1258680721600000
  dip_coef(2276) =      -0.7566106488110000
  dip_coef(2277) =      -1.8713751352100001
  dip_coef(2278) =      -0.0787613871568400
  dip_coef(2279) =       1.0466822106700000
  dip_coef(2280) =       0.2650399080930000
  dip_coef(2281) =      -0.1887494463510000
  dip_coef(2282) =      -0.3413735438100000
  dip_coef(2283) =       1.3453392907699999
  dip_coef(2284) =      -0.4145154060820000
  dip_coef(2285) =       2.1260851150100000
  dip_coef(2286) =       0.1482170678020000
  dip_coef(2287) =      -1.2777191753799999
  dip_coef(2288) =       0.6189043232620000
  dip_coef(2289) =      -0.0201930639350500
  dip_coef(2290) =      -0.7111287675980000
  dip_coef(2291) =      -0.1722648313040000
  dip_coef(2292) =      -0.5807529955080000
  dip_coef(2293) =      -0.3367712784970000
  dip_coef(2294) =      -0.1026199072110000
  dip_coef(2295) =      -0.1681689904250000
  dip_coef(2296) =       0.7953812781720000
  dip_coef(2297) =       2.1842133180899999
  dip_coef(2298) =      -0.3753893824890000
  dip_coef(2299) =      -1.0112936736200000
  dip_coef(2300) =      -1.3338415543000000
  dip_coef(2301) =      -0.0983760382401700
  dip_coef(2302) =       0.1814570859540000
  dip_coef(2303) =      -1.1571389952000000
  dip_coef(2304) =       2.5610051892999999
  dip_coef(2305) =       0.0220171710007900
  dip_coef(2306) =       1.2090294076200001
  dip_coef(2307) =       0.1255114950300000
  dip_coef(2308) =      -0.9267640936410000
  dip_coef(2309) =      -0.7255893594900000
  dip_coef(2310) =       1.6149538323099999
  dip_coef(2311) =      -2.7470676272499999
  dip_coef(2312) =       1.1969847068799999
  dip_coef(2313) =       0.9748061436170000
  dip_coef(2314) =      -0.4100876911670000
  dip_coef(2315) =       0.7661347649300000
  dip_coef(2316) =       0.0223401147459500
  dip_coef(2317) =      -0.1366909921000000
  dip_coef(2318) =      -0.3467425822760000
  dip_coef(2319) =       0.1264589527690000
  dip_coef(2320) =      -0.4445631881880000
  dip_coef(2321) =      -0.3023770732080000
  dip_coef(2322) =       0.6703865590080000
  dip_coef(2323) =       0.3035688458260000
  dip_coef(2324) =       0.0440490220716600
  dip_coef(2325) =       0.9034213972150000
  dip_coef(2326) =       0.5843152958910000
  dip_coef(2327) =      -0.3323078929340000
  dip_coef(2328) =      -0.3570984524200000
  dip_coef(2329) =      -0.0495573422014800
  dip_coef(2330) =       1.0196950581699999
  dip_coef(2331) =       1.1087521722500000
  dip_coef(2332) =       0.0493635721966300
  dip_coef(2333) =      -0.3673168405740000
  dip_coef(2334) =      -0.0509043553572600
  dip_coef(2335) =      -0.2964960126910000
  dip_coef(2336) =       0.3028870086870000
  dip_coef(2337) =      -0.5352355886440000
  dip_coef(2338) =       0.7187552010670000
  dip_coef(2339) =      -1.7565914575399999
  dip_coef(2340) =      -0.4854664558110000
  dip_coef(2341) =       1.1323643778800001
  dip_coef(2342) =      -0.1210395330280000
  dip_coef(2343) =       0.1038903256870000
  dip_coef(2344) =       0.3351688218310000
  dip_coef(2345) =       0.2595457987580000
  dip_coef(2346) =       0.1938947076630000
  dip_coef(2347) =       0.2134954427150000
  dip_coef(2348) =       0.1414633736100000
  dip_coef(2349) =      -0.6522075044750000
  dip_coef(2350) =       0.0576381682813600
  dip_coef(2351) =      -1.5642301099700000
  dip_coef(2352) =      -0.6716753380490000
  dip_coef(2353) =       0.8565006382300000
  dip_coef(2354) =       0.9360013632990000
  dip_coef(2355) =       0.0060682959906390
  dip_coef(2356) =       0.0036346780990390
  dip_coef(2357) =      -0.0943288017241700
  dip_coef(2358) =       0.0789579155161700
  dip_coef(2359) =      -0.0267324850564200
  dip_coef(2360) =      -0.0174685867853300
  dip_coef(2361) =      -0.0229192147971800
  dip_coef(2362) =      -0.0246415909831700
  dip_coef(2363) =       0.1454900694300000
  dip_coef(2364) =       0.0450247302472600
  dip_coef(2365) =      -0.0170365842276400
  dip_coef(2366) =      -0.0164967555625300
  dip_coef(2367) =       0.1532472631400000
  dip_coef(2368) =       0.0267452454441300
  dip_coef(2369) =       0.0662902998240400
  dip_coef(2370) =       0.0043469813302780
  dip_coef(2371) =      -0.0427903471761900
  dip_coef(2372) =      -0.0099034336949510
  dip_coef(2373) =      -0.0222703109326900
  dip_coef(2374) =      -0.0429434426935500
  dip_coef(2375) =       0.0218890981124000
  dip_coef(2376) =      -0.1225734450910000
  dip_coef(2377) =       0.1312544422120000
  dip_coef(2378) =      -0.0151003123259800
  dip_coef(2379) =      -0.1074767504640000
  dip_coef(2380) =      -0.1226444068590000
  dip_coef(2381) =      -0.1578536227420000
  dip_coef(2382) =      -0.0396309852060200
  dip_coef(2383) =      -0.0065261457625690
  dip_coef(2384) =       0.0295397842070100
  dip_coef(2385) =       0.2356387001300000
  dip_coef(2386) =       0.0381980833470200
  dip_coef(2387) =      -0.1588378842190000
  dip_coef(2388) =      -0.0088717943126630
  dip_coef(2389) =      -0.0235752838312500
  dip_coef(2390) =      -0.4636804204910000
  dip_coef(2391) =       0.0611309412382500
  dip_coef(2392) =      -0.2366050260410000
  dip_coef(2393) =       0.0356713100942500
  dip_coef(2394) =       0.3689024526520000
  dip_coef(2395) =      -0.1185022531670000
  dip_coef(2396) =      -0.1035280332820000
  dip_coef(2397) =       0.0398208168454000
  dip_coef(2398) =       0.0851847547239900
  dip_coef(2399) =      -0.0004187320480887
  dip_coef(2400) =       0.1085860514100000
  dip_coef(2401) =      -0.0030995359272880
  dip_coef(2402) =      -0.0384432125126500
  dip_coef(2403) =       0.2662529842910000
  dip_coef(2404) =      -0.2157531465460000
  dip_coef(2405) =      -0.2638218380370000
  dip_coef(2406) =       0.1253869437530000
  dip_coef(2407) =       0.1477361209070000
  dip_coef(2408) =       0.3169228328430000
  dip_coef(2409) =       0.1698779983920000
  dip_coef(2410) =      -0.0439373935659700
  dip_coef(2411) =       0.0001460516176439
  dip_coef(2412) =      -0.0087037710879050
  dip_coef(2413) =      -0.0104127454844500
  dip_coef(2414) =       0.9090451765960000
  dip_coef(2415) =       0.0304854888296300
  dip_coef(2416) =       0.2413232878090000
  dip_coef(2417) =      -0.3119184669820000
  dip_coef(2418) =       1.8788852653799999
  dip_coef(2419) =       0.0751705837512500
  dip_coef(2420) =      -0.0868270331976900
  dip_coef(2421) =       1.1229456820399999
  dip_coef(2422) =      -0.0891675595170200
  dip_coef(2423) =      -0.0395434486237500
  dip_coef(2424) =       0.0983499324541000
  dip_coef(2425) =      -0.1145255077380000
  dip_coef(2426) =      -0.3219819008590000
  dip_coef(2427) =       0.0021674245617680
  dip_coef(2428) =      -0.0837961226790200
  dip_coef(2429) =       0.0848740738792800
  dip_coef(2430) =      -0.4695177595420000
  dip_coef(2431) =      -0.0263877460127400
  dip_coef(2432) =      -0.2230023384380000
  dip_coef(2433) =      -0.2069298030540000
  dip_coef(2434) =      -0.9045631267840000
  dip_coef(2435) =      -0.0608401271372900
  dip_coef(2436) =      -0.0978672937537100
  dip_coef(2437) =      -0.1568722159510000
  dip_coef(2438) =       1.8563552131300001
  dip_coef(2439) =       0.2468318810920000
  dip_coef(2440) =      -0.0784329842289800
  dip_coef(2441) =       0.9704459971750000
  dip_coef(2442) =       0.7307724887450000
  dip_coef(2443) =       0.3569027960670000
  dip_coef(2444) =      -0.1307232119400000
  dip_coef(2445) =       0.0028062223348260
  dip_coef(2446) =      -1.1104660675300000
  dip_coef(2447) =      -0.2725333719160000
  dip_coef(2448) =      -0.4217494253740000
  dip_coef(2449) =      -2.1953920646500000
  dip_coef(2450) =       0.0655726507123000
  dip_coef(2451) =      -0.3183121027810000
  dip_coef(2452) =       0.1099925127140000
  dip_coef(2453) =       1.5817980641900000
  dip_coef(2454) =       0.0156229603473300
  dip_coef(2455) =       0.1058338369630000
  dip_coef(2456) =       2.1591287244399999
  dip_coef(2457) =       0.1695115476300000
  dip_coef(2458) =       0.2961048700950000
  dip_coef(2459) =      -0.1836295442850000
  dip_coef(2460) =       0.9401884378730000
  dip_coef(2461) =       0.3786756178390000
  dip_coef(2462) =      -0.2025484692290000
  dip_coef(2463) =       0.4784310736870000
  dip_coef(2464) =       0.3014573222030000
  dip_coef(2465) =      -0.0722876506540400
  dip_coef(2466) =      -0.0648926502305900
  dip_coef(2467) =       0.0251238672779000
  dip_coef(2468) =       0.6723739437380000
  dip_coef(2469) =      -0.1771596565710000
  dip_coef(2470) =       0.3848793126490000
  dip_coef(2471) =      -0.3003117077960000
  dip_coef(2472) =       0.1003673145170000
  dip_coef(2473) =      -0.0524712021620300
  dip_coef(2474) =       0.0362162926499800
  dip_coef(2475) =      -0.0914959125579000
  dip_coef(2476) =       0.7254747279850000
  dip_coef(2477) =      -0.0155964268980400
  dip_coef(2478) =      -0.0504460884960700
  dip_coef(2479) =       0.1609183189450000
  dip_coef(2480) =       0.1237348614860000
  dip_coef(2481) =      -0.1596252141760000
  dip_coef(2482) =       0.2536028181890000
  dip_coef(2483) =       0.0935333041939600
  dip_coef(2484) =       0.2603053950520000
  dip_coef(2485) =       0.2627002078690000
  dip_coef(2486) =      -0.2969774516250000
  dip_coef(2487) =       0.1412642833750000
  dip_coef(2488) =      -0.7524240780440000
  dip_coef(2489) =      -0.1719867285450000
  dip_coef(2490) =       0.5146500223429999
  dip_coef(2491) =      -0.3150296822450000
  dip_coef(2492) =       0.5768336382100000
  dip_coef(2493) =      -0.6142724690350000
  dip_coef(2494) =      -3.7582010043100000
  dip_coef(2495) =       0.7554799930190000
  dip_coef(2496) =      -0.2637597597590000
  dip_coef(2497) =      -0.2144693642400000
  dip_coef(2498) =      -1.3221906050300001
  dip_coef(2499) =      -0.1364420696190000
  dip_coef(2500) =       0.3530809458160000
  dip_coef(2501) =       0.6307038311230000
  dip_coef(2502) =       0.9190582779960000
  dip_coef(2503) =      -0.0904206791717900
  dip_coef(2504) =       0.8098131512200000
  dip_coef(2505) =       1.2032805843000001
  dip_coef(2506) =      -0.2487603973370000
  dip_coef(2507) =       0.9085590462820000
  dip_coef(2508) =       0.1353798056050000
  dip_coef(2509) =      -0.3226133890740000
  dip_coef(2510) =       0.0631379698282700
  dip_coef(2511) =       0.1276409261620000
  dip_coef(2512) =      28.0999574105000001
  dip_coef(2513) =      22.9982807109000014
  dip_coef(2514) =      16.3594562502999992
  dip_coef(2515) =     -30.6114087963000010
  dip_coef(2516) =      -8.1320947985899998
  dip_coef(2517) =      94.9714272763999929
  dip_coef(2518) =     -79.9833546994999978
  dip_coef(2519) =     -70.8216577174000008
  dip_coef(2520) =       0.5455269168869999
  dip_coef(2521) =      29.6287376555000002
  dip_coef(2522) =      15.5509454008999999
  dip_coef(2523) =      -2.4131807974799999
  dip_coef(2524) =      12.6346111550000000
  dip_coef(2525) =       1.6543071447500000
  dip_coef(2526) =      -1.8786329770200001
  dip_coef(2527) =      -1.8604885515900000
  dip_coef(2528) =     -34.2118272266000005
  dip_coef(2529) =      -9.5915343883700004
  dip_coef(2530) =      -5.5001762916099999
  dip_coef(2531) =      36.0581295890999982
  dip_coef(2532) =      32.6568167555999977
  dip_coef(2533) =     -19.5643091320000018
  dip_coef(2534) =      15.1531245696999992
  dip_coef(2535) =      65.6087224924000054
  dip_coef(2536) =     -62.3927154124000012
  dip_coef(2537) =     -47.8713356470999969
  dip_coef(2538) =     -58.5945321888000024
  dip_coef(2539) =      20.3045332913000003
  dip_coef(2540) =      30.6214182601000005
  dip_coef(2541) =      22.5841819165999986
  dip_coef(2542) =      45.9719001120000001
  dip_coef(2543) =      -1.7764591677299999
  dip_coef(2544) =       4.8807082197699998
  dip_coef(2545) =      28.9532085897000009
  dip_coef(2546) =      15.9918440522999994
  dip_coef(2547) =     -12.3402299307999996
  dip_coef(2548) =     -27.7995205574999993
  dip_coef(2549) =      83.1302612906000036
  dip_coef(2550) =     -25.6287051261999999
  dip_coef(2551) =      38.7240704908000026
  dip_coef(2552) =       0.4706728610480000
  dip_coef(2553) =      -5.4280725154800002
  dip_coef(2554) =     -54.5007725639000000
  dip_coef(2555) =      -3.8611758924499999
  dip_coef(2556) =      -0.1185789062500000
  dip_coef(2557) =      -9.6797903610499993
  dip_coef(2558) =     -31.2470624512999997
  dip_coef(2559) =     -28.0674578648999997
  dip_coef(2560) =       4.3814859483299999
  dip_coef(2561) =      36.2269851614999965
  dip_coef(2562) =     -10.2535895865000004
  dip_coef(2563) =       0.5475241080480000
  dip_coef(2564) =      -4.1645860280100004
  dip_coef(2565) =       4.9787850650300003
  dip_coef(2566) =      -1.9126169361300001
  dip_coef(2567) =       5.0122047811200003
  dip_coef(2568) =      -9.3690375394099998
  dip_coef(2569) =      -7.3542256014899996
  dip_coef(2570) =      -6.7678587697500001
  dip_coef(2571) =      -3.1694463898700000
  dip_coef(2572) =      -0.4276780099320000
  dip_coef(2573) =      10.7026703059999999
  dip_coef(2574) =       0.5874496147750000
  dip_coef(2575) =       1.9392828169800000
  dip_coef(2576) =      20.3748573190999984
  dip_coef(2577) =       1.7857933095000000
  dip_coef(2578) =       0.0332774218726000
  dip_coef(2579) =     -13.5504837162000005
  dip_coef(2580) =     -20.6414209898000003
  dip_coef(2581) =       5.4358662666699997
  dip_coef(2582) =       0.1275098273630000
  dip_coef(2583) =     -18.8982772300999997
  dip_coef(2584) =       4.4847659414100001
  dip_coef(2585) =       0.5170761645460000
  dip_coef(2586) =       0.6731706215000000
  dip_coef(2587) =      -0.7497698235530000
  dip_coef(2588) =       6.3109842262800004
  dip_coef(2589) =      -7.3938959890899998
  dip_coef(2590) =       1.0876432216800000
  dip_coef(2591) =      -9.6713078152699996
  dip_coef(2592) =       9.1410186448699999
  dip_coef(2593) =       1.3381571555500000
  dip_coef(2594) =       4.2099051836100001
  dip_coef(2595) =      -1.9315539504599999
  dip_coef(2596) =     -16.0295374503999994
  dip_coef(2597) =     -10.0522223328999996
  dip_coef(2598) =      -0.7902493903910000
  dip_coef(2599) =      -5.0095639005199999
  dip_coef(2600) =      21.3352257955000013
  dip_coef(2601) =       3.0677126609599998
  dip_coef(2602) =      39.0028405520000021
  dip_coef(2603) =     -48.0656147304000001
  dip_coef(2604) =     -37.5152016936999999
  dip_coef(2605) =      -6.1040209389899998
  dip_coef(2606) =      32.4605429815999997
  dip_coef(2607) =      42.6171392430999987
  dip_coef(2608) =       2.3216827591000002
  dip_coef(2609) =      22.8454699056999999
  dip_coef(2610) =       0.4402986753740000
  dip_coef(2611) =       0.6534906904830000
  dip_coef(2612) =      -4.7837915526800003
  dip_coef(2613) =     -34.6196512674000019
  dip_coef(2614) =     -15.6091386387999993
  dip_coef(2615) =      -9.8888762956599994
  dip_coef(2616) =      46.6144567882000018
  dip_coef(2617) =      32.3273668952000008
  dip_coef(2618) =     -15.5973595457999998
  dip_coef(2619) =     -21.4801483562999991
  dip_coef(2620) =     -35.1726174885000020
  dip_coef(2621) =       9.5125943595999995
  dip_coef(2622) =     -19.1365401044999999
  dip_coef(2623) =      14.4449540922999997
  dip_coef(2624) =      -1.4197363217900001
  dip_coef(2625) =       6.4745272515099996
  dip_coef(2626) =      26.6482241128000013
  dip_coef(2627) =      12.1734403099000001
  dip_coef(2628) =       7.5308966497499998
  dip_coef(2629) =     -48.9243540243000012
  dip_coef(2630) =     -28.0697196475999995
  dip_coef(2631) =      13.3085162111000006
  dip_coef(2632) =     -18.0048934946000010
  dip_coef(2633) =       8.9212627863300007
  dip_coef(2634) =      -0.7589853133430000
  dip_coef(2635) =       3.9036353023400001
  dip_coef(2636) =      20.6198774616000016
  dip_coef(2637) =       7.0115746637400003
  dip_coef(2638) =       4.3831258189600000
  dip_coef(2639) =     -28.9321017319000013
  dip_coef(2640) =     -26.8828360862999993
  dip_coef(2641) =      -1.3365856202499999
  dip_coef(2642) =      -0.1235088513250000
  dip_coef(2643) =      -1.3728106325400000
  dip_coef(2644) =       8.2292786885700000
  dip_coef(2645) =      -1.8223408341400000
  dip_coef(2646) =      -1.2209602692699999
  dip_coef(2647) =      -3.9020473461199998
  dip_coef(2648) =      -3.6579746076899999
  dip_coef(2649) =      -0.6742967036810000
  dip_coef(2650) =       1.6554141604499999
  dip_coef(2651) =      10.5231417194999999
  dip_coef(2652) =      -2.3904733984200002
  dip_coef(2653) =      -0.5322032555130000
  dip_coef(2654) =       6.2559998876999998
  dip_coef(2655) =       0.0160244093881400
  dip_coef(2656) =     -10.2153503498999996
  dip_coef(2657) =      18.6128482103000010
  dip_coef(2658) =       3.2512419404399999
  dip_coef(2659) =      -7.3134056365399998
  dip_coef(2660) =       8.9799054278800003
  dip_coef(2661) =       4.1815250628299996
  dip_coef(2662) =     -19.3441370289000005
  dip_coef(2663) =      -3.4558739129300000
  dip_coef(2664) =      -9.6771411063099997
  dip_coef(2665) =      -1.7708520836799999
  dip_coef(2666) =      -3.9202497355500001
  dip_coef(2667) =      -1.8657259599400000
  dip_coef(2668) =      17.4230550320000006
  dip_coef(2669) =       7.0551192179999997
  dip_coef(2670) =       8.2940341372699997
  dip_coef(2671) =      11.2559730927999997
  dip_coef(2672) =      -2.7793201325500001
  dip_coef(2673) =      17.3536854679000001
  dip_coef(2674) =      -8.8912012878399995
  dip_coef(2675) =      -0.4561397070930000
  dip_coef(2676) =      -0.8212470696210000
  dip_coef(2677) =      -5.6158635753799997
  dip_coef(2678) =      -7.7419408965400001
  dip_coef(2679) =      -4.8604335933199998
  dip_coef(2680) =      15.1970018620000005
  dip_coef(2681) =       8.3434996112000004
  dip_coef(2682) =     -11.1680665672000003
  dip_coef(2683) =       1.6620830694800000
  dip_coef(2684) =      -6.6993811445700002
  dip_coef(2685) =      -0.5473230414930000
  dip_coef(2686) =      -1.5903012472900000
  dip_coef(2687) =     -10.7918096213000005
  dip_coef(2688) =      -2.4800931224900000
  dip_coef(2689) =       1.5630091168900000
  dip_coef(2690) =      23.0629462266999994
  dip_coef(2691) =       9.2937769640500001
  dip_coef(2692) =     -13.5141879900999999
  dip_coef(2693) =       0.3772051509640000
  dip_coef(2694) =       1.2407381404500000
  dip_coef(2695) =      -6.3101184602600000
  dip_coef(2696) =       2.3361260687500001
  dip_coef(2697) =       1.8422712299399999
  dip_coef(2698) =      -3.7417012621699999
  dip_coef(2699) =      -2.7335518839300001
  dip_coef(2700) =       7.3765205545099999
  dip_coef(2701) =      -0.6515410625410000
  dip_coef(2702) =       1.8038846442100001
  dip_coef(2703) =       2.3166537739800002
  dip_coef(2704) =      -1.0737395203300000
  dip_coef(2705) =      -6.8051748371600000
  dip_coef(2706) =       0.3165527342890000
  dip_coef(2707) =       2.7603565207199998
  dip_coef(2708) =      -6.6829028869100000
  dip_coef(2709) =       0.4313950705530000
  dip_coef(2710) =       1.4816752796899999
  dip_coef(2711) =       0.4734786705630000
  dip_coef(2712) =      -1.6484810748500001
  dip_coef(2713) =       7.7108322781799998
  dip_coef(2714) =       2.4942106819199998
  dip_coef(2715) =       4.0037031679900004
  dip_coef(2716) =       0.4940205388860000
  dip_coef(2717) =       2.1074791069600001
  dip_coef(2718) =       0.3630210301180000
  dip_coef(2719) =      -4.4633867302599999
  dip_coef(2720) =      -0.7796950208270000
  dip_coef(2721) =       5.1201631832799999
  dip_coef(2722) =      -0.2933378229450000
  dip_coef(2723) =      -1.6049637302799999
  dip_coef(2724) =      -0.0684971927983600
  dip_coef(2725) =      -8.6489080019800006
  dip_coef(2726) =      -3.4922778382700002
  dip_coef(2727) =      -1.5803322936799999
  dip_coef(2728) =       9.8163439937499994
  dip_coef(2729) =      14.7922493933000005
  dip_coef(2730) =      -6.3362925588000003
  dip_coef(2731) =       0.5373607591400000
  dip_coef(2732) =       1.0576273080900001
  dip_coef(2733) =      -4.7426424269699998
  dip_coef(2734) =       3.0265578639499999
  dip_coef(2735) =       1.4755539022199999
  dip_coef(2736) =      -5.5216248854299996
  dip_coef(2737) =      -4.8255476704999998
  dip_coef(2738) =       9.0343425420900001
  dip_coef(2739) =      -0.2787068184340000
  dip_coef(2740) =       3.2796285311000002
  dip_coef(2741) =      -1.9220130103099999
  dip_coef(2742) =      -0.1697056941430000
  dip_coef(2743) =      -5.5794675510299996
  dip_coef(2744) =       0.1811179867630000
  dip_coef(2745) =       2.0010406543600001
  dip_coef(2746) =      -1.7173851865800001
  dip_coef(2747) =      -2.1583500088699998
  dip_coef(2748) =       1.9080483169300000
  dip_coef(2749) =      -2.7836130433799999
  dip_coef(2750) =      -2.2272648792899998
  dip_coef(2751) =       3.1353621046900000
  dip_coef(2752) =       3.4359096762400001
  dip_coef(2753) =       2.4337673344100001
  dip_coef(2754) =       2.8672188293200001
  dip_coef(2755) =      -0.5396460941170000
  dip_coef(2756) =      -2.3288097861099999
  dip_coef(2757) =       3.9040192816800001
  dip_coef(2758) =      -1.7459790723499999
  dip_coef(2759) =      -0.0585051412842200
  dip_coef(2760) =       0.0461480262074200
  dip_coef(2761) =       0.0082096652798800
  dip_coef(2762) =      -0.2138465269920000
  dip_coef(2763) =      -0.0490753066445500
  dip_coef(2764) =       0.9477186903400000
  dip_coef(2765) =       0.7335543666160000
  dip_coef(2766) =      -1.3128960500400000
  dip_coef(2767) =      -0.0513365490026000
  dip_coef(2768) =      -2.2705017758000001
  dip_coef(2769) =      -0.1009691710320000
  dip_coef(2770) =      -0.0210234228138500
  dip_coef(2771) =       0.7203710434780000
  dip_coef(2772) =      -0.1561577436800000
  dip_coef(2773) =       0.5622297133970000
  dip_coef(2774) =       0.2032848566640000
  dip_coef(2775) =      -0.5071816555920000
  dip_coef(2776) =       0.5996068643280000
  dip_coef(2777) =       0.1733542920040000
  dip_coef(2778) =      -0.2341393913040000
  dip_coef(2779) =       1.6817590132700000
  dip_coef(2780) =       1.0294379218900001
  dip_coef(2781) =       1.1564718896299999
  dip_coef(2782) =       1.3379562686599999
  dip_coef(2783) =      -0.9819127929960000
  dip_coef(2784) =      -0.4598554798540000
  dip_coef(2785) =      -0.1403953860670000
  dip_coef(2786) =      -0.9463054582860000
  dip_coef(2787) =      -0.3563843311230000
  dip_coef(2788) =       3.1301508652400001
  dip_coef(2789) =      -0.5919906103960000
  dip_coef(2790) =       0.0665185167251100
  dip_coef(2791) =      -2.1321087978500000
  dip_coef(2792) =      -1.1244111170100000
  dip_coef(2793) =       0.3543926103760000
  dip_coef(2794) =       1.5952649096100000
  dip_coef(2795) =       8.4286851599099997
  dip_coef(2796) =       4.9807161430100004
  dip_coef(2797) =     -12.2801205597000003
  dip_coef(2798) =      -5.3497768652900000
  dip_coef(2799) =       5.2445812402499996
  dip_coef(2800) =       0.9557248774790000
  dip_coef(2801) =       0.7573694638720000
  dip_coef(2802) =      -5.4159976371800003
  dip_coef(2803) =      -4.4627696993899999
  dip_coef(2804) =       2.7127793338799999
  dip_coef(2805) =      -0.6836773043110000
  dip_coef(2806) =      -4.8296178895599997
  dip_coef(2807) =      -3.7478293253900001
  dip_coef(2808) =       0.8557911577590001
  dip_coef(2809) =      -0.1941919587990000
  dip_coef(2810) =      -3.6537113699599999
  dip_coef(2811) =      -0.8761159520610000
  dip_coef(2812) =       0.1539528781450000
  dip_coef(2813) =       9.9753871688700002
  dip_coef(2814) =      14.0589988910999999
  dip_coef(2815) =      -6.3289084151299999
  dip_coef(2816) =       3.3258895230799999
  dip_coef(2817) =      -0.6814611120770000
  dip_coef(2818) =      -1.7225839225599999
  dip_coef(2819) =      -1.1274508085199999
  dip_coef(2820) =      -0.2029902453810000
  dip_coef(2821) =      -0.3114036294540000
  dip_coef(2822) =      -0.3733309486380000
  dip_coef(2823) =      -0.0968873008634000
  dip_coef(2824) =       1.2250149813500000
  dip_coef(2825) =      -2.0905229722200001
  dip_coef(2826) =       2.9662351359199999
  dip_coef(2827) =      -0.6738542000740000
  dip_coef(2828) =       0.5765205044980000
  dip_coef(2829) =       0.8447112455390000
  dip_coef(2830) =      -1.2966939791500001
  dip_coef(2831) =       1.2060553371100000
  dip_coef(2832) =      -0.9609297023850000
  dip_coef(2833) =       1.5484008629899999
  dip_coef(2834) =       5.4686297893900004
  dip_coef(2835) =       1.6968892119700001
  dip_coef(2836) =       0.3179499044080000
  dip_coef(2837) =       0.5357703372900000
  dip_coef(2838) =      -6.0805677088600003
  dip_coef(2839) =      -1.8502626913000000
  dip_coef(2840) =      -2.2755818848099998
  dip_coef(2841) =       2.7784040548500002
  dip_coef(2842) =       1.1507182090200001
  dip_coef(2843) =      -0.4044225256750000
  dip_coef(2844) =       0.0123744435872000
  dip_coef(2845) =      -2.4623425721899999
  dip_coef(2846) =      -0.5546670407730000
  dip_coef(2847) =      -1.3496431680300001
  dip_coef(2848) =       2.6079248410400000
  dip_coef(2849) =       0.7620021740120000
  dip_coef(2850) =       1.0318155901999999
  dip_coef(2851) =      -1.0885621810999999
  dip_coef(2852) =      -2.8372851159599999
  dip_coef(2853) =       4.7370998401200000
  dip_coef(2854) =      -0.7901618800010000
  dip_coef(2855) =      -0.9026484503350000
  dip_coef(2856) =       1.7707216143500000
  dip_coef(2857) =       0.3113250067520000
  dip_coef(2858) =       2.8922286395999999
  dip_coef(2859) =      -0.8633800168830000
  dip_coef(2860) =      -0.1166646132370000
  dip_coef(2861) =      -0.3509508095020000
  dip_coef(2862) =       1.3593464157499999
  dip_coef(2863) =      -6.7755809231199997
  dip_coef(2864) =      -0.4385368287800000
  dip_coef(2865) =      -2.3895677171599998
  dip_coef(2866) =      -2.3799159769200000
  dip_coef(2867) =       3.9452342881500000
  dip_coef(2868) =       1.1148869520500000
  dip_coef(2869) =     -35.3475612084000019
  dip_coef(2870) =     -25.0395121906000000
  dip_coef(2871) =      -5.1393684407800002
  dip_coef(2872) =     -41.2980871832999981
  dip_coef(2873) =      21.5480206060000015
  dip_coef(2874) =      48.4928952110999987
  dip_coef(2875) =      38.9699557925000022
  dip_coef(2876) =      -0.0198652405563900
  dip_coef(2877) =      16.1157320569000007
  dip_coef(2878) =      -8.0544667455599992
  dip_coef(2879) =      -5.3512641612599996
  dip_coef(2880) =     -20.2613085356000013
  dip_coef(2881) =       3.9298602413500001
  dip_coef(2882) =      61.4497689613000020
  dip_coef(2883) =      -1.0301002181500001
  dip_coef(2884) =      56.8966214869999973
  dip_coef(2885) =      -9.6046424736299993
  dip_coef(2886) =      -6.0541109219999996
  dip_coef(2887) =     -37.9979438435000034
  dip_coef(2888) =      -2.9216127148300002
  dip_coef(2889) =       0.8184371637900000
  dip_coef(2890) =      -5.1874575795900002
  dip_coef(2891) =       4.5813469854199997
  dip_coef(2892) =     -32.8790271558000029
  dip_coef(2893) =      10.0870653170000004
  dip_coef(2894) =      -9.0818944451599997
  dip_coef(2895) =     -42.8684478476999971
  dip_coef(2896) =      39.4231332912000028
  dip_coef(2897) =      -9.9102638608800007
  dip_coef(2898) =       0.0948960598313500
  dip_coef(2899) =       1.1948108207000001
  dip_coef(2900) =      18.9476585630000010
  dip_coef(2901) =       9.2094578597600005
  dip_coef(2902) =      -0.2756029081150000
  dip_coef(2903) =     -10.3433667293999996
  dip_coef(2904) =     -14.3537664143000008
  dip_coef(2905) =    -113.3702495689999949
  dip_coef(2906) =      29.5544300146000012
  dip_coef(2907) =      -0.2061104758790000
  dip_coef(2908) =       2.9840875431999998
  dip_coef(2909) =      28.4508511478000017
  dip_coef(2910) =       2.4049693146600002
  dip_coef(2911) =      -0.0524577547404600
  dip_coef(2912) =     -20.3454959990000006
  dip_coef(2913) =     -36.5658525277999971
  dip_coef(2914) =       0.1107559392530000
  dip_coef(2915) =      -3.1521812822299999
  dip_coef(2916) =     -18.4028386963999999
  dip_coef(2917) =       5.7945376056500004
  dip_coef(2918) =       2.2517299594600000
  dip_coef(2919) =       2.3457510406600002
  dip_coef(2920) =       5.2916385336299996
  dip_coef(2921) =      -0.9483209610260001
  dip_coef(2922) =       9.2962432888500004
  dip_coef(2923) =       7.5988314307999998
  dip_coef(2924) =     -19.9486365485000015
  dip_coef(2925) =      36.0238736668000001
  dip_coef(2926) =      14.1967557440000007
  dip_coef(2927) =      -3.5299010359000000
  dip_coef(2928) =      -3.4896309200800002
  dip_coef(2929) =     -25.2575950380000016
  dip_coef(2930) =     -16.9294630198000000
  dip_coef(2931) =      -8.7853033655699999
  dip_coef(2932) =      -7.6244622124900001
  dip_coef(2933) =      28.6813576857000001
  dip_coef(2934) =      12.0860429370000002
  dip_coef(2935) =     -16.0698342754999999
  dip_coef(2936) =       7.2631892658400004
  dip_coef(2937) =       6.6314417856599999
  dip_coef(2938) =     -21.8776768703000002
  dip_coef(2939) =       1.6743982614699999
  dip_coef(2940) =      -1.8596805106300001
  dip_coef(2941) =     -11.1921248318999993
  dip_coef(2942) =      25.3527168160000009
  dip_coef(2943) =      20.3336542935000004
  dip_coef(2944) =       9.4808197359400008
  dip_coef(2945) =     -66.6676022369999970
  dip_coef(2946) =       0.1334368457460000
  dip_coef(2947) =     -24.1291552980999988
  dip_coef(2948) =      31.2384301682000007
  dip_coef(2949) =      -5.8255479381299997
  dip_coef(2950) =      -2.7063535686000000
  dip_coef(2951) =     -28.0570626113000010
  dip_coef(2952) =     122.4609172769999930
  dip_coef(2953) =       5.0377015399799996
  dip_coef(2954) =      -7.5146961000400001
  dip_coef(2955) =      25.6439512277000006
  dip_coef(2956) =      23.0980610981000005
  dip_coef(2957) =       3.4310323413899999
  dip_coef(2958) =       0.2280623920230000
  dip_coef(2959) =      -1.8102353039200001
  dip_coef(2960) =     -20.6463386661000001
  dip_coef(2961) =      -3.6992638813899998
  dip_coef(2962) =       0.7957854380870000
  dip_coef(2963) =       5.8598865378599996
  dip_coef(2964) =       3.1036321347700002
  dip_coef(2965) =       3.6317188457700000
  dip_coef(2966) =       0.5067886367700000
  dip_coef(2967) =       1.3813332840300001
  dip_coef(2968) =      -7.0806486485000004
  dip_coef(2969) =       1.5041744474600001
  dip_coef(2970) =       4.1319943646899997
  dip_coef(2971) =      -1.4351588699800000
  dip_coef(2972) =       8.3978150983800006
  dip_coef(2973) =      29.8372741015999985
  dip_coef(2974) =       2.5163404646299998
  dip_coef(2975) =      12.0227449861999993
  dip_coef(2976) =      -6.4986078295200000
  dip_coef(2977) =       1.7794691531400000
  dip_coef(2978) =       3.5585728440400000
  dip_coef(2979) =      -4.1643360095500004
  dip_coef(2980) =       6.3373113332199997
  dip_coef(2981) =     -21.2984346902000006
  dip_coef(2982) =       5.3185183887100003
  dip_coef(2983) =      -4.5008413166799999
  dip_coef(2984) =     -32.7528106910000005
  dip_coef(2985) =       7.7060887979699997
  dip_coef(2986) =      -7.2492515826500004
  dip_coef(2987) =       3.1760183493900001
  dip_coef(2988) =      -1.9096889285500001
  dip_coef(2989) =     -12.3380449965000007
  dip_coef(2990) =      18.1865461157000006
  dip_coef(2991) =      16.0445207363999991
  dip_coef(2992) =      10.5566291137999997
  dip_coef(2993) =      13.0714067901999993
  dip_coef(2994) =      17.4525730072999998
  dip_coef(2995) =     -18.4603129346999992
  dip_coef(2996) =       0.0961174966350000
  dip_coef(2997) =      -1.8696193870900000
  dip_coef(2998) =     -37.4448349391999997
  dip_coef(2999) =      -0.5322155196780000
  dip_coef(3000) =      -0.0059467580337070
  dip_coef(3001) =       1.7903525186800000
  dip_coef(3002) =      -2.6671112023200001
  dip_coef(3003) =       7.1253057912799997
  dip_coef(3004) =      -5.3637384172599996
  dip_coef(3005) =      14.7601908907000006
  dip_coef(3006) =      -0.7251565318990000
  dip_coef(3007) =      -4.2550548533299999
  dip_coef(3008) =     -13.4055954190000008
  dip_coef(3009) =      -2.3560603975899999
  dip_coef(3010) =       8.8736696112699995
  dip_coef(3011) =      -6.4285245240600002
  dip_coef(3012) =      -6.7266093484400002
  dip_coef(3013) =      21.4896987917999986
  dip_coef(3014) =       3.3322943845499999
  dip_coef(3015) =      -3.4406163938100001
  dip_coef(3016) =      -5.0966759747700001
  dip_coef(3017) =     -23.8155866293999985
  dip_coef(3018) =      24.8792862925000016
  dip_coef(3019) =      -9.7673111447699998
  dip_coef(3020) =      -7.5112441910600003
  dip_coef(3021) =       1.9473564824600000
  dip_coef(3022) =     -23.2293947695000007
  dip_coef(3023) =       6.6236684516700004
  dip_coef(3024) =       0.0532875305335100
  dip_coef(3025) =       0.1822755910750000
  dip_coef(3026) =      -0.3900155729710000
  dip_coef(3027) =      -0.0339794250659500
  dip_coef(3028) =       0.0353212457561500
  dip_coef(3029) =       0.6719277685430000
  dip_coef(3030) =       0.4385779656650000
  dip_coef(3031) =      -0.2706196563860000
  dip_coef(3032) =      -1.0903328109199999
  dip_coef(3033) =      -9.8461677225600006
  dip_coef(3034) =       1.4361686551599999
  dip_coef(3035) =       0.9093145476050000
  dip_coef(3036) =      -4.7578163071899997
  dip_coef(3037) =      -1.2353623071399999
  dip_coef(3038) =       4.5222021643000003
  dip_coef(3039) =      -3.0771833807900002
  dip_coef(3040) =       0.3422477298100000
  dip_coef(3041) =       7.8921974503500003
  dip_coef(3042) =      -1.1710627923500001
  dip_coef(3043) =       0.6868172368350000
  dip_coef(3044) =       7.8311306199099997
  dip_coef(3045) =       6.3999538389900001
  dip_coef(3046) =      15.3777751484999996
  dip_coef(3047) =      10.5901559077999998
  dip_coef(3048) =       0.9221240082830000
  dip_coef(3049) =       0.1922787181440000
  dip_coef(3050) =      -2.6084468293900001
  dip_coef(3051) =      -1.8390903972299999
  dip_coef(3052) =       0.9737402600340000
  dip_coef(3053) =     -20.8458346325000008
  dip_coef(3054) =       2.9103941844499999
  dip_coef(3055) =       2.5968763719200001
  dip_coef(3056) =       2.5593276684299999
  dip_coef(3057) =     -16.0049301224000011
  dip_coef(3058) =       7.3043534318900001
  dip_coef(3059) =      11.8132281630999998
  dip_coef(3060) =       2.6340776578799998
  dip_coef(3061) =       9.9362937239499995
  dip_coef(3062) =      -1.2528210029600000
  dip_coef(3063) =     -16.7734264218000000
  dip_coef(3064) =     -19.4973443754000009
  dip_coef(3065) =      -1.6857045787400000
  dip_coef(3066) =      -0.9948028588590000
  dip_coef(3067) =      10.7974750981999996
  dip_coef(3068) =      -5.2172256219699999
  dip_coef(3069) =       6.4534047036100004
  dip_coef(3070) =      -0.9678997953500000
  dip_coef(3071) =     -10.9204768368000007
  dip_coef(3072) =      -0.4864582780320000
  dip_coef(3073) =       6.0176235412599999
  dip_coef(3074) =       0.9558922232070000
  dip_coef(3075) =      13.5562359688999994
  dip_coef(3076) =       1.3575166328199999
  dip_coef(3077) =       0.3823202488040000
  dip_coef(3078) =       2.0008386906800002
  dip_coef(3079) =      64.6358222889000018
  dip_coef(3080) =     -30.6579137103000008
  dip_coef(3081) =      52.9715159809000014
  dip_coef(3082) =     -19.2810850725999998
  dip_coef(3083) =       4.5982192278899996
  dip_coef(3084) =       2.4961147895100000
  dip_coef(3085) =       0.0004060448358919
  dip_coef(3086) =      -2.8666776225200001
  dip_coef(3087) =       0.2790773466690000
  dip_coef(3088) =      -0.8969029219090000
  dip_coef(3089) =       0.3000192885920000
  dip_coef(3090) =      -3.2154389372100001
  dip_coef(3091) =      -8.1545036453600002
  dip_coef(3092) =      -2.9363122644400002
  dip_coef(3093) =      -0.8828080959320000
  dip_coef(3094) =       5.9437574869100001
  dip_coef(3095) =      -9.4134043248899992
  dip_coef(3096) =      -2.0033825728700001
  dip_coef(3097) =      -6.0061287485300001
  dip_coef(3098) =       0.4899785973590000
  dip_coef(3099) =      -0.0236986032102600
  dip_coef(3100) =      -0.2844075247490000
  dip_coef(3101) =       1.1850892380600000
  dip_coef(3102) =      -0.6040930345640000
  dip_coef(3103) =      -3.5638186528400002
  dip_coef(3104) =      -0.2772797833680000
  dip_coef(3105) =      -7.0707876383899997
  dip_coef(3106) =      13.5662789676000006
  dip_coef(3107) =       5.3117572147600001
  dip_coef(3108) =       4.3744923329000001
  dip_coef(3109) =       3.5590366886300000
  dip_coef(3110) =     -18.0455318037999994
  dip_coef(3111) =      -0.4635629382060000
  dip_coef(3112) =     -11.8436448575999993
  dip_coef(3113) =       0.8366990331440000
  dip_coef(3114) =       0.0485255441553800
  dip_coef(3115) =       3.2110951906200000
  dip_coef(3116) =       3.9202475490599999
  dip_coef(3117) =     -10.2121889337000002
  dip_coef(3118) =      13.4827632151000003
  dip_coef(3119) =      -1.8275342818800000
  dip_coef(3120) =      -7.9079634506299996
  dip_coef(3121) =       5.4910334152300004
  dip_coef(3122) =      -0.3855026133600000
  dip_coef(3123) =       9.0818008187799997
  dip_coef(3124) =       1.2239387966799999
  dip_coef(3125) =      -0.3243664506600000
  dip_coef(3126) =      -0.9758111363220000
  dip_coef(3127) =       0.9224711489560000
  dip_coef(3128) =     -16.0723715451999993
  dip_coef(3129) =       6.8078133743400002
  dip_coef(3130) =      -3.4101487155800001
  dip_coef(3131) =      -2.8073409834700001
  dip_coef(3132) =       8.2620378790199993
  dip_coef(3133) =       6.9589040205100003
  dip_coef(3134) =      -3.2604602222999999
  dip_coef(3135) =      -2.5225842820100000
  dip_coef(3136) =      -5.3697301606299996
  dip_coef(3137) =       0.3065284647600000
  dip_coef(3138) =      -3.8067173775500001
  dip_coef(3139) =     -21.4361860449999995
  dip_coef(3140) =      -7.8033721413899997
  dip_coef(3141) =      -1.8555482185800001
  dip_coef(3142) =       6.8703147124499999
  dip_coef(3143) =       1.5044185078600001
  dip_coef(3144) =       0.0050882713922330
  dip_coef(3145) =      -3.6248892363600000
  dip_coef(3146) =      -2.8939493011300002
  dip_coef(3147) =       1.7589895201500001
  dip_coef(3148) =      -0.0178335296797000
  dip_coef(3149) =     -13.3320647035000004
  dip_coef(3150) =      -6.5837356003299998
  dip_coef(3151) =       0.9892902059930000
  dip_coef(3152) =      -2.8061392909400000
  dip_coef(3153) =      27.6228900949999989
  dip_coef(3154) =      13.0476947948999999
  dip_coef(3155) =       0.1469270729360000
  dip_coef(3156) =       0.3326714751050000
  dip_coef(3157) =       5.4488156042500000
  dip_coef(3158) =       1.2093645103599999
  dip_coef(3159) =       0.3733668857260000
  dip_coef(3160) =      -2.0459786234099999
  dip_coef(3161) =      -1.9854647687700000
  dip_coef(3162) =       1.0930271977799999
  dip_coef(3163) =       0.7390800451010000
  dip_coef(3164) =      -5.4109454840600000
  dip_coef(3165) =       3.1361191697500002
  dip_coef(3166) =      -0.0113405809627200
  dip_coef(3167) =       0.5806427096050000
  dip_coef(3168) =      -1.4528607089400001
  dip_coef(3169) =      10.4533242732999998
  dip_coef(3170) =      -7.7784032565299999
  dip_coef(3171) =       1.5149456041300000
  dip_coef(3172) =       1.8251324865000000
  dip_coef(3173) =      -6.3396522160000002
  dip_coef(3174) =      -3.3613894876599999
  dip_coef(3175) =      14.8397574432999999
  dip_coef(3176) =       4.1203994287199999
  dip_coef(3177) =       6.5539721319400002
  dip_coef(3178) =       1.6714561704999999
  dip_coef(3179) =      -0.3881252706900000
  dip_coef(3180) =      -0.5381574764420000
  dip_coef(3181) =       4.6707208555899999
  dip_coef(3182) =      -2.4262977989499999
  dip_coef(3183) =     -18.2525807584000006
  dip_coef(3184) =      -3.6856069000799998
  dip_coef(3185) =       2.1173491225999999
  dip_coef(3186) =      -3.1993240524300002
  dip_coef(3187) =       1.7722335405700000
  dip_coef(3188) =      -5.6668281861100001
  dip_coef(3189) =      11.1345288163999996
  dip_coef(3190) =      40.1958118893999981
  dip_coef(3191) =       2.1598239963100001
  dip_coef(3192) =       6.7194491959200002
  dip_coef(3193) =      -0.0599069803987900
  dip_coef(3194) =      -0.7365326781440000
  dip_coef(3195) =       6.7957797143400001
  dip_coef(3196) =       2.6222757987300001
  dip_coef(3197) =      -1.3034717434600001
  dip_coef(3198) =       1.1914299572400000
  dip_coef(3199) =       5.7457438128699998
  dip_coef(3200) =      -4.2579437812599998
  dip_coef(3201) =      -0.0677533055657400
  dip_coef(3202) =      -5.1083570234700000
  dip_coef(3203) =       0.8460388993710000
  dip_coef(3204) =      -4.0160665014400001
  dip_coef(3205) =     -13.1591232234000000
  dip_coef(3206) =      -7.6398330774899996
  dip_coef(3207) =       1.9980123070100000
  dip_coef(3208) =      -1.5607557924900000
  dip_coef(3209) =      -0.6397168594640000
  dip_coef(3210) =       4.2391264897000003
  dip_coef(3211) =      -2.8132853967700000
  dip_coef(3212) =      -6.8660276887000000
  dip_coef(3213) =      -2.0759804935999999
  dip_coef(3214) =       4.1039305702400002
  dip_coef(3215) =       0.6886003578750000
  dip_coef(3216) =       4.9641686591900003
  dip_coef(3217) =      -0.8719230278620000
  dip_coef(3218) =       1.1203636782999999
  dip_coef(3219) =     -26.9815193133999998
  dip_coef(3220) =     -24.6044567675000003
  dip_coef(3221) =      -0.0736115630776400
  dip_coef(3222) =      -0.1393938733750000
  dip_coef(3223) =       0.3242801133430000
  dip_coef(3224) =      -0.2521162382510000
  dip_coef(3225) =      -0.3651659707100000
  dip_coef(3226) =       0.8470197746780000
  dip_coef(3227) =       0.2414158849060000
  dip_coef(3228) =      -1.1334495521700001
  dip_coef(3229) =       0.5887244856640000
  dip_coef(3230) =       1.6650995186999999
  dip_coef(3231) =      -0.0572507456380800
  dip_coef(3232) =       0.3879534456850000
  dip_coef(3233) =       3.1161941190100002
  dip_coef(3234) =       0.6782045796000000
  dip_coef(3235) =      -2.0027861297800000
  dip_coef(3236) =       0.8678878327980000
  dip_coef(3237) =       0.3053785935160000
  dip_coef(3238) =      -1.1900183430300000
  dip_coef(3239) =      -0.3264117089190000
  dip_coef(3240) =       0.2893255841600000
  dip_coef(3241) =      -3.6154533516999998
  dip_coef(3242) =      -3.0220131308300000
  dip_coef(3243) =      -2.1811063519600000
  dip_coef(3244) =      -1.5470160968100000
  dip_coef(3245) =       0.5082827599140000
  dip_coef(3246) =       1.1799946135899999
  dip_coef(3247) =       0.7470953028210000
  dip_coef(3248) =       1.7097339905800000
  dip_coef(3249) =      -0.0497749855006300
  dip_coef(3250) =       1.9026722560100000
  dip_coef(3251) =       1.4057463649999999
  dip_coef(3252) =       0.7917115464409999
  dip_coef(3253) =      12.3064236480000009
  dip_coef(3254) =       9.6347281275099999
  dip_coef(3255) =       0.0927607044580000
  dip_coef(3256) =      -0.7673329335930000
  dip_coef(3257) =      12.2048912187000003
  dip_coef(3258) =      -6.4257655954199997
  dip_coef(3259) =     -25.2014455212999984
  dip_coef(3260) =      -4.4679680649600000
  dip_coef(3261) =     -17.8761119776000008
  dip_coef(3262) =       2.0802956669100001
  dip_coef(3263) =       0.7038392508950000
  dip_coef(3264) =      -7.6330903887300003
  dip_coef(3265) =      -2.1420806956899998
  dip_coef(3266) =       2.2616436882199999
  dip_coef(3267) =      -1.9325820950999999
  dip_coef(3268) =      -6.7144298596100001
  dip_coef(3269) =       2.4385739073799999
  dip_coef(3270) =      -3.8708395377299998
  dip_coef(3271) =      -0.1301197256140000
  dip_coef(3272) =      -0.5080444732190000
  dip_coef(3273) =      -1.8484810827100000
  dip_coef(3274) =       1.4077802207300001
  dip_coef(3275) =      12.8940153805000008
  dip_coef(3276) =      -7.9171225346499998
  dip_coef(3277) =      20.0633541519000005
  dip_coef(3278) =      -0.9080606512510000
  dip_coef(3279) =      -0.0746409301500400
  dip_coef(3280) =      -8.1698672646400006
  dip_coef(3281) =      -0.5728203503310000
  dip_coef(3282) =      -0.5855641566599999
  dip_coef(3283) =       0.1482374969940000
  dip_coef(3284) =      -0.3156899857750000
  dip_coef(3285) =      -0.4996619960910000
  dip_coef(3286) =      -1.1195629778600000
  dip_coef(3287) =      -5.3438892710900001
  dip_coef(3288) =       6.1239552386199998
  dip_coef(3289) =      -2.3277122234199998
  dip_coef(3290) =       0.3107681844240000
  dip_coef(3291) =      -1.5191915539400001
  dip_coef(3292) =      -0.7158163970870000
  dip_coef(3293) =       0.0305807299323600
  dip_coef(3294) =      -1.3453505547900000
  dip_coef(3295) =       3.3778597878899999
  dip_coef(3296) =      -1.4874485891500000
  dip_coef(3297) =       1.2012496548200000
  dip_coef(3298) =       1.6729580095000001
  dip_coef(3299) =      -0.3144946355580000
  dip_coef(3300) =      -9.0684436718200008
  dip_coef(3301) =      -2.2231994709400000
  dip_coef(3302) =      -0.4477014197980000
  dip_coef(3303) =      -2.5487099693199999
  dip_coef(3304) =       2.4183702287200002
  dip_coef(3305) =       3.6095662063299998
  dip_coef(3306) =       1.8499246443099999
  dip_coef(3307) =       0.2333060406080000
  dip_coef(3308) =      -0.6480855886660000
  dip_coef(3309) =      -3.7127343768799999
  dip_coef(3310) =       1.2648029274199999
  dip_coef(3311) =       0.0906359199182500
  dip_coef(3312) =      -0.5396139719930000
  dip_coef(3313) =       1.2723995740200000
  dip_coef(3314) =       1.4373730864600001
  dip_coef(3315) =       1.2823948360399999
  dip_coef(3316) =      -1.9120380464700000
  dip_coef(3317) =      -1.0264336473300000
  dip_coef(3318) =      -1.4878213964500000
  dip_coef(3319) =       0.8415660739290000
  dip_coef(3320) =      10.1548710275000005
  dip_coef(3321) =      -1.4089046012799999
  dip_coef(3322) =       4.1616267194500001
  dip_coef(3323) =      -0.7357590516770000
  dip_coef(3324) =       6.7875770435299998
  dip_coef(3325) =       4.1394573742300000
  dip_coef(3326) =       5.9654410035099996
  dip_coef(3327) =      -4.3995519213599996
  dip_coef(3328) =      -1.5357537168900000
  dip_coef(3329) =       3.6279000421699998
  dip_coef(3330) =      -1.2735227523899999
  dip_coef(3331) =      -0.2788075659490000
  dip_coef(3332) =       0.3912789765710000
  dip_coef(3333) =      -8.7630586761099991
  dip_coef(3334) =      -1.8873578855599999
  dip_coef(3335) =      -1.7740731485200001
  dip_coef(3336) =       0.9521898412720000
  dip_coef(3337) =      12.7008371500999999
  dip_coef(3338) =       8.7745479935400006
  dip_coef(3339) =       0.2875782830210000
  dip_coef(3340) =       0.0873938140680200
  dip_coef(3341) =       3.0399246688799999
  dip_coef(3342) =       0.3405242121210000
  dip_coef(3343) =      -0.5606370034780001
  dip_coef(3344) =       0.2755925924720000
  dip_coef(3345) =      -0.2554647659710000
  dip_coef(3346) =       1.2258685226799999
  dip_coef(3347) =       0.4294854456500000
  dip_coef(3348) =       0.1443549929810000
  dip_coef(3349) =       0.7767835432410000
  dip_coef(3350) =      -0.1128761248940000
  dip_coef(3351) =      -4.0265254676200000
  dip_coef(3352) =      -3.7129900514899998
  dip_coef(3353) =       0.4641791309670000
  dip_coef(3354) =      -9.4901078126700007
  dip_coef(3355) =      -4.1243682388999998
  dip_coef(3356) =       0.2578687866100000
  dip_coef(3357) =      -3.6529793434900002
  dip_coef(3358) =      -4.3533395354200000
  dip_coef(3359) =      -3.2628047530500002
  dip_coef(3360) =      -3.1471065983400002
  dip_coef(3361) =      -3.6691482571099998
  dip_coef(3362) =       3.1387979026299999
  dip_coef(3363) =      -0.1747969973220000
  dip_coef(3364) =      -7.5256832878999997
  dip_coef(3365) =       4.9355651068600004
  dip_coef(3366) =      -5.8305786448200001
  dip_coef(3367) =      -0.0843218460273000
  dip_coef(3368) =      -0.1334411974300000
  dip_coef(3369) =       0.0635504863758300
  dip_coef(3370) =      -0.6646711358560000
  dip_coef(3371) =      -0.2643857201510000
  dip_coef(3372) =       1.0791957323200001
  dip_coef(3373) =       1.5331316805299999
  dip_coef(3374) =      -2.0341483709000001
  dip_coef(3375) =       0.8405534069440000
  dip_coef(3376) =       2.9775496196200000
  dip_coef(3377) =       0.0854749848760000
  dip_coef(3378) =       1.3144325627800000
  dip_coef(3379) =       5.1991411967500003
  dip_coef(3380) =       0.9560745674250000
  dip_coef(3381) =      -0.7163874709600000
  dip_coef(3382) =      -0.9923067755640000
  dip_coef(3383) =      -0.2882683537740000
  dip_coef(3384) =      -0.8766645048160000
  dip_coef(3385) =      -1.0149899031000000
  dip_coef(3386) =       0.5225083382080000
  dip_coef(3387) =      -1.3521118185700001
  dip_coef(3388) =      -2.0165092270099998
  dip_coef(3389) =      -0.9639892959060000
  dip_coef(3390) =      -1.8891818679300001
  dip_coef(3391) =       3.0560726370200002
  dip_coef(3392) =       3.0295653164399998
  dip_coef(3393) =      -4.2787199810100001
  dip_coef(3394) =       0.6240526238380000
  dip_coef(3395) =       0.2763140501310000
  dip_coef(3396) =      -9.2738838787999995
  dip_coef(3397) =       0.2960844571300000
  dip_coef(3398) =      -1.6012250699199999
  dip_coef(3399) =       9.1404123498899992
  dip_coef(3400) =       7.0276726957799998
  dip_coef(3401) =       1.1918320093799999
  dip_coef(3402) =      10.6568786622000005
  dip_coef(3403) =       0.8337804065940000
  dip_coef(3404) =      13.0840294798999999
  dip_coef(3405) =     -18.9998865744000014
  dip_coef(3406) =       0.3691076878710000
  dip_coef(3407) =      -9.1840545203499993
  dip_coef(3408) =      -0.0378706051803300
  dip_coef(3409) =      -0.3052250762060000
  dip_coef(3410) =       0.2745425790350000
  dip_coef(3411) =     -10.4468792918000002
  dip_coef(3412) =      -1.0280200353000000
  dip_coef(3413) =       0.9546770303500000
  dip_coef(3414) =      -0.4088048589820000
  dip_coef(3415) =     -11.0572299137000005
  dip_coef(3416) =      -1.2228600699600001
  dip_coef(3417) =       0.5830865568100000
  dip_coef(3418) =      -3.6339701000300000
  dip_coef(3419) =       0.4106688285930000
  dip_coef(3420) =      -1.6680090627799999
  dip_coef(3421) =      -5.8331817306999998
  dip_coef(3422) =      -0.1803242591560000
  dip_coef(3423) =      -7.7675887176299998
  dip_coef(3424) =      -5.7478844371499997
  dip_coef(3425) =       2.6176018051200001
  dip_coef(3426) =      -4.4077661801000003
  dip_coef(3427) =       2.2877685145300002
  dip_coef(3428) =       0.4904830326350000
  dip_coef(3429) =       0.5686931460040000
  dip_coef(3430) =      -0.6234006735360000
  dip_coef(3431) =      -0.7970775795380000
  dip_coef(3432) =      -1.2756127735700000
  dip_coef(3433) =      -0.2056235078400000
  dip_coef(3434) =       1.0190915818099999
  dip_coef(3435) =       0.0152720334204500
  dip_coef(3436) =       1.6240168603900000
  dip_coef(3437) =      -1.9921952129800000
  dip_coef(3438) =      -0.9926866150570000
  dip_coef(3439) =       6.2532743529800001
  dip_coef(3440) =       0.2279084811470000
  dip_coef(3441) =       1.8132725831000001
  dip_coef(3442) =      -0.1187147834510000
  dip_coef(3443) =       2.5520647797999998
  dip_coef(3444) =       4.8636969605799996
  dip_coef(3445) =       2.1208631250900001
  dip_coef(3446) =       2.2738246242200000
  dip_coef(3447) =      -0.4174564294200000
  dip_coef(3448) =       3.9473877299900000
  dip_coef(3449) =      -2.9169107335000000
  dip_coef(3450) =       0.5301248281760000
  dip_coef(3451) =       4.5368263179600001
  dip_coef(3452) =       1.7623769114600001
  dip_coef(3453) =      -3.5686993795299999
  dip_coef(3454) =      -2.3274950866299999
  dip_coef(3455) =       3.5868477864699999
  dip_coef(3456) =       0.7470649695060000
  dip_coef(3457) =       3.0488323057700000
  dip_coef(3458) =       0.3204413177830000
  dip_coef(3459) =       2.3507165030300001
  dip_coef(3460) =      -0.8851450418130000
  dip_coef(3461) =      14.3401037687000006
  dip_coef(3462) =      -1.8350254847700000
  dip_coef(3463) =      -0.8958498786969999
  dip_coef(3464) =       8.2019535567399995
  dip_coef(3465) =       4.2336670621100003
  dip_coef(3466) =       7.3046011153699997
  dip_coef(3467) =      -4.2868665277400000
  dip_coef(3468) =       0.8696646909100000
  dip_coef(3469) =      -0.9648615863480000
  dip_coef(3470) =       3.2560073638499998
  dip_coef(3471) =       1.7963051662200000
  dip_coef(3472) =       4.8373278536499997
  dip_coef(3473) =      -2.2106011116500000
  dip_coef(3474) =      -0.5740129775270000
  dip_coef(3475) =       0.0064140307845970
  dip_coef(3476) =      -0.0124587402725800
  dip_coef(3477) =       0.1197012678940000
  dip_coef(3478) =       0.1191523173390000
  dip_coef(3479) =       0.0854358981462000
  dip_coef(3480) =      -0.3182407390780000
  dip_coef(3481) =      -0.2374932004120000
  dip_coef(3482) =       0.3576178718830000
  dip_coef(3483) =      -0.0304055534905600
  dip_coef(3484) =      -0.0377936870991700
  dip_coef(3485) =      -0.1991173379900000
  dip_coef(3486) =      -0.2122774160920000
  dip_coef(3487) =      -0.1385087133150000
  dip_coef(3488) =       0.0599715787749500
  dip_coef(3489) =      -0.0091396746852320
  dip_coef(3490) =       0.3755158507970000
  dip_coef(3491) =       0.0018431415291590
  dip_coef(3492) =       0.0341199867871900
  dip_coef(3493) =       0.1107424262090000
  dip_coef(3494) =      -0.1261007973040000
  dip_coef(3495) =      -0.2156464201230000
  dip_coef(3496) =      -0.3530346524490000
  dip_coef(3497) =      -0.0872069296222900
  dip_coef(3498) =      -0.3696186090200000
  dip_coef(3499) =      -0.3816926603840000
  dip_coef(3500) =      -0.2742379715720000
  dip_coef(3501) =       0.6344674119540000
  dip_coef(3502) =       0.3411453855010000
  dip_coef(3503) =       0.0526191548134100
  dip_coef(3504) =      -0.7362610558340000
  dip_coef(3505) =      -0.0350667777863900
  dip_coef(3506) =      -0.2641766589740000
  dip_coef(3507) =       0.6324812100540000
  dip_coef(3508) =       1.0235319917900001
  dip_coef(3509) =      -0.0061028652728510
  dip_coef(3510) =      -5.5340869772800003
  dip_coef(3511) =      -1.8569316326500001
  dip_coef(3512) =      -0.5233387236150000
  dip_coef(3513) =       6.6915111258300000
  dip_coef(3514) =      12.7539057421000006
  dip_coef(3515) =      -2.3222343743199998
  dip_coef(3516) =       0.1013174364130000
  dip_coef(3517) =       0.0301048485586600
  dip_coef(3518) =       1.2211815604300000
  dip_coef(3519) =       1.0133774645100000
  dip_coef(3520) =       0.0516281080638900
  dip_coef(3521) =       0.4487558830940000
  dip_coef(3522) =       0.7384742821319999
  dip_coef(3523) =       0.3283028764410000
  dip_coef(3524) =       0.9834169452800000
  dip_coef(3525) =       0.1314903143720000
  dip_coef(3526) =       3.3175037334100002
  dip_coef(3527) =       0.0323629817303600
  dip_coef(3528) =      -0.1628593772390000
  dip_coef(3529) =      -1.8487191281699999
  dip_coef(3530) =      -1.3959717570700001
  dip_coef(3531) =      -0.1498406292570000
  dip_coef(3532) =      -0.5384199044640000
  dip_coef(3533) =      -0.8971519500460000
  dip_coef(3534) =       1.9089931433500000
  dip_coef(3535) =       0.7922359377230000
  dip_coef(3536) =       0.2598387972020000
  dip_coef(3537) =       0.4815753107520000
  dip_coef(3538) =       0.4612053026510000
  dip_coef(3539) =       0.4171335048920000
  dip_coef(3540) =      -0.6722888998630000
  dip_coef(3541) =       0.7519990847880000
  dip_coef(3542) =      -0.9938764595789999
  dip_coef(3543) =       0.5045425623010000
  dip_coef(3544) =      -0.7547046428810000
  dip_coef(3545) =      -1.1007375827300001
  dip_coef(3546) =       0.5059331451220000
  dip_coef(3547) =      -0.6721340635500000
  dip_coef(3548) =       0.3088031327910000
  dip_coef(3549) =      -0.6310458908760000
  dip_coef(3550) =      -2.1572418210700000
  dip_coef(3551) =      -1.1498075736300000
  dip_coef(3552) =      -0.5943652316130000
  dip_coef(3553) =      -0.3350117796840000
  dip_coef(3554) =       1.5405244285699999
  dip_coef(3555) =       0.6742307892610000
  dip_coef(3556) =       1.3476579031000000
  dip_coef(3557) =      -4.0940474629299999
  dip_coef(3558) =      -2.0887666023999998
  dip_coef(3559) =      -1.1849593659200000
  dip_coef(3560) =      -0.8590889140830000
  dip_coef(3561) =       0.6681980517060000
  dip_coef(3562) =       0.4450770084040000
  dip_coef(3563) =       0.2881557635300000
  dip_coef(3564) =      -0.7369542915030000
  dip_coef(3565) =      -0.2034983654710000
  dip_coef(3566) =      -1.7173139341699999
  dip_coef(3567) =      -1.0380352568600000
  dip_coef(3568) =       0.8402810921380000
  dip_coef(3569) =      -2.1308751301000002
  dip_coef(3570) =       0.6282047640970000
  dip_coef(3571) =       0.7012262094500000
  dip_coef(3572) =      -1.5552276168700001
  dip_coef(3573) =      -0.6629145177769999
  dip_coef(3574) =      -1.0468302142200001
  dip_coef(3575) =       0.1643333294620000
  dip_coef(3576) =      -0.7632740934580000
  dip_coef(3577) =       0.0066884778504940
  dip_coef(3578) =      -1.1621797655999999
  dip_coef(3579) =       1.6272062574399999
  dip_coef(3580) =       0.7910366566290000
  dip_coef(3581) =       0.7962803936130000
  dip_coef(3582) =       0.4301769072430000
  dip_coef(3583) =      -0.0707597972959000
  dip_coef(3584) =       5.7099147703200002
  dip_coef(3585) =       0.1417572733100000
  dip_coef(3586) =       0.8473370580200000
  dip_coef(3587) =      -0.4610734978480000
  dip_coef(3588) =       0.3493819626850000
  dip_coef(3589) =      -0.5525834364510001
  dip_coef(3590) =      -3.1512095509200000
  dip_coef(3591) =      -0.7919426463080000
  dip_coef(3592) =       1.2404727792600001
  dip_coef(3593) =      -1.7086438376399999
  dip_coef(3594) =       0.0660248197009900
  dip_coef(3595) =      -0.0579965973302000
  dip_coef(3596) =       0.3247567388680000
  dip_coef(3597) =       3.8052795597100002
  dip_coef(3598) =       3.3356625420500001
  dip_coef(3599) =       0.2789025822050000
  dip_coef(3600) =       0.2306322710660000
  dip_coef(3601) =      -4.1706714437099999
  dip_coef(3602) =      -1.3000964007600000
  dip_coef(3603) =       1.9635245925000000
  dip_coef(3604) =      -2.7823149492399999
  dip_coef(3605) =      -3.2415887571500002
  dip_coef(3606) =      -3.4559154472500002
  dip_coef(3607) =       4.5808865066399997
  dip_coef(3608) =      -2.4786859531099998
  dip_coef(3609) =       0.4651033960400000
  dip_coef(3610) =       2.4756034733900001
  dip_coef(3611) =      -0.7544981771520000
  dip_coef(3612) =       4.0860873700699996
  dip_coef(3613) =      -5.8053081642700004
  dip_coef(3614) =      -3.1873202858499998
  dip_coef(3615) =      -0.8553182152730000
  dip_coef(3616) =      -1.4106385775200001
  dip_coef(3617) =      -2.5851824670400001
  dip_coef(3618) =      -3.0114203623600000
  dip_coef(3619) =       3.5924131892300002
  dip_coef(3620) =       2.8789382791399998
  dip_coef(3621) =       1.0695019162299999
  dip_coef(3622) =      -3.0916005224999998
  dip_coef(3623) =      -0.4076009259590000
  dip_coef(3624) =       0.1853821125160000
  dip_coef(3625) =      -0.7684557836510000
  dip_coef(3626) =       0.0265056156421100
  dip_coef(3627) =      -0.7690152214480001
  dip_coef(3628) =       0.0810907433258300
  dip_coef(3629) =      -1.4336949961500001
  dip_coef(3630) =       1.7362047160100000
  dip_coef(3631) =      -0.9824779103100000
  dip_coef(3632) =       0.3287444259660000
  dip_coef(3633) =       1.6831436538900000
  dip_coef(3634) =       0.9947990348560000
  dip_coef(3635) =      -2.1709225317600001
  dip_coef(3636) =      -0.4190543605970000
  dip_coef(3637) =       0.1762529564440000
  dip_coef(3638) =       0.6300856203580000
  dip_coef(3639) =      -0.6239986708790000
  dip_coef(3640) =      -0.1833247603770000
  dip_coef(3641) =      -2.2323342537499999
  dip_coef(3642) =      -0.5758038297640000
  dip_coef(3643) =      -0.3586334375820000
  dip_coef(3644) =      -2.1728576084500002
  dip_coef(3645) =      -0.1668727724720000
  dip_coef(3646) =      -0.5216447602410000
  dip_coef(3647) =       0.1468175868890000
  dip_coef(3648) =       0.4723683563410000
  dip_coef(3649) =       1.3905251676499999
  dip_coef(3650) =       1.2069606271300000
  dip_coef(3651) =       0.0855004730724000
  dip_coef(3652) =       0.3209670486080000
  dip_coef(3653) =       1.4665206592600000
  dip_coef(3654) =       0.9131681791289999
  dip_coef(3655) =       2.5321942204600001
  dip_coef(3656) =       2.0048303610299998
  dip_coef(3657) =      -0.6549881387320000
  dip_coef(3658) =      -1.8240567645100001
  dip_coef(3659) =       0.0361163972893700
  dip_coef(3660) =       0.1914345098090000
  dip_coef(3661) =      -2.4051362130100000
  dip_coef(3662) =       0.0042675439303210
  dip_coef(3663) =      -0.0189191287628800
  dip_coef(3664) =       2.0881032106699999
  dip_coef(3665) =      -0.3657566464270000
  dip_coef(3666) =      -0.1817600398800000
  dip_coef(3667) =      -2.4153470560299999
  dip_coef(3668) =       2.9735242041699999
  dip_coef(3669) =       0.5070975490590000
  dip_coef(3670) =       0.0560690628249100
  dip_coef(3671) =       0.4251506297440000
  dip_coef(3672) =      -0.9289140633890000
  dip_coef(3673) =       0.8444739921360001
  dip_coef(3674) =       0.8761495677900000
  dip_coef(3675) =      -0.1702133239380000
  dip_coef(3676) =      -5.3489504228799998
  dip_coef(3677) =       0.4153192533480000
  dip_coef(3678) =       0.3150638264470000
  dip_coef(3679) =      -2.4728068602500000
  dip_coef(3680) =       2.9128198474599998
  dip_coef(3681) =       1.9494108107799999
  dip_coef(3682) =       0.4505245625610000
  dip_coef(3683) =       1.0516350554100000
  dip_coef(3684) =       1.8109370263200000
  dip_coef(3685) =      -3.3016341500599999
  dip_coef(3686) =      -9.2962915646499997
  dip_coef(3687) =       3.7369838741099999
  dip_coef(3688) =      -2.8309744927299998
  dip_coef(3689) =       4.5713887612499997
  dip_coef(3690) =      -6.2438800085999997
  dip_coef(3691) =      -0.0194453891655400
  dip_coef(3692) =       2.1999977441400000
  dip_coef(3693) =      -3.2150338049900000
  dip_coef(3694) =      -8.3427412311100007
  dip_coef(3695) =      -9.4947749913399999
  dip_coef(3696) =       1.8086412924299999
  dip_coef(3697) =       3.1374197053900001
  dip_coef(3698) =       2.0132334763899999
  dip_coef(3699) =      -2.7700062443000002
  dip_coef(3700) =      -0.9192549660960000
  dip_coef(3701) =       0.6050373840620000
  dip_coef(3702) =      -0.2600109363190000
  dip_coef(3703) =       3.4338448225099998
  dip_coef(3704) =      -8.8028868517800003
  dip_coef(3705) =       6.4074828230199996
  dip_coef(3706) =       8.1255307339799998
  dip_coef(3707) =       0.9188441661480000
  dip_coef(3708) =       3.6475504440300002
  dip_coef(3709) =      -8.8917351474299995
  dip_coef(3710) =     -11.9643042673999993
  dip_coef(3711) =       3.1224406994399998
  dip_coef(3712) =       3.4979508201799998
  dip_coef(3713) =       1.5363670441199999
  dip_coef(3714) =      -0.0838376470760600
  dip_coef(3715) =      -5.0445135132600001
  dip_coef(3716) =      -0.1780787675750000
  dip_coef(3717) =       4.1369343381499997
  dip_coef(3718) =      -0.0183462743163600
  dip_coef(3719) =      -1.1929935970100001
  dip_coef(3720) =       2.0932083595800002
  dip_coef(3721) =      15.4179337813000004
  dip_coef(3722) =       6.1913999820800001
  dip_coef(3723) =      -4.7116713858899999
  dip_coef(3724) =      -1.8515943788500000
  dip_coef(3725) =      -3.2756358654200000
  dip_coef(3726) =       2.8699991639900002
  dip_coef(3727) =      -0.7712962833070000
  dip_coef(3728) =      -0.4035511441870000
  dip_coef(3729) =       0.1449890373900000
  dip_coef(3730) =       0.3176757888030000
  dip_coef(3731) =      -0.6218409609000000
  dip_coef(3732) =      -0.1034515119690000
  dip_coef(3733) =      -0.1746259052030000
  dip_coef(3734) =       0.7655506576909999
  dip_coef(3735) =       0.0193120433203700
  dip_coef(3736) =       0.7309517750350000
  dip_coef(3737) =       1.2042771731799999
  dip_coef(3738) =      -1.1978367861999999
  dip_coef(3739) =       0.5276736302510000
  dip_coef(3740) =       0.6136290257230000
  dip_coef(3741) =       0.4212835332950000
  dip_coef(3742) =      -0.9021542624350000
  dip_coef(3743) =       1.3812362224600001
  dip_coef(3744) =      -0.1134497778270000
  dip_coef(3745) =      -0.5416206984080000
  dip_coef(3746) =       0.9225529510450000
  dip_coef(3747) =      -0.2031525054280000
  dip_coef(3748) =      -0.3603851041280000
  dip_coef(3749) =       0.0216884225427800
  dip_coef(3750) =       0.3043672643040000
  dip_coef(3751) =       0.0456759246058700
  dip_coef(3752) =       0.0186287408297400
  dip_coef(3753) =       0.3712070818400000
  dip_coef(3754) =       0.0852693712761900
  dip_coef(3755) =       0.0068541189400250
  dip_coef(3756) =       1.2934158350300000
  dip_coef(3757) =      -0.4585128678410000
  dip_coef(3758) =      -0.6453210845150000
  dip_coef(3759) =       0.1411622563800000
  dip_coef(3760) =       2.4228930632400001
  dip_coef(3761) =       0.8551141633450000
  dip_coef(3762) =       1.1224855799600000
  dip_coef(3763) =       0.4642283782650000
  dip_coef(3764) =      -0.4518803098210000
  dip_coef(3765) =      -0.3513206488150000
  dip_coef(3766) =      -2.3355457669900002
  dip_coef(3767) =      -0.6921320190819999
  dip_coef(3768) =      -1.3279224057500001
  dip_coef(3769) =      -1.0195909326199999
  dip_coef(3770) =      -0.4113069932280000
  dip_coef(3771) =      -0.3537934505060000
  dip_coef(3772) =      -0.6078433818710000
  dip_coef(3773) =       4.6217654850600001
  dip_coef(3774) =       0.2960485028010000
  dip_coef(3775) =      -0.6911911922020000
  dip_coef(3776) =      -0.2783117017240000
  dip_coef(3777) =       0.8427768192190001
  dip_coef(3778) =       1.1050374349100001
  dip_coef(3779) =       2.7747505652400002
  dip_coef(3780) =       0.1208913030040000
  dip_coef(3781) =      -0.4186434551670000
  dip_coef(3782) =       0.7657585620670000
  dip_coef(3783) =       0.0787350357525500
  dip_coef(3784) =       0.6728334527470000
  dip_coef(3785) =      -0.0768101701787100
  dip_coef(3786) =       0.0864285342394800
  dip_coef(3787) =      -0.0886741709210800
  dip_coef(3788) =      -0.3027553722630000
  dip_coef(3789) =      -0.8909983072270000
  dip_coef(3790) =      -0.9436855872510000
  dip_coef(3791) =       0.3057776245860000
  dip_coef(3792) =      -0.3548738204780000
  dip_coef(3793) =      -0.2423873079410000
  dip_coef(3794) =      -0.7129955600590000
  dip_coef(3795) =       2.0308641645500001
  dip_coef(3796) =      -0.3335640744620000
  dip_coef(3797) =       2.0157759407800002
  dip_coef(3798) =      -2.1930287696700002
  dip_coef(3799) =       2.6137200462800001
  dip_coef(3800) =       0.0223683384750100
  dip_coef(3801) =       1.6903423799999999
  dip_coef(3802) =      -1.3266445682200001
  dip_coef(3803) =       1.5553187371999999
  dip_coef(3804) =      -0.1163384392650000
  dip_coef(3805) =       1.1373788826100000
  dip_coef(3806) =      -1.1697953094500000
  dip_coef(3807) =      -1.3371323845500001
  dip_coef(3808) =      -0.3768030909880000
  dip_coef(3809) =       0.0337035517355500
  dip_coef(3810) =      -1.3201903698499999
  dip_coef(3811) =      -0.3712737430120000
  dip_coef(3812) =       0.5835203176580001
  dip_coef(3813) =       0.4139985334020000
  dip_coef(3814) =       0.4442641642770000
  dip_coef(3815) =      -0.2042402183000000
  dip_coef(3816) =       0.1605057096160000
  dip_coef(3817) =      -1.8547495068199999
  dip_coef(3818) =      -0.9333888893770000
  dip_coef(3819) =       6.1076966126499999
  dip_coef(3820) =      -1.1652316067899999
  dip_coef(3821) =      -2.1071269455100001
  dip_coef(3822) =      -1.1303674217199999
  dip_coef(3823) =      -5.7807844142500002
  dip_coef(3824) =      -1.7792366060600000
  dip_coef(3825) =       2.2069187585800001
  dip_coef(3826) =       0.5652068731070000
  dip_coef(3827) =       2.0627178139000000
  dip_coef(3828) =      -0.7527366183880000
  dip_coef(3829) =      -0.0732708374383500
  dip_coef(3830) =       0.5796591504500000
  dip_coef(3831) =      -2.5621146503799999
  dip_coef(3832) =      -2.6282982820499998
  dip_coef(3833) =       0.4720769122340000
  dip_coef(3834) =       0.4379442200840000
  dip_coef(3835) =      -3.0392151360700002
  dip_coef(3836) =      -1.1111858012200000
  dip_coef(3837) =       0.3883366516600000
  dip_coef(3838) =       0.1493471126090000
  dip_coef(3839) =       1.7988424579100000
  dip_coef(3840) =       1.1690113883200000
  dip_coef(3841) =       2.4091964604500000
  dip_coef(3842) =      -1.3032684065000000
  dip_coef(3843) =       0.6837009561750000


  return
end subroutine predip1
