!#######################################################################
      module utils

use decimal, only: long,dop
use op1lib,  only: zerovxi, zerovxd, zeromxd, zeromxi
use ioqc,    only: getintyp, getnm, nm2xmat
use logdat, only: errormsg

use vcglobal
use global, only:pi, tiny, logisopen, lwarn, keyorig
use maxv, only: maxdim,maxsta,maxset
use keyunitsmod
use vchammod
use vcpntmod
use vctransmod
use vcplotmod
use vchfitmod
use functions, only: ddipole,dpolar
use vcpntmod
use keyunitsmod

use functions, only: mosc,dmosc,vpes,dfridr
use logdat, only: message, routine, inptit

      implicit none
      private
      public :: vchdefault,vcwarnmsg,vcerrormsg,vccntrl,vcwrline,q2intcoo,&
                x2q,q2x,rmblnk0,rotatex,rotatey,rotatez,qvec,dihed,angel,&
                setintcoo,vec2par2d,vcrmsd,dvcrmsd,par2vec2d,zeropar,&
                zerocons,active_mask,numpar,numtpar,general_cons,parstats,&
                vcnorm,parnumt,parnum,integralsym,comcalc,qvec2,&
                getlabel,dattrans,translate0,rotate1,rexcoo,&
                chkcoocut,chkcoovec,par2vec_lm,vec2par_lm,lmrmsd,&
                rotatemodes,uppercase,getcharacters,j2x,x2poly,poly2x,&
                force_pos,set_pair_cons,cleanirreps,nmeckart,q2xf,x2qf,&
                qvec2f,rotrans,switchdat2,rotx,roty,rotz,getfreq,q2curv,&
                curv2q,rotpairvec,reweight,x2dihed,transformq1,nm2dihed

      contains

!#######################################################################
!
! VCHDEFAULT
!
! general parameters used by more than one vcham program
!
!#######################################################################

      subroutine vchdefault

      implicit none

      integer(long)       :: i,j,k

!-----------------------------------------------------------------------
! set parameters
!-----------------------------------------------------------------------
      call setunits

!     definition of surface to be fitted
      do i=1,maxsta
         do j=1,maxdim
            ncoeff(j,i)=0
            ifdiab(j,i)=0
            do k=1,maxsta
               ncoeffcp(j,i,k)=0
               ifdiabcp(j,i,k)=0
            enddo
         enddo
      enddo

!     turns lambda*q into lambda*sin(q) in pes routine
      lsincoup = .false.

!     turns lambda*q into lambda*cos(q) in pes routine
         do j=1,maxdim
           lcoscoup(j) = .false.
         enddo

!     curval modes:
      nmcurv = 0.0
      lnmcurv = .false.
      lspecies = .false.
      lqtransform = .false.
      qtransform = 0
      nmcurv = 0
      nm2curv = 0
      nmcurvtype = 0

!     the paramters for the fitting type
      lortho=.true.

!     eigenvectors of the Hessian to be used (only vibrations or all)
      lalleig=.false.

!-----------------------------------------------------------------------
! zero standard arrays
!-----------------------------------------------------------------------
      do i = 1, maxdim
 !        nmlab='???'
         nmlab='A'
      enddo
      do i = 1, int(maxdim/2)+1
         rotang = 0.0_dop
         rotmxfq = 0.0_dop
      enddo

      do i = 1, (int(maxdim/2)+1)
        do j = 1, 2
           rotm(i,j) = 0
        enddo
      enddo
      lmodesym = .false.
      call zerovxi(info,maxset*3)
      call zerovxd(mass,maxdim)
      call zerovxd(xcoo0,maxdim)
      call zerovxd(q0,maxdim)
      call zerovxd(freq0,maxdim)
      call zerovxi(iatsym,maxdim)
      call zeromxd(nmcoo,maxdim,maxdim)
      call zeromxd(coonm,maxdim,maxdim)
      
      linternal = .false.
      call zerovxi(cootyp,maxdim)
      call zerovxi(adof,maxdim)
      call zeromxi(coodef,4,maxdim)
      call zerovxd(intcoo0,maxdim)
      call zerovxi(nmcurv,(INT(maxdim/2)+1)*maxdim)
      call zerovxi(nmcurvtype,INT(maxdim/2)+1)
      call zerovxd(c0,(INT(maxdim/2)+1)*maxdim)

      ignoredof = .false.
      i_ignore = 0      

! 2d  functions
      cp2dtyp= 0
      stacp2d = 0  
      stacp2d = 0
      dofcp2d = 0
      dofcp2d = 0
      ocp2d = 0

!-----------------------------------------------------------------------
! set energy and coordinate bounds for data sets to be included
!-----------------------------------------------------------------------
      edsup = 99999.0_dop
      edslow = -99999.0_dop
      do i=1,maxdim
         qbnd(i,1) = -99999.0
         qbnd(i,2) = 99999.0
      enddo
       
!     ignoring to print some modes onto the info file, see vchfit outpar and readguess
      i_ignore = 0
       
      return
      end subroutine vchdefault

!#######################################################################

      subroutine genq(q1,q,v,xincr,nincr,nmodes)

      implicit none

      integer(long)                             :: i
      integer(long), intent(in)                 :: nmodes,nincr
      real(dop), intent(in)                     :: xincr
      real(dop), dimension(nmodes), intent(in)  :: q,v
      real(dop), dimension(nmodes), intent(out) :: q1


      do i=1,nmodes
         q1(i)=q(i)+xincr*nincr*v(i)
      enddo

      return
      end subroutine genq

!#######################################################################

      subroutine logq(q1,q,v,unit,nmodes)

      implicit none

      integer(long)                             :: i
      integer(long), intent(in)                 :: nmodes,unit
      real(dop), dimension(nmodes), intent(in)  :: q1,q,v

      write(unit,'(/,a)') '   New       Old      Vector  '
      do i=1,nmodes
         write(unit,'(3(2x,f8.4))') q1(i),q(i),v(i)
      enddo

      return
      end subroutine logq

!#######################################################################

      subroutine x2q(q,xcoo,xcoo0,coonm,nmodes,ncoo,maxdim)

      implicit none

      integer(long)                                     :: i,j
      integer(long), intent(in)                         :: nmodes,ncoo,maxdim
      real(dop), dimension(ncoo), intent(in)            :: xcoo0
      real(dop), dimension(nmodes), intent(out)         :: q
      real(dop), dimension(maxdim,maxdim), intent(in)   :: coonm
      real(dop), dimension(ncoo), intent(inout)         :: xcoo
      real(dop), dimension(1000)                        :: xcoo_old

      do i=1,ncoo
         xcoo_old(i)=xcoo(i)
         xcoo(i)=xcoo(i)-xcoo0(i)
      enddo

      do j=1,nmodes
         q(j)=0.0
         do i=1,ncoo
            q(j)=q(j)+coonm(j,i)*xcoo(i)
         enddo
      enddo

!     restore old values
      do i=1,ncoo
         xcoo(i)=xcoo_old(i)
      enddo

      return
      end subroutine x2q

!#######################################################################

      subroutine q2x(xcoo,xcoo0,q,nmcoo,nmodes,ncoo,maxdim)


      implicit none

      integer(long)                                   :: i,j
      integer(long), intent(in)                       :: nmodes,ncoo,maxdim
      real(dop), dimension(ncoo), intent(in)          :: xcoo0
      real(dop), dimension(nmodes), intent(in)        :: q
      real(dop), dimension(maxdim,maxdim), intent(in) :: nmcoo
      real(dop), dimension(ncoo), intent(out)         :: xcoo

      do i=1,ncoo
         xcoo(i)=xcoo0(i)
         do j=1,nmodes
            xcoo(i)=xcoo(i)+nmcoo(i,j)*q(j)
         enddo
      enddo

      return
      end subroutine q2x

!#######################################################################

      subroutine q2xf(xcoo,xcoo0,q,nmcoo,nmodes,ncoo,maxdim)


      implicit none

      integer(long)                                   :: i,j
      integer(long), intent(in)                       :: nmodes,ncoo,maxdim
      real(dop), dimension(ncoo), intent(in)          :: xcoo0
      real(dop), dimension(ncoo), intent(in)          :: q
      real(dop), dimension(maxdim,maxdim), intent(in) :: nmcoo
      real(dop), dimension(ncoo), intent(out)         :: xcoo

      do i=1,ncoo
         xcoo(i)=xcoo0(i)
         do j=1,ncoo
            xcoo(i)=xcoo(i)+nmcoo(i,j)*q(j)
         enddo
      enddo

      return
    end subroutine q2xf

!#######################################################################

      subroutine x2qf(q,xcoo,xcoo0,coonm,nmodes,ncoo,maxdim,freq0)

      implicit none

      integer(long)                                     :: i,j
      integer(long), intent(in)                         :: nmodes,ncoo,maxdim
      real(dop), dimension(ncoo), intent(in)            :: xcoo0,freq0
      real(dop), dimension(ncoo), intent(out)           :: q
      real(dop), dimension(maxdim,maxdim), intent(in)   :: coonm
      real(dop), dimension(ncoo), intent(inout)         :: xcoo
      real(dop), dimension(1000)                        :: xcoo_old

      do i=1,ncoo
         xcoo_old(i)=xcoo(i)
         xcoo(i)=xcoo(i)-xcoo0(i)
      enddo

      do j=1,ncoo
         q(j)=0.0
         do i=1,ncoo
            q(j)=q(j)+coonm(j,i)*xcoo(i)
         enddo
         if (j.le.nmodes) q(j)=q(j)*sqrt(freq0(j))
      enddo

!     restore old values
      do i=1,ncoo
         xcoo(i)=xcoo_old(i)
      enddo

      return
    end subroutine x2qf

!#######################################################################

      subroutine qvec(xcoo,xcoo0,q,nmcoo,nmodes,ncoo,maxdim,npnts,p)

      implicit none

      integer(long)                                   :: i,j
      integer(long), intent(in)                       :: nmodes,ncoo,maxdim,npnts,p
      real(dop), dimension(ncoo), intent(in)          :: xcoo0
      real(dop), dimension(nmodes), intent(in)        :: q
      real(dop), dimension(maxdim,maxdim), intent(in) :: nmcoo
      real(dop), dimension(ncoo), intent(out)         :: xcoo
      
      do i=1,ncoo
         xcoo(i)=xcoo0(i)
         do j=1,nmodes
            xcoo(i)=xcoo(i)+(p*nmcoo(i,j)*(q(j)/npnts))
         enddo
      enddo

      return
      end subroutine qvec

!#######################################################################

      subroutine qvec2(xcoo,xcoo0,q,qini,nmcoo,nmodes,ncoo,maxdim,&
           dumnpnts,dump)
      
      implicit none

      integer(long)                                   :: i,j
      integer(long), intent(in)                       :: nmodes,ncoo,maxdim,dump,&
                                                         dumnpnts
      real(dop), dimension(maxdim), intent(in)        :: xcoo0
      real(dop), dimension(maxdim), intent(in)        :: q,qini
      real(dop), dimension(maxdim,maxdim), intent(in) :: nmcoo
      real(dop), dimension(maxdim), intent(out)       :: xcoo
      real(dop)                                       :: p,npnts

      p=dump
      npnts=dumnpnts

      do i=1,ncoo
         xcoo(i)=xcoo0(i)
         do j=1,nmodes
            xcoo(i)=xcoo(i)+nmcoo(i,j)*(qini(j)+((p/npnts)*(q(j)-qini(j))))
         enddo
      enddo

      return
      end subroutine qvec2

!#######################################################################

      subroutine qvec2f(xcoo,xcoo0,q,qini,nmcoo,nmodes,ncoo,maxdim,&
           dumnpnts,dump)
      
      implicit none

      integer(long)                                   :: i,j
      integer(long), intent(in)                       :: nmodes,ncoo,maxdim,dump,&
                                                         dumnpnts
      real(dop), dimension(maxdim), intent(in)        :: xcoo0
      real(dop), dimension(maxdim), intent(in)        :: q,qini
      real(dop), dimension(maxdim,maxdim), intent(in) :: nmcoo
      real(dop), dimension(maxdim), intent(out)       :: xcoo
      real(dop)                                       :: p,npnts

      p=dump
      npnts=dumnpnts

      do i=1,ncoo
         xcoo(i)=xcoo0(i)
         do j=1,ncoo
            xcoo(i)=xcoo(i)+nmcoo(i,j)*(qini(j)+((p/npnts)*(q(j)-qini(j))))
         enddo
      enddo

      return
    end subroutine qvec2f

!#######################################################################

      subroutine q2xy(xcoo,xcoo0,qr,nmcoo,nmodes,ncoo,maxdim,nin)


      implicit none

      integer(long)                                   :: i,j
      integer(long), intent(in)                       :: nmodes,ncoo,maxdim,nin
      real(dop), dimension(ncoo), intent(in)          :: xcoo0
      real(dop), dimension(maxdim,nmodes), intent(in) :: qr
      real(dop), dimension(maxdim,maxdim), intent(in) :: nmcoo
      real(dop), dimension(ncoo), intent(out)         :: xcoo

      do i=1,ncoo
         xcoo(i)=xcoo0(i)
         do j=1,nmodes
            xcoo(i)=xcoo(i)+nmcoo(i,j)*qr(nin,j)
         enddo
      enddo

      return
      end subroutine q2xy

!#######################################################################

      subroutine simtrans(mat1,umat,mat,umat1,dim,ncoo)

      implicit none

      integer(long)                              :: i,j,k,l
      integer(long), intent(in)                  :: dim,ncoo
      real(dop), dimension(dim,dim), intent(in)  :: umat,mat,umat1
      real(dop), dimension(dim,dim), intent(out) :: mat1


      do i=1,ncoo
         do j=1,ncoo
            mat1(i,j)=0.0
         enddo
      enddo

      do i=1,ncoo
         do j=1,ncoo
            do k=1,ncoo
               do l=1,ncoo
                  mat1(i,j)=mat1(i,j)+umat(k,i)*mat(k,l)*umat1(l,j)
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine simtrans

!#######################################################################

      subroutine mkmat(diag,mat,dim,ncoo)

      implicit none

      integer(long)                              :: i,j
      integer(long), intent(in)                  :: dim,ncoo
      real(dop), dimension(dim), intent(in)      :: diag
      real(dop), dimension(dim,dim), intent(out) :: mat


      do i=1,ncoo
         do j=1,ncoo
            mat(i,j)=0.0
         enddo
      enddo

      do i=1,ncoo
         mat(i,i)=diag(i)*diag(i)
      enddo
      
      return
      end subroutine mkmat

!#######################################################################

      subroutine cpmat(mat1,mat,dim,ncoo)

      implicit none


      integer(long)                              :: i,j
      integer(long), intent(in)                  :: dim,ncoo
      real(dop), dimension(dim,dim), intent(in)  :: mat
      real(dop), dimension(dim,dim), intent(out) :: mat1

      do i=1,ncoo
         do j=1,ncoo
            mat1(i,j)=mat(i,j)
         enddo
      enddo

      return
      end subroutine cpmat

!#######################################################################

      subroutine xmat(mat1,mat,x,dim,ncoo)

      implicit none

      integer(long)                              :: i,j
      integer(long), intent(in)                  :: dim,ncoo
      real(dop), intent(in)                      :: x
      real(dop), dimension(dim,dim), intent(in)  :: mat
      real(dop), dimension(dim,dim), intent(out) :: mat1

      do i=1,ncoo
         do j=1,ncoo
            mat1(j,i)=x*mat(j,i)
         enddo
      enddo

      return
      end subroutine xmat

!####################################################################################################
      subroutine force_pos(kappa,gamma,expcoe,termk,kmask,gmask,xmask,tmask)

      implicit none

      integer(long) :: n,i,j,k,l,nhi,m,o,tot_order
      real(dop), dimension(nadd), intent(inout)                    :: termk
      real(dop), dimension(nmodes,nsta), intent(inout)             :: kappa
      real(dop), dimension(nmodes,nmodes,nsta), intent(inout)      :: gamma
      real(dop), dimension(maxcoe,nmodes,nsta), intent(inout)      :: expcoe
      integer(long), dimension(nmodes,nsta), intent(in) :: kmask
      integer(long), dimension(nmodes,nmodes,nsta), intent(in) :: gmask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(in) :: xmask
      integer(long), dimension(nadd), intent(in) :: tmask

!     make some ondiagonal order positive (ussually the highest one!
      do i=1, nsta
       do j= 1, nmodes
        if (forcepos(j,j,i) .ne. 0 ) then
          if(forcepos(j,j,i) .eq. 1 .and. kmask(j,i) .eq. 1 ) kappa(j,i) = abs( kappa(j,i) )
          if(forcepos(j,j,i) .eq. 2 .and. gmask(j,j,i) .eq. 1 ) gamma(j,j,i) = abs( gamma(j,j,i) )
          if (ifdiab(j,i) .eq. 1) then
            if  (forcepos(j,j,i) .eq. 1 .and. xmask(1,j,i) .eq. 1) expcoe(1,j,i) = abs( expcoe(1,j,i) )
            if  (forcepos(j,j,i) .eq. 2 .and. xmask(2,j,i) .eq. 1) expcoe(2,j,i) = abs( expcoe(2,j,i) )
            if  (forcepos(j,j,i) .eq. 4 .and. xmask(3,j,i) .eq. 1) expcoe(3,j,i) = abs( expcoe(3,j,i) )
!            if  (forcepos(j,j,i) .eq.  4) print*, "yes! mode: ",j,' state : ',i
          elseif (ifdiab(j,i) .eq. 18) then
            if  (forcepos(j,j,i) .eq. 2 .and. xmask(1,j,i) .eq. 1) expcoe(1,j,i) = abs( expcoe(1,j,i) )
            if  (forcepos(j,j,i) .eq. 2 .and. xmask(3,j,i) .eq. 1) expcoe(3,j,i) = abs( expcoe(3,j,i) )
            if  (forcepos(j,j,i) .eq. 4 .and. xmask(2,j,i) .eq. 1) expcoe(2,j,i) = abs( expcoe(2,j,i) )
            if  (forcepos(j,j,i) .eq. 4 .and. xmask(4,j,i) .eq. 1) expcoe(4,j,i) = abs( expcoe(4,j,i) )
!	BEWARE THIS BODGE MAKES THE UPPER QUADRATIC TERM ALWAYS POSSITIVE (often a hassle)
            if  (forcepos(j,j,i) .eq. 4 .and. xmask(3,j,i) .eq. 1) expcoe(3,j,i) = abs( expcoe(3,j,i) )
          endif
        endif
       enddo
      enddo
      do k = 1, nadd
       tot_order = 0
       o = 0
       do j = 1, nmodes
        if( termdef(j,k) .ne. 0) &
        tot_order = tot_order + termdef(j,k)
! booodggge:
        if( termdef(j,k) .ne. 0) o = o + 1
!        if( termdef(j,k) .ne. 1) o = o + 1
       enddo
       do j =1, nmodes
        do l =j, nmodes
          do i = 1, nsta
            if( termst(1,k) .eq. termst(2,k) ) then
             if( termst(1,k) .eq. i ) then
               if (forcepos(l,j,i) .eq. tot_order .and. tmask(k) .eq. 1 .and. o .ne. 4) &
                   termk(k) = abs( termk(k) )
               if (forcepos(j,l,i) .eq. tot_order .and. tmask(k) .eq. 1 .and. o .ne. 4) &
                   termk(k) = abs( termk(k) )
!       SUPER MASSIVE BODGE< REMOVE LATER:
!               if ( o .eq. 4 .and. tot_order .ge. 4 )  termk(k) = abs( termk(k) )
!   temporary bodge: make sure you comment out afterwards (CHRIS!)
!               if( tot_order .ge. 4)  termk(k) = abs(termk(k))
!       more bodges:
               if( tot_order .eq. 4 .and. o .eq. 4 )  termk(k) = abs(termk(k))
               if( tot_order .eq. 2) termk(k) = abs( termk(k) ) 
             endif
            endif
          enddo
        enddo
       enddo
      enddo
      
      end subroutine

!#######################################################################

      subroutine par2vec2d(optcoo,const,kappa,lambda,gamma,iota, &
                 mu,expcoe,higher,termk,cpcoe,switch,&
                 cp2d,const_mask,kappa_mask,lambda_mask, &
                 gamma_mask,iota_mask,mu_mask,expcoe_mask,termk_mask, &
                 cpcoe_mask,switch_mask,cp2d_mask)

      implicit none

      integer(long) :: n,i,j,k,l,nhi,m,m1,o, tot_order,s
      integer(long), dimension(nsta,nsta), intent(in) :: const_mask
      integer(long), dimension(nmodes,nsta), intent(in) :: kappa_mask
      integer(long), dimension(nmodes,nsta,nsta), intent(in) :: lambda_mask
      integer(long), dimension(nmodes,nmodes,nsta), intent(in) :: gamma_mask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(in) :: expcoe_mask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: &
                                                      iota_mask,mu_mask
      integer(long), dimension(maxcoe,nmodes,nsta,nsta), intent(in) :: cpcoe_mask
      integer(long), dimension(nadd), intent(in) :: termk_mask
      integer(long), dimension(nswitch,maxcoe), intent(in) :: switch_mask

      real(dop), dimension(nhigher), intent(inout)                 :: higher
      real(dop), dimension(nadd), intent(inout)                    :: termk
      real(dop), dimension(nsta,nsta), intent(inout)               :: const
      real(dop), dimension(nmodes,nsta), intent(inout)             :: kappa
      real(dop), dimension(nmodes,nsta,nsta), intent(inout)        :: lambda
      real(dop), dimension(nmodes,nmodes,nsta), intent(inout)      :: gamma
      real(dop), dimension(maxcoe,nmodes,nsta), intent(inout)      :: expcoe
      real(dop), dimension(nmodes,nmodes,nsta,nsta), intent(inout) :: iota,mu
      real(dop), dimension(maxcoe,nmodes,nsta,nsta), intent(inout) :: cpcoe
      real(dop), dimension(maxcoe,maxadd), intent(inout)           :: cp2d
      integer(long), dimension(maxcoe,maxadd), intent(in)          :: cp2d_mask

      real(dop), dimension(nswitch,maxcoe), intent(in) :: switch

      real(dop), dimension(*), intent(out)                      :: optcoo

      if(lforcepos) call force_pos(kappa,gamma,expcoe,termk, kappa_mask,gamma_mask,expcoe_mask,termk_mask)
    
!        print*, 'utils befo LAMBDA(14,1,5) = ', lambda(14,2,5)

!      print*,'before par2vec: optcoo(1) = ',optcoo(1)
      n=0
      do i=1,nsta
         do k=i,nsta
            if (const_mask(i,k).eq.1) then
               n=n+1
               optcoo(n)=const(i,k)
            endif
         enddo
      enddo

      do i=1,nsta
         do j=1,nmodes
            if (kappa_mask(j,i).eq.1) then
               n=n+1
               optcoo(n)=kappa(j,i)
            endif
         enddo
      enddo
      do i=1,nsta-1
         do k=i+1,nsta
            do j=1,nmodes
               if (lambda_mask(j,i,k).eq.1) then
                  n=n+1
                  optcoo(n)=lambda(j,i,k)
               endif
            enddo
         enddo
      enddo

      do i=1,nsta
         do j=1,nmodes
            do k=j,nmodes
               if (gamma_mask(k,j,i).eq.1) then
                  n=n+1
                  optcoo(n)=gamma(k,j,i)
               endif
            enddo
         enddo
      enddo
      do i=1,nsta
         do j=1,nmodes
            do k=1,nmodes
               if (iota_mask(k,j,i,i).eq.1) then
                  n=n+1
                  optcoo(n)=iota(k,j,i,i)
               endif
            enddo
         enddo
      enddo
      do i=1,nsta-1
         do l=i+1,nsta
            do j=1,nmodes
               do k=j,nmodes
                  if (mu_mask(k,j,i,l).eq.1) then
                     n=n+1
                     optcoo(n)=mu(k,j,i,l)
                  endif
               enddo
            enddo
         enddo
      enddo
      do i=1,nsta
         do j=1,nmodes
            do k=1,ncoeff(j,i)
               if (expcoe_mask(k,j,i).eq.1) then
                  n=n+1
                  optcoo(n)=expcoe(k,j,i)
               endif
            enddo
         enddo
      enddo
      do i=1,nsta-1
         do l=i+1,nsta
            do j=1,nmodes
               do k=1,nmodes
                  if (iota_mask(k,j,i,l).eq.1) then
                     n=n+1
                     optcoo(n)=iota(k,j,i,l)
                  endif
               enddo
            enddo
         enddo
      enddo
      do nhi=1,nhigher
         n=n+1
         optcoo(n)=higher(nhi)         
      enddo

      do nhi=1,nadd
         if(termk_mask(nhi).eq.1) then
            n=n+1
            optcoo(n)=termk(nhi)
         endif
      enddo

     s = 1
     do while (cp2dtyp(s) .ne. 0)
       select case (cp2dtyp(s))
       case (1 : 2) 
         if (cp2d_mask(i,s) .eq. 1) then
            n=n+1
            optcoo(n)=cp2d(1,s)
          endif
!        return
       case (3 : 5)
        do i = 1, 4
         if (cp2d_mask(i,s) .eq. 1) then
            n=n+1
            optcoo(n)=cp2d(i,s)
!            print*, 'CP = ', cp2d(i,s), 'i = ', i
          endif
        enddo 
       case ( 6)
        do i = 1, 3
         if (cp2d_mask(i,s) .eq. 1) then
            n=n+1
            optcoo(n)=cp2d(i,s)
!           print*, 'CP = ', cp2d(i,s), 'i = ', i
          endif
        enddo 
       end select
     s = s + 1
     enddo


      do i=1,nsta-1
         do j=i+1,nsta
            do m=1,nmodes
               do k=1,ncoeffcp(m,j,i)
                  if (cpcoe_mask(k,m,j,i).eq.1) then
                     n=n+1
                     optcoo(n)=cpcoe(k,m,j,i)
                  endif
               enddo
            enddo
         enddo
      enddo

! New switching functions
      do i=1,nswitch
         do j=1,nswitchcoe(i)
            if (switch_mask(i,j).eq.1) then
               n=n+1
               optcoo(n)=switch(i,j)
            endif
         enddo
      enddo

      return
      end subroutine par2vec2d
      
!#######################################################################

      subroutine vec2par2d(optcoo,const,kappa,lambda,gamma,iota,mu, &
                 expcoe,higher,termk,cpcoe,switch,cp2d,&
                 const_mask,kappa_mask,lambda_mask,gamma_mask,iota_mask,&
                 mu_mask,expcoe_mask,termk_mask,cpcoe_mask,switch_mask,cp2d_mask)

      implicit none

      integer(long) :: n,i,j,k,l,n1,n2,nhi,m,m1,o,tot_order,s

      integer(long), dimension(nsta,nsta), intent(in) :: const_mask
      integer(long), dimension(nmodes,nsta), intent(in) :: kappa_mask
      integer(long), dimension(nmodes,nsta,nsta), intent(in) :: lambda_mask
      integer(long), dimension(nmodes,nmodes,nsta), intent(in) :: gamma_mask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(in) :: expcoe_mask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: &
                                                      iota_mask,mu_mask
      integer(long), dimension(maxcoe,nmodes,nsta,nsta), intent(in) :: cpcoe_mask
      integer(long), dimension(nadd), intent(in) :: termk_mask
      integer(long), dimension(nswitch,maxcoe) :: switch_mask

      real(dop), dimension(nhigher), intent(out)                 :: higher
      real(dop), dimension(nadd), intent(out)                    :: termk
      real(dop), dimension(nsta,nsta), intent(out)               :: const
      real(dop), dimension(nmodes,nsta), intent(out)             :: kappa
      real(dop), dimension(nmodes,nsta,nsta), intent(out)        :: lambda
      real(dop), dimension(nmodes,nmodes,nsta), intent(out)      :: gamma
      real(dop), dimension(maxcoe,nmodes,nsta), intent(out)      :: expcoe      
      real(dop), dimension(nmodes,nmodes,nsta,nsta), intent(out) :: iota,mu
      real(dop), dimension(maxcoe,nmodes,nsta,nsta), intent(out)      :: cpcoe

      real(dop), dimension(nswitch,maxcoe) :: switch

      real(dop), dimension(maxcoe,maxadd), intent(out)            :: cp2d
      integer(long), dimension(maxcoe,maxadd), intent(in)         :: cp2d_mask

      real(dop), dimension(*), intent(out) :: optcoo

      n=0
      do i=1,nsta
         do k=i,nsta
! optimised parameters
            if (const_mask(i,k).eq.1) then
               n=n+1
               const(i,k)=optcoo(n)
               const(k,i)=const(i,k)
! constrained parameters
            else if (const_mask(i,k).lt.0) then
               n1=abs(const_mask(i,k))
               n2=parcons(n1)
               if (n2 .gt. 0) then
                  const(i,k)=pconfact(n1)*optcoo(n2)
                  const(k,i)=const(i,k)
               endif
            endif
         enddo
      enddo 
      do i=1,nsta
         do j=1,nmodes
! optimised parameters
            if (kappa_mask(j,i).eq.1) then
               n=n+1
               kappa(j,i)=optcoo(n)
! constrained parameters
            else if (kappa_mask(j,i).lt.0) then
               n1=abs(kappa_mask(j,i))
               n2=parcons(n1)
               if (n2 .gt. 0) kappa(j,i)=pconfact(n1)*optcoo(n2)
            endif
         enddo
      enddo
      do i=1,nsta-1
         do k=i+1,nsta
            do j=1,nmodes
! optimised parameters
               if (lambda_mask(j,i,k).eq.1) then
                  n=n+1
                  lambda(j,i,k)=optcoo(n)
                  lambda(j,k,i)=lambda(j,i,k)
! constrained parameters
               else if (lambda_mask(j,i,k).lt.0) then
                  n1=abs(lambda_mask(j,i,k))
                  n2=parcons(n1)
                  if (n2 .gt. 0) then
                     lambda(j,i,k)=pconfact(n1)*optcoo(n2)
                     lambda(j,k,i)=lambda(j,i,k)
                  endif
               endif
            enddo
         enddo
      enddo
      do i=1,nsta
         do j=1,nmodes
            do k=j,nmodes
! optimised parameters
               if (gamma_mask(k,j,i).eq.1) then
                  n=n+1
                  gamma(k,j,i)=optcoo(n)
                  if (k.ne.j) then 
                     gamma(j,k,i)=gamma(k,j,i)
                  endif
! constrained parameters
               else if (gamma_mask(k,j,i).lt.0) then
                  n1=abs(gamma_mask(k,j,i))
                  n2=parcons(n1)

                  if (n2 .gt. 0) then
                     gamma(k,j,i)=pconfact(n1)*optcoo(n2)
                     if (k.ne.j) gamma(j,k,i)=gamma(k,j,i)
                  endif
               endif
            enddo
         enddo
      enddo
      do i=1,nsta
         do j=1,nmodes
            do k=1,nmodes
! optimised parameters
               if (iota_mask(k,j,i,i).eq.1) then
                  n=n+1
                  iota(k,j,i,i)=optcoo(n)
! constrained parameters
               else if (iota_mask(k,j,i,i).lt.0) then
                  n1=abs(iota_mask(k,j,i,i))
                  n2=parcons(n1)
                  if (n2 .gt. 0) then
                     iota(k,j,i,i)=pconfact(n1)*optcoo(n2)
                  endif
               endif
            enddo
         enddo
      enddo
      do i=1,nsta-1
         do l=i+1,nsta
            do j=1,nmodes
               do k=j,nmodes
! optimised parameters
                  if (mu_mask(k,j,i,l).eq.1) then
                     n=n+1
                     mu(k,j,i,l)=optcoo(n)
                     mu(k,j,l,i)=mu(k,j,i,l)
                     if (k.ne.j) then
                        mu(j,k,i,l)=mu(k,j,i,l)
                        mu(j,k,l,i)=mu(k,j,l,i)
                     endif
! constrained parameters
                  else if (mu_mask(k,j,i,l).lt.0) then
                     n1=abs(mu_mask(k,j,i,l))
                     n2=parcons(n1)
                     if (n2 .gt. 0) then
                        mu(k,j,i,l)=pconfact(n1)*optcoo(n2)
                        mu(k,j,l,i)=mu(k,j,i,l)
                        if (k.ne.j) then
                           mu(j,k,i,l)=mu(k,j,i,l)
                           mu(j,k,l,i)=mu(k,j,l,i)
                        endif
                     endif
                  endif
               enddo
            enddo
         enddo
      enddo
      do i=1,nsta
         do j=1,nmodes
            do k=1,ncoeff(j,i)
! optimised parameters
               if (expcoe_mask(k,j,i).eq.1) then
                  n=n+1
                  expcoe(k,j,i)=optcoo(n)
! constrained diabatic states
               else if (expcoe_mask(k,j,i).lt.0) then
                  n1=abs(expcoe_mask(k,j,i))
                  n2=parcons(n1)
                  if (n2 .gt. 0) expcoe(k,j,i)=pconfact(n1)*optcoo(n2)
               endif
            enddo
         enddo
      enddo           
      do i=1,nsta-1
         do l=i+1,nsta
            do j=1,nmodes
               do k=1,nmodes
! optimised parameters
                  if (iota_mask(k,j,i,l).eq.1) then
                     n=n+1
                     iota(k,j,i,l)=optcoo(n)
! constrained parameters
                  else if (iota_mask(k,j,i,l).lt.0) then
                     n1=abs(iota_mask(k,j,i,l))
                     n2=parcons(n1)
                     if (n2 .gt. 0) then
                        iota(k,j,i,l)=pconfact(n1)*optcoo(n2)
                        iota(k,j,l,i)=iota(k,j,i,l)
                     endif
                  endif
               enddo
            enddo
         enddo
      enddo
      do nhi=1,nhigher
         n=n+1
         higher(nhi) = optcoo(n)
      enddo

      do i=1,nadd
         if (termk_mask(i).eq.1) then
            n=n+1            
            termk(i) = optcoo(n)
         else if (termk_mask(i).lt.0) then
            n1=abs(termk_mask(i))
            n2=parcons(n1)
            if (n2 .gt. 0) then
               termk(i)=pconfact(n1)*optcoo(n2)
            endif
         endif
      enddo
     s = 1
     do while (cp2dtyp(s) .ne. 0)
       select case (cp2dtyp(s))
       case (1 : 2)
         if (cp2d_mask(1,s) .eq. 1) then
            n=n+1
            cp2d(1,s)= optcoo(n)
          endif
       case ( 3 : 5 ) 
        do i = 1, 4
         if (cp2d_mask(i,s) .eq. 1) then
           n=n+1
           cp2d(i,s)= optcoo(n)
           if (i .eq. 2 .or. i .eq. 3) cp2d(i,s) = abs(cp2d(i,s)) 
!           print*, 'cp = ', cp2d(i,s), optcoo(n), 'i = ', i
         endif
        enddo
       case (  6 ) 
        do i = 1, 3 
         if (cp2d_mask(i,s) .eq. 1) then
           n=n+1
           cp2d(i,s)= optcoo(n)
           if (i .eq. 2 .or. i .eq. 3) cp2d(i,s) = abs(cp2d(i,s)) 
!           print*, 'cp = ', cp2d(i,s), optcoo(n), 'i = ', i
         endif
        enddo
       end select
       s = s + 1
     enddo
      do i=1,nsta-1
         do j=i+1,nsta
            do m=1,nmodes
               do k=1,ncoeffcp(m,j,i)
                  if (cpcoe_mask(k,m,j,i).eq.1) then
                     n=n+1
                     cpcoe(k,m,j,i)=optcoo(n)
                     cpcoe(k,m,i,j)=cpcoe(k,m,j,i)
                  else if (cpcoe_mask(k,m,j,i).lt.0) then
                     n1=abs(cpcoe_mask(k,m,j,i))
                     n2=parcons(n1)                     
                     if (n2 .gt. 0) then                        
                        cpcoe(k,m,j,i)=pconfact(n1)*optcoo(n2)
                        cpcoe(k,m,i,j)=cpcoe(k,m,j,i)
                     endif
                  endif
               enddo
            enddo
         enddo
      enddo

! New switching functions
      do i=1,nswitch
         do j=1,nswitchcoe(i)
            if (switch_mask(i,j).eq.1) then
               n=n+1
               switch(i,j)=optcoo(n)
            else if (switch_mask(i,j).lt.0) then
               n1=abs(switch_mask(i,j))
               n2=parcons(n1)
               if (n2 .gt. 0) then
                  switch(i,j)=pconfact(n1)*optcoo(n2)
               endif
            endif
         enddo
      enddo

      if(lforcepos)&
       call force_pos(kappa,gamma,expcoe,termk,kappa_mask,gamma_mask,expcoe_mask,termk_mask)

!-----------------------------------------------------------------------
! load appropriate general potential parameters in kapp, gamma, etc.
!-----------------------------------------------------------------------
      do i=1,nsta
         do j=1,nmodes
            if (ifdiab(j,i) .eq. 1) then
               kappa(j,i) = expcoe(1,j,i)
            else if (ifdiab(j,i) .eq. 9 .or. ifdiab(j,i) .eq. 12) then
               kappa(j,i) = expcoe(1,j,i)
               gamma(j,j,i) = expcoe(2,j,i)
            endif
         enddo
      enddo
 
      return
      end subroutine vec2par2d


!#######################################################################
!       this routine is used in GA: I wanted to make it more efficient,
!       but beacuse of the arbitrary assignement of 1 or -ve masks of 
!       the pair this just looks similar to a merged 
!       par2vec2d + vec2par2d
!#######################################################################
     subroutine set_pair_cons(const,kappa,lambda,gamma,iota, &
                 mu,expcoe,termk,const_mask,kappa_mask,&
                 lambda_mask,gamma_mask,iota_mask,mu_mask,&
                 expcoe_mask,termk_mask,cpcoe_mask)

      implicit none

      integer(long) :: n,i,j,k,l,n1,n2,nhi,m,m1,o,tot_order

      integer(long), dimension(nsta,nsta), intent(in)             :: const_mask
      integer(long), dimension(nmodes,nsta), intent(in)           :: kappa_mask
      integer(long), dimension(nmodes,nsta,nsta), intent(in)      :: lambda_mask
      integer(long), dimension(nmodes,nmodes,nsta), intent(in)    :: gamma_mask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(in)    :: expcoe_mask
      integer(long),dimension(nmodes,nmodes,nsta,nsta),intent(in) :: iota_mask,mu_mask
      integer(long), dimension(nadd), intent(in)                  :: termk_mask
      integer(long),dimension(maxcoe,nmodes,nsta,nsta),intent(in) :: cpcoe_mask

      real(dop), dimension(nadd), intent(out)                    :: termk
      real(dop), dimension(nsta,nsta), intent(out)               :: const
      real(dop), dimension(nmodes,nsta), intent(out)             :: kappa
      real(dop), dimension(nmodes,nsta,nsta), intent(out)        :: lambda
      real(dop), dimension(nmodes,nmodes,nsta), intent(out)      :: gamma
      real(dop), dimension(maxcoe,nmodes,nsta), intent(out)      :: expcoe      
      real(dop), dimension(nmodes,nmodes,nsta,nsta), intent(out) :: iota,mu
      real(dop), dimension(optcoodim)                            :: temp_optcoo

      n=0
      do i=1,nsta
         do k=i,nsta
            if (const_mask(i,k).eq.1) then
               n=n+1
               temp_optcoo(n)=const(i,k)
            endif
         enddo
      enddo

      do i=1,nsta
         do k=i,nsta
            if (const_mask(i,k).lt.0) then
               n1=abs(const_mask(i,k))
               n2=parcons(n1)
               if (n2 .gt. 0) then
                   const(i,k)=pconfact(n1)*temp_optcoo(n2)
                   const(k,i)=const(i,k)
               endif
            endif
         enddo
      enddo

      do i=1,nsta
         do j=1,nmodes
            if (kappa_mask(j,i).eq.1) then 
               n=n+1
               temp_optcoo(n)=kappa(j,i)
            endif
         enddo
      enddo
      do i=1,nsta
         do j=1,nmodes
            if (kappa_mask(j,i).lt.0) then
               n1=abs(kappa_mask(j,i))
               n2=parcons(n1)
               if (n2 .gt. 0) kappa(j,i)=pconfact(n1)*temp_optcoo(n2)
            endif
         enddo
      enddo

      do i=1,nsta-1
         do k=i+1,nsta
            do j=1,nmodes
               if (lambda_mask(j,i,k).eq.1) then
                 n=n+1
                 temp_optcoo(n)=lambda(j,i,k)
               endif
            enddo
         enddo
      enddo
      do i=1,nsta-1
         do k=i+1,nsta
            do j=1,nmodes
               if (lambda_mask(j,i,k).lt.0) then
                  n1=abs(lambda_mask(j,i,k))
                  n2=parcons(n1)
                  if (n2 .gt. 0) then
                     lambda(j,i,k)=pconfact(n1)*temp_optcoo(n2)
                     lambda(j,k,i)=lambda(j,i,k)
                  endif
               endif
            enddo
         enddo
      enddo

      do i=1,nsta
         do j=1,nmodes
            do k=j,nmodes
               if (gamma_mask(k,j,i).eq.1) then
                  n=n+1
                  temp_optcoo(n)=gamma(k,j,i)
               endif
            enddo
         enddo
      enddo

      do i=1,nsta
         do j=1,nmodes
            do k=j,nmodes
               if (gamma_mask(k,j,i).lt.0) then
                  n1=abs(gamma_mask(k,j,i))
                  n2=parcons(n1)
                  if (n2 .gt. 0) then
                     gamma(k,j,i)=pconfact(n1)*temp_optcoo(n2)
                     if (k.ne.j) gamma(j,k,i)=gamma(k,j,i)
                  endif
               endif
            enddo
         enddo
      enddo

      do i=1,nsta
         do j=1,nmodes
            do k=1,nmodes
               if (iota_mask(k,j,i,i).eq.1) then
                  n=n+1
                 temp_optcoo(n)=iota(k,j,i,i)
               endif
            enddo
         enddo
      enddo
      do i=1,nsta
         do j=1,nmodes
            do k=1,nmodes
               if (iota_mask(k,j,i,i).lt.0) then
                  n1=abs(iota_mask(k,j,i,i))
                  n2=parcons(n1)
                  if (n2 .gt. 0) then
                     iota(k,j,i,i)=pconfact(n1)*temp_optcoo(n2)
                  endif
               endif
            enddo
         enddo
      enddo

      do i=1,nsta-1
         do l=i+1,nsta
            do j=1,nmodes
               do k=j,nmodes
                  if (mu_mask(k,j,i,l).eq.1) then
                     n=n+1
                    temp_optcoo(n)=mu(k,j,i,l)
                  endif
               enddo
            enddo
         enddo
      enddo
      do i=1,nsta-1
         do l=i+1,nsta
            do j=1,nmodes
               do k=j,nmodes
                  if (mu_mask(k,j,i,l).lt.0) then
                     n1=abs(mu_mask(k,j,i,l))
                     n2=parcons(n1)
                     if (n2 .gt. 0) then
                        mu(k,j,i,l)=pconfact(n1)*temp_optcoo(n2)
                        mu(k,j,l,i)=mu(k,j,i,l)
                        if (k.ne.j) then
                           mu(j,k,i,l)=mu(k,j,i,l)
                           mu(j,k,l,i)=mu(k,j,l,i)
                        endif
                     endif
                  endif
               enddo
            enddo
         enddo
      enddo
      do i=1,nsta
         do j=1,nmodes
            do k=1,ncoeff(j,i)
               if (expcoe_mask(k,j,i).eq.1) then
                  n=n+1
                 temp_optcoo(n)=expcoe(k,j,i)
               endif
            enddo
         enddo
      enddo

      do i=1,nsta
         do j=1,nmodes
            do k=1,ncoeff(j,i)
               if (expcoe_mask(k,j,i).lt.0) then
                  n1=abs(expcoe_mask(k,j,i))
                  n2=parcons(n1)
                  if (n2 .gt. 0) expcoe(k,j,i)=pconfact(n1)*temp_optcoo(n2)
               endif
            enddo
         enddo
      enddo

      do i=1,nsta-1
         do l=i+1,nsta
            do j=1,nmodes
               do k=1,nmodes
                  if (iota_mask(k,j,i,l).eq.1) then
                     n=n+1
                    temp_optcoo(n)=iota(k,j,i,l)
                  endif
               enddo
            enddo
         enddo
      enddo
      do i=1,nsta-1
         do l=i+1,nsta
            do j=1,nmodes
               do k=1,nmodes
                  if (iota_mask(k,j,i,l).lt.0) then
                     n1=abs(iota_mask(k,j,i,l))
                     n2=parcons(n1)
                     if (n2 .gt. 0) then
                        iota(k,j,i,l)=pconfact(n1)*temp_optcoo(n2)
                        iota(k,j,l,i)=iota(k,j,i,l)
                     endif
                  endif
               enddo
            enddo
         enddo
      enddo
      do nhi=1,nhigher
         n=n+1
!        temp_optcoo(n)=higher(nhi)         
      enddo
      do nhi=1,nadd
         if(termk_mask(nhi).eq.1) then
            n=n+1
           temp_optcoo(n)=termk(nhi)
         endif
      enddo
      do i=1,nsta-1
         do j=i+1,nsta
            do m=1,nmodes
               do k=1,ncoeffcp(m,j,i)
                  if (cpcoe_mask(k,m,j,i).eq.1) then
                     n=n+1
!                    temp_optcoo(n)=cpcoe(k,m,j,i)
                  endif
               enddo
            enddo
         enddo
      enddo

      return

      end subroutine set_pair_cons

!#######################################################################

      subroutine general_cons(const_mask,kappa_mask,lambda_mask,&
              gamma_mask,iota_mask,mu_mask,expcoe_mask,expcoe,&
              termk_mask,cpcoe_mask,switch_mask,lambda)

      implicit none

! common needed for constraining Morse oscillators
      real(dop)       :: freqj,target
      common /ftarget/ target,freqj

      integer(long)                                                 :: i,j,k,n1,n2,iter1,type,md,st,md1,st1,koe,attermk
      integer(long), dimension(1)                                   :: idum
      integer(long), dimension(nsta,nsta), intent(in)               :: const_mask
      integer(long), dimension(nmodes,nsta), intent(in)             :: kappa_mask
      integer(long), dimension(nmodes,nsta,nsta), intent(in)        :: lambda_mask
      integer(long), dimension(nmodes,nmodes,nsta), intent(in)      :: gamma_mask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(in)      :: expcoe_mask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: iota_mask,mu_mask
      integer(long), dimension(nadd), intent(in)                    :: termk_mask
      integer(long), dimension(maxcoe,nmodes,nsta,nsta), intent(in) :: cpcoe_mask
      integer(long), dimension(nswitch,maxcoe)                      :: switch_mask
      real(dop), dimension(maxcoe,nmodes,nsta), intent(in)          :: expcoe
      real(dop), dimension(nmodes,nsta,nsta), intent(inout)         :: lambda

      real(dop)                     :: kappa1,kappa2,d0,x0,freq,alpha,fret,tol1
      real(dop), dimension(1)       :: rdum
!      external                      :: mosc,dmosc
      logical(kind=4)               :: docons
!
! if general diabatic potentials are used and constraints
! have been set on kappa / lambda. Optimise potential parameters to meet 
! constraints. Special cases....
!

      do i=1,nsta
         do j=1,nmodes
            docons = ifdiab(j,i) .eq. 2
            if (docons .and. kappa_mask(j,i) .lt. 0) then
! find parameter to which kappa(j,i) is constrained to
               n1=abs(kappa_mask(j,i))
               n2=parcons(n1)
               n2=-n2
               call numtpar(n2,type,md,st,md1,st1,koe,attermk,const_mask,&
                    kappa_mask,lambda_mask,gamma_mask,iota_mask,mu_mask,&
                    expcoe_mask,termk_mask,cpcoe_mask,switch_mask)
                   
! get kappa value to which kappa(j,i) is constrained to
               if (type .eq. 1) then
                  d0 = expcoe(1,md,st)
                  freq = freq0(md) + expcoe(2,md,st)
                  x0 = expcoe(3,md,st)

                  if (freq .gt. 0) then
                     alpha=freq/(2.0*d0)
                     alpha=sqrt(alpha)
                  else
                     alpha=0.0
                  endif

                  kappa1=exp(alpha*x0) -&
                            exp(2.0*alpha*x0)
                  kappa1=2.0*alpha*d0*kappa1
                  kappa1 = pconfact(n1)*kappa1

                  d0 = expcoe(1,j,i)
                  freq = freq0(j) + expcoe(2,j,i)
                  x0 = expcoe(3,j,i)

                  if (freq .gt. 0) then
                     alpha=freq/(2.0*d0)
                     alpha=sqrt(alpha)
                  else
                     alpha=0.0
                  endif

                  kappa2=exp(alpha*x0) -&
                            exp(2.0*alpha*x0)
                  kappa2=2.0*alpha*d0*kappa2
                  if (abs(kappa1 -kappa2) .gt. 1.0e-3_dop) then
                     freqj = freq0(j)
                     target = kappa1
!                 print*,'CONS TO',kappa1,kappa2
                     tol1 = 0.00001
                     iter1 = 10
                     call frprmn(expcoe(1,j,i),3,tol1,iter1,fret,&
                          mosc,dmosc,ilog,0.0_dop,1,rdum,idum)

                  endif
               endif
            endif
         enddo
      enddo


      do i=1,nsta
         do k=i+1,nsta
            do j=1,nmodes
               if (lambda_mask(j,i,k).lt.0) then
                  n1=abs(lambda_mask(j,i,k))
                  n2=parcons(n1)
! only interested if n2 points to a non-active kappa that is same
! mode as a general potential
                  if (n2 .lt. 0) then
                     n2=-n2
                     call numtpar(n2,type,md,st,md1,st1,koe,attermk,&
                          const_mask,kappa_mask,lambda_mask,gamma_mask,&
                          iota_mask,mu_mask,expcoe_mask,termk_mask,&
                          cpcoe_mask,switch_mask)
                   
                     docons = ifdiab(md,st) .eq. 2 
                     if (type .eq. 1 .and. docons .and. &
                             kappa_mask(md,st) .gt. 0) then
                        d0 = expcoe(1,md,st)
                        freq = freq0(md) + expcoe(2,md,st)
                        x0 = expcoe(3,md,st)

                        if (freq .gt. 0) then
                           alpha=freq/(2.0*d0)
                           alpha=sqrt(alpha)
                        else
                           alpha=0.0
                        endif

                        kappa1=exp(alpha*x0) -&
                               exp(2.0*alpha*x0)
                        kappa1=2.0*alpha*d0*kappa1
                        lambda(j,i,k)=pconfact(n1)*kappa1
                        lambda(j,k,i)=lambda(j,i,k)
                      endif
                   endif
                endif
            enddo
         enddo
      enddo

      return
      end subroutine general_cons

!#######################################################################

      subroutine active_mask(n,const_mask,kappa_mask,lambda_mask,&
              gamma_mask,iota_mask,mu_mask,expcoe_mask,termk_mask)

      implicit none

      integer(long)                                                    :: i,j,k,l
      integer(long), intent(out)                                       :: n
      integer(long), dimension(nsta,nsta), intent(inout)               :: const_mask
      integer(long), dimension(nmodes,nsta), intent(inout)             :: kappa_mask
      integer(long), dimension(nmodes,nsta,nsta), intent(inout)        :: lambda_mask
      integer(long), dimension(nmodes,nmodes,nsta), intent(inout)      :: gamma_mask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(inout)      :: expcoe_mask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(inout) :: iota_mask,mu_mask
      integer(long), dimension(nadd), intent(inout)                    :: termk_mask

      n=0
      do i=1,nsta
         do k=i,nsta
            if (const_mask(i,k).ne.0) then
               n=n+1
               const_mask(i,k) = 1
               const_mask(k,i) = 1
            endif
         enddo
      enddo

      do i=1,nsta
         do j=1,nmodes
            if (kappa_mask(j,i).ne.0) then
               n=n+1
               kappa_mask(j,i) = 1
            endif
         enddo
      enddo
      do i=1,nsta-1
         do k=i+1,nsta
            do j=1,nmodes
               if (lambda_mask(j,i,k).ne.0) then
                  n=n+1
                  lambda_mask(j,i,k) = 1
                  lambda_mask(j,k,i) = 1
               endif
            enddo
         enddo
      enddo
      do i=1,nsta
         do j=1,nmodes
            do k=j,nmodes
               if (gamma_mask(k,j,i).ne.0) then
                  n=n+1
                  gamma_mask(k,j,i) = 1
                  if (k.ne.j) then 
                     gamma_mask(j,k,i)=gamma_mask(k,j,i)
                  endif
               endif
            enddo
         enddo
      enddo
      do i=1,nsta
         do j=1,nmodes
            do k=1,nmodes
               if (iota_mask(k,j,i,i).ne.0) then
                  n=n+1
                  iota_mask(k,j,i,i) = 1
               endif
            enddo
         enddo
      enddo
      do i=1,nsta-1
         do l=i+1,nsta
            do j=1,nmodes
               do k=j,nmodes
                  if (mu_mask(k,j,i,l).ne.0) then
                     n=n+1
                     mu_mask(k,j,i,l) = 1
                     mu_mask(k,j,l,i)=mu_mask(k,j,i,l)
                     if (k.ne.j) then
                        mu_mask(j,k,i,l)=mu_mask(k,j,i,l)
                        mu_mask(j,k,l,i)=mu_mask(k,j,l,i)
                     endif
                  endif
               enddo
            enddo
         enddo
      enddo
      do i=1,nsta
         do j=1,nmodes
            do k=1,ncoeff(j,i)
               if (expcoe_mask(k,j,i).ne.0) then
                  n=n+1
                  expcoe_mask(k,j,i) = 1
               endif
            enddo
         enddo
      enddo
      do i=1,nsta-1
         do l=i+1,nsta
            do j=1,nmodes
               do k=1,nmodes
                  if (iota_mask(k,j,i,l).ne.0) then
                     n=n+1
                     iota_mask(k,j,i,l) = 1
                     iota_mask(k,j,l,i) = 1
                  endif
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine active_mask
      
!#######################################################################
!
! zeros parameters with mask value imask
!
! NB constants are never zerod - must have a value
!
!#######################################################################

      subroutine zeropar(imask,const,kappa,lambda,gamma,iota,mu,expcoe,&
           termk,cpcoe,switch,cp2d,const_mask,kappa_mask,&
           lambda_mask,gamma_mask,iota_mask,mu_mask,expcoe_mask,&
           termk_mask,cpcoe_mask,switch_mask,cp2d_mask)

      implicit none

      integer(long) :: i,j,k,l,m,n,imask,s
      integer(long), dimension(nsta,nsta), intent(in) :: const_mask
      integer(long), dimension(nmodes,nsta), intent(in) :: kappa_mask
      integer(long), dimension(nmodes,nsta,nsta), intent(in) :: lambda_mask
      integer(long), dimension(nmodes,nmodes,nsta), intent(in) :: gamma_mask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(in) :: expcoe_mask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: &
                                                      iota_mask,mu_mask
      integer(long), dimension(maxcoe,nmodes,nsta,nsta), intent(in) :: cpcoe_mask
      integer(long), dimension(nadd), intent(in) :: termk_mask
      integer(long), dimension(nswitch,maxcoe), intent(in) :: switch_mask

      real(dop), dimension(nadd), intent(out)                    :: termk
      real(dop), dimension(nsta,nsta), intent(out)               :: const
      real(dop), dimension(nmodes,nsta), intent(out)             :: kappa
      real(dop), dimension(nmodes,nsta,nsta), intent(out)        :: lambda
      real(dop), dimension(nmodes,nmodes,nsta), intent(out)      :: gamma
      real(dop), dimension(maxcoe,nmodes,nsta), intent(out)      :: expcoe
      real(dop), dimension(nmodes,nmodes,nsta,nsta), intent(out) :: iota,mu
      real(dop), dimension(maxcoe,nmodes,nsta,nsta), intent(out) :: cpcoe
      real(dop), dimension(nswitch,maxcoe), intent(out) :: switch
      real(dop), dimension(maxcoe,maxadd), intent(inout)            :: cp2d
      integer(long), dimension(maxcoe,maxadd), intent(in)         :: cp2d_mask

      do i=1,nsta
         do j=1,nmodes
            if (kappa_mask(j,i).eq.imask) kappa(j,i)=0.0
         enddo
      enddo

      do i=1,nsta-1
         do k=i+1,nsta
            do j=1,nmodes
               if (lambda_mask(j,i,k).eq.imask) then
                  lambda(j,i,k)=0.0
                  lambda(j,k,i)=0.0
               endif
            enddo
         enddo
      enddo

      do i=1,nsta
         do j=1,nmodes
            do k=j,nmodes
               if (gamma_mask(k,j,i).eq.imask) then
                  gamma(k,j,i)=0.0
                  if (k.ne.j) then
                     gamma(j,k,i)=0.0
                  endif
               endif
            enddo
         enddo
      enddo

      do i=1,nsta
         do j=1,nmodes
            do k=1,nmodes
               if (iota_mask(k,j,i,i).eq.imask) then
                  iota(k,j,i,i)=0.0
               endif
            enddo
         enddo
      enddo

      do i=1,nsta-1
         do l=i+1,nsta
            do j=1,nmodes
               do k=j,nmodes
                  if (mu_mask(k,j,i,l).eq.imask) then
                     mu(k,j,i,l)=0.0
                     mu(k,j,l,i)=mu(k,j,i,l)
                     if (k.ne.j) then
                        mu(j,k,i,l)=mu(k,j,i,l)
                        mu(j,k,l,i)=mu(k,j,l,i)
                     endif
                  endif
               enddo
            enddo
         enddo
      enddo

      do i=1,nsta
         do j=1,nmodes
            do k=1,ncoeff(j,i)
               if (expcoe_mask(k,j,i).eq.imask) expcoe(k,j,i)=0.0
            enddo
         enddo
      enddo 

      do i=1,nsta
         do j=1,nsta
            do m=1,nmodes
               do k=1,ncoeffcp(m,j,i)
                  if (cpcoe_mask(k,m,j,i).eq.imask) cpcoe(k,m,j,i)=0.0
               enddo
            enddo
         enddo
      enddo

      do i=1,nsta-1
         do l=i+1,nsta
            do j=1,nmodes
               do k=1,nmodes
                  if (iota_mask(k,j,i,l).eq.imask) then
                     iota(k,j,i,l)=0.0
                     iota(k,j,l,i)=0.0
                  endif
               enddo
            enddo
         enddo
      enddo

      do i=1,nadd
         if(termk_mask(i).eq.imask) then
            termk(i)=0
         endif
      enddo

     s = 1
     do while (cp2dtyp(s) .ne. 0)
       select case (cp2dtyp(s))
       case (1 : 2)
         if (cp2d_mask(1,s) .eq. imask) cp2d(1,s) = 0.0
       case (3 : 5)
        do i = 1, 4
         if (cp2d_mask(i,s) .eq. imask) then
          cp2d(i,s) = 0.0
         endif
        enddo
       case ( 6)
        do i = 1, 3
         if (cp2d_mask(i,s) .eq. imask) then
          cp2d(i,s) = 0.0
         endif
        enddo
       end select
     s = s + 1
     enddo

       do i=1,nswitch
         do j=1,nswitchcoe(i)
            if (switch_mask(i,j).eq.imask) then
               switch(i,j)=0.0
            endif
         enddo
      enddo

      return
      end subroutine zeropar

!#######################################################################
!
! zero parameters constrained to non-active parameters
!
!#######################################################################

      subroutine zerocons(const,kappa,lambda,gamma,iota,mu,expcoe,&
                 const_mask,kappa_mask,lambda_mask,gamma_mask,iota_mask,&
                 mu_mask,expcoe_mask)

      implicit none

      integer(long)                                                 :: i,j,k,l,n1,n2
      integer(long), dimension(nsta,nsta), intent(in)               :: const_mask
      integer(long), dimension(nmodes,nsta), intent(in)             :: kappa_mask
      integer(long), dimension(nmodes,nsta,nsta), intent(in)        :: lambda_mask
      integer(long), dimension(nmodes,nmodes,nsta), intent(in)      :: gamma_mask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(in)      :: expcoe_mask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: iota_mask,mu_mask

      real(dop), dimension(nsta,nsta), intent(out)               :: const
      real(dop), dimension(nmodes,nsta), intent(out)             :: kappa
      real(dop), dimension(nmodes,nsta,nsta), intent(out)        :: lambda
      real(dop), dimension(nmodes,nmodes,nsta), intent(out)      :: gamma
      real(dop), dimension(maxcoe,nmodes,nsta), intent(out)      :: expcoe
      real(dop), dimension(nmodes,nmodes,nsta,nsta), intent(out) :: iota,mu

!-----------------------------------------------------------------------
! if n1=mask < 0 then parameter is constrained to variable parcons(n1).
! if n2=parcons(n1) < 0 then variable is not to be optimised, and 
!       parameter is set to zero
!-----------------------------------------------------------------------
      do i=1,nsta
         do j=1,nmodes
            if (kappa_mask(j,i).lt.0) then
               n1=abs(kappa_mask(j,i))
               n2=parcons(n1)
               if (n2 .lt. 0) kappa(j,i)=0.0
            endif
         enddo
      enddo

      do i=1,nsta-1
         do k=i+1,nsta
            do j=1,nmodes
               if (lambda_mask(j,i,k).lt.0) then
                  n1=abs(lambda_mask(j,i,k))
                  n2=parcons(n1)
                  if (n2 .lt. 0) then
                     lambda(j,i,k)=0.0
                     lambda(j,k,i)=0.0
                  endif
               endif
            enddo
         enddo
      enddo

      do i=1,nsta
         do j=1,nmodes
            do k=j,nmodes
               if (gamma_mask(k,j,i).lt.0) then
                  n1=abs(gamma_mask(k,j,i))
                  n2=parcons(n1)
                  if (n2 .lt. 0) then
                     gamma(k,j,i)=0.0
                     if (k.ne.j) then 
                        gamma(j,k,i)=0.0
                     endif
                  endif
               endif
            enddo
         enddo
      enddo

      do i=1,nsta
         do j=1,nmodes
            do k=1,nmodes
               if (iota_mask(k,j,i,i).lt.0) then
                  n1=abs(iota_mask(k,j,i,i))
                  n2=parcons(n1)
                  if (n2 .lt. 0) then
                     iota(k,j,i,i)=0.0
                  endif
               endif
            enddo
         enddo
      enddo

      do i=1,nsta-1
         do l=i+1,nsta
            do j=1,nmodes
               do k=j,nmodes
                  if (mu_mask(k,j,i,l).lt.0) then
                     n1=abs(mu_mask(k,j,i,l))
                     n2=parcons(n1)
                     if (n2 .lt. 0) then
                        mu(k,j,i,l)=0.0
                        mu(k,j,l,i)=mu(k,j,i,l)
                        if (k.ne.j) then
                           mu(j,k,i,l)=mu(k,j,i,l)
                           mu(j,k,l,i)=mu(k,j,l,i)
                        endif
                     endif
                  endif
               enddo
            enddo
         enddo
      enddo
      
      do i=1,nsta
         do j=1,nmodes
            do k=1,ncoeff(j,i)
               if (expcoe_mask(k,j,i).lt.0) then
                  n1=abs(expcoe_mask(k,j,i))
                  n2=parcons(n1)
                  if (n2 .lt. 0) expcoe(k,j,i)=0.0
               endif
            enddo
         enddo
      enddo                                                                     

      do i=1,nsta-1
         do l=i+1,nsta
            do j=1,nmodes
               do k=1,nmodes
                  if (iota_mask(k,j,i,l).lt.0) then
                     n1=abs(iota_mask(k,j,i,l))
                     n2=parcons(n1)
                     if (n2 .lt. 0) then
                        iota(k,j,i,l)=0.0
                        iota(k,j,l,i)=0.0
                     endif
                  endif
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine zerocons

!#######################################################################

      subroutine parstats(const_mask,kappa_mask,lambda_mask,&
           gamma_mask,iota_mask,mu_mask,expcoe_mask,termk_mask,&
           cpcoe_mask,switch_mask,cp2d_mask,modus)

      implicit none

      integer(long) :: i,j,k,l,m,n,o,s
      integer(long), intent(inout) :: modus
      integer(long), dimension(nsta,nsta), intent(in) :: const_mask
      integer(long), dimension(nmodes,nsta), intent(in) :: kappa_mask
      integer(long), dimension(nmodes,nsta,nsta), intent(in) :: lambda_mask
      integer(long), dimension(nmodes,nmodes,nsta), intent(in) :: gamma_mask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(in) :: expcoe_mask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: &
                                                         iota_mask,mu_mask
      integer(long), dimension(nadd), intent(in) :: termk_mask

      integer(long), dimension(maxcoe,nmodes,nsta,nsta), intent(in) :: cpcoe_mask

      integer(long), dimension(nswitch,maxcoe), intent(in) :: switch_mask

      integer(long)       :: ntpar0,ntconst0,ntkappa0,ntlambda0,ntgamma0,&
                             ntiota0,ntmu0,ntgen0,ntvar0,ntiota20
      integer(long), dimension(maxcoe,maxadd), intent(in)         :: cp2d_mask
!
! save old values
!
      if (modus .eq. 2) then
         write(ilog,'(/,a)') '*** Read Constraints ****'
         ntpar0=ntpar
         ntvar0=ntvar
         ntconst0=ntconst
         ntkappa0=ntkappa
         ntlambda0=ntlambda
         ntgamma0=ntgamma
         ntiota0=ntiota
         ntmu0=ntmu
         ntgen0=ntgen
         ntiota20=ntiota2
      endif

      ntpar=0

      nconst=0
      ntconst=0
      do i=1,nsta
         do j=i,nsta
            ntpar=ntpar+1
            if (const_mask(i,j).eq.1 .or.&
                const_mask(j,i).eq.1) nconst=nconst+1
            if (const_mask(i,j).ne.0) ntconst=ntconst+1
         enddo
      enddo

      nkappa=0
      ntkappa=0
      do i=1,nsta
         do j=1,nmodes
            ntpar=ntpar+1
            if (kappa_mask(j,i).eq.1) nkappa=nkappa+1
            if (kappa_mask(j,i).ne.0) ntkappa=ntkappa+1
         enddo
      enddo

      nlambda=0
      ntlambda=0
      do i=1,nsta-1
         do k=i+1,nsta
            do j=1,nmodes
               ntpar=ntpar+1
               if (lambda_mask(j,i,k).eq.1 .or.&
                    lambda_mask(j,k,i).eq.1) nlambda=nlambda+1
               if (lambda_mask(j,i,k).ne.0) ntlambda=ntlambda+1
            enddo
         enddo
      enddo

      ngamma=0
      ntgamma=0
      do i=1,nsta
         do j=1,nmodes
            do k=j,nmodes
               if (hamord .gt. 1) ntpar=ntpar+1
               if (gamma_mask(k,j,i).eq.1 .or.&
                   gamma_mask(j,k,i).eq.1) ngamma=ngamma+1
               if (gamma_mask(k,j,i).ne.0) ntgamma=ntgamma+1
            enddo
         enddo
      enddo

      niota=0
      ntiota=0
      do i=1,nsta
         do j=1,nmodes
            do k=1,nmodes
               if (hamord .gt. 2) ntpar=ntpar+1
               if (iota_mask(k,j,i,i).eq.1) niota=niota+1
               if (iota_mask(k,j,i,i).ne.0) ntiota=ntiota+1
               if (iota_mask(k,j,i,i).eq.1) print*, '=', k,j,i,i
            enddo
         enddo
      enddo

      nmu=0
      ntmu=0
      do i=1,nsta-1
         do l=i+1,nsta
            do j=1,nmodes
               do k=j,nmodes
                  if (hamord .gt. 2) ntpar=ntpar+1
                  if (mu_mask(k,j,i,l).eq.1 .or.&
                      mu_mask(j,k,i,l).eq.1 .or.&
                      mu_mask(k,j,l,i).eq.1 .or.&
                      mu_mask(j,k,l,i).eq.1) nmu=nmu+1
                  if (mu_mask(k,j,i,l).ne.0) ntmu=ntmu+1
               enddo
            enddo
         enddo
      enddo

      ngen=0
      ntgen=0
      do i=1,nsta
         do j=1,nmodes
! remove kappa and gamma parameter from total no. of parameters 
            if (ncoeff(j,i) .gt. 0) then
               ntpar=ntpar-1
               if (hamord .gt. 1) ntpar=ntpar-1
            endif
            do k=1,ncoeff(j,i)
               if (expcoe_mask(k,j,i).eq.1) then
                  ngen=ngen+1
               endif
            enddo
            do k=1,ncoeff(j,i)
               if (expcoe_mask(k,j,i).ne.0) then
                  ntgen=ntgen+1
               endif
            enddo
         enddo
      enddo
      ntpar=ntpar+ntgen

      ntcpgen=0
      ncpgen=0
      do i=1,nsta-1
         do j=i+1,nsta
            do m=1,nmodes
! remove lambda and mu parameters from total no. of parameters
               if (ncoeffcp(m,i,j).gt.0) then
                  if (llambda) ntpar=ntpar-1
                  if (lmu) ntpar=ntpar-1
               endif
               do k=1,ncoeffcp(m,j,i)
                  if (cpcoe_mask(k,m,j,i).eq.1.or. &
                       cpcoe_mask(k,m,i,j).eq.1) then
                     ncpgen=ncpgen+1
                  endif
               enddo
               do k=1,ncoeffcp(m,j,i)
                  if (cpcoe_mask(k,m,j,i).ne.0) then
                     ntcpgen=ntcpgen+1
                  endif
               enddo
            enddo
         enddo
      enddo

      niota2=0
      ntiota2=0
      do i=1,nsta-1
         do l=i+1,nsta
            do j=1,nmodes
               do k=1,nmodes
                  if (hamord .gt. 2) ntpar=ntpar+1
                  if (iota_mask(k,j,i,l).eq.1 .or. &
                      iota_mask(k,j,l,i).eq.1 ) niota2=niota2+1
                  if (iota_mask(k,j,i,l).ne.0 .or.&
                      iota_mask(k,j,l,i).eq.0 ) ntiota2=ntiota2+1
               enddo
            enddo
         enddo
      enddo

      ntermk=0
      nttermk=0
      do i=1,nadd
         if (termk_mask(i).eq.1) ntermk=ntermk+1
         if (termk_mask(i).ne.0.or.termk_mask(i).eq.0) nttermk=nttermk+1
      enddo

      nswitchpar=0
      ntswitchpar=0
      do i=1,nswitch
         do j=1,nswitchcoe(i)
            if (switch_mask(i,j).eq.1) nswitchpar=nswitchpar+1
            if (switch_mask(i,j).ne.0.or.switch_mask(i,j).eq.0) ntswitchpar=ntswitchpar+1
         enddo
      enddo

     ncp2d=0
     ntcp2d=0
     s = 1
     do while (cp2dtyp(s) .ne. 0)
       select case (cp2dtyp(s))
       case (1 : 2)
         if (cp2d_mask(1,s) .eq. 1) ncp2d=ncp2d+1
         if (cp2d_mask(1,s) .ne. 0) ntcp2d=ntcp2d+1
       case (3 : 5 )
        do i = 1, 4
         if (cp2d_mask(i,s) .eq. 1) ncp2d=ncp2d+1
         if (cp2d_mask(i,s) .ne. 0) ntcp2d=ntcp2d+1
        enddo
       case ( 6 )
        do i = 1, 4
         if (cp2d_mask(i,s) .eq. 1) ncp2d=ncp2d+1
         if (cp2d_mask(i,s) .ne. 0) ntcp2d=ntcp2d+1
        enddo
       end select
     s = s + 1
     enddo

     nvar=nconst+nkappa+nlambda+ngamma+niota+nmu+ngen+niota2+ &
          ncpgen+nswitchpar+ncp2d

     ntvar=ntconst+ntkappa+ntlambda+ntgamma+ntiota+ntmu+ntgen+ &
          ntiota2+ntcpgen+ntswitchpar+ntcp2d
     
     nvar = nvar+nhigher+ntermk
     ntvar = ntvar+nhigher+nttermk

!
! check whether total number of parameters has changed - ERROR
!
      if (modus .eq. 2 .and. ntpar0 .ne. ntpar) then
         routine='Parstats'
         write(6,*) 'Change in number of parameters (old/new)',&
                ntpar0,ntpar
         call vcerrormsg
      endif
         
!
! check whether total number of variables has changed due to constraints
! set by hand
!
      if (modus .eq. 2 .and. ntvar0 .ne. ntvar) then
         write(ilog,*) 'Change in number of parameters (old/new)',&
                ntvar0,ntvar
         if (ntconst0 .ne. ntconst) then
            write(ilog,*) 'Change in number of constants (old/new) ',&
                ntconst0,ntconst
         endif
         if (ntkappa0 .ne. ntkappa) then
            write(ilog,*) 'Change in number of kappa (old/new) ',&
                ntkappa0,ntkappa
         endif
         if (ntlambda0 .ne. ntlambda) then
            write(ilog,*) 'Change in number of lambda (old/new) ',&
                ntlambda0,ntlambda
         endif
         if (ntgamma0 .ne. ntgamma) then
            write(ilog,*) 'Change in number of gamma (old/new) ',&
                ntgamma0,ntgamma
         endif
         if (ntiota0 .ne. ntiota) then
            write(ilog,*) 'Change in number of iota (old/new) ',&
                ntiota0,ntiota
         endif
         if (ntmu0 .ne. ntmu) then
            write(ilog,*) 'Change in number of mu (old/new) ',&
                ntmu0,ntmu
         endif
         if (ntgen0 .ne. ntgen) then
            write(ilog,*) 'Change in number of coefficients (old/new) ',&
                ntgen0,ntgen
         endif
         if (ntiota20 .ne. ntiota2) then
            write(ilog,*) 'Change in number of iota2 (old/new) ',&
                ntiota0,ntiota
         endif
 
         modus=-2
      endif
         

      return
      end subroutine parstats
      
!#######################################################################
!
! type = 0 const(st,st1)) 
!        1 kappa(md,st)
!        2 lambda(md,st,st1)
!        3 gamma(md,md1,st)
!        4 iota(md,md1,st,st)
!        5 mu(md,md1,st,st1)
!        6 general(koe,md,st)
!        7 iota2(md,md1,st,st1)
!        8 termk(nadd) termst((st1,st2),nadd ) termdef(nmodes,nadd)
!        9 cpgeneral(koe,md,st,st1)
!       10 switch(attermk,k)
!
!#######################################################################

      subroutine parnumt(n1,type,md,st,md1,st1,koe,attermk,const_mask,&
                 kappa_mask,lambda_mask,gamma_mask,iota_mask,&
                 mu_mask,expcoe_mask,termk_mask,cpcoe_mask,switch_mask)

      implicit none

      integer(long)                                                 :: i,j,k,n,l,m
      integer(long), intent(out)                                    :: n1
      integer(long), intent(in)                                     :: type,md,st,md1,st1,koe,attermk
      integer(long), dimension(nsta,nsta), intent(in)               :: const_mask
      integer(long), dimension(nmodes,nsta), intent(in)             :: kappa_mask
      integer(long), dimension(nmodes,nsta,nsta), intent(in)        :: lambda_mask
      integer(long), dimension(nmodes,nmodes,nsta), intent(in)      :: gamma_mask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(in)      :: expcoe_mask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: iota_mask,mu_mask
      integer(long), dimension(nadd), intent(in)                    :: termk_mask
      integer(long), dimension(maxcoe,nmodes,nsta,nsta), intent(in) :: cpcoe_mask
      integer(long), dimension(nswitch,maxcoe) :: switch_mask
     
      n1 = -999 
      if (type .eq. 0) then
         n=0
         do i=1,nsta
            do j=i,nsta
               if (const_mask(i,j).ne.0) then
                  n=n+1
                  if (st .eq. i .and. st1 .eq. j) n1=n
                  if (st .eq. j .and. st1 .eq. i) n1=n
               endif
            enddo
         enddo

      else if (type .eq. 1) then
         n=ntconst
         do i=1,nsta
            do j=1,nmodes
               if (kappa_mask(j,i).ne.0) then
                  n=n+1
                  if (md .eq. j .and. st .eq. i) n1=n
               endif
            enddo
         enddo

      else if (type .eq. 2) then 
         n=ntconst+ntkappa
         do i=1,nsta-1
            do k=i+1,nsta
               do j=1,nmodes
                  if (lambda_mask(j,i,k).ne.0) then
                     n=n+1
                     if (md .eq. j .and. st .eq. i .and. &
                                 st1 .eq. k) n1=n
                     if (md .eq. j .and. st .eq. k .and. &
                                 st1 .eq. i) n1=n
                  endif
               enddo
            enddo
         enddo

      else if (type .eq. 3) then
         n=ntconst+ntkappa+ntlambda
         do i=1,nsta
            do j=1,nmodes
               do k=j,nmodes
                  if (gamma_mask(k,j,i).ne.0) then
                     n=n+1
                     if (md .eq. k .and. st .eq. i .and. &
                                 md1 .eq. j) n1=n
                     if (md .eq. j .and. st .eq. i .and. &
                                 md1 .eq. k) n1=n
                  endif
               enddo
            enddo
         enddo

      else if (type .eq. 4) then
         n=ntconst+ntkappa+ntlambda+ntgamma
         do i=1,nsta
            do j=1,nmodes
               do k=1,nmodes
                  if (iota_mask(k,j,i,i).ne.0) then
                     n=n+1
                     if (md .eq. k .and. st .eq. i .and.&
                                 md1 .eq. j) n1=n
                  endif
               enddo
            enddo
         enddo

      else if (type .eq. 5) then
         n=ntconst+ntkappa+ntlambda+ntgamma+ntiota
         do i=1,nsta-1
            do l=i+1,nsta
               do j=1,nmodes
                  do k=j,nmodes
                     if (mu_mask(k,j,i,l).ne.0) then
                        n=n+1
                        if (md .eq. k .and. st .eq. i .and. &
                              st1 .eq. l .and. md1 .eq. j) n1=n
                        if (md .eq. j .and. st .eq. i .and. &
                              st1 .eq. l .and. md1 .eq. k) n1=n
                        if (md .eq. j .and. st .eq. l .and. &
                              st1 .eq. i .and. md1 .eq. k) n1=n
                        if (md .eq. k .and. st .eq. l .and. &
                              st1 .eq. i .and. md1 .eq. j) n1=n
                     endif
                  enddo
               enddo
            enddo
         enddo

      else if (type .eq. 6) then
         n=ntconst+ntkappa+ntlambda+ntgamma+ntiota+ntmu
         do i=1,nsta
            do j=1,nmodes
               do k=1,ncoeff(j,i)
                  if (expcoe_mask(k,j,i).ne.0) then
                     n=n+1
                     if (md .eq. j .and. st .eq. i .and. &
                            koe .eq. k) n1=n
                     
                  endif
               enddo
            enddo
         enddo

      else if (type .eq. 7) then
         n=ntconst+ntkappa+ntlambda+ntgamma+ntiota+ntmu+ntgen
         do i=1,nsta-1
            do l=i+1,nsta
               do j=1,nmodes
                  do k=1,nmodes
                     if (iota_mask(k,j,i,l).ne.0) then
                        n=n+1
                        if (md .eq. k .and. st .eq. i .and. &
                              st1 .eq. l .and. md1 .eq. j) n1=n

                        if (md .eq. k .and. st .eq. l .and. &
                              st1 .eq. i .and. md1 .eq. j) n1=n
                     endif
                  enddo
               enddo
            enddo
         enddo
         
      else if (type .eq. 8) then
      n=ntconst+ntkappa+ntlambda+ntgamma+ntiota+ntmu+ntgen+ntiota2&
         +nhigher
      do i = 1, nadd
        if( termk_mask(i) .ne. 0) then
          n = n + 1
          if( i .eq. attermk) n1=n
        endif
      enddo
     else if (type .eq. 9) then
        n=ntconst+ntkappa+ntlambda+ntgamma+ntiota+ntmu+ntgen+ntiota2&
           +nhigher+nttermk
        do i=1,nsta-1
           do j=i+1,nsta
              do m=1,nmodes
                 do k=1,ncoeffcp(m,j,i)
                    if (cpcoe_mask(k,m,j,i).ne.0) then
                       n=n+1
                       if (md .eq. m .and. st .eq. i .and. st1 .eq. j &
                            .and. koe .eq. k) n1=n
                       if (md .eq. m .and. st .eq. j .and. st1 .eq. i &
                            .and. koe .eq. k) n1=n          
                    endif
                 enddo
              enddo
           enddo
        enddo

     else if (type .eq. 10) then
        n=ntconst+ntkappa+ntlambda+ntgamma+ntiota+ntmu+ntgen+ntiota2&
             +nhigher+nttermk+ntcpgen
        do i=1,attermk
           do j=1,nswitchcoe(i)
              if (switch_mask(i,j).ne.0) then
                 n=n+1
                 if (i.eq.attermk.and.j.eq.koe) n1=n
              endif
           enddo
        enddo
        
     endif
     if( n1 .eq. -999) print*, 'Could not find constraint (parnumt routine) for type:',type

      return
      end subroutine parnumt
      
!#######################################################################
!
! type = 0 const(st,st1)
!        1 kappa(md,st)
!        2 lambda(md,st,st1)
!        3 gamma(md,md1,st)
!        4 iota(md,md1,st,st)
!        5 mu(md,md1,st,st1)
!        6 general(koe,md,st)
!        7 iota(md,md1,st,st1)
!        8 termk(attetmk)
!        9 cpgeneral(koe,md,st,st1)
!       11 switch(attermk,k)
!
!#######################################################################

      subroutine parnum(n1,type,md,st,md1,st1,koe,attermk,const_mask,&
                 kappa_mask,lambda_mask,gamma_mask,iota_mask,&
                 mu_mask,expcoe_mask,termk_mask,cpcoe_mask,switch_mask)

      implicit none

      integer(long)                                                 :: i,j,k,n,l,m
      integer(long), intent(in)                                     :: type,md,st,md1,st1,koe,attermk
      integer(long), intent(out)                                    :: n1
      integer(long), dimension(nsta,nsta), intent(in)               :: const_mask
      integer(long), dimension(nmodes,nsta), intent(in)             :: kappa_mask
      integer(long), dimension(nmodes,nsta,nsta), intent(in)        :: lambda_mask
      integer(long), dimension(nmodes,nmodes,nsta), intent(in)      :: gamma_mask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(in)      :: expcoe_mask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: iota_mask,mu_mask
      integer(long), dimension(nadd), intent(in)                    :: termk_mask
      integer(long), dimension(maxcoe,nmodes,nsta,nsta), intent(in) :: cpcoe_mask
      integer(long), dimension(nswitch,maxcoe)                      :: switch_mask

      if (type .eq. 0) then
         n=0
         do i=1,nsta
            do j=i,nsta
               if (const_mask(i,j).eq.1 .or.&
                   const_mask(j,i).eq.1) then
                  n=n+1
                  if (st .eq. i .and. st1 .eq. j) n1=n
                  if (st .eq. j .and. st1 .eq. i) n1=n
               endif
            enddo
         enddo

      else if (type .eq. 1) then
         n=nconst
         do i=1,nsta
            do j=1,nmodes
               if (kappa_mask(j,i).eq.1) then
                  n=n+1
                  if (md .eq. j .and. st .eq. i) n1=n
               endif
            enddo
         enddo

      else if (type .eq. 2) then
         n=nconst+nkappa
         do i=1,nsta-1
            do k=i+1,nsta
               do j=1,nmodes
                  if (lambda_mask(j,i,k).eq.1 .or.&
                      lambda_mask(j,k,i).eq.1) then
                     n=n+1
                     if (md .eq. j .and. st .eq. i .and. &
                                 st1 .eq. k) n1=n
                     if (md .eq. j .and. st .eq. k .and. &
                                 st1 .eq. i) n1=n
                  endif
               enddo
            enddo
         enddo

      else if (type .eq. 3) then
         n=nconst+nkappa+nlambda
         do i=1,nsta
            do j=1,nmodes
               do k=j,nmodes
                  if (gamma_mask(k,j,i).eq.1 .or.&
                      gamma_mask(j,k,i).eq.1) then
                     n=n+1
                     if (md .eq. k .and. st .eq. i .and. &
                                 md1 .eq. j) n1=n
                     if (md .eq. j .and. st .eq. i .and. &
                                 md1 .eq. k) n1=n
                  endif
               enddo
            enddo
         enddo

      else if (type .eq. 4) then
         n=nconst+nkappa+nlambda+ngamma
         do i=1,nsta
            do j=1,nmodes
               do k=1,nmodes
                  if (iota_mask(k,j,i,i).eq.1) then
                     n=n+1
                     if (md .eq. k .and. st .eq. i .and.&
                                 md1 .eq. j) n1=n
                  endif
               enddo
            enddo
         enddo

      else if (type .eq. 5) then
         n=nconst+nkappa+nlambda+ngamma+niota
         do i=1,nsta-1
            do l=i+1,nsta
               do j=1,nmodes
                  do k=j,nmodes
                     if (mu_mask(k,j,i,l).eq.1 .or.&
                         mu_mask(j,k,i,l).eq.1 .or.&
                         mu_mask(k,j,l,i).eq.1 .or.&
                         mu_mask(j,k,l,i).eq.1) then
                        n=n+1
                        if (md .eq. k .and. st .eq. i .and. &
                              st1 .eq. l .and. md1 .eq. j) n1=n
                        if (md .eq. j .and. st .eq. i .and. &
                              st1 .eq. l .and. md1 .eq. k) n1=n
                        if (md .eq. k .and. st .eq. l .and. &
                              st1 .eq. i .and. md1 .eq. j) n1=n
                        if (md .eq. j .and. st .eq. l .and. &
                              st1 .eq. i .and. md1 .eq. k) n1=n
                     endif
                  enddo
               enddo
            enddo
         enddo

      else if (type .eq. 6) then
         n=nconst+nkappa+nlambda+ngamma+niota+nmu
         do i=1,nsta
            do j=1,nmodes
               do k=1,ncoeff(j,i)
                  if (expcoe_mask(k,j,i).eq.1) then
                     n=n+1
                     if (md .eq. j .and. st .eq. i .and. &
                            koe .eq. k) n1=n
                  endif
               enddo
            enddo
         enddo

      else if (type .eq. 7) then
         n=nconst+nkappa+nlambda+ngamma+niota+nmu+ngen
         do i=1,nsta-1
            do l=i+1,nsta
               do j=1,nmodes
                  do k=1,nmodes
                     if (iota_mask(k,j,i,l).eq.1) then
                        n=n+1
                        if (md .eq. k .and. st .eq. i .and. &
                              st1 .eq. l .and. md1 .eq. j) n1=n
                        if (md .eq. k .and. st .eq. l .and. &
                              st1 .eq. i .and. md1 .eq. j) n1=n
                     endif
                  enddo
               enddo
            enddo
         enddo

      else if (type .eq. 8) then

         n=nconst+nkappa+nlambda+ngamma+niota+nmu+ngen+niota2&
            +nhigher
         do i = 1, nadd 
           if( termk_mask(i) .eq.1) then
              n=n+1
              if(i .eq. attermk) n1=n
            endif
         enddo

      else if (type .eq. 9) then
         n=nconst+nkappa+nlambda+ngamma+niota+nmu+ngen+niota2&
              +nhigher+ntermk
         do i=1,nsta-1
           do j=i+1,nsta
              do m=1,nmodes
                 do k=1,ncoeffcp(m,j,i)
                    if (cpcoe_mask(k,m,j,i).eq.1) then
                       n=n+1
                       if (md .eq. m .and. st .eq. i .and. st1 .eq. j &
                            .and. koe .eq. k) n1=n
                       if (md .eq. m .and. st .eq. j .and. st1 .eq. i &
                            .and. koe .eq. k) n1=n          
                    endif
                 enddo
              enddo
           enddo
        enddo

     else if (type .eq. 10) then
        n=nconst+nkappa+nlambda+ngamma+niota+nmu+ngen+niota2&
             +nhigher+ntermk+ncpgen
        do i=1,attermk
           do j=1,nswitchcoe(i)
              if (switch_mask(i,j).ne.0) then
                 n=n+1
                 if (i.eq.attermk.and.j.eq.koe) n1=n
              endif
           enddo
        enddo

     endif

      return
      end subroutine parnum
      
!#######################################################################
!
! type = 0 const(st)
!        1 kappa(md,st)
!        2 lambda(md,st,st1)
!        3 gamma(md,md1,st)
!        4 iota(md,md1,st,st)
!        5 mu(md,md1,st,st1)
!        6 general(koe,md,st)
!        7 iota(md,md1,st,st1)
!        8 termk(nadd) termst((st1,st2),nadd ) termdef(nmodes,nadd)
!        9 cpgeneral(koe,md,st,st1)
!       10 switch(attermk,koe)
!
!#######################################################################

      subroutine numtpar(n1,type,md,st,md1,st1,koe,attermk,const_mask,&
                 kappa_mask,lambda_mask,gamma_mask,iota_mask,&
                 mu_mask,expcoe_mask,termk_mask,cpcoe_mask,switch_mask)

      implicit none

      integer(long)                                                 :: i,j,k,n,l,n2,m
      integer(long), intent(in)                                     :: n1
      integer(long), intent(out)                                    :: type,md,st,md1,st1,koe,attermk
      integer(long), dimension(nsta,nsta), intent(in)               :: const_mask
      integer(long), dimension(nmodes,nsta), intent(in)             :: kappa_mask
      integer(long), dimension(nmodes,nsta,nsta), intent(in)        :: lambda_mask
      integer(long), dimension(nmodes,nmodes,nsta), intent(in)      :: gamma_mask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(in)      :: expcoe_mask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: iota_mask,mu_mask
      integer(long), dimension(nadd), intent(in)                    :: termk_mask
      integer(long), dimension(maxcoe,nmodes,nsta,nsta), intent(in) :: cpcoe_mask
      integer(long), dimension(nswitch,maxcoe)                      :: switch_mask


      type = 0
      md=0
      st=0
      md1=0
      st1=0
      koe=0
      n2=abs(n1)
      if (n2 .gt. ntconst) type=1
      if (n2 .gt. ntconst+ntkappa) type=2
      if (n2 .gt. ntconst+ntkappa+ntlambda) type=3
      if (n2 .gt. ntconst+ntkappa+ntlambda+ntgamma) type=4
      if (n2 .gt. ntconst+ntkappa+ntlambda+ntgamma+ntiota) type=5
      if (n2 .gt. ntconst+ntkappa+ntlambda+ntgamma+ntiota+ntmu) type=6
      if (n2 .gt. &
           ntconst+ntkappa+ntlambda+ntgamma+ntiota+ntmu+ntgen) type=7
      if (n2 .gt. &
           ntconst+ntkappa+ntlambda+ntgamma+ntiota+ntmu+ntgen+ntiota2&
             +nhigher) type=8
      if (n2 .gt. &
           ntconst+ntkappa+ntlambda+ntgamma+ntiota+ntmu+ntgen+ntiota2&
           +nhigher+nttermk) type=9
      if (n2 .gt. &
           ntconst+ntkappa+ntlambda+ntgamma+ntiota+ntmu+ntgen+ntiota2&
           +nhigher+nttermk+ntcpgen) type=10

      if (type .eq. 0) then
         n=0
         do i=1,nsta
            do j=i,nsta
               if (const_mask(i,j).ne.0) then
                  n=n+1
                  if (n .eq. n2) then
                     st=i
                     st1=j
                  endif
               endif
            enddo
         enddo

      else if (type .eq. 1) then
         n=ntconst
         do i=1,nsta
            do j=1,nmodes
               if (kappa_mask(j,i).ne.0) then
                  n=n+1
                  if (n .eq. n2) then
                     md=j
                     st=i
                  endif
               endif
            enddo
         enddo

      else if (type .eq. 2) then
         n=ntconst+ntkappa
         do i=1,nsta-1
            do k=i+1,nsta
               do j=1,nmodes
                  if (lambda_mask(j,i,k).ne.0) then
                     n=n+1
                     if (n .eq. n2) then
                        md=j
                        st=i
                        st1=k
                     endif
                  endif
               enddo
            enddo
         enddo

      else if (type .eq. 3) then
         n=ntconst+ntkappa+ntlambda
         do i=1,nsta
            do j=1,nmodes
               do k=j,nmodes
                  if (gamma_mask(k,j,i).ne.0) then
                     n=n+1
                     if (n .eq. n2) then
                        md=k
                        md1=j
                        st=i
                     endif
                  endif
               enddo
            enddo
         enddo

      else if (type .eq. 4) then
         n=ntconst+ntkappa+ntlambda+ntgamma
         do i=1,nsta
            do j=1,nmodes
               do k=1,nmodes
                  if (iota_mask(k,j,i,i).ne.0) then
                     n=n+1
                     if (n .eq. n2) then
                        md=k
                        md1=j
                        st=i
                     endif
                  endif
               enddo
            enddo
         enddo

      else if (type .eq. 5) then
         n=ntconst+ntkappa+ntlambda+ntgamma+ntiota
         do i=1,nsta-1
            do l=i+1,nsta
               do j=1,nmodes
                  do k=j,nmodes
                     if (mu_mask(k,j,i,l).ne.0) then
                        n=n+1
                        if (n .eq. n2) then
                           md=k
                           md1=j
                           st=i
                           st1=l
                        endif
                     endif
                  enddo
               enddo
            enddo
         enddo

      else if (type .eq. 6) then
         n=ntconst+ntkappa+ntlambda+ntgamma+ntiota+ntmu
         do i=1,nsta
            do j=1,nmodes
               do k=1,ncoeff(j,i)
                  if (expcoe_mask(k,j,i).ne.0) then
                     n=n+1
                     if (n .eq. n2) then
                        koe=k
                        md=j
                        st=i
                     endif
                  endif
               enddo
            enddo
         enddo

      else if (type .eq. 7) then
         n=ntconst+ntkappa+ntlambda+ntgamma+ntiota+ntmu+ntgen
         do i=1,nsta-1
            do l=i+1,nsta
               do j=1,nmodes
                  do k=1,nmodes
                     if (iota_mask(k,j,i,l).ne.0) then
                        n=n+1
                        if (n .eq. n2) then
                           md=k
                           md1=j
                           st=i
                           st1=l
                        endif
                     endif
                  enddo
               enddo
            enddo
         enddo

      else if (type .eq. 8) then
        n=ntconst+ntkappa+ntlambda+ntgamma+ntiota+ntmu+ntgen+ntiota2+nhigher
         do i=1,nadd
            if (termk_mask(i).ne.0) then
                n=n+1
                if (n .eq. n2) then
                   attermk=i
                endif
            endif
         enddo

      else if (type .eq. 9) then
         n=ntconst+ntkappa+ntlambda+ntgamma+ntiota+ntmu+ntgen+ntiota2&
              +nhigher+nttermk
         do i=1,nsta-1
            do j=i+1,nsta
               do m=1,nmodes
                  do k=1,ncoeffcp(m,j,i)
                     if (cpcoe_mask(k,m,j,i).ne.0) then
                        n=n+1
                        if (n .eq. n2) then
                           koe=k
                           md=m
                           st=i
                           st1=j
                        endif
                     endif
                  enddo
               enddo
            enddo
         enddo

      else if (type .eq. 10) then
         n=ntconst+ntkappa+ntlambda+ntgamma+ntiota+ntmu+ntgen+ntiota2&
              +nhigher+nttermk+ntcpgen
         do i=1,nswitch
            do j=1,nswitchcoe(i)
               if (switch_mask(i,j).ne.0) then
                  n=n+1
                  if (n.eq.n2) then
                     attermk=i
                     koe=j
                  endif
               endif
            enddo
         enddo

      endif

      return
      end subroutine numtpar
      
!#######################################################################
!
! type = 0 const(st)
!        1 kappa(md,st)
!        2 lambda(md,st,st1)
!        3 gamma(md,md1,st)
!        4 iota(md,md1,st,st)
!        5 mu(md,md1,st,st1)
!        6 general(koe,md,st)
!        7 iota(md,md1,st,st1)
!        8 termk(nadd) termst((st1,st2),nadd ) termdef(nmodes,nadd)
!        9 cpgeneral(koe,md,st,st1)
!
!#######################################################################

      subroutine numpar(n1,type,md,st,md1,st1,koe,attermk,const_mask,&
                 kappa_mask,lambda_mask,gamma_mask,iota_mask,&
                 mu_mask,expcoe_mask,termk_mask,cpcoe_mask)

      implicit none

      integer(long)     :: i,j,k,n,l,n2,m,n3
      integer(long), intent(in)                                     :: n1
      integer(long), intent(out)                                    :: type,md,st,md1,st1,koe,attermk
      integer(long), dimension(nsta,nsta), intent(in)               :: const_mask
      integer(long), dimension(nmodes,nsta), intent(in)             :: kappa_mask
      integer(long), dimension(nmodes,nsta,nsta), intent(in)        :: lambda_mask
      integer(long), dimension(nmodes,nmodes,nsta), intent(in)      :: gamma_mask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(in)      :: expcoe_mask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: iota_mask,mu_mask
      integer(long), dimension(nadd), intent(in)                    :: termk_mask
      integer(long), dimension(maxcoe,nmodes,nsta,nsta), intent(in) :: cpcoe_mask

      type = 0
      md=0
      st=0
      md1=0
      st1=0
      koe=0

      n2=abs(n1)
      n3 = nconst
      if (n2 .gt. n3) type=1
      n3 = n3 + nkappa
      if (n2 .gt. n3) type=2
      n3 = n3 + nlambda
      if (n2 .gt. n3) type=3
      n3 = n3 + ngamma
      if (n2 .gt. n3) type=4
      n3 = n3 + niota
      if (n2 .gt. n3) type=5
      n3 = n3 + nmu
      if (n2 .gt. n3) type=6
      n3 = n3 + ngen
      if (n2 .gt. n3) type=7
      n3 = n3 + niota2+nhigher
      if (n2 .gt. n3) type=8
      n3 = n3 + ntermk
      if (n2 .gt. n3) type=9

!      print*, 'in numpar with n2 = abs(n1) =', n2, 'with type =', type
      n=0
      if (type .eq. 0) then
         do i=1,nsta
            do j=i,nsta
               if (const_mask(i,j).eq.1 .or.&
                   const_mask(j,i).eq.1) then
                  n=n+1
                  if (n .eq. n2) then
                     st=i
                     st1=j
                  endif
               endif
            enddo
         enddo

      else if (type .eq. 1) then
         n=nconst
         do i=1,nsta
            do j=1,nmodes
               if (kappa_mask(j,i).eq.1) then
                  n=n+1
                  if (n .eq. n2) then
                     md=j
                     st=i
                  endif
               endif
            enddo
         enddo

      else if (type .eq. 2) then
         n=nconst+nkappa
         do i=1,nsta-1
            do k=i+1,nsta
               do j=1,nmodes
                  if (lambda_mask(j,i,k).eq.1 .or.&
                      lambda_mask(j,k,i).eq.1) then
                     n=n+1
                     if (n .eq. n2) then
                        md=j
                        st=i
                        st1=k
                     endif
                  endif
               enddo
            enddo
         enddo

      else if (type .eq. 3) then
         n=nconst+nkappa+nlambda
         do i=1,nsta
            do j=1,nmodes
               do k=j,nmodes
                  if (gamma_mask(k,j,i).eq.1 .or.&
                      gamma_mask(j,k,i).eq.1) then
                     n=n+1
                     if (n .eq. n2) then
                        md=k
                        md1=j
                        st=i
                     endif
                  endif
               enddo
            enddo
         enddo

      else if (type .eq. 4) then
         n=nconst+nkappa+nlambda+ngamma
         do i=1,nsta
            do j=1,nmodes
               do k=1,nmodes
                  if (iota_mask(k,j,i,i).eq.1) then
                     n=n+1
                     if (n .eq. n2) then
                        md=k
                        md1=j
                        st=i
                     endif
                  endif
               enddo
            enddo
         enddo

      else if (type .eq. 5) then
         n=nconst+nkappa+nlambda+ngamma+niota
         do i=1,nsta-1
            do l=i+1,nsta
               do j=1,nmodes
                  do k=j,nmodes
                     if (mu_mask(k,j,i,l).eq.1 .or.&
                         mu_mask(j,k,i,l).eq.1 .or.&
                         mu_mask(j,k,l,i).eq.1 .or.&
                         mu_mask(k,j,l,i).eq.1) then
                        n=n+1
                        if (n .eq. n2) then
                           md=k
                           md1=j
                           st=i
                           st1=l
                        endif
                     endif
                  enddo
               enddo
            enddo
         enddo

      else if (type .eq. 6) then
         n=nconst+nkappa+nlambda+ngamma+niota+nmu
         do i=1,nsta
            do j=1,nmodes
               do k=1,ncoeff(j,i)
                  if (expcoe_mask(k,j,i).eq.1) then
                     n=n+1
                     if (n .eq. n2) then
                        koe=k
                        md=j
                        st=i
                     endif
                  endif
               enddo
            enddo
         enddo

      else if (type .eq. 7) then
         n=nconst+nkappa+nlambda+ngamma+niota+nmu+ngen
         do i=1,nsta-1
            do l=i+1,nsta
               do j=1,nmodes
                  do k=1,nmodes
                     if (iota_mask(k,j,i,l).eq.1) then
                        n=n+1
                        if (n .eq. n2) then
                           md=k
                           md1=j
                           st=i
                           st1=l
                        endif
                     endif
                  enddo
               enddo
            enddo
         enddo
     
      else if (type .eq. 8) then
         n=nconst+nkappa+nlambda+ngamma+niota+nmu+ngen+niota2+nhigher
         do i=1,nadd
            if (termk_mask(i).eq.1) then
               n=n+1
               if (n .eq. n2) then
                  attermk = i
               endif
            endif
         enddo


      else if (type .eq. 9) then
         n=nconst+nkappa+nlambda+ngamma+niota+nmu+ngen+niota2&
              +nhigher+ntermk
         do i=1,nsta-1
            do j=i+1,nsta
               do m=1,nmodes
                  do k=1,ncoeffcp(m,j,i)
                     if (cpcoe_mask(k,m,j,i).eq.1) then
                        n=n+1
                        if (n .eq. n2) then
                           koe=k
                           md=m
                           st=i
                           st1=j
                        endif
                     endif
                  enddo
               enddo
            enddo
         enddo

      endif

      return
      end subroutine numpar

!#######################################################################

      subroutine integralsym (check,nmchk,stachk,axchk)

      implicit none

      integer(long) :: i,j,k,l,r,numsym,prodnum
      integer(long), dimension(maxdim), intent(in) :: nmchk,stachk
      integer(long), dimension(3), intent(in)      :: axchk
      integer(long), dimension(maxdim)             :: prodchar
      integer(long), intent(out)                   :: check
      real(dop)                                    :: junk      
      character(len=16)                            :: dummy

      dummy=''
      check=0
      prodnum=0

!----------------------------------------------------------------------------
! N.B. In the matching of the mode/state/axis symmetry labels to those
! of the irreps of the point group, all letters are converted to 
! uppercase, and all instances of ` are converted to ' in order to allow
! for a free(ish) input format
!
! In the following,
!
! chdim: order of the point group
! numsym: no. of times that the totally symmetric irrep enters into the
!         decomposition of the direct product representation
! chel(i,j): character of the ith irrep of the jth symmetry operation
!----------------------------------------------------------------------------

! Contributions from differentiation wrt the normal modes
      do i=1,nmodes
         if (nmchk(i).gt.0) then            
            do j=1,nmchk(i)
               prodnum=prodnum+1
               do k=1,chdim
                  if (nmlab(i).eq.irreplab(k)) prodchar(prodnum)=k
               enddo
            enddo
         endif
      enddo

! Contributions from the electronic wavefunctions
      do i=1,nsta
         if (stachk(i).gt.0) then            
            do j=1,stachk(i)
               prodnum=prodnum+1
               do k=1,chdim
                  if (stalab(i).eq.irreplab(k)) prodchar(prodnum)=k
               enddo
            enddo
         endif
      enddo

! Contributions from differentiation wrt electric field components
! (for use in LMFIT only)
      do i=1,3
         if (axchk(i).gt.0) then            
            do j=1,axchk(i)
               prodnum=prodnum+1
               do k=1,chdim
                  if (axislab(i).eq.irreplab(k)) prodchar(prodnum)=k
               enddo
            enddo
         endif
      enddo

      numsym=0
! Loop over operations
      do r=1,chdim
! Loop over irreps intering into direct product
         junk=1
         do i=1,prodnum
            junk=junk*chel(prodchar(i),r)
         enddo
         numsym=numsym+int(junk)
      enddo

      numsym=numsym/chdim

      if (numsym.ne.0) check=1

      return
      end subroutine integralsym

!#######################################################################
!
! getcharacters determines the symmetry labels and characters of the 
! irreps of the point group of interest
!
!#######################################################################

      subroutine getcharacters

        implicit none

        integer(long)           :: i,j
        character(len=80)       :: dummy

!-----------------------------------------------------------------------
! BEFORE ADDING CHARACTERS FOR A NEW POINT GROUP, READ AND OBEY THE 
! FOLLOWING (unless you can think of a better way of doing it) 
!-----------------------------------------------------------------------
!
! 1.) All pointgroup labels added here must not contain any lowercase 
!     letters.
!
! 2.) All irrep symmetry labels must not contain any lowercase letters,
!     and also must not contain any ` characters (only ' characters are
!     allowed).
!
! 3.) No " characters are allowed, only ''.
!
! 4.) All numbers must appear before instances of ' and ''
!
! chdim:     order of the point group
! chel(i,j): character of the ith irrep of the jth symmetry operation
!-----------------------------------------------------------------------
        do i=1,maxdim
           irreplab(i)=''
        enddo

        dummy=pntgrp 
        call uppercase(dummy,len(dummy))

        if (dummy.eq.'C1') then

           chdim=1
           
           irreplab(1)='A'

           chel(1,1)=1.0d0

        else if (dummy.eq.'C2V') then
        
           chdim=4

           irreplab(1)='A1'
           irreplab(2)='A2'
           irreplab(3)='B1'
           irreplab(4)='B2'
           
           chel(1,1)=1.0d0
           chel(1,2)=1.0d0
           chel(1,3)=1.0d0
           chel(1,4)=1.0d0
           
           chel(2,1)=1.0d0
           chel(2,2)=1.0d0
           chel(2,3)=-1.0d0
           chel(2,4)=-1.0d0
           
           chel(3,1)=1.0d0
           chel(3,2)=-1.0d0
           chel(3,3)=1.0d0
           chel(3,4)=-1.0d0

           chel(4,1)=1.0d0
           chel(4,2)=-1.0d0
           chel(4,3)=-1.0d0
           chel(4,4)=1.0d0

        else if (dummy.eq.'CS') then
           
           chdim=2
           
           irreplab(1)='A'''
           irreplab(2)='A'''''

           chel(1,1)=1.0d0
           chel(1,2)=1.0d0

           chel(2,1)=1.0d0
           chel(2,2)=-1.0d0

        else if (dummy.eq.'D2H') then

           chdim=8
           
           irreplab(1)='AG'
           irreplab(2)='B1G'
           irreplab(3)='B2G'
           irreplab(4)='B3G'
           irreplab(5)='AU'
           irreplab(6)='B1U'
           irreplab(7)='B2U'
           irreplab(8)='B3U'

           chel(1,1)=1.0d0
           chel(1,2)=1.0d0
           chel(1,3)=1.0d0
           chel(1,4)=1.0d0
           chel(1,5)=1.0d0
           chel(1,6)=1.0d0
           chel(1,7)=1.0d0
           chel(1,8)=1.0d0

           chel(2,1)=1.0d0
           chel(2,2)=1.0d0
           chel(2,3)=-1.0d0
           chel(2,4)=-1.0d0
           chel(2,5)=1.0d0
           chel(2,6)=1.0d0
           chel(2,7)=-1.0d0
           chel(2,8)=-1.0d0

           chel(3,1)=1.0d0
           chel(3,2)=-1.0d0
           chel(3,3)=1.0d0
           chel(3,4)=-1.0d0
           chel(3,5)=1.0d0
           chel(3,6)=-1.0d0
           chel(3,7)=1.0d0
           chel(3,8)=-1.0d0

           chel(4,1)=1.0d0
           chel(4,2)=-1.0d0
           chel(4,3)=-1.0d0
           chel(4,4)=1.0d0
           chel(4,5)=1.0d0
           chel(4,6)=-1.0d0
           chel(4,7)=-1.0d0
           chel(4,8)=1.0d0

           chel(5,1)=1.0d0
           chel(5,2)=1.0d0
           chel(5,3)=1.0d0
           chel(5,4)=1.0d0
           chel(5,5)=-1.0d0
           chel(5,6)=-1.0d0
           chel(5,7)=-1.0d0
           chel(5,8)=-1.0d0

           chel(6,1)=1.0d0
           chel(6,2)=1.0d0
           chel(6,3)=-1.0d0
           chel(6,4)=-1.0d0
           chel(6,5)=-1.0d0
           chel(6,6)=-1.0d0
           chel(6,7)=1.0d0
           chel(6,8)=1.0d0

           chel(7,1)=1.0d0
           chel(7,2)=-1.0d0
           chel(7,3)=1.0d0
           chel(7,4)=-1.0d0
           chel(7,5)=-1.0d0
           chel(7,6)=1.0d0
           chel(7,7)=-1.0d0
           chel(7,8)=1.0d0

           chel(8,1)=1.0d0
           chel(8,2)=-1.0d0
           chel(8,3)=-1.0d0
           chel(8,4)=1.0d0
           chel(8,5)=-1.0d0
           chel(8,6)=1.0d0
           chel(8,7)=1.0d0
           chel(8,8)=-1.0d0
           
        else if (dummy.eq.'CI') then
           
           chdim=2

           irreplab(1)='AG'
           irreplab(2)='AU'

           chel(1,1)=1.0d0
           chel(1,2)=1.0d0

           chel(2,1)=1.0d0
           chel(2,2)=-1.0d0

        else if (dummy.eq.'C2') then

           chdim=2

           irreplab(1)='A'
           irreplab(2)='B'

           chel(1,1)=1.0d0
           chel(1,2)=1.0d0

           chel(2,1)=1.0d0
           chel(2,2)=-1.0d0

        else if (dummy.eq.'D2') then

           chdim=4

           irreplab(1)='A'
           irreplab(2)='B1'
           irreplab(3)='B2'
           irreplab(4)='B3'
           
           chel(1,1)=1.0d0
           chel(1,2)=1.0d0
           chel(1,3)=1.0d0
           chel(1,4)=1.0d0

           chel(2,1)=1.0d0
           chel(2,2)=1.0d0
           chel(2,3)=-1.0d0
           chel(2,4)=-1.0d0
           
           chel(3,1)=1.0d0
           chel(3,2)=-1.0d0
           chel(3,3)=1.0d0
           chel(3,4)=-1.0d0

           chel(4,1)=1.0d0
           chel(4,2)=-1.0d0
           chel(4,3)=-1.0d0
           chel(4,4)=1.0d0

        else if (dummy.eq.'C2H') then

           chdim=4

           irreplab(1)='AG'
           irreplab(2)='BG'
           irreplab(3)='AU'
           irreplab(4)='BU'
           
           chel(1,1)=1.0d0
           chel(1,2)=1.0d0
           chel(1,3)=1.0d0
           chel(1,4)=1.0d0

           chel(2,1)=1.0d0
           chel(2,2)=-1.0d0
           chel(2,3)=1.0d0
           chel(2,4)=-1.0d0
           
           chel(3,1)=1.0d0
           chel(3,2)=1.0d0
           chel(3,3)=-1.0d0
           chel(3,4)=-1.0d0

           chel(4,1)=1.0d0
           chel(4,2)=-1.0d0
           chel(4,3)=-1.0d0
           chel(4,4)=1.0d0
           
        else
           write(6,'(/,3(a,x),/)') 'The point group',&
                pntgrp(1:len_trim(pntgrp)),'is not supported.'
           STOP
        endif
        
        return
      end subroutine getcharacters
      
!#######################################################################

      subroutine cleanirreps(string,dim)

        implicit none

        integer(long)      :: dim,j,i
        character(len=dim) :: string,tmp
        logical(kind=4)    :: check

        tmp=''
        check=.false.

! Convert to upper case letters 
        call uppercase(string,len(string))

! Convert all instances of ` to '
        do j=1,len_trim(string)
           if (string(j:j).eq.'`') string(j:j)=''''
        enddo

! Convert all instances of " to ''
        do j=1,len_trim(string)
           if (string(j:j).eq.'"') then
              check=.true.
              string(j:j)=''''
              do i=1,j
                 tmp(i:i)=string(i:i)
              enddo
              do i=j+1,len_trim(string)+1
                 tmp(i:i)=string(i-1:i-1)
              enddo
           endif
        enddo
        if (check) then 
           string=tmp
           tmp=''
        endif

! Place all numbers before primes
        do j=1,len_trim(string)
           if (string(j:j).ge.'1'.and.string(j:j).le.'5') then
              if (string(j-1:j-1).eq.'''') then
                 ! Double prime
                 if (string(j-2:j-2).eq.'''') then
                    tmp(1:1)=string(j:j)
                    tmp(2:2)=string(j-1:j-1)
                    tmp(3:3)=string(j-2:j-2)
                    string(j-2:j-2)=tmp(1:1)
                    string(j-1:j-1)=tmp(2:2)
                    string(j:j)=tmp(3:3)
                 ! Single prime
                 else
                    tmp(1:1)=string(j:j)
                    tmp(2:2)=string(j-1:j-1)
                    string(j-1:j-1)=tmp(1:1)
                    string(j:j)=tmp(2:2)              
                 endif
              endif
           endif
        enddo

      end subroutine cleanirreps

!#######################################################################

      subroutine subvec(vec1,vec,dim)

      implicit none

      integer(long)                            :: i
      integer(long), intent(in)                :: dim
      real(dop), dimension(dim), intent(in)    :: vec
      real(dop), dimension(dim), intent(inout) :: vec1

      do i=1,dim
         vec1(i)=vec1(i)-vec(i)
      enddo

      return
      end subroutine subvec

!######################################################################
! calculate the rotation matrix
!######################################################################

      subroutine getrot(m,phi)

      implicit none

      real(dop)                                       :: r
      real(dop), intent(in)                           :: phi
      real(dop), dimension(maxdim,maxdim), intent(out):: m
      integer(long)                                   :: i,j
!      integer(long), external                         :: signum

      r=1.0
      do i=1,ncoo
         do j=i,ncoo
            if (i.eq.j) then
               m(i,j)=1.0
            else
               m(i,j)=0.0
               m(j,i)=0.0
            endif
         enddo
      enddo
      print*,'Phi : ',phi
      do i=1,ncoo,3
         if (rottype(i).eq.'x') then
            m(i+1,i+1)=r*cos(phi)
            m(i+2,i+2)=m(i+1,i+1)
            m(i+1,i+2)=r*sin(phi*signum(xcoo0(i)))
            m(i+2,i+1)=-m(i+1,i+2)
          else if (rottype(i).eq.'y') then
            m(i,i)=r*cos(phi)
            m(i+2,i+2)=m(i,i)
            m(i,i+2)=r*sin(phi*signum(xcoo0(i+1)))
            m(i+2,i)=-m(i,i+2)
         else if (rottype(i).eq.'z') then
            m(i,i)=r*cos(phi)
            m(i+1,i+1)=m(i,i)
            m(i,i+1)=r*sin(phi*signum(xcoo0(i+2)))
            m(i+1,i)=-m(i,i+1)
         endif
      enddo

      return
      end subroutine getrot

!#######################################################################
!
! normal mode may be substituted by an internal coordinate
!
! xcoo is in Angstroem
!#######################################################################

      subroutine q2intcoo(xcoo,qcoo)

      implicit none

      real(dop), dimension(maxdim), intent(out) :: qcoo
      real(dop), dimension(3,natm), intent(in)  :: xcoo

      real(dop)       :: xdiff,ydiff,zdiff,xdiff1,ydiff1,zdiff1,norm,norm1,&
                         r,r0
      integer(long)   :: i,j,k,m

!----------------------------------------------------------------------
! cootyp=4 : mass- and intertia-weighted internal coordinates
!
! uses spherical polar coords theta, phi, r
! phi: angle between z-axis and atom1-atom2 bond in x,z-plane
! theta: angle between z-axis and atom1-atom2 bond in y,z-plane
! r: atom1-atom2 bond length
! k=1: Theta
! k=2: Phi
! k=3: R - R0
!
! TO DO: -Make this general and work for any choice of axis about
!         which to rotate
!---------------------------------------------------------------------- 

      do m=1,nmodes
         if (cootyp(m).eq.4) then
            i = coodef(1,m)
            j = coodef(2,m)
            k = coodef(3,m)
            xdiff=xcoo(1,j)-xcoo0(i*3-2)
            ydiff=xcoo(2,j)-xcoo0(i*3-1)
            zdiff=xcoo(3,j)-xcoo0(i*3)
            r=xdiff**2+ydiff**2+zdiff**2
            r=dsqrt(r)

            xdiff = xcoo0(j*3-2) - xcoo0(i*3-2)
            ydiff = xcoo0(j*3-1) - xcoo0(i*3-1)
            zdiff = xcoo0(j*3) - xcoo0(i*3)
            r0 = xdiff**2 + ydiff**2 + zdiff**2
            r0 = sqrt(r0)

            if (k.eq.1) then
               ydiff=xcoo(2,j)-xcoo(2,i)
               qcoo(m)=(ydiff)/r
!               qcoo(m)=(xcoo(2,j)/r)
               qcoo(m)=asin(qcoo(m))
               qcoo(m)=qcoo(m)*(180/pi)
!               print*,"theta:",qcoo(m)
            endif
            if (k.eq.2) then
               zdiff=xcoo(3,j)-xcoo(3,i)
               xdiff=xcoo(1,j)-xcoo(1,i)
               qcoo(m)=(xdiff/zdiff)
!               qcoo(m)=(xcoo(1,j)/zdiff)
               qcoo(m)=atan(qcoo(m))
               qcoo(m)=qcoo(m)*(180/pi)
!               print*,"phi:",qcoo(m)
            endif
            if (k.eq.3) then
               qcoo(m)=(r-r0)*sqrt(mass(j*3))/angstroem
               print*,sqrt(mass(j*3))
!               print*,"R:",qcoo(m)
            endif

         else if (cootyp(m) .eq. 3) then
            i = coodef(1,m)
            j = coodef(2,m)
            xdiff = xcoo(1,j) - xcoo(1,i)
            ydiff = xcoo(2,j) - xcoo(2,i)
            zdiff = xcoo(3,j) - xcoo(3,i)
            qcoo(m) = xdiff**2 + ydiff**2 + zdiff**2
            qcoo(m) = dsqrt(qcoo(m))/angstroem

!            xdiff = xcoo0(j*3-2) - xcoo0(i*3-2)
!            ydiff = xcoo0(j*3-1) - xcoo0(i*3-1)
!            zdiff = xcoo0(j*3) - xcoo0(i*3)
!            r0 = xdiff**2 + ydiff**2 + zdiff**2
!            r0 = sqrt(r0)/angstroem
!
!            qcoo(m)= qcoo(m)-r0
!
!            print*,"R:",qcoo(m)

         else if (cootyp(m) .eq. 2) then
            i = coodef(1,m)
            j = coodef(2,m)
            k = coodef(3,m)
            xdiff = xcoo(1,i) - xcoo(1,j)
            ydiff = xcoo(2,i) - xcoo(2,j)
            zdiff = xcoo(3,i) - xcoo(3,j)
            xdiff1 = xcoo(1,k) - xcoo(1,j)
            ydiff1 = xcoo(2,k) - xcoo(2,j)
            zdiff1 = xcoo(3,k) - xcoo(3,j)
            norm = xdiff**2 + ydiff**2 + zdiff**2
            norm = dsqrt(norm)
            norm1 = xdiff1**2 + ydiff1**2 + zdiff1**2
            norm1 = dsqrt(norm1)
            qcoo(m) = xdiff*xdiff1 + ydiff*ydiff1 + zdiff*zdiff1
            qcoo(m) = qcoo(m)/(norm*norm1)
            qcoo(m) = dacos(qcoo(m))

         else if (cootyp(m) .eq. 1) then
            print*,'q2internal'
            print*,'Dihedral not yet implemented'
         endif

         qcoo(m) = qcoo(m) - intcoo0(m)

      enddo

      return
      end subroutine q2intcoo


!#######################################################################

      subroutine q2intcoo_old(xcoo,qcoo)

      implicit none

!      real(dop), external                         :: dihed
      real(dop), dimension(maxdim)                :: q_0,q,xcoo_br
      real(dop), dimension(maxdim), intent(in)    :: xcoo
      real(dop), dimension(maxdim), intent(inout) :: qcoo
      integer(long)                               :: i,j,k

      do i=1,nmodes
!        q_0(i)=qcoo(i)
         q(i)=qcoo(i)
      enddo


!     take only the internal coordinates 
!     do i=1,nmodes
!        if ((cootyp(i).eq.0)
!    +        .and.(nmlab(i)(1:2).ne.'AG')) then
!           q(i)=0.0
!        else
!           q(i)=q_0(i)
!        endif
!     enddo

!      write(6,'(18F6.3)') (q(i),i=1,nmodes)

!     call q2x(xcoo,xcoo0,q,nmcoo,nmodes,ncoo,maxdim)

!      print*,'x-Geometrie : '
!      do i=1,ncoo,3
!         write(6,'(3F10.5)') (xcoo(j),j=i,i+2)
!      enddo
      do i=1,nmodes
!        if ((q_type(i).name).eq.'distance') then
!           q_0(i)=dist(xcoo,q_type(i).a,q_type(i).b)
!        else if ((q_type(i).name).eq.'angel') then
!           q_0(i)=angel(xcoo,q_type(i).a,q_type(i).b,q_type(i).c)
!        endif
         if (cootyp(i).eq.1) then
            q_0(i)=dihed(xcoo,coodef(1,i),ncoo)
         endif
      enddo

      do i=1,ncoo
         xcoo_br(i)=xcoo(i)
      enddo

! fix only for butatriene
      if (abs(q_0(5)) .gt. 0.001) then
         k=0
         do i=1,4
            if (i .eq. 1) k=2
            if (i .eq. 2) k=3
            if (i .eq. 3) k=7
            if (i .eq. 4) k=8
            do j=1,3
               xcoo_br(3*(k-1)+j) = xcoo0(3*(k-1)+j)
            enddo
         enddo
         call x2q(q,xcoo_br,xcoo0,coonm,nmodes,ncoo,maxdim)
         q(5)=q_0(5)
      else
         q(5)=q_0(5)
      endif
         
      do i=1,nmodes
!        if (q_type(i).name.eq.'distance') then
!            print*,'Distance : ',i,q_0(i)
!        else if (cootyp(i).eq.1) then
         if (cootyp(i).eq.1) then
             print*,'Angel    : ',i,q(i)*360.0/(2*PI)
!           alpha = -q_0(i)
         endif
      enddo            
    
!     call getrot(D,alpha)
!     do j=1,ncoo
!        xcoo_br(j)=0.0
!        do i=1,ncoo
!           xcoo_br(j)=xcoo_br(j)+D(j,i)*xcoo(i)
!        enddo
!     enddo
!      print*,'After back-rotation : '
!      do i=1,ncoo,3
!         write(6,'(3F10.5)') (xcoo_br(j),j=i,i+2)
!      enddo

!     call x2q(q,xcoo_br,xcoo0,coonm,nmodes,ncoo,maxdim)

!      write(6,'(18F6.3)') (q(i),i=1,nmodes)
!     do i=1,nmodes
!        if (nmlab(i)(1:2).eq.'AG') q_0(i)=q(i) 
!        if (i.ne.5) q_0(i)=0.0
!     enddo

      do i=1,nmodes
         qcoo(i)=q(i)
      enddo

      return
      end subroutine q2intcoo_old

!######################################################################
! calculates the distance between the point a and b
!######################################################################
      real(dop) function dist(xcoo,a,b)

      implicit none

      integer(long)                       :: i
      integer(long), intent(in)           :: a,b
      real(dop)                           :: r
      real(dop), dimension(*), intent(in) :: xcoo

      r=0.0
      do i=1,3
         r=r + (xcoo(3*(a-1)+i)-xcoo(3*(b-1)+i))**2
      enddo
      dist=sqrt(r)
      return
      end function dist


!######################################################################
! calcultes the angel between the vector(abc), b>0,c>0 
!        or the angel between  vector (a proj plane(bc)) ,b<0,c<0
!######################################################################

      real(dop) function angel(xcoo,a,b,c)

      implicit none

      integer(long)                       :: i
      integer(long), intent(in)           :: a,b,c
      real(dop)                           :: sc,norm1,norm2
      real(dop), dimension(3)             :: v1,v2
      real(dop), dimension(*), intent(in) :: xcoo

      do i=1,3
         v1(i)=0.0
         if (b.lt.0) then            
!        the projection from vector a to plane perp. to axis b
            if (b.ne.(-i)) v1(i) = xcoo(3*(a-1)+i)
         else
!        the vector (ab)
            v1(i) = xcoo(3*(b-1)+i)-xcoo(3*(a-1)+i)
         endif
         v2(i) = 0.0
         if (c.lt.0) then
!        the axis c
            if (i.eq.(-c)) v2(i)=1.0
         else
!        the vector(cb)
            v2(i)= xcoo(3*(b-1)+i)-xcoo(3*(c-1)+i)
         endif
      enddo
      sc=0.0
      norm1=0.0
      norm2=0.0
      do i=1,3
         sc=sc+v1(i)*v2(i)
         norm1=norm1+v1(i)*v1(i)
         norm2=norm2+v2(i)*v2(i)
      enddo
      norm1=sqrt(norm1)
      norm2=sqrt(norm2)
      angel = acos(sc/(norm1*norm2))

      return
      end function angel

!######################################################################
! calculates the angel between two vectors(abc)
!######################################################################

      real(dop) function angel1(xcoo,a,b,c)

      implicit none

      integer(long)                       :: i
      integer(long), intent(in)           :: a,b,c
      real(dop)                           :: sc,norm1,norm2
      real(dop), dimension(3)             :: v1,v2
      real(dop), dimension(*), intent(in) :: xcoo

! v1 is ba
! v2 is bc
      do i=1,3
         v1(i) = xcoo(3*(a-1)+i)-xcoo(3*(b-1)+i)
         v2(i)= xcoo(3*(c-1)+i)-xcoo(3*(b-1)+i)
      enddo
      sc=0.0
      norm1=0.0
      norm2=0.0
      do i=1,3
         sc=sc+v1(i)*v2(i)
         norm1=norm1+v1(i)*v1(i)
         norm2=norm2+v2(i)*v2(i)
      enddo
      norm1=dsqrt(norm1)
      norm2=dsqrt(norm2)
      angel1 = acos(sc/(norm1*norm2))

      return
      end function angel1

!######################################################################

      real(dop) function dihed(xcoo,coodef,ncoo)

      implicit none

      integer(long)                           :: i,j,k,l
      integer(long), intent(in)               :: ncoo
      integer(long), dimension(4), intent(in) :: coodef
      real(dop), dimension(ncoo), intent(in)  :: xcoo

      real(dop)       :: rccx,rccy,rccz,rch1x,rch1y,rch1z,rch2x,rch2y,rch2z
      real(dop)       :: cross1x,cross1y,cross1z,cross2x,cross2y,cross2z,norm1,norm2
      real(dop)       :: cross3x,cross3y,cross3z,sign

      i=(coodef(1)-1)*3 + 1
      j=(coodef(2)-1)*3 + 1
      k=(coodef(3)-1)*3 + 1
      l=(coodef(4)-1)*3 + 1

      rccx=xcoo(k) - xcoo(j)
      rccy=xcoo(k+1) - xcoo(j+1)
      rccz=xcoo(k+2) - xcoo(j+2)
      rch1x=xcoo(i) - xcoo(j)
      rch1y=xcoo(i+1) - xcoo(j+1)
      rch1z=xcoo(i+2) - xcoo(j+2)
      rch2x=xcoo(l) - xcoo(k)
      rch2y=xcoo(l+1) - xcoo(k+1)
      rch2z=xcoo(l+2) - xcoo(k+2)
 
! cross1 is plane ijk
! cross2 is plane jkl
      cross1x = rch1y*rccz-rch1z*rccy
      cross1y = rch1z*rccx-rch1x*rccz
      cross1z = rch1x*rccy-rch1y*rccx
      cross2x = rch2y*rccz-rch2z*rccy
      cross2y = rch2z*rccx-rch2x*rccz
      cross2z = rch2x*rccy-rch2y*rccx
      norm1= cross1x**2+cross1y**2+cross1z**2
      norm1=sqrt(norm1)
      cross1x=cross1x/norm1
      cross1y=cross1y/norm1
      cross1z=cross1z/norm1
      norm2= cross2x**2+cross2y**2+cross2z**2
      norm2=sqrt(norm2)
      cross2x=cross2x/norm2
      cross2y=cross2y/norm2
      cross2z=cross2z/norm2
 
      dihed=cross1x*cross2x+cross1y*cross2y+cross1z*cross2z
      dihed=acos(dihed)     

! get sign by direction of cross products of plane normals
! wrt to central bond
      cross3x = cross1y*cross2z-cross1z*cross2y
      cross3y = cross1z*cross2x-cross1x*cross2z
      cross3z = cross1x*cross2y-cross1y*cross2x
      sign = cross3x*rccx + cross3y*rccy + cross3z*rccz
      if (sign .lt. 0.0) dihed = -dihed

      return
      end function dihed
      
!###################################################################

      subroutine setintcoo

      implicit none

      routine='Setintcoo'
      message='Rotate option does not work - use the dihed option'
      call vcerrormsg
! ! only for butatriene
!     q_type(5).name='angel'
!     q_type(5).dim='radian'
!     q_type(5).a=2
!c     (x,y)-plane
!     q_type(5).b=-3
!c     y-axis
!     q_type(5).c=-2

      return
      end subroutine setintcoo
      

!###################################################################
      integer(long) function signum(x)

      implicit none

      real(dop), intent(in) :: x


      if (x.gt.0.0) then
         signum = 1
      else if (x.lt.0.0) then
         signum = -1
      else
         signum = 0
      endif
      return
      end function signum

!###################################################################
!
! Calculations the norm of a vector
!
!###################################################################

      real(dop) function vcnorm(q)

      implicit none
      
      real(dop)                           :: help
      real(dop), dimension(*), intent(in) :: q
      integer(long)                       :: n

      help = 0.0e0
      do n=1,nmodes
         help = help + q(n)**2
      enddo
      vcnorm = sqrt(help)
      
      return 
      end function vcnorm
      
!-----------------------------------------------------------------------
! remove leading blanks from a string
!-----------------------------------------------------------------------

      subroutine rmblnk0(string)
      implicit none

!      integer(long), external         :: len
      integer(long)                   :: ii,jj
      character(len=*), intent(inout) :: string

      ii = 1
      do while( string(ii:ii) .eq. ' ' )
         ii = ii + 1
      end do

      do jj = ii,len(string)
         string(jj-ii+1:jj-ii+1) = string(jj:jj)
      end do
      
      do jj = len(string)-ii,len(string)
         string(jj:jj) = ' '
      end do
      
      return
      end subroutine rmblnk0

!-----------------------------------------------------------------------
! returns index of first non-space character in a string
!-----------------------------------------------------------------------

      integer(long) function sstart(string)

      implicit none

      character(len=*), intent(in) :: string
      integer(long)                :: plength,i

      plength=len(string)
      do i=1,plength
         if (string(i:i).ne.' ') goto 10
      enddo
 10   sstart=i

      return
      end function sstart

!#######################################################################
!
! Writes an error message to the log file and to the screen.
! The calculation is then terminated.
!
! from errormsg in mctdh/errordat.F
!
!#######################################################################

      Subroutine vcerrormsg

      Implicit None


      integer(long), external :: slen
      integer(long)           :: routlength,messlength,messl1

! --- DETERMINE THE LENGTH OF THE ARGUMENTS ---

      routlength = slen(routine)
      messlength = slen(message)
      if(messlength .gt. 80) then
         messl1=index(message(70:messlength),' ')+69
         if(messl1.gt.80) messl1=index(message(60:88),' ')+59
      else
         messl1=0
      endif
! --- WRITE ERROR MESSAGE AND CLOSE FILES---

      if (logisopen) Then
         Write(ilog,*)
         Write(ilog,*) 'ERROR in subroutine ',&
                       routine(1:routlength),' :'
         if(messlength .gt. 80) then
            write(ilog,*) message(1:messl1-1)
            write(ilog,*) message(messl1+1:messlength)
         else
            write(ilog,*) message(1:messlength)
         endif
         write(ilog,*)
      endif
      write(6,*)
      write(6,*) 'ERROR in subroutine ', routine(1:routlength),' :'
      if(messlength .gt. 80) then
         write(6,*) message(1:messl1-1)
         write(6,*) message(messl1+1:messlength)
      else
         write(6,*) message(1:messlength)
      endif
      write(6,*)

      Stop
      End Subroutine vcerrormsg

!#######################################################################
!                       ~~~~~~~~~~~~~~~~~~~~~~~~~~
!
! Writes an error message to the log file and to the screen, but does
! not stop the calculation. Cf subroutine vcerrormsg
!
! The logical lwarn is set.
!
! from warnmsg in mctdh/errordat.F
!
!#######################################################################

      subroutine vcwarnmsg

      implicit None

      integer(long), external :: slen
      integer(long)           :: routlength,messlength,messl1,messlength1

      lwarn = .true.
      logisopen=.true.

! --- DETERMINE THE LENGTH OF THE ARGUMENTS ---

      routlength = slen(routine)
      messlength = slen(message)
      messlength1 = slen(message1)
      if(messlength .gt. 80) then
         messl1=index(message(70:messlength),' ')+69
         if(messl1.gt.80) messl1=index(message(60:88),' ')+59
      else
         messl1=0
      endif

! --- WRITE WARNING MESSAGE TO LOG FILE AND SCREEN ---

      if (logisopen) then
         write(ilog,*)
         write(ilog,*) 'ERROR in subroutine ',&
                       routine(1:routlength),' :'
         if(messlength .gt. 80) then
            write(ilog,*) message(1:messl1-1)
            write(ilog,*) message(messl1+1:messlength)
         else
            write(ilog,*) message(1:messlength)
         endif
         if(messlength1 .gt. 0) then
            write(ilog,*) message1
         endif
         call myflush(ilog)
      endif

      write(6,*)
      write(6,*) 'ERROR in subroutine ', routine(1:routlength),' :'
      if(messlength .gt. 80) then
         write(6,*) message(1:messl1-1)
         write(6,*) message(messl1+1:messlength)
      else
         write(6,*) message(1:messlength)
      endif
      if(messlength1 .gt. 0) then
         write(6,*) message1
      endif

      return
      end subroutine vcwarnmsg

!***********************************************************************
!
!                 VCWRLINE
!
! Writes some additional information to screen and to the 'log' file.
! Helps finding wrong input parameters in the ~.inp file.
!
! i  : no. of keyword associated with error message (input)
! iz : no. of line in input file containing error   (input)
! iz0: no. of last error containing line in input file (prevents
!      repeated messages) (input/output)
!
!   taken from mctdh/einglib.F
!***********************************************************************

      subroutine vcwrline (i,iz,iz0)

      implicit none

      integer(long)                :: ilbl
      integer(long), intent(in)    :: i,iz
      integer(long), intent(inout) :: iz0

      if (iz.ne.iz0) write(6,900) iz, inptit
      iz0     = iz
      ilbl=index(message,':')+1
      if (i.gt.0) message=message(1:ilbl)//keyorig(i)
      call vcwarnmsg

 900  format(/,' Input Error in line no.',i4,' The wrong line reads :',&
             /,  a80 )

      return
      end subroutine vcwrline

!***********************************************************************
!
!            Subroutine   VCCNTRL
!
! Contolls whether the input read via rdinpf is correct.
! Calls ERRORMSG.
! 
! taken from cntrl(mctdh/eingutils.F)
!
!***********************************************************************

      subroutine vccntrl(ierr)

      implicit None

      integer(long), intent(in) :: ierr

      if (ierr .gt. 0) then
         message=inptit
         write(6,'(a)') 'rdinpf returned ierr=1'
         if (message(1:31) .eq. '*****  End of file found! *****') then
            call vcwarnmsg
            message = 'Missing end-input or end-keyword ? '
            call vcerrormsg
         else
            call vcerrormsg
         endif
      endif

      return
      end subroutine vccntrl


!#######################################################################
!  Rotates the molucule by angle theta about the x axis
!  Used in the dihedral pnts generation as it simplifies the creation
!  of the xyz coords from the dihedral angle by placing the
!  BC bond parallel to the Z axis
!  Performs a rotation about the x axis using a rotation matrix
!#######################################################################

      subroutine rotatex(thetax,xcoo,xcoot)

      implicit none

      real(dop)           :: costheta,sintheta
!      real*8 tmp1,tmp2,tmp3,tmp4
      integer(long)       :: i,j,x1,y1,z1
      real(dop), intent(in)                    :: thetax
      real(dop), dimension(maxdim), intent(in)  :: xcoo
      real(dop), dimension(maxdim), intent(out) :: xcoot

      j=ncoo/3
      do i=1,j
         x1=(i-1)*3+1
         y1=(i-1)*3+2
         z1=(i-1)*3+3

         costheta=cos(thetax)
         sintheta=sin(thetax)
         xcoot(x1) = xcoo(x1)
         xcoot(y1) = (xcoo(y1)*costheta)+(xcoo(z1)*sintheta)
         xcoot(z1) = (-(xcoo(y1)*sintheta))+(xcoo(z1)*costheta)
      enddo

      return
      end subroutine rotatex

!#######################################################################
!  Rotates the molucule by angle theta about the y axis
!  Used in the dihedral pnts generation as it simplifies the creation
!  of the xyz coords from the dihedral angle by placing the
!  BC bond parallel to the Z axis
!  Performs a rotation about the y axis using a rotation matrix
!#######################################################################
 
      subroutine rotatey(thetay,xcoo,xcoot)

      implicit none
 
      real(dop)           :: sintheta,costheta
      integer(long)       :: i,j,x1,y1,z1
      real(dop), intent(in)                    :: thetay
      real(dop), dimension(maxdim), intent(in)  :: xcoo
      real(dop), dimension(maxdim), intent(out) :: xcoot

      j=ncoo/3
      do i=1,j
         x1=(i-1)*3+1
         y1=(i-1)*3+2
         z1=(i-1)*3+3
 
         costheta=cos(thetay)
         sintheta=sin(thetay)
         xcoot(x1) = (-(xcoo(z1)*sintheta))+(xcoo(x1)*costheta)
         xcoot(y1) = xcoo(y1)
         xcoot(z1) = (xcoo(x1)*sintheta)+(xcoo(z1)*costheta)
      enddo
 
      return
      end subroutine rotatey

!#######################################################################
!  Rotates the molucule by angle theta about the z axis
!  Performs a rotation about the z axis using a rotation matrix
!#######################################################################
 
      subroutine rotatez(thetaz,xcoo,xcoot)

      implicit none
 
      real(dop)           :: costheta,sintheta
      integer(long)       :: i,j,x1,y1,z1
      real(dop), intent(in)                     :: thetaz
      real(dop), dimension(maxdim), intent(in)  :: xcoo
      real(dop), dimension(maxdim), intent(out) :: xcoot

      j=ncoo/3
      do i=1,j
         x1=(i-1)*3+1
         y1=(i-1)*3+2
         z1=(i-1)*3+3
 
         costheta=cos(thetaz)
         sintheta=sin(thetaz)
         xcoot(x1) = (xcoo(x1)*costheta)+(xcoo(y1)*sintheta)
         xcoot(y1) = (-(xcoo(x1)*sintheta))+(xcoo(y1)*costheta)
         xcoot(z1) = xcoo(z1)
      enddo
 
      return
      end subroutine rotatez

!##############################################################
 
      subroutine savemask

      implicit none
 
!     real*8        optcoo(optcoodim)
!     complex*16    mc(mcdim)
!     real*8        mr(mrdim)
!     integer       mi(midim)
!     logical       ml(mldim)
 
      integer(long)       :: i,j,s,s1
!     integer npopgen,temp,c
!     character*7 filename
 
          do i=1,nmodes
 
!            Frequencies
 
          enddo
          do i=1,nsta
 
!            Energies
 
          enddo
          do s=1,nsta
             do i=1,nmodes
 
!               Kappa
 
             enddo
          enddo
          do s=1,nsta-1
             do s1=s+1,nsta
                do i=1,nmodes
 
!                  lambda
 
                enddo
             enddo
          enddo
       do s=1,nsta
          do i=1,nmodes
             do j=1,nmodes
 
!               Gamma
          
             enddo
          enddo 
        enddo 
       do s=1,nsta-1
          do s1=s+1,nsta
             do i=1,nmodes
                do j=1,nmodes
    
!                  Mu
    
                enddo
             enddo
          enddo
       enddo
    
       return
       end subroutine savemask

!##############################################################
 
      subroutine move_constants

      implicit none
 
!     real*8        optcoo(optcoodim)
!     complex*16    mc(mcdim)
!     real*8        mr(mrdim)
!     integer       mi(midim)
!     logical       ml(mldim)
 
      integer(long)       :: i,j,s,s1
!     integer npopgen,temp,c
!     character*7 filename
 
          do i=1,nmodes
 
!            Frequencies
 
          enddo
          do i=1,nsta
 
!            Energies
 
          enddo
          do s=1,nsta
             do i=1,nmodes
 
!               Kappa
 
             enddo
          enddo
          do s=1,nsta-1
             do s1=s+1,nsta
                do i=1,nmodes
 
!                  lambda
 
                enddo
             enddo
          enddo
       do s=1,nsta
          do i=1,nmodes
             do j=1,nmodes
 
!               Gamma
 
             enddo
          enddo
        enddo
       do s=1,nsta-1
          do s1=s+1,nsta
             do i=1,nmodes
                do j=1,nmodes
 
!                  Mu
 
                enddo
             enddo
          enddo
       enddo
 
       return
       end subroutine move_constants

!##############################################################

      subroutine comcalc

      implicit none

      real(dop) :: tmass
      integer(long) :: i,k

      tmass=0.0
      do k=1,ncoo/3
         tmass=tmass+mass(k*3)
      enddo

      do i=1,3
         xcom0(i)=0.0
         do k=1,natm
            xcom0(i)=xcom0(i)+xcoo0(k*3-(3-i))*mass(k*3-(3-i))/tmass
         enddo
      enddo

      return
      end subroutine comcalc

!#######################################################################

      real(dop) function vcrmsd(optcoo,mr,mi)

      implicit none

      integer(long), dimension(midim), intent(inout) :: mi
      real(dop), dimension(mrdim), intent(inout)     :: mr
      integer(long)                                  :: n,i
      real(dop), dimension(optcoodim), intent(inout) :: optcoo
      real(dop), dimension(maxdim)                :: qcoo
      real(dop)                                   :: diff1,diff2

!-----------------------------------------------------------------------
! extract parameters from "coordinate" vector
!-----------------------------------------------------------------------
      if (lgenetica .and. .not. lusecons .and. .not. gaoptim ) then
         goto 32
      elseif((lgenetica .and. lusecons .and. .not. gaoptim) .or. gaoptim2 ) then
      call par2vec2d(optcoo,mr(mrconst),mr(mrkappa),mr(mrlambda),&
           mr(mrgamma),mr(mriota),mr(mrmu),mr(mrexpcoe),&
           mr(mrhigher),mr(mrtermk),mr(mrcpcoe),&
           mr(mrswitch),mr(mrcp2d),mi(miconstmask),&
           mi(mikappamask),mi(milambdamask),mi(migammamask),&
           mi(miiotamask),mi(mimumask),mi(miexpcoemask),&
           mi(mitermkmask),mi(micpcoemask),&
           mi(miswitchmask),mi(micp2d))
           if(gaoptim2)  goto 32
      endif
      call vec2par2d(optcoo,mr(mrconst),mr(mrkappa),mr(mrlambda),&
           mr(mrgamma),mr(mriota),mr(mrmu),mr(mrexpcoe),&
           mr(mrhigher),mr(mrtermk),mr(mrcpcoe),&
           mr(mrswitch),mr(mrcp2d),mi(miconstmask),&
           mi(mikappamask),mi(milambdamask),mi(migammamask),&
           mi(miiotamask),mi(mimumask),mi(miexpcoemask),&
           mi(mitermkmask),mi(micpcoemask),&
           mi(miswitchmask),mi(micp2d))

!-----------------------------------------------------------------------
! calculate model energies
!-----------------------------------------------------------------------
 32   continue
      qcoo = 0d0
      do n=1,nsets
         call getqcoo(mr(mrqcoo),qcoo,nmodes,nsets,n)
         currsta = ista(n)
         denergy(n)=vpes(qcoo,mr,mi)
      enddo

!-----------------------------------------------------------------------
! calculate RMSD (actually only squared difference)
! Cap difference at 5eV or else outliers can dominate error
!-----------------------------------------------------------------------
      diff2=0.0
      do n=1,nsets
         diff1=mr(mrenergy+n-1)-denergy(n)
         diff1=min(abs(diff1),5.0)
         diff2=diff2+w0(n)*diff1*diff1/dble(nsets)
         !diff2=diff2+w0(n)*diff1*diff1
      enddo

      vcrmsd=diff2

      return
      end function vcrmsd

!#######################################################################

      subroutine getqcoo(qcoo,qcoo1,nmodes,nsets,n)

      implicit none

      integer(long)                                  :: i
      integer(long), intent(in)                      :: nmodes,nsets,n
      real(dop), dimension(nmodes), intent(out)      :: qcoo1
      real(dop), dimension(nmodes,nsets), intent(in) :: qcoo


      do i=1,nmodes
         qcoo1(i) = qcoo(i,n)
      enddo

      return
      end subroutine getqcoo

!#######################################################################

      subroutine dvcrmsd(q,dq,mr,mi)

      implicit none

      integer(long)                                 :: n
      integer(long), dimension(*), intent(in)       :: mi
      real(dop), dimension(*), intent(in)           :: q
      real(dop), dimension(*), intent(in)           :: mr
      real(dop), dimension(*), intent(out)          :: dq
      real(dop)                                     :: err,h
      real(dop), dimension(4*maxdim*maxdim*2)       :: work
!      real(dop), external                           :: vcrmsd,dfridr

      do n=1,nvar
         h=0.001
         dq(n)=dfridr(vcrmsd,q,nvar,n,h,err,work,mr,mi)
      enddo

      return
      end subroutine dvcrmsd

!#######################################################################

      subroutine getlabel(n,string)
        
        implicit none
        
        integer(long), intent(in)      :: n
        character(len=8), intent(out) :: string
        
        string='     '
        if (n.le.9) then
           write(string(1:1),'(i1)') n
        else if (n.le.99) then
           write(string(1:2),'(i2)') n
        else if (n.le.999) then
           write(string(1:3),'(i3)') n
        else if (n.le.9999) then
           write(string(1:4),'(i4)') n
        else if (n.le.99999) then
           write(string(1:5),'(i5)') n
        else if (n.le.999999) then
           write(string(1:6),'(i6)') n
        else if (n.le.9999999) then
           write(string(1:7),'(i7)') n
        else if (n.le.99999999) then
           write(string(1:8),'(i8)') n
        endif
        
        return
      end subroutine getlabel
      

!#######################################################################

      subroutine dattrans(filter,n)

      implicit none

      integer(long) :: i,j,k,l,tag,junk,count,ifiletyp,ierr,unit,ilbl,iau
      integer(long), intent(out) :: n,filter
      real(dop), dimension(ncoo) :: nxcoo,nxcoo0,tmpxcoo,tmp,work
      real(dop), dimension(nmodes) :: nqcoo
      real(dop), dimension(maxdim) :: nmass
      real(dop) :: theta0,phi0,r0,rcom,r,xdiff,ydiff,zdiff
      real(dop) :: small,small2,transx,transy,transz,ang1,ang2,ang3,&
                   rat,theta,phi,stheta,ctheta
      character(len=80) :: filename

      logical(kind=4) :: negx,negz 

      real(dop) :: phidiff

      real(dop) :: comx,comy,comz

      small=0.09
      small2=0.000001
      
      tag=sysatom

      ang1=0.0
      ang2=0.0
      ang3=0.0

!-----------------------------------------------------------------------
! Calculate reference R, theta, phi
!-----------------------------------------------------------------------
      call translate0(tmpxcoo,xcoo0,transx,transy,transz)
      call rotate1(tmpxcoo,ang1,ang2,ang3,negx,negz)

      xdiff=tmpxcoo(ratm1*3-2)-tmpxcoo(ratm2*3-2)
      ydiff=tmpxcoo(ratm1*3-1)-tmpxcoo(ratm2*3-1)
      zdiff=tmpxcoo(ratm1*3)-tmpxcoo(ratm2*3)

      r0=xdiff**2+ydiff**2+zdiff**2
      r0=sqrt(r0)

      phi0=tmpxcoo(ratm1*3)/r0
      phi0=acos(phi0)

      theta0=tmpxcoo(ratm1*3-1)/tmpxcoo(ratm1*3)
      theta0=atan(theta0)

!-----------------------------------------------------------------------
! Calculate current R, theta, phi
!-----------------------------------------------------------------------
!      call translate0(tmp,xcoo,transx,transy,transz)
!      call rotate1(tmp,ang1,ang2,ang3,negx,negz)

      do i=1,ncoo
         tmp(i)=xcoo(i)
      enddo

! Rotate around y-axis by ang1
      do i=1,ncoo
         work(i)=0
      enddo

      do i=1,ncoo/3
         j=i*3-2
         k=i*3-1
         l=i*3
         tmp(j)=xcoo(j)-transx
         tmp(k)=xcoo(k)-transy
         tmp(l)=xcoo(l)-transz
      enddo

      if (abs(ang1).gt.small2) then         
         stheta=sin(ang1)
         ctheta=cos(ang1)
         
         do i=1,ncoo/3
            j=i*3-2
            k=i*3-1
            l=i*3
            work(j) = tmp(j)*ctheta + tmp(l)*stheta
            work(k) = tmp(k)
            work(l) = -tmp(j)*stheta + tmp(l)*ctheta
         enddo

         do i=1,ncoo
            tmp(i)=work(i)
         enddo
         
         do i=1,ncoo
            work(i)=0
         enddo
      endif

! Rotate around x-axis by ang2
      if (abs(ang2).gt.small2) then
         stheta=sin(ang2)
         ctheta=cos(ang2)
         
         do i=1,ncoo/3
            j=i*3-2
            k=i*3-1
            l=i*3
            work(j) = tmp(j)
            work(k) = tmp(k)*ctheta - tmp(l)*stheta
            work(l) = tmp(k)*stheta + tmp(l)*ctheta
         enddo

         do i=1,ncoo
            tmp(i)=work(i)
         enddo
      
         do i=1,ncoo
            work(i)=0
         enddo
      endif

! Rotate around z-axis by ang3
      if (abs(ang3).gt.small2) then
         stheta=sin(ang3)
         ctheta=cos(ang3)
      
         do i=1,ncoo/3
            j=i*3-2
            k=i*3-1
            l=i*3
            work(j) = tmp(j)*ctheta - tmp(k)*stheta
            work(k) = tmp(j)*stheta + tmp(k)*ctheta
            work(l) = tmp(l)
         enddo
         
         do i=1,ncoo
            tmp(i)=work(i)
         enddo

      endif
         
      tmpxcoo(tag*3-2)=tmp(tag*3-2)
      tmpxcoo(tag*3-1)=tmp(tag*3-1)
      tmpxcoo(tag*3)=tmp(tag*3)
      
      xdiff=tmpxcoo(ratm1*3-2)-tmpxcoo(ratm2*3-2)
      ydiff=tmpxcoo(ratm1*3-1)-tmpxcoo(ratm2*3-1)
      zdiff=tmpxcoo(ratm1*3)-tmpxcoo(ratm2*3)
      
      r=xdiff**2+ydiff**2+zdiff**2
      r=sqrt(r)
      
      phi=tmpxcoo(ratm1*3-2)/r
      phi=asin(phi)
      
      if (negx.eqv..true.) phi=-phi
      
      if (zdiff.lt.0.0) then         
         phidiff=((pi/2)-abs(phi))
         if (phi.lt.0.0) then
            phi=phi-2*phidiff
         else
            phi=phi+2*phidiff
         endif
      endif

      theta=tmpxcoo(ratm1*3-1)/tmpxcoo(ratm1*3)
      theta=atan(theta)

      nqcoo(nmodes-2)=(r-r0)/angstroem
      nqcoo(nmodes-1)=theta-theta0
      nqcoo(nmodes)=phi-phi0

!---------------------------------------------------------------------
! read elements of fragment hessian eigenvector matrix
!---------------------------------------------------------------------
      filename=fragmentfile
      unit=55
      open(unit,file=filename,form='formatted',status='old',err=1050)
      call getintyp(unit,ifiletyp,filename,ilog)

      fragncoo=ncoo-3
      iau=1
      call getnm(unit,ifiletyp,fragnmlab,fragnmcoo,fragfreq0,iatsym,iau,&
           maxdim,fragncoo,nsta,fragnmodes,ilog,1,.true.)
      call nm2xmat(fragnmcoo,fragcoonm,iatsym,fragfreq0,iau,fragncoo,maxdim,&
           fragnmodes,ilog)

      close(unit)

!---------------------------------------------------------------------
! calculate point in fragment Q
!---------------------------------------------------------------------
      call rexcoo(nxcoo,nxcoo0,nmass)
      
      do i=1,nmodes-3
         nqcoo(i)=0.0
         do k=1,ncoo-3
            nqcoo(i)=nqcoo(i)+fragcoonm(i,k)*(nxcoo(k)-nxcoo0(k))
         enddo
      enddo

      if (n.gt.nsta) then
         if (lfilter.eqv..true.) then
            if ((nqcoo(nmodes-2).gt.small)) then
               filter=1
            else
               n=n-1
            endif
         else
            filter=1
         endif
      else
         filter=1
      endif
         
      do i=1,nmodes
         qcoo(i)=nqcoo(i)
      enddo

 1050 continue

      return
 1040 routine='dattrans (utils.f90)'
      ilbl=index(filename,' ')-1
      write(message,'(2a)') 'Nmodes not found in: ',filename(1:ilbl)
      call errormsg

      end subroutine dattrans


!#######################################################################

      subroutine translate0(tmpxcoo,tmp,transx,transy,transz)

      implicit none

      integer(long) :: i,j,k,l
      real(dop), dimension(ncoo), intent(out) :: tmpxcoo,tmp
      real(dop), intent(out) :: transx,transy,transz

      transx=tmp(ratm2*3-2)
      transy=tmp(ratm2*3-1)
      transz=tmp(ratm2*3)

      do i=1,ncoo/3
         j=i*3-2
         k=i*3-1
         l=i*3
         tmpxcoo(j)=tmp(j)-transx
         tmpxcoo(k)=tmp(k)-transy
         tmpxcoo(l)=tmp(l)-transz
      enddo


      return
      end subroutine translate0

!#######################################################################

      subroutine rotate1(tmpxcoo,ang1,ang2,ang3,negx,negz)

      implicit none

      integer(long) :: i,j,k,l
      real(dop), dimension(ncoo), intent(out) :: tmpxcoo
      real(dop), intent(out) :: ang1,ang2,ang3
      real(dop), dimension(ncoo):: work
      real(dop) :: stheta,ctheta
      real(dop), dimension(3) :: tmp,mid
      logical(kind=4), intent(out) :: negx,negz

      negx=.false.
      negz=.false.

!-----------------------------------------------------------------------
! Step (i): rotation about y-axis such that mid(x)=0
!-----------------------------------------------------------------------
! Calculate midpoint between ratm3 and ratm4
      do i=1,3
         tmp(i)=tmpxcoo(ratm4*3-3+i)-tmpxcoo(ratm3*3-3+i)
         mid(i)=tmpxcoo(ratm3*3-3+i)+tmp(i)/2
      enddo

      if (mid(1).ne.0) then

! Calculate angle between midpoint and z-axis
         ang1=mid(1)/mid(3)
         ang1=atan(ang1)

! Ensure that we rotate in the correct direction
         if (ang1.gt.0) ang1=-ang1
         if (mid(3).lt.0) then
            if (mid(1).gt.0) ang1=-ang1
         else if (mid(3).gt.0) then
            if (mid(1).lt.0) ang1=-ang1
         endif
      
         stheta=sin(ang1)
         ctheta=cos(ang1)

! Rotate around y-axis by ang1
         do i=1,ncoo
            work(i)=0
         enddo

         do i=1,ncoo/3
            j=i*3-2
            k=i*3-1
            l=i*3
            work(j) = tmpxcoo(j)*ctheta + tmpxcoo(l)*stheta
            work(k) = tmpxcoo(k)
            work(l) = -tmpxcoo(j)*stheta + tmpxcoo(l)*ctheta
         enddo

         do i=1,ncoo
            tmpxcoo(i)=work(i)
         enddo
         
      endif
      
      do i=1,ncoo
         work(i)=0
      enddo

!-----------------------------------------------------------------------
! Step (ii): rotation about x-axis such that mid(y)=0
!-----------------------------------------------------------------------
! Calculate midpoint between ratm3 and ratm4
      do i=1,3
         tmp(i)=tmpxcoo(ratm4*3-3+i)-tmpxcoo(ratm3*3-3+i)
         mid(i)=tmpxcoo(ratm3*3-3+i)+tmp(i)/2
      enddo

      if (mid(2).ne.0) then

! Calculate angle between midpoint and z-axis
         ang2=mid(2)/mid(3)
         ang2=atan(ang2)
      
! Ensure that we rotate in the correct direction
         if (ang2.lt.0) ang2=-ang2
         if (mid(2).lt.0) then
            if (mid(3).gt.0) ang2=-ang2
         else if (mid(2).gt.0) then
            if (mid(3).lt.0) ang2=-ang2
         endif
         
         stheta=sin(ang2)
         ctheta=cos(ang2)

! Rotate around x-axis by ang2
         do i=1,ncoo/3
            j=i*3-2
            k=i*3-1
            l=i*3
            work(j) = tmpxcoo(j)
            work(k) = tmpxcoo(k)*ctheta - tmpxcoo(l)*stheta
            work(l) = tmpxcoo(k)*stheta + tmpxcoo(l)*ctheta
         enddo

         do i=1,ncoo
            tmpxcoo(i)=work(i)
         enddo
         
      endif

      do i=1,ncoo
         work(i)=0
      enddo

!-----------------------------------------------------------------------
! Step (iii): rotation about z-axis such that ratm1(y)=0
!-----------------------------------------------------------------------
      if (tmpxcoo(ratm1*3-1).eq.0) then
         ang3=0
      else
! Calculate angle between ratm1 and x-axis      
         ang3=tmpxcoo(ratm1*3-1)/tmpxcoo(ratm1*3-2)
         ang3=atan(ang3)

! Ensure that we rotate in the correct direction
         if (ang3.gt.0) ang3=-ang3
         if (tmpxcoo(ratm1*3-1).lt.0) then
            if (tmpxcoo(ratm1*3-2).gt.0) ang3=-ang3
         else if (tmpxcoo(ratm1*3-1).gt.0) then
            if (tmpxcoo(ratm1*3-2).lt.0) ang3=-ang3
         endif

         stheta=sin(ang3)
         ctheta=cos(ang3)      

! Rotate around z-axis by ang3
         do i=1,ncoo/3
            j=i*3-2
            k=i*3-1
            l=i*3
            work(j) = tmpxcoo(j)*ctheta - tmpxcoo(k)*stheta
            work(k) = tmpxcoo(j)*stheta + tmpxcoo(k)*ctheta
            work(l) = tmpxcoo(l)
         enddo

         do i=1,ncoo
            tmpxcoo(i)=work(i)
         enddo

      endif

! Ensure that ratm1 has +ve x- and z-components
      if (tmpxcoo(ratm1*3-2).lt.0) then
         negx=.true.
         do i=1,ncoo/3
            tmpxcoo(i*3-2)=-tmpxcoo(i*3-2)
         enddo
      endif

      if (tmpxcoo(ratm1*3).lt.0) then
         negz=.true.
         do i=1,ncoo/3
            tmpxcoo(i*3)=-tmpxcoo(i*3)
         enddo
      endif

      return
      end subroutine rotate1

!#######################################################################
! Rearranges xcoo so that system atom corresponds to last three elements
!#######################################################################
      subroutine rexcoo(nxcoo,nxcoo0,nmass)

      implicit none

      integer(long) :: i,j,k,tag,junk,count
      real(dop), dimension(maxdim),intent(out) :: nmass
      real(dop), dimension(maxdim),intent(out) :: nxcoo,nxcoo0

      tag=sysatom

      do i=1,ncoo
         nxcoo(i)=0.0
         nxcoo0(i)=0.0
      enddo
      do i=1,(tag*3)-3
         nxcoo(i)=xcoo(i)
         nxcoo0(i)=xcoo0(i)
         nmass(i)=mass(i)
      enddo
      count=0
      do i=ncoo-2,ncoo
         count=count+1
         junk=(tag*3)-3+count
         nxcoo(i)=xcoo(junk)
         nxcoo0(i)=xcoo0(junk)
         nmass(i)=mass(junk)
      enddo
      do i=(tag*3)-2,ncoo-3
         junk=i+3
         nxcoo(i)=xcoo(junk)
         nxcoo0(i)=xcoo0(junk)
         nmass(i)=mass(junk)
      enddo

      nxcoo(ncoo-2)=xcoo(tag*3-2)
      nxcoo(ncoo-1)=xcoo(tag*3-1)
      nxcoo(ncoo)=xcoo(tag*3)

      nmass(ncoo-2)=mass(tag*3-2)
      nmass(ncoo-1)=mass(tag*3-1)
      nmass(ncoo)=mass(tag*3)

!      do i=1,ncoo,3
!         write(6,'(3F15.10)') (nxcoo(j),j=i,i+2)
!      enddo

      return
      end subroutine rexcoo


!#######################################################################
!
! the set of coordinates is on a cut along ncut or ncut,ncut1 if
! all other coordinates are 0.0
!
!#######################################################################

      subroutine chkcoocut(qcoo,ncut,ncut2,nmodes,ismode,xscl,negdiag)

      implicit none

      integer(long)                            :: m
      integer(long), intent(in)                :: ncut,ncut2,nmodes
      real(dop), dimension(nmodes), intent(in) :: qcoo
      real(dop), intent(in)                    :: xscl
      logical(kind=4), intent(inout)           :: ismode
      logical(kind=4), intent(in)              :: negdiag

      do m=1,nmodes
         if (m.ne.ncut .and. m.ne.ncut2)then
            ismode=(ismode.and.(abs(qcoo(m) - q0val(m)).le.cuttol))
         end if
      enddo

      if (negdiag) then
         ismode=(ismode.and.(abs(xscl*qcoo(ncut)+qcoo(ncut2)).le.cuttol))
      else
         ismode=(ismode.and.(abs(xscl*qcoo(ncut)-qcoo(ncut2)).le.cuttol))
      endif 

      return
      end subroutine chkcoocut


!#######################################################################
!
! the set of coordinates is on a vector in normal mode space, checking 
! that the point lies on this vector
! S Neville 18/04/11
! based on routine by TPenfold 29/1/10 which had all other q = 0
!
!#######################################################################

      subroutine chkcoovec(qcoo,nmodes,ismode,atvmd,qvp,qvpini)     

      implicit none

      integer(long), intent(in)                      :: nmodes
      integer(long)                                  :: m
      real(dop), dimension(nmodes), intent(in)       :: qcoo,qvp,qvpini
      real(dop), dimension(nmodes)                   :: vectrue, vectest
      real(dop)                                      :: theta,lentrue,&
                                                        lentest
      real(dop), parameter                           :: tol=0.02
      logical(kind=4), intent(out)                   :: ismode
      logical(kind=4), dimension(nmodes), intent(in) :: atvmd

!-----------------------------------------------------------------------
! Calculate the angle between the vector pointing from qvpini to qvp
! and the vector pointing from qvpini to qcoo.
!
! If this angle is greater than the tolerance, take the point not to lie
! on the vector.
!-----------------------------------------------------------------------
      theta=0
      lentrue=0
      lentest=0

      do m=1,nmodes         
         vectrue(m)=qvp(m)-qvpini(m)
         vectest(m)=qcoo(m)-qvpini(m)
         lentrue=lentrue+vectrue(m)*vectrue(m)
         lentest=lentest+vectest(m)*vectest(m)
      enddo
      lentrue=sqrt(lentrue)
      lentest=sqrt(lentest)

      do m=1,nmodes
         vectrue(m)=vectrue(m)/lentrue
         vectest(m)=vectest(m)/lentest
         theta=theta+vectrue(m)*vectest(m)
      enddo
      theta=acos(theta)


      if (theta.gt.tol) ismode=.false.
      end subroutine chkcoovec

!#######################################################################

      subroutine par2vec_lm(s,s1,cmpnt,cmpnt1,optcoo,c,dipder1,dipder2,&
           dipder3,dipder4,dipder1_mask,dipder2_mask,dipder3_mask,&
           dipder4_mask,&
           polarder1,polarder2,polarder3,polarder4,polarder1_mask,&
           polarder2_mask,polarder3_mask,polarder4_mask,diprmsd,polarrmsd)

        implicit none

        real(dop), dimension(*), intent(out) :: optcoo
        
        integer(long) :: s,s1,cmpnt,cmpnt1,m,n,c

!        integer(long) :: j,k,s,s1,m,n,c
      
        integer(long), dimension(nsta,nsta,3,nmodes)        :: dipder1_mask
        integer(long), dimension(nsta,nsta,3,nmodes,nmodes) :: dipder2_mask
        integer(long), dimension(nsta,nsta,3,nmodes)        :: dipder3_mask
        integer(long), dimension(nsta,nsta,3,nmodes)        :: dipder4_mask
        
        real(dop), dimension(nsta,nsta,3,nmodes)        :: dipder1
        real(dop), dimension(nsta,nsta,3,nmodes,nmodes) :: dipder2
        real(dop), dimension(nsta,nsta,3,nmodes)        :: dipder3
        real(dop), dimension(nsta,nsta,3,nmodes)        :: dipder4


        integer(long), dimension(nsta,nsta,3,3,nmodes)        :: polarder1_mask
        integer(long), dimension(nsta,nsta,3,3,nmodes,nmodes) :: polarder2_mask
        integer(long), dimension(nsta,nsta,3,3,nmodes)        :: polarder3_mask
        integer(long), dimension(nsta,nsta,3,3,nmodes)        :: polarder4_mask
        
        real(dop), dimension(nsta,nsta,3,3,nmodes)        :: polarder1
        real(dop), dimension(nsta,nsta,3,3,nmodes,nmodes) :: polarder2
        real(dop), dimension(nsta,nsta,3,3,nmodes)        :: polarder3
        real(dop), dimension(nsta,nsta,3,3,nmodes)        :: polarder4
        
        logical(kind=4) :: diprmsd,polarrmsd

        c=0

        if (diprmsd) then
! First derivatives wrt Q
           do m=1,nmodes
              if (dipder1_mask(s,s1,cmpnt,m).eq.1) then
                 c=c+1
                 optcoo(c)=dipder1(s,s1,cmpnt,m)
              endif
           enddo
        
! Second derivatives wrt Q
           do m=1,nmodes
              do n=m,nmodes
                 if (dipder2_mask(s,s1,cmpnt,m,n).eq.1) then
                    c=c+1
                    optcoo(c)=dipder2(s,s1,cmpnt,m,n)
                 endif
              enddo
           enddo
        
! Third derivatives wrt Q (cubic terms only)
           do m=1,nmodes
              if (dipder3_mask(s,s1,cmpnt,m).eq.1) then
                 c=c+1
                 optcoo(c)=dipder3(s,s1,cmpnt,m)
              endif
           enddo

! Fourth derivatives wrt Q (quartic terms only)
           do m=1,nmodes
              if (dipder4_mask(s,s1,cmpnt,m).eq.1) then
                 c=c+1
                 optcoo(c)=dipder4(s,s1,cmpnt,m)
              endif
           enddo
        endif

        if (polarrmsd) then
! First derivatives wrt Q
           do m=1,nmodes
              if (polarder1_mask(s,s1,cmpnt,cmpnt1,m).eq.1) then
                 c=c+1
                 optcoo(c)=polarder1(s,s1,cmpnt,cmpnt1,m)
              endif
           enddo
        
! Second derivatives wrt Q
           do m=1,nmodes
              do n=m,nmodes
                 if (polarder2_mask(s,s1,cmpnt,cmpnt1,m,n).eq.1) then
                    c=c+1
                    optcoo(c)=polarder2(s,s1,cmpnt,cmpnt1,m,n)
                 endif
              enddo
           enddo

! Third derivatives wrt Q (cubic terms only)
           do m=1,nmodes
              if (polarder3_mask(s,s1,cmpnt,cmpnt1,m).eq.1) then
                 c=c+1
                 optcoo(c)=polarder3(s,s1,cmpnt,cmpnt1,m)
              endif
           enddo

! Fourth derivatives wrt Q (quartic terms only)
           do m=1,nmodes
              if (polarder4_mask(s,s1,cmpnt,cmpnt1,m).eq.1) then
                 c=c+1
                 optcoo(c)=polarder4(s,s1,cmpnt,cmpnt1,m)
              endif
           enddo
        endif

        return
      end subroutine par2vec_lm

!#######################################################################

      subroutine vec2par_lm(s,s1,cmpnt,cmpnt1,optcoo,dipder1,dipder2,dipder3,&
           dipder4,dipder1_mask,dipder2_mask,dipder3_mask,dipder4_mask,&
           polarder1,polarder2,polarder3,polarder4,polarder1_mask,&
           polarder2_mask,polarder3_mask,polarder4_mask,diprmsd,polarrmsd)

        implicit none

        real(dop), dimension(*), intent(out) :: optcoo
        
        integer(long) :: s,s1,cmpnt,cmpnt1,m,n,c
      
        integer(long), dimension(nsta,nsta,3,nmodes)        :: dipder1_mask
        integer(long), dimension(nsta,nsta,3,nmodes,nmodes) :: dipder2_mask
        integer(long), dimension(nsta,nsta,3,nmodes)        :: dipder3_mask
        integer(long), dimension(nsta,nsta,3,nmodes)        :: dipder4_mask
        
        real(dop), dimension(nsta,nsta,3,nmodes)        :: dipder1
        real(dop), dimension(nsta,nsta,3,nmodes,nmodes) :: dipder2
        real(dop), dimension(nsta,nsta,3,nmodes)        :: dipder3
        real(dop), dimension(nsta,nsta,3,nmodes)        :: dipder4

        integer(long), dimension(nsta,nsta,3,3,nmodes)        :: polarder1_mask
        integer(long), dimension(nsta,nsta,3,3,nmodes,nmodes) :: polarder2_mask
        integer(long), dimension(nsta,nsta,3,3,nmodes)        :: polarder3_mask
        integer(long), dimension(nsta,nsta,3,3,nmodes)        :: polarder4_mask
        
        real(dop), dimension(nsta,nsta,3,3,nmodes)        :: polarder1
        real(dop), dimension(nsta,nsta,3,3,nmodes,nmodes) :: polarder2
        real(dop), dimension(nsta,nsta,3,3,nmodes)        :: polarder3
        real(dop), dimension(nsta,nsta,3,3,nmodes)        :: polarder4

        logical(kind=4) :: diprmsd,polarrmsd

        c=0

        if (diprmsd) then
! First derivatives wrt Q
           do m=1,nmodes
              if (dipder1_mask(s,s1,cmpnt,m).eq.1) then
                 c=c+1
                 dipder1(s,s1,cmpnt,m)=optcoo(c)
                 dipder1(s1,s,cmpnt,m)=dipder1(s,s1,cmpnt,m)
              endif
           enddo

! Second derivatives wrt Q
           do m=1,nmodes
              do n=m,nmodes
                 if (dipder2_mask(s,s1,cmpnt,m,n).eq.1) then
                    c=c+1
                    dipder2(s,s1,cmpnt,m,n)=optcoo(c)
                    dipder2(s1,s,cmpnt,m,n)=dipder2(s,s1,cmpnt,m,n)
                    dipder2(s,s1,cmpnt,n,m)=dipder2(s,s1,cmpnt,m,n)
                    dipder2(s1,s,cmpnt,n,m)=dipder2(s,s1,cmpnt,m,n)
                 endif
              enddo
           enddo

! Third derivatives wrt Q (cubic terms only)
           do m=1,nmodes
              if (dipder3_mask(s,s1,cmpnt,m).eq.1) then
                 c=c+1
                 dipder3(s,s1,cmpnt,m)=optcoo(c)
                 dipder3(s1,s,cmpnt,m)=dipder3(s,s1,cmpnt,m)
              endif
           enddo

! Fourth derivatives wrt Q (cubic terms only)
           do m=1,nmodes
              if (dipder4_mask(s,s1,cmpnt,m).eq.1) then
                 c=c+1
                 dipder4(s,s1,cmpnt,m)=optcoo(c)
                 dipder4(s1,s,cmpnt,m)=dipder4(s,s1,cmpnt,m)
              endif
           enddo
        endif
           
        if (polarrmsd) then
! First derivatives wrt Q
           do m=1,nmodes
              if (polarder1_mask(s,s1,cmpnt,cmpnt1,m).eq.1) then
                 c=c+1
                 polarder1(s,s1,cmpnt,cmpnt1,m)=optcoo(c)
                 polarder1(s1,s,cmpnt,cmpnt1,m)=polarder1(s,s1,cmpnt,cmpnt1,m)
              endif
           enddo

! Second derivatives wrt Q
           do m=1,nmodes
              do n=m,nmodes
                 if (polarder2_mask(s,s1,cmpnt,cmpnt1,m,n).eq.1) then
                    c=c+1
                    polarder2(s,s1,cmpnt,cmpnt1,m,n)=optcoo(c)
                    polarder2(s1,s,cmpnt,cmpnt1,m,n)=polarder2(s,s1,cmpnt,cmpnt1,m,n)
                    polarder2(s,s1,cmpnt,cmpnt1,n,m)=polarder2(s,s1,cmpnt,cmpnt1,m,n)
                    polarder2(s1,s,cmpnt,cmpnt1,n,m)=polarder2(s,s1,cmpnt,cmpnt1,m,n)
                 endif
              enddo
           enddo

! Third derivatives wrt Q (cubic terms only)
           do m=1,nmodes
              if (polarder3_mask(s,s1,cmpnt,cmpnt1,m).eq.1) then
                 c=c+1
                 polarder3(s,s1,cmpnt,cmpnt1,m)=optcoo(c)
                 polarder3(s1,s,cmpnt,cmpnt1,m)=polarder3(s,s1,cmpnt,cmpnt1,m)
              endif
           enddo

! Fourth derivatives wrt Q (cubic terms only)
           do m=1,nmodes
              if (polarder4_mask(s,s1,cmpnt,cmpnt1,m).eq.1) then
                 c=c+1
                 polarder4(s,s1,cmpnt,cmpnt1,m)=optcoo(c)
                 polarder4(s1,s,cmpnt,cmpnt1,m)=polarder4(s,s1,cmpnt,cmpnt1,m)
              endif
           enddo
           

        endif

        return
      end subroutine vec2par_lm

!#######################################################################

      real(dop) function lmrmsd(optcoo,mr,mi,s,s1,cmpnt,cmpnt1,diprmsd,&
           polarrmsd)

        implicit none

        integer(long)                                  :: s,s1,cmpnt,cmpnt1
        integer(long), dimension(midim), intent(in)    :: mi
        real(dop), dimension(mrdim), intent(out)       :: mr
        integer(long)                                  :: n,i
        real(dop), dimension(optcoodim), intent(inout) :: optcoo
        real(dop), dimension(maxdim)                   :: qcoo
        real(dop)                                      :: diff1,diff2
        real(dop), dimension(nsets)                    :: dat

        logical(kind=4)                                :: diprmsd,polarrmsd

!        print*,'d, p',diprmsd,polarrmsd
        
!-----------------------------------------------------------------------
! extract parameters from "coordinate" vector
!-----------------------------------------------------------------------
        call vec2par_lm(s,s1,cmpnt,cmpnt1,optcoo,mr(mrdipder1),&
             mr(mrdipder2),mr(mrdipder3),mr(mrdipder4),&
             mi(midipder1mask),mi(midipder2mask),&
             mi(midipder3mask),mi(midipder4mask),&
             mr(mrpolarder1),mr(mrpolarder2),mr(mrpolarder3),&
             mr(mrpolarder4),mi(mipolarder1mask),mi(mipolarder2mask),&
             mi(mipolarder3mask),mi(mipolarder4mask),diprmsd,polarrmsd)



        if (diprmsd) then
!-----------------------------------------------------------------------
! calculate model dipoles
!-----------------------------------------------------------------------
           do n=1,nsets
              call getqcoo(mr(mrqcoo),qcoo,nmodes,nsets,n)
              moddip(n)=ddipole(qcoo,mr,s,s1,cmpnt)
           enddo

!-----------------------------------------------------------------------
! calculate RMSD
!-----------------------------------------------------------------------
           call getdipole(mr(mrdipole),dat,s,s1,cmpnt)
        
           diff2=0
           do n=1,nsets
              diff1=dat(n)-moddip(n)
              diff2=diff2+diff1*diff1
              ! print*,moddip(n),dip(n)
           enddo
           
           lmrmsd=diff2
        
        else if (polarrmsd) then
!-----------------------------------------------------------------------
! calculate model polarisabilities
!-----------------------------------------------------------------------
           do n=1,nsets
              call getqcoo(mr(mrqcoo),qcoo,nmodes,nsets,n)
              modpolar(n)=dpolar(qcoo,mr,s,s1,cmpnt,cmpnt1)
           enddo

!-----------------------------------------------------------------------
! calculate RMSD
!-----------------------------------------------------------------------
           call getpolar(mr(mrpolar),dat,s,s1,cmpnt,cmpnt1)
           
           diff2=0
           do n=1,nsets
              diff1=dat(n)-modpolar(n)
              diff2=diff2+diff1*diff1
              ! print*,moddip(n),dip(n)
           enddo
           
           lmrmsd=diff2

        endif

        return
      end function lmrmsd

!#######################################################################

      subroutine getdipole(alldip,dip,s,s1,cmpnt)

        implicit none
        
        integer(long)                           :: s,s1,cmpnt,n
        real(dop), dimension(nsets,nsta,nsta,3) :: alldip
        real(dop), dimension(nsets)             :: dip

        do n=1,nsets
           dip(n)=alldip(n,s,s1,cmpnt)
        enddo

        return
      end subroutine getdipole

!#######################################################################

      subroutine getpolar(allpolar,polar,s,s1,cmpnt,cmpnt1)

        implicit none
        
        integer(long)                           :: s,s1,cmpnt,cmpnt1,n
        real(dop), dimension(nsets,nsta,nsta,3,3) :: allpolar
        real(dop), dimension(nsets)             :: polar

        do n=1,nsets
           polar(n)=allpolar(n,s,s1,cmpnt,cmpnt1)
        enddo

        return
      end subroutine getpolar

!#######################################################################
      subroutine rotpairvec(vec,rotm,rotang)

        implicit none
        
        integer(long)                       :: i,j,k,l,n
        real(dop)                           :: mixang,sinphi,cosphi
        real(dop), dimension(maxdim), intent(inout)                 :: vec
        integer(long),dimension(INT(maxdim/2)+1,1:2), intent(in)     :: rotm
        real(dop)                                                   :: tmp1,tmp2
        real(dop), dimension(2,2)                                   :: rmat,invrmat,omega,tomega
        real(dop),dimension(INT(maxdim/2)+1), intent(in)            :: rotang

        n = 1
        do while (rotm(n,1) .ne. 0)
           sinphi=sin(rotang(n)*pi/180.0)
           cosphi=cos(rotang(n)*pi/180.0)

!           do i=1,ncoo
              tmp1=cosphi*vec(rotm(n,1))-sinphi*vec(rotm(n,2))
              tmp2=sinphi*vec(rotm(n,1))+cosphi*vec(rotm(n,2))
!              print*, 'tmp1(i) = ', tmp1(i) , 'tmp2(i) = ', tmp2(i)
              vec(rotm(n,1))=tmp1       
              vec(rotm(n,2))=tmp2       
!           enddo
!           print*, 'vec = ', vec(1:nmodes)
!           print*, 'N = ', n
!           print*, 'sinphi = ', sinphi, 'cosphi =', cosphi
!           print*, 'rotm(n,1) = ', rotm(n,1), 'rotm(n,2) =', rotm(n,2)
!           print*,'ANGLE = ', rotang(n)
           n = n + 1
        enddo

        return
      end subroutine rotpairvec
!#######################################################################
      
      subroutine rotatemodes(nmcoo,freq0,rotm,rotang,rotmxfq)

        implicit none
        
        integer(long)                       :: i,j,k,l,n
        real(dop)                           :: mixang,sinphi,cosphi
        real(dop), dimension(maxdim,maxdim), intent(inout)          :: nmcoo
        real(dop), dimension(maxdim), intent(inout)                 :: freq0
        real(dop), dimension(ncoo)                                  :: tmp1,tmp2
        real(dop), dimension(2,2)                                   :: rmat,invrmat,omega,tomega
        integer(long),dimension(INT(maxdim/2)+1,1:2), intent(in)    :: rotm
        real(dop),dimension(INT(maxdim/2)+1), intent(inout)         :: rotmxfq  
        real(dop),dimension(INT(maxdim/2)+1), intent(in)            :: rotang

        n = 1
        do while (rotm(n,1) .ne. 0)
           sinphi=sin(rotang(n)*pi/180.0)
           cosphi=cos(rotang(n)*pi/180.0)
!          print*, 'rotand(n) = ', rotang(n)
           rmat(1,1)=cosphi
           rmat(2,2)=rmat(1,1)
           rmat(1,2)=sinphi
           rmat(2,1)=-sinphi

!           omega(1,1)=freq0(rotm(n,1))**2
!           omega(2,2)=freq0(rotm(n,2))**2

           omega(1,1)=freq0(rotm(n,1))
           omega(2,2)=freq0(rotm(n,2))
           omega(1,2)=0.0
           omega(2,1)=0.0
!----------------------------------------------------------------------
! (i) Transform eigenvectors
!----------------------------------------------------------------------
!           print*, 'sinphi = ', sinphi, 'cosphi =', cosphi
!           print*, 'rotm(n,1) = ', rotm(n,1), 'rotm(n,2) =', rotm(n,2)
!           do i=1,ncoo
!              print*, 'i = ', i, 'nmcoo(i,rotm(n,1)) = ', nmcoo(i,rotm(n,1)), 'nmcoo(i,rotm(n,2)) =', nmcoo(i,rotm(n,2))   
!           enddo
           do i=1,ncoo
              tmp1(i)=cosphi*nmcoo(i,rotm(n,1))-sinphi*nmcoo(i,rotm(n,2))
              tmp2(i)=sinphi*nmcoo(i,rotm(n,1))+cosphi*nmcoo(i,rotm(n,2))
!              print*, 'tmp1(i) = ', tmp1(i) , 'tmp2(i) = ', tmp2(i)
              nmcoo(i,rotm(n,1))=tmp1(i)       
              nmcoo(i,rotm(n,2))=tmp2(i)       
           enddo
           

!----------------------------------------------------------------------
! (ii) Transform eigenvalues
!----------------------------------------------------------------------
           do i=1,2
              do j=1,2
                 tomega(i,j)=0           
                 do k=1,2
                    do l=1,2
                       tomega(i,j)=tomega(i,j)+omega(k,l)*rmat(i,k)*rmat(j,l)
                    enddo
                 enddo
              enddo
           enddo
           


!           freq0(rotm(n,1))=sqrt(tomega(1,1))
           freq0(rotm(n,1))=tomega(1,1)
           freq0(rotm(n,2))=tomega(2,2)
           rotmxfq(n)=tomega(1,2)


           print*, 'mode1 ', rotm(n,1) , freq0(rotm(n,1))
!           freq0(rotm(n,2))=sqrt(tomega(2,2))
           print*, 'mode2 ', rotm(n,2), freq0(rotm(n,2))
          
!           rotmxfq(n)=sqrt(tomega(1,2))
!           print*, 'tomega(1,2) = ', tomega(1,2)
!           print*, 'mix freq =', rotmxfq(n)
           print*, 'ANGLE ROTATED + ', rotang(n)
           n = n + 1
        enddo

        return
      end subroutine rotatemodes

!#######################################################################

      subroutine uppercase(string,dim)

        implicit none

        integer(long)      :: dim,j
        character(len=dim) :: string

        do j = 1,len(string)
           if(string(j:j).ge."a".and.string(j:j).le."z")&
                then
              string(j:j)=achar(iachar(string(j:j))-32)
           end if
        enddo

        return
      end subroutine uppercase

!#######################################################################
! x2poly: Calculates the polyspherical coordinates pcoo for a point in 
!         Cartesian coordinates xcoo based on user-specified vectors
!
! For an N-atom system, we have 3N-6 internal coordinates: 
!
! N-1 vector lengths, R_i;
! N-2 in-plane angles, theta_i;
! N-3 dihedral angles, phi_i;
!
! pcoo(1) to pcoo(N-1) are the R_i
! 
! pcoo(N) to pcoo(2N-3) are the theta_i
!
! pcoo(2N-2) to pcoo(3N-6) are the phi_i
!
! The system is first translated to and rotated to the body-fixed frame.
!
! For Radau coordinates, patm(1) is taken as the heliocentre.
!#######################################################################

      subroutine x2poly(xcoo,pcoo,ncoo,nmodes,patm,mass,vcon)

        implicit none

        integer(long)                                     :: i,j,n,natm
        integer(long), intent(inout)                      :: ncoo,nmodes
        real(dop), dimension(nmodes), intent(inout)       :: pcoo
        real(dop), dimension(ncoo), intent(inout)         :: xcoo
        integer(long), dimension(maxdim), intent(inout)   :: patm,vcon
        real(dop), dimension(3)                           :: vec1,com
        real(dop)                                         :: work,pi,&
             nbmass,alpha,beta,gamma,tmp1,tmp2
        real(dop), dimension(maxdim), intent(inout)       :: mass
        real(dop), dimension(ncoo)                        :: xcoo_orig
        real(dop), dimension((ncoo/3)-1,3)                :: pvec

        pi=2.0*acos(0.0d0)
        natm=ncoo/3

        do i=1,ncoo
           xcoo_orig(i)=xcoo(i)
        enddo
      
!--------------------------------------------------------
! Translate to the SF frame
!--------------------------------------------------------
        do i=1,3
           nbmass=0.0
           com(i)=0.0
           do j=1,natm
              nbmass=nbmass+mass(j*3)
              com(i)=com(i)+xcoo(j*3-3+i)*mass(j*3)
           enddo
           com(i)=com(i)/nbmass
        enddo
        
        do i=1,3
           do j=1,natm
              xcoo(j*3-3+i)=xcoo(j*3-3+i)-com(i)
           enddo
        enddo
      
!--------------------------------------------------------
! Calculate the first two of the three Euler angles that 
! define the BF frame
!--------------------------------------------------------
        call getvectors(pvec,nmodes,xcoo,mass,ncoo,patm,vcon)

        work=0.0
        do i=1,3
           work=work+pvec((natm-1),i)*pvec((natm-1),i)
        enddo
        work=sqrt(work)
      
        if(pvec((natm-1),1).ne.0) then
           alpha=atan(pvec((natm-1),2)/pvec((natm-1),1))
        else
           alpha=0.0
        endif
        beta=acos(pvec((natm-1),3)/work)

!--------------------------------------------------------
! Rotate to the E2 frame
!--------------------------------------------------------
        call rotz(alpha,xcoo,ncoo)
        call roty(beta,xcoo,ncoo)
        
!--------------------------------------------------------
! Calculate the third Euler angle
!--------------------------------------------------------
        call getvectors(pvec,nmodes,xcoo,mass,ncoo,patm,vcon)

        if (pvec((natm-2),1).ne.0) then
           gamma=atan(pvec((natm-2),2)/pvec((natm-2),1))
        else
           gamma=0.0
        endif
        
!--------------------------------------------------------
! Rotate to the BF frame
!--------------------------------------------------------
        call rotz(gamma,xcoo,ncoo)

!--------------------------------------------------------
! Check that R_{N-1} points along the z-axis of BF in the
! positive direction and that R_{N-1} lies in the correct
! half-plane
!--------------------------------------------------------
        if (xcoo(patm(natm)*3).lt.0) then           
           do n=1,natm
              xcoo(n*3)=-xcoo(n*3)
           enddo
        endif

        if (xcoo(patm(natm-1)*3-2).lt.0) then
           do n=1,natm
              xcoo(n*3-2)=-xcoo(n*3-2)
           enddo
        endif
        
!--------------------------------------------------------
! Calculate all 3N-6 spherical coordinates in the BF
! frame
!--------------------------------------------------------   
        call getvectors(pvec,nmodes,xcoo,mass,ncoo,patm,vcon)

! Vector lengths
        do n=1,natm-1
           pcoo(n)=0.0
           do i=1,3
              pcoo(n)=pcoo(n)+pvec(n,i)*pvec(n,i)
           enddo
           pcoo(n)=sqrt(pcoo(n))
        enddo

! Planar angles
        do n=1,natm-2
           pcoo(natm-1+n)=acos(pvec(n,3)/pcoo(n))         
        enddo

! Dihedral angles (azimuthal angles)
        do n=1,natm-3
           work=pvec(n,2)/pvec(n,1)
           work=abs(work)
           if (pvec(n,1).gt.0) then
              pcoo(2*natm-3+n)=atan(work)
           else
              if (pvec(n,1).lt.0.and.pvec(n,2).ge.0) then
                 pcoo(2*natm-3+n)=atan(work)+pi
              else if (pvec(n,1).lt.0.and.pvec(n,2).lt.0) then
                 pcoo(2*natm-3+n)=atan(work)-pi
              else if (pvec(n,1).eq.0.and.pvec(n,2).gt.0) then
                 pcoo(2*natm-3+n)=pi/2.0
              else if (pvec(n,1).eq.0.and.pvec(n,2).lt.0) then
                 pcoo(2*natm-3+n)=-pi/2.0
              else if (pvec(n,1).eq.0.and.pvec(n,2).eq.0) then
                 pcoo(2*natm-3+n)=0
              endif
           endif
           if (pcoo(2*natm-3+n).lt.0) &
                pcoo(2*natm-3+n)=pcoo(2*natm-3+n)+2*pi
        enddo

! Convert vector lengths to atomic units
        do n=1,natm-1
           pcoo(n)=pcoo(n)/angstroem
        enddo

        do i=1,ncoo
           xcoo(i)=xcoo_orig(i)
        enddo
        
        return
      end subroutine x2poly

!#######################################################################
! poly2x: Calculates the Cartesian coordinates xcoo for a point in 
!         polyspherical coordinates pcoo based on user-specified vectors
!#######################################################################

    subroutine poly2x(xcoo,xcoo0,pcoo,ncoo,nmodes,patm,mass,vcon)

      implicit none
      
      integer(long)                                     :: i,j,k,n,natm
      integer(long), intent(in)                         :: ncoo,nmodes
      real(dop), dimension(nmodes), intent(inout)       :: pcoo
      real(dop), dimension(ncoo), intent(inout)         :: xcoo,xcoo0
      integer(long), dimension(maxdim), intent(inout)   :: patm,vcon
      real(dop), dimension(3)                           :: vec1,com
      real(dop)                                         :: work,nbmass,pi
      real(dop), dimension(maxdim), intent(inout)       :: mass
      real(dop), dimension(maxdim,3)                    :: nbcom,pvec

      pi=2.0*acos(0.0d0)
      natm=ncoo/3

      do i=1,ncoo
         xcoo(i)=0.0         
      enddo

! Convert vector lengths to Angstrom
      do n=1,natm-1
         pcoo(n)=pcoo(n)*angstroem
      enddo

!--------------------------------------------------------
! 1.) Calculate vectors from the given BF frame spherical 
!     coordinates
!-------------------------------------------------------- 
      do n=1,natm-1
         if (n.lt.natm-2) then
            pvec(n,1)=pcoo(n)*sin(pcoo(natm-1+n))*cos(pcoo(2*natm-3+n))
            pvec(n,2)=pcoo(n)*sin(pcoo(natm-1+n))*sin(pcoo(2*natm-3+n))
            pvec(n,3)=pcoo(n)*cos(pcoo(natm-1+n))         
         else if (n.eq.natm-2) then
            pvec(n,1)=pcoo(n)*sin(pcoo(natm-1+n))
            pvec(n,2)=0.0
            pvec(n,3)=pcoo(n)*cos(pcoo(natm-1+n))
         else
            pvec(n,1)=0.0
            pvec(n,2)=0.0
            pvec(n,3)=pcoo(n)
         endif
      enddo

! Convert vector lengths back to atomic units
      do n=1,natm-1
         pcoo(n)=pcoo(n)/angstroem
      enddo

!--------------------------------------------------------
! 2.) Calculate Cartesian coordinates
!--------------------------------------------------------
      if (ljacobi) call j2x(pvec,xcoo,patm,mass,ncoo)
      if (lradau) call r2x(pvec,xcoo,patm,mass,ncoo)
      if (lvalence) call v2x(pvec,xcoo,patm,vcon,ncoo)

      return
    end subroutine poly2x

    subroutine j2x(pvec,xcoo,patm,mass,ncoo)

      implicit none

      integer(long), intent(in)                         :: ncoo
      integer(long)                                     :: i,j,k,natm      
      real(dop), dimension(maxdim,3)                    :: nbcom,pvec
      real(dop), dimension(ncoo), intent(inout)         :: xcoo
      real(dop)                                         :: nbmass
      real(dop), dimension(3)                           :: com
      integer(long), dimension(maxdim), intent(inout)   :: patm
      real(dop), dimension(maxdim), intent(inout)       :: mass

      natm=ncoo/3

      do i=1,3
         xcoo(patm(1)*3-3+i)=0.0
         xcoo(patm(2)*3-3+i)=xcoo(patm(1)*3-3+i)+pvec(1,i)
      enddo
      
      do j=3,natm
         nbmass=0.0
         do i=1,3
            com(i)=0.0
            do k=1,j-1
               if (i.eq.1) nbmass=nbmass+mass(patm(k)*3)
               com(i)=com(i)+xcoo(patm(k)*3-3+i)*mass(patm(k)*3)
            enddo
            com(i)=com(i)/nbmass
            xcoo(patm(j)*3-3+i)=com(i)+pvec(j-1,i)
         enddo
      enddo

      return
    end subroutine j2x

!#######################################################################

    subroutine r2x(pvec,xcoo,patm,mass,ncoo)
      
      implicit none

      integer(long), intent(in)                         :: ncoo
      integer(long)                                     :: i,k,n,natm      
      real(dop), dimension(maxdim,3)                    :: pvec
      real(dop), dimension(ncoo), intent(inout)         :: xcoo
      integer(long), dimension(maxdim), intent(inout)   :: patm
      real(dop), dimension(maxdim), intent(inout)       :: mass
      real(dop)                                         :: tmass,fac,&
                                                           alpha,msub,tmp

      natm=ncoo/3

      msub=0.0
      tmass=0.0
      do n=1,natm
         tmass=tmass+mass(n*3)
         if (n.gt.1) msub=msub+mass(patm(n)*3)
      enddo

! Cartesian coordinates of the heliocentre (patm(1))
      fac=tmass*mass(patm(1)*3)
      fac=sqrt(fac)
      fac=-1/fac
      do i=1,3
         xcoo(patm(1)*3-3+i)=0.0
         do n=2,natm
            xcoo(patm(1)*3-3+i)=xcoo(patm(1)*3-3+i)+&
                 mass(patm(n)*3)*pvec(n-1,i)
         enddo
         xcoo(patm(1)*3-3+i)=fac*xcoo(patm(1)*3-3+i)
      enddo

! Cartesian coordinates of the atoms of the (natm-1)-atom
! subunit
      fac=mass(patm(1)*3)/tmass
      fac=sqrt(fac)
      alpha=1.0-sqrt(tmass/mass(patm(1)*3))
      alpha=(1.0/msub)*alpha
      do n=2,natm
         tmp=msub-mass(patm(n)*3)
         do i=1,3
            xcoo(patm(n)*3-3+i)=(1.0-alpha*tmp)*pvec(n-1,i)
            do k=2,natm
               if (k.ne.n) then
                  xcoo(patm(n)*3-3+i)=xcoo(patm(n)*3-3+i)+&
                       alpha*mass(patm(k)*3)*pvec(k-1,i)
               endif
            enddo
            xcoo(patm(n)*3-3+i)=fac*xcoo(patm(n)*3-3+i)
         enddo
      enddo

      return
    end subroutine r2x

!#######################################################################

    subroutine v2x(pvec,xcoo,patm,vcon,ncoo)

      implicit none

      integer(long), intent(in)                         :: ncoo
      integer(long)                                     :: n,i,natm,check
      real(dop), dimension(maxdim,3)                    :: pvec
      real(dop), dimension(ncoo), intent(inout)         :: xcoo
      integer(long), dimension(maxdim), intent(inout)   :: patm,vcon
      integer(long), dimension(ncoo/3)                  :: xass

      integer(long) :: j

      natm=ncoo/3

      do n=1,natm
         xass(n)=0
      enddo

! Take first reference atom as the origin
      do i=1,3
         xcoo(vcon(1)*3-3+i)=0.0
         xass(vcon(1))=1
      enddo

! Assign remaining coordinates
! N.B. as the connectivity is not checked after it is read, it is posible
! to enter an infinite loop here...
10    continue
      check=0
      do n=2,natm
         if (xass(patm(n)).eq.0.and.xass(vcon(n-1)).eq.1) then            
            do i=1,3
               xcoo(patm(n)*3-3+i)=xcoo(vcon(n-1)*3-3+i)+pvec(n-1,i)
            enddo
            xass(patm(n))=1
         endif
         if (xass(patm(n)).eq.0) check=check+1
      enddo
      if (check.gt.0) goto 10

      return
    end subroutine v2x

!#######################################################################

    subroutine getvectors(pvec,nmodes,xcoo,mass,ncoo,patm,vcon)
      
      implicit none
      
      integer(long)                                     :: i,n
      real(dop), dimension((ncoo/3)-1,3)                :: pvec
      integer(long), intent(inout)                      :: nmodes
      real(dop), dimension(ncoo), intent(inout)         :: xcoo
      real(dop), dimension(maxdim), intent(inout)       :: mass
      integer(long), intent(inout)                      :: ncoo
      integer(long), dimension(maxdim), intent(inout)   :: patm,vcon

      do i=1,3
         do n=1,(ncoo/3)-1
            pvec(n,i)=0.0
         enddo
      enddo

      if (ljacobi) call getjacobivec(xcoo,mass,ncoo,patm,pvec,nmodes)
      if (lradau) call getradauvec(xcoo,mass,ncoo,patm,pvec,nmodes)
      if (lvalence) call getvalencevec(xcoo,mass,ncoo,patm,pvec,vcon)

      return
    end subroutine getvectors

!#######################################################################

    subroutine getradauvec(xcoo,mass,ncoo,patm,pvec,nmodes)
      
      implicit none
      
      integer(long)                                     :: i,n,natm
      real(dop), dimension(ncoo), intent(inout)         :: xcoo
      real(dop), dimension(maxdim), intent(inout)       :: mass
      integer(long), intent(inout)                      :: ncoo
      integer(long), dimension(maxdim), intent(inout)   :: patm
      real(dop), dimension((ncoo/3)-1,3)                :: pvec
      integer(long), intent(inout)                      :: nmodes
      real(dop), dimension(3)                           :: Dcom,Ccom,vec1,&
                                                           canonical
      real(dop)                                         :: tmass,lenod,&
                                                           lencd,lenbd,&
                                                           lenob,tmp

      natm=ncoo/3

! Ccom: centre of mass of the whole system
      do i=1,3
         Ccom(i)=0.0
         tmass=0.0
         do n=1,natm
            tmass=tmass+mass(n*3)
            Ccom(i)=Ccom(i)+xcoo(n*3-3+i)*mass(n*3)
         enddo
         Ccom(i)=Ccom(i)/tmass
      enddo
      
! Dcom: centre of mass of the system minus the heliocentre (patm(1))
      do i=1,3
         Dcom(i)=0.0
         tmass=0.0
         do n=2,natm
            tmass=tmass+mass(patm(n)*3)
            Dcom(i)=Dcom(i)+xcoo(patm(n)*3-3+i)*mass(patm(n)*3)
         enddo
         Dcom(i)=Dcom(i)/tmass
      enddo

! Determine the length from the heliocentre to the canonical point (lenob)
      lenod=0.0
      lencd=0.0
      do i=1,3
         tmp=Dcom(i)-xcoo(patm(1)*3-3+i)
         lenod=lenod+tmp*tmp
         tmp=Dcom(i)-Ccom(i)
         lencd=lencd+tmp*tmp
      enddo
      lenod=sqrt(lenod)
      lencd=sqrt(lencd)
      lenbd=lenod*lencd
      lenbd=sqrt(lenbd)
      lenob=lenod-lenbd

! Determine the Cartesian coordinates of the canonical point      
      tmp=0.0
      do i=1,3
         vec1(i)=Dcom(i)-xcoo(patm(1)*3-3+i)
         tmp=tmp+vec1(i)*vec1(i)
      enddo
      tmp=sqrt(tmp)
      
      do i=1,3
         vec1(i)=vec1(i)/tmp
         canonical(i)=xcoo(patm(1)*3-3+i)+vec1(i)*lenob
      enddo

! Calculate Radau vectors
      do n=1,natm-1
         do i=1,3
            pvec(n,i)=xcoo(patm(n+1)*3-3+i)-canonical(i)
         enddo
      enddo

      return
    end subroutine getradauvec
      
!#######################################################################

    subroutine getjacobivec(xcoo,mass,ncoo,patm,pvec,nmodes)
      
      implicit none

      integer(long)                                     :: i,n
      real(dop), dimension(ncoo), intent(inout)         :: xcoo
      real(dop), dimension(maxdim), intent(inout)       :: mass
      integer(long), intent(inout)                      :: ncoo
      integer(long), dimension(maxdim), intent(inout)   :: patm
      real(dop), dimension(maxdim,3)                    :: nbcom
      real(dop), dimension((ncoo/3)-1,3)                :: pvec
      integer(long), intent(inout)                      :: nmodes
                     
      call eval_nbcom(xcoo,mass,ncoo,nbcom,patm)

      do n=1,(ncoo/3)-1
         do i=1,3
            pvec(n,i)=xcoo(patm(n+1)*3-3+i)-nbcom(n,i)
         enddo
      enddo

      return
    end subroutine getjacobivec

!#######################################################################

    subroutine getvalencevec(xcoo,mass,ncoo,patm,pvec,vcon)

      implicit none

      integer(long)                                     :: i,n
      real(dop), dimension(ncoo), intent(inout)         :: xcoo
      real(dop), dimension(maxdim), intent(inout)       :: mass
      integer(long), intent(inout)                      :: ncoo
      integer(long), dimension(maxdim), intent(inout)   :: patm,vcon
      real(dop), dimension(maxdim,3)                    :: nbcom
      real(dop), dimension((ncoo/3)-1,3)                :: pvec
      
      do n=1,(ncoo/3)-1
         do i=1,3
            pvec(n,i)=xcoo(patm(n+1)*3-3+i)-xcoo(vcon(n)*3-3+i)
         enddo
      enddo

      return
    end subroutine getvalencevec

!#######################################################################

    subroutine eval_nbcom(xcoo,mass,ncoo,nbcom,jatm)

      implicit none

      integer(long)                                     :: i,j,k,natm
      real(dop)                                         :: nbmass
      
      integer(long), intent(in)                         :: ncoo
      real(dop), dimension(ncoo), intent(inout)         :: xcoo
      real(dop), dimension(maxdim), intent(inout)       :: mass
      real(dop), dimension(maxdim,3), intent(inout)     :: nbcom
      integer(long), dimension(maxdim), intent(inout)   :: jatm

      natm=ncoo/3

      do i=1,natm
         nbmass=0
         do j=1,i
            nbmass=nbmass+mass(jatm(j)*3)
         enddo
         do k=1,3
            nbcom(i,k)=0.0
            do j=1,i
               nbcom(i,k)=nbcom(i,k)+xcoo(jatm(j)*3-3+k)*mass(jatm(j)*3)
            enddo
            nbcom(i,k)=nbcom(i,k)/nbmass
         enddo
      enddo

      return
    end subroutine eval_nbcom

!#######################################################################

    subroutine rotx(ang,xcoo,ncoo)

      implicit none
      
      real(dop), dimension(ncoo), intent(inout)  :: xcoo
      real(dop), intent(in)                      :: ang
      integer(long), intent(in)                  :: ncoo
      
      real(dop), dimension(ncoo)                 :: tmp
      integer(long)                              :: i,j,k,l
      
      do i=1,ncoo/3
         j=i*3-2
         k=i*3-1
         l=i*3
         tmp(j)=xcoo(j)
         tmp(k)=xcoo(k)*cos(ang)+xcoo(l)*sin(ang)
         tmp(l)=-xcoo(k)*sin(ang)+xcoo(l)*cos(ang)
      enddo
      
      do j=1,ncoo
         xcoo(j)=tmp(j)
      enddo

      return
    end subroutine rotx

!#######################################################################

    subroutine roty(ang,xcoo,ncoo)

      implicit none
      
      real(dop), dimension(ncoo), intent(inout)  :: xcoo
      real(dop), intent(in)                      :: ang
      integer(long), intent(in)                  :: ncoo
      
      real(dop), dimension(ncoo)                 :: tmp
      integer(long)                              :: i,j,k,l

      do i=1,ncoo/3
         j=i*3-2
         k=i*3-1
         l=i*3
         tmp(j)=xcoo(j)*cos(ang)-xcoo(l)*sin(ang)
         tmp(k)=xcoo(k)
         tmp(l)=xcoo(j)*sin(ang)+xcoo(l)*cos(ang)
      enddo
      
      do j=1,ncoo
         xcoo(j)=tmp(j)
      enddo
      
      return
    end subroutine roty

!#######################################################################

    subroutine rotz(ang,xcoo,ncoo)

      implicit none
      
      real(dop), dimension(ncoo), intent(inout)  :: xcoo
      real(dop), intent(in)                      :: ang
      integer(long), intent(in)                  :: ncoo
      
      real(dop), dimension(ncoo)                 :: tmp
      integer(long)                              :: i,j,k,l

      do i=1,ncoo/3
         j=i*3-2
         k=i*3-1
         l=i*3
         tmp(j)=xcoo(j)*cos(ang)+xcoo(k)*sin(ang)
         tmp(k)=-xcoo(j)*sin(ang)+xcoo(k)*cos(ang)
         tmp(l)=xcoo(l)
      enddo
      
      do j=1,ncoo
         xcoo(j)=tmp(j)
      enddo

      return
    end subroutine rotz

!#######################################################################

      subroutine curv2q(qn)

        implicit none
        
        integer(long)                              :: j,p
        real(dop)                                  :: n0, p1
        integer(long)                              :: i
        real(dop), dimension(nmodes)               :: tq1  
        real(dop), dimension(nmodes),intent(inout) :: qn 
        
        tq1 = -1.0
        j = 1
        print*,nmcurvtype(1), nmcurv(1,1),nmcurv(1,2), nmcurv(1,3), c0(1,1),c0(1,2),c0(1,3)
        print*,nmcurvtype(2), nmcurv(2,1),nmcurv(2,2), nmcurv(2,3), c0(2,1),c0(2,2),c0(2,3)
        do while (nmcurvtype(j) .ne. 0)
           if( nmcurvtype(j) .eq. 1 ) then   
!       note: this n0 is not generalisable, it is only for acetylene's c-H bond  (this transforms the 
!	c-h bondlength into a normal mode basis:
              n0=(coonm(6,12)*c0(j,1))**2
!	note that the 0.531 is a massive bodge to make the new bonds slightly more like actual c-h angles
              n0 = sqrt(n0)+0.533
!           n0 = sqrt(n0)+0.533

!           n0 = sqrt(n0)+0.533
!!                      this is the polar spherical transformation of 3 normal modes
!!		       but where there is a shift in the (in this case q1(nmcurv(k,3)) ) direction by a distance
!!		       of the bondlength that one is trying to then describe by the 2 sph. pol. angles and R
!!		       ( the shift allows one to start moving throught phi or theta while  R=0 (c0(,1) = 0) )
!!                      the y x and z type normal modes will be replaced by R theta phi (in this order)
!          tq1(nmcurv(j,1)) = (qn(nmcurv(j,1))+n0)*cos(pi*(qn(nmcurv(j,2))+c0(j,2))/180)
!          tq1(nmcurv(j,2)) = (qn(nmcurv(j,1))+n0)*sin(pi*(qn(nmcurv(j,2))+c0(j,2))/180)*sin(pi*(qn(nmcurv(j,3))+c0(j,3))/180)
!          tq1(nmcurv(j,3)) = (qn(nmcurv(j,1))+n0)*sin(pi*(qn(nmcurv(j,2))+c0(j,2))/180)*cos(pi*(qn(nmcurv(j,3))+c0(j,3))/180) - n0
            
              tq1(nmcurv(j,1)) = ((qn(nmcurv(j,1)))+n0)*cos(qn(nmcurv(j,2))+c0(j,2))
              tq1(nmcurv(j,2)) = ((qn(nmcurv(j,1)))+n0)*sin(qn(nmcurv(j,2))+c0(j,2))*sin(qn(nmcurv(j,3))+c0(j,3))
              tq1(nmcurv(j,3)) = ((qn(nmcurv(j,1)))+n0)*sin(qn(nmcurv(j,2))+c0(j,2))*cos(qn(nmcurv(j,3))+c0(j,3)) + n0

              print*, 'SIN(phi +c0(j,2)) =', sin(qn(nmcurv(j,2))+c0(j,2))
              print*, 'COS(theta +c0(j,3)) =', cos(qn(nmcurv(j,3))+c0(j,3) )
!	FOR DEBUGGGING:
              print*, 'curv_2_Q'
              print*, 'NO + ', n0
              print*, 'c0(j,2) = ', c0(j,2) ,  ' c0(j,3) = ', c0(j,3)
              print*, 'nmcurv(j,1) =', nmcurv(j,1), ' qn(nmcurv(j,1)) =',&
                   qn(nmcurv(j,1)) , ' tn(nmcurv(j,1)) =', tq1(nmcurv(j,1))
              print*, 'nmcurv(j,2) =', nmcurv(j,2), ' qn(nmcurv(j,2)) =',&
                   qn(nmcurv(j,2)) , ' tn(nmcurv(j,2)) =', tq1(nmcurv(j,2))
              print*, 'nmcurv(j,3) =', nmcurv(j,3), ' qn(nmcurv(j,3)) =',&
                   qn(nmcurv(j,3)) , ' tn(nmcurv(j,3)) =', tq1(nmcurv(j,3))
              
           endif
           j = j + 1
        enddo


!       print*, 't = ', t, xincr , c0(j,2), ' j = ', j, 'cmode =', cmode
!      those modes that are not in any set of transformations:
        do j = 1, nmodes
           if (tq1(j) .eq. -1.0 ) tq1(j) = qn(j)
           if (tq1(j) .eq. -1.0 ) print*, 'mode ',i,' is not used;'
        enddo
        qn = tq1
        


!	IF WE DO FREQUENCY SCALE THE COORDINATES
!!         y 
!          q1(nmcurv(j,1)) = (p*inc(1)+n0)*cos(pi*(p*inc(2)+c0(j,2))/180)&
!          *(sqrt(freq0(nmcurv(j,1))/freq0(nmcurv(j,3))))
!!         x
!          q1(nmcurv(j,2)) = (p*inc(1)+n0)*sin(pi*(p*inc(2)+c0(j,2))/180)*sin(pi*(p*inc(3)+c0(j,3))/180)&
!          *(sqrt(freq0(nmcurv(j,2))/freq0(nmcurv(j,3))))
!!     z
!          q1(nmcurv(j,3)) = (p*inc(1)+n0)*sin(pi*(p*inc(2)+c0(j,2))/180)*cos(pi*(p*inc(3)+c0(j,3))/180) - n0

      end subroutine   curv2q
!###########################################################################################################

      subroutine q2curv(qcoo1)

      implicit none
      
      real(dop),dimension(nmodes),intent(inout)  :: qcoo1
      real(dop),dimension(nmodes)                :: ccoo
      real(dop)                                  :: n0, f3f1,f3f2,co,si,x,y
      integer(long)                              :: i, j

      ccoo = -1.0
      j = 1
      do while ( nmcurv(j,1) .ne. 0  )
       print*, 'IN HERE'   
       if( nmcurvtype(j) .eq. 1) then
!	note: this n0 only works for acetylene (its equal to the length of the C-H bond in normal mode coordiantes length)
          n0=(coonm(6,12)*c0(j,1))**2
          n0 = sqrt(n0)+0.533

          ccoo(nmcurv(j,1)) = sqrt( qcoo1(nmcurv(j,1))**2 + qcoo1(nmcurv(j,2))**2 + &
                                    ( qcoo1(nmcurv(j,3)) - n0 )**2  ) - n0 
            
!          ccoo(nmcurv(j,2)) =&
!          atan( sqrt( (qcoo(nmcurv(j,3)) + n0)**2 + qcoo(nmcurv(j,2))**2  )/qcoo(nmcurv(j,1)))  -c0(j,2) 

!          ccoo(nmcurv(j,3)) = atan2( qcoo(nmcurv(j,2)),(qcoo(nmcurv(j,3)) + n0 ) )-c0(j,3)
!       NOTE: THERE IS SOMETHING ABOUT ATAN THAT I DONT SEEM TO NEED TO SUBTRACT THE 180 OUT OF IT...
!       PLEASE USE WITH CARE...
         si = atan2( qcoo1(nmcurv(j,2)),(qcoo1(nmcurv(j,3)) - n0 ) )/abs(atan2( qcoo1(nmcurv(j,2)),(qcoo1(nmcurv(j,3)) - n0 ) ))
          ccoo(nmcurv(j,3)) = atan2( qcoo1(nmcurv(j,2)),(qcoo1(nmcurv(j,3)) - n0 ) ) - c0(j,3)*si

         
          
!          ccoo(nmcurv(j,3)) = acos( (qcoo(nmcurv(j,3)) - n0 )**2/( (sqrt( (qcoo(nmcurv(j,3)) - n0 )**2 + &
!           qcoo(nmcurv(j,2))**2   ) )*qcoo(nmcurv(j,2)))) - c0(j,3)
!          ccoo(nmcurv(j,3)) = atan2( qcoo(nmcurv(j,2)),(qcoo(nmcurv(j,3)) - n0 ) ) 
!          if(qcoo(nmcurv(j,3)) + n0 )  y/x

!           x = (qcoo(nmcurv(j,3)) - n0 )
!           y = qcoo(nmcurv(j,2))
!           if( x .lt. 0 .and. y .ge. 0) then
!            ccoo(nmcurv(j,3)) = atan(y/x)+pi-c0(j,3)           
!           elseif(  x .lt. 0 .and. y .lt. 0) then
!            ccoo(nmcurv(j,3)) = atan(y/x)-pi-c0(j,3)
!           elseif(  x .eq. 0 .and. y .gt. 0) then
!            ccoo(nmcurv(j,3)) = pi/2.0-c0(j,3)
!           elseif(  x .eq. 0 .and. y .lt. 0) then
!            ccoo(nmcurv(j,3)) = -pi/2.0-c0(j,3)
!           elseif(  abs(x) .gt. 0.0001 .and. abs(y) .gt. 0.001) then
!            ccoo(nmcurv(j,3)) = 0.0
!           endif

!          ccoo(nmcurv(j,3)) = atan2( qcoo(nmcurv(j,2)),(qcoo(nmcurv(j,3)) - n0 ) )
!          print*, 'one',  atan2( (qcoo(nmcurv(j,3)) + n0 ),qcoo(nmcurv(j,2)) )
!          print*, 'two',  atan2( qcoo(nmcurv(j,2)),(qcoo(nmcurv(j,3)) + n0 ) )


!          ccoo(nmcurv(j,3)) = atan2( qcoo(nmcurv(j,2))/(qcoo(nmcurv(j,3)) + n0 )  )-c0(j,3)
          if( abs(qcoo1(nmcurv(j,2))) .lt. 0.00001 ) ccoo(nmcurv(j,3)) = 0.0d0
!          if(ISNAN(  ccoo(nmcurv(j,3)) ) )  ccoo(nmcurv(j,3) = 0.0   

          ccoo(nmcurv(j,2)) = acos(qcoo1(nmcurv(j,1))/(ccoo(nmcurv(j,1)) + n0 )) - c0(j,2)  

!          ccoo(nmcurv(j,3)) = atan(qcoo(nmcurv(j,2))/(qcoo(nmcurv(j,3)) +n0 ))
!          ccoo(nmcurv(j,3)) = atan(qcoo(nmcurv(j,2))/(qcoo(nmcurv(j,3)) +n0 ))-c0(j,3)


         print*, 'Q_2_curv'
         print*, 'NO + ', n0
         print*, 'c0(j,2) = ', c0(j,2) ,  ' c0(j,3) = ', c0(j,3)
         print*, 'nmcurv(j,1) =', nmcurv(j,1), ' qcoo(nmcurv(j,1)) =',&
            qcoo1(nmcurv(j,1)), ' ccoo(nmcurv(j,1)) =', ccoo(nmcurv(j,1))
         print*, 'nmcurv(j,2) =', nmcurv(j,2), ' qcoo(nmcurv(j,2)) =',&
            qcoo1(nmcurv(j,2)), ' ccoo(nmcurv(j,2)) =', ccoo(nmcurv(j,2))
         print*, 'nmcurv(j,3) =', nmcurv(j,3), ' qcoo(nmcurv(j,3)) =',&
            qcoo1(nmcurv(j,3)), ' ccoo(nmcurv(j,3)) =', ccoo(nmcurv(j,3))

       endif

       j = j + 1
      enddo      

!      those modes that are not in any set of transformations:
      do j = 1, nmodes
        if (ccoo(j) .eq. -1.0 ) ccoo(j) = qcoo1(j)
      enddo
       qcoo1 = ccoo

      end subroutine

!#######################################################################
!
! Get the dihedral from a set of Cartesian coordinates
!   Defined dihedral is calculated and molecule rotated
!   back to 0 dihedral
!
!#######################################################################

      subroutine x2dihed(xcoo,qtheta,xcoo0,theta0)

      implicit none

      integer(long)             :: nd,o,p,d1,h
      integer(long), dimension(4) :: temp
      real(dop), dimension(3,natm), intent(inout) :: xcoo
      real(dop), dimension(3,natm), intent(in)    :: xcoo0
      real(dop), intent(out)                      :: qtheta,theta0
      integer(long)                               :: a,b,c
      integer(long), dimension(maxdim)            :: d

      real(dop)                          :: adj,adj1,adj2,opp,opp1,opp2,&
                                            thetax,thetay,thetaz,x,y,z,&
                                            tmptheta,rotang
      logical(kind=4), save              :: lrotatex,lrotatey,ltransl, &
                                            lrotatez,lreverse
      real(dop), dimension(3,natm)       :: q1,xcoot,xcoo0t
      real(dop), dimension(maxdim)       :: phi,r,theta
      real(dop), dimension(3,2)          :: vec
      real(dop), dimension(3)            :: vcross,vcross1
      real(dop)                          :: x2,y2,z2,norm

!-------------------------------------------------------
!   Dihedral
!-------------------------------------------------------
!   Dihedral works using polar spherical coordinates
!   To achieve this the entire molecular must be rotated
!   so that the a-b bond lies in the x plane, atom c
!   must be the origin and the b-c bond must lie along
!   the z plane.
! 
!-------------------------------------------------------

!-----------------------------------------------------------------------
! set defaults 
!-----------------------------------------------------------------------
      lrotatex=.false.
      lrotatey=.false.
      lrotatez=.false.
      ltransl=.true.

      opp=0.0_dop
      adj=0.0_dop
      opp1=0.0_dop
      adj1=0.0_dop
      thetax=0.0_dop
      thetay=0.0_dop
      thetaz=0.0_dop

!-----------------------------------------------------------------------
! setting a,b,c,d defining dihedral. nd is no. of "d" atoms
!-----------------------------------------------------------------------
      nd=0
      do p=1,natm
         if (dihedflag(p) .eq. 'a' .or. dihedflag(p) .eq. 'A') then
            a = p
         elseif (dihedflag(p) .eq. 'b' .or. dihedflag(p) .eq. 'B') then
            b = p
         elseif (dihedflag(p) .eq. 'c' .or. dihedflag(p) .eq. 'C') then
            c = p
         elseif (dihedflag(p) .eq. 'd' .or. dihedflag(p) .eq. 'D') then
            nd=nd+1
            d(nd) = p
         elseif (dihedflag(p) .ne. 'f' .or. dihedflag(p) .eq. 'F') then
            routine = 'x2dihed (Vcham utils.f90)'
            message = 'Invalid dihedral flag (a,b,c,d,f only)'
            call vcerrormsg
         endif
      enddo

! first orient reference structure xcoo0 -> xcoo0t
!-----------------------------------------------------------------------
! translation to put atom c as the origin
!-----------------------------------------------------------------------
      x=xcoo0(1,c)
      y=xcoo0(2,c)
      z=xcoo0(3,c)
      do p=1,natm
         xcoo0t(1,p)=xcoo0(1,p)-x
         xcoo0t(2,p)=xcoo0(2,p)-y
         xcoo0t(3,p)=xcoo0(3,p)-z
      enddo

!-----------------------------------------------------------------------
! rotation to put the b-c bond in the xz plane
! rotates about the x axis
!-----------------------------------------------------------------------
      opp=xcoo0t(2,b)-xcoo0t(2,c)
      adj=xcoo0t(3,b)-xcoo0t(3,c)
      if (abs(opp) .lt. tiny) then
         lrotatex=.false.
         thetax=0.0_dop
      elseif (abs(adj) .lt. tiny) then 
         lrotatex=.true.
         thetax=-90*(pi/180)
      else
         lrotatex=.true.
         thetax=-atan(opp/adj)
      endif
      if (lrotatex) then
         call rotatex(thetax,xcoo0t,xcoot)
         xcoo0t=xcoot
      endif

!-----------------------------------------------------------------------
! rotation to put the b-c bond along the z axis
! rotates about the y axis
!-----------------------------------------------------------------------
      opp=xcoo0t(1,b)-xcoo0t(1,c)
      adj=xcoo0t(3,b)-xcoo0t(3,c)
      if (abs(opp) .lt. tiny) then
         lrotatey=.false.
         thetay=0.0_dop
      elseif(abs(adj) .lt. tiny)then
         lrotatey=.true.
         thetay=-90*(pi/180)
      else
         lrotatey=.true.
         thetay=-atan(opp/adj)
      endif
      if (lrotatey) then
         call rotatey(thetay,xcoo0t,xcoot)
         xcoo0t=xcoot
      endif

!-----------------------------------------------------------------------
! rotation to put the a-b bond in the xz plane
! rotates about the z axis
!-----------------------------------------------------------------------
      opp=xcoo0t(2,a)-xcoo0t(2,b)
      adj=xcoo0t(1,a)-xcoo0t(1,b)
      if (abs(opp) .lt. tiny) then
         lrotatez=.false.    
         thetaz=0.0_dop
      elseif (abs(adj) .lt. tiny) then
         thetaz=90*(pi/180)
! print*,'Perpendicular to Y axis, rotation set to 90 degrees'
         lrotatez=.true.
      else 
         thetaz=atan(opp/adj)
         lrotatez=.true.
      endif
      if (lrotatez) then
         call rotatez(thetaz,xcoo0t,xcoot)
         xcoo0t=xcoot
      endif

!-----------------------------------------------------------------------
! translate and rotate new structure to same orientation as standard
!-----------------------------------------------------------------------
      do p=1,natm
         xcoo(1,p)=xcoo(1,p)-x
         xcoo(2,p)=xcoo(2,p)-y
         xcoo(3,p)=xcoo(3,p)-z
      enddo
      if (lrotatex) then
         call rotatex(thetax,xcoo,xcoot)
         xcoo=xcoot
      endif
      if (lrotatey) then
         call rotatey(thetay,xcoo,xcoot)
         xcoo=xcoot
      endif
      if (lrotatez) then
         call rotatez(thetaz,xcoo,xcoot)
         xcoo=xcoot
      endif
!-----------------------------------------------------------------------
! define plane for d1-c-d2
!-----------------------------------------------------------------------
      do h=1,2
         d1=d(h)
         vec(:,h) = xcoo(:,d1) - xcoo(:,c)
      enddo

      vcross(1) = vec(2,1)*vec(3,2) - vec(3,1)*vec(2,2)
      vcross(2) = -(vec(1,1)*vec(3,2) - vec(3,1)*vec(1,2))
      vcross(3) = vec(1,1)*vec(2,2) - vec(2,1)*vec(1,2)
! normalise projection onto xy plane
      norm = vcross(1)**2 + vcross(2)**2
      norm = dsqrt(norm)
      if (norm .lt. tiny) then
         routine='X2dihed'
         message='atoms to be moved do not define a plane'
         call vcerrormsg
      endif
      vcross = vcross/norm

! dihedral is given by dot product of vcross with y axis. Sign depends on
! which side of y-axis a lies
      if (xcoo0t(1,a) .ge. 0.0) then
         qtheta = acos(vcross(2))
      else
         qtheta = acos(-vcross(2))
      endif

! sign given by direction of z-component of cross product
      vcross1(3) = -vcross(1)
      if (vcross1(3) .gt. 0.0) qtheta = -qtheta

!-----------------------------------------------------------------------
! rotate dihedrals back to original values. Rotation around z-axis 
! of theta0-qtheta 
!-----------------------------------------------------------------------
! get dihedral of xcoo0
      temp(1) = a
      temp(2) = b
      temp(3) = c
      temp(4) = d(1)
      theta0 = dihed(xcoo0,temp,ncoo)

      rotang = theta0-qtheta
      do h=1,nd
         d1 = d(h)
         xcoot(1,d1) = cos(rotang) * xcoo(1,d1) + sin(rotang) * xcoo(2,d1)
         xcoot(2,d1) = -sin(rotang) * xcoo(1,d1) + cos(rotang) * xcoo(2,d1)
         xcoot(3,d1) = xcoo(3,d1)

         xcoo(:,d1) = xcoot(:,d1)
      enddo

!-----------------------------------------------------------------------
! translate and rotate molecule back to original orientation
!-----------------------------------------------------------------------
      thetax=-thetax
      thetay=-thetay
      thetaz=-thetaz
      if (lrotatez) then
         call rotatez(thetaz,xcoo,xcoot)
         xcoo=xcoot
      endif
      if (lrotatey) then
         call rotatey(thetay,xcoo,xcoot)
         xcoo=xcoot
      endif
      if (lrotatex) then
         call rotatex(thetax,xcoo,xcoot)
         xcoo=xcoot
      endif
      do p=1,natm
         xcoo(1,p)=xcoo(1,p)+x
         xcoo(2,p)=xcoo(2,p)+y
         xcoo(3,p)=xcoo(3,p)+z
      enddo

      return
      end subroutine x2dihed

!#######################################################################
!
! Set the dihedral for a structure
!   Defined dihedral is calculated and molecule rotated
!   back to 0 dihedral
!
!#######################################################################

      subroutine nm2dihed(xcoo,qtheta,xcoo0,theta0)

      implicit none

      integer(long)             :: nd,p,d1,h
      integer(long), dimension(4) :: temp
      real(dop), dimension(3,natm), intent(inout) :: xcoo
      real(dop), dimension(3,natm), intent(in)    :: xcoo0
      real(dop), intent(out)                      :: qtheta,theta0
      integer(long)                               :: a,b,c
      integer(long), dimension(maxdim)            :: d

      real(dop)                          :: adj,adj1,opp,opp1,&
                                            thetax,thetay,thetaz,x,y,z,&
                                            tmptheta,rotang
      logical(kind=4), save              :: lrotatex,lrotatey,ltransl, &
                                            lrotatez
      real(dop), dimension(3,natm)       :: q1,xcoot,xcoo0t
      real(dop), dimension(maxdim)       :: r,theta
      real(dop), dimension(3,2)          :: vec
      real(dop), dimension(3)            :: vcross,vcross1
      real(dop)                          :: x2,y2,z2

!-------------------------------------------------------
!   Dihedral
!-------------------------------------------------------
!   Dihedral works using polar spherical coordinates
!   To achieve this the entire molecular must be rotated
!   so that the a-b bond lies in the x plane, atom c
!   must be the origin and the b-c bond must lie along
!   the z plane.
! 
!-------------------------------------------------------

!-----------------------------------------------------------------------
! set defaults 
!-----------------------------------------------------------------------
      lrotatex=.false.
      lrotatey=.false.
      lrotatez=.false.
      ltransl=.true.

      opp=0.0_dop
      adj=0.0_dop
      opp1=0.0_dop
      adj1=0.0_dop
      thetax=0.0_dop
      thetay=0.0_dop
      thetaz=0.0_dop

!-----------------------------------------------------------------------
! setting a,b,c,d defining dihedral. nd is no. of "d" atoms
!-----------------------------------------------------------------------
      nd=0
      do p=1,natm
         if (dihedflag(p) .eq. 'a' .or. dihedflag(p) .eq. 'A') then
            a = p
         elseif (dihedflag(p) .eq. 'b' .or. dihedflag(p) .eq. 'B') then
            b = p
         elseif (dihedflag(p) .eq. 'c' .or. dihedflag(p) .eq. 'C') then
            c = p
         elseif (dihedflag(p) .eq. 'd' .or. dihedflag(p) .eq. 'D') then
            nd=nd+1
            d(nd) = p
         elseif (dihedflag(p) .ne. 'f' .or. dihedflag(p) .eq. 'F') then
            routine = 'x2dihed (Vcham utils.f90)'
            message = 'Invalid dihedral flag (a,b,c,d,f only)'
            call vcerrormsg
         endif
      enddo

! first orient reference structure xcoo0 -> xcoo0t
!-----------------------------------------------------------------------
! translation to put atom c as the origin
!-----------------------------------------------------------------------
      x=xcoo0(1,c)
      y=xcoo0(2,c)
      z=xcoo0(3,c)
      do p=1,natm
         xcoo0t(1,p)=xcoo0(1,p)-x
         xcoo0t(2,p)=xcoo0(2,p)-y
         xcoo0t(3,p)=xcoo0(3,p)-z
      enddo

!-----------------------------------------------------------------------
! rotation to put the b-c bond in the xz plane
! rotates about the x axis
!-----------------------------------------------------------------------
      opp=xcoo0t(2,b)-xcoo0t(2,c)
      adj=xcoo0t(3,b)-xcoo0t(3,c)
      if (abs(opp) .lt. tiny) then
         lrotatex=.false.
         thetax=0.0_dop
      elseif (abs(adj) .lt. tiny) then 
         lrotatex=.true.
         thetax=-90*(pi/180)
      else
         lrotatex=.true.
         thetax=-atan(opp/adj)
      endif
      if (lrotatex) then
         call rotatex(thetax,xcoo0t,xcoot)
         xcoo0t=xcoot
      endif

!-----------------------------------------------------------------------
! rotation to put the b-c bond along the z axis
! rotates about the y axis
!-----------------------------------------------------------------------
      opp=xcoo0t(1,b)-xcoo0t(1,c)
      adj=xcoo0t(3,b)-xcoo0t(3,c)
      if (abs(opp) .lt. tiny) then
         lrotatey=.false.
         thetay=0.0_dop
      elseif(abs(adj) .lt. tiny)then
         lrotatey=.true.
         thetay=-90*(pi/180)
      else
         lrotatey=.true.
         thetay=-atan(opp/adj)
      endif
      if (lrotatey) then
         call rotatey(thetay,xcoo0t,xcoot)
         xcoo0t=xcoot
      endif

!-----------------------------------------------------------------------
! rotation to put the a-b bond in the xz plane
! rotates about the z axis
!-----------------------------------------------------------------------
      opp=xcoo0t(2,a)-xcoo0t(2,b)
      adj=xcoo0t(1,a)-xcoo0t(1,b)
      if (abs(opp) .lt. tiny) then
         lrotatez=.false.    
         thetaz=0.0_dop
      elseif (abs(adj) .lt. tiny) then
         thetaz=90*(pi/180)
! print*,'Perpendicular to Y axis, rotation set to 90 degrees'
         lrotatez=.true.
      else 
         thetaz=atan(opp/adj)
         lrotatez=.true.
      endif
      if (lrotatez) then
         call rotatez(thetaz,xcoo0t,xcoot)
         xcoo0t=xcoot
      endif

!-----------------------------------------------------------------------
! translate and rotate new structure to same orientation as standard
!-----------------------------------------------------------------------
      do p=1,natm
         xcoo(1,p)=xcoo(1,p)-x
         xcoo(2,p)=xcoo(2,p)-y
         xcoo(3,p)=xcoo(3,p)-z
      enddo
      if (lrotatex) then
         call rotatex(thetax,xcoo,xcoot)
         xcoo=xcoot
      endif
      if (lrotatey) then
         call rotatey(thetay,xcoo,xcoot)
         xcoo=xcoot
      endif
      if (lrotatez) then
         call rotatez(thetaz,xcoo,xcoot)
         xcoo=xcoot
      endif

!-----------------------------------------------------------------------
! rotate dihedral. Rotation around z-axis of theta0+qtheta 
!-----------------------------------------------------------------------
! get dihedral of xcoo0
      temp(1) = a
      temp(2) = b
      temp(3) = c
      temp(4) = d(1)
      theta0 = dihed(xcoo0,temp,ncoo)

      rotang = qtheta
      do h=1,nd
         d1 = d(h)
         xcoot(1,d1) = cos(rotang) * xcoo(1,d1) - sin(rotang) * xcoo(2,d1)
         xcoot(2,d1) = sin(rotang) * xcoo(1,d1) + cos(rotang) * xcoo(2,d1)
         xcoot(3,d1) = xcoo(3,d1)

         xcoo(:,d1) = xcoot(:,d1)
      enddo

!-----------------------------------------------------------------------
! translate and rotate molecule back to original orientation
!-----------------------------------------------------------------------
      thetax=-thetax
      thetay=-thetay
      thetaz=-thetaz
      if (lrotatez) then
         call rotatez(thetaz,xcoo,xcoot)
         xcoo=xcoot
      endif
      if (lrotatey) then
         call rotatey(thetay,xcoo,xcoot)
         xcoo=xcoot
      endif
      if (lrotatex) then
         call rotatex(thetax,xcoo,xcoot)
         xcoo=xcoot
      endif
      do p=1,natm
         xcoo(1,p)=xcoo(1,p)+x
         xcoo(2,p)=xcoo(2,p)+y
         xcoo(3,p)=xcoo(3,p)+z
      enddo

      return
      end subroutine nm2dihed

!#######################################################################

!      LOGICAL FUNCTION ISNAN(R)
         
!        real(dop), intent(in) :: R
        
!        IF (R .GT. 0.0) THEN
!           ISNAN = .FALSE.
!        ELSEIF (R .LE. 0.0) THEN
!           ISNAN = .FALSE.
!        ELSE
!           ISNAN = .TRUE.
!        END IF
!      END function isnan

!#######################################################################

    subroutine nmeckart(nmcoo,itvec,nmodes,ncoo,maxdim)

      implicit none
      
      integer(long)                                      :: i,j,k,l
      integer(long), intent(in)                          :: nmodes,ncoo,&
                                                            maxdim      
      real(dop), dimension(maxdim,maxdim), intent(inout) :: nmcoo
      real(dop), dimension(3,3), intent(in)              :: itvec
      real(dop), dimension(ncoo,nmodes)                  :: tmp

! Loop over the Hessian eigenvectors
      do i=1,nmodes         
! Loop over the atoms
         do j=1,ncoo/3            
! Loop over the x,y,z components of each atom, rotating as we go
            do k=1,3
               tmp(j*3-3+k,i)=0.0
               do l=1,3
                  tmp(j*3-3+k,i)=tmp(j*3-3+k,i)+&
                       itvec(k,l)*nmcoo(j*3-3+l,i)
               enddo
            enddo
         enddo
      enddo

      do i=1,nmodes 
         do j=1,ncoo
            nmcoo(j,i)=tmp(j,i)
         enddo
      enddo

      return
    end subroutine nmeckart

!#######################################################################

    subroutine rotrans(nmcoo,maxdim,nmodes,ncoo,mass,xcoo0)

      implicit none
      
      integer(long)                                      :: i,j,k,error
      integer(long), intent(in)                          :: maxdim,nmodes,&
                                                            ncoo
      real(dop), dimension(maxdim,maxdim), intent(inout) :: nmcoo
      real(dop), dimension(ncoo), intent(inout)          :: xcoo0
      real(dop), dimension(ncoo)                         :: txcoo0
      real(dop), dimension(maxdim), intent(in)           :: mass
      real(dop), dimension(3,3)                          :: itensor
      real(dop), dimension(3)                            :: ieig,com
      real(dop), dimension(9)                            :: work
      real(dop)                                          :: tmass,tmp
      real(dop), dimension(3,ncoo/3)                     :: rdx

      do i=1,3
         do j=1,3
            itensor(i,j)=0.0
         enddo
      enddo

!-----------------------------------------------------------------------
! Translations
!-----------------------------------------------------------------------
      do j=1,ncoo,3
         ! Tx
         nmcoo(j,nmodes+1)=sqrt(mass(j))
         ! Ty
         nmcoo(j+1,nmodes+2)=sqrt(mass(j+1))
         ! Tz
         nmcoo(j+2,nmodes+3)=sqrt(mass(j+2))
      enddo

!-----------------------------------------------------------------------
! Rotations
!-----------------------------------------------------------------------
! Translate the origin to the CoM
!-----------------------------------------------------------------------
      tmass=0.0
      do i=1,ncoo/3
         tmass=tmass+mass(i*3)
      enddo      
      do j=1,3
         com(j)=0.0
         do i=1,ncoo/3
            com(j)=com(j)+(mass(i*3-3+j)*xcoo0(i*3-3+j))
         enddo
         com(j)=com(j)/tmass
      enddo
      do i=1,ncoo/3
         do j=1,3
            txcoo0(i*3-3+j)=xcoo0(i*3-3+j)-com(j)
         enddo
      enddo

!-----------------------------------------------------------------------
! Construct the moment of inertia tensor
!-----------------------------------------------------------------------
      do i=1,ncoo/3
           itensor(1,1)=itensor(1,1)+mass(i*3)*(txcoo0(i*3-1)**2+txcoo0(i*3)**2)
           itensor(2,2)=itensor(2,2)+mass(i*3)*(txcoo0(i*3-2)**2+txcoo0(i*3)**2)
           itensor(3,3)=itensor(3,3)+mass(i*3)*(txcoo0(i*3-2)**2+txcoo0(i*3-1)**2)
           itensor(1,2)=itensor(1,2)-mass(i*3)*(txcoo0(i*3-2)*txcoo0(i*3-1))
           itensor(1,3)=itensor(1,3)-mass(i*3)*(txcoo0(i*3-2)*txcoo0(i*3))
           itensor(2,3)=itensor(2,3)-mass(i*3)*(txcoo0(i*3-1)*txcoo0(i*3))
        enddo
        itensor(2,1)=itensor(1,2)
        itensor(3,1)=itensor(1,3)
        itensor(3,2)=itensor(2,3)

!-----------------------------------------------------------------------
! Diagonalise the moment of inertia tensor to obtain the principal
! axes of inertia
!-----------------------------------------------------------------------
        call dsyev('V','U',3,itensor,3,ieig,work,9,error)
        
        if (error.ne.0) then
           write(6,'(/,a,/)') 'Diagonalisation of the moment of inertia tensor in rottrans failed'
           STOP
        endif

!-----------------------------------------------------------------------
! Construct the vectors for infinitesimal rotations
!-----------------------------------------------------------------------
      do i=1,3
         do j=1,ncoo/3
            rdx(i,j)=0.0d0
            do k=1,3
               rdx(i,j)=rdx(i,j)+txcoo0(j*3-3+k)*itensor(k,i)
            enddo
         enddo
      enddo

      do i=1,ncoo/3
         ! Rx
         ! x-components
         nmcoo(i*3-2,nmodes+4)=(rdx(2,i)*itensor(3,1)-rdx(3,i)*itensor(2,1))/sqrt(mass(i))
         ! y-components
         nmcoo(i*3-1,nmodes+4)=(rdx(2,i)*itensor(3,2)-rdx(3,i)*itensor(2,2))/sqrt(mass(i))
         ! z-components
         nmcoo(i*3,nmodes+4)=(rdx(2,i)*itensor(3,3)-rdx(3,i)*itensor(2,3))/sqrt(mass(i))

         ! Ry
         ! x-components
         nmcoo(i*3-2,nmodes+5)=(rdx(3,i)*itensor(1,1)-rdx(1,i)*itensor(3,1))/sqrt(mass(i))
         ! y-components
         nmcoo(i*3-1,nmodes+5)=(rdx(3,i)*itensor(1,2)-rdx(1,i)*itensor(3,2))/sqrt(mass(i))
         ! z-components
         nmcoo(i*3,nmodes+5)=(rdx(3,i)*itensor(1,3)-rdx(1,i)*itensor(3,3))/sqrt(mass(i))

         ! Rz
         ! x-components
         nmcoo(i*3-2,nmodes+6)=(rdx(1,i)*itensor(2,1)-rdx(2,i)*itensor(1,1))/sqrt(mass(i))
         ! y-components
         nmcoo(i*3-1,nmodes+6)=(rdx(1,i)*itensor(2,2)-rdx(2,i)*itensor(1,2))/sqrt(mass(i))
         ! z-components
         nmcoo(i*3,nmodes+6)=(rdx(1,i)*itensor(2,3)-rdx(2,i)*itensor(1,3))/sqrt(mass(i))
      enddo

!-----------------------------------------------------------------------
! Normalise the eigenvectors
!-----------------------------------------------------------------------
        do i=nmodes+1,nmodes+6
           tmp=0
           do j=1,ncoo
              tmp=tmp+nmcoo(j,i)*nmcoo(j,i)
           enddo
           do j=1,ncoo
              nmcoo(j,i)=nmcoo(j,i)/sqrt(tmp)
           enddo
        enddo

    end subroutine rotrans
    
!#######################################################################

    subroutine switchdat2(aswitch,ms,m,m1,s,s1,k,swpar,swfunc,nswitchcoe)

      implicit none

      integer(long) :: i,j,narg,ms,m,m1,s,s1,k,ilbl,swpar,swfunc,nswitchcoe
      integer(long), dimension(nmodes) :: defchk,chk
      character(len=80) :: aswitch
      character(len=20) :: apar,afunc
      integer(long), parameter :: dimaarg=15
      character(len=20), dimension(dimaarg) :: aarg

      do i=1,dimaarg
         aarg(i)=''
      enddo
      apar=''

      narg=1
      j=0
      do i=1,len_trim(aswitch)
         if (aswitch(i:i).eq.' ') then
            narg=narg+1
            j=0
         else
            j=j+1
            aarg(narg)(j:j)=aswitch(i:i)
         endif
      enddo

! Read switch mode
      read(aarg(1),'(i2)') ms

! Read parameter type
      read(aarg(2),'(a)') apar

! Read modes, states, and, if appropriate, the coefficient no.
      if (apar.eq.'kappa') then
         swpar=1
         read(aarg(3),*) m
         read(aarg(4),*) s
         read(aarg(5),*) afunc

      else if (apar.eq.'lambda') then
         swpar=2
         read(aarg(3),*) m
         read(aarg(4),*) s
         read(aarg(5),*) s1
         read(aarg(6),*) afunc

      else if (apar.eq.'gamma') then
         swpar=3
         read(aarg(3),*) m
         read(aarg(4),*) m1
         read(aarg(5),*) s
         read(aarg(6),*) afunc

      else if (apar.eq.'mu') then
         swpar=4
         read(aarg(3),*) m
         read(aarg(4),*) m1
         read(aarg(5),*) s
         read(aarg(6),*) s1
         read(aarg(7),*) afunc

      else if (apar.eq.'iota') then
         swpar=5
         read(aarg(3),*) m
         read(aarg(4),*) m1
         read(aarg(5),*) s
         read(aarg(6),*) afunc

      else if (apar.eq.'general') then
         swpar=6
         read(aarg(3),*) k
         read(aarg(4),*) m
         read(aarg(5),*) s
         read(aarg(6),*) afunc

      else if (apar.eq.'explicit') then
         swpar=7
         read(aarg(3),*) s
         read(aarg(4),*) s1
! Read modes and powers in order to determine termk number by
! comparing to termdef array
         do i=1,nmodes
            defchk(i)=0
         enddo
         do i=5,narg-1
            ilbl=index(aarg(i),'^')
            read(aarg(i)(1:ilbl-1),*) j
            read(aarg(i)(ilbl+1:len_trim(aarg(i))),*) defchk(j)
         enddo
! Compare current modes and powers to those of the read-in 
! explicitely-added terms
         do i=1,nadd
            do j=1,nmodes
               chk(j)=1
            enddo
            if (s.eq.termst(1,i).and.s1.eq.termst(2,i).or. &
                 (s.eq.termst(2,i)).and.s1.eq.termst(1,i)) then
               do j=1,nmodes
                  chk(j)=0
               enddo
               do j=1,nmodes
                  chk(j)=defchk(j)-termdef(j,i)
                  if (chk(j).ne.0) go to 10
               enddo
               k=i
               goto 20
            endif
10          continue
         enddo
20       continue
         read(aarg(narg),*) afunc

      else if (apar.eq.'cpgeneral') then
         swpar=8
         read(aarg(3),*) k
         read(aarg(4),*) m
         read(aarg(5),*) s
         read(aarg(6),*) s1
         read(aarg(7),*) afunc

      endif

      if (afunc.eq.'tanh') then
         swfunc=1
         nswitchcoe=2
      else if (afunc.eq.'dtanh') then
         swfunc=2
         nswitchcoe=1
      else if (afunc.eq.'cdtanh') then
         swfunc=3
         nswitchcoe=2
      else if (afunc.eq.'adtanh') then
         swfunc=4
         nswitchcoe=4
      else
         write(6,'(/,3(x,a),/)') 'The switching function type',&
              afunc(1:len_trim(afunc)),'is not recognised.'
         STOP
      endif

      return
    end subroutine switchdat2

!#######################################################################

    subroutine getfreq(unit)

      implicit none

      integer(long)     :: i,unit
      character(len=80) :: string

      rewind(unit)

10    continue
      read(unit,'(a)',end=999) string
      if (string.ne.'*** Frequencies ***') goto 10

      do i=1,nmodes
         read(unit,'(13x,F7.4)') freq0(i)
      enddo

      return

999   continue
      write(6,'(/,a,/)') 'Error reading VCHAM file!'
      STOP

    end subroutine getfreq

!#######################################################################

    subroutine reweight(energy,weight,nsets)

      implicit none

      integer(long)               :: nsets,n
      real(dop), dimension(nsets) :: energy
      real(dop)                   :: weight,ezero

      if (weighttype .eq. 4) then ! yapol
         do n=1,nsets
            ezero = energy(ista(n))
            w0(n)=0.5_dop*(1-tanh((energy(n)-(ezero+0.7_dop))/0.2_dop))
         enddo
      else
         do n=1,nsets
            ezero = energy(ista(n))
            w0(n) = exp(weight*(ezero-energy(n)))
         enddo
      endif

    end subroutine reweight

!#######################################################################

    subroutine transformq1(qcoo)

    implicit none

    integer(long)                               :: m,n
    real(dop), dimension(nmodes), intent(inout) :: qcoo

    do m=1,nmodes
       if (qtransform(m) .eq. 1) then
          qcoo(m) = cos(qcoo(m))
       else if (qtransform(m) .eq. 2) then
          qcoo(m) = sin(qcoo(m))
       else if (qtransform(m) .eq. 3) then
          qcoo(m) = cos(0.5*qcoo(m))
       else if (qtransform(m) .eq. 4) then
          qcoo(m) = sin(0.5*qcoo(m))
       endif
    enddo

    end subroutine transformq1


  end module utils

