!!-----------------------------------------------------------------------
!
      program vchfit

use decimal, only: long,dop
use vcglobal
use global, only:pi, fs, ev, keyword,keyorig,maxkey,lc,lwarn
use maxv, only: maxdim,maxsta,maxset
use versions
use vchammod
use vchfitmod
use versions
use invchammod

use module_interface

use op1lib, only: zeromxi, zerovxz, zerovxd, zerovxl, zerovxi, unitqxd, cpvxd
use output_vcham, only: outmask,outstat,writepar,writeop,writeqinp
use rddatamod
use symmetry, only: createmask,renumcons,readmask,setcons,delmodes,&
     delstates,addparams,rdexplicit
use iofiles, only: readguess,findtermkdat
use functions, only: conv_fdiab,vpes,dpes,diabpes,ddiabpes
use density_vcham, only: statedensity
use calc_low_cimod
use findminmod
use utils
use logdat, only: message, routine, inptit
use randommod, only: drandom,randgaussian

use timing
use keyunitsmod

      implicit none

      integer(long)       :: ilbl,modus,ierr,k,l,s,ierror,iarg,pos
      character(len=15)   :: cerr

      real(dop), dimension(:), allocatable        :: optcoo
      complex(dop), dimension(:), allocatable     :: mc
      real(dop), dimension(:), allocatable        :: mr
      integer(long), dimension(:), allocatable    :: mi
      logical(kind=4), dimension(:), allocatable  :: ml
      integer(long)                               :: i
      character(len=80)                           :: filename,string
      real(dop)           :: fret
      integer(long), save :: tid=0
      integer(long), save :: tid1=0
      integer(long), external :: myiargc
      character(len=240)      :: cstartup

! arrays to read in a Q0 coordinate
      integer(long), save  :: nval=0 , ival(10)
      real(dop), save      :: qval0(10)

      call get_timer(tid, 'Tot')
      call start_timer(tid)


      iarg = myiargc()
      if (iarg .lt. 1) goto 200
      i = 0
110      i=i+1
      call mygetarg(i,string)

!-----------------------------------------------------------------------
! options start with -
!-----------------------------------------------------------------------

      if (string(1:1) .eq. '-') then
         if(string .eq. '-qval')then
            nval = nval+1
            if (nval .gt. 10) then
               routine='vchfit'
               message='A Maximum of 10 values can be given'
               call vcerrormsg
            else
               i=i+1
               call mygetarg(i,string)
               read(string,*) ival(nval)
               i=i+1
               call mygetarg(i,string)
               read(string,*) qval0(nval)
            endif


         else if (string .eq. '-h' .or. string .eq. '-?') then
            write(6,'(1x,a)')&
       'Purpose : To fit the output from VCTRANS.'
            write(6,'(1x,2a)')&
               'Usage   : vchfitXX [options]',&
               '[arguments].'
            write(6,'(/,a)') ' Options : '
            write(6,'(a)')&
            ' -qval m val: accepts a value val for mode m non equa',&
                 'l to zero at the fc geometry'
            write(6,'(a)')&
            ' -filename filename: writes customfilenames (filename.vcham, filename.op)'

         else
            write(6,'(2a)') ' unknown option : ', string
            stop
         end if

! read next option
         go to 110


      else if (i .eq. iarg) then
         filename=string
      endif

  200 continue

!      call rempath(progname,lprogname,ierror)
!      if(ierror.eq.1) then
!         routine='VCHFIT'
!         message=progname//' is a directory, not a file name.'
!         call vcerrormsg
!      endif

!-----------------------------------------------------------------------
! Initialise calculation
! get input file name, set maxdim and then allocate memory
!-----------------------------------------------------------------------
      pos = i !position of filename

      call initvcham(pos)
      call fpstartup(cstartup)
      write(ilog,'(/a)') trim(cstartup(1:80))
      call wrversion(ilog)

      call alloc_vchfitmod
      call alloc_vchammod

      call wrversion(ilog)

!-----------------------------------------------------------------------
! set default values for parameters and flags
!-----------------------------------------------------------------------
      call vchdefault
      call vchfdefault

!-----------------------------------------------------------------------
! open files
! read input file
!-----------------------------------------------------------------------
      call vchfinp

!-----------------------------------------------------------------------
! If the GA is being used, initialise the corresponding arrays
!-----------------------------------------------------------------------
      if(lgenetica) call initgamut

      if ( (lrdguess .eqv. .false.) .and. &
           ( offdiagsignopt .eqv. .true.)) then
         routine = 'VCHFIT'
         message = &
              'odd-diagonal sign optimisation must come with a guess.vcham.'
         call vcerrormsg
      endif
      if (lwarn) then
         routine = 'VCHFIT'
         message = 'Finished with input error. See log file'
         call vcerrormsg
      endif

!-----------------------------------------------------------------------
! read explicitely-added terms
!-----------------------------------------------------------------------
      if (nadd.gt.0) call rdexplicit

!-----------------------------------------------------------------------
! read switching functions
!-----------------------------------------------------------------------
      if (lswitch) call rdswitch

!----------------------------------------------------------------------
! log calculation
!-----------------------------------------------------------------------
      call wrlog

!-----------------------------------------------------------------------
! assign memory for arrays
!-----------------------------------------------------------------------
      optcoodim = nsta*nsta & ! const
                  + nmodes*nsta & ! kappa
                  + nmodes*nsta*nsta & !lambda
                  + nmodes*nmodes*nsta & ! gamma
                  + nmodes*nmodes*nsta*nsta & ! mu
                  + nmodes*nmodes*nsta*nsta & ! iota
                  + nsta*nmodes*maxcoe & ! expcoe
                  + nhigher & ! 'higher-order terms'
                  + nadd & ! explicitly added terms
                  + nsta*nsta*nmodes*maxcoe & ! cpcoe
                  + nswitch*maxcoe & !switching functions
                  + nsta*nsta*nmodes*nmodes*maxcoe ! I think this is the 2D coupling functions...

      call memvcham
      if (lconfidence) then
         mrhess = mrdim
         mrdim = mrhess + optcoodim*optcoodim
         mrcovar = mrdim
         mrdim = mrcovar + optcoodim*optcoodim
         mrcorrel = mrdim
         mrdim = mrcorrel + optcoodim*optcoodim
         mrdpar = mrdim
         mrdim = mrdpar + optcoodim
         mrerrpar = mrdim
         mrdim = mrerrpar + optcoodim
      else
         mrhess = 1
         mrcovar = 1
         mrcorrel = 1
         mrdpar = 1
         mrerrpar = 1
      endif

!-----------------------------------------------------------------------
! Allocate memory
!-----------------------------------------------------------------------
      write(ilog,'(/,a)') 'MEMORY ALLOCATED'
      ilbl = 8*optcoodim+16*mcdim+8*mrdim+4*midim+4*mldim
      write(ilog,'( 5(a,i10,/),a,i13,a,f10.2,a)')&
       ' optcoodim  : ', optcoodim,&
       ' complex*16 : ', mcdim,&
       ' real*8     : ', mrdim,&
       ' integer    : ', midim,&
       ' logical    : ', mldim,&
       ' Total   : ', ilbl, ' Bytes ,', dble(ilbl)/1048576.0_dop,' MB'
      call myflush(ilog)

      if(allocated(optcoo))deallocate(optcoo)
      if(allocated(mc))deallocate(mc)
      if(allocated(mr))deallocate(mr)
      if(allocated(mi))deallocate(mi)
      if(allocated(ml))deallocate(ml)

      cerr = 'vchfit_optcoo '
      allocate(optcoo(optcoodim),stat=ierr)
      if (ierr > 0) go to 2000
      cerr = 'vchfit_mc '
      allocate(mc(mcdim),stat=ierr)
      if (ierr > 0) go to 2000
      cerr = 'vchfit_mr '
      allocate(mr(mrdim),stat=ierr)
      if (ierr > 0) go to 2000
      cerr = 'vchfit_mi '
      allocate(mi(midim),stat=ierr)
      if (ierr > 0) go to 2000
      cerr = 'vchfit_ml '
      allocate(ml(mldim),stat=ierr)
      if (ierr > 0) go to 2000
      call zerovxz(mc,mcdim)
      call zerovxd(mr,mrdim)
      call zerovxi(mi,midim)
      call zerovxl(ml,mldim)

      q0val(:) =0.0
!-----------------------------------------------------------------------
! get information from the dataset-file
!-----------------------------------------------------------------------
      write(6,'(/,a)') ' Reading datasets... '
      open(idat,file=datfile,form='formatted',status='old',err=999)
      call read_dataset(mr(mrenergy),mr(mrxcoo),mr(mrqcoo),mr(mrder1),&
           mr(mrdcp),mr(mrder2),mr(mrtdip),mr(mrxder1),mr(mrxdcp),&
           mr(mrxder2),q0val)
      close(idat)

      if (lqtransform) then
         call transformq(mr(mrqcoo))
      endif

!-----------------------------------------------------------------------
! read a guess-file. This must be done before creating the mask as
! the diabatic potentials may be defined here.
!-----------------------------------------------------------------------
      if (lrdguess) then
         ilbl=index(guessfile,' ')-1
         write(6,'(/,2a)')&
              'Load guess from file : ',guessfile(1:ilbl)
         open(iguess,file=guessfile,form='formatted',status='old',&
              err=1000)
         call readguess(iguess,mr(mrconst),mr(mrkappa),mr(mrlambda),&
              mr(mrgamma),mr(mriota),mr(mrmu),mr(mrexpcoe),&
              mi(miexpcoemask),mr(mrhigher),mr(mrtermk),&
              mr(mrcpcoe),mr(mrswitch),mr(mrcp2d))
         close(iguess)
      endif

!-----------------------------------------------------------------------
! read or create symmetry mask for fit
!-----------------------------------------------------------------------
      if (lrdmask) then
         ilbl=index(guessfile,' ')-1
         write(6,'(/,2a)')&
              'Reading mask from file : ',guessfile(1:ilbl)
         open(iguess,file=guessfile,form='formatted',status='old',&
            err=1000)
         call readmask(mi(miconstmask),mi(mikappamask),mi(milambdamask),&
              mi(migammamask),mi(miiotamask),mi(mimumask),&
              mi(miexpcoemask),mi(mitermkmask))
         close(iguess)
      else
         write(6,'(/,1x,a)') 'Creating mask ...'
         call createmask(mi(miconstmask),mi(mikappamask),&
              mi(milambdamask),mi(migammamask),mi(miiotamask),&
              mi(mimumask),mi(miexpcoemask),mi(mitermkmask),&
              mi(micpcoemask),mi(miswitchmask),mi(micp2d))
      endif

!-----------------------------------------------------------------------
! If a guess-file wasn't read, create one now.
!-----------------------------------------------------------------------
      if (.not. lrdguess) then
         write(6,'(/,1x,a)') 'Creating guess ...'
         call guess(mr(mrenergy),mr(mrqcoo),mr(mrder1),mr(mrdcp),&
              mr(mrder2),mr(mrconst),mr(mrkappa),mr(mrlambda),&
              mr(mrgamma),mr(mriota),mr(mrmu),mr(mrexpcoe),&
              mr(mrhigher),mr(mrtermk),mr(mrcp2d),mi(miconstmask),&
              mi(mikappamask),mi(milambdamask),mi(migammamask),&
              mi(miiotamask),mi(mimumask),mi(miexpcoemask),mi(micp2d))
      endif

!-----------------------------------------------------------------------
! skip fit if not required
!-----------------------------------------------------------------------
!      if (.not. lfit) go to 100
       if (.not. lfit) iter = 0

!-----------------------------------------------------------------------
! add parameters by hand
!-----------------------------------------------------------------------
      if (laddpar) &
           call addparams(mi(miconstmask),mi(mikappamask),&
                mi(milambdamask),mi(migammamask),&
                mi(miiotamask),mi(mimumask),mi(miexpcoemask),mi(mitermkmask))

!-----------------------------------------------------------------------
! remove unwanted modes from fit
!-----------------------------------------------------------------------
      if (nfit.lt.nmodes) &
           call delmodes(mi(miconstmask),mi(mikappamask),&
                mi(milambdamask),mi(migammamask),mi(miiotamask),&
                mi(mimumask),mi(miexpcoemask),mi(mitermkmask),&
                mi(micpcoemask),mi(miswitchmask))

!-----------------------------------------------------------------------
! remove unwanted states from fit
!-----------------------------------------------------------------------
      if (nsfit.lt.nsta) &
         call delstates(mi(miconstmask),mi(mikappamask),&
              mi(milambdamask),mi(migammamask),mi(miiotamask),&
              mi(mimumask),mi(miexpcoemask),mi(mitermkmask),&
              mi(micpcoemask),mi(miswitchmask))

!-----------------------------------------------------------------------
! set up constraints on coeffs. Need to first find out no. of parameters
! modus=2 after setting parameters so that a check is made in
! parstats as to whether any extra parameters have been generated
!-----------------------------------------------------------------------
      ncons=0
      modus=1
      if (lusecons) then
         call parstats(mi(miconstmask),mi(mikappamask),&
              mi(milambdamask),mi(migammamask),mi(miiotamask),&
              mi(mimumask),mi(miexpcoemask),mi(mitermkmask),&
              mi(micpcoemask),mi(miswitchmask),mi(micp2d),modus)

         call setcons(mi(miconstmask),mi(mikappamask),mi(milambdamask),&
              mi(migammamask),mi(miiotamask),&
              mi(mimumask),mi(miexpcoemask),mi(mitermkmask),&
              mi(micpcoemask),mi(miswitchmask),mi(micp2d),mr(mrkappa),mr(mrlambda),&
              mr(mrexpcoe))
         modus=2
     write(*,*)"ntpar",ntpar
     write(*,*)"nvar",nvar
      endif

!-----------------------------------------------------------------------
! Calculate how many parameters
!-----------------------------------------------------------------------
      call parstats(mi(miconstmask),mi(mikappamask),&
           mi(milambdamask),mi(migammamask),mi(miiotamask),&
           mi(mimumask),mi(miexpcoemask),mi(mitermkmask),&
           mi(micpcoemask),mi(miswitchmask),mi(micp2d),modus)

      write(ilog,'(/,a,i5)')&
          'Total number of parameters           :',ntpar
      write(ilog,'(a,i5)')&
          'Number of non-zero parameters        :',ntvar
      write(ilog,'(a,i5)')&
           'Number of parameters to be optimised :',nvar
      write(ilog,'(a,i5)')&
           'Number of constrained parameters     :',ncons

!-----------------------------------------------------------------------
! renumber constraint labels to reflect parameters chosen for fit
!-----------------------------------------------------------------------
      if (ncons .gt. 0) call renumcons(mi(miconstmask),mi(mikappamask),&
              mi(milambdamask),mi(migammamask),mi(miiotamask),&
              mi(mimumask),mi(miexpcoemask),mi(mitermkmask),&
              mi(micpcoemask),mi(miswitchmask))

!-----------------------------------------------------------------------
! if fit is to be done, set constraints
! and zero non-optimised parameters
!
! then fit parameters to adiabatic surfaces
!----------------------------------------------------------------------
      write(6,'(/,a)') ' Initialising parameters ...'
      if (lsetpar) call setpar(mr(mrconst),mr(mrkappa),mr(mrlambda),&
                        mr(mrgamma),mr(mriota),mr(mrmu),mr(mrexpcoe),&
                        mr(mrtermk),mr(mrcpcoe),mr(mrswitch),mr(mrcp2d))

      call initpar(optcoo,mr,mi)

      if ((iter .gt. 0) .or. lgenetica) then
         call get_timer(tid1, 'Fitpar')
         call start_timer(tid1)
         call fitpar(optcoo,mr,mi)
         call stop_timer(tid1)
      else
         fret=vcrmsd(optcoo,mr,mi)
         write(ilog,'(/,a,e16.8)') '  RMSD (guess - data) = ',SQRT(fret/nsets)
         write(6,'(/,a,e16.8)') '  RMSD (guess - data) = ',SQRT(fret/nsets)

      endif

      call stop_timer(tid)

!-----------------------------------------------------------------------
! Output parameters
!-----------------------------------------------------------------------
      open(iout,file=fileout,form='formatted',status='unknown')
      call outpar(iout,mr(mrconst),mr(mrkappa),mr(mrlambda),&
           mr(mrgamma),mr(mriota),mr(mrmu),mr(mrexpcoe),&
           mr(mrhigher),mr(mrtermk),mr(mrcpcoe),mr(mrswitch),mr(mrcp2d))

!-----------------------------------------------------------------------
! write coupling mask
!-----------------------------------------------------------------------
      call outmask(mi(miconstmask),mi(mikappamask),mi(milambdamask),&
           mi(migammamask),mi(miiotamask),mi(mimumask),mi(miexpcoemask),&
           mi(mitermkmask),mi(micpcoemask),mi(miswitchmask))

      close(iout)

!-----------------------------------------------------------------------
! find minima and write to log file
!-----------------------------------------------------------------------
      if (lmin) call findmin(mr,mi)
      if (lminrd0) call findminrd0(mr,mi)

!----------------------------------------------------------------------
! determine some important conical intersections
!----------------------------------------------------------------------
      if (lcalc_low_ci) then
         call calc_low_ci(mr,mi)
      endif

!----------------------------------------------------------------------
! calculates the density of states,
! approximated for harm. oscillatrors
! ---------------------------------------------------------------------
      if (ldensity) then
         open(idens,file=filedens,form='formatted',status='unknown')
         call statedensity(mr(mrconst),mr(mrconst)+2.0)
         close(idens)
      endif

!-----------------------------------------------------------------------
! write some statistics into the log-file
!-----------------------------------------------------------------------
      call outstat(mr(mrenergy),mr(mrconst),mr(mrkappa),mr(mrlambda),&
           mr(mrgamma),mr(mriota),mr(mrmu),mr(mrexpcoe),mr(mrconstini),&
           mr(mrkappaini),mr(mrlambdaini),mr(mrgammaini),mr(mriotaini),&
           mr(mrmuini),mr(mrexpcoeini),mr(mrhigher),mr(mrhigherini),&
           mr(mrtermk),mr(mrtermkini))

!-----------------------------------------------------------------------
! write the Operator file for Quantics
!-----------------------------------------------------------------------
      if (lwrop) then
         write(6,'(/,a)') ' ------->  writing the operator file'
         open(iop,file=fileoper,form='formatted',status='unknown')

         write(ilog,'(/,a)') ' *** Writing operator file ***'
         write(ilog,'(a)') trim(fileoper)
         if (abs(cutoff_op) .lt. 1.0d-8) then
            write(ilog,'(a)') 'All parameters and terms included'
         else
            write(ilog,'(a,f10.6)') &
               'Parameters and terms only included if > ',cutoff_op
         endif

         call writeop(iop,mr(mrconst),mr(mrkappa),mr(mrlambda),&
              mr(mrgamma),mr(mrmu),mr(mriota),mr(mrexpcoe),&
              mr(mrhigher),mr(mrtermk),mi(miconstmask),&
              mi(mikappamask),mi(milambdamask),mi(migammamask),&
              mi(miiotamask),mi(mimumask),mi(miexpcoemask),mi(mitermkmask))

         close(iop)
      endif

!-----------------------------------------------------------------------
! write the parameter file for Quantics
!-----------------------------------------------------------------------
      if (lwrpar) then
         write(6,'(/,a)') ' ------->  writing the parameter file'
         open(ipar,file=filepar,form='formatted',status='unknown')
         call writepar(ipar,mr(mrconst),mr(mrkappa),mr(mrlambda),&
              mr(mrgamma),mr(mrmu),mr(mriota),mr(mrexpcoe),&
              mr(mrhigher),mr(mrtermk),mi(miconstmask),mi(mikappamask),&
              mi(milambdamask),mi(migammamask),mi(miiotamask),&
              mi(mimumask),mi(miexpcoemask),mi(mitermkmask))

         close(ipar)
      endif

!-----------------------------------------------------------------------
! write the Quantics input file
!-----------------------------------------------------------------------
      if (lwrqinp) then
         write(6,'(/,a)') ' ------->  writing the quantics input file'
         open(ipar,file=fileqinp,form='formatted',status='unknown')
         call writeqinp(ipar)
         close(ipar)
      endif

!-----------------------------------------------------------------------
! determine the confidence errors of the parameters
!----------------------------------------------------------------------
      if (lconfidence) call confidence(optcoo,mr(mrconst),mr(mrkappa),&
              mr(mrlambda),mr(mrgamma),mr(mriota),mr(mrmu),mr(mrexpcoe),&
              mi(miconstmask),mi(mikappamask),mi(milambdamask),&
              mi(migammamask),mi(miiotamask),mi(mimumask),&
              mi(miexpcoemask),mi(mitermkmask),&
              mr(mrhess),mr(mrcovar),mr(mrcorrel),mr(mrdpar),&
              mr(mrerrpar),mr,mi)

!-----------------------------------------------------------------------
! output timing to the log file
!-----------------------------------------------------------------------
      call outtiming
      call fpstartup(cstartup)
      write(ilog,'(/a)') trim(cstartup(1:80))

!-----------------------------------------------------------------------
! close files and finish
!-----------------------------------------------------------------------
      close(iin)
      close(ilog)

      cerr = 'vchfit_...'

      deallocate(optcoo,mc,mr,mi,ml,stat=ierr)
      if (ierr > 0) go to 2001

      write(6,'(/,a)') '  FINI '

      stop

  999 continue
      ilbl=index(datfile,' ')-1
      print*
      print*,'ERROR: Data file not found : ',datfile(1:ilbl)
      print*
      stop

 1000 continue
      ilbl=index(guessfile,' ')-1
      print*
      print*,'ERROR: Guess file not found : ',guessfile(1:ilbl)
      print*
      stop

 2000 write (6,*) 'Error in Memory Allocation of ',cerr,' Error code is ',ierr
      stop

 2001 write (6,*) 'Error in Memory Deallocation of ',cerr,' Error code is ',ierr
      stop

      contains

!#######################################################################

      subroutine vchfdefault

      implicit none

      integer(long)       :: i,j,k


      lpoo = .false.
!     default values for the numerical stuff
      ioptim=1
      iter=100
      tol =1.0E-10
      fragnmodes=nmodes-3
      lspecies = .false.
      gaoptim = .false.
      gaoptim2 = .false.
      lforcepos = .false.
      lmodesym = .false.
      covar_adapt = .false.
      perparam = 0
! ! for butatriene
      do i=1,24
         rottype(i)=' '
      enddo

      muti = 0
      comb_nsta = 500
      rottype(1)='z'
      rottype(2)='z'
      rottype(3)='z'

      rottype(4)='z'
      rottype(5)='z'
      rottype(6)='z'

      rottype(19)='z'
      rottype(20)='z'
      rottype(21)='z'

      rottype(22)='z'
      rottype(23)='z'
      rottype(24)='z'

      lparzero=.false.

! Defaults for Genetic Algorithm
      lgenetica = .false.
      lseed = .false.
      min_stdev = 0
      npop = 50
      ngen = 150
      nmut = 0.1
      nseed = 0
      seed = 4
      maxcoup0=4
      muttyp=3
      mutchange=4
      mutchance=0.3
      branch_cutoff=99
      scale_stdev=5.0
      do i=1,maxsta
         do j=1,nmodes
            do k=1,maxcoe
               expcoemut(1,k,j,i)=-999.0
            enddo
         enddo
      enddo
      increase_mut = 0.0
      cc = 0
      old_eval_covar = 0.0
      kurtosis = 0.0
      skew = 0.0
      DPtest = 0.0
      e_st_dev = 0.0
      gencount = 0
      lstats = .false.
! 2x2 roatation of normal modes
      lrotmodes=.false.
!    mutation range:
      kappamut = 0.0
      lambdamut = 0.0
      gammamut = 0.0
      mumut = 0.0
      iotamut = 0.0
      termkmut = 0.0
      expcoemut = 0.0

      offdiagsignopt = .false.


      return
      end subroutine vchfdefault

!#######################################################################

      subroutine vchfinp

      integer(long) :: i,iz,iz0,ic,ierr,j,k,s,ilbl,n,m,m1,s1,ijt,p,l,a,b,c,o
      real(dop), dimension(nmodes) :: sysfreq
      real(dop)                    :: rotang
      character(len=16), dimension(nmodes) :: syslab
      character(len=16) :: string,string1
      logical(kind=4) :: file_exists

      iz0 = 0
      iz = 0
!      print*,'Reading input'
!-----------------------------------------------------------------------
! set initial values
!-----------------------------------------------------------------------
! ___ equal weight is default
      weighttype=3
      wfact=0.0
!     energy_shift=0
      lrdguess=.false.
      lrdmask=.false.
      lusecons=.false.
      lrdcons=.false.
      lsetpar=.false.
      laddpar=.false.
      lfit=.true.
      lwrop = .false.

      lconfidence=.false.

      lcalc_low_ci = .false.
      ldensity     = .false.

      hamord=1
      nfit=0
      nsfit=0
      highord = 0
      nhigher = 0
      nadd = 0

      call zeromxi(termdef,maxdim,maxadd)
      call zeromxi(termst,2,maxadd)
!
! ldiagfit starts out true. Can be set to false in input in which case it
! overides constall, diabfit etc. Is then also set to false if no
! diagonal terms asked for.
!
      lconst=.false.
      lconstall=.false.
      ldirot=.false.
      lkappa=.false.
      lgamma=.false.
      lgamdiag=.false.
      llambda=.false.
      lmu=.false.
      lmudiag=.false.
      liota=.false.
      liotadiag=.false.
      liota2=.false.
      lhigher=.false.
      ldifit=.false.
      ldiagfit=.true.
      lgeneral=.false.
      lparzero=.false.
      lzerozero=.true.
      ldiab=.false.
      lfulllog=.false.
      maxmem = -1
      lcoscoup(:)=.false.
      lautosym=.false.

      lgeneralcp=.false.
      qtransform=0
      lder2_dcp = .true.

      call zerovxi(jtsta,maxsta)

      call zeromxi(cointype,maxsta,maxsta)

      njt=0
      symmode = '   '
      do s=1,maxsta
         jtmode(s) = '   '
         do s1=1,maxsta
            cpmode(s1,s) = '   '
            cpmode(s,s1) = '   '
            ncp2(s,s1) = 0
            ncp2(s1,s) = 0
            do n = 1,10
               cpmode2(s1,s,1,n) = '   '
               cpmode2(s1,s,2,n) = '   '
               cpmode2(s,s1,1,n) = '   '
               cpmode2(s,s1,2,n) = '   '
            enddo
         enddo
      enddo

      lfit=.true.
      nfit=0
      call zerovxi(mfit,maxdim)

!-----------------------------------------------------------------------
! get file names
!-----------------------------------------------------------------------
      ilbl=index(filein,'.')-1
      if (ilbl .le. 0) then
         ilbl=index(filein,' ')-1
         filein=filein(1:ilbl)//'.inp'
      endif
      fileout  = filein(1:ilbl)//'.vcham'
      filelog  = filein(1:ilbl)//'.log'
      filepar  = filein(1:ilbl)//'.par'
      fileoper   = filein(1:ilbl)//'.op'
      filedens = filein(1:ilbl)//'.dens'
      fileqinp = filein(1:ilbl)//'.qinp'

!-----------------------------------------------------------------------
! Open files
!-----------------------------------------------------------------------
      open(iin,file=filein,form='formatted',status='old',err=1010)

!-----------------------------------------------------------------------
! read input file
!-----------------------------------------------------------------------
   10 continue
      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      routine = 'vchfinp (read VCHFIT input)'
      call vccntrl(ierr)
      i=1
   15 continue
      if (keyword(i) .eq. 'end-input') then
         goto 200

      else if (keyword(i) .eq. 'infofile') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            datfile=keyword(i)
         else
            routine = 'vchfinp (read VCHFIT input)'
            message = 'No argument specified with infofile keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i) .eq. 'root') then
           ldiab = .true.
      else if (keyword(i) .eq. 'cpmode') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            if (ic .lt. i+2) then
               routine = 'vchfinp (read VCHFIT input)'
               message = 'Not enough arguments with cpmode keyword'
               call vcwrline(0,iz,iz0)
            else
               read(keyword(i),*) s
               read(keyword(i+1),*) s1
               cpmode(s,s1)=keyorig(i+2)(1:c1)
               cpmode(s1,s)=keyorig(i+2)(1:c1)
               i=i+2
            endif
         else
            routine = 'vchfinp (read VCHFIT input)'
            message = 'No argument specified with cpmode keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i) .eq. 'symmode') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            symmode=keyorig(i)(1:c1)
         else
            routine = 'vchfinp (read VCHFIT input)'
            message = 'No argument specified with symmode keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i) .eq. 'jtmode') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            if (ic .lt. i+1) then
               routine = 'vchfinp (read VCHFIT input)'
               message = 'Not enough arguments with jtmode keyword'
               call vcwrline(0,iz,iz0)
            else
               read(keyword(i),*) s
               jtmode(s)=keyorig(i+1)(1:c1)
               i=i+1
            endif
         else
            routine = 'vchfinp (read VCHFIT input)'
            message = 'No argument specified with jtmode keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i).eq.'jtstates') then
         if (keyword(i+1).eq.'=' .and. keyword(i+3) .eq. ',') then
            read(keyword(i+2),*,iostat=ierr) s
            read(keyword(i+4),*,iostat=ierr) s1
            njt=njt+1
            jtsta(s)=njt
            jtsta(s1)=njt
            i=i+4
   20       continue
            if (keyword(i+1).eq.',') then
               read(keyword(i+2),*,iostat=ierr) s1
               jtsta(s1)=njt
               i=i+2
               go to 20
            endif

            if (ierr .ne. 0) then
               routine = 'vchfinp (read VCHFIT input)'
               message ='jtstates must be defined by integers.'
               call vcwrline(i,iz,iz0)
            endif

         else
            routine = 'vchfinp (read VCHFIT input)'
            message = 'No argument specified with jtstates keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i) .eq. 'cpmode2') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            if (ic .lt. i+3) then
               routine = 'vchfinp (read VCHFIT input)'
               message = 'Not enough arguments with cpmode2 keyword'
               call vcwrline(0,iz,iz0)
            else
               read(keyword(i),*) s
               read(keyword(i+1),*) s1
               ncp2(s,s1) = ncp2(s,s1)+1
               ncp2(s1,s) = ncp2(s,s1)
               n = ncp2(s1,s)
               cpmode2(s,s1,1,n)=keyorig(i+2)(1:c1)
               cpmode2(s1,s,1,n)=keyorig(i+2)(1:c1)
               cpmode2(s1,s,2,n)=keyorig(i+3)(1:c1)
               cpmode2(s,s1,2,n)=keyorig(i+3)(1:c1)
               i=i+3
            endif
         else
            routine = 'vchfinp (read VCHFIT input)'
            message = 'No argument specified with cpmode2 keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i) .eq. 'guess') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            guessfile=keyorig(i)
            lrdguess=.true.
         else
            routine = 'vchfinp (read VCHFIT input)'
            message = 'No argument specified with guess keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i) .eq. 'read_mask') then
         lrdmask=.true.

      else if (keyword(i)(1:8) .eq. 'use_cons') then
         lusecons=.true.


! determine the Hamiltonian order
      else if (keyword(i) .eq. 'linear_mod') then
         hamord=1
      else if (keyword(i) .eq. 'adiab2_mod') then
         hamord=2
      else if (keyword(i) .eq. 'diab2_mod') then
         hamord=3

! mix specific diabatic states at Q0
      else if (keyword(i) .eq. 'diab_mix') then
         if (keyword(i+1).eq.'=' .and. keyword(i+3) .eq. ','&
          .and. keyword(i+5) .eq. ',') then
            read(keyword(i+2),*,iostat=ierr) s
            read(keyword(i+4),*,iostat=ierr) s1
            read(keyword(i+6),*,iostat=ierr) rotang
            rotang = rotang*pi/180.0
            dirot(s,s)=cos(rotang)
            dirot(s1,s1)=cos(rotang)
            dirot(s1,s)=sin(rotang)
            dirot(s,s1)=-dirot(s1,s)
            ldirot=.true.
            i=i+6
            if (ic .lt. i) then
               routine = 'vchfinp (read VCHFIT input)'
               message = 'Not enough arguments with diab_mix keyword'
               call vcwrline(0,iz,iz0)
            endif

         else
            routine = 'vchfinp (read VCHFIT input)'
            message = 'No argument specified with diab_mix keyword'
            call vcwrline(0,iz,iz0)
         endif

!     determine the fittype
      else if (keyword(i).eq.'linfit') then
         lconst=.true.
         lkappa=.true.
         llambda=.true.
      else if (keyword(i).eq.'fullfit') then
         lconst=.true.
         lkappa=.true.
         llambda=.true.
         lgamma=.true.
         lmu=.true.
      else if (keyword(i).eq.'constfit') then
         lconst=.true.
      else if (keyword(i).eq.'constall') then
         lconstall=.true.
         lconst=.true.
      else if (keyword(i).eq.'kappafit') then
         lkappa=.true.
      else if (keyword(i).eq.'gammafit ') then
         lgamma=.true.
         ldifit=.true.
      else if (keyword(i).eq.'iotafit') then
         liota=.true.
         hamord = 4
      else if (keyword(i).eq.'iotafit_diag') then
         liota=.true.
         liotadiag=.true.
         hamord = 4
      else if (keyword(i).eq.'iota2fit ') then
         liota2=.true.
         hamord = 4
      else if (keyword(i).eq.'poofit ') then
         lpoo=.true.
      else if (keyword(i).eq.'highfit ') then
         lhigher=.true.
         hamord = 4
         if (keyword(i+1).eq.'= ') then
            i=i+2
            read(keyword(i),*) highord
         else
            highord = 3
         endif
      else if (keyword(i).eq.'gammafit_diag') then
         lgamdiag=.true.
         ldifit=.true.
      else if (keyword(i).eq.'lambdafit') then
         llambda=.true.
      else if (keyword(i).eq.'mufit ') then
         lmu=.true.
      else if (keyword(i).eq.'mufit_diag') then
         lmudiag=.true.
         lmu=.true.
      else if (keyword(i).eq.'diabfit') then
         ldifit=.true.
      else if (keyword(i).eq.'nodiabfit') then
         ldifit=.false.
      else if (keyword(i).eq.'nodiag') then
         ldiagfit=.false.

      else if (keyword(i).eq.'nonopt_zero') then
         lparzero=.true.
      else if (keyword(i).eq.'nozero_zero') then
         lzerozero=.false.
      else if (keyword(i).eq.'sin_coupling') then
         lsincoup = .true.
      else if (keyword(i).eq.'cos_coupling') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*) m
            lcoscoup(m) = .true.
          endif
            write(*,*)lcoscoup(:)

!     determine diabatic state function, if not HO
      else if (keyword(i) .eq. 'diabatic_function ') then
         if (i .ne. 1) then
            routine = 'vchfinp (read VCHFIT input)'
            message='Diabatic_function keyword must be alone on line'
            call vcwrline(i,iz,iz0)
         else
            lgeneral=.true.
            n=0
 30         continue
            call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,&
                 maxkey)
            routine = 'vchfinp (read VCHFIT input)'
            call vccntrl(ierr)

            if (keyword(1) .ne. 'end-define') then
               n=n+1
               if (ic .eq. 3) then
                  read(keyword(1),*) m
                  read(keyword(2),*) s
                  if (keyword(3) .eq. 'quartic') then
                     ifdiab(m,s)=1
                     ncoeff(m,s)=3
                  else if (keyword(3) .eq. 'morse') then
                     ifdiab(m,s)=2
                     ncoeff(m,s)=3
                  else if (keyword(3) .eq. 'buckingham') then
                     ifdiab(m,s)=3
                     ncoeff(m,s)=7
                  else if (keyword(3) .eq. 'anti-morse' .or. &
                        keyword(3) .eq. 'antimorse') then
                     ifdiab(m,s)=4
                     ncoeff(m,s)=3
                  else if (keyword(3) .eq. 'exp') then
                     ifdiab(m,s)=5
                     ncoeff(m,s)=2
                  else if (keyword(3) .eq. 'avoided_crossing') then
                     ifdiab(m,s)=6
                     ncoeff(m,s)=8
                  else if (keyword(3) .eq. 'general_morse') then
                     ifdiab(m,s)=7
                     ncoeff(m,s)=3
                  else if (keyword(3) .eq. 'general_harmonic') then
                     ifdiab(m,s)=8
                     ncoeff(m,s)=1
                  else if (keyword(3) .eq. 'general_quartic') then
                     ifdiab(m,s)=9
                     ncoeff(m,s)=3
                  else if (keyword(3) .eq. 'lennard-jones') then
                     ifdiab(m,s)=10
                     ncoeff(m,s)=3
                  else if (keyword(3) .eq. 'anti-lennard-jones') then
                     ifdiab(m,s)=11
                     ncoeff(m,s)=3
                  else if (keyword(3) .eq. 'general_aquartic') then
                     ifdiab(m,s)=12
                     ncoeff(m,s)=4
                  else if (keyword(3) .eq. 'morse_lorentzian') then
                     ifdiab(m,s)=13
                     ncoeff(m,s)=7
                  else if (keyword(3) .eq. 'morse_gaussian') then
                     ifdiab(m,s)=14
                     ncoeff(m,s)=7
                  else if (keyword(3) .eq. 'general_quadratic') then
                     ifdiab(m,s)=15
                     ncoeff(m,s)=2
                  else if (keyword(3) .eq. 'avoided_crossing2') then
                     ifdiab(m,s)=16
                     ncoeff(m,s)=9
                  else if (keyword(3) .eq. 'anti_avoided_crossing2') then
                     ifdiab(m,s)=24
                     ncoeff(m,s)=9
                  else if (keyword(3) .eq. 'avoided_crossing_upper2') then
                     ifdiab(m,s)=17
                     ncoeff(m,s)=9
                  else if (keyword(3) .eq. 'avoided_crossing_quartic') then
                     ifdiab(m,s)=18
                     ncoeff(m,s)=6
                  else if (keyword(3) .eq. 'jt4') then
                     ifdiab(m,s)=19
                     ncoeff(m,s)=8
                  else if (keyword(3) .eq. 'rrp') then
                     ifdiab(m,s)=20
                     ncoeff(m,s)=3
                  else if (keyword(3) .eq. 'constant') then
                     ifdiab(m,s)=22
                     ncoeff(m,s)=1
                  else if (keyword(3) .eq. '1/r') then
                     ifdiab(m,s)=23
                     ncoeff(m,s)=2
                  endif
               else
                  routine = 'vchfinp (read VCHFIT input)'
                  message='Error reading diabatic function definition'
                  call vcwrline(i,iz,iz0)
               endif
               go to 30
            endif
         endif
! determine mono-mode off-diagonal element function, if not a
! Taylor expansion
      else if (keyword(i) .eq. 'coupling_function') then
         if (i .ne. 1) then
            routine = 'vchfinp (read VCHFIT input)'
            message='coupling_function keyword must be alone on line'
            call vcwrline(i,iz,iz0)
         else
            lgeneralcp=.true.
            n=0
40          continue
            call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,&
                 maxkey)
            routine = 'vchfinp (read VCHFIT input)'
            call vccntrl(ierr)
            if (keyword(1) .ne. 'end-coupling_function') then
               n=n+1
                if (ic.eq.4) then
                   read(keyword(1),*) m
                   read(keyword(2),*) s
                   read(keyword(3),*) s1
                   if (keyword(4) .eq. 'tanh') then
                      ifdiabcp(m,s,s1)=1
                      ncoeffcp(m,s,s1)=2
                   else if (keyword(4) .eq. 'poly_to_tanh') then
                      ifdiabcp(m,s,s1)=2
                      ncoeffcp(m,s,s1)=5
                   else if (keyword(4) .eq. 'tanh2') then
                      ifdiabcp(m,s,s1)=3
                      ncoeffcp(m,s,s1)=3
                   endif
                   ifdiabcp(m,s1,s)=ifdiabcp(m,s,s1)
                   ncoeffcp(m,s1,s)=ncoeffcp(m,s,s1)
                elseif (ic.eq.5) then
                   read(keyword(1),*) m
                   read(keyword(2),*) m1
                   read(keyword(3),*) s
                   read(keyword(4),*) s1
                   k = 1
                   do while (cp2dtyp(k) .ne. 0 )
                      k = k + 1
                   enddo
                   if (keyword(5) .eq. 'ltanh2d' .or. keyword(5) .eq. 'mtanh2d'  &
             .or. keyword(5) .eq. 'radtanch' .or.  keyword(5) .eq. 'vectanch' ) then
!                   this type only works for coupling between totally symmetryic modes
                     if (keyword(5) .eq. 'ltanh2d') cp2dtyp(k) = 1
                     if (keyword(5) .eq. 'mtanh2d') cp2dtyp(k) = 2
                     if (keyword(5) .eq. 'radtanch') cp2dtyp(k) = 3
                     if (keyword(5) .eq. 'vectanch') cp2dtyp(k) = 4
                     stacp2d(1,k) = s
                     stacp2d(2,k) = s1
                     dofcp2d(1,k) = m
                     dofcp2d(2,k) = m1
                   endif
                else
                   routine = 'vchfinp (read VCHFIT input)'
                   message='Error reading coupling function definition'
                   call vcwrline(i,iz,iz0)
                endif
                goto 40
             endif
          endif
      else if (keyword(i) .eq. '2d_function') then
         if (i .ne. 1) then
            routine = 'vchfinp (read VCHFIT input)'
            message='coupling_function keyword must be alone on line'
            call vcwrline(i,iz,iz0)
         else
            lgeneralcp=.true.
            n=0
44          continue
            call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,&
                 maxkey)
            routine = 'vchfinp (read VCHFIT input)'
            call vccntrl(ierr)
            if (keyword(1) .ne. 'end-coupling_function') then
               n=n+1
                if (ic.eq.6) then
                   read(keyword(1),*) m
                   read(keyword(2),*) m1
                   read(keyword(3),*) s
                   read(keyword(4),*) s1
                   read(keyword(5),*) o
                   k = 1
                   do while (cp2dtyp(k) .ne. 0 )
                      k = k + 1
                   enddo
                   if( keyword(5) .eq. 'polybox' ) then
!                   this type only works for coupling between totally symmetryic modes
                     if (keyword(5) .eq. 'polybox') cp2dtyp(k) = 5
                     if (keyword(5) .eq. 'polybox2') cp2dtyp(k) = 6
                     stacp2d(1,k) = s
                     stacp2d(2,k) = s1
                     dofcp2d(1,k) = m
                     dofcp2d(2,k) = m1
                     ocp2d(k) = o
                   endif
                else
                   routine = 'vchfinp (read VCHFIT input)'
                   message='Error reading coupling function definition'
                   call vcwrline(i,iz,iz0)
                endif
                goto 44
             endif
          endif

! determine number of switching functions
      else if (keyword(i) .eq. 'switching_function') then
         lswitch=.true.
43       continue
         call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,&
              maxkey)
         if (keyword(1).ne.'end-switching_function') then
            nswitch=nswitch+1
            goto 43
         endif

      else if (keyword(i) .eq. 'state_symmetry') then
         lautosym=.true.
 330     continue
         call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         routine = 'vchfinp (read VCHFIT input)'
         call vccntrl(ierr)

         if (keyword(1) .ne. 'end-state_symmetry') then
            read(keyword(1),*) s
            read(keyword(2),'(a)') stalab(s)
            goto 330
         endif

      else if (keyword(i) .eq. 'mode_symmetry') then
         lmodesym=.true.
         freq0 = -1
 331     continue
         call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
         routine = 'vchfinp (read VCHFIT input)'
         call vccntrl(ierr)
         if (keyword(1) .ne. 'end-mode_symmetry') then
            read(keyword(1),*) s
            read(keyword(2),'(a)') nmlab(s)
            if(ic .gt. 2) read(keyword(3),*) freq0(s)
            goto 331
         endif


      else if(keyword(i).eq.'point_group') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),'(a)') pntgrp
         endif

      else if (keyword(i).eq.'force_positive') then
         if (lforcepos .eqv. .false. ) &
         call zerovxi(forcepos,maxdim*maxdim*maxsta)
         lforcepos = .true.
         if (keyword(i+1).eq.'=') then
            i = i + 2
            if ( keyword(i+4) .eq. ',' .or. ( ((i+3)) .eq. ic)) then
155            read(keyword(i),*) j
               read(keyword(i+1),*) m
               read( keyword(i+2),*) s
               if ( j .lt. m ) then
!       swapp j and m (upper diagonal)
                  k = m
                  m = j
                  j = k
               endif
!       forcepos(mode,mode,state) = order (that one wishes to be forced
!       positive; eg. kappa is 1 , gamma is 2 , Ax^3y^3 is 6 etc..)
               if(len(trim(keyword(i+3))) .eq. 1 ) &
                    read(keyword(i+3),'(I1)') forcepos(j,m,s)
               if(len(trim(keyword(i+3))) .eq. 2 ) &
                    read(keyword(i+3),'(I2)') forcepos(j,m,s)
               i = i + 5
               if (keyword(i-1).eq.',') go to 155
            else
               print*,'Error in force_positive format input'
            endif
         endif

      else if (keyword(i) .eq. 'transform_coord ') then
         if (i .ne. 1) then
            routine = 'vchfinp (read VCHFIT input)'
            message='transform_coord keyword must be alone on line'
            call vcwrline(i,iz,iz0)
         else
            lqtransform=.true.
160         continue
            call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
            routine = 'vchfinp (read VCHFIT input)'
            call vccntrl(ierr)

            if (keyword(1) .ne. 'end-transform_coord') then
               if (ic .eq. 2) then
                  read(keyword(1),*) m
                  if (keyword(2) .eq. 'cos') then
                     qtransform(m) = 1
                  else if (keyword(2) .eq. 'sin') then
                     qtransform(m) = 2
                  else if (keyword(2) .eq. 'halfcos') then
                     qtransform(m) = 3
                  else if (keyword(2) .eq. 'halfsin') then
                     qtransform(m) = 4
                  endif

               else
                  routine = 'vchfinp (read VCHFIT input)'
                  message='Error reading coordinate transformation definition'
                  call vcwrline(i,iz,iz0)
               endif
               go to 160
            endif
         endif
! determine mono-mode off-diagonal element function, if not a

!     determine the weight-type for the fit
      else if (keyword(i).eq.'energyweight') then
         weighttype = 2
         wfact=1.0
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*) wfact
         endif

      else if (keyword(i).eq.'energyweight_tanh') then
         weighttype = 4
         wfact=1.0
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*) wfact
         endif

      else if (keyword(i).eq.'positionweight') then
         weighttype = 1
         wfact=2.0
         if (keyword(i+1) .eq. '=') then
            i=i+2
            read(keyword(i),*) wfact
         endif
      else if (keyword(i).eq.'equalweight') then
         weighttype = 3


!     use internal coordinates
      else if (keyword(i) .eq.'torsion') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            if (ic .lt. i+4) then
               routine = 'vchfinp (read VCHFIT input)'
               message = 'Not enough arguments with torsion keyword'
               call vcwrline(0,iz,iz0)
            else
               read(keyword(i),*) m
               cootyp(m) = 1
               read(keyword(i+1),*) coodef(1,m)
               read(keyword(i+2),*) coodef(2,m)
               read(keyword(i+3),*) coodef(3,m)
               read(keyword(i+4),*) coodef(4,m)
               i=i+4
            endif
         else
            routine = 'vchfinp (read VCHFIT input)'
            message = 'No argument specified with torsion keyword'
            call vcwrline(0,iz,iz0)
         endif

!     adjustments
      else if (keyword(i).eq.'eshift') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*) e_shift
            if (keyword(i+1) .eq. ',') then
               i=i+2
               call keyeunits(e_shift,keyword(i),ierr)
               if (ierr .ne. 0) then
                  routine = 'vchfinp (read VCHFIT input)'
                  message = 'Incorrect unit for energy : '
                  call vcwrline(1,iz,iz0)
               endif
            endif
            e_shift=e_shift*ev

         else
            routine = 'vchfinp (read VCHFIT input)'
            message = 'No argument specified with eshift keyword'
            call vcwrline(0,iz,iz0)
         endif

!     set upper bound for data set energies
      else if (keyword(i).eq.'eupper') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*) edsup
            if (keyword(i+1) .eq. ',') then
               i=i+2
               call keyeunits(edsup,keyword(i),ierr)
               if (ierr .ne. 0) then
                  routine = 'vchfinp (read VCHFIT input)'
                  message = 'Incorrect unit for energy : '
                  call vcwrline(1,iz,iz0)
               endif
            endif
            edsup=edsup*ev

         else
            routine = 'vchfinp (read VCHFIT input)'
            message = 'No argument specified with eupper keyword'
            call vcwrline(0,iz,iz0)
         endif

!     set lower bound for data set energies
      else if (keyword(i).eq.'elower') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*) edslow
            if (keyword(i+1) .eq. ',') then
               i=i+2
               call keyeunits(edslow,keyword(i),ierr)
               if (ierr .ne. 0) then
                  routine = 'vchfinp (read VCHFIT input)'
                  message = 'Incorrect unit for energy : '
                  call vcwrline(1,iz,iz0)
               endif
            endif
            edslow=edslow*ev

         else
            routine = 'vchfinp (read VCHFIT input)'
            message = 'No argument specified with elower keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i).eq.'qshift') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*) q_shift
            if (keyword(i+1) .eq. ',') then
               i=i+2
               call keylunits(q_shift,keyword(i),ierr)
               if (ierr .ne. 0) then
                  routine = 'vchfinp (read VCHFIT input)'
                  message = 'Incorrect unit for length : '
                  call vcwrline(1,iz,iz0)
               endif
            endif

         else
            routine = 'vchfinp (read VCHFIT input)'
            message = 'No argument specified with qshift keyword'
            call vcwrline(0,iz,iz0)
         endif

!     set bounds for a coordinate
      else if (keyword(i).eq.'qbound') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*) m
            read(keyword(i+1),*) qbnd(m,1)
            i=i+1
            if (keyword(i+1) .eq. ',') then
               i=i+2
               call keylunits(qbnd(m,1),keyword(i),ierr)
               if (ierr .ne. 0) then
                  routine = 'vchfinp (read VCHFIT input)'
                  message = 'Incorrect unit for length : '
                  call vcwrline(1,iz,iz0)
               endif
            endif
            read(keyword(i+1),*) qbnd(m,2)
            i=i+1
            if (keyword(i+1) .eq. ',') then
               i=i+2
               call keylunits(qbnd(m,1),keyword(i),ierr)
               if (ierr .ne. 0) then
                  routine = 'vchfinp (read VCHFIT input)'
                  message = 'Incorrect unit for length : '
                  call vcwrline(1,iz,iz0)
               endif
            endif

         else
            routine = 'vchfinp (read VCHFIT input)'
            message = 'No argument specified with xbonbd keyword'
            call vcwrline(0,iz,iz0)
         endif

!     numerical settings
      else if (keyword(i).eq.'iter') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*) iter
         else
            routine = 'vchfinp (read VCHFIT input)'
            message = 'No argument specified with iter keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i).eq.'tol') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*) tol
         else
            routine = 'vchfinp (read VCHFIT input)'
            message = 'No argument specified with tol keyword'
            call vcwrline(0,iz,iz0)
         endif

      elseif(keyword(i) .eq. 'mut_kappa') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) kappamut(1)
            read(keyword(i+1),*,iostat=ierr) kappamut(2)
            i=i+2
         else
            routine = 'vchfinp (read VCHFIT input)'
            message = 'Invalid argument specified with mut_** keyword'
            call vcwrline(0,iz,iz0)
         endif

      elseif(keyword(i) .eq. 'mut_lambda') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) lambdamut(1)
            read(keyword(i+1),*,iostat=ierr) lambdamut(2)
!           print*, 'got = ', lambdamut(2), lambdamut(1)
            i=i+2
         else
            routine = 'vchfinp (read VCHFIT input)'
            message = 'Invalid argument specified with mut_** keyword'
            call vcwrline(0,iz,iz0)
         endif

      elseif(keyword(i) .eq. 'mut_gamma') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) gammamut(1)
            read(keyword(i+1),*,iostat=ierr) gammamut(2)
            i=i+2
         else
            routine = 'vchfinp (read VCHFIT input)'
            message = 'Invalid argument specified with mut_** keyword'
            call vcwrline(0,iz,iz0)
         endif

      elseif(keyword(i) .eq. 'mut_mu') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) mumut(1)
            read(keyword(i+1),*,iostat=ierr) mumut(2)
            i=i+2
         else
            routine = 'vchfinp (read VCHFIT input)'
            message = 'Invalid argument specified with mut_** keyword'
            call vcwrline(0,iz,iz0)
         endif

      elseif(keyword(i) .eq. 'mut_termk') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) termkmut(1)
            read(keyword(i+1),*,iostat=ierr) termkmut(2)
            i=i+2
         else
            routine = 'vchfinp (read VCHFIT input)'
            message = 'Invalid argument specified with mut_** keyword'
            call vcwrline(0,iz,iz0)
         endif
      elseif(keyword(i) .eq. 'mut_general') then
         if (keyword(i+4).eq.'=') then
            read(keyword(i+1),*,iostat=ierr) k
            read(keyword(i+2),*,iostat=ierr) m
            read(keyword(i+3),*,iostat=ierr) s
            i=i+5
            read(keyword(i),*,iostat=ierr) expcoemut(1,k,m,s)
            read(keyword(i+1),*,iostat=ierr) expcoemut(2,k,m,s)
!            print*, 'general k, m, s, 1 2 = ', k, m, s, expcoemut(1,k,m,s), expcoemut(2,k,m,s)
            i=i+2
         else
            routine = 'vchfinp (read VCHFIT input)'
            message = 'Invalid argument specified with mut_** keyword'
            call vcwrline(0,iz,iz0)
         endif

      elseif(keyword(i)(1:3) .eq. 'CMA' .or. &
           keyword(i)(1:10) .eq. "covariance") then
         covar_adapt = .true.
      elseif(keyword(i) .eq. 'Stat' .or. &
           keyword(i) .eq. "stat") then
          lstats = .true.
      elseif(keyword(i) .eq. 'speciate') then
         lspecies = .true.
         if (keyword(i+1).eq.'=') then
            i = i + 2
            read(keyword(i),*,iostat=ierr) perparam
            if(i .ne. ic)  read(keyword(i+1),'(I3)') branch_cutoff
            if(i .ne. ic) i = ic
         else
            routine = 'vchfinp (read VCHFIT input)'
            message = 'No argument specified with speciate keyword'
            call vcwrline(0,iz,iz0)
         endif
      else if (keyword(i).eq.'rngseed') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*) seed
         else
            routine = 'vchfinp (read VCHFIT input)'
            message = 'No argument specified with tol keyword'
            call vcwrline(0,iz,iz0)
         endif
      elseif(keyword(i) .eq. 'ignore_print_modes' ) then

         if (keyword(i+1).eq.'=') then
157        i = i + 2
           if (keyword(i+1) .eq. ',' .or. (i .eq. ic))then
             read(keyword(i),'(I2)') k
             ignoredof(k) = .true.
!             print*, 'ignore mode = ', k
             i_ignore = i_ignore + 1
             goto 157
           endif
           k = 0
           do j = 1, maxdim
            if(.not. ignoredof(j) ) k = k + 1
            if(.not. ignoredof(j) ) adof(k) = j
!            print*, 'ACT_DOF =', adof(k)
           enddo
         endif
      elseif(keyword(i).eq.'mutchance') then
         i=i+2
         read(keyword(i) ,'(F4.2)') mutchance

      elseif(keyword(i).eq.'scale_stdev') then
         i=i+2
         read(keyword(i) ,'(F4.2)') scale_stdev

      else if ((keyword(i).eq.'optimiser') .or. (keyword(i) .eq. 'optimizer')) then
         if (keyword(i+1).eq.'=') then
            i=i+2
            if (keyword(i) .eq. 'conj_grad') then
               ioptim=1
            else if (keyword(i) .eq. 'powell') then
               ioptim=2
            else if (keyword(i) .eq. 'simplex') then
               ioptim=3
            else if (keyword(i) .eq. 'bfgs') then
               ioptim=4
            elseif (keyword(i) .eq. 'offdiag_sign_opt') then
!       see routine  mutate
               lgenetica = .true.
               lspecies = .false.
               offdiagsignopt = .true.
            else if (keyword(i) .eq. 'genetic') then
               lgenetica=.true.
               ioptim=3
            endif
         else
            routine = 'vchfinp (read VCHFIT input)'
            message = 'No argument specified with optimiser keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i) .eq. 'no-ortho') then
         lortho=.false.
      else if (keyword(i).eq.'confidence') then
         lconfidence=.true.

      else if (keyword(i).eq.'nofit') then
         lfit=.false.
         nfit=0
         call zerovxi(mfit,maxdim)
      else if (keyword(i).eq.'fit_all') then
         lfit=.true.
         nfit=nmodes
         do m=1,nmodes
            mfit(m)=1
         enddo

      else if (keyword(i).eq.'fit_select') then
         lfit=.true.
         if (keyword(i+1).eq.'=') then
            call zerovxi(mfit,maxdim)
   60       continue
            nfit=nfit+1
            i=i+2
            read(keyword(i),*,iostat=ierr) m
            mfit(m)=1
            if (ierr .ne. 0) then
               routine = 'vchfinp (read VCHFIT input)'
               message ='selected mode must be an integer. Not :'
               call vcwrline(i,iz,iz0)
            endif

            if (keyword(i+1).eq.',') go to 60
         else
            routine = 'vchfinp (read VCHFIT input)'
            message = 'No argument specified with fit_select keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i).eq.'state_select') then
         if (keyword(i+1).eq.'=') then
            call zerovxi(sfit,maxsta)
 110        continue
            nsfit=nsfit+1
            i=i+2
            read(keyword(i),*,iostat=ierr) s
            sfit(s)=1
            if (keyword(i+1).eq.',') go to 110
         else
            routine = 'vchfinp (read VCHFIT input)'
            message = 'No argument specified with state_select keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i).eq.'minima' .or. keyword(i).eq.'findmin') then
         lmin=.true.
      else if (keyword(i).eq.'minima_in_x') then
         lmininx=.true.
      else if (keyword(i).eq.'nomin') then
         lmin=.false.
      else if (keyword(i).eq.'minima_rd0') then
         lminrd0=.true.
      else if (keyword(i).eq.'quantics_op') then
         lwrop=.true.
      else if (keyword(i).eq.'quantics_par') then
         lwrpar=.true.
      else if (keyword(i).eq.'quantics_inp') then
         lwrqinp=.true.
      else if (keyword(i).eq.'calc_low_ci') then
         lcalc_low_ci=.true.
!     calculates the density of state
      else if (keyword(i).eq.'density') then
         ldensity=.true.

      else if (keyword(i).eq.'cutoff_op') then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*,iostat=ierr) cutoff_op
         else
            cutoff_op = 0.0001
         endif

      else if (keyword(i).eq.'constraints-section') then
         lrdcons=.true.
   70    continue
         call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,&
              maxkey)
         routine = 'vchfinp (read VCHFIT input)'
         call vccntrl(ierr)

         if (keyword(1) .ne. 'end-constraints-section') go to 70

      else if (keyword(i).eq.'set_parameters-section') then
         lsetpar=.true.
   80    continue
         call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,&
              maxkey)
         routine = 'vchfinp (read VCHFIT input)'
         call vccntrl(ierr)

         if (keyword(1) .ne. 'end-set_parameters-section') go to 80

      else if (keyword(i).eq.'addpar-section' .or. &
         keyword(1) .eq. 'add_parameters-section') then
         laddpar=.true.
   90    continue
         call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,&
              maxkey)
         routine = 'vchfinp (read VCHFIT input)'
         call vccntrl(ierr)
         if (keyword(1).eq.'explicit') nadd=nadd+1

         if (keyword(1) .ne. 'end-addpar-section' .and.&
             keyword(1) .ne. 'end-add_parameters-section') go to 90

      else if ((keyword(i) .eq. 'seeds') .or. (keyword(i) .eq. 'Seeds')&
         .or. (keyword(i) .eq. 'seed') .or. (keyword(i) .eq. 'Seed'))&
        then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*) nseed
            lseed=.true.
         else
            routine = 'vchfinp (read VCHFIT input)'
            message = 'No argument specified with seeds keyword'
            call vcwrline(0,iz,iz0)
         endif
      else if ((keyword(i) .eq. 'population') .or. (keyword(i) &
       .eq. 'Population')) then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*) npop
         else
            routine = 'vchfinp (read VCHFIT input)'
            message = 'No argument specified with population keyword'
            call vcwrline(0,iz,iz0)
         endif
      else if ((keyword(i) .eq. 'generations') .or. (keyword(i)&
       .eq. 'Generations')) then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*) maxgen
            maxgen = maxgen
         else
            routine = 'vchfinp (read VCHFIT input)'
            message = 'No argument specified with population keyword'
            call vcwrline(0,iz,iz0)
         endif
      else if ((keyword(i) .eq. 'mutate') .or. (keyword(i) &
       .eq. 'Mutate')) then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*) nmut
         else
            routine = 'vchfinp (read VCHFIT input)'
            message = 'No argument specified with mutate keyword'
            call vcwrline(0,iz,iz0)
         endif
      else if ((keyword(i) .eq. 'mutation_type') &
       .or. (keyword(i) .eq. 'mutate_type')) then
         if (keyword(i+1) .eq. '=') then
           i=i+2
           read(keyword(i),*) muttyp
           i=i+1
           read(keyword(i),*) mutchange
         else
            routine = 'vchfinp (read VCHFIT input)'
            message = 'No argument specified with mutation_type keyword'
            call vcwrline(0,iz,iz0)
         endif
      else if ((keyword(i) .eq. 'mutation_chance')&
       .or. (keyword(i) .eq. 'mutate_chance')) then
         if (keyword(i+1) .eq. '=') then
           i=i+2
           read(keyword(i),*) mutchance
         else
            routine = 'vchfinp (read VCHFIT input)'
            message = 'No argument specified with mutation_chance keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if ((keyword(i) .eq. 'fulllog') .or. &
       (keyword(i) .eq. 'Fulllog')) then
           lfulllog = .true.

      else if ((keyword(i) .eq. 'Max-Coupling') .or. (keyword(i) &
       .eq. 'max-coupling')) then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*) maxcoup0
         else
            routine = 'vchfinp (read VCHFIT input)'
            message = 'No argument specified with Max-Coupling keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if ((keyword(i) .eq. 'maxmem') .or.&
       (keyword(i) .eq. 'Maxmem')) then
         if (keyword(i+1).eq.'=') then
            i=i+2
            read(keyword(i),*) maxmem
         else
            routine = 'vchfinp (read VCHFIT input)'
            message = 'No argument specified with maxmem keyword'
            call vcwrline(0,iz,iz0)
         endif

      else if (keyword(i) .eq. 'no_der2_dcp') then
         lder2_dcp = .false.

      else
         routine = 'vchfinp (read VCHFIT input)'
         message = ' unknown keyword : '
         call vcwrline(i,iz,iz0)
      endif

!-----------------------------------------------------------------------
! check if more keywords are to be read, else read new line
!-----------------------------------------------------------------------
      if (i.lt.ic) then
         i=i+1
         go to 15
      else
         go to 10
      endif

 200  continue

!-----------------------------------------------------------------------
! get system information from the dataset-file
!-----------------------------------------------------------------------
      ilbl=index(datfile,' ')-1
      write(ilog,'(/,2a)')&
              'Data read from file : ',datfile(1:ilbl)
      open(idat,file=datfile,form='formatted',status='old',iostat=ierr)
      if (ierr .ne. 0) then
         routine = 'vchfinp (read VCHFIT input)'
         write(message,'(2a)') 'Error opening infofile: ',datfile(1:ilbl)
         call vcerrormsg
      endif
      call dataset_info
      close(idat)

!-----------------------------------------------------------------------
! check dimensions
!-----------------------------------------------------------------------
      if (nadd .gt. maxadd) then
         routine = 'vchfinp (read VCHFIT input)'
         write(message,'(2a,i5)') 'Number of added parameters exceeds ',&
              'maximum. Increase MAXADD in vchammod.f90 to at least:',nadd
         call vcerrormsg
      endif

      do i=1,nsta
         do j=1,nmodes
            if (ncoeff(j,i) .gt. maxcoe) then
               write(6,'(a,i7)') 'ERROR: Increase MAXCOE to :',&
                    ncoeff(j,i)
               stop
            endif
         enddo
      enddo

!-----------------------------------------------------------------------
! check symmetry of symmetric modes specified
!-----------------------------------------------------------------------
      if (symmode .eq. '   ') then
         routine = 'vchfinp (read VCHFIT input)'
         message='Symmetry of symmetric mode not defined'
         call vcwarnmsg
      endif

!-----------------------------------------------------------------------
! check symmetries of JT modes
!-----------------------------------------------------------------------
      do s=1,nsta
         if (jtsta(s) .gt. 0) then
            if (jtmode(s) .eq. ' ') then
               routine = 'vchfinp (read VCHFIT input)'
               write(message,'(a,2i5)')&
            'Symmetry of JT mode not defined for states: ',s,s1
               call vcwarnmsg
            endif

            do s1=s+1,nsta
               if (jtsta(s1) .eq. jtsta(s)) then
                  if (jtmode(s) .ne. jtmode(s1)) then
                     routine = 'vchfinp (read VCHFIT input)'
                     write(message,'(a,2i5)')&
               'Symmetry of JT modes differ for states: ',s,s1
                     call vcwarnmsg
                  endif
                  if (cpmode(s1,s) .eq. ' ') then
                     routine = 'vchfinp (read VCHFIT input)'
                     write(message,'(a,2i5)')&
               'Coupling mode not defined for states: ',s,s1
                     call vcwarnmsg
                  endif
               endif
            enddo
         endif
      enddo

!-----------------------------------------------------------------------
! get type of intersection
! cointype = 0   no intersection
!            1   symmetry allowed intersection
!            2   Exe JT
!            3   Exb JT
!            4   accidental intersection / avoided crossing
!
! ijt: degeneracy of JT system
!-----------------------------------------------------------------------
      call zeromxi(cointype,maxsta,maxsta)
      do s=1,nsta
         if (jtsta(s) .gt. 0) then
            ijt=1
            do s1=s+1,nsta
               if (jtsta(s1) .eq. jtsta(s)) ijt=ijt+1
            enddo

            do s1=s+1,nsta
               if (jtsta(s1) .eq. jtsta(s)) then
                  if (ijt .eq. 2) then
                     if (jtmode(s)(1:1) .eq. 'E' .and.&
                         cpmode(s1,s)(1:1) .eq. 'E') then
                        cointype(s1,s)=2
                        cointype(s,s1)=2
                     else if (jtmode(s)(1:1) .eq. 'B' .and.&
                        cpmode(s,s1)(1:1) .eq. 'B') then
                        cointype(s1,s)=3
                        cointype(s,s1)=3
                     else
                        routine = 'vchfinp (read VCHFIT input)'
                        message = 'Unimplemeted or unknown JT type'
                        call vcwarnmsg
                     endif
                  else
                     routine = 'vchfinp (read VCHFIT input)'
                     message = 'Unimplemeted or unknown JT type'
                     call vcwarnmsg
                  endif
               endif
            enddo
         endif
      enddo

!-----------------------------------------------------------------------
! if conical intersection hasn't been assigned as Jahn-Teller, assign
! now
!-----------------------------------------------------------------------
      do s=1,nsta
         do s1=s+1,nsta
            if (cointype(s1,s) .eq. 0) then
               if (cpmode(s1,s) .ne. symmode) then
                  cointype(s1,s) = 1
                  cointype(s,s1) = 1

               else if (cpmode(s1,s) .eq. symmode) then
                  cointype(s1,s) = 4
                  cointype(s,s1) = 4

               else if (cpmode(s1,s) .eq. ' ') then
                  cointype(s1,s) = 0
                  cointype(s,s1) = 0

               else
                  routine = 'vchfinp (read VCHFIT input)'
                  write(message,'(a,2i5)')&
                 'Unknown intersection type for states: ',s,s1
                  call vcwarnmsg
               endif
            endif
         enddo
      enddo

!-----------------------------------------------------------------------
! guess file must be specified from which mask is read
!-----------------------------------------------------------------------
      if (lrdmask .and. .not. lrdguess) then
         routine = 'vchfinp (read VCHFIT input)'
         message='Guess file containing symmetry mask not specified'
         call vcwarnmsg
      endif

!-----------------------------------------------------------------------
! check selections of parameters are compatible
!-----------------------------------------------------------------------
      if (lgeneral) then
         if (lkappa .or. lgamdiag) ldifit = .true.
      endif

      if (.not. ldiagfit) then
         ldifit = .false.
         lkappa = .false.
         llambda = .false.
         lgamdiag = .false.
      endif

      if (lhigher) then
         if (highord .lt. 3 .or. highord .gt. 4) then
            routine = 'vchfinp (read VCHFIT input)'
            message='Illegal value for highfit. Must be 3 or 4'
            call vcwarnmsg
         endif
         call getnhigher(nhigher,highord)
         if (nhigher .eq. 0) then
            lhigher = .false.
            highord = 0
         endif
      endif
      if (highord .ge. 3) liota = .true.

!-----------------------------------------------------------------------
! if no mode selection made, fit all modes
!-----------------------------------------------------------------------
      if (nfit .eq. 0) then
         nfit=nmodes
         do m=1,nmodes
            mfit(m)=1
         enddo
      endif

!-----------------------------------------------------------------------
! if no state selection made, fit all states
!-----------------------------------------------------------------------
      if (nsfit .eq. 0) then
         nsfit=nsta
         do s=1,nsta
            sfit(s)=1
         enddo
      endif

!-----------------------------------------------------------------------
! if user wants to print out all modes, then make adof usseless
!-----------------------------------------------------------------------
     if(i_ignore .eq. 0 ) then
       do i = 1 , nmodes
        adof(i) = i
       enddo
     endif

      return

 1010 continue
      ilbl=index(filein,' ')-1
      inquire(file=filein,exist=file_exists)
      if (.not. file_exists) then
         print*,'Input file does not exist :',filein
         stop
      endif

      end subroutine vchfinp

!#######################################################################

      subroutine getnhigher(nhigher,highord)

      implicit none

      integer(long), intent(in)  :: highord
      integer(long), intent(out) :: nhigher
      integer(long)              :: i,j,k,l,s

      nhigher = 0

! gam_ijk q_i q_j q_k terms
      do s=1,nsta
         do i=1,nmodes-1
            do j = i+1,nmodes
               do k = j+1,nmodes
                  nhigher = nhigher+1
               enddo
            enddo
         enddo
      enddo

      if (highord .eq. 3) return

! gam_iijj q_i^2 q_j^2 terms
      do s=1,nsta
         do i=1,nmodes-1
            do j = i+1,nmodes
               nhigher = nhigher+1
            enddo
         enddo
      enddo

! gam_iiij q_i^3 q_j terms
      do s=1,nsta
         do i=1,nmodes-1
            do j = i+1,nmodes
               nhigher = nhigher+1
            enddo
         enddo
         do i=2,nmodes
            do j = 1,i-1
               nhigher = nhigher+1
            enddo
         enddo
      enddo

! gam_iijk q_i^2 q_j q_k terms
      do s=1,nsta
         do i=1,nmodes-2
            do j = i+1,nmodes-1
               do k = j+1,nmodes
                  nhigher = nhigher+1
               enddo
            enddo
         enddo
         do i=3,nmodes
            do j = 1,i-2
               do k = j+1,i-1
                  nhigher = nhigher+1
               enddo
            enddo
         enddo
         do i=2,nmodes-1
            do j = 1,i-1
               do k = i+1,nmodes
                  nhigher = nhigher+1
               enddo
            enddo
         enddo
      enddo

! gam_ijkl q_i q_j q_k q_l terms
      do s=1,nsta
         do i=1,nmodes-3
            do j = i+1,nmodes-2
               do k = j+1,nmodes-1
                  do l = k+1,nmodes
                     nhigher = nhigher+1
                  enddo
               enddo
            enddo
         enddo
      enddo

      return
      end subroutine getnhigher

!#######################################################################

      subroutine wrlog

      implicit none

      integer(long)           :: i,j,s,s1,n,ilbl,jlbl,m
      integer(long), external :: slen
      character(len=10)       :: atmp
      character(len=80)       :: string

      write(ilog,'(a,f8.5)') '# VCHam Version :',progver
      write(ilog,'(a)') 'Data taken from : '
      write(ilog,'(10x,a)') trim(datfile(1:70))
      write(ilog,'(a)') 'Output file : '
      write(ilog,'(10x,a)') trim(fileout(1:70))
      if (lrdguess) then
         write(ilog,'(a)') 'Guess file : '
         write(ilog,'(10x,a)') trim(guessfile(1:70))
      endif

      if (.not. lrdmask) then
         write(ilog,'(a)') '--------------------------'
         write(ilog,'(a)') ' The fitting type is:'
         if (lconst) write(ilog,'(a)') ' const   '
         if (lkappa) write(ilog,'(a)') ' kappa   '
         if (lgamma) write(ilog,'(a)') ' gamma   '
         if (ldifit) write(ilog,'(a)') ' Diabatic PES   '
         if (llambda) write(ilog,'(a)') ' lambda  '
         if (lmu) write(ilog,'(a)') ' mu      '
         if (liota .and. .not. lhigher) write(ilog,'(a)') ' iota    '
         if (lhigher) then
            write(ilog,'(a,i5)') 'Higher order terms. Order: ',highord
            write(ilog,'(a,i7)') '             no. of terms: ',nhigher
         endif
         if (nadd .gt. 0) then
            write(ilog,'(a,i5)') 'No. of explicit terms: ',nadd
         endif
         write(ilog,'(a)') '--------------------------'
      else
         write(ilog,'(a)') '-----------------------------------------'
         write(ilog,'(a)') ' The fitting mask is read from guess file'
         write(ilog,'(a)') '-----------------------------------------'
      endif

      write(ilog,*)
      write(ilog,'(a,a3,a)') &
             'The symmetric-mode has ',symmode,' -symmetry'
      do s=1,nsta
         if (jtsta(s) .gt. 0) then
            write(ilog,'(a,i3,a,a3,a)') &
                'The Jahn-Teller-mode for state',s,' has ',jtmode(s),&
                      ' -symmetry'
         endif
         do s1=s+1,nsta
            write(ilog,'(a,2i3,a,a3,a)') &
                'The coupling-mode for states',s,s1,' has ',&
                cpmode(s1,s),' -symmetry'
         enddo
      enddo

      write(ilog,*)
      do s=1,nsta
         do s1=s+1,nsta
            if (cointype(s,s1) .eq. 0) then
               write(ilog,'(a,2i3)') &
                'No conical intersection between states :',s,s1
            else if (cointype(s,s1) .eq. 1) then
               write(ilog,'(a,2i3)') &
                'Symmetry allowed intersection between states :',s,s1
            else if (cointype(s,s1) .eq. 2) then
               write(ilog,'(a,2i3)') &
                'Exe Jahn-Teller intersection between states :',s,s1
            else if (cointype(s,s1) .eq. 3) then
               write(ilog,'(a,2i3)') &
                'Exb Jahn-Teller intersection between states :',s,s1
            else if (cointype(s,s1) .eq. 4) then
               write(ilog,'(a,2i3)') &
                'Accidental intersection between states :',s,s1
            endif
         enddo
      enddo

      if (lgeneral) then
         write(ilog,*)
         write(ilog,'(a)') 'General diabatic potential used for :'
         do s=1,nsta
            do i=1,nmodes
               if (ifdiab(i,s) .eq. 1) then
                  write(ilog,'(2(a,i3),a)') &
                       'mode:',i,' state:',s,'  Quartic'
               else if (ifdiab(i,s) .eq. 2) then
                  write(ilog,'(2(a,i3),a)') &
                       'mode:',i,' state:',s,'  Morse'
               else if (ifdiab(i,s) .eq. 3) then
                  write(ilog,'(2(a,i3),a)') &
                       'mode:',i,' state:',s,'  Buckingham'
               else if (ifdiab(i,s) .eq. 4) then
                  write(ilog,'(2(a,i3),a)')&
                       'mode:',i,' state:',s,'  Anti-Morse'
               else if (ifdiab(i,s) .eq. 5) then
                  write(ilog,'(2(a,i3),a)')&
                       'mode:',i,' state:',s,'  Exponential'
               else if (ifdiab(i,s) .eq. 7) then
                  write(ilog,'(2(a,i3),a)')&
                       'mode:',i,' state:',s,'  General Morse'
               else if (ifdiab(i,s) .eq. 8) then
                  write(ilog,'(2(a,i3),a)')&
                       'mode:',i,' state:',s,'  General Harmonic'
               else if (ifdiab(i,s) .eq. 9) then
                  write(ilog,'(2(a,i3),a)')&
                       'mode:',i,' state:',s,'  General Quartic'
               else if (ifdiab(i,s) .eq. 10) then
                  write(ilog,'(2(a,i3),a)')&
                       'mode:',i,' state:',s,'  Lennard-Jones'
               else if (ifdiab(i,s) .eq. 11) then
                  write(ilog,'(2(a,i3),a)')&
                       'mode:',i,' state:',s,'  Anti Lennard-Jones'
               else if (ifdiab(i,s) .eq. 12) then
                  write(ilog,'(2(a,i3),a)')&
                       'mode:',i,' state:',s,'  General Aquartic'
               else if (ifdiab(i,s) .eq. 13) then
                  write(ilog,'(2(a,i3),a)')&
                       'mode:',i,' state:',s,'  Morse Lorentzian'
               else if (ifdiab(i,s) .eq. 14) then
                  write(ilog,'(2(a,i3),a)')&
                       'mode:',i,' state:',s,'  Morse Gaussian'
               else if (ifdiab(i,s) .eq. 15) then
                  write(ilog,'(2(a,i3),a)')&
                       'mode:',i,' state:',s,'  General Quadratic'
               else if (ifdiab(i,s) .eq. 16) then
                  write(ilog,'(2(a,i3),a)')&
                       'mode:',i,' state:',s,'  Avoided Crossing 2'
               else if (ifdiab(i,s) .eq. 24) then
                  write(ilog,'(2(a,i3),a)')&
                       'mode:',i,' state:',s,'  Anti Avoided Crossing 2'
               else if (ifdiab(i,s) .eq. 17) then
                  write(ilog,'(2(a,i3),a)')&
                       'mode:',i,' state:',s,'  Avoided Crossing Upper 2'
               else if (ifdiab(i,s) .eq. 18) then
                  write(ilog,'(2(a,i3),a)')&
                       'mode:',i,' state:',s,'  Avoided Crossing Quartic'
               else if (ifdiab(i,s) .eq. 19) then
                  write(ilog,'(2(a,i3),a)')&
                       'mode:',i,' state:',s,'  4th order Exe JT'
               else if (ifdiab(i,s) .eq. 20) then
                  write(ilog,'(2(a,i3),a)')&
                       'mode:',i,' state:',s,'  RRP'
               else if (ifdiab(i,s) .eq. 22) then
                  write(ilog,'(2(a,i3),a)')&
                       'mode:',i,' state:',s,'  Constant'
               else if (ifdiab(i,s) .eq. 23) then
                  write(ilog,'(2(a,i3),a)')&
                       'mode:',i,' state:',s,'  1/R'
               endif
            enddo
         enddo
      endif

      if (nadd .gt. 0) then
         write(ilog,*)
         write(ilog,'(a)') 'General polynomial terms added :'
         do n=1,nadd
            ilbl = 1
            string= ' '
            do m=1,nmodes
               if (termdef(m,n) .gt. 0) then
                  write(atmp,'(i4,a,i4)') m,'^',termdef(m,n)
                  call rmblnk(atmp)
                  jlbl=slen(atmp)
                  string(ilbl:ilbl+jlbl-1)=atmp(1:jlbl)
                  ilbl=ilbl+jlbl+1
               endif
            enddo
            write(ilog,'(2(a,i3),5x,a60)') 's= ',termst(1,n),&
                  ' s1= ',termst(2,n),string(1:60)
         enddo
      endif

      write(ilog,'(/,a)') 'The weight-type is :'
      if (weighttype.eq.2) then
         write(ilog,'(a)') '    energy'
      else if (weighttype.eq.1) then
         write(ilog,'(a)') '    position'
      else if (weighttype.eq.3) then
         write(ilog,'(a)') '    equal'
      else
         write(ilog,'(a)') '    none'
      endif
      write(ilog,'(a,f8.4,a)') ' Energyshift : ',e_shift,' eV'
      write(ilog,'(a,i5)') 'Maximum iterations :  ',iter
      write(ilog,'(a,f8.5)') 'Tolerance          :  ',tol
      write(ilog,*) '=============================================='
      write(ilog,'(a,f8.4,a)') 'Shifting of the higher PES : ',e_shift,&
           ' eV'

      if (linternal) then
!        call setintcoo
         write(ilog,'(/,a)') 'coordinate type : internal'
         do i=1,nmodes
            if (cootyp(i) .eq. 1) then
                write(ilog,'(a,i5,a)') 'Normal mode ',i,&
                    '  replaced by torsion'
                write(ilog,'(6x,4i5)') (coodef(j,i),j=1,4)
            endif
         enddo
      else
         write(ilog,'(a)') 'coordinate type : normal modes'
      endif

      if (lusecons) then
         write(ilog,'(a)') 'Constraints used on coefficients'
      endif

      return
      end subroutine wrlog

!#######################################################################

      subroutine guess(energy,qcoo,der1,dcp,der2,const,kappa,lambda,&
                 gamma,iota,mu,expcoe,higher,termk,cp2d,const_mask,&
                 kappa_mask,lambda_mask,gamma_mask,iota_mask,mu_mask,&
                 expcoe_mask,cp2d_mask)

      implicit none

      integer(long)                                                 :: i,j,s,s1,n,m,&
                                                                       nact
      integer(long), dimension(maxsta,maxsta)                       :: dcpset
      integer(long), dimension(nsta,nsta), intent(in)               :: const_mask
      integer(long), dimension(nmodes,nsta), intent(in)             :: kappa_mask
      integer(long), dimension(nmodes,nsta,nsta), intent(in)        :: lambda_mask
      integer(long), dimension(nmodes,nmodes,nsta), intent(in)      :: gamma_mask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(out)     :: expcoe_mask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: iota_mask,mu_mask
      integer(long), dimension(maxcoe,maxadd), intent(in)         :: cp2d_mask

      real(dop), dimension(nadd), intent(out)                    :: termk
      real(dop), dimension(nhigher), intent(out)                 :: higher
      real(dop), dimension(nsta,nsta), intent(out)               :: const
      real(dop), dimension(nmodes,nsta), intent(out)             :: kappa
      real(dop), dimension(nmodes,nsta,nsta), intent(out)        :: lambda
      real(dop), dimension(nmodes,nmodes,nsta), intent(out)      :: gamma
      real(dop), dimension(maxcoe,nmodes,nsta), intent(out)      :: expcoe
      real(dop), dimension(nmodes,nmodes,nsta,nsta), intent(out) :: iota,mu
      real(dop), dimension(maxcoe,maxadd), intent(out)         :: cp2d

      real(dop)                                          :: delta,x,sum,diff,norm
      real(dop), dimension(maxsta)                       :: rbuf
      real(dop), dimension(nmodes,nsets), intent(in)     :: qcoo
      real(dop), dimension(nsets), intent(in)            :: energy
      real(dop), dimension(ncoo,nsets1), intent(in)      :: der1
      real(dop), dimension(ncoo,nsta,nsets3), intent(in) :: dcp
      real(dop), dimension(ncoo,ncoo,nsets2), intent(in) :: der2
      real(dop), dimension(nsta)                         :: energy0
      real(dop), dimension(ncoo,nsta)                    :: der10
      logical(kind=4)                                    :: check,check1

      q0val(:) = 0.0_dop
      if (nval .gt. 0) then
         print*,'Zero point has non-zero coordinates for :'
         do j=1,nval
            m=ival(i)
            q0val(m) = qval0(i)
            write(6,'(a,i5,a,f8.5)') 'Mode : ',m,'  Value :',q0val(m)
         enddo
      endif

!-----------------------------------------------------------------------
! check that first nsta data sets are at Q0
! If shifted along symmetric modes, get energy + der1 at Q0
!-----------------------------------------------------------------------
      check=.true.
      check1=.true.
      energy0(:) = 0.0_dop
      der10(:,:) = 0.0_dop
      do s=1,nsta
         do i=1,nmodes
            diff = abs(qcoo(i,s) - q0val(i))
            if (diff .gt. 0.01) then
               check = .false.
               if (kappa_mask(i,s) .eq. 0) check1 = .false.
            endif
         enddo
      enddo

      if (.not. check) then
         print*,'First Nsta data sets must be at Q0'
         stop
      endif
      if (.not. check1) then
         print*,&
           'Q0 must not be displaced from 0 along non-symmetric modes'
         stop
      endif

      energy0(1:nsta) = energy(1:nsta)
      der10(1:nmodes,1:nsta) = der1(1:nmodes,1:nsta)

!-----------------------------------------------------------------------
! assign energy at Q0
!-----------------------------------------------------------------------
      do s=1,nsta
         const(s,s)=energy0(s)
      enddo

      check=.true.
      do s=2,nsta
         if (energy0(s)+0.0001 .lt. energy0(s-1)) check = .false.
         i = s
         j = s-1
      enddo

      if (.not. check) then
         print*,'First Nsta data sets in wrong order'
         print*,'State ',i,' (',energy(i),') has lower', &
       ' energy than State ',j,' (',energy(j),')'
!         stop
      endif

!-----------------------------------------------------------------------
! mix diabatic states
!-----------------------------------------------------------------------
      if (ldirot) then
! normalise matrix
         do s=1,nsta
            norm=0.0
            do s1=1,nsta
               norm=norm+dirot(s1,s)*dirot(s1,s)
            enddo

            if (abs(norm-0.0) .lt. 0.00001) then
               dirot(s,s)=1.0
            else
               norm=sqrt(norm)
               do s1=1,nsta
                  dirot(s1,s)=dirot(s1,s)/norm
               enddo
            endif
         enddo

! rotate constants
         do s=1,nsta
            rbuf(s)=const(s,s)
            const(s,s)=0.0
         enddo
         do s=1,nsta
            do s1=1,nsta
               do j=1,nsta
                  const(s1,s)=const(s1,s) + &
                              dirot(s1,j)*rbuf(j)*dirot(s,j)
               enddo
            enddo
         enddo
      endif

!-----------------------------------------------------------------------
! assign gradients at Q0
!-----------------------------------------------------------------------
      do s=1,nsta
         do i=1,nmodes
            if (info(s,1) .ge. 1) kappa(i,s)=der10(i,s)
         enddo
      enddo

!-----------------------------------------------------------------------
! now assign to lambda
!-----------------------------------------------------------------------
      delta=0.0_dop
      do s=1,nsta-1
         do s1=s+1,nsta
! take lambda from calculated DCPs
            if (info(s,2) .ge. 1) then
               n=0
               do nact=1,nsta
                  if (jsta(nact,s) .eq. s1) n = nact
               enddo
               do i=1,nmodes
                  lambda(i,s,s1)=dcp(i,n,s)
                  lambda(i,s1,s)=lambda(i,s,s1)
               enddo

! assign 2nd derivative to lambda only if maxima in S1 at Q0
            else if (info(s,2) .ge. 2) then
               delta=(energy(s1)-energy(s))
               do i=1,nmodes
                  if (der2(i,i,s) .lt. -1.0e-6_dop) then
                     x=der2(i,i,s1)-der2(i,i,s)
                     x=x*delta/2.0

                     if (x .gt. 0) then
                        lambda(i,s,s1)=sqrt(x)
                        lambda(i,s1,s)=sqrt(x)
                     else
                        x=sqrt(-x)
                        lambda(i,s,s1)=-x
                        lambda(i,s1,s)=-x
                     endif
                  endif
               enddo
            endif
         enddo
      enddo

!-----------------------------------------------------------------------
! now assign gamma
!
! Correct for derivative coupling and then remove freq0
!-----------------------------------------------------------------------
!     if ((lgamma .or. lgamdiag) .and. nsta .eq. 2) then
!        do i=1,nmodes
!           if ((i .eq. j) .and. (lambda_mask(i,1,2).ne.0)) then
!              sum=0.5*(der2(i,i,2)+der2(i,i,1))
!              sum=sum-freq0(i)
!              diff=0.5*(der2(i,i,2)-der2(i,i,1))
!              diff=diff - lambda(i,1,2)*lambda(i,1,2)/delta

!              gamma(i,i,2)=0.5*(sum+diff)
!              gamma(i,i,1)=0.5*(sum-diff)

!           endif
!        enddo
!     endif

      if (lgamma .or. lgamdiag) then
         do s=1,nsta
            do i=1,nmodes
               do j=1,nmodes
                  gamma(i,j,s)=der2(i,j,s)
               enddo
            enddo
         enddo

         if (lder2_dcp) then    ! correct second derivatives for DCP
            do s=1,nsta
               do s1=1,nsta
                  if (s .eq. s1) cycle
                  delta=(energy(s1)-energy(s))

                  do i=1,nmodes
                     do j=1,nmodes
                        if ((lambda_mask(i,s,s1).eq.0) .or. &
                                     (lambda_mask(j,s,s1).eq.0)) cycle

                        gamma(i,j,s)=gamma(i,j,s) + &
                             2.0d0*lambda(i,s,s1)*lambda(j,s1,s)/delta
                     enddo
                  enddo
               enddo
            enddo
         endif

! subtract freq0 so that curvature = freq0 + gamma
         do s=1,nsta
            do i=1,nmodes
                gamma(i,i,s) = gamma(i,i,s) - freq0(i)
            enddo
         enddo

      endif

!-----------------------------------------------------------------------
! assign initial parameters for general functions
!-----------------------------------------------------------------------
      do s=1,nsta
         do i=1,nmodes
            if (ifdiab(i,s) .gt. 0) then
               call conv_fdiab(ifdiab(i,s),freq0(i),kappa(i,s),&
                       gamma(i,i,s),expcoe(1,i,s),ncoeff(i,s),&
                       expcoe_mask(1,i,s),0,ierr)
            endif
         enddo
      enddo

!-----------------------------------------------------------------------
! zero higher order terms
!-----------------------------------------------------------------------
      call zerovxd(iota,nmodes*nmodes*nsta*nsta)
      call zerovxd(higher,nhigher)
      if ( .not. lgenetica ) call zerovxd(termk,nadd)

!-----------------------------------------------------------------------
! delete unwanted parameters
!-----------------------------------------------------------------------
      do s=1,nsta
         do i=1,nmodes
            if (kappa_mask(i,s) .eq. 0) kappa(i,s)=0.0
            do j=1,nmodes
               if (gamma_mask(j,i,s) .eq. 0) gamma(j,i,s)=0.0
            enddo
         enddo
      enddo

!        print*, 'guess LAMBDA(14,1,5) = ', lambda(14,2,5)
!        print*, 'guess LAMBDAMASK(14,1,5) = ', lambda_mask(14,2,5)


      return
      end subroutine guess

!#######################################################################

      subroutine shiftq0(qcoo,energy,der1,der2,energy0,der10,kappa_mask,&
                 ncoo,nmodes,nsta)

      implicit none

      integer(long), intent(in)                          :: ncoo,nmodes,nsta
      integer(long)                                      :: s,i,j
      integer(long), dimension(nmodes,nsta), intent(in)  :: kappa_mask
      real(dop), dimension(nmodes,nsta), intent(in)      :: qcoo
      real(dop), dimension(nsta), intent(in)             :: energy
      real(dop), dimension(ncoo,nsta), intent(in)        :: der1
      real(dop), dimension(ncoo,ncoo,nsta), intent(in)   :: der2
      real(dop), dimension(nsta)                         :: energy0
      real(dop), dimension(ncoo,nsta)                    :: der10

! shift by -qcoo to Q0
      do s=1,nsta
         energy0(s) = energy(s)
         do i=1,nmodes
            if (kappa_mask(i,s) .eq. 0) cycle

            energy0(s) = energy0(s) - der1(i,s)*qcoo(i,s)
            der10(i,s) = der1(i,s)

            do j=1,nmodes
               if (kappa_mask(j,s) .eq. 0) cycle

               energy0(s) = energy0(s) + 0.5*der2(i,j,s)*qcoo(i,s)*qcoo(j,s)
               der10(i,s) = der10(i,s) - der2(i,j,s)*qcoo(j,s)
            enddo
         enddo
      enddo

      return
      end subroutine shiftq0

!#######################################################################
! set any parameters to specified values
!#######################################################################

      subroutine setpar(const,kappa,lambda,gamma,iota,mu,expcoe,termk,&
                        cpcoe,switch,cp2d)

      implicit none

      character(len=80)                :: atmp,achk,achk1

      integer(long)                    :: s,s1,m,m1,k,p,i,j,ptmp,ieq,o
      integer(long)                    :: iz,ic,ierr,iz0,ilbl,jlbl,slen
      integer(long), dimension(nmodes) :: defchk,chk
      real(dop)                                                  :: nn
      real(dop), dimension(nsta,nsta), intent(out)               :: const
      real(dop), dimension(nmodes,nsta), intent(out)             :: kappa
      real(dop), dimension(nmodes,nsta,nsta), intent(out)        :: lambda
      real(dop), dimension(nmodes,nmodes,nsta), intent(out)      :: gamma
      real(dop), dimension(maxcoe,nmodes,nsta), intent(out)      :: expcoe
      real(dop), dimension(nmodes,nmodes,nsta,nsta), intent(out) :: iota,mu
      real(dop), dimension(maxcoe,nmodes,nsta,nsta), intent(out) :: cpcoe
      real(dop), dimension(nadd), intent(out)                    :: termk
      real(dop), dimension(nswitch,maxcoe)                       :: switch
      real(dop), dimension(maxcoe,maxadd), intent(out)            :: cp2d

      rewind(iin)
      iz=0
      iz0=0

 10   continue
      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      routine = 'setpar (read VCHFIT input)'
      call vccntrl(ierr)

      if (keyword(1).ne.'set_parameters-section') go to 10
 20   continue
      call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      routine = 'readcons (read VCHFIT input)'
      call vccntrl(ierr)

      if (keyword(1).eq.'end-set_parameters-section') then
         go to 999

      else if (keyword(1) .eq. 'e') then
         read(keyword(2),*) s
         read(keyword(3),*) s1
         read(keyword(5),*) const(s,s1)
         if (keyword(6) .eq. ',') then
            call keyeunits(const(s,s1),keyword(7),ierr)
            if (ierr .ne. 0) then
               routine = 'Setpar (VCHFIT)'
               message = 'Incorrect unit for energy : '
               call vcwrline(1,iz,iz0)
            endif
         endif
         const(s1,s)=const(s,s1)

      else if (keyword(1) .eq. 'kappa') then
         read(keyword(2),*) m
         read(keyword(3),*) s
         read(keyword(5),*) kappa(m,s)
         if (keyword(6) .eq. ',') then
            call keyeunits(kappa(m,s),keyword(7),ierr)
            if (ierr .ne. 0) then
               routine = 'Setpar (VCHFIT)'
               message = 'Incorrect unit for energy : '
               call vcwrline(1,iz,iz0)
            endif
         endif

      else if (keyword(1) .eq. 'lambda') then
         read(keyword(2),*) m
         read(keyword(3),*) s
         read(keyword(4),*) s1
         read(keyword(6),*) lambda(m,s,s1)
         if (keyword(7) .eq. ',') then
            call keyeunits(lambda(m,s,s1),keyword(8),ierr)
            if (ierr .ne. 0) then
               routine = 'Setpar (VCHFIT)'
               message = 'Incorrect unit for energy : '
               call vcwrline(1,iz,iz0)
            endif
         endif
         lambda(m,s1,s) = lambda(m,s,s1)

      else if (keyword(1) .eq. 'gamma') then
         read(keyword(2),*) m
         read(keyword(3),*) m1
         read(keyword(4),*) s
         read(keyword(6),*) gamma(m,m1,s)
         if (keyword(7) .eq. ',') then
            call keyeunits(gamma(m,m1,s),keyword(8),ierr)
            if (ierr .ne. 0) then
               routine = 'Setpar (VCHFIT)'
               message = 'Incorrect unit for energy : '
               call vcwrline(1,iz,iz0)
            endif
         endif
         if (m .ne. m1) gamma(m1,m,s) = gamma(m,m1,s)

      else if (keyword(1) .eq. 'mu') then
         read(keyword(2),*) m
         read(keyword(3),*) m1
         read(keyword(4),*) s
         read(keyword(5),*) s1
         read(keyword(7),*) mu(m,m1,s,s1)
         if (keyword(8) .eq. ',') then
            call keyeunits(mu(m,m1,s,s1),keyword(9),ierr)
            if (ierr .ne. 0) then
               routine = 'Setpar (VCHFIT)'
               message = 'Incorrect unit for energy : '
               call vcwrline(1,iz,iz0)
            endif
         endif
         mu(m,m1,s1,s) = mu(m,m1,s,s1)
         if (m .ne. m1) then
            mu(m1,m,s,s1) = mu(m,m1,s,s1)
            mu(m1,m,s1,s) = mu(m,m1,s,s1)
         endif

      else if (keyword(1) .eq. 'iota') then
         read(keyword(2),*) m
         read(keyword(3),*) m1
         read(keyword(4),*) s
         read(keyword(6),*) iota(m,m1,s,s)
         if (keyword(7) .eq. ',') then
            call keyeunits(iota(m,m1,s,s),keyword(10),ierr)
            if (ierr .ne. 0) then
               routine = 'Setpar (VCHFIT)'
               message = 'Incorrect unit for energy : '
               call vcwrline(1,iz,iz0)
            endif
         endif

      else if (keyword(1) .eq. 'general') then
         read(keyword(2),*) k
         read(keyword(3),*) m
         read(keyword(4),*) s
         read(keyword(6),*) expcoe(k,m,s)
         if (keyword(7) .eq. ',') then
            call keyeunits(expcoe(k,m,s),keyword(8),ierr)
            if (ierr .ne. 0) then
               routine = 'Setpar (VCHFIT)'
               message = 'Incorrect unit for energy : '
               call vcwrline(1,iz,iz0)
            endif
         endif

      else if (keyword(1) .eq. 'cpgeneral') then
         read(keyword(2),*) k
         read(keyword(3),*) m
         read(keyword(4),*) s
         read(keyword(5),*) s1
         read(keyword(7),*) cpcoe(k,m,s,s1)
         if (keyword(8) .eq. ',') then
            call keyeunits(cpcoe(k,m,s,s1),keyword(9),ierr)
            if (ierr .ne. 0) then
               routine = 'Setpar (VCHFIT)'
               message = 'Incorrect unit for energy : '
               call vcwrline(1,iz,iz0)
            endif
         endif
         cpcoe(k,m,s1,s)=cpcoe(k,m,s,s1)

      elseif( index(keyword(1),'tanch') .ne. 0 .or.  index(keyword(1),'polybox2') .ne. 0) then
         read(keyword(2),*,iostat=ierr) m
         read(keyword(3),*,iostat=ierr) p
         read(keyword(4),*,iostat=ierr) s
         read(keyword(5),*,iostat=ierr) s1
!       this is the parameter index for that type of function
         read(keyword(6),*,iostat=ierr) k
         do j = 1, maxadd
          if( dofcp2d(1,j) .eq. m .and. dofcp2d(2,j) .eq. p &
              .and. stacp2d(1,j) .eq. s .and. stacp2d(2,j) .eq. s1 ) &
           read(keyword(8),*,iostat=ierr) cp2d(k,j)
         enddo

      elseif(index(keyword(1),'polybox') .ne. 0 ) then
!       modes
         read(keyword(2),*,iostat=ierr) m
         read(keyword(3),*,iostat=ierr) p
!       states
         read(keyword(4),*,iostat=ierr) s
         read(keyword(5),*,iostat=ierr) s1
!       order
         read(keyword(6),*,iostat=ierr) o
!       number parameter
         read(keyword(7),*,iostat=ierr) k
         do j = 1, maxadd
            if( dofcp2d(1,j) .eq. m .and. dofcp2d(2,j) .eq. p &
                .and. stacp2d(1,j) .eq. s .and. stacp2d(2,j) .eq. s1  &
                .and. ocp2d(j) .eq. o ) &
              read(keyword(7),*,iostat=ierr) cp2d(k,j)
         enddo
      else if (keyword(1).eq.'explicit') then
         do i=1,nmodes
            defchk(i)=0.0
         enddo
         read(keyword(2),*,iostat=ierr) s
         read(keyword(3),*,iostat=ierr) s1
         do i=4,ic-2
            ilbl=index(keyword(i),'^')
            jlbl=slen(keyword(i))
            if (ilbl .eq. 0) then
               p = 1
               read(keyword(i),*,iostat=ierr) m
            else
               read(keyword(i)(1:ilbl-1),*,iostat=ierr) m
               read(keyword(i)(ilbl+1:jlbl),*,iostat=ierr) p
            endif
            defchk(m) = p
         enddo
         do i=1,nadd
            do j=1,nmodes
               chk(j)=1
            enddo
            if (s.eq.termst(1,i).and.s1.eq.termst(2,i).or.   &
                 (s.eq.termst(2,i)).and.s1.eq.termst(1,i)) then
               do j=1,nmodes
                  chk(j)=0
               enddo
               do j=1,nmodes
                  chk(j)=defchk(j)-termdef(j,i)
                  if (chk(j).ne.0) go to 300
               enddo

               read(keyword(ic),*,iostat=ierr) termk(i)
  300          continue
            endif
         enddo

      else if (keyword(1).eq.'switch') then
         read(keyword(2),*) k
         do i=1,ic
            if (keyword(i).eq.'=') ieq=i
         enddo
         j=1
         atmp=''
         do i=3,ieq-1
            l=len_trim(keyword(i))
            write(atmp(j:j+l),'(a)') keyword(i)(1:len_trim(keyword(i)))
            j=j+l+1
            write(atmp(j:j),'(a)') ' '
         enddo
         do i=1,nswitch
            achk=''
            achk1=''
            write(achk(1:len_trim(atmp)),'(a)') atmp(1:len_trim(atmp))
            write(achk1(1:len_trim(atmp)),'(a)') aswitch(i)(1:len_trim(atmp))
            if (achk.eq.achk1) then
               read(keyword(ieq+1),*,iostat=ierr) switch(i,k)
            endif
         enddo

      else
         routine = 'Setpar'
         write(message,'(2a)') 'Unknown parameter type: ',keyword(1)
         call vcerrormsg

      endif

!        print*, 'newguess LAMBDA(14,1,5) = ', lambda(14,1,5)
      go to 20




  999 continue


      return
      end subroutine setpar

!#######################################################################

      subroutine rdswitch

        implicit none

        integer(long), external :: slen
        integer(long)   :: n,m,k,iz,ic,ierr

        rewind(iin)
10      continue
        call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
        if (keyword(1).ne.'switching_function') goto 10

        m=0
20      continue
        call rdinpf(iin,keyword,keyorig,inptit,lc,ic,iz,ierr,&
             maxkey)
        routine = 'vchfinp (read VCHFIT input)'
        call vccntrl(ierr)
        if (keyword(1) .ne. 'end-switching_function') then
           m=m+1
           n=-1
           do k=1,ic
              write(aswitch(m)(n+2:n+2+len_trim(keyword(k))),'(a)') &
                   keyword(k)(1:len_trim(keyword(k)))
              n=n+1+len_trim(keyword(k))
           enddo
           call switchdat2(aswitch(m),swmd(m),swm(m),swm1(m),sws(m),&
                sws1(m),swk(m),swpar(m),swfunc(m),nswitchcoe(m))
           goto 20
        endif

        return
      end subroutine rdswitch

!#######################################################################


      subroutine initpar(optcoo,mr,mi)

      implicit none

      integer(long), dimension(midim), intent(inout) :: mi
      real(dop), dimension(mrdim), intent(out)       :: mr
      real(dop), dimension(optcoodim), intent(out)   :: optcoo
      real(dop)                                      :: fret


!-----------------------------------------------------------------------
! set to zero all parameters that are zero by symmetry
!-----------------------------------------------------------------------
      if (lzerozero) call zeropar(0,mr(mrconst),mr(mrkappa),&
           mr(mrlambda),mr(mrgamma),mr(mriota),mr(mrmu),&
           mr(mrexpcoe),mr(mrtermk),mr(mrcpcoe),&
           mr(mrswitch),mr(mrcp2d),mi(miconstmask),mi(mikappamask),mi(milambdamask),&
           mi(migammamask),mi(miiotamask),mi(mimumask),&
           mi(miexpcoemask),mi(mitermkmask),mi(micpcoemask),&
           mi(miswitchmask),mi(micp2d))

!-----------------------------------------------------------------------
! set to zero all parameters not selected for optimisation
! and then zero any parameters constrained to these
!-----------------------------------------------------------------------
      if (lparzero) then
         call zeropar(2,mr(mrconst),mr(mrkappa),&
              mr(mrlambda),mr(mrgamma),mr(mriota),mr(mrmu),&
              mr(mrexpcoe),mr(mrtermk),mr(mrcpcoe),&
              mr(mrswitch),mr(mrcp2d),&
              mi(miconstmask),mi(mikappamask),mi(milambdamask),&
              mi(migammamask),mi(miiotamask),mi(mimumask),&
              mi(miexpcoemask),mi(mitermkmask),mi(micpcoemask),&
              mi(miswitchmask),mi(micp2d))
         call zerocons(mr(mrconst),mr(mrkappa),mr(mrlambda),&
              mr(mrgamma),mr(mriota),mr(mrmu),mr(mrexpcoe),&
              mi(miconstmask),&
              mi(mikappamask),mi(milambdamask),mi(migammamask),&
              mi(miiotamask),mi(mimumask),mi(miexpcoemask))
      endif

!-----------------------------------------------------------------------
! set kappa and lambda parameters to be optimised
!-----------------------------------------------------------------------
      call initpar1(mr(mrkappa),mr(mrlambda),mi(mikappamask),&
           mi(milambdamask),mr(mrgamma),mr(mriota),mr(mrmu),&
           mr(mrexpcoe),mr(mrtermk),mr(mrcpcoe),&
           mr(mrswitch),mr(mrcp2d),mi(migammamask),mi(miiotamask),mi(mimumask),&
           mi(miexpcoemask),mi(mitermkmask),mi(micpcoemask),&
           mi(miswitchmask),mi(micp2d))

!-----------------------------------------------------------------------
! calculate rmsd of functions from data set. Doing this automatically
! sets any constraints
!-----------------------------------------------------------------------
      call par2vec2d(optcoo,mr(mrconst),mr(mrkappa),mr(mrlambda),&
           mr(mrgamma),mr(mriota),mr(mrmu),mr(mrexpcoe),&
           mr(mrhigher),mr(mrtermk),mr(mrcpcoe),&
           mr(mrswitch),mr(mrcp2d),mi(miconstmask),mi(mikappamask),&
           mi(milambdamask),mi(migammamask),mi(miiotamask),&
           mi(mimumask),mi(miexpcoemask),mi(mitermkmask),&
           mi(micpcoemask),mi(miswitchmask),mi(micp2d))

      if (.not.lgenetica) then
        fret=vcrmsd(optcoo,mr,mi)
        write(ilog,'(/,a,e16.8)') '  RMSD (guess - data) = ',SQRT(fret/nsets)
!        write(6,'(/,a,e16.8)') '  RMSD (guess - data) = ',SQRT(fret/nsets)
      endif
!      if(lgenetica) fret=vcrmsd(optcoo,mr,mi)
      return
      end subroutine initpar

!#######################################################################

      subroutine initpar1(kappa,lambda,kappa_mask,lambda_mask,&
                 gamma,iota,mu,expcoe,termk,cpcoe,switch,cp2d,&
                 gamma_mask,iota_mask,mu_mask,expcoe_mask,&
                 termk_mask,cpcoe_mask,switch_mask,cp2d_mask)

      integer(long)       :: s,s1,m,m1,k,i,j,l

      integer(long), dimension(nmodes,nsta), intent(in)             :: kappa_mask
      integer(long), dimension(nmodes,nsta,nsta), intent(in)        :: lambda_mask
      integer(long), dimension(nmodes,nmodes,nsta), intent(in)      :: gamma_mask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(in)      :: expcoe_mask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: iota_mask,mu_mask
      integer(long), dimension(maxcoe,nmodes,nsta,nsta), intent(in) :: cpcoe_mask
      integer(long), dimension(nswitch,maxcoe), intent(in)          :: switch_mask

      real(dop), dimension(nmodes,nsta), intent(inout)             :: kappa
      real(dop), dimension(nmodes,nsta,nsta), intent(inout)        :: lambda
      real(dop), dimension(nmodes,nmodes,nsta), intent(inout)      :: gamma
      real(dop), dimension(maxcoe,nmodes,nsta), intent(inout)      :: expcoe
      real(dop), dimension(nmodes,nmodes,nsta,nsta), intent(inout) :: iota,mu
      real(dop), dimension(maxcoe,nmodes,nsta,nsta), intent(inout) :: cpcoe
      real(dop), dimension(nadd), intent(inout)                    :: termk

      integer(long), dimension(nadd), intent(in)                   :: termk_mask
      real(dop), dimension(nswitch,maxcoe), intent(inout)          :: switch
      real(dop)                                                    :: x,small,small2
      real(dop), dimension(maxcoe,maxadd), intent(inout)           :: cp2d
      integer(long), dimension(maxcoe,maxadd), intent(in)          :: cp2d_mask

      small=0.04
!      small=0.000000005

      if (lwrop) small=0.000000005

!      small=0.000000005
      small2=0.000005

!-----------------------------------------------------------------------
! check all kappa parameters to be optimised are non-zero.
!-----------------------------------------------------------------------
      do s=1,nsta
         do m=1,nmodes
            if (kappa_mask(m,s) .eq. 1) then
               x=abs(kappa(m,s))
               x=max(x,0.000005_dop)
               kappa(m,s) = sign(x,kappa(m,s))
            endif
         enddo
      enddo

!-----------------------------------------------------------------------
! check all lambda parameters to be optimised are non-zero. (as lambda^2
! is related to adiabatic surfaces, if lambda = 0, then gradient of RMSD
! calculated by numerical differentiation is 0).
!-----------------------------------------------------------------------
      do s=1,nsta
         do s1=1,nsta
            do m=1,nmodes
               if (lambda_mask(m,s,s1) .eq. 1) then
                  x=abs(lambda(m,s,s1))
                  x=max(x,0.000005_dop)
                  lambda(m,s,s1) = sign(x,lambda(m,s,s1))
               endif
            enddo
         enddo
      enddo

!-----------------------------------------------------------------------
! check all gamma parameters to be optimised are non-zero.
!-----------------------------------------------------------------------
      do s=1,nsta
         do m=1,nmodes
            do m1=m,nmodes
               if (gamma_mask(m1,m,s) .eq. 1) then
                  x=abs(gamma(m1,m,s))
                  x=max(x,0.000005_dop)
                  gamma(m1,m,s) = sign(x,gamma(m1,m,s))
                  if (m .ne. m1) gamma(m,m1,s) = gamma(m1,m,s)
               endif
            enddo
         enddo
      enddo

!-----------------------------------------------------------------------
! check all iota parameters to be optimised are non-zero.
!-----------------------------------------------------------------------
      do s=1,nsta
         do m=1,nmodes
            do m1=1,nmodes
               if (iota_mask(m1,m,s,s) .eq. 1) then
                  x=abs(iota(m1,m,s,s))
                  x=max(x,0.000005_dop)
                  iota(m1,m,s,s) = sign(x,iota(m1,m,s,s))
               endif
            enddo
         enddo
      enddo
      do s=1,nsta-1
         do s1=s+1,nsta
            do m=1,nmodes
               do m1=1,nmodes
                  if (iota_mask(m1,m,s,s1) .eq. 1) then
                     x=abs(iota(m1,m,s,s1))
                     x=max(x,0.000005_dop)
                     iota(m1,m,s,s1) = sign(x,iota(m1,m,s,s1))
                     iota(m1,m,s1,s) = iota(m1,m,s,s1)
                  endif
               enddo
            enddo
         enddo
      enddo

!-----------------------------------------------------------------------
! check all mu parameters to be optimised are non-zero.
!-----------------------------------------------------------------------
      do s=1,nsta
         do s1=1,nsta
            do m=1,nmodes
               do m1=m,nmodes
                  if (mu_mask(m1,m,s1,s) .eq. 1) then
                     x=abs(mu(m1,m,s1,s))
                     x=max(x,0.000005_dop)
                     mu(m1,m,s1,s) = sign(x,mu(m1,m,s1,s))
                     mu(m,m1,s,s1) = mu(m1,m,s1,s)
                     if (m .ne. m1) then
                        mu(m1,m,s,s1) = mu(m1,m,s1,s)
                        mu(m1,m,s1,s) = mu(m1,m,s1,s)
                     endif
                  endif
               enddo
            enddo
         enddo
      enddo

!-----------------------------------------------------------------------
! check all coefficients to be optimised are non-zero.
!-----------------------------------------------------------------------
      do s=1,nsta
         do m=1,nmodes
            do k=1,ncoeff(m,s)
               if (expcoe_mask(k,m,s) .eq. 1) then
                  x=abs(expcoe(k,m,s))
                  if (ifdiab(m,s).ne.12) then
                     x=max(x,0.000005_dop)
                  else
                     x=max(x,0.00000005_dop)
                  endif
                  expcoe(k,m,s) = sign(x,expcoe(k,m,s))
               endif
            enddo
         enddo
      enddo

     s = 1
     do while (cp2dtyp(s) .ne. 0)
       select case (cp2dtyp(s))
       case (1 : 2)
         if (cp2d_mask(1,s) .eq. 1) then
            cp2d(1,s)=0.5
          endif
       case (3)
         if (cp2d_mask(1,s) .eq. 1 .and. cp2d(1,s) .eq. 0.0 ) cp2d(1,s) = 0
         if (cp2d_mask(2,s) .eq. 1 .and. cp2d(2,s) .eq. 0.0 ) cp2d(2,s) = 0.5
         if (cp2d_mask(3,s) .eq. 1 .and. cp2d(3,s) .eq. 0.0 ) cp2d(3,s) = 5
         if (cp2d_mask(4,s) .eq. 1 .and. cp2d(4,s) .eq. 0.0 ) cp2d(4,s) = 1
       case (4)
         if (cp2d_mask(1,s) .eq. 1 .and. cp2d(1,s) .eq. 0.0 ) cp2d(1,s) = 0
         if (cp2d_mask(2,s) .eq. 1 .and. cp2d(2,s) .eq. 0.0 ) cp2d(2,s) = 0.5
         if (cp2d_mask(3,s) .eq. 1 .and. cp2d(3,s) .eq. 0.0 ) cp2d(3,s) = 5
         if (cp2d_mask(4,s) .eq. 1 .and. cp2d(4,s) .eq. 0.0 ) cp2d(4,s) = pi/4.0
       case (5)
         if (cp2d_mask(1,s) .eq. 1 .and. cp2d(1,s) .eq. 0.0 ) cp2d(1,s) = 0
         if (cp2d_mask(2,s) .eq. 1 .and. cp2d(2,s) .eq. 0.0 ) cp2d(2,s) = 1
         if (cp2d_mask(3,s) .eq. 1 .and. cp2d(3,s) .eq. 0.0 ) cp2d(3,s) = 5
         if (cp2d_mask(4,s) .eq. 1 .and. cp2d(4,s) .eq. 0.0 ) cp2d(4,s) = pi/4.0
         if (cp2d_mask(5,s) .eq. 1 .and. cp2d(5,s) .eq. 0.0 ) cp2d(5,s) = 2
       case (6)
         if (cp2d_mask(1,s) .eq. 1 .and. cp2d(1,s) .eq. 0.0 ) cp2d(1,s) = 0
         if (cp2d_mask(3,s) .eq. 1 .and. cp2d(3,s) .eq. 0.0 ) cp2d(3,s) = 5
         if (cp2d_mask(4,s) .eq. 1 .and. cp2d(4,s) .eq. 0.0 ) cp2d(4,s) = pi/4.0
       end select
     s = s + 1
     enddo

      do s=1,nsta
         do s1=1,nsta
            do m=1,nmodes
               do k=1,ncoeffcp(m,s,s1)
                  if (cpcoe_mask(k,m,s,s1) .eq. 1) then
                     x=abs(cpcoe(k,m,s,s1))
                     x=max(x,0.00000005_dop)
                     cpcoe(k,m,s,s1) = sign(x,cpcoe(k,m,s,s1))
                  endif
               enddo
            enddo
         enddo
      enddo

!-----------------------------------------------------------------------
! check all explicitly added terms to be optimised are non-zero.
!-----------------------------------------------------------------------
      do i=1,nadd
         if (termk_mask(i).eq.1) then
            x=abs(termk(i))
            x=max(x,0.00000005_dop)
            termk(i)=sign(x,termk(i))
         endif
      enddo

!-----------------------------------------------------------------------
! switching functions
!-----------------------------------------------------------------------
      do i=1,nswitch
         do j=1,nswitchcoe(i)
            if (switch_mask(i,j).eq.1) then
               x=abs(switch(i,j))
               x=max(x,0.00000005_dop)
               switch(i,j)=sign(x,switch(i,j))
            endif
         enddo
      enddo

      return
      end subroutine initpar1

!#######################################################################

      subroutine fitpar(optcoo,mr,mi)

      implicit none


      integer(long), parameter :: powdim=20

      integer(long), dimension(midim), intent(inout) :: mi
      real(dop), dimension(mrdim), intent(inout)     :: mr
      real(dop), dimension(optcoodim), intent(out)   :: optcoo
!      integer, dimension(nmodes,nmodes,nsta), intent(in)    :: forcepos
!     real*8 xi(powdim*powdim),work(powdim,3)
      real(dop), dimension(:,:), allocatable :: xi !commented MAP to test simplex issues 18/07/19
      real(dop), dimension(:,:), allocatable :: work !commented MAP to test simplex issues 18/07/19
      !real*8 xi(nvar,nvar)!added MAP to test simplex issues 18/07/19
      !real*8 work(nvar,3)!added MAP to test simplex issues 18/07/19

      integer(long)                         :: iter0, ifail, irevcm
      real(dop)                             :: fret


      allocate(xi(nvar,nvar+1)) !MAP added + 1 to match amoeba 18/07/19
      allocate(work(nvar+1,3)) !MAP added + 1 to match amoeba 18/07/19

      ifail = 0
      irevcm = 0
!----------------------------------------------------------------------
! save parameters from the initial hamiltonian
!----------------------------------------------------------------------
      if( .not. lspecies) then
         call cpvxd(mr(mrconst),mr(mrconstini),nsta*nsta)
         call cpvxd(mr(mrkappa),mr(mrkappaini),nmodes*nsta)
         call cpvxd(mr(mrlambda),mr(mrlambdaini),nmodes*nsta*nsta)
         call cpvxd(mr(mrgamma),mr(mrgammaini),nmodes*nmodes*nsta)
         call cpvxd(mr(mriota),mr(mriotaini),nmodes*nmodes*nsta*nsta)
         call cpvxd(mr(mrmu),mr(mrmuini),nmodes*nmodes*nsta*nsta)
         call cpvxd(mr(mrexpcoe),mr(mrexpcoeini),nmodes*nsta*maxcoe)
         call cpvxd(mr(mrhigher),mr(mrhigherini),nhigher)
         call cpvxd(mr(mrtermk),mr(mrtermkini),nadd)
      endif
!
! save specified maximum iterations
!

      iter0=iter
! optimise lambda, energy, gamma, mu, kappa
! offdiagsingopt is used in routing mutate (see routine!)
      if ((nvar .eq. 0) .and. (offdiagsignopt .eqv. .false.)) then
         print*
         write(6,'(a)') 'WARNING: No parameters to optimise !'
         write(6,'(a)') 'Check input flags'
         print*
         return
      elseif (ioptim .eq. 2 .and. nvar .lt. powdim) then
         print*
         write(6,'(a)') 'WARNING: Not enough variables to use Powell '
         write(6,'(a)') 'Changing to conjugate gradient'
         print*
         ioptim=1
      endif

      fret=vcrmsd(optcoo,mr,mi)
      if(.not. lspecies) write(ilog,'(a,e16.8)') '  function value = ',fret
      if(.not. lspecies .and. .not. lgenetica)&
         write(6,'(/,1x,a,e16.8)') 'Initial function value: ',fret
      if(.not. lgenetica) write(6,'(/,1x,a,e16.8)') 'Initial RMSD: ',sqrt(fret/nsets)

      if (.not. (lgenetica .or. gaoptim)) then
         fret=vcrmsd(optcoo,mr,mi)
         write(ilog,'(a,e16.8)') '  function value = ',fret
         write(6,'(a,e16.8)') '  function value = ',fret
         write(6,'(/,1x,a,e16.8)') 'Initial function value: ',fret
         write(ilog,'(/,1x,a,e16.8)') 'Initial function value: ',fret
! Output weighted RMSD
         write(6,'(/,1x,a,e16.8)') 'Initial weighted RMSD: ',sqrt(fret/nsets)
         write(ilog,'(/,1x,a,e16.8)') 'Initial weighted RMSD: ',sqrt(fret/nsets)
! Output unweighted RMSD
         call reweight(mr(mrenergy),0.0d0,nsets)
         fret=vcrmsd(optcoo,mr,mi)
         write(6,'(/,1x,a,e16.8)') 'Initial unweighted RMSD: ',sqrt(fret/nsets)
         write(ilog,'(/,1x,a,e16.8)') 'Initial unweighted RMSD: ',sqrt(fret/nsets)
! Revert back to original weights
         call reweight(mr(mrenergy),wfact,nsets)
      else
         write(6,'(/,1x,a)') 'Genetic Algorithm fitting is beginning...'
         call geneticalgor(optcoo,mr,mi)
         goto 10
      endif

      call par2vec2d(optcoo,mr(mrconst),mr(mrkappa),mr(mrlambda),&
           mr(mrgamma),mr(mriota),mr(mrmu),mr(mrexpcoe),&
           mr(mrhigher),mr(mrtermk),mr(mrcpcoe),&
           mr(mrswitch),mr(mrcp2d),mi(miconstmask),mi(mikappamask),&
           mi(milambdamask),mi(migammamask),mi(miiotamask),&
           mi(mimumask),mi(miexpcoemask),mi(mitermkmask),&
           mi(micpcoemask),mi(miswitchmask),mi(micp2d))
      if( .not. lspecies) then
         write(6,*)
         write(6,'(a,x,i5)') "NVAR:",nvar
         write(6,*)
      endif

      if (ioptim .eq. 1) then
        print*,'CONJUGATE GRADIENT'
        iter=iter0
        call frprmn(optcoo,nvar,tol,iter,fret,vcrmsd,dvcrmsd,&
             ilog,0.0_dop,1,mr,mi)
      else if (ioptim .eq. 2) then
         print*,'POWELL'
         iter=iter0
         call unitqxd(xi,nvar)
         call powell(optcoo,xi,nvar,nvar,tol,iter,fret,vcrmsd,&
              work(1,1),work(1,2),work(1,3),mr,mi)
      else if (ioptim .eq. 3) then
         print*,'SIMPLEX'
         iter=iter0
         call init_amoeba(xi,work(1,1),optcoo,nvar,vcrmsd,mr,mi)
         print*,"Amoeba Initialised"
         call amoeba(xi,work(1,1),nvar,0.001_dop,vcrmsd,iter,work(1,2),&
               work(1,3),mr,mi)
         write(6,'(a,i5)') 'converged in Iterations: ',iter
         call cpvxd(xi,optcoo,nvar)
      else if (ioptim .eq. 4) then
        print*,'BFGS (Quasi-Newton)'
        iter=iter0
        tol = 10.0d-5
        call bfgs_driver(optcoo,nvar,tol,iter,fret,vcrmsd,dvcrmsd,&
             ilog,mr,mi)
      else if ( ioptim .eq. 4) then
         print*,'IOPTIM 4 needs code'
         stop
      endif

      call vec2par2d(optcoo,mr(mrconst),mr(mrkappa),mr(mrlambda),&
           mr(mrgamma),mr(mriota),mr(mrmu),mr(mrexpcoe),&
           mr(mrhigher),mr(mrtermk),mr(mrcpcoe),&
           mr(mrswitch),mr(mrcp2d),mi(miconstmask),mi(mikappamask),&
           mi(milambdamask),mi(migammamask),mi(miiotamask),&
           mi(mimumask),mi(miexpcoemask),mi(mitermkmask),&
           mi(micpcoemask),mi(miswitchmask),mi(micp2d))

      if (lgeneral .and. lusecons .and. .not. gaoptim  ) then
         fret=vcrmsd(optcoo,mr,mi)
         call general_cons(mi(miconstmask),mi(mikappamask),&
              mi(milambdamask),mi(migammamask),&
              mi(miiotamask),mi(mimumask),&
              mi(miexpcoemask),mr(mrexpcoe),mi(mitermkmask),&
              mi(micpcoemask),mi(miswitchmask),&
              mr(mrlambda))

         call par2vec2d(optcoo,mr(mrconst),mr(mrkappa),mr(mrlambda),&
              mr(mrgamma),mr(mriota),mr(mrmu),mr(mrexpcoe),&
              mr(mrhigher),mr(mrtermk),mr(mrcpcoe),&
              mr(mrswitch),mr(mrcp2d),mi(miconstmask),mi(mikappamask),&
              mi(milambdamask),mi(migammamask),mi(miiotamask),&
              mi(mimumask),mi(miexpcoemask),mi(mitermkmask),&
              mi(micpcoemask),mi(miswitchmask),mi(micp2d))
      endif

10   fret=vcrmsd(optcoo,mr,mi)

! Output weighted RMSD
      write(ilog,'(/,a,e16.8)') '  Weighted RMSD (guess - data)   = ',SQRT(fret/nsets)
      write(6,'(/,a,e16.8)') '  Weighted RMSD (guess - data)   = ',SQRT(fret/nsets)

! Output unweighted RMSD
      call reweight(mr(mrenergy),0.0d0,nsets)
      fret=vcrmsd(optcoo,mr,mi)
      write(6,'(/,a,e16.8)') '  Unweighted RMSD (guess - data) = ',SQRT(fret/nsets)
      write(ilog,'(/,a,e16.8)') '  Unweighted RMSD (guess - data) = ',SQRT(fret/nsets)
! Revert back to original weights
      call reweight(mr(mrenergy),wfact,nsets)

      !deallocate(xi,work)

      write(ilog,*)
      write(6,*)
      return
      end subroutine fitpar


!#######################################################################

      subroutine readgamask(io,kmask,lmask,gmask,imask,mmask,xmask,tmask,spec,cspec)

      implicit none

      integer(long)             :: i,j,s,n,s1,m,m1,x
      integer(long),intent(in)                                       :: io
      integer(long), dimension(nadd), intent(out)                    :: tmask
      integer(long), dimension(nmodes,nsta), intent(out)             :: kmask
      integer(long), dimension(nmodes,nsta,nsta), intent(out)        :: lmask
      integer(long), dimension(nmodes,nmodes,nsta), intent(out)      :: gmask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(out)      :: xmask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(out) :: imask,mmask
      character(len=200)                     :: string, dum
      character(len=39),intent(out)          :: cspec
      logical(kind=4) , intent(inout)        :: spec

       cspec = ''
       spec = .false.
       kmask = 0
       lmask = 0
       gmask = 0
       mmask = 0
       imask = 0
       xmask = 0
       tmask = 0
200   read(io,'(A)',err=998,end=999) string
      string = trim(adjustl(string))
      if( index(string,'SPECIES') .ne. 0 ) then
           spec = .true.
!           read(trim(string),'(9X,A9)') cspec
           cspec = trim(adjustl(string(10:len(trim(adjustl(string))))))
      endif
!      print*, trim(string),' and ',  string(len(trim(string)):len(trim(string)))
      if( (index(string,'f') .eq. 0 .and. spec .eqv. .false.) .or. &
        ( lspecies .eqv. .true. .and. spec .eqv. .true. .and. &
        string(len(trim(string)):len(trim(string))) .eq. 'f' )) then
       if(string(1:1) .eq. 'k') then
         read(string,'(A1,2I3)',err=998) dum,i,s
         kmask(i,s) = 1
       elseif(string(1:1) .eq. 'l' )then
         read(string,'(A1,3I3)',err=998) dum,i,s,s1
         lmask(i,s,s1) = 1
       elseif(string(1:1) .eq. 'g') then
         read(string,'(A1,3I3)',err=998) dum,i,j,s
         gmask(i,j,s) = 1
         gmask(j,i,s) = 1
       elseif(string(1:1) .eq. 'm' )then
         read(string,'(A1,4I3)',err=998) dum,j,i,s,s1
         mmask(j,i,s,s1) = 1
         mmask(j,i,s1,s) = 1
         mmask(i,j,s1,s) = 1
         mmask(i,j,s,s1) = 1
       elseif(string(1:1) .eq. 'i') then
         read(string,'(A1,4I3)',err=998) dum,j,i,s,s1
         imask(j,i,s,s1) = 1
         imask(j,i,s1,s) = 1
         imask(i,j,s1,s) = 1
         imask(i,j,s,s1) = 1
       elseif(string(1:1) .eq. 'x') then
         read(string,'(A1,3I3)',err=998) dum,k,i,s
         xmask(k,i,s) = 1
       elseif(string(1:1) .eq. 't' ) then
         read(string,'(X,I3)',err=998) i
         tmask(i) = 1
       endif
      endif
     goto 200

999  return

998  print*, 'ERROR readgin mask,(readgamask) param:', dum
     return

      end subroutine readgamask
!#######################################################################

      subroutine readgapar(io,kappa,lambda,gamma,iota,mu,expcoe,termk)

      implicit none

      integer(long)             :: i,j,s,s1
      integer(long),intent(in)                                  :: io
      real(dop)                 :: rdum
      real(dop), dimension(nadd), intent(out)                    :: termk
      real(dop), dimension(nmodes,nsta), intent(out)             :: kappa
      real(dop), dimension(nmodes,nsta,nsta), intent(out)        :: lambda
      real(dop), dimension(nmodes,nmodes,nsta), intent(out)      :: gamma
      real(dop), dimension(maxcoe,nmodes,nsta), intent(out)      :: expcoe
      real(dop), dimension(nmodes,nmodes,nsta,nsta), intent(out) :: iota,mu
      character(len=50)       :: string, dum

200   read(io,'(50A)',err=998,end=999) string
      string = trim(adjustl(string))
!      print*, 'string = ', string
      if(string(1:1) .eq. 'k') then
        read(string,'(A1,2I3,F15.8)',err=998) dum,i,s,rdum
        kappa(i,s) = rdum
      elseif(string(1:1) .eq. 'l') then
        read(string,'(A1,3I3,F15.8)',err=998) dum,i,s,s1,rdum
        lambda(i,s,s1) = rdum
        lambda(i,s1,s) = lambda(i,s,s1)
      elseif(string(1:1) .eq. 'g') then
        read(string,'(A1,3I3,F15.8)',err=998) dum,i,j,s,rdum
        gamma(i,j,s) = rdum
        gamma(j,i,s) = gamma(i,j,s)
      elseif(string(1:1) .eq. 'm') then
        read(string,'(A1,4I3,F15.8)',err=998) dum,j,i,s,s1,rdum
        mu(j,i,s,s1) = rdum
        mu(j,i,s1,s) = mu(j,i,s,s1)
        mu(i,j,s1,s) = mu(j,i,s,s1)
        mu(i,j,s,s1) = mu(j,i,s,s1)
      elseif(string(1:1) .eq. 'i') then
        read(string,'(A1,4I3,F15.8)',err=998) dum,j,i,s,s1,rdum
        iota(j,i,s,s1) = rdum
        iota(j,i,s1,s) = iota(j,i,s,s1)
        iota(i,j,s1,s) = iota(j,i,s,s1)
        iota(i,j,s,s1) = iota(j,i,s,s1)
      elseif(string(1:1) .eq. 'x') then
        read(string,'(A1,3I3,F15.8)',err=998) dum,k,i,s,rdum
        expcoe(k,i,s) = rdum
      elseif(string(1:1) .eq. 't') then
        read(string,'(X,I3)',err=998) i
!       NOTE: IF YOU WANT A ROUTINE THAT READS THE TERMDEF AND TERMST, SEE VCEXTRACT
        j = len(trim(string(:len(trim(string))-15 ) ) )
!        print*, 'j = ', j, 'cahriut=', trim(adjustl(charit(j,0))), string
        read(string,'('//trim(adjustl(charit(j,0)))//'X,F15.8)',err=998)  termk(i)
!!        print*, 'TERMK = ', termk(i)
      endif

      goto 200

999   return

998   print*, 'ERROR READING PARAMETERS (readgapar)'
      return

      end subroutine readgapar
!#######################################################################
      subroutine gaout(io,kappa,lambda,gamma,iota,mu,expcoe,termk,&
                       kmask,lmask,gmask,imask,mmask,xmask,tmask,printfz,cspec)



      implicit none

      integer(long)             :: i,j,s,s1,m
      integer(long),intent(in)                                  :: io
      real(dop), dimension(nadd), intent(in)                    :: termk
      real(dop), dimension(nmodes,nsta), intent(in)             :: kappa
      real(dop), dimension(nmodes,nsta,nsta), intent(in)        :: lambda
      real(dop), dimension(nmodes,nmodes,nsta), intent(in)      :: gamma
      real(dop), dimension(maxcoe,nmodes,nsta), intent(inout)   :: expcoe
      real(dop), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: iota,mu
      integer(long), dimension(nadd), intent(in)                    :: tmask
      integer(long), dimension(nmodes,nsta), intent(in)             :: kmask
      integer(long), dimension(nmodes,nsta,nsta), intent(in)        :: lmask
      integer(long), dimension(nmodes,nmodes,nsta), intent(in)      :: gmask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(inout)   :: xmask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: imask,mmask
      character(len=39), intent(in)  :: cspec
      logical(kind=4),intent(in)     :: printfz
      character(len=200)             :: string

           write(io,'(A2,A30)',err=998) 'f ',trim(adjustl(datfile))
           if(printfz) write(io,'(A9,A25)',err=998) 'SPECIES :',trim(adjustl(cspec))
           do s=1,nsta
              do i=1,nmodes
                 if (kmask(i,s) .eq. 1 ) then
                   write(io,'(A1,2I3,F15.8)',err=998) 'k', i, s, kappa(i,s)
                 elseif (kmask(i,s) .eq. 5 .and. printfz .and. kappa(i,s) .ne. 0.0 ) then
                   write(io,'(A1,2I3,F15.8,X,A1)',err=998) 'k', i, s, kappa(i,s), 'f'
                 endif
                 do s1=s,nsta
                   if( s .le. nsta-1 .and. s1 .gt. s .and. ( lmask(i,s,s1) .eq. 1)  ) then
                       write(io,'(A1,3I3,F15.8)',err=998) 'l',i,s,s1,lambda(i,s,s1)
                   elseif( s .le. nsta-1 .and. s1 .gt. s .and. &
                   ( (lmask(i,s,s1) .eq. 5 .and. printfz .and. lambda(i,s,s1) .ne. 0.0)  ) ) then
                       write(io,'(A1,3I3,F15.8,X,A1)',err=998) 'l',i,s,s1,lambda(i,s,s1), 'f'
                   endif
                   do j=i,nmodes
                     if( s1 .eq. s .and. (gmask(i,j,s) .eq. 1) ) then
                       write(io,'(A1,3I3,F15.8)',err=998) 'g',i,j,s,gamma(i,j,s)
                     elseif( s1 .eq. s .and. (gmask(i,j,s) .eq. 5 .and. printfz .and. gamma(i,j,s) .ne. 0.0)  ) then
                       write(io,'(A1,3I3,F15.8,X,A1)',err=998) 'g',i,j,s,gamma(i,j,s), 'f'
                     endif
                     if( s1 .gt. s .and. s .le.  nsta-1 .and. ( mmask(j,i,s,s1) .eq. 1 ) ) then
                       write(io,'(A1,4I3,F15.8)',err=998) 'm', j, i, s, s1, mu(j,i,s,s1)
                     elseif( s1 .gt. s .and. s .le.  nsta-1 .and. &
                      (mmask(i,j,s,s1) .eq. 5 .and. printfz .and. mu(j,i,s,s1) .ne. 0.0)  ) then
                       write(io,'(A1,4I3,F15.8,X,A1)',err=998) 'm', j, i, s, s1, mu(j,i,s,s1), 'f'
                     endif
                      if (imask(i,j,s,s1) .eq. 1 ) then
                       write(io,'(A1,4I3,F15.8)',err=998)  'i', j, i, s, s1, iota(j,i,s,s1)
                      elseif (imask(i,j,s,s1) .eq. 5 .and. printfz .and. iota(j,i,s,s1) .ne. 0.0)  then
                       write(io,'(A1,4I3,F15.8,X,A1)',err=998)  'i', j, i, s, s1, iota(j,i,s,s1), 'f'
                      endif
                   enddo
                 enddo
                 if (ncoeff(i,s).gt.0) then
                    do k=1,ncoeff(i,s)
                      if( xmask(k,i,s) .eq. 1) then
                        write(io,'(A1,3I3,F15.8)',err=998) 'x', k,i,s, expcoe(k,i,s)
                      elseif(xmask(k,i,s) .eq. 5 .and. printfz .and. expcoe(k,i,s) .ne. 0.0)   then
                        write(io,'(A1,3I3,F15.8,X,A1)',err=998) 'x', k,i,s, expcoe(k,i,s),'f'
                      endif
                    enddo
                 endif
              enddo
           enddo
           do i = 1, nadd
            if ((tmask(i) .eq. 1 .or. (tmask(i) .eq. 5 .and. printfz )) .and. termk(i) .ne. 0.0 )  then
              string = ''
              do m=1,nmodes
                if( termdef(m,i) .ne. 0) &
                string = trim(trim(string)//' '//trim(adjustl(charit(m,0)))//'^'//&
                trim(adjustl(charit(termdef(m,i),0)) ))
              enddo

              if (tmask(i) .eq. 1  ) then
                write(io,'(A1,3I3,A'//trim(charit(len(trim(string)),0))//',F15.8)',err=998) &
               't',i,termst(1,i),termst(2,i), trim(string), termk(i)
              elseif ((tmask(i) .eq. 5 .and. printfz .and. termk(i) .ne. 0.0 )  ) then
                write(io,'(A1,3I3,A'//trim(charit(len(trim(string)),0))//',F15.8,X,A1)',err=998) &
               't',i,termst(1,i),termst(2,i), trim(string), termk(i),'f'
              endif
            endif
           enddo


       return
998    print*, 'ERROR WRITING GUESS (gabinout)'
       return

      end subroutine gaout
!#######################################################################

      subroutine outpar(io,const,kappa,lambda,gamma,iota,mu,expcoe,&
                        higher,termk,cpcoe,switch,cp2d)

      implicit none

      integer(long), intent(in) :: io
      integer(long), external   :: slen
      integer(long)             :: i,j,s,j1,j2,j3,nblck,n,s1,k,ilbl,jlbl,nhi,&
                                   l,m,itmp
      real(dop)                 :: freq,x0,d0,alpha,rtmp

      real(dop), dimension(nhigher), intent(in)                 :: higher
      real(dop), dimension(nadd), intent(in)                    :: termk
      real(dop), dimension(nsta,nsta), intent(in)               :: const
      real(dop), dimension(nmodes,nsta), intent(in)             :: kappa
      real(dop), dimension(nmodes,nsta)                         :: tempkappa
      real(dop), dimension(nmodes,nsta,nsta), intent(in)        :: lambda
      real(dop), dimension(nmodes,nmodes,nsta), intent(in)      :: gamma
      real(dop), dimension(nmodes,nmodes,nsta)                  :: tempgamma
      real(dop), dimension(maxcoe,nmodes,nsta), intent(inout)   :: expcoe
      real(dop), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: iota,mu
      real(dop), dimension(maxcoe,nmodes,nsta,nsta), intent(inout)  :: cpcoe

      real(dop), dimension(nswitch,maxcoe), intent(in)           :: switch
      real(dop), dimension(maxcoe,maxadd), intent(inout)         :: cp2d
      character(len=70)       :: buffer
      character(len=60)       :: string
      character(len=10)       :: atmp
      character(len=50)       :: fmt
      logical(kind=4)         :: igno

      write(io,'(a,f8.5)') '# VCHam Version :',progver
      ilbl=index(datfile,' ')-1
      write(io,'(2a)') '# Data file : ',datfile(1:ilbl)
      if (highord .gt. 0) &
           write(io,'(a,2i7)') '# High order terms : ',highord,nhigher
      write(io,*)

      write(io,'(a)') '*** Frequencies ***'
      do i=1,nmodes
         write(io,'(i3,2x,a5,f10.4)') i,nmlab(i),freq0(i)
      enddo
      write(io,*)

    !  if( lalleig ) write(io,'(a)') '*** Mass-scaled Only ***'
    !  if( lalleig ) write(io,*)
      if (i_ignore .gt. 0) then
         write(io,'(a)') '*** Omitted DOFs ***'
         do i = 1, nmodes
           if(ignoredof(i) )  write(io,'(i3,2x,a5,f10.4)') i,nmlab(i),freq0(i)
         enddo
         nmodes = nmodes - i_ignore
         write(io,*)
      endif

      if (lqtransform) then
         write(io,'(a)') '*** Transformed Coordinates ***'
         do i=1,nmodes
            if (qtransform(i) .eq. 1) then
               write(io,'(i3,2x,a10)') i,'cos'
            else if (qtransform(i) .eq. 2) then
               write(io,'(i3,2x,a10)') i,'sin'
            else if (qtransform(i) .eq. 3) then
               write(io,'(i3,2x,a10)') i,'halfcos'
            else if (qtransform(i) .eq. 4) then
               write(io,'(i3,2x,a10)') i,'halfsin'
            endif
         enddo
         write(io,*)
      endif

!      if (lrotmodes) then
!         write(io,'(/,a)') '*** Zeroth-order bilinear contribution ***'
!         write(iout,'(i3,x,i3,F20.10)') rotm1,rotm2,mixfreq
!      endif
!----------------------------------------------------------------------
! Zeroth-order bilinear contribution due to the rotation of a pair of
! normal modes (new version)
!----------------------------------------------------------------------
      if (lrotmodes) then
         j = 1
         igno = .false.
         write(io,'(a)') '*** Zeroth-order bilinear contributions ***'
         do while (rotm(j,1) .ne. 0)
            if( lnmcurv) then
              i = 1
              do while(nmcurvtype(i) .ne. 0)
               if(nmcurvtype(i) .eq. 1)then
                 if( nmcurv(i,1) .eq. rotm(j,1) .or. nmcurv(i,2) .eq. rotm(j,1) .or.&
                     nmcurv(i,3) .eq. rotm(j,1) ) igno = .true.
                 if( nmcurv(i,1) .eq. rotm(j,2) .or. nmcurv(i,2) .eq. rotm(j,2) .or.&
                     nmcurv(i,3) .eq. rotm(j,2) ) igno = .true.
               endif
              i = i + 1
              enddo
            endif
            if (igno .eqv. .false.) &
            write(io,'(i3,x,i3,F15.10)') rotm(j,1),rotm(j,2),rotmxfq(j)
            j = j + 1
            igno = .false.
         enddo
         write(io,*)
      endif

      if (lgeneral) then
         write(io,'(a)') '*** Diabatic Potential Form ***'
         do i=1,nsta
            do j=1,nmodes
               if (ifdiab(adof(j),i) .gt. 0) then
                  if (ifdiab(adof(j),i) .eq. 1) then
                     buffer='Quartic Oscillator'
                  else if (ifdiab(adof(j),i) .eq. 2) then
                     buffer='Morse Oscillator'
                  else if (ifdiab(adof(j),i) .eq. 3) then
                     buffer='Buckingham potential'
                  else if (ifdiab(adof(j),i) .eq. 4) then
                     buffer='Anti-Morse'
                  else if (ifdiab(adof(j),i) .eq. 5) then
                     buffer='Exponential'
                  else if (ifdiab(adof(j),i) .eq. 6) then
                     buffer='Avoided-Crossing'
                  else if (ifdiab(adof(j),i) .eq. 7) then
                     buffer='General Morse'
                  else if (ifdiab(adof(j),i) .eq. 8) then
                     buffer='General Harmonic'
                  else if (ifdiab(adof(j),i) .eq. 9) then
                     buffer='General Quartic'
                  else if (ifdiab(adof(j),i) .eq. 10) then
                     buffer='Lennard-Jones'
                  else if (ifdiab(adof(j),i) .eq. 11) then
                     buffer='Anti-Lennard-Jones'
                  else if (ifdiab(adof(j),i) .eq. 12) then
                     buffer='General AQuartic'
                  else if (ifdiab(adof(j),i) .eq. 13) then
                     buffer='Morse + Lorentzian'
                  else if (ifdiab(adof(j),i) .eq. 14) then
                     buffer='Morse + Gaussian'
                  else if (ifdiab(adof(j),i) .eq. 15) then
                     buffer='General Quadratic'
                  else if (ifdiab(adof(j),i) .eq. 16) then
                     buffer='Avoided-Crossing2'
                  else if (ifdiab(adof(j),i) .eq. 17) then
                     buffer='Avoided-Crossing_Upper2'
                  else if (ifdiab(adof(j),i) .eq. 18) then
                     buffer='avoided_crossing_quartic'
                  else if (ifdiab(adof(j),i) .eq. 19) then
                     buffer='JT4'
                  else if (ifdiab(adof(j),i) .eq. 20) then
                     buffer='RRP'
                  else if (ifdiab(adof(j),i) .eq. 22) then
                     buffer='const'
                  else if (ifdiab(adof(j),i) .eq. 23) then
                     buffer='1/R'
                  endif
                  n=ncoeff(adof(j),i)
                  write(io,'(i3,2x,i3,2x,a5,2i5,2x,a20)') &
                       adof(j),i,nmlab(adof(j)),ifdiab(adof(j),i),n,buffer
               endif
            enddo
         enddo

         buffer=''
         write(io,'(/,a)') '*** Expansion Coefficients ***'
         do i=1,nsta
            do j=1,nmodes
               n=ncoeff(adof(j),i)
               if (n .gt. 0) then
                  if (ifdiab(adof(j),i) .eq. 1) then
                     tempkappa(adof(j),i)=expcoe(1,adof(j),i)
                     tempgamma(adof(j),adof(j),i)=expcoe(2,adof(j),i)
                     buffer = 'kappa, gamma, epsilon'

                  else if (ifdiab(adof(j),i) .eq. 2) then
                     d0 = expcoe(1,adof(j),i)
                     tempgamma(adof(j),adof(j),i) = expcoe(2,adof(j),i)
                     x0 = expcoe(3,adof(j),i)
                     freq = freq0(adof(j)) + tempgamma(adof(j),adof(j),i)
                     alpha=freq/(2.0*d0)
                     alpha=sqrt(alpha)

                     tempkappa(adof(j),i)=exp(alpha*x0) - &
                          exp(2.0*alpha*x0)
                     tempkappa(adof(j),i)=2.0*alpha*d0*kappa(adof(j),i)
                     tempkappa(adof(j),i) = min(kappa(adof(j),i), 999.0_dop)

                     D0=D0/27.21128
                     E0=-(D0*(exp(alpha*x0)-1)**2)

                     buffer = 'D0, gamma, x0'

                  else if (ifdiab(adof(j),i) .eq. 3) then
                     d0 = expcoe(1,adof(j),i)
                     tempgamma(adof(j),adof(j),i) = expcoe(2,adof(j),i)
                     x0 = expcoe(3,adof(j),i)

                     freq = freq0(adof(j)) + tempgamma(adof(j),adof(j),i)
                     alpha=freq/(2.0*d0)
                     alpha=sqrt(alpha)

                     tempkappa(adof(j),i)=exp(alpha*x0) - exp(2.0*alpha*x0)
                     tempkappa(adof(j),i)=2.0*alpha*d0*kappa(adof(j),i)

                     buffer = 'D0, gamma, x0, B6, xs, x06'

                  else if (ifdiab(adof(j),i) .eq. 4) then
                     d0 = expcoe(1,adof(j),i)
                     tempgamma(adof(j),adof(j),i) = expcoe(2,adof(j),i)
                     x0 = expcoe(3,adof(j),i)

                     freq = freq0(adof(j)) + tempgamma(adof(j),adof(j),i)
                     alpha=freq/(2.0*d0)
                     alpha=sqrt(alpha)
                     alpha=-alpha
                     tempkappa(adof(j),i)=exp(alpha*x0) -&
                          exp(2.0*alpha*x0)
                     tempkappa(adof(j),i)=2.0*alpha*d0*kappa(adof(j),i)
                     D0=D0/27.21128
                     E0=-(D0*(exp(alpha*x0)-1)**2)
                     buffer = 'D0, gamma, x0'

                  else if (ifdiab(adof(j),i) .eq. 5) then
                     buffer = 'Einf, alpha'

                  else if (ifdiab(adof(j),i) .eq. 6) then
                     buffer = 'something useful'

                  else if (ifdiab(adof(j),i) .eq. 7) then
                     buffer = 'D0, alpha, q0'

                  else if (ifdiab(adof(j),i) .eq. 8) then
                     buffer = 'k1'

                  else if (ifdiab(adof(j),i) .eq. 9) then
                     buffer = 'k1,k2,k3'

                  else if (ifdiab(adof(j),i) .eq. 10) then
                     buffer = 'D0,alpha,X'

                  else if (ifdiab(adof(j),i) .eq. 11) then
                     buffer = 'D0,alpha,X'

                  else if (ifdiab(adof(j),i) .eq. 12) then
                     buffer = 'k1,k2,k3,k4'

                  else if (ifdiab(adof(j),i) .eq. 13) then
                     buffer = 'D0,alpha,X0,E0,d,e,f'

                  else if (ifdiab(adof(j),i) .eq. 14) then
                     buffer = 'D0,alpha,X0,E0,d,e,f'

                  else if (ifdiab(adof(j),i) .eq. 15) then
                     buffer = 'k1,k2'

                  else if (ifdiab(adof(j),i) .eq. 18) then
                     buffer = 'gL,eL,gU,eU,lambda,dE'
                     expcoe(6,adof(j),i)=abs(expcoe(6,adof(j),i))

                  else if (ifdiab(adof(j),i) .eq. 19) then
                     buffer = 'kappa,gamma,eta,zeta,iota,tau,alpha,beta'
                  else if (ifdiab(adof(j),i) .eq. 19) then
                     buffer = 'A,order,shift'
                  else if (ifdiab(adof(j),i) .eq. 21) then
                     buffer = 'lambda'
                  else if (ifdiab(adof(j),i) .eq. 22) then
                     buffer = 'const'
                  else if (ifdiab(adof(j),i) .eq. 23) then
                     buffer = 'A6, q0'
                  endif

                  fmt=''
                  write(fmt(1:9),'(a8)') '(3i5,2x,'
                  if (len_trim(buffer).lt.10) then
                     write(fmt(9:11),'(a1,i1,a1)') 'a',&
                          len_trim(buffer),')'
                  else
                     write(fmt(9:13),'(a1,i2,a2)') 'a',&
                          len_trim(buffer),')'
                  endif

                  write(io,fmt) adof(j),i,n,buffer(1:len_trim(buffer))
                  write(io,'(6e13.5)') (expcoe(k,adof(j),i),k=1,n)
               endif
            enddo
         enddo
      endif

      if (lgeneralcp) then
         write(io,'(/,a)') '*** Nonadiabatic Coupling Form ***'
         do j=1,nsta-1
            do i=j+1,nsta
               do m=1,nmodes
                  if (ifdiabcp(adof(m),i,j).gt.0) then
                     if (ifdiabcp(adof(m),i,j).eq.1) then
                        buffer='tanh'
                     else if (ifdiabcp(adof(m),i,j).eq.2) then
                        buffer='poly_to_tanh'
                     else if (ifdiabcp(adof(m),i,j).eq.3) then
                        buffer='tanh2'
                     endif
                     n=ncoeffcp(adof(m),i,j)
                     write(io,'(i3,2x,i3,2x,i3,2x,a5,2i5,2x,a20)') &
                          adof(m),j,i,nmlab(m),ifdiabcp(m,i,j),n,buffer
                  endif
               enddo
            enddo
         enddo
         write(io,'(/,a)') '*** Nonadiabatic Coupling Parameters ***'
         do j=1,nsta-1
            do i=j+1,nsta
               do m=1,nmodes
                  n=ncoeffcp(adof(m),j,i)
                  if (n.gt.0) then
                     if (ifdiabcp(adof(m),i,j).eq.2) cpcoe(5,m,j,i)=abs(cpcoe(5,m,j,i))
                     write(io,'(4i5)') adof(m),j,i,n
                     write(io,'(6e13.5)') (cpcoe(k,adof(m),j,i),k=1,n)
                  endif
               enddo
            enddo
         enddo
      endif

      if (lswitch) then
         write(io,'(/,a)') '*** Switching Functions ***'
         write(io,'(i5)') nswitch
         do i=1,nswitch
            write(io,'(x,a)') aswitch(i)(1:len_trim(aswitch(i)))
            do j=1,nswitchcoe(i)
               write(io,'(x,i2,e13.5)') j,switch(i,j)
            enddo
         enddo
      endif

      if (ldirot .or. lconstall) then
         write(io,'(/,a)') '*** Constants ***'
         write(io,'(1x,a6,8(2x,i5,3x))') 'State:',(s,s=1,nsta)
         do s1=1,nsta
            write(io,'(i8,8f10.4)') s1,(const(s1,s),s=1,s1)
         enddo
      else
         write(io,'(/,a)') '*** On-diagonal constants ***'
         write(io,'(1x,a6)') 'State:'
         do s1=1,nsta
            write(io,'(i8,f10.4)') s1,const(s1,s1)
         enddo
      endif

! OLD FORMAT: DOES NOT ALLOW FOR MORE THAN 8 STATES
!      write(io,'(/,a)') &
!          '*** On-diagonal linear (kappa) coupling constants ***'
!      write(io,'(4x,a6,8(6x,i1,3x))') 'State:',(mod(s,10),s=1,nsta)
!      do i=1,nmodes
!         write(io,'(i3,2x,a5,8f10.4)') i,nmlab(i),&
!            (kappa(i,s),s=1,nsta)
!      enddo
! NEW FORMAT: ARBITRARY NUMBER OF STATES WRITTEN IN BLOCKS OF 5
      write(io,'(/,a)') &
          '*** On-diagonal linear coupling constants (kappa) ***'
      rtmp=(nsta/5.0)
      itmp=int(rtmp)
      nblck=itmp
      if ((rtmp-itmp).gt.0.0) nblck=nblck+1
      do n=1,nblck
         j=(n-1)*5+1
         k=0
         if (n.eq.nblck) then
            k=j+4-nsta
         endif
         write(io,'(4x,a6,5(6x,i2,2x))') 'State:', (s, s=j,j+4-k)
         do i=1,nmodes
            write(io,'(i3,2x,a5,8f10.4)') adof(i),nmlab(adof(i)),&
                 (kappa(adof(i),s),s=j,j+4-k)
         enddo
         write(io,*)
      enddo
      backspace(io)

      if (lsincoup)  write(io,'(/,a)') '*** Fourier Lambda Used,sin ***'
     do i=1,nmodes
      if (lcoscoup(i))  then
      write(io,'(/,a,i2)') '*** Fourier Lambda Used,cos ***, for normal mode',i
      endif
     enddo
      write(io,'(/,a)') &
          '*** Off-diagonal linear (lambda) coupling constants ***'
      do s=1,nsta-1
         do s1=s+1,nsta
            write(io,'(4x,a6,2(6x,i1,3x))') 'State:',&
              mod(s,10),mod(s1,10)
            do i=1,nmodes
               write(io,'(i3,2x,a5,f10.4)') adof(i),nmlab(adof(i)),&
                    lambda(adof(i),s,s1)
               write(string,'(f10.4)') lambda(adof(i),s,s1)
!       DEBUGGY
               if( index(string, "NaN") .ne. 0 ) then
                print*,  'DEBUG, lambda: ', lambda(adof(i),s,s1)
                 write(6,'(a15,F10.4)') 'DEBUG, lambda: ', lambda(adof(i),s,s1)
               endif
!,lambda1(i,s,s1),lambda2(i,s,s1)
            enddo
         enddo
      enddo

! if linear model do not output 2nd order coefficients
      if (hamord .eq. 1) go to 100

! OLD FORMAT: DOES NOT ALLOW FOR MORE THAN 8 STATES
!      write(io,'(/,a)')&
!         '*** On-diagonal quadratic (gamma) constants ***'
!      write(io,'(4x,a6,8(6x,i1,3x))') 'State:',(mod(s,10),s=1,nsta)
!      do i=1,nmodes
!         write(io,'(i3,2x,a5,8f10.4)') i,nmlab(i),&
!            (gamma(i,i,s),s=1,nsta)
!      enddo
! NEW FORMAT: ARBITRARY NUMBER OF STATES WRITTEN IN BLOCKS OF 5
      write(io,'(/,a)')&
           '*** On-diagonal quadratic constants (gamma) ***'
      rtmp=(nsta/5.0)
      itmp=int(rtmp)
      nblck=itmp
      if ((rtmp-itmp).gt.0.0) nblck=nblck+1
      do n=1,nblck
         j=(n-1)*5+1
         k=0
         if (n.eq.nblck) then
            k=j+4-nsta
         endif
         write(io,'(4x,a6,5(6x,i2,2x))') 'State:', (s, s=j,j+4-k)
         do i=1,nmodes
            write(io,'(i3,2x,a5,8f10.4)') adof(i),nmlab(adof(i)),&
                 (gamma(adof(i),adof(i),s),s=j,j+4-k)
         enddo
         write(io,*)
      enddo
      backspace(io)

      write(io,'(/,a)') &
         '*** All on-diagonal bilinear (gamma) constants ***'
!      print*, 'i_ignore =', i_ingore
      nblck=((nmodes-1)/6)+1
      do s=1,nsta
         write(io,'(4x,a6,6x,i1)') 'State:',s
         do n=1,nblck
            j1=(n-1)*6+1
            j2=min(j1+5,nmodes)
            write(io,'(10x,6(5x,i2,3x))') (adof(j),j=j1,j2)
            do i=j1,nmodes
               j3=min(i,j2)
               write(io,20) &
               adof(i),nmlab(adof(i)),&
               (gamma(adof(i),adof(j),s),j=j1,j3)
            enddo
         enddo
         write(io,*)
      enddo

! if 2nd order adiabatic model do not output mu coefficients
      if (hamord .eq. 2) go to 100

      write(io,'(/,a)') '*** Off-diagonal bilinear (mu) constants ***'
      nblck=((nmodes-1)/6)+1
!     print*, 'in outpar'
!       print*,    mu(1,9,1,3)
      do s=1,nsta-1
         do s1=s+1,nsta
            write(io,'(4x,a6,2(6x,i1,3x))') 'State:',&
              mod(s,10),mod(s1,10)
            do n=1,nblck
               j1=(n-1)*6+1
               j2=min(j1+5,nmodes)
               write(io,'(10x,6(5x,i2,3x))') (adof(j),j=j1,j2)
               do i=j1,nmodes
                  j3=min(i,j2)
                  write(io,20) adof(i),nmlab(adof(i)),&
                  (mu(adof(j),adof(i),s,s1),j=j1,j3)
!      write(6,20) adof(i), nmlab(adof(i)), (mu(adof(i),adof(j),s,s1),j=j1,j3)
!      write(6,20) i,nmlab(i), (mu(j,i,s,s1),j=j1,j3)
               enddo
            enddo
            write(io,*)
         enddo
      enddo

      if (hamord .eq. 3) go to 100


! Writes 'quadratic-lin' instead of 'quad-linear' to distinguish between
! the old (icorrect) format and the new format. Do not change!
      write(io,'(/,a)')&
         '*** On-diagonal quad-lin (iota) constants ***'
      nblck=((nmodes-1)/3)+1
      do s=1,nsta
         write(io,'(4x,a6,6x,i1)') 'State:',s
         do n=1,nblck
            j1=(n-1)*3+1
            j2=min(j1+2,nmodes)
            write(io,'(9x,3(5x,i2,7x))') (adof(j),j=j1,j2)
            do i=1,nmodes
               j3=j2
               write(io,21) adof(i),nmlab(adof(i)),&
               (iota(adof(i),adof(j),s,s),j=j1,j3)
            enddo
         enddo
         write(io,*)
      enddo

      write(io,'(/,a)')&
         '*** Off-diagonal quadratic-linear (iota2) constants ***'
      nblck=((nmodes-1)/6)+1
      do s=1,nsta-1
         do s1=s+1,nsta
            write(io,'(4x,a6,2(6x,i1,3x))') 'State:',&
              mod(s,10),mod(s1,10)

            do n=1,nblck
               j1=(n-1)*6+1
               j2=min(j1+5,nmodes)
               write(io,'(10x,6(5x,i2,3x))') (adof(j),j=j1,j2)
               do i=j1,nmodes
                  j3=j2
                  write(io,20) adof(i),nmlab(adof(i)),&
                  (iota(adof(i),adof(j),s,s1),j=j1,j3)
               enddo
            enddo
            write(io,*)
         enddo
      enddo

  100 continue

      if (highord .ge. 3) then
         nhi = 0

! gam_ijk q_i q_j q_k terms
         write(io,'(/,a)')&
            '*** On-diagonal lin-lin-lin third order constants ***'
         do s = 1,nsta
            write(io,'(4x,a6,6x,i1)') 'State:',s
            do i=1,nmodes
               do j = i+1,nmodes
                  do k = j+1,nmodes
                     nhi = nhi+1
                     write(io,'(3i5,f10.4)') adof(i),adof(j),adof(k),higher(nhi)
                  enddo
               enddo
            enddo
            write(io,*)
         enddo
      endif

      if (highord .eq. 4) then

! gam_iijj q_i^2 q_j^2 terms
         write(io,'(/,a)')&
            '*** On-diagonal quad-quad fourth order constants ***'

         do s=1,nsta
            write(io,'(4x,a6,6x,i1)') 'State:',s
            do i=1,nmodes-1
               do j = i+1,nmodes
                  nhi = nhi+1
                  write(io,'(4i5,f10.4)') adof(i),adof(i),adof(j),adof(j),higher(nhi)
               enddo
            enddo
            write(io,*)
         enddo

! gam_iiij q_i^3 q_j terms
         write(io,'(/,a)')&
            '*** On-diagonal cube-lin fourth order constants ***'
         do s=1,nsta
            write(io,'(4x,a6,6x,i1)') 'State:',s
            do i=1,nmodes-1
               do j = i+1,nmodes
                  nhi = nhi+1
                  write(io,'(4i5,f10.4)') adof(i),adof(i),adof(i),adof(j),higher(nhi)
               enddo
            enddo
            do i=2,nmodes
               do j = 1,i-1
                  nhi = nhi+1
                  write(io,'(4i5,f10.4)') adof(i),adof(i),adof(i),adof(j),higher(nhi)
               enddo
            enddo
            write(io,*)
         enddo

! gam_iijk q_i^2 q_j q_k terms
         write(io,'(/,a)')&
            '*** On-diagonal quad-lin-lin fourth order constants ***'
         do s=1,nsta
            write(io,'(4x,a6,6x,i1)') 'State:',s
            do i=1,nmodes-2
               do j = i+1,nmodes-1
                  do k = j+1,nmodes
                     nhi = nhi+1
                     write(io,'(4i5,f10.4)') adof(i),adof(i),adof(j),adof(k),higher(nhi)
                  enddo
               enddo
            enddo
            do i=3,nmodes
               do j = 1,i-2
                  do k = j+1,i-1
                     nhi = nhi+1
                     write(io,'(4i5,f10.4)') adof(i),adof(i),adof(j),adof(k),higher(nhi)
                  enddo
               enddo
            enddo
            do i=2,nmodes-1
               do j = 1,i-1
                  do k = i+1,nmodes
                     nhi = nhi+1
                     write(io,'(4i5,f10.4)') adof(i),adof(i),adof(j),adof(k),higher(nhi)
                  enddo
               enddo
            enddo
            write(io,*)
         enddo

! gam_ijkl q_i q_j q_k q_l terms
         write(io,'(/,a)')&
            '*** On-diagonal lin-lin-lin-lin fourth order constants ***'
         do s=1,nsta
            write(io,'(4x,a6,6x,i1)') 'State:',s
            do i=1,nmodes-3
               do j = i+1,nmodes-2
                  do k = j+1,nmodes-1
                     do l = k+1,nmodes
                        nhi = nhi+1
                        write(io,'(4i5,f10.4)') adof(i),adof(j),adof(k),adof(l),higher(nhi)
                     enddo
                  enddo
               enddo
            enddo
            write(io,*)
         enddo

      endif

      if (nadd .gt. 0) then
! Early formats did not include n in list of parameters. The character 'N'
! in the header line allows the new format to be detected.
         write(io,'(/,a)')&
            '*** Explicitely added polynomial terms ***'
         write(io,'(a3,i5)') '  N',nadd
         do n = 1,nadd
            ilbl = 1
            string= ' '
            do m=1,nmodes
               if (termdef(adof(m),n) .gt. 0) then
                  write(atmp,'(i4,a,i4)') adof(m),'^',termdef(adof(m),n)
                  call rmblnk(atmp)
                  jlbl=slen(atmp)
                  string(ilbl:ilbl+jlbl-1)=atmp(1:jlbl)
                  ilbl=ilbl+jlbl+1
               endif
            enddo
            write(io,'(i3,1x,E13.5,2x,2i4,2x,a50)') n,termk(n),termst(1,n),&
                  termst(2,n),string(1:50)
         enddo
      endif

     s = 1
     if(cp2dtyp(s) .ne. 0) write(io,'(/,a)')&
      '***2D Coupling Parameters***'
     do while (cp2dtyp(s) .ne. 0)
       select case (cp2dtyp(s))
       case (1)
         write(io,'(A10,2X,5I2,A3,E13.5)') 'ltanh2d',cp2dtyp(s),dofcp2d(1,s),dofcp2d(2,s),&
         stacp2d(1,s),stacp2d(2,s), ' = ', cp2d(1,s)
       case (2)
         write(io,'(A10,2X,5I2,A3,E13.5)') 'mtanh2d',cp2dtyp(s),dofcp2d(1,s),dofcp2d(2,s),&
         stacp2d(1,s),stacp2d(2,s), ' = ', cp2d(1,s)
       case (3)
         write(io,'(A8,2X,4A2,A3,A8,X,A8,X,A8,X,A8)') 'type ',' ','m1','m2','s1','s2',' = ',&
         'height','grad','shift','cuadrant'
         string = '(A8,2X,5I2,A3,F8.4,X,F8.4,X,F8.4,X,F8.4)'
         write(io,trim(string)) 'radtanch',cp2dtyp(s),dofcp2d(1,s),dofcp2d(2,s),&
         stacp2d(1,s),stacp2d(2,s), ' = ', cp2d(1,s), cp2d(2,s), cp2d(3,s), cp2d(4,s)
         print*, cp2d(1,s), cp2d(2,s), cp2d(3,s), cp2d(4,s)
       case (4)
         write(io,'(A8,2X,4A2,A3,A8,X,A8,X,A8,X,A8)') 'type ',' ','m1','m2','s1','s2',' = ',&
         'height','grad','shift','angle'
         string = '(A8,2X,5I2,A3,F8.4,X,F8.4,X,F8.4,X,F8.4)'
         write(io,trim(string)) 'vectanch',cp2dtyp(s),dofcp2d(1,s),dofcp2d(2,s),&
         stacp2d(1,s),stacp2d(2,s), ' = ', cp2d(1,s), cp2d(2,s), cp2d(3,s), cp2d(4,s)
         print*, cp2d(1,s), cp2d(2,s), cp2d(3,s), cp2d(4,s)
       case (5)
         write(io,'(A8,2X,4A2,A3,A8,X,A8,X,A8,X,A8,X,A8)') 'type ',' ','m1','m2','s1','s2',' = ',&
         'height','grad','shift','angle','order'
         string = '(A8,2X,5I2,A3,F8.4,X,F8.4,X,F8.4,X,F8.4,X,F8.4)'
         write(io,trim(string)) 'polybox',cp2dtyp(s),dofcp2d(1,s),dofcp2d(2,s),&
         stacp2d(1,s),stacp2d(2,s), ' = ', cp2d(1,s), cp2d(2,s), cp2d(3,s), cp2d(4,s),cp2d(5,s)
         print*, cp2d(1,s), cp2d(2,s), cp2d(3,s), cp2d(4,s),cp2d(5,s)
       case (6)
         write(io,'(A8,2X,5A2,A3,A8,X,A8,X,A8)') 'type ',' ','m1','m2','s1','s2',' o',' = ',&
         'prefact','shift','angle'
         string = '(A8,2X,6I2,A3,F8.4,X,F8.4,X,F8.4)'
         write(io,trim(string)) 'polybox2',cp2dtyp(s),dofcp2d(1,s),dofcp2d(2,s),&
         stacp2d(1,s),stacp2d(2,s),ocp2d(s),  ' = ', cp2d(1,s), cp2d(2,s), cp2d(3,s)
       end select
         s = s + 1
     enddo
     if(cp2dtyp(1) .ne. 0) write(io,*)

20    format(i3,2x,a5,6f10.4)
21    format(i3,2x,a5,6f14.7)

! Due to the way these files are read 3 blank lines are needed.  This is usually done
! when writing the mask but in the genetic algorithm this is not used so it is added
! here.

      if (lgenetica) then
          write(io,'(a1)') ' '
          write(io,'(a1)') ' '
          write(io,'(a1)') ' '
      endif

      if (i_ignore .gt. 0) nmodes = nmodes + i_ignore

      return
      end subroutine outpar

!#######################################################################
! SMALE
! due to how memory is allocated, confidence requires a lot as
! hess, covar, correl are all optcoodim*optcoodim but really only
! need to be ntvar*ntvar. Should really be called after fitting
! is finished through its own memory allocation routine
!
! no idea whether this all works properly anyway
! dodgy passing mr/mi AND arrays const etc.
!#######################################################################

      subroutine confidence(optcoo,const,kappa,lambda,gamma,iota,&
                 mu,expcoe,&
                 const_mask,kappa_mask,lambda_mask,gamma_mask,iota_mask,&
                 mu_mask,&
                 expcoe_mask,termk_mask,hess,covar,correl,dpar,errpar,mr,mi)

      implicit none

      integer(long)       :: i,j,nblck,j1,j2,j3,n

      integer(long), dimension(nsta,nsta), intent(inout)               :: const_mask
      integer(long), dimension(nmodes,nsta), intent(inout)             :: kappa_mask
      integer(long), dimension(nmodes,nsta,nsta), intent(inout)        :: lambda_mask
      integer(long), dimension(nmodes,nmodes,nsta), intent(inout)      :: gamma_mask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(inout)      :: expcoe_mask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(inout) :: iota_mask,mu_mask
      integer(long), dimension(nadd), intent(inout) :: termk_mask

      real(dop), dimension(optcoodim), intent(out)              :: optcoo
      real(dop), dimension(nsta,nsta), intent(in)               :: const
      real(dop), dimension(nmodes,nsta), intent(in)             :: kappa
      real(dop), dimension(nmodes,nsta,nsta), intent(in)        :: lambda
      real(dop), dimension(nmodes,nmodes,nsta), intent(in)      :: gamma
      real(dop), dimension(maxcoe,nmodes,nsta), intent(in)      :: expcoe
      real(dop), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: iota,mu

      integer(long), dimension(midim), intent(inout) :: mi
      real(dop), dimension(mrdim), intent(inout)     :: mr

      real(dop)                                      :: h,f1,f2,f3,f4,h1,h2,optcoo_i,optcoo_j,wssr,stdfit,redchis
      real(dop), dimension(ntvar,ntvar), intent(out) :: hess,covar,correl
      real(dop), dimension(nvar), intent(out)        :: dpar,errpar
      integer(long)                                  :: ndf
!      real(dop), external                            :: vcrmsd

      call unitqxd(hess,ntvar)

      print*,'confidence estimation:'

      write(ilog,'(a)') ' '
      write(ilog,'(20x,a)')'###########################################'
      write(ilog,'(20x,a)')'             confidence estimation '
      write(ilog,'(20x,a)')'###########################################'
      do i=1,4
         write(ilog,'(a)') ' '
      enddo

!-----------------------------------------------------------------------
! make all non-zero parameters active
! load all non-zero parameters into variable vector
!-----------------------------------------------------------------------
      call active_mask(n,const_mask,kappa_mask,lambda_mask,gamma_mask,&
           iota_mask,mu_mask,expcoe_mask,termk_mask)
      if (n .ne. ntvar) then
         routine='Confidence'
         write(message,'(a,2i5)') &
               'wrong no. of active parameters:',n,ntvar
         call vcerrormsg
      endif

      call par2vec2d(optcoo,mr(mrconst),mr(mrkappa),mr(mrlambda),&
           mr(mrgamma),mr(mriota),mr(mrmu),mr(mrexpcoe),&
           mr(mrhigher),mr(mrtermk),mr(mrcpcoe),&
           mr(mrswitch),mr(mrcp2d),mi(miconstmask),mi(mikappamask),&
           mi(milambdamask),mi(migammamask),mi(miiotamask),&
           mi(mimumask),mi(miexpcoemask),mi(mitermkmask),&
           mi(micpcoemask),mi(miswitchmask),mi(micp2d))

!     calculate the correlation for all parameters
!     degree of freedom
      ndf = nsets - ntvar
!     weighted sum of squares of residual
      wssr=vcrmsd(optcoo,mr,mi)
!     rms of residuals
      stdfit=sqrt(wssr/ndf)
!     reduced chisquare
      redchis=wssr/ndf

!     write it
      write(ilog,'(a,I6)')   &
           ' total sum of points (nsets)             '//&
           '             : ',nsets
      write(ilog,'(a,I6)')   &
           ' total sum of fitted parameters (nvar)   '//&
           '             : ',nvar
      write(ilog,'(a,F10.5)')&
           ' weighted sum of squares of residual (wss'//&
           'r)           : ',wssr
      write(ilog,'(a,I6)')   &
           ' degrees of freedom (ndf)                '//&
           '             : ',ndf
      write(ilog,'(a,F10.5)')&
           ' rms of residuals      (stdfit) = sqrt(ws'//&
           'sr/ndf)      : ',stdfit
      write(ilog,'(a,F10.5)')&
           ' variance of residuals (reduced chisquare'//&
           ') = wssr/ndf : ',redchis

      h=0.01_dop
! calculate the hessian of the parameters (numerical)
      print*, '--> calculate the hessian '
      do i=1,ntvar
         print*,i
         do j=1,ntvar
            optcoo_i =optcoo(i)
            optcoo_j =optcoo(j)
            h1=abs(optcoo_i)/100.0+1.0e-6
            h2=abs(optcoo_j)/100.0+1.0e-6
!            h1=.20E-3
!            h2=.20E-3
            optcoo(i)=optcoo(i)+h1
            optcoo(j)=optcoo(j)+h2
!           rms2d(i+h,j+h)
            f1=vcrmsd(optcoo,mr,mi)
            optcoo(i)=optcoo(i)-h1-h1
!           rms2d(i-h,j+h)
            f2=vcrmsd(optcoo,mr,mi)
            optcoo(j)=optcoo(j)-h2-h2
!           rms2d(i-h,j-h)
            f3=vcrmsd(optcoo,mr,mi)
            optcoo(i)=optcoo(i)+h1+h1
!           rms2d(i+h,j-h)
            f4=vcrmsd(optcoo,mr,mi)
            hess(i,j)=(f1-f2+f3-f4)/(4.0*h1*h2)
!            if (i.ne.j) hess(j,i)=hess(i,j)
            optcoo(i)=optcoo_i
            optcoo(j)=optcoo_j
         enddo
      enddo


!     stabilize the numeric hessian
      do i=1,ntvar
!c        the model depends not on optcoo(i)
!         if (hess(i,i).eq.0.0_dop) hess(i,i)=1.0_dop
         do j=i,ntvar
            hess(i,j)=0.5*(hess(i,j)+hess(j,i))
            hess(j,i)=hess(i,j)
         enddo
!c         write(6,'(4F8.2,a,F10.5)')(hess(i,j),j=1,4),'|',optcoo(i)
      enddo

      do i=1,ntvar
         if (hess(i,i).le.0.0) then
            print*,i,hess(i,i)
            hess(i,i)=1.0
         endif
      enddo

!     calculate the inverse if the hessian
      print*,'--> calculate the inverse of the hessian'
      call invert(hess,covar,ntvar,ntvar)

!     calculate the variance
      do i=1,ntvar
         if (covar(i,i).ge.0.0) then
            dpar(i)=sqrt(covar(i,i))
         else
            print*,' !! '
            print*,'  ==> Non-positive diagonal element in covar.'//&
                 ' matrix !'
            print*,' !! '
            print*,' Can this still happens ?!'
            print*,covar(i,i)
            dpar(i)=0.0
         endif
!        the error in per cent

!         errpar(i)=dpar(i)
         errpar(i)=abs(dpar(i))

      enddo
!     calculate the correlation matrix
      do i=1,ntvar
         do j=1,ntvar
            correl(i,j)=covar(i,j)/(dpar(i)*dpar(j))
         enddo
      enddo

!     copy the errors for printing in the parameters
      call vec2par2d(optcoo,mr(mrconst),mr(mrkappa),mr(mrlambda),&
           mr(mrgamma),mr(mriota),mr(mrmu),mr(mrexpcoe),&
           mr(mrhigher),mr(mrtermk),mr(mrcpcoe),&
           mr(mrswitch),mr(mrcp2d),mi(miconstmask),mi(mikappamask),&
           mi(milambdamask),mi(migammamask),mi(miiotamask),&
           mi(mimumask),mi(miexpcoemask),mi(mitermkmask),&
           mi(micpcoemask),mi(miswitchmask),mi(micp2d))
      write(ilog,'(a)')
      write(ilog,'(a)')'-------------------------------------------'//&
           '--------------------------'
      write(ilog,'(a)')'     asymptotic standard errors for the par'//&
           'ameters in per cent :'
      write(ilog,'(a)')'-------------------------------------------'//&
           '--------------------------'
      write(ilog,'(a)') ' '
!     1 is a flag for printing only fitted values
      call outpar(iout,mr(mrconst),mr(mrkappa),mr(mrlambda),&
           mr(mrgamma),mr(mriota),mr(mrmu),mr(mrexpcoe),&
           mr(mrhigher),mr(mrtermk),mr(mrcpcoe),&
           mr(mrswitch),mr(mrcp2d))
!     restore the true parameters
!     call vec2par2d(optcoo,mr(mrconst),mr(mrkappa),mr(mrlambda),
!    +     mr(mrgamma),mr(mrmu),mr(mrexpcoe),mr(mrhigher),mr(mrtermk),
!    +     mi(miconstmask),
!    +     mi(mikappamask),mi(milambdamask),mi(migammamask),
!    +     mi(mimumask),mi(miexpcoemask))

!     write the correlation between the parameters
      write(ilog,'(a)') ' '
      write(ilog,'(a)')'-------------------------------------------'//&
           '-----------------------------------------'
      write(ilog,'(a)')'     the correlation between the parameters'//&
           ' '
      write(ilog,'(a)')'-------------------------------------------'//&
           '-----------------------------------------'
      write(ilog,'(a)') ' '
      nblck=((nvar-1)/10)+1
      do n=1,nblck
         j1=(n-1)*10+1
         j2=min(j1+9,ntvar)
         write(ilog,'(5x,10I7)') (j,j=j1,j2)
         do i=j1,ntvar
            j3=min(i,j2)
            write(ilog,'(I5,10F7.3)') i,(correl(i,j),j=j1,j3)
         enddo
         write(ilog,'(a)') &
              '-------------------------------------------'//&
              '--------------------------------------'
      enddo

      return
      end subroutine confidence

!##############################################################
! invert the matrix A -> B
!##############################################################

      subroutine invert(A,B,max,n)

      implicit none

      integer(long), parameter                     :: maxdim=500

      integer(long), intent(in)                    :: max,n
      real(dop), dimension(max,max), intent(inout) :: A
      real(dop), dimension(max,max), intent(out)   :: B
      real(dop), dimension(maxdim,maxdim)          :: C,restore
      real(dop)                                    :: pivot,help
      integer(long)                                :: i,j,k

      if (max.gt.maxdim) then
         write(6,'(a)') '!! Dimension-error in subroutine invert !!'
         stop
      endif

      do j=1,n
         do i=1,n
            restore(i,j)=A(i,j)
         enddo
      enddo

      do i=1,n
         B(i,i)=1.0_dop
         C(i,i)=1.0_dop
         do j=i+1,n
            B(i,j)=0.0_dop
            B(j,i)=0.0_dop
            C(i,j)=A(i,j)
            C(j,i)=A(j,i)
         enddo
      enddo

!      do i=1,n
!         write(6,'(10F5.2)') (A(i,j),j=1,n)
!         write(6,'(10F5.2)') (B(i,j),j=1,n)
!         write(6,'(10F5.2)') (C(i,j),j=1,n)
!      enddo

      do i=1,n
         j=i
         pivot=A(j,i)
         if (j.gt.i) then
            do k=1,n
               help=A(j,k)
               A(j,k)=A(i,k)
               A(i,k)=help

               help=B(j,k)
               B(j,k)=B(i,k)
               B(i,k)=help
            enddo
         endif
         do j=1,n
            A(i,j)=A(i,j)/pivot
            B(i,j)=B(i,j)/pivot
         enddo
         do j=1,n
            if(j.ne.i) then
               help=A(j,i)
               do k=1,n
                  A(j,k)=A(j,k) - A(i,k)*help
                  B(j,k)=B(j,k) - B(i,k)*help
               enddo
            endif
         enddo
      enddo

      do i=1,n
         do j=1,n
            A(i,j)=restore(i,j)
         enddo
      enddo

      return
      end subroutine invert

!##############################################################
! Genetic Algorithm fitting
!    A global optimiser for the VCHFIT program
!
!    This optimiser is very different from those above as it
!    produces random guesses then 'breeds' these guess together
!    removing the least fit with every subsequent generation.
!
!    VCHAM documentation includes more details.  Routine structure
!    consists of this main routine which deals with all file handling
!    and calls all the genetic operators.
!##############################################################

      subroutine geneticalgor(optcoo, mr, mi)

      implicit none

      integer(long)                        :: i,m,j,k,temp,parent,duelist,child,&
                                              d,iter0,nanticonv,num,bfit
      integer(long), dimension(10000)      :: mut,crossover,tourn,surv,sellist
      real(dop)                            :: mm,a,b,armsd,brmsd,sd,tempr,tot1,&
                                              tot2
      real(dop), dimension(10000)          :: sdrmsd,pop_rmsd,tempri
      character(len=19)                    :: filename,solution,bestfit,filename2
      character(len=80)                    :: string

      real(dop), dimension(optcoodim), intent(inout) :: optcoo
      real(dop), dimension(mrdim), intent(inout)     :: mr
      integer(long), dimension(midim), intent(inout) :: mi
      logical(kind=4)                                :: check,lex
      character(len=39)                              :: cdum

      integer(long), save :: tid=0
      integer(long), save :: tid1=0
      integer(long), save :: tid2=0

      cdum=' '
      check = .false.
      ilog = 3
      iout = 13
      lfulllog=.false.
      nanticonv = 0
      pop_rmsd = 0.0
      tempri = 0.0
      bfit = 0
      if (nmut .gt. 1) then
          print*,'Number of mutations greater than population'
          goto 70
      endif
! this will avoid odd numbered populants...
      if( mod(npop,2) .ne. 0 ) npop = npop + 1
      if(( mod(npop,2) .ne. 0) .and. (.not. lspecies)) &
      print*, 'added 1 to odd number of populants'
      npop0 = npop

! Setting local arrays to zero, rest set in vchfdefault
      do i=1,10000
         mut(i)=0
         crossover(i)=0
         tourn(i)=0
         surv(i)=0
         co_weights(i,1) = 0
         co_weights(i,2) = 0
         co_weights(i,3) = 0
      enddo

      if(lseed .eqv. .true. ) call getseeds()
      if(ngenga .eq. 0 .and. nseed .eq. 0) then
         inquire(file='Pop', exist=lex)
         if (lex) call system('rm -r Pop')
         call system('mkdir Pop ; cp '//trim(datfile)//' Pop/.')
      elseif (nseed .ne. 0 .and. ngenga .eq. 0) then

      endif


      write(ilog,*)
      write(ilog,*) '=============================================='
      write(ilog,600) 'Genetic Algorithm optimisation'
      write(ilog,601) 'Total population = ',npop
      write(ilog,604) 'Maximum number of Generations = ',maxgen
      write(ilog,605) 'RNG Seed =',seed
      write(ilog,*)

      call initpop(mr(mrconst),mr(mrkappa),mr(mrlambda),&
           mr(mrgamma),mr(mriota),mr(mrtermk),mr(mrmu),mr(mrhigher),&
           mr(mrexpcoe),mi(miconstmask),mi(mikappamask),mi(milambdamask),&
           mi(migammamask),mi(miiotamask),mi(mitermkmask),mi(mimumask),&
           mi(miexpcoemask),mr(mrcpcoe),mr(mrcp2d),mi,mr)

!       close(iguess)
!       close(iout)

      iter0=iter
      armsd=500000000

      call get_timer(tid, 'Mut')
512   call start_timer(tid)

      write(6,'(a)')'++++++++++++++++++++++++++++++++++++++++++++++++++++++'
      write(6,'(1x,a,1x,i5)') 'Starting Generation: ',ngenga

      if(covar_adapt .or. lstats ) &
           call get_par_stat(mr(mrkappa),mr(mrlambda),mr(mrgamma),mr(mrmu),mr(mriota),&
           mr(mrexpcoe),mr(mrtermk),mi(mikappamask),mi(milambdamask),&
           mi(migammamask),mi(mimumask),mi(miiotamask),mi(miexpcoemask),mi(mitermkmask),mr)

      do i=1,npop0
         sellist(i)=i
      enddo
      print*, 'I THINK BFIT IS = ', bfit, 'rmsd = ',  pop_rmsd(bfit)
      if( covar_adapt .and. ngenga .gt. 1 ) then
       do while (i .le. npop0*nmut)
        mm=drandom(seed)
        tot1 = 0.0
        do j=1, npop0
          tot1 = tot1 + mutcake(j)
          if( tot1 .gt. mm ) then
            check = .false.
            if( bfit .eq. j ) check = .true.
            do k = 1, i
                if( j .eq. mut(k) ) check = .true.
            enddo
            if(.not. check ) then
                 mut(i) = j
            else
              i = i -1
            endif
            EXIT
          endif
        enddo
        i= i + 1
       enddo
      else
        do i=1,npop0
           mm=drandom(seed)
           mm=mm*(npop0+1-i)
           m=nint(mm)
           if (m .eq. 0) then
              m=1
           elseif( m .eq. bfit) then
              m = m + 1
           endif
!--------------------------------------------------------------------------
! m is the lucky mutation number, so add the first mutatant into the mut
! array from the selection list
!--------------------------------------------------------------------------
           mut(i)=sellist(m)
!--------------------------------------------------------------------------
! this loop "gets rid of" (PUSHES TO THE RIGHT OF ARRAY) that (now mutant)
! participant from the selection list
!--------------------------------------------------------------------------
            do j=1,npop0-m
               sellist(m+j-1)=sellist(m+j)
            enddo
         enddo
       endif
!       print*, 'mutlist = ',nmut,  mut(1:nint(nmut*npop0))

       if (lfulllog) then
          call mutlog(mut)
       endif

!--------------------------------------------------------------------------
! HERE THE MUT ARRAY HAS BEEN FILLED WITH ALL POPULATION BUT IN ORDER OF
! THEIR MUTATION RANDOM SELECTION, AND PERFORMS THE MUATION FOR THE FIRST
! NMUT LUCKY PEOPLE
!--------------------------------------------------------------------------
       if(int(nmut*npop0) .lt. 1) print*, 'WARNING: not enough populants to', &
            'harbour mutants in speciate routine, increase speciate (perparam)', &
            'factor'

!--------------------------------------------------------------------------
! here it does the actual mutation for every lucky winner!
!--------------------------------------------------------------------------
       if( ngenga .eq. 0 .and. covar_adapt) check = .true.
       if( ngenga .eq. 0 .and. covar_adapt) covar_adapt = .false.

44     continue

       if( .not. lspecies) then
         do i=1,int(nmut*npop0)
            temp = mut(i)
            call gafilename(ngenga,temp,filename)
            open(iguess,file=trim(adjustl(filename)),form='formatted',status='old')

!--------------------------------------------------------------------------
! HERE IT READS THE DATA FROM THE POPLUATION INTO THE MR ARRAY
!--------------------------------------------------------------------------
            call readgapar(iguess,mr(mrkappa),mr(mrlambda),mr(mrgamma),mr(mriota),mr(mrmu)&
                           ,mr(mrexpcoe),mr(mrtermk) )
            close(iguess)
!----- ---------------------------------------------------------------------
! IT MUTATES THE ARRAY
!----- ---------------------------------------------------------------------
!           print*, 'before mut = ', mr(mrtermk:mrtermk+nadd)
            call mutate(mr(mrkappa),mr(mrlambda),mr(mrgamma),mr(mrmu),&
                 mr(mriota),mr(mrexpcoe),mr(mrtermk),mi(mikappamask),mi(milambdamask),&
                 mi(migammamask),mi(mimumask),mi(miiotamask),mi(miexpcoemask),mi(mitermkmask),i)
            pop_rmsd(temp)=vcrmsd(optcoo,mr,mi)
            pop_rmsd(temp)=sqrt(pop_rmsd(temp)/nsets)
!           print*, 'after mut = ', mr(mrtermk:mrtermk+nadd)
!--------------------------------------------------------------------------
! RE-WRITES THAT MEMBER OF THE POP NOW MUTATED
!--------------------------------------------------------------------------
            open(iout,file=trim(adjustl(filename)),form='formatted',status='unknown')
            call gaout(iout,mr(mrkappa),mr(mrlambda),mr(mrgamma),mr(mriota),&
                       mr(mrmu),mr(mrexpcoe),mr(mrtermk),mi(mikappamask), &
                       mi(milambdamask),mi(migammamask),mi(miiotamask), &
                       mi(mimumask),mi(miexpcoemask),mi(mitermkmask),&
                       .false.,cdum)
            close(iout)
          call file_copy(trim(filename), trim(filename)//'mut')

         enddo
         call stop_timer(tid)
!--------------------------------------------------------------------------
! in the first generation, dont use the covariance matrix to give a
! mutation
!--------------------------------------------------------------------------
         if( check) covar_adapt = .true.
         if( check) check = .false.
       endif


!--------------------------------------------------------------------------
! Crossover
! Selects 2 members of the population at random and mates them
!--------------------------------------------------------------------------
       call get_timer(tid1, 'crosso')
       call start_timer(tid1)

       do i=1,npop0
          sellist(i)=i
       enddo
       do i=1,npop0
          mm=drandom(seed)
          mm=mm*(npop0+1-i)
          m=nint(mm)
          if (m .eq. 0) then
             m=1
          endif
          crossover(i)=sellist(m)
          do j=1,npop0-m
             sellist(m+j-1)=sellist(m+j)
          enddo
       enddo
!--------------------------------------------------------------------------
! HERE IT FORMS AN ARRAY CALLED CROSSOVER WITH THE POPLUATION RANDOMY
! CHOSEN INTO IT, SUCH THAT PARTICIPANT D WILL SHAGG WITH PARTICIPANT D+1
!--------------------------------------------------------------------------

!--------------------------------------------------------------------------
! This loads the first parent into the mr2 arrays, and the second into the
! mr3 arrays.
! The subrouine firstchild generates the first child and puts it's values
! in the the normal mr arrays.
! The subroutine also generates a mask depicting which parent gave which
! value to which child (mi 'co' arrays)
! This is used by the subroutine second child to make the second child.
! The first child is saved before the second is generated, so that only
! 3 sets or parameters are in memory
!--------------------------------------------------------------------------
!--------------------------------------------------------------------------
! IF COMASK IS 1 THEN THE PARAMETER IN MR1 CAME FROM MR2 ELSE (IF ITS 2
! THE PARAMETER IN MR1 CAME FROM MR3
! NOTE THAT POP_RMSD KEEPS THE FITTNESS OF EVERY PARTICIPANT, (USED IN TOURNAMENT)
!-------------------------------------------------------------------------

       do i=1,npop0/2
          d= (i*2)-1
          parent=crossover(d)
          call gafilename(ngenga,parent,filename)
          open(iguess,file=filename,form='formatted',status='old')
          call readgapar(iguess,mr(mrkappa2),mr(mrlambda2),mr(mrgamma2),mr(mriota2),mr(mrmu2)&
                         ,mr(mrexpcoe2),mr(mrtermk2) )
          close(iguess)
          parent=crossover(d+1)
          call gafilename(ngenga,parent,filename)
          open(iguess,file=filename,form='formatted',status='old')
          call readgapar(iguess,mr(mrkappa3),mr(mrlambda3),mr(mrgamma3),mr(mriota3),mr(mrmu3)&
                         ,mr(mrexpcoe3),mr(mrtermk3) )
          close(iguess)

          child = npop0 + (i*2) - 1
          call firstchild(mr(mrkappa),mr(mrlambda),mr(mrgamma),mr(mrmu),&
               mr(mriota),mr(mrexpcoe),mr(mrtermk),mr(mrkappa2),&
               mr(mrlambda2),mr(mrgamma2),mr(mrmu2),mr(mriota2),&
               mr(mrexpcoe2),mr(mrtermk2),&
               mr(mrkappa3),mr(mrlambda3),mr(mrgamma3),mr(mrmu3),&
               mr(mriota3),mr(mrexpcoe3),mr(mrtermk3),mi(mikappacomask),&
               mi(milambdacomask),mi(migammacomask),mi(mimucomask),&
               mi(miiotacomask),mi(miexpcoecomask),mi(mitermkcomask),&
               mi(mikappamask),mi(milambdamask),mi(migammamask),mi(mimumask),&
               mi(miiotamask),mi(miexpcoemask),mi(mitermkmask) )
          pop_rmsd(child)=vcrmsd(optcoo,mr,mi)
          pop_rmsd(child)=sqrt(pop_rmsd(child)/nsets)
!          print*, 'POPFRIST = [', pop_rmsd(child), 'child =', child

          call gafilename(ngenga,child,filename)
          open(iout,file=filename,form='formatted',status='new')
          call gaout(iout,mr(mrkappa),mr(mrlambda),mr(mrgamma),mr(mriota),&
                     mr(mrmu),mr(mrexpcoe),mr(mrtermk),mi(mikappamask),mi(milambdamask),&
                     mi(migammamask),mi(miiotamask),mi(mimumask),mi(miexpcoemask),&
                     mi(mitermkmask),.false.,cdum)
          close(iout)

!--------------------------------------------------------------------------
! SECOND CHILD IS THE COMPLEMENTARY DNA OF FIRST ONE
!--------------------------------------------------------------------------
          child = child+1
          call secondchild(mr(mrkappa),mr(mrlambda),mr(mrgamma),mr(mrmu),&
               mr(mriota),mr(mrexpcoe),mr(mrtermk),mr(mrkappa2),&
               mr(mrlambda2),mr(mrgamma2),mr(mrmu2),mr(mriota2),&
               mr(mrexpcoe2),mr(mrtermk2),&
               mr(mrkappa3),mr(mrlambda3),mr(mrgamma3),mr(mrmu3),&
               mr(mriota3),mr(mrexpcoe3),mr(mrtermk3),mi(mikappacomask),&
               mi(milambdacomask),mi(migammacomask),mi(mimucomask),&
               mi(miiotacomask),mi(miexpcoecomask),mi(mitermkcomask),&
               mi(mikappamask),mi(milambdamask),mi(migammamask),mi(mimumask),&
               mi(miiotamask),mi(miexpcoemask),mi(mitermkmask) )
          pop_rmsd(child)=vcrmsd(optcoo,mr,mi)
          pop_rmsd(child)=sqrt(pop_rmsd(child)/nsets)

          call gafilename(ngenga,child,filename)

          if( lusecons) then
             call par2vec2d(optcoo,mr(mrconst),mr(mrkappa),mr(mrlambda),&
                  mr(mrgamma),mr(mriota),mr(mrmu),mr(mrexpcoe),&
                  mr(mrhigher),mr(mrtermk),mr(mrcpcoe),&
                  mr(mrswitch),mr(mrcp2d),mi(miconstmask),mi(mikappamask),&
                  mi(milambdamask),mi(migammamask),mi(miiotamask),&
                  mi(mimumask),mi(miexpcoemask),mi(mitermkmask),&
                  mi(micpcoemask),mi(miswitchmask),mi(micp2d))

             call vec2par2d(optcoo,mr(mrconst),mr(mrkappa),mr(mrlambda),&
                  mr(mrgamma),mr(mriota),mr(mrmu),mr(mrexpcoe),&
                  mr(mrhigher),mr(mrtermk),mr(mrcpcoe),&
                  mr(mrswitch),mr(mrcp2d),mi(miconstmask),mi(mikappamask),&
                  mi(milambdamask),mi(migammamask),mi(miiotamask),&
                  mi(mimumask),mi(miexpcoemask),mi(mitermkmask),&
                  mi(micpcoemask),mi(miswitchmask),mi(micp2d))
          endif

          open(iout,file=filename,form='formatted',status='new')
          call gaout(iout,mr(mrkappa),mr(mrlambda),mr(mrgamma),mr(mriota),&
                     mr(mrmu),mr(mrexpcoe),mr(mrtermk),mi(mikappamask),mi(milambdamask),&
                     mi(migammamask),mi(miiotamask),mi(mimumask),mi(miexpcoemask),&
                     mi(mitermkmask),.false.,cdum)
          close(iout)
          if (lfulllog) then
             write(ilog,606) 'Parents = ', crossover(d),' ',&
                  crossover(d+1),' Children = ',child-1,' ',child
          endif
          continue
       enddo

!      on the first generation after using speciate routine, we wish to mix all the genes obtained from
!      the speciation (so that there is a big rmsd difference between competitors and all genes are given enought time
!      to be  mixed into the pool.
       if (lspecies .and. ngenga .eq. 0) then
        do i=1,npop0
          call gafilename(ngenga,npop0+i,filename)
          call gafilename(ngenga,i,filename2)
          call rename(filename,filename2)
        enddo
        lspecies = .false.
        goto 44
       endif


       call stop_timer(tid1)

!      this so called 'anti-convergance' just checks to remove any dupliocates. quite a wastefull and
!       slightly useless routine... but dune every 20 generations. Would be good to do something similar
!       but that checks "similar" participants (since rarely two are identical).
!       if (nanticonv .ge. 40) then
!          call anticonv(mr(mrkappa),mr(mrlambda),mr(mrgamma),mr(mrmu),&
!               mr(mriota),mr(mrtermk),mr(mrkappa2),mr(mrlambda2),&
!               mr(mrgamma2),mr(mrmu2),mr(mriota2),mr(mrtermk2),&
!               mi(miconstmask),mi(mikappamask),mi(milambdamask),mi(migammamask),&
!               mi(mimumask),mi(miiotamask),mi(mitermkmask),&
!               mr(mrconst),mr(mrexpcoe),&
!               mi(miexpcoemask),mr(mrhigher),mr(mrekappa),&
!               mr(mregamma),mr(mrelambda),mr(mremu),mr(mre2xpcoe),&
!               mr(mreiota),mr(mrdexpcoe),mr(mrselambda),mr(mrsemu),&
!               mr(mrcpcoe),nanticonv,pop_rmsd)
!          nanticonv=0
!       elseif( (nanticonv .eq. 20) ) then
!          call anticonv(mr(mrkappa),mr(mrlambda),mr(mrgamma),mr(mrmu),&
!               mr(mriota),mr(mrtermk),mr(mrkappa2),mr(mrlambda2),&
!               mr(mrgamma2),mr(mrmu2),mr(mriota2),mr(mrtermk2),&
!               mi(miconstmask),mi(mikappamask),mi(milambdamask),mi(migammamask),&
!               mi(mimumask),mi(miiotamask),mi(mitermkmask),&
!               mr(mrconst),mr(mrexpcoe),&
!               mi(miexpcoemask),mr(mrhigher),mr(mrekappa),&
!               mr(mregamma),mr(mrelambda),mr(mremu),mr(mre2xpcoe),&
!               mr(mreiota),mr(mrdexpcoe),mr(mrselambda),mr(mrsemu),&
!               mr(mrcpcoe),nanticonv,pop_rmsd)
!               nanticonv=nanticonv+1
!       else
!          nanticonv=nanticonv+1
!       endif

       num = 0
!--------------------------------------------------------------------------
! -- Tournament Selection
! Randomly selects members of the population and puts them in
! the queue for the tournament
!--------------------------------------------------------------------------
       call get_timer(tid1, 'tourn')
       call start_timer(tid2)
       do i=1,npop0*2
          sellist(i)=i
       enddo
       do i=1,npop0*2
          mm=drandom(seed)
          mm=mm*((npop0*2)+1-i)
          m=nint(mm)
          if (m .eq. 0) then
             m=1
          endif
          tourn(i)=sellist(m)
          do j=1,npop0*2-m
             sellist(m+j-1)=sellist(m+j)
          enddo
       enddo

!--------------------------------------------------------------------------
! Now that the entrants are selected it compares their RMSD (the penalty
! function) 2 entrants and a time and the winner's number is saved in the
! survivor arrary (surv)
!--------------------------------------------------------------------------
       armsd=0
       brmsd=999999999
       do i = 1, npop0
          j = i*2-1
          duelist=tourn(j)
          if( ngenga .eq. 0 .and. pop_rmsd(duelist) .eq. 0.0 ) then
            call gafilename(ngenga,duelist,filename)
            open(iguess,file=filename,form='formatted',status='old')
            call readgapar(iguess,mr(mrkappa),mr(mrlambda),mr(mrgamma),mr(mriota),mr(mrmu)&
                           ,mr(mrexpcoe),mr(mrtermk) )
            close(iguess)
            pop_rmsd(duelist)=vcrmsd(optcoo,mr,mi)
            pop_rmsd(duelist)=sqrt(pop_rmsd(duelist)/nsets)
          endif
!          if (pop_rmsd(duelist) .eq. 0) pop_rmsd(duelist) =  sqrt(vcrmsd(optcoo,mr,mi)/nsets)
          a=pop_rmsd(duelist)

          close(iguess)

          k = i*2
          duelist=tourn(k)
          if( ngenga .eq. 0 .and. pop_rmsd(duelist) .eq. 0.0 ) then
             call gafilename(ngenga,duelist,filename)
             open(iguess,file=filename,form='formatted',status='old')
             call readgapar(iguess,mr(mrkappa),mr(mrlambda),mr(mrgamma),mr(mriota),mr(mrmu)&
                            ,mr(mrexpcoe),mr(mrtermk))
             close(iguess)
!            if( pop_rmsd(duelist) .eq. 0.0) pop_rmsd(duelist)=vcrmsd(optcoo,mr,mi)
             pop_rmsd(duelist)=vcrmsd(optcoo,mr,mi)
             pop_rmsd(duelist)=sqrt(pop_rmsd(duelist)/nsets)
          endif
!          if (pop_rmsd(duelist) .eq. 0) pop_rmsd(duelist) =  sqrt(vcrmsd(optcoo,mr,mi)/nsets)
          b=pop_rmsd(duelist)

!--------------------------------------------------------------------------
! this will put in co_weights(x,2) the position what ranking that
! participant (in the next generation) has given its fitness, and then turn
! it into a weight for the covariance matrix
!--------------------------------------------------------------------------
          if (a .lt. b) then
             surv(i) = tourn(j)
!             tempri(i) = pop_rmsd(tourn(j))
             if(covar_adapt) co_weights(i,2) = a
             do k = 1,int(nmut*npop0)
               if(surv(i) .eq. mut(k) ) num = num + 1
             enddo
          else if (a .gt. b) then
             surv(i) = tourn(k)
!             tempri(i) = pop_rmsd(tourn(k))
             if(covar_adapt) co_weights(i,2) = b
             do k = 1,int(nmut*npop0)
                if(surv(i) .eq. mut(k) ) num = num + 1
             enddo
          else
             surv(i) = tourn(k)
!             tempri(i) = pop_rmsd(tourn(k))
             if(covar_adapt) co_weights(i,2) = b
          endif

!--------------------------------------------------------------------------
! Decides whether to stop and finds the best fit so far
!--------------------------------------------------------------------------
          if (a .lt. tol) then
             lend = .true.
             temp = tourn(j)
             call gafilename(ngenga,temp,solution)
          else if (b .lt. tol) then
             lend = .true.
             temp = tourn(k)
             call gafilename(ngenga,temp,solution)
          endif
!       B IN BRSMD STANDS FOR "BEST"
          if (a .lt. brmsd) then
             brmsd = a
             duelist=tourn(j)
             call gafilename(ngenga,duelist,filename)
             bestfit=filename
             bfit= duelist
!             print*, 'winner =', a
          endif
          if (b .lt. brmsd) then
             brmsd = b
             duelist=tourn(k)
             call gafilename(ngenga,duelist,filename)
             bestfit=filename
             bfit= duelist
!             print*, 'winner =', b
          endif
          armsd=armsd+a+b
       enddo
!       A IN ARMSD STANDS FOR "AVERAGE"
       armsd=armsd/(npop0 * 2)
       tempri = 0.0
!       WRITES BEST PARTICIPANT AS A VCHAM FILE
       open(iguess,file=bestfit,form='formatted',status='old')
       call readgapar(iguess,mr(mrkappa),mr(mrlambda),mr(mrgamma),mr(mriota),mr(mrmu)&
            ,mr(mrexpcoe),mr(mrtermk))
         if(lusecons) then
            call par2vec2d(optcoo,mr(mrconst),mr(mrkappa),mr(mrlambda),&
                 mr(mrgamma),mr(mriota),mr(mrmu),mr(mrexpcoe),&
                 mr(mrhigher),mr(mrtermk),mr(mrcpcoe),&
                 mr(mrswitch),mr(mrcp2d),mi(miconstmask),mi(mikappamask),&
                 mi(milambdamask),mi(migammamask),mi(miiotamask),&
                 mi(mimumask),mi(miexpcoemask),mi(mitermkmask),&
                 mi(micpcoemask),mi(miswitchmask),mi(micp2d))

            call vec2par2d(optcoo,mr(mrconst),mr(mrkappa),mr(mrlambda),&
                 mr(mrgamma),mr(mriota),mr(mrmu),mr(mrexpcoe),&
                 mr(mrhigher),mr(mrtermk),mr(mrcpcoe),&
                 mr(mrswitch),mr(mrcp2d),mi(miconstmask),mi(mikappamask),&
                 mi(milambdamask),mi(migammamask),mi(miiotamask),&
                 mi(mimumask),mi(miexpcoemask),mi(mitermkmask),&
                 mi(micpcoemask),mi(miswitchmask),mi(micp2d))
         endif

       close(iguess)
       open(iout,file='Best_'//trim(adjustl(charit(ngenga,0)))//'.vcham',form='formatted')
!      open(iout,file='Best_'//trim(adjustl(charit(ngenga,0)))//'.vcham',form='formatted',status='old')
       call outpar(iout,mr(mrconst),mr(mrkappa),mr(mrlambda),&
            mr(mrgamma),mr(mriota),mr(mrmu),mr(mrexpcoe),&
            mr(mrhigher),mr(mrtermk),mr(mrcpcoe),&
            mr(mrswitch),mr(mrcp2d))
       call outmask(mi(miconstmask),mi(mikappamask),mi(milambdamask),&
           mi(migammamask),mi(miiotamask),mi(mimumask),mi(miexpcoemask),&
           mi(mitermkmask),mi(micpcoemask),mi(miswitchmask))
       close(iout)

       print*, 'average rmsd =', armsd
       print*, 'mutants that succeded:', num,&
       '(out of'//trim(charit(int(npop0*nmut),0))//')'

       call stop_timer(tid2)

!--------------------------------------------------------------------------
! Save surviors in next generation
! Reads the files listed in the survivor array and and then saves them in
! the next generation
! note of caution: the Id for every participant changes every generation, so be sure you take this into account generally.
!--------------------------------------------------------------------------
       if(ngenga .eq. 0) then
          inquire(file='Best', exist=lex)
          if (lex) call system('rm -r Best')
          call system('mkdir Best')
       endif
!       if(ngenga .eq. 0) call system('rm Best/*vcham')
       if(ngenga .eq. 0) call system('cp '//trim(datfile)//' Best/.')
       call system('mv Best_'//trim(adjustl(charit(ngenga,0)))//'.vcham'//' Best/.')

       temp = ngenga
       if (lfulllog) then
          call survlog(surv)
       endif
       do i=1,npop0
          m=surv(i)
          tempri(i) = pop_rmsd(m)
          call gafilename(ngenga,m,filename)
          call gafilename((ngenga+1),i,filename2)
!       QUESTION: which way of copying is more efficient: using fortran (line by line copy, see simple routine),
!       or to use the os 'cp ' script. I reckon using fortran, but if anyone knows,  swap the commented lines!.
!          call system('cp '//filename//' '//filename2)
          call file_copy(trim(filename), trim(filename2))
          if( m .eq. bfit ) bfit = i
!          open(iguess,file=filename,form='formatted',status='old')
!          call readgapar(iguess,mr(mrkappa),mr(mrlambda),mr(mrgamma),mr(mriota),mr(mrmu)&
!                         ,mr(mrexpcoe),mr(mrtermk) )
!          close(iguess)
!          call gafilename((ngenga+1),i,filename)
!          open(iout,file=filename,form='formatted',status='new')
!          call gaout(iout,mr(mrkappa),mr(mrlambda),mr(mrgamma),mr(mriota),&
!                     mr(mrmu),mr(mrexpcoe),mr(mrtermk),mi(mikappamask),mi(milambdamask),&
!                     mi(migammamask),mi(miiotamask),mi(mimumask),mi(miexpcoemask),&
!                     mi(mitermkmask),.false.)
!          close(iout)
       enddo
       sd=0
!      THIS PUTS THE RMSD'S OF THE WINNERS IN THEIR NEW INDEX:
       pop_rmsd = tempri

       if(covar_adapt) &
!        CALCULATES THE COVARIANCE MATRIX AND MUTANTS!
         call get_par_covar(mr(mrkappa),mr(mrlambda),mr(mrgamma),mr(mrmu),mr(mriota),&
         mr(mrexpcoe),mr(mrtermk),mi(mikappamask),mi(milambdamask),&
         mi(migammamask),mi(mimumask),mi(miiotamask),mi(miexpcoemask),mi(mitermkmask),mr)

!--------------------------------------------------------------------------
! Information for log file
!--------------------------------------------------------------------------
       call galog(sd,sdrmsd,brmsd,bestfit,armsd)

!--------------------------------------------------------------------------
! To save file space, deleteing all but the last ten generations
!--------------------------------------------------------------------------
       if (maxmem .lt. 0) then
          maxgensaved = 1
       else
          tempr = (maxmem/(50*ntpar))-1
          if (tempr .lt. 1) then
             tempr = 1
          endif
          maxgensaved = nint(tempr)
       endif

       string=' '
       if (ngenga .gt. maxgensaved) then
          do i=1,npop0*2
             call gafilename((ngenga-maxgensaved),i,filename)
             call system('rm '//trim(filename))
          enddo
       endif

!--------------------------------------------------------------------------
! Are the end conditions met?
!--------------------------------------------------------------------------
       if (ngenga .ge. maxgen) then
          print*,'Maximum number of generations met'
          write(ilog,*)
          write(ilog,630) 'Maximum number of generations met'
          write(ilog,*)
          open(iguess,file=bestfit,form='formatted',status='old')
          call readgapar(iguess,mr(mrkappa),mr(mrlambda),mr(mrgamma),mr(mriota),mr(mrmu)&
                         ,mr(mrexpcoe),mr(mrtermk) )
          close(iguess)
       elseif (lend) then
          write(ilog,*)
          write(ilog,631) 'Solution found'
          write(ilog,*)
          open(iguess,file=solution,form='formatted',status='old')
          call readgapar(iguess,mr(mrkappa),mr(mrlambda),mr(mrgamma),mr(mriota),mr(mrmu)&
                         ,mr(mrexpcoe),mr(mrtermk) )
          close(iguess)
       else
          ngenga=ngenga+1
          write(ilog,*)

          write(6,'(1x,a,3x,e16.8)') &
               'Best RMSD from this Generation:', brmsd
          write(6,'(a,/)')&
               '++++++++++++++++++++++++++++++++++++++++++++++++++++++'

          write(ilog,80) 'Starting Generation: ',ngenga
          write(ilog,*)
          goto 512
       endif

       write(ilog,*)
       write(ilog,*) 'Completed Genetic Algorithm Optimisation'
       write(ilog,*)

       do i=1,npop0*2
          call gafilename(ngenga,i,filename)
          write(string(1:30),609) 'rm ',filename//'*'
          call excmd(string)
       enddo

70     continue
       return
       80     format(a21,i4)

600    format(a30)
601    format(a19,i4,a33,i4)
604    format(a32,i4)
605    format(a10,e10.4)
606    format(a10,i4,a1,i4,a12,i4,a1,i4)
609    format(a3,a25)
630    format(a33)
631    format(a15)

     end subroutine geneticalgor


!#######################################################################
      subroutine getseeds()

      implicit none

      integer(long)              :: k,l,a
      logical(kind=4) :: file_exists
      character(len=20) :: string2

            print*, 'Reading User Given Populants...'
            file_exists = .false.
            do k =0, 9999
             l = 9999-k
             string2 = 'Pop/'//charit(l,3)//'_'//charit(nseed,3)//'.vcham'
             INQUIRE(FILE=trim(string2), EXIST=file_exists)
             if( file_exists) then
              do a=1,nseed
               INQUIRE(FILE='Pop/'//charit(l,3)//'_'//charit(a,3)//'.vcham', EXIST=file_exists)
               if(.not. file_exists) EXIT
              enddo
              if(file_exists) then
              call system('mkdir Pop2')
!             move all the wanted files to generation zero
               do a=1,nseed
                CALL RENAME('Pop/'//charit(l,3)//'_'//charit(a,3)//'.vcham ','Pop2/'//charit(0,3)//'_'//charit(a,3)//'.vcham ')
               enddo
               do a=nseed, 9998
                INQUIRE(FILE='Pop/'//charit(l,3)//'_'//charit(a,3)//'.vcham', EXIST=file_exists)
                 if(file_exists) call system('rm -rf Pop/'//charit(l,3)//'_'//charit(a,3)//'.vcham')
               enddo
               call system('rm -rf Pop/  ')
               CALL RENAME('Pop2','Pop')
              endif
             endif
            enddo

      end subroutine getseeds

!#######################################################################
      subroutine initpop(const,kappa,lambda,gamma,&
              iota,termk,mu,higher,expcoe,&
              constmask,kappamask,lambdamask,gammamask,&
              iotamask,termkmask,mumask,expcoemask,&
              cpcoe,cp2d,mi,mr)

      implicit none

      integer(long)                         :: sp_max, isp,md1,md2
      integer(long), dimension(midim), intent(inout)          :: mi
      real(dop), dimension(mrdim), intent(inout)     :: mr
      integer(long), dimension(midim)                      :: mi_sp,mi_sp2
      real(dop), dimension(mrdim)                          :: mr_sp
      real(dop), dimension(nsta,nsta), intent(inout)          :: const

      real(dop), dimension(nhigher), intent(inout)                  :: higher
      real(dop), dimension(nmodes,nsta), intent(out)             :: kappa
      real(dop), dimension(nmodes,nsta,nsta), intent(out)        :: lambda
      real(dop), dimension(nmodes,nmodes,nsta), intent(out)      :: gamma
      real(dop), dimension(maxcoe,nmodes,nsta), intent(out)      :: expcoe
      real(dop), dimension(nadd), intent(inout)                  :: termk
      real(dop), dimension(nmodes,nmodes,nsta,nsta), intent(out) :: iota,mu
      real(dop), dimension(maxcoe,maxadd), intent(inout)            :: cp2d

      real(dop), dimension(maxcoe,nmodes,nsta,nsta), intent(out)      :: cpcoe

      integer(long), dimension(nsta,nsta), intent(inout)               :: constmask
      integer(long), dimension(nmodes,nsta), intent(inout)             :: kappamask
      integer(long), dimension(nmodes,nsta,nsta), intent(inout)        :: lambdamask
      integer(long), dimension(nmodes,nmodes,nsta), intent(inout)      :: gammamask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(inout)      :: expcoemask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(inout) :: iotamask,mumask
      integer(long), dimension(nadd), intent(inout)                    :: termkmask

      integer(long), dimension(nmodes,nsta)             :: kmask,skmask
      integer(long), dimension(nmodes,nsta,nsta)        :: lmask,slmask
      integer(long), dimension(nmodes,nmodes,nsta)      :: gmask,sgmask
      integer(long), dimension(maxcoe,nmodes,nsta)      :: xmask,sxmask
      integer(long), dimension(nmodes,nmodes,nsta,nsta) :: imask,mmask,simask,smmask
      integer(long), dimension(nadd)                    :: tmask,stmask

      integer(long), dimension(nsta)                     :: ibranch
      character(len=40), dimension(nsta,comb_nsta,3) :: species
      integer(long), dimension(nsta, comb_nsta,2)      :: specimen_count
      integer(long)           :: npopgen,c,i,j,p,sp,spn,k,&
                                 tot_spec,ntpar0,ntvar0,nvar0,&
                                 ncons0,extra, counts,ioff,fncons
      character(len=19)       :: filename
      character(len=39)       :: cspec,cdum
      real(dop)               :: mutchance0
      integer(long)           :: ngenga0, npopgen0
      logical(kind=4)         :: file_exists, oddnumber, waslrdguess, check, ltemp
      integer(long),dimension(maxcons)    :: fparcons
      real(dop)    ,dimension(maxcons)    :: fpconfact


      cdum=' '
      fparcons =  parcons
      fpconfact=  pconfact
      fncons   =  ncons
      mi_sp2   = mi

      mr_sp = mr
      mi_sp = mi
      kmask = kappamask
      lmask = lambdamask
      gmask = gammamask
      mmask = mumask
      imask = iotamask
      xmask = expcoemask
      tmask = termkmask
      iout=13
      isp = 14
      counts= 0
      waslrdguess = .false.
      check = .false.
      specimen_count = 0
      write(6,'(/,1x,a,/)')'Generating initial Population'

! TO DO: IMPORTANT: MAKE IT SUCH THAT, ONLY THE ON-DIAGONAL TERMS THAT ARE INVOLVED IN THE COUPLING TERMS OF THE
!       SPEACIES BEING MINIMISED BY SIMPLEX, ARE OPTIMISED BY THE SIMPLEX (AND GIVE A RANDOM VALUE TO THE OTHER ONES).
!       THIS SHOULD MAKE IT EASY FOR THE PROGRAM TO COPE WITH MANY SIMPLEX CALCULATIONS)


!--------------------------------------------------------------------------
! First need to check for seeds
!
! lseed is set if one wants the first nseed individuals to be picked up
! from a previous calculations
!
! note that they will only be picked up if they have the format
! "0000_xxxx.vcham", so you may need to change their name.
!
! if lseed and lspecies is chosen then the previous calculation's initial
! population of SPECIES (once they have been optimised) is read to be either
! further optimised (setting sp_gen > 0) or to use as the begining for the
! merging of species (sp_gen = 0) (dont forget to rename them as
! sp_0000_xxxx.vcham
!
! note that perparam tell us how many individuals there will be, by being
! proportional to the overall number of parameters. A given species in a
! "genus" (defined by their state sequence) will have
! perparam*(#of parameters of that species) specimens in the population
!--------------------------------------------------------------------------
      extra = 0
      npopgen0 = 0
      c = 0
      tot_spec = 0
      oddnumber = .false.
      if(lspecies) then
         do i = 1, nsta
            do j = 1, comb_nsta
               do p = 1,3
                  species(i,j,p) = ""
               enddo
               specimen_count(i,j,1) = 0
               specimen_count(i,j,2) = 0
            enddo
            ibranch(i) = 0
         enddo
         k = 0
         i = 1
         j = 0
         p = 0
         npopgen0 = npop
         npopgen = 0
         counts = nseed
         if(lseed)then
!--------------------------------------------------------------------------
! read as many 0000_xxxxx.vcham as you find
!--------------------------------------------------------------------------
            call gafilename(ngenga,i,filename)
            INQUIRE(FILE=filename, EXIST=file_exists)
            print*,'Adding Specimens...'

!--------------------------------------------------------------------------
! one can choose how many seed specimens as one wants, but one needs
! to set nseed > what one wants
!--------------------------------------------------------------------------
!--------------------------------------------------------------------------
! note that p that comes out is = # coupling parameters that the specimen
! has (equivalent to content in species(x,x,3), what species it belong to)
! if the routine has already found that species in the array, then p = 0
! specimen_count is an array that keeps track of how many of each species
! the user adds
!--------------------------------------------------------------------------
            ibranch = 0
            do while ( (file_exists .eqv. .true.) .or. (i .le. nseed) )
               open(iguess,file=filename,form='formatted',status='old',err=99)
               call add_species(iguess,species,slmask,smmask,stmask,p,&
                  ibranch,specimen_count,i,cspec)
               close(iguess)
!--------------------------------------------------------------------------
! the participants that have been made from the speciate routine have an f
! for "frozen" in the parameters that have been used in the simplex optimization
! here we need to invert the 1's and 2's
!--------------------------------------------------------------------------

!          k counts how many species have been added
               if(p .gt. 0) k = k + 1
!          i counts the number of files provided (number of specimens)
               i = i + 1
               call gafilename(ngenga,i,filename)
               INQUIRE(FILE=filename, EXIST=file_exists)
               if( (.not. file_exists) .and. (i-1 .lt. nseed) ) then
                  nseed = i-1
                  print*, &
                     'Not as many nseeds as user specified, taking instead: ',i-1
                  counts = nseed
               endif
            enddo
            c = npopgen
         endif
         print*, 'Commencing Speciation ...'
         call speciate(species,lmask,mmask,tmask,tot_spec,ibranch)
!--------------------------------------------------------------------------
! this calculates how many more specimens have to be added  besides the
! ones given by the user:
!--------------------------------------------------------------------------
!         npopgen = npopgen - c + 1
         c = 1
         gaoptim = .true.
         ntpar0 = ntpar
         ntvar0 = ntvar
         nvar0 = nvar
         ncons0 = ncons
      else
         if (lseed) then
            npopgen = npop - nseed
         else
            npopgen = npop
            nseed = 0
         endif

      endif
      if( lspecies) then
        p = 1
        spn = 1
        sp = 2
        npopgen = 0
        do while( sp .le. nsta )
           if( trim(species(sp,spn,1)) .ne. "" ) sp_max = sp
!           print*, 'its sp=', sp, 'spn = ',spn,'species=', trim(species(sp,spn,1)),'max = ', sp_max
           do while( trim(species(sp,spn,1)) .ne. "" )
              npopgen = npopgen + int(numit(species(sp,spn,3))*perparam) - specimen_count(sp,spn,1)
              spn = spn + 1
           enddo
           sp = sp + 1
           spn = 1
        enddo
        sp = 2
        if( npopgen .gt. npop0) npop0 = npopgen
        if(.not.lseed .and. lspecies ) c = 1
        if( lrdguess ) lrdguess = .false.
        if( lrdguess ) waslrdguess = .true.
      else
        c = 1
        npopgen0 = npop0
      endif
      if( nseed .gt. npop0) then
        npopgen0 = nseed
        npop0 = nseed
      endif

      do while (counts .lt. npopgen0)
!--------------------------------------------------------------------------
!     if the population turns out to be odd, add one more, which will just
!     be a dumb random newguess (see above)
!--------------------------------------------------------------------------
          if( mod(npopgen+nseed,2) .ne. 0 .and. c .gt. npopgen0 ) then
             npopgen0 = npopgen0 + 1
          endif

!       IF A GUESS GIVEN, THEN THE INITIAL POP WILL BE A BUNCH OF MUTANTS OF THAT GUESS
         if( lrdguess .eqv. .true.) then
            open(iguess,file=guessfile,form='formatted',status='old',err=99 )
            call readguess(iguess,const,kappa,lambda,gamma,iota,mu,expcoe,&
                 expcoemask,higher,termk,cpcoe,mr(mrswitch),cp2d)
            close(iguess)
            ltemp = covar_adapt
            covar_adapt = .false.
            mutchance0 = mutchance
            mutchance = mutchance*0.5
            ngenga0 = ngenga
            ngenga = int(maxgen/1.1)
            call mutate(kappa,lambda,gamma,mu,iota,expcoe,termk,kappamask,lambdamask,&
                 gammamask,mumask,iotamask,expcoemask,termkmask,0)
            ngenga = ngenga0
            mutchance = mutchance0
            covar_adapt = ltemp
         elseif(lspecies) then
            ioptim = 3
!--------------------------------------------------------------------------
! "x" means that the species was given by the user
! for every species, (#off-diag partmeters of that specimen)*perparam -1
! will be created with their DNA strip (+ ondiag parameters) simplex optimised
! and the rest will be assigned random values.
!--------------------------------------------------------------------------
!         print*, 'READING SPECIES = ', species(sp,spn,1) ,  species(sp,spn,2)
!         print*, 'amount = ',int(numit(species(sp,spn,3)))*perparam, ' = ', specimen_count(sp,spn,1)
            if( ( (index(species(sp,spn,1),'x') .eq. 0) .and. (species(sp,spn,1) .ne. "") ) &
               .or. (( index(species(sp,spn,1),'x') .ne. 0) .and. &
                 (specimen_count(sp,spn,1) .lt. int(numit(species(sp,spn,3))*perparam) ) )) then
!--------------------------------------------------------------------------
! if user inputted the wrong number specimens for a species,
! add as many extra specimens to make up to the right
! number of them according to int(numit(species(sp,spn,3))*perparam
!--------------------------------------------------------------------------
               ltemp = .false.
               if( index(species(sp,spn,1),'x') .ne. 0 .and. &
           specimen_count(sp,spn,1) .lt. int(numit(species(sp,spn,3))*perparam) ) then
                ltemp = .true.
                p = 1
               endif
!--------------------------------------------------------------------------
! NOTE: array specimen_count(sp,spn,2) contains the file index where the species
! in species(sp,spn,) was given by the user
!--------------------------------------------------------------------------
                cspec = trim(species(sp,spn,1))//' '//trim(species(sp,spn,2))

               if(p .eq. 1) then
!       if user gave wrong number of specimens of species:
!                  print*, 'spc = ', specimen_count(sp,spn,1),'perp=', int(numit(species(sp,spn,3))*perparam), &
!                   'species = ', species(sp,spn,1)
                  if(  index(species(sp,spn,1),'x') .ne. 0 .and. &
                  specimen_count(sp,spn,1) .lt. int(numit(species(sp,spn,3))*perparam) ) then
                    p = specimen_count(sp,spn,1)
                    specimen_count(sp,spn,1) =  specimen_count(sp,spn,1) +1
!                    print*, 'User given specimen, count = ', specimen_count(sp,spn,1)
                  endif

                  ioff = 0
                  if(  lseed .and. index(species(sp,spn,1),'x') .ne. 0  ) &
                  print*, 're-Populating species; DNA states: ',trim(species(sp,spn,1)),&
                       ' modes: ',trim(species(sp,spn,2)), ' (', c ,' out of ', npopgen,')'
                  if( .not. lseed .or. ( lseed .and.  index(species(sp,spn,1),'x') .eq. 0 ) ) &
                  print*, 'Populating species; DNA states: ',trim(species(sp,spn,1)),&
                       ' modes: ',trim(species(sp,spn,2)), ' (',c -1  + &
                       int(numit(species(sp,spn,3))*perparam)-specimen_count(sp,spn,1)&
                       ,' out of ', npopgen,')'

                  call set_species_masks(species(sp,spn,1:3),kappamask,lambdamask,gammamask,&
                       mumask,iotamask,expcoemask,termkmask,kmask,lmask,gmask,&
                       mmask,imask,xmask,tmask,ioff)

!--------------------------------------------------------------------------
! IOFF counts how many parameters have been switched off
!--------------------------------------------------------------------------
!   SAVE THE MASKS OF BEFORE FREEZING THE NON-INVOLVED MODES:
                  skmask = kmask
                  slmask = lmask
                  sgmask = gmask
                  sxmask = xmask
                  smmask = mmask
                  simask = imask
                  stmask = tmask
                  call set_othermodes_masks_off(species(sp,spn,1:3),kmask,lmask,gmask,&
                       imask,mmask,xmask,tmask,ioff)
!       if the modes that were switched off where actually used by some constrined 'active' modes, then
!       recover them:
                  md1 = numit(trim(species(sp,spn,2)(1:2))//"   ")
                  md2 = numit(trim(species(sp,spn,2)(3:4))//"   ")
                  if(lusecons) call remask_constraints(iin,kmask,lmask,gmask,imask,mmask,xmask,tmask,md1,md2)

                  if(  index(species(sp,spn,1),'x') .ne. 0  )   then
!                 get the simplexed values from a specimen given by user (to avoid doing simplex again.
                     call gafilename(ngenga,specimen_count(sp,spn,2),filename)
                     print*, 'got data from file:', filename
                     open(iguess,file=filename,form='formatted',status='old',err=99 )
                     call readgapar(iguess,kappa,lambda,gamma,iota,mu,expcoe,termk)
                     close(iguess)
                  else
                   parcons = 0
                   pconfact=0d0
                   ncons=0
                   modus=1
                   optcoo = 0d0
                   ntpar = 0

                   if (lusecons) then
                      call parstats(mi(miconstmask),kmask,lmask,gmask,imask,&
                           mmask,xmask,tmask,mi(micpcoemask),&
                           mi(miswitchmask),mi(micp2d),modus)

                      call setcons(mi(miconstmask),kmask,lmask,&
                           gmask,imask,mmask,xmask,tmask,&
                           mi(micpcoemask),mi(miswitchmask),mi(micp2d),kappa,lambda,expcoe)
                      modus=2
                   endif
!--------------------------------------------------------------------------
! Calculate how many parameters
!--------------------------------------------------------------------------
                  call parstats(mi(miconstmask),kmask,lmask,gmask,imask,&
                       mmask,xmask,tmask,mi(micpcoemask),&
                       mi(miswitchmask),mi(micp2d),modus)

!--------------------------------------------------------------------------
!  renumber constraint labels to reflect parameters chosen for fit
!--------------------------------------------------------------------------
                  if (ncons .gt. 0) call renumcons(mi(miconstmask),kmask,&
                       lmask,gmask,imask,mmask,xmask,tmask,mi(micpcoemask),&
                       mi(miswitchmask))

                   print*, 'Number of parameters to be optimised :',nvar
                   call zerovxi(mi_sp,midim)
                   call set_mi_array(mi_sp(mikappamask),mi_sp(milambdamask),&
                        mi_sp(migammamask),mi_sp(miiotamask),mi_sp(mimumask),&
                        mi_sp(miexpcoemask),mi_sp(mitermkmask),&
                        kmask,lmask,gmask,imask,mmask,xmask,tmask)

                   mr = mr_sp
                   call initpar(optcoo,mr,mi_sp)
                   call fitpar(optcoo,mr,mi_sp)
!      THIS IS BECAUSE WE WANT TO MAKE A BUNCH OF PARTICIPANTS WITH THE
!       SIMPLEXED PARAMETERS FROZEN (NOT RANDOMLY SET), BUT THE REST RANDOMLY SET.
!       (SO WE SAVE SMASK WILL HAVE ONLY THE NON-SIMPLEXED PARAMETERS AS MASK = 1 )
                 endif
!       TEST taht prints out the simplexed parameters:
!                call gafilename(ngenga,npop0+c+1,filename)
!                 open(isp,file=trim(adjustl(filename))//'_test',form='formatted',status='new')
!                call outpar(isp,mr(mrconst),kappa,lambda,&
!                  gamma,iota,mu,expcoe,mr(mrhigher),termk,mr(mrcpcoe),mr(mrswitch),mr(mrcp2d))
!                 close(isp)

                 call makenegativemask(kmask,skmask,lmask,slmask,gmask,sgmask,xmask,sxmask,&
                       imask,simask,mmask,smmask,tmask,stmask)
!---------------------------------------------------------------------------------------
!      NOW RE-SET CONSTRAINTS FOR ALL THE PARAMETERS THAT WERE NOT SIMPLEXED BUT RANDOMLY ASSIGNED.
!      im not sure if this is necessary, as the gaout routine only print the variables that are not
!      masked with -ve values.
!---------------------------------------------------------------------------------------

                 parcons = 0
                 pconfact=0d0
                 ncons=0
                 modus=1
                 optcoo = 0d0
                 ntpar = 0

                 if (lusecons) then
                    call parstats(mi(miconstmask),kmask,lmask,gmask,imask,&
                         mmask,xmask,tmask,mi(micpcoemask),&
                         mi(miswitchmask),mi(micp2d),modus)
                    call setcons(mi(miconstmask),kmask,lmask,&
                         gmask,imask,mmask,xmask,tmask,&
                         mi(micpcoemask),mi(miswitchmask),mi(micp2d),kappa,lambda,expcoe)
                    modus=2
                 endif
!------------------------------------------------------------------------
! Calculate how my parameters
!------------------------------------------------------------------------
                 call parstats(mi(miconstmask),kmask,lmask,gmask,imask,&
                      mmask,xmask,tmask,mi(micpcoemask),&
                      mi(miswitchmask),mi(micp2d),modus)

!------------------------------------------------------------------------
!  renumber constint labels to reflect parameters chosen for fit
!------------------------------------------------------------------------
                 if (ncons .gt. 0) call renumcons(mi(miconstmask),kmask,&
                      lmask,gmask,imask,mmask,xmask,tmask,mi(micpcoemask),&
                       mi(miswitchmask))

               endif

               call newguess(kappa,lambda,gamma,iota,mu,expcoe,             &
                    termk,kmask,lmask,gmask,imask,mmask,xmask,tmask)

               if( p .eq. int(numit(species(sp,spn,3))*perparam))then
                   spn = spn + 1
                   p =0
                   do while(species(sp,spn,1) .eq. "" .and. sp .lt. nsta)
                      spn = 1
                      sp = sp + 1
                   enddo
               endif
               p = p + 1
!--------------------------------------------------------------------------
            else
! if there is an "x" and the user gave the right number of them,
! thensimply skip:
!--------------------------------------------------------------------------
               spn = spn + 1
               do while(species(sp,spn,1) .eq. "" .and. sp .lt. nsta)
                  sp = sp + 1
                  spn = 1
               enddo
!               spn = 1
               p = 0
               if( sp .le. nsta  .and. sp .le. sp_max) cycle
! this only happens when the user adds species..its a bit of a bodgy
               if ( sp .gt. nsta .or. sp .gt. sp_max ) &
                 call newguess(kappa,lambda,gamma,iota,mu,expcoe,&
                      termk,kmask,lmask,gmask,imask,mmask,xmask,tmask)
            endif
         else

!--------------------------------------------------------------------------
! if the user wants more number of poulants than there are species
! and there were odd number of populants (so that in the routine
! lspecies is set to false)
!--------------------------------------------------------------------------

!            if( npopgen0 .ne. 0 .and. c .eq. npopgen .and. .not. oddnumber) lspecies = .true.
            call newguess(kappa,lambda,gamma,iota,mu,expcoe,&
                 termk,kmask,lmask,gmask,imask,mmask,xmask,tmask)
         endif


         counts = counts + 1
         if(lforcepos .and. .not. lspecies) &
         call force_pos(kappa,gamma,expcoe,termk,kappamask,gammamask,expcoemask,termkmask)

!--------------------------------------------------------------------------
! calling this routine sets the constraints between two parameters:
!--------------------------------------------------------------------------
         if(lusecons .and. .not. lspecies) then
            call par2vec2d(optcoo,mr(mrconst),kappa,lambda,&
                 gamma,iota,mu,expcoe,mr(mrhigher),termk,mr(mrcpcoe),&
                 mr(mrswitch),mr(mrcp2d),mi(miconstmask),kappamask,&
                 lambdamask,gammamask,iotamask,mumask,expcoemask,termkmask,&
                 mi(micpcoemask),mi(miswitchmask),mi(micp2d))

            call vec2par2d(optcoo,mr(mrconst),kappa,lambda,&
                 gamma,iota,mu,expcoe,mr(mrhigher),termk,mr(mrcpcoe),&
                 mr(mrswitch),mr(mrcp2d),mi(miconstmask),kappamask,&
                 lambdamask,gammamask,iotamask,mumask,expcoemask,termkmask,&
                 mi(micpcoemask),mi(miswitchmask),mi(micp2d))
         endif
         npop0 = counts
         call gafilename(ngenga,npop0,filename)

!       creates an species backup file for reruns:
         if(lspecies .and. .not. ltemp ) then
          open(isp,file=trim(adjustl(filename))//'_sp',form='formatted',status='new')
          call gaout(isp,kappa,lambda,gamma,iota,mu,expcoe,termk,&
                         kmask,lmask,gmask,imask,mmask,xmask,tmask,.true.,cspec)
          close(isp)
         endif
!         if(lspecies .and. .not. ltemp ) then
!          open(isp,file=trim(adjustl(filename))//'_test',form='formatted',status='new')
!         call outpar(isp,mr(mrconst),kappa,lambda,&
!           gamma,iota,mu,expcoe,mr(mrhigher),termk,mr(mrcpcoe),mr(mrswitch),mr(mrcp2d))
!          close(isp)
!         endif

         open(iout,file=trim(adjustl(filename)),form='formatted',status='new')
         if(lspecies ) then
          call gaout(iout,kappa,lambda,gamma,iota,mu,expcoe,termk,&
                         kmask,lmask,gmask,imask,mmask,xmask,tmask,.true.,cspec)
         else
          call gaout(iout,kappa,lambda,gamma,iota,mu,expcoe,termk,&
                         kmask,lmask,gmask,imask,mmask,xmask,tmask,.false.,cdum)
         endif
         close(iout)

         if( lspecies .and. c .eq. npopgen .and. counts .le. npopgen0 ) then
          if ( waslrdguess) then
           lrdguess = .true.
           if( .not. check ) print*, 'Adding ', int(npopgen-counts), 'mutated guesses. '
          else
           if( .not. check ) print*, 'Adding ', int(npopgen0-(npopgen+nseed)), 'random guesses. '
!         sets parameters back to normal:
            kmask = kappamask
            lmask = lambdamask
            gmask = gammamask
            mmask = mumask
            imask = iotamask
            xmask = expcoemask
            tmask = termkmask
            parcons =  fparcons
            pconfact=  fpconfact
            ncons   =  fncons
            ntpar = ntpar0
            ntvar = ntvar0
            nvar = nvar0
            ncons = ncons0
          endif
          check = .true.
          lspecies = .false.
         endif


         c = c + 1
      enddo
      if(check) lspecies = .true.
! this sets the parstats back to normal,
      close(iout)
      if (lspecies) then
         gaoptim2 = .false.
         gaoptim  = .false.
         modus=1
         parcons =  fparcons
         pconfact=  fpconfact
         ncons   =  fncons
         mi   = mi_sp2
         ntpar = ntpar0
         ntvar = ntvar0
         nvar = nvar0
         ncons = ncons0
         mr = mr_sp
      endif
      return
99    continue
      print*, 'could not open guess file (INITPOP routine)'
      return

!       for debugging:
!        i = 0
!         do j =1, nsta-1
!          print*, '#of states (branch_level): ',j
!          print*, '#of species : ',ibranch(j)
!          do k = 1, ibranch(j)
!          print*, 'spn = ',k
!            print*, 'species_states: ',species(j,k,1)
!            print*, 'species_modes : ',species(j,k,2)
!            print*, 'species_optpar: ',species(j,k,3)
!       i = i + 1
!          enddo
!         enddo
!        print*, 'total number of species : ',i
!        debbuging 1D array at mrgamma:
!        l = 0
!        do s=1,nsta
!          do timp=1,nmodes
!           do j=1, nmodes
!             print*, trim(charit(l+1,0)),' m1:',trim(charit(j,0)),' m2:',trim(charit(timp,0)),&
!             ' s1:',trim(charit(s,0)), mi(migammamask + l)
!             l = l + 1
!           enddo
!          enddo
!        enddo
!       debugging array expcoe:
!         l = 0
!          do s = 1, nsta
!            do i=1, nmodes
!             do j = 1, maxcoe
!               print*, 'x = ',j,'m = ',i,'s=',s
!               print*, 'mi_sp = ', mi_sp(miexpcoemask + l)
!               print*, 'mi =    ', mi(miexpcoemask + l)
!               print*, 'xmask = ', xmask(j,i,s)
!               l = l + 1
!             enddo
!            enddo
!         enddo

    end subroutine initpop

!##############################################################


      subroutine remask_constraints(unit,kmask,lmask,gmask,imask,mmask,xmask,tmask,md1,md2)

      implicit none

      integer(long) :: m,m1,s,k,iz,ic,ierr
      integer(long) :: ilbl,jlbl,slen,schk,s2chk,p,i,j,ieq,cont,cant
      integer(long), dimension(nmodes) :: defchk,chk
      integer(long), intent(in)                                        :: unit,md1,md2
      integer(long), dimension(nmodes,nsta), intent(inout)             :: kmask
      integer(long), dimension(nmodes,nsta,nsta), intent(inout)        :: lmask
      integer(long), dimension(nmodes,nmodes,nsta), intent(inout)      :: gmask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(inout)      :: xmask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(inout) :: imask,mmask
      integer(long), dimension(nadd), intent(inout) :: tmask

      rewind(unit)

!-----------------------------------------------------------------------
! read down to constraint list
! could be in iin or iguess
!-----------------------------------------------------------------------
   10 continue
      call rdinpf(unit,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      if (unit .eq. iin) then
         routine = 'readcons (read VCHFIT input)'
      else if (unit .eq. iguess) then
         routine = 'readcons (read GUESS)'
      endif
      call vccntrl(ierr)

      if (keyword(1).ne.'constraints-section') go to 10

!-----------------------------------------------------------------------
! read constraints
! find which parameter is being constrained
!-----------------------------------------------------------------------
 20   continue
      call rdinpf(unit,keyword,keyorig,inptit,lc,ic,iz,ierr,maxkey)
      routine = 'readcons (read VCHFIT input)'
      call vccntrl(ierr)

      if (keyword(1).eq.'end-constraints-section') then
         go to 999

      else if (keyword(1) .eq. 'kappa') then
         read(keyword(2),*) m
         read(keyword(3),*) s
         if( (m .eq. md1 .or. m .eq. md2 ) .and. &
          ( index(trim(keyword(5)),'freeze') .eq. 0) )then
           call getcons(5,kmask,lmask,gmask,imask,mmask,xmask,tmask,ic)
         endif

      else if (keyword(1) .eq. 'gamma') then
         read(keyword(2),*) m
         read(keyword(3),*) m1
         read(keyword(4),*) s
         if(((md1 .eq. md2 .and. m .eq. m1 .and. m .eq. md1 ) .or. &
         (md1 .ne. md2 .and. (( m .eq. m1 .and. (m .eq. md1 .or. m .eq. md2)) .or. &
         ( m .ne. m1 .and. (( m .eq. md1 .and. m1 .eq. md2) .or. ( m .eq. md2 .and. m1 .eq. md1)))))) &
         .and. ( index(trim(keyword(6)),'freeze') .eq. 0) )then
           call getcons(6,kmask,lmask,gmask,imask,mmask,xmask,tmask,ic)
         endif

      else if (keyword(1) .eq. 'iota') then
         read(keyword(2),*) m
         read(keyword(3),*) m1
         read(keyword(4),*) s
         if(((md1 .eq. md2 .and. m .eq. m1 .and. m .eq. md1 ) .or. &
         (md1 .ne. md2 .and. (( m .eq. m1 .and. (m .eq. md1 .or. m .eq. md2)) .or. &
         ( m .ne. m1 .and. (( m .eq. md1 .and. m1 .eq. md2) .or. ( m .eq. md2 .and. m1 .eq. md1)))))) &
         .and. ( index(trim(keyword(6)),'freeze') .eq. 0) )then
           call getcons(6,kmask,lmask,gmask,imask,mmask,xmask,tmask,ic)
         endif

      else if (keyword(1) .eq. 'general') then
         read(keyword(2),*) k
         read(keyword(3),*) m
         read(keyword(4),*) s
         if( (m .eq. md1 .or. m .eq. md2 ) .and. &
         ( index(trim(keyword(6)),'freeze') .eq. 0) )then
!          print*, 'modes = ', md1,md2,'constraied gen=',k,m,s, 'mask=', xmask(k,m,s)
           call getcons(6,kmask,lmask,gmask,imask,mmask,xmask,tmask,ic)
         endif
      else if ((keyword(1).eq.'explicit').or.&
           (keyword(1).eq. 'Explicit')) then
         do i=1,ic
            if (keyword(i).eq.'=') ieq=i
         enddo

         do i=1,nmodes
            defchk(i)=0.0
         enddo
         read(keyword(2),*,iostat=ierr) schk
         read(keyword(3),*,iostat=ierr) s2chk
        cont = 0
        cant = 0
         do i=4,ieq-1
            ilbl=index(keyword(i),'^')
            jlbl=slen(keyword(i))
            if (ilbl .eq. 0) then
               p = 1
               read(keyword(i),*,iostat=ierr) m
            else
               read(keyword(i)(1:ilbl-1),*,iostat=ierr) m
               read(keyword(i)(ilbl+1:jlbl),*,iostat=ierr) p
            endif
            defchk(m) = p
            cont = cont + 1
            if( m .eq. md1 .or. m .eq. md2 ) cant = cant + 1
         enddo
!       this has not been properly tested:
         if( ( (md1 .eq. md2 .and. cont .eq. 1 .and. cant .eq. 1 ) .or.  &
             ( md1 .ne. md2 .and. cant .eq. 2 .and. cont .eq. 2 ))&
          .and. s2chk .eq. schk ) then
           do i=1,nadd
              do j=1,nmodes
                 chk(j)=1
              enddo
              if (schk.eq.termst(1,i).and.s2chk.eq.termst(2,i).or.    &
                   (schk.eq.termst(2,i)).and.s2chk.eq.termst(1,i)) then
                 do j=1,nmodes
                    chk(j)=0
                 enddo
                 do j=1,nmodes
                    chk(j)=defchk(j)-termdef(j,i)
                    if (chk(j).ne.0) go to 300
                 enddo

                 if ( ( index(trim(keyword(ieq+1)),'freeze') .eq. 0) )then
                    call getcons(ieq+1,kmask,lmask,gmask,imask,mmask,xmask,tmask,ic)
                 endif
              endif

    300       continue
           enddo
         endif

     endif
     go to 20

  999 continue
     return
     end subroutine remask_constraints

!#######################################################################

      subroutine getcons(ik,kmask,lmask,gmask,imask,mmask,xmask,tmask,ic)

      implicit none

      integer(long)                                                 :: m,m1,s,s1,k,&
                                                   i,l,schk,s2chk,ilbl,jlbl,slen,p,j
      integer(long), intent(in)                                     :: ik,ic
      integer(long), dimension(nmodes,nsta), intent(inout)             :: kmask
      integer(long), dimension(nmodes,nsta,nsta), intent(inout)        :: lmask
      integer(long), dimension(nmodes,nmodes,nsta), intent(inout)      :: gmask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(inout)      :: xmask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(inout) :: imask,mmask
      integer(long), dimension(nadd), intent(inout)                    :: tmask
      integer(long), dimension(nmodes) :: defchk,chk

! get parameter for constraint.
      m=0
      m1=0
      s=0
      s1=0
      k=0
      if (keyword(ik) .eq. 'kappa') then
         if (ic .lt. ik+2) go to 990
         read(keyword(ik+1),*,err=999) m
         read(keyword(ik+2),*,err=999) s
         kmask(m,s) = 1
      else if (keyword(ik) .eq. 'lambda') then
         if (ic .lt. ik+3) go to 990
         read(keyword(ik+1),*,err=999) m
         read(keyword(ik+2),*,err=999) s
         read(keyword(ik+3),*,err=999) s1
         lmask(m,s,s1) = 1
         lmask(m,s1,s) = 1
      else if (keyword(ik) .eq. 'gamma') then
         if (ic .lt. ik+3) go to 990
         read(keyword(ik+1),*,err=999) m
         read(keyword(ik+2),*,err=999) m1
         read(keyword(ik+3),*,err=999) s
         gmask(m,m1,s) = 1
         gmask(m1,m,s) = 1
      else if (keyword(ik) .eq. 'iota') then
         if (ic .lt. ik+3) go to 990
         read(keyword(ik+1),*,err=999) m
         read(keyword(ik+2),*,err=999) m1
         read(keyword(ik+3),*,err=999) s
         imask(m,m1,s,s) = 1
         imask(m1,m,s,s) = 1
      else if (keyword(ik) .eq. 'mu') then
         if (ic .lt. ik+4) go to 990
         read(keyword(ik+1),*,err=999) m
         read(keyword(ik+2),*,err=999) m1
         read(keyword(ik+3),*,err=999) s
         read(keyword(ik+4),*,err=999) s1
         mmask(m,m1,s,s1) = 1
         mmask(m1,m,s,s1) = 1
         mmask(m,m1,s1,s) = 1
         mmask(m,m1,s1,s) = 1
      else if (keyword(ik) .eq. 'general') then
         if (ic .lt. ik+3) go to 990
         read(keyword(ik+1),*,err=999) k
         read(keyword(ik+2),*,err=999) m
         read(keyword(ik+3),*,err=999) s
         xmask(k,m,s) = 1
!         print*,' into gen = ', k,m,s,'mask = ', xmask(k,m,s)
      else if (keyword(ik) .eq. 'iota2') then
         if (ic .lt. ik+3) go to 990
         read(keyword(ik+1),*,err=999) m
         read(keyword(ik+2),*,err=999) m1
         read(keyword(ik+3),*,err=999) s
         read(keyword(ik+4),*,err=999) s1
         imask(m,m1,s,s1) = 1
         imask(m1,m,s,s1) = 1
         imask(m,m1,s1,s) = 1
         imask(m,m1,s1,s) = 1
      else if (keyword(ik) .eq. 'explicit') then
! determine termk number
         do i=1,nmodes
            defchk(i)=0.0
         enddo
         read(keyword(ik+1),*,err=999) schk
         read(keyword(ik+2),*,err=999) s2chk
         m = 0
         do l=ik+3,ic-1
            ilbl=index(keyword(l),'^')
            jlbl=slen(keyword(l))
            if (ilbl .eq. 0) then
              p = 1
              read(keyword(l),*,err=999) m
            else
               read(keyword(l)(1:ilbl-1),*,err=999) m
               read(keyword(l)(ilbl+1:jlbl),*,err=999) p
            endif
            defchk(m) = p
         enddo
         do i=1,nadd
            do j=1,nmodes
               chk(j)=1
            enddo
            if (schk.eq.termst(1,i).and.s2chk.eq.termst(2,i).or.    &
                 (schk.eq.termst(2,i)).and.s2chk.eq.termst(1,i)) then
               do j=1,nmodes
                  chk(j)=0
               enddo
               do j=1,nmodes
                  chk(j)=defchk(j)-termdef(j,i)
                  if (chk(j).ne.0) go to 300
               enddo
               tmask(i) = 1
            endif
300         continue
         enddo
      else
         routine = 'getcons'
         write(message,'(2a)') 'Unknown constraint type: ',keyword(ik)
         call vcerrormsg
      endif

      return
  990 continue
      routine='getcons'
      write(message,'(a,a10,a10,a)') &
           'Not enough arguments for constraint: ',keyword(ik),&
           keyword(ik+1),'....'
      call vcerrormsg
  999 continue
      routine='getcons'
      write(message,'(a,a10,a10,a)') 'Error reading constraints: ',&
           keyword(ik),keyword(ik+1),'....'
      call vcerrormsg
      end subroutine getcons

!#######################################################################



     subroutine set_mi_array(mkmask,mlmask,mgmask,mimask,mmmask,mxmask,mtmask,&
                               kmask,lmask,gmask,imask,mmask,xmask,tmask)

      implicit none
      integer(long) ::i,j,s,s1
      integer(long), dimension(nmodes,nsta), intent(in)             :: kmask
      integer(long), dimension(nmodes,nsta,nsta), intent(in)        :: lmask
      integer(long), dimension(nmodes,nmodes,nsta), intent(in)      :: gmask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(in)      :: xmask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: imask,mmask

      integer(long), dimension(nadd), intent(out)                    :: tmask
      integer(long), dimension(nmodes,nsta), intent(out)             :: mkmask
      integer(long), dimension(nmodes,nsta,nsta), intent(out)        :: mlmask
      integer(long), dimension(nmodes,nmodes,nsta), intent(out)      :: mgmask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(out)      :: mxmask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(out) :: mimask,mmmask
      integer(long), dimension(nadd), intent(out)                    :: mtmask


!--------------------------------------------------------------------------
! for some strange reason the above methods where causing problems...so i
! had to do it the lame way:
!--------------------------------------------------------------------------

      do s=1,nsta
         do i=1,nmodes
            mkmask(i,s) = kmask(i,s)
         enddo
      enddo
      do s=1,nsta-1
         do s1=s+1,nsta
            do i=1,nmodes
               mlmask(i,s,s1) = lmask(i,s,s1)
               mlmask(i,s1,s) = lmask(i,s,s1)
               if((s .eq. 1 .and. s1 .eq. 5) .or. (s .eq. 5 .and. s1 .eq. 1)) then
               endif
            enddo
         enddo
      enddo
      do s=1,nsta
         do i=1,nmodes
            do j=i,nmodes
               mgmask(i,j,s) = gmask(i,j,s)
               mgmask(j,i,s) = gmask(i,j,s)
            enddo
         enddo
      enddo

      do s=1,nsta-1
         do s1=s+1,nsta
            do i=1,nmodes
               do j=i,nmodes
                  mmmask(i,j,s,s1) = mmask(i,j,s,s1)
                  mmmask(j,i,s,s1) = mmask(i,j,s,s1)
                  mmmask(j,i,s1,s) = mmask(i,j,s,s1)
                  mmmask(i,j,s1,s) = mmask(i,j,s,s1)
               enddo
            enddo
         enddo
      enddo

      do s=1,nsta
         do s1=s,nsta
            do i=1,nmodes
              do j=1,nmodes
               mimask(i,j,s,s1) = imask(i,j,s,s1)
               mimask(i,j,s1,s) = imask(i,j,s,s1)
               mimask(j,i,s,s1) = imask(i,j,s,s1)
               mimask(j,i,s1,s) = imask(i,j,s,s1)
              enddo
            enddo
         enddo
      enddo

      do i=1, maxcoe
         do s=1,nsta
            do j=1,nmodes
               mxmask(i,j,s) = xmask(i,j,s)
            enddo
         enddo
      enddo

      do i=1, nadd
         mtmask(i) = tmask(i)
      enddo

    end subroutine set_mi_array

!##############################################################

    subroutine add_species(io,species,lambdamask,mumask,termkmask,&
         tnum,ibranch,specimen_count,fidx,cspec)


      integer(long)     :: i,num,z
      character(len=40),dimension(nsta,comb_nsta,3),intent(inout)   :: species
      integer(long), dimension(nsta), intent(inout)                 :: ibranch
      integer(long),                              intent(out)       :: tnum
      integer(long), dimension(nmodes,nsta,nsta), intent(in)        :: lambdamask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: mumask
      integer(long), dimension(nadd), intent(in)                    :: termkmask
      integer(long), dimension(nsta,comb_nsta,2), intent(inout)       :: specimen_count
      character(len=40)                                             :: states,modes, string
      character(len=39)                                             :: cspec
      logical(kind=4)                                               :: found
      integer(long), intent(in)                                     :: fidx,io

      found = .false.
      num = 0
      tnum = 0
200   read(io,'(A)',end=999) string
      string = trim(adjustl(string))
      if( index(string,'SPECIES') .ne. 0 ) then
           cspec = trim(adjustl(string(10:len(trim(adjustl(string))))))
           found = .true.
      endif
!      print*, trim(string),' and ',  string(len(trim(string)):len(trim(string)))
      if(found) then
        if( index(string,'f') .ne. 0 ) then
         if(string(1:1) .eq. 'l' )then
           num = num + 1
         elseif(string(1:1) .eq. 'm' )then
           num = num + 1
         elseif(string(1:1) .eq. 't' ) then
           num = num + 1
         endif
        endif
      endif
     goto 200
999  continue

!-----------------------------------------------------------------------
! if this species has already been accounted for, then dont add it to
! species array.
!-----------------------------------------------------------------------
      if (found) then
       states = cspec(1:len(trim(adjustl(cspec)))-4)
       modes = cspec(len(trim(adjustl(cspec)))-4:len(trim(adjustl(cspec))))
       states = trim(adjustl(states))
       modes = trim(adjustl(modes))
       string(1:len(states)) = trim(states)
       tnum = len(trim(adjustl(string)))
!       do i = 1, len(trim(string))
!          if( string(i:i) .ne. " " ) tnum = tnum + 1
!       enddo

!      print*, 'states = ', states, 'modes = ', modes, 'num = ', num, 'tnum = ', tnum
!      print*, 'tot_c = ', tot_c, 'ibranch(tnum) = ', ibranch(tnum)
!      print*, 'Species available: ', species(tnum,1:ibranch(tnum),1:2)
      if(tnum .gt. 0) then
       if( ibranch(tnum) .eq. 0 ) ibranch(tnum) = 1
          if( .not. &
               species_exists(states,modes,species(tnum,1:ibranch(tnum),1:2),&
               ibranch(tnum),z))then
! if it has a full set of coupling parameters (= mi array) then its not a
! species
             i = ibranch(tnum)
             ibranch(tnum) = ibranch(tnum) + 1
             specimen_count(tnum,i,1) = specimen_count(tnum,i,1) + 1
! this second dimension carries information about what file number this
! species has (specimen_count(,,2)
             specimen_count(tnum,i,2) = fidx
             species(tnum,i,1) = 'x'//trim(adjustl(states))
             species(tnum,i,2) = trim(adjustl(modes))
             species(tnum,i,3) = charit(num,0)
          else
             specimen_count(tnum,z,1) = specimen_count(tnum,z,1) + 1
          endif
       endif
      endif
      return

    end subroutine add_species

!#####################################################################################
! SPECIATE ROUTINE:
! All possible combinations  of states that actually couple will be a
! "species". but only coupling states of one or two ('bi-linear bi-quadra...') modes
! will be species. So species can optimise diag-mode (2d) terms between 2
! modes, maximum (so that bilinear terms get optimised properly too!).
! species(,,) = array that stores all the species, each species is identified
! by their species "genus" (the number of states opim and in this routine pointed by
! the branch_level), an index within that genus (in this routine pointed by ibranch(branch_level)
! and 3 elements that contain 3 pieces of information about that species:
! 1) a string of numbers that represent the states of that species (which
! species it belongs to)
! 2) a string of 2 numbers defining which modes the species is optimising
!    the first two characters are mode1 and the last two are mode2 (4 in
!    total)
! 3) a string giving the total number of inter-state couplings for that
! species.
! so for species species(3,12,1) = "258" (all the on-diagonal elements always and) only the
! off-diagonal elements that couple states 2 5 and 8 will be optimised (so its the 12th parameter
! with 3 coupling states (12th mewmber of the "genus"))
!
!#####################################################################################

    subroutine speciate(species,lmask,mmask,tmask,tot_species,add_ibranch)

      integer(long)     :: i,j,k,m,n,num,ifound
      character(len=40) :: cmodes, comb
      logical(kind=4)   :: endcomb,pass
      integer(long),intent(inout)                                   :: tot_species
      integer(long), dimension(nsta)                                :: ibranch
      integer(long), dimension(nsta), intent(inout)                 :: add_ibranch
      character(len=40),dimension(nsta,comb_nsta,3),intent(out)     :: species
      integer(long), dimension(nmodes,nsta,nsta), intent(in)        :: lmask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: mmask
      integer(long), dimension(nadd), intent(in)                    :: tmask
      integer(long),dimension(nsta)                                 :: biny

!-----------------------------------------------------------------------
! add_ibranch tell us how many species where read for each genus in
! the routine add_species
!-----------------------------------------------------------------------
      do i = 1, nsta
         ibranch(i) = add_ibranch(i)
         tot_species = tot_species + ibranch(i)
      enddo
      num = 0
      do n=1, nmodes
         if(mfit(n) .eq. 0 ) cycle
         do m=n, nmodes
             if(mfit(m) .eq. 0 ) cycle
             comb = ''
             endcomb = .false.
             cmodes = adjustl(trim(adjustl(charit(n,1)))//trim(adjustl(charit(m,1))))
             biny = 0
!       find next possible unique combination of coupling states
             call nextcomb(comb,biny,endcomb)
             do while (.not. endcomb)
!       if this combination is actually realised by our system then pass = true
                  call findcomb(lmask,mmask,tmask,comb,pass,n,m)
                  if( len(trim(adjustl(comb))) .gt. branch_cutoff ) pass = .false.
                  if (pass) then
!       if the user has already given this species as seed then dont add to list
                    if( .not. species_exists(comb,cmodes,&
                    species(len(trim(comb)),1:ibranch(len(trim(comb))),1:2),&
                    ibranch(len(trim(comb))),ifound)) then
                      ibranch(len(trim(comb))) = ibranch(len(trim(comb))) +1
                      species(len(trim(comb)),ibranch(len(trim(comb))),1) = comb
                      species(len(trim(comb)),ibranch(len(trim(comb))),2) = cmodes
                      species(len(trim(comb)),ibranch(len(trim(comb))),3) = &
                      charit(countcoupling(comb,lmask,mmask,tmask,n,m),0)
                      tot_species = tot_species + 1
                    endif
                  endif
                  call nextcomb(comb,biny,endcomb)
            enddo
         enddo
      enddo
      do i = 1, nsta
         add_ibranch(i) = ibranch(i)
      enddo
!       for debugging:
!        print*, 'out', branch_cutoff
        write(ilog,*) '#### SPECIATE ROUTINE: species list. ####'
        write(ilog,*) 'branch_cutoff:', branch_cutoff
        i = 0
         do j =1, nsta
          write(ilog,*) '#of states: ',j
          write(ilog,*) '#of species : ',ibranch(j)
!          print*, '#of states: ',j
!          print*, '#of species : ',ibranch(j)
          do k = 1, ibranch(j)
            write(ilog,*) 'species_states: ',species(j,k,1)
!            print*, 'species_states: ',species(j,k,1)
            write(ilog,*) 'species_modes : ',species(j,k,2)
!            print*, 'species_modes : ',species(j,k,2)
            write(ilog,*) 'species_optpar: ',species(j,k,3)
!            print*, 'species_optpar: ',species(j,k,3)
            i = i + 1
          enddo
         enddo
!        print*, 'Total number of species : ',i
        write(ilog,*) 'Total number of species : ',i

      return
      end subroutine speciate

!#######################################################################

    subroutine findcomb(lmask,mmask,tmask,comb,pass,md1,md2)
      integer(long)                                                 :: m,i,j,k,s1,s2,h,c
      integer(long), intent(in)                                     :: md1,md2
      integer(long),dimension(nmodes,nmodes,nsta,nsta), intent(in)  :: mmask
      integer(long),dimension(nmodes,nsta,nsta), intent(in)         :: lmask
      integer(long), dimension(nadd), intent(in)                    :: tmask
      character(len=40)                                             :: netw
      character(len=40),intent(in)                                  :: comb
      integer(long), dimension(nsta,nsta)                           :: icombs
      integer(long), dimension(nsta)                                :: icomb,history,ns
      logical(kind=4)                                               :: couple,ends
      logical(kind=4),intent(out)                                   :: pass

      netw = ''
      icombs = 0
      ends = .false.
      if( md1 .ne. md2) then
       ends = .false.
       do s1= 1, nsta
         do s2= 1, nsta
           if(s2 .eq. s1) cycle
             couple = .false.
             do i=1, nadd
              if( tmask(i) .eq. 1) then
               k = 0
               do j = 1, nmodes
                 if( termdef(j,i) .ne. 0 ) k = k + 1
               enddo
               if( (termdef(md1,i) .ne. 0 .and. termdef(md2,i) .ne. 0 ) .and.&
                 ( (termst(1,i) .eq. s1 .and. termst(2,i) .eq. s2 ) .or.   &
                 (termst(1,i) .eq. s2 .and. termst(2,i) .eq. s1 ) ) .and. &
                 ( ( md1 .ne. md2 .and. k .eq. 2 )  )  )  couple = .true.
               endif
             enddo
             if(mmask(md1,md2,s1,s2) .eq. 1 .or. mmask(md1,md2,s2,s1) .eq. 1 .or.  &
                mmask(md2,md1,s1,s2) .eq. 1 .or. mmask(md2,md1,s2,s1) .eq. 1 ) couple = .true.
             if( couple ) then
                   icombs(s1,s2) = 1
                   icombs(s2,s1) = 1
                  ends = .true.
!                  print*, 'STATE = ', s1,s2
             endif
         enddo
       enddo
      endif
      if(ends .or. md1 .eq. md2) then
       do s1= 1, nsta
         do s2= 1, nsta
          if(s2 .eq. s1) cycle
           couple = .false.
           if( (lmask(md1,s1,s2) .eq. 1 .or. lmask(md2,s2,s1) .eq. 1 )  )  couple = .true.
           if(mmask(md1,md1,s1,s2) .eq. 1 .or. mmask(md1,md1,s2,s1) .eq. 1 .or.  &
             mmask(md2,md2,s1,s2) .eq. 1 .or. mmask(md2,md2,s2,s1) .eq. 1 ) couple = .true.
           do i=1, nadd
            if( tmask(i) .eq. 1) then
               k = 0
               do j = 1, nmodes
                 if( termdef(j,i) .ne. 0 ) k = k + 1
               enddo
               if( (termdef(md1,i) .ne. 0 .and. termdef(md2,i) .ne. 0 ) .and.&
                 ( (termst(1,i) .eq. s1 .and. (termst(2,i) .eq. s2) ) .or.   &
                 (termst(1,i) .eq. s2 .and. (termst(2,i) .eq. s1) ) ) .and. &
                 ( md1 .eq. md2 .and. k .eq. 1 )  )  couple = .true.
            endif
           enddo
           if (couple ) then
                icombs(s1,s2) = 1
                icombs(s2,s1) = 1
   !             print*, 'STATE = ', s1,s2
               ends = .true.
           endif

         enddo
       enddo
      endif

     do i = 1, len(trim(comb))
       read(comb(i:i), '(I1)') j
       ns(i) = j
     enddo
     icomb = 0
     i = 1
     k = 1
     h = 1
     c = 1
     j = 2
     history = 0
     history(1) = 1
     icomb(c) = ns(i)
!      this routine seearches throught the coupling links to see if this combination
!      does form a connected network (conected nodes in graph) by doing a tree search
       do
          ends = .false.
          do m=1,c
            if( ns(j) .eq. icomb(m) ) ends = .true.
          enddo
          if( j .gt. len(trim(comb)) ) then
            if(h .eq. 1) exit
            h = h - 1
            i = history(h)
            history(h) = 0
            j = i + 1
          elseif( icombs(ns(i),ns(j)) .eq. 0 .or. ends ) then
            j = j + 1
          elseif( icombs(ns(i),ns(j)) .eq. 1  ) then
            history(h) = i
            c = c + 1
            icomb(c) = ns(j)
            h = h + 1
            i = j
            j = 1
          endif
       enddo
       pass = .true.
       do i = 1,c
        netw=   trim(netw)//trim(charit(icomb(i),0))
       enddo
!       print*, 'network =', netw, 'comb = ', comb, 'MODES = ',md1, md2
       do i=1, len(trim(comb))
           if( index(netw,comb(i:i)) .eq. 0 ) then
               pass = .false.
               exit
           endif
       enddo
!       if (pass) print*, 'PASSED'
      return
    end subroutine findcomb

!#######################################################################

    subroutine nextcomb(comb,biny,endcomb)
      integer(long)                                                 :: i,j,k
      character(len=40),intent(out)                                 :: comb
      integer(long),dimension(nsta), intent(inout)                  :: biny
      logical(kind=4),intent(inout)                                 :: endcomb
!       every state represents a binary number
       i = 1
       do
        biny(i) = biny(i) + 1
        k = 0
        do j = 1, nsta
          if(biny(j) .ne. 0 ) k = k + 1
        enddo
        if( biny(nsta) .eq. 2 .or. k .eq. nsta) exit
        if( biny(i) .eq. 2) then
           biny(i) = 0
           i = i + 1
        elseif( k .gt. 1) then
           exit
        else
           i = 1
        endif
       enddo

       comb = ''
       do i = 1, nsta
        if( biny(i) .ne. 0 ) comb = trim(charit(i,0))//trim(comb)
       enddo
       if ( biny(nsta) .eq. 2 .or. k .eq. nsta ) endcomb = .true.

      return
    end subroutine nextcomb

!#######################################################################

    logical(kind=4) function comb_pass(comb,combs)
    integer(long)                                                :: i,j
    character(len=40),intent(in)                                 :: comb
    character(len=40),dimension(nsta),intent(in)                 :: combs

!    if comb in any of the combs then pass
     comb_pass = .true.
     j = 1
     do while (trim(combs(j)) .ne. '' .and. j .le. nsta )
        if(.not. comb_pass) comb_pass = .true.
        do i=1, len(trim(comb))
            if( index(combs(j),comb(i:i)) .eq. 0 ) then
                comb_pass = .false.
                exit
            endif
        enddo
        if( comb_pass) exit
        j = j + 1
     enddo
     if( trim(combs(1)) .eq. '' ) comb_pass = .false.
     return
    end function comb_pass

!#######################################################################

    integer(long) function countcoupling(comb,lmask,mmask,tmask,md1,md2)
      integer(long)                                                 :: l,n,i,j,k
      integer(long), intent(in)                                     :: md1,md2
      integer(long),dimension(nmodes,nmodes,nsta,nsta), intent(in)  :: mmask
      integer(long),dimension(nmodes,nsta,nsta), intent(in)         :: lmask
      integer(long), dimension(nadd), intent(in)                    :: tmask
      character(len=40),intent(in)                                  :: comb

      countcoupling = 0
      do i = 1, nsta-1
        do j = i+1, nsta
          if( index(comb,trim(charit(i,0))) .ne. 0 .and. &
              index(comb,trim(charit(j,0))) .ne. 0   ) then
             if(( mmask(md1,md2,i,j) .eq. 1 .or. mmask(md1,md2,j,i) .eq. 1 &
            .or. mmask(md2,md1,i,j) .eq. 1 .or. mmask(md2,md1,j,i) .eq. 1 )  )&
                countcoupling = countcoupling + 1
             if(md1 .ne. md2 ) then
               if(mmask(md1,md1,i,j) .eq. 1 .or. (mmask(md1,md1,j,i) .eq. 1 ) ) &
                  countcoupling = countcoupling + 1
               if(mmask(md2,md2,i,j) .eq. 1 .or. (mmask(md2,md2,j,i) .eq. 1 ) ) &
                  countcoupling = countcoupling + 1
             endif
             if( lmask(md1,i,j) .eq. 1 .or. lmask(md1,j,i) .eq. 1 ) &
                countcoupling = countcoupling + 1
             if( (lmask(md2,i,j) .eq. 1 .or. lmask(md2,j,i) .eq. 1) .and. md1 .ne. md2 ) &
                countcoupling = countcoupling + 1
             do k=1, nadd
              if( tmask(k) .eq. 1 ) then
               n = 0
               do l = 1, nmodes
                 if( termdef(l,k) .ne. 0 ) n = n + 1
               enddo
!                if( j .eq. 4 .and. i .eq. 3 .and. md1 .eq. 2 .and. md2 .eq. 6 ) 'count = ', countcoupling
               if( (termdef(md1,k) .ne. 0 .and. termdef(md2,k) .ne. 0 ) .and. &
                 ( (termst(1,k) .eq. i .and. (termst(2,k) .eq. j) ) .or.      &
                 (termst(1,k) .eq. j .and. (termst(2,k) .eq. i) ) ) .and.     &
                 (( n .eq. 2 .and. md1 .ne. md2 ) .or. ( n .eq. 1 .and. md1 .eq. md2 ) ) ) &
                countcoupling = countcoupling + 1
!                if( j .eq. 4 .and. i .eq. 3 .and. md1 .eq. 2 .and. md2 .eq. 6 ) 'count = ', countcoupling
              endif
             enddo
          endif
        enddo
      enddo


      return
    end function countcoupling

!#######################################################################

      character(len=10) function rem_char(string,chara)
      character(len=10), intent(in)  :: string
      character(len=8), intent(in)   :: chara
      integer(long)                  :: idx
      integer(long)                  :: leng

11    if( index(string,trim(chara)) .ne. 0) then
        idx = index(string,trim(chara))-1
        leng = len(trim(chara)) + 1
        rem_char = trim(string(1:idx))//&
                trim(string((idx+leng):10))
      else
        rem_char = string
      endif
      if (index(string,trim(chara)) .ne. 0) GoTo 11
      return
      end function rem_char

 !#######################################################################

      character(len=40) function DNAstrip(branch_level,history)

      integer(long)     :: z
      integer(long), intent(in) :: branch_level
      integer(long), dimension(nsta), intent(in) ::history
      character(len=40)   :: string
        z = branch_level
        DNAstrip = ''
        do while (z .gt. 0)
        string = charit(history(z),0)
        if(history(z) .lt. 10 ) then
           DNAstrip = trim(trim(DNAstrip)//trim(string))
        else
           DNAstrip = trim(trim(DNAstrip)//' '//trim(string))
        endif
          z = z - 1
        enddo

      end function

!#######################################################################

      logical(kind=4) function species_exists(cpstates,cpmodes,splist,num,ifound)

      character(len=40), intent(in)                          :: cpstates,cpmodes
      integer(long),intent(in)                               :: num
      character(len=40), intent(in),dimension(num,1:2)       :: splist
      integer(long)                                          :: i, j, k
      logical(kind=4)                                        :: match
      integer(long), intent(out)                             :: ifound


      i = 1
      k = 1
      match = .false.
      species_exists = .true.
      ifound = 0
!      print*, 'num = ', num
      do j=1,num
        i = 1
        do while (i .lt. len(trim(cpstates)) )
          if( trim(cpstates(i:i)) .eq. " ") i = i + 1
          if( trim(cpstates(i+1:i+1)) .ne. " ") then
                  k = i + 1
          else
                  k = i
          endif
!          print*, 'STRINGS= ', splist(j,1), 'and', splist(j,2), 'num =', num
!          print*, 'cptates= ', cpstates, 'modes=', cpmodes
          if(index( trim(splist(j,1)), cpstates(i:k) )  .ne. 0) match = .true.
          if ( match .eqv. .true. ) then
            match = .false.
!            print*, 'MATCH'
          else
            species_exists = .false.
!            print*, 'NO MATCH'
          endif
          i = k + 1
        enddo
        if(species_exists .eqv. .false.) then
          species_exists = .true.
        else
         if( splist(j,2)(1:2) .eq. cpmodes(1:2) .and. splist(j,2)(3:4) .eq. cpmodes(3:4) )then
!           print*, 'MODES in list = ', splist(j,2)(1:2),  splist(j,2)(3:4), ' got ', cpmodes
           species_exists = .true.
           if( index(splist(j,1),'x') .ne. 0 ) &
           ifound = j
           return
         else
          species_exists = .true.
!          print*, 'MODES DONT MATCH'
         endif
        endif
      enddo
      species_exists = .false.
      return
      end function species_exists

!#######################################################################

      integer(long) function numit(untrimmed)
      character(len=5), intent(in) :: untrimmed
      character(len=3)             :: string

      string = trim(untrimmed)
      if( len(trim(string)) .lt. 2) then
                      read(string,'(I1)') numit
      elseif( len(trim(string)) .lt. 3) then
              if(trim(string(1:1)) .eq. "0") then
                     read(string(2:2),'(I1)') numit
              else
                     read(string,'(I2)') numit
              endif
      elseif( len(trim(string)) .lt. 4) then
              if(trim(string(1:2)) .eq. "00") then
                    read(string(3:3),'(I1)') numit
              elseif (trim(string(1:1)) .eq. "0") then
                    read(string(2:3),'(I2)') numit
              else
                    read(string,'(I3)') numit
              endif
      endif
      return
      end function

!#######################################################################

      character(len=4) function charit(num,zeros)
      integer, intent(in) :: num, zeros
      if( zeros .eq. 0) then
        if( num .lt. 10) write(charit,'(I1)')  num
        if( num .lt. 100) write(charit,'(I2)') num
        if( num .lt. 1000) write(charit,'(I3)') num
        if( num .ge. 1000) write(charit,'(I4)') num
      elseif( zeros .eq. 1) then
        if( num .lt. 10) write(charit,'(A1,I1)') '0', num
        if((num .ge. 10) .and. (num .lt. 100)) write(charit,'(I2)') num
        if((num .ge. 100) .and. (num .lt. 1000)) write(charit,'(I3)') num
        if( num .ge. 1000) write(charit,'(I4)') num
      elseif( zeros .eq. 2) then
        if( num .lt. 10) write(charit,'(A2,I1)') '00', num
        if((num .ge. 10) .and. (num .lt. 100)) write(charit,'(A1,I2)') '0',num
        if((num .ge. 100) .and. (num .lt. 1000)) write(charit,'(I3)') num
        if( num .ge. 1000) write(charit,'(I4)') num
      elseif( zeros .eq. 3) then
        if( num .lt. 10) write(charit,'(A3,I1)') '000', num
        if((num .ge. 10) .and. (num .lt. 100)) write(charit,'(A2,I2)') '00',num
        if((num .ge. 100) .and. (num .lt. 1000)) write(charit,'(A1,I3)') '0',num
        if( num .ge. 1000) write(charit,'(I4)') num
      endif
      charit = trim(adjustl(charit))
      return
      end function charit

!#######################################################################

      integer(long) function comb(N,m)
      integer(long), intent(in) :: N, m
      comb = 0
      comb = comb + factorial(N) / (factorial(m) * factorial(N-m))
      return
      end function

!#######################################################################

      integer(long) function sumofcomb(N)
      integer :: o, N
      do o =1, N
       sumofcomb = sumofcomb + factorial(N) / (factorial(o) * factorial(N - o))
      enddo
      return
      end function

!#######################################################################

      integer(long) function factorial(x)
      integer(long), intent(in) :: x
      integer(long) :: i
      factorial = 1
      do i =1, x
       factorial = factorial*i
      enddo
      return
      end function

!#######################################################################

      subroutine set_species_masks(species,kmask1,lmask1,gmask1,mmask1,imask1,xmask1,tmask1,&
                                           kmask2,lmask2,gmask2,mmask2,imask2,xmask2,tmask2,ioff)

      implicit none
      integer(long) :: i,j,k,l,n,m,c
      integer(long), intent(inout)                                     :: ioff
      character(len=40),dimension(3),intent(in)                      :: species
      integer(long), dimension(nmodes,nsta), intent(in)              :: kmask1
      integer(long), dimension(nmodes,nsta), intent(out)             :: kmask2
      integer(long), dimension(nmodes,nsta,nsta), intent(in)         :: lmask1
      integer(long), dimension(nmodes,nsta,nsta), intent(out)        :: lmask2
      integer(long), dimension(nmodes,nmodes,nsta), intent(in)       :: gmask1
      integer(long), dimension(nmodes,nmodes,nsta), intent(out)      :: gmask2
      integer(long), dimension(maxcoe,nmodes,nsta), intent(in)       :: xmask1
      integer(long), dimension(maxcoe,nmodes,nsta), intent(out)      :: xmask2
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(in)  :: imask1,mmask1
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(out) :: imask2,mmask2
      integer(long), dimension(nadd), intent(in)                     :: tmask1
      integer(long), dimension(nadd), intent(out)                    :: tmask2
      integer(long),dimension(2)                                     :: modes
      ioff = 0
      kmask2 = 0
      lmask2 = 0
      gmask2 = 0
      mmask2 = 0
      imask2 = 0
      xmask2 = 0
      tmask2 = 0
!     Bodge: off-diag coup parameters that are constrained dont count, but on-diag parameters that are constrained
!     are then de-constrained and fitted (see main initpop routine for a bit more).


      do i=1,nmodes
       do j=1,nsta
         if(lkappa .and. (kmask1(i,j) .eq. 1 .or. kmask1(i,j) .lt. 0 ))&
            kmask2(i,j) = 1
         if (ldifit) then
          do k=1,maxcoe
            if( (xmask1(k,i,j) .eq. 1 .or.xmask1(k,i,j) .lt. 0) )&
               xmask2(k,i,j) = 1
          enddo
         endif
         do k=1, nsta
           if( k .ne. j .and. llambda .and. (lmask1(i,j,k).eq. 1))&
              lmask2(i,j,k) = 1
         enddo
         do k=1, nmodes
           if(lgamma .and. ( gmask1(i,k,j) .eq.1 .or.gmask1(i,k,j) .lt.0 ) )&
              gmask2(i,k,j) = 1
           do l=1, nsta
             if( liota .and. (imask1(i,k,j,l) .eq.1 .or. imask1(i,k,j,l) .lt.0 ) ) &
                imask2(i,k,j,l) = 1
             if( l .ne. j .and. lmu .and. (mmask1(i,k,j,l) .eq. 1) ) &
                mmask2(i,k,j,l) = 1
           enddo
         enddo
       enddo
      enddo
      do i=1,nadd
         if( termst(1,i) .ne. termst(2,i) .and. tmask1(i) .eq.1 ) then
                tmask2(i) = 1
         elseif(termst(1,i) .eq. termst(2,i) .and.&
               ( tmask1(i) .eq.1 .or.  tmask2(i) .lt.0 ))then
                tmask2(i) = 1
         endif
      enddo
      modes(1) = numit(trim(species(2)(1:2))//"   ")
      modes(2) = numit(trim(species(2)(3:4))//"   ")
      if( modes(1) .eq. modes(2) ) c = 1
      if( modes(1) .ne. modes(2) ) c = 2
      do m = 1, c
       do n = m, c
        do k=1,nsta-1
         do l=k+1,nsta
            if( (index (species(1),trim(adjustl(charit(k,0)))) .eq. 0) .or. &
              (index (species(1),trim(adjustl(charit(l,0)))) .eq. 0)  ) then
               call set_offdiag_off(lmask2,mmask2,tmask2,modes(m),modes(n),k,l,ioff)
             endif
         enddo
        enddo
       enddo
      enddo

      end subroutine

!######################################################################################################

      subroutine makenegativemask(kmask1,skmask,lmask1,slmask,gmask1,sgmask,xmask1,sxmask,&
                             imask1,simask,mmask1,smmask,tmask1,stmask)

      implicit none
      integer(long) :: i,j,k,l
      integer(long), dimension(nmodes,nsta), intent(inout)              :: kmask1
      integer(long), dimension(nmodes,nsta), intent(in)                 :: skmask
      integer(long), dimension(nmodes,nsta,nsta), intent(inout)         :: lmask1
      integer(long), dimension(nmodes,nsta,nsta), intent(in)            :: slmask
      integer(long), dimension(nmodes,nmodes,nsta), intent(inout)       :: gmask1
      integer(long), dimension(nmodes,nmodes,nsta), intent(in)          :: sgmask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(inout)       :: xmask1
      integer(long), dimension(maxcoe,nmodes,nsta), intent(in)          :: sxmask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(inout)  :: imask1,mmask1
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(in)     :: simask,smmask
      integer(long), dimension(nadd), intent(inout)                     :: tmask1
      integer(long), dimension(nadd), intent(in)                        :: stmask

      integer(long), dimension(nmodes,nsta)                             :: tkmask
      integer(long), dimension(nmodes,nsta,nsta)                        :: tlmask
      integer(long), dimension(nmodes,nmodes,nsta)                      :: tgmask
      integer(long), dimension(maxcoe,nmodes,nsta)                      :: txmask
      integer(long), dimension(nmodes,nmodes,nsta,nsta)                 :: timask, tmmask
      integer(long), dimension(nadd)                                    :: ttmask


      tkmask = skmask
      tlmask = slmask
      tgmask = sgmask
      txmask = sxmask
      timask = simask
      ttmask = stmask
      tmmask = smmask

      do i=1,nmodes
       do j=1,nsta
         if( kmask1(i,j) .eq. 1) tkmask(i,j) = 5
         do k=1,maxcoe
           if( xmask1(k,i,j) .eq. 1) txmask(k,i,j) = 5
         enddo
         do k=1, nsta
           if( lmask1(i,j,k) .eq. 1) tlmask(i,j,k) = 5
         enddo
         do k=1, nmodes
           if( gmask1(i,k,j) .eq. 1) tgmask(i,k,j) = 5
           do l=1, nsta
             if( imask1(i,k,j,l) .eq. 1 .or.  imask1(i,k,l,j) .eq. 1 .or. &
                 imask1(k,i,j,l) .eq. 1 .or.  imask1(k,i,l,j) .eq. 1 ) then
                 timask(i,k,j,l) = 5
                 timask(k,i,j,l) = 5
                 timask(i,k,l,j) = 5
                 timask(k,i,l,j) = 5
             endif
             if( mmask1(i,k,j,l) .eq. 1 .or.  mmask1(i,k,l,j) .eq. 1 .or. &
                 mmask1(k,i,j,l) .eq. 1 .or.  mmask1(k,i,l,j) .eq. 1 ) then
                 tmmask(i,k,j,l) = 5
                 tmmask(i,k,l,j) = 5
                 tmmask(k,i,j,l) = 5
                 tmmask(k,i,l,j) = 5
             endif
           enddo
         enddo
       enddo
      enddo
      do i=1,nadd
        if(tmask1(i) .eq. 1)  ttmask(i) = 5
      enddo
      kmask1 = tkmask
      xmask1 = txmask
      lmask1 = tlmask
      gmask1 = tgmask
      imask1 = timask
      mmask1 = tmmask
      tmask1 = ttmask

      end subroutine

!#######################################################################
      subroutine set_othermodes_masks_off(species,kmask1,lmask1,gmask1,&
                       imask1,mmask1,xmask1,tmask1,ioff)

      implicit none
      integer(long)                                             :: i,j,k,s2,n,m,md1,md2
      integer(long), dimension(nmodes,nsta,nsta), intent(inout)         :: lmask1
      integer(long), dimension(nmodes,nmodes,nsta), intent(inout)       :: gmask1
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(inout)  :: imask1,mmask1
      integer(long), dimension(nadd), intent(inout)                     :: tmask1
      integer(long), dimension(maxcoe,nmodes,nsta), intent(inout)       :: xmask1
      integer(long), dimension(nmodes,nsta), intent(inout)              :: kmask1
      character(len=40),dimension(3),intent(in)                         :: species
      integer(long), intent(inout)                                      :: ioff
      logical(kind=4)                                                   :: check


      md1 = numit(trim(species(2)(1:2))//"   ")
      md2 = numit(trim(species(2)(3:4))//"   ")



      do n = 1, nmodes
        do m = n, nmodes
         check = .false.

         if( ( md2 .eq. md1 ) .and. ( ( m .ne. md2 ) .or. ( n .ne. md1 ) ) ) check = .true.
         if( ( md2 .ne. md1 ) .and. (m .ne. n) .and. ( ( m .ne. md2 ) .or. (n .ne. md1 ) ) ) check = .true.
         if( ( md2 .ne. md1 ) .and. (m .eq. n) .and. ( ( m .ne. md2 ) .and. (n .ne. md1 ) ) ) check = .true.
!          print*, 'modes ', n,m , 'check = ', check
          if( check ) then
            do s = 1, nsta
             if( imask1(n,m,s,s).eq. 1 ) imask1(n,m,s,s) = 2
             if( imask1(m,n,s,s).eq. 1 ) imask1(m,n,s,s) = 2
             if( gmask1(n,m,s).eq. 1 ) gmask1(n,m,s) = 2
             if( gmask1(m,n,s).eq. 1 ) gmask1(m,n,s) = 2
             do s2 = s, nsta
               if( imask1(n,m,s,s2).eq. 1 .or.imask1(n,m,s2,s).eq. 1 ) then
                   imask1(n,m,s,s2) = 2
                   imask1(n,m,s2,s) = 2
                   ioff = ioff + 1
               elseif( imask1(m,n,s,s2).eq. 1 .or. imask1(m,n,s2,s).eq. 1 ) then
                   imask1(m,n,s,s2) = 2
                   imask1(m,n,s2,s) = 2
                   ioff = ioff + 1
               endif
               if( s2 .ne. s ) then
                if( mmask1(n,m,s,s2).eq. 1 .or.mmask1(n,m,s2,s).eq. 1 &
               .or. mmask1(m,n,s,s2).eq. 1 .or.mmask1(m,n,s2,s).eq. 1) then
                    mmask1(n,m,s,s2) = 2
                    mmask1(n,m,s2,s) = 2
                    mmask1(m,n,s,s2) = 2
                    mmask1(m,n,s2,s) = 2
!                   print*, 'Switching modes:', n,m,'off', 'MU'
                    ioff = ioff + 1
                endif
               endif
               if (n .eq. m .and. s .ne. s2) then
                   if( lmask1(n,s,s2) .eq. 1 .or. lmask1(n,s2,s) .eq. 1) then
                        lmask1(n,s,s2) = 2
                        lmask1(n,s2,s) = 2
                   endif
               endif
             enddo
             if (n .eq. m) then
              if(kmask1(n,s) .eq. 1 ) kmask1(n,s) = 2
              do i= 1, ncoeff(n,s)
                if (xmask1(i,n,s) .eq. 1 ) xmask1(i,n,s) = 2
              enddo
             endif
            enddo
          endif
        enddo
      enddo

          do i= 1, nadd
            k = 0
            do j = 1, nmodes
             if( termdef(j,i) .ne. 0 ) k = k + 1
            enddo
!            print*, 'set other masks off TERMK= ', tmask1(i),'modes='
!               print*, 'defmds = ',md1,'^',termdef(md1,i), md2,'^',termdef(md2,i), termst(1,i) , termst(2,i)
            if( ((termdef(md1,i) .eq. 0 .or. termdef(md2,i) .eq. 0 ) .or. (md1 .eq. md2 .and. k .gt. 1) .or. &
                ( md1 .ne. md2 .and. k .gt. 2)  ) .and. &
              tmask1(i) .eq. 1 ) then
               tmask1(i) = 2
!                print*, 'OFFFFF'
               if( termst(1,i) .ne. termst(2,i) ) ioff = ioff + 1
            endif
          enddo


      end subroutine

!#######################################################################

     subroutine set_offdiag_off(lmask2,mmask2,tmask2,m1,m2,s1,s2,ioff)

      implicit none
      integer(long)                                                  :: i,j,p
      integer(long), intent(in)                                      :: m1,m2,s1,s2
      integer(long), dimension(nmodes,nsta,nsta), intent(out)        :: lmask2
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(out) :: mmask2
      integer(long), dimension(nadd), intent(out)                    :: tmask2
      integer(long), intent(inout)                                   :: ioff

!        print*, 'INERE', m1,m2,s1,s2
      if( m1 .eq. m2 ) then
       if(lmask2(m1,s1,s2) .eq. 1  )then
                lmask2(m1,s1,s2) = 2
                lmask2(m1,s2,s1) = 2
                ioff = ioff + 1
       endif
      endif
       if( (mmask2(m1,m2,s1,s2) .eq. 1)  .or. &
           (mmask2(m1,m2,s2,s1) .eq. 1)  .or. &
           (mmask2(m2,m1,s1,s2) .eq. 1)  .or. &
           (mmask2(m2,m1,s2,s1) .eq. 1)  )then
                mmask2(m1,m2,s1,s2) = 2
                mmask2(m1,m2,s2,s1) = 2
                mmask2(m2,m1,s1,s2) = 2
                mmask2(m2,m1,s2,s1) = 2
                ioff = ioff + 1
       endif
      do i= 1, nadd
       if( tmask2(i) .eq. 1 ) then
         if(((termst(1,i) .eq. s1) .and. (termst(2,i) .eq. s2)) .or. &
            ((termst(1,i) .eq. s2) .and. (termst(2,i) .eq. s1)) ) then
           if( (termdef(m1,i) .ne. 0) .and. (termdef(m2,i) .ne. 0) ) then
              p = 0
              do j = 1, nmodes
                 if (termdef(j,i) .ne. 0 ) p = p + 1
              enddo
              if ( (p .eq. 2 .and. m1 .ne. m2) .or. &
                   (p .eq. 1 .and. m1 .eq. m2) ) then
                   tmask2(i) = 2
                   ioff = ioff + 1
!                   print*, 'setting off ', termdef(m2,i), termdef(m1,i), termst(1,i) ,termst(2,i)
              endif
           endif
          endif
        endif
      enddo
      return
      end subroutine


!#######################################################################

      real(dop) function mutdis(mean2, maxval0)

      real(dop), intent(in) :: maxval0, mean2
      real(dop)             :: stdevgen,gaussgen,y,stdev,x, mean


!     if the current value (the "mean") is greater that the mutval(i,i) (here maxval0). This can only happen
!     when the covariance matrix produces that "mean" value. this sets the mean near (but under) maxval0 so as to avoid any
!     annoying infinite floating numbers.
      if( abs(mean2) .gt. maxval0 ) then
        mean = maxval0*(abs(mean2)/mean2)*0.95
      else
        mean = mean2
      endif
      x = drandom(seed) - 0.5
      y=ngenga
!     standard deviation of gaussian function determining generational standard deviation of
!     the generated normal distribuition that depends on a maximum value and the
!     current (named "mean" here) value
!      stdevgen = maxgen * sqrt( 1 / log(1/0.2) )
      stdevgen =  sqrt( - maxgen**2 / log(0.2) )
      gaussgen = 1 * exp( - ( y / stdevgen ) ** 2)

      stdev = abs((maxval0 * abs(x)/x - mean)/3)
      if( stdev .eq. 0 ) stdev = 0.0001
!      print*, 'stdev =', stdev
      stdev = gaussgen*stdev
      mutdis = (abs(x)/x) * abs(mean+stdev*real(randgaussian()))
!      print*, 'mutDIS =', mutdis
      return

      end function

!#######################################################################

      real(dop) function mutdis2(mean, stdev2, min_std)

      real(dop), intent(in) :: mean, stdev2, min_std
      real(dop)             :: stdevgen,gaussgen,y,stdev

      y=ngenga
!     standard deviation of gaussian function determining generational standard deviation of
!     the generated normal distribuition
      stdevgen =  sqrt( - maxgen**2 / log(0.2) )
      gaussgen = 1 * exp( - ( y / stdevgen ) ** 2 )

      if(stdev2 .lt. min_std*gaussgen ) then
          stdev = min_std*gaussgen
      else
          stdev = stdev2
      endif
      stdev = gaussgen*stdev
      if(stdev .lt. 0.0001) stdev = 0.0001
!      print*, 'gaaussgen=', gaussgen, 'stvegen =', stdevgen , 'stdev =', stdev, &
!      ', 2=', stdev2
      mutdis2 = mean+stdev*real(randgaussian())
!      print*, 'mutdis2 =', mutdis2

      return

      end function

!##############################################################
! mutmag is a bogdy function that regulates the magnitude of the mutation:
! a gaussian function slowly determines the maximum value of a mutation at
! a given generation (decreses as generation pass and also depends on the difference
! between a asensible maximum and the current value).Simultaneously, at every generation
! a (choice of 2 types of) inverted gaussian (with maximum determined by the first gaussian) determies
! the magnitude of the mutation for that parameter (in that generation); this gaussian function has a SHAPE
! that ensures that most mutations are often small, but with the possibility of getting big mutations
! everynow and then that allow for escaping local minima better.
! THIS SAMPLING PROCEDURE DOES NOT RESULT IN A GAUSSIAN DISTRIBUITION.
! again, notice that the maximum and minimum magnitudes of the mutation
! also depend on the difference between the current value (val0) and the maximum value (maxval0)
! this parameter can posses. Deadly mexican, arriba-arriba¬!
! if x is negative, then the mutation will be towards a negative value, and
! viceversa
!##################################################################

      real(dop) function mutmag(val0,maxval0,ordertyp)

      real(dop), intent(in) :: val0,ordertyp,maxval0
      real(dop)             :: maxim, minim, stdevgen, stdevran,ran, ran2
      real(dop)             :: gaussgen,x,y,gaussran,rangeran, A,z

      y=ngenga
      if(ordertyp .eq. 2 ) then
!     here the sampling range is extended to a big number and then
!     scaled down logarithmically (base e) and normalised
!     this bassically allows for small mutations that span a
!     good range of orders of magnitude
              rangeran = exp(10.0)
              ran = drandom(seed)
              ran2 = 2*drandom(seed)-1
              x = ran2/abs(ran2) * log( ran*rangeran) / 10
      elseif( (ordertyp .eq. 3) .or. (ordertyp .eq. 1)) then
              rangeran = 1.0
              x=(2*drandom(seed)-1)*rangeran
      endif
88    z = 2*drandom(seed)-1
      if( z .eq. 0) goto 88
      maxim = maxval0 * z/abs(z) - val0
      if( maxim .eq. 0)  goto 88
      if(ordertyp .eq. 1) maxim = abs(maxim)
      minim = maxim * 0.05

!     standard deviation of gaussian determining generational maximal amplitude
      stdevgen = maxgen * sqrt( 1 / log(maxim/minim) )
      gaussgen = maxim * exp(- ( y / stdevgen ) ** 2 )

!     gaussian types determined by the random number
!     my "natural log gaussian" function
!     like ordertyp2, this helps to sample ranges over orders of magnitude.
      if(ordertyp .eq. 1) then
         A = exp(gaussgen)-1
         if( .not. (abs(A) .gt. huge(A)) ) then
           stdevran = rangeran * sqrt( 1 / (gaussgen - minim*0.0001) )
!           print*, 'BooBS'
         else
           stdevran = rangeran * sqrt( 1 / log( (A)/(exp(minim*0.0001) -1) ) )
!           print*, 'BAABS'
         endif
         gaussran = exp(- (abs(x) / stdevran)**2 )
         mutmag = z/abs(z) * ( gaussgen + minim*0.001 - log( A * gaussran + 1 ) )
!         print*, 'ITS TYPE 1'
!     a typpical gaussian function
      else
!        the standard dev of this one depends on the amplitude obtained from
!        gaussgen (how tall the overall function at generation y)
!        notice that the gaussian is inverted, so the maximal amplitude of
!        gaussgen is now the lowest value of mutation (very close to zero)
         stdevran = rangeran * sqrt( 1 / log(gaussgen/(minim*0.001 )) )
         gaussran = exp(- ( abs(x) / stdevran ) ** 2 )

         mutmag = ( gaussgen + minim*0.001 - gaussgen * gaussran )
         if( abs(mutmag) .lt. 0.000000001) mutmag = 0
!         print*, 'ITS TYPE 3'
      endif

!        for debugging:
!         print*, 'x = ', x, 'y = ', y
!         print*, 'minim =', minim
!         print*, 'maxim =', maxim
!         print*, 'val0 =', val0
!         print*, 'maxgen =', maxgen
!         print*, 'rangeran =', rangeran
!         print*, 'gaussran =', gaussran
!         print*, 'stdevran =', stdevran
!         print*, 'log(gaussgen/(minim*0.001 ) =', log(gaussgen/(minim*0.001))
!         print*, 'log( (A)/(exp(minim*0.0001) -1) ) =', log((A)/(exp(minim*0.0001) -1) )
!         print*, 'A = exp(gaussgen)-1 =' , A
!         print*, 'gaussgen =', gaussgen
!         print*, 'stdevgen =', stdevgen
!         print*, 'gaussgen * gaussran =',gaussgen * gaussran
!         print*, 'mutmag =',mutmag
      return

      end function

!##############################################################

      subroutine mutate(kappa,lambda,gamma,mu,iota,expcoe,termk,kappamask,&
           lambdamask,gammamask,mumask,iotamask,expcoemask,termkmask,imut)

      implicit none

      real(dop)           :: mm,t1,prod_freq0,tempmutchance

      real(dop), dimension(nmodes,nsta), intent(inout)             :: kappa
      real(dop), dimension(nmodes,nsta,nsta), intent(inout)        :: lambda
      real(dop), dimension(nmodes,nmodes,nsta), intent(inout)      :: gamma
      real(dop), dimension(maxcoe,nmodes,nsta), intent(inout)      :: expcoe
      real(dop), dimension(nadd), intent(inout)                    :: termk
      real(dop), dimension(nmodes,nmodes,nsta,nsta), intent(inout) :: iota,mu


      integer(long), dimension(nmodes,nsta), intent(in)             :: kappamask
      integer(long), dimension(nmodes,nsta,nsta), intent(in)        :: lambdamask
      integer(long), dimension(nmodes,nmodes,nsta), intent(in)      :: gammamask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(in)      :: expcoemask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: iotamask,mumask
      integer(long), dimension(nadd), intent(in)                  :: termkmask
      integer(long)       :: i,j,s,s1,k,m,tot_order,x
      integer(long), intent(in) :: imut

      if (covar_adapt) then
         tempmutchance = mutchance
         mutchance = 0.5 - mutchance_w(imut)*0.40
         if( gencount .ne. ngenga) muti = muti + 1
         if( gencount .ne. ngenga) gencount = ngenga
      endif

      if (covar_adapt .and. muti .gt. 5) then
         mutchance = tempmutchance
         muti = -2
         covar_adapt = .false.
      endif

      x = 0
      prod_freq0 = 1
      tot_order = 0
      t1 = real(maxgen)

      if (offdiagsignopt .eqv. .false. ) then
      do s=1,nsta
         do i=1,nmodes
            if (kappamask(i,s) .eq. 1) then
               x = x + 1
               mm=drandom(seed)
               if (mm .gt. mutchance) then
                  if(covar_adapt) then
                     kappa(i,s) = mutval(imut,x)
                  else
                     kappa(i,s) = mutdis(kappa(i,s),kappamut(1))
!                     mutmag(kappa(i,s),gamut(1,1),gamut(1,2))
                  endif
               endif
            endif
         enddo
      enddo
      do s=1,nsta-1
         do s1=s+1,nsta
            do i=1,nmodes
               if (lambdamask(i,s,s1) .eq. 1) then
                  x = x + 1
                  mm=drandom(seed)

                  if (mm .gt. mutchance) then
                     if(covar_adapt) then
                        lambda(i,s,s1) = mutval(imut,x)
                     else
                        lambda(i,s,s1) = mutdis(lambda(i,s,s1),lambdamut(1))
!                       mutmag(lambda(i,s,s1),gamut(2,1),gamut(2,2))
                     endif
                  endif
               endif
            enddo
         enddo
      enddo
      do s=1,nsta
         do i=1,nmodes
            do j=i,nmodes
               if (gammamask(i,j,s) .eq. 1) then
                  x = x + 1
                  mm=drandom(seed)
                  if (mm .gt. mutchance) then
                     if(covar_adapt) then
                        gamma(i,j,s) = mutval(imut,x)
                     else
                        gamma(i,j,s) = mutdis(gamma(i,j,s),gammamut(1))
!                       mutmag(gamma(i,j,s),gamut(3,1),gamut(3,2))
                     endif
                  endif
               endif
            enddo
         enddo
      enddo

!-----------------------------------------------------------------------
! termk (explicitly added terms)
! added by c.r aka deadly mexican
!-----------------------------------------------------------------------
      do i = 1, nadd
         if (termkmask(i) .eq. 1) then
            x = x + 1
            tot_order = 0
            do j = 1, nmodes
               if( termdef(j,i) .gt. 0) then
                  tot_order = tot_order + termdef(j,i)
               endif
            enddo
            mm = drandom(seed)
            if (mm .gt. mutchance) then
               if(covar_adapt) then
                  termk(i) = mutval(imut,x)
               else
                  termk(i) = mutdis(termk(i)*(factorial(tot_order)),&
                       termkmut(1))
                  termk(i) = termk(i)/(factorial(tot_order))
               endif
            endif
         endif
      enddo
! mu
      do s=1,nsta-1
         do s1=s+1,nsta
            do i=1,nmodes
               do j=i,nmodes
                  if (mumask(j,i,s,s1) .eq. 1) then
                     x = x + 1
                     mm=drandom(seed)
!                  print*,'mu =', j,i,s,s1, 'x = ', x
                     if (mm .gt. mutchance) then
                        if(covar_adapt) then
                           mu(j,i,s,s1) = mutval(imut,x)
!                          if( abs(mutval(imut,x)) .eq. 0.0) print*, 'MUTANT ZEROed'
                        else
                           mu(j,i,s,s1) = mutdis(mu(j,i,s,s1), mumut(1))
                          if( mu(j,i,s,s1) .eq. 0 ) print*, 'ITS MUTDIS'
!                        mutmag( mu(j,i,s,s1) ,gamut(4,1),gamut(4,2))
                        endif
                     endif
                  endif
               enddo
            enddo
         enddo
      enddo

! iota
      do s=1,nsta
         do s1=s,nsta
            do i=1,nmodes
               do j=i,nmodes
                  if (iotamask(i,j,s,s1) .eq. 1) then
                     x = x + 1
                     mm=drandom(seed)
                     if (mm .gt. mutchance) then
                        if(covar_adapt) then
                           iota(i,j,s,s1) = mutval(imut,x)
                        else
                           iota(i,j,s,s1) = mutdis(iota(i,j,s,s1),iotamut(1))
!                       mutmag( iota(i,j,s,s1) ,gamut(12,1),gamut(12,2))
                        endif
                     endif
                  endif
               enddo
            enddo
         enddo
      enddo

! general diabatic potential functions
      do s=1,nsta
         do m=1,nmodes
            if (ncoeff(m,s).gt.0) then
               do k=1,ncoeff(m,s)
                  if (expcoemask(k,m,s).eq.1) then
                     x=x+1
                     if (expcoemut(1,k,m,s).eq.-999.0) then
                        write(6,'(/,a,x,i2,x,a,/)') 'The diabatic potential function type',&
                             ifdiab(m,s),'is not yet supported for use with the genetic algorithm.'
                        STOP
                     endif
                     mm=drandom(seed)
                     if (mm.gt.mutchance) then
                        if(covar_adapt) then
                           expcoe(k,m,s)=mutval(imut,x)
                        else
                           expcoe(k,m,s)=mutdis(expcoe(k,m,s),expcoemut(1,k,m,s))
!           print*, 'i NONE expcoe k =', k,'m s =',m,s,'x = ', x, 'imut = ', imut, 'xcoe=', &
!                          expcoe(k,m,s)
!                          if ( k .eq. 2 .and. s .eq. 1 ) print*, 'xcoe = ', expcoe(k,m,s)
                        endif
!	term only need to be positive!
                        if(ifdiab(m,s) .eq. 18 .and. k .eq. 6 ) expcoe(k,m,s) = abs(expcoe(k,m,s))
                        if(ifdiab(m,s) .eq. 18 .and. k .eq. 3 ) expcoe(k,m,s) = abs(expcoe(k,m,s))
                        if(ifdiab(m,s) .eq. 2 .and. k .eq. 1 ) expcoe(k,m,s) = abs(expcoe(k,m,s))
                        if(ifdiab(m,s) .eq. 4 .and. k .eq. 1 ) expcoe(k,m,s) = abs(expcoe(k,m,s))
                        if(ifdiab(m,s) .eq. 2 .and. k .eq. 3 .and. expcoe(k,m,s) .eq. 0 )&
                         expcoe(k,m,s) = abs(expcoe(k,m,s)) + 0.00001
                        if(ifdiab(m,s) .eq. 4 .and. k .eq. 3 .and. expcoe(k,m,s) .eq. 0 )&
                         expcoe(k,m,s) = abs(expcoe(k,m,s)) + 0.00001

                     endif
                  endif
               enddo
            endif
         enddo
      enddo

!----------------------------------------------------------------------------
! TO OPTIMISE THE CONSTELLATION OF SIGNS OF THE OFF DIAGONAL TERMS;
! THEY DONT AFFECT THE FITTING ALONG A GIVEN MODE, BUT GIVE DIFFERENT
! RESULTS WHEN LOOKING ALONG THE DIAGONAL BETWEEN 2 MODES ("bi-"
! DIRECTION)DEPENDING ON THE SIGN (LAMBDA'S MU'S ETC).
! THE GA ALGORITHM IS IDEAL FOR HANDLING SUCH
! OPTIMISATIONS. IDEALLY ONE WANTS TO USE THIS BEFORE BEGINING TO FIT ANY
! PARAMETERS ALONG THE DIAGONAL OF TWO MODES, OTHERWISE IF DONE IN TANDEM
! WITH PARAMETER OPTIMISATION, GA WOULD TAKE MUCH LONGER TO OPTIMISE THE
! PARAMETERS (deadly mexican)
!----------------------------------------------------------------------------
      elseif (offdiagsignopt .eqv. .true. ) then
         do s=1,nsta-1
            do s1=s+1,nsta
               do i=1,nmodes
                  do j=i,nmodes-1
                     mm=drandom(seed)
                     if ((mm .gt. mutchance) .and.  &
                          ((mfit(j) .eq. 1) .and. (mfit(i) .eq. 1)) .and. &
                          ( mumask(j,i,s,s1) .ne. 2) ) then
                        mu(j,i,s,s1) = -mu(i,j,s,s1)
                        mu(j,i,s,s1) = mu(j,i,s,s1)
                     endif
                  enddo
                  mm=drandom(seed)
                  if ((mm .gt. mutchance) .and. &
                       ( mfit(i) .eq. 1 )  .and. &
                       ( lambdamask(i,s,s1) .ne. 2) ) &
                     lambda(i,s,s1) = -lambda(i,s,s1)

               enddo
            enddo
         enddo
        do i = 1, nadd
           if (termkmask(i) .eq. 1) then
              if( (offdiagsignopt .eqv. .true.) .and. &
                   (termst(1,i) .ne. termst(2,i)) .and. &
                   (mm .gt. mutchance)   ) &
                   termk(i) = -termk(i)
           endif
        enddo
      endif
      if(lforcepos) call force_pos(kappa,gamma,expcoe,termk,kappamask,gammamask,expcoemask,termkmask)
      if(lusecons) then
         call par2vec2d(optcoo,mr(mrconst),mr(mrkappa),mr(mrlambda),&
              mr(mrgamma),mr(mriota),mr(mrmu),mr(mrexpcoe),&
              mr(mrhigher),mr(mrtermk),mr(mrcpcoe),&
              mr(mrswitch),mr(mrcp2d),mi(miconstmask),mi(mikappamask),&
              mi(milambdamask),mi(migammamask),mi(miiotamask),&
              mi(mimumask),mi(miexpcoemask),mi(mitermkmask),&
              mi(micpcoemask),mi(miswitchmask),mi(micp2d))

         call vec2par2d(optcoo,mr(mrconst),mr(mrkappa),mr(mrlambda),&
              mr(mrgamma),mr(mriota),mr(mrmu),mr(mrexpcoe),&
              mr(mrhigher),mr(mrtermk),mr(mrcpcoe),&
              mr(mrswitch),mr(mrcp2d),mi(miconstmask),mi(mikappamask),&
              mi(milambdamask),mi(migammamask),mi(miiotamask),&
              mi(mimumask),mi(miexpcoemask),mi(mitermkmask),&
              mi(micpcoemask),mi(miswitchmask),mi(micp2d))
      endif

      if (muti .lt. 0) then
         covar_adapt = .true.
         muti = 0
      elseif (covar_adapt) then
         mutchance = tempmutchance
      endif

      return
    end subroutine mutate

!##############################################################

      subroutine firstchild(kappa,lambda,gamma,mu,iota,&
              expcoe,termk,kappa2,lambda2,gamma2,mu2,iota2,&
              expcoe2,termk2,kappa3,lambda3,gamma3,mu3,iota3,&
              expcoe3,termk3,kappacomask,lambdacomask,gammacomask,&
              mucomask,iotacomask,expcoecomask,termkcomask,kappamask,&
              lambdamask,gammamask,mumask,iotamask,expcoemask,termkmask)

      implicit none

      real(dop), dimension(nmodes,nsta), intent(out)             :: kappa
      real(dop), dimension(nmodes,nsta,nsta), intent(out)        :: lambda
      real(dop), dimension(nmodes,nmodes,nsta), intent(out)      :: gamma
      real(dop), dimension(nmodes,nmodes,nsta,nsta), intent(out) :: iota,mu
      real(dop), dimension(nadd), intent(out)                    :: termk
      real(dop), dimension(maxcoe,nmodes,nsta), intent(out)      :: expcoe

      real(dop), dimension(nmodes,nsta), intent(in)             :: kappa2
      real(dop), dimension(nmodes,nsta,nsta), intent(in)        :: lambda2
      real(dop), dimension(nmodes,nmodes,nsta), intent(in)      :: gamma2
      real(dop), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: iota2,mu2
      real(dop), dimension(nadd), intent(in)                    :: termk2
      real(dop), dimension(maxcoe,nmodes,nsta), intent(in)      :: expcoe2

      real(dop), dimension(nmodes,nsta), intent(in)             :: kappa3
      real(dop), dimension(nmodes,nsta,nsta), intent(in)        :: lambda3
      real(dop), dimension(nmodes,nmodes,nsta), intent(in)      :: gamma3
      real(dop), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: iota3,mu3
      real(dop), dimension(nadd), intent(in)                    :: termk3
      real(dop), dimension(maxcoe,nmodes,nsta), intent(in)      :: expcoe3

      integer(long), dimension(nmodes,nsta), intent(in)             :: kappamask
      integer(long), dimension(nmodes,nsta,nsta), intent(in)        :: lambdamask
      integer(long), dimension(nmodes,nmodes,nsta), intent(in)      :: gammamask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: iotamask,mumask
      integer(long), dimension(nadd), intent(in)                    :: termkmask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(in)      :: expcoemask

      integer(long), dimension(nmodes,nsta), intent(out)             :: kappacomask
      integer(long), dimension(nmodes,nsta,nsta), intent(out)        :: lambdacomask
      integer(long), dimension(nmodes,nmodes,nsta), intent(out)      :: gammacomask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(out) :: iotacomask,mucomask
      integer(long), dimension(nadd), intent(out)                    :: termkcomask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(out)      :: expcoecomask

      integer(long)       :: i,j,s,s1,p
      real(dop)           :: mm

!       for debugging array:
!        l = 1
!       do s=1,nsta
!         do i=1,nmodes
!          do p=1,maxcoe
!            print*,   trim(charit(l,0)),'A (',trim(charit(p,0)),'): s',trim(charit(s,0)),&
!            ' m',trim(charit(i,0)),expcoe2(p,i,s)
!            print*, trim(charit(l+1,0)),'A (2): s',trim(charit(s,0)),&
!            ' m',trim(charit(i,0)),expcoe2(p+1,i,s)
!            print*, trim(charit(l+2,0)),'A (3): s',trim(charit(s,0)),&
!            ' m',trim(charit(i,0)),expcoe2(p+2,i,s)
!             l = l + 1
!          enddo
!         enddo
!       enddo

      do s=1,nsta
         do i=1,nmodes
            if (kappamask(i,s) .eq. 1 ) then
               mm=drandom(seed)
               if (mm .gt. 0.5) then
                  kappa(i,s)=kappa2(i,s)
                  kappacomask(i,s)=1
               else
                  kappa(i,s)=kappa3(i,s)
                  kappacomask(i,s)=2
               endif
            endif
         enddo
      enddo
      p = 0
      do s=1,nsta-1
         do s1=s+1,nsta
            do i=1,nmodes
               if (lambdamask(i,s,s1) .eq. 1) then
                  mm=drandom(seed)
                  if (mm .gt. 0.5) then
                     lambda(i,s,s1)=lambda2(i,s,s1)
                     lambdacomask(i,s,s1)=1
                  else
                     lambda(i,s,s1)=lambda3(i,s,s1)
                     lambdacomask(i,s,s1)=2
                  endif
               endif

            enddo
         enddo
      enddo
      do s=1,nsta
         do i=1,nmodes
            do j=1,nmodes
               if (gammamask(i,j,s) .eq. 1) then
                  mm=drandom(seed)
                  if (mm .gt. 0.5) then
                     gamma(i,j,s)=gamma2(i,j,s)
                     gammacomask(i,j,s)=1
                  else
                     gamma(i,j,s)=gamma3(i,j,s)
                     gammacomask(i,j,s)=2
                  endif
               endif
            enddo
         enddo
      enddo
      do s=1,nsta-1
         do s1=s+1,nsta
            do i=1,nmodes
               do j=1,nmodes
                  if (mumask(j,i,s,s1) .eq. 1) then
                     mm=drandom(seed)
                     if (mm .gt. 0.5) then
                        mu(j,i,s,s1)=mu2(i,j,s,s1)
                        mucomask(i,j,s,s1)=1
                     else
                        mu(j,i,s,s1)=mu3(i,j,s,s1)
                        mucomask(i,j,s,s1)=2
                     endif
                  endif
               enddo
            enddo
         enddo
      enddo

      do i = 1, nadd
         if (termkmask(i) .eq. 1) then
            mm=drandom(seed)
            if (mm .gt. 0.5) then
               termk(i) = termk2(i)
               termkcomask(i) = 1
            else
               termk(i) = termk3(i)
               termkcomask(i) = 2
            endif
         endif
      enddo

      do s=1,nsta
         do s1=s,nsta
            do i=1,nmodes
               do j=1,nmodes
                  if (iotamask(i,j,s,s1) .eq. 1) then
                     mm=drandom(seed)
                     if (mm .gt. 0.5) then
                        iota(i,j,s,s1)=iota2(i,j,s,s1)
                        iotacomask(i,j,s,s1)=1
                     else
                        iota(i,j,s,s1)=iota3(i,j,s,s1)
                        iotacomask(i,j,s,s1)=2
                     endif
                  endif
               enddo
            enddo
         enddo
      enddo

      do s=1,nsta
         do i=1,nmodes
            do p = 1, ncoeff(i,s)
               if(expcoemask(p,i,s) .eq. 1)then
                  mm=drandom(seed)
                  if (mm .gt. 0.5) then
                     expcoe(p,i,s) = expcoe2(p,i,s)
                     expcoecomask(p,i,s) = 1
                  else
                     expcoe(p,i,s) = expcoe3(p,i,s)
                     expcoecomask(p,i,s) = 2
                  endif
               endif
            enddo
         enddo
      enddo
      if( lusecons ) then
         call par2vec2d(optcoo,mr(mrconst),kappa,lambda,&
              gamma,iota,mu,expcoe,mr(mrhigher),termk,mr(mrcpcoe),&
              mr(mrswitch),mr(mrcp2d),mi(miconstmask),kappamask,&
              lambdamask,gammamask,iotamask,mumask,expcoemask,termkmask,&
              mi(micpcoemask),mi(miswitchmask),mi(micp2d))

         call vec2par2d(optcoo,mr(mrconst),kappa,lambda,&
              gamma,iota,mu,expcoe,mr(mrhigher),termk,mr(mrcpcoe),&
              mr(mrswitch),mr(mrcp2d),mi(miconstmask),kappamask,&
              lambdamask,gammamask,iotamask,mumask,expcoemask,termkmask,&
              mi(micpcoemask),mi(miswitchmask),mi(micp2d))

      endif

      return
    end subroutine firstchild

!##############################################################

      subroutine secondchild(kappa,lambda,gamma,mu,iota,&
              expcoe,termk,kappa2,lambda2,gamma2,mu2,iota2,&
              expcoe2,termk2,kappa3,lambda3,gamma3,mu3,iota3,&
              expcoe3,termk3,kappacomask,lambdacomask,gammacomask,&
              mucomask,iotacomask,expcoecomask,termkcomask,kappamask,&
              lambdamask,gammamask,mumask,iotamask,expcoemask,termkmask)

      implicit none

      real(dop), dimension(nmodes,nsta), intent(out)             :: kappa
      real(dop), dimension(nmodes,nsta,nsta), intent(out)        :: lambda
      real(dop), dimension(nmodes,nmodes,nsta), intent(out)      :: gamma
      real(dop), dimension(nmodes,nmodes,nsta,nsta), intent(out) :: iota,mu
      real(dop), dimension(nadd), intent(out)                    :: termk
      real(dop), dimension(maxcoe,nmodes,nsta), intent(out)      :: expcoe

      real(dop), dimension(nmodes,nsta), intent(in)             :: kappa2
      real(dop), dimension(nmodes,nsta,nsta), intent(in)        :: lambda2
      real(dop), dimension(nmodes,nmodes,nsta), intent(in)      :: gamma2
      real(dop), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: iota2,mu2
      real(dop), dimension(nadd), intent(in)                    :: termk2
      real(dop), dimension(maxcoe,nmodes,nsta), intent(in)      :: expcoe2

      real(dop), dimension(nmodes,nsta), intent(in)             :: kappa3
      real(dop), dimension(nmodes,nsta,nsta), intent(in)        :: lambda3
      real(dop), dimension(nmodes,nmodes,nsta), intent(in)      :: gamma3
      real(dop), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: iota3,mu3
      real(dop), dimension(nadd), intent(in)                    :: termk3
      real(dop), dimension(maxcoe,nmodes,nsta), intent(in)      :: expcoe3

      integer(long), dimension(nmodes,nsta), intent(in)             :: kappamask
      integer(long), dimension(nmodes,nsta,nsta), intent(in)        :: lambdamask
      integer(long), dimension(nmodes,nmodes,nsta), intent(in)      :: gammamask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: iotamask,mumask
      integer(long), dimension(nadd), intent(in)                    :: termkmask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(in)      :: expcoemask

      integer(long), dimension(nmodes,nsta), intent(in)             :: kappacomask
      integer(long), dimension(nmodes,nsta,nsta), intent(in)        :: lambdacomask
      integer(long), dimension(nmodes,nmodes,nsta), intent(in)      :: gammacomask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: iotacomask,mucomask
      integer(long), dimension(nadd), intent(in)                    :: termkcomask
      integer(long), dimension(maxcoe,nmodes,nsta), intent(in)      :: expcoecomask

      integer(long) :: i,j,s,s1,p

          do s=1,nsta
             do i=1,nmodes
                if (kappamask(i,s) .eq. 1) then
                   if (kappacomask(i,s) .eq. 1) then
                     kappa(i,s) = kappa3(i,s)
                   else
                     kappa(i,s) = kappa2(i,s)
                   endif
                endif
             enddo
          enddo
          do s=1,nsta-1
             do s1=s+1,nsta
                do i=1,nmodes
                if (lambdamask(i,s,s1) .eq. 1) then
                   if (lambdacomask(i,s,s1) .eq. 1) then
                     lambda(i,s,s1) = lambda3(i,s,s1)
                   else
                     lambda(i,s,s1) = lambda2(i,s,s1)
                   endif
                endif
                enddo
             enddo
          enddo
       do s=1,nsta
          do i=1,nmodes
             do j=1,nmodes
                if( gammamask(i,j,s) .eq. 1) then
                   if (gammacomask(i,j,s) .eq. 1) then
                     gamma(i,j,s) = gamma3(i,j,s)
                   else
                     gamma(i,j,s) = gamma2(i,j,s)
                   endif
                endif
             enddo
          enddo
        enddo
       do s=1,nsta-1
          do s1=s+1,nsta
             do i=1,nmodes
                do j=1,nmodes
                 if (mumask(j,i,s,s1) .eq. 1) then
                   if (mucomask(i,j,s,s1) .eq. 1) then
                     mu(j,i,s,s1) = mu3(i,j,s,s1)
                   else
                     mu(j,i,s,s1) = mu2(i,j,s,s1)
                   endif
                endif
                enddo
             enddo
          enddo
       enddo
       do s=1,nsta
         do s1=s,nsta
            do i=1,nmodes
               do j=1,nmodes
                if (iotamask(i,j,s,s1) .eq. 1) then
                  if (iotacomask(i,j,s,s1) .eq. 1) then
                    iota(i,j,s,s1) = iota3(i,j,s,s1)
                  else
                    iota(i,j,s,s1) = iota2(i,j,s,s1)
                  endif
               endif
               enddo
            enddo
         enddo
      enddo
      do i = 1, nadd
        if (termkmask(i) .eq. 1) then
          if (termkcomask(i) .eq. 1) then
            termk(i) = termk3(i)
          else
            termk(i) = termk2(i)
          endif
        endif
      enddo

       do s=1,nsta
          do i=1,nmodes
             do p = 1, ncoeff(i,s)
                if( expcoemask(p,i,s) .eq. 1 )then
                   if (expcoecomask(p,i,s) .eq. 1) then
                     expcoe(p,i,s) = expcoe3(p,i,s)
                   else
                     expcoe(p,i,s) = expcoe2(p,i,s)
                   endif
                endif
             enddo
          enddo
       enddo
      if( lusecons ) then
         call par2vec2d(optcoo,mr(mrconst),kappa,lambda,&
              gamma,iota,mu,expcoe,mr(mrhigher),termk,mr(mrcpcoe),&
              mr(mrswitch),mr(mrcp2d),mi(miconstmask),kappamask,&
              lambdamask,gammamask,iotamask,mumask,expcoemask,termkmask,&
              mi(micpcoemask),mi(miswitchmask),mi(micp2d))

         call vec2par2d(optcoo,mr(mrconst),kappa,lambda,&
              gamma,iota,mu,expcoe,mr(mrhigher),termk,mr(mrcpcoe),&
              mr(mrswitch),mr(mrcp2d),mi(miconstmask),kappamask,&
              lambdamask,gammamask,iotamask,mumask,expcoemask,termkmask,&
              mi(micpcoemask),mi(miswitchmask),mi(micp2d))

      endif

       return
       end subroutine secondchild

!##############################################################

      subroutine gafilename(ngenga,npop,filename)

      implicit none
      integer(long), intent(in)     :: ngenga,npop
      character(len=19)             :: filename

!      if(lspecies) then
!
!         if (ngenga .lt. 10) then
!            if (npop .lt. 10) then
!               write(filename,513) 'sp',sp,'_000',ngenga,'_000',npop,'.vcham'
!            else if (npop .lt. 100) then
!               write(filename,514) 'sp',sp,'_000',ngenga,'_00',npop,'.vcham'
!            else if (npop .lt. 1000) then
!               write(filename,515) 'sp',sp,'_000',ngenga,'_0',npop,'.vcham'
!            else if (npop .lt. 10000) then
!               write(filename,516) 'sp',sp,'_000',ngenga,'_',npop,'.vcham'
!            else
!               print*,'Populations above 4998 not supported'
!            endif
!         elseif (ngenga .lt. 100) then
!            if (npop .lt. 10) then
!               write(filename,517) 'sp',sp,'_00',ngenga,'_000',npop,'.vcham'
!            else if (npop .lt. 100) then
!               write(filename,518) 'sp',sp,'_00',ngenga,'_00',npop,'.vcham'
!            else if (npop .lt. 1000) then
!               write(filename,519) 'sp',sp,'_00',ngenga,'_0',npop,'.vcham'
!            else if (npop .lt. 10000) then
!               write(filename,520) 'sp',sp,'_00',ngenga,'_',npop,'.vcham'
!            else
!               print*,'Populations above 4998 not supported'
!            endif
!         elseif (ngenga .lt. 1000) then
!            if (npop .lt. 10) then
!               write(filename,521) 'sp',sp,'_0',ngenga,'_000',npop,'.vcham'
!            else if (npop .lt. 100) then
!               write(filename,522) 'sp',sp,'_0',ngenga,'_00',npop,'.vcham'
!            else if (npop .lt. 1000) then
!               write(filename,523) 'sp',sp,'_0',ngenga,'_0',npop,'.vcham'
!            else if (npop .lt. 10000) then
!               write(filename,524) 'sp',sp,'_0',ngenga,'_',npop,'.vcham'
!            else
!               print*,'Populations above 4998 not supported'
!            endif
!          else if (ngenga .lt. 10000) then
!            if (npop .lt. 10) then
!               write(filename,525) 'sp',sp,'_',ngenga,'_000',npop,'.vcham'
!            else if (npop .lt. 100) then
!               write(filename,526) 'sp',sp,'_',ngenga,'_00',npop,'.vcham'
!            else if (npop .lt. 1000) then
!               write(filename,527) 'sp',sp,'_',ngenga,'_0',npop,'.vcham'
!            else if (npop .lt. 10000) then
!               write(filename,528) 'sp',sp,'_',ngenga,'_',npop,'.vcham'
!            else
!               print*,'Populations above 4998 not supported'
!            endif
!          else
!            print*,'More than 4998 generations not supported'
!          endif
!      else
         if (ngenga .lt. 10) then
            if (npop .lt. 10) then
               write(filename,513) '000',ngenga,'_000',npop,'.vcham'
            else if (npop .lt. 100) then
               write(filename,514) '000',ngenga,'_00',npop,'.vcham'
            else if (npop .lt. 1000) then
               write(filename,515) '000',ngenga,'_0',npop,'.vcham'
            else if (npop .lt. 10000) then
               write(filename,516) '000',ngenga,'_',npop,'.vcham'
            else
               print*,'Populations above 4998 not supported'
            endif
         elseif (ngenga .lt. 100) then
            if (npop .lt. 10) then
               write(filename,517) '00',ngenga,'_000',npop,'.vcham'
            else if (npop .lt. 100) then
               write(filename,518) '00',ngenga,'_00',npop,'.vcham'
            else if (npop .lt. 1000) then
               write(filename,519) '00',ngenga,'_0',npop,'.vcham'
            else if (npop .lt. 10000) then
               write(filename,520) '00',ngenga,'_',npop,'.vcham'
            else
               print*,'Populations above 4998 not supported'
            endif
         elseif (ngenga .lt. 1000) then
            if (npop .lt. 10) then
               write(filename,521) '0',ngenga,'_000',npop,'.vcham'
            else if (npop .lt. 100) then
               write(filename,522) '0',ngenga,'_00',npop,'.vcham'
            else if (npop .lt. 1000) then
               write(filename,523) '0',ngenga,'_0',npop,'.vcham'
            else if (npop .lt. 10000) then
               write(filename,524) '0',ngenga,'_',npop,'.vcham'
            else
               print*,'Populations above 4998 not supported'
            endif
          else if (ngenga .lt. 10000) then
            if (npop .lt. 10) then
               write(filename,525) ngenga,'_000',npop,'.vcham'
            else if (npop .lt. 100) then
               write(filename,526) ngenga,'_00',npop,'.vcham'
            else if (npop .lt. 1000) then
               write(filename,527) ngenga,'_0',npop,'.vcham'
            else if (npop .lt. 10000) then
               write(filename,528) ngenga,'_',npop,'.vcham'
            else
               print*,'Populations above 4998 not supported'
            endif
          else
            print*,'More than 4998 generations not supported'
          endif
!       endif

        filename = 'Pop/'//trim(filename)

! 533   format(a2,i1,a4,i1,a4,i1,a6)
! 534   format(a2,i1,a4,i1,a3,i2,a6)
! 535   format(a2,i1,a4,i1,a2,i3,a6)
! 536   format(a2,i1,a4,i1,a1,i4,a6)
! 537   format(a2,i1,a3,i2,a4,i1,a6)
! 538   format(a2,i1,a3,i2,a3,i2,a6)
! 539   format(a2,i1,a3,i2,a2,i3,a6)
! 540   format(a2,i1,a3,i2,a1,i4,a6)
! 541   format(a2,i1,a2,i3,a4,i1,a6)
! 542   format(a2,i1,a2,i3,a3,i2,a6)
! 543   format(a2,i1,a2,i3,a2,i3,a6)
! 544   format(a2,i1,a2,i3,a2,i4,a6)
! 545   format(a2,i1,a1,i4,a4,i1,a6)
! 546   format(a2,i1,a1,i4,a3,i2,a6)
! 547   format(a2,i1,a1,i4,a2,i3,a6)
! 548   format(a2,i1,a1,i4,a1,i4,a6)



 513   format(a3,i1,a4,i1,a6)
 514   format(a3,i1,a3,i2,a6)
 515   format(a3,i1,a2,i3,a6)
 516   format(a3,i1,a1,i4,a6)
 517   format(a2,i2,a4,i1,a6)
 518   format(a2,i2,a3,i2,a6)
 519   format(a2,i2,a2,i3,a6)
 520   format(a2,i2,a1,i4,a6)
 521   format(a1,i3,a4,i1,a6)
 522   format(a1,i3,a3,i2,a6)
 523   format(a1,i3,a2,i3,a6)
 524   format(a1,i3,a1,i4,a6)
 525   format(i4,a4,i1,a6)
 526   format(i4,a3,i2,a6)
 527   format(i4,a2,i3,a6)
 528   format(i4,a1,i4,a6)

       return
       end subroutine gafilename

!##############################################################
!
! Subroutine mutlog:
!      Writes out the list of mutationss in the log file, this
!      is only called when lfulllog is true
!
!##############################################################

      subroutine mutlog(mut)

      implicit none

      integer(long)                              :: a,b
      integer(long), dimension(npop), intent(in) :: mut
      character(len=80)                          :: string

      string = ' '
      b = 1
      ilog = 3

      write(ilog,10) 'Mutation targets are:'
      do a=1,int(nmut*npop0) -1
        if (mut(a) .lt. 10) then
           write(string(b:b+1),11) mut(a)
           b=b+1
        else if (mut(a) .lt. 100) then
           write(string(b:b+2),12) mut(a)
           b=b+2
        else if (mut(a) .lt. 1000) then
           write(string(b:b+3),13) mut(a)
           b=b+3
        else if (mut(a) .lt. 10000) then
           write(string(b:b+4),16) mut(a)
        else
           print*,'Populations above 4998 are not supported'
        endif
        write(string(b:b+1),15) ' '
        b=b+1
        if (b .gt. 76) then
           write(ilog,14) string
           string = ' '
           b = 1
        endif
      enddo

      write(ilog,14) string

      return

 10   format(a23)
 11   format(i1)
 12   format(i2)
 13   format(i3)
 14   format(a80)
 15   format(a1)
 16   format(i4)

      end subroutine mutlog

!##############################################################
!
! Subroutine survlog:
!      Writes out the list of surviors in the log file, this
!      is only called when lfulllog is true
!
!##############################################################


      subroutine survlog(surv)

      implicit none

      integer(long)                              :: a,b
      integer(long), dimension(npop), intent(in) :: surv
      character(len=80)                          :: string

      string = ' '
      b = 1
      ilog = 3

      write(ilog,10) 'Tournament Survivors are:'
      do a=1,npop0
        if (surv(a) .lt. 10) then
           write(string(b:b+1),11) surv(a)
           b=b+1
        else if (surv(a) .lt. 100) then
           write(string(b:b+2),12) surv(a)
           b=b+2
        else if (surv(a) .lt. 1000) then
           write(string(b:b+3),13) surv(a)
           b=b+3
        else if (surv(a) .lt. 10000) then
           write(string(b:b+4),16) surv(a)
           b=b+4
        else
           print*,'Populations above 4998 are not supported'
        endif
        write(string(b:b+1),15) ' '
        b=b+1
        if (b .gt. 76) then
           write(ilog,14) string
           string = ' '
           b = 1
        endif
      enddo

      write(ilog,14) string

      return

 10   format(a27)
 11   format(i1)
 12   format(i2)
 13   format(i3)
 14   format(a80)
 15   format(a1)
 16   format(i4)

      end subroutine survlog

!##############################################################
! Subroutine loptim:
!    Performs local optimisation using the simplex algorithm
!    Effectively copied from the rout-ine fitpar above
!##############################################################

      subroutine loptim(optcoo,mr,mi,iter0)

      implicit none

      integer(long), parameter :: powdim=20

      integer(long), dimension(midim), intent(inout) :: mi
      real(dop), dimension(mrdim), intent(inout)  :: mr
      real(dop), dimension(optcoodim), intent(out):: optcoo
      real(dop), dimension(1000*1000)             :: xi
      real(dop), dimension(1000,3)                :: work
      integer(long), intent(in)                   :: iter0
      real(dop)                                   :: fret
!      real(dop), external                         :: vcrmsd
!      external                                    :: dvcrmsd

!----------------------------------------------------------------------
! save parameters from the initial hamiltonian
!----------------------------------------------------------------------
      call cpvxd(mr(mrconst),mr(mrconstini),nsta*nsta)
      call cpvxd(mr(mrkappa),mr(mrkappaini),nmodes*nsta)
      call cpvxd(mr(mrlambda),mr(mrlambdaini),nmodes*nsta*nsta)
      call cpvxd(mr(mrgamma),mr(mrgammaini),nmodes*nmodes*nsta)
      call cpvxd(mr(mrmu),mr(mrmuini),nmodes*nmodes*nsta*nsta)
      call cpvxd(mr(mrexpcoe),mr(mrexpcoeini),nmodes*nsta*maxcoe)


      iter=iter0

      ioptim=1

!      print*,'Iter = ',iter,' ioptim = ',ioptim
!      print*,'Before optim: optcoo(1) = ',optcoo(1)
      call par2vec2d(optcoo,mr(mrconst),mr(mrkappa),mr(mrlambda),&
           mr(mrgamma),mr(mriota),mr(mrmu),mr(mrexpcoe),&
           mr(mrhigher),mr(mrtermk),mr(mrcpcoe),&
           mr(mrswitch),mr(mrcp2d),mi(miconstmask),mi(mikappamask),&
           mi(milambdamask),mi(migammamask),mi(miiotamask),&
           mi(mimumask),mi(miexpcoemask),mi(mitermkmask),&
           mi(micpcoemask),mi(miswitchmask),mi(micp2d))

         fret=vcrmsd(optcoo,mr,mi)/nsets
         if (ioptim .eq. 1) then
!            print*,'CONJUGATE-GRADIENTS'
            iter=iter0
            call frprmn(optcoo,nvar,tol,iter,fret,vcrmsd,dvcrmsd,&
                 ilog,0.0_dop,0,mr,mi)

         else if (ioptim .eq. 2) then
!            print*,'POWELL'
            iter=iter0
            call unitqxd(xi,nvar)
            call powell(optcoo,xi,nvar,nvar,tol,iter,fret,vcrmsd,&
                 work(1,1),work(1,2),work(1,3),mr,mi)

         else if (ioptim .eq. 3) then
!            print*,'SIMPLEX'
            iter=iter0
            call init_amoeba(xi,work(1,1),optcoo,nvar,vcrmsd,mr,&
                 mi)
            call amoeba(xi,work(1,1),nvar,0.001_dop,vcrmsd,iter,&
                 work(1,2),work(1,3),mr,mi)
!            write(6,'(a,i5)') 'converged in Iterations: ',iter
            call cpvxd(xi,optcoo,nvar)
         endif
         call vec2par2d(optcoo,mr(mrconst),mr(mrkappa),mr(mrlambda),&
              mr(mrgamma),mr(mriota),mr(mrmu),mr(mrexpcoe),&
              mr(mrhigher),mr(mrtermk),mr(mrcpcoe),&
              mr(mrswitch),mr(mrcp2d),mi(miconstmask),mi(mikappamask),&
              mi(milambdamask),mi(migammamask),mi(miiotamask),&
              mi(mimumask),mi(miexpcoemask),mi(mitermkmask),&
              mi(micpcoemask),mi(miswitchmask),mi(micp2d))

      return
      end subroutine loptim


!##############################################################
!
! Subroutine : galog
!  Write useful statistics to the log file
!
!##############################################################

      subroutine galog(sd,sdrmsd,brmsd,bestfit,armsd)

      implicit none

      real(dop), intent(in)                     :: brmsd,armsd
      real(dop), dimension(npop0*8), intent(in) :: sdrmsd
      real(dop), intent(inout)                  :: sd
      real(dop)                                 :: cputime,totalcputime
      character(len=15), intent(in)             :: bestfit
      integer(long)                             :: i
      integer(long), save :: tid=0
      integer(long), save :: tid1=0
      integer(long), save :: tid2=0
      integer(long), save :: tid3=0
      integer(long), save :: tid4=0

      call get_timer(tid, 'mut')
      call get_timer(tid1, 'crosso')
! local optimisation timer does not seem to be used
      call get_timer(tid2, 'loptim')
      call get_timer(tid3, 'anticonv')
      call get_timer(tid4, 'tourn')

      do i=1,npop0*2
         sd=sd+((sdrmsd(i)-armsd)*(sdrmsd(i)-armsd))
      enddo
      sd=sqrt(sd/(npop0*2))

      if (iter .eq. 0) then
         gacput(:) = 0
      else
! CPU time this generation = cpu time total - cpu time last generation
         gacput(1) = timers(tid)%cputime - gacput(6)
         gacput(2) = timers(tid1)%cputime - gacput(7)
         gacput(3) = timers(tid2)%cputime - gacput(8)
         gacput(4) = timers(tid3)%cputime - gacput(9)
         gacput(5) = timers(tid4)%cputime - gacput(10)

! Records total CPU time for next generation
         gacput(6) = timers(tid)%cputime
         gacput(7) = timers(tid1)%cputime
         gacput(8) = timers(tid2)%cputime
         gacput(9) = timers(tid3)%cputime
         gacput(10) = timers(tid4)%cputime
      endif

      cputime = gacput(1)+gacput(2)+gacput(3)+gacput(4)+gacput(5)
      totalcputime = gacput(6)+gacput(7)+gacput(8)+gacput(9)+gacput(10)

      write(ilog,70) '*Summary************************************'
      write(ilog,5)  'Generation: - ',ngenga
      write(ilog,10) 'Best RMSD from this Generation = ', brmsd
      write(ilog,20) 'Best fit from the Generation = ', bestfit
      write(ilog,30) 'Average RMSD from this Generation = ', armsd
      write(ilog,40) 'Standard Deviation of RMSD = ', sd
      write(ilog,70) '*Timing*************************************'
      write(ilog,75) 'Algorithm:','Mutation','Crossover ',&
       'Local Optim.','Anti-Converg.','Tournament'
      write(ilog,76) 'CPU Time(s):',gacput(1),gacput(2),gacput(3),&
       gacput(4),gacput(5)
      write(ilog,50) 'CPU time this Generation(s) = ', cputime
      write(ilog,60) 'Total CPU time so far(s) = ', totalcputime
      write(ilog,70) '********************************************'

      cputime = 0

      return

 5    format(a14,i4)
 10   format(a33,3x,e16.8)
 20   format(a31,7x,a15)
 30   format(a36,e16.8)
 40   format(a29,7x,e16.8)
 50   format(a30,2x,e10.4)
 60   format(a27,5x,e10.4)
 70   format(a)
 75   format(a10,6x,a9,6x,a10,6x,a12,2x,a13,2x,a10)
 76   format(a12,5x,e10.4,4x,e10.4,6x,e10.4,4x,e10.4,5x,e10.4)

      end subroutine galog

!##############################################################
!      subroutine anticonv
!    A simple subroutine which reduces the number of indentical
!    population members to stop pre-convergence (increases the
!    diversity of the population dramatically)
!
!    Variables used:
!    nlocpop =           number of local populations
!    locpopexample(i) =  number (second hald of filename) of an example of localpopulation i
!    locpop(i) =         which local population member i is a member of, e.g. if pop member 6 is
!                        in population 3 locpop(6) will equal 3
!    maxlocpop =         maximum allowed members of a local population
!    llocpop =           whether the population member currently in memory is the same as population
!                        loaded into secondary arrays (kapp2,lambda2 etc.)
!    assigned =          whether the current population member under evaulation has been assigned to
!                        a local population
!##############################################################

      subroutine anticonv(kappa,lambda,gamma,mu,iota,termk,kappa2,lambda2,gamma2,mu2,&
         iota2,termk2,constmask,kappamask,lambdamask,gammamask,mumask,iotamask,termkmask,&
         const,expcoe,expcoemask,higher,cpcoe,nanticonv,pop_rmsd)

      implicit none

      real(dop), dimension(nadd)                                :: termk,termk2
      real(dop), dimension(nsta,nsta)                           :: const
      real(dop), dimension(nhigher)                             :: higher
      real(dop), dimension(nmodes,nsta), intent(out)            :: kappa,kappa2
      real(dop), dimension(nmodes,nsta,nsta), intent(out)       :: lambda,lambda2
      real(dop), dimension(nmodes,nmodes,nsta), intent(out)     :: gamma,gamma2
      real(dop), dimension(maxcoe,nmodes,nsta)                  :: expcoe,expcoe2
      real(dop), dimension(nmodes,nmodes,nsta,nsta), intent(out):: mu,mu2
      real(dop), dimension(nmodes,nmodes,nsta,nsta)             :: iota,iota2
      real(dop), dimension(maxcoe,nmodes,nsta,nsta)             :: cpcoe

      integer(long), dimension(nsta,nsta), intent(in)               :: constmask
      integer(long), dimension(nmodes,nsta), intent(in)             :: kappamask
      integer(long), dimension(nmodes,nsta,nsta), intent(in)        :: lambdamask
      integer(long), dimension(nmodes,nmodes,nsta), intent(in)      :: gammamask
      integer(long), dimension(maxcoe,nmodes,nsta)                  :: expcoemask
      integer(long), dimension(nmodes,nmodes,nsta,nsta)             :: iotamask,mumask
      integer(long), dimension(nadd), intent(in)                    :: termkmask

      real(dop), dimension(10000),intent(inout)          :: pop_rmsd

      integer(long)                        :: i,j,k,npopcomp,temp,nlocpop,l
      integer                              :: dead
      integer(long), dimension(npop)       :: locpopexample
      integer(long)                        :: maxlocpop,nreplaced,nanticonv
      logical(kind=4)                      :: llocpop,assigned,ltemp
      integer(long), dimension(npop)       :: locpop
      character(len=80)                    :: filename
      character(len=39)                    :: cdum

      cdum=' '
      ilog = 3
      iout = 13
      llocpop=.false.
      nlocpop=0
      maxlocpop=10
      nreplaced=0
      filename=''
      dead = 0
!NOTE: this first part of the code simply matches identical copies of
! participants and then mutates all but one. this helps Ga not to waste time
! the second part is jon doing the same but more time consuming; however,
! it has the advantage that it gives information about "local populations"
! this sort of statistical info sounds usefull, so im allowing for this to happen
! every 40 generations (it shows how many local populations are formed in 15 generations,
! see "call anticonv")
! viva la revolucion!
      if(nanticonv .lt. 40) then
      do i=2,npop0
        call gafilename(ngenga,i,filename)
        open(iguess,file=filename,form='formatted',status='old')
         call readgapar(iguess,kappa,lambda,gamma,iota,mu,expcoe,termk)
        close(iguess)
        do j=1,i-1
          call gafilename(ngenga,j,filename)
          call readgapar(iguess,kappa2,lambda2,gamma2,iota2,mu2,expcoe2,termk2)
                close(iguess)
          call locpopcompare(kappa,lambda,gamma,mu,iota,termk,kappa2,&
                lambda2,gamma2,mu2,iota2,termk2,kappamask,lambdamask,&
                gammamask,mumask,iotamask,termkmask,llocpop)
          if (llocpop .eqv. .true.) then
              ltemp = covar_adapt
              covar_adapt = .false.
              do l = 1, 2
                  call mutate(kappa,lambda,gamma,mu,&
                  iota,expcoe,termk,kappamask,lambdamask,&
                  gammamask,mumask,iotamask,expcoemask,termkmask,0)
              enddo
              pop_rmsd(i)=vcrmsd(optcoo,mr,mi)
              pop_rmsd(i)=sqrt(pop_rmsd(temp)/nsets)
              covar_adapt = ltemp
              call gafilename(ngenga,i,filename)
              open(iout,file=filename,form='formatted',status='old')
               call gaout(iout,kappa,lambda,gamma,iota,mu,expcoe,termk,&
                    kappamask,lambdamask,gammamask,iotamask,mumask,&
                   expcoemask,termkmask,.false.,cdum)
              dead = dead + 1
              exit
          endif
        enddo
      enddo
      write(6,'(I2,A35)') dead, ' copycats mutated!'
      else
!   Compare each member of the population with the rest of the population
!   Do this by loading both, one into kappa,lambda etc the other into
!   kappa2,lambda2 etc.

!   the locpopcompare will return a result of true if both are the same
      do i=2,npop0
        assigned = .false.
        npopcomp = npop0-i
! first load population member to compare to (i)
        temp=i
        call gafilename(ngenga,temp,filename)
!       print*,'DEBUG: Filename for guess: ',filename
        open(iguess,file=filename,form='formatted',status='old')
        call readgapar(iguess,kappa,lambda,gamma,iota,mu,expcoe,termk)
        close(iguess)
! now compare to each member of the population (with higher number than itself to avoid repetition)
        do j=1,npopcomp
          if (assigned .eqv. .false.) then
           temp=i+j
           call gafilename(ngenga,temp,filename)
!          print*,'DEBUG: Filename for guess: ',filename

           open(iguess,file=filename,form='formatted',status='old')
           call readgapar(iguess,kappa2,lambda2,gamma2,iota2,mu2,expcoe2,termk2)
           close(iguess)
           call locpopcompare(kappa,lambda,gamma,mu,iota,termk,kappa2,&
                lambda2,gamma2,mu2,iota2,termk2,kappamask,lambdamask,&
                gammamask,mumask,iotamask,termkmask,llocpop)

!    If llocpop is true it will then compare to all (if any) previous identified
!    local populations and label it as a member of a local population already found
!    or create a new local population and add it as an example of the local population

           if (llocpop .eqv. .true.) then
              if (nlocpop .eq. 0) then
                 nlocpop=nlocpop+1
                 locpop(i)=nlocpop
!
                 locpopexample(nlocpop)=i
                 assigned=.true.
              else
                 do k=1,nlocpop
                    temp=locpopexample(k)
                    call gafilename(ngenga,temp,filename)
!          print*,'DEBUG: Filename for guess: ',filename
                    open(iguess,file=filename,form='formatted',status='old')
                    call readgapar(iguess,kappa,lambda,gamma,iota,mu,expcoe,termk)
                    close(iguess)
                    call locpopcompare(kappa,lambda,gamma,mu,iota,termk,kappa2,&
                         lambda2,gamma2,mu2,iota2,termk2,kappamask,lambdamask,&
                         gammamask,mumask,iotamask,termkmask,llocpop)
                    if (llocpop .eqv. .true.) then
                       locpop(i) = k
                       assigned = .true.
                    endif
                 enddo
              endif
           endif
           if (assigned .eqv. .false.) then
              nlocpop=nlocpop+1
              locpop(i)=nlocpop
              locpopexample(nlocpop)=i
              assigned=.true.
           endif
         endif
        enddo
      enddo


!    if there are any local populations this reduces the number in the local population
!    to the maximum defined in maxlocpop.  Any guesses removed are replaced by a new
!    randomly generated guess (same procedure as in initpop)

      if (nlocpop .gt. 0) then
         do i=1,nlocpop
           k=0
           do j=1,npop0
              if (locpop(j) .eq. i) then
                 k=k+1
                 if (k .gt. maxlocpop) then
                    call gafilename(ngenga,j,filename)

                  if (offdiagsignopt .eqv. .false.) then
                    open(iout,file=filename,form='formatted',status='old')
                    call newguess(kappa,lambda,gamma,iota,mu,expcoe,&
                                  termk,kappamask,lambdamask,gammamask,&
                                  iotamask,mumask,expcoemask,termkmask )
                     call gaout(iout,kappa,lambda,gamma,iota,mu,expcoe,termk,&
                         kappamask,lambdamask,gammamask,iotamask,mumask,&
                         expcoemask,termkmask,.false.,cdum)
                     pop_rmsd(j)=vcrmsd(optcoo,mr,mi)
                     pop_rmsd(j)=sqrt(pop_rmsd(temp)/nsets)
                  else
!       if we are just mutating signs then take the local pop and mutate the
!       hell out of it! (to make our "new" participants)
                     open(iguess,file=filename,form='formatted',status='old')
                     call readgapar(iguess,kappa,lambda,gamma,iota,mu,expcoe,termk)
                     ltemp = covar_adapt
                     covar_adapt = .false.
                     do l = 1, 2
                         call mutate(kappa,lambda,gamma,mu,&
                         iota,expcoe,termk,kappamask,lambdamask,&
                         gammamask,mumask,iotamask,expcoemask,termkmask,0)
                         close(iguess)
                     enddo
                     pop_rmsd(j)=vcrmsd(optcoo,mr,mi)
                     pop_rmsd(j)=sqrt(pop_rmsd(temp)/nsets)
                     covar_adapt = ltemp
                     open(iout,file=filename,form='formatted',status='old')
                     call gaout(iout,kappa,lambda,gamma,iota,mu,expcoe,termk,&
                         kappamask,lambdamask,gammamask,iotamask,mumask,&
                         expcoemask,termkmask,.false.,cdum)
                  endif
                  close(iout)
                  nreplaced=nreplaced+1
                 endif
              endif
            enddo
         enddo
      write(ilog,'(a,i2)') 'Number of local populations = ',nlocpop
      endif
!      print*,'Number of guesses replaced = ',nreplaced
      endif
      write(ilog,'(a,i2)') 'Number of guesses replaced = ',nreplaced
      end subroutine anticonv

!##############################################################
!      subroutine locpopcompar
!   A routine which compares to loaded sets of comparisons and
!   returns whether they are the same.  It does this by comparing
!   value and if they are not the same (difference less than
!   cutoff(set below)) changinging llocpop to false
!
!    Variables used:
!    nlocpop =           number of local populations
!    locpopexample(i) =  number (second half of filename) of an example of localpopulation i
!    locpop(i) =         which local population member i is a member of, e.g. if pop member 6 is
!                        in population 3 locpop(6) will equal 3
!    maxlocpop =         maximum allowed members of a local population
!    llocpop =           whether the population member currently in memory is the same as population
!                        loaded into secondary arrays (kapp2,lambda2 etc.)
!    assigned =          whether the current population member under evaulation has been assigned to
!                        a local population
         !##############################################################


      subroutine locpopcompare(kappa,lambda,gamma,mu,iota,termk,&
            kappa2,lambda2,gamma2,mu2,iota2,termk2,kappamask,&
            lambdamask,gammamask,mumask,iotamask,termkmask,llocpop)

      implicit none

      real(dop), dimension(nmodes,nsta), intent(in)                 :: kappa,kappa2
      real(dop), dimension(nmodes,nsta,nsta), intent(in)            :: lambda,lambda2
      real(dop), dimension(nmodes,nmodes,nsta), intent(in)          :: gamma,gamma2
      real(dop), dimension(nmodes,nmodes,nsta,nsta), intent(in)     :: iota,iota2,mu,mu2
      real(dop), dimension(nadd), intent(in)                        :: termk,termk2

      integer(long), dimension(nmodes,nsta), intent(in)             :: kappamask
      integer(long), dimension(nmodes,nsta,nsta), intent(in)        :: lambdamask
      integer(long), dimension(nmodes,nmodes,nsta), intent(in)      :: gammamask
      integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: iotamask,mumask
      integer(long), dimension(nadd), intent(in)                    :: termkmask

      logical(kind=4), intent(out) :: llocpop
      real(dop)                    :: diff,cutoff
      integer(long)                :: s,s1,i,j

      llocpop=.true.
      cutoff=0.005

       do s=1,nsta
          do i=1,nmodes
             if (kappamask(i,s) .eq. 1) then
                 diff=kappa(i,s)-kappa2(i,s)
                 diff=abs(diff)
                 if (diff .gt. cutoff) then
                     llocpop=.false.
                 endif
             endif
          enddo
       enddo

       if (.not. llocpop) goto 10

       do s=1,nsta-1
          do s1=s+1,nsta
             do i=1,nmodes
             if (lambdamask(i,s,s1) .eq. 1) then
                 diff=lambda(i,s,s1)-lambda2(i,s,s1)
                 diff=abs(diff)
                 if (diff .gt. cutoff) then
                     llocpop=.false.
                 endif
             endif
             enddo
          enddo
       enddo

       if (.not. llocpop) goto 10

       do s=1,nsta
          do i=1,nmodes
             do j=1,nmodes
              if (gammamask(i,j,s) .eq. 1) then
                  diff=gamma(i,j,s)-gamma2(i,j,s)
                  diff=abs(diff)
                  if (diff .gt. cutoff) then
                      llocpop=.false.
                  endif
              endif
             enddo
          enddo
       enddo

      if (.not. llocpop) goto 10

       do s=1,nsta-1
          do s1=s+1,nsta
             do i=1,nmodes
                do j=1,nmodes
                if (mumask(j,i,s,s1) .eq. 1) then
                    diff=mu(i,j,s,s1)-mu2(i,j,s,s1)
                    diff=abs(diff)
                    if (diff .gt. cutoff) then
                        llocpop=.false.
                    endif
                endif
                enddo
             enddo
          enddo
       enddo

       if (.not. llocpop) goto 10

       do s=1,nsta
          do s1=s,nsta
             do i=1,nmodes
                do j=1,nmodes
                if (iotamask(i,j,s,s1) .eq. 1) then
                    diff=iota(i,j,s,s1)-iota2(i,j,s,s1)
                    diff=abs(diff)
                    if (diff .gt. cutoff) then
                        llocpop=.false.
                    endif
                endif
                enddo
             enddo
          enddo
       enddo


       if (.not. llocpop) goto 10

       do i = 1, nadd
         if( termk(i) .eq. 1) then
           diff=termk(i)- termk2(i)
           if( diff .gt. cutoff) then
             llocpop=.false.
           endif
         endif
       enddo

 10    continue
       return
       end subroutine locpopcompare

!###############################################################
!  Subroutine newguess
!       Generates a new guess at random.  Copied from old
!       initpop routine, and then rewrote initpop routine to use
!       this subroutine to avoid duplication of efforts.
!
!##############################################################

       subroutine newguess(kappa,lambda,gamma,iota,mu,expcoe,&
            termk,kappamask,lambdamask,gammamask,&
            iotamask,mumask,expcoemask,termkmask)

         implicit none

         real(dop), dimension(nadd), intent(inout)                 :: termk
         real(dop), dimension(nmodes,nsta), intent(out)            :: kappa
         real(dop), dimension(nmodes,nsta,nsta), intent(out)       :: lambda
         real(dop), dimension(nmodes,nmodes,nsta), intent(out)     :: gamma
         real(dop), dimension(maxcoe,nmodes,nsta), intent(out)     :: expcoe
         real(dop), dimension(nmodes,nmodes,nsta,nsta), intent(out):: iota,mu

         integer(long), dimension(nmodes,nsta), intent(in)             :: kappamask
         integer(long), dimension(nmodes,nsta,nsta), intent(in)        :: lambdamask
         integer(long), dimension(nmodes,nmodes,nsta), intent(in)      :: gammamask
         integer(long), dimension(maxcoe,nmodes,nsta), intent(in)   :: expcoemask
         integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: iotamask,mumask
         integer(long), dimension(nadd), intent(in)                    :: termkmask

         integer(long)       :: i,j,s,s1,tot_order,k,m
         real(dop)           :: prod_freq0



         prod_freq0 = 1
         tot_order = 0
         do s=1,nsta
            do i=1,nmodes
               if (kappamask(i,s) .eq. 1) then
!                  kappa(i,s)=&
!                       mutmag(0.0d0,kappamut(1),kappamut(2))
!                  kappa(i,s) = mutdis(0.0d0,kappamut(1))
                  kappa(i,s) = 2*(drandom(seed)-0.5)*kappamut(1)
               elseif (lzerozero .eqv. .true.) then
                  kappa(i,s)=0
               endif
            enddo
         enddo

         do s=1,nsta-1
            do s1=s+1,nsta
               do i=1,nmodes
                  if (lambdamask(i,s,s1) .eq. 1) then

!		      lambda(i,s,s1)= mutdis(0.0d0,lambdamut(1))
                      lambda(i,s,s1) = 2*(drandom(seed)-0.5)*lambdamut(1)
!                     lambda(i,s,s1)=&
!                          mutmag(0.0d0,lambdamut(1),lambdamut(2))
                     lambda(i,s1,s) = lambda(i,s,s1)

!                    print*, 'LAMBDA ', lambda(i,s,s1)
                  elseif (lzerozero .eqv. .true.) then
                     lambda(i,s,s1)=0
                  endif
               enddo
            enddo
         enddo
         do s=1,nsta
            do i=1,nmodes
               do j=i,nmodes
                  if(gammamask(i,j,s) .eq. 1 )then
!                     gamma(i,j,s)=  mutdis(0.0d0,gammamut(1))
                     gamma(i,j,s)  = 2*(drandom(seed)-0.5)*gammamut(1)
!                     gamma(i,j,s)=&
!                          mutmag(0.0d0,gammamut(1),gammamut(2))
                     gamma(j,i,s) = gamma(i,j,s)
                  elseif (lzerozero .eqv. .true.)then
                     gamma(i,i,s)=0
                  endif
               enddo
            enddo
         enddo
         do s=1,nsta-1
            do s1=s+1,nsta
               do i=1,nmodes
                  do j=i,nmodes
                     if (mumask(j,i,s,s1) .eq. 1) then
!                         print*, 'mode mode state state'
!                          print*, i, j, s, s1
!			 mu(j,i,s,s1)= mutdis(0.0d0,mumut(1) )
                         mu(j,i,s,s1) = 2*(drandom(seed)-0.5)*mumut(1)
                         mu(j,i,s1,s)= mu(j,i,s,s1)
                         mu(i,j,s1,s)= mu(j,i,s,s1)
                         mu(i,j,s,s1)= mu(j,i,s,s1)
!                        mu(j,i,s,s1)=&
!                             mutmag( 0.0d0 ,mumut(1),mumut(2))
!                        print*, mu(j,i,s,s1)
                     elseif (lzerozero .eqv. .true.) then
                        mu(j,i,s,s1)=0
                     endif
                  enddo
               enddo
            enddo
         enddo
         do s=1,nsta
            do s1=s,nsta
               do i=1,nmodes
                  do j=1,nmodes
                     if (iotamask(i,j,s,s1) .eq. 1) then
!			 iota(i,j,s,s1)= mutdis(0.0d0, iotamut(1) )
                         iota(i,j,s,s1) = 2*(drandom(seed)-0.5)*iotamut(1)
                         iota(j,i,s1,s)= iota(j,i,s,s1)
                         iota(i,j,s1,s)= iota(j,i,s,s1)
                         iota(i,j,s,s1)= iota(j,i,s,s1)
!                        iota(i,j,s,s1)=&
!                             mutmag( 0.0d0 ,iotamut(1),iotamut(2))
                     elseif (lzerozero .eqv. .true.)then
                        iota(i,j,s,s1)=0
                     endif
                  enddo
               enddo
            enddo
         enddo

!!       termk (explicitly added terms)
!!       added by c.r aka deadly mexican
         do i = 1, nadd
            if (termkmask(i) .eq. 1) then
               tot_order = 0
               do j = 1, nmodes
                  if( termdef(j,i) .gt. 0) then
                     tot_order = tot_order + termdef(j,i)
                  endif
               enddo
!               termk(i) = &
!                    mutmag(0.0d0,termkmut(1),termkmut(2) )
!               termk(i) =  mutdis(0.0d0, termkmut(1) )
               termk(i) = 2*(drandom(seed)-0.5)*termkmut(1)
               termk(i) = termk(i)/ factorial(tot_order)
!               print*, 'TERMK + ', termk(i)
            elseif (termkmask(i) .eq. 2) then
!             print*, 'NOT ALLWED BY SYM APPARENTLY i = ', i
!             print*, 'modes =', termdef(1:7,i), 'sta=', termst(1,i), termst(2,i)
            elseif (lzerozero .eqv. .true.) then
               termk(i) = 0
               print*, 'TERMK ZERO'
            endif
         enddo

! General diabatic potential functions
         do s=1,nsta
            do m=1,nmodes
               if (ncoeff(m,s).gt.0) then
                  do k=1,ncoeff(m,s)
                     if (expcoemut(1,k,m,s).eq.-999.0) then
                        write(6,'(/,a,x,i2,x,a,/)') 'The diabatic potential function type',&
                             ifdiab(m,s),'is not yet supported for use with the genetic algorithm.'
                        STOP
                     endif
!                    if( m .eq. 56) print*, 'values = k = ',k, expcoe(k,m,s)
                     if (expcoemask(k,m,s).eq.1) then
!                          expcoe(k,m,s)=&
!                          mutmag(0.0d0,expcoemut(1,k,m,s),expcoemut(2,k,m,s))
                            expcoe(k,m,s) = 2*(drandom(seed)-0.5)*expcoemut(1,k,m,s)
                        if(ifdiab(m,s) .eq. 18 .and. k .eq. 6 ) expcoe(k,m,s) = abs(expcoe(k,m,s))
                        if(ifdiab(m,s) .eq. 18 .and. k .eq. 3 ) expcoe(k,m,s) = abs(expcoe(k,m,s))
                        if(ifdiab(m,s) .eq. 2 .and. k .eq. 1 ) expcoe(k,m,s) = abs(expcoe(k,m,s))
                        if(ifdiab(m,s) .eq. 4 .and. k .eq. 1 ) expcoe(k,m,s) = abs(expcoe(k,m,s))
                        if(ifdiab(m,s) .eq. 2 .and. k .eq. 3 .and. expcoe(k,m,s) .eq. 0 )&
                         expcoe(k,m,s) = abs(expcoe(k,m,s)) + 0.00001
                        if(ifdiab(m,s) .eq. 4 .and. k .eq. 3 .and. expcoe(k,m,s) .eq. 0 )&
                         expcoe(k,m,s) = abs(expcoe(k,m,s)) + 0.00001
!                        print*, expcoe(k,m,s)
                     elseif( lzerozero .eqv. .true.) then
                         expcoe(k,m,s) = 0
                        if(ifdiab(m,s) .eq. 2 .and. k .eq. 3 .and. expcoe(k,m,s) .eq. 0 )&
                         expcoe(k,m,s) = 0.00001
                        if(ifdiab(m,s) .eq. 4 .and. k .eq. 3 .and. expcoe(k,m,s) .eq. 0 )&
                         expcoe(k,m,s) = 0.00001

                     endif
                  enddo
               endif
            enddo
         enddo


         return

       end subroutine newguess

!#######################################################################

       subroutine get_par_stat(kappa,lambda,gamma,mu,iota,expcoe,termk,&
            kappamask,lambdamask,gammamask,mumask,iotamask,expcoemask,&
            termkmask,mr)

         implicit none

         real(dop), dimension(mrdim), intent(inout)                   :: mr
         real(dop), dimension(nmodes,nsta), intent(inout)             :: kappa
         real(dop), dimension(nmodes,nsta,nsta), intent(inout)        :: lambda
         real(dop), dimension(nmodes,nmodes,nsta), intent(inout)      :: gamma
         real(dop), dimension(maxcoe,nmodes,nsta), intent(inout)      :: expcoe
         real(dop), dimension(nadd), intent(inout)                    :: termk
         real(dop), dimension(nmodes,nmodes,nsta,nsta), intent(inout) :: iota,mu


         integer(long), dimension(nmodes,nsta), intent(in)             :: kappamask
         integer(long), dimension(nmodes,nsta,nsta), intent(in)        :: lambdamask
         integer(long), dimension(nmodes,nmodes,nsta), intent(in)      :: gammamask
         integer(long), dimension(maxcoe,nmodes,nsta), intent(inout)   :: expcoemask
         integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: iotamask,mumask
         integer(long), dimension(nadd), intent(in)                    :: termkmask
         integer(long)       :: i,j,s,s1,c,k,x

         character(len=300)                         :: string,frmt
         character(len=30)                          :: ctemp, cmean,cstdev,ckurt,&
                                                       cskew,cdptest
         character(len=19)                          :: filename
         real(dop), dimension(300,5000)             :: temp
         real(dop)                                  :: avg_kurt, avg_std,rvar1,rvar2


         do i=1,300
            param_mean(i)= 0.0
            st_dev(i)    = 0.0
            kurtosis(i)  = 0.0
            skew(i)      = 0.0
            DPtest(i)    = 0.0
           do j =1, 5000
             temp(i,j) =0.0
           enddo
         enddo
!	THE NAMES ARE ONLY RECORDED ONCE AT THE BEGIGNIGN:
         if( ngenga .eq. 0) then
          x = 0
           do s=1,nsta
              do i=1,nmodes
                 if (kappamask(i,s) .eq. 1) then
                    x = x + 1
                    write(ctemp,'(2I2)') i, s
                    pname(x) = 'kappa'//trim(ctemp)
                 endif
              enddo
           enddo
           do s=1,nsta-1
              do s1=s+1,nsta
                 do i=1,nmodes
                    if (lambdamask(i,s,s1) .eq. 1) then
                       x = x + 1
                       write(ctemp,'(3I2)') i, s, s1
                       pname(x) = 'lambda'//trim(ctemp)
                    endif
                 enddo
              enddo
           enddo
           do s=1,nsta
              do i=1,nmodes
                 do j=i,nmodes
                    if (gammamask(i,j,s) .eq. 1) then
                       x = x + 1
                       write(ctemp,'(3I2)') i, j, s
                       pname(x) = 'gamma'//trim(ctemp)
                    endif
                 enddo
              enddo
           enddo
           do i = 1, nadd
              if (termkmask(i) .eq. 1) then
                 x = x + 1
                 write(ctemp,'(1I2)') i
!       not bothering with the termdef...bit too clunky
                 pname(x) = 'termk'//trim(ctemp)//' '//trim(charit(termst(1,i),0))//&
                 trim(charit(termst(2,i),0))
              endif
           enddo
           do s=1,nsta-1
              do s1=s+1,nsta
                 do i=1,nmodes
                    do j=i,nmodes
                       if (mumask(j,i,s,s1) .eq. 1) then
                          x = x + 1
                          write(ctemp,'(4I2)') i, j, s, s1
                          pname(x) = 'mu'//trim(ctemp)
                       endif
                    enddo
                 enddo
              enddo
           enddo

           do s=1,nsta
              do s1=s,nsta
                 do i=1,nmodes
                    do j=i,nmodes
                       if (iotamask(j,i,s,s1) .eq. 1) then
                          x = x + 1
                          write(ctemp,'(4I2)') j, i, s, s1
                          pname(x) = 'iota'//trim(ctemp)
                       endif
                    enddo
                 enddo
              enddo
           enddo

           do s=1,nsta
              do i=1,nmodes
                 if (ncoeff(i,s).gt.0) then
                    do k=1,ncoeff(i,s)
                       if(expcoemask(k,i,s).eq.1) then
                          x=x+1
                          write(ctemp,'(4I2)') k, i, s
                          pname(x) = 'exp'//trim(ctemp)
                       endif
                    enddo
                 endif
              enddo
           enddo
        endif

         do c=1,npop0

            call gafilename(ngenga,c,filename)
            open(iguess,file=trim(adjustl(filename)),form='formatted',status='old')
            call readgapar(iguess,mr(mrkappa),mr(mrlambda),mr(mrgamma),mr(mriota),mr(mrmu)&
                         ,mr(mrexpcoe),mr(mrtermk) )
            close(iguess)
            x = 0
            do s=1,nsta
               do i=1,nmodes
                  if (kappamask(i,s) .eq. 1) then
                     x = x + 1
                     if(abs(kappa(i,s)) .lt. kappamut(1)*1.5)then
                       temp(x,c) = kappa(i,s)
                     else
                       temp(x,c) = kappamut(1)*1.5*(kappa(i,s)/abs(kappa(i,s)))
                     endif
                     param_mean(x) = param_mean(x) + temp(x,c)
                     if(c .eq. 1 ) temp(x,5000) = kappamut(1)
                  endif
               enddo
            enddo
            do s=1,nsta-1
               do s1=s+1,nsta
                  do i=1,nmodes
                     if (lambdamask(i,s,s1) .eq. 1) then
                        x = x + 1
                        if(abs(lambda(i,s,s1)) .lt. lambdamut(1)*1.5)then
                          temp(x,c) = lambda(i,s,s1)
                        else
                          temp(x,c) = lambdamut(1)*1.5*(lambda(i,s,s1)/abs(lambda(i,s,s1)))
                        endif
                        param_mean(x) = param_mean(x) + temp(x,c)
                        if(c .eq. 1 ) temp(x,5000) = lambdamut(1)
                     endif
                  enddo
               enddo
            enddo

            do s=1,nsta
               do i=1,nmodes
                  do j=i,nmodes
                     if (gammamask(i,j,s) .eq. 1 ) then
                        x = x + 1
                        if(abs(gamma(i,j,s)) .lt. gammamut(1)*1.5) then
                          temp(x,c) = gamma(i,j,s)
                        else
                          temp(x,c) =  gammamut(1)*1.5*(gamma(i,j,s)/abs(gamma(i,j,s)))
                        endif
                        param_mean(x) = param_mean(x) + temp(x,c)
                        if(c .eq. 1 ) temp(x,5000) = gammamut(1)
                     endif
                  enddo
               enddo
            enddo

            do i = 1, nadd
               if (termkmask(i) .eq. 1 ) then
                  x = x + 1
                  if( abs(termk(i)) .lt. termkmut(1)*1.5 )then
                    temp(x,c) = termk(i)
                  else
                    temp(x,c) = termkmut(1)*1.5*(termk(i)/abs(termk(i)))
                  endif
                  param_mean(x) = param_mean(x) + temp(x,c)
                  if(c .eq. 1 ) temp(x,5000) = termkmut(1)
               endif
            enddo

            do s=1,nsta-1
               do s1=s+1,nsta
                  do i=1,nmodes
                     do j=i,nmodes
                        if (mumask(j,i,s,s1) .eq. 1 ) then
                           x = x + 1
                           if( abs(mu(j,i,s,s1)) .lt. mumut(1)*1.5) then
                             temp(x,c) = mu(j,i,s,s1)
                           else
                             temp(x,c) = mumut(1)*1.5*(mu(j,i,s,s1)/abs(mu(j,i,s,s1)))
                           endif
                           param_mean(x) = param_mean(x) + temp(x,c)
                           if(c .eq. 1 ) temp(x,5000) = mumut(1)
                        endif
                     enddo
                  enddo
               enddo
            enddo

            do s=1,nsta
               do s1=s,nsta
                  do i=1,nmodes
                     do j=i,nmodes
                        if (iotamask(j,i,s,s1) .eq. 1) then
                           x = x + 1
                           if( abs(iota(j,i,s,s1)) .lt. iotamut(1)*1.5 ) then
                              temp(x,c) = iota(j,i,s,s1)
                           else
                              temp(x,c) = iotamut(1)*1.5*(iota(j,i,s,s1)/abs(iota(j,i,s,s1)))
                           endif
                           param_mean(x) = param_mean(x) + temp(x,c)
                           if(c .eq. 1 ) temp(x,5000) = iotamut(1)
                        endif
                     enddo
                  enddo
               enddo
            enddo
!            print*, 'GENERATIO = ', ngenga, 'expoce = ',  expcoe(2,7,1), expcoemut(1,2,7,1)
            do s=1,nsta
               do i=1,nmodes
                  if (ncoeff(i,s).ne.0) then
                     do k=1,ncoeff(i,s)
                        if (expcoemask(k,i,s).eq.1) then
                           x=x+1
                           if( abs(expcoe(k,i,s)) .lt. expcoemut(1,k,i,s)*1.5) then
                             temp(x,c) = expcoe(k,i,s)
                           else
                             temp(x,c) =  expcoemut(1,k,i,s)*1.5*( expcoe(k,i,s)/abs(expcoe(k,i,s)))
                           endif
                           param_mean(x)=param_mean(x)+ temp(x,c)
                           if(c .eq. 1 ) temp(x,5000) = expcoemut(1,k,i,s)
                        endif
                     enddo
                  endif
               enddo
            enddo

         enddo

         avg_kurt = 0.0
         do i = 1, x
            param_mean(i) = param_mean(i)/npop0
         enddo
         do c = 1, npop0
           do i = 1, x
              st_dev(i)   = st_dev(i) + (temp(i,c) - param_mean(i) )**2
              skew(i)     = skew(i) + (temp(i,c) - param_mean(i) )**3
              kurtosis(i) = kurtosis(i) + (temp(i,c) - param_mean(i) )**4
           enddo
         enddo
         do i = 1, x
           st_dev(i)   = sqrt(st_dev(i)/dble(npop0))
           skew(i)     = (skew(i) / ( (dble(npop0)-1)*(st_dev(i))**3 ) )
           kurtosis(i) = (kurtosis(i)/( (dble(npop0)-1)*(st_dev(i))**4 ) )-3
           DPtest(i)   = skew(i)**2 + kurtosis(i)**2
           avg_kurt = avg_kurt + kurtosis(i)
           avg_std = avg_std + st_dev(i)/temp(i,5000)
         enddo
        avg_kurt = avg_kurt/x
        avg_std = avg_std/x
        print*, 'average kurtosis = ', avg_kurt
        print*, 'average std = ', avg_std
        kurtosis(300) = avg_kurt
        write(ilog,*) 'average kurtosis', avg_kurt
        write(ilog,*) 'average std', avg_std, 'increase_mut=', increase_mut


        if( covar_adapt .and. ngenga .gt. 5) then
!       return nmut to original:
         nmut = nmut/(1+increase_mut)
         rvar1 = 2*0.99-1
!        inverse tanh:
         rvar2 = 0.5*log( (1+rvar1)/(1-rvar1) )/(-0.05)
!        if standard deviation is ~0.05 of the user given maximumrange of the parameter (eg kappamut(1) )
!        then increase the number of mutants!
!        tanh function platooing at ~0 and ~0.05 (centred at 0.05)
         increase_mut = 0.5*(tanh(rvar2*(avg_std-0.05)) + 1)
!        set nmut approprialey
         nmut = nmut + nmut*increase_mut
        endif

        if( ngenga .eq. 0) then
         call system(' echo " Simply grep the parameter you are interested to see Stat history  " > Generation_stats.dat')
          write(string,'(A9,A9,9X,A10,9X,A9,9X,A9,9X,A9,9X,A9,9X,A40 )') ' echo  " ','      NAME     ','GENERATION',&
          '  MEAN   ',' ST.DEV  ','SKEWNESS ','KURTOSIS ',' DP-TEST" >> Generation_stats.dat '
          call system(string)
        endif
!        if( mod(ngenga,2) .eq. 0) then
!          call system(' cd Pop/ ; seeparams.py  ; mkdir Gen'//trim(adjustl(charit(ngenga,0)))&
!           //' ; mv *dat Gen'//trim(adjustl(charit(ngenga,0))))
!          do i = 1, npop0/20
!            j = i*20
!            call gafilename(ngenga,j,filename)
!            call file_copy(trim(adjustl(filename(1:20))), 'Pop/Gen'&
!            //trim(adjustl(charit(ngenga,0)))//'/'//trim(adjustl(filename(5:20))) )
!          enddo
!        endif
        do i =1,x
         write( cmean,'(F11.7)') param_mean(i)
         write( cstdev,'(F11.7)') st_dev(i)
         write( ckurt,'(F11.7)') kurtosis(i)
         write( cskew,'(F11.7)') skew(i)
         write( cdptest,'(F11.7)') DPtest(i)
         if(ngenga .eq. 0) then
           frmt = '(A9,I3,2X,A14,9X,A6,9X,A9,9X,A9,9X,A9,9X,A9,9X,A9,A26 )'
           write(string,frmt) ' echo  " ',i, pname(i),charit(ngenga,0),&
           trim(cmean), trim(cstdev),trim(cskew),trim(ckurt),trim(cdptest), ' " >> Generation_stats.dat'
         else
           frmt = '(A9,A14,9X,A6,9X,A9,9X,A9,9X,A9,9X,A9,9X,A9,A26 )'
           write(string,frmt) ' echo  " ', pname(i),charit(ngenga,0),trim(cmean), trim(cstdev),&
           trim(cskew),trim(ckurt),trim(cdptest), ' " >> Generation_stats.dat'
         endif
         call system(trim(string))
        enddo


        return
      end subroutine get_par_stat

!#######################################################################
      subroutine file_copy(old,new)
         character(len=30), intent(in)        :: old, new
         character(len=50) :: ch
         open(33,file=old,form='formatted',status='old')
         open(22,file=new,form='formatted',status='new')
         do
            read (33,'(A)',end=101) ch
            write (22,'(A)') ch
         end do
101      continue
         close(33)
         close(22)
      end subroutine

!#######################################################################

      subroutine get_par_covar(kappa,lambda,gamma,mu,iota,expcoe,termk,kappamask,&
           lambdamask,gammamask,mumask,iotamask,expcoemask,termkmask,mr)

        implicit none

        real(dop), dimension(mrdim), intent(inout)                   :: mr
        real(dop), dimension(nmodes,nsta), intent(inout)             :: kappa
        real(dop), dimension(nmodes,nsta,nsta), intent(inout)        :: lambda
        real(dop), dimension(nmodes,nmodes,nsta), intent(inout)      :: gamma
        real(dop), dimension(maxcoe,nmodes,nsta), intent(inout)      :: expcoe
        real(dop), dimension(nadd), intent(inout)                    :: termk
        real(dop), dimension(nmodes,nmodes,nsta,nsta), intent(inout) :: iota,mu

        integer(long), dimension(nmodes,nsta), intent(in)             :: kappamask
        integer(long), dimension(nmodes,nsta,nsta), intent(in)        :: lambdamask
        integer(long), dimension(nmodes,nmodes,nsta), intent(in)      :: gammamask
        integer(long), dimension(maxcoe,nmodes,nsta), intent(inout)   :: expcoemask
        integer(long), dimension(nmodes,nmodes,nsta,nsta), intent(in) :: iotamask,mumask
        integer(long), dimension(nadd), intent(in)                    :: termkmask
        integer(long)       :: i,j,s1,s,c,k,infos,x,y,ipop
        real(dop), dimension(300)                  :: rarray
        real(dop), dimension(300,3)                :: arry
        integer(long), dimension(300)              :: nord
        integer(long), dimension(300,300)          :: tarray
        logical(kind=4) :: once,exity
        real(dop) :: prod,prod2,prod3,noise,jj,avrg2,tot2, tot_mutcake,rvar1,rvar2

!       INITIALISING SOME PARAMETERS
        tarray = 0
        arry = 0.0
        ipop = 0
        do j=1,300
          nord(j) = 0
          rarray = 0.0
          do c=1,5000
              paramvec(c,j) = 0
           enddo
        enddo

        co_weights(1:npop0,4) = 1.0d0
        co_weights(1:npop0,3) = 0.0d0
!        print*, 'COWEIGHT 2 ==', co_weights(1:npop0,2)
        co_weights(1:npop0,1) = co_weights(1:npop0,2)
!        print*, 'COWEIGHT 1 ==', co_weights(1:npop0,2)
        call sort(co_weights(1:npop0,3),co_weights(1:npop0,1),npop0)
!        print*, 'COWEIGHT after ==', co_weights(1:npop0,1)


          do j = 1, npop0
             do i = 1, npop0
                if( co_weights(j,2) .gt. 50 .and. ngenga .gt. 20) co_weights(j,4) = 0.0d0
                if( co_weights(j,2) .gt. 50 .and. ngenga .gt. 20) ipop = ipop + 1
                if( co_weights(j,2) .gt. 50 .and. ngenga .gt. 20) print*, 'ITS HAPPENING', co_weights(j,2), j
                if( co_weights(j,2) .eq. co_weights(i,1)) co_weights(j,2) = npop0 - i + 1
!                if( co_weights(i,1) .gt. 50 .and. ngenga .gt. 20) co_weights(j,4) = 0.0d0
!                if( co_weights(i,1) .gt. 50 .and. ngenga .gt. 20) print*, 'RMSD GOT =', co_weights(i,)
!      if( co_weights(j,4) = 0.0d0 ) print*, 'its got one :', co_weights(j,4), co_weights(i,1)
                if( co_weights(j,2) .eq. npop0 - i + 1  ) EXIT
             enddo

          enddo

!       SO IF CO_WEIGHT(J,2) = 1, PARTICIPANT J IS THE BEST

!	using the actual RMSD values of each participant:
!       avrg = 0
!       do i = 1, npop0
!        co_weights(i,2) =  co_weights(i,2)/armsd
!        avrg = avrg + co_weights(i,2)
!       enddo
!	or using an 1/x decay:
!          avrg = 0
!          do i = 1, npop0
!             avrg = avrg + i
!          enddo
!          avrg = avrg/npop0

!	co_we(4) is zero if the RMSD of that participant is absurdly high (anything above 40,id say).
!	using an exponential decay for co_we(2) and gaussian for co_we(1) distribuition
           avrg2 = 0.0
           prod = sqrt( - (real(npop0*(2.0/3.0)))**2 / log(0.05) )
           prod3 = sqrt( - (real(npop0))**2 / log(0.05) )
           mutcake = 0.0
!        print*, 'before switch'
           tot_mutcake = 0.0
           do i = 1, npop0
            co_weights(i,1) = co_weights(i,2)
!       MUTCHANCE IS USED IN THE MUTATE ROUTINE; IF THE DISTRIBUITION'S KURTOSIS
!       BECOMES HIGH, THE BEST PARTICIPANTS,IF MUTATED, WILL HAVE MORE TERMS MUTATED
!       (FROM A VERY NOISY COVARIANCE MATRIX THAT WILL GIVE OUT DECREASE THE KURTOSIS)
!       this looks a bit like a decaying exponential:
            mutchance_w(i) = 1-tanh(co_weights(i,1)*4/npop0)

!       0.99 is arbitrarily close to 1
            rvar1 = 2*0.99-1
!       inverse tanh:
            rvar2 = 0.5*log( (1+rvar1)/(1-rvar1) )/(20)
!       20 is an arbitraty value for a high kurtosis
!       tanh function platooing at ~0 and ~40 (centred at 20)
            prod2 = 0.5*(tanh(rvar2*(abs(kurtosis(300))-20)) + 1)
!       mutcake is a function that gives every candidate in the population a probability of mutation
!       that depend on a gaussian distribuition whose centre depends on the kurtosis of the overall population
!       this is to mutate more of the good participants if the population has too many of them.
            mutcake(i) = 1* exp( - ( (co_weights(i,1) - prod2*npop0  ) / prod3 )**2 )
            tot_mutcake = tot_mutcake + mutcake(i)
!	   gaussian
!            print*, i, co_weights(i,1)
            co_weights(i,1) =  1* exp( - ( (co_weights(i,1)) / prod )**2 )
!       centered at the centre (median) of the distribuition:
!            co_weights(i,1) =  1* exp( - ( (co_weights(i,1)-npop0/2) / prod )**2 )
!       centered at the top 1/3 of the distribuition (why not? :P):
!            co_weights(i,1) =  1* exp( - ( (co_weights(i,1)-npop0*(1/3)) / prod )**2 )

!	   exp decay (the exponential constant is a function of the generation)
            co_weights(i,2) = co_weights(i,4)*exp( -co_weights(i,2)*(log(2.0)/(npop0/(10 + (ngenga/maxgen)*10 )   )))
            avrg2 = avrg2 + co_weights(i,2)
           enddo

            print*, 'centre of mutation distro = ', prod2
           tot2 = avrg2
!	 what is the average of a exponentail decay distribuition?
           avrg2= avrg2/dble(npop0-ipop)
!           print*, 'AVERAGE 2 WEIGHT =', avrg2
           mutcake= mutcake/tot_mutcake

!	  DEBUG 'TOOL' this overwrites on mutchance_w or co_weight(1,) for the sake of printing out
!	  how the exponential weighting looks:
!         co_weights(1:npop0,1) = co_weights(1:npop0,2)
!          print*, 'ORDEDEREASDFGSDAFASDFASDFASDF'
!         co_weights(1:npop0,3) = 0.0d0
!         call sort(co_weights(1:npop0,3),mutcake(1:npop0),npop0)
!         call sort(co_weights(1:npop0,3),mutchance_w(1:npop0),npop0)
!         call sort(co_weights(1:npop0,3),co_weights(1:npop0,1),npop0)
!         do i= 1, npop0
!           print*, i , mutcake(i)
!           print*, i , mutchance_w(i)
!           print*, i , co_weights(i,1)
!         enddo

!	FINALY CO_WEIGHTS(2) HAS AN NEGATIVE EXPONONETIAL DISTRIBUITION (NORMALISED = NPOP0)
!	WHERE THE HIGHEST VALUES ARE GIVEN TO THE HIGHEST RMSD'S (USED IN CONVARIANCE MATRIX)
!	CO_WEIGHTS(1) HAS INSTEAD AN (NPOP) NORMALISED GAUSSIAN DISTRIBUITION CENTRED AT THE (MEDIANorBEST) RMSD PARTICIPANT
!	(USED IN THE 'NOISE' WHEN FEEDING PARAMETERS INTO COVARIANCE MATRIX, TO IMPORVE KURTOSIS
!	normalizing:
          do i = 1, npop0
             co_weights(i,2) = (co_weights(i,2)/tot2)*(npop0-ipop)
!             print*, i,  co_weights(i,2)
          enddo



!	normalized:
!         avrg2 = 0.0
!         do i =1, npop0
!           avrg2 = avrg2 + co_weights(i,2)
!         enddo

!       NOW GATHER PARAMETER DATA  TO BUILD COVARIANCE MATRIX
        do c=1,npop0
           call gafilename(ngenga,c,filename)
           open(iguess,file=filename,form='formatted',status='unknown')

          call readgapar(iguess,mr(mrkappa),mr(mrlambda),mr(mrgamma),mr(mriota),mr(mrmu)&
                         ,mr(mrexpcoe),mr(mrtermk) )
           close(iguess)
           x = 0

!	  how much noise is 'injected' into the covariance (to reduce the kurtosis(x) in the population) will go down as
!	  the generations progress. it seems like a good idea to make the better value parameters more noisy
!         (given by the co_weight(c,1)) since there
!         are already too many of those in the pop (hence the large kurtosis) and we want the covariance to start sugggestting other
!         direction (simultaneously, in the mutate routine the number of parameters mutated will increase from good contestants)

          prod = sqrt( - maxgen**2 / log(0.05) )
          noise = exp( - ( ngenga / prod ) ** 2 )*co_weights(c,1)
!          noise = co_weights(c,1)
          prod = -log(0.01)/30.0
!          print*, 'NOISE =', noise
!          print*, 'CO_WEIGHTS = ', co_weights(c,2)
           do s=1,nsta
              do i=1,nmodes
                 if (kappamask(i,s) .eq. 1) then
                    x = x + 1
                    if(kurtosis(x) .gt. 0 ) then
!                      paramvec(c,x) = rand_normal(kappa(i,s),(1.00000001-exp(-kurtosis(x)*prod) )*noise*kappamut(2) )
!                      paramvec(c,x) = rand_normal(kappa(i,s),(1.00000001-exp(-kurtosis(x)*prod) )*noise*st_dev(x)**2 )
          paramvec(c,x) = &
!          rand_normal(kappa(i,s),(1.00000001-exp(-kurtosis(x)*prod))*noise*( kappamut(1)/5 ))
!          rand_normal(kappa(i,s),(1.00000001-exp(-kurtosis(x)*prod))*noise*(5*(st_dev(x)) ))
!          rand_normal(kappa(i,s),(1.00000001-exp(-kurtosis(x)*prod))*noise*(kappamut(1)/10)/(st_dev(x)) )
          kappa(i,s)+(1.00000001-exp(-kurtosis(x)*prod))*noise*(kappamut(1)/10)/(st_dev(x))*real(randgaussian())
                    else
                      paramvec(c,x) = kappa(i,s)
                    endif
                 endif
              enddo
           enddo
           do s=1,nsta-1
              do s1=s+1,nsta
                 do i=1,nmodes
                    if (lambdamask(i,s,s1) .eq. 1) then
                       x = x + 1

                    if(kurtosis(x) .gt. 0 ) then
!                    paramvec(c,x) = rand_normal(lambda(i,s,s1),(1.00000001-exp(-kurtosis(x)*prod) )*noise*st_dev(x)**2 )
          paramvec(c,x) = &
!          rand_normal(lambda(i,s,s1),(1.00000001-exp(-kurtosis(x)*prod) )*noise*( lambdamut(1)/5 ) )
!          rand_normal(lambda(i,s,s1),(1.00000001-exp(-kurtosis(x)*prod) )*noise*(lambdamut(1)/10)/(st_dev(x)) )
          lambda(i,s,s1)+(1.00000001-exp(-kurtosis(x)*prod) )*noise*(lambdamut(1)/10)/(st_dev(x))*real(randgaussian())
                  if (c .eq. 1 .or. c .eq. 2) print*, 'l =', lambda(i,s,s1), 'n = ', noise, 'k= ', kurtosis(x), &
          's = ', 1.00000001-exp(-kurtosis(x)*prod) , 'p=', prod, 'sd=', &
          (1.00000001-exp(-kurtosis(x)*prod) )*noise*( lambdamut(1)/5 ) , 'p = ', paramvec(c,x)
                  if (c .eq. 1 .or. c .eq. 2) write(ilog,*) 'l =', lambda(i,s,s1), 'n = ', noise, 'k= ', kurtosis(x), &
          's = ', 1.00000001-exp(-kurtosis(x)*prod) , 'p=', prod, 'sd=', &
          (1.00000001-exp(-kurtosis(x)*prod) )*noise*( lambdamut(1)/5 ) ,  'p = ', paramvec(c,x)
!                    paramvec(c,x) = rand_normal(lambda(i,s,s1),(1.00000001-exp(-kurtosis(x)*prod) )*noise*lambdamut(1) )
!                    if(kurtosis(x) .gt. 10) then
!                print*,' EFEECT = ', noise*(lambdamut(1)/10)/(st_dev(x)**2)
!                print*, 'RATIO = ', (lambdamut(1)/10)/(st_dev(x)**2)
!                   print*, 'exp(-x) = ', exp(-kurtosis(x)*prod), &
!               ' so effective st_dev =',(1.00000001-exp(-kurtosis(x)*prod) )*noise*(lambdamut(1)/10)/(st_dev(x))
!                    print*, 'LAMDBA = ', lambda(i,s,s1), 'KURT = ', kurtosis(x), ' paramvec=', paramvec(c,x), 'st_dev=', st_dev(x)
!                    endif
                    else
                       paramvec(c,x) = lambda(i,s,s1)
                    endif
                    endif
                 enddo
              enddo
           enddo

           do s=1,nsta
              do i=1,nmodes
                 do j=i,nmodes
                    if (gammamask(i,j,s) .eq. 1) then
                       x = x + 1

                    if(kurtosis(x) .gt. 0 ) then
!                    paramvec(c,x) = rand_normal(gamma(i,j,s),(1.00000001-exp(-kurtosis(x)*prod) )*noise*gammamut(1) )
          paramvec(c,x) = &
          gamma(i,j,s)+(1.00000001-exp(-kurtosis(x)*prod) )*noise*(gammamut(1)/10)/(st_dev(x))*real(randgaussian())
!          rand_normal(gamma(i,j,s),(1.00000001-exp(-kurtosis(x)*prod) )*noise*(gammamut(1)/10)/(st_dev(x)) )
!          rand_normal(gamma(i,j,s),(1.00000001-exp(-kurtosis(x)*prod) )*noise*(gammamut(1)/5 ))
!          rand_normal(gamma(i,j,s),(1.00000001-exp(-kurtosis(x)*prod) )*noise*(5*(st_dev(x)) ))
!                    paramvec(c,x) = rand_normal(gamma(i,j,s),(1.00000001-exp(-kurtosis(x)*prod) )*noise*st_dev(x)**2 )i
!                    if(kurtosis(x) .gt. 10) print*, 'gammas ', paramvec(c,x), 'ratio =', (gammamut(1)/10)/(st_dev(x))
                    else
                       paramvec(c,x) = gamma(i,j,s)
                    endif

                    endif
                 enddo
              enddo
           enddo

           do i = 1, nadd
              if (termkmask(i) .eq. 1) then
                    x = x + 1
                    if(kurtosis(x) .gt. 0 ) then
!                  paramvec(c,x) = rand_normal(termk(i),(1.00000001-exp(-kurtosis(x)*prod) )*noise*termkmut(1) )
!                  paramvec(c,x) = rand_normal(termk(i),(1.00000001-exp(-kurtosis(x)*prod) )*noise*st_dev(x)**2 )
          paramvec(c,x) = &
!          rand_normal(termk(i),(1.00000001-exp(-kurtosis(x)*prod) )*noise*(termkmut(1)/5 ) )
!          rand_normal(termk(i),(1.00000001-exp(-kurtosis(x)*prod) )*noise*(5*(st_dev(x)) ) )
!          rand_normal(termk(i),(1.00000001-exp(-kurtosis(x)*prod) )*noise*(termkmut(1)/10)/(st_dev(x)) )
          termk(i)+(1.00000001-exp(-kurtosis(x)*prod) )*noise*(termkmut(1)/10)/(st_dev(x))*real(randgaussian())
!                    if(kurtosis(x) .gt. 10) print*, 'termk ', paramvec(c,x), 'ratio =', (termkmut(1)/10)/(st_dev(x))
                    else
                 paramvec(c,x) = termk(i)
                    endif
              endif
           enddo

           do s=1,nsta-1
              do s1=s+1,nsta
                 do i=1,nmodes
                    do j=i,nmodes
                       if (mumask(j,i,s,s1) .eq. 1) then
                          x = x + 1
!                  print*,'mu =', j,i,s,s1, 'x = ', x
                    if(kurtosis(x) .gt. 0 ) then
!                  paramvec(c,x) = rand_normal(mu(j,i,s,s1),(1.00000001-exp(-kurtosis(x)*prod) )*noise*mutmut(1) )
!                  paramvec(c,x) = rand_normal(mu(j,i,s,s1),(1.00000001-exp(-kurtosis(x)*prod) )*noise*st_dev(x)**2 )
          paramvec(c,x) = &
!          rand_normal(mu(j,i,s,s1),(1.00000001-exp(-kurtosis(x)*prod) )*noise*(5*(st_dev(x)) ))
!          rand_normal(mu(j,i,s,s1),(1.00000001-exp(-kurtosis(x)*prod) )*noise*( mumut(1)/5 ))
!          rand_normal(mu(j,i,s,s1),(1.00000001-exp(-kurtosis(x)*prod) )*noise*(mumut(1)/10)/(st_dev(x)) )
          mu(j,i,s,s1)+(1.00000001-exp(-kurtosis(x)*prod) )*noise*(mumut(1)/10)/(st_dev(x))*real(randgaussian())
!                    if(kurtosis(x) .gt. 10) print*, 'mu ', paramvec(c,x),'ration=', (mumut(1)/10)/(st_dev(x))
                    else
                          paramvec(c,x) = mu(j,i,s,s1)
                    endif
                       endif
                    enddo
                 enddo
              enddo
           enddo

           do s=1,nsta
              do s1=s,nsta
                 do i=1,nmodes
                    do j=1,nmodes
                       if (iotamask(j,i,s,s1) .eq. 1) then
                          x = x + 1
                    if(kurtosis(x) .gt. 0 ) then
!                  paramvec(c,x) = rand_normal(iota(i,j,s,s1),(1.00000001-exp(-kurtosis(x)*prod) )*noise*iotamut(1) )
!                  paramvec(c,x) = rand_normal(iota(j,i,s,s1),(1.00000001-exp(-kurtosis(x)*prod) )*noise*st_dev(x)**2 )
          paramvec(c,x) = &
!          rand_normal(iota(i,j,s,s1),(1.00000001-exp(-kurtosis(x)*prod) )*noise*( iotamut(1)/5 ) )
!          rand_normal(iota(i,j,s,s1),(1.00000001-exp(-kurtosis(x)*prod) )*noise*(5*(st_dev(x)) ) )
!          rand_normal(iota(i,j,s,s1),(1.00000001-exp(-kurtosis(x)*prod) )*noise*(iotamut(1)/10)/(st_dev(x)) )
          iota(i,j,s,s1)+(1.00000001-exp(-kurtosis(x)*prod) )*noise*(iotamut(1)/10)/(st_dev(x))*real(randgaussian())
!                    if(kurtosis(x) .gt. 10) print*, 'iota ', paramvec(c,x), 'ratio=', (iotamut(1)/10)/(st_dev(x))
                    else
                          paramvec(c,x) = iota(j,i,s,s1)
                    endif
                       endif
                    enddo
                 enddo
              enddo
           enddo

           do s=1,nsta
              do i=1,nmodes
                 if (ncoeff(i,s).gt.0) then
                    do k=1,ncoeff(i,s)
                       if(expcoemask(k,i,s).eq.1) then
                          x=x+1
                    if(kurtosis(x) .gt. 0 ) then
!                  paramvec(c,x) = rand_normal(expcoe(k,i,s),(1.00000001-exp(-kurtosis(x)*prod) )*noise*expcoemut(1,k,i,s) )
          paramvec(c,x) = &
!          rand_normal(expcoe(k,i,s),(1.00000001-exp(-kurtosis(x)*prod) )*noise*(expcoemut(1,k,i,s)/5 ))
!          rand_normal(expcoe(k,i,s),(1.00000001-exp(-kurtosis(x)*prod) )*noise*(5*(st_dev(x)) ))
!          rand_normal(expcoe(k,i,s),(1.00000001-exp(-kurtosis(x)*prod) )*noise*(expcoemut(1,k,i,s)/10)/(st_dev(x)) )
          expcoe(k,i,s)+(1.00000001-exp(-kurtosis(x)*prod) )*noise*(expcoemut(1,k,i,s)/10)/(st_dev(x))*real(randgaussian())
!                    if(kurtosis(x) .gt. 10) print*, 'expcoe ', paramvec(c,x),'ratio=', (expcoemut(1,k,i,s)/10)/(st_dev(x))

!                  paramvec(c,x) = rand_normal(expcoe(k,i,s),(1.00000001-exp(-kurtosis(x)*prod) )*noise*st_dev(x)**2 )
                    else
                          paramvec(c,x)=expcoe(k,i,s)
                    endif
                       endif
                    enddo
                 endif
              enddo
           enddo

        enddo

        do i = 1, 300
           eval_covar(i) = 0d0
        enddo
        do i = 1, 800
           lwork(i) = 0d0
        enddo
        do i = 1, 300
           do j = 1, 300
              param_covar(i,j) = 0d0
           enddo
        enddo

!       do i = 1, x
!        print*, 'I=',trim(charit(i,0)),' ',trim(pname(i)), 'M=', param_mean(i), 'SD=', st_dev(i),'sk=',skew(i),&
!         'k=',  kurtosis(i)
!        print*, 'I=',trim(charit(i,0)),' ',trim(pname(i)), 'M=', param_mean(i), 'SD=', st_dev(i),'sk=',skew(i),&
!         'k=',  kurtosis(i),'DP=',DPtest(i)
!       enddo


!	makes the covariance matrix
!	the old_param_covar matrix is not the actual, last generations covariance matrix but the parameters of last generation shifted by
!	THIS GENBERATIONS MEAN, this will make the basis between both the same (span the same space) and we can identify what e-vectors match between
!	generations( NOTE: THIS DID NOT WORK BECAUSE THEN WE NEED TO MATCH THAT SET OF EVECTORS WITH THE OLD GEN E-VECTORS, WHICH ADDS ANOTHER MATCHING
!	REQUIREMENT, MAKING THE WHOLE THING SLIGHTLY INEFFICIENT, can you think of a solution?)
        do i = 1, x
           do j = 1, x
              param_covar(i,j) = 0.0d0
!              old_param_covar(i,j) = 0.0d0
              do c = 1, npop0
                 param_covar(i,j) = param_covar(i,j) + &
                      (paramvec(c,i) - param_mean(i) ) * (paramvec(c,j) - param_mean(j)) * &
                      co_weights(c,2)
!              print*, 'MEAN = ', param_mean(i), 'val = ', paramvec(c,j)

!                 old_param_covar(i,j) = old_param_covar(i,j) + &
!                      (old_paramvec(c,i) - param_mean(i) ) * (old_paramvec(c,j) - param_mean(j)) * &
!                      co_weights(c,2)
              enddo
!              print*, 'COVAR DIAG= ', param_covar(i,j)
              param_covar(i,j) = param_covar(i,j)/dble(npop0-ipop)
           enddo
        enddo

        p_evec = 0d0
        p_evec = param_covar
!        old_p_evec2 = old_param_covar

!	PRINT COVARIANCE MATRIX TO SCREEN:
!        print*, 'x = ', x
!       print*,'*********************************'
!       print*,'Statistic Results:'
!       do i = 1, x
!         print*, 'I = ',trim(charit(i,0)),' param_mean(I)=',param_mean(i),'cov st.dev(I) =', param_covar(i,i)
!         print*, '###########Covariance DATA:##############'
!         do j = 1, x
!           print*, 'J = ',trim(charit(j,0)), 'Covariance(I,J)=',param_covar(i,j)
!         enddo
!       enddo

       rarray = 0d0

!	diagnoalizes:
        call dsyev( 'V', 'L', x, p_evec, 300, eval_covar, lwork, 800, infos)
!        lwork = 0.0
!        call dsyev( 'V', 'L', x, old_p_evec2, 300, rarray, lwork, 800, infos)

!      print the diagonal covariance matrix and e-vectors
!      print*, 'ERROR?=', infos
!      print*,'DIAGNOALISED COVARIANCE MATRIX:'
!      print*,' E-VECTORS :'
!      do i = 1, x
!         print*, i , eval_covar(i)
!        print*, '####E-VECTOR####'
!        print*, 'I = ',trim(charit(i,0)), '   E-VAL=     ', eval_covar(i)
!        do j = 1, x
!          print*, 'J = ',trim(charit(j,0)), p_evec(i,j)
!        enddo
!      enddo
!        print*, eval_covar(1:x)

!     since the mean is slightly nugded to ignore really big valued outliers, but the covariance
!     does not, as a result the e-values are on some rare occassions insignificantly negative
      do i = 1, x
        if (eval_covar(i) .lt. 0.0) then
          if( eval_covar(i) .lt. -0.000001) &
              print*, 'Warning evalues of covariance matrix ', &
                      'negative, population is probably unstable '
          eval_covar(i) = 0.000000001
        endif
      enddo

!	this enures that the gaussian width of the distribuition does not become too small
!	too quickly and converges on a local minima. (used in mutdis2)
!       note that its generation 2, since the first one does not use the covariance matrix
!	
        scale_stdev = 5
!        if (ngenga .eq. 4) then
!           do i=1,x
!!       scale_stdev is a variable that may now be set in the input
!             min_stdev(i) = (sqrt(eval_covar(i))/scale_stdev)
!           enddo
!        elseif(ngenga .eq. 0) then
!!	  on the generation zero, make the old covariance the same as the current:
!           do i=1,x
!            old_eval_covar(i) = eval_covar(i)
!           enddo
!           old_p_evec = p_evec
!        endif



!	the minimum standard deviation is going to be based on a laplace distribuition that peaks at the
!	highest E-value (i = x)  and tails at i = 0. based on some rought expiriance that is a decent
!	balance of eigenvalues
!	the smaller the denominator in 'prod', the bigger the min for smaller e-values (normal value ~2*0.25)
        do i = 1, x-1
          prod = (1/(2*0.25) )
          jj = ((dble(i)+1)*10/(dble(x)))
!          j = (dble(i)/(dble(x)/10)) + 1
          prod2 = (1/(2*prod))*exp(-abs(jj-10)/prod)*(eval_covar(x)*0.9)
          min_stdev(i) = sqrt(prod2)
!          min_stdev(i) = sqrt(min_stdev(i) )
        enddo

        if(ngenga .eq. 0) then
!	  on the generation zero, make the old covariance the same as the current:
           do i=1,x
            old_eval_covar(i) = eval_covar(i)
           enddo
           old_p_evec = p_evec
        endif

!	determines the length of every e-vector of the new and old covariance matrix (dysev. it turns out, makes them = 1)
!     do j = 1, x
!      len_evec(j,1) = 0
!      len_evec(j,2) = 0
!      do i=1,x
!        len_evec(j,1) = len_evec(j,1) + old_p_evec(i,j)**2
!        len_evec(j,2) = len_evec(j,2) + p_evec(i,j)**2
!      enddo
!       len_evec(j,1) = sqrt(len_evec(j,1))
!       len_evec(j,2) = sqrt(len_evec(j,2))
!!       print*, 'nlens = ', len_evec(j,1) , len_evec(j,2)
!     enddo

!     matches the new e-vectors of gen(n) with the gen(n-1). nord() matches old_eval_covar with current eval_covar (and evec's)
      arry = 0.0
      do k = 1, x
!       print*, 'x-k+1 = ', x-k+1
       do j = 1, x
         prod = 0.0
         do i = 1, x
           prod = prod + p_evec(i,j)*old_p_evec(i,x-k+1 )
         enddo
         arry(j,1) = abs(prod)
!         arry(j,1) = abs(prod)/(len_evec(j,1)*len_evec(j,2))
!         print*, 'k = ', k, 'j = ', j, 'angle =', acos(abs(prod))*180/pi
       enddo
!	it sorts the products (of k with all j's) in decreasing order into arry(,2)
        arry(1:x,2) = arry(1:x,1)
        call sort(arry(1:x,3),arry(1:x,2),x)
        do y = 1, x
         do i = 1, x
          if(arry(y,2) .eq. arry(i,1) ) tarray(x-k+1,y) = i
         enddo
        enddo

!        do i = 1, x
!          print*, 'arry(y,2) = ', arry(i,2) , ' arry(y,2) = ', arry(i,1), 'tarray(x-k+1,y) =', tarray(x-k+1,i)
!        enddo
      enddo



!!	since the basis that describes the old and new e-vectors changes slightly, often two vectors 'comptete' for the same one (in the other space)
!     we want to give priority in the following way:
!     evector+big_evalue + big_overlap > evector+low_evalue + big_overlap > evector+big_evalue + low_overlap > evecto+low_evalue + low_overlap
!     recall that the old_evector order in the array gives the evalue (small evalues come first, that is why I start with x-k+1)
      exity = .false.
!	loop over every new vector (simply indexed by i)
      do i = 1, x
!	loop from the best to worst positions of every old vector
!        print*, ' i = ', i
       do y = 1, x
!	loop over every old vector
        do k = 1, x
!         print*, 'old index = ', x-k+1,' y = ', y, ' = ', tarray( x-k+1 , y )
!	if the old vector's bests
          if( tarray( x-k+1 , y ) .eq. i ) then
           once = .false.
           do s = 1, i
!	 if any previous new vector already directs the that old vector
            if( nord(s) .eq. x-k+1 ) once = .true.
           enddo

           if (once .eqv. .false.) then
!            print*, 'take it i = ', i
             nord( i ) = x-k+1
!           print*, 'old_eval = ', old_eval_covar(x-k+1)
             exity = .true.
             EXIT

           else
             nord( i ) = x-k+1
             exity = .true.

            print*, 'already taken i = ', i
!           print*, 'old_eval = ', old_eval_covar(x-k+1)
!           print*, 'when it is now = ', eval_covar(i)
            once = .false.
            EXIT
           endif
          endif
        enddo
        if( exity) EXIT
       enddo
       exity = .false.
      enddo

!      do i = 1, x
!        print*, ' nord =', nord(i)
!      enddo


      tarray = 0

! 	THIS BIT BELOW JUST SHOWS THAT THE E-VECTORS OF OLD_P_EVEC AND P_EVEC ARE ORTHOGONAL
!      do k = 1, x
!
!       do j = 1, x
!         prod = 0
!         do i = 1, x
!           prod = prod + old_p_evec(i,j)*old_p_evec(i,k)
!         enddo
!         print*,'k = ', k, 'j = ', j,  ' self_angle =', acos(abs(prod))*180/pi
!       enddo
!      enddo
!      print*, 'ESDFASFDAFDASDFASDFASDFASDFASDF'
!      do k = 1, x
!       do j = 1, x
!         prod = 0
!         do i = 1, x
!           prod = prod + p_evec(i,j)*p_evec(i,k)
!         enddo
!         print*,'k = ', k, 'j = ', j,  ' self_angle =', acos(abs(prod))*180/pi
!       enddo
!      enddo

!	updates old_paramvec
!        old_paramvec = paramvec
        old_p_evec = p_evec

!	switches the index values of min_e_st_dev array to match this generations e-vals (the correct ones since nord points old_eval_covar
!	which uses the previous generation e_val index order:
!      if( ngenga .gt. 4) then
!        do i = 1, x
!          rarray(i) = min_stdev(i)
!        enddo
!        do i = 1, x
!          min_stdev(i) = rarray(nord(i))
!        enddo
!      endif

      if (ngenga .ge. 1) then
         do i=1,x
!	   this sets the standard deviation of the normal distribuition of the new parameters of mutants in such a way as to keep the
!	   standard deviation of this parameter of the whole population constant (except for the normal gaussian function that
!          depends on the current generation that
!          multiplies the maximum possible value of the distribuition, see mutdis2)
!          n = number of mutants every gen, N = total pop num, v_n = variance of this gen, v_o =  variance of last gen, v_m = variance of
!	   this generations mutants  (note that the these are the variance of the E-values, and have the mean at zero so is just = sum(var)/N
!          so to keep e_st_dev semi- constant between gen's,
!          ((N -n)*v_n + n*v_m)/N = v_o    =>  v_m = ( N*(v_o -v_n) + n*v_n)/n
!          NOTE: I MAKE N => N/2  (npop0/2) BECAUSE THE COVARIANCE MATRIX IS WEIGHTED AND IT ALREADY EVAGGERATES THE E-VALUES

!         print*,  'E_ST_DEV  =', e_st_dev(i)
!         print*,'EVALCOVA = ', eval_covar(i)
!         print*,'oldEALCOVA = ', old_eval_covar(nord(i))
!           print*,'duff = ', npop0*(old_eval_covar(nord(i)) - eval_covar(i))
!         print*, 'other bit', int(npop0*nmut)*eval_covar(i)

           if( old_eval_covar(nord(i)) .gt. eval_covar(i) ) then
            e_st_dev(i) = ( (npop0/2)*(old_eval_covar(nord(i)) - eval_covar(i)) + int(npop0*nmut)*eval_covar(i))/ &
            int(npop0*nmut)
           else
            e_st_dev(i) = 0.0
           endif


!	   this is a bodge: sometimes the above equation gives really high values that become usseless
           if(e_st_dev(i) .gt. eval_covar(i)*1.5) e_st_dev(i) = eval_covar(i)*1.5
           if(e_st_dev(i) .gt. eval_covar(i)*1.5) print*, 'GREATER THAN EVAL'

!	   if the new variance is bigger than the old variance, then the variance for the new mutants should just be set to the current
!	   variance (since the variance typically has the tendency to become smaller) (since we cant have sqrt(-ve), it means that the cannot
!	   decrease the variance by adding mutants )
           if (e_st_dev(i) .le. 0.0) then
             e_st_dev(i) = sqrt(eval_covar(i))
!             print*, 'a_st_dec = ', e_st_dev(i)
           else
             e_st_dev(i) = sqrt(e_st_dev(i))
           endif
         enddo
      endif
!      print*, 'MINSDDEV =', min_stdev(1:x)
!      print*, 'E_ST_DEV  =', e_st_dev(1:x)
!      print*,'EVALCOVA = ', eval_covar(1:x)
!      print*,'oldEALCOVA = ', old_eval_covar(1:x)

!	this makes the old_eval_covar a weighted average of this generation and the previous one (giving the previous one a weight of 1)
!	this parameter could potentially be optimized, but who has the time? not me!
        do i = 1,x
         if(old_eval_covar(nord(i)) .gt. eval_covar(i) ) &
            old_eval_covar(nord(i)) = (old_eval_covar(nord(i)) + eval_covar(i))/2
         if( sqrt(old_eval_covar(nord(i))) .lt. min_stdev(i) ) old_eval_covar(nord(i)) = min_stdev(i)**2
!         print*, 'MINSDDEV i = ', i, ' = ', min_stdev(i)
!         print*, 'E_ST_DEV i = ', i, ' = ', e_st_dev(i)
!         print*, 'EVALCOVA i = ', i, ' = ', e_st_dev(i)
        enddo


!	calculates the inverse matrix of p_evec (which, should just be the transpose, for similarity transormations..but jsut in case)
!        inv_p_evec = p_evec
!        ipiv = 0d0
!        call dgetrf( x, x, inv_p_evec, 300, ipiv, infos )
!      print*, 'ERROR1=', infos
!        call dgetri( x, inv_p_evec, 300, ipiv, lwork, 800, infos )
!      print*, 'ERROR2 =', infos
        emutval = 0d0
        mutval = 0d0

!	creates an array called emutval that has 'lamarkian' mutations that will be used in the mutate routine:
!	mutdis2 is just a Normal distribuition with the e_st_dev needed to explore the usefull parts of parameter space

        do c = 1, int(npop0*(2*nmut) )
           do i = 1, x
!              emutval(c,i) = mutdis2(0d0,e_st_dev(i),0.0d0)
!	standard deviation = sqrt(variance) = sqrt(e_st_dev)
              emutval(c,i) = mutdis2(0d0,e_st_dev(i),min_stdev(i))
!              emutval(c,i) = mutdis2(0d0,e_st_dev(i)*0.85,min_stdev(i))
           enddo
        enddo

!	print some data:
!        do i =1,x
!         print*, 'I = ',i
!         print*, 'min_std_dev = ', min_stdev(i), 'eval_covar     = ', eval_covar(i)
!        print*, 'e_st_dev      = ', e_st_dev(i),    'old_eval_covar = ', old_eval_covar(nord(i))
!       enddo


!	DIAGNOALISATION OF MATRIX TO MAKE SURE I HAVE THE RIGHT MATRIX TO GET THE TRANSPOSE/INVERSE
!        temp = 0
!        do i = 1, x
!!         print*, 'i = ', i
!         do k = 1, x
!!          print*, 'k = ', k
!             do j = 1, x
!              do l = 1, x
!!					  TRANSP*MAT*ORIGINAL
!                 temp(i,k) = temp(i,k) + p_evec(j,i) * param_covar(j,l) * p_evec(l,k)
!              enddo
!             enddo
!         enddo
!        enddo
!!      do i = 1, x
!        do j = 1, x
!          print*, eval_covar(j)
!          print*, j, temp(j,j)
!         enddo
!!      enddo
!        print*, '1 EVECTOR=', p_evec(1:x,1)
!        print*, '2 EVECTOR=', p_evec(1:x,2)
!        print*, '3 EVECTOR=', p_evec(1:x,3)
!        print*, '4 EVECTOR=', p_evec(1:x,4)
!        print*, '5 EVECTOR=', p_evec(1:x,5)
!        print*, '6 EVECTOR=', p_evec(1:x,6)

!       the reason why its (2*nmut ) is because sometimes the mutation number increases (gets changed by the program)
!       if theaverage standard dev  becomes low (typically as the Ga finds stabel minima) (see get_par_stats and mutate routines)
        do c = 1, int(npop*(2*nmut) )
           do i = 1, x
              do j = 1, x
                 mutval(c,i) = mutval(c,i) + p_evec(j,i) * emutval(c,j)

!                 mutval(c,i) = mutval(c,i) + inv_p_evec(i,j) * emutval(c,j)
              enddo
              mutval(c,i) = mutval(c,i) + param_mean(i)
!              if( abs(mutval(c,i)) .eq. 0.0 ) print*, 'A ZERO COMING OUT'

!	THIS PRINTS THE MUTATION DISTRIBUITION FOR EACH PARAMETERS (E-PARAMETERS AND NORMAL ONES) INTO FILES TO PLOT IN GNUPLOT:
!              write(poo,'(F10.6)')  mutval(c,i)
!              cc(i) = cc(i) + 1
!              if( cc(i) .ge. 9999) cc(i) = 0
!              call system(' echo  '//charit(cc(i),0)//'  '//trim(poo)//'  >> out_mutval'//trim(charit(i,1))//'.dat')
!              write(poo,'(F10.6)') emutval(c,i)
!              call system(' echo  '//charit(cc(i),0)//'  '//trim(poo)//'  >> out_emutval'//trim(charit(i,1))//'.dat')
           enddo
        enddo
        return
      end subroutine get_par_covar

!#######################################################################

      real(dop) function mutcomag(stdev)

      real(dop), intent(in) :: stdev
      real(dop)             :: x

      x = 2*drandom(seed)-1
      x = 3*stdev
      mutcomag =  (abs(x)/x)*3*stdev* exp( - ( x / stdev )**2 )

      return

      end function

!#######################################################################

      SUBROUTINE sort (array_x, array_y, datasize)
!      Global Definitions
       REAL(dop) array_x(*)
       REAL(dop) array_y(*)
       INTEGER datasize
!       Local
      REAL(dop) x_temp
      REAL(dop) y_temp
      LOGICAL inorder
      inorder = .false.
      do 90 while (inorder.eqv..false.)
       inorder = .true.
       do 91 i=1, datasize
! Check Equilivant Points and swap those on Y
       if (array_x(i).eq.array_x(i+1) ) then
        if (array_y(i).lt.array_y(i+1) ) then
         x_temp = array_x(i)
         y_temp = array_y(i)
         array_x(i) = array_x(i+1)
         array_y(i) = array_y(i+1)
         array_x(i+1) = x_temp
         array_y(i+1) = y_temp
         inorder = .false.
        endif
       endif
! If x needs to be swapped, do so
       if (array_x(i).lt.array_x(i+1) )then
        x_temp = array_x(i)
        y_temp = array_y(i)
        array_x(i) = array_x(i+1)
        array_y(i) = array_y(i+1)
        array_x(i+1) = x_temp
        array_y(i+1) = y_temp
        inorder = .false.
       endif
91    continue
90    continue
      END SUBROUTINE sort

!#######################################################################

      subroutine outtiming


        implicit none

        real(dop)     :: cpu_now,totcpu
        integer(long), save :: tid=0
        integer(long), save :: tid1=0
        integer(long), save :: tid2=0

        call cpu_time(cpu_now) ! F95 intrinsic
        totcpu = cpu_now - initcpu

        call get_timer(tid, 'Wconstruct')
        call get_timer(tid1, 'fitpar')
        call get_timer(tid2, 'wdiag')

        write(ilog,'(/,a)') '------------------------------------------'
        write(ilog,'(1x,a)') 'Timing (s)'
        write(ilog,'(a)') '------------------------------------------'
        write(ilog,'(a15,/,F8.2,a,F8.2,a,/)') &
             'Total cpu time:',totcpu,'s,',totcpu/3600.d0,'h'
        write(ilog,'(a23,/,F8.2,a,F8.2,a,/)') 'Total fitting cpu time:',&
             timers(tid)%cputime,'s,',timers(tid)%cputime/3600.d0,'h'
        write(ilog,'(a47,/,F8.2,/)') &
             'Total diabatic potential construction cpu time:',&
             timers(tid1)%cputime
        write(ilog,'(a50,/,F8.2,/)') &
             'Total diabatic potential diagonalisation cpu time:',&
             timers(tid2)%cputime

      end subroutine outtiming

!#######################################################################

      subroutine initgamut

        implicit none

        integer(long) :: m,s

!-----------------------------------------------------------------------
! *mut(x,1) gives you the value for the maximum range of the parameter *
! allowed in the mutation
!
! *mut(x,2) gives you the options 1 2 & 3, three different types of
! mutation testing (1 and 2 are logarithmic, so they are
! used for parameters who's uncertainty on their value spans several
! orders of magnitude)
!-----------------------------------------------------------------------

! Kappa
      if(kappamut(1) .eq. 0.0) kappamut(1) = 1.5
      if(kappamut(2) .eq. 0.0) kappamut(2) = 3.0

! Lambda
      if(lambdamut(1) .eq. 0.0) lambdamut(1) = 3.0
      if(lambdamut(2) .eq. 0.0)lambdamut(2) = 3.0

! Gamma
      if(gammamut(1) .eq. 0.0 ) gammamut(1) = 2.5
      if(gammamut(2) .eq. 0.0 ) gammamut(2) = 3.0

! Mu
      if(mumut(1) .eq. 0.0 ) mumut(1) = 2.5
      if(mumut(2) .eq. 0.0 ) mumut(2) = 3.0

! Iota
      if(iotamut(1) .eq. 0.0 ) iotamut(1) = 2.0
      if(iotamut(2) .eq. 0.0 ) iotamut(2) = 3.0

! Explicitely added terms
      if(termkmut(1) .eq. 0.0) termkmut(1) = 3.0
      if(termkmut(2) .eq. 0.0) termkmut(2) = 3.0

! General diabatic potentials
      do s=1,nsta
         do m=1,nmodes
            if (ncoeff(m,s).gt.0) then
               select case (ifdiab(m,s))
               case(1) ! HO + quartic term
                  if(expcoemut(1,1,m,s) .eq. 0.0 ) expcoemut(1,1,m,s)=0.8 ! kappa
                  if(expcoemut(2,1,m,s) .eq. 0.0 ) expcoemut(2,1,m,s)=3.0 ! kappa
                  if(expcoemut(1,2,m,s) .eq. 0.0 ) expcoemut(1,2,m,s)=1.5 ! gamma
                  if(expcoemut(2,2,m,s) .eq. 0.0 ) expcoemut(2,2,m,s)=3.0 ! gamma
                  if(expcoemut(1,3,m,s) .eq. 0.0 ) expcoemut(1,3,m,s)=1.0 ! epsilo
                  if(expcoemut(2,3,m,s) .eq. 0.0 ) expcoemut(2,3,m,s)=3.0 ! epsilo
               case(2) ! Morse
                  if(expcoemut(1,1,m,s) .eq. 0.0 ) expcoemut(1,1,m,s)=20.0 ! D0
                  if(expcoemut(2,1,m,s) .eq. 0.0 ) expcoemut(2,1,m,s)=3.0 ! D0
                  if(expcoemut(1,2,m,s) .eq. 0.0 ) expcoemut(1,2,m,s)=2.0 ! 2*alpha - omega
                  if(expcoemut(2,2,m,s) .eq. 0.0 ) expcoemut(2,2,m,s)=3.0 ! 2*alpha - omega
                  if(expcoemut(1,3,m,s) .eq. 0.0 ) expcoemut(1,3,m,s)=2.0 ! X0
                  if(expcoemut(2,3,m,s) .eq. 0.0 ) expcoemut(2,3,m,s)=3.0 ! X0
               case(4) ! anti-Morse
                  if(expcoemut(1,1,m,s) .eq. 0.0 ) expcoemut(1,1,m,s)=15.0 ! D0
                  if(expcoemut(2,1,m,s) .eq. 0.0 ) expcoemut(2,1,m,s)=3.0 ! D0
                  if(expcoemut(1,2,m,s) .eq. 0.0 ) expcoemut(1,2,m,s)=2.0 ! 2*alpha - omega
                  if(expcoemut(2,2,m,s) .eq. 0.0 ) expcoemut(2,2,m,s)=3.0 ! 2*alpha - omega
                  if(expcoemut(1,3,m,s) .eq. 0.0 ) expcoemut(1,3,m,s)=2.0 ! X0
                  if(expcoemut(2,3,m,s) .eq. 0.0 ) expcoemut(2,3,m,s)=3.0 ! X0
               case(5) ! Exponential function
                  if(expcoemut(1,1,m,s) .eq. 0.0 ) expcoemut(1,1,m,s)=8.0 ! Einf
                  if(expcoemut(2,1,m,s) .eq. 0.0 ) expcoemut(2,1,m,s)=3.0 ! Einf
                  if(expcoemut(1,2,m,s) .eq. 0.0 ) expcoemut(1,2,m,s)=4.0 ! alpha
                  if(expcoemut(2,2,m,s) .eq. 0.0 ) expcoemut(2,2,m,s)=3.0 ! alpha
               case(9) ! General quartic potential
                  if(expcoemut(1,1,m,s) .eq. 0.0 ) expcoemut(1,1,m,s)=2.0 ! linear term
                  if(expcoemut(2,1,m,s) .eq. 0.0 ) expcoemut(2,1,m,s)=3.0 ! linear term
                  if(expcoemut(1,2,m,s) .eq. 0.0 ) expcoemut(1,2,m,s)=8.0 ! quadratic term
                  if(expcoemut(2,2,m,s) .eq. 0.0 ) expcoemut(2,2,m,s)=3.0 ! quadratic term
                  if(expcoemut(1,3,m,s) .eq. 0.0 ) expcoemut(1,3,m,s)=8.0 ! quartic term
                  if(expcoemut(2,3,m,s) .eq. 0.0 ) expcoemut(2,3,m,s)=3.0 ! quartic term
               case(18) ! Quartic, quartic avoided crossing
                  if(expcoemut(1,1,m,s) .eq. 0.0 ) expcoemut(1,1,m,s)= 3.0 ! gammaL
                  if(expcoemut(2,1,m,s) .eq. 0.0 ) expcoemut(2,1,m,s)= 3.0 ! gammaL
                  if(expcoemut(1,2,m,s) .eq. 0.0 ) expcoemut(1,2,m,s)= 3.0 ! epsilonL
                  if(expcoemut(2,2,m,s) .eq. 0.0 ) expcoemut(2,2,m,s)= 3.0 ! epsilonL
                  if(expcoemut(1,3,m,s) .eq. 0.0 ) expcoemut(1,3,m,s)= 3.0 ! gammaU
                  if(expcoemut(2,3,m,s) .eq. 0.0 ) expcoemut(2,3,m,s)= 3.0 ! gammaU
                  if(expcoemut(1,4,m,s) .eq. 0.0 ) expcoemut(1,4,m,s)= 3.0 ! epsilonU
                  if(expcoemut(2,4,m,s) .eq. 0.0 ) expcoemut(2,4,m,s)= 3.0 ! epsilonU
                  if(expcoemut(1,5,m,s) .eq. 0.0 ) expcoemut(1,5,m,s)= 3.0 ! lambda
                  if(expcoemut(2,5,m,s) .eq. 0.0 ) expcoemut(2,5,m,s)= 3.0 ! lambda
                  if(expcoemut(1,6,m,s) .eq. 0.0 ) expcoemut(1,6,m,s)= 10.0 ! EU
                  if(expcoemut(2,6,m,s) .eq. 0.0 ) expcoemut(2,6,m,s)= 3.0 ! EU
               case(19) ! 4th-order Exe Jahn-Teller expansion
                  if(expcoemut(1,1,m,s) .eq. 0.0 ) expcoemut(1,1,m,s)= 5.0 ! kappa
                  if(expcoemut(2,1,m,s) .eq. 0.0 ) expcoemut(2,1,m,s)= 2.0 ! kappa
                  if(expcoemut(1,2,m,s) .eq. 0.0 ) expcoemut(1,2,m,s)= 8.0 ! gamma
                  if(expcoemut(2,2,m,s) .eq. 0.0 ) expcoemut(2,2,m,s)= 2.0 ! gamma
                  if(expcoemut(1,3,m,s) .eq. 0.0 ) expcoemut(1,3,m,s)= 3.0 ! eta
                  if(expcoemut(2,3,m,s) .eq. 0.0 ) expcoemut(2,3,m,s)= 2.0 ! eta
                  if(expcoemut(1,4,m,s) .eq. 0.0 ) expcoemut(1,4,m,s)= 2.0 ! zeta
                  if(expcoemut(2,4,m,s) .eq. 0.0 ) expcoemut(2,4,m,s)= 2.0 ! zeta
                  if(expcoemut(1,5,m,s) .eq. 0.0 ) expcoemut(1,5,m,s)= 2.0 ! iota
                  if(expcoemut(2,5,m,s) .eq. 0.0 ) expcoemut(2,5,m,s)= 2.0 ! iota
                  if(expcoemut(1,6,m,s) .eq. 0.0 ) expcoemut(1,6,m,s)= 8.0 ! tau
                  if(expcoemut(2,6,m,s) .eq. 0.0 ) expcoemut(2,6,m,s)= 2.0 ! tau
                  if(expcoemut(1,7,m,s) .eq. 0.0 ) expcoemut(1,7,m,s)= 5.0 ! alpha
                  if(expcoemut(2,7,m,s) .eq. 0.0 ) expcoemut(2,7,m,s)= 2.0 ! alpha
                  if(expcoemut(1,8,m,s) .eq. 0.0 ) expcoemut(1,8,m,s)= 5.0 ! beta
                  if(expcoemut(2,8,m,s) .eq. 0.0 ) expcoemut(2,8,m,s)= 2.0 ! beta
                case(20)
                case(21)
                  if(expcoemut(1,3,m,s) .eq. 0.0 ) expcoemut(1,3,m,s)=2.0 ! X0
!       later...
               end select
            endif
         enddo
      enddo

    end subroutine initgamut

!#######################################################################

    subroutine transformq(qcoo)

    implicit none

    integer(long)                                     :: n
    real(dop), dimension(nmodes,nsets), intent(inout) :: qcoo

    do n=1,nsets
       call transformq1(qcoo(1,n))
    enddo

    end subroutine transformq

!#######################################################################

    end program vchfit

